/// <reference path="../typings/index.d.ts" />
/**
 * Shared base class for all EVA services
 *
 * @export
 * @abstract
 * @class EvaService
 */
export interface IEvaServiceDefinition {
    name: string;
    path: string;
    request?: unknown;
    response?: unknown;
}
/**
 * The EVA Service factory is used to created a fully type enabled instance
 * of an EVA backend service
 *
 * @export
 * @template SVC The type of EVA service
 * @param {new () => SVC} service The EVA service definition
 * @returns {SVC} Instantiated EVA service definition
 */
export declare function createServiceDefinition<SVC extends IEvaServiceDefinition>(service: new () => SVC): SVC;
/**
 * Expose the service definitions API version
 */
export * from './api-version.js';
/**
 * Expose the service definitions for EVA.Core
 */
export * from './EVA.Core.Services.js';
//# sourceMappingURL=eva-services-core.d.ts.map