declare module EVA.API.Barcodes {

  /**
   * Generate a barcode image.
   * 
   * The `Format` is one of;
   * 
   * - QR = 0
   * - EAN = 1
   * - Code39 = 2
   * - Code128 = 3
   * 
   * :lock: This is the safer variant to our `/barcode` endpoint, because the data moves _inside_ the POST body instead of through GET query params
   * :bulb: You can use the returned `Data` in combination with `MimeType` to render the image directly, for example;
   * 
   * ```
   *   <img src='data:{MimeType};base64,{Data}' />
   * ```
   */
  export interface Barcode {
    Width?: number;
    Height?: number;
    Code?: string;
    Format?: Barcode.BarcodeFormat;
    BlobID?: string;
    NoMargin?: boolean;
    NoText?: boolean;
    BackgroundHex?: string;
    ForegroundHex?: string;
  }

  export namespace Barcode {
    export const enum BarcodeFormat {
      QR = 0,
      EAN = 1,
      Code39 = 2,
      Code128 = 3
    }

  }

  export interface BarcodeResponse {
    MimeType: string;
    Data: string;
    Error?: EVA.Core.ServiceError;
    Metadata: EVA.Core.ResponseMessageMetadata;
  }

}
