declare module EVA.Admin {

  export namespace Errors {
    export const enum Workspaces {
      /**
       * You are not authorized to view this workspace.
       */
      UnauthorizedToViewWorkspace = 'Workspaces:UnauthorizedToViewWorkspace',
      /**
       * Workspace is owned by other user.
       */
      WorkspaceOwnedByOtherUser = 'Workspaces:WorkspaceOwnedByOtherUser',
      /**
       * The workspace is private.
       */
      WorkspaceIsPrivate = 'Workspaces:WorkspaceIsPrivate',
      /**
       * The workspace is shared.
       */
      WorkspaceIsShared = 'Workspaces:WorkspaceIsShared',
      /**
       * Cannot set a workspace without key to be the active one.
       */
      CannotSetWorkspaceWithoutKeyToActive = 'Workspaces:CannotSetWorkspaceWithoutKeyToActive',
      /**
       * Cannot remove a shared workspace as it still has Active members.
       */
      CannotRemoveSharedWorkspace = 'Workspaces:CannotRemoveSharedWorkspace',
      /**
       * Cannot remove an active workspace as it's still active
       */
      CannotRemoveActiveWorkspace = 'Workspaces:CannotRemoveActiveWorkspace',
      /**
       * Workspaces can't have duplicate names
       */
      DuplicateNames = 'Workspaces:DuplicateNames',
      /**
       * RoleIds: {0:string} are not connected to the provided workspace.
       */
      RoleDoesNotHaveSharedWorkspace = 'Workspaces:RoleDoesNotHaveSharedWorkspace'
    }

  }


  /**
   * List all shared workspaces based on the given filter
   */
  export interface ListSharedWorkspaces {
    InitialPageConfig?: EVA.Core.PageTokenConfig<ListSharedWorkspacesFilter>;
    PageToken?: string;
  }

  export interface ListSharedWorkspacesFilter {
    Key?: string;
    /**
     * Entity type: Role
     */
    RoleID?: number;
    Name: string;
  }

  export interface ListSharedWorkspacesResponse {
    Results?: ListSharedWorkspacesResponse.Workspace[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListSharedWorkspacesResponse {
    export interface Workspace {
      /**
       * Entity type: Workspace
       */
      ID: number;
      Key: string;
      Name: string;
      Settings: string;
      IsAllowedToEdit: boolean;
      RolesWithDefault: Workspace.RoleWithDefault[];
    }

    export namespace Workspace {
      export interface RoleWithDefault {
        /**
         * Entity type: Role
         */
        ID: number;
        IsDefault: boolean;
        Name: string;
      }

    }

  }

  /**
   * Get a shared workspace by ID
   */
  export interface GetSharedWorkspaceByID {
    /**
     * Entity type: Workspace
     */
    ID: number;
  }

  export interface GetSharedWorkspaceByIDResponse {
    /**
     * Entity type: Workspace
     */
    ID: number;
    /**
     * Entity type: User
     */
    OwnerID: number;
    Key: string;
    Name: string;
    Settings: string;
    RolesWithDefault: GetSharedWorkspaceByIDResponse.RoleWithDefault[];
    IsAllowedToEdit: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSharedWorkspaceByIDResponse {
    export interface RoleWithDefault {
      /**
       * Entity type: Role
       */
      ID: number;
      IsDefault: boolean;
      Name: string;
    }

  }

  /**
   * Creates a new shared workspace.
   */
  export interface CreateSharedWorkspace {
    /**
     * Identifier of workspace target
     */
    Key: string;
    /**
     * Max. length 128 characters
     */
    Name: string;
    Settings: string;
    /**
     * Entity type: Role
     */
    RoleIDs?: number[];
  }

  export interface CreateWorkspaceResponse {
    /**
     * Entity type: Workspace
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a shared workspace by ID
   */
  export interface UpdateSharedWorkspace {
    /**
     * Entity type: Workspace
     */
    ID: number;
    /**
     * Max. length 128 characters
     */
    Name: string;
    Settings: string;
  }

  /**
   * Add roles to a shared workspace
   */
  export interface AddRolesToSharedWorkspace {
    /**
     * Entity type: Workspace
     */
    ID: number;
    /**
     * Entity type: Role
     */
    RoleIDs: number[];
  }

  /**
   * Remove (all) roles from a workspace
   */
  export interface RemoveRolesFromSharedWorkspace {
    /**
     * Entity type: Workspace
     */
    ID?: number;
    RemoveAllRoles?: boolean;
    /**
     * Entity type: Role
     */
    RoleIDs?: number[];
  }

  /**
   * Delete a shared workspace by it's ID
   */
  export interface DeleteSharedWorkspace {
    /**
     * Entity type: Workspace
     */
    ID: number;
    UnlinkActiveWorkspace?: boolean;
  }

  /**
   * Transfer shared wordspace ownership with ID and optional user Update
   */
  export interface TransferSharedWorkspaceOwnership {
    /**
     * Entity type: Workspace
     */
    ID: number;
    /**
     * Entity type: User
     */
    OwnerID?: number;
  }

  /**
   * Set default shared workspace for roles
   */
  export interface SetDefaultSharedWorkspaceForRoles {
    /**
     * Entity type: Workspace
     */
    ID: number;
    /**
     * Entity type: Role
     */
    RoleIDs: number[];
    IsDefault?: boolean;
  }

  /**
   * Get all workspaces for the current user
   */
  export interface GetWorkspaces {
    /**
     * Identifier of workspace target
     */
    Key?: string;
  }

  export interface GetWorkspacesResponse {
    Result: GetWorkspacesResponse.Workspace[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetWorkspacesResponse {
    export interface Workspace {
      /**
       * Entity type: Workspace
       */
      ID: number;
      Key: string;
      Name: string;
      Settings: string;
      IsShared: boolean;
      RolesWithDefault: Workspace.RoleWithDefault[];
      IsActive: boolean;
    }

    export namespace Workspace {
      export interface RoleWithDefault {
        /**
         * Entity type: Role
         */
        ID: number;
        IsDefault: boolean;
        Name: string;
      }

    }

  }

  /**
   * Get a workspace by ID
   */
  export interface GetWorkspaceByID {
    /**
     * Entity type: Workspace
     */
    ID: number;
  }

  export interface GetWorkspaceByIDResponse {
    /**
     * Entity type: Workspace
     */
    ID: number;
    Key?: string;
    Name: string;
    Settings: string;
    IsShared: boolean;
    IsActive: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new workspace for the current user.
   */
  export interface CreateWorkspace {
    /**
     * Identifier of workspace target
     */
    Key?: string;
    /**
     * Max. length 128 characters
     */
    Name: string;
    Settings: string;
  }

  /**
   * Update a workspace by ID
   */
  export interface UpdateWorkspace {
    /**
     * Entity type: Workspace
     */
    ID: number;
    /**
     * Max. length 128 characters
     */
    Name: string;
    Settings: string;
  }

  /**
   * Set active workspace for the current user
   */
  export interface SetActiveWorkspace {
    /**
     * Entity type: Workspace
     */
    ID: number;
  }

  /**
   * Delete a Workspace by it's ID
   */
  export interface DeleteWorkspace {
    /**
     * Entity type: Workspace
     */
    ID: number;
  }

  /**
   * Gives a list of all admin modules visible by the current user.
   */
  export interface AdminGetAllModules {
  }

  export interface AdminGetAllModulesResponse {
    Modules: AdminGetAllModulesResponse.ModuleDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace AdminGetAllModulesResponse {
    export interface ModuleDto {
      /**
       * Entity type: Module
       */
      ID: number;
      Name: string;
      Code: string;
      IsActive: boolean;
      Data?: any;
    }

  }

  /**
   * Gives a list of all roles that are allowed to see this module
   */
  export interface AdminGetModuleAllowedRoles {
    /**
     * Entity type: Module
     */
    ModuleID: number;
  }

  export interface AdminGetModuleAllowedRolesResponse {
    Roles: AdminGetModuleAllowedRolesResponse.Role[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace AdminGetModuleAllowedRolesResponse {
    export interface Role {
      /**
       * Entity type: Role
       */
      ID: number;
      Name: string;
      Code: string;
    }

  }

  /**
   * Gives a list of all modules that the given role isallowed to see
   */
  export interface AdminGetRoleAllowedModules {
    /**
     * Entity type: Role
     */
    RoleID: number;
  }

  export interface AdminGetRoleAllowedModulesResponse {
    Modules: AdminGetRoleAllowedModulesResponse.AllowedModule[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace AdminGetRoleAllowedModulesResponse {
    export interface AllowedModule {
      /**
       * Entity type: Module
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Set which roles are allowed to see the specified module. Will replace existing values.
   */
  export interface AdminUpdateModuleAllowedRoles {
    /**
     * Entity type: Module
     */
    ModuleID: number;
    Roles?: AdminUpdateModuleAllowedRoles.AllowedRole[];
  }

  export namespace AdminUpdateModuleAllowedRoles {
    export interface AllowedRole {
      /**
       * Entity type: Role
       */
      RoleID: number;
    }

  }

  /**
   * Set which modules are allowed to see by the specified role. Will replace existing values.
   */
  export interface AdminUpdateRoleAllowedModules {
    /**
     * Entity type: Role
     */
    RoleID: number;
    Modules?: AdminUpdateRoleAllowedModules.AllowedModule[];
  }

  export namespace AdminUpdateRoleAllowedModules {
    export interface AllowedModule {
      /**
       * Entity type: Module
       */
      ModuleID: number;
    }

  }

  /**
   * Get a Module by ID
   */
  export interface AdminGetModuleByID {
    /**
     * Entity type: Module
     */
    ID: number;
  }

  export interface AdminGetModuleByIDResponse {
    /**
     * Entity type: Module
     */
    ID: number;
    Name: string;
    Code: string;
    IsActive: boolean;
    Data?: any;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new Module
   */
  export interface AdminCreateModule {
    Name: string;
    Code: string;
    IsActive?: boolean;
    Data?: any;
  }

  export interface AdminCreateModuleResponse {
    /**
     * Entity type: Module
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing Module
   */
  export interface AdminUpdateModule {
    /**
     * Entity type: Module
     */
    ID: number;
    Name: string;
    Code: string;
    IsActive?: boolean;
    Data?: any;
  }

  /**
   * Delete a Module
   */
  export interface AdminDeleteModule {
    /**
     * Entity type: Module
     */
    ID: number;
  }

  /**
   * List the Modules
   */
  export interface AdminListModules {
    PageConfig?: EVA.Core.PageConfig<ListModulesFilter>;
  }

  export interface ListModulesFilter {
    Name: string;
    Code: string;
    IsActive?: boolean;
  }

  export interface AdminListModulesResponse {
    Result?: EVA.Core.PagedResult<AdminListModulesResponse.ModuleDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace AdminListModulesResponse {
    export interface ModuleDto {
      /**
       * Entity type: Module
       */
      ID: number;
      Name: string;
      Code: string;
      IsActive: boolean;
      Data?: any;
    }

  }

}
