declare module EVA.Auditing {

  export namespace Errors {
    export const enum ValidationError {
      /**
       * ShopID exceeds maximum length of 4 characters, it was: {0:string}.
       */
      ShopNumberExceedsMaximumLength = 'ValidationError:ShopNumberExceedsMaximumLength',
      /**
       * TenantCode exceeds maximum length of 5 characters, it was: {0:string}.
       */
      TenantCodeExceedsMaximumLength = 'ValidationError:TenantCodeExceedsMaximumLength',
      /**
       * Register (counter) number exceeds maximum length of 3 characters, it was: {0:string}.
       */
      RegisterNumberExceedsMaximumLength = 'ValidationError:RegisterNumberExceedsMaximumLength',
      /**
       * The Infrasec RegisterID / FiscalSystemID of the station cannot be null or whitespace.
       */
      InfrasecRegisterIDNullOrWhitespace = 'ValidationError:InfrasecRegisterIDNullOrWhitespace',
      /**
       * The Infrasec Organization Number (taken from the Company RegistrationNumber) must be exactly 10 characters, but it was {0:string}.
       */
      CompanyRegistrationNumberNotWithinRange = 'ValidationError:CompanyRegistrationNumberNotWithinRange',
      /**
       * {0:string}
       */
      ExceptionInNTHRelatedFlow = 'ValidationError:ExceptionInNTHRelatedFlow',
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * An exception occured in a flow involving the RT Server!
       * The RT Server might not be the culprit, see the full error message for more details: 
       * {0:string}
       */
      ExceptionInRTServerRelatedFlow = 'ValidationError:ExceptionInRTServerRelatedFlow',
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * The station (till) fiscal system ID must be exactly 8 characters, but it was: {0:string}
       */
      StationFiscalSystemIdOutOfRangeError = 'ValidationError:StationFiscalSystemIdOutOfRangeError',
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * RT Server did not return a valid content: 
       * {0:string}
       */
      RtServerInvalidResponseContent = 'ValidationError:RtServerInvalidResponseContent',
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * More than one station (till) with this {0:string} FiscalSystemID found in EVA! The FiscalSystemID of the newly created station will be set to null and the station cannot be used further for operations.
       */
      StationWithThisFiscalSystemIdAlreadyCreatedInEva = 'ValidationError:StationWithThisFiscalSystemIdAlreadyCreatedInEva'
    }

    export const enum InfrasecErrors {
      /**
       * The station with ID '{0:number}' is not a point of sale
       */
      StationNotPointOfSale = 'InfrasecErrors:StationNotPointOfSale',
      /**
       * The FiscalID/RegisterID for station with ID '{0:number}' is null or whitespace.
       */
      StationFiscalIDNotSet = 'InfrasecErrors:StationFiscalIDNotSet',
      /**
       * The FiscalID/RegisterID for station with ID '{0:number}' cannot be modified (Infrasec does not support changing the Register Number of a station.
       */
      StationFiscalIDCanNeverBeChanged = 'InfrasecErrors:StationFiscalIDCanNeverBeChanged',
      /**
       * The Register (Station) with this ID: {0:string} was already enrolled. The same combination of properties [Counter Number, StoreID (ShopNumber), Chain Code (TenantCode), POS Authority Code, Authority Code] was supplied before to build this RegisterID. Note that the 'Chain' (ChainName) does not matter, only the combination of the properties above
       */
      RegisterWithThisIDAlreadyEnrolled = 'InfrasecErrors:RegisterWithThisIDAlreadyEnrolled',
      /**
       * {0:string}
       */
      EnrollmentFailed = 'InfrasecErrors:EnrollmentFailed',
      /**
       * {0:string}
       */
      ReceiptError = 'InfrasecErrors:ReceiptError',
      /**
       * Auth Error Code 189. RequestID: {0:string}. Check the Organization Number (Organization RegistrationNumber) and the RegisterID from the request, the station from where the order was paid must belong to thesame OU.
       */
      ReceiptAuthError = 'InfrasecErrors:ReceiptAuthError',
      /**
       * The Infrasec API is offline, should proceed with Offline Cash invoice fallback flow{0:string}
       */
      InfrasecCCUApiOffline = 'InfrasecErrors:InfrasecCCUApiOffline'
    }

    export const enum FiskalySignEsErrors {
      /**
       * Fiskaly is unresponsive
       */
      FiskalyUnresponsive = 'FiskalySignEsErrors:FiskalyUnresponsive',
      /**
       * Station FiscalSystemID cannot be empty. The FiscalSystemID must be set to the value of the SignES associated ClientID
       */
      MissingFiscalSystemID = 'FiskalySignEsErrors:MissingFiscalSystemID',
      /**
       * Station must exist. The FiscalSystemID of the station must be set to the value of the SignES associated ClientID
       */
      MissingStation = 'FiskalySignEsErrors:MissingStation',
      /**
       * Setting EVA.Framework.Configuration.Setting`1[System.String] cannot be empty. The setting must be a UUIDv4 set to the value of the SignES associated ClientID
       */
      MissingWebShopClientID = 'FiskalySignEsErrors:MissingWebShopClientID',
      /**
       * The original invoice ID cannot be empty. The ReferenceInvoiceID should contain the associated Fiskaly SignES Invoice ID as a UUIDv4 format.
       */
      MissingReferenceInvoiceID = 'FiskalySignEsErrors:MissingReferenceInvoiceID'
    }

    export const enum FinancialPeriodAudit {
      /**
       * The 'From' {0:string} is greater than the 'To' {1:string} date
       */
      FromGreaterThanTo = 'FinancialPeriodAudit:FromGreaterThanTo',
      /**
       * The maximum period is {0:number} days
       */
      PeriodExceeded = 'FinancialPeriodAudit:PeriodExceeded',
      /**
       * Got a 'From' date but missing the 'To' date
       */
      MissingToDate = 'FinancialPeriodAudit:MissingToDate',
      /**
       * 'From' is in the future
       */
      DateIsInTheFuture = 'FinancialPeriodAudit:DateIsInTheFuture'
    }

    export const enum GenerateFinancialPeriodAuditExportService {
      /**
       * The specified audit file type is not handled.
       */
      AuditTypeCannotBeProcessed = 'GenerateFinancialPeriodAuditExportService:AuditTypeCannotBeProcessed'
    }

    export namespace Auditing {
      export const enum ReturnReport {
        /**
         * The report is empty.
         */
        InvalidReport = 'Auditing:ReturnReport:InvalidReport'
      }

      export namespace Romania {
        export const enum FiscalPrinter {
          /**
           * Invalid value for cashier code '{0:string}'
           */
          InvalidCashierCode = 'Auditing:Romania:FiscalPrinter:InvalidCashierCode',
          /**
           * The device '{0:number}' must have the OrganizationUnitID set
           */
          MissingDeviceOrganizationUnit = 'Auditing:Romania:FiscalPrinter:MissingDeviceOrganizationUnit',
          /**
           * Cannot find organization unit with ID {0:number}
           */
          MissingOrganizationUnit = 'Auditing:Romania:FiscalPrinter:MissingOrganizationUnit',
          /**
           * Cannot find country ID for organization unit with ID {0:number}
           */
          MissingCountryID = 'Auditing:Romania:FiscalPrinter:MissingCountryID',
          /**
           * Sale receipts require a fiscal receipt number
           */
          MissingFiscalReceipt = 'Auditing:Romania:FiscalPrinter:MissingFiscalReceipt',
          /**
           * Missing tax information, line '{0:string}'
           */
          MissingTaxInfo = 'Auditing:Romania:FiscalPrinter:MissingTaxInfo',
          /**
           * Incorrect VAT rate on the product, please check the configuration of the tax rates. Invalid value for category '{0:string}'
           */
          InvalidTaxCategory = 'Auditing:Romania:FiscalPrinter:InvalidTaxCategory',
          /**
           * Command '{0:string}', execution failure, error: {1:string}
           */
          FiscalPrinterExecutionError = 'Auditing:Romania:FiscalPrinter:FiscalPrinterExecutionError',
          /**
           * Did not expect to print an invoice {0:number} with a fiscal receipt number set {1:string}
           */
          FiscalReceiptNumberIndicatesReprint = 'Auditing:Romania:FiscalPrinter:FiscalReceiptNumberIndicatesReprint'
        }

      }

      export namespace Italy {
        export const enum SDI {
          /**
           * Invoice {0:number} does not have a customer
           */
          MissingCustomer = 'Auditing:Italy:SDI:MissingCustomer',
          /**
           * Invoice {0:number} does not have a billing address
           */
          MissingBillingAddress = 'Auditing:Italy:SDI:MissingBillingAddress',
          /**
           * OrganizationUnit {0:number} for Invoice {1:number} does not have a Company attached to it
           */
          MissingParentCompany = 'Auditing:Italy:SDI:MissingParentCompany',
          /**
           * Missing 'codice fiscale' to export invoice {0:number} to the SDI
           */
          MissingFiscalID = 'Auditing:Italy:SDI:MissingFiscalID',
          /**
           * OrganizationUnit {0:number} has not SDI url configured
           */
          MissingSDIServiceUrl = 'Auditing:Italy:SDI:MissingSDIServiceUrl',
          /**
           * OrganizationUnit {0:number} is missing the SDI transmitter country code
           */
          MissingSDITransmitterCountryCode = 'Auditing:Italy:SDI:MissingSDITransmitterCountryCode',
          /**
           * OrganizationUnit {0:number} is missing the SDI transmitter VAT number
           */
          MissingSDITransmitterVatNumber = 'Auditing:Italy:SDI:MissingSDITransmitterVatNumber'
        }

      }

    }

    export const enum ReprocessFinancialPeriodAuditService {
      /**
       * The specified audit cannot be reprocessed.
       */
      AuditCannotBeReprocessed = 'ReprocessFinancialPeriodAuditService:AuditCannotBeReprocessed'
    }

    export const enum SAFTPaymentCreationValidator {
      /**
       * The payment surpasses the maximum amount of {amount:number} {currency:string} in cash.
       */
      PaymentForbidden = 'SAFTPaymentCreationValidator:PaymentForbidden'
    }

    export const enum SAFT {
      /**
       * InvalidClientConfiguration
       */
      Series = 'SAFT:Series',
      /**
       * Failed to list series (one or more settings are invalid)
       */
      InvalidConfiguration = 'SAFT:InvalidConfiguration'
    }

    export const enum RomaniaPaymentCreationValidator {
      /**
       * You cannot create a sale containing gift cards and normal products.
       */
      MixedOrder = 'RomaniaPaymentCreationValidator:MixedOrder',
      /**
       * No thermal printer configured on the station.
       */
      NoThermalPrinter = 'RomaniaPaymentCreationValidator:NoThermalPrinter',
      /**
       * No fiscal thermal printer configured on the station.
       */
      NoFiscalThermalPrinter = 'RomaniaPaymentCreationValidator:NoFiscalThermalPrinter',
      /**
       * Thermal printer not reachable.
       */
      ThermalPrinterNotReachable = 'RomaniaPaymentCreationValidator:ThermalPrinterNotReachable',
      /**
       * Printer memory is full.
       */
      ThermalPrinterMemory = 'RomaniaPaymentCreationValidator:ThermalPrinterMemory',
      /**
       * Unable to complete the transaction. The financial period is open for longer than 24 hours. Please close the financial period and re-open the period in order to continue.
       */
      ThermalPrinterNeedToPrintZReport = 'RomaniaPaymentCreationValidator:ThermalPrinterNeedToPrintZReport',
      /**
       * Unable to complete the transaction. Printer in error state: {0:string}
       */
      ThermalPrinterError = 'RomaniaPaymentCreationValidator:ThermalPrinterError'
    }

    export const enum InvoiceValidationError {
      /**
       * {0:string}
       */
      KSeFRejectedTheInvoice = 'InvoiceValidationError:KSeFRejectedTheInvoice',
      /**
       * {0:string}
       */
      KSeFRejectedInvoice = 'InvoiceValidationError:KSeFRejectedInvoice',
      /**
       * Validation of the EVA created KSeF Invoice XML against the XSD Schema failed! At least one validation error was encountered:
       * {0:string}
       */
      InvoiceXsdValidationError = 'InvoiceValidationError:InvoiceXsdValidationError'
    }

    export const enum PolandFiscalPosPrinter {
      /**
       * Station required on the fiscal printer device
       */
      StationNotSetOnDevice = 'PolandFiscalPosPrinter:StationNotSetOnDevice',
      /**
       * Fiscal system ID required on the station
       */
      NoFiscalSystemID = 'PolandFiscalPosPrinter:NoFiscalSystemID'
    }

    export const enum PolandPaymentCreationValidator {
      /**
       * You cannot create a sale containing gift cards and normal products.
       */
      MixedOrder = 'PolandPaymentCreationValidator:MixedOrder',
      /**
       * No thermal printer configured on the station.
       */
      NoThermalPrinter = 'PolandPaymentCreationValidator:NoThermalPrinter',
      /**
       * No fiscal thermal printer configured on the station.
       */
      NoFiscalThermalPrinter = 'PolandPaymentCreationValidator:NoFiscalThermalPrinter',
      /**
       * Thermal printer not reachable.
       */
      ThermalPrinterNotReachable = 'PolandPaymentCreationValidator:ThermalPrinterNotReachable',
      /**
       * Thermal Printer is in autonomous mode, please exit this mode or restart your printer to continue.
       */
      ThermalPrinterAutonomous = 'PolandPaymentCreationValidator:ThermalPrinterAutonomous',
      /**
       * Check printer paper.
       */
      ThermalPrinterPaper = 'PolandPaymentCreationValidator:ThermalPrinterPaper',
      /**
       * Printer memory is full.
       */
      ThermalPrinterMemory = 'PolandPaymentCreationValidator:ThermalPrinterMemory'
    }

    export const enum NF525PaymentCreationValidator {
      /**
       * The payment surpasses the maximum amount of {amount:number} {currency:string} in cash.
       */
      PaymentForbidden = 'NF525PaymentCreationValidator:PaymentForbidden',
      /**
       * The maximum duration of {0:number} day(s) in the offline financial period has been exceeded. Please close and reopen the period.
       */
      OfflinePeriodDurationExceeded = 'NF525PaymentCreationValidator:OfflinePeriodDurationExceeded'
    }

    export const enum SDiCoopInvoiceMiddlewareErrors {
      /**
       * The request message was empty
       */
      EmptyRequestMessage = 'SDiCoopInvoiceMiddlewareErrors:EmptyRequestMessage',
      /**
       * The request message did not contain a body.
       */
      EmptyRequestMessageBody = 'SDiCoopInvoiceMiddlewareErrors:EmptyRequestMessageBody',
      /**
       * The message body type '{0:string}' is unknown.
       */
      UnknownRequestMessageBodyType = 'SDiCoopInvoiceMiddlewareErrors:UnknownRequestMessageBodyType',
      /**
       * The SDI ID '{0:string}' is unknown.
       */
      UnknownInvoiceIdentificationNumber = 'SDiCoopInvoiceMiddlewareErrors:UnknownInvoiceIdentificationNumber'
    }

    export const enum RtServerResponse {
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * RT Server did not return a successful response: 
       * {0:string}
       */
      ResponseError = 'RtServerResponse:ResponseError',
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * Received 'Server Closure To Do' response from RT Server. This should indicate that there is at least one station which is open. All stations have to be closed if you want to remove a station from the RT Server. 
       * {0:string}
       */
      ServerClosureToDo = 'RtServerResponse:ServerClosureToDo'
    }

    export const enum ReceiptError {
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * No matching RT Server Vat Rate found for this Product. Check the Tax Rates from the Organization Unit that the product belongs to. Details: {0:string}
       */
      RtServerVatRateMismatch = 'ReceiptError:RtServerVatRateMismatch'
    }

    export const enum ItalyIntegrationErrors {
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * The station (till) fiscal system ID must be exactly 8 characters, but it was null. Station details: {0:string}
       */
      FiscalSystemIDCannotBeNull = 'ItalyIntegrationErrors:FiscalSystemIDCannotBeNull'
    }

    export const enum LotteryNumber {
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * Expected lottery number to be exactly 8 characters, but was: {0:string}
       */
      InvalidLotteryNumberProvided = 'LotteryNumber:InvalidLotteryNumberProvided'
    }

    export const enum LotteryErrors {
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW 
       * Expected lottery number to be exactly 8 characters, but was: {0:string}
       */
      InvalidLotteryNumberProvided = 'LotteryErrors:InvalidLotteryNumberProvided',
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW
       * {0:string}
       */
      InvalidInstantLotteryComputedVerificationCode = 'LotteryErrors:InvalidInstantLotteryComputedVerificationCode',
      /**
       * EXCEPTION_IN_RT_RELATED_FLOW
       * {0:string}
       */
      RtServerNotInActiveState = 'LotteryErrors:RtServerNotInActiveState'
    }

    export const enum GermanPaymentCreationValidator {
      /**
       * The station {0:number} is not a fiscal station.
       */
      NotAFicalStation = 'GermanPaymentCreationValidator:NotAFicalStation',
      /**
       * The station {0:number} is not a fiscal station.
       */
      NotSyncedToTheExternalSystem = 'GermanPaymentCreationValidator:NotSyncedToTheExternalSystem'
    }

    export const enum GermanyIntegrationErrors {
      /**
       * The station fiscal system ID cannot be null. Station details: {0:string}
       */
      FiscalSystemIDCannotBeNull = 'GermanyIntegrationErrors:FiscalSystemIDCannotBeNull',
      /**
       * The station fiscal system ID cannot be changed. Station details: {0:string}
       */
      FiscalSystemIDOfAStationCannotBeChanged = 'GermanyIntegrationErrors:FiscalSystemIDOfAStationCannotBeChanged'
    }

    export const enum GenericPaymentCreationValidator {
      /**
       * The payment surpasses the maximum amount of {amount:number} {curency:string} in cash.
       */
      PaymentForbidden = 'GenericPaymentCreationValidator:PaymentForbidden'
    }

    export const enum FiscalPrinting {
      /**
       * Printing {typeOfReport:string} failed on station {stationID:number}
       */
      FailureToPrintPeriodicReport = 'FiscalPrinting:FailureToPrintPeriodicReport'
    }

    export const enum AuditingPaymentCreationValidator {
      /**
       * Additional fiscal configuration is required
       */
      FiscalConfigurationRequired = 'AuditingPaymentCreationValidator:FiscalConfigurationRequired'
    }

    export const enum AustriaFiskalyErrors {
      /**
       * Fiskaly is unresponsive
       */
      FiskalyUnresponsive = 'AustriaFiskalyErrors:FiskalyUnresponsive'
    }

    export const enum AustriaPaymentCreationValidator {
      /**
       * The station {0:number} is not a fiscal station.
       */
      NotAFicalStation = 'AustriaPaymentCreationValidator:NotAFicalStation'
    }

  }


  export interface PrintControlDocument {
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  /**
   * Used to get the required fields for the company.
   */
  export interface GetAuditingFieldsForCompany {
    /**
     * Entity type: Company
     */
    CompanyID: number;
  }

  export interface GetAuditingFieldsForCompanyResponse {
    Fields: { [key: string]: GetAuditingFieldsForCompanyResponse.FieldModel };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAuditingFieldsForCompanyResponse {
    export interface FieldModel {
      Label: string;
      Value: string;
      Type: EVA.Core.AuditingFieldDataType;
      MinLength: number;
      MaxLength: number;
    }

  }

  /**
   * Used to set the required fields for the company.
   */
  export interface SetAuditingFieldsForCompany {
    /**
     * Entity type: Company
     */
    CompanyID: number;
    AuditingFields: { [key: string]: string };
  }

  export interface CorrectInvoiceInformation {
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    FiscalID?: string;
    RemoveFiscalID?: boolean;
    VatNumber?: string;
    RemoveVatNumber?: boolean;
    RemoveCompany?: boolean;
    CompanyName?: string;
    FirstName?: string;
    LastName?: string;
    RecipientCode?: string;
    RecipientEmail?: string;
    PublicProjectCode?: string;
    TenderIdentificationCode?: string;
    BillingAddressAddress1?: string;
    BillingAddressNumber?: string;
    BillingAddressZipCode?: string;
    BillingAddressCity?: string;
    BillingAddressRegion?: string;
    BillingAddressCountryID?: string;
  }

  /**
   * Creates financial period audits based on the given organization unit set.
   */
  export interface CreateFinancialPeriodAuditForOrganizationUnitSet {
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Start of the date range to create an audit for
     */
    From: string;
    /**
     * End of the date range to create an audit for
     */
    To: string;
  }

  export interface CreateFinancialPeriodAuditForOrganizationUnitSetResponse {
    /**
     * Entity type: FinancialPeriodAudit
     */
    IDs: number[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new audit collection of all currently open `Financial Period`s for the given `OrganizationUnitID`. The `OrganizationUnitID` act as root
   * selector and will cascade for all 'child' organizations as well.
   * 
   * Supply a `To` parameter to only collect `Financial Period`s from before that moment for audit.
   * Supply the `Type` to specify which or what kind of output you want to have. If none is passed the first available one is used. See the
   * `GetFinancialPeriodAuditOutputTypes` service to fetch all of the available types.
   */
  export interface CreateFinancialPeriodAudit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Start of the date range to create an audit for
     */
    From?: string;
    /**
     * End of the date range to create an audit for
     */
    To?: string;
    /**
     * Use `GetFinancialPeriodAuditOutputTypes` to fetch the available types
     */
    Type?: string;
    IsPreview?: boolean;
  }

  export interface CreateFinancialPeriodAuditResponse {
    /**
     * Entity type: FinancialPeriodAudit
     */
    IDs: number[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Generates and prints the periodic monthly report that can be printed on-demand at the given station over the given period.
   * 
   * - `StationID` is where the periodic report should be printed
   * - `Year` is the year of the period
   * - `Month` is the month of the period (1=january, 12=december)
   */
  export interface CreateMonthlyPeriodicReport {
    /**
     * Entity type: Station
     */
    StationID: number;
    Year: number;
    Month: number;
  }

  /**
   * Helper service to decommission a station
   */
  export interface DecommissionStation {
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * Generates the audit file for the given audit and optionally its type, if there's one or more to choose from. Get these via the `GetFinancialPeriodAuditOutputTypes`
   */
  export interface GenerateFinancialPeriodAuditExport {
    /**
     * Financial period audit to generate the document for
     * Entity type: FinancialPeriodAudit
     */
    FinancialPeriodAuditID?: number;
    /**
     * Type of audit document to generate
     */
    Type?: string;
  }

  /**
   * Used to get the auditing summary.
   */
  export interface GetAuditingSummary {
  }

  export interface GetAuditingSummaryResponse {
    Summary: { [key: string]: string };
    Documents: { [key: string]: string };
    ControlDocuments: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Service for retrieving a list of blobs by the given `FinancialPeriodAudit` ID
   */
  export interface GetFinancialPeriodAuditBlobs {
    /**
     * Entity type: FinancialPeriodAudit
     */
    ID: number;
  }

  export interface GetFinancialPeriodAuditBlobsResponse {
    Items: GetFinancialPeriodAuditBlobsResponse.BlobItem[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetFinancialPeriodAuditBlobsResponse {
    export interface BlobItem {
      BlobID: string;
      BlobUrl: string;
      Type?: string;
    }

  }

  /**
   * Returns all of the available outputs that are available for the financial period audits for the given organization unit
   */
  export interface GetFinancialPeriodAuditOutputTypes {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetFinancialPeriodAuditOutputTypesResponse {
    Types: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Used to get the required fields for the organization unit.
   */
  export interface GetAuditingFieldsForOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetAuditingFieldsForOrganizationUnitResponse {
    Fields: { [key: string]: GetAuditingFieldsForOrganizationUnitResponse.FieldModel };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAuditingFieldsForOrganizationUnitResponse {
    export interface FieldModel {
      Label: string;
      Value: string;
      Type: EVA.Core.AuditingFieldDataType;
      MinLength: number;
      MaxLength: number;
    }

  }

  /**
   * Used to set the required fields for the organization unit.
   */
  export interface SetAuditingFieldsForOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    AuditingFields: { [key: string]: string };
  }

  /**
   * Generate and print a terminal report. This is a report for a terminal, p/a a X- or Z-report. It is required to supply the StationID where to print the report.
   */
  export interface PrintTerminalReport {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    DocumentID?: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  export interface PrintTerminalReportResponse {
    CopyReceiptsPrinted: number;
    TotalCopyReceiptsAmount: number;
    ReceiptsPrinted: number;
    CashDrawerOpenings: number;
    ReturnCount: number;
    ReturnAmount: number;
    DiscountCount: number;
    DiscountAmount: number;
    Change: number;
    OpeningAmount: number;
    ClosingAmount: number;
    CashDepositsTotalAmount: number;
    Payments: PrintTerminalReportResponse.Payment[];
    CreditTaxes: PrintTerminalReportResponse.Tax[];
    DebitTaxes: PrintTerminalReportResponse.Tax[];
    Taxes: PrintTerminalReportResponse.Tax[];
    ProductGroups: PrintTerminalReportResponse.ProductGroup[];
    TotalPayments: number;
    TotalTaxes: number;
    TotalAmount: number;
    PaymentsPerUser: PrintTerminalReportResponse.PaymentPerUser[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace PrintTerminalReportResponse {
    export interface Payment {
      Description: string;
      Amount: number;
    }

    export interface PaymentPerUser {
      /**
       * Entity type: User
       */
      UserID: number;
      EmployeeNumber: string;
      Description: string;
      Amount: number;
    }

    export interface Tax {
      Code: string;
      Rate: number;
      Amount: number;
    }

    export interface ProductGroup {
      Code: string;
      Amount: number;
      Count: number;
    }

  }

  /**
   * Generates a monthly report on returned orders.
   */
  export interface ProduceReturnReport {
    /**
     * Min: 1
     */
    Year: number;
    Month: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  /**
   * Retry processing of a FinancialPeriodAudit, in case it is stuck or had an issue processing earlier.
   */
  export interface ReprocessFinancialPeriodAudit {
    /**
     * Entity type: FinancialPeriodAudit
     */
    ID: number;
  }

  /**
   * Validates the given Fiscal ID by the current organization unit auditing provider or by providing a country in the request
   */
  export interface ValidateFiscalID {
    FiscalID: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  export interface ValidateFiscalIDResponse {
    Success: EVA.Core.AuditingFiscalIDValidationResult;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListSeries {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Identifier?: string;
    Type?: string;
    From?: string;
    To?: string;
  }

  export interface ListSeriesResponse {
    Result: ListSeriesResponse.Model[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListSeriesResponse {
    export interface Model {
      Identifier: string;
      SerialType: string;
      Classification: string;
      DocumentType: string;
      StartSequence: number;
      StartDate: string;
      ProcessingMediumCode: string;
      CertificateNumber: string;
      ValidationCode: string;
      RegistrationDate: string;
      State: string;
      StatusChangeReasonCode: string;
      LastChangeDate: string;
      TaxRegistrationNumber: string;
    }

  }

  export interface CancelSeries {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    Identifier?: string;
    Classification?: string;
    DocumentType?: string;
    ValidationCode?: string;
  }

  export interface CancelSeriesResponse {
    Success: boolean;
    Code: string;
    Message: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Utility service to easily set all of the settings required for enabling ATCUD in Portugal (SAFT)
   */
  export interface SetSeriesSettings {
    /**
     * Organization unit to which the settings should be applied
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Certificate that was handed out by the AT after completing the CSR steps (.cer file)
     */
    SigningCertificate?: string;
    /**
     * Certificate that was generated by combining the private key with the key received from AT
     */
    ClientCertificate?: string;
    /**
     * Recommended, password to the client certificate
     */
    ClientCertificatePassword?: string;
    /**
     * Username of the web service user created in the AT portal with at least WSE rights
     */
    Username?: string;
    /**
     * Password of the web service user
     */
    Password?: string;
    /**
     * Endpoint to connect to when registering series. Use https://servicos.portaldasfinancas.gov.pt:722/SeriesWSService for TEST and https://servicos.portaldasfinancas.gov.pt:422/SeriesWSService for PRODUCTION
     */
    Endpoint?: string;
  }

}
