declare module EVA.Authentication.OpenID {

  /**
   * Get all available openid provider configurations
   */
  export interface GetAvailableOpenIDConfigurations {
  }

  export interface GetAvailableOpenIDConfigurationsResponse {
    Providers: AvailableOpenIDConfiguration[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface AvailableOpenIDConfiguration {
    /**
     * Entity type: Provider
     */
    ID: number;
    Name?: string;
    Primary: boolean;
    BaseUrl: string;
    ClientID: string;
    UserType: EVA.Core.UserTypes;
  }

  /**
   * Get all info for a Azure OpenID Provider
   */
  export interface GetOpenIDProviderByID {
    /**
     * Entity type: Provider
     */
    ID: number;
  }

  export interface GetOpenIDProviderByIDResponse {
    /**
     * Entity type: Provider
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    Enabled?: boolean;
    Primary?: boolean;
    BaseUrl?: string;
    ClientID?: string;
    FirstNameClaim?: string;
    LastNameClaim?: string;
    EmailAddressClaim?: string;
    NicknameClaim?: string;
    UsernameClaim?: string;
    LanguageClaim?: string;
    CountryClaim?: string;
    UserType?: EVA.Core.UserTypes;
    CreateUsers?: boolean;
    DisallowOtherMethods?: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new OpenID Provider Registration
   */
  export interface CreateOpenIDProvider {
    BackendID?: string;
    Name?: string;
    Enabled?: boolean;
    BaseUrl?: string;
    ClientID?: string;
    FirstNameClaim?: string;
    LastNameClaim?: string;
    /**
     * To identify the user an EmailAddress or Nickname is required, so at least one of EmailAddressClaim/NicknameClaim/UsernameClaim should be set.
     */
    EmailAddressClaim?: string;
    /**
     * To identify the user an EmailAddress or Nickname is required, so at least one of EmailAddressClaim/NicknameClaim/UsernameClaim should be set.
     */
    NicknameClaim?: string;
    /**
     * To identify the user an EmailAddress or Nickname is required, so at least one of EmailAddressClaim/NicknameClaim/UsernameClaim should be set.
     */
    UsernameClaim?: string;
    LanguageClaim?: string;
    CountryClaim?: string;
    UserType?: EVA.Core.UserTypes;
    CreateUsers?: boolean;
    DisallowOtherMethods?: boolean;
  }

  export interface CreateOpenIDProviderResponse {
    /**
     * Entity type: Provider
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing OpenID Provider Registration
   */
  export interface UpdateOpenIDProvider {
    /**
     * Entity type: Provider
     */
    ID: number;
    BackendID?: string;
    Name?: string;
    Enabled?: boolean;
    BaseUrl?: string;
    ClientID?: string;
    FirstNameClaim?: string;
    LastNameClaim?: string;
    /**
     * To identify the user an EmailAddress or Nickname is required, so at least one of EmailAddressClaim/NicknameClaim/UsernameClaim should be set.
     */
    EmailAddressClaim?: string;
    /**
     * To identify the user an EmailAddress or Nickname is required, so at least one of EmailAddressClaim/NicknameClaim/UsernameClaim should be set.
     */
    NicknameClaim?: string;
    /**
     * To identify the user an EmailAddress or Nickname is required, so at least one of EmailAddressClaim/NicknameClaim/UsernameClaim should be set.
     */
    UsernameClaim?: string;
    LanguageClaim?: string;
    CountryClaim?: string;
    UserType?: EVA.Core.UserTypes;
    CreateUsers?: boolean;
    DisallowOtherMethods?: boolean;
  }

  /**
   * Delete an existing OpenID Provider Registration
   */
  export interface DeleteOpenIDProvider {
    /**
     * Entity type: Provider
     */
    ID: number;
  }

  /**
   * List OpenID Provider Registrations
   */
  export interface ListOpenIDProviders {
    PageConfig?: EVA.Core.PageConfig<ListOpenIDProvidersFilter>;
  }

  export interface ListOpenIDProvidersFilter {
    Name?: string;
    Enabled?: boolean;
    Primary?: boolean;
    BaseUrl?: string;
    ClientID?: string;
  }

  export interface ListOpenIDProvidersResponse {
    Result?: EVA.Core.PagedResult<OpenIDProviderDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface OpenIDProviderDto {
    /**
     * Entity type: Provider
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    Enabled?: boolean;
    Primary?: boolean;
    BaseUrl?: string;
    ClientID?: string;
    FirstNameClaim?: string;
    LastNameClaim?: string;
    /**
     * To identify the user an EmailAddress or Nickname is required, so at least one of EmailAddressClaim/NicknameClaim/UsernameClaim should be set.
     */
    EmailAddressClaim?: string;
    /**
     * To identify the user an EmailAddress or Nickname is required, so at least one of EmailAddressClaim/NicknameClaim/UsernameClaim should be set.
     */
    NicknameClaim?: string;
    /**
     * To identify the user an EmailAddress or Nickname is required, so at least one of EmailAddressClaim/NicknameClaim/UsernameClaim should be set.
     */
    UsernameClaim?: string;
    LanguageClaim?: string;
    CountryClaim?: string;
    UserType?: EVA.Core.UserTypes;
    CreateUsers?: boolean;
    DisallowOtherMethods?: boolean;
  }

  /**
   * Set the provider that will be the default available when logging in, non-primary providers have to be requested.
   */
  export interface SetPrimaryOpenIDProvider {
    /**
     * Entity type: Provider
     */
    ID: number;
  }

}
