declare module EVA.BI {

  /**
   * Returns data that can be displayed as graphs in the frontend. This data is provided by an external system. When no external system is configured, mockdata will be returned.
   */
  export interface GetCustomerCharts {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  export interface GetChartsResponse {
    Elements: ChartElement[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ChartElement {
    Type: ChartElementTypes;
    Title: string;
    Description: string;
    DataSet: any;
  }

  export const enum ChartElementTypes {
    Text = 1,
    Bar = 2,
    Pie = 3,
    Table = 4
  }

  export interface GetDailyStats {
  }

  export interface GetDailyStatsResponse {
    Orders: number;
    Exported: number;
    Shipped: number;
    Products: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetOrderCounts {
    StartDate?: string;
    EndDate?: string;
  }

  export interface GetOrderCountsResponse {
    Days: GetOrderCountsResponse.DayWithCount[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetOrderCountsResponse {
    export interface DayWithCount {
      Day: string;
      Count: number;
    }

  }

  export interface GetPopularProducts {
    StartDate?: string;
    EndDate?: string;
    Amount?: number;
  }

  export interface GetPopularProductsResponse {
    Products: GetPopularProductsResponse.ProductWithCount[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPopularProductsResponse {
    export interface ProductWithCount {
      /**
       * Entity type: Product
       */
      ID: number;
      Name: string;
      Count: number;
    }

  }

}
