declare module EVA.Cloe {

  export interface CloeGetAllProducts {
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
    Filters?: { [key: string]: EVA.Core.FilterModel };
    SearchStrategyCode?: string;
  }

  export interface CloeGetAllProductsResponse {
    Products: any[];
    Total: number;
    Error?: EVA.Core.ServiceError;
    Metadata: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Generates a tracking identifier for `Cloe`. If available, the `SessionID` can be provided.
   * The generated tracking identifier should be used to call the `CloeTrackEvent` service.
   */
  export interface CloeGetTrackingID {
    SessionID?: string;
  }

  export interface CloeGetTrackingIDResponse {
    TrackingID: string;
    Error?: EVA.Core.ServiceError;
    Metadata: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Track events for `Cloe`. All fields are mandatory. The `TrackingID` can be generated with the `CloeGetTrackingID` service.
   */
  export interface CloeTrackEvent {
    TrackingID: string;
    EventID: string;
    Data: { [key: string]: any };
  }

  /**
   * Accepts a list of product/Fingerprint pairs and updates the cloe_fingerprint property on the given products.
   * 
   * To identify the product, either specify ProductID or ProductIdentifier. When ProductIdentifier is used, it's matched to an EVA product
   * using the value of the `Cloe:ProductIdentifierField` setting (for example `backend_id`).
   * 
   *  The service returns immediately and processing happens in the background.
   */
  export interface CloeUpdateProductFingerprints {
    Fingerprints: ProductFingerprint[];
  }

  export interface ProductFingerprint {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    ProductIdentifier?: string;
    Fingerprint: string;
  }

}
