declare module EVA.Cookbook {

  export interface CreateAccountingRecipe {
    Recipe?: string;
    IsActive?: boolean;
    Name?: string;
  }

  export interface CreateAccountingRecipeResponse {
    /**
     * Entity type: AccountingRecipe
     */
    ID: number;
    HasErrors: boolean;
    Errors: ParseRecipeError[];
    CompletionSuggestions: CompletionSuggestion[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ParseRecipeError {
    Type: ParsingErrorType;
    Message: string;
    SourceLine: string;
    SourceLineNumber: number;
    SourceColumn: number;
  }

  export const enum ParsingErrorType {
    InvalidSyntax = 0,
    UnknownVariable = 1,
    InvalidComparison = 2,
    InvalidOperator = 3,
    MissingCredit = 4,
    MissingDebit = 5,
    AmountMustBeNumerical = 6,
    InvalidBookOnOrganizationUnit = 7,
    InvalidEventType = 8,
    DuplicateVariable = 9,
    InvalidVariable = 10
  }

  export interface CompletionSuggestion {
    Label: string;
    Code: string;
    Type: CompletionSuggestionType;
    Documentation: string;
  }

  export const enum CompletionSuggestionType {
    Variable = 0,
    Keyword = 1,
    Enum = 2,
    Account = 3
  }

  export interface UpdateAccountingRecipe {
    /**
     * Entity type: AccountingRecipe
     */
    ID: number;
    Recipe?: string;
    IsActive?: boolean;
    Name?: string;
  }

  export interface UpdateAccountingRecipeResponse {
    HasErrors: boolean;
    Errors: ParseRecipeError[];
    CompletionSuggestions: CompletionSuggestion[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface DeleteAccountingRecipe {
    /**
     * Entity type: AccountingRecipe
     */
    ID: number;
  }

  export interface ListAccountingRecipes {
    IsActive?: boolean;
    Type?: EVA.Core.FinancialEventTypes;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListAccountingRecipesResponse {
    Result?: EVA.Core.PagedResult<AccountingRecipeDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface AccountingRecipeDto {
    /**
     * Entity type: AccountingRecipe
     */
    ID: number;
    Recipe: string;
    Name: string;
    EventType: EVA.Core.FinancialEventTypes;
    IsActive: boolean;
  }

  export interface GetAccountingRecipe {
    /**
     * Entity type: AccountingRecipe
     */
    ID: number;
  }

  export interface GetAccountingRecipeResponse {
    Recipe: AccountingRecipeDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ParseAccountingRecipe {
    Recipe: string;
    CursorPosition?: number;
  }

  export interface ParseRecipeResponse {
    HasErrors: boolean;
    Errors: ParseRecipeError[];
    CompletionSuggestions: CompletionSuggestion[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface PreviewAccountingRecipe {
    Recipes?: AccountingRecipeToPreview[];
    /**
     * Entity type: FinancialEvent
     */
    FinancialEventIDs?: number[];
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Type?: EVA.Core.FinancialEventTypes;
    RecipeType: AccountingRecipeToPreviewType;
  }

  export interface AccountingRecipeToPreview {
    /**
     * Entity type: AccountingRecipe
     */
    ID?: number;
    Recipe: string;
  }

  export const enum AccountingRecipeToPreviewType {
    V1 = 0,
    V2 = 1
  }

  export interface PreviewAccountingRecipeResponse {
    Warnings: string[];
    BookingPreviewSummaries: AccountBookingPreview[];
    BookingPreviewDetails: AccountBookingDetails[];
    HasErrors: boolean;
    Errors: ParseRecipeError[];
    CompletionSuggestions: CompletionSuggestion[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface AccountBookingPreview {
    Date: string;
    /**
     * Entity type: OrganizationUnit
     */
    BookedOnOrganizationUnitID: number;
    BookedOnOrganizationUnitName: string;
    /**
     * Entity type: Account
     */
    BookedOnAccountID: number;
    BookedOnAccountNumber: string;
    BookedOnAccountName: string;
    Amount: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Offset1: string;
    Offset2: string;
    Offset3: string;
    Offset4: string;
    Offset5: string;
    Offset6: string;
    Offset7: string;
    Offset8: string;
    Offset9: string;
  }

  export interface AccountBookingDetails {
    Amount: number;
    /**
     * Entity type: OrganizationUnit
     */
    BookedOnOrganizationUnitID: number;
    BookedOnOrganizationUnitName: string;
    /**
     * Entity type: Account
     */
    BookedOnAccountID: number;
    BookedOnAccountNumber: string;
    BookedOnAccountName: string;
    /**
     * Entity type: AccountingRecipe
     */
    AccountingRecipeID: number;
    Date: string;
    Description: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Offset1: string;
    Offset2: string;
    Offset3: string;
    Offset4: string;
    Offset5: string;
    Offset6: string;
    Offset7: string;
    Offset8: string;
    Offset9: string;
  }

  export interface GetAccountingRecipeHistory {
    /**
     * Entity type: AccountingRecipe
     */
    ID: number;
  }

  export interface GetAccountingRecipeHistoryResponse {
    Entries: GetAccountingRecipeHistoryResponse.AccountingRecipeEntry[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAccountingRecipeHistoryResponse {
    export interface AccountingRecipeEntry {
      /**
       * Entity type: AccountingRecipe
       */
      ID: number;
      Recipe: string;
      Name: string;
      Revision: number;
      EventType: EVA.Core.FinancialEventTypes;
      IsActive: boolean;
      ModifiedByEmail?: string;
      /**
       * Entity type: User
       */
      ModifiedByUserID?: number;
      ModifiedAt?: string;
      ExistedFrom: string;
      ExistedTill: string;
    }

  }

  /**
   * Compares V1 and V2 scripts for a given set of events. This is a temporary service to aid in the migration of V1 scripts to V2.
   */
  export interface CookbookMigrationComparison {
    /**
     * The list of V1 scripts to use in the comparison. If not supplied, will use all active V1 scripts.
     * Entity type: AccountingRecipe
     */
    ScriptsV1?: number[];
    /**
     * The list of V2 scripts to use in the comparison. If not supplied, will use all active V2 scripts.
     * Entity type: Script
     */
    ScriptsV2?: number[];
    /**
     * List of event IDs to use in the comparison.
     * Entity type: FinancialEvent
     */
    EventIDs?: number[];
    /**
     * The start date to use in the comparison.
     */
    StartDate?: string;
    /**
     * The end date to use in the comparison.
     */
    EndDate?: string;
    /**
     * The financial period ID to use in the comparison.
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
  }

  export interface CookbookMigrationComparisonResponse {
    /**
     * The list of events that are missing in the new processing.
     */
    MissingEvents: GeneralLedgerRecord[];
    /**
     * The list of events that are unexpected in the new processing.
     */
    UnexpectedEvents: GeneralLedgerRecord[];
    /**
     * The list of events that are matching in both processings.
     */
    MatchingEvents: GeneralLedgerRecord[];
    /**
     * The list of events that are returned by the old processing.
     */
    OldEvents: GeneralLedgerRecord[];
    /**
     * The list of events that are returned by the new processing.
     */
    NewEvents: GeneralLedgerRecord[];
    /**
     * The list of warnings for the old processing.
     */
    WarningsOld: Warning[];
    /**
     * The list of warnings for the new processing.
     */
    WarningsNew: Warning[];
    /**
     * Indicates whether the event limit was reached.
     */
    ReachedEventsLimit: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GeneralLedgerRecord {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Account
     */
    AccountID: number;
    Amount: number;
    Remark: string;
    IsProcessed: boolean;
    /**
     * Entity type: StockMutation
     */
    StockMutationID?: number;
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    Reference: string;
    Offset1: string;
    Offset2: string;
    Offset3: string;
    Offset4: string;
    Offset5: string;
    Offset6: string;
    Offset7: string;
    Offset8: string;
    Offset9: string;
    TaxCodeID?: number;
    TaxAmount?: number;
    TaxRate?: number;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    CurrencyID: string;
    Source: string;
    /**
     * Entity type: FinancialEvent
     */
    FinancialEventID?: number;
    GeneratedTime?: string;
    Description: string;
    TaxBookingCode: string;
  }

  export interface Warning {
    Type: string;
    Message: string;
  }

}
