declare module EVA.Core.LiveGuard {

  export namespace Errors {
    export const enum LiveGuard {
      /**
       * You're not allowed to generate this liveguard api key.
       */
      NotAllowedToGenerateApiKey = 'LiveGuard:NotAllowedToGenerateApiKey',
      /**
       * You're not allowed to generate this liveguard secret.
       */
      NotAllowedToGenerateSecret = 'LiveGuard:NotAllowedToGenerateSecret',
      /**
       * This is a stub order for offline evapay payments, no mutations allowed.
       */
      UnableToMutateEVAPayStub = 'LiveGuard:UnableToMutateEVAPayStub'
    }

    export const enum LiveGuardCertificate {
      /**
       * The configuration is invalid.
       */
      InvalidConfiguration = 'LiveGuardCertificate:InvalidConfiguration',
      /**
       * No valid Watchtower for this request.
       */
      NoValidWatchtower = 'LiveGuardCertificate:NoValidWatchtower',
      /**
       * CSR could not be parsed.
       */
      CsrInvalid = 'LiveGuardCertificate:CsrInvalid',
      /**
       * Failed to generate the certificate.
       */
      CertificateGenerationFailed = 'LiveGuardCertificate:CertificateGenerationFailed',
      /**
       * Hostname for this Watchtower does not match the hostname in the CSR.
       */
      HostnameMismatch = 'LiveGuardCertificate:HostnameMismatch'
    }

    export const enum LiveGuardVPN {
      /**
       * The configuration is invalid.
       */
      InvalidConfiguration = 'LiveGuardVPN:InvalidConfiguration',
      /**
       * No valid Watchtower for this request.
       */
      NoValidWatchtower = 'LiveGuardVPN:NoValidWatchtower',
      /**
       * Failed to check for existing config.
       */
      ExistsCheckFailed = 'LiveGuardVPN:ExistsCheckFailed',
      /**
       * Failed to create new config.
       */
      CreateFailed = 'LiveGuardVPN:CreateFailed'
    }

    export const enum LiveGuardExecuteTask {
      /**
       * There is no current connection to the selected device.
       */
      NoConnection = 'LiveGuardExecuteTask:NoConnection'
    }

    export const enum LiveGuardLockStatus {
      /**
       * This watchtower is not connected.
       */
      NotConnected = 'LiveGuardLockStatus:NotConnected',
      /**
       * Failed to lock the device.
       */
      LockFailed = 'LiveGuardLockStatus:LockFailed',
      /**
       * Failed to unlock the device.
       */
      UnlockFailed = 'LiveGuardLockStatus:UnlockFailed'
    }

    export const enum WatchtowerExecuteCommand {
      /**
       * There is no current connection to the selected device.
       */
      NoConnection = 'WatchtowerExecuteCommand:NoConnection'
    }

    export const enum WatchtowerIO {
      /**
       * Failed to connect to the device.
       */
      ConnectionFailed = 'WatchtowerIO:ConnectionFailed'
    }

    export const enum MenderClient {
      /**
       * No registration request found for this device. (Please reboot the device and try again)
       */
      NoRegistrationRequest = 'MenderClient:NoRegistrationRequest'
    }

    export const enum LocalModeManager {
      /**
       * You can only force local mode on shops.
       */
      CanForceLocalModeOnlyOnShop = 'LocalModeManager:CanForceLocalModeOnlyOnShop'
    }

    export const enum FleetManagerErrors {
      /**
       * Device with HardwareID {hardwareID:string} already exists with ID {deviceID:number}.
       */
      DeviceAlreadyExists = 'FleetManagerErrors:DeviceAlreadyExists',
      /**
       * OrganizationUnit {organizationUnitID:number} already has a watchtower with ID {deviceID:number}.
       */
      DuplicateOrganizationUnit = 'FleetManagerErrors:DuplicateOrganizationUnit'
    }

    export const enum FleetManager {
      /**
       * Could not determine a matching FleetManager.
       */
      UnableToDetermineFleetManager = 'FleetManager:UnableToDetermineFleetManager',
      /**
       * The FleetManager {0:string} is not available.
       */
      FleetManagerNotAvailable = 'FleetManager:FleetManagerNotAvailable',
      /**
       * Device with HardwareID {hardwareID:string} was not found in FleetManager {fleetManager:string}.
       */
      DeviceNotInFleetManager = 'FleetManager:DeviceNotInFleetManager',
      /**
       * Device is not valid for this FleetManager.
       */
      InvalidDevice = 'FleetManager:InvalidDevice',
      /**
       * Device was already boarded.
       */
      DeviceAlreadyBoarded = 'FleetManager:DeviceAlreadyBoarded',
      /**
       * Error while boarding the device: {0:string}
       */
      BoardingFailed = 'FleetManager:BoardingFailed'
    }

  }


  /**
   * Get user information to sync to LiveGuard
   */
  export interface GetUserForLiveGuard {
    /**
     * Entity type: User
     */
    UserID?: number;
    EmailAddress?: string;
    NickName?: string;
  }

  export interface GetUserForLiveGuardResponse {
    EmailAddress?: string;
    Nickname?: string;
    FirstName?: string;
    LastName?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Type?: EVA.Core.UserTypes;
    BackendSystemID?: string;
    BackendID?: string;
    TimeZone?: string;
    AuthenticationData?: GetUserForLiveGuardResponse.AuthenticationDataDto[];
    Roles?: GetUserForLiveGuardResponse.RoleDto[];
    /**
     * Entity type: User
     */
    CloudIDs?: number[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetUserForLiveGuardResponse {
    export interface RoleDto {
      Name: string;
      Code: string;
      UserType: EVA.Core.UserTypes;
      GlobalID: string;
      DeactivateUserAfterInactiveDays?: number;
      Functionalities: GetUserForLiveGuardResponse.FunctionalityDto[];
    }

    export interface FunctionalityDto {
      Functionality: string;
      FunctionalityScope: EVA.Core.FunctionalityScope;
      ElevationType: EVA.Core.ElevationType;
      /**
       * @deprecated Replaced by `ElevationType`
       */
      RequiresElevation: boolean;
    }

    export interface AuthenticationDataDto {
      Type: string;
      Data: string;
      SearchKey: string;
    }

  }

  export interface LiveGuardGenerateApiKey {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface LiveGuardGenerateApiKeyResponse {
    ApiKey: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Generate a trust token that needs to be provided in the Login call to the cloud to verify local presence
   */
  export interface LiveGuardGenerateTrustToken {
  }

  export interface LiveGuardGenerateTrustTokenResponse {
    TrustToken: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Retreive current chain status for the connected watchtower
   */
  export interface LiveGuardGetAuditChains {
  }

  export interface LiveGuardGetAuditChainsResponse {
    Chains: ChainDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ChainDto {
    /**
     * Entity type: AuditChain
     */
    ID: number;
    Identifier: string;
    Discriminator: number;
    Type: EVA.Core.AuditChainTypes;
    Version: number;
    CurrentSequenceNumber: number;
    PreviousSequenceNumber?: number;
    CurrentSignature: string;
    PreviousSignature: string;
    RecentLedgers: LiveGuardPushOrder.ChainLedgerDto[];
  }

  /**
   * Service called by a liveguard instance to generate an authentication token for an authorized user on the liveguard
   * 
   * Can only be called by the liveguard user
   */
  export interface LiveGuardGetTokenForUser {
    Username: string;
    UseJwtTokens?: boolean;
  }

  export interface LiveGuardGetTokenForUserResponse {
    AuthenticationToken: string;
    RefreshToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Retreive current secret for TOTP token usage
   */
  export interface LiveGuardGetTOTPSecret {
  }

  export interface LiveGuardGetTOTPSecretResponse {
    Secret: EVA.Core.TOTPSecretWrapper;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Accepts an Order pushed from a LiveGuard instance
   */
  export interface LiveGuardPushOrder {
    LiveGuardOrderID?: string;
    DisplayID?: string;
    CreationTime?: string;
    Remark?: string;
    Customer?: LiveGuardPushOrder.CustomerDto;
    ShippingAddress?: EVA.Core.AddressDataDto;
    BillingAddress?: EVA.Core.AddressDataDto;
    PickupAddress?: EVA.Core.AddressDataDto;
    Lines?: LiveGuardPushOrder.OrderLineDto[];
    Payments?: LiveGuardPushOrder.PaymentDto[];
    IsPaid?: boolean;
    SoldBy?: string;
    Blobs?: LiveGuardPushOrder.OrderBlobDto[];
    Invoices?: LiveGuardPushOrder.InvoiceDto[];
    Chains?: ChainDto[];
    BlobID: string;
    IsShipped?: boolean;
    CustomerIdentifiers?: LiveGuardPushOrder.OrderCustomerIdentifiersDto;
  }

  export namespace LiveGuardPushOrder {
    export interface OrderCustomerIdentifiersDto {
      UserID?: string;
      UserSubscriptionID?: string;
      UserBackendID?: string;
      UserCustomID?: string;
      EmailAddress?: string;
    }

    export interface OrderBlobDto {
      /**
       * Entity type: Invoice
       */
      InvoiceID?: number;
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      Name: string;
      Type: EVA.Core.OrderBlobTypes;
      BlobID: string;
    }

    export interface InvoiceLineTaxDetailDto {
      TaxRate: number;
      TaxAmount: number;
      TaxName: string;
      TaxType: string;
      TaxSubType: string;
      TaxSubTypeName: string;
    }

    export interface InvoiceLineDiscountDetailDto {
      CloudID?: number;
      BackendID: string;
      Description: string;
      Amount: number;
    }

    export interface InvoiceLineDto {
      InvoiceData: LiveGuardPushOrder.InvoiceDataDto[];
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      TaxRate: number;
      TaxCode?: string;
      TaxRateCategory?: EVA.Core.TaxRateCategory;
      UnitCost?: number;
      UnitPrice?: number;
      Quantity: number;
      TotalAmountInvoiced: number;
      TotalAmountInvoicedInTax: number;
      DiscountAmount: number;
      DiscountDetails: LiveGuardPushOrder.InvoiceLineDiscountDetailDto[];
      TaxDetails: LiveGuardPushOrder.InvoiceLineTaxDetailDto[];
      InvoicedBy: string;
      Type: EVA.Core.InvoiceLineTypes;
      TaxAmount?: number;
      Description?: string;
      UnitPriceInTax: number;
      NetTaxAmount: number;
      TotalTaxAmount: number;
    }

    export interface InvoiceDataDto {
      Type: string;
      Data: string;
      SearchKey: string;
    }

    export interface InvoiceBlobDto {
      BackendID: string;
      OriginalName: string;
      MimeType: string;
      Category: string;
      BlobID: string;
    }

    export interface InvoiceUserDataDto {
      EmailAddress: string;
      BackendRelationID: string;
      Initials: string;
      FirstName: string;
      LastName: string;
      Gender: string;
      BillingAddress: EVA.Core.AddressDataDto;
      ShippingAddress: EVA.Core.AddressDataDto;
      Company: EVA.Core.CompanyDto;
      FiscalID: string;
      BankAccount: string;
      PhoneNumber: string;
      LanguageID: string;
      CountryID: string;
      OrganizationUnitName: string;
      OrganizationUnitBackendID: string;
      OrganizationUnitBranchNumber: string;
      OrganizationUnitCompany: EVA.Core.CompanyDto;
      OrganizationUnitCompanyVatNumber: string;
      OrganizationUnitEstablishmentNumber?: string;
      OrganizationUnitCountryID?: string;
      OrganizationUnitAddress1?: string;
      OrganizationUnitAddress2?: string;
      OrganizationUnitHouseNumber?: string;
      OrganizationUnitDistrict?: string;
      OrganizationUnitSubDistrict?: string;
      OrganizationUnitCity?: string;
      OrganizationUnitZipCode?: string;
      OrganizationUnitState?: string;
      OrganizationUnitEmailAddress?: string;
      OrganizationUnitPhoneNumber?: string;
      OrganizationUnitWebsite?: string;
      OrganizationUnitBIC?: string;
    }

    export interface InvoiceDocumentLedgerDto {
      Type: EVA.Core.InvoiceDocumentLedgerTypes;
      /**
       * Entity type: AuditChain
       */
      ChainID?: number;
      SequenceNumber: number;
      PreviousSequenceNumber?: number;
      Signature: string;
      PreviousSignature: string;
      IsSuccess: boolean;
      FailureReason: string;
      ReportContentBlobID?: string;
      InvoiceDocumentLedgerData: EVA.Core.InvoiceDocumentLedgerData;
      Format: EVA.Core.InvoiceDocumentLedgerFormats;
      ReprintNumber?: number;
      /**
       * Entity type: Device
       */
      OriginalDeviceID?: number;
    }

    export interface InvoiceDto {
      /**
       * Entity type: Invoice
       */
      ID: number;
      InvoiceNumber: string;
      InvoiceDate?: string;
      Lines: LiveGuardPushOrder.InvoiceLineDto[];
      TotalAmount: number;
      Description: string;
      Status: EVA.Core.InvoiceStatuses;
      Blobs: LiveGuardPushOrder.InvoiceBlobDto[];
      Type: EVA.Core.InvoiceTypes;
      AuditingProvider: string;
      BackendTypeID?: number;
      CurrencyID: string;
      PrintCount: number;
      FiscalID: string;
      VatNumber: string;
      InvoiceData: LiveGuardPushOrder.InvoiceDataDto[];
      UserData: LiveGuardPushOrder.InvoiceUserDataDto;
      DocumentLedger: LiveGuardPushOrder.InvoiceDocumentLedgerDto[];
      /**
       * Entity type: AuditChain
       */
      ChainID?: number;
      Signature: string;
      PreviousSignature: string;
      SequenceNumber?: number;
      /**
       * Entity type: Station
       */
      StationID?: string;
      TaxReverseCharge: boolean;
      CalculationMethod: EVA.Core.InvoiceCalculationMethod;
      BackendSystemID: string;
      BackendID: string;
      ReferencedInvoice?: string;
      ExpirationDate?: string;
    }

    export interface CustomerDto {
      /**
       * Entity type: User
       */
      LiveGuardID: number;
      FirstName: string;
      LastName: string;
      EmailAddress: string;
      PhoneNumber: string;
      LanguageID: string;
      CountryID: string;
      Gender: string;
    }

    export interface OrderLineDto {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      BackendID?: string;
      CustomID?: string;
      Amount: number;
      TaxRate: number;
      TaxRateCategory?: EVA.Core.TaxRateCategory;
      Quantity: number;
      Description: string;
      SerialNumber?: string;
      UnitPriceBeforeCorrection?: number;
      UnitPriceBeforeCorrectionInTax?: number;
      ReturnReason?: string;
      ReturnReasonType?: EVA.Core.ReturnReasonType;
      UnitPriceCorrectionReason: string;
      Discounts: LiveGuardPushOrder.OrderLineDiscount[];
      TaxDetails: LiveGuardPushOrder.OrderLineTaxDetail[];
    }

    export interface OrderLineTaxDetail {
      TaxRate: number;
      TaxAmount: number;
      TaxName: string;
      TaxType: string;
      TaxSubType: string;
      TaxSubTypeName: string;
    }

    export interface OrderLineDiscount {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      BackendID: string;
      Description: string;
      Amount: number;
      CloudID?: number;
    }

    export interface PaymentDto {
      BackendID: string;
      Method: string;
      Type: string;
      Amount: number;
      Data: string;
      DataObjects: string;
      /**
       * Entity type: PaymentTransaction
       */
      ID: number;
      /**
       * Entity type: Device
       */
      OriginalDeviceID?: number;
    }

    export interface ChainLedgerDto {
      NewSequenceNumber: number;
      PreviousSequenceNumber?: number;
      NewSignature: string;
      PreviousSignature: string;
      CreationTime: string;
    }

  }

  export interface LiveGuardPushOrderResponse {
    /**
     * Entity type: Order
     */
    ID: number;
    AlreadyExists: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface LiveGuardSyncCatalogByBlob {
  }

  export interface LiveGuardSyncCatalogByBlobResponse {
    Products: string;
    Bundles: string;
    PriceList: string;
    PromotionPriceList: string;
    EcoTaxPriceList: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface LiveGuardSyncGeneralByBlob {
  }

  export interface LiveGuardSyncGeneralByBlobResponse {
    Url?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface LiveGuardSyncUsersByBlob {
  }

  export interface LiveGuardSyncUsersByBlobResponse {
    Url?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Trigger a full sync on a liveguard instance, this has major performance impact for a while
   */
  export interface LiveGuardTriggerSync {
  }

  /**
   * Validate the given token with the cloud, and if valid, trust the chain.
   */
  export interface LiveGuardValidateToken {
    LocalChainID?: string;
    CloudChainID?: string;
    AuthenticationToken?: string;
  }

  export interface LiveGuardValidateTokenResponse {
    ChainID?: string;
    /**
     * Entity type: User
     */
    UserID?: number;
    EmailAddress?: string;
    Nickname?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface EVAStatus {
  }

  export interface EVAStatusResponse {
    Ready: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns some basic information about the current state of the liveGuard
   */
  export interface LiveGuardGetInformation {
  }

  export interface LiveGuardGetInformationResponse {
    HardwareID?: string;
    Version: string;
    OrganizationUnitName: string;
    OrganizationUnitBackendID: string;
    ProductsCount: number;
    UsersCount: number;
    SettingsCount: number;
    PendingOrderExports: number;
    LastGeneralSync?: string;
    LastUserSync?: string;
    LastCatalogSync?: string;
    GeneralTaskState?: EVA.Core.TaskState;
    CatalogTaskState?: EVA.Core.TaskState;
    UserTaskState?: EVA.Core.TaskState;
    GeneralTaskLastRun?: string;
    CatalogTaskLastRun?: string;
    UserTaskLastRun?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Push metrics of the watchtower device to the cloud.
   */
  export interface LiveGuardPushMetrics {
    HardwareID?: string;
    State?: string;
    EVAVersion?: string;
    IOVersion?: string;
    OSVersion?: string;
    CPUUsage?: number;
    DiskUsage?: number;
    RAMUsage?: number;
    LastBootTime?: string;
    Temperature?: number;
    Latency?: number;
    AvgLatency?: number;
    DNSLatency?: number;
    Logs?: EVA.Core.WatchtowerLog[];
  }

  /**
   * This service overrides the operational flags for a liveguard instance.
   * 
   * This should never be used in production environments. This is used to mimick offline scenario's in liveguard deployments in the cloud.
   */
  export interface LiveGuardSetStatus {
    /**
     * Flag this environment as if it's running on a watchtower device. Will disable a lot of functionality.
     */
    RunningOnWatchtower?: boolean;
    /**
     * Flag this environment as if it has no working internet connection. Will disable a lot of functionality.
     */
    InternetAvailable?: boolean;
    /**
     * Flag this environment as if the cloud deployment of EVA is not availabled. Will disable a lot of functionality.
     */
    EVACloudAvailable?: boolean;
    /**
     * Disable the automatic task that checks for Internet/EVACloud. This should be set to true if these properties need to be managed by hand.
     */
    DisableCheckTask?: boolean;
  }

  /**
   * This service returns the operational flags for a liveguard instance.
   */
  export interface LiveGuardGetStatus {
  }

  export interface LiveGuardGetStatusResponse {
    RunningOnWatchtower: boolean;
    InternetAvailable: boolean;
    EVACloudAvailable: boolean;
    DisableCheckTask: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new SSL certificate for a Watchtower device.
   */
  export interface CreateWatchtowerCertificate {
    CertificateSigningRequest: string;
  }

  export interface CreateWatchtowerCertificateResponse {
    Certificate: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new watchtower device
   * 
   * To create a device for fallback IO, leave the HardwareID null
   * When a HardwareID is set, the device will be boarded and prepared for use.
   */
  export interface CreateWatchtowerDevice {
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    HardwareID?: string;
  }

  export interface CreateWatchtowerDeviceResponse {
    /**
     * Entity type: Device
     */
    ID?: number;
    ApiKey?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new VPN Configuration for a Watchtower device.
   */
  export interface CreateWatchtowerVPNConfiguration {
  }

  export interface CreateWatchtowerVPNConfigurationResponse {
    Config: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Execute a command on a device
   */
  export interface LiveGuardExecuteTask {
    /**
     * Entity type: Device
     */
    DeviceID: number;
    Catalog?: boolean;
    General?: boolean;
    Users?: boolean;
  }

  /**
   * Push the current configuration for the given device to mender
   */
  export interface PushWatchtowerConfigurationToDevice {
    /**
     * Entity type: Device
     */
    DeviceID: number;
  }

  /**
   * Update the lock status of a watchtower device.
   * 
   * This can only be done when the device is online
   */
  export interface ToggleLockStatusForWatchtowerDevice {
    /**
     * Entity type: Device
     */
    DeviceID: number;
    IsLocked: boolean;
  }

  /**
   * Execute a command on a device
   */
  export interface WatchtowerExecuteCommand {
    /**
     * Entity type: Device
     */
    DeviceID: number;
    Type: string;
    Address?: string;
    Port?: number;
  }

  export interface WatchtowerExecuteCommandResponse {
    Output?: string;
    Success?: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get details about the given device.
   */
  export interface WatchtowerGetDeviceDetails {
    /**
     * Entity type: Device
     */
    DeviceID: number;
  }

  export interface WatchtowerGetDeviceDetailsResponse {
    ID?: number;
    FleetManagerID?: string;
    HardwareID?: string;
    Virtual?: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName?: string;
    OrganizationUnitBackendID?: string;
    /**
     * Entity type: Country
     */
    OrganizationUnitCountryID?: string;
    OrganizationUnitStatus?: EVA.Core.OrganizationUnitStatus;
    FleetManagerLastCheckIn?: string;
    WatchtowerIOConnected?: boolean;
    WatchtowerIOLastSeen?: string;
    WatchtowerConnected?: boolean;
    WatchtowerLastSeen?: string;
    Online?: boolean;
    LastSeen?: string;
    Locked?: boolean;
    ForceLocalMode?: boolean;
    OSVersion?: string;
    OSVersionLastCheckinTime?: string;
    State?: string;
    StateLastCheckinTime?: string;
    EVAVersion?: string;
    EVAVersionLastCheckinTime?: string;
    IOVersion?: string;
    IOVersionLastCheckinTime?: string;
    LiveGuardStateLastCheckinTime?: string;
    GeneralTaskState?: EVA.Core.TaskState;
    GeneralTaskStateLastRunTime?: string;
    CatalogTaskState?: EVA.Core.TaskState;
    CatalogTaskStateLastRunTime?: string;
    UserTaskState?: EVA.Core.TaskState;
    UserTaskStateLastRunTime?: string;
    ProductsCount?: number;
    UsersCount?: number;
    SettingsCount?: number;
    PendingOrderExports?: number;
    NetworkInterfaces?: WatchtowerGetDeviceDetailsResponse.NetworkInterface[];
    FleetManagerStatus?: FleetManagerDeviceStatus;
    Logs?: EVA.Core.WatchtowerLog[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace WatchtowerGetDeviceDetailsResponse {
    export interface NetworkInterface {
      Name: string;
      MacAddress: string;
      IPV4?: string;
      IPV6?: string;
    }

  }

  export const enum FleetManagerDeviceStatus {
    Accepted = 1,
    Pending = 2,
    Rejected = 3,
    Preauthorized = 4,
    Noauth = 5
  }

  /**
   * List the configured watchtower devices with their status
   * 
   * Sorting is supported on the following properties:
   * - ID
   * - Name
   * - HardwareID
   * - OrganizationUnitID
   * - OrganizationUnitBackendID
   * - OrganizationUnitName
   * - OrganizationUnitCountryID
   * - OrganizationUnitStatus
   * - WatchtowerIOConnected
   * - WatchtowerIOLastSeen
   * - ForceLocalMode
   */
  export interface ListWatchtowerDevices {
    PageConfig?: EVA.Core.PageConfig<ListWatchtowerDevicesFilter>;
  }

  export interface ListWatchtowerDevicesFilter {
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitStatus?: EVA.Core.OrganizationUnitStatus;
    IncludeChildOrganizationUnits?: boolean;
    /**
     * Entity type: Device
     */
    DeviceID?: number;
    FleetManagerID?: string;
    HardwareID?: string;
    IsWatchtowerIOConnected?: boolean;
    IsWatchtowerConnected?: boolean;
    IsWatchtowerOnline?: boolean;
    IsLocked?: boolean;
    ForceLocalMode?: boolean;
  }

  export interface ListWatchtowerDevicesResponse {
    Result?: EVA.Core.PagedResult<ListWatchtowerDevicesResponse.Watchtower>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListWatchtowerDevicesResponse {
    export interface Watchtower {
      /**
       * Entity type: Device
       */
      ID: number;
      Name: string;
      HardwareID: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitBackendID?: string;
      OrganizationUnitName: string;
      /**
       * Entity type: Country
       */
      OrganizationUnitCountryID: string;
      OrganizationUnitStatus: EVA.Core.OrganizationUnitStatus;
      WatchtowerIOConnected: boolean;
      WatchtowerIOLastSeen?: string;
      WatchtowerConnected: boolean;
      WatchtowerLastSeen?: string;
      Online: boolean;
      LastSeen?: string;
      Locked: boolean;
      ForceLocalMode: boolean;
    }

  }

  /**
   * Get an authentication token that's valid in the cloud for the current user
   */
  export interface LiveGuardGetCloudToken {
    UseJwtTokens?: boolean;
  }

  export interface LiveGuardGetCloudTokenResponse {
    AuthenticationToken: string;
    RefreshToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns id mapping for cloud vs liveguard entities
   */
  export interface LiveGuardGetMapping {
  }

  export interface LiveGuardGetMappingResponse {
    Mapping: { [key: string]: { [key: number]: number } };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get a trust token that needs to be provided in the Login call to the cloud to verify local presence
   */
  export interface LiveGuardGetTrustToken {
  }

  export interface LiveGuardGetTrustTokenResponse {
    TrustToken: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

}
