declare module EVA.Global {

  export interface GetGlobalBlob {
    BlobID: string;
  }

  export interface GetGlobalBlobResponse {
    Data: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetGlobalBlobInfo {
    BlobID: string;
  }

  export interface GetGlobalBlobInfoResponse {
    Found: boolean;
    OriginalName: string;
    Category: string;
    ExpireDate?: string;
    MimeType: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetGlobalCurrencies {
  }

  export interface GetGlobalCurrenciesResponse {
    Currencies: GetGlobalCurrenciesResponse.Currency[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalCurrenciesResponse {
    export interface Currency {
      Name: string;
      Description: string;
      Precision: number;
    }

  }

  export interface GetGlobalMessageTemplates {
  }

  export interface GetGlobalMessageTemplatesResponse {
    Templates: GetGlobalMessageTemplatesResponse.MessageTemplate[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalMessageTemplatesResponse {
    export interface MessageTemplate {
      Name: string;
      OrganizationUnitGlobalID: string;
      LanguageID: string;
      CountryID: string;
      Template: string;
      Helpers: string;
      Type: EVA.Core.MessageTemplateTypes;
      Layout: string;
      Destination: EVA.Core.MessageTemplateDestinations;
      PaperPropertiesData: string;
      IsDisabled: boolean;
      GlobalID: string;
    }

  }

  export interface GetGlobalOpeningHours {
  }

  export interface GetGlobalOpeningHoursResponse {
    Types: GetGlobalOpeningHoursResponse.TypeData[];
    Templates: GetGlobalOpeningHoursResponse.Template[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalOpeningHoursResponse {
    export interface TypeData {
      Name: string;
      BackendID: string;
      Description: string;
    }

    export interface Template {
      GlobalID: string;
      TypeName: string;
      OrganizationUnitGlobalID: string;
      Description: string;
      IsImported: boolean;
      Details: GetGlobalOpeningHoursResponse.TemplateDetail[];
      OrganizationUnitSetGlobalIDs: string[];
    }

    export interface TemplateDetail {
      GlobalID: string;
      DayOfWeek?: EVA.Core.DayOfWeek;
      Date?: string;
      Description: string;
      OpeningSlots: GetGlobalOpeningHoursResponse.TemplateDetailSlot[];
    }

    export interface TemplateDetailSlot {
      StartTime: string;
      EndTime: string;
    }

  }

  export interface GetGlobalOrganizationUnitSets {
  }

  export interface GetGlobalOrganizationUnitSetsResponse {
    OrganizationUnitSets: GetGlobalOrganizationUnitSetsResponse.GlobalOrganizationUnitSet[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalOrganizationUnitSetsResponse {
    export interface GlobalOrganizationUnitSet {
      GlobalID: string;
      OrganizationUnitGlobalID: string;
      SerializedDefinition: string;
      Name: string;
      Type: EVA.Core.OrganizationUnitSetTypes;
      Subsets: GetGlobalOrganizationUnitSetsResponse.Subset[];
    }

    export interface Subset {
      GlobalID: string;
      Type: EVA.Core.OrganizationUnitSetOperatorTypes;
      SequenceNumber: number;
    }

  }

  export interface GetGlobalOrganizationUnits {
  }

  export interface GetGlobalOrganizationUnitsResponse {
    OrganizationUnits: GetGlobalOrganizationUnitsResponse.GlobalOrganizationUnit[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalOrganizationUnitsResponse {
    export interface GlobalOrganizationUnit {
      GlobalID: string;
      BackendID: string;
      Name: string;
      Description: string;
      Notes: string;
      ParentID: string;
      RegisterCashLimit?: number;
      SafeCashLimit?: number;
      CashHandler: GlobalOrganizationUnit.CashHandlerDto;
      BackendRelationID: string;
      BackendCompanyID: string;
      BranchNumber: string;
      GlobalLocationNumber: string;
      OpeningHours: GlobalOrganizationUnit.OpeningHour[];
      Address: EVA.Core.AddressDataDto;
      Latitude?: number;
      Longitude?: number;
      Type: EVA.Core.OrganizationUnitTypes;
      Status: EVA.Core.OrganizationUnitStatus;
      Subnet: string;
      BankAccount: string;
      VatNumber: string;
      RegistrationNumber: string;
      EmailAddress: string;
      PhoneNumber: string;
      UseForAccounting: boolean;
      AttachedToUserID: string;
      IpAddress: string;
      CountryID: string;
      LanguageID: string;
      CurrencyID: string;
      CostPriceCurrencyID: string;
      CashFloat?: number;
      TimeZone: string;
      TaxProvider: GlobalOrganizationUnit.TaxProviderDto;
      AccountingOrganizationUnitID: string;
      AssortmentCode: string;
      RoleSetID: string;
      RegionID: string;
      ReturnOrganizationUnitID: string;
    }

    export namespace GlobalOrganizationUnit {
      export interface OpeningHour {
        DayOfWeek?: EVA.Core.DayOfWeek;
        StartTime?: string;
        EndTime?: string;
        Date?: string;
        Description: string;
        ClosedForReceipts: boolean;
        Type: string;
      }

      export interface CashHandlerDto {
        Name: string;
        CurrencyID: string;
        RoundingFactor: number;
        Coins: number[];
        BankNotes: number[];
      }

      export interface TaxProviderDto {
        Name: string;
        Type: string;
        BackendID: string;
        Description?: string;
      }

    }

  }

  export interface GetGlobalPaymentTypes {
  }

  export interface GetGlobalPaymentTypesResponse {
    PaymentTypes: GetGlobalPaymentTypesResponse.PaymentTypeDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalPaymentTypesResponse {
    export interface PaymentTypeDto {
      IsActive: boolean;
      OrganizationUnitSetGlobalID: string;
      PaymentMethodCode: string;
      Name: string;
      Code: string;
      CashJournalMethod: EVA.Core.PaymentCashJournalMethod;
      LedgerClassID: string;
      PrintOnDocuments: boolean;
      BackendRelationID: string;
      AutoFinalizeOnOrderPaid: boolean;
    }

  }

  export interface GetGlobalRegions {
  }

  export interface GetGlobalRegionsResponse {
    Regions: GetGlobalRegionsResponse.Region[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalRegionsResponse {
    export interface Region {
      Name: string;
      Code: string;
      ApiEndPoint: string;
      IsIsolated: boolean;
      IsPrimary: boolean;
      Prefix: string;
    }

  }

  export interface GetGlobalRoles {
  }

  export interface GetGlobalRolesResponse {
    Roles: GetGlobalRolesResponse.Role[];
    RoleSets: GetGlobalRolesResponse.RoleSet[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalRolesResponse {
    export interface Role {
      Name: string;
      Code: string;
      UserType: EVA.Core.UserTypes;
      GlobalID: string;
      Functionalities: GetGlobalRolesResponse.Functionality[];
      DeactivateUserAfterInactiveDays?: number;
    }

    export interface Functionality {
      Name: string;
      Scope: EVA.Core.FunctionalityScope;
    }

    export interface RoleSet {
      Name: string;
      BackendID: string;
      GlobalID: string;
      Roles: string[];
    }

  }

  export interface GetGlobalSettings {
    RegionCode?: string;
  }

  export interface GetGlobalSettingsResponse {
    OrganizationUnits: GetGlobalSettingsResponse.OrganizationUnitSettings[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalSettingsResponse {
    export interface OrganizationUnitSettings {
      GlobalID: string;
      Settings: GetGlobalSettingsResponse.Setting[];
    }

    export interface Setting {
      Name: string;
      Value: string;
      SensitivityTypeID: number;
    }

  }

  export interface GetGlobalUser {
    RegionCode: string;
    Username: string;
    Password: string;
    AsEmployee?: boolean;
  }

  export interface GetGlobalUserResponse {
    User: GetGlobalUserResponse.UserDto;
    Roles: GetGlobalUserResponse.OrganizationUnitRole[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetGlobalUserResponse {
    export interface UserDto {
      EmailAddress: string;
      BackendRelationID: string;
      Initials: string;
      FirstName: string;
      LastName: string;
      DateOfBirth?: string;
      Gender: string;
      PlaceOfBirth: string;
      LanguageID: string;
      CountryID: string;
      Type: EVA.Core.UserTypes;
      Nickname: string;
      FiscalID: string;
      SocialSecurityNumber: string;
      BankAccount: string;
      PhoneNumber: string;
      TimeZone: string;
      BackendSystemID: string;
      BackendID: string;
      GlobalID: string;
      Title: string;
      Salutation: string;
      PricingGroupID?: string;
    }

    export interface OrganizationUnitRole {
      OrganizationUnitGlobalID: string;
      OrganizationUnitSetGlobalID: string;
      RoleGlobalID: string;
      UserType: EVA.Core.UserTypes;
    }

  }

}
