declare module EVA.Intersolve {

  export namespace Errors {
    export const enum Intersolve {
      /**
       * Failed to issue giftcard on order {0:number}
       */
      IssuingFailed = 'Intersolve:IssuingFailed',
      /**
       * A CardNumber is required to activate a giftcard
       */
      MissingCardNumber = 'Intersolve:MissingCardNumber',
      /**
       * A PaymentTransaction can only be refunded for the full amount
       */
      RefundsCanOnlyBeDoneForTheFullAmount = 'Intersolve:RefundsCanOnlyBeDoneForTheFullAmount',
      /**
       * No CardActionLedger could be found for the PaymentTransaction
       */
      NoCardActionLedgerFound = 'Intersolve:NoCardActionLedgerFound',
      /**
       * Card with SerialNumber {0:string} was not found
       */
      CardNotFound = 'Intersolve:CardNotFound',
      /**
       * Trying to pay with '{currencyID:string}' but the card is in '{cardCurrencyID:string}'
       */
      CurrencyMismatch = 'Intersolve:CurrencyMismatch',
      /**
       * Failed to create a payment: {0:string}
       */
      CardActivationFailed = 'Intersolve:CardActivationFailed',
      /**
       * Failed to cancel the transaction: {0:string}
       */
      CardTransactionCancellationFailed = 'Intersolve:CardTransactionCancellationFailed',
      /**
       * Code {Code:number}: {Message:string}
       */
      UnknownResultCode = 'Intersolve:UnknownResultCode',
      /**
       * Error in the configuration at the Intersolve side. Please contact Intersolve.
       */
      ConfigurationError = 'Intersolve:ConfigurationError',
      /**
       * Service failed. This can be a typo in the command.
       */
      ServiceFailed = 'Intersolve:ServiceFailed',
      /**
       * Invalid operation.
       */
      Error_1 = 'Intersolve:Error_1',
      /**
       * XML Syntax error
       */
      Error_2 = 'Intersolve:Error_2',
      /**
       * Authorization failed, not logged in.
       */
      Error_3 = 'Intersolve:Error_3',
      /**
       * Login failed (Reason not specified).
       */
      Error_4 = 'Intersolve:Error_4',
      /**
       * Invalid parameters (e.g. no CardId specified).
       */
      Error_5 = 'Intersolve:Error_5',
      /**
       * Not allowed to import this card holder.
       */
      Error_6 = 'Intersolve:Error_6',
      /**
       * Retailer not in Group.
       */
      Error_7 = 'Intersolve:Error_7',
      /**
       * Invalid Identifier.
       */
      Error_8 = 'Intersolve:Error_8',
      /**
       * Invalid/unknown Retailer.
       */
      Error_9 = 'Intersolve:Error_9',
      /**
       * Invalid/unknown Terminal.
       */
      Error_10 = 'Intersolve:Error_10',
      /**
       * Invalid Terminal/password does not match. Not authorized.
       */
      Error_11 = 'Intersolve:Error_11',
      /**
       * Invalid Terminal originating address.
       */
      Error_12 = 'Intersolve:Error_12',
      /**
       * No configuration available.
       */
      Error_13 = 'Intersolve:Error_13',
      /**
       * Origination address not authorized.
       */
      Error_14 = 'Intersolve:Error_14',
      /**
       * No transaction available to confirm or cancel.
       */
      Error_15 = 'Intersolve:Error_15',
      /**
       * RetailerId/TerminalId combination does not exist.
       */
      Error_16 = 'Intersolve:Error_16',
      /**
       * Unable to cancel. Pos cannot cancel transaction.
       */
      Error_18 = 'Intersolve:Error_18',
      /**
       * Transaction can not be cancelled at this moment, please try again later.
       */
      Error_19 = 'Intersolve:Error_19',
      /**
       * Add card failed.
       */
      Error_20 = 'Intersolve:Error_20',
      /**
       * Card already exists.
       */
      Error_21 = 'Intersolve:Error_21',
      /**
       * Unknown brand type.
       */
      Error_22 = 'Intersolve:Error_22',
      /**
       * Login failed: Device blocked due to license agreement violation.
       */
      Error_23 = 'Intersolve:Error_23',
      /**
       * Login failed: Device usage exceeds license agreement.
       */
      Error_24 = 'Intersolve:Error_24',
      /**
       * Post issue failed.
       */
      Error_100 = 'Intersolve:Error_100',
      /**
       * Card or Customer does not exist.
       */
      Error_101 = 'Intersolve:Error_101',
      /**
       * Card is expired.
       */
      Error_102 = 'Intersolve:Error_102',
      /**
       * Card is blocked.
       */
      Error_103 = 'Intersolve:Error_103',
      /**
       * Insufficient balance to perform transaction.
       */
      Error_104 = 'Intersolve:Error_104',
      /**
       * Usr/pwd does not match with cardholder.
       */
      Error_105 = 'Intersolve:Error_105',
      /**
       * Cardholder not registered.
       */
      Error_106 = 'Intersolve:Error_106',
      /**
       * Invalid sequence number.
       */
      Error_107 = 'Intersolve:Error_107',
      /**
       * Card not related to a Group.
       */
      Error_108 = 'Intersolve:Error_108',
      /**
       * Invalid CardId. Invalid EAN.
       */
      Error_109 = 'Intersolve:Error_109',
      /**
       * Invalid Email address.
       */
      Error_110 = 'Intersolve:Error_110',
      /**
       * Invalid Country.
       */
      Error_111 = 'Intersolve:Error_111',
      /**
       * Invalid ObjectId/ObjectType.
       */
      Error_112 = 'Intersolve:Error_112',
      /**
       * Invalid CardCode in CustomerId.
       */
      Error_113 = 'Intersolve:Error_113',
      /**
       * Card is already active.
       */
      Error_114 = 'Intersolve:Error_114',
      /**
       * Invalid GroupId.
       */
      Error_115 = 'Intersolve:Error_115',
      /**
       * Invalid Card Format.
       */
      Error_116 = 'Intersolve:Error_116',
      /**
       * Invalid Scheme.
       */
      Error_117 = 'Intersolve:Error_117',
      /**
       * Card is transferred.
       */
      Error_118 = 'Intersolve:Error_118',
      /**
       * Card is not active / Inactive Brand Type.
       */
      Error_119 = 'Intersolve:Error_119',
      /**
       * Incorrect activation or purchase value / Pre dominated rule violated / Minimum balance rule violated.
       */
      Error_120 = 'Intersolve:Error_120',
      /**
       * Customer already exists.
       */
      Error_121 = 'Intersolve:Error_121',
      /**
       * Card already exists.
       */
      Error_122 = 'Intersolve:Error_122',
      /**
       * Customer does not exist.
       */
      Error_123 = 'Intersolve:Error_123',
      /**
       * Maximum number of transactions per day reached.
       */
      Error_124 = 'Intersolve:Error_124',
      /**
       * Maximum number of transactions per day per retailer reached.
       */
      Error_125 = 'Intersolve:Error_125',
      /**
       * Maximum issue points per day is reached.
       */
      Error_126 = 'Intersolve:Error_126',
      /**
       * Card not available. / Out of cards stock.
       */
      Error_127 = 'Intersolve:Error_127',
      /**
       * Not allowed to redeem on date of first transaction (business rule 107 indicates that you are only allowed to perform a redeem one day after the first issue transaction).
       */
      Error_128 = 'Intersolve:Error_128',
      /**
       * Number of points for issue or redeem transaction is too high (MaxPointsPerIssue or MaxPointsPerRedeem).
       */
      Error_129 = 'Intersolve:Error_129',
      /**
       * The service is not available on the card.
       */
      Error_130 = 'Intersolve:Error_130',
      /**
       * The service cannot be verified.
       */
      Error_131 = 'Intersolve:Error_131',
      /**
       * Receipt not found.
       */
      Error_132 = 'Intersolve:Error_132',
      /**
       * Receipt already claimed.
       */
      Error_133 = 'Intersolve:Error_133',
      /**
       * Invalid block date.
       */
      Error_134 = 'Intersolve:Error_134',
      /**
       * SchemeId already exists.
       */
      Error_135 = 'Intersolve:Error_135',
      /**
       * Balance not available.
       */
      Error_136 = 'Intersolve:Error_136',
      /**
       * Invalid Brand.
       */
      Error_140 = 'Intersolve:Error_140',
      /**
       * Invalid BrandType.
       */
      Error_141 = 'Intersolve:Error_141',
      /**
       * Maximum card balance is reached.
       */
      Error_161 = 'Intersolve:Error_161',
      /**
       * Card is not reloadable. / Too many reload transactions. / Wrong balance on Reload.
       */
      Error_162 = 'Intersolve:Error_162',
      /**
       * Card is already assigned to a customer.
       */
      Error_163 = 'Intersolve:Error_163',
      /**
       * Wrong currency.
       */
      Error_164 = 'Intersolve:Error_164',
      /**
       * Transaction would exceed configured limits.
       */
      Error_165 = 'Intersolve:Error_165',
      /**
       * Login failed: Unknown user.
       */
      Error_201 = 'Intersolve:Error_201',
      /**
       * Maximum simultaneous logins reached.
       */
      Error_202 = 'Intersolve:Error_202',
      /**
       * User is already logged in.
       */
      Error_203 = 'Intersolve:Error_203',
      /**
       * Login account is set inactive.
       */
      Error_204 = 'Intersolve:Error_204',
      /**
       * Database is locked.
       */
      Error_205 = 'Intersolve:Error_205',
      /**
       * Database is expired and now set to locked.
       */
      Error_206 = 'Intersolve:Error_206',
      /**
       * Login failed: Invalid password.
       */
      Error_207 = 'Intersolve:Error_207',
      /**
       * Invalid Username.
       */
      Error_208 = 'Intersolve:Error_208',
      /**
       * Invalid SecretAnswer.
       */
      Error_209 = 'Intersolve:Error_209',
      /**
       * Invalid SecretQuestion.
       */
      Error_210 = 'Intersolve:Error_210',
      /**
       * Invalid PIN.
       */
      Error_211 = 'Intersolve:Error_211',
      /**
       * Invalid EAN code.
       */
      Error_212 = 'Intersolve:Error_212',
      /**
       * Invalid E-mail address.
       */
      Error_250 = 'Intersolve:Error_250',
      /**
       * E-mail or customer information is missing.
       */
      Error_251 = 'Intersolve:Error_251',
      /**
       * E-mail address already in use.
       */
      Error_252 = 'Intersolve:Error_252',
      /**
       * Invalid ConfigurationId.
       */
      Error_300 = 'Intersolve:Error_300',
      /**
       * Invalid VoucherId.
       */
      Error_301 = 'Intersolve:Error_301',
      /**
       * Invalid IssuedVoucherId. The specified issued voucher does not exist or does not have the correct status.
       */
      Error_302 = 'Intersolve:Error_302',
      /**
       * Voucher not found.
       */
      Error_303 = 'Intersolve:Error_303',
      /**
       * Voucher not available.
       */
      Error_304 = 'Intersolve:Error_304',
      /**
       * Invalid Voucher/Owner combination.
       */
      Error_305 = 'Intersolve:Error_305',
      /**
       * Invalid MediaId/MediaType.
       */
      Error_306 = 'Intersolve:Error_306',
      /**
       * Voucher has been redeemed already.
       */
      Error_307 = 'Intersolve:Error_307',
      /**
       * Refund not supported for this voucher.
       */
      Error_308 = 'Intersolve:Error_308',
      /**
       * Voucher redeem currently denied for this voucher.
       */
      Error_309 = 'Intersolve:Error_309',
      /**
       * Invalid redeem currently denied for this voucher.
       */
      Error_310 = 'Intersolve:Error_310',
      /**
       * Maximum number of voucher issues exceeded for this voucher.
       */
      Error_311 = 'Intersolve:Error_311',
      /**
       * Invalid Brand Type ID. / Voucher has been purchased already.
       */
      Error_312 = 'Intersolve:Error_312',
      /**
       * Voucher Reservation Expired.
       */
      Error_313 = 'Intersolve:Error_313',
      /**
       * Invalid Delivery Method.
       */
      Error_314 = 'Intersolve:Error_314',
      /**
       * CardId/InitialAccessCode mismatch.
       */
      Error_401 = 'Intersolve:Error_401',
      /**
       * Scheme not found.
       */
      Error_402 = 'Intersolve:Error_402',
      /**
       * Invalid RegisterCode.
       */
      Error_403 = 'Intersolve:Error_403',
      /**
       * Card does not belong to the customer.
       */
      Error_404 = 'Intersolve:Error_404',
      /**
       * The source and destination card must be specified.
       */
      Error_405 = 'Intersolve:Error_405',
      /**
       * May not transfer to the same card.
       */
      Error_406 = 'Intersolve:Error_406',
      /**
       * Transfer customer failed.
       */
      Error_407 = 'Intersolve:Error_407',
      /**
       * Customer not found.
       */
      Error_408 = 'Intersolve:Error_408',
      /**
       * Invalid CustomerId format.
       */
      Error_409 = 'Intersolve:Error_409',
      /**
       * No active draw.
       */
      Error_500 = 'Intersolve:Error_500',
      /**
       * Draw is closed. New sales starts tomorrow.
       */
      Error_501 = 'Intersolve:Error_501',
      /**
       * No lottery tickets available.
       */
      Error_502 = 'Intersolve:Error_502',
      /**
       * No active draw for this product.
       */
      Error_503 = 'Intersolve:Error_503',
      /**
       * Sold out. Select a different day.
       */
      Error_504 = 'Intersolve:Error_504',
      /**
       * Customer does not exist.
       */
      Error_550 = 'Intersolve:Error_550',
      /**
       * Validation field 1 not valid.
       */
      Error_551 = 'Intersolve:Error_551',
      /**
       * Validation field 2 not valid.
       */
      Error_552 = 'Intersolve:Error_552',
      /**
       * Validation field 3 not valid.
       */
      Error_553 = 'Intersolve:Error_553',
      /**
       * Delivery type is not supported.
       */
      Error_554 = 'Intersolve:Error_554',
      /**
       * Card already has a PIN.
       */
      Error_555 = 'Intersolve:Error_555',
      /**
       * Card does have a PIN.
       */
      Error_556 = 'Intersolve:Error_556',
      /**
       * Invalid PIN.
       */
      Error_557 = 'Intersolve:Error_557',
      /**
       * Could not encrypt PIN.
       */
      Error_558 = 'Intersolve:Error_558',
      /**
       * Payment Ok.
       */
      Error_700 = 'Intersolve:Error_700',
      /**
       * Payment failed.
       */
      Error_701 = 'Intersolve:Error_701',
      /**
       * Payment declined.
       */
      Error_702 = 'Intersolve:Error_702',
      /**
       * Payment pending.
       */
      Error_703 = 'Intersolve:Error_703',
      /**
       * Payment revoked.
       */
      Error_704 = 'Intersolve:Error_704',
      /**
       * Transaction not found.
       */
      Error_705 = 'Intersolve:Error_705',
      /**
       * Payment needs to be cancelled.
       */
      Error_706 = 'Intersolve:Error_706',
      /**
       * Special fixed value activation already done.
       */
      Error_914 = 'Intersolve:Error_914',
      /**
       * Special fixed value activation.
       */
      Error_919 = 'Intersolve:Error_919',
      /**
       * No bueno.
       */
      NoBueno = 'Intersolve:NoBueno'
    }

  }


  export interface AddVoucherToShoppingCart {
    /**
     * Entity type: Order
     */
    OrderID: number;
    VoucherID: number;
  }

  export interface ListVoucherConfigurations {
  }

  export interface ListVoucherConfigurationsResponse {
    VoucherProviders: VoucherProvider[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface VoucherProvider {
    Options: VoucherOption[];
    Name: string;
  }

  export interface VoucherOption {
    Name: string;
    VoucherID: number;
    EAN: string;
    Value: number;
    LayoutID: string;
    Layout: string;
    UnitCost: number;
  }

}
