declare module EVA.Monitor {

  export namespace Errors {
    export const enum Monitors {
      /**
       * You are not allowed to edit this monitor.
       */
      NotAllowedToEditMonitor = 'Monitors:NotAllowedToEditMonitor',
      /**
       * This handler does not support conditions.
       */
      HandlerDoesNotSupportConditions = 'Monitors:HandlerDoesNotSupportConditions',
      /**
       * The handler with name {0:string} does not exist
       */
      InvalidHandler = 'Monitors:InvalidHandler',
      /**
       * An unknown error occurred while executing this monitor
       */
      UnknownError = 'Monitors:UnknownError'
    }

  }


  /**
   * Creates a monitor. See documentation for handler arguments.
   */
  export interface CreateMonitor {
    Handler?: string;
    Name?: string;
    DefaultLevel?: MonitorLevel;
    Description?: string;
    Cron?: string;
    Data?: { [key: string]: any };
    /**
     * Entity type: Script
     */
    ScriptID?: number;
  }

  export const enum MonitorLevel {
    Info = 0,
    Warning = 1,
    Error = 2,
    Fatal = 3
  }

  export interface CreateMonitorResponse {
    /**
     * Entity type: Monitor
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Deletes a monitor.
   */
  export interface DeleteMonitor {
    /**
     * Entity type: Monitor
     */
    ID: number;
  }

  /**
   * Download the last monitor results as excel
   */
  export interface DownloadMonitorResult {
    /**
     * Entity type: Monitor
     */
    MonitorID: number;
  }

  /**
   * Get available conditions for monitors.
   */
  export interface GetAvailableConditionsForMonitorHandler {
    Handler?: string;
  }

  export interface GetAvailableConditionsForMonitorHandlerResponse {
    AvailableConditions?: GetAvailableConditionsForMonitorHandlerResponse.AvailableCondition[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAvailableConditionsForMonitorHandlerResponse {
    export interface AvailableCondition {
      Name?: string;
      ValueOperators?: string[];
    }

  }

  /**
   * Returns available monitor handlers.
   */
  export interface GetAvailableMonitorHandlers {
  }

  export interface GetAvailableMonitorHandlersResponse {
    Handlers?: GetAvailableMonitorHandlersResponse.AvailableHandler[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAvailableMonitorHandlersResponse {
    export interface AvailableHandler {
      Name?: string;
      SupportsConditions?: boolean;
      ScriptingExtensionType?: string;
    }

  }

  /**
   * Gets the last result of a monitor.
   */
  export interface GetMonitorResult {
    /**
     * Entity type: Monitor
     */
    ID?: number;
  }

  export interface GetMonitorResultResponse {
    /**
     * Entity type: Monitor
     */
    ID?: number;
    Name?: string;
    Handler?: string;
    IsDefaultMonitor?: boolean;
    Description?: string;
    LastResult?: { [key: string]: any };
    LastResultCount?: number;
    LastRun?: string;
    DefaultLevel?: MonitorLevel;
    LastNotification?: NotificationDetails;
    LastNotificationDate?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface NotificationDetails {
    /**
     * Entity type: MonitorNotification
     */
    ID?: number;
    Description?: string;
    Threshold?: number;
    Inverse?: boolean;
    Level?: MonitorLevel;
  }

  /**
   * Gets Monitor by ID and returns basic information
   */
  export interface GetMonitor {
    /**
     * Entity type: Monitor
     */
    ID?: number;
  }

  export interface GetMonitorResponse {
    /**
     * Entity type: Monitor
     */
    ID?: number;
    Name?: string;
    Description?: string;
    Handler?: string;
    Data?: { [key: string]: any };
    IsDefaultMonitor?: boolean;
    CreationTime?: string;
    RoleOwners?: GetMonitorResponse.RoleOwnerData[];
    ConfiguredNotifications?: NotificationDetails[];
    DefaultLevel?: MonitorLevel;
    LastNotification?: NotificationDetails;
    LastNotificationDate?: string;
    IsAllowedToEdit?: boolean;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    ScriptingExtensionType?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetMonitorResponse {
    export interface RoleOwnerData {
      /**
       * Entity type: Role
       */
      RoleID?: number;
      Name?: string;
      IsAllowedToEdit?: boolean;
    }

    export interface UserOwnerData {
      /**
       * Entity type: User
       */
      UserID?: number;
      Name?: string;
      IsAllowedToEdit?: boolean;
    }

  }

  /**
   * Gets all visible monitors by current user.
   */
  export interface GetUserMonitors {
    Handler?: string;
  }

  export interface GetUserMonitorsResponse {
    Monitors?: GetUserMonitorsResponse.UserMonitor[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetUserMonitorsResponse {
    export interface UserMonitor {
      /**
       * Entity type: Monitor
       */
      ID?: number;
      Name?: string;
      Handler?: string;
      IsDefaultMonitor?: boolean;
      Description?: string;
      IsAllowedToEdit?: boolean;
      LastRun?: string;
      LastResultCount?: number;
      DefaultLevel?: MonitorLevel;
      LastNotification?: NotificationDetails;
      LastNotificationDate?: string;
    }

  }

  /**
   * Gets the last result of a monitor.
   */
  export interface ListScriptedMonitorResults {
    InitialPageConfig?: EVA.Core.PageTokenConfig<ListScriptedMonitorResultFilter>;
    PageToken?: string;
  }

  export interface ListScriptedMonitorResultFilter {
    /**
     * Entity type: Monitor
     */
    MonitorID: number;
  }

  export interface ListScriptedMonitorResultsResponse {
    Results?: ListScriptedMonitorResultsResponse.ResultItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListScriptedMonitorResultsResponse {
    export interface ResultItem {
      DataModelID?: string;
      Data?: any;
    }

  }

  /**
   * This service can be called to manually trigger a monitor.
   */
  export interface RunMonitor {
    /**
     * Entity type: Monitor
     */
    ID: number;
  }

  /**
   * Set permissions for a monitor.
   */
  export interface SetMonitorPermissions {
    /**
     * Entity type: Monitor
     */
    ID?: number;
    RoleOwners?: SetMonitorPermissions.RoleOwnerData[];
  }

  export namespace SetMonitorPermissions {
    export interface RoleOwnerData {
      /**
       * Entity type: Role
       */
      RoleID: number;
      IsAllowedToEdit: boolean;
    }

    export interface UserOwnerData {
      /**
       * Entity type: User
       */
      UserID: number;
      IsAllowedToEdit: boolean;
    }

  }

  /**
   * Updates a monitor.
   */
  export interface UpdateMonitor {
    /**
     * Entity type: Monitor
     */
    ID: number;
    Description?: string;
    Cron?: string;
    Name?: string;
    Data?: { [key: string]: any };
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    DefaultLevel?: MonitorLevel;
  }

  /**
   * Creates a role owner for a monitor.
   */
  export interface CreateMonitorRoleOwner {
    /**
     * Entity type: Monitor
     */
    MonitorID?: number;
    /**
     * Entity type: Role
     */
    RoleID?: number;
    IsAllowedToEdit?: boolean;
  }

  export interface CreateMonitorRoleOwnerResponse {
    Result: CreateMonitorRoleOwnerResult;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export const enum CreateMonitorRoleOwnerResult {
    Created = 0,
    Updated = 1
  }

  /**
   * Deletes a monitor role owner.
   */
  export interface DeleteMonitorRoleOwner {
    /**
     * Entity type: Monitor
     */
    MonitorID: number;
    /**
     * Entity type: Role
     */
    RoleID: number;
  }

  /**
   * Creates a email recipient for a Notification.
   */
  export interface CreateMonitorNotificationEmailRecipient {
    /**
     * Entity type: MonitorNotification
     */
    NotificationID?: number;
    EmailAddress?: string;
  }

  export interface CreateMonitorNotificationEmailRecipientResponse {
    /**
     * Entity type: MonitorNotificationEmailRecipient
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a monitor Notification.
   */
  export interface CreateMonitorNotification {
    /**
     * Entity type: Monitor
     */
    MonitorID?: number;
    Level?: MonitorLevel;
    Threshold?: number;
    /**
     * @default false
     */
    Inverse?: boolean;
    Description?: string;
    EmailSubject?: string;
    EmailRecipients?: CreateMonitorNotification.EmailRecipientData[];
  }

  export namespace CreateMonitorNotification {
    export interface EmailRecipientData {
      EmailAddress?: string;
    }

  }

  export interface CreateMonitorNotificationResponse {
    /**
     * Entity type: MonitorNotification
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Deletes email recipient for given monitor Notification.
   */
  export interface DeleteMonitorNotificationEmailRecipient {
    /**
     * Entity type: MonitorNotificationEmailRecipient
     */
    ID: number;
  }

  /**
   * Deletes a monitor Notification.
   */
  export interface DeleteMonitorNotification {
    /**
     * Entity type: MonitorNotification
     */
    ID: number;
  }

  /**
   * Returns information about given recipient.
   */
  export interface GetMonitorNotificationEmailRecipient {
    /**
     * Entity type: MonitorNotificationEmailRecipient
     */
    ID?: number;
  }

  export interface GetMonitorNotificationEmailRecipientResponse {
    /**
     * Entity type: MonitorNotificationEmailRecipient
     */
    ID?: number;
    EmailAddress?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Gets all email recipients for given monitor Notification.
   */
  export interface GetMonitorNotificationEmailRecipients {
    /**
     * Entity type: MonitorNotification
     */
    NotificationID?: number;
  }

  export interface GetMonitorNotificationEmailRecipientsResponse {
    Recipients?: GetMonitorNotificationEmailRecipientsResponse.EmailRecipient[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetMonitorNotificationEmailRecipientsResponse {
    export interface EmailRecipient {
      /**
       * Entity type: MonitorNotificationEmailRecipient
       */
      ID?: number;
      EmailAddress?: string;
    }

  }

  /**
   * Returns information about given monitor Notification.
   */
  export interface GetMonitorNotification {
    /**
     * Entity type: MonitorNotification
     */
    ID?: number;
  }

  export interface GetMonitorNotificationResponse {
    /**
     * Entity type: MonitorNotification
     */
    ID?: number;
    Level?: MonitorLevel;
    Threshold?: number;
    Inverse?: boolean;
    Description?: string;
    EmailSubject?: string;
    EmailRecipients?: GetMonitorNotificationResponse.EmailRecipientData[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetMonitorNotificationResponse {
    export interface EmailRecipientData {
      EmailAddress?: string;
    }

  }

  /**
   * Returns all Notifications for given monitor.
   */
  export interface GetMonitorNotifications {
    /**
     * Entity type: Monitor
     */
    MonitorID?: number;
  }

  export interface GetMonitorNotificationsResponse {
    Notifications?: GetMonitorNotificationsResponse.MonitorNotification[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetMonitorNotificationsResponse {
    export interface MonitorNotification {
      /**
       * Entity type: MonitorNotification
       */
      ID?: number;
      Level?: MonitorLevel;
      Threshold?: number;
      Inverse?: boolean;
      Description?: string;
      EmailSubject?: string;
      EmailRecipients?: MonitorNotification.EmailRecipientData[];
    }

    export namespace MonitorNotification {
      export interface EmailRecipientData {
        EmailAddress?: string;
      }

    }

  }

  export interface UpdateMonitorNotificationEmailRecipient {
    /**
     * Entity type: MonitorNotification
     */
    ID?: number;
    EmailAddress?: string;
  }

  /**
   * Updates monitor Notification details.
   */
  export interface UpdateMonitorNotification {
    /**
     * Entity type: MonitorNotification
     */
    ID: number;
    Level?: MonitorLevel;
    Threshold?: number;
    Inverse?: boolean;
    Description?: string;
    EmailSubject?: string;
    EmailRecipients?: UpdateMonitorNotification.EmailRecipientData[];
  }

  export namespace UpdateMonitorNotification {
    export interface EmailRecipientData {
      EmailAddress?: string;
    }

  }

}
