declare module EVA.PIM {

  export namespace Errors {
    export const enum Assortment {
      /**
       * Assortment is missing content culture.
       */
      MissingContentCulture = 'Assortment:MissingContentCulture'
    }

    export const enum SecondChanceProductRegistration {
      /**
       * MimeType {MimeType:string} for BlobID {BlobID:string} is an invalid MimeType for an image
       */
      InvalidMimeTypeForImage = 'SecondChanceProductRegistration:InvalidMimeTypeForImage',
      /**
       * The specified serial number {serialNumber:string} must either be new or it must belong to the original product {productID:number}.
       */
      InvalidSerialNumber = 'SecondChanceProductRegistration:InvalidSerialNumber',
      /**
       * The specified serial number {serialNumber:string} must match the regex of configuration {configurationID:number}.
       */
      InvalidSerialNumberForConfiguration = 'SecondChanceProductRegistration:InvalidSerialNumberForConfiguration'
    }

    export const enum CustomContent {
      /**
       * Error converting property {0:string}: {1:string}
       */
      DataTypeError = 'CustomContent:DataTypeError'
    }

    export const enum ImportProducts {
      /**
       * Product {productID:string} uses unknown TaxCode {taxCode:string}
       */
      UnknownTaxCode = 'ImportProducts:UnknownTaxCode',
      /**
       * The provided request did not pass validation. Failures: {0:string}
       */
      ValidationFailures = 'ImportProducts:ValidationFailures',
      /**
       * The assortment '{0:string}' is not known.
       */
      UnknownAssortment = 'ImportProducts:UnknownAssortment'
    }

    export const enum ProductCapacityPlans {
      /**
       * Either a ProductSearchTemplateID or ProductSearchFilters are required for a dynamic ProductCapacityPlan.
       */
      FiltersAreRequiredForADynamicCapacityPlan = 'ProductCapacityPlans:FiltersAreRequiredForADynamicCapacityPlan',
      /**
       * Cannot add or remove products on a dynamic ProductCapacityPlan.
       */
      CannotAddOrRemoveProductsOnADynamicCapacityPlan = 'ProductCapacityPlans:CannotAddOrRemoveProductsOnADynamicCapacityPlan',
      /**
       * Products already linked to a static ProductCapacityPlan.
       */
      ProductsAlreadyHaveAStaticProductCapacityPlan = 'ProductCapacityPlans:ProductsAlreadyHaveAStaticProductCapacityPlan',
      /**
       * Cannot convert ProductCapacityPlan from static to dynamic.
       */
      CannotConvertProductCapacityPlanFromStaticToDynamic = 'ProductCapacityPlans:CannotConvertProductCapacityPlanFromStaticToDynamic',
      /**
       * Exception requires `DayOfWeek` or `Date` to be given.
       */
      ExceptionRequiresDayOfWeekOrDate = 'ProductCapacityPlans:ExceptionRequiresDayOfWeekOrDate'
    }

    export const enum ProductExcelServices {
      /**
       * Cannot find header row
       */
      MissingHeaderRow = 'ProductExcelServices:MissingHeaderRow',
      /**
       * Unit Of Measure {0:string} does not exist
       */
      UnitOfMeasureNotFound = 'ProductExcelServices:UnitOfMeasureNotFound',
      /**
       * Product with CustomID {0:string} cannot be found
       */
      ProductNotFound = 'ProductExcelServices:ProductNotFound',
      /**
       * Invalid quantity {quantity:number} for Unit of Measure {unitOfMeasure:string} for product {productCustomID:string}
       */
      InvalidUnitOfMeasureQuantity = 'ProductExcelServices:InvalidUnitOfMeasureQuantity',
      /**
       * Duplicate barcode {0:string} found
       */
      DuplicateBarcode = 'ProductExcelServices:DuplicateBarcode'
    }

    export const enum ProductSearchSynonyms {
      /**
       * The line {0:string} is not a valid synonym rule.
       */
      InvalidRule = 'ProductSearchSynonyms:InvalidRule'
    }

    export const enum ProductContent {
      /**
       * Property {0:string} has not been defined, a property must be defined before the content for that property can be stored.
       */
      UnknownProductPropertyType = 'ProductContent:UnknownProductPropertyType'
    }

    export const enum ProductPropertyType {
      /**
       * The value {0:string} cannot be converted to a data type of {1:ProductPropertyTypeDataTypes}
       */
      CannotConvertValueToDataType = 'ProductPropertyType:CannotConvertValueToDataType'
    }

  }


  /**
   * Get the current DB values for a product, used to update these properties with the UpdateProduct service.
   */
  export interface GetProductByID {
    /**
     * Entity type: Product
     */
    ID: number;
  }

  export interface GetProductByIDResponse {
    /**
     * Entity type: Product
     */
    ID: number;
    BackendID: string;
    CustomID: string;
    PrimitiveName: string;
    TypeID: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID: number;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update the core properties of a Product
   */
  export interface UpdateProduct {
    /**
     * Entity type: Product
     */
    ID: number;
    BackendID?: string;
    CustomID?: string;
    PrimitiveName?: string;
    TypeID?: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
    LedgerClassID?: string;
    CommodityCode?: string;
    CountryOfOriginID?: string;
    ManufacturerID?: string;
  }

  /**
   * Replaces all search synonyms for a given LanguageID/CountryID with the provided ones.
   * Any synonyms not present in the request but that currently exist will be deleted.
   * 
   * ## Rules
   * The `Rule` must be formatted according to [what Elasticsearch accepts](https://www.elastic.co/guide/en/elasticsearch/reference/current/analysis-synonym-tokenfilter.html#_solr_synonyms):
   * 
   * - Comma-separated values: `jump, leap, hop` causes any of the matched terms to be replaced by all of the terms.
   * - Arrow notation: `britain=>england` causes the left side to be replaced by the right side.
   * 
   * To create a synonym, a `LanguageID` is required.
   */
  export interface ReplaceProductSearchSynonyms {
    Rules: string[];
    /**
     * Entity type: Language
     */
    LanguageID: string;
  }

  /**
   * Gets all currently created search synonyms for the provided LanguageID (required) and CountryID (optional).
   */
  export interface GetProductSearchSynonyms {
    /**
     * Entity type: Language
     */
    LanguageID: string;
  }

  export interface GetProductSearchSynonymsResponse {
    Result: GetProductSearchSynonymsResponse.ProductSearchSynonym[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetProductSearchSynonymsResponse {
    export interface ProductSearchSynonym {
      ID: number;
      Rule: string;
      LanguageID: string;
    }

  }

  export interface ApplyRevision {
    /**
     * Entity type: Revision
     */
    RevisionID: number;
  }

  export interface CreateRevision {
    Name?: string;
  }

  export interface CreateRevisionResponse {
    Result: CreatedRevision;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface CreatedRevision {
    /**
     * Entity type: Revision
     */
    ID: number;
    Name: string;
  }

  /**
   * Delete a revision
   * 
   * All Edits in this revision will be deleted
   */
  export interface DeleteRevision {
    /**
     * Entity type: Revision
     */
    ID: number;
  }

  export interface GetRevisionStatus {
    /**
     * Entity type: Revision
     */
    RevisionID: number;
  }

  export interface GetRevisionStatusResponse {
    Status: RevisionProgress;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface RevisionProgress {
    TotalItems: number;
    HandledItems: number;
    State: ProductCompositionProgressState;
    Progress: number;
    ValidationErrors: number;
  }

  export const enum ProductCompositionProgressState {
    None = 0,
    Composing = 1,
    PostProcessing = 2,
    Publishing = 3,
    Complete = 4
  }

  export interface ListRevisions {
    PageConfig?: EVA.Core.PageConfig<ListRevisionsFilter>;
  }

  export interface ListRevisionsFilter {
    Name?: string;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    Status?: RevisionStatus;
    IncludeSystemRevisions?: boolean;
  }

  export const enum RevisionStatus {
    Open = 1,
    Applied = 2,
    Inactive = 4
  }

  export interface ListRevisionsResponse {
    Result?: EVA.Core.PagedResult<RevisionDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface RevisionDto {
    /**
     * Entity type: Revision
     */
    ID?: number;
    Name?: string;
    /**
     * Entity type: Application
     */
    ApplicationID?: number;
    Status?: RevisionStatus;
    AppliedOn?: string;
    CreatedBy?: ListRevisionUser;
    CreationTime?: string;
  }

  export interface ListRevisionUser {
    FullName?: string;
  }

  export interface ListRevisionValidationErrors {
    /**
     * Entity type: Revision
     */
    RevisionID: number;
    /**
     * Min: 0
     */
    Offset?: number;
    /**
     * Min: 0
     */
    Cursor?: number;
  }

  export interface ListRevisionValidationErrorsResponse {
    Offset?: number;
    Cursor?: number;
    Errors: { [key: string]: any }[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface CreateNewSearchDataIndex {
    DeleteOldIndex?: boolean;
  }

  /**
   * Recreate aliases on all productsearch indices. Use to recover from a state where the aliases exist, but without filters.
   */
  export interface RecreateAliases {
  }

  export interface RequestReindexSearchData {
    Script?: string;
    LanguageID?: string;
    CountryID?: string;
    ApplyProductSearchConfiguration?: boolean;
  }

  export interface RequestReindexSearchDataResponse {
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface AddReferenceDataToProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Data?: { [key: string]: any };
    Name?: string;
  }

  /**
   * Creates a new Product, given at least your `PrimitiveName` and `ProductType`.
   * 
   * - [x]  Given `PrimitiveName` is a name for the product to help you recognise it, but is rarely exposed to users or customers. This property does not support localization or translations!
   * - [x]  `TaxCodeID` is an integer value indicating one of the configured TaxCodes.
   * - [x]  The `ProductType` is a flag enumeration value with one or more of the following possible values;
   * 
   *   * `0` None
   *   * `1` Stock
   *   * `2` Insurance
   *   * `4` Marketing
   *   * `8` Gift Card
   *   * `16` Service
   *   * `64` Custom Pricing
   *   * `128` External
   *   * `256` Order Costs
   *   * `512` System Generated
   *   * `1024` Product Set
   *   * `2048` Bundle Product
   *   * `16384` Configurable
   *   * `32768` Supplier Product
   *   * `65536` Template
   * 
   * Optionally, you can supply some additional information;
   * 
   * - [] `BackendID` is a number to help you identify this product. **Unique per `OrganizationUnitID` and `BackendSystemID`.**
   * - [] `BackendSystemID` is a string value to idenfity the source this product originated from - can be anything.
   * - [] `OrganizationUnitID` is the Organization Unit this product is linked to, or `null` when this is a generic EVA product.
   * - [] `CustomID` is a string value to help you identify this product, without any constraints, but indexed for optimal performance.
   * - [] `BrandID` to link it to any `Brand` entity.
   */
  export interface CreateProduct {
    PrimitiveName?: string;
    ProductType?: EVA.Core.ProductTypes;
    /**
     * Entity type: Brand
     */
    BrandID?: number;
    BackendID?: string;
    BackendSystemID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    CustomID?: string;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
    LogicalLevel?: string;
    LedgerClassID?: string;
    CommodityCode?: string;
    CountryOfOriginID?: string;
    ManufacturerID?: string;
  }

  export interface CreateProductResponse {
    /**
     * Entity type: Product
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a second chance product.
   */
  export interface CreateSecondChanceProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductName: string;
    Description: string;
    Price: number;
    BlobImageUrls?: string[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * @default false
     */
    NotReturnable?: boolean;
    /**
     * Specify a serial number.
     */
    SerialNumber?: string;
    /**
     * Moves stock from the original product to the second chance product.
     * @default false
     */
    MoveStock?: boolean;
    /**
     * Stock label for original product after stock movement. If unspecified, defaults to Damaged.
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * Please ensure the key is a valid ProductPropertyTypes.
     */
    CustomContent?: { [key: string]: any };
  }

  export interface CreateSecondChanceProductResponse {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Barcode: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Modifies given product by its `ID` for a specific `RevisionID`.
   * 
   * The `Edits` is a list of `ProductEdit` objects, that defines some `Content` and the `LanguageID`, `CountryID` and  `LayerID` it applies to. The `Content` is a Json object with key-value pairs that holds
   * product information. The keys are simple strings, but the value can be a scalar value (like an int or string), an array of scalar values or an array of objects.
   * 
   * A `Revision` is a versioning container in which one or more product modifications are buffered. The changes to the product do not apply directly, but instead are stored in this `Revision` until it is
   * ready to be published. Also see the `CreateRevision` and `ApplyRevision` services. When omitted, a new 'quick' `Revision` is created and used here.
   * 
   * A `Layer` is a hierarchical solution to building product data. Every layer has its own `Level`, where the bottom layers will be overwritten by higher layers for colliding properties. A `Layer` has a
   * `Revision`. Also see the `CreateLayer` service. When omitted, a new 'quick' `Layer` is created and used here.
   */
  export interface EditProduct {
    /**
     * Entity type: Product
     */
    ID: number;
    /**
     * Entity type: Revision
     */
    RevisionID?: number;
    Edits: ProductEdit[];
  }

  export interface ProductEdit {
    /**
     * Entity type: ContentLayer
     */
    LayerID?: number;
    Content: { [key: string]: any };
    LanguageID: string;
    CountryID: string;
  }

  /**
   * Retrieves a `Product` with detailed information per `Layer`, from the given `ID` and `RevisionID` parameter.
   * 
   * In the response, `Content` is a Json formatted object containing key-value pair data, where the keys are simple strings and the value can be a scalar value (like an int or string), an array of scalar
   * values or an array of objects.
   * 
   * The `Types` property in the response data holds meta-data information about the property keys used in the `Content` object like property name, group and (of course) data type.
   * 
   * :point_right: Use the `GetProductForQuickEditByID` service when you want to apply all published `Revision`'s and only need the compressed `Layer` version of `Content`.
   */
  export interface GetProductForEditByID {
    /**
     * Entity type: Product
     */
    ID: number;
    /**
     * Entity type: Revision
     */
    RevisionID: number;
  }

  export interface GetProductForEditByIDResponse {
    /**
     * Entity type: Product
     */
    ID: number;
    /**
     * Entity type: Catalog
     */
    CatalogID?: number;
    DisplayValue: string;
    LayersWithContent: LayerWithContentDto[];
    Types: { [key: string]: ProductPropertyTypeModel };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface LayerWithContentDto {
    /**
     * Entity type: ContentLayer
     */
    ID: number;
    Name: string;
    Level: number;
    /**
     * Entity type: Application
     */
    ApplicationID?: number;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    Content: { [key: string]: any };
    Description: string;
    DisplayValue: string;
  }

  export interface ProductPropertyTypeModel {
    ID: string;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    EditFunctionality: string;
    SearchType: ProductPropertyTypeSearchTypes;
    DataType: ProductPropertyTypeDataTypes;
    IsArray: boolean;
    Category: ProductPropertyTypeModel.CategoryModel;
    type_id: string;
    language_id: string;
    country_id: string;
    functionality: string;
    /**
     * Entity type: Application
     */
    application_id: number;
    search_type: ProductPropertyTypeSearchTypes;
    data_type: ProductPropertyTypeDataTypes;
    array: boolean;
    Intent: ProductPropertyTypeIntents;
    intent: ProductPropertyTypeIntents;
    category_id: string;
    category: ProductPropertyTypeModel.CategoryModel;
  }

  export namespace ProductPropertyTypeModel {
    export interface CategoryModel {
      ID: string;
      EditFunctionality: string;
      category_id: string;
      functionality: string;
    }

  }

  export const enum ProductPropertyTypeSearchTypes {
    None = 0,
    Keyword = 1,
    Text = 2,
    IsHidden = 4
  }

  export const enum ProductPropertyTypeDataTypes {
    None = 0,
    Integer = 1,
    Double = 2,
    Boolean = 3,
    String = 4,
    Blob = 5,
    Date = 6,
    RichText = 7
  }

  export const enum ProductPropertyTypeIntents {
    RichAttribute = 0,
    SimpleField = 1,
    RootLevelOnly = 2
  }

  /**
   * Retrieves a `Product` with detailed information, based on all currently applied `Revision`'s and the `Layer` data compressed into a single truth, from the given `ID` parameter.
   * 
   * In the response, `Content` is a Json formatted object containing key-value pair data, where the keys are simple strings and the value can be a scalar value (like an int or string), an array of scalar
   * values or an array of objects.
   * 
   * The `Types` property in the response data holds meta-data information about the property keys used in the `Content` object like property name, group and (of course) data type.
   * 
   * :point_right: Use the `GetProductForEditByID` service when you want to specify the `Revision` to apply and receive per-layer `Content`.
   */
  export interface GetProductForQuickEditByID {
    /**
     * Entity type: Product
     */
    ID: number;
  }

  export interface GetProductForQuickEditByIDResponse {
    /**
     * Entity type: Product
     */
    ID: number;
    /**
     * Entity type: Catalog
     */
    CatalogID?: number;
    DisplayValue: string;
    Content: { [key: string]: any };
    Types: { [key: string]: ProductPropertyTypeModel };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetReferenceDataForProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  export interface GetReferenceDataForProductResponse {
    Data: { [key: string]: { [key: string]: any } };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * This service lets you create and update a set of products and their content.
   * 
   * The set is products is hierarchical in nature. For example, if you have a shirt that comes in 3 colors and 5 sizes per color, you would normally define a root product that then contains the 3 color products
   * and each color product would contain the 5 different sizes, which would be the SKU. How you define the hierarchy is up to you, it's also fine to have the 3 color products as root products and only have them
   * contain the SKUs, or to forego a hierarchy entirely and just create the SKUs, but creating the root and color products is useful for search and frontend purposes.
   * 
   * An example request of a hierarchy:
   * 
   * ```
   * {
   *   "ID": "special_shirt",
   *   "Name": "Special Shirt",
   *   "TaxCode": "High",
   *   "Variations": {
   *     "Property": "color_code",
   *     "LogicalLevel": "color",
   *     "Products": [
   *       {
   *         "ID": "special_shirt_blue",
   *         "Name": "Special Shirt - Blue",
   *         "VariationValues": [
   *           {
   *             "LanguageID": null,
   *             "Value": "BLUE"
   *           }
   *         ],
   *         "Variations": {
   *           "Property": "size_code",
   *           "LogicalLevel": "size",
   *           "Products": [
   *             {
   *               "ID": "sku_special_shirt_blue_xl",
   *               "VariationValues": [
   *                 {
   *                   "LanguageID": null,
   *                   "Value": "XL"
   *                 }
   *               ]
   *             }
   *           ]
   *         }
   *       }
   *     ]
   *   }
   * }
   * ```
   */
  export interface ImportProducts {
    /**
     * A string that identifies the system that was used to call this service. In combination with a product's ID, this is what uniquely identifies the product in EVA.
     * The SystemID must be the same for every call, if it's different from a previous call then new products will be created.
     */
    SystemID: string;
    /**
     * Determines the behavior of this service.
     */
    Type: ImportProducts.ImportType;
    /**
     * The LogicalLevel that will be assigned to the root level of a product hierarcy. If not specified a default value is used.
     * 
     * See ProductVariation.LogicalLevel for more information.
     * @default root
     */
    RootLogicalLevel?: string;
    /**
     * A string that identifies the category in which the ProductImages will be stored. It's optional, when not provided the SystemID will be used instead.
     */
    ImageBlobCategory?: string;
    /**
     * The list of products to create or update. If a product does not exist it's created and if a product with the same ID and SystemID already exists it is updated.
     */
    Products: ImportProducts.Product[];
    /**
     * By default this service only blocks for the time that it takes to store the requested modifications. After that time, some processing still needs to happen before the product is fully ready and searchable, but this happens in the background.
     * If this parameter is specified as true then this service blocks for the full duration that it takes to complete the full process.
     */
    WaitForProductComposition?: boolean;
    /**
     * When not specified, or explicitly specified as `false`, images will be downloaded while you wait and may cause this service to run for a long time. Defaults to `true`.
     */
    DownloadImagesInBackground?: boolean;
    /**
     * Custom properties used in product content can be defined by specifying their definition in this collection.
     */
    CustomPropertyTypes?: ImportProducts.ProductPropertyConfiguration[];
    /**
     * The layer on which the content will be stored. Defaults to 'ImportProducts-Base'. If you want to store the content on a different layer, you can specify it here. If the content also
     * specifies a language/country this will be combined with the provided layer name to create a layer name like '<name> <language> <country>', so the provided LayerName is not necessarily the absolute
     * value that is used for the layer, only if you leave LanguageID/CountryID as null on the content.
     */
    LayerName?: string;
  }

  export namespace ImportProducts {
    export const enum ProductPropertyDataTypes {
      String = 0,
      Int = 1,
      Double = 2,
      Bool = 3,
      Date = 4
    }

    export const enum ProductPropertyIndexTypes {
      /**
       * The property is not indexed, meaning it's not searchable.
       */
      NotIndexed = 0,
      /**
       * The property is indexed but not analyzed, which means it will only result in a hit if it's an exact match.
       */
      IndexedNotAnalyzed = 1,
      /**
       * The property is indexed and analyzed, meaning the value is broken down to its individual tokens according to the index analyzer.
       */
      IndexedAnalyzed = 2
    }

    export interface ProductPropertyConfiguration {
      /**
       * The ID of the custom property, by convention in snake_case.
       */
      ProductPropertyTypeID?: string;
      /**
       * The ID of the category the property falls into. Defaults to `default`.
       */
      CategoryID?: string;
      /**
       * Defines the data type of the property.
       */
      DataType?: ImportProducts.ProductPropertyDataTypes;
      /**
       * Defines whether and how the content of this property is searchable. Note that this is a flags enum so IndexType can both be IndexedNotAnalyzed and IndexedAnalyzed, meaning the field will be indexed in an analyzed *and* a non-analyzed manner.
       */
      IndexType?: ImportProducts.ProductPropertyIndexTypes;
      /**
       * Determines if the property allows multiple values or if it's just a single value.
       */
      IsArray?: boolean;
      /**
       * If provided, the values of this property will be copied to the provided property on the parent products.
       * This can be useful if you have a `clothing_color` property on the `color` logical level and would like to have an `available_colors` property on the root level that contains all the colors that are available for the product.
       * In that example the value of the CopyToParentProductPropertyTypeID on the `clothing_color` property would be `available_colors` and the value of the CopyToParentProductPropertyTypeID on the `available_colors` property would be null.
       * If there is no property yet with this name, it will be automatically created with the same data type as the source property.
       * 
       * If there already exists a property with this name, the data type must match the data type of the source property and the IsArray property must be true as the parent products will be given multiple values from their children.
       */
      CopyToParentProductPropertyTypeID?: string;
    }

    export interface ProductAssortment {
      AssortmentCode: string;
      ProductStatus: ImportProducts.ProductStatus;
      StartDate?: string;
      EndDate?: string;
      AnnouncementDate?: string;
      PreSaleDate?: string;
      ShipmentDate?: string;
    }

    export interface Product {
      /**
       * The unique identifier for the product.
       */
      ID: string;
      /**
       * The culture independent name of the product by which it can be recognized even if it otherwise does not have any content. Required unless Type = ContentOnly.
       */
      Name?: string;
      /**
       * The custom identifier for the product.
       */
      CustomID?: string;
      /**
       * The (optional) brand of the product. Only has to be specified on the root level of a hierarchy.
       */
      Brand?: string;
      /**
       * The (optional) ledger class ID of the product. Only has to be specified on the root level of a hierarchy.
       */
      LedgerClassID?: string;
      /**
       * The TaxCode determines how sales tax applies to this product. This must refer to an existing TaxCode in EVA, if it doesn't exist yet an error is returned.
       * 
       * Required unless Type = ContentOnly.
       * 
       * The string expected here is the Name property of the tax code, so 'High' for TaxCode High.
       * 
       * EVA defines the following default tax codes:
       * 
       * - High
       * - Intermediate
       * - Low
       * - Zero
       * - Exempt
       * 
       * In a hierarchy, the TaxCode only has to be specified for the root level; all products part of the hierarchy will always have the same TaxCode.
       */
      TaxCode?: string;
      /**
       * An (optional) list of barcodes for this product. A barcode should be globally unique and any duplicate barcodes will result in an error.
       */
      Barcodes?: string[];
      /**
       * An (optional) list of barcodes with the associated quantity and unit of measure for this product. A barcode should be globally unique and any duplicate barcodes will result in an error.
       */
      UnitBarcodes?: ImportProducts.UnitBarcode[];
      /**
       * Defines what kind of product it is. If the product is part of a hierarchy, this is only relevant for the SKU level, all other levels will have type Configurable.
       */
      Type?: ImportProducts.ProductType;
      /**
       * Defines the status of a product.
       */
      Status?: ImportProducts.ProductStatus;
      /**
       * Defines the variations hierarchy of the product.
       */
      Variations?: ImportProducts.ProductVariation;
      /**
       * The per-language content of the product.
       * 
       * A small set of default properties has been defined, such as Name and MarketingDescription. Additional content can be defined inside `CustomContent`, which is a key-value container
       * where the key is the name of a property and value should be a single scalar value (string, number, boolean or null) or an array of values.
       * 
       * If LanguageID is left null, this will be the fallback content for languages that have no content of their own.
       * For example, you may offer a French website but you only have English product content, if you specify your English content with LanguageID = null
       * then products on the French website will have the English content.
       * 
       * In general, it's a good idea to choose a language that will serve as the default fallback language and to not specify LanguageID for that language's content so it will function as the fallback content.
       */
      Content?: ImportProducts.ProductContent[];
      /**
       * Defines the per-language value of the variation property for this product. For example, if the variation property is `color` then the value could be 'red' or 'blue'.
       * 
       * If the variation value isn't language sensitive then the LanguageID should be left null. If there isn't an item where LanguageID is null, all languages for which this product has content must have
       * a value for the variation property.
       */
      VariationValues?: ImportProducts.ProductVariationValue[];
      /**
       * Set to `True` to delete the product.
       */
      IsDeleted?: boolean;
      /**
       * In case a type GiftCard product is uploaded, its `GiftCardType` or `GiftCardConfigurationID` is mandatory to connect to its provider.
       * Entity type: GiftCardConfiguration
       */
      GiftCardConfigurationID?: number;
      /**
       * In case a type GiftCard product is uploaded, its `GiftCardType` or `GiftCardConfigurationID` is mandatory to connect to its provider.
       */
      GiftCardType?: string;
      /**
       * In case a type GiftCard product is uploaded, this defines if the product is digital, so it will not require a serialnumber during checkout.
       */
      GiftCardIsDigital?: boolean;
      /**
       * In case a type GiftCard product is uploaded, this defines the optional data for configuration.
       */
      GiftCardData?: any;
      /**
       * If specified this will add the product to those assortments.
       */
      Assortments?: ImportProducts.ProductAssortment[];
      /**
       * The CommodityCode is used for customs declarations and product tariffs. Can be used to store the HS code of the product.
       */
      CommodityCode?: string;
      /**
       * The country of origin for the product. Value must conform to the 2-letter country code as defined by ISO 3166-1.
       */
      CountryOfOriginID?: string;
      /**
       * Optional field that can be used to store the manufacturer's identifier for this product.
       */
      ManufacturerID?: string;
      /**
       * Optional field to indicate a serialnumber will be required when selling this product.
       */
      RequireSerialNumberForSale?: boolean;
      /**
       * Optional field to indicate what kind of serial number requirements apply to this product.
       * Entity type: SerialNumberConfiguration
       */
      SerialNumberConfigurationID?: number;
      /**
       * Optional field to set up the product requirements on a product.
       */
      ProductRequirements?: ImportProducts.ProductRequirement[];
      /**
       * Determines whether or not any images set on a parent product will inherit down to its children. This defaults to true.
       * Can be useful to give the root product of a hierarchy an ensemble of the available colors, without inheriting these images to the children.
       */
      ImagesWillBeInheritedByChildren?: boolean;
    }

    export interface ProductRequirement {
      Name: string;
      DataType: EVA.Core.ProductRequirementDataTypes;
      OrderType?: EVA.Core.OrderTypes;
      IsArray?: boolean;
      IsRequired?: boolean;
      EnumValues?: { [key: string]: string };
      BackendID?: string;
      Handler?: string;
    }

    export interface ProductVariation {
      /**
       * The property on which the product hierarchy varies. For example 'size' or 'color'.
       */
      Property: string;
      /**
       * The LogicalLevel of a product defines the 'level' on which the product can be thought to sit in the hierarchy.
       * 
       * For example, the various different colors of a shirt can be thought to have LogicalLevel = 'color' and the actual SKU might have LogicalLevel = 'size' or 'sku'.
       * 
       * You are free to use any value, but it's recommended to keep this consistent between all products, or at least between products of the same kind, as it's a useful way to
       * structure a frontend by for example filtering on the color level and only show sizes when navigating to the product detail.
       */
      LogicalLevel: string;
      /**
       * The actual variations of the product on the specified LogicalLevel.
       */
      Products: ImportProducts.Product[];
    }

    export interface ProductVariationValue {
      /**
       * Can be left null if the variation value isn't culture dependent.
       * Entity type: Language
       */
      LanguageID?: string;
      /**
       * Can be left null if the variation value isn't culture dependent.
       * Entity type: Country
       */
      CountryID?: string;
      Value?: string;
    }

    export interface ProductContent {
      /**
       * Entity type: Language
       */
      LanguageID?: string;
      /**
       * Entity type: Country
       */
      CountryID?: string;
      MarketingDescription?: string;
      Name?: string;
      ShortDescription?: string;
      LongDescription?: string;
      /**
       * The publication status determines the visibility of the product. A product generally won't be visible to non-employee users unless it has publication status 'public'.
       */
      PublicationStatuses?: string[];
      Images?: ImportProducts.ProductImage[];
      Tags?: string[];
      UniqueSellingPointTexts?: string[];
      UniqueSellingPointBlobIDs?: string[];
      XDimension: ImportProducts.ProductDimension;
      YDimension: ImportProducts.ProductDimension;
      CustomContent?: { [key: string]: any };
    }

    export interface ProductDimension {
      PropertyName: string;
      Value: string;
    }

    export interface ProductImage {
      /**
       * The unique identifier for the image. Optional, if null then an ID is generated based on the hash of the ImageUrl
       */
      ID?: string;
      /**
       * Where can the image be downloaded? Required unless ImageData is provided.
       */
      ImageUrl?: string;
      /**
       * How the image should be stored. Usually empty.
       */
      LocationType?: string;
      /**
       * The image mime type. Optional, if null then the mime type will be guessed by the ImageUrl
       */
      MimeType?: string;
      /**
       * The human readable name of the image. Optional, if null then the name will be the ID of the image.
       */
      Name?: string;
      /**
       * The raw image data. Only required if ImageUrl is left empty.
       */
      ImageData: string;
      /**
       * The sequence number of the image, can be used to determine the display order of images and will be used to determine the primary image of the product.
       * 
       * Smaller numbers mean a higher priority.
       * 
       * By default this will be null and the sequence of the images will be determined by their ordering in the Images array.
       * 
       * The number provided here does not necessarily translate to the actual sequence number stored in EVA, only the relative order will be used. Given the sequence 1, 2, 4, 7 the actual sequence stored in EVA
       * will be 0, 1, 2, 3.
       */
      Sequence?: number;
      /**
       * If an image is specified as IsPrimaryImage = true then it will be used as the primary image on the product.
       * If there is for some reason multiple images for a product that have IsPrimaryImage set to true then the last image that have this set will become the actual primary image.
       */
      IsPrimaryImage?: boolean;
    }

    export interface ProductStatus {
      /**
       * The product will be unavailable until the pre-order date.
       */
      PreRelease?: boolean;
      /**
       * The product won't be available for endless aisle pickup from shop.
       */
      DeliveryOnly?: boolean;
      /**
       * The product will be unavailable for delivery.
       */
      DisableDelivery?: boolean;
      /**
       * The product won't be available from a shop at all.
       */
      DisablePickup?: boolean;
      /**
       * The product won't be orderable if there is no available stock.
       */
      DisableBackorder?: boolean;
      /**
       * Indicates the product won't be back in store and all current stock is all there is going to be.
       */
      UseUp?: boolean;
      /**
       * Indicates the product isn't sellable.
       */
      NonSellable?: boolean;
    }

    export interface UnitBarcode {
      /**
       * The barcode of the product. Must be unique.
       */
      Barcode: string;
      /**
       * The quantity of the product (in the given unit of measure) that the barcode represents.
       */
      Quantity: number;
      /**
       * The name of the unit of measure to use for this barcode. If not specified, 'Each' will be used. Must be created beforehand.
       */
      UnitOfMeasureName?: string;
    }

    export interface ProductType {
      /**
       * A normal stock-keeping product. This will be the default if not specified.
       */
      Stock?: boolean;
      /**
       * A giftcard product.
       */
      GiftCard?: boolean;
      /**
       * A non-physical product that represents a service of some sort.
       */
      Service?: boolean;
      /**
       * Indicates the product is of a bundle type.
       */
      BundleProduct?: boolean;
      /**
       * Indicates if custom pricing on the product is allowed.
       */
      AllowCustomPricing?: boolean;
      /**
       * Indicates if the product is an insurance product.
       */
      Insurance?: boolean;
    }

    export const enum ImportType {
      /**
       * A normal import; creates and updates the product structure and stores content, if present.
       */
      Normal = 0,
      /**
       * Only imports content, does not create or update the products themselves. It's required that the products already exist.
       */
      ContentOnly = 1,
      /**
       * A dry-run, does not make any modifications, only returns a representation of the product structure that would be generated in the response.
       */
      PreviewOnly = 2
    }

  }

  export interface ImportProductsResponse {
    /**
     * The EVA IDs of the products were touched by this call.
     * Entity type: Product
     */
    UpdatedProductIDs: number[];
    /**
     * The EVA IDs of the products that were newly created by this call.
     * Entity type: Product
     */
    CreatedProductIDs: number[];
    /**
     * If the Type of the request was PreviewOnly this property will contain the product structure that would be generated.
     */
    PreviewResult: string;
    /**
     * Contains an entry for every product mapping the input string ID to its internal EVA integer ID.
     */
    ProductMap: ProductMapItem[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ProductMapItem {
    BackendID: string;
    /**
     * Entity type: Product
     */
    ID: number;
  }

  /**
   * Create a new product capacity plan, which limits the stock of a product based on the capacity of production.
   */
  export interface CreateProductCapacityPlan {
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
    Name?: string;
    Description?: string;
    ProductCapacityTimeFrame?: EVA.Core.ProductCapacityTimeFrame;
    /**
     * Min: 0
     */
    Capacity?: number;
    /**
     * Range: 1-14
     */
    AvailabilityCalculationCutOffInDays?: number;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    ProductSearchFilters?: { [key: string]: EVA.Core.FilterModel };
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    OverwriteExistingProductCapacityPlans?: boolean;
  }

  export interface CreateProductCapacityPlanResponse {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a product capacity plan.
   * When the product capacity plan is changed from static to dynamic, and this is as requested, the service requires the `AllowStaticToDynamicTypeChange` to be set to true.
   * If `AllowStaticToDynamicTypeChange` is false, the service will return an error when the product capacity plan switches from static to dynamic.
   */
  export interface UpdateProductCapacityPlan {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID: number;
    Name: string;
    Description?: string;
    ProductCapacityTimeFrame: EVA.Core.ProductCapacityTimeFrame;
    /**
     * Min: 0
     */
    Capacity: number;
    /**
     * Min: 1
     */
    AvailabilityCalculationCutOffInDays?: number;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    ProductSearchFilters?: { [key: string]: EVA.Core.FilterModel };
    AllowStaticToDynamicTypeChange?: boolean;
  }

  /**
   * Deletes a product capacity plan.
   */
  export interface DeleteProductCapacityPlan {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID: number;
  }

  /**
   * Set a product capacity plan exception. The capacity is prioritized as follows;
   * - Exception based on date (time is stripped from the given datetime).
   * - Exception based on day of week.
   * - Default product capacity plan capacity
   */
  export interface SetProductCapacityPlanException {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID: number;
    DayOfWeek?: EVA.Core.DayOfWeek;
    Date?: string;
    /**
     * Min: 0
     */
    Capacity: number;
  }

  /**
   * Deletes a product capacity plan exception
   */
  export interface DeleteProductCapacityPlanException {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID: number;
    DayOfWeek?: EVA.Core.DayOfWeek;
    Date?: string;
  }

  /**
   * Add a product to product capacity plan. This is only allowed on static product capacity plans. A product can not be in more then one static product capacity plan per supplier.
   */
  export interface AddProductsToProductCapacityPlan {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    OverwriteExistingProductCapacityPlans: boolean;
  }

  /**
   * Remove a product to product capacity plan. This is only allowed on static product capacity plans.
   */
  export interface RemoveProductsFromProductCapacityPlan {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
  }

  export interface ListProductCapacityPlans {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ProductCapacityPlanFilter>;
  }

  export interface ListProductCapacityPlansResponse {
    Result?: EVA.Core.PagedResult<ListProductCapacityPlansResponse.Data>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListProductCapacityPlansResponse {
    export interface Data {
      /**
       * Entity type: ProductCapacityPlan
       */
      ID: number;
      /**
       * Entity type: OrganizationUnit
       */
      SupplierOrganizationUnitID: number;
      SupplierOrganizationUnitName: string;
      Name: string;
      Description?: string;
      ProductCapacityTimeFrame: EVA.Core.ProductCapacityTimeFrame;
      Capacity: number;
      AvailabilityCalculationCutOffInDays?: number;
      IsDynamic: boolean;
    }

  }

  /**
   * Get ProductCapacityPlan by the given ID
   */
  export interface GetProductCapacityPlanByID {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID: number;
  }

  export interface GetProductCapacityPlanByIDResponse {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID: number;
    SupplierOrganizationUnitName: string;
    Name: string;
    Description?: string;
    IsDynamic: boolean;
    ProductCapacityTimeFrame: EVA.Core.ProductCapacityTimeFrame;
    Capacity: number;
    AvailabilityCalculationCutOffInDays?: number;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    ProductSearchFilters: { [key: string]: EVA.Core.FilterModel };
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    Exceptions: GetProductCapacityPlanByIDResponse.ProductCapacityPlanException[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetProductCapacityPlanByIDResponse {
    export interface ProductCapacityPlanException {
      DayOfWeek?: EVA.Core.DayOfWeek;
      Date?: string;
      Capacity: number;
    }

  }

  /**
   * Get ProductCapacityPlans by the given ProductID
   */
  export interface GetProductCapacityPlansByProductID {
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  export interface GetProductCapacityPlansByProductIDResponse {
    ProductCapacityPlans: GetProductCapacityPlansByProductIDResponse.Plan[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetProductCapacityPlansByProductIDResponse {
    export interface Plan {
      /**
       * Entity type: ProductCapacityPlan
       */
      ID: number;
      /**
       * Entity type: OrganizationUnit
       */
      SupplierOrganizationUnitID: number;
      Name: string;
      Description?: string;
      IsDynamic: boolean;
      ProductCapacityTimeFrame: EVA.Core.ProductCapacityTimeFrame;
      Capacity: number;
      AvailabilityCalculationCutOffInDays?: number;
      IsActive: boolean;
    }

  }

  export interface UploadProductExcel {
    Data: string;
  }

  export interface UploadProductExcelResponse {
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ExportProductBaseExcel {
  }

  export interface GenerateProductExcelSample {
  }

  export interface UploadProductContentExcel {
    LanguageID?: string;
    Data: string;
  }

  export interface UploadProductContentExcelResponse {
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GenerateProductContentExcelSample {
  }

  export interface ExportProductContentExcel {
    Query?: string;
    Filters?: { [key: string]: EVA.Core.FilterModel };
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface UploadProductBarcodeExcel {
    Data: string;
  }

  export interface UploadProductBarcodeExcelResponse {
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GenerateProductBarcodeExcelSample {
  }

  /**
   * Create a new ProductRelationType.
   */
  export interface CreateProductRelationType {
    /**
     * This is the primary name the relation. For example; if primary name is 'Blocks', the secondary name could be something in the lines of 'Blocked by'.
     */
    PrimaryName: string;
    /**
     * This is the secondary name the relation. For example; if primary name is 'Blocks', the secondary name could be something in the lines of 'Blocked by'.
     */
    SecondaryName?: string;
    BackendID?: string;
    /**
     * The direction of the relation. This can either be;
     * Unidirectional; one-way relation. For example 'parent-child' or 'replaced by'
     * Bidirectional; two-way relation. For example 'maybe your also intrested in' or 'others also bought'
     * Group; collective relation which allows for many products as part of a group. For example 'all products that are blue' or 'all products from summer-2022'. This type does not accept a StockRelationType.
     */
    Direction: EVA.Core.ProductRelationDirection;
    /**
     * Indicates which stock relation type the relation has. In most cases this will be 'None'. The following options are available:
     * None; No stock relation
     * Replacing; When the PrimaryProduct is no longer in stock, the system can automaticly use the RelatedProduct as it's replacement (or vice versa if `Direction` is set to `Bidirectional`).
     */
    StockRelationType: EVA.Core.StockRelationType;
    /**
     * You can create dynamic bi- or uni directional relations by defining a `DynamicRelationProductID` in combination with a `ProductSearchTemplateID` or `ProductSearchFilters`.
     * By setting the `UseDynamicRelationProductAsRelatedProduct` to false (default), the `DynamicRelationProductID` will be considered as the primary product and the result from the `ProductSearchTemplateID` or `ProductSearchFilters` will be considered as related products
     * By setting the `UseDynamicRelationProductAsRelatedProduct` to true , the result from the `ProductSearchTemplateID` or `ProductSearchFilters` will be considered as primary products and the `DynamicRelationProductID` will be considered as the related product
     * Entity type: Product
     */
    DynamicRelationProductID?: number;
    UseDynamicRelationProductAsRelatedProduct?: boolean;
    /**
     * Only available when Direction is set to `Group`.
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    /**
     * Only available when Direction is set to `Group`.
     */
    ProductSearchFilters?: { [key: string]: EVA.Core.FilterModel };
    Relations?: CreateProductRelationType.Relation[];
  }

  export namespace CreateProductRelationType {
    export interface Relation {
      /**
       * Entity type: Product
       */
      PrimaryProductID: number;
      /**
       * Entity type: Product
       */
      RelatedProductID?: number;
      Sequence?: number;
    }

  }

  export interface CreateProductRelationTypeResponse {
    /**
     * Entity type: ProductRelationType
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update the the ProductRelationType.
   */
  export interface UpdateProductRelationType {
    /**
     * Entity type: ProductRelationType
     */
    ID: number;
    BackendID?: string;
    /**
     * This is the primary name the relation. For example; if primary name is 'Blocks', the secondary name could be something in the lines of 'Blocked by'.
     */
    PrimaryName: string;
    /**
     * This is the secondary name the relation. For example; if primary name is 'Blocks', the secondary name could be something in the lines of 'Blocked by'.
     */
    SecondaryName?: string;
    /**
     * Entity type: Product
     */
    DynamicRelationProductID?: number;
    UseDynamicRelationProductAsRelatedProduct?: boolean;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    ProductSearchFilters?: { [key: string]: EVA.Core.FilterModel };
    AllowTypeChange?: boolean;
  }

  /**
   * Deletes the ProductRelationType.
   */
  export interface DeleteProductRelationType {
    /**
     * Entity type: ProductRelationType
     */
    ID: number;
  }

  /**
   * List the ProductRelationTypes.
   */
  export interface ListProductRelationTypes {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ProductRelationTypesFilter>;
  }

  export interface ListProductRelationTypesResponse {
    Result?: EVA.Core.PagedResult<ListProductRelationTypesResponse.Data>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListProductRelationTypesResponse {
    export interface Data {
      /**
       * Entity type: ProductRelationType
       */
      ID: number;
      PrimaryName: string;
      SecondaryName?: string;
      BackendID: string;
      Direction: EVA.Core.ProductRelationDirection;
      StockRelationType: EVA.Core.StockRelationType;
      IsDynamic: boolean;
      /**
       * Entity type: Product
       */
      DynamicRelationProductID?: number;
      UseDynamicRelationProductAsRelatedProduct: boolean;
    }

  }

  /**
   * Get ProductRelationType by ID
   */
  export interface GetProductRelationTypeByID {
    /**
     * Entity type: ProductRelationType
     */
    ID: number;
  }

  export interface GetProductRelationTypeByIDResponse {
    /**
     * Entity type: ProductRelationType
     */
    ID: number;
    PrimaryName: string;
    SecondaryName?: string;
    BackendID: string;
    Direction: EVA.Core.ProductRelationDirection;
    StockRelationType: EVA.Core.StockRelationType;
    IsDynamic: boolean;
    /**
     * Entity type: Product
     */
    DynamicRelationProductID?: number;
    UseDynamicRelationProductAsRelatedProduct: boolean;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    ProductSearchFilters?: { [key: string]: EVA.Core.FilterModel };
    Relations: GetProductRelationTypeByIDResponse.Relation[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetProductRelationTypeByIDResponse {
    export interface Relation {
      /**
       * Entity type: ProductRelation
       */
      ID?: number;
      /**
       * Entity type: Product
       */
      PrimaryProductIDs?: number[];
      /**
       * Entity type: Product
       */
      RelatedProductIDs?: number[];
      Sequence?: number;
    }

  }

  /**
   * List all product relations for the given Product and (optional) ProductRelationType
   */
  export interface ListProductRelations {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: ProductRelationType
     */
    ProductRelationTypeID?: number;
  }

  export interface ListProductRelationsResponse {
    RelationTypeCollections: ListProductRelationsResponse.RelationTypeCollection[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListProductRelationsResponse {
    export interface RelationTypeCollection {
      /**
       * Entity type: ProductRelationType
       */
      ID: number;
      PrimaryName: string;
      SecondaryName?: string;
      BackendID: string;
      Direction: EVA.Core.ProductRelationDirection;
      StockRelationType: EVA.Core.StockRelationType;
      IsDynamic: boolean;
      /**
       * Entity type: Product
       */
      DynamicRelationProductID?: number;
      UseDynamicRelationProductAsRelatedProduct: boolean;
      Relations: ListProductRelationsResponse.Relation[];
    }

    export interface Relation {
      /**
       * Entity type: ProductRelation
       */
      ID?: number;
      /**
       * Entity type: Product
       */
      PrimaryProductIDs?: number[];
      /**
       * Entity type: Product
       */
      RelatedProductIDs?: number[];
      Sequence?: number;
    }

  }

  /**
   * Create a relation of the given ProductRelationType for the given ProductRelations
   */
  export interface AddProductRelations {
    /**
     * Entity type: ProductRelationType
     */
    ID: number;
    Relations?: AddProductRelations.ProductRelation[];
  }

  export namespace AddProductRelations {
    export interface ProductRelation {
      /**
       * Entity type: Product
       */
      PrimaryProductID: number;
      /**
       * Entity type: Product
       */
      RelatedProductID?: number;
      Sequence?: number;
    }

  }

  /**
   * Update a relation of the given ProductRelationType for the given ProductRelations
   */
  export interface UpdateProductRelations {
    /**
     * Entity type: ProductRelationType
     */
    ID: number;
    Relations?: UpdateProductRelations.ProductRelation[];
  }

  export namespace UpdateProductRelations {
    export interface ProductRelation {
      /**
       * Entity type: Product
       */
      PrimaryProductID: number;
      /**
       * Entity type: Product
       */
      RelatedProductID?: number;
      Sequence?: number;
    }

  }

  /**
   * Remove the given ProductRelations by the ProductRelationID
   */
  export interface RemoveProductRelationsByID {
    /**
     * Entity type: ProductRelation
     */
    IDs: number[];
  }

  /**
   * Remove the given ProductRelations by the ProductRelationID
   */
  export interface RemoveProductRelationsByProductID {
    /**
     * Entity type: Product
     */
    IDs: number[];
    /**
     * Entity type: ProductRelationType
     */
    ProductRelationTypeID?: number;
  }

  /**
   * Search product property types
   */
  export interface SearchProductPropertyTypes {
    Query?: string;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface SearchProductPropertyTypesResponse {
    Result?: EVA.Core.PagedResult<ProductPropertyTypeModel>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List product property types
   */
  export interface ListProductPropertyTypes {
    PageConfig?: EVA.Core.PageConfig<ListProductPropertyTypesFilter>;
  }

  export interface ListProductPropertyTypesFilter {
    /**
     * Fuzzy search for ID of the property
     * Entity type: ProductPropertyType
     */
    ID?: string;
    /**
     * Non-fuzzy search for properties for the given ID's
     * Entity type: ProductPropertyType
     */
    IDs?: string[];
    /**
     * Entity type: ProductPropertyCategory
     */
    CategoryID?: string;
    DataTypeID?: ProductPropertyTypeDataTypes;
    DataType?: ProductPropertyTypeDataTypes;
    Intent?: ProductPropertyTypeIntents;
    SearchType?: ProductPropertyTypeSearchTypes;
    SearchTypeMatchingMethod?: ProductPropertyTypeSearchTypeMatchingMethod;
    IsArray?: boolean;
    IsEnum?: boolean;
    InheritanceType?: ProductPropertyTypeInheritanceTypes;
    Options?: ProductPropertyTypeOptions;
  }

  export const enum ProductPropertyTypeSearchTypeMatchingMethod {
    MatchesAll = 0,
    MatchesAtLeastOne = 1
  }

  export const enum ProductPropertyTypeInheritanceTypes {
    NormalInheritance = 0,
    NoInheritance = 1
  }

  export const enum ProductPropertyTypeOptions {
    None = 0,
    AlphabeticallySortable = 1
  }

  export interface ListProductPropertyTypesResponse {
    Result?: EVA.Core.PagedResult<ListProductPropertyTypesDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListProductPropertyTypesDto {
    /**
     * Entity type: ProductPropertyType
     */
    ID: string;
    DisplayName: string;
    /**
     * Entity type: ProductPropertyCategory
     */
    CategoryID: string;
    CategoryDisplayName: string;
    DataType: ProductPropertyTypeDataTypes;
    Intent: ProductPropertyTypeIntents;
    SearchType: ProductPropertyTypeSearchTypes;
    InheritanceType: ProductPropertyTypeInheritanceTypes;
    Options: ProductPropertyTypeOptions;
  }

  /**
   * Get product property type
   */
  export interface GetProductPropertyTypeByID {
    ID: string;
    /**
     * Entity type: Revision
     */
    RevisionID?: number;
  }

  export interface GetProductPropertyTypeByIDResponse {
    /**
     * Entity type: ProductPropertyType
     */
    ID: string;
    /**
     * Entity type: Catalog
     */
    CatalogID?: number;
    LayersWithContent?: LayerWithContentDto[];
    /**
     * Entity type: ProductPropertyCategory
     */
    CategoryID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get available enum values for product property type
   */
  export interface GetAvailableEnumValuesForProductPropertyType {
    TypeID?: string;
  }

  export interface GetAvailableEnumValuesForProductPropertyTypeResponse {
    /**
     * Entity type: ProductPropertyType
     */
    Values: { [key: number]: string };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new ProductPropertyType
   */
  export interface CreateProductPropertyType {
    TypeID: string;
    CategoryID: string;
    DataType: ProductPropertyTypeDataTypes;
    SearchType: ProductPropertyTypeSearchTypes;
    Options?: ProductPropertyTypeOptions;
    IsArray?: boolean;
    IsEnum?: boolean;
    EditFunctionality?: string;
    CopyToParentProductPropertyTypeID?: string;
  }

  /**
   * Update a ProductPropertyType
   * 
   * The SearchType or Options can be changed
   * 
   * For the Edits there is a specific set of properties that can be set:
   * - display_name
   * - name
   * - unit_of_measure
   */
  export interface EditProductPropertyType {
    /**
     * Entity type: ProductPropertyType
     */
    ID?: string;
    Edits?: ProductPropertyTypeEdit[];
    SearchType?: ProductPropertyTypeSearchTypes;
    Options?: ProductPropertyTypeOptions;
    CopyToParentProductPropertyTypeID?: string;
  }

  export interface ProductPropertyTypeEdit {
    /**
     * Entity type: ContentLayer
     */
    LayerID?: number;
    Content?: { [key: string]: any };
  }

  export interface EditProductPropertyTypeResponse {
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Deletes the given product property type based on its identifier. This is irreversible and will cause products that are composed after this action to no longer have the property.
   */
  export interface DeleteProductPropertyType {
    ID?: string;
  }

  /**
   * Get product property type enum value
   */
  export interface GetProductPropertyTypeEnumValueByID {
    /**
     * Entity type: ProductPropertyTypeEnumValue
     */
    ID: number;
  }

  export interface GetProductPropertyTypeEnumValueByIDResponse {
    /**
     * Entity type: ProductPropertyTypeEnumValue
     */
    ID: number;
    ProductPropertyTypeID: string;
    Identifier: number;
    Value: string;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create product property type enum value
   */
  export interface CreateProductPropertyTypeEnumValue {
    ProductPropertyTypeID: string;
    Identifier: number;
    Value?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  export interface CreateProductPropertyTypeEnumValueResponse {
    /**
     * Entity type: ProductPropertyTypeEnumValue
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update product property type enum value
   */
  export interface UpdateProductPropertyTypeEnumValue {
    /**
     * Entity type: ProductPropertyTypeEnumValue
     */
    ID: number;
    Identifier: number;
    Value?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  /**
   * Delete product property type enum value
   */
  export interface DeleteProductPropertyTypeEnumValue {
    /**
     * Entity type: ProductPropertyTypeEnumValue
     */
    ID: number;
  }

  /**
   * List product property type enum values
   */
  export interface ListProductPropertyTypeEnumValues {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListProductPropertyTypeEnumValuesResponse {
    Result?: EVA.Core.PagedResult<ListProductPropertyTypeEnumValuesResponse.ProductPropertyTypeEnumValueDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListProductPropertyTypeEnumValuesResponse {
    export interface ProductPropertyTypeEnumValueDto {
      /**
       * Entity type: ProductPropertyTypeEnumValue
       */
      ID: number;
      ProductPropertyTypeID: string;
      Identifier: number;
      Value: string;
      LanguageID: string;
      CountryID: string;
    }

  }

  export interface CreateProductPropertyCategory {
    CategoryID?: string;
    EditFunctionality?: string;
  }

  export interface CreateProductPropertyCategoryResponse {
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface EditProductPropertyCategory {
    ID?: string;
    Edits?: ProductPropertyCategoryEdit[];
  }

  export interface ProductPropertyCategoryEdit {
    /**
     * Entity type: ContentLayer
     */
    LayerID: number;
    Content: { [key: string]: any };
  }

  export interface EditProductPropertyCategoryResponse {
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetProductPropertyCategoryByID {
    ID?: string;
    /**
     * Entity type: Revision
     */
    RevisionID: number;
  }

  export interface GetProductPropertyCategoryByIDResponse {
    ID: string;
    /**
     * Entity type: Catalog
     */
    CatalogID?: number;
    LayersWithContent: LayerWithContentDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListProductPropertyCategories {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListProductPropertyCategoriesResponse {
    Result?: EVA.Core.PagedResult<ListProductPropertyCategoriesDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListProductPropertyCategoriesDto {
    ID: string;
    DisplayName: string;
  }

  /**
   * Returns the AssortmentContentCultures
   */
  export interface GetAssortmentContentCultures {
  }

  export interface GetAssortmentContentCulturesResponse {
    ContentCultures: GetAssortmentContentCulturesResponse.AssortmentContentCulture[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAssortmentContentCulturesResponse {
    export interface AssortmentContentCulture {
      /**
       * Entity type: Assortment
       */
      AssortmentID: number;
      AssortmentName: string;
      /**
       * Entity type: Culture
       */
      CultureID: string;
      /**
       * Entity type: Language
       */
      LanguageID: string;
      /**
       * Entity type: Country
       */
      CountryID: string;
    }

  }

  /**
   * Creates a new AssortmentContentCulture.
   * If it already exists, nothing will be done.
   */
  export interface CreateAssortmentContentCulture {
    /**
     * Entity type: Assortment
     */
    AssortmentID: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  /**
   * Deletes an existing AssortmentContentCulture. If there are any existing mappings to this ContentCulture, they will also be deleted.
   */
  export interface DeleteAssortmentContentCulture {
    /**
     * Entity type: Assortment
     */
    AssortmentID: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  /**
   * Returns the current ContentCultureMapping.
   */
  export interface ListContentCultureMapping {
    PageConfig?: EVA.Core.PageConfig<ContentCultureMappingFilter>;
  }

  export interface ContentCultureMappingFilter {
    /**
     * Entity type: Language
     */
    SourceLanguageID: string;
    /**
     * Entity type: Country
     */
    SourceCountryID: string;
    /**
     * Entity type: Language
     */
    ContentLanguageID: string;
    /**
     * Entity type: Country
     */
    ContentCountryID: string;
    /**
     * Entity type: Assortment
     */
    ContentAssortmentID?: number;
  }

  export interface ListContentCultureMappingResponse {
    Result?: EVA.Core.PagedResult<ListContentCultureMappingResponse.ContentCultureMappingDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListContentCultureMappingResponse {
    export interface ContentCultureMappingDto {
      /**
       * Entity type: ContentCultureMapping
       */
      ID: number;
      /**
       * Entity type: Assortment
       */
      ContentAssortmentID: number;
      /**
       * Entity type: Language
       */
      SourceLanguageID: string;
      /**
       * Entity type: Country
       */
      SourceCountryID: string;
      /**
       * Entity type: Culture
       */
      ContentCultureID: string;
      /**
       * Entity type: Language
       */
      ContentLanguageID: string;
      /**
       * Entity type: Country
       */
      ContentCountryID: string;
    }

  }

  /**
   * Creates a new ContentCultureMapping for the current application.
   */
  export interface CreateContentCultureMapping {
    /**
     * Entity type: Assortment
     */
    AssortmentID: number;
    SourceLanguageID?: string;
    SourceCountryID?: string;
    ContentLanguageID: string;
    ContentCountryID: string;
  }

  /**
   * Deletes an existing ContentCultureMapping.
   */
  export interface DeleteContentCultureMapping {
    /**
     * Entity type: ContentCultureMapping
     */
    ID: number;
  }

  export interface ListApplicationCultures {
  }

  export interface ListApplicationCulturesResponse {
    Result: ListApplicationContentCulture[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListApplicationContentCulture {
    /**
     * Entity type: Application
     */
    ApplicationID: number;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    Culture: string;
  }

  export interface CreateLayer {
    /**
     * Entity type: Application
     */
    ApplicationID?: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Description?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    Level: number;
    Name?: string;
  }

  export interface CreateLayerResponse {
    Result: CreatedLayer;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface CreatedLayer {
    /**
     * Entity type: ContentLayer
     */
    ID: number;
    Name: string;
    /**
     * Entity type: Application
     */
    ApplicationID?: number;
    Description: string;
    Level: number;
    /**
     * Entity type: Country
     */
    CountryID: string;
  }

  export interface GetLayerByID {
    /**
     * Entity type: ContentLayer
     */
    ID: number;
  }

  export interface GetLayerByIDResponse {
    Layer: LayerDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface LayerDto {
    /**
     * Entity type: ContentLayer
     */
    ID: number;
    Description: string;
    Level: number;
    /**
     * Entity type: Application
     */
    ApplicationID?: number;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Catalog
     */
    CatalogID?: number;
    Name: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    Type: ContentLayerTypes;
  }

  export const enum ContentLayerTypes {
    UserLayer = 0,
    SystemLayer = 1
  }

  export interface ListLayers {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListLayersResponse {
    Result?: EVA.Core.PagedResult<LayerDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface UpdateLayer {
    /**
     * Entity type: ContentLayer
     */
    ID: number;
    Name?: string;
    Description?: string;
    Level: number;
  }

  /**
   * Creates a search strategy for the provided LanguageID and CountryID (both are optional).
   * 
   * All parts of the `Strategy` field are optional.
   */
  export interface CreateProductSearchStrategy {
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    Name: string;
    Code?: string;
    Strategy: ProductSearchStrategyModel;
    Type: ProductSearchStrategyTypes;
  }

  export interface ProductSearchStrategyModel {
    Condition: ProductSearchStrategyModel.StrategyCondition;
    Scoring: ProductSearchStrategyModel.ScoringStrategy;
    Sorting: ProductSearchStrategyModel.SortingStrategy;
    Query: ProductSearchStrategyModel.QueryStrategy;
    Aggregation: ProductSearchStrategyModel.AggregationStrategy;
  }

  export namespace ProductSearchStrategyModel {
    export interface StrategyCondition {
      Query: ProductSearchStrategyModel.QueryCondition;
      Filters: ProductSearchStrategyModel.FilterCondition[];
    }

    export interface FilterCondition {
      ProductPropertyTypeID: string;
      FilterValue: string;
    }

    export interface QueryCondition {
      Texts: string[];
      Type: ProductSearchStrategyModel.QueryConditionTypes;
    }

    export const enum QueryConditionTypes {
      Equals = 0,
      Contains = 1,
      StartsWith = 2
    }

    export interface AggregationStrategy {
      StockFilterType: ProductSearchStrategyModel.StockFilterTypes;
    }

    export interface ProductPropertyTypeToSearch {
      ID: string;
      Boost?: number;
    }

    export interface QueryStrategy {
      Queries: ProductSearchStrategyModel.MultiMatchQuery[];
      MinimumShouldMatch: string;
      StockFilter: ProductSearchStrategyModel.StockFilterStrategy;
    }

    export interface StockFilterStrategy {
      FilterByStock: boolean;
      Type: ProductSearchStrategyModel.StockFilterTypes;
    }

    export interface MultiMatchQuery {
      Type: ProductSearchStrategyModel.MultiMatchType;
      Slop?: number;
      Fuzziness: ProductSearchStrategyModel.MultiMatchFuzziness;
      Operator: ProductSearchStrategyModel.MatchOperator;
      Boost?: number;
      RequirementType: ProductSearchStrategyModel.QueryRequirementType;
      PropertiesToSearch: ProductSearchStrategyModel.ProductPropertyTypeToSearch[];
      Analyzer: string;
      MinimumShouldMatch: string;
    }

    export interface ScoringStrategy {
      ScoreMode: ProductSearchStrategyModel.ScoreModes;
      BoostMode: ProductSearchStrategyModel.BoostMode;
      ScoringFunctions: ProductSearchStrategyModel.ScoringFunction[];
      MaxBoost?: number;
      MinScore?: number;
      StockScoring: ProductSearchStrategyModel.StockScoringStrategy;
    }

    export interface StockScoringStrategy {
      AdjustScoreByStock: boolean;
      BoostAmount: number;
      Type: ProductSearchStrategyModel.StockFilterTypes;
    }

    export const enum StockFilterTypes {
      CurrentOrganizationUnit = 0,
      Suppliers = 1
    }

    export interface ScoringFunction {
      Type: ProductSearchStrategyModel.ScoringFunctionTypes;
      Options: { [key: string]: any };
      Weight?: number;
    }

    export interface SortingStrategySort {
      Order: ProductSearchStrategyModel.SortingStrategyOrder;
      Script: ProductSearchStrategyModel.SortingStrategySortScript;
      Field: string;
      MissingValue: any;
    }

    export interface SortingStrategySortScript {
      Type: ProductSearchStrategyModel.SortingScriptType;
      Source: string;
      Language: string;
    }

    export interface StockSortingStrategy {
      SortByStock: boolean;
      Type: ProductSearchStrategyModel.StockFilterTypes;
    }

    export interface SortingStrategy {
      Sort: ProductSearchStrategyModel.SortingStrategySort[];
      StockSorting: ProductSearchStrategyModel.StockSortingStrategy;
    }

    export const enum SortingStrategyOrder {
      Ascending = 0,
      Descending = 1
    }

    export const enum SortingScriptType {
      Number = 0,
      String = 1
    }

    export const enum QueryRequirementType {
      Must = 0,
      Should = 1
    }

    export const enum BoostMode {
      Multiply = 0,
      Replace = 1,
      Sum = 2,
      Average = 3,
      Max = 4,
      Min = 5
    }

    export const enum ScoreModes {
      Multiply = 0,
      Sum = 1,
      Average = 2,
      First = 3,
      Max = 4,
      Min = 5
    }

    export const enum ScoringFunctionTypes {
      FieldValueFactor = 0,
      Script = 1
    }

    export const enum MatchOperator {
      And = 0,
      Or = 1
    }

    export const enum MultiMatchType {
      BestFields = 0,
      MostFields = 1,
      CrossFields = 2,
      Phrase = 3,
      PhrasePrefix = 4
    }

    export interface MultiMatchFuzziness {
      EditDistance?: number;
      PrefixLength?: number;
      MaxExpansions?: number;
    }

  }

  export const enum ProductSearchStrategyTypes {
    Full = 0,
    Partial = 1
  }

  export interface CreateProductSearchStrategyResponse {
    /**
     * Entity type: ProductSearchStrategy
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Retrieves a paged list of search strategies, optionally filtered by a CountryID or LanguageID.
   */
  export interface ListProductSearchStrategies {
    PageConfig?: EVA.Core.PageConfig<ListProductSearchStrategiesFilter>;
  }

  export interface ListProductSearchStrategiesFilter {
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
  }

  export interface ListProductSearchStrategiesResponse {
    Result?: EVA.Core.PagedResult<ListProductSearchStrategyItem>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListProductSearchStrategyItem {
    /**
     * Entity type: ProductSearchStrategy
     */
    ID: number;
    /**
     * Entity type: Catalog
     */
    CatalogID?: number;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    Name: string;
    Code: string;
    TypeID: number;
    Type: ProductSearchStrategyTypes;
    OrganizationUnitTypeID?: number;
    OrganizationUnitType?: EVA.Core.OrganizationUnitTypes;
    Strategy: ProductSearchStrategyModel;
    SerializedStrategy: string;
  }

  /**
   * Updates a search strategy, replacing the current strategy with the one provided and setting the LanguageID/CountryID
   * to the provided ones (both are optional).
   * 
   * All parts of the `Strategy` field are optional.
   */
  export interface UpdateProductSearchStrategy {
    /**
     * Entity type: ProductSearchStrategy
     */
    ID: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    Name?: string;
    Code?: string;
    Strategy: ProductSearchStrategyModel;
  }

  /**
   * Deletes a search strategy by the provided ID.
   */
  export interface DeleteProductSearchStrategy {
    /**
     * Entity type: ProductSearchStrategy
     */
    ID: number;
  }

  /**
   * Gets the default product search strategy, which can then be used as the basis to create a new strategy.
   */
  export interface GetDefaultProductSearchStrategy {
  }

  export interface GetDefaultProductSearchStrategyResponse {
    Strategy: ProductSearchStrategyModel;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a ProductSearchConfiguration for your current Catalog if it doesn't exist, or replaces the existing one.
   * All properties on the configuration are optional.
   */
  export interface CreateOrReplaceProductSearchConfiguration {
    Configuration?: ProductSearchConfigurationModel;
  }

  export interface ProductSearchConfigurationModel {
    Analyzers: ProductSearchConfigurationModel.Analyzer[];
    TokenFilters: ProductSearchConfigurationModel.TokenFilter[];
    CharFilters: ProductSearchConfigurationModel.CharFilter[];
    ProductPropertyTypes: ProductSearchConfigurationModel.ProductPropertyTypeConfiguration[];
    DisableSynonyms: boolean;
    IndexPipelineScript: string;
  }

  export namespace ProductSearchConfigurationModel {
    export interface IHasCulture {
      LanguageID: string;
    }

    export interface Analyzer {
      Name: string;
      LanguageID: string;
      Type: string;
      Tokenizer: string;
      Filters: string[];
      CharFilters: string[];
    }

    export interface TokenFilter {
      Name: string;
      LanguageID: string;
      Type: string;
      Options: { [key: string]: any };
    }

    export interface CharFilter {
      Name: string;
      LanguageID: string;
      Type: string;
      Options: { [key: string]: any };
    }

    export interface ProductPropertyTypeConfiguration {
      Name: string;
      LanguageID: string;
      Analyzer: string;
      SearchAnalyzer: string;
      CopyTo: string;
    }

  }

  /**
   * Retrieves the ProductSearchConfiguration for your current Catalog, or an empty one if it doesn't exist.
   */
  export interface GetProductSearchConfiguration {
  }

  export interface GetProductSearchConfigurationResponse {
    Configuration: ProductSearchConfigurationModel;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface CreateBrand {
    BackendID?: string;
    Name?: string;
  }

  export interface GetBrand {
    /**
     * Entity type: Brand
     */
    ID: number;
  }

  export interface GetBrandResponse {
    Result: BrandDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface BrandDto {
    /**
     * Entity type: Brand
     */
    ID: number;
    Name: string;
    BackendID: string;
  }

  export interface UpdateBrand {
    ToUpdate: BrandDto;
  }

  /**
   * Composes the given set of products
   */
  export interface ComposeProducts {
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    WaitForCompletion?: boolean;
    /**
     * Entity type: Language
     */
    LanguageIDs: string[];
    UseNewIndices?: boolean;
  }

  export interface ComposeResponse {
    CompositionID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface CancelComposeProducts {
    CompositionID: string;
  }

  export interface GetProductCompositionState {
    CompositionID: string;
  }

  export interface GetProductCompositionStateResponse {
    Status: string;
    TotalProductCount: number;
    TotalComposeCount: number;
    ProductsComposed: number;
    CompositionCount: number;
    DocumentsCopiedToElasticsearch: number;
    DocumentsCopiedToDatabase: number;
    BytesReadFromDatabase: number;
    BytesWrittenToElasticsearch: number;
    BytesWrittenToDatabase: number;
    BytesReadFromDatabasePerSecond: number;
    BytesWrittenToElasticsearchPerSecond: number;
    BytesWrittenToDatabasePerSecond: number;
    CompositionsPerSecond: number;
    ProductsComposedPerSecond: number;
    DocumentsCopiedToElasticsearchPerSecond: number;
    DocumentsCopiedToDatabasePerSecond: number;
    Progress: number;
    RemainingBatchCount: number;
    RemainingCopyBatchCount: number;
    ElapsedTimeInSeconds: number;
    ExpectedTimeRemainingInSeconds?: number;
    ExpectedTotalTimeInSeconds?: number;
    ExpectedCompletionTime?: string;
    CreationTime: string;
    CompletionTime?: string;
    WaitingOnDatabaseTimeInSeconds: number;
    WaitingOnConsumerTimeInSeconds: number;
    WaitingOnProducerTimeInSeconds: number;
    WaitingOnCompositionTimeInSeconds: number;
    WaitingOnSerializationTimeInSeconds: number;
    WaitingOnElasticsearchTimeInSeconds: number;
    WaitingOnDatabaseWriteTimeInSeconds: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Lists ProductContentEdits, optionally filtered by ProductID/RevisionID/LayerID
   */
  export interface ListProductContentEdits {
    InitialPageConfig?: EVA.Core.PageTokenConfig<ListProductContentEditsFilter>;
    PageToken?: string;
  }

  export interface ListProductContentEditsFilter {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    /**
     * Entity type: Revision
     */
    RevisionID?: number;
    /**
     * Entity type: ContentLayer
     */
    LayerID?: number;
  }

  export interface ListProductContentEditsResponse {
    Results?: ListProductContentEditsItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListProductContentEditsItem {
    /**
     * Entity type: ProductContentEdit
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: ContentLayer
     */
    LayerID: number;
    /**
     * Entity type: Revision
     */
    RevisionID?: number;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreationTime: string;
    /**
     * Entity type: User
     */
    LastModifiedByID?: number;
    LastModificationTime?: string;
    RevisionName: string;
    LayerName: string;
    /**
     * Entity type: Language
     */
    LayerLanguageID: string;
    /**
     * Entity type: Country
     */
    LayerCountryID: string;
    LayerLevel: number;
    ProductName: string;
    ProductType: EVA.Core.ProductTypes;
    ProductLogicalLevel: string;
    Content: ProductEditItem;
  }

  export interface ProductEditItem {
    Properties: { [key: string]: ProductEditItem.ProductProperty };
    MediaAdd: ProductEditItem.ProductMedia[];
    MediaRemove: ProductEditItem.ProductMedia[];
  }

  export namespace ProductEditItem {
    export interface ProductMedia {
      ID: string;
      BlobID: string;
      Name: string;
      Type: string;
      Sequence?: number;
      IsPrimaryImage?: boolean;
      Inherit?: boolean;
      MediaType: ProductMediaTypes;
      Url: string;
    }

    export interface ProductProperty {
      Value?: ProductEditItem.PropertyValue;
      AddValues: ProductEditItem.PropertyValue[];
      RemoveValues: ProductEditItem.PropertyValue[];
      IsDeleted?: boolean;
    }

    export interface PropertyValue {
      Value: any;
      Type: PropertyValueType;
    }

  }

  export const enum PropertyValueType {
    None = 0,
    Int = 1,
    String = 2,
    Bool = 3,
    DateTime = 4,
    Float = 5
  }

  export const enum ProductMediaTypes {
    Default = 0,
    Swatch = 1
  }

}
