declare module EVA.Payment.Core {

  export namespace Errors {
    export const enum LoyaltyPayment {
      /**
       * A Customer is required to do a Loyalty payment
       */
      CustomerRequired = 'LoyaltyPayment:CustomerRequired',
      /**
       * Invalid loyalty program selected
       */
      InvalidLoyaltyProgram = 'LoyaltyPayment:InvalidLoyaltyProgram',
      /**
       * Failed to execute the LoyaltyPayment in the handler
       */
      PaymentFailed = 'LoyaltyPayment:PaymentFailed'
    }

    export const enum APIGiftCard {
      /**
       * Redemption of card {0:string} failed
       */
      RedemptionFailed = 'APIGiftCard:RedemptionFailed',
      /**
       * A CardNumber is required to activate a giftcard
       */
      MissingCardNumber = 'APIGiftCard:MissingCardNumber',
      /**
       * Card with number {0:string} was not found.
       */
      CardNotFound = 'APIGiftCard:CardNotFound',
      /**
       * The card has currency {cardCurrency:string}, but the requested currency is {transactionCurrency:string}.
       */
      CurrencyMismatch = 'APIGiftCard:CurrencyMismatch'
    }

    export const enum EVAPay {
      /**
       * The given data if for a different Order.
       */
      InvalidOrder = 'EVAPay:InvalidOrder',
      /**
       * The given token is not valid.
       */
      InvalidToken = 'EVAPay:InvalidToken',
      /**
       * Customer is required.
       */
      CustomerRequired = 'EVAPay:CustomerRequired',
      /**
       * Invalid amount.
       */
      InvalidAmount = 'EVAPay:InvalidAmount',
      /**
       * EmailAddress is required.
       */
      EmailAddressRequired = 'EVAPay:EmailAddressRequired',
      /**
       * BaseUrl is required and should either be configured (Setting) or supplied as parameter.
       */
      BaseUrlRequired = 'EVAPay:BaseUrlRequired',
      /**
       * No PIN configured.
       */
      NoPinConfigured = 'EVAPay:NoPinConfigured',
      /**
       * Service only supports calls from users authenticated with an EVAPay token.
       */
      InvalidAuthenticationToken = 'EVAPay:InvalidAuthenticationToken'
    }

    export const enum CashPayment {
      /**
       * There is no open financial period for the current organizationunit.
       */
      NoOpenFinancePeriod = 'CashPayment:NoOpenFinancePeriod',
      /**
       * Invalid amounts provided for a CashPayment.
       */
      InvalidAmount = 'CashPayment:InvalidAmount',
      /**
       * The amount given can never be equal to or less than 0.
       */
      InvalidAmountGiven = 'CashPayment:InvalidAmountGiven',
      /**
       * The amount to refund can never be equal to or less than 0.
       */
      InvalidRefundAmount = 'CashPayment:InvalidRefundAmount',
      /**
       * Amount to be paid can't be greater than the open amount on the order.
       */
      OverPaymentNotAllowed = 'CashPayment:OverPaymentNotAllowed',
      /**
       * Cash payment limit has been reached for this currency type.
       */
      PaidAmountAboveCashHandlerLimit = 'CashPayment:PaidAmountAboveCashHandlerLimit',
      /**
       * Cash payment limit has been reached.
       */
      PaidAmountAboveCashLimit = 'CashPayment:PaidAmountAboveCashLimit'
    }

    export const enum CashDrawer {
      /**
       * Opening cash drawers requires Remark field to be set.
       */
      OpeningCashDrawerRequiresRemark = 'CashDrawer:OpeningCashDrawerRequiresRemark',
      /**
       * Opening a cash drawer with a PaymentTransaction requires a recent, confirmed cash payment.
       */
      InvalidPaymentTransactionProvided = 'CashDrawer:InvalidPaymentTransactionProvided'
    }

    export const enum BankTransfer {
      /**
       * AccountHolderName is required.
       */
      AccountHolderNameRequired = 'BankTransfer:AccountHolderNameRequired',
      /**
       * IBAN is required.
       */
      IBANRequired = 'BankTransfer:IBANRequired',
      /**
       * IBAN is not valid: {0:string}
       */
      IBANInvalid = 'BankTransfer:IBANInvalid'
    }

  }


  /**
   * Returns the available UserCardTypes
   */
  export interface GetUserCardTypes {
  }

  export interface GetUserCardTypesResponse {
    Types: EVA.Core.UserCardTypeDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Lists all user cards, with their number, type and linked user (full) name.
   * Using the `PageConfig`, you can filter on `Barcode`, `TypeID`, `FullName` and/or `UserName`.
   */
  export interface ListUserCards {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListUserCardsFilter>;
  }

  export interface ListUserCardsResponse {
    Result?: EVA.Core.PagedResult<ListUserCardDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListUserCardDto {
    Barcode: string;
    /**
     * Entity type: UserCard
     */
    ID: number;
    Type: EVA.Core.UserCardTypeDto;
    UserFullName: string;
  }

  /**
   * Returns all the UserCard for the current or requested User.
   * The currency is used to convert UserCards without a currency to their monetary value. Defaults to the session Currency.
   */
  export interface GetUserCardsForUser {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  export interface GetUserCardsForUserResponse {
    UserCards: EVA.Core.UserCardDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get the UserCard including the current balance
   */
  export interface GetUserCardDetails {
    /**
     * Entity type: UserCard
     */
    UserCardID?: number;
    CardNumber?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  export interface GetUserCardDetailsResponse {
    UserCard: EVA.Core.UserCardDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get the currenct balance of an UserCard
   */
  export interface GetUserCardBalance {
    /**
     * Entity type: UserCard
     */
    UserCardID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  export interface GetUserCardBalanceResponse {
    CurrentBalance: EVA.Core.UserCardBalance;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the mutations for a UserCard. Customers can only view the mutations of their own usercards.
   */
  export interface ListUserCardMutations {
    /**
     * Entity type: UserCard
     */
    UserCardID: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListUserCardMutationsResponse {
    Result?: EVA.Core.PagedResult<ListUserCardMutationsDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListUserCardMutationsDto {
    Amount: number;
    CreationTime: string;
    Description: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Status: EVA.Core.UserCardMutationStatuses;
    OrderDisplayID?: string;
  }

  /**
   * Create a new UserCard
   * 
   * The type is required, it can optionally be attached to an User.
   * When the CurrencyID is not supplied the currency of the current session will be used.
   */
  export interface CreateUserCard {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Required for UserCards that hold normal Currencies
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Entity type: UserCardType
     */
    UserCardTypeID: number;
    Description?: string;
  }

  export interface CreateUserCardResponse {
    /**
     * Entity type: UserCard
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Mutate the balance on an UserCard with the given amount.
   */
  export interface CreateUserCardMutation {
    /**
     * Entity type: UserCard
     */
    UserCardID: number;
    Amount: number;
    Description?: string;
  }

  /**
   * Abort a current pin paymenttransaction
   */
  export interface AbortTransaction {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  export interface AbortTransactionResponse {
    ResultStatus: PinTransactionResultStatus;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export const enum PinTransactionResultStatus {
    None = 0,
    TimeoutFromPinDevice = 1,
    TransactionStillInProgressForPayment = 2,
    Success = 3,
    Aborted = 4,
    Failure = 5,
    DeviceUnavailable = 6,
    PrintLastTicket = 7,
    TimedOut = 8,
    TransactionAlreadyInProgressForUser = 9,
    UserMustScanStationToAbort = 10,
    PaymentValidationError = 11,
    InProgress = 12,
    ClientFailure = 13,
    SelectPaymentMethod = 14,
    MustRevalidate = 16,
    ConnectToTerminal = 17
  }

  /**
   * Returns all `PinHandlers` available to use on `PaymentType.Options`
   */
  export interface GetAvailablePinHandlers {
  }

  export interface GetAvailablePinHandlersResponse {
    Handlers: GetAvailablePinHandlersResponse.PinHandler[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAvailablePinHandlersResponse {
    export interface PinHandler {
      Name: string;
    }

  }

  /**
   * Idle the terminal display and make it show the standby screen - this screen shows the configured branded logo as it is usually shown when no other process is going on.
   */
  export interface IdlePinTerminal {
  }

  export interface PrintLastReceiptCallback {
    /**
     * Entity type: Station
     */
    StationID?: number;
    ReceiptLines: string[];
  }

  export interface PrintPinReceipt {
    ECRID?: string;
    OrderReference: string;
    ReceiptLines: string[];
    TransactionSucceeded?: boolean;
    SignatureLines: SignatureLine[];
    ReceiptType: string;
  }

  export namespace PrintPinReceipt {
    export interface ReceiptTypes {
    }

  }

  export interface SignatureLine {
    X1: number;
    Y1: number;
    X2: number;
    Y2: number;
  }

  /**
   * Start the pin transaction
   * 
   * This service needs to be called after creating a pin payment. It will block until the transaction is completed.
   * To cancel this process the AbortTransaction service can be called.
   */
  export interface StartTransaction {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  export interface StartTransactionResponse {
    ResultStatus: PinTransactionResultStatus;
    ResultStatusString: string;
    OpenAmount: number;
    CardCircuit: string;
    CountryID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Retrieve order information based on an EVAPay generated token (that you authenticated with).
   */
  export interface EVAPayGetInfo {
    ProductProperties?: string[];
    IncludeAllPayments?: boolean;
    ShowOnlyShippableLines?: boolean;
  }

  export interface EVAPayGetInfoResponse {
    Amounts: EVA.Core.OpenOrderAmounts;
    ReturnUrl?: string;
    RequestedAmount?: number;
    TotpToken?: string;
    TotpData?: string;
    Result: EVA.Core.OrderDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Open a CashDrawer on the given station
   * 
   * Without a PaymentTransactionID, this service requires additional OpenCashDrawer functionality and you can then specify a ReasonID why you are opening it.
   */
  export interface OpenCashDrawer {
    /**
     * Entity type: Station
     */
    StationID?: number;
    Remark?: string;
    /**
     * When the opening of the drawer is non-payment related, it's possible to specify a predefined ReasonID for opening the drawer.
     * Entity type: OpenCashDrawerReason
     */
    ReasonID?: number;
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
  }

  export interface OpenCashDrawerResponse {
    Result: EVA.Core.OpenCashDrawerResults;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

}
