declare module EVA.Payment.Mollie {

  /**
   * When you would like to control payment type and issuer selection yourself, you can use `ListMollieGateways` to retrieve a list of supported payment types ("gateways") from Mollie. Currently they
   * support iDEAL, Bitcoin, bank transfer, credit card and a couple more, all configurable through Mollie. This response will include icons and descriptions in the supplied locale, and in case of iDEAL
   * will include a list of "issuers" like ING, Rabobank, etcetera.
   * 
   * Then when creating a payment, include a `GatewayID` and this payment type is automatically chosen. Additionally include `Issuer` and this too is automatically selected.
   * 
   * Configure `Mollie:Api:BaseUrl` (although it doesn't seem you want to deviate from the default) and specify a `Mollie:Api:Token`.
   * 
   * EVA will require the selected payment type configured in the "MOLLIE_`GatewayID`" format, for instance "MOLLIE_BITCOIN" and "MOLLIE_IDEAL", and the "MOLLIE" payment type is no longer required.
   */
  export interface ListMollieGateways {
  }

  export interface ListMollieGatewaysResponse {
    Gateways: Method[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface Method {
    ID: string;
    Description: string;
    Amount: Amount;
    Image: Image;
    Issuers: Issuer[];
  }

  export interface Amount {
    Minimum: number;
    Maximum: number;
  }

  export interface Image {
    Normal: string;
    Bigger: string;
  }

  export interface Issuer {
    ID: string;
    Name: string;
    Image: Image;
  }

}
