declare module EVA.Sync {

  /**
   * Compares synchronization file with current environment.
   */
  export interface CompareSynchronization {
    BlobUrl?: string;
    IncludeIdentical?: boolean;
  }

  export interface CompareSynchronizationResponse {
    Files?: CompareSynchronizationResponse.CompareSynchronizationFile[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace CompareSynchronizationResponse {
    export interface CompareSynchronizationFile {
      Name?: string;
      Data?: CompareSynchronizationResponse.CompareSynchronizationFileData[];
    }

    export interface CompareSynchronizationFileData {
      ID?: string;
      Description?: string;
      ObjectInfo?: { [key: string]: any };
      IsIdentical?: boolean;
      Values?: CompareSynchronizationResponse.CompareSynchronizationFileDataValue[];
    }

    export interface CompareSynchronizationFileDataValue {
      Type?: string;
      NewValue?: string;
      CurrentValue?: string;
      CanCompare?: boolean;
      CanSync?: boolean;
    }

  }

  /**
   * Creates a synchronization file. Compare with CompareSynchronization and upload with UploadSynchronization
   */
  export interface CreateSynchronization {
    Managers?: CreateSynchronization.CreateSynchronizationManager[];
  }

  export namespace CreateSynchronization {
    export interface CreateSynchronizationManager {
      Name?: string;
    }

  }

  export interface CreateSynchronizationResponse {
    BlobUrl?: string;
    ExpireDate?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns available synchronization managers.
   */
  export interface GetAvailableSynchronizationManagers {
  }

  export interface GetAvailableSynchronizationManagersResponse {
    Managers?: GetAvailableSynchronizationManagersResponse.Manager[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAvailableSynchronizationManagersResponse {
    export interface Manager {
      Name?: string;
    }

  }

  /**
   * Import data stored in provided synchronization file. Provide a list of data IDs that should be imported.
   */
  export interface UploadSynchronization {
    BlobUrl?: string;
    Import?: UploadSynchronization.FileDataToImport[];
  }

  export namespace UploadSynchronization {
    export interface FileDataToImport {
      ID?: string;
      Values?: FileDataToImport.FileDataToImportValue[];
    }

    export namespace FileDataToImport {
      export interface FileDataToImportValue {
        Type?: string;
        Value?: string;
      }

    }

  }

  export interface UploadSynchronizationResponse {
    NumProcessed?: number;
    Processed?: UploadSynchronizationResponse.UploadSynchronizationResult[];
    Errors?: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace UploadSynchronizationResponse {
    export interface UploadSynchronizationResult {
      Type?: string;
      ID?: string;
      IsSuccess?: boolean;
      ErrorDescription?: string;
      IsNew?: boolean;
    }

  }

}
