declare module EVA.Waldo {

  export interface CreateOrderStatus {
    Description: string;
    BackendID?: string;
    Script: string;
    ScriptLanguage: string;
    Sequence: number;
    Once?: boolean;
  }

  export interface CreateOrderStatusResponse {
    /**
     * Entity type: OrderDefinition
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface UpdateOrderStatus {
    /**
     * Entity type: OrderDefinition
     */
    ID: number;
    Description: string;
    BackendID?: string;
    Script: string;
    ScriptLanguage: string;
    Sequence: number;
    Once?: boolean;
  }

  export interface DeleteOrderStatus {
    /**
     * Entity type: OrderDefinition
     */
    ID: number;
  }

  export interface GetOrderStatus {
    /**
     * Entity type: OrderDefinition
     */
    ID: number;
  }

  export interface GetOrderStatusResponse {
    Description: string;
    BackendID: string;
    Script: string;
    ScriptLanguage: string;
    Sequence: number;
    Once: boolean;
    Translations: GetOrderStatusResponse.TranslationModel[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetOrderStatusResponse {
    export interface TranslationModel {
      LanguageID: string;
      CountryID: string;
      Title: string;
      Subtitle: string;
    }

  }

  export interface ListOrderStatuses {
    PageConfig?: EVA.Core.PageConfig<OrderDefinitionFilter>;
  }

  export interface OrderDefinitionFilter {
    BackendID?: string;
    Description?: string;
    IncludeDeleted?: boolean;
  }

  export interface ListOrderStatusesResponse {
    Result?: EVA.Core.PagedResult<ListOrderStatusesResponse.Model>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListOrderStatusesResponse {
    export interface Model {
      /**
       * Entity type: OrderDefinition
       */
      ID: number;
      Description: string;
      BackendID?: string;
      Script: string;
      ScriptLanguage: string;
      Sequence: number;
      Once: boolean;
    }

  }

  export interface CreateOrUpdateOrderStatusTranslation {
    /**
     * Entity type: OrderDefinition
     */
    StatusID: number;
    LanguageID?: string;
    CountryID?: string;
    Title: string;
    Subtitle?: string;
  }

  export interface DeleteOrderStatusTranslation {
    /**
     * Entity type: OrderDefinition
     */
    StatusID: number;
    LanguageID?: string;
    CountryID?: string;
  }

  export interface CreateOrderType {
    Description: string;
    BackendID?: string;
    Script: string;
    ScriptLanguage: string;
    Sequence: number;
    Once?: boolean;
  }

  export interface CreateOrderTypeResponse {
    /**
     * Entity type: OrderDefinition
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface UpdateOrderType {
    /**
     * Entity type: OrderDefinition
     */
    ID: number;
    Description: string;
    BackendID?: string;
    Script: string;
    ScriptLanguage: string;
    Sequence: number;
    Once?: boolean;
  }

  export interface DeleteOrderType {
    /**
     * Entity type: OrderDefinition
     */
    ID: number;
  }

  export interface GetOrderType {
    /**
     * Entity type: OrderDefinition
     */
    ID: number;
  }

  export interface GetOrderTypeResponse {
    Description: string;
    BackendID: string;
    Script: string;
    ScriptLanguage: string;
    Sequence: number;
    Once: boolean;
    Translations: GetOrderTypeResponse.TranslationModel[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetOrderTypeResponse {
    export interface TranslationModel {
      LanguageID: string;
      CountryID: string;
      Title: string;
      Subtitle: string;
    }

  }

  export interface ListOrderTypes {
    PageConfig?: EVA.Core.PageConfig<OrderDefinitionFilter>;
  }

  export interface ListOrderTypesResponse {
    Result?: EVA.Core.PagedResult<ListOrderTypesResponse.Model>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListOrderTypesResponse {
    export interface Model {
      /**
       * Entity type: OrderDefinition
       */
      ID: number;
      Description: string;
      BackendID?: string;
      Script: string;
      ScriptLanguage: string;
      Sequence: number;
      Once: boolean;
    }

  }

  export interface CreateOrUpdateOrderTypeTranslation {
    /**
     * Entity type: OrderDefinition
     */
    DefinitionID: number;
    LanguageID?: string;
    CountryID?: string;
    Title: string;
    Subtitle?: string;
  }

  export interface DeleteOrderTypeTranslation {
    /**
     * Entity type: OrderDefinition
     */
    DefinitionID: number;
    LanguageID?: string;
    CountryID?: string;
  }

  /**
   * Get the last orderstatus for a set of Orders
   */
  export interface GetOrderStatusForOrders {
    /**
     * Entity type: Order
     */
    OrderIDs?: number[];
  }

  export interface GetOrderStatusForOrdersResponse {
    /**
     * Entity type: Order
     */
    Results: { [key: number]: GetOrderStatusForOrdersResponse.Model };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetOrderStatusForOrdersResponse {
    export interface Model {
      Description: string;
      BackendID: string;
      Title: string;
      Subtitle: string;
      CreationTime: string;
      Sequence: number;
      Data: { [key: string]: any };
    }

  }

  /**
   * @deprecated Use `GetOrderCustomStatusLedger`
   */
  export interface GetOrderStatusHistory {
    /**
     * Entity type: Order
     */
    OrderID?: number;
  }

  export interface GetOrderStatusHistoryResponse {
    Results: GetOrderStatusHistoryResponse.Model[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetOrderStatusHistoryResponse {
    export interface Model {
      Description: string;
      BackendID: string;
      Title: string;
      Subtitle: string;
      CreationTime: string;
      Sequence: number;
      Data: { [key: string]: any };
    }

  }

}
