declare module EVA.Workforce {

  export namespace Errors {
    export const enum FinancialPeriodClosingImpediments {
      /**
       * There are {0:number} clocked-in users remaining that must be clocked out before the period can be closed.
       */
      ClockedInUsers = 'FinancialPeriodClosingImpediments:ClockedInUsers'
    }

  }


  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListAvailabilityForTimeslot {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    StartDateTime: string;
    EndDateTime: string;
  }

  export interface ListAvailabilityForTimeslotResponse {
    Availability: ConsecutiveTimeChunkWithUser[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ConsecutiveTimeChunkWithUser {
    /**
     * Entity type: User
     */
    UserID: number;
    FullName: string;
    Date: string;
    StartTime: string;
    EndTime: string;
    IsAvailable: boolean;
    Duration: string;
    StartDateTime: string;
    EndDateTime: string;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListPlanning {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    StartDate: string;
    EndDate: string;
  }

  export interface ListPlanningResponse {
    Items: ListPlanningResponse.Planning[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPlanningResponse {
    export interface Planning {
      OrganizationUnit: EVA.Core.OrganizationUnitDto;
      DateTime: string;
      Result: number;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   * Clocks the given employee out for the given `FinancialPeriodID` (if no value is passed, the curerntly active period is
   * used instead). The `UserID` indicates the user that should be clocked out. `AbsentFrom` indicates at what time the
   * employee should be registered as AFK.
   */
  export interface ClockEmployeeOutAsAbsent {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    AbsentFrom: string;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface CorrectWorkedHours {
    /**
     * Entity type: WorkedHours
     */
    ID: number;
    ClockInDateTime?: string;
    ClockOutDateTime?: string;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface CreateWorkedHours {
    /**
     * Entity type: User
     */
    UserID: number;
    Date: string;
    ClockInTime: string;
    ClockOutTime: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  /**
   * @deprecated Feature not supported anymore.
   * Returns all of the employees that haven't yet clocked out for the given financial period (`PeriodID`). Use this service
   * in combination with the `ClockEmployeeOutAsAbsent` service.
   */
  export interface ListClockedInEmployeesForFinancialPeriod {
    /**
     * Entity type: FinancialPeriod
     */
    PeriodID: number;
  }

  export interface ListClockedInEmployeesForFinancialPeriodResponse {
    Employees: ListClockedInEmployeesForFinancialPeriodResponse.ClockedInUser[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListClockedInEmployeesForFinancialPeriodResponse {
    export interface ClockedInUser {
      User: EVA.Core.UserDto;
      ClockedIn: string;
      ClockedOut?: string;
      IsAbsent: boolean;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListUnrosteredWorkedHours {
    /**
     * Entity type: User
     */
    UserID: number;
    StartDate: string;
    EndDate: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface ListUnrosteredWorkedHoursResponse {
    WorkedHours: WorkedPeriod[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface WorkedPeriod {
    StartDate: string;
    EndDate?: string;
  }

  /**
   * @deprecated Feature not supported anymore.
   * List the WorkedHours for the current user
   */
  export interface ListWorkedHoursForCurrentEmployee {
  }

  export interface ListWorkedHoursForCurrentEmployeeResponse {
    WorkedHours: ListWorkedHoursForCurrentEmployeeResponse.WorkedHour[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListWorkedHoursForCurrentEmployeeResponse {
    export interface WorkedHour {
      ID: number;
      /**
       * Entity type: User
       */
      UserID: number;
      User: EVA.Core.UserDto;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnit: EVA.Core.OrganizationUnitDto;
      Date: string;
      ClockInTime: string;
      ClockOutTime?: string;
      ClockInDateTime: string;
      ClockOutDateTime?: string;
      IsExported: boolean;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   * List the WorkedHours within a specific financialperiod
   */
  export interface ListWorkedHoursForFinancialPeriod {
    PageConfig?: EVA.Core.PageConfig<ListWorkedHoursInFinancialPeriodFilter>;
  }

  export interface ListWorkedHoursInFinancialPeriodFilter {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
  }

  export interface ListWorkedHoursForFinancialPeriodResponse {
    Result?: EVA.Core.PagedResult<ListWorkedHoursForFinancialPeriodResponse.WorkedPeriod>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListWorkedHoursForFinancialPeriodResponse {
    export interface WorkedPeriod {
      /**
       * Entity type: WorkedHours
       */
      ID: number;
      /**
       * Entity type: User
       */
      UserID: number;
      UserFullName: string;
      /**
       * Start date/time of the worked hours as set by clocking in action or manager.
       */
      StartDate: string;
      /**
       * End date/time of the worked hours as set by clocking out action or manager.
       */
      EndDate?: string;
      /**
       * Indicates if the worked hours have been corrected by a manager.
       */
      IsCorrected: boolean;
      /**
       * Indicates that the users didn't clock out, and the manager did it on period closing.
       */
      IsAbsent: boolean;
      /**
       * Indicates if the worked hours can be corrected, based on the `WorkedHours:CorrectionTime` setting.
       */
      CanBeCorrected: boolean;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   * List the WorkedHours from a specific user based on start- and end date/time
   */
  export interface ListWorkedHours {
    /**
     * Entity type: User
     */
    UserIDs?: number[];
    StartDate: string;
    EndDate: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface ListWorkedHoursResponse {
    /**
     * Entity type: User
     */
    WorkedPeriods: { [key: number]: ListWorkedHoursResponse.WorkedPeriod[] };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListWorkedHoursResponse {
    export interface WorkedPeriod {
      /**
       * The ID of the worked hours registration.
       */
      StartID: number;
      /**
       * Start date/time of the worked hours as set by clocking in action or manager.
       */
      StartDate: string;
      /**
       * End date/time of the worked hours as set by clocking out action or manager.
       */
      EndDate?: string;
      /**
       * Indicates if the user has clocked out.
       */
      ClockedOut: boolean;
      /**
       * Indicates if the worked hours can be corrected, based on the `WorkedHours:CorrectionTime` setting.
       */
      CanBeCorrected: boolean;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface DeleteRosterItem {
    /**
     * Entity type: Roster
     */
    ID: number;
    ExpireDate?: string;
  }

  /**
   * Returns a list of employees with an active Role in the given organization unit
   */
  export interface ListEmployeesForOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Role
     */
    RoleIDs?: number[];
  }

  export interface ListEmployeesForOrganizationUnitResponse {
    Users: EVA.Core.UserDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListOccupiedPeriods {
    /**
     * Entity type: User
     */
    UserID: number;
    StartDate: string;
    EndDate: string;
  }

  export interface ListOccupiedPeriodsResponse {
    Absence: ListOccupiedPeriodsResponse.OccupiedPeriod[];
    Leave: ListOccupiedPeriodsResponse.OccupiedPeriod[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListOccupiedPeriodsResponse {
    export interface OccupiedPeriod {
      StartDate: string;
      EndDate?: string;
      TypeDescription: string;
      Reason: string;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListRoster {
    Types?: number[];
    /**
     * Entity type: User
     */
    UserID?: number;
    BadgeID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    StartDate: string;
    EndDate: string;
  }

  export interface ListRosterResponse {
    Items: ListRosterResponse.RosterItem[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListRosterResponse {
    export interface RosterItem {
      TypeID: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      OrganizationUnitName: string;
      /**
       * Entity type: User
       */
      UserID?: number;
      BadgeID?: number;
      FullName: string;
      Date: string;
      StartTime?: string;
      EndTime?: string;
      DayOfWeekID?: number;
      ID: number;
      StartDateTime?: string;
      EndDateTime?: string;
      Description: string;
      AllDay: boolean;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface SaveRosterItem {
    /**
     * Entity type: Roster
     */
    ID: number;
    TypeID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    Date?: string;
    DayOfWeekID?: number;
    AllDay?: boolean;
    /**
     * Entity type: Roster
     */
    AvailabilityID?: number;
    /**
     * Entity type: Roster
     */
    SlotID?: number;
    StartDateTime?: string;
    EndDateTime?: string;
    Description?: string;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ExportPayrollPeriod {
    /**
     * Entity type: PayrollPeriod
     */
    PayrollPeriodID: number;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface GetPayrollPeriodReport {
    /**
     * Entity type: PayrollPeriod
     */
    PayrollPeriodID: number;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface GetPayrollPeriodWorkedHoursReport {
    /**
     * Entity type: PayrollPeriod
     */
    PayrollPeriodID: number;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface GetSalaryComponentTypes {
  }

  export interface GetSalaryComponentTypesResponse {
    Result: GetSalaryComponentTypesResponse.SalaryComponentTypeDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSalaryComponentTypesResponse {
    export interface SalaryComponentTypeDto {
      ID: number;
      Name: string;
      Description: string;
      Code: string;
      ExportNextPeriod: boolean;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListPayrollPeriods {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListPayrollPeriodsResponse {
    Result?: EVA.Core.PagedResult<ListPayrollPeriodsResponse.PayrollPeriodDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPayrollPeriodsResponse {
    export interface PayrollPeriodDto {
      ID: number;
      PreviousPeriodID?: number;
      StartDate: string;
      EndDate: string;
      Weeks: number;
      IsExported: boolean;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListSalaryComponents {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListSalaryComponentsResponse {
    Result?: EVA.Core.PagedResult<ListSalaryComponentsResponse.SalaryComponentDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListSalaryComponentsResponse {
    export interface SalaryComponentDto {
      PayrollPeriodID: number;
      UserID: number;
      User: EVA.Core.UserDto;
      TypeID: number;
      Param1?: number;
      Param2?: number;
      Param3?: number;
      Reason: string;
      IsExported: boolean;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface RecalculatePayrollPeriod {
    /**
     * Entity type: PayrollPeriod
     */
    PayrollPeriodID: number;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListLeaveBalancesForUser {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  export interface ListLeaveBalancesForUserResponse {
    Items: ListLeaveBalancesForUserResponse.LeaveBalanceDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListLeaveBalancesForUserResponse {
    export interface LeaveBalanceDto {
      LeaveType: string;
      Balance: number;
    }

  }

}
