import { IEvaServiceDefinition } from './eva-services-core.js';
export declare namespace Core {
    /**
     * Adds a boarding pass to specified order, by barcode.
     *
     * @export
     * @class AddBoardingPassToOrder
     * @implements {EvaService}
     */
    class AddBoardingPassToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddBoardingPassToOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * The normal `AddProductToOrder` cannot be used for adding a Product with type `BundleProduct` to an order. Instead, `AddBundleProductToOrder` must be
used. In addition to the `BundleProductID`, this service accepts an optional `LineSelection`, which represents the bundle's configuration in terms of
which products you've chosen.

The `LineSelection` data structure contains a `SelectedProductID`, which must refer to a ProductID that is a valid option for this bundle, according
to what `GetBundleProductDetails` returns.

If no `LineSelection` is provided then the default products of the bundle are added to the Order.
     *
     * @export
     * @class AddBundleProductToOrder
     * @implements {EvaService}
     */
    class AddBundleProductToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddBundleProductToOrder;
        response?: EVA.Core.AddBundleProductToOrderResponse;
    }
    /**
     * A specific implementation for `AddProductToOrder` for GiftCards. Adds a new order line for a digital GiftCard to specified order, where
`OrderID` identifies the order for the customer and `Amount` is the desired amount to create a GiftCard for.

Optionally supply a `DetailsUrl` pointing to the website you want the buyer of the GiftCard to land to see/manage the order.

Additionally, the `DisplayOptions` property need to be supplied, this contains a bunch of default options to personalize the giftcard.
It is also possible to provide some Metadata. We won't do any processing on DisplayOptions.

Except: DetailsUrl will be changed to include the `OrderID` in the URL as well as a `token` that can be used to access a few services.
     *
     * @export
     * @class AddDigitalGiftCardToOrder
     * @implements {EvaService}
     */
    class AddDigitalGiftCardToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddDigitalGiftCardToOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Add a discount coupon to an order
     *
     * @export
     * @class AddDiscountCouponToOrder
     * @implements {EvaService}
     */
    class AddDiscountCouponToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddDiscountCouponToOrder;
        response?: EVA.Core.AddDiscountToOrderResponse;
    }
    /**
     * Add a Discount to an Order

- To add a Coupon discount, specify the `CouponCode`.
- To add a Manual discount, specify the `DiscountID` and the `DiscountAmount`. Optionally a `Reason` can be added.
     *
     * @export
     * @class AddDiscountToOrder
     * @implements {EvaService}
     */
    class AddDiscountToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddDiscountToOrder;
        response?: EVA.Core.SimpleShoppingCartResponse;
    }
    /**
     * Adds a label to specified full stock count.
     *
     * @export
     * @class AddLabelToFullStockCount
     * @implements {EvaService}
     */
    class AddLabelToFullStockCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddLabelToFullStockCount;
        response?: EVA.Core.AddLabelToFullStockCountResponse;
    }
    /**
     * Add a loyalty Discount to an order
     *
     * @export
     * @class AddLoyaltyDiscountToOrder
     * @implements {EvaService}
     */
    class AddLoyaltyDiscountToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddLoyaltyDiscountToOrder;
        response?: EVA.Core.AddDiscountToOrderResponse;
    }
    /**
     * Add a manual Discount to an order
     *
     * @export
     * @class AddManualDiscountToOrder
     * @implements {EvaService}
     */
    class AddManualDiscountToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddManualDiscountToOrder;
        response?: EVA.Core.AddDiscountToOrderResponse;
    }
    /**
     * Adds a product count result to specified Full Stock Count label. User task must be active and assigned to the current user.
     *
     * @export
     * @class AddProductToFullStockCountLabel
     * @implements {EvaService}
     */
    class AddProductToFullStockCountLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddProductToFullStockCountLabel;
        response?: EVA.Core.AddProductToFullStockCountLabelResponse;
    }
    /**
     * Add a Product to an Order

A product can be defined by `ProductID` or `Barcode`.
If the product is unstackable (e.g. gift cards, same product added with different serial numbers), new orderlines are created.
     *
     * @export
     * @class AddProductToOrder
     * @implements {EvaService}
     */
    class AddProductToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddProductToOrder;
        response?: EVA.Core.AddProductToOrderResponse;
    }
    /**
     * Add product(s) to an anonymous wishlist.
     *
     * @export
     * @class AddProductsToAnonymousWishlist
     * @implements {EvaService}
     */
    class AddProductsToAnonymousWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddProductsToAnonymousWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Add product(s) to the user wishlist.
     *
     * @export
     * @class AddProductsToUserWishlist
     * @implements {EvaService}
     */
    class AddProductsToUserWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddProductsToUserWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Add lines to an existing purchase order

It is required to supply an OrderID and at least one line with a ProductID and a Quantity. All other line properties are optional.
It's not allowed to create multiple lines with the same product on the same stocklabel.
     *
     * @export
     * @class AddPurchaseOrderLines
     * @implements {EvaService}
     */
    class AddPurchaseOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddPurchaseOrderLines;
        response?: EVA.Core.AddPurchaseOrderLinesResponse;
    }
    /**
     * Adds a service product to an order.
     *
     * @export
     * @class AddServiceProductToOrder
     * @implements {EvaService}
     */
    class AddServiceProductToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddServiceProductToOrder;
        response?: EVA.Core.AddServiceProductToOrderResponse;
    }
    /**
     * Add a tracking code to the ReturnToSupplier task shipment.
If tracking code is already present, it will be overwritten.
     *
     * @export
     * @class AddTrackingCodeToReturnToSupplierShipment
     * @implements {EvaService}
     */
    class AddTrackingCodeToReturnToSupplierShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddTrackingCodeToReturnToSupplierShipment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Add user borrowed products to order. When no quantity is given, all borrowed products with the given product ID are added to the order.
     *
     * @export
     * @class AddUserBorrowedProductToOrder
     * @implements {EvaService}
     */
    class AddUserBorrowedProductToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddUserBorrowedProductToOrder;
        response?: EVA.Core.AddUserBorrowedProductToOrderResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class AddUserToGroup
     * @implements {EvaService}
     */
    class AddUserToGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AddUserToGroup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Book a set of Adjustments to modify stock.

An Adjustment should have a `ProductID` to identify the product, an `OrganizationUnitID` to identify the stock keeping organization, a `StockLabelID` (see `GetEnumValues` with type `StockLabel`),
a `StockMutationReasonID` (see `GetEnumValues` with type `StockMutationReasons`) and of course a `Quantity` to actually book.

Supply a `BackendSystemID` on the request and `BackendID`'s on the Adjustments to make the request idempotent (optional). When one adjustment in your request fails, all of them will fail. Duplicate
keys will not be processed again and yield a warning (but thus will not block!).

Optionally supports a `Remark` for each Adjustment.

When specifying a `SerialNumber`, please note the `Quantity` of this adjustment then should either be -1 or 1!
     *
     * @export
     * @class AdjustStock
     * @implements {EvaService}
     */
    class AdjustStock implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AdjustStock;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used to apply a selection of products to the provided `BundleProductOrderLineID`.
The `BundleProductOrderLineID` field refers to the ID of an `OrderLine` on the provided `Order` that has a `BundleProduct` Product on it.
The `LineSelection` field is a list that couples a `SelectedProductID` to a `ProductBundleLineID` and is identical to how it's used in `AddBundleProductToOrder`.
     *
     * @export
     * @class ApplyBundleProductSelection
     * @implements {EvaService}
     */
    class ApplyBundleProductSelection implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ApplyBundleProductSelection;
        response?: EVA.Core.SimpleShoppingCartResponse;
    }
    /**
     * Will apply and return the estimated tax for specified order. Its effect is persistent through all subsequent calls for this orders' amounts.
     *
     * @export
     * @class ApplySalesTaxEstimateForOrder
     * @implements {EvaService}
     */
    class ApplySalesTaxEstimateForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ApplySalesTaxEstimateForOrder;
        response?: EVA.Core.SalesTaxEstimateForOrderResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ApplyScriptEdits
     * @implements {EvaService}
     */
    class ApplyScriptEdits implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ApplyScriptEdits;
        response?: EVA.Core.UpdateScriptResponse;
    }
    /**
     * Approve a non-confirmed payment transaction capture
     *
     * @export
     * @class ApproveCapture
     * @implements {EvaService}
     */
    class ApproveCapture implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ApproveCapture;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Approve a non-confirmed payment transaction
     *
     * @export
     * @class ApprovePayment
     * @implements {EvaService}
     */
    class ApprovePayment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ApprovePayment;
        response?: EVA.Core.ApprovePaymentResponse;
    }
    /**
     * Archive an organization unit.
NOTE: Only do this when it is no longer in use. While this action is reversible, settings and other configurations previously in place may be lost.
     *
     * @export
     * @class ArchiveOrganizationUnit
     * @implements {EvaService}
     */
    class ArchiveOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ArchiveOrganizationUnit;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Attach addresses to specified order(s), requiring either `OrderID` or `OrderIDs`.
     *
     * @export
     * @class AttachAddressToOrder
     * @implements {EvaService}
     */
    class AttachAddressToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AttachAddressToOrder;
        response?: EVA.Core.AttachAddressToOrderResponse;
    }
    /**
     * If the wishlist is created by anonymous, but the user logs in afterwards, this service can be used to attach the anonymous wishlist to the logged in user.
The service requires a unique name.
     *
     * @export
     * @class AttachAnonymousWishlistToUser
     * @implements {EvaService}
     */
    class AttachAnonymousWishlistToUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AttachAnonymousWishlistToUser;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Attaches a blob to an invoice, either by `BlobID` or `BlobUrl`.
     *
     * @export
     * @class AttachBlobToInvoice
     * @implements {EvaService}
     */
    class AttachBlobToInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AttachBlobToInvoice;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Attaches given blob to an order, either by `BlobID` or `BlobUrl`.
     *
     * @export
     * @class AttachBlobToOrder
     * @implements {EvaService}
     */
    class AttachBlobToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AttachBlobToOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Attaches a consumer company to an order. Optionally, provide a user ID to associate with it.
     *
     * @export
     * @class AttachConsumerCompanyToOrder
     * @implements {EvaService}
     */
    class AttachConsumerCompanyToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AttachConsumerCompanyToOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Attach a Customer to an Order. When no UserID is provided the current user will be attached. If the selected User has addresses, these will also be assigned to the Order.
     *
     * @export
     * @class AttachCustomerToOrder
     * @implements {EvaService}
     */
    class AttachCustomerToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AttachCustomerToOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Attach an identification document to an order. This will replace any existing identification document attached to the order.
     *
     * @export
     * @class AttachIdentificationToOrder
     * @implements {EvaService}
     */
    class AttachIdentificationToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AttachIdentificationToOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Attaches an OpeningHoursTemplate to the given OrganizationUnitSets.
When `OverrideIfOpeningHoursTypeAlreadyAttached` is set to true, the service detaches the template of the same type before attaching the new template.
When `OverrideIfOpeningHoursTypeAlreadyAttached` is set to false, the service will fail if one of the given OrganizationUnitSets is already attached to a OpeningHoursTemplate of the same Type.
     *
     * @export
     * @class AttachOpeningHoursTemplateToOrganizationUnitSets
     * @implements {EvaService}
     */
    class AttachOpeningHoursTemplateToOrganizationUnitSets implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AttachOpeningHoursTemplateToOrganizationUnitSets;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Attaches an user to a consumer company.
     *
     * @export
     * @class AttachUserToConsumerCompany
     * @implements {EvaService}
     */
    class AttachUserToConsumerCompany implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AttachUserToConsumerCompany;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Automatically places, ships and receives an interbranch order between 2 organization units.
     *
     * @export
     * @class AutoTransferInterbranchOrder
     * @implements {EvaService}
     */
    class AutoTransferInterbranchOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AutoTransferInterbranchOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for retrieving a list of suggested addresses
     *
     * @export
     * @class AutocompleteAddress
     * @implements {EvaService}
     */
    class AutocompleteAddress implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AutocompleteAddress;
        response?: EVA.Core.AutocompleteAddressResponse;
    }
    /**
     * Attempts to autocomplete the order by:

- Cancelling the lines that can still be cancelled
- Invoicing the lines that've been (partially) shipped
- Optionally creating a payment to bring the open amount to 0

:bikini: __The actions this service performs are irreverisble__

*With great power comes great responsibility*
-Uncle Ben
     *
     * @export
     * @class AutocompleteOrder
     * @implements {EvaService}
     */
    class AutocompleteOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AutocompleteOrder;
        response?: EVA.Core.AutocompleteOrderResponse;
    }
    /**
     * Generate a barcode image.

The `Format` is one of;

- QR = 0
- EAN = 1
- Code39 = 2
- Code128 = 3

:lock: This is the safer variant to our `/barcode` endpoint, because the data moves _inside_ the POST body instead of through GET query params
:bulb: You can use the returned `Data` in combination with `MimeType` to render the image directly, for example;

```
  <img src='data:{MimeType};base64,{Data}' />
```
     *
     * @export
     * @class Barcode
     * @implements {EvaService}
     */
    class Barcode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.Barcode;
        response?: EVA.Core.BarcodeResponse;
    }
    /**
     * Books the given invoice by its `ID`.

Booking an invoice has the side effect of creating a payment on the order for the amount of the invoice.
     *
     * @export
     * @class BookInvoice
     * @implements {EvaService}
     */
    class BookInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.BookInvoice;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates some of the non-unique the OrganizationUnit's settings. This service can be used to only update a subset of the settings of the OrganizationUnit by providing only the fields you want to update.
If a field is not part of the request message (or has value `undefined`) then the setting is not updated. If the field is present but has a value of `null` then it's assumed you want to unset the setting.

The targeted OrganizationUnits can be identified by providing `OrganizationUnitIDs` or an `OrganizationUnitSetID`.

All OrganizationUnits will be updated, if one of them fails, no changes will be persisted.
     *
     * @export
     * @class BulkUpdateOrganizationUnitSettings
     * @implements {EvaService}
     */
    class BulkUpdateOrganizationUnitSettings implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.BulkUpdateOrganizationUnitSettings;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CanProcessInitialCycleCount
     * @implements {EvaService}
     */
    class CanProcessInitialCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CanProcessInitialCycleCount;
        response?: EVA.Core.CanProcessInitialCycleCountResponse;
    }
    /**
     * Cancel the pending cash PaymentTransactions on the Order to remove the AML checks.
     *
     * @export
     * @class CancelAMLChecksOnOrder
     * @implements {EvaService}
     */
    class CancelAMLChecksOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CancelAMLChecksOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Cancel a discount order line.

When the discount of the given line is a coupon triggerd discount, the coupon will be removed from the order.
When the discount of the given line is a manual discount, the manual discount will be cancelled.
When the discount of the given line is an automatic discount, the discount will be disabled on the order. To re-enable it you must call the 'EnableDisabledDiscountOnOrder' service. You can check which discount are disabled on an order by calling 'ListDisabledDiscountsOnOrder'
     *
     * @export
     * @class CancelDiscountOrderLine
     * @implements {EvaService}
     */
    class CancelDiscountOrderLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CancelDiscountOrderLine;
        response?: EVA.Core.SimpleShoppingCartResponse;
    }
    /**
     * Cancel a full stock count, cancelling all related labels and their results.
     *
     * @export
     * @class CancelFullStockCount
     * @implements {EvaService}
     */
    class CancelFullStockCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CancelFullStockCount;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Cancel a specific Full Stock Count label.
     *
     * @export
     * @class CancelFullStockCountLabel
     * @implements {EvaService}
     */
    class CancelFullStockCountLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CancelFullStockCountLabel;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Cancel a complete order or a subset of lines.

The order can be identified by an OrderID or by BackendID + BackendSystemID, the lines can be identified by an OrderLineID or by BackendID.

The result will tell you if it was successful.
     *
     * @export
     * @class CancelOrder
     * @implements {EvaService}
     */
    class CancelOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CancelOrder;
        response?: EVA.Core.CancelOrderResponse;
    }
    /**
     * Cancels the OrderFulfillmentLines for the given OrderID and provided OrderLineIDs.
It will cancel all not already cancelled OrderFulfillmentLines for the orderlines.
This will cause them to be re-routed and possible be fulfilled through another supplier or method.
     *
     * @export
     * @class CancelOrderLineFulfillments
     * @implements {EvaService}
     */
    class CancelOrderLineFulfillments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CancelOrderLineFulfillments;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Cancels a non-confirmed payment transaction
     *
     * @export
     * @class CancelPayment
     * @implements {EvaService}
     */
    class CancelPayment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CancelPayment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CancelSecondChanceProductRegistration
     * @implements {EvaService}
     */
    class CancelSecondChanceProductRegistration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CancelSecondChanceProductRegistration;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Cancels shipments without receipt of goods, also removing stock in transit.
     *
     * @export
     * @class CancelShipments
     * @implements {EvaService}
     */
    class CancelShipments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CancelShipments;
        response?: EVA.Core.CancelShipmentsResponse;
    }
    /**
     * Sends a capture request to the PSP for the given PaymentTransaction with the given amount.
     *
     * @export
     * @class CapturePayment
     * @implements {EvaService}
     */
    class CapturePayment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CapturePayment;
        response?: EVA.Core.CapturePaymentResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CaptureShipment
     * @implements {EvaService}
     */
    class CaptureShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CaptureShipment;
        response?: EVA.Core.CaptureShipmentResponse;
    }
    /**
     * Checks a balance of given card, where `Handler` or `Barcode` determines the giftcard provider to consult.
     *
     * @export
     * @class CardBalanceCheck
     * @implements {EvaService}
     */
    class CardBalanceCheck implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CardBalanceCheck;
        response?: EVA.Core.CardBalanceCheckResponse;
    }
    /**
     * Update some OrderLines to a state so that they can be carried out in a store

If the products should be ordered from a supplier first, the `OrderTheLinesFromASupplier` property should be true.

When no OrderLineIDs are provided all eligable lines in the Order are updated
     *
     * @export
     * @class ChangeOrderLinesToCarryOut
     * @implements {EvaService}
     */
    class ChangeOrderLinesToCarryOut implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ChangeOrderLinesToCarryOut;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update some OrderLines to a state so that they can be delivered to the ShippingAddress on the order

When no OrderLineIDs are provided all eligable lines in the Order are updated
     *
     * @export
     * @class ChangeOrderLinesToDelivery
     * @implements {EvaService}
     */
    class ChangeOrderLinesToDelivery implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ChangeOrderLinesToDelivery;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update some OrderLines to a state so that they can be pickup up in a store by the customer attached to the order

If no PickupOrganizationUnit was choses yet, it should be provided in the `PickupOrganizationUnitID` property.

When no OrderLineIDs are provided all eligable lines in the Order are updated
     *
     * @export
     * @class ChangeOrderLinesToPickup
     * @implements {EvaService}
     */
    class ChangeOrderLinesToPickup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ChangeOrderLinesToPickup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Change the password of a user.

When invalid, the service will return a 400 BAD REQUEST;

- SecurityPolicy:UserOrPasswordInvalid
- SecurityPolicy:TooShort
- SecurityPolicy:TooLong
- SecurityPolicy.NewPasswordIsRequired
- SecurityPolicy.Pwned
- General
     *
     * @export
     * @class ChangeUserPassword
     * @implements {EvaService}
     */
    class ChangeUserPassword implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ChangeUserPassword;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Checks if the given EmailAddress is available to create an account for.

When creating Employee accounts, the `AsEmployee` property should be set to true.
     *
     * @export
     * @class CheckEmailAddressAvailability
     * @implements {EvaService}
     */
    class CheckEmailAddressAvailability implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CheckEmailAddressAvailability;
        response?: EVA.Core.CheckUsernameAvailabilityResponse;
    }
    /**
     * Checks if a magic login was confirmed already.
     *
     * @export
     * @class CheckMagicLink
     * @implements {EvaService}
     */
    class CheckMagicLink implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CheckMagicLink;
        response?: EVA.Core.CheckMagicLinkResponse;
    }
    /**
     * Checks if the given Nickname is available to create an account for.

When creating Employee accounts, the `AsEmployee` property should be set to true.
     *
     * @export
     * @class CheckNicknameAvailability
     * @implements {EvaService}
     */
    class CheckNicknameAvailability implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CheckNicknameAvailability;
        response?: EVA.Core.CheckUsernameAvailabilityResponse;
    }
    /**
     * Check if the order has fraudulent shipping or billing addresses
     *
     * @export
     * @class CheckOrderAddressFraud
     * @implements {EvaService}
     */
    class CheckOrderAddressFraud implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CheckOrderAddressFraud;
        response?: EVA.Core.CheckOrderAddressFraudResponse;
    }
    /**
     * Check if order BackendID exists
     *
     * @export
     * @class CheckOrderBackendIDExists
     * @implements {EvaService}
     */
    class CheckOrderBackendIDExists implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CheckOrderBackendIDExists;
        response?: EVA.Core.BackendIDExistsResponse;
    }
    /**
     * Check if shipment BackendID exists
     *
     * @export
     * @class CheckShipmentBackendIDExists
     * @implements {EvaService}
     */
    class CheckShipmentBackendIDExists implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CheckShipmentBackendIDExists;
        response?: EVA.Core.BackendIDExistsResponse;
    }
    /**
     * Closes the financial period for your current OrganizationUnit.
     *
     * @export
     * @class CloseFinancialPeriod
     * @implements {EvaService}
     */
    class CloseFinancialPeriod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CloseFinancialPeriod;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service allows you to manually commit the stock for the given OrderLineIDs.
The response contains whether or not the commitment was successful, per OrderLine.

Commitment can be unsucessful for two reasons:

- No commitment is necessary. This is because it's an OrderLine that can't be committed, for example one without a stock product, or because the order type doesn't allow commitments.
- Insufficient stock. This service only allows creating a commitment when there's sufficient available stock. Use the MoveCommitments or UncommitOrderLines services to create available stock.
     *
     * @export
     * @class CommitOrderLines
     * @implements {EvaService}
     */
    class CommitOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CommitOrderLines;
        response?: EVA.Core.CommitOrderLinesResponse;
    }
    /**
     * Completes the closing of a cash journal, making the device once again unavailable for cash transactions until a new journal is opened.

This service expects a list of coins and notes, which summed up must equal the given `TotalAmount`.

If the `TotalAmount` is different from what was expected (returned as `ExpectedClosingAmount` in `StartCloseCashJournal` ) then this deviation
must be corrected and provided through a list of corrections in the `Corrections` field.

For cash journals that either do not use or ignore cash journals, most properties will automatically be set to the expected amounts. Should you
want to enforce validation, regardless, specify the `ForceValidation` as `true`.

Pending cash deposits need to be accounted for as well - supply their `ID`s as `PendingCashDeposits`. Do not add the `Amount` of cash deposits
to the `TotalAmount`, as the user cannot usually verify the cash deposit content.
     *
     * @export
     * @class CompleteCloseCashJournal
     * @implements {EvaService}
     */
    class CompleteCloseCashJournal implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteCloseCashJournal;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Completes a ConsolidatedPickTask task.
     *
     * @export
     * @class CompleteConsolidatedPickTask
     * @implements {EvaService}
     */
    class CompleteConsolidatedPickTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteConsolidatedPickTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Completes a ConsolidatedZonePickTask
     *
     * @export
     * @class CompleteConsolidatedZonePickTask
     * @implements {EvaService}
     */
    class CompleteConsolidatedZonePickTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteConsolidatedZonePickTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Completes a custom usertask
     *
     * @export
     * @class CompleteCustomUserTask
     * @implements {EvaService}
     */
    class CompleteCustomUserTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteCustomUserTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Completes a full stock count task. User task must be active and assigned to the current user.
     *
     * @export
     * @class CompleteFullStockCount
     * @implements {EvaService}
     */
    class CompleteFullStockCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteFullStockCount;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Completes a Full Stock Count label task. User task must be active and assigned to the current user.
     *
     * @export
     * @class CompleteFullStockCountLabel
     * @implements {EvaService}
     */
    class CompleteFullStockCountLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteFullStockCountLabel;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Complete multiple print SFS tasks.
     *
     * @export
     * @class CompleteMultipleShipFromStoreTasks
     * @implements {EvaService}
     */
    class CompleteMultipleShipFromStoreTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteMultipleShipFromStoreTasks;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Completes the opening of a cash journal. After this service has been called, it's possible to make cash transactions on this device.

This service expects a list of the coins and notes that were counted.

When the opening amount is different from last period's closing amount, it's required that a correction is provided for this difference
in `OpeningDeviationCorrection`. This simply consists of a description and an amount.

For cash journals that either do not use or ignore cash journals, most properties will automatically be set to the expected amounts. Should you
want to enforce validation, regardless, specify the `ForceValidation` as `true`.

Pending cash deposits need to be accounted for as well - supply their `ID`s as `PendingCashDeposits`. Do not add the `Amount` of cash deposits
to the `TotalAmount`, as the user cannot usually verify the cash deposit content.
     *
     * @export
     * @class CompleteOpenCashJournal
     * @implements {EvaService}
     */
    class CompleteOpenCashJournal implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteOpenCashJournal;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CompletePrintPriceLabelTask
     * @implements {EvaService}
     */
    class CompletePrintPriceLabelTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompletePrintPriceLabelTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CompletePrintTask
     * @implements {EvaService}
     */
    class CompletePrintTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompletePrintTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CompleteReservationCleanup
     * @implements {EvaService}
     */
    class CompleteReservationCleanup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteReservationCleanup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Complete a ReservationPickupTask task
     *
     * @export
     * @class CompleteReservationPickupTask
     * @implements {EvaService}
     */
    class CompleteReservationPickupTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteReservationPickupTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Completes the ReturnToSupplier task.

When CompleteReturnToSupplierTask is called and there are no lines on the task shipment, the shipment will be removed.
     *
     * @export
     * @class CompleteReturnToSupplierTask
     * @implements {EvaService}
     */
    class CompleteReturnToSupplierTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteReturnToSupplierTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CompleteSecondChanceProductRegistration
     * @implements {EvaService}
     */
    class CompleteSecondChanceProductRegistration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteSecondChanceProductRegistration;
        response?: EVA.Core.CompleteSecondChanceProductRegistrationResponse;
    }
    /**
     * Completes the ShipFromOrder task, indicating that the shipment of the order has taken place.

The order will be shipped and the shop stock will be moved to Transit.
     *
     * @export
     * @class CompleteShipFromStoreTask
     * @implements {EvaService}
     */
    class CompleteShipFromStoreTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteShipFromStoreTask;
        response?: EVA.Core.CompleteShipFromStoreTaskResponse;
    }
    /**
     * Completes a Stock Movement Follow Up task, using the `WorkSet` returned by the `StartStockMovementFollowUp` service.
     *
     * @export
     * @class CompleteStockMovementFollowUp
     * @implements {EvaService}
     */
    class CompleteStockMovementFollowUp implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteStockMovementFollowUp;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Completes a Stock Replenishment task, using the UserTaskIDs from the `WorkSet` returned by the `StartStockReplenishment` service.
     *
     * @export
     * @class CompleteStockReplenishment
     * @implements {EvaService}
     */
    class CompleteStockReplenishment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteStockReplenishment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Complete a StockReservation task that has been started with the StartStockReservation(2) service

The service should be called with a line for each orderline that should be reserved. It is not required anymore to send a line for each item on a line.

Provide each line with an OrderLineID and a QuantityTo(Cancel/Reserve/Order).
     *
     * @export
     * @class CompleteStockReservationTask
     * @implements {EvaService}
     */
    class CompleteStockReservationTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteStockReservationTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CompleteZonedCycleCountPreCount
     * @implements {EvaService}
     */
    class CompleteZonedCycleCountPreCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteZonedCycleCountPreCount;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CompleteZonedCycleCounts
     * @implements {EvaService}
     */
    class CompleteZonedCycleCounts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CompleteZonedCycleCounts;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Confirms a magic login by `ManualCode`.
     *
     * @export
     * @class ConfirmMagicLink
     * @implements {EvaService}
     */
    class ConfirmMagicLink implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ConfirmMagicLink;
        response?: EVA.Core.ConfirmMagicLinkResponse;
    }
    /**
     * Confirms that specified Purchase Order will be delivered, identified by the `OrderID` parameter.

Optionally use the `BackendID` parameter to set the BackendID on the Purchase Order.
     *
     * @export
     * @class ConfirmPurchaseOrder
     * @implements {EvaService}
     */
    class ConfirmPurchaseOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ConfirmPurchaseOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Confirm a Subscription with a Token

The service returns the status of the subscription
     *
     * @export
     * @class ConfirmSubscription
     * @implements {EvaService}
     */
    class ConfirmSubscription implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ConfirmSubscription;
        response?: EVA.Core.ConfirmSubscriptionResponse;
    }
    /**
     * Confirms an email address validation by entering the code that was sent through `ValidateUserEmailAddress`.
     *
     * @export
     * @class ConfirmUserEmailAddressValidation
     * @implements {EvaService}
     */
    class ConfirmUserEmailAddressValidation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ConfirmUserEmailAddressValidation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Copies EntityTranslations from one entity to another. Replaces any existing translations of the target entity.
     *
     * @export
     * @class CopyEntityTranslations
     * @implements {EvaService}
     */
    class CopyEntityTranslations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CopyEntityTranslations;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CountProductForInitialCycleCount
     * @implements {EvaService}
     */
    class CountProductForInitialCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CountProductForInitialCycleCount;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CountStockLabelForZonedCycleCount
     * @implements {EvaService}
     */
    class CountStockLabelForZonedCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CountStockLabelForZonedCycleCount;
        response?: EVA.Core.CountStockLabelForZonedCycleCountResponse;
    }
    /**
     * Create a new Address in the AddressBook of an User
     *
     * @export
     * @class CreateAddressBookItem
     * @implements {EvaService}
     */
    class CreateAddressBookItem implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateAddressBookItem;
        response?: EVA.Core.CreateAddressBookItemResponse;
    }
    /**
     * Create an anonymous token for specified `OrganizationUnitID` and `LanguageID`. The response is your new token, or a current one if it already existed.
     *
     * @export
     * @class CreateAnonymousToken
     * @implements {EvaService}
     */
    class CreateAnonymousToken implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateAnonymousToken;
        response?: EVA.Core.CreateAnonymousTokenResponse;
    }
    /**
     * Create an anonymous wishlist.
     *
     * @export
     * @class CreateAnonymousWishlist
     * @implements {EvaService}
     */
    class CreateAnonymousWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateAnonymousWishlist;
        response?: EVA.Core.CreateAnonymousWishlistResponse;
    }
    /**
     * Creates an API key for specified user
     *
     * @export
     * @class CreateApiKey
     * @implements {EvaService}
     */
    class CreateApiKey implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateApiKey;
        response?: EVA.Core.CreateApiKeyResponse;
    }
    /**
     * Creates a API user
     *
     * @export
     * @class CreateApiUser
     * @implements {EvaService}
     */
    class CreateApiUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateApiUser;
        response?: EVA.Core.CreateApiUserResponse;
    }
    /**
     * Create app rule
     *
     * @export
     * @class CreateAppRule
     * @implements {EvaService}
     */
    class CreateAppRule implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateAppRule;
        response?: EVA.Core.CreateAppRuleResponse;
    }
    /**
     * Creates a new AuditChainConfiguration, which allows you to configure the starting number of a sequence and its prefix.
Creating a configuration for an audit chain that already exists does not apply this configuration. A new chain would need to be created by incrementing the Auditing:Version setting.
     *
     * @export
     * @class CreateAuditChainConfiguration
     * @implements {EvaService}
     */
    class CreateAuditChainConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateAuditChainConfiguration;
        response?: EVA.Core.CreateAuditChainConfigurationResponse;
    }
    /**
     * Create a cash correction reason
     *
     * @export
     * @class CreateCashCorrectionReason
     * @implements {EvaService}
     */
    class CreateCashCorrectionReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCashCorrectionReason;
        response?: EVA.Core.CreateCashCorrectionReasonResponse;
    }
    /**
     * Creates a cash deposit for the provided device, currency and payment type.

When not explicitly delaying the drop-off, it will be booked to the current active cash journal and financial period.
     *
     * @export
     * @class CreateCashDeposit
     * @implements {EvaService}
     */
    class CreateCashDeposit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCashDeposit;
        response?: EVA.Core.CreateCashDepositResponse;
    }
    /**
     * Creates a cash expense.

This service expects:
- `DeviceID`, which is the cash device the money was used from.
- `TypeID` which is one of the available expense types.
- An `Amount`, which is the amount of cash that was taken from or added to the device.
The amount should always be positive, but the semantics of the amount depends on the AmountType of the ExpenseType:
if ExpenseType.AmountType = Expense, then the amount is booked as an expense, if it's Income then it's booked as income. Either way, it should be a positive number.
- A `TaxCodeID`, which refers to one of the available tax codes.
- A `Description`, which is the reason for this expense (for example: 'lunch').
- A `BlobID`, which contains an image of the receipt.
- A `SignatureBlobID`, which contains an image of the signature
     *
     * @export
     * @class CreateCashExpense
     * @implements {EvaService}
     */
    class CreateCashExpense implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCashExpense;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create and attach a new company to an User

Only Name is required.
     *
     * @export
     * @class CreateCompanyForUser
     * @implements {EvaService}
     */
    class CreateCompanyForUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCompanyForUser;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Creates consolidated pick task
     *
     * @export
     * @class CreateConsolidatedPickTask
     * @implements {EvaService}
     */
    class CreateConsolidatedPickTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateConsolidatedPickTask;
        response?: EVA.Core.CreateConsolidatedPickTaskResponse;
    }
    /**
     * Creates consolidated pick task
     *
     * @export
     * @class CreateConsolidatedPickTask_Async
     * @implements {EvaService}
     */
    class CreateConsolidatedPickTask_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateConsolidatedPickTask;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * Creates consolidated pick task
     *
     * @export
     * @class CreateConsolidatedPickTask_AsyncResult
     * @implements {EvaService}
     */
    class CreateConsolidatedPickTask_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.CreateConsolidatedPickTaskResponse;
    }
    /**
     * Creates a new company associated with the consumer type
     *
     * @export
     * @class CreateConsumerCompany
     * @implements {EvaService}
     */
    class CreateConsumerCompany implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateConsumerCompany;
        response?: EVA.Core.CreateConsumerCompanyResponse;
    }
    /**
     * Creates a custom order status which can be set on an order
     *
     * @export
     * @class CreateCustomOrderStatus
     * @implements {EvaService}
     */
    class CreateCustomOrderStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCustomOrderStatus;
        response?: EVA.Core.CreateCustomOrderStatusResponse;
    }
    /**
     * Creates a new custom usertask
     *
     * @export
     * @class CreateCustomUserTask
     * @implements {EvaService}
     */
    class CreateCustomUserTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCustomUserTask;
        response?: EVA.Core.CreateCustomUserTaskResponse;
    }
    /**
     * Creates a new template for custom user tasks
     *
     * @export
     * @class CreateCustomUserTaskTemplate
     * @implements {EvaService}
     */
    class CreateCustomUserTaskTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCustomUserTaskTemplate;
        response?: EVA.Core.CreateCustomUserTaskTemplateResponse;
    }
    /**
     * Creates a new custom usertask type.
     *
     * @export
     * @class CreateCustomUserTaskType
     * @implements {EvaService}
     */
    class CreateCustomUserTaskType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCustomUserTaskType;
        response?: EVA.Core.CreateCustomUserTaskTypeResponse;
    }
    /**
     * Creates a customer.

This is allowed when you are an employee, or by anonymous if the CustomerCanRegister (default true) is enabled.

Be sure to handle errors for registering, which could be (amongst others);

  * InvalidEmailAddress
  * EmailAddressInUse
  * NicknameInUse
  * FiscalIDInUse
     *
     * @export
     * @class CreateCustomer
     * @implements {EvaService}
     */
    class CreateCustomer implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCustomer;
        response?: EVA.Core.CreateCustomerResponse;
    }
    /**
     * Create a return order from an existing sales order (`OrderID`).

When a `TargetOrderID` has been specified, newly created lines are added to that specific order. When omitted, the lines are either added to the
existing, original order (in case that is still permitted) or a new order is created automatically.

Returns information on the (newly created?) return order.
     *
     * @export
     * @class CreateCustomerReturn
     * @implements {EvaService}
     */
    class CreateCustomerReturn implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCustomerReturn;
        response?: EVA.Core.SimpleShoppingCartResponse;
    }
    /**
     * Creates a return order for a set of products for which there is no original order in EVA.
     *
     * @export
     * @class CreateCustomerReturnWithoutOrder
     * @implements {EvaService}
     */
    class CreateCustomerReturnWithoutOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCustomerReturnWithoutOrder;
        response?: EVA.Core.SimpleShoppingCartResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateCycleCountZone
     * @implements {EvaService}
     */
    class CreateCycleCountZone implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCycleCountZone;
        response?: EVA.Core.CreateCycleCountZoneResponse;
    }
    /**
     * Creates a new CycleCountZoneGroup.
     *
     * @export
     * @class CreateCycleCountZoneGroup
     * @implements {EvaService}
     */
    class CreateCycleCountZoneGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateCycleCountZoneGroup;
        response?: EVA.Core.CreateCycleCountZoneGroupResponse;
    }
    /**
     * Creates a blob intended for app diagnostics - will log the user context to determine user, organization unit and user agent.
     *
     * @export
     * @class CreateDiagnosticsBlob
     * @implements {EvaService}
     */
    class CreateDiagnosticsBlob implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateDiagnosticsBlob;
        response?: EVA.Core.CreateDiagnosticsBlobResponse;
    }
    /**
     * Create event export configurations (HTTPS only).

Event Export Configurations allow you to configure specific endpoints (webhooks) for EventTargets and EventTypes.

- Not setting EventTarget & EventType will configure the endpoint for all targets and types
- Set AuthenticationHeaderType and AuthenticationHeaderValue if you want to configure some sort of authentication
- Make sure to catch & store the `SecretKey1` and `SecretKey2`, as there will be no other way of retrieving them after this service call!
     *
     * @export
     * @class CreateEventExportConfiguration
     * @implements {EvaService}
     */
    class CreateEventExportConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateEventExportConfiguration;
        response?: EVA.Core.CreateEventExportConfigurationResponse;
    }
    /**
     * Create a new Flight object
     *
     * @export
     * @class CreateFlight
     * @implements {EvaService}
     */
    class CreateFlight implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateFlight;
        response?: EVA.Core.CreateFlightResponse;
    }
    /**
     * Initiate the Full Stock Count process for the current store. **Prevents basic store functionalities once started.**
     *
     * @export
     * @class CreateFullStockCount
     * @implements {EvaService}
     */
    class CreateFullStockCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateFullStockCount;
        response?: EVA.Core.CreateFullStockCountResponse;
    }
    /**
     * Create a return that is returned in the store. This will handle the return in the original OU and move stock to the current OU. For now, this only supports orders sold from a webshop.
     *
     * @export
     * @class CreateInStoreReturn
     * @implements {EvaService}
     */
    class CreateInStoreReturn implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInStoreReturn;
        response?: EVA.Core.CreateInStoreReturnResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateInitialCycleCount
     * @implements {EvaService}
     */
    class CreateInitialCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInitialCycleCount;
        response?: EVA.Core.CreateInitialCycleCountResponse;
    }
    /**
     * Convert the given `Order` to a InterbranchOrder.
     *
     * @export
     * @class CreateInterbranchOrder
     * @implements {EvaService}
     */
    class CreateInterbranchOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInterbranchOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Creates a new invoice for the given `OrganizationUnitID` (the one who writes the invoice)
and the `SupplierOrganizationUnitID` (the one who receives good/services).

- `InvoiceDate` is the date on which the invoice was created.
- `InvoiceNumber` is the unique number for the invoice.
- `Blobs` can contain PDF's or other media that can for example be a copy of the invoice
- `OriginalTotalAmount` is the amount that is expected on the invoice. E.g. this can differ from the amount that is computed based on the invoice lines
- `Type` indicates if this is an invoice for a sales or purchase orders
     *
     * @export
     * @class CreateInvoice
     * @implements {EvaService}
     */
    class CreateInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoice;
        response?: EVA.Core.CreateInvoiceResponse;
    }
    /**
     * Create an InvoiceAdditionalAmount
     *
     * @export
     * @class CreateInvoiceAdditionalAmount
     * @implements {EvaService}
     */
    class CreateInvoiceAdditionalAmount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoiceAdditionalAmount;
        response?: EVA.Core.CreateInvoiceAdditionalAmountResponse;
    }
    /**
     * Create a InvoiceAdditionalAmountType
     *
     * @export
     * @class CreateInvoiceAdditionalAmountType
     * @implements {EvaService}
     */
    class CreateInvoiceAdditionalAmountType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoiceAdditionalAmountType;
        response?: EVA.Core.CreateInvoiceAdditionalAmountTypeResponse;
    }
    /**
     * Create an InvoiceDispute
     *
     * @export
     * @class CreateInvoiceDispute
     * @implements {EvaService}
     */
    class CreateInvoiceDispute implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoiceDispute;
        response?: EVA.Core.CreateInvoiceDisputeResponse;
    }
    /**
     * Create a InvoiceDisputeReason
     *
     * @export
     * @class CreateInvoiceDisputeReason
     * @implements {EvaService}
     */
    class CreateInvoiceDisputeReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoiceDisputeReason;
        response?: EVA.Core.CreateInvoiceDisputeReasonResponse;
    }
    /**
     * Create an InvoiceDisputeResolveAction
     *
     * @export
     * @class CreateInvoiceDisputeResolveAction
     * @implements {EvaService}
     */
    class CreateInvoiceDisputeResolveAction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoiceDisputeResolveAction;
        response?: EVA.Core.CreateInvoiceDisputeResolveActionResponse;
    }
    /**
     * Creates lines on the given invoice (`InvoiceID`).

It is required to supply an OrderLine or a ShipmentLine. All other properties are optional and will be copied from the shipment-/order- line.
     *
     * @export
     * @class CreateInvoiceLines
     * @implements {EvaService}
     */
    class CreateInvoiceLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoiceLines;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Allocates a payment transaction to either a specific invoice, or to all open invoices sequentially.
     *
     * @export
     * @class CreateInvoicePayment
     * @implements {EvaService}
     */
    class CreateInvoicePayment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoicePayment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create a correction on an already invoiced line.

This can be used to refund a partial amount on a completed OrderLine, in cases where the product should still be returnable.

Specify either `OrderLineID` and `CorrectionAmount`, or `Lines`.

- If the product is not returnable anymore, a return without retake should be created
- This will result in a new line that will be invoiced immediately and shipped by default (AutoProcess.Ship is the default variant).
- If the shipping isn't desired, use AutoProcess.None;
- If a refund is desired, use AutoProcess.ShipAndRefund.
     *
     * @export
     * @class CreateInvoicedOrderLineCorrection
     * @implements {EvaService}
     */
    class CreateInvoicedOrderLineCorrection implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoicedOrderLineCorrection;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create invoices for an Order. Optionally specify the specific lines with quantities.
     *
     * @export
     * @class CreateInvoicesForOrder
     * @implements {EvaService}
     */
    class CreateInvoicesForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateInvoicesForOrder;
        response?: EVA.Core.CreateInvoicesForOrderResponse;
    }
    /**
     * Creates an order with a manually issued invoice. This will insert a committed Invoice with a custom sequencing number and additional
`InvoiceManualData`. It will be fully paid with a custom payment type.

The specified Order will be paid, placed, shipped and completed, but without any additional side effects like stock ledgers or commitments.
     *
     * @export
     * @class CreateManualInvoice
     * @implements {EvaService}
     */
    class CreateManualInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateManualInvoice;
        response?: EVA.Core.CreateManualInvoiceResponse;
    }
    /**
     * Create a new manual tax exemption reason.
     *
     * @export
     * @class CreateManualTaxExemptionReason
     * @implements {EvaService}
     */
    class CreateManualTaxExemptionReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateManualTaxExemptionReason;
        response?: EVA.Core.CreateManualTaxExemptionReasonResponse;
    }
    /**
     * Converts an order to an offer
     *
     * @export
     * @class CreateOfferFromOrder
     * @implements {EvaService}
     */
    class CreateOfferFromOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOfferFromOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create a new OpeningHoursTemplate
     *
     * @export
     * @class CreateOpeningHoursTemplate
     * @implements {EvaService}
     */
    class CreateOpeningHoursTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOpeningHoursTemplate;
        response?: EVA.Core.CreateOpeningHoursTemplateResponse;
    }
    /**
     * Provides a mapping between an organization unit and a company based on a country. This mapping is used in scenarios for where delivery orders are
in play. E.g endless aisle (delivery order from a store) and an online order from a website for example.

To comply with regulations, these invoices that are generated require the tax registration number from the company that will be delivering the goods
with its specific registration from the country that the goods are send to.

If a mapping already exists for given `OrganizationUnitID` and `CountryID`, the `CompanyID` will be updated and `AlreadyExisted` in the response will be `true`.
     *
     * @export
     * @class CreateOrUpdateInvoiceCompanyMapping
     * @implements {EvaService}
     */
    class CreateOrUpdateInvoiceCompanyMapping implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrUpdateInvoiceCompanyMapping;
        response?: EVA.Core.CreateOrUpdateInvoiceCompanyMappingResponse;
    }
    /**
     * Create a new Order of the given Type
     *
     * @export
     * @class CreateOrder
     * @implements {EvaService}
     */
    class CreateOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrder;
        response?: EVA.Core.CreateOrderResponse;
    }
    /**
     * Creates and sets an app token on specified order, to allow accessing it from an anonymous context. The order may not have a Customer attached.
     *
     * @export
     * @class CreateOrderAppToken
     * @implements {EvaService}
     */
    class CreateOrderAppToken implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrderAppToken;
        response?: EVA.Core.CreateOrderAppTokenResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateOrderCustomStatus
     * @implements {EvaService}
     */
    class CreateOrderCustomStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrderCustomStatus;
        response?: EVA.Core.CreateOrderCustomStatusResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateOrderCustomType
     * @implements {EvaService}
     */
    class CreateOrderCustomType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrderCustomType;
        response?: EVA.Core.CreateOrderCustomTypeResponse;
    }
    /**
     * Create an Order from an Offer specified by `OrderID`.
     *
     * @export
     * @class CreateOrderFromOffer
     * @implements {EvaService}
     */
    class CreateOrderFromOffer implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrderFromOffer;
        response?: EVA.Core.CreateOrderFromOfferResponse;
    }
    /**
     * Create a new order ledger of the given type
     *
     * @export
     * @class CreateOrderLedger
     * @implements {EvaService}
     */
    class CreateOrderLedger implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrderLedger;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create a price correction on the order line, specified by `OrderID` and `OrderLineID`. Provide the new `UnitPrice` this order line should have, and include
a property `IncludingTax` to indicate if you mean IN or EX taxes.
     *
     * @export
     * @class CreateOrderLineUnitPriceCorrection
     * @implements {EvaService}
     */
    class CreateOrderLineUnitPriceCorrection implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrderLineUnitPriceCorrection;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateOrderLink
     * @implements {EvaService}
     */
    class CreateOrderLink implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrderLink;
        response?: EVA.Core.CreateOrderLinkResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateOrderLinkType
     * @implements {EvaService}
     */
    class CreateOrderLinkType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrderLinkType;
        response?: EVA.Core.CreateOrderLinkTypeResponse;
    }
    /**
     * Create a new order verification reason.
     *
     * @export
     * @class CreateOrderVerificationReason
     * @implements {EvaService}
     */
    class CreateOrderVerificationReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrderVerificationReason;
        response?: EVA.Core.CreateOrderVerificationReasonResponse;
    }
    /**
     * Create a new OrganizationUnit
     *
     * @export
     * @class CreateOrganizationUnit
     * @implements {EvaService}
     */
    class CreateOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrganizationUnit;
        response?: EVA.Core.CreateOrganizationUnitResponse;
    }
    /**
     * Service for creating a `OrganizationUnitSupplier`
     *
     * @export
     * @class CreateOrganizationUnitSupplier
     * @implements {EvaService}
     */
    class CreateOrganizationUnitSupplier implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateOrganizationUnitSupplier;
        response?: EVA.Core.CreateOrganizationUnitSupplierResponse;
    }
    /**
     * After having called `GetPasskeyChallenge` to obtain a challenge and some information from the server, and after pushing that through WebAuthn using `navigator.credentials.create`,
you extract information from the generated credential and send it back to the server for storage.

1. The `Challenge` matching the one originally provided through `GetPasskeyChallenge`
2. The `PublicKey` generated (base64 encoded)
3. The `AuthenticatorData` generated (base64 encoded)

The creation will succeed if the `Challenge` matches the one we created for you and all the data checks out, after which you can use your new key to log in!

When you want to log in, simply call `Login` using `CustomAuthenticatorType` set to "Passkey" and it will reply with a challenge and some information inside the
`CustomAuthenticateResponseData` property, which you can push through WebAuthn using `navigator.credentials.get` to sign that challenge with your private key. Exact the information
from the generated credential and call `Login` again with the same `CustomAuthenticatorType`, but this time include `CustomAuthenticateData` with an object of type
`PasskeyAuthenticateMessage` containing;

- The `ID` as the "rawId" of the used credential (base64 encoded)
- The `Signature` that was created to sign the challenge (base64 encoded)
- The `ClientData` to include some client information (base64 encoded json)
- The `AuthenticatorData` to include some authenticator data (base64 encoded)

If this all checks out, you're logged in! As with every `Login`, be sure to specify user types and organization units of your preference.
     *
     * @export
     * @class CreatePasskey
     * @implements {EvaService}
     */
    class CreatePasskey implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreatePasskey;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreatePayment
     * @implements {EvaService}
     */
    class CreatePayment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreatePayment;
        response?: EVA.Core.CreatePaymentResponse;
    }
    /**
     * Create a new paymenttransaction ledger of the given type. Only custom types are allowed
     *
     * @export
     * @class CreatePaymentTransactionLedger
     * @implements {EvaService}
     */
    class CreatePaymentTransactionLedger implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreatePaymentTransactionLedger;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Creates a new Print Price Label task with specified products, or append these to existing tasks when applicable.
     *
     * @export
     * @class CreatePrintPriceLabelTask
     * @implements {EvaService}
     */
    class CreatePrintPriceLabelTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreatePrintPriceLabelTask;
        response?: EVA.Core.CreatePrintPriceLabelTaskResponse;
    }
    /**
     * Create a UserTask in specified organization unit to print a specific `BlobID` or `BlobUrl`.
     *
     * @export
     * @class CreatePrintTask
     * @implements {EvaService}
     */
    class CreatePrintTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreatePrintTask;
        response?: EVA.Core.CreatePrintTaskResponse;
    }
    /**
     * This service creates a ProductBundle for the given BundleProductID.

The request body object contains three parameters:

- BackendID: Optional. Can be filled in with the identifier of this bundle by which it is known in another system.
- BundleProductID: Required. This should refer to an EVA product with type `BundleProduct`.
- Type: Required. This specifies the type of bundle: discount or configurable. The type can not be changed after the bundle has been created.
- Lines: Optional. These specify the `ProductBundle`'s lines. If left empty or null at creation, these can be filled in by calling `UpdateProductBundle`. When specified, a line consists of the following:
    - BackendID: Optional. If the bundle exists in another system, this can refer to it.
    - Options: A list of available options for this line from which can be chosen. Only contains a ProductID for now, which may refer to a normal sellable product, a product with Type `ConfigurableProduct` or a `BundleProduct` (which would create bundles of bundles).
    - DefaultProductID: Optional, but required if there are multiple options or if the line option is a Configurable product. The DefaultProductID cannot be a `ConfigurableProduct` or a `BundleProduct`.
    - IsRequired: Is this line an optional part of the bundle or not?
    - IsPromptSuppressed: Should the FE suppress the modal that allows the user to select a matching bundle for the scanned product
     *
     * @export
     * @class CreateProductBundle
     * @implements {EvaService}
     */
    class CreateProductBundle implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateProductBundle;
        response?: EVA.Core.CreateProductBundleResponse;
    }
    /**
     * Create a new structure of products.
     *
     * @export
     * @class CreateProductStructure
     * @implements {EvaService}
     */
    class CreateProductStructure implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateProductStructure;
        response?: EVA.Core.CreateProductStructureResponse;
    }
    /**
     * Creates a new ProductZone.
     *
     * @export
     * @class CreateProductZone
     * @implements {EvaService}
     */
    class CreateProductZone implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateProductZone;
        response?: EVA.Core.CreateProductZoneResponse;
    }
    /**
     * Creates a new ProductZoneGroup.
     *
     * @export
     * @class CreateProductZoneGroup
     * @implements {EvaService}
     */
    class CreateProductZoneGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateProductZoneGroup;
        response?: EVA.Core.CreateProductZoneGroupResponse;
    }
    /**
     * Creates a Purchase Order for supplied `OrganizationUnitID`, from optional `SupplierID`.

Use the `BackendID` to indicate the supplier order reference.
     *
     * @export
     * @class CreatePurchaseOrder
     * @implements {EvaService}
     */
    class CreatePurchaseOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreatePurchaseOrder;
        response?: EVA.Core.CreatePurchaseOrderResponse;
    }
    /**
     * Create a price correction on the purchase order line.
     *
     * @export
     * @class CreatePurchaseOrderLineUnitPriceCorrection
     * @implements {EvaService}
     */
    class CreatePurchaseOrderLineUnitPriceCorrection implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreatePurchaseOrderLineUnitPriceCorrection;
        response?: EVA.Core.CreatePurchaseOrderLineUnitPriceCorrectionResponse;
    }
    /**
     * Service for creating a `QuickBuy`
     *
     * @export
     * @class CreateQuickBuy
     * @implements {EvaService}
     */
    class CreateQuickBuy implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateQuickBuy;
        response?: EVA.Core.CreateQuickBuyResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateRecurringTask
     * @implements {EvaService}
     */
    class CreateRecurringTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateRecurringTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create a refund

With this service a refund can be done based on an existing PaymentTransaction, or without it for an Order.

It is required to specify a transaction or to specify the PaymentType to use.

When no Amount is specified the open amount of the order will be used.
     *
     * @export
     * @class CreateRefund
     * @implements {EvaService}
     */
    class CreateRefund implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateRefund;
        response?: EVA.Core.CreateRefundResponse;
    }
    /**
     * Create a repair
     *
     * @export
     * @class CreateRepair
     * @implements {EvaService}
     */
    class CreateRepair implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateRepair;
        response?: EVA.Core.CreateRepairResponse;
    }
    /**
     * Creates a new interaction associated with a specific repair
     *
     * @export
     * @class CreateRepairInteraction
     * @implements {EvaService}
     */
    class CreateRepairInteraction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateRepairInteraction;
        response?: EVA.Core.CreateRepairInteractionResponse;
    }
    /**
     * Convert the given `Order` to a ReturnToSupplierOrder. Existing none-product lines will be cancelled.
     *
     * @export
     * @class CreateReturnToSupplierOrder
     * @implements {EvaService}
     */
    class CreateReturnToSupplierOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateReturnToSupplierOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create a new ReturnToSupplierRequest

A ReturnToSupplierRequest is used to generate ReturnToSupplier orders for Shops

- Name: For identification of the request (required)
- OrganizationUnitSetID: The organizationunits (filtered on Type = Shop) that will get a RTS (required)
- ProductSearchTemplateID: Only products matching this template will be added to the orders, if supplied (optional)
- RecallPercentage: Percentage of the current stock to recall. Value should be between 0.00 and 1.00. (required)
- Filters, CountryID, LanguageID: Instead of a productsearchtemplate, products can also be defined by providing a manual filter (all optional)
     *
     * @export
     * @class CreateReturnToSupplierRequest
     * @implements {EvaService}
     */
    class CreateReturnToSupplierRequest implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateReturnToSupplierRequest;
        response?: EVA.Core.CreateReturnToSupplierRequestResponse;
    }
    /**
     * Creates a new script.
     *
     * @export
     * @class CreateScript
     * @implements {EvaService}
     */
    class CreateScript implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateScript;
        response?: EVA.Core.CreateScriptResponse;
    }
    /**
     * Create a new Serial Number
     *
     * @export
     * @class CreateSerialNumber
     * @implements {EvaService}
     */
    class CreateSerialNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateSerialNumber;
        response?: EVA.Core.CreateSerialNumberResponse;
    }
    /**
     * Creates a new entity for Serial Number Configuration, defining a set of requirements where linked products should adhere to.
     *
     * @export
     * @class CreateSerialNumberConfiguration
     * @implements {EvaService}
     */
    class CreateSerialNumberConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateSerialNumberConfiguration;
        response?: EVA.Core.CreateSerialNumberConfigurationResponse;
    }
    /**
     * Associate a product with a Serial Number Configuration, or re-associate with this Serial Number Configuration in case it was previously assigned to another.
     *
     * @export
     * @class CreateSerialNumberProduct
     * @implements {EvaService}
     */
    class CreateSerialNumberProduct implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateSerialNumberProduct;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create a new shipment

It is required to specify an Order. This can be done with `OrderID` or by sending `OrderBackendID` and `OrderBackendSystemID`.

To create a pending Shipment, set `IsPending` to true. A pending shipment has to be shipped with the ShipShipment service.
     *
     * @export
     * @class CreateShipment
     * @implements {EvaService}
     */
    class CreateShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateShipment;
        response?: EVA.Core.CreateShipmentResponse;
    }
    /**
     * Create a new shipment identifier.
     *
     * @export
     * @class CreateShipmentIdentifier
     * @implements {EvaService}
     */
    class CreateShipmentIdentifier implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateShipmentIdentifier;
        response?: EVA.Core.CreateShipmentIdentifierResponse;
    }
    /**
     * Creates a shipment receipt with `Lines` for specified `ShipmentID`.

* `CompletesShipment` / `Lines.CompletesShipmentLine` will close the shipment (line) for futher receipts
* `Lines.QuantityReceived` is the delta quantity received in this receipt, so not the new total quantity received
* `StockLabelID` is the target StockLabel for the received stock - overrides the StockLabel of the purchase order line
     *
     * @export
     * @class CreateShipmentReceipt
     * @implements {EvaService}
     */
    class CreateShipmentReceipt implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateShipmentReceipt;
        response?: EVA.Core.CreateShipmentReceiptResponse;
    }
    /**
     * Creates a stock mutation for the given organization unit and product. Note that this service supports creating delta's as well as modifying in absolute quantities based on the `Delta` property.'
     *
     * @export
     * @class CreateStockMutations
     * @implements {EvaService}
     */
    class CreateStockMutations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateStockMutations;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Creates a StockNotification for a product and organization, for specified EmailAddress - will return its ID.
     *
     * @export
     * @class CreateStockNotification
     * @implements {EvaService}
     */
    class CreateStockNotification implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateStockNotification;
        response?: EVA.Core.StockNotificationResponse;
    }
    /**
     * Creates a StockNotification for a product and organization (or current when null), for the current user - will return its ID.
     *
     * @export
     * @class CreateStockNotificationForCurrentUser
     * @implements {EvaService}
     */
    class CreateStockNotificationForCurrentUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateStockNotificationForCurrentUser;
        response?: EVA.Core.StockNotificationResponse;
    }
    /**
     * Creates a new Stock Replenishment task.
     *
     * @export
     * @class CreateStockReplenishmentTask
     * @implements {EvaService}
     */
    class CreateStockReplenishmentTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateStockReplenishmentTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Creates new Stock Replenishment tasks in bulk, with `DetailedProducts` being an array of products details.
     *
     * @export
     * @class CreateStockReplenishmentTasks
     * @implements {EvaService}
     */
    class CreateStockReplenishmentTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateStockReplenishmentTasks;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateTaxCode
     * @implements {EvaService}
     */
    class CreateTaxCode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateTaxCode;
        response?: EVA.Core.CreateTaxCodeResponse;
    }
    /**
     * Create a new TaxCodeMapping. Only one mapping per TaxCode & TaxProvider is allowed.
     *
     * @export
     * @class CreateTaxCodeMapping
     * @implements {EvaService}
     */
    class CreateTaxCodeMapping implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateTaxCodeMapping;
        response?: EVA.Core.CreateTaxCodeMappingResponse;
    }
    /**
     * Create a new TaxProvider
     *
     * @export
     * @class CreateTaxProvider
     * @implements {EvaService}
     */
    class CreateTaxProvider implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateTaxProvider;
        response?: EVA.Core.CreateTaxProviderResponse;
    }
    /**
     * Creates a tax rate.

* `CountryID` is a foreign key to the Countries table, which uses the ISO 3166-2 standard as ID
* `CountrySubdivisionID` is a foreign key to the CountrySubdivisions table, which uses the ISO 3166-2 standard as ID
* `TaxCodeID` is a foreign key to the TaxCodes table, which can be listed using the `ListTaxCodes` service

Percentages are a human readable formatted result of a fraction - applications use a `Rate` instead as an actual result of fraction to maintain
precision and integrity. If you wish to set a 21% tax percentage, that means a 1.21 tax rate. Setting a rate of 1.00 implies a 0% tax percentage,
and setting to 2.00 would mean a rediculous 100% tax percentage, doubling prices. You can probably tell that a `Rate` of 2.00 here is much more
readable than a 100% increase, but hey; nobody's forcing you to stick to percentages. At least - we're not!

Oh, and you can set a tax rate of 0.00, which would mean it will be fully sponsored by the government and will be free to purchase. The services
block anything lower than 0.00 and higher than 2.00 for sanity reasons. Will recap when governments gone mad.

:key: The combination of `CountryID`, `CountrySubdivision` and `TaxCodeID` is unique and will be enforced by EVA.

:point_right: `StartDate` and `EndDate` are optional, and can be used to transition from one rate to a next when it changes. Both these dates
are inclusive, meaning you'd want to set the `EndDate` of the expiring tax rate to 2018-12-31 and the `StartDate` of the new tax rate to
2019-01-01 (for example).
     *
     * @export
     * @class CreateTaxRate
     * @implements {EvaService}
     */
    class CreateTaxRate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateTaxRate;
        response?: EVA.Core.CreateTaxRateResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateUserAssociation
     * @implements {EvaService}
     */
    class CreateUserAssociation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateUserAssociation;
        response?: EVA.Core.CreateUserAssociationResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateUserAssociationType
     * @implements {EvaService}
     */
    class CreateUserAssociationType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateUserAssociationType;
        response?: EVA.Core.CreateUserAssociationTypeResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateUserGroup
     * @implements {EvaService}
     */
    class CreateUserGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateUserGroup;
        response?: EVA.Core.CreateUserGroupResponse;
    }
    /**
     * Creates a UserInteraction. A UserInteraction logs an interaction between an employee and a customer, for example the fact the customer was called.

UserID is optional but only if OrderID is also provided and the given Order is attached to a customer. If so then the UserInteraction is attached to the Order.Customer.
     *
     * @export
     * @class CreateUserInteraction
     * @implements {EvaService}
     */
    class CreateUserInteraction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateUserInteraction;
        response?: EVA.Core.CreateUserInteractionResponse;
    }
    /**
     * Creates a UserPhoneNumber for the current user, or the UserID specified if you're allowed to do so.
     *
     * @export
     * @class CreateUserPhoneNumber
     * @implements {EvaService}
     */
    class CreateUserPhoneNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateUserPhoneNumber;
        response?: EVA.Core.CreateUserPhoneNumberResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateUserTaskNotCompletedReason
     * @implements {EvaService}
     */
    class CreateUserTaskNotCompletedReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateUserTaskNotCompletedReason;
        response?: EVA.Core.CreateUserTaskNotCompletedReasonResponse;
    }
    /**
     * Creates a new User Task Priority.
     *
     * @export
     * @class CreateUserTaskPriority
     * @implements {EvaService}
     */
    class CreateUserTaskPriority implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateUserTaskPriority;
        response?: EVA.Core.CreateUserTaskPriorityResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class CreateUserTaskTypeOrganizationUnitSet
     * @implements {EvaService}
     */
    class CreateUserTaskTypeOrganizationUnitSet implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateUserTaskTypeOrganizationUnitSet;
        response?: EVA.Core.UserTaskTypeOrganizationUnitSetResponse;
    }
    /**
     * Creates a user wishlist.
     *
     * @export
     * @class CreateUserWishlist
     * @implements {EvaService}
     */
    class CreateUserWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateUserWishlist;
        response?: EVA.Core.CreateUserWishlistResponse;
    }
    /**
     * Creates one cycle count for the requested Product and Organization Unit (with origin Manual)
     *
     * @export
     * @class CreateZonedCycleCount
     * @implements {EvaService}
     */
    class CreateZonedCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateZonedCycleCount;
        response?: EVA.Core.CreateZonedCycleCountResponse;
    }
    /**
     * Creates a Zoned Cycle Count Schedule.

It is required to specify either a set `Date`, or a `CycleCountIntervalInDays`.
     *
     * @export
     * @class CreateZonedCycleCountSchedule
     * @implements {EvaService}
     */
    class CreateZonedCycleCountSchedule implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateZonedCycleCountSchedule;
        response?: EVA.Core.CreateZonedCycleCountScheduleResponse;
    }
    /**
     * Creates cycle counts for all requested combinations of Products and Organization Units (with origin Manual). Use this service through the async service endpoints.
     *
     * @export
     * @class CreateZonedCycleCounts
     * @implements {EvaService}
     */
    class CreateZonedCycleCounts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateZonedCycleCounts;
        response?: EVA.Core.CreateZonedCycleCountsResponse;
    }
    /**
     * Creates cycle counts for all requested combinations of (type Stock) Products and Organization Units that match specified filters (with origin Manual). Use this service through the async service endpoints.
     *
     * @export
     * @class CreateZonedCycleCountsByQuery
     * @implements {EvaService}
     */
    class CreateZonedCycleCountsByQuery implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateZonedCycleCountsByQuery;
        response?: EVA.Core.CreateZonedCycleCountsByQueryResponse;
    }
    /**
     * Creates cycle counts for all requested combinations of (type Stock) Products and Organization Units that match specified filters (with origin Manual). Use this service through the async service endpoints.
     *
     * @export
     * @class CreateZonedCycleCountsByQuery_Async
     * @implements {EvaService}
     */
    class CreateZonedCycleCountsByQuery_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateZonedCycleCountsByQuery;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * Creates cycle counts for all requested combinations of (type Stock) Products and Organization Units that match specified filters (with origin Manual). Use this service through the async service endpoints.
     *
     * @export
     * @class CreateZonedCycleCountsByQuery_AsyncResult
     * @implements {EvaService}
     */
    class CreateZonedCycleCountsByQuery_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.CreateZonedCycleCountsByQueryResponse;
    }
    /**
     * Creates cycle counts for all requested combinations of Products and Organization Units (with origin Manual). Use this service through the async service endpoints.
     *
     * @export
     * @class CreateZonedCycleCounts_Async
     * @implements {EvaService}
     */
    class CreateZonedCycleCounts_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CreateZonedCycleCounts;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * Creates cycle counts for all requested combinations of Products and Organization Units (with origin Manual). Use this service through the async service endpoints.
     *
     * @export
     * @class CreateZonedCycleCounts_AsyncResult
     * @implements {EvaService}
     */
    class CreateZonedCycleCounts_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.CreateZonedCycleCountsResponse;
    }
    /**
     * Service for customers to add a blob to a case.
     *
     * @export
     * @class CustomerAddBlobToCase
     * @implements {EvaService}
     */
    class CustomerAddBlobToCase implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerAddBlobToCase;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for customers to add a blob to a case interacion.
     *
     * @export
     * @class CustomerAddBlobToCaseInteraction
     * @implements {EvaService}
     */
    class CustomerAddBlobToCaseInteraction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerAddBlobToCaseInteraction;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for customers to create a new case.
     *
     * @export
     * @class CustomerCreateCase
     * @implements {EvaService}
     */
    class CustomerCreateCase implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerCreateCase;
        response?: EVA.Core.CustomerCreateCaseResponse;
    }
    /**
     * Service for customers to create a case
     *
     * @export
     * @class CustomerCreateCaseInteraction
     * @implements {EvaService}
     */
    class CustomerCreateCaseInteraction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerCreateCaseInteraction;
        response?: EVA.Core.CustomerCreateCaseInteractionResponse;
    }
    /**
     * Service for customers to delete a case interaction
     *
     * @export
     * @class CustomerDeleteCaseInteraction
     * @implements {EvaService}
     */
    class CustomerDeleteCaseInteraction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerDeleteCaseInteraction;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for customers to get a case by its ID.
     *
     * @export
     * @class CustomerGetCaseByID
     * @implements {EvaService}
     */
    class CustomerGetCaseByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerGetCaseByID;
        response?: EVA.Core.CustomerGetCaseByIDResponse;
    }
    /**
     * Service for customers to get a case interaction by id
     *
     * @export
     * @class CustomerGetCaseInteraction
     * @implements {EvaService}
     */
    class CustomerGetCaseInteraction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerGetCaseInteraction;
        response?: EVA.Core.CustomerGetCaseInteractionResponse;
    }
    /**
     * Service for customers to list case interactions
     *
     * @export
     * @class CustomerListCaseInteractions
     * @implements {EvaService}
     */
    class CustomerListCaseInteractions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerListCaseInteractions;
        response?: EVA.Core.CustomerListCaseInteractionsResponse;
    }
    /**
     * Service for customers to list cases. Returns a paged response.
     *
     * @export
     * @class CustomerListCases
     * @implements {EvaService}
     */
    class CustomerListCases implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerListCases;
        response?: EVA.Core.CustomerListCasesResponse;
    }
    /**
     * Service for customers to remove a blob from a case.
     *
     * @export
     * @class CustomerRemoveCaseBlob
     * @implements {EvaService}
     */
    class CustomerRemoveCaseBlob implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerRemoveCaseBlob;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for customers to remove a blob from a case interaction.
     *
     * @export
     * @class CustomerRemoveCaseInteractionBlob
     * @implements {EvaService}
     */
    class CustomerRemoveCaseInteractionBlob implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerRemoveCaseInteractionBlob;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for customers to update an existing case.
     *
     * @export
     * @class CustomerUpdateCase
     * @implements {EvaService}
     */
    class CustomerUpdateCase implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerUpdateCase;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for customers to update a case interaction
     *
     * @export
     * @class CustomerUpdateCaseInteraction
     * @implements {EvaService}
     */
    class CustomerUpdateCaseInteraction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.CustomerUpdateCaseInteraction;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deactivates all active user tasks based on organization unit. Not supported on production environments!
     *
     * @export
     * @class DeactivateAllUserTasks
     * @implements {EvaService}
     */
    class DeactivateAllUserTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeactivateAllUserTasks;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeactivateUserTask
     * @implements {EvaService}
     */
    class DeactivateUserTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeactivateUserTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deactivate the given ZonedCycleCounts

Also deactivates linked PreCounts
     *
     * @export
     * @class DeactivateZonedCycleCount
     * @implements {EvaService}
     */
    class DeactivateZonedCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeactivateZonedCycleCount;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete an Address in the AddressBook of an User

It's not allowed to delete an address when it's the default and there are more than 1 remaining items in the address book
     *
     * @export
     * @class DeleteAddressBookItem
     * @implements {EvaService}
     */
    class DeleteAddressBookItem implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteAddressBookItem;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes an existing anonymous token.
     *
     * @export
     * @class DeleteAnonymousToken
     * @implements {EvaService}
     */
    class DeleteAnonymousToken implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteAnonymousToken;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes an anonymous wishlist.
     *
     * @export
     * @class DeleteAnonymousWishlist
     * @implements {EvaService}
     */
    class DeleteAnonymousWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteAnonymousWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes an API key
     *
     * @export
     * @class DeleteApiKey
     * @implements {EvaService}
     */
    class DeleteApiKey implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteApiKey;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete app rule
     *
     * @export
     * @class DeleteAppRule
     * @implements {EvaService}
     */
    class DeleteAppRule implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteAppRule;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes an AuditChainConfiguration
     *
     * @export
     * @class DeleteAuditChainConfiguration
     * @implements {EvaService}
     */
    class DeleteAuditChainConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteAuditChainConfiguration;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a cash correction reason
     *
     * @export
     * @class DeleteCashCorrectionReason
     * @implements {EvaService}
     */
    class DeleteCashCorrectionReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteCashCorrectionReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a deposit for the given `ID`.

Cash deposits in status Pending (`-1`) can always be removed, status Done (`0`) ("Dropped-off") requires the cash journal and the financial period to still be open.
     *
     * @export
     * @class DeleteCashDeposit
     * @implements {EvaService}
     */
    class DeleteCashDeposit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteCashDeposit;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes an expense for the given `ID`. This is only possible while the cash journal and the financial period are still open.
     *
     * @export
     * @class DeleteCashExpense
     * @implements {EvaService}
     */
    class DeleteCashExpense implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteCashExpense;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a custom order status
     *
     * @export
     * @class DeleteCustomOrderStatus
     * @implements {EvaService}
     */
    class DeleteCustomOrderStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteCustomOrderStatus;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a specific custom usertask template
     *
     * @export
     * @class DeleteCustomUserTaskTemplate
     * @implements {EvaService}
     */
    class DeleteCustomUserTaskTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteCustomUserTaskTemplate;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteCycleCountZone
     * @implements {EvaService}
     */
    class DeleteCycleCountZone implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteCycleCountZone;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a CycleCountZoneGroup by its ID
     *
     * @export
     * @class DeleteCycleCountZoneGroup
     * @implements {EvaService}
     */
    class DeleteCycleCountZoneGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteCycleCountZoneGroup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for deleting an existing `EntityTranslation`
     *
     * @export
     * @class DeleteEntityTranslation
     * @implements {EvaService}
     */
    class DeleteEntityTranslation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteEntityTranslation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for deleting a existing `EventExportConfiguration`
     *
     * @export
     * @class DeleteEventExportConfiguration
     * @implements {EvaService}
     */
    class DeleteEventExportConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteEventExportConfiguration;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete an existing Flight
     *
     * @export
     * @class DeleteFlight
     * @implements {EvaService}
     */
    class DeleteFlight implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteFlight;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteInitialCycleCountResult
     * @implements {EvaService}
     */
    class DeleteInitialCycleCountResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteInitialCycleCountResult;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes the given invoice. Note, can only delete invoices that have not been booked yet.
     *
     * @export
     * @class DeleteInvoice
     * @implements {EvaService}
     */
    class DeleteInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteInvoice;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete an InvoiceAdditionalAmount
     *
     * @export
     * @class DeleteInvoiceAdditionalAmount
     * @implements {EvaService}
     */
    class DeleteInvoiceAdditionalAmount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteInvoiceAdditionalAmount;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a InvoiceAdditionalAmountType, this is only allowed if the type is never used
     *
     * @export
     * @class DeleteInvoiceAdditionalAmountType
     * @implements {EvaService}
     */
    class DeleteInvoiceAdditionalAmountType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteInvoiceAdditionalAmountType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Removes a blob from an invoice, either by `BlobID` or `BlobUrl`.
     *
     * @export
     * @class DeleteInvoiceBlob
     * @implements {EvaService}
     */
    class DeleteInvoiceBlob implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteInvoiceBlob;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes an `InvoiceCompanyMapping`.
     *
     * @export
     * @class DeleteInvoiceCompanyMapping
     * @implements {EvaService}
     */
    class DeleteInvoiceCompanyMapping implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteInvoiceCompanyMapping;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete an InvoiceDispute
     *
     * @export
     * @class DeleteInvoiceDispute
     * @implements {EvaService}
     */
    class DeleteInvoiceDispute implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteInvoiceDispute;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a InvoiceDisputeReason, this is only allowed if the reason is never used
     *
     * @export
     * @class DeleteInvoiceDisputeReason
     * @implements {EvaService}
     */
    class DeleteInvoiceDisputeReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteInvoiceDisputeReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete an InvoiceDisputeResolveAction
     *
     * @export
     * @class DeleteInvoiceDisputeResolveAction
     * @implements {EvaService}
     */
    class DeleteInvoiceDisputeResolveAction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteInvoiceDisputeResolveAction;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a manual tax exemption reason.
     *
     * @export
     * @class DeleteManualTaxExemptionReason
     * @implements {EvaService}
     */
    class DeleteManualTaxExemptionReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteManualTaxExemptionReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a manual tax exemption reason line.
     *
     * @export
     * @class DeleteManualTaxExemptionReasonLine
     * @implements {EvaService}
     */
    class DeleteManualTaxExemptionReasonLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteManualTaxExemptionReasonLine;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete an OpeningHoursTemplate
     *
     * @export
     * @class DeleteOpeningHoursTemplate
     * @implements {EvaService}
     */
    class DeleteOpeningHoursTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteOpeningHoursTemplate;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteOrderCustomStatus
     * @implements {EvaService}
     */
    class DeleteOrderCustomStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteOrderCustomStatus;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteOrderCustomType
     * @implements {EvaService}
     */
    class DeleteOrderCustomType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteOrderCustomType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteOrderLink
     * @implements {EvaService}
     */
    class DeleteOrderLink implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteOrderLink;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteOrderLinkType
     * @implements {EvaService}
     */
    class DeleteOrderLinkType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteOrderLinkType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete an order verification reason
     *
     * @export
     * @class DeleteOrderVerificationReason
     * @implements {EvaService}
     */
    class DeleteOrderVerificationReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteOrderVerificationReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteOrganizationUnit
     * @implements {EvaService}
     */
    class DeleteOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteOrganizationUnit;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteOrganizationUnitSupplier
     * @implements {EvaService}
     */
    class DeleteOrganizationUnitSupplier implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteOrganizationUnitSupplier;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Remove a passkey configuration for the current user.
     *
     * @export
     * @class DeletePasskey
     * @implements {EvaService}
     */
    class DeletePasskey implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeletePasskey;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Remove all passkey configurations for the specified user, as employee management feature.
     *
     * @export
     * @class DeletePasskeyForUser
     * @implements {EvaService}
     */
    class DeletePasskeyForUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeletePasskeyForUser;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a ProductBundle.
     *
     * @export
     * @class DeleteProductBundle
     * @implements {EvaService}
     */
    class DeleteProductBundle implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteProductBundle;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes an existing structure of products by `ID`.
     *
     * @export
     * @class DeleteProductStructure
     * @implements {EvaService}
     */
    class DeleteProductStructure implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteProductStructure;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a ProductZone by its ID
     *
     * @export
     * @class DeleteProductZone
     * @implements {EvaService}
     */
    class DeleteProductZone implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteProductZone;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a ProductZoneGroup by its ID
     *
     * @export
     * @class DeleteProductZoneGroup
     * @implements {EvaService}
     */
    class DeleteProductZoneGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteProductZoneGroup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for deleting an existing `QuickBuy` by ID
     *
     * @export
     * @class DeleteQuickBuy
     * @implements {EvaService}
     */
    class DeleteQuickBuy implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteQuickBuy;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteRecurringTask
     * @implements {EvaService}
     */
    class DeleteRecurringTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteRecurringTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a ReturnToSupplierRequest

This is only allowed for Request in the state New or Error
     *
     * @export
     * @class DeleteReturnToSupplierRequest
     * @implements {EvaService}
     */
    class DeleteReturnToSupplierRequest implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteReturnToSupplierRequest;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a script
     *
     * @export
     * @class DeleteScript
     * @implements {EvaService}
     */
    class DeleteScript implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteScript;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes an existing Serial Number Configuration by it's `ID` if this is not linked to any existing Serial Number Product.
     *
     * @export
     * @class DeleteSerialNumberConfiguration
     * @implements {EvaService}
     */
    class DeleteSerialNumberConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteSerialNumberConfiguration;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Removes a product association with a Serial Number Configuration.
     *
     * @export
     * @class DeleteSerialNumberProduct
     * @implements {EvaService}
     */
    class DeleteSerialNumberProduct implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteSerialNumberProduct;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete an existing shipment that has status Pending.

Works only for SalesOrders at this moment.
     *
     * @export
     * @class DeleteShipment
     * @implements {EvaService}
     */
    class DeleteShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteShipment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete an existing shipment identifier.
     *
     * @export
     * @class DeleteShipmentIdentifier
     * @implements {EvaService}
     */
    class DeleteShipmentIdentifier implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteShipmentIdentifier;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a StockNotification by ID. When it was created with an EmailAddress, repeat that in this request by means of authentication.
     *
     * @export
     * @class DeleteStockNotification
     * @implements {EvaService}
     */
    class DeleteStockNotification implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteStockNotification;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for deleting an existing `StringTranslation`
     *
     * @export
     * @class DeleteStringTranslation
     * @implements {EvaService}
     */
    class DeleteStringTranslation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteStringTranslation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteTaxCode
     * @implements {EvaService}
     */
    class DeleteTaxCode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteTaxCode;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a tax mapping.
     *
     * @export
     * @class DeleteTaxCodeMapping
     * @implements {EvaService}
     */
    class DeleteTaxCodeMapping implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteTaxCodeMapping;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a TaxProvider
     *
     * @export
     * @class DeleteTaxProvider
     * @implements {EvaService}
     */
    class DeleteTaxProvider implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteTaxProvider;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a tax rate.
     *
     * @export
     * @class DeleteTaxRate
     * @implements {EvaService}
     */
    class DeleteTaxRate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteTaxRate;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a User by it's ID.
     *
     * @export
     * @class DeleteUser
     * @implements {EvaService}
     */
    class DeleteUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUser;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteUserAssociation
     * @implements {EvaService}
     */
    class DeleteUserAssociation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUserAssociation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteUserAssociationType
     * @implements {EvaService}
     */
    class DeleteUserAssociationType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUserAssociationType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteUserGroup
     * @implements {EvaService}
     */
    class DeleteUserGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUserGroup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a UserInteraction.
     *
     * @export
     * @class DeleteUserInteraction
     * @implements {EvaService}
     */
    class DeleteUserInteraction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUserInteraction;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete a user's phone number based on the provided ID. If you're trying to delete a user's primary phone number and that user has
more than two phone numbers currently, this service will return a `CannotDeletePrimaryPhoneNumber` error,
because it cannot determine what number should now become the user's primary phone number. If the user has two numbers, the other number
will become the primary.

This service only allows the number's own user or an employee to delete it.
     *
     * @export
     * @class DeleteUserPhoneNumber
     * @implements {EvaService}
     */
    class DeleteUserPhoneNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUserPhoneNumber;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteUserTaskNotCompletedReason
     * @implements {EvaService}
     */
    class DeleteUserTaskNotCompletedReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUserTaskNotCompletedReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes a User Task Priority.
     *
     * @export
     * @class DeleteUserTaskPriority
     * @implements {EvaService}
     */
    class DeleteUserTaskPriority implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUserTaskPriority;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DeleteUserTaskTypeOrganizationUnitSet
     * @implements {EvaService}
     */
    class DeleteUserTaskTypeOrganizationUnitSet implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUserTaskTypeOrganizationUnitSet;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Deletes the user wishlist.
     *
     * @export
     * @class DeleteUserWishlist
     * @implements {EvaService}
     */
    class DeleteUserWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUserWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delete multiple users by ID's.
This service is async only and only accepts 1000 ID's per request
     *
     * @export
     * @class DeleteUsers
     * @implements {EvaService}
     */
    class DeleteUsers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUsers;
        response?: EVA.Core.DeleteUsersResponse;
    }
    /**
     * Delete multiple users by ID's.
This service is async only and only accepts 1000 ID's per request
     *
     * @export
     * @class DeleteUsers_Async
     * @implements {EvaService}
     */
    class DeleteUsers_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteUsers;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * Delete multiple users by ID's.
This service is async only and only accepts 1000 ID's per request
     *
     * @export
     * @class DeleteUsers_AsyncResult
     * @implements {EvaService}
     */
    class DeleteUsers_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.DeleteUsersResponse;
    }
    /**
     * Deletes a Zoned Cycle Count Schedule.
     *
     * @export
     * @class DeleteZonedCycleCountSchedule
     * @implements {EvaService}
     */
    class DeleteZonedCycleCountSchedule implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeleteZonedCycleCountSchedule;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Delivers an undelivered SalesOrder shipment. This will give both the shipment and the orderline the status that they have been delivered.
     *
     * @export
     * @class DeliverShipment
     * @implements {EvaService}
     */
    class DeliverShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DeliverShipment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DetachCompanyFromUser
     * @implements {EvaService}
     */
    class DetachCompanyFromUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DetachCompanyFromUser;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Detaches the consumer company from an order.
     *
     * @export
     * @class DetachConsumerCompanyFromOrder
     * @implements {EvaService}
     */
    class DetachConsumerCompanyFromOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DetachConsumerCompanyFromOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Detach a Customer from an Order. The addresses on the Order will also be cleared.
     *
     * @export
     * @class DetachCustomerFromOrder
     * @implements {EvaService}
     */
    class DetachCustomerFromOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DetachCustomerFromOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DetachFromRepairUserTask
     * @implements {EvaService}
     */
    class DetachFromRepairUserTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DetachFromRepairUserTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Detach an identification document from an order. This will wipe the document and data.
     *
     * @export
     * @class DetachIdentificationFromOrder
     * @implements {EvaService}
     */
    class DetachIdentificationFromOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DetachIdentificationFromOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Attaches an OpeningHoursTemplate to the given OrganizationUnitSets.
     *
     * @export
     * @class DetachOpeningHoursTemplateToOrganizationUnitSets
     * @implements {EvaService}
     */
    class DetachOpeningHoursTemplateToOrganizationUnitSets implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DetachOpeningHoursTemplateToOrganizationUnitSets;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Detach the given Order from the attached customer and a session. A ReminderToken will be returned to be able to use the Order later with the `RecoverShoppingCartService`.

This service can be used to transfer an Order to a different User:
- DetachOrder
- Logout
- Login
- RecoverShoppingCart
     *
     * @export
     * @class DetachOrder
     * @implements {EvaService}
     */
    class DetachOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DetachOrder;
        response?: EVA.Core.DetachOrderResponse;
    }
    /**
     * Detaches an user from a consumer company.
     *
     * @export
     * @class DetachUserFromConsumerCompany
     * @implements {EvaService}
     */
    class DetachUserFromConsumerCompany implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DetachUserFromConsumerCompany;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Disables all two-factor authenticator for the given user
     *
     * @export
     * @class DisableAllTwoFactorAuthentication
     * @implements {EvaService}
     */
    class DisableAllTwoFactorAuthentication implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DisableAllTwoFactorAuthentication;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Disable a specific Discount for an Order, if the discount is active it will be removed from the Order, if it not active, it will not be applied anymore.
     *
     * @export
     * @class DisableDiscountOnOrder
     * @implements {EvaService}
     */
    class DisableDiscountOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DisableDiscountOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Disable all discounts of the given trigger(s) on the order.
     *
     * @export
     * @class DisableDiscountTriggersOnOrder
     * @implements {EvaService}
     */
    class DisableDiscountTriggersOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DisableDiscountTriggersOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Disable a given two-factor authenticator for the current user
     *
     * @export
     * @class DisableTwoFactorAuthentication
     * @implements {EvaService}
     */
    class DisableTwoFactorAuthentication implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DisableTwoFactorAuthentication;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Indicate that this was the current order, but has been dismissed.

This is purely a ledger and does not affect the order in any other way.
     *
     * @export
     * @class DismissOrder
     * @implements {EvaService}
     */
    class DismissOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DismissOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Download an Excel sample which can be filled with cycle count info. The populated excel can be uploaded through the `UploadCycleCountExcel` service.
     *
     * @export
     * @class DownloadCycleCountSample
     * @implements {EvaService}
     */
    class DownloadCycleCountSample implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadCycleCountSample;
        response?: EVA.Core.ResourceWithBlobIDResponseMessage;
    }
    /**
     * Download the GiftCard data for a specific order, based on given `OrderID` and `GiftCardID`. This order needs to be visible for the current user. Requires
the order to be shipped (automatically when it is fully paid).

Additionally, this service can be called with an `AccessToken` and `GiftCardID` combination. In this case, no user validation is done - you know both
keys to the GiftCard, so you are allowed to get it's information.

If you want, you can specify a `LanguageID` to use for rendering the report - when omitted, the `LanguageID` of the current user.

See `GetDigitalGiftCardsForOrder` service for more details.
     *
     * @export
     * @class DownloadDigitalGiftCard
     * @implements {EvaService}
     */
    class DownloadDigitalGiftCard implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadDigitalGiftCard;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * Download a template format for the full stock count result to upload.
     *
     * @export
     * @class DownloadExternalFullStockCountResultTemplate
     * @implements {EvaService}
     */
    class DownloadExternalFullStockCountResultTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadExternalFullStockCountResultTemplate;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * Downloads all generated audit files from the selected period as a single zip file, with a separate folder for each organization unit.

When selecting a container organization unit, all underlying organization units will be included.
     *
     * @export
     * @class DownloadFinancialPeriodAudits
     * @implements {EvaService}
     */
    class DownloadFinancialPeriodAudits implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadFinancialPeriodAudits;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DownloadFinancialPeriodDeposit
     * @implements {EvaService}
     */
    class DownloadFinancialPeriodDeposit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadFinancialPeriodDeposit;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * Download a template format for the flights file to upload.
     *
     * @export
     * @class DownloadFlightsFileTemplate
     * @implements {EvaService}
     */
    class DownloadFlightsFileTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadFlightsFileTemplate;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class DownloadFullStockCountLabelUploadTemplate
     * @implements {EvaService}
     */
    class DownloadFullStockCountLabelUploadTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadFullStockCountLabelUploadTemplate;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * Exports the summary of specified Full Stock Count to Excel.
     *
     * @export
     * @class DownloadFullStockCountSummaryExcel
     * @implements {EvaService}
     */
    class DownloadFullStockCountSummaryExcel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadFullStockCountSummaryExcel;
        response?: EVA.Core.ResourceWithBlobIDResponseMessage;
    }
    /**
     * Exports the summary of specified Full Stock Count to Excel.
     *
     * @export
     * @class DownloadFullStockCountSummaryExcel_Async
     * @implements {EvaService}
     */
    class DownloadFullStockCountSummaryExcel_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadFullStockCountSummaryExcel;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * Exports the summary of specified Full Stock Count to Excel.
     *
     * @export
     * @class DownloadFullStockCountSummaryExcel_AsyncResult
     * @implements {EvaService}
     */
    class DownloadFullStockCountSummaryExcel_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.ResourceWithBlobIDResponseMessage;
    }
    /**
     * Exports the summary of specified Full Stock Count to PDF.
     *
     * @export
     * @class DownloadFullStockCountSummaryPDF
     * @implements {EvaService}
     */
    class DownloadFullStockCountSummaryPDF implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadFullStockCountSummaryPDF;
        response?: EVA.Core.ResourceWithBlobIDResponseMessage;
    }
    /**
     * This service is used only for downloading invoices.
     *
     * @export
     * @class DownloadInvoice
     * @implements {EvaService}
     */
    class DownloadInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadInvoice;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * This service is used only for downloading offers.
     *
     * @export
     * @class DownloadOffer
     * @implements {EvaService}
     */
    class DownloadOffer implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadOffer;
        response?: EVA.Core.ResourceWithBlobIDResponseMessage;
    }
    /**
     * This service is used only for printing paper invoices.
     *
     * @export
     * @class DownloadOrderConfirmation
     * @implements {EvaService}
     */
    class DownloadOrderConfirmation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadOrderConfirmation;
        response?: EVA.Core.DownloadOrderConfirmationResponse;
    }
    /**
     * Download a sample Excel file which can be filled with special opening hours. The populated Excel file can be uploaded through the `ImportSpecialOpeningHours` service.
     *
     * @export
     * @class DownloadSpecialOpeningHoursExcelTemplate
     * @implements {EvaService}
     */
    class DownloadSpecialOpeningHoursExcelTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadSpecialOpeningHoursExcelTemplate;
        response?: EVA.Core.ResourceWithBlobIDResponseMessage;
    }
    /**
     * Download a template format for the stock file to upload.
     *
     * @export
     * @class DownloadStockExcelFileTemplate
     * @implements {EvaService}
     */
    class DownloadStockExcelFileTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadStockExcelFileTemplate;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * Prepare an attachment of an UserInteraction to be downloaded, will return an url that is valid for 5min.
     *
     * @export
     * @class DownloadUserInteractionCommunicationAttachment
     * @implements {EvaService}
     */
    class DownloadUserInteractionCommunicationAttachment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DownloadUserInteractionCommunicationAttachment;
        response?: EVA.Core.ResourceWithBlobIDResponseMessage;
    }
    /**
     * Drops-off a cash deposit previously created and stored on a device (`CreateCashDeposit.DelayDropOff`).

Cash deposits created this way will remain in status Pending (`-1`) until it is either dropped-off (`DropOffCashDeposit`) or deleted (`DeleteCashDeposit`). Whilst
pending, it will not create any events, will not be attached to any `CashTransactionLedger` and therefor will not be attached to any `FinancialPeriod`. You can find
these pending cash deposits using `ListCashDeposits`, but then only when filtering for a `DeviceID` and/or `OrganizationUnitID` (and optionally `StatusID`).
     *
     * @export
     * @class DropOffCashDeposit
     * @implements {EvaService}
     */
    class DropOffCashDeposit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DropOffCashDeposit;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Make a new `Order` based on an existing one.

* Optionally you can provide some OrderLineIDs to duplicate only a subset of the order.
* When an `OrganizationUnitID` is set, the Order will be created Originating and SoldFrom the specified OrganizationUnit
* To include the orignal GiftWrapping the flag `IncludeGiftWrapping` should be set to true. When combining this with a subset of lines, this can generate an error when some wrapped lines are not duplicated.
* To ignore shipping costs, set `IgnoreShippingCosts` to true (defaults to false)
     *
     * @export
     * @class DuplicateOrder
     * @implements {EvaService}
     */
    class DuplicateOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.DuplicateOrder;
        response?: EVA.Core.DuplicateOrderResponse;
    }
    /**
     * Edits an existing structure of products by `ID`.
     *
     * @export
     * @class EditProductStructure
     * @implements {EvaService}
     */
    class EditProductStructure implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EditProductStructure;
        response?: EVA.Core.EditProductStructureResponse;
    }
    /**
     * This service is used only for emailing invoices.
     *
     * @export
     * @class EmailInvoice
     * @implements {EvaService}
     */
    class EmailInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EmailInvoice;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used for emailing all invoices linked to an order ID.
     *
     * @export
     * @class EmailInvoices
     * @implements {EvaService}
     */
    class EmailInvoices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EmailInvoices;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used only for emailing offers.
     *
     * @export
     * @class EmailOffer
     * @implements {EvaService}
     */
    class EmailOffer implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EmailOffer;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used only for printing paper invoices.
     *
     * @export
     * @class EmailOrderConfirmation
     * @implements {EvaService}
     */
    class EmailOrderConfirmation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EmailOrderConfirmation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used to retrieve the TaxFreeForms associated with an order & send them to the specified email address.
     *
     * @export
     * @class EmailOrderTaxFreeForm
     * @implements {EvaService}
     */
    class EmailOrderTaxFreeForm implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EmailOrderTaxFreeForm;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used only for emailing thermal invoices
     *
     * @export
     * @class EmailThermalInvoice
     * @implements {EvaService}
     */
    class EmailThermalInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EmailThermalInvoice;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used only for emailing thermal invoices based on an order ID.
     *
     * @export
     * @class EmailThermalInvoices
     * @implements {EvaService}
     */
    class EmailThermalInvoices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EmailThermalInvoices;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Empty the full amount of a giftcard into an empty order.
     *
     * @export
     * @class EmptyGiftCard
     * @implements {EvaService}
     */
    class EmptyGiftCard implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EmptyGiftCard;
        response?: EVA.Core.EmptyGiftCardResponse;
    }
    /**
     * Enable a previous disabled Discount for an Order. If the Discount wasn't disabled, no error will be given.
     *
     * @export
     * @class EnableDisabledDiscountOnOrder
     * @implements {EvaService}
     */
    class EnableDisabledDiscountOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EnableDisabledDiscountOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Enable previously disabled discounts of the given trigger(s) on the order
     *
     * @export
     * @class EnableDiscountTriggersOnOrder
     * @implements {EvaService}
     */
    class EnableDiscountTriggersOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EnableDiscountTriggersOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Enable a given two-factor authenticator for the current user
     *
     * @export
     * @class EnableTwoFactorAuthentication
     * @implements {EvaService}
     */
    class EnableTwoFactorAuthentication implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EnableTwoFactorAuthentication;
        response?: EVA.Core.EnableTwoFactorAuthenticationResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class EnqueueTask
     * @implements {EvaService}
     */
    class EnqueueTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.EnqueueTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Executes the given barcode with an available handler that can execute it. To know if a barcode is executable, run ParseBarcode and look for the IsExecutable in the response.
     *
     * @export
     * @class ExecuteBarcode
     * @implements {EvaService}
     */
    class ExecuteBarcode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ExecuteBarcode;
        response?: EVA.Core.ExecuteBarcodeResponse;
    }
    /**
     * Marks the provided Order or OrderLine as exempt from tax, with specified code and reason.

When exempting the Order, this also marks all current OrderLines of the Order that haven't been invoiced yet as tax-exempt. All future
OrderLines created on this Order will then also automatically be made tax-exempt.

Calling this service is irreversible; once the Order is marked as tax exempt, it cannot be undone and can only be worked around by creating a new Order.
     *
     * @export
     * @class ExemptOrderFromTax
     * @implements {EvaService}
     */
    class ExemptOrderFromTax implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ExemptOrderFromTax;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Returns a tree representation of how the price was derived for the given ProductIDs in the given PriceListID. NOTICE THAT you can provide a maximum of 100 products to explain.
     *
     * @export
     * @class ExplainProductPrices
     * @implements {EvaService}
     */
    class ExplainProductPrices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ExplainProductPrices;
        response?: EVA.Core.ExplainProductPricesResponse;
    }
    /**
     * Attempts to explain the given script in plain English.
     *
     * @export
     * @class ExplainScript
     * @implements {EvaService}
     */
    class ExplainScript implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ExplainScript;
        response?: EVA.Core.ExplainScriptResponse;
    }
    /**
     * Fetches the user by its BackendID from the configured external provider. This service is usually used in combination with a call to SearchUsers. This service will create a user in EVA based on the details returned by the external provider if we don't already have a user with the same BackendID. Passing in the OrderID will attach the (created) user to the given order directly as well.
     *
     * @export
     * @class FetchUserFromExternalProvider
     * @implements {EvaService}
     */
    class FetchUserFromExternalProvider implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.FetchUserFromExternalProvider;
        response?: EVA.Core.FetchUserFromExternalProviderResponse;
    }
    /**
     * Validate all AML requirements and finalize the pending PaymentTransactions on the Order.
     *
     * @export
     * @class FinalizeAMLChecksOnOrder
     * @implements {EvaService}
     */
    class FinalizeAMLChecksOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.FinalizeAMLChecksOnOrder;
        response?: EVA.Core.FinalizeAMLChecksOnOrderResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class FinishRepair
     * @implements {EvaService}
     */
    class FinishRepair implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.FinishRepair;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Finish repair user task
     *
     * @export
     * @class FinishRepairUserTask
     * @implements {EvaService}
     */
    class FinishRepairUserTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.FinishRepairUserTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class FlushCache
     * @implements {EvaService}
     */
    class FlushCache implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.FlushCache;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ForcePlaceOrder
     * @implements {EvaService}
     */
    class ForcePlaceOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ForcePlaceOrder;
        response?: EVA.Core.ForcePlaceOrderResponse;
    }
    /**
     * Verify the customer of an order without QR code check.
     *
     * @export
     * @class ForceVerifyCustomer
     * @implements {EvaService}
     */
    class ForceVerifyCustomer implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ForceVerifyCustomer;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Receives a shipment completely as-is. Supply either `ID` or `BackendID`.
     *
     * @export
     * @class FullyReceiveShipment
     * @implements {EvaService}
     */
    class FullyReceiveShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.FullyReceiveShipment;
        response?: EVA.Core.FullyReceiveShipmentResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class FullyShipPurchaseOrder
     * @implements {EvaService}
     */
    class FullyShipPurchaseOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.FullyShipPurchaseOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for generating a new device hub barcode
     *
     * @export
     * @class GenerateDeviceHubBarcode
     * @implements {EvaService}
     */
    class GenerateDeviceHubBarcode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GenerateDeviceHubBarcode;
        response?: EVA.Core.GenerateDeviceHubBarcodeResponse;
    }
    /**
     * Generates an elevation barcode to elevate this user's rights, which can be scanned by a manager with sufficient rights to grant elevation.

Show the returned `Barcode` to a manager for scanning through the `ParseBarcode` service, preferably as QR code through our `/barcode` endpoint.

:shield: Requires `Security:ElevatedFunctionalityProvider` to contain "ElevationBarcode".

:point_right: Send the returned `RequestToken` as elevation header with your original call, but before you do, it needs to be confirmed first through a manager scan. You can use this
same token to subscribe to the `/elevationBarcode` SignalR hub to listen for this scan event.

:bulb: Optionally supply the `OrganizationUnitID`, `Functionality` and `Scope` of your original request, so we can check the manager rights directly from the scan
on their device. Omitting these here will fall back to the default security enforced from the user attempting to use the (confirmed) elevation token, which just gives
a nasty 403 on the employee's device.
     *
     * @export
     * @class GenerateElevationBarcode
     * @implements {EvaService}
     */
    class GenerateElevationBarcode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GenerateElevationBarcode;
        response?: EVA.Core.GenerateElevationBarcodeResponse;
    }
    /**
     * Service for generating a new scan mode barcode
     *
     * @export
     * @class GenerateScanModeBarcode
     * @implements {EvaService}
     */
    class GenerateScanModeBarcode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GenerateScanModeBarcode;
        response?: EVA.Core.GenerateScanModeBarcodeResponse;
    }
    /**
     * Service for generating a new session barcode
     *
     * @export
     * @class GenerateSessionBarcode
     * @implements {EvaService}
     */
    class GenerateSessionBarcode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GenerateSessionBarcode;
        response?: EVA.Core.GenerateSessionBarcodeResponse;
    }
    /**
     * Generates a temporary 5-digit code that can be used for elevation.

A manager generates this code on their own device, and can then manually copy this code on the user's device when requested.

:shield: Requires `Security:ElevatedFunctionalityProvider` to contain "TemporaryElevationCode".

:point_right: Send this temporary code as elevation header with your original call.

:bulb: Note it's only valid for a limited period of time (as per `Security:TemporaryElevationCode:ExpireInMinutes`) and is a /separate/ code from your login PIN!
     *
     * @export
     * @class GenerateTemporaryElevationCode
     * @implements {EvaService}
     */
    class GenerateTemporaryElevationCode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GenerateTemporaryElevationCode;
        response?: EVA.Core.GenerateTemporaryElevationCodeResponse;
    }
    /**
     * Creates a temporary, numeric password for specified user, overwriting his/her current password, intended for employees to be able to help resetting a user password.
     *
     * @export
     * @class GenerateTemporaryPasswordForUser
     * @implements {EvaService}
     */
    class GenerateTemporaryPasswordForUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GenerateTemporaryPasswordForUser;
        response?: EVA.Core.GenerateTemporaryPasswordForUserResponse;
    }
    /**
     * Returns all active discounts within an organization unit
     *
     * @export
     * @class GetActiveDiscounts
     * @implements {EvaService}
     */
    class GetActiveDiscounts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetActiveDiscounts;
        response?: EVA.Core.GetActiveDiscountsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetActivePaymentTypes
     * @implements {EvaService}
     */
    class GetActivePaymentTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetActivePaymentTypes;
        response?: EVA.Core.GetActivePaymentTypesResponse;
    }
    /**
     * Returns the active shopping cart for the current user.
Works only for webshop OUs.
If there is no active shopping cart, no orderID will be returned.
     *
     * @export
     * @class GetActiveShoppingCart
     * @implements {EvaService}
     */
    class GetActiveShoppingCart implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetActiveShoppingCart;
        response?: EVA.Core.GetActiveShoppingCartResponse;
    }
    /**
     * Service for retrieving a `Address` by specifying a Zip Code and House Number.

This only works for addresses in the Netherlands, for a more generic approach look a the AutocompleteAddress service
     *
     * @export
     * @class GetAddressForZipCode
     * @implements {EvaService}
     */
    class GetAddressForZipCode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAddressForZipCode;
        response?: EVA.Core.GetAddressForZipCodeResponse;
    }
    /**
     * Get all available address requirements.
     *
     * @export
     * @class GetAddressRequirements
     * @implements {EvaService}
     */
    class GetAddressRequirements implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAddressRequirements;
        response?: EVA.Core.GetAddressRequirementsResponse;
    }
    /**
     * Get all User subscriptions for all OUs.
     *
     * @export
     * @class GetAllSubscriptionsByUserID
     * @implements {EvaService}
     */
    class GetAllSubscriptionsByUserID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAllSubscriptionsByUserID;
        response?: EVA.Core.GetUserSubscriptionsResponse;
    }
    /**
     * Retrieve the anonymous token for specified `OrganizationUnitID` and `LanguageID`. The response is your token, or null when not present.
     *
     * @export
     * @class GetAnonymousToken
     * @implements {EvaService}
     */
    class GetAnonymousToken implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAnonymousToken;
        response?: EVA.Core.GetAnonymousTokenResponse;
    }
    /**
     * Get an anonymous wishlist by token.
     *
     * @export
     * @class GetAnonymousWishlist
     * @implements {EvaService}
     */
    class GetAnonymousWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAnonymousWishlist;
        response?: EVA.Core.GetAnonymousWishlistResponse;
    }
    /**
     * Gets the description of the entire API
     *
     * @export
     * @class GetApiDefinition
     * @implements {EvaService}
     */
    class GetApiDefinition implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetApiDefinition;
        response?: EVA.Core.GetApiDefinitionResponse;
    }
    /**
     * Get all app rules
     *
     * @export
     * @class GetAppRules
     * @implements {EvaService}
     */
    class GetAppRules implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAppRules;
        response?: EVA.Core.GetAppRulesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetApplicationConfiguration
     * @implements {EvaService}
     */
    class GetApplicationConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetApplicationConfiguration;
        response?: EVA.Core.GetApplicationConfigurationResponse;
    }
    /**
     * Returns the available assembly names for the device types. Can optionally filter by device types.
     *
     * @export
     * @class GetAssemblyNamesForDeviceTypes
     * @implements {EvaService}
     */
    class GetAssemblyNamesForDeviceTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAssemblyNamesForDeviceTypes;
        response?: EVA.Core.GetAssemblyNamesForDeviceTypesResponse;
    }
    /**
     * Returns a specific async request for your current user
     *
     * @export
     * @class GetAsyncRequestForUser
     * @implements {EvaService}
     */
    class GetAsyncRequestForUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAsyncRequestForUser;
        response?: EVA.Core.GetAsyncRequestForUserResponse;
    }
    /**
     * Returns all async requests for your current user
     *
     * @export
     * @class GetAsyncRequestsForUser
     * @implements {EvaService}
     */
    class GetAsyncRequestsForUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAsyncRequestsForUser;
        response?: EVA.Core.GetAsyncRequestsForUserResponse;
    }
    /**
     * Gets all current audit chain configurations.
     *
     * @export
     * @class GetAuditChainConfigurations
     * @implements {EvaService}
     */
    class GetAuditChainConfigurations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAuditChainConfigurations;
        response?: EVA.Core.GetAuditChainConfigurationsResponse;
    }
    /**
     * Returns the full authorizationstructure of the current user.

It is a list of all the roles the user has, with their functionalities and the OrganizationUnits where these are active.
     *
     * @export
     * @class GetAuthorizationStructure
     * @implements {EvaService}
     */
    class GetAuthorizationStructure implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAuthorizationStructure;
        response?: EVA.Core.GetAuthorizationStructureResponse;
    }
    /**
     * Service for retrieving address suggestion details by specifying a address suggestion retrieved from `AutocompleteAddress` service
     *
     * @export
     * @class GetAutocompleteAddressByReference
     * @implements {EvaService}
     */
    class GetAutocompleteAddressByReference implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAutocompleteAddressByReference;
        response?: EVA.Core.GetAutocompleteAddressByReferenceResponse;
    }
    /**
     * Returns the pickup and/or delivery availability of the given products
     *
     * @export
     * @class GetAvailabilityIndication
     * @implements {EvaService}
     */
    class GetAvailabilityIndication implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailabilityIndication;
        response?: EVA.Core.GetAvailabilityIndicationResponse;
    }
    /**
     * Retrieves the current available cash amount of given device, to be able to e.g. validate user given amounts in `CreateCashDeposit` or `CreateCashExpense` services.

Supply a `CurrencyID` and `PaymentTypeID` to indicate the Cash Journal to be used.
     *
     * @export
     * @class GetAvailableCashAmountForDevice
     * @implements {EvaService}
     */
    class GetAvailableCashAmountForDevice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableCashAmountForDevice;
        response?: EVA.Core.GetAvailableCashAmountForDeviceResponse;
    }
    /**
     * Returns all `CashHandlers` available to use on `OrganizationUnits`
     *
     * @export
     * @class GetAvailableCashHandlers
     * @implements {EvaService}
     */
    class GetAvailableCashHandlers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableCashHandlers;
        response?: EVA.Core.GetAvailableCashHandlersResponse;
    }
    /**
     * Services for retrieving a list of `Currency`
     *
     * @export
     * @class GetAvailableCurrencies
     * @implements {EvaService}
     */
    class GetAvailableCurrencies implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableCurrencies;
        response?: EVA.Core.GetAvailableCurrenciesResponse;
    }
    /**
     * Returns a list of available outputs for an order by taking an `OrderID`.
     *
     * @export
     * @class GetAvailableDocumentsForOrder
     * @implements {EvaService}
     */
    class GetAvailableDocumentsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableDocumentsForOrder;
        response?: EVA.Core.GetAvailableDocumentsForOrderResponse;
    }
    /**
     * Get the AvailableGiftCardConfigurations taking into account the OrganizationUnitID from ClientContext.

`GiftCardConfiguration`: an item that describes the giftcard properties of a product with type GiftCard
     *
     * @export
     * @class GetAvailableGiftCardConfigurations
     * @implements {EvaService}
     */
    class GetAvailableGiftCardConfigurations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableGiftCardConfigurations;
        response?: EVA.Core.GetAvailableGiftCardConfigurationsResponse;
    }
    /**
     * Get the available GiftWrapping options for an Order.
     *
     * @export
     * @class GetAvailableGiftWrappingLinesForOrder
     * @implements {EvaService}
     */
    class GetAvailableGiftWrappingLinesForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableGiftWrappingLinesForOrder;
        response?: EVA.Core.GetAvailableGiftWrappingLinesForOrderResponse;
    }
    /**
     * Retrieve all available payment methods, in general, or for a specific order.

When ReturnUnusablePaymentMethods is set to true, the system will also return unusable payment methods
     *
     * @export
     * @class GetAvailablePaymentMethods
     * @implements {EvaService}
     */
    class GetAvailablePaymentMethods implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailablePaymentMethods;
        response?: EVA.Core.GetAvailablePaymentMethodsResponse;
    }
    /**
     * Get all available options for refunds on the given order

Refunds can be executed with the CreateRefund service
     *
     * @export
     * @class GetAvailableRefundPaymentMethodsForOrder
     * @implements {EvaService}
     */
    class GetAvailableRefundPaymentMethodsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableRefundPaymentMethodsForOrder;
        response?: EVA.Core.GetAvailableRefundPaymentMethodsResponse;
    }
    /**
     * Service for retrieving a list available service details
     *
     * @export
     * @class GetAvailableServiceDetails
     * @implements {EvaService}
     */
    class GetAvailableServiceDetails implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableServiceDetails;
        response?: EVA.Core.GetAvailableServiceDetailsResponse;
    }
    /**
     * Service for retrieving a list of available services
     *
     * @export
     * @class GetAvailableServices
     * @implements {EvaService}
     */
    class GetAvailableServices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableServices;
        response?: EVA.Core.GetAvailableServicesResponse;
    }
    /**
     * Get all available subscriptions
     *
     * @export
     * @class GetAvailableSubscriptions
     * @implements {EvaService}
     */
    class GetAvailableSubscriptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableSubscriptions;
        response?: EVA.Core.GetAvailableSubscriptionsResponse;
    }
    /**
     * Gets the available two-factor authenticators for the current user
     *
     * @export
     * @class GetAvailableTwoFactorAuthenticators
     * @implements {EvaService}
     */
    class GetAvailableTwoFactorAuthenticators implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableTwoFactorAuthenticators;
        response?: EVA.Core.GetAvailableTwoFactorAuthenticatorsResponse;
    }
    /**
     * Get all visibility groups that the current user can see.
     *
     * @export
     * @class GetAvailableVisibilityGroups
     * @implements {EvaService}
     */
    class GetAvailableVisibilityGroups implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetAvailableVisibilityGroups;
        response?: EVA.Core.GetAvailableVisibilityGroupsResponse;
    }
    /**
     * Gets boarding pass information set to given order.
     *
     * @export
     * @class GetBoardingPass
     * @implements {EvaService}
     */
    class GetBoardingPass implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetBoardingPass;
        response?: EVA.Core.GetBoardingPassResponse;
    }
    /**
     * Gets the details of a bundle by its ProductID. The input to this service is `BundleProductID` which refers to the ID
of a `Product` that has `Type` = `BundleProduct`.

The details of a bundle is a structure containing the lines of the bundle, the available options for each line
of the bundle and, if it's a bundle of bundles or a bundle of configurable products, this structure is recursive; each option
can have lines of its own again. So the output of this service contains a tree of all the available products for the bundle.

The `IncludedFields` parameter is optional and allows you to specify which fields you wish to get back from this service.
     *
     * @export
     * @class GetBundleProductDetails
     * @implements {EvaService}
     */
    class GetBundleProductDetails implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetBundleProductDetails;
        response?: EVA.Core.GetBundleProductDetailResponse;
    }
    /**
     * This service returns a list of bundle products that contain the `ProductID` in the request.
The service is meant to be used on a product detail page as a "this product is also available in these bundles" component.
The request allows you to specify what data you'd like to get back from the bundle product, through the `IncludedFields` field.
     *
     * @export
     * @class GetBundleProductsForProduct
     * @implements {EvaService}
     */
    class GetBundleProductsForProduct implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetBundleProductsForProduct;
        response?: EVA.Core.GetBundleProductsForProductResponse;
    }
    /**
     * Gets info about EVA caches, only accessible by New Black.
     *
     * @export
     * @class GetCacheInfo
     * @implements {EvaService}
     */
    class GetCacheInfo implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCacheInfo;
        response?: EVA.Core.GetCacheInfoResponse;
    }
    /**
     * Retrieves details of a cash correction
     *
     * @export
     * @class GetCashCorrection
     * @implements {EvaService}
     */
    class GetCashCorrection implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCashCorrection;
        response?: EVA.Core.GetCashCorrectionResponse;
    }
    /**
     * Get cash correction reason by it's ID
     *
     * @export
     * @class GetCashCorrectionReason
     * @implements {EvaService}
     */
    class GetCashCorrectionReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCashCorrectionReason;
        response?: EVA.Core.GetCashCorrectionReasonResponse;
    }
    /**
     * Get all cash correction reasons
     *
     * @export
     * @class GetCashCorrectionReasons
     * @implements {EvaService}
     */
    class GetCashCorrectionReasons implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCashCorrectionReasons;
        response?: EVA.Core.GetCashCorrectionReasonsResponse;
    }
    /**
     * Retrieves details of a cash deposit
     *
     * @export
     * @class GetCashDeposit
     * @implements {EvaService}
     */
    class GetCashDeposit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCashDeposit;
        response?: EVA.Core.GetCashDepositResponse;
    }
    /**
     * Retrieves details of a cash expense
     *
     * @export
     * @class GetCashExpense
     * @implements {EvaService}
     */
    class GetCashExpense implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCashExpense;
        response?: EVA.Core.GetCashExpenseResponse;
    }
    /**
     * Returns all cash expenses that are available for your current OrganizationUnit, or available to the provided OrganizationUnitID (optional).
     *
     * @export
     * @class GetCashExpenseTypes
     * @implements {EvaService}
     */
    class GetCashExpenseTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCashExpenseTypes;
        response?: EVA.Core.GetCashExpenseTypesResponse;
    }
    /**
     * Gets the currency and a list of available coins and notes for the provided `DeviceID`.
     *
     * @export
     * @class GetCashJournalDenominations
     * @implements {EvaService}
     */
    class GetCashJournalDenominations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCashJournalDenominations;
        response?: EVA.Core.GetCashJournalDenominationsResponse;
    }
    /**
     * Retrieves details of a cash movement
     *
     * @export
     * @class GetCashMovement
     * @implements {EvaService}
     */
    class GetCashMovement implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCashMovement;
        response?: EVA.Core.GetCashMovementResponse;
    }
    /**
     * Get the CheckoutOptions
     *
     * @export
     * @class GetCheckoutOptions
     * @implements {EvaService}
     */
    class GetCheckoutOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCheckoutOptions;
        response?: EVA.Core.GetCheckoutOptionsResponse;
    }
    /**
     * Return the Company attached to the given User
     *
     * @export
     * @class GetCompanyForUser
     * @implements {EvaService}
     */
    class GetCompanyForUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCompanyForUser;
        response?: EVA.Core.GetCompanyForUserResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetConfigurableOrderView
     * @implements {EvaService}
     */
    class GetConfigurableOrderView implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetConfigurableOrderView;
        response?: EVA.Core.GetConfigurableOrderViewResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetConfigurableProductDetail
     * @implements {EvaService}
     */
    class GetConfigurableProductDetail implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetConfigurableProductDetail;
        response?: EVA.Core.GetConfigurableProductDetailResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetConfigurableProductsDetail
     * @implements {EvaService}
     */
    class GetConfigurableProductsDetail implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetConfigurableProductsDetail;
        response?: EVA.Core.GetConfigurableProductsDetailResponse;
    }
    /**
     * Returns a detailed view of a ConsolidatedPickTask by the ID of its UserTask.
     *
     * @export
     * @class GetConsolidatedPickTask
     * @implements {EvaService}
     */
    class GetConsolidatedPickTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetConsolidatedPickTask;
        response?: EVA.Core.GetConsolidatedPickTaskResponse;
    }
    /**
     * Retrieves the details of the consumer company corresponding to the specified ID.
     *
     * @export
     * @class GetConsumerCompany
     * @implements {EvaService}
     */
    class GetConsumerCompany implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetConsumerCompany;
        response?: EVA.Core.GetConsumerCompanyResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetContractNumber
     * @implements {EvaService}
     */
    class GetContractNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetContractNumber;
        response?: EVA.Core.GetContractNumberResponse;
    }
    /**
     * Get the CountrySubdivisions

Optionally the request can be filtered on countryid
     *
     * @export
     * @class GetCountrySubdivisions
     * @implements {EvaService}
     */
    class GetCountrySubdivisions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCountrySubdivisions;
        response?: EVA.Core.GetCountrySubdivisionsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetCurrency
     * @implements {EvaService}
     */
    class GetCurrency implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCurrency;
        response?: EVA.Core.GetCurrencyResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetCurrentApplication
     * @implements {EvaService}
     */
    class GetCurrentApplication implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCurrentApplication;
        response?: EVA.Core.GetCurrentApplicationResponse;
    }
    /**
     * Returns information about the current cash journals for the given DeviceID.
     *
     * @export
     * @class GetCurrentCashJournals
     * @implements {EvaService}
     */
    class GetCurrentCashJournals implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCurrentCashJournals;
        response?: EVA.Core.GetCurrentCashJournalsResponse;
    }
    /**
     * Returns a summary of the current financial period of the organization unit that the user is currently logged on to. The
result of this service can be used to retrieve more details of the financial period via the
`GetFinancialPeriodDetails` service.
     *
     * @export
     * @class GetCurrentFinancialPeriodSummary
     * @implements {EvaService}
     */
    class GetCurrentFinancialPeriodSummary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCurrentFinancialPeriodSummary;
        response?: EVA.Core.GetCurrentFinancialPeriodSummaryResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetCurrentInitialCycleCount
     * @implements {EvaService}
     */
    class GetCurrentInitialCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCurrentInitialCycleCount;
        response?: EVA.Core.GetCurrentInitialCycleCountResponse;
    }
    /**
     * Return the current user for the given Authorization header
     *
     * @export
     * @class GetCurrentUser
     * @implements {EvaService}
     */
    class GetCurrentUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCurrentUser;
        response?: EVA.Core.GetCurrentUserResponse;
    }
    /**
     * Return identification info for the current employee.
     *
     * @export
     * @class GetCurrentUserIdentificationInfo
     * @implements {EvaService}
     */
    class GetCurrentUserIdentificationInfo implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCurrentUserIdentificationInfo;
        response?: EVA.Core.GetCurrentUserIdentificationInfoResponse;
    }
    /**
     * Get the version status of the client compared to the current version of the API. The header EVA-API-Version is required.
     *
     * @export
     * @class GetCurrentVersion
     * @implements {EvaService}
     */
    class GetCurrentVersion implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCurrentVersion;
        response?: EVA.Core.GetCurrentVersionResponse;
    }
    /**
     * Lists the available custom order types that exists which can be used in the order editing process.
     *
     * @export
     * @class GetCustomOrderTypes
     * @implements {EvaService}
     */
    class GetCustomOrderTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCustomOrderTypes;
        response?: EVA.Core.GetCustomOrderTypesResponse;
    }
    /**
     * Retrieve the set values
     *
     * @export
     * @class GetCustomUserTaskData
     * @implements {EvaService}
     */
    class GetCustomUserTaskData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCustomUserTaskData;
        response?: EVA.Core.GetCustomUserTaskDataResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetCustomerOrderVerificationCode
     * @implements {EvaService}
     */
    class GetCustomerOrderVerificationCode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCustomerOrderVerificationCode;
        response?: EVA.Core.GetVerificationCodeResponse;
    }
    /**
     * Return the customer order return details originally set via `CreateCustomerReturn` or `CreateCustomerReturnWithoutOrder
     *
     * @export
     * @class GetCustomerReturn
     * @implements {EvaService}
     */
    class GetCustomerReturn implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCustomerReturn;
        response?: EVA.Core.GetCustomerReturnResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetCycleCountOriginTypes
     * @implements {EvaService}
     */
    class GetCycleCountOriginTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCycleCountOriginTypes;
        response?: EVA.Core.GetCycleCountOriginTypesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetCycleCountSettings
     * @implements {EvaService}
     */
    class GetCycleCountSettings implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCycleCountSettings;
        response?: EVA.Core.GetCycleCountSettingsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetCycleCountZones
     * @implements {EvaService}
     */
    class GetCycleCountZones implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetCycleCountZones;
        response?: EVA.Core.GetCycleCountZonesResponse;
    }
    /**
     * Get additional order information intended for delivery data.
     *
     * @export
     * @class GetDeliveryOrderData
     * @implements {EvaService}
     */
    class GetDeliveryOrderData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDeliveryOrderData;
        response?: EVA.Core.GetDeliveryOrderDataResponse;
    }
    /**
     * Get detailed organization unit opening hours
     *
     * @export
     * @class GetDetailedOrganizationUnitOpeningHours
     * @implements {EvaService}
     */
    class GetDetailedOrganizationUnitOpeningHours implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDetailedOrganizationUnitOpeningHours;
        response?: EVA.Core.GetDetailedOrganizationUnitOpeningHoursResponse;
    }
    /**
     * Service for retrieving a existing `Device`
     *
     * @export
     * @class GetDevice
     * @implements {EvaService}
     */
    class GetDevice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDevice;
        response?: EVA.Core.GetDeviceResponse;
    }
    /**
     * Service for retrieving a list of possible `Device` types
     *
     * @export
     * @class GetDeviceTypes
     * @implements {EvaService}
     */
    class GetDeviceTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDeviceTypes;
        response?: EVA.Core.GetDeviceTypesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetDigitalGiftCardMailOptions
     * @implements {EvaService}
     */
    class GetDigitalGiftCardMailOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDigitalGiftCardMailOptions;
        response?: EVA.Core.GetDigitalGiftCardOptionsResponse;
    }
    /**
     * Gets the GiftCard mail information identified by the `OrderID` and `GiftCardID`.
     *
     * @export
     * @class GetDigitalGiftCardOptions
     * @implements {EvaService}
     */
    class GetDigitalGiftCardOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDigitalGiftCardOptions;
        response?: EVA.Core.GetDigitalGiftCardOptionsResponse;
    }
    /**
     * Retrieve the GiftCard data for a specific order, based on given `OrderID`. This order needs to be visible for the current user.

Will return a list of `GiftCard` data, for every orderline with a GiftCard. Will return the same data supplied in the `AddGiftCardToOrder`
service. When the order was shipped (automatically when it is fully paid), the details `GiftCardNumber` and
`Pin` are also included. An additional property `IsAvailable` let's you check if it was indeed shipped and paid.

The `GiftCardID` returned on the GiftCard is it's unique identifier, to be used in conjunction with the `AccessToken` to make it globally
unique. Using the `AccessToken` and `GiftCardID`, we are pretty sure you are allowed to see the GiftCard data. Use this data to build the link
to share directly, or use the approperiate services to mail/text/print this.

Additionally, this service can be called with an `AccessToken` and `GiftCardID` combination, as explained above. In this case, no user validation
is done - you know both keys to the GiftCard, so you are allowed to get it's information.
     *
     * @export
     * @class GetDigitalGiftCardsForOrder
     * @implements {EvaService}
     */
    class GetDigitalGiftCardsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDigitalGiftCardsForOrder;
        response?: EVA.Core.GetDigitalGiftCardsForOrderResponse;
    }
    /**
     * Get discount product options for order
     *
     * @export
     * @class GetDiscountProductOptionsForOrder
     * @implements {EvaService}
     */
    class GetDiscountProductOptionsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDiscountProductOptionsForOrder;
        response?: EVA.Core.GetDiscountProductOptionsForOrderResponse;
    }
    /**
     * Get discount product options for a specific discount line
     *
     * @export
     * @class GetDiscountProductOptionsForOrderLine
     * @implements {EvaService}
     */
    class GetDiscountProductOptionsForOrderLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDiscountProductOptionsForOrderLine;
        response?: EVA.Core.GetDiscountProductOptionsForOrderLineResponse;
    }
    /**
     * Get a list of all document store categories.

:warning: For debugging and support purposes only.
     *
     * @export
     * @class GetDocumentStoreCategories
     * @implements {EvaService}
     */
    class GetDocumentStoreCategories implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDocumentStoreCategories;
        response?: EVA.Core.GetDocumentStoreCategoriesResponse;
    }
    /**
     * Get a specific document from the document store.

:warning: For debugging and support purposes only.
     *
     * @export
     * @class GetDocumentStoreDocument
     * @implements {EvaService}
     */
    class GetDocumentStoreDocument implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetDocumentStoreDocument;
        response?: EVA.Core.GetDocumentStoreDocumentResponse;
    }
    /**
     * Used to retrieve the e-invoicing data that was set before via `SetElectronicInvoiceData`
     *
     * @export
     * @class GetElectronicInvoiceData
     * @implements {EvaService}
     */
    class GetElectronicInvoiceData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetElectronicInvoiceData;
        response?: EVA.Core.GetElectronicInvoiceDataResponse;
    }
    /**
     * This service returns a list of the most likely email address domains (so 'gmail.com', 'hotmail.com', etc) based on
your current context.
     *
     * @export
     * @class GetEmailAddressDomainSuggestions
     * @implements {EvaService}
     */
    class GetEmailAddressDomainSuggestions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetEmailAddressDomainSuggestions;
        response?: EVA.Core.GetEmailAddressDomainSuggestionsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetEmployeeOrderVerificationCode
     * @implements {EvaService}
     */
    class GetEmployeeOrderVerificationCode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetEmployeeOrderVerificationCode;
        response?: EVA.Core.GetVerificationCodeResponse;
    }
    /**
     * Service for retrieving an existing `EntityTranslation`
     *
     * @export
     * @class GetEntityTranslation
     * @implements {EvaService}
     */
    class GetEntityTranslation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetEntityTranslation;
        response?: EVA.Core.GetEntityTranslationResponse;
    }
    /**
     * A generic service to return all possible values in specified list, e.g. `Currency`, `TaxCode` or `StockLabel`.

The `Name` of the enumeration is unique within EVA, and all of the `Values` returned are also unique and identifiable
by their respective `Name`. Services that consume a value of an enumeration will expect the numeric `Value` returned.

Some of these enumerations are static and are defined by code, meaning the `Values`' combination of `Name` and `Value`
is contant and will never change. It is important to know, though, that other enumerations may be dynamic and can be
altered per environment by their own respective services. When using these, keep in mind they can vary between
environments and new values could be added. It is guaranteed, however, that once a certain `Name`-`Value` pair is set,
it will never change in that particular environment and you are safe treat it as a constant.

:point_right: It is important to realize these dynamics, because at first sight, all of these values may seem contant between
environments - but that is the mere result of an initialization sequence that is usually identical, but definitely
not guaranteed!

:point_right: `IsFlags` will indicate if the values should be treated as bit fields (or, a set of flags). For example,
the enumeration `OrganizationUnitTypes` demonstrates that an organization unit can both be a Shop (`1`) and a Warehouse (`16`)
at the same time, indicated by value `17`. Always use bitwise operands on these fields!
     *
     * @export
     * @class GetEnumValues
     * @implements {EvaService}
     */
    class GetEnumValues implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetEnumValues;
        response?: EVA.Core.GetEnumValuesResponse;
    }
    /**
     * Service for retrieving an existing `EventExportConfiguration`.
     *
     * @export
     * @class GetEventExportConfiguration
     * @implements {EvaService}
     */
    class GetEventExportConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetEventExportConfiguration;
        response?: EVA.Core.GetEventExportConfigurationResponse;
    }
    /**
     * Service for retrieving a list of existing `EventExportConfiguration`'s
     *
     * @export
     * @class GetEventExportConfigurations
     * @implements {EvaService}
     */
    class GetEventExportConfigurations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetEventExportConfigurations;
        response?: EVA.Core.GetEventExportConfigurationsResponse;
    }
    /**
     * Service for retrieving a list of possible event types by `EventExportTarget`. If no target is provided, all targets and their types are returned.
     *
     * @export
     * @class GetEventExportTargetTypes
     * @implements {EvaService}
     */
    class GetEventExportTargetTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetEventExportTargetTypes;
        response?: EVA.Core.GetEventExportTargetTypesResponse;
    }
    /**
     * List available event ledger types.
     *
     * @export
     * @class GetEventLedgerTypes
     * @implements {EvaService}
     */
    class GetEventLedgerTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetEventLedgerTypes;
        response?: EVA.Core.GetEventLedgerTypesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetFinancialPeriodAccountsSummary
     * @implements {EvaService}
     */
    class GetFinancialPeriodAccountsSummary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFinancialPeriodAccountsSummary;
        response?: EVA.Core.GetFinancialPeriodAccountsSummaryResponse;
    }
    /**
     * Get status summary for the devices of a FinancialPeriod
     *
     * @export
     * @class GetFinancialPeriodCashJournalDeviceSummary
     * @implements {EvaService}
     */
    class GetFinancialPeriodCashJournalDeviceSummary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFinancialPeriodCashJournalDeviceSummary;
        response?: EVA.Core.GetFinancialPeriodCashJournalDeviceSummaryResponse;
    }
    /**
     * Gets a list of reasons (if any) why this period cannot be closed yet.

The current possible reasons are (which is available in the `Type` property of each item):

## CLOCKED_IN_USER

The user is still clocked in and must be clocked out.

### Data

The `Data` property will contain:

```json
{
  "UserID" : 1,
  "ClockedIn" : "2018-08-01 12:00",
  "FirstName" : "John",
  "LastName"  : "Doe"
}
```

## ACTIVE_USER_TASK

The user task hasn't been completed yet and it's either attached to this financial period, or its past its deadline.

### Data

The `Data` property will contain:

```json
{
    "UserTaskID" : 5546,
    "UserTaskTypeID" : 9,
    "UserID" : null,
    "UserFullName" : null,
    "IsActive" : true,
    "IsCompleted" : false,
    "Deadline" : null,
    "CreationTime" : "2018-08-01"
}
```

## OPEN_CASH_JOURNAL

There is still an open cash journal that needs to be closed before the period can be closed.

### Data

The `Data` property will contain:

```json
{
"CashJournalID" : 3,
"DeviceID" : 1,
"OpeningAmount" : 150.15
}
```

## PCI_QUESTION

The PCI compliancy question has not been answered yet, this is required to close the period if the OrganizationUnit has been configured to require it.

### Data

No Data.

## NEGATIVE_STOCK

There is negative stock that must be corrected (to at least zero) before the period can be closed.

### Data

The `Data` property will contain:

```json
{
  "ProductID" : 7684,
  "ProductBackendID" : "ABC123",
  "QuantityOnHand" : -10,
  "StockLabelID" : 1,
  "StockLabelName" : "Sellable",
  "OrganizationUnitID" : 182
}
```

This data can then be used to correct the stock through AdjustStock/MoveStock.
     *
     * @export
     * @class GetFinancialPeriodClosingImpediments
     * @implements {EvaService}
     */
    class GetFinancialPeriodClosingImpediments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFinancialPeriodClosingImpediments;
        response?: EVA.Core.GetFinancialPeriodClosingImpedimentsResponse;
    }
    /**
     * Gets information regarding financial period stock
     *
     * @export
     * @class GetFinancialPeriodClosingStockInformation
     * @implements {EvaService}
     */
    class GetFinancialPeriodClosingStockInformation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFinancialPeriodClosingStockInformation;
        response?: EVA.Core.GetFinancialPeriodClosingStockInformationResponse;
    }
    /**
     * Returns a `FinancialPeriodDetailsDto` that contains details about the given financial period (`FinancialPeriodID`).
     *
     * @export
     * @class GetFinancialPeriodDetails
     * @implements {EvaService}
     */
    class GetFinancialPeriodDetails implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFinancialPeriodDetails;
        response?: EVA.Core.GetFinancialPeriodDetailsResponse;
    }
    /**
     * Get financial period export documents
     *
     * @export
     * @class GetFinancialPeriodExportDocuments
     * @implements {EvaService}
     */
    class GetFinancialPeriodExportDocuments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFinancialPeriodExportDocuments;
        response?: EVA.Core.GetFinancialPeriodExportDocumentsResponse;
    }
    /**
     * Gets a detailed overview of the provided `FinancialPeriodID`. If no `FinancialPeriodID` is provided, the current period for your user is used.
     *
     * @export
     * @class GetFinancialPeriodInformation
     * @implements {EvaService}
     */
    class GetFinancialPeriodInformation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFinancialPeriodInformation;
        response?: EVA.Core.GetFinancialPeriodInformationResponse;
    }
    /**
     * Gets a list of reasons (if any) why this period cannot be opened yet.

The current possible reasons are (which is available in the `Type` property of each item):

## LINKED_EMPLOYEES

Employees should be linked to this financial periods (with an optional commission share).

### Data

No Data.

## LOYALTY_PROGRAM

A loyalty program should be linked to this financial period.

### Data

No Data.

## MOBILE_ORGANIZATION_UNIT_LOCATION

This organization unit should get an updated location before starting this financial period.

### Data

No Data.
     *
     * @export
     * @class GetFinancialPeriodOpeningImpediments
     * @implements {EvaService}
     */
    class GetFinancialPeriodOpeningImpediments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFinancialPeriodOpeningImpediments;
        response?: EVA.Core.GetFinancialPeriodOpeningImpedimentsResponse;
    }
    /**
     * Get the fiscal properties of an Order.
     *
     * @export
     * @class GetFiscalOrderData
     * @implements {EvaService}
     */
    class GetFiscalOrderData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFiscalOrderData;
        response?: EVA.Core.GetFiscalOrderDataResponse;
    }
    /**
     * Retrieves an existing Flight
     *
     * @export
     * @class GetFlight
     * @implements {EvaService}
     */
    class GetFlight implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFlight;
        response?: EVA.Core.GetFlightResponse;
    }
    /**
     * Get a Full Stock count task with details.
     *
     * @export
     * @class GetFullStockCount
     * @implements {EvaService}
     */
    class GetFullStockCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFullStockCount;
        response?: EVA.Core.GetFullStockCountResponse;
    }
    /**
     * Returns the information of a single label product. Most notably, this also includes serialnumber information.
     *
     * @export
     * @class GetFullStockCountLabelProduct
     * @implements {EvaService}
     */
    class GetFullStockCountLabelProduct implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetFullStockCountLabelProduct;
        response?: EVA.Core.GetFullStockCountLabelProductResponse;
    }
    /**
     * Fetch the options for a giftcard to determine the value ranges, currency etc.
     *
     * @export
     * @class GetGiftCardOptions
     * @implements {EvaService}
     */
    class GetGiftCardOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetGiftCardOptions;
        response?: EVA.Core.GetGiftCardOptionsResponse;
    }
    /**
     * Get the GiftWrapping options for an Order.
     *
     * @export
     * @class GetGiftWrappingOptionsForOrder
     * @implements {EvaService}
     */
    class GetGiftWrappingOptionsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetGiftWrappingOptionsForOrder;
        response?: EVA.Core.GetGiftWrappingOptionsForOrderResponse;
    }
    /**
     * Returns a list of greeting cards.
     *
     * @export
     * @class GetGreetingCards
     * @implements {EvaService}
     */
    class GetGreetingCards implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetGreetingCards;
        response?: EVA.Core.GetGreetingCardsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetInitialCycleCountLabel
     * @implements {EvaService}
     */
    class GetInitialCycleCountLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInitialCycleCountLabel;
        response?: EVA.Core.GetInitialCycleCountLabelResponse;
    }
    /**
     * Gets the initialization state for the EVA environment.
Can either way be Created or Initialized.
If Created, the environment is not configured and can not be used.
If Initialized, the environment is configured and ready to be used.
     *
     * @export
     * @class GetInitializationState
     * @implements {EvaService}
     */
    class GetInitializationState implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInitializationState;
        response?: EVA.Core.InitializationStateResponse;
    }
    /**
     * Returns inquiry details
     *
     * @export
     * @class GetInquiry
     * @implements {EvaService}
     */
    class GetInquiry implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInquiry;
        response?: EVA.Core.GetInquiryResponse;
    }
    /**
     * Returns all the details of an invoice by its ID.
     *
     * @export
     * @class GetInvoice
     * @implements {EvaService}
     */
    class GetInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoice;
        response?: EVA.Core.GetInvoiceResponse;
    }
    /**
     * Get a InvoiceAdditionalAmountType by ID
     *
     * @export
     * @class GetInvoiceAdditionalAmountTypeByID
     * @implements {EvaService}
     */
    class GetInvoiceAdditionalAmountTypeByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoiceAdditionalAmountTypeByID;
        response?: EVA.Core.GetInvoiceAdditionalAmountTypeByIDResponse;
    }
    /**
     * Get all InvoiceAdditionalAmountTypes
     *
     * @export
     * @class GetInvoiceAdditionalAmountTypes
     * @implements {EvaService}
     */
    class GetInvoiceAdditionalAmountTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoiceAdditionalAmountTypes;
        response?: EVA.Core.GetInvoiceAdditionalAmountTypesResponse;
    }
    /**
     * Get all AdditionalAmounts for an Invoice
     *
     * @export
     * @class GetInvoiceAdditionalAmountsForInvoice
     * @implements {EvaService}
     */
    class GetInvoiceAdditionalAmountsForInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoiceAdditionalAmountsForInvoice;
        response?: EVA.Core.GetInvoiceAdditionalAmountsForInvoiceResponse;
    }
    /**
     * Get a InvoiceDisputeReason by ID
     *
     * @export
     * @class GetInvoiceDisputeReasonByID
     * @implements {EvaService}
     */
    class GetInvoiceDisputeReasonByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoiceDisputeReasonByID;
        response?: EVA.Core.GetInvoiceDisputeReasonByIDResponse;
    }
    /**
     * Get all InvoiceDisputeReasons
     *
     * @export
     * @class GetInvoiceDisputeReasons
     * @implements {EvaService}
     */
    class GetInvoiceDisputeReasons implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoiceDisputeReasons;
        response?: EVA.Core.GetInvoiceDisputeReasonsResponse;
    }
    /**
     * Get all Disputes for an Invoice
     *
     * @export
     * @class GetInvoiceDisputesForInvoice
     * @implements {EvaService}
     */
    class GetInvoiceDisputesForInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoiceDisputesForInvoice;
        response?: EVA.Core.GetInvoiceDisputesForInvoiceResponse;
    }
    /**
     * Get the details of an InvoiceExport
     *
     * @export
     * @class GetInvoiceExport
     * @implements {EvaService}
     */
    class GetInvoiceExport implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoiceExport;
        response?: EVA.Core.GetInvoiceExportResponse;
    }
    /**
     * Get the documents for an export when available
     *
     * @export
     * @class GetInvoiceExportDocuments
     * @implements {EvaService}
     */
    class GetInvoiceExportDocuments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoiceExportDocuments;
        response?: EVA.Core.GetInvoiceExportDocumentsResponse;
    }
    /**
     * Service for retrieving a list of available `InvoiceOutputType`'s providing a `Invoice`
     *
     * @export
     * @class GetInvoiceOutputTypes
     * @implements {EvaService}
     */
    class GetInvoiceOutputTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoiceOutputTypes;
        response?: EVA.Core.GetInvoiceOutputTypesResponse;
    }
    /**
     * Retrieves payments for an order, with context about allocated invoices, filtered on specified optional `InvoiceID`.
     *
     * @export
     * @class GetInvoicePayments
     * @implements {EvaService}
     */
    class GetInvoicePayments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetInvoicePayments;
        response?: EVA.Core.GetInvoicePaymentsResponse;
    }
    /**
     * Returns all organization units where this user can login
     *
     * @export
     * @class GetLoginOrganizationUnitsForUser
     * @implements {EvaService}
     */
    class GetLoginOrganizationUnitsForUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetLoginOrganizationUnitsForUser;
        response?: EVA.Core.GetLoginOrganizationUnitsForUserResponse;
    }
    /**
     * Retrieves the lottery number for a given order
     *
     * @export
     * @class GetLotteryNumber
     * @implements {EvaService}
     */
    class GetLotteryNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetLotteryNumber;
        response?: EVA.Core.GetLotteryNumberResponse;
    }
    /**
     * Gets the possible valid reasons that have been defined for opening the cash drawer manually.
     *
     * @export
     * @class GetOpenCashDrawerReasons
     * @implements {EvaService}
     */
    class GetOpenCashDrawerReasons implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOpenCashDrawerReasons;
        response?: EVA.Core.GetOpenCashDrawerReasonsResponse;
    }
    /**
     * Get OpeningHoursTemplate by ID
     *
     * @export
     * @class GetOpeningHoursTemplateByID
     * @implements {EvaService}
     */
    class GetOpeningHoursTemplateByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOpeningHoursTemplateByID;
        response?: EVA.Core.GetOpeningHoursTemplateResponseByID;
    }
    /**
     * Get a list of all OpeningHoursTemplates translation keys for description translation.
     *
     * @export
     * @class GetOpeningHoursTemplatesTranslationKeys
     * @implements {EvaService}
     */
    class GetOpeningHoursTemplatesTranslationKeys implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOpeningHoursTemplatesTranslationKeys;
        response?: EVA.Core.GetOpeningHoursTemplatesTranslationKeysResponse;
    }
    /**
     * Get an Order by ID.

This service is readonly, it will not update the order just by calling it like the `GetShoppingCart` service does.

To include the result of `GetRequiredDataForOrder`, the property `AdditionalOrderDataOptions.RequiredFor` can be set to the desired level of requirements.
To include the result of `GetProductRequirementValuesForOrder`, the property `AdditionalOrderDataOptions.IncludeProductRequirements` can be set to `true`.
To include the result of `GetGiftWrappingOptionsForOrder` and `GetAvailableGiftWrappingLinesForOrder`, the property `AdditionalOrderDataOptions.IncludeGiftWrapping` can be set to `true`.
To include the result of `PrefigureDiscounts` and `GetPotentialDiscountsForOrder`, the property `AdditionalOrderDataOptions.IncludePrefigureDiscounts` can be set to `true`.
To include the result of `GetPickProductDiscountOptionsForOrder`, the property `AdditionalOrderDataOptions.IncludePickProductOptions` can be set to `true`.
To include the result of `ListAvailableShippingMethods`, the property `AdditionalOrderDataOptions.IncludeAvailableShippingMethods` can be set to `true`.
To include the result of `GetAvailablePaymentMethods`, the property `AdditionalOrderDataOptions.IncludeAvailablePaymentMethods` can be set to `true`.
To include the result of `GetAvailableRefundPaymentMethodsForOrder`, the property `AdditionalOrderDataOptions.IncludeAvailableRefundPaymentMethods` can be set to `true`.
To include the result of `ValidateOrderShipment`, the property `AdditionalOrderDataOptions.IncludeValidateShipment` can be set to `true`.
To include the available actions for the PaymentTransactions, the property `AdditionalOrderDataOptions.IncludePaymentTransactionActions` can be set to `true`.
     *
     * @export
     * @class GetOrder
     * @implements {EvaService}
     */
    class GetOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrder;
        response?: EVA.Core.GetOrderResponse;
    }
    /**
     * Returns a list of custom status changes for a specific order
     *
     * @export
     * @class GetOrderCustomStatusLedger
     * @implements {EvaService}
     */
    class GetOrderCustomStatusLedger implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderCustomStatusLedger;
        response?: EVA.Core.GetOrderCustomStatusLedgerResponse;
    }
    /**
     * This services returns info about the discounts on the given order
     *
     * @export
     * @class GetOrderDiscountInfo
     * @implements {EvaService}
     */
    class GetOrderDiscountInfo implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderDiscountInfo;
        response?: EVA.Core.GetOrderDiscountInfoResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrderExportByID
     * @implements {EvaService}
     */
    class GetOrderExportByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderExportByID;
        response?: EVA.Core.GetOrderExportByIDResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrderExportDocuments
     * @implements {EvaService}
     */
    class GetOrderExportDocuments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderExportDocuments;
        response?: EVA.Core.GetOrderExportDocumentsResponse;
    }
    /**
     * Gets the details of an OrderFulfillment, except for its Lines which can be paged in through ListOrderFulfillmentLines with a filter on OrderFulfillmentID.
     *
     * @export
     * @class GetOrderFulfillment
     * @implements {EvaService}
     */
    class GetOrderFulfillment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderFulfillment;
        response?: EVA.Core.GetOrderFulfillmentResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrderFulfillmentOptions
     * @implements {EvaService}
     */
    class GetOrderFulfillmentOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderFulfillmentOptions;
        response?: EVA.Core.GetOrderFulfillmentOptionsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrderFulfillmentSupplierPerformance
     * @implements {EvaService}
     */
    class GetOrderFulfillmentSupplierPerformance implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderFulfillmentSupplierPerformance;
        response?: EVA.Core.GetOrderFulfillmentSupplierPerformanceResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrderLinkTypeByID
     * @implements {EvaService}
     */
    class GetOrderLinkTypeByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderLinkTypeByID;
        response?: EVA.Core.GetOrderLinkTypeByIDResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrderLinkTypes
     * @implements {EvaService}
     */
    class GetOrderLinkTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderLinkTypes;
        response?: EVA.Core.GetOrderLinkTypesResponse;
    }
    /**
     * Return the current order links for the given order
     *
     * @export
     * @class GetOrderLinks
     * @implements {EvaService}
     */
    class GetOrderLinks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderLinks;
        response?: EVA.Core.GetOrderLinksResponse;
    }
    /**
     * Get the signature for an Order
     *
     * @export
     * @class GetOrderSignature
     * @implements {EvaService}
     */
    class GetOrderSignature implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderSignature;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrderStatistics
     * @implements {EvaService}
     */
    class GetOrderStatistics implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderStatistics;
        response?: EVA.Core.GetOrderStatisticsResponse;
    }
    /**
     * Returns a summarized version of the order with less details than `GetOrder`. This service should be used to expose
less data to the frontend, also this variant to retrieve the order can be used when authenticated via the `GetTemporaryTokenForOrder`
service. Where `GetOrder` is not accessible.
     *
     * @export
     * @class GetOrderSummary
     * @implements {EvaService}
     */
    class GetOrderSummary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderSummary;
        response?: EVA.Core.GetOrderSummaryResponse;
    }
    /**
     * Get a summary of an Order to be used when a shipment will be made.

Caution: the ProductID is nullable, all non-cancelled lines are returned
     *
     * @export
     * @class GetOrderSummaryForShipping
     * @implements {EvaService}
     */
    class GetOrderSummaryForShipping implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderSummaryForShipping;
        response?: EVA.Core.GetOrderSummaryForShippingResponse;
    }
    /**
     * Get tax details on the specified order.
     *
     * @export
     * @class GetOrderTaxDetails
     * @implements {EvaService}
     */
    class GetOrderTaxDetails implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderTaxDetails;
        response?: EVA.Core.GetOrderTaxDetailsResponse;
    }
    /**
     * Get order verification reason.
     *
     * @export
     * @class GetOrderVerificationReason
     * @implements {EvaService}
     */
    class GetOrderVerificationReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrderVerificationReason;
        response?: EVA.Core.GetOrderVerificationReasonResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrdersForReturnOrder
     * @implements {EvaService}
     */
    class GetOrdersForReturnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrdersForReturnOrder;
        response?: EVA.Core.GetOrdersForReturnOrderResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrganizationUnit
     * @implements {EvaService}
     */
    class GetOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnit;
        response?: EVA.Core.GetOrganizationUnitResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrganizationUnitCreditBalance
     * @implements {EvaService}
     */
    class GetOrganizationUnitCreditBalance implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnitCreditBalance;
        response?: EVA.Core.GetOrganizationUnitCreditBalanceResponse;
    }
    /**
     * Retrieve detailed organization unit information.
     *
     * @export
     * @class GetOrganizationUnitDetailed
     * @implements {EvaService}
     */
    class GetOrganizationUnitDetailed implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnitDetailed;
        response?: EVA.Core.GetOrganizationUnitDetailedResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrganizationUnitNotes
     * @implements {EvaService}
     */
    class GetOrganizationUnitNotes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnitNotes;
        response?: EVA.Core.GetOrganizationUnitNotesResponse;
    }
    /**
     * Get flattened organization unit opening hours
     *
     * @export
     * @class GetOrganizationUnitOpeningHours
     * @implements {EvaService}
     */
    class GetOrganizationUnitOpeningHours implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnitOpeningHours;
        response?: EVA.Core.GetOrganizationUnitOpeningHoursResponse;
    }
    /**
     * Gets the organization unit visiting opening hours for the given period.
     *
     * @export
     * @class GetOrganizationUnitOpeningHoursForPeriod
     * @implements {EvaService}
     */
    class GetOrganizationUnitOpeningHoursForPeriod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnitOpeningHoursForPeriod;
        response?: EVA.Core.GetOrganizationUnitOpeningHoursForPeriodResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrganizationUnitSettings
     * @implements {EvaService}
     */
    class GetOrganizationUnitSettings implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnitSettings;
        response?: EVA.Core.GetOrganizationUnitSettingsResponse;
    }
    /**
     * Returns a summary of a specific organizationunit. Since this service is publicly available, this will return minimal information (aka a summary) about the OU.
Service can also be called for archived organization units to fetch minimal information.
     *
     * @export
     * @class GetOrganizationUnitSummary
     * @implements {EvaService}
     */
    class GetOrganizationUnitSummary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnitSummary;
        response?: EVA.Core.GetOrganizationUnitSummaryResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetOrganizationUnitsForUser
     * @implements {EvaService}
     */
    class GetOrganizationUnitsForUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnitsForUser;
        response?: EVA.Core.GetOrganizationUnitsForUserResponse;
    }
    /**
     * Returns the full hierarchy of organization units, starting from the root and then containing a nested list of child OU's.
     *
     * @export
     * @class GetOrganizationUnitsHierarchy
     * @implements {EvaService}
     */
    class GetOrganizationUnitsHierarchy implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetOrganizationUnitsHierarchy;
        response?: EVA.Core.GetOrganizationUnitsHierarchyResponse;
    }
    /**
     * As a logged in user, you can create a passkey as additional login method, which is a totally awesome, passwordless, usernameless method to log in!

Before you can create a new passkey, use this `GetPasskeyChallenge` service to obtain a challenge and some information from the server, push that through WebAuthn using
`navigator.credentials.create` to create the credential, which you use to call `CreatePasskey`.

:point_right: See `CreatePasskey` for more details!
     *
     * @export
     * @class GetPasskeyChallenge
     * @implements {EvaService}
     */
    class GetPasskeyChallenge implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPasskeyChallenge;
        response?: EVA.Core.GetPasskeyChallengeResponse;
    }
    /**
     * Retrieves some basic information about the current user's passkeys.
     *
     * @export
     * @class GetPasskeyInfo
     * @implements {EvaService}
     */
    class GetPasskeyInfo implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPasskeyInfo;
        response?: EVA.Core.GetPasskeyInfoResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetPaymentTransaction
     * @implements {EvaService}
     */
    class GetPaymentTransaction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPaymentTransaction;
        response?: EVA.Core.GetPaymentTransactionResponse;
    }
    /**
     * Get the PaymentTransactions attached to the given Invoice

The response also includes an object `AvailableActions`:
- Cancel: can the payment be cancelled
- Refund: can the payment be refunded
- Update: can an update for this payment be requested
- Approve: can the payment be approved
     *
     * @export
     * @class GetPaymentTransactionsForInvoice
     * @implements {EvaService}
     */
    class GetPaymentTransactionsForInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPaymentTransactionsForInvoice;
        response?: EVA.Core.GetPaymentTransactionsResponse;
    }
    /**
     * Get the PaymentTransactions attached to the given Order

The response also includes an object `AvailableActions`:
- Cancel: can the payment be cancelled
- Refund: can the payment be refunded
- Update: can an update for this payment be requested
- Approve: can the payment be approved
     *
     * @export
     * @class GetPaymentTransactionsForOrder
     * @implements {EvaService}
     */
    class GetPaymentTransactionsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPaymentTransactionsForOrder;
        response?: EVA.Core.GetPaymentTransactionsResponse;
    }
    /**
     * Returns the quantity that is to be shipped for order lines. Includes pending shipments.
     *
     * @export
     * @class GetPendingShipmentInformationForOrderLines
     * @implements {EvaService}
     */
    class GetPendingShipmentInformationForOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPendingShipmentInformationForOrderLines;
        response?: EVA.Core.GetPendingShipmentInformationForOrderLinesResponse;
    }
    /**
     * Get the PickProductDiscountOptions for an Order

For each line there is an object with the options:
- Options: the available products to choose from
- CurrentSelection: the currently selected product
- CurrentTier: the current tier of the discount
- BaseAmount: the amount used to determine the tiers
- Tiers:
 - ID
 - Description
 - From: The amount at which this tier is available
 - Difference: The amount that should be added to be eligable for this tier
 - DefaultProductID: The product that should be selected by default when hitting this tier
 - Prompt: Should the frontend show a prompt to pick a product?
 - ProductIDs: The avaialble products in this tier
 - IsCurrentTier: Is this tier the current one?
     *
     * @export
     * @class GetPickProductDiscountOptionsForOrder
     * @implements {EvaService}
     */
    class GetPickProductDiscountOptionsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPickProductDiscountOptionsForOrder;
        response?: EVA.Core.GetPickProductDiscountOptionsForOrderResponse;
    }
    /**
     * Get the PickProductDiscountOptions for an OrderLine

See: `GetPickProductDiscountOptionsForOrder`
     *
     * @export
     * @class GetPickProductDiscountOptionsForOrderLine
     * @implements {EvaService}
     */
    class GetPickProductDiscountOptionsForOrderLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPickProductDiscountOptionsForOrderLine;
        response?: EVA.Core.GetPickProductDiscountOptionsForOrderLineResponse;
    }
    /**
     * This service returns the bundles for which you currently have at least one product in the provided order.
This can then be used to suggest bundles to the end-user that they're missing out on.

The IncludedFields field controls which product content you get back for each bundle product. By default, only the product's display value is returned.

Specifying the ProductIDs field means that only suggestions for the given products are returned.
If the Order does not contain the provided ProductIDs, nothing will be returned.
     *
     * @export
     * @class GetPotentialBundleProductsForOrder
     * @implements {EvaService}
     */
    class GetPotentialBundleProductsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPotentialBundleProductsForOrder;
        response?: EVA.Core.GetPotentialBundleProductsForOrderResponse;
    }
    /**
     * List the discounts that potentially could be activated.
When an `Order` is given, identified by `OrderID`, the discounts will be based on the ordercontext. That will include possibly User specific discounts etc.
If a `ProductID` is set, also discounts that will be triggered when that Product is added to the current Order will be returned.

It is required to supply an Order a Product or both.
     *
     * @export
     * @class GetPotentialDiscountsForOrder
     * @implements {EvaService}
     */
    class GetPotentialDiscountsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPotentialDiscountsForOrder;
        response?: EVA.Core.GetPotentialDiscountsResponse;
    }
    /**
     * List the discounts that potentially could be activated.
This service has the same behavior as the `GetPotentialDiscountsForOrder`, only in this service the discounts are not calculated based on one `ProductID`, but on the result of a `ProductSearch` request.
This creates the possibility to fire a `ProductSearch` and a `GetPotentialDiscountsForProductSearch` at the same time.
     *
     * @export
     * @class GetPotentialDiscountsForProductSearch
     * @implements {EvaService}
     */
    class GetPotentialDiscountsForProductSearch implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPotentialDiscountsForProductSearch;
        response?: EVA.Core.GetPotentialDiscountsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetPotentialProductPricing
     * @implements {EvaService}
     */
    class GetPotentialProductPricing implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPotentialProductPricing;
        response?: EVA.Core.GetPotentialProductPricingResponse;
    }
    /**
     * Retrieves a pre-picked ShipFromStoreTask based on a product ID.
Optionally filters by a ConsolidatedPickTask ID or defaults to the latest done, stopped, or paused task.
     *
     * @export
     * @class GetPrePickedShipFromStoreTask
     * @implements {EvaService}
     */
    class GetPrePickedShipFromStoreTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPrePickedShipFromStoreTask;
        response?: EVA.Core.GetPrePickedShipFromStoreTaskResponse;
    }
    /**
     * Get the selected preferred refund method to use when this order will be refunded.

See SetPreferredRefundMethod for more information.
     *
     * @export
     * @class GetPreferredRefundMethod
     * @implements {EvaService}
     */
    class GetPreferredRefundMethod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPreferredRefundMethod;
        response?: EVA.Core.GetPreferredRefundMethodResponse;
    }
    /**
     * This service returns information about product availability.
     *
     * @export
     * @class GetProductAvailability
     * @implements {EvaService}
     */
    class GetProductAvailability implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductAvailability;
        response?: EVA.Core.GetProductAvailabilityResponse;
    }
    /**
     * This service is used to fetch a `ProductBundle` by its ID. The requested bundle will be returned with its lines and for each line, its options.
     *
     * @export
     * @class GetProductBundle
     * @implements {EvaService}
     */
    class GetProductBundle implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductBundle;
        response?: EVA.Core.GetProductBundleResponse;
    }
    /**
     * Get product details by id
     *
     * @export
     * @class GetProductDetail
     * @implements {EvaService}
     */
    class GetProductDetail implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductDetail;
        response?: EVA.Core.GetProductDetailResponse;
    }
    /**
     * This service is used to list prices associated with the products determined by IDs.

Price - this value has TaxRate applied
PriceInTax - value without TaxRate

A product has more associated prices with different usage types, e.g. Cost, Sales..

Request e.g.:
{
  "ProductIDs": [
    "productId1",
    "productId2"
  ]
}
     *
     * @export
     * @class GetProductPrices
     * @implements {EvaService}
     */
    class GetProductPrices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductPrices;
        response?: EVA.Core.GetProductPricesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetProductPricing
     * @implements {EvaService}
     */
    class GetProductPricing implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductPricing;
        response?: EVA.Core.GetProductPricingResponse;
    }
    /**
     * Get product relations by product id
     *
     * @export
     * @class GetProductRelations
     * @implements {EvaService}
     */
    class GetProductRelations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductRelations;
        response?: EVA.Core.GetProductRelationsResponse;
    }
    /**
     * List all ProductRequirement values for an Order
     *
     * @export
     * @class GetProductRequirementValuesForOrder
     * @implements {EvaService}
     */
    class GetProductRequirementValuesForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductRequirementValuesForOrder;
        response?: EVA.Core.GetProductRequirementValuesForOrderResponse;
    }
    /**
     * List all ProductRequirement values for an OrderLine
     *
     * @export
     * @class GetProductRequirementValuesForOrderLine
     * @implements {EvaService}
     */
    class GetProductRequirementValuesForOrderLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductRequirementValuesForOrderLine;
        response?: EVA.Core.GetProductRequirementValuesForOrderLineResponse;
    }
    /**
     * Calculates the run-rates for the provided products for the provided warehouse OrganizationUnit, based on a date (`Since`).

The run-rates are returned per SoldFromOrganizationUnit.
     *
     * @export
     * @class GetProductRunRates
     * @implements {EvaService}
     */
    class GetProductRunRates implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductRunRates;
        response?: EVA.Core.GetProductRunRatesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetProductSearchSuggestions
     * @implements {EvaService}
     */
    class GetProductSearchSuggestions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductSearchSuggestions;
        response?: EVA.Core.GetProductSearchSuggestionsResponse;
    }
    /**
     * Retrieve the structure of products by `Code` (and for given `LanguageID`/`CountryID`).
     *
     * @export
     * @class GetProductStructure
     * @implements {EvaService}
     */
    class GetProductStructure implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductStructure;
        response?: EVA.Core.GetProductStructureResponse;
    }
    /**
     * Get the supplier product info for a set of product IDs (not supplier product IDs) and the optional supplier OU.
     *
     * @export
     * @class GetProductSupplierInfoForProducts
     * @implements {EvaService}
     */
    class GetProductSupplierInfoForProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductSupplierInfoForProducts;
        response?: EVA.Core.GetProductSupplierInfoForProductsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetProductUnitOfMeasureQuantities
     * @implements {EvaService}
     */
    class GetProductUnitOfMeasureQuantities implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductUnitOfMeasureQuantities;
        response?: EVA.Core.GetProductUnitOfMeasureQuantitiesResponse;
    }
    /**
     * Get product warranty information by making a request to an endpoint set with the 'Product:WarrantyEndpoint' setting.

If 'SerialNumber' is not set in the request, we'll try to get it using the 'OrderLineID', so at least one or the other must be present and valid.
     *
     * @export
     * @class GetProductWarranty
     * @implements {EvaService}
     */
    class GetProductWarranty implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductWarranty;
        response?: EVA.Core.GetProductWarrantyResponse;
    }
    /**
     * Retrieves a ProductZone by its ID
     *
     * @export
     * @class GetProductZone
     * @implements {EvaService}
     */
    class GetProductZone implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductZone;
        response?: EVA.Core.ProductZoneDto;
    }
    /**
     * Retrieves a ProductZoneGroup by its ID
     *
     * @export
     * @class GetProductZoneGroup
     * @implements {EvaService}
     */
    class GetProductZoneGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProductZoneGroup;
        response?: EVA.Core.ProductZoneGroupDto;
    }
    /**
     * Fetch one or more products by ID.
     *
     * @export
     * @class GetProducts
     * @implements {EvaService}
     */
    class GetProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetProducts;
        response?: EVA.Core.GetProductsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetPurchaseOrderShipment
     * @implements {EvaService}
     */
    class GetPurchaseOrderShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetPurchaseOrderShipment;
        response?: EVA.Core.GetPurchaseOrderShipmentResponse;
    }
    /**
     * Calculate and return the current `QuantityOnHand` for supplied `ProductID`s on given organization unit.

Returns a dictionary with `ProductID` as key, and the `QuantityOnHand` as value.
     *
     * @export
     * @class GetQuantityOnHandForProducts
     * @implements {EvaService}
     */
    class GetQuantityOnHandForProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetQuantityOnHandForProducts;
        response?: EVA.Core.GetQuantityOnHandForProductsResponse;
    }
    /**
     * Service for retrieving an existing `QuickBuy` by ID
     *
     * @export
     * @class GetQuickBuy
     * @implements {EvaService}
     */
    class GetQuickBuy implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetQuickBuy;
        response?: EVA.Core.GetQuickBuyResponse;
    }
    /**
     * Service for retrieving an list of `QuickBuy` options
     *
     * @export
     * @class GetQuickBuyOptions
     * @implements {EvaService}
     */
    class GetQuickBuyOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetQuickBuyOptions;
        response?: EVA.Core.GetQuickBuyOptionsResponse;
    }
    /**
     * Service for retrieving an list of `QuickBuy`
     *
     * @export
     * @class GetQuickBuys
     * @implements {EvaService}
     */
    class GetQuickBuys implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetQuickBuys;
        response?: EVA.Core.GetQuickBuysResponse;
    }
    /**
     * Get the logging of the run of a recurring task.
     *
     * @export
     * @class GetRecurringTaskLogging
     * @implements {EvaService}
     */
    class GetRecurringTaskLogging implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetRecurringTaskLogging;
        response?: EVA.Core.GetRecurringTaskLoggingResponse;
    }
    /**
     * Get the list of runs for a recurring task.
     *
     * @export
     * @class GetRecurringTaskRuns
     * @implements {EvaService}
     */
    class GetRecurringTaskRuns implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetRecurringTaskRuns;
        response?: EVA.Core.GetRecurringTaskRunsResponse;
    }
    /**
     * Get all refund reasons available in the current context
     *
     * @export
     * @class GetRefundCorrectionReasons
     * @implements {EvaService}
     */
    class GetRefundCorrectionReasons implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetRefundCorrectionReasons;
        response?: EVA.Core.GetRefundCorrectionReasonsResponse;
    }
    /**
     * List all the related orderlines for the given OrderLineID.

The orderlines will be sorted by CreationTime.
     *
     * @export
     * @class GetRelatedOrderLines
     * @implements {EvaService}
     */
    class GetRelatedOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetRelatedOrderLines;
        response?: EVA.Core.GetRelatedOrderLinesResponse;
    }
    /**
     * Get all the related orders for the given OrderID.

The orders will be sorted by CreationTime.
     *
     * @export
     * @class GetRelatedOrders
     * @implements {EvaService}
     */
    class GetRelatedOrders implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetRelatedOrders;
        response?: EVA.Core.GetRelatedOrdersResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetRepair
     * @implements {EvaService}
     */
    class GetRepair implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetRepair;
        response?: EVA.Core.GetRepairResponse;
    }
    /**
     * Get the required data for the order
     *
     * @export
     * @class GetRequiredDataForOrder
     * @implements {EvaService}
     */
    class GetRequiredDataForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetRequiredDataForOrder;
        response?: EVA.Core.GetRequiredDataForOrderResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetRequiredOrganizationUnitFields
     * @implements {EvaService}
     */
    class GetRequiredOrganizationUnitFields implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetRequiredOrganizationUnitFields;
        response?: EVA.Core.GetRequiredOrganizationUnitFieldsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetReturnOrdersForOrder
     * @implements {EvaService}
     */
    class GetReturnOrdersForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetReturnOrdersForOrder;
        response?: EVA.Core.GetReturnOrdersForOrderResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetReturnToSupplierData
     * @implements {EvaService}
     */
    class GetReturnToSupplierData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetReturnToSupplierData;
        response?: EVA.Core.GetReturnToSupplierDataResponse;
    }
    /**
     * Get a ReturnToSupplierRequest
     *
     * @export
     * @class GetReturnToSupplierRequestByID
     * @implements {EvaService}
     */
    class GetReturnToSupplierRequestByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetReturnToSupplierRequestByID;
        response?: EVA.Core.GetReturnToSupplierRequestByIDResponse;
    }
    /**
     * Gets whether or not an Order is currently returnable.

For each OrderLine on the Order you'll get back a result that specifies whether or not it can be returned,
how much can be returned and which (if any) other OrderLines must be returned before the line itself can be returned
(think of parent-child lines where you can't return a child OrderLine without also returning the parent OrderLine).

In addition, this service may return `ReturnSets` which specify that 'if you return these OrderLines (specified by `OrderLineIDs`) then
you must *also* return these lines (specified by `RequiredOrderLineIDsToReturn`). For example, if you return all normal OrderLines on an Order
then you must also return all the free products you have received.

Finally, specify the `ReturningQuantities` parameter as an object where the key is an `OrderLineID` with a (positive) `Quantity` you wish to be
returned. The `Result` can be affected in case rules have been configured to, for example, allow _returns without products_ only when the total
return order amount is below a certain amount.
     *
     * @export
     * @class GetReturnableStatusForOrder
     * @implements {EvaService}
     */
    class GetReturnableStatusForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetReturnableStatusForOrder;
        response?: EVA.Core.GetReturnableStatusForOrderResponse;
    }
    /**
     * Will return the estimated tax for specified order, but does not persist through the order amounts retrieved elsewhere.
     *
     * @export
     * @class GetSalesTaxEstimateForOrder
     * @implements {EvaService}
     */
    class GetSalesTaxEstimateForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetSalesTaxEstimateForOrder;
        response?: EVA.Core.SalesTaxEstimateForOrderResponse;
    }
    /**
     * Gets a script by ID
     *
     * @export
     * @class GetScript
     * @implements {EvaService}
     */
    class GetScript implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetScript;
        response?: EVA.Core.GetScriptResponse;
    }
    /**
     * Returns all available dialects
     *
     * @export
     * @class GetScriptDialects
     * @implements {EvaService}
     */
    class GetScriptDialects implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetScriptDialects;
        response?: EVA.Core.GetScriptDialectsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetScriptExtensionPoints
     * @implements {EvaService}
     */
    class GetScriptExtensionPoints implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetScriptExtensionPoints;
        response?: EVA.Core.GetScriptExtensionPointsResponse;
    }
    /**
     * Gets information about a serial number
     *
     * @export
     * @class GetSerialNumber
     * @implements {EvaService}
     */
    class GetSerialNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetSerialNumber;
        response?: EVA.Core.GetSerialNumberResponse;
    }
    /**
     * Get all Serial Number Configurations, without paging.
     *
     * @export
     * @class GetSerialNumberConfigurations
     * @implements {EvaService}
     */
    class GetSerialNumberConfigurations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetSerialNumberConfigurations;
        response?: EVA.Core.GetSerialNumberConfigurationsResponse;
    }
    /**
     * Shows in which OU sets an OU is included.
     *
     * @export
     * @class GetSetsByOrganizationUnit
     * @implements {EvaService}
     */
    class GetSetsByOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetSetsByOrganizationUnit;
        response?: EVA.Core.GetSetsByOrganizationUnitResponse;
    }
    /**
     * Get the shared wishlist by token. Requires the share token.
     *
     * @export
     * @class GetSharedWishlist
     * @implements {EvaService}
     */
    class GetSharedWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetSharedWishlist;
        response?: EVA.Core.GetSharedWishlistResponse;
    }
    /**
     * Gets an active ShipFromStore type User Task for your store, that contains a product with specified `Barcode`.

Use the optional `DeterminationType` to indicate if you prefer the `Oldest`, `Smallest` or `Largest` task that contains your product.
     *
     * @export
     * @class GetShipFromStoreTaskByBarcode
     * @implements {EvaService}
     */
    class GetShipFromStoreTaskByBarcode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShipFromStoreTaskByBarcode;
        response?: EVA.Core.GetShipFromStoreTaskByBarcodeResponse;
    }
    /**
     * Get a Shipment by its ID, returning information about where it came from, where it's shipped to, its shipping or delivery status and payment status
of the order(s) contained in the shipment.
     *
     * @export
     * @class GetShipment
     * @implements {EvaService}
     */
    class GetShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShipment;
        response?: EVA.Core.GetShipmentResponse;
    }
    /**
     * Get details of a shipment to receive, either by `ID` or by `BackendID`/`OrganizationUnitID` combination.
     *
     * @export
     * @class GetShipmentDetails
     * @implements {EvaService}
     */
    class GetShipmentDetails implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShipmentDetails;
        response?: EVA.Core.GetShipmentDetailsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetShipmentExportByID
     * @implements {EvaService}
     */
    class GetShipmentExportByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShipmentExportByID;
        response?: EVA.Core.GetShipmentExportByIDResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetShipmentExportDocuments
     * @implements {EvaService}
     */
    class GetShipmentExportDocuments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShipmentExportDocuments;
        response?: EVA.Core.GetShipmentExportDocumentsResponse;
    }
    /**
     * Get a shipment identifier by ID.
     *
     * @export
     * @class GetShipmentIdentifier
     * @implements {EvaService}
     */
    class GetShipmentIdentifier implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShipmentIdentifier;
        response?: EVA.Core.GetShipmentIdentifierResponse;
    }
    /**
     * Get an existing ShipmentReceipt
     *
     * @export
     * @class GetShipmentReceipt
     * @implements {EvaService}
     */
    class GetShipmentReceipt implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShipmentReceipt;
        response?: EVA.Core.GetShipmentReceiptResponse;
    }
    /**
     * @see: SetShipmentSettings
     *
     * @export
     * @class GetShipmentSettings
     * @implements {EvaService}
     */
    class GetShipmentSettings implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShipmentSettings;
        response?: EVA.Core.GetShipmentSettingsResponse;
    }
    /**
     * Returns all of the shipping methods that are being used on the order.
     *
     * @export
     * @class GetShippingMethodsForOrder
     * @implements {EvaService}
     */
    class GetShippingMethodsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShippingMethodsForOrder;
        response?: EVA.Core.GetShippingMethodsForOrderResponse;
    }
    /**
     * This returns the requested order. The order is retreived based on the OrderID.

The order will be recalculated:
- Discounts
- ShippingCosts
- GiftWrappingCosts
- OrderLine prices

If no recalculation is needed, use the GetOrder service instead. In general the approach would be: use GetShoppingCart while building the order, after PrepareOrderForCheckout has been called, switch to GetOrder.

To include the result of `GetRequiredDataForOrder`, the property `AdditionalOrderDataOptions.RequiredFor` can be set to the desired level of requirements.
To include the result of `GetProductRequirementValuesForOrder`, the property `AdditionalOrderDataOptions.IncludeProductRequirements` can be set to `true`.
To include the result of `GetGiftWrappingOptionsForOrder` and `GetAvailableGiftWrappingLinesForOrder`, the property `AdditionalOrderDataOptions.IncludeGiftWrapping` can be set to `true`.
To include the result of `PrefigureDiscounts` and `GetPotentialDiscountsForOrder`, the property `AdditionalOrderDataOptions.IncludePrefigureDiscounts` can be set to `true`.
To include the result of `GetPickProductDiscountOptionsForOrder`, the property `AdditionalOrderDataOptions.IncludePickProductOptions` can be set to `true`.
To include the result of `ListAvailableShippingMethods`, the property `AdditionalOrderDataOptions.IncludeAvailableShippingMethods` can be set to `true`.
To include the result of `GetAvailablePaymentMethods`, the property `AdditionalOrderDataOptions.IncludeAvailablePaymentMethods` can be set to `true`.
To include the result of `GetAvailableRefundPaymentMethodsForOrder`, the property `AdditionalOrderDataOptions.IncludeAvailableRefundPaymentMethods` can be set to `true`.
To include the result of `ValidateOrderShipment`, the property `AdditionalOrderDataOptions.IncludeValidateShipment` can be set to `true`.
To include the available actions for the PaymentTransactions, the property `AdditionalOrderDataOptions.IncludePaymentTransactionActions` can be set to `true`.
     *
     * @export
     * @class GetShoppingCart
     * @implements {EvaService}
     */
    class GetShoppingCart implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShoppingCart;
        response?: EVA.Core.ShoppingCartResponse;
    }
    /**
     * Fetch the summary of an Order. This summary contains:
- QuantityOrdered: the total count of products in the Order
- TotalAmount: the total amount of the Order
- ForeignTotalAmount: the total amount of the Order in the foreign currency
- CurrencyID: the currency of the Order

It is required to supply an OrderID
     *
     * @export
     * @class GetShoppingCartInfo
     * @implements {EvaService}
     */
    class GetShoppingCartInfo implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShoppingCartInfo;
        response?: EVA.Core.GetShoppingCartInfoResponse;
    }
    /**
     * Returns stores in proximity of a set of coordinates.

If `ProductID` and `HasStock` is set, only the stores with stock for that product will be returned.
     *
     * @export
     * @class GetShopsByProximity
     * @implements {EvaService}
     */
    class GetShopsByProximity implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetShopsByProximity;
        response?: EVA.Core.GetShopsByProximityResponse;
    }
    /**
     * Service for retieving a existing `Station`
     *
     * @export
     * @class GetStation
     * @implements {EvaService}
     */
    class GetStation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStation;
        response?: EVA.Core.GetStationResponse;
    }
    /**
     * Gets the status of the pre-counts of a cyclecount, if any. The input for this service is either the ID of the main count task (`ZonedCycleCountUserTaskID`)
or the ID of a pre-count task (`PreCountUserTaskID`). If `PreCountUserTaskID` is specified, it returns the status of the sibling pre-counts.
     *
     * @export
     * @class GetStatusForZonedCycleCountPreCounts
     * @implements {EvaService}
     */
    class GetStatusForZonedCycleCountPreCounts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStatusForZonedCycleCountPreCounts;
        response?: EVA.Core.GetStatusForZonedCycleCountPreCountsResponse;
    }
    /**
     * Calculates the stock availability for the provided OrderLines that are part of the provided Order.

The return value of this service is a list that contains, per OrderLine, whether or not it can be fulfilled (`CanBeFulfilled`) on the `RequestedDate`.

When an OrderLine doesn't already have a RequestedDate, a RequestedDate of tomorrow is used.

Additionally, a list of AlternativeDates is returned per OrderLine with dates on which it is expected that the OrderLine can be fulfilled.

When OrderLineIDs is left empty, this service will return an estimate for all unshipped lines of the order.

Doesn't work for orders with more than 25 lines.
     *
     * @export
     * @class GetStockAvailabilityEstimateForOrder
     * @implements {EvaService}
     */
    class GetStockAvailabilityEstimateForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockAvailabilityEstimateForOrder;
        response?: EVA.Core.GetStockAvailabilityEstimateForOrderResponse;
    }
    /**
     * Given a set of products this service returns a detailed estimate for each product when it's expected to be available.
     *
     * @export
     * @class GetStockAvailabilityEstimateForProducts
     * @implements {EvaService}
     */
    class GetStockAvailabilityEstimateForProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockAvailabilityEstimateForProducts;
        response?: EVA.Core.GetStockAvailabilityEstimateForProductsResponse;
    }
    /**
     * Returns the expected availability timeline for the provided `ProductIDs`, from the perspective of the provided `OrganizationUnitID` (optional, defaults to the user's current OrganizationUnitID).

The timeline is ordered chronologically and includes all events that are relevant to the availability of the requested products, including events that happen on the suppliers of the requested OrganizationUnit (such as PurchaseOrder receipts).

Additionally, a `FromDate` (inclusive) and `ToDate` (exclusive) can be used to filter the response by date and a `Type` can be provided to only return timeline items of a certain type.
     *
     * @export
     * @class GetStockAvailabilityTimeline
     * @implements {EvaService}
     */
    class GetStockAvailabilityTimeline implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockAvailabilityTimeline;
        response?: EVA.Core.GetStockAvailabilityTimelineResponse;
    }
    /**
     * Requests the stock per stock label, per product for the given organization unit. If there is no stock at all for a product the product is not
returned in the response.
     *
     * @export
     * @class GetStockByStockLabelForProducts
     * @implements {EvaService}
     */
    class GetStockByStockLabelForProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockByStockLabelForProducts;
        response?: EVA.Core.GetStockByStockLabelForProductsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetStockDetailsForProduct
     * @implements {EvaService}
     */
    class GetStockDetailsForProduct implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockDetailsForProduct;
        response?: EVA.Core.GetStockDetailsForProductResponse;
    }
    /**
     * Returns all organizationunits with the configured setting for a single stocklabel.

Every organizationunit that has any setting set for the stocklabel will be returned.
If an organizationunit has no setting for a specific feature, it will inherit the setting from it’s parent, this is indicated by the `IsInherited` property.
     *
     * @export
     * @class GetStockLabelSettingsForLabel
     * @implements {EvaService}
     */
    class GetStockLabelSettingsForLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockLabelSettingsForLabel;
        response?: EVA.Core.GetStockLabelSettingsForLabelResponse;
    }
    /**
     * Get stock labels
     *
     * @export
     * @class GetStockLabels
     * @implements {EvaService}
     */
    class GetStockLabels implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockLabels;
        response?: EVA.Core.GetStockLabelsResponse;
    }
    /**
     * Returns all stocklabels with the configured setting for a single organizationunit.
     *
     * @export
     * @class GetStockLabelsForFeature
     * @implements {EvaService}
     */
    class GetStockLabelsForFeature implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockLabelsForFeature;
        response?: EVA.Core.GetStockLabelsForFeatureResponse;
    }
    /**
     * Returns all stocklabels with the configured setting for a single organizationunit.
     *
     * @export
     * @class GetStockLabelsForFeatures
     * @implements {EvaService}
     */
    class GetStockLabelsForFeatures implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockLabelsForFeatures;
        response?: EVA.Core.GetStockLabelsForFeaturesResponse;
    }
    /**
     * Retrieve a StockNotification for a product and organization, for an EmailAddress - will return its ID, or null when it does not exist.
     *
     * @export
     * @class GetStockNotification
     * @implements {EvaService}
     */
    class GetStockNotification implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockNotification;
        response?: EVA.Core.StockNotificationResponse;
    }
    /**
     * Retrieve a StockNotification for a product and organization, for the current user - will return its ID, or null when it does not exist.
     *
     * @export
     * @class GetStockNotificationForCurrentUser
     * @implements {EvaService}
     */
    class GetStockNotificationForCurrentUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockNotificationForCurrentUser;
        response?: EVA.Core.StockNotificationResponse;
    }
    /**
     * Get the additional Stock Replenishment task data.
     *
     * @export
     * @class GetStockReplenishmentTaskData
     * @implements {EvaService}
     */
    class GetStockReplenishmentTaskData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockReplenishmentTaskData;
        response?: EVA.Core.GetStockReplenishmentTaskDataResponse;
    }
    /**
     * Get the configured zones for your current organization unit.
     *
     * @export
     * @class GetStockReplenishmentTaskZones
     * @implements {EvaService}
     */
    class GetStockReplenishmentTaskZones implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockReplenishmentTaskZones;
        response?: EVA.Core.GetStockReplenishmentTaskZonesResponse;
    }
    /**
     * Retrieve details on a Stock Reservation Task based on an OrderID.
     *
     * @export
     * @class GetStockReservationTask
     * @implements {EvaService}
     */
    class GetStockReservationTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStockReservationTask;
        response?: EVA.Core.GetStockReservationTaskResponse;
    }
    /**
     * Service for retrieving a `StringTranslation`
     *
     * @export
     * @class GetStringTranslation
     * @implements {EvaService}
     */
    class GetStringTranslation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetStringTranslation;
        response?: EVA.Core.GetStringTranslationResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetSupplierStockForProduct
     * @implements {EvaService}
     */
    class GetSupplierStockForProduct implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetSupplierStockForProduct;
        response?: EVA.Core.GetSupplierStockForProductResponse;
    }
    /**
     * Service for retrieving a list of `OrganizationUnitSupplier` by `OrganizationUnit`
     *
     * @export
     * @class GetSuppliersForOrganizationUnit
     * @implements {EvaService}
     */
    class GetSuppliersForOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetSuppliersForOrganizationUnit;
        response?: EVA.Core.GetSuppliersForOrganizationUnitResponse;
    }
    /**
     * Gets the available reasons for tax exemption.
The `Code` property of each result can then be supplied to `ExemptOrderFromTax` as the `TaxExemptionCode`.
     *
     * @export
     * @class GetTaxExemptionReasons
     * @implements {EvaService}
     */
    class GetTaxExemptionReasons implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetTaxExemptionReasons;
        response?: EVA.Core.GetTaxExemptionReasonsResponse;
    }
    /**
     * Get a tax provider by ID.
     *
     * @export
     * @class GetTaxProvider
     * @implements {EvaService}
     */
    class GetTaxProvider implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetTaxProvider;
        response?: EVA.Core.GetTaxProviderResponse;
    }
    /**
     * Get the available tax provider types.
     *
     * @export
     * @class GetTaxProviderTypes
     * @implements {EvaService}
     */
    class GetTaxProviderTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetTaxProviderTypes;
        response?: EVA.Core.GetTaxProviderTypesResponse;
    }
    /**
     * Gets a tax rate.

:point_right: See `CreateTaxRate` service for more info.
     *
     * @export
     * @class GetTaxRate
     * @implements {EvaService}
     */
    class GetTaxRate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetTaxRate;
        response?: EVA.Core.GetTaxRateResponse;
    }
    /**
     * Create a temporary token, based off of your current authentication token, which can be passed in query parameters for redirect requests. On the
receiving side, post this token in the `Login` service request body as `AuthenticationToken` to retrieve a new token, similar to the original
token, preserving user and context.

Note this token lifetime is only 30 seconds, and has no rights to do anything else.
     *
     * @export
     * @class GetTemporaryRedirectToken
     * @implements {EvaService}
     */
    class GetTemporaryRedirectToken implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetTemporaryRedirectToken;
        response?: EVA.Core.GetTemporaryRedirectTokenResponse;
    }
    /**
     * Allows customers to grab a token to be able to view their order for a limited amount of time. The token is
obtained through the user of either `OrderID` or both a `BackendSystemID` and a `BackendID` in combination
with the `EmailAddress` of the customer that is attached to the order.
     *
     * @export
     * @class GetTemporaryTokenForOrder
     * @implements {EvaService}
     */
    class GetTemporaryTokenForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetTemporaryTokenForOrder;
        response?: EVA.Core.GetTemporaryTokenForOrderResponse;
    }
    /**
     * Retrieve information about a temporary access token from specified `Authorization` parameter.
     *
     * @export
     * @class GetTokenInfo
     * @implements {EvaService}
     */
    class GetTokenInfo implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetTokenInfo;
        response?: EVA.Core.GetTokenInfoResponse;
    }
    /**
     * Get additional order line information intended for transport data.
     *
     * @export
     * @class GetTransportOrderLineData
     * @implements {EvaService}
     */
    class GetTransportOrderLineData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetTransportOrderLineData;
        response?: EVA.Core.GetTransportOrderLineDataResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetTransputJobTypes
     * @implements {EvaService}
     */
    class GetTransputJobTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetTransputJobTypes;
        response?: EVA.Core.GetTransputJobTypesResponse;
    }
    /**
     * Get all unitPrice correction reasons
     *
     * @export
     * @class GetUnitPriceCorrectionReasons
     * @implements {EvaService}
     */
    class GetUnitPriceCorrectionReasons implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUnitPriceCorrectionReasons;
        response?: EVA.Core.GetUnitPriceCorrectionReasonsResponse;
    }
    /**
     * Get the estimated amount of unprocessed financial events.
     *
     * @export
     * @class GetUnprocessedFinancialEventsCount
     * @implements {EvaService}
     */
    class GetUnprocessedFinancialEventsCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUnprocessedFinancialEventsCount;
        response?: EVA.Core.GetUnprocessedFinancialEventsCountResponse;
    }
    /**
     * Get a user by ID
     *
     * @export
     * @class GetUser
     * @implements {EvaService}
     */
    class GetUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUser;
        response?: EVA.Core.GetUserResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetUserAssociationTypeByID
     * @implements {EvaService}
     */
    class GetUserAssociationTypeByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserAssociationTypeByID;
        response?: EVA.Core.GetUserAssociationTypeByIDResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetUserAssociationTypes
     * @implements {EvaService}
     */
    class GetUserAssociationTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserAssociationTypes;
        response?: EVA.Core.GetUserAssociationTypesResponse;
    }
    /**
     * Return the current user associations for the given user
     *
     * @export
     * @class GetUserAssociations
     * @implements {EvaService}
     */
    class GetUserAssociations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserAssociations;
        response?: EVA.Core.GetUserAssociationsResponse;
    }
    /**
     * Find an User by their SocialSecurityNumber. This service only returns an UserID.
When more info is needed, additional data can be fetched with the `GetUser` service.
     *
     * @export
     * @class GetUserBySocialSecurityNumber
     * @implements {EvaService}
     */
    class GetUserBySocialSecurityNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserBySocialSecurityNumber;
        response?: EVA.Core.GetUserBySocialSecurityNumberResponse;
    }
    /**
     * Get the user Consignment property.
     *
     * @export
     * @class GetUserConsignment
     * @implements {EvaService}
     */
    class GetUserConsignment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserConsignment;
        response?: EVA.Core.GetUserConsignmentResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetUserDebtorData
     * @implements {EvaService}
     */
    class GetUserDebtorData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserDebtorData;
        response?: EVA.Core.GetUserDebtorDataResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetUserGroupByID
     * @implements {EvaService}
     */
    class GetUserGroupByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserGroupByID;
        response?: EVA.Core.GetUserGroupByIDResponse;
    }
    /**
     * Fetch user information from an external API based on the phone number. This can be used to enrich the user in the frontend.
Depending on the phone number provider, this may require Customer or Employee rights.
     *
     * @export
     * @class GetUserInfoByPhoneNumber
     * @implements {EvaService}
     */
    class GetUserInfoByPhoneNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserInfoByPhoneNumber;
        response?: EVA.Core.GetUserInfoResponse;
    }
    /**
     * Fetch user information from an external API based on the social security number. This can be used to enrich the user in the frontend.
Depending on the SSN provider, this may require Customer or Employee rights.
     *
     * @export
     * @class GetUserInfoBySocialSecurityNumber
     * @implements {EvaService}
     */
    class GetUserInfoBySocialSecurityNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserInfoBySocialSecurityNumber;
        response?: EVA.Core.GetUserInfoResponse;
    }
    /**
     * Returns user inquiries
     *
     * @export
     * @class GetUserInquiries
     * @implements {EvaService}
     */
    class GetUserInquiries implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserInquiries;
        response?: EVA.Core.GetUserInquiriesResponse;
    }
    /**
     * Returns a user inquiry with data
     *
     * @export
     * @class GetUserInquiry
     * @implements {EvaService}
     */
    class GetUserInquiry implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserInquiry;
        response?: EVA.Core.GetUserInquiryResponse;
    }
    /**
     * Get a UserInteration by its ID. A UserInteraction logs an interaction between an employee and a customer, for example the fact the customer was called.
     *
     * @export
     * @class GetUserInteractionByID
     * @implements {EvaService}
     */
    class GetUserInteractionByID implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserInteractionByID;
        response?: EVA.Core.GetUserInteractionByIDResponse;
    }
    /**
     * Get the user loyalty balance.
     *
     * @export
     * @class GetUserLoyaltyBalance
     * @implements {EvaService}
     */
    class GetUserLoyaltyBalance implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserLoyaltyBalance;
        response?: EVA.Core.GetUserLoyaltyBalanceResponse;
    }
    /**
     * Get all user loyalty subscriptions, this takes into account the OU this request is done on and will only show subscriptions available in that OU.
     *
     * @export
     * @class GetUserLoyaltySubscriptions
     * @implements {EvaService}
     */
    class GetUserLoyaltySubscriptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserLoyaltySubscriptions;
        response?: EVA.Core.GetUserLoyaltySubscriptionsResponse;
    }
    /**
     * Get the requested user's phone numbers. If UserID is left null, this service will return the phone numbers for your current user.
Only employee users are allowed to access the phone numbers of other users.
     *
     * @export
     * @class GetUserPhoneNumbers
     * @implements {EvaService}
     */
    class GetUserPhoneNumbers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserPhoneNumbers;
        response?: EVA.Core.GetUserPhoneNumbersResponse;
    }
    /**
     * Get all User subscriptions, this takes into account the OU this request is done on and will only show subscriptions available in that OU.
     *
     * @export
     * @class GetUserSubscriptions
     * @implements {EvaService}
     */
    class GetUserSubscriptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserSubscriptions;
        response?: EVA.Core.GetUserSubscriptionsResponse;
    }
    /**
     * Return the total number of open tasks, and the count per task type.
     *
     * @export
     * @class GetUserTaskCounts
     * @implements {EvaService}
     */
    class GetUserTaskCounts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserTaskCounts;
        response?: EVA.Core.GetUserTaskCountsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetUserTaskDetails
     * @implements {EvaService}
     */
    class GetUserTaskDetails implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserTaskDetails;
        response?: EVA.Core.GetUserTaskDetailsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetUserTaskNotCompletedReasons
     * @implements {EvaService}
     */
    class GetUserTaskNotCompletedReasons implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserTaskNotCompletedReasons;
        response?: EVA.Core.GetUserTaskNotCompletedReasonsResponse;
    }
    /**
     * Return all of the configured User Task Priority configurations.

These are used at real-time on the User Task details whenever fetched, translating the User Task `Priority` into a `PriorityDescription`. The
`LanguageID` and `CountryID` filters are compared against the user context requesting this information, and the `UserTaskTypeID`, `UserTaskSubTypeID`
and `Priority` are taken from the User Task itself. The `Priority` from the User Task will be compared against the `From` and `To` ranges defined,
where the `From` priority is an *inclusive* lower bound, while `To` is the *exclusive* upper bound.

The `Description` will be used to describe the priority, but when left empty, will fall back to `Name`.

An example configuration could be as follows;

```
Language | From | To | Description
---------|------|----|------------
      NL |      |    | Standaard
      NL |   10 |    | Spoed
      DE |      | 10 | Unwichtig
         |      |    | Default
```

A User Task with priority 9 for a Dutch employee would be labelled "Standaard", while for a German colleague this would be called "Unwichtig". A
User Task of priority 10 would be called "Spoed" in Dutch, but "Default" in German.

:sunglasses: For added awesomeness, if you define overlapping priority ranges, we'll select the most narrow range for you.
     *
     * @export
     * @class GetUserTaskPriorities
     * @implements {EvaService}
     */
    class GetUserTaskPriorities implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserTaskPriorities;
        response?: EVA.Core.GetUserTaskPrioritiesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetUserTaskTypeOrganizationUnitSet
     * @implements {EvaService}
     */
    class GetUserTaskTypeOrganizationUnitSet implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserTaskTypeOrganizationUnitSet;
        response?: EVA.Core.UserTaskTypeOrganizationUnitSetResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetUserTaskTypes
     * @implements {EvaService}
     */
    class GetUserTaskTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserTaskTypes;
        response?: EVA.Core.GetUserTaskTypesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class GetUserWhoDeletedCycleCountZoneAndMakeItGreatAgain
     * @implements {EvaService}
     */
    class GetUserWhoDeletedCycleCountZoneAndMakeItGreatAgain implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserWhoDeletedCycleCountZoneAndMakeItGreatAgain;
        response?: EVA.Core.GetUserWhoDeletedCycleCountZoneAndMakeItGreatAgainResponse;
    }
    /**
     * Get the user wishlist by ID.
     *
     * @export
     * @class GetUserWishlist
     * @implements {EvaService}
     */
    class GetUserWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetUserWishlist;
        response?: EVA.Core.GetUserWishlistResponse;
    }
    /**
     * Get additional order information intended for warehouse data.
     *
     * @export
     * @class GetWarehouseOrderData
     * @implements {EvaService}
     */
    class GetWarehouseOrderData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetWarehouseOrderData;
        response?: EVA.Core.GetWarehouseOrderDataResponse;
    }
    /**
     * This service return the available webshop OrganizationUnits
     *
     * @export
     * @class GetWebshops
     * @implements {EvaService}
     */
    class GetWebshops implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetWebshops;
        response?: EVA.Core.GetWebshopsResponse;
    }
    /**
     * Returns a detailed view of a ZonedCycleCount by the ID of its UserTask.

The response contains:

- Which StockLabels to count (`StockLabelsToCount`)
- The product that needs to be counted. This product contains all the fields in its `Content` object that were requested through the `IncludedFields` property on the request, or a default set of fields if not specified.
- Which StockMutations have been created since the first pre-count (if any) per StockLabel (`StockMutationsSincePreCount`)
- A list of all pre-counts, complete with the zone that needs to be counted, the user who did the count (if the count already happened) and a list of count results per StockLabel
- A list of pre-counted quantityies per StockLabel (`PreCountedQuantities`)
- A list of the total quantity of stock mutated per StockLabel (a summary of `StockMutationsSincePreCount`).
- The current stock per StockLabel
- ExpectedPreCountedQuantity: This is stored on the ZonedCyleCount the moment the last pre-count has been completed, and is a sum of the expected Stock at that moment.
- PreCountedQuantity: The quantity that was actually counted in all pre-counts over all StockLabels. This is recorded when all pre-counts have been completed.
     *
     * @export
     * @class GetZonedCycleCount
     * @implements {EvaService}
     */
    class GetZonedCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetZonedCycleCount;
        response?: EVA.Core.GetZonedCycleCountResponse;
    }
    /**
     * Returns a detailed view of a ZonedCycleCountPreCount by the ID of its UserTask.

The response contains:

- Which StockLabels to count (`StockLabelsToCount`)
- The product that needs to be counted (`Product`); this product contains all the fields in its `Content` object that were requested through the `IncludedFields` property on the request, or a default set of fields if not specified
- Zone that needs to be counted (`CycleCountZone`)
- List of stock labels to be counted (`StockLabelsToCount`)
- List of required resource types (`RequiredResourceTypes`)
     *
     * @export
     * @class GetZonedCycleCountPreCount
     * @implements {EvaService}
     */
    class GetZonedCycleCountPreCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetZonedCycleCountPreCount;
        response?: EVA.Core.GetZonedCycleCountPreCountResponse;
    }
    /**
     * Returns a count of potential cycle counts that would be created for the combinations of (type Stock) Products and Organization Units that match specified filters. Use this service to preview results before actual creation.
     *
     * @export
     * @class GetZonedCycleCountsEstimateByQuery
     * @implements {EvaService}
     */
    class GetZonedCycleCountsEstimateByQuery implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetZonedCycleCountsEstimateByQuery;
        response?: EVA.Core.GetZonedCycleCountsEstimateByQueryResponse;
    }
    /**
     * Returns a count of potential cycle counts that would be created for the combinations of (type Stock) Products and Organization Units that match specified filters. Use this service to preview results before actual creation.
     *
     * @export
     * @class GetZonedCycleCountsEstimateByQuery_Async
     * @implements {EvaService}
     */
    class GetZonedCycleCountsEstimateByQuery_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.GetZonedCycleCountsEstimateByQuery;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * Returns a count of potential cycle counts that would be created for the combinations of (type Stock) Products and Organization Units that match specified filters. Use this service to preview results before actual creation.
     *
     * @export
     * @class GetZonedCycleCountsEstimateByQuery_AsyncResult
     * @implements {EvaService}
     */
    class GetZonedCycleCountsEstimateByQuery_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.GetZonedCycleCountsEstimateByQueryResponse;
    }
    /**
     * When a PaymentTransaction is disputed due to a Chargeback or a Reversal, the difference can be booked in a different PaymentTransaction to allow the order to be completed.

Calling this service will result in an additional PaymentTransaction of type `DISPUTE`.
     *
     * @export
     * @class HandlePaymentTransactionDispute
     * @implements {EvaService}
     */
    class HandlePaymentTransactionDispute implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.HandlePaymentTransactionDispute;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Creates multiple Flight objects for a specific Organization Unit.
    The flights are identified from a GRC type file containing the necessary information.
     *
     * @export
     * @class ImportFlights
     * @implements {EvaService}
     */
    class ImportFlights implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ImportFlights;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Creates or updates specified `OrganizationUnits`, identified by your `ID` (that we map to our `BackendID`).

Omit any field you don't want altered, or send in null values of nullable fields to unset their current value.

- Does not apply to `Address` - if you specify a new or different one from the current, a new Address will be created with only the data specify!
- Does not apply to `OpeningHours` - every value you specify will either update or create a record for that day
     *
     * @export
     * @class ImportOrganizationUnits
     * @implements {EvaService}
     */
    class ImportOrganizationUnits implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ImportOrganizationUnits;
        response?: EVA.Core.ImportOrganizationUnitsResponse;
    }
    /**
     * Set up a payment type
     *
     * @export
     * @class InitializePaymentType
     * @implements {EvaService}
     */
    class InitializePaymentType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.InitializePaymentType;
        response?: EVA.Core.InitializePaymentTypeResponse;
    }
    /**
     * Links the shared wishlist to the user. Requires the share token. When linked the wishlist is returned in the users overview.
     *
     * @export
     * @class LinkSharedWishlist
     * @implements {EvaService}
     */
    class LinkSharedWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.LinkSharedWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for retrieving a list of `AddressBook`
     *
     * @export
     * @class ListAddressBook
     * @implements {EvaService}
     */
    class ListAddressBook implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListAddressBook;
        response?: EVA.Core.ListAddressBookResponse;
    }
    /**
     * List API keys, optionally filtered.
     *
     * @export
     * @class ListApiKeys
     * @implements {EvaService}
     */
    class ListApiKeys implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListApiKeys;
        response?: EVA.Core.ListApiKeysResponse;
    }
    /**
     * Get a list of all active Applications
     *
     * @export
     * @class ListApplications
     * @implements {EvaService}
     */
    class ListApplications implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListApplications;
        response?: EVA.Core.ListApplicationsResponse;
    }
    /**
     * Lists all tasks that are able to be configured to run.
     *
     * @export
     * @class ListAvailableRecurringTasks
     * @implements {EvaService}
     */
    class ListAvailableRecurringTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListAvailableRecurringTasks;
        response?: EVA.Core.ListAvailableRecurringTasksResponse;
    }
    /**
     * Returns a set of all the shipping methods that can be used for the given order. Use `SetShippingMethod` to update the shipping method on the order.
     *
     * @export
     * @class ListAvailableShippingMethods
     * @implements {EvaService}
     */
    class ListAvailableShippingMethods implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListAvailableShippingMethods;
        response?: EVA.Core.ListAvailableShippingMethodsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListAvailableTimeZones
     * @implements {EvaService}
     */
    class ListAvailableTimeZones implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListAvailableTimeZones;
        response?: EVA.Core.ListAvailableTimeZonesResponse;
    }
    /**
     * List the available UserTasks for the current user.

The response has a `Aggregations` property. This property contains the available filters for this set of UserTasks.
To apply such a filter it should added in the `Filters` property on the RequestMessage.

Example response:
```
{
...
,
  "Aggregations": {
      "ZonedCycleCountPreCount": {
        "CycleCountZoneID": [
          {
            "Name": "Shelves",
            "Value": 10,
            "Count": 1
          },
          {
            "Name": "Stockroom",
            "Value": 12,
            "Count": 6
          },
          {
            "Name": "Store",
            "Value": 13,
            "Count": 4
          }
        ]
      }
    }
  }
}
```

To apply a matching filter:
```
{
  "Filters": {
    "ZonedCycleCountPreCount": {
      "CycleCountZoneID": [10]
    }
  }
}
```
     *
     * @export
     * @class ListAvailableUserTasks
     * @implements {EvaService}
     */
    class ListAvailableUserTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListAvailableUserTasks;
        response?: EVA.Core.ListAvailableUserTasksResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListBlobsForFinancialPeriod
     * @implements {EvaService}
     */
    class ListBlobsForFinancialPeriod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListBlobsForFinancialPeriod;
        response?: EVA.Core.ListBlobsForFinancialPeriodResponse;
    }
    /**
     * List all the blobs that're attached to the given invoice (`InvoiceID`).
     *
     * @export
     * @class ListBlobsForInvoice
     * @implements {EvaService}
     */
    class ListBlobsForInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListBlobsForInvoice;
        response?: EVA.Core.ListBlobsForInvoiceResponse;
    }
    /**
     * Get the blobs that are attached to an Order

It's required to specify the order, optionally the blobs can also be filtered on `InvoiceID` of `ShipmentID`.
     *
     * @export
     * @class ListBlobsForOrder
     * @implements {EvaService}
     */
    class ListBlobsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListBlobsForOrder;
        response?: EVA.Core.ListBlobsForOrderResponse;
    }
    /**
     * Lists all of the bookings that've been made for the given financial period. Which you can retrieve
by setting the filter `FinancialPeriodID` on the `PageConfig`.
     *
     * @export
     * @class ListBookings
     * @implements {EvaService}
     */
    class ListBookings implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListBookings;
        response?: EVA.Core.ListBookingsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListBrands
     * @implements {EvaService}
     */
    class ListBrands implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListBrands;
        response?: EVA.Core.ListBrandsResponse;
    }
    /**
     * Lists all cash deposits for your current financial period.
     *
     * @export
     * @class ListCashDeposits
     * @implements {EvaService}
     */
    class ListCashDeposits implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCashDeposits;
        response?: EVA.Core.ListCashDepositsResponse;
    }
    /**
     * Lists all cash expenses for your current financial period.
     *
     * @export
     * @class ListCashExpenses
     * @implements {EvaService}
     */
    class ListCashExpenses implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCashExpenses;
        response?: EVA.Core.ListCashExpensesResponse;
    }
    /**
     * This service allows you to view the cash transaction ledger, which underlies all cash transactions,
such as received payments, moves between devices and expenses.

By default, you only get access to the cash transaction ledger for your current financial period. If
you have the `ManageCashTransactionLedger` permission, you're also allowed to filter on another financial
period.
     *
     * @export
     * @class ListCashTransactionLedger
     * @implements {EvaService}
     */
    class ListCashTransactionLedger implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCashTransactionLedger;
        response?: EVA.Core.ListCashTransactionLedgerResponse;
    }
    /**
     * This service allows you to get a grouped view of the cash transaction ledger, which underlies all cash
transactions, such as received payments, moves between devices and expenses.

By default, you only get access to the cash transaction ledger for your current financial period. If
you have the `ManageCashTransactionLedger` permission, you're also allowed to filter on another financial
period.
     *
     * @export
     * @class ListCashTransactionLedgerGroups
     * @implements {EvaService}
     */
    class ListCashTransactionLedgerGroups implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCashTransactionLedgerGroups;
        response?: EVA.Core.ListCashTransactionLedgerGroupsResponse;
    }
    /**
     * This service lists the OrderLines for the given ProductID that currently have committed Stock.
     *
     * @export
     * @class ListCommittedOrderLines
     * @implements {EvaService}
     */
    class ListCommittedOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCommittedOrderLines;
        response?: EVA.Core.ListCommittedOrderLinesResponse;
    }
    /**
     * List the available ConsolidatedPickTask tasks for the current user.
     *
     * @export
     * @class ListConsolidatedPickTasks
     * @implements {EvaService}
     */
    class ListConsolidatedPickTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListConsolidatedPickTasks;
        response?: EVA.Core.ListConsolidatedPickTasksResponse;
    }
    /**
     * List the available ConsolidatedZonePickTask tasks for the current user.
     *
     * @export
     * @class ListConsolidatedZonePickTasks
     * @implements {EvaService}
     */
    class ListConsolidatedZonePickTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListConsolidatedZonePickTasks;
        response?: EVA.Core.ListConsolidatedZonePickTasksResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListControlDocuments
     * @implements {EvaService}
     */
    class ListControlDocuments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListControlDocuments;
        response?: EVA.Core.ListControlDocumentsResponse;
    }
    /**
     * Gets the full list of countries. When `FetchAll` is set to true, all the `Country` entries are returned, rather than just the ones limited by the `OrganizationUnitCountry` constraint (`false` or `null`, default).
     *
     * @export
     * @class ListCountries
     * @implements {EvaService}
     */
    class ListCountries implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCountries;
        response?: EVA.Core.ListCountriesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListCurrencies
     * @implements {EvaService}
     */
    class ListCurrencies implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCurrencies;
        response?: EVA.Core.ListCurrenciesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListCustomOrderStatus
     * @implements {EvaService}
     */
    class ListCustomOrderStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCustomOrderStatus;
        response?: EVA.Core.ListCustomOrderStatusResponse;
    }
    /**
     * Lists all of the custom usertask templates
     *
     * @export
     * @class ListCustomUserTaskTemplate
     * @implements {EvaService}
     */
    class ListCustomUserTaskTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCustomUserTaskTemplate;
        response?: EVA.Core.ListCustomUserTaskTemplateResponse;
    }
    /**
     * List all custom usertask types
     *
     * @export
     * @class ListCustomUserTaskTypes
     * @implements {EvaService}
     */
    class ListCustomUserTaskTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCustomUserTaskTypes;
        response?: EVA.Core.ListCustomUserTaskTypesResponse;
    }
    /**
     * Gets all the CycleCountZoneGroups based on the given filter and returns a paged response.
     *
     * @export
     * @class ListCycleCountZoneGroups
     * @implements {EvaService}
     */
    class ListCycleCountZoneGroups implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCycleCountZoneGroups;
        response?: EVA.Core.ListCycleCountZoneGroupsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListCycleZonesForOrganizationUnits
     * @implements {EvaService}
     */
    class ListCycleZonesForOrganizationUnits implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListCycleZonesForOrganizationUnits;
        response?: EVA.Core.ListCycleZonesForOrganizationUnitsResponse;
    }
    /**
     * Service for retrieving a list of `Device`
     *
     * @export
     * @class ListDevices
     * @implements {EvaService}
     */
    class ListDevices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListDevices;
        response?: EVA.Core.ListDevicesResponse;
    }
    /**
     * List blobs intended for app diagnostics.
     *
     * @export
     * @class ListDiagnosticsBlobs
     * @implements {EvaService}
     */
    class ListDiagnosticsBlobs implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListDiagnosticsBlobs;
        response?: EVA.Core.ListDiagnosticsBlobsResponse;
    }
    /**
     * List disabled discounts triggers on order
     *
     * @export
     * @class ListDisabledDiscountTriggersOnOrder
     * @implements {EvaService}
     */
    class ListDisabledDiscountTriggersOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListDisabledDiscountTriggersOnOrder;
        response?: EVA.Core.ListDisabledDiscountTriggersOnOrderResponse;
    }
    /**
     * List disabled discounts on order
     *
     * @export
     * @class ListDisabledDiscountsOnOrder
     * @implements {EvaService}
     */
    class ListDisabledDiscountsOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListDisabledDiscountsOnOrder;
        response?: EVA.Core.ListDisabledDiscountsOnOrderResponse;
    }
    /**
     * This services returns a list of discount coupon handlers
     *
     * @export
     * @class ListDiscountCouponHandlers
     * @implements {EvaService}
     */
    class ListDiscountCouponHandlers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListDiscountCouponHandlers;
        response?: EVA.Core.ListDiscountCouponHandlersResponse;
    }
    /**
     * Service for retrieving a list of `EntityTranslation`
     *
     * @export
     * @class ListEntityTranslations
     * @implements {EvaService}
     */
    class ListEntityTranslations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListEntityTranslations;
        response?: EVA.Core.ListEntityTranslationsResponse;
    }
    /**
     * This service lets you query the EventLedger for auditing purposes.

The request provides a PageConfig (of type ScrollablePageConfig) that lets you filter on the given properties in the EventLedger.
Paging is done by providing the request with the `NextResultToken` that you got back from the previous request.
As this method of paging is forward-only, you can't go back to a previous page unless you store the token or the results,
and the total number of results is unknown, only whether or not there are more results (determined by if the NextResultToken is null).

The response contains the whole EventLedger document.

Possible filters:

- Types: A list of strings that refer to the `Type` field that is in every document.
- OrderID: Only return documents that contain a reference to the given OrderID
- OrganizationUnitID: Only return documents that were created by someone logged into the given OrganizationUnitID. It's recommended that this filter is always used, due to performance reasons.
- DeviceID: Filters on documents that have a reference to the given device, usually associated with a payment or a print command.
- FromDate: Only return documents newer than this date. It's recommended that this filter is always used, due to performance reasons.
- ToDate: Only return documents older than this date. It's recommended that this filter is always used, due to performance reasons.
- UserIDs: Only return documents created by these users. The IDs for this can be gotten by first doing a user search.
- FirstName: Filter on documents created by users whose first name starts with this. Achieves the same as filtering on UserIDs, but with one step less. However, due to privacy reasons, this information isn't stored for customers.
- FullName: Filter on documents created by users whose full name starts with this. Achieves the same as filtering on UserIDs, but with one step less. However, due to privacy reasons, this information isn't stored for customers.
- EmailAddress: Filter on documents created by users whose email starts with this. Achieves the same as filtering on UserIDs, but with one step less. However, due to privacy reasons, this information isn't stored for customers.
     *
     * @export
     * @class ListEventLedger
     * @implements {EvaService}
     */
    class ListEventLedger implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListEventLedger;
        response?: EVA.Core.ListEventLedgerResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListExternalOrderStatusForOrder
     * @implements {EvaService}
     */
    class ListExternalOrderStatusForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListExternalOrderStatusForOrder;
        response?: EVA.Core.ListExternalOrderStatusForOrderResponse;
    }
    /**
     * List the active feed subscriptions for the current user's `ApplicationID`.
     *
     * @export
     * @class ListFeedSubscriptions
     * @implements {EvaService}
     */
    class ListFeedSubscriptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFeedSubscriptions;
        response?: EVA.Core.ListFeedSubscriptionsResponse;
    }
    /**
     * List financial events.
     *
     * @export
     * @class ListFinancialEvents
     * @implements {EvaService}
     */
    class ListFinancialEvents implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFinancialEvents;
        response?: EVA.Core.ListFinancialEventsResponse;
    }
    /**
     * Lists all `Financial Period Audit` with their possible BlobID's for audition purposes.
     *
     * @export
     * @class ListFinancialPeriodAudits
     * @implements {EvaService}
     */
    class ListFinancialPeriodAudits implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFinancialPeriodAudits;
        response?: EVA.Core.ListFinancialPeriodAuditsResponse;
    }
    /**
     * List financial period exports
     *
     * @export
     * @class ListFinancialPeriodExports2
     * @implements {EvaService}
     */
    class ListFinancialPeriodExports2 implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFinancialPeriodExports2;
        response?: EVA.Core.ListFinancialPeriodExports2Response;
    }
    /**
     * List financialperiods for an OrganizationUnit
     *
     * @export
     * @class ListFinancialPeriods
     * @implements {EvaService}
     */
    class ListFinancialPeriods implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFinancialPeriods;
        response?: EVA.Core.ListFinancialPeriodsResponse;
    }
    /**
     * List all the existing Flights mathing the specified filter
     *
     * @export
     * @class ListFlights
     * @implements {EvaService}
     */
    class ListFlights implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFlights;
        response?: EVA.Core.ListFlightsResponse;
    }
    /**
     * Lists the products of specified full stock count label.
     *
     * @export
     * @class ListFullStockCountLabelProducts
     * @implements {EvaService}
     */
    class ListFullStockCountLabelProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFullStockCountLabelProducts;
        response?: EVA.Core.ListFullStockCountLabelProductsResponse;
    }
    /**
     * Lists the summary of specified Full Stock Count.
     *
     * @export
     * @class ListFullStockCountLabelSummary
     * @implements {EvaService}
     */
    class ListFullStockCountLabelSummary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFullStockCountLabelSummary;
        response?: EVA.Core.ListFullStockCountLabelSummaryResponse;
    }
    /**
     * List the available FullStockCountLabel tasks for the current user.
     *
     * @export
     * @class ListFullStockCountLabelTasks
     * @implements {EvaService}
     */
    class ListFullStockCountLabelTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFullStockCountLabelTasks;
        response?: EVA.Core.ListFullStockCountLabelTasksResponse;
    }
    /**
     * Lists the Full Stock Count labels.
     *
     * @export
     * @class ListFullStockCountLabels
     * @implements {EvaService}
     */
    class ListFullStockCountLabels implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFullStockCountLabels;
        response?: EVA.Core.ListFullStockCountLabelsResponse;
    }
    /**
     * Lists the result of specified Full Stock Count.
     *
     * @export
     * @class ListFullStockCountResult
     * @implements {EvaService}
     */
    class ListFullStockCountResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFullStockCountResult;
        response?: EVA.Core.ListFullStockCountResultResponse;
    }
    /**
     * Retrieves the status of serial numbers for a specific product and stock label within a Full Stock Count.
     *
     * @export
     * @class ListFullStockCountSerialNumbersStatus
     * @implements {EvaService}
     */
    class ListFullStockCountSerialNumbersStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFullStockCountSerialNumbersStatus;
        response?: EVA.Core.ListFullStockCountSerialNumbersStatusResponse;
    }
    /**
     * Lists the summary of specified Full Stock Count.
     *
     * @export
     * @class ListFullStockCountSummary
     * @implements {EvaService}
     */
    class ListFullStockCountSummary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFullStockCountSummary;
        response?: EVA.Core.ListFullStockCountSummaryResponse;
    }
    /**
     * List the available FullStockCount tasks for the current user.
     *
     * @export
     * @class ListFullStockCountTasks
     * @implements {EvaService}
     */
    class ListFullStockCountTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFullStockCountTasks;
        response?: EVA.Core.ListFullStockCountTasksResponse;
    }
    /**
     * Lists the full stock counts.
     *
     * @export
     * @class ListFullStockCounts
     * @implements {EvaService}
     */
    class ListFullStockCounts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListFullStockCounts;
        response?: EVA.Core.ListFullStockCountsResponse;
    }
    /**
     * Returns a paged view of all the general ledgers.

The page size `Limit` has a maximum value of `1.000` for this service.
     *
     * @export
     * @class ListGeneralLedgers
     * @implements {EvaService}
     */
    class ListGeneralLedgers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListGeneralLedgers;
        response?: EVA.Core.ListGeneralLedgersResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListInitialCycleCountLabels
     * @implements {EvaService}
     */
    class ListInitialCycleCountLabels implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListInitialCycleCountLabels;
        response?: EVA.Core.ListInitialCycleCountLabelsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListInitialCycleCountResults
     * @implements {EvaService}
     */
    class ListInitialCycleCountResults implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListInitialCycleCountResults;
        response?: EVA.Core.ListInitialCycleCountResultsResponse;
    }
    /**
     * Lists the OrganizationUnits that are available for the given OrderID to be used as the target for an interbranch order.
     *
     * @export
     * @class ListInterbranchOrganizationUnitForOrder
     * @implements {EvaService}
     */
    class ListInterbranchOrganizationUnitForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListInterbranchOrganizationUnitForOrder;
        response?: EVA.Core.ListInterbranchOrganizationUnitForOrderResponse;
    }
    /**
     * Lists the `InvoiceCompanyMapping` items.
     *
     * @export
     * @class ListInvoiceCompanyMapping
     * @implements {EvaService}
     */
    class ListInvoiceCompanyMapping implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListInvoiceCompanyMapping;
        response?: EVA.Core.ListInvoiceCompanyMappingResponse;
    }
    /**
     * List invoice exports
     *
     * @export
     * @class ListInvoiceExports
     * @implements {EvaService}
     */
    class ListInvoiceExports implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListInvoiceExports;
        response?: EVA.Core.ListInvoiceExportResponse;
    }
    /**
     * List the exports for the given Invoice
     *
     * @export
     * @class ListInvoiceExportsForInvoice
     * @implements {EvaService}
     */
    class ListInvoiceExportsForInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListInvoiceExportsForInvoice;
        response?: EVA.Core.ListInvoiceExportsForInvoiceResponse;
    }
    /**
     * List ledgers for invoices

Available filters:
- InvoiceID
- InvoiceLineID
- InvoiceDisputeID
- Type
     *
     * @export
     * @class ListInvoiceLedger
     * @implements {EvaService}
     */
    class ListInvoiceLedger implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListInvoiceLedger;
        response?: EVA.Core.ListInvoiceLedgerResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListInvoices
     * @implements {EvaService}
     */
    class ListInvoices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListInvoices;
        response?: EVA.Core.ListInvoicesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListInvoicesForOrder
     * @implements {EvaService}
     */
    class ListInvoicesForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListInvoicesForOrder;
        response?: EVA.Core.ListInvoicesForOrderResponse;
    }
    /**
     * List the available manual discounts. Optionally, the OrderType or DiscountAppliesTo filter can be given.
     *
     * @export
     * @class ListManualDiscounts
     * @implements {EvaService}
     */
    class ListManualDiscounts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListManualDiscounts;
        response?: EVA.Core.ListManualDiscountsResponse;
    }
    /**
     * List manually created invoices. These invoices are created via the `CreateManualInvoice` service and are used as a worst-case scenario when everything has gone offline and the retailer has to make invoices on actual paper. These services are then used to enter the paper invoices back in to EVA.
     *
     * @export
     * @class ListManualInvoices
     * @implements {EvaService}
     */
    class ListManualInvoices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListManualInvoices;
        response?: EVA.Core.ListManualInvoicesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListManualTaxExemptionReasons
     * @implements {EvaService}
     */
    class ListManualTaxExemptionReasons implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListManualTaxExemptionReasons;
        response?: EVA.Core.ListManualTaxExemptionReasonsResponse;
    }
    /**
     * Lists the errors from the message queue.
     *
     * @export
     * @class ListMessageQueueErrors
     * @implements {EvaService}
     */
    class ListMessageQueueErrors implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListMessageQueueErrors;
        response?: EVA.Core.ListMessageQueueErrorsResponse;
    }
    /**
     * List the UserTasks for the current user.
     *
     * @export
     * @class ListMyUserTasks
     * @implements {EvaService}
     */
    class ListMyUserTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListMyUserTasks;
        response?: EVA.Core.ListMyUserTasksResponse;
    }
    /**
     * List OpeningHoursTemplates
     *
     * @export
     * @class ListOpeningHoursTemplates
     * @implements {EvaService}
     */
    class ListOpeningHoursTemplates implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOpeningHoursTemplates;
        response?: EVA.Core.ListOpeningHoursTemplatesResponse;
    }
    /**
     * List all the attached OrganizationUnitSets for OpeningHoursTemplates
     *
     * @export
     * @class ListOpeningHoursTemplatesOrganizationUnitSets
     * @implements {EvaService}
     */
    class ListOpeningHoursTemplatesOrganizationUnitSets implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOpeningHoursTemplatesOrganizationUnitSets;
        response?: EVA.Core.ListOpeningHoursTemplatesOrganizationUnitSetsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListOrderCustomStatus
     * @implements {EvaService}
     */
    class ListOrderCustomStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderCustomStatus;
        response?: EVA.Core.ListOrderCustomStatusResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListOrderCustomTypes
     * @implements {EvaService}
     */
    class ListOrderCustomTypes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderCustomTypes;
        response?: EVA.Core.ListOrderCustomTypesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListOrderExportsForOrder
     * @implements {EvaService}
     */
    class ListOrderExportsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderExportsForOrder;
        response?: EVA.Core.ListOrderExportsForOrderResponse;
    }
    /**
     * Lists the OrderFulfillmentLines for a given OrderID or OrderFulfillmentID.
     *
     * @export
     * @class ListOrderFulfillmentLines
     * @implements {EvaService}
     */
    class ListOrderFulfillmentLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderFulfillmentLines;
        response?: EVA.Core.ListOrderFulfillmentLinesResponse;
    }
    /**
     * List fulfillment rejections for one order.
Filter on: OrderID, SupplierOrganizationUnitID, OrderLineID, Reason, FulfillmentMethod
     *
     * @export
     * @class ListOrderFulfillmentRejections
     * @implements {EvaService}
     */
    class ListOrderFulfillmentRejections implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderFulfillmentRejections;
        response?: EVA.Core.ListOrderFulfillmentRejectionsResponse;
    }
    /**
     * Lists OrderFulfillments, optionally filtered.
     *
     * @export
     * @class ListOrderFulfillments
     * @implements {EvaService}
     */
    class ListOrderFulfillments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderFulfillments;
        response?: EVA.Core.ListOrderFulfillmentsResponse;
    }
    /**
     * Retrieves a paged list of Order Ledgers for specified OrderID.
     *
     * @export
     * @class ListOrderLedgerForOrder
     * @implements {EvaService}
     */
    class ListOrderLedgerForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderLedgerForOrder;
        response?: EVA.Core.ListOrderLedgerForOrderResponse;
    }
    /**
     * List the OrderLines of an Order in a paginated fashion, allowing you to filter with a `PageConfig` and include product properties using `ProductProperties`.
   * Use this service when an Order contains a lot of lines, and the `GetOrder` service doesn't cut it - it will tell you it's time to switch using a property called `HasMoreLines`.
     *
     * @export
     * @class ListOrderLines
     * @implements {EvaService}
     */
    class ListOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderLines;
        response?: EVA.Core.ListOrderLinesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListOrderLinesInvoiceSummary
     * @implements {EvaService}
     */
    class ListOrderLinesInvoiceSummary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderLinesInvoiceSummary;
        response?: EVA.Core.ListOrderLinesInvoiceSummaryResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListOrderShipments
     * @implements {EvaService}
     */
    class ListOrderShipments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderShipments;
        response?: EVA.Core.ListOrderShipmentsResponse;
    }
    /**
     * Lists order verification reasons.
     *
     * @export
     * @class ListOrderVerificationReasons
     * @implements {EvaService}
     */
    class ListOrderVerificationReasons implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrderVerificationReasons;
        response?: EVA.Core.ListOrderVerificationReasonsResponse;
    }
    /**
     * List orders for specific customer.
     *
     * @export
     * @class ListOrdersForCustomer
     * @implements {EvaService}
     */
    class ListOrdersForCustomer implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrdersForCustomer;
        response?: EVA.Core.ListOrdersForCustomerResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListOrdersWithCustomerReferences
     * @implements {EvaService}
     */
    class ListOrdersWithCustomerReferences implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrdersWithCustomerReferences;
        response?: EVA.Core.ListOrdersWithCustomerReferencesResponse;
    }
    /**
     * Returns summaries of a list of organizationunits. Since this service is publicly available, this will return minimal information (aka a summary) about the OUs.
     *
     * @export
     * @class ListOrganizationUnitSummaries
     * @implements {EvaService}
     */
    class ListOrganizationUnitSummaries implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrganizationUnitSummaries;
        response?: EVA.Core.ListOrganizationUnitSummariesResponse;
    }
    /**
     * Service for retrieving a list of `OrganizationUnitSupplier`
     *
     * @export
     * @class ListOrganizationUnitSuppliers
     * @implements {EvaService}
     */
    class ListOrganizationUnitSuppliers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrganizationUnitSuppliers;
        response?: EVA.Core.ListOrganizationUnitSuppliersResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListOrganizationUnits
     * @implements {EvaService}
     */
    class ListOrganizationUnits implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrganizationUnits;
        response?: EVA.Core.ListOrganizationUnitsResponse;
    }
    /**
     * Returns all data of a list of organizationunits. Since this service is not publicly available, this will return all information about the OUs.
     *
     * @export
     * @class ListOrganizationUnitsDetailed
     * @implements {EvaService}
     */
    class ListOrganizationUnitsDetailed implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListOrganizationUnitsDetailed;
        response?: EVA.Core.ListOrganizationUnitsDetailedResponse;
    }
    /**
     * List PaymentTransactionLedger

All available filters are typed in the PageConfig.Filter property.
     *
     * @export
     * @class ListPaymentTransactionLedger
     * @implements {EvaService}
     */
    class ListPaymentTransactionLedger implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListPaymentTransactionLedger;
        response?: EVA.Core.ListPaymentTransactionLedgerResponse;
    }
    /**
     * List the available PrintPriceLabel tasks for the current user.
     *
     * @export
     * @class ListPrintPriceLabelTasks
     * @implements {EvaService}
     */
    class ListPrintPriceLabelTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListPrintPriceLabelTasks;
        response?: EVA.Core.ListPrintPriceLabelTasksResponse;
    }
    /**
     * List the available Print tasks for the current user.
     *
     * @export
     * @class ListPrintTasks
     * @implements {EvaService}
     */
    class ListPrintTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListPrintTasks;
        response?: EVA.Core.ListPrintTasksResponse;
    }
    /**
     * This service returns a paged summary of `ProductBundles`, for each bundle it contains the `BundleProduct` and the pricing information for the bundle.
     *
     * @export
     * @class ListProductBundles
     * @implements {EvaService}
     */
    class ListProductBundles implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListProductBundles;
        response?: EVA.Core.ListProductBundlesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListProductPrices
     * @implements {EvaService}
     */
    class ListProductPrices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListProductPrices;
        response?: EVA.Core.ListProductPricesResponse;
    }
    /**
     * List product structures.
     *
     * @export
     * @class ListProductStructures
     * @implements {EvaService}
     */
    class ListProductStructures implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListProductStructures;
        response?: EVA.Core.ListProductStructuresResponse;
    }
    /**
     * Gets all the ProductZoneGroups based on the given filter and returns a paged response.
     *
     * @export
     * @class ListProductZoneGroups
     * @implements {EvaService}
     */
    class ListProductZoneGroups implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListProductZoneGroups;
        response?: EVA.Core.ListProductZoneGroupsResponse;
    }
    /**
     * Gets all the ProductZones based on the given filter and returns a paged response.
     *
     * @export
     * @class ListProductZones
     * @implements {EvaService}
     */
    class ListProductZones implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListProductZones;
        response?: EVA.Core.ListProductZonesResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListPurchaseOrderShipments
     * @implements {EvaService}
     */
    class ListPurchaseOrderShipments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListPurchaseOrderShipments;
        response?: EVA.Core.ListPurchaseOrderShipmentsResponse;
    }
    /**
     * List the available ReceiveShipment tasks for the current user.
     *
     * @export
     * @class ListReceiveShipmentTasks
     * @implements {EvaService}
     */
    class ListReceiveShipmentTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListReceiveShipmentTasks;
        response?: EVA.Core.ListReceiveShipmentTasksResponse;
    }
    /**
     * Lists all tasks that are scheduled recurringly.
     *
     * @export
     * @class ListRecurringScheduledTasks
     * @implements {EvaService}
     */
    class ListRecurringScheduledTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListRecurringScheduledTasks;
        response?: EVA.Core.ListRecurringScheduledTasksResponse;
    }
    /**
     * Lists all tasks that are configured to run recurringly.
     *
     * @export
     * @class ListRecurringTasks
     * @implements {EvaService}
     */
    class ListRecurringTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListRecurringTasks;
        response?: EVA.Core.ListRecurringTasksResponse;
    }
    /**
     * Lists all tasks that are configured to run recurringly.
     *
     * @export
     * @class ListRecurringTasks2
     * @implements {EvaService}
     */
    class ListRecurringTasks2 implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListRecurringTasks2;
        response?: EVA.Core.ListRecurringTasks2Response;
    }
    /**
     * List the repair interactions
     *
     * @export
     * @class ListRepairInteractions
     * @implements {EvaService}
     */
    class ListRepairInteractions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListRepairInteractions;
        response?: EVA.Core.ListRepairInteractionsResponse;
    }
    /**
     * List the available Repair tasks for the current user.
     *
     * @export
     * @class ListRepairTasks
     * @implements {EvaService}
     */
    class ListRepairTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListRepairTasks;
        response?: EVA.Core.ListRepairTasksResponse;
    }
    /**
     * List Repairs, optionally filtered.
     *
     * @export
     * @class ListRepairs
     * @implements {EvaService}
     */
    class ListRepairs implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListRepairs;
        response?: EVA.Core.ListRepairsResponse;
    }
    /**
     * List the available ReservationCleanup tasks for the current user.
     *
     * @export
     * @class ListReservationCleanupTasks
     * @implements {EvaService}
     */
    class ListReservationCleanupTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListReservationCleanupTasks;
        response?: EVA.Core.ListReservationCleanupTasksResponse;
    }
    /**
     * List the available ReservationDeviation tasks for the current user.
     *
     * @export
     * @class ListReservationDeviationTasks
     * @implements {EvaService}
     */
    class ListReservationDeviationTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListReservationDeviationTasks;
        response?: EVA.Core.ListReservationDeviationTasksResponse;
    }
    /**
     * List the available ReservationPickup tasks for the current user.
     *
     * @export
     * @class ListReservationPickupTasks
     * @implements {EvaService}
     */
    class ListReservationPickupTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListReservationPickupTasks;
        response?: EVA.Core.ListReservationPickupTasksResponse;
    }
    /**
     * List ReturnToSupplierRequests

Available filters:
- Name
- Status
     *
     * @export
     * @class ListReturnToSupplierRequests
     * @implements {EvaService}
     */
    class ListReturnToSupplierRequests implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListReturnToSupplierRequests;
        response?: EVA.Core.ListReturnToSupplierRequestsResponse;
    }
    /**
     * List available ReturnToSupplier order lines with quantities.
     *
     * @export
     * @class ListReturnToSupplierTaskOrderLines
     * @implements {EvaService}
     */
    class ListReturnToSupplierTaskOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListReturnToSupplierTaskOrderLines;
        response?: EVA.Core.ListReturnToSupplierTaskOrderLinesResponse;
    }
    /**
     * List lines for the tasks shipment with quantities.
     *
     * @export
     * @class ListReturnToSupplierTaskShipmentLines
     * @implements {EvaService}
     */
    class ListReturnToSupplierTaskShipmentLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListReturnToSupplierTaskShipmentLines;
        response?: EVA.Core.ListReturnToSupplierTaskShipmentLinesResponse;
    }
    /**
     * List the available ReturnToSupplier tasks for the current user.
     *
     * @export
     * @class ListReturnToSupplierTasks
     * @implements {EvaService}
     */
    class ListReturnToSupplierTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListReturnToSupplierTasks;
        response?: EVA.Core.ListReturnToSupplierTasksResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListReturnableSuppliersForOrder
     * @implements {EvaService}
     */
    class ListReturnableSuppliersForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListReturnableSuppliersForOrder;
        response?: EVA.Core.ListReturnableSuppliersForOrderResponse;
    }
    /**
     * Returns a list of revisions for a specific script.
     *
     * @export
     * @class ListScriptHistory
     * @implements {EvaService}
     */
    class ListScriptHistory implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListScriptHistory;
        response?: EVA.Core.ListScriptHistoryResponse;
    }
    /**
     * Lists scripts, optionally filtered.
     *
     * @export
     * @class ListScripts
     * @implements {EvaService}
     */
    class ListScripts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListScripts;
        response?: EVA.Core.ListScriptsResponse;
    }
    /**
     * List the available SecondChanceProductRegistration tasks for the current user.
     *
     * @export
     * @class ListSecondChanceProductRegistrationTasks
     * @implements {EvaService}
     */
    class ListSecondChanceProductRegistrationTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListSecondChanceProductRegistrationTasks;
        response?: EVA.Core.ListSecondChanceProductRegistrationTasksResponse;
    }
    /**
     * Lists all products associated with a Serial Number Configuration.
     *
     * @export
     * @class ListSerialNumberProducts
     * @implements {EvaService}
     */
    class ListSerialNumberProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListSerialNumberProducts;
        response?: EVA.Core.ListSerialNumberProductsResponse;
    }
    /**
     * List serial number registrations for a serial number.
     *
     * @export
     * @class ListSerialNumberRegistrations
     * @implements {EvaService}
     */
    class ListSerialNumberRegistrations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListSerialNumberRegistrations;
        response?: EVA.Core.ListSerialNumberRegistrationsResponse;
    }
    /**
     * Lists Serial Numbers
     *
     * @export
     * @class ListSerialNumbers
     * @implements {EvaService}
     */
    class ListSerialNumbers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListSerialNumbers;
        response?: EVA.Core.ListSerialNumbersResponse;
    }
    /**
     * List the available ShipFromStore tasks for the current user.
     *
     * @export
     * @class ListShipFromStoreTasks
     * @implements {EvaService}
     */
    class ListShipFromStoreTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShipFromStoreTasks;
        response?: EVA.Core.ListShipFromStoreTasksResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListShipmentExportsForShipment
     * @implements {EvaService}
     */
    class ListShipmentExportsForShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShipmentExportsForShipment;
        response?: EVA.Core.ListShipmentExportsForShipmentResponse;
    }
    /**
     * List shipment identifiers.
     *
     * @export
     * @class ListShipmentIdentifiers
     * @implements {EvaService}
     */
    class ListShipmentIdentifiers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShipmentIdentifiers;
        response?: EVA.Core.ListShipmentIdentifiersResponse;
    }
    /**
     * Retrieves a paged list of Shipment Ledgers for specified ShipmentID.
     *
     * @export
     * @class ListShipmentLedgerForShipment
     * @implements {EvaService}
     */
    class ListShipmentLedgerForShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShipmentLedgerForShipment;
        response?: EVA.Core.ListShipmentLedgerForShipmentResponse;
    }
    /**
     * Returns serial numbers on a shipment line with pagination and filtering capabilities.
     *
     * @export
     * @class ListShipmentLineSerialNumbers
     * @implements {EvaService}
     */
    class ListShipmentLineSerialNumbers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShipmentLineSerialNumbers;
        response?: EVA.Core.ListShipmentLineSerialNumbersResponse;
    }
    /**
     * List shipmentlines of a shipment, this service replaces the GetShipmentLines and the ListPurchaseOrderShipmentLines services.

The page size `Limit` has a maximum value of `10.000` for this service.
     *
     * @export
     * @class ListShipmentLines
     * @implements {EvaService}
     */
    class ListShipmentLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShipmentLines;
        response?: EVA.Core.ListShipmentLinesResponse;
    }
    /**
     * List all of the lines of a shipment that have not been completely invoiced.
     *
     * @export
     * @class ListShipmentLinesToInvoice
     * @implements {EvaService}
     */
    class ListShipmentLinesToInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShipmentLinesToInvoice;
        response?: EVA.Core.ListShipmentLinesToInvoiceResponse;
    }
    /**
     * Lists shipment packages for a specified shipment
     *
     * @export
     * @class ListShipmentPackages
     * @implements {EvaService}
     */
    class ListShipmentPackages implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShipmentPackages;
        response?: EVA.Core.ListShipmentPackagesResponse;
    }
    /**
     * List all of the shipments that have not been completely invoiced.
     *
     * @export
     * @class ListShipmentsToInvoice
     * @implements {EvaService}
     */
    class ListShipmentsToInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShipmentsToInvoice;
        response?: EVA.Core.ListShipmentsToInvoiceResponse;
    }
    /**
     * Service that returns OrganizationUnitDtos that are approximately in range of the given latitude and longitude. The result are returned in ascending order.

This service is useful for displaying a list of the nearest shops (eg: checkout and choosing a pick-up shop).

The page size `Limit` has a maximum value of `100` for this service.
     *
     * @export
     * @class ListShopsByProximity
     * @implements {EvaService}
     */
    class ListShopsByProximity implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShopsByProximity;
        response?: EVA.Core.ListShopsByProximityResponse;
    }
    /**
     * List the Shop OrganizationUnits in a certain area

Included are the OpeningHours in the next 7 days, and the Regular OpeningHours.

This service is useful for showing all shops in a map, where you want to search in a given rectangle.
     *
     * @export
     * @class ListShopsInArea
     * @implements {EvaService}
     */
    class ListShopsInArea implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListShopsInArea;
        response?: EVA.Core.ListShopsInAreaResponse;
    }
    /**
     * List stations

This service is meant for management purposes, will return all stations you're allowed to view.
     *
     * @export
     * @class ListStations
     * @implements {EvaService}
     */
    class ListStations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListStations;
        response?: EVA.Core.ListStationsResponse;
    }
    /**
     * List stations for specified organization unit.
     *
     * @export
     * @class ListStationsForOrganizationUnit
     * @implements {EvaService}
     */
    class ListStationsForOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListStationsForOrganizationUnit;
        response?: EVA.Core.ListStationsForOrganizationUnitResponse;
    }
    /**
     * Returns a paginated list of all stock, optionally filtered.
     *
     * @export
     * @class ListStock
     * @implements {EvaService}
     */
    class ListStock implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListStock;
        response?: EVA.Core.ListStockResponse;
    }
    /**
     * List the available StockMovementFollowUp tasks for the current user.
     *
     * @export
     * @class ListStockMovementFollowUpTasks
     * @implements {EvaService}
     */
    class ListStockMovementFollowUpTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListStockMovementFollowUpTasks;
        response?: EVA.Core.ListStockMovementFollowUpTasksResponse;
    }
    /**
     * The page size `Limit` has a maximum value of `1.000` for this service.
     *
     * @export
     * @class ListStockMutations
     * @implements {EvaService}
     */
    class ListStockMutations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListStockMutations;
        response?: EVA.Core.ListStockMutationsResponse;
    }
    /**
     * Lists the additional Stock Replenishment task data.
     *
     * @export
     * @class ListStockReplenishmentTaskData
     * @implements {EvaService}
     */
    class ListStockReplenishmentTaskData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListStockReplenishmentTaskData;
        response?: EVA.Core.ListStockReplenishmentTaskDataResponse;
    }
    /**
     * List the available StockReplenishment tasks for the current user.
     *
     * @export
     * @class ListStockReplenishmentTasks
     * @implements {EvaService}
     */
    class ListStockReplenishmentTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListStockReplenishmentTasks;
        response?: EVA.Core.ListStockReplenishmentTasksResponse;
    }
    /**
     * List the available StockReservation tasks for the current user.
     *
     * @export
     * @class ListStockReservationTasks
     * @implements {EvaService}
     */
    class ListStockReservationTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListStockReservationTasks;
        response?: EVA.Core.ListStockReservationTasksResponse;
    }
    /**
     * Service for retrieving a list of `StringTranslation`
     *
     * @export
     * @class ListStringTranslations
     * @implements {EvaService}
     */
    class ListStringTranslations implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListStringTranslations;
        response?: EVA.Core.ListStringTranslationsResponse;
    }
    /**
     * To continue with a suspended order, use the `ResumeSuspendedOrder` service.
     *
     * @export
     * @class ListSuspendedOrders
     * @implements {EvaService}
     */
    class ListSuspendedOrders implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListSuspendedOrders;
        response?: EVA.Core.ListSuspendedOrdersResponse;
    }
    /**
     * Returns the current tasks that prevent the current organization units' financial period from closing.
     *
     * @export
     * @class ListTasksThatBlockPeriodClosing
     * @implements {EvaService}
     */
    class ListTasksThatBlockPeriodClosing implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListTasksThatBlockPeriodClosing;
        response?: EVA.Core.ListTasksThatBlockPeriodClosingResponse;
    }
    /**
     * Lists tax code mappings.
     *
     * @export
     * @class ListTaxCodeMappings
     * @implements {EvaService}
     */
    class ListTaxCodeMappings implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListTaxCodeMappings;
        response?: EVA.Core.ListTaxCodeMappingsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListTaxCodes
     * @implements {EvaService}
     */
    class ListTaxCodes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListTaxCodes;
        response?: EVA.Core.ListTaxCodesResponse;
    }
    /**
     * Lists tax providers.
     *
     * @export
     * @class ListTaxProviders
     * @implements {EvaService}
     */
    class ListTaxProviders implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListTaxProviders;
        response?: EVA.Core.ListTaxProvidersResponse;
    }
    /**
     * Lists all tax rates.

Supports the filter `OnlyActive`, and when specified, will filter out all tax rates that have a `StartDate` or `EndDate` out of bounds for
'today'. The actual value of the filter is irrelevant.

:point_right: See `CreateTaxRate` service for more info.
     *
     * @export
     * @class ListTaxRates
     * @implements {EvaService}
     */
    class ListTaxRates implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListTaxRates;
        response?: EVA.Core.ListTaxRatesResponse;
    }
    /**
     * Service for listing transput jobs
     *
     * @export
     * @class ListTransputJobs
     * @implements {EvaService}
     */
    class ListTransputJobs implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListTransputJobs;
        response?: EVA.Core.ListTransputJobsResponse;
    }
    /**
     * List the UserTasks for the current user.
     *
     * @export
     * @class ListUrgentTasks
     * @implements {EvaService}
     */
    class ListUrgentTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUrgentTasks;
        response?: EVA.Core.ListUrgentTasksResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListUserApiKeys
     * @implements {EvaService}
     */
    class ListUserApiKeys implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUserApiKeys;
        response?: EVA.Core.ListUserApiKeysResponse;
    }
    /**
     * List borrowed products based on the given filter
     *
     * @export
     * @class ListUserBorrowedProducts
     * @implements {EvaService}
     */
    class ListUserBorrowedProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUserBorrowedProducts;
        response?: EVA.Core.ListUserBorrowedProductsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListUserGroups
     * @implements {EvaService}
     */
    class ListUserGroups implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUserGroups;
        response?: EVA.Core.ListUserGroupsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListUserInteractions
     * @implements {EvaService}
     */
    class ListUserInteractions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUserInteractions;
        response?: EVA.Core.ListUserInteractionsResponse;
    }
    /**
     * List the user loyalty transactions.
     *
     * @export
     * @class ListUserLoyaltyTransactions
     * @implements {EvaService}
     */
    class ListUserLoyaltyTransactions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUserLoyaltyTransactions;
        response?: EVA.Core.ListUserLoyaltyTransactionsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListUserTaskTypeOrganizationUnitSets
     * @implements {EvaService}
     */
    class ListUserTaskTypeOrganizationUnitSets implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUserTaskTypeOrganizationUnitSets;
        response?: EVA.Core.ListUserTaskTypeOrganizationUnitSetsResponse;
    }
    /**
     * Returns a paged list of UserTasks.
     *
     * @export
     * @class ListUserTasks
     * @implements {EvaService}
     */
    class ListUserTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUserTasks;
        response?: EVA.Core.ListUserTasksResponse;
    }
    /**
     * List the users wishlists.
     *
     * @export
     * @class ListUserWishlists
     * @implements {EvaService}
     */
    class ListUserWishlists implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUserWishlists;
        response?: EVA.Core.ListUserWishlistsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ListUsersForUserGroup
     * @implements {EvaService}
     */
    class ListUsersForUserGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListUsersForUserGroup;
        response?: EVA.Core.ListUsersForUserGroupResponse;
    }
    /**
     * Lists results associated with a Zoned Cycle Count PreCount
     *
     * @export
     * @class ListZonedCycleCountPreCountResult
     * @implements {EvaService}
     */
    class ListZonedCycleCountPreCountResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListZonedCycleCountPreCountResult;
        response?: EVA.Core.ListZonedCycleCountPreCountResultResponse;
    }
    /**
     * List the available ZonedCycleCountPreCount tasks for the current user.
     *
     * @export
     * @class ListZonedCycleCountPreCountTasks
     * @implements {EvaService}
     */
    class ListZonedCycleCountPreCountTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListZonedCycleCountPreCountTasks;
        response?: EVA.Core.ListZonedCycleCountPreCountTasksResponse;
    }
    /**
     * Lists the Zoned Cycle Count Schedules.
     *
     * @export
     * @class ListZonedCycleCountSchedules
     * @implements {EvaService}
     */
    class ListZonedCycleCountSchedules implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListZonedCycleCountSchedules;
        response?: EVA.Core.ListZonedCycleCountSchedulesResponse;
    }
    /**
     * Retrieves the status of serial numbers for a specific product and stock label within a Zoned Cycle Count.
     *
     * @export
     * @class ListZonedCycleCountSerialNumbersStatus
     * @implements {EvaService}
     */
    class ListZonedCycleCountSerialNumbersStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListZonedCycleCountSerialNumbersStatus;
        response?: EVA.Core.ListZonedCycleCountSerialNumbersStatusResponse;
    }
    /**
     * List the available ZonedCycleCount tasks for the current user.
     *
     * @export
     * @class ListZonedCycleCountTasks
     * @implements {EvaService}
     */
    class ListZonedCycleCountTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListZonedCycleCountTasks;
        response?: EVA.Core.ListZonedCycleCountTasksResponse;
    }
    /**
     * The page size `Limit` has a maximum value of `250` for this service.
     *
     * @export
     * @class ListZonedCycleCounts
     * @implements {EvaService}
     */
    class ListZonedCycleCounts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ListZonedCycleCounts;
        response?: EVA.Core.ListZonedCycleCountsResponse;
    }
    /**
     * Lock the Order in order to prevent some editing functionalities on it.
     *
     * @export
     * @class LockOrder
     * @implements {EvaService}
     */
    class LockOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.LockOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Authenticates a user
     *
     * @export
     * @class Login
     * @implements {EvaService}
     */
    class Login implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.Login;
        response?: EVA.Core.LoginResponse;
    }
    /**
     * Ends the current session of the user and invalidates the used AuthenticationToken

Optionally the ExpireAllSessions property can be set to terminate all current sessions for the User.

The Clockout property can be used the register a clock out event for time tracking.
     *
     * @export
     * @class Logout
     * @implements {EvaService}
     */
    class Logout implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.Logout;
        response?: EVA.Core.LogoutResponse;
    }
    /**
     * This service marks the provided phone number as the user's primary phone number,
which will be used for communication with the user.

This service only allows the number's own user or an employee to mark it as the primary number.
     *
     * @export
     * @class MakeUserPhoneNumberPrimary
     * @implements {EvaService}
     */
    class MakeUserPhoneNumberPrimary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.MakeUserPhoneNumberPrimary;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class MigrateOrdersToUnified
     * @implements {EvaService}
     */
    class MigrateOrdersToUnified implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.MigrateOrdersToUnified;
        response?: EVA.Core.MigrateOrdersToUnifiedResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class MissingProductForInitialCycleCount
     * @implements {EvaService}
     */
    class MissingProductForInitialCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.MissingProductForInitialCycleCount;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ModifyOrderLinePrice
     * @implements {EvaService}
     */
    class ModifyOrderLinePrice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ModifyOrderLinePrice;
        response?: EVA.Core.SimpleShoppingCartResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ModifyPurchaseOrderLine
     * @implements {EvaService}
     */
    class ModifyPurchaseOrderLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ModifyPurchaseOrderLine;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ModifyQuantityOrdered
     * @implements {EvaService}
     */
    class ModifyQuantityOrdered implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ModifyQuantityOrdered;
        response?: EVA.Core.ModifyQuantityOrderedResponse;
    }
    /**
     * Modify the QuantityShipped on one or more ShipmentLines of a Shipment.

When the new QuantityShipped is 0 it will be cancelled.
     *
     * @export
     * @class ModifyQuantityShipped
     * @implements {EvaService}
     */
    class ModifyQuantityShipped implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ModifyQuantityShipped;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Moves cash!
     *
     * @export
     * @class MoveCash
     * @implements {EvaService}
     */
    class MoveCash implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.MoveCash;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service allows you to move stock commitments from one OrderLine (`SourceOrderLineID`) to another OrderLine (`DestinationOrderLineID`).

- If the Source and Destination do not share the same ProductID, a `CannotMoveCommitment` error is returned.
- If the Source does not have a commitment, a `CannotMoveCommitment` error is returned.
     *
     * @export
     * @class MoveCommitments
     * @implements {EvaService}
     */
    class MoveCommitments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.MoveCommitments;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Books a set of Movements.

A Movement should have a `ProductID` to identify the product, an `OrganizationUnitID` to identify the stock keeping
organization, a `SourceStockLabelID` and `DestinationStockLabelID` (see `GetEnumValues` with type `StockLabel`) and (of course) a `Quantity`
to actually book.

Supply a `BackendSystemID` on the request and `BackendID`'s on the Movements to make the request idempotent.

Optionally supports a `StockMutationReasonID` and `Remark` for each Movement.
     *
     * @export
     * @class MoveStock
     * @implements {EvaService}
     */
    class MoveStock implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.MoveStock;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Opens a financial period for your current OrganizationUnit.
     *
     * @export
     * @class OpenFinancialPeriod
     * @implements {EvaService}
     */
    class OpenFinancialPeriod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.OpenFinancialPeriod;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Parses the given barcode with an available handler that can process it.
     *
     * @export
     * @class ParseBarcode
     * @implements {EvaService}
     */
    class ParseBarcode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ParseBarcode;
        response?: EVA.Core.ParseBarcodeResponse;
    }
    /**
     * Parses a script for validation and auto-completion.

Can be used as the user types their script, for that purpose use these parameters:
- Type: Fast, doing Full performs a full compilation of the script which can take longer. Using Full can be done before the user hits the Save button, or for example every 10th keystroke or so.

Use IncludeCompletionSuggestions if you want to display auto-completion suggestions to the user.

You can verify the entire script and get completion results by passing in the full script as the Source and only the text until the user's cursor in the CompletionSource.

A list of the available dialects can be retrieved by calling GetScriptDialects.
     *
     * @export
     * @class ParseScript
     * @implements {EvaService}
     */
    class ParseScript implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ParseScript;
        response?: EVA.Core.ParseScriptResponse;
    }
    /**
     * Pauses a ConsolidatedPickTask task.
     *
     * @export
     * @class PauseConsolidatedPickTask
     * @implements {EvaService}
     */
    class PauseConsolidatedPickTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PauseConsolidatedPickTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Stub service to validate the current token for employees
     *
     * @export
     * @class Ping
     * @implements {EvaService}
     */
    class Ping implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.Ping;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Placing an order means it is final for all parties; both agree on the exchange in goods, services and/or money that this order implies.

Will set the `Order`.`IsPlaced` flag.

This service can be used to place a single order, or to place a batch at once.
     *
     * @export
     * @class PlaceOrder
     * @implements {EvaService}
     */
    class PlaceOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PlaceOrder;
        response?: EVA.Core.PlaceOrderResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class PollExternalOrderStatus
     * @implements {EvaService}
     */
    class PollExternalOrderStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PollExternalOrderStatus;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class PreCountInitialCycleCountLabel
     * @implements {EvaService}
     */
    class PreCountInitialCycleCountLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PreCountInitialCycleCountLabel;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * List the discounts that potentially could be activated.

A order or user can be given to look at the context of the order or user. When order is given the user will be ignored and the customer of the order will be used (if present)

If a `ProductIDs`, `ProductSearchID` or `ProductSearch` is set, the service will check if the resulting products are a eligible to one or more discounts. If an `OrderID` is provided, the context of the given order is taken into account, that can include user specific discounts, order OU specific discount, etc.

Be aware that:
- If `ProductIDs` is given, `ProductSearchID` and `ProductSearch` are ignored
- If `ProductSearchID` is given, `ProductIDs` and `ProductSearch` are ignored
- If `ProductSearch` is given, `ProductIDs` and `ProductSearchID` are ignored
     *
     * @export
     * @class PrefigureDiscounts
     * @implements {EvaService}
     */
    class PrefigureDiscounts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrefigureDiscounts;
        response?: EVA.Core.PrefigureDiscountsResponse;
    }
    /**
     * Prepare the order to go into the checkout

If the order still has status ShoppingCart, update it to Order and:
 - Validate lineactiontypes (customer are not allowed to use ShipLine p/a)
 - Set the SoldBy on the lines to the current user
 - Set the addresses of the customer on the order, if not already set
 - (Re)calculate the taxes
 - Recalculate the order: discounts, shippingcosts, giftwrappingcosts etc.
     *
     * @export
     * @class PrepareOrderForCheckout
     * @implements {EvaService}
     */
    class PrepareOrderForCheckout implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrepareOrderForCheckout;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Render a UserInteraction of Type Communication in something that can be displayed.

This can be used to view the send emails in a webview/iframe.
     *
     * @export
     * @class PreviewUserInteractionCommunication
     * @implements {EvaService}
     */
    class PreviewUserInteractionCommunication implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PreviewUserInteractionCommunication;
        response?: EVA.Core.PreviewUserInteractionCommunicationResponse;
    }
    /**
     * Pushes a blob (by `BlobID` or `BlobUrl`) to a device of specified `Type` on given `StationID`.
     *
     * @export
     * @class Print
     * @implements {EvaService}
     */
    class Print implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.Print;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This is service is used only for printing couopons.
     *
     * @export
     * @class PrintCouponsForOrder
     * @implements {EvaService}
     */
    class PrintCouponsForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintCouponsForOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Prints label(s) for specified Full Stock Count to specified station.
     *
     * @export
     * @class PrintFullStockCountLabels
     * @implements {EvaService}
     */
    class PrintFullStockCountLabels implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintFullStockCountLabels;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Print the InterbranchOrder receipt.
     *
     * @export
     * @class PrintInterbranchOrderReceipt
     * @implements {EvaService}
     */
    class PrintInterbranchOrderReceipt implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintInterbranchOrderReceipt;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used only for printing paper invoices.
     *
     * @export
     * @class PrintPaperInvoice
     * @implements {EvaService}
     */
    class PrintPaperInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintPaperInvoice;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This is service is used for printing sales paper invoices for one order.
     *
     * @export
     * @class PrintPaperInvoices
     * @implements {EvaService}
     */
    class PrintPaperInvoices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintPaperInvoices;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used only for printing offers.
     *
     * @export
     * @class PrintPaperOffer
     * @implements {EvaService}
     */
    class PrintPaperOffer implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintPaperOffer;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used only for printing paper invoices.
     *
     * @export
     * @class PrintPaperOrderConfirmation
     * @implements {EvaService}
     */
    class PrintPaperOrderConfirmation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintPaperOrderConfirmation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class PrintPriceLabel
     * @implements {EvaService}
     */
    class PrintPriceLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintPriceLabel;
        response?: EVA.Core.PrintPriceLabelResponse;
    }
    /**
     * This service is used to print labels containing the price details of one product
     *
     * @export
     * @class PrintProductPriceLabel
     * @implements {EvaService}
     */
    class PrintProductPriceLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintProductPriceLabel;
        response?: EVA.Core.PrintProductPriceLabelResponse;
    }
    /**
     * Print repair thermal receipt
     *
     * @export
     * @class PrintRepairThermalReceipt
     * @implements {EvaService}
     */
    class PrintRepairThermalReceipt implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintRepairThermalReceipt;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Print the return to supplier order label.
     *
     * @export
     * @class PrintReturnToSupplierOrderTicket
     * @implements {EvaService}
     */
    class PrintReturnToSupplierOrderTicket implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintReturnToSupplierOrderTicket;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Print the return to supplier order receipt.
     *
     * @export
     * @class PrintReturnToSupplierReceipt
     * @implements {EvaService}
     */
    class PrintReturnToSupplierReceipt implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintReturnToSupplierReceipt;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Print the tasks shipment label.
     *
     * @export
     * @class PrintReturnToSupplierTaskShipment
     * @implements {EvaService}
     */
    class PrintReturnToSupplierTaskShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintReturnToSupplierTaskShipment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class PrintSecondChanceProductPriceLabel
     * @implements {EvaService}
     */
    class PrintSecondChanceProductPriceLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintSecondChanceProductPriceLabel;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Generates a combined pick list for given Ship From Store tasks, starting and assigning them all to the current user.

These tasks need to be ShipFromStore type with subtype Pick, or a CompositeUserTask type with such a task as first child.
     *
     * @export
     * @class PrintShipFromStoreTasks
     * @implements {EvaService}
     */
    class PrintShipFromStoreTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintShipFromStoreTasks;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * Print the Reservation receipt for the given Order
     *
     * @export
     * @class PrintStockReservationReceipt
     * @implements {EvaService}
     */
    class PrintStockReservationReceipt implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintStockReservationReceipt;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Print a *gift* receipt; a regular receipt, but without any price- or payment details. This document is intended to accompany the purchase when gifted, so the recipient can return or exchange it without (direct) knowledge of the price.
    The `Lines` property is optional, when omitted the entire order (from `OrderID`) will be printed. Also, `Name` and `Remark` are optional and intended for the gift recipient, only.
     *
     * @export
     * @class PrintThermalGiftReceipt
     * @implements {EvaService}
     */
    class PrintThermalGiftReceipt implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintThermalGiftReceipt;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This is service is used only for printing thermal invoices.
     *
     * @export
     * @class PrintThermalInvoice
     * @implements {EvaService}
     */
    class PrintThermalInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintThermalInvoice;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This is service is used for printing sales thermal invoices for one order.
     *
     * @export
     * @class PrintThermalInvoices
     * @implements {EvaService}
     */
    class PrintThermalInvoices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintThermalInvoices;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class PrintThermalOrderConfirmation
     * @implements {EvaService}
     */
    class PrintThermalOrderConfirmation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintThermalOrderConfirmation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Print an order summary as a thermal non-fiscal receipt.
It can be used when a payment is refunded or canceled during the sale process, the cashier needs to hand a proof to the customer with the current state of the order.
     *
     * @export
     * @class PrintThermalOrderSummary
     * @implements {EvaService}
     */
    class PrintThermalOrderSummary implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PrintThermalOrderSummary;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ProcessInitialCycleCountResults
     * @implements {EvaService}
     */
    class ProcessInitialCycleCountResults implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProcessInitialCycleCountResults;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Processes the print tasks for each required ID.

This service does not throws an expeption in the case of mulptiple user task ids, but the output will be a dictionary containing
an Error (EVA.Framework.Error) for every print task that encounters an issue. The UserTaskID will be the key in that specific Dictionary.

In the case of a single user task id, the service will behave exactly as the depracated ProcessPrintTask service did, throwing the encountered exception.
     *
     * @export
     * @class ProcessPrintTasks
     * @implements {EvaService}
     */
    class ProcessPrintTasks implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProcessPrintTasks;
        response?: EVA.Core.ProcessPrintTasksResponse;
    }
    /**
     * Start the processing of a ReturnToSupplierRequest

This will put the request in a processing state and will generate the orders based on the set options.
     *
     * @export
     * @class ProcessReturnToSupplierRequest
     * @implements {EvaService}
     */
    class ProcessReturnToSupplierRequest implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProcessReturnToSupplierRequest;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Processes a (delta)stock mutation file.

Expected format of the file is JSON with the following markup:

```
[
  {
    "ProductID": [Optional, int],
    "ProductBackendID": [Optional, string],
    "QuantityOnHand": [Required, int, value >= 0],
    "StockMutationReasonID": [Optional, int],
    "Remark": [Optional, string]
  },
  ...
]
```

Mutation will only be processed for valid ProductIDs or ProductBackendIDs. In case ProductID and ProductBackendID are provided,
ProductBackendID will be selected as the search parameter.
     *
     * @export
     * @class ProcessStockMutationFile
     * @implements {EvaService}
     */
    class ProcessStockMutationFile implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProcessStockMutationFile;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Process unprocessed financial events.
     *
     * @export
     * @class ProcessUnprocessedFinancialEvents
     * @implements {EvaService}
     */
    class ProcessUnprocessedFinancialEvents implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProcessUnprocessedFinancialEvents;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for producing a cash deposit document
     *
     * @export
     * @class ProduceCashDepositDocument
     * @implements {EvaService}
     */
    class ProduceCashDepositDocument implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProduceCashDepositDocument;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for producing a cash expense document
     *
     * @export
     * @class ProduceCashExpenseDocument
     * @implements {EvaService}
     */
    class ProduceCashExpenseDocument implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProduceCashExpenseDocument;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ProduceDocuments
     * @implements {EvaService}
     */
    class ProduceDocuments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProduceDocuments;
        response?: EVA.Core.ProduceDocumentsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ProduceInitialCycleCountLabels
     * @implements {EvaService}
     */
    class ProduceInitialCycleCountLabels implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProduceInitialCycleCountLabels;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ProduceInvoice
     * @implements {EvaService}
     */
    class ProduceInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProduceInvoice;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ProducePackingSlip
     * @implements {EvaService}
     */
    class ProducePackingSlip implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProducePackingSlip;
        response?: EVA.Core.ProducePackingSlipResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ProducePickSlip
     * @implements {EvaService}
     */
    class ProducePickSlip implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProducePickSlip;
        response?: EVA.Core.ProducePickSlipResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ProducePurchaseOrderDocument
     * @implements {EvaService}
     */
    class ProducePurchaseOrderDocument implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProducePurchaseOrderDocument;
        response?: EVA.Core.ResourceResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ProduceReceipt
     * @implements {EvaService}
     */
    class ProduceReceipt implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProduceReceipt;
        response?: EVA.Core.ProduceReceiptResponse;
    }
    /**
     * Generate shipment document for an order or shipment.

It is required to supply an `OrderID` or a `ShipmentID`.

When using an `OrderID` it is also required to set the `LinesToShip`.

When a `StationID` is supplied the documents will be printed, if not, the service will return urls for downloading the documents.
     *
     * @export
     * @class ProduceShipmentDocuments
     * @implements {EvaService}
     */
    class ProduceShipmentDocuments implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProduceShipmentDocuments;
        response?: EVA.Core.ProduceShipmentDocumentsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ProduceTrackingInformation
     * @implements {EvaService}
     */
    class ProduceTrackingInformation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProduceTrackingInformation;
        response?: EVA.Core.ProduceTrackingInformationResponse;
    }
    /**
     *
     *
     * @export
     * @class ProduceZonedCycleCountHandout
     * @implements {EvaService}
     */
    class ProduceZonedCycleCountHandout implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ProduceZonedCycleCountHandout;
        response?: EVA.Core.ProduceZonedCycleCountHandoutResponse;
    }
    /**
     * A service that allows to manage the entirety of a price list.
     *
     * @export
     * @class PushPriceList
     * @implements {EvaService}
     */
    class PushPriceList implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushPriceList;
        response?: EVA.Core.PushPriceListResponse;
    }
    /**
     * A service that allows to manage the entirety of a price list.
     *
     * @export
     * @class PushPriceList_Async
     * @implements {EvaService}
     */
    class PushPriceList_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushPriceList;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * A service that allows to manage the entirety of a price list.
     *
     * @export
     * @class PushPriceList_AsyncResult
     * @implements {EvaService}
     */
    class PushPriceList_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.PushPriceListResponse;
    }
    /**
     * Creates a returnorder with all specified data

It is required to send the identifier for the ReturnOrder of the external system by providing a BackendSystemID and a BackendID.

A return is based of an OriginalOrder. This order can be identifier by the OriginalBackendID or the OriginalOrderID. One of those is required.

If the stock is already received, set the `AlreadyReceived` to true. This will ship the order immediately.

If the stock is already received and the open amount should be refunded directly, set `AutoRefund` to true.

The Lines have to be identified by the original line, this should be done by providing the OriginalBackendID or the OriginalOrderLineID.
If those are not available the CustomID of the product can be set, but matching based on that is not advised.
     *
     * @export
     * @class PushReturnOrder
     * @implements {EvaService}
     */
    class PushReturnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushReturnOrder;
        response?: EVA.Core.PushReturnOrderResponse;
    }
    /**
     * Creates an order with all specified data, places it, and raises all follow-up events.

Whenever the order already exists with the same BackendSystemID/BackendID combination, nothing will be modified.
     *
     * @export
     * @class PushSalesOrder
     * @implements {EvaService}
     */
    class PushSalesOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushSalesOrder;
        response?: EVA.Core.PushSalesOrderResponse;
    }
    /**
     * Push multiple serial numbers into EVA.

- The `ProductID` is required
- You must either specify *both*, or *neither* of `StockLabelID` and `OrganizationUnitID`

:bulb: Our recommended JSON request size limit is 10 MB
:muscle: So when calling this service with just a `SerialNumber` and a `ProductID`, that should be somewhere between 100.000 and 200.000 serials per call
:warning: When a product belongs to a Serial Number Configuration that is configured with a validation regex, serial numbers pushed for that product must match this regex

This service is built for speeeeeeeeeeeed :rocket:, and so has a slightly different idempotency from other services;

- Skips over errors and attempts to persist as many serial numbers as possible
- Gathers any error encountered and returns (up to 1.000 of) them, so you can fix & retry
- Ignores serial numbers that already exist in EVA
- Ignores duplicates within the same call (but it's undefined which serial number is used)
- Ignores duplicates within multiple simultaneous calls (but it's undefined which serial number is used)
- Skips over serial number validation for second chance products
     *
     * @export
     * @class PushSerialNumbers
     * @implements {EvaService}
     */
    class PushSerialNumbers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushSerialNumbers;
        response?: EVA.Core.PushSerialNumbersResponse;
    }
    /**
     * Push multiple serial numbers into EVA.

- The `ProductID` is required
- You must either specify *both*, or *neither* of `StockLabelID` and `OrganizationUnitID`

:bulb: Our recommended JSON request size limit is 10 MB
:muscle: So when calling this service with just a `SerialNumber` and a `ProductID`, that should be somewhere between 100.000 and 200.000 serials per call
:warning: When a product belongs to a Serial Number Configuration that is configured with a validation regex, serial numbers pushed for that product must match this regex

This service is built for speeeeeeeeeeeed :rocket:, and so has a slightly different idempotency from other services;

- Skips over errors and attempts to persist as many serial numbers as possible
- Gathers any error encountered and returns (up to 1.000 of) them, so you can fix & retry
- Ignores serial numbers that already exist in EVA
- Ignores duplicates within the same call (but it's undefined which serial number is used)
- Ignores duplicates within multiple simultaneous calls (but it's undefined which serial number is used)
- Skips over serial number validation for second chance products
     *
     * @export
     * @class PushSerialNumbers_Async
     * @implements {EvaService}
     */
    class PushSerialNumbers_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushSerialNumbers;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * Push multiple serial numbers into EVA.

- The `ProductID` is required
- You must either specify *both*, or *neither* of `StockLabelID` and `OrganizationUnitID`

:bulb: Our recommended JSON request size limit is 10 MB
:muscle: So when calling this service with just a `SerialNumber` and a `ProductID`, that should be somewhere between 100.000 and 200.000 serials per call
:warning: When a product belongs to a Serial Number Configuration that is configured with a validation regex, serial numbers pushed for that product must match this regex

This service is built for speeeeeeeeeeeed :rocket:, and so has a slightly different idempotency from other services;

- Skips over errors and attempts to persist as many serial numbers as possible
- Gathers any error encountered and returns (up to 1.000 of) them, so you can fix & retry
- Ignores serial numbers that already exist in EVA
- Ignores duplicates within the same call (but it's undefined which serial number is used)
- Ignores duplicates within multiple simultaneous calls (but it's undefined which serial number is used)
- Skips over serial number validation for second chance products
     *
     * @export
     * @class PushSerialNumbers_AsyncResult
     * @implements {EvaService}
     */
    class PushSerialNumbers_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.PushSerialNumbersResponse;
    }
    /**
     * Used to create or update a user from 3rd party system.
     *
     * @export
     * @class PushUser
     * @implements {EvaService}
     */
    class PushUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushUser;
        response?: EVA.Core.PushUserResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class PushUserInteractionContent
     * @implements {EvaService}
     */
    class PushUserInteractionContent implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushUserInteractionContent;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Use this to push multiple users at once. This is more efficient than pushing them one by one. Please get in touch with your contact at New Black if you plan on pushing a lot of users. This service follows the "all-or-nothing" approach, either everything is imported, or nothing is imported. There is a limit (100) on the number of users that can be pushed at once.
     *
     * @export
     * @class PushUsers
     * @implements {EvaService}
     */
    class PushUsers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushUsers;
        response?: EVA.Core.PushUsersResponse;
    }
    /**
     * Use this to push multiple users at once. This is more efficient than pushing them one by one. Please get in touch with your contact at New Black if you plan on pushing a lot of users. This service follows the "all-or-nothing" approach, either everything is imported, or nothing is imported. There is a limit (100) on the number of users that can be pushed at once.
     *
     * @export
     * @class PushUsers_Async
     * @implements {EvaService}
     */
    class PushUsers_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.PushUsers;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * Use this to push multiple users at once. This is more efficient than pushing them one by one. Please get in touch with your contact at New Black if you plan on pushing a lot of users. This service follows the "all-or-nothing" approach, either everything is imported, or nothing is imported. There is a limit (100) on the number of users that can be pushed at once.
     *
     * @export
     * @class PushUsers_AsyncResult
     * @implements {EvaService}
     */
    class PushUsers_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.PushUsersResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class RaisePurchaseOrderImportedEvent
     * @implements {EvaService}
     */
    class RaisePurchaseOrderImportedEvent implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RaisePurchaseOrderImportedEvent;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Reactivates an User that was deactivated due to no login
     *
     * @export
     * @class ReactivateUser
     * @implements {EvaService}
     */
    class ReactivateUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReactivateUser;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Receive (part of) a Shipment for a PurchaseOrder.

A ShipmentID is required.

When doing partial receipts it is strongly advised to supply a BackendSystemID and BackendID to make sure Receipts are not created twice.

To do partial receipts the FinalShipment property should be set to false, otherwise the shipment will be close afterwards and the remaining lines will be booked as missing.

When available it is preferred to identify the ShipmentLines with the ShipmentLineID and not with the ProductID.
     *
     * @export
     * @class ReceivePurchaseOrderShipment
     * @implements {EvaService}
     */
    class ReceivePurchaseOrderShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReceivePurchaseOrderShipment;
        response?: EVA.Core.ReceivePurchaseOrderShipmentResponse;
    }
    /**
     * Receive a customer return order
     *
     * @export
     * @class ReceiveReturnOrder
     * @implements {EvaService}
     */
    class ReceiveReturnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReceiveReturnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Receive a shipment.
     *
     * @export
     * @class ReceiveShipment
     * @implements {EvaService}
     */
    class ReceiveShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReceiveShipment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Initiates a recount of the given FullStockCountLabel. FullStockCount will not be completed.
     *
     * @export
     * @class RecountFullStockCountLabel
     * @implements {EvaService}
     */
    class RecountFullStockCountLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RecountFullStockCountLabel;
        response?: EVA.Core.RecountFullStockCountLabelResponse;
    }
    /**
     * Initiates a recount of the given FullStockCountProduct for the given FullStockCount. FullStockCount will not be completed.
     *
     * @export
     * @class RecountFullStockCountProduct
     * @implements {EvaService}
     */
    class RecountFullStockCountProduct implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RecountFullStockCountProduct;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Recovers an employee account by changing the email address and optionally generating a temporary password.
  This is intended as last resort for when an employee has lost access to their emailaddress.
     *
     * @export
     * @class RecoverEmployeeAccount
     * @implements {EvaService}
     */
    class RecoverEmployeeAccount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RecoverEmployeeAccount;
        response?: EVA.Core.RecoverEmployeeAccountResponse;
    }
    /**
     * Refresh a JWT token
     *
     * @export
     * @class RefreshToken
     * @implements {EvaService}
     */
    class RefreshToken implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RefreshToken;
        response?: EVA.Core.RefreshTokenResponse;
    }
    /**
     * Triggers a refund request for the given Order.

The amount to refund is optional, if not specified it will be the amount available to refund on the order.
     *
     * @export
     * @class RefundOrder
     * @implements {EvaService}
     */
    class RefundOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RefundOrder;
        response?: EVA.Core.RefundOrderResponse;
    }
    /**
     * Register an app for push notifications. This will register the device token for the given application identifier so the app can receive PushNotifications for the current User/OrganizationUnit.
     *
     * @export
     * @class RegisterAppForNotifications
     * @implements {EvaService}
     */
    class RegisterAppForNotifications implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RegisterAppForNotifications;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Register user borrowed products
     *
     * @export
     * @class RegisterUserBorrowedProducts
     * @implements {EvaService}
     */
    class RegisterUserBorrowedProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RegisterUserBorrowedProducts;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Remove offer expiration date for an order specified by `OrderID`.
     *
     * @export
     * @class RemoveOfferStatusFromOrder
     * @implements {EvaService}
     */
    class RemoveOfferStatusFromOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RemoveOfferStatusFromOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Reset the unitprice on an OrderLine with a price correction.
     *
     * @export
     * @class RemoveOrderLineUnitPriceCorrection
     * @implements {EvaService}
     */
    class RemoveOrderLineUnitPriceCorrection implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RemoveOrderLineUnitPriceCorrection;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Remove an Organization's openinghours exception. When neither a DayOfWeek or Date is given, all openinghours exception for the given type will be removed and all openinghours will revert to the template value (if present).
     *
     * @export
     * @class RemoveOrganizationUnitOpeningHoursException
     * @implements {EvaService}
     */
    class RemoveOrganizationUnitOpeningHoursException implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RemoveOrganizationUnitOpeningHoursException;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Removes product(s) from an anonymous wishlist.
     *
     * @export
     * @class RemoveProductsFromAnonymousWishlist
     * @implements {EvaService}
     */
    class RemoveProductsFromAnonymousWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RemoveProductsFromAnonymousWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Removes product(s) from the user wishlist.
     *
     * @export
     * @class RemoveProductsFromUserWishlist
     * @implements {EvaService}
     */
    class RemoveProductsFromUserWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RemoveProductsFromUserWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Remove a special date from an OpeningHoursTemplate
     *
     * @export
     * @class RemoveSpecialDateFromOpeningHoursTemplate
     * @implements {EvaService}
     */
    class RemoveSpecialDateFromOpeningHoursTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RemoveSpecialDateFromOpeningHoursTemplate;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class RemoveUserFromGroup
     * @implements {EvaService}
     */
    class RemoveUserFromGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RemoveUserFromGroup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Renders the result of a previous thermal print job to HTML and/or PDF. Supply either a `BlobID` or `BlobUrl`.
     *
     * @export
     * @class RenderThermalPrintResult
     * @implements {EvaService}
     */
    class RenderThermalPrintResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RenderThermalPrintResult;
        response?: EVA.Core.RenderThermalPrintResultResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ReplaceCycleCountZonesForOrganizationUnit
     * @implements {EvaService}
     */
    class ReplaceCycleCountZonesForOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReplaceCycleCountZonesForOrganizationUnit;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Creates an EntityTranslation if one does not exist yet, otherwise it replaces the current one.
     *
     * @export
     * @class ReplaceEntityTranslation
     * @implements {EvaService}
     */
    class ReplaceEntityTranslation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReplaceEntityTranslation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Creates an StringTranslation if one does not exist yet, otherwise it replaces the current one.
     *
     * @export
     * @class ReplaceStringTranslation
     * @implements {EvaService}
     */
    class ReplaceStringTranslation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReplaceStringTranslation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ReplayEvents
     * @implements {EvaService}
     */
    class ReplayEvents implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReplayEvents;
        response?: EVA.Core.ReplayEventsResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class RepostFinancialPeriod
     * @implements {EvaService}
     */
    class RepostFinancialPeriod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RepostFinancialPeriod;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Reprints the results of a previous thermal print job to a specified station. Supply either a `BlobID` or `BlobUrl`.
     *
     * @export
     * @class ReprintThermalPrintResult
     * @implements {EvaService}
     */
    class ReprintThermalPrintResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReprintThermalPrintResult;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for retrieving a device authentication token
     *
     * @export
     * @class RequestDeviceAuthenticationToken
     * @implements {EvaService}
     */
    class RequestDeviceAuthenticationToken implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RequestDeviceAuthenticationToken;
        response?: EVA.Core.RequestDeviceAuthenticationTokenResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class RequestPasswordResetToken
     * @implements {EvaService}
     */
    class RequestPasswordResetToken implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RequestPasswordResetToken;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Certain payment methods support retrieving the current status. Given a PaymentTransactionID this service will attempt to poll thestatus for that particular payment and directly update its status.
     *
     * @export
     * @class RequestPaymentUpdate
     * @implements {EvaService}
     */
    class RequestPaymentUpdate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RequestPaymentUpdate;
        response?: EVA.Core.RequestPaymentUpdateResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class RequestReindexOrderSearchData
     * @implements {EvaService}
     */
    class RequestReindexOrderSearchData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RequestReindexOrderSearchData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class RequestReindexUserSearchData
     * @implements {EvaService}
     */
    class RequestReindexUserSearchData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RequestReindexUserSearchData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Requeues specified error message(s).
     *
     * @export
     * @class RequeueErrorMessage
     * @implements {EvaService}
     */
    class RequeueErrorMessage implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RequeueErrorMessage;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Resend a UserInteraction of Type Communication

When no EmailAddress is supplied, the original recipient will be used.
     *
     * @export
     * @class ResendUserInteractionCommunication
     * @implements {EvaService}
     */
    class ResendUserInteractionCommunication implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ResendUserInteractionCommunication;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Reserve an Order.

The Lines on the Order with LineActionType ReserveLine will be reserved:
- The Stock will be moved from Sellable to Reserved
- A receipt will be printed
     *
     * @export
     * @class ReserveOrder
     * @implements {EvaService}
     */
    class ReserveOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReserveOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service resets processed financial events back to unprocessed,
which means all associated financial ledgers are deleted.

Only financial events of periods that have not been fully processed (for example exported to an external ERP system)
can be reset.

As this can be a time consuming service, the request is handled in the background and upon completion a mail
is sent to the caller of this service with the results.

The scope of which events to reset can be limited by specifying a `Filter` in the request. Only the events
that match the filter will be reset.
     *
     * @export
     * @class ResetFinancialEvents
     * @implements {EvaService}
     */
    class ResetFinancialEvents implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ResetFinancialEvents;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ResetUserPassword
     * @implements {EvaService}
     */
    class ResetUserPassword implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ResetUserPassword;
        response?: EVA.Core.ResetUserPasswordResponse;
    }
    /**
     * Resolve product backend IDs
     *
     * @export
     * @class ResolveProductBackendIDs
     * @implements {EvaService}
     */
    class ResolveProductBackendIDs implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ResolveProductBackendIDs;
        response?: EVA.Core.ResolveProductBackendIDsResponse;
    }
    /**
     * When you would like to resume a suspended order, you can use `ResumeSuspendedOrder` to do so. This will remove the suspended state from the order.
     *
     * @export
     * @class ResumeSuspendedOrder
     * @implements {EvaService}
     */
    class ResumeSuspendedOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ResumeSuspendedOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Retry financial period export. Use `Force` to retry eventhough the status is `Finished`
     *
     * @export
     * @class RetryFinancialPeriodExport
     * @implements {EvaService}
     */
    class RetryFinancialPeriodExport implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RetryFinancialPeriodExport;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Retry financial period exports.
     *
     * @export
     * @class RetryFinancialPeriodExports
     * @implements {EvaService}
     */
    class RetryFinancialPeriodExports implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RetryFinancialPeriodExports;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Retry the given InvoiceExports
     *
     * @export
     * @class RetryInvoiceExport
     * @implements {EvaService}
     */
    class RetryInvoiceExport implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RetryInvoiceExport;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class RetryOrderExport
     * @implements {EvaService}
     */
    class RetryOrderExport implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RetryOrderExport;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class RetryShipmentExport
     * @implements {EvaService}
     */
    class RetryShipmentExport implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RetryShipmentExport;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Supports the creation of return orders and return reversal orders. Unreferenced reversals will always be treated as normal returns.
     *
     * @export
     * @class ReturnOrderLines
     * @implements {EvaService}
     */
    class ReturnOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReturnOrderLines;
        response?: EVA.Core.SimpleShoppingCartResponse;
    }
    /**
     * Return user borrowed products. When IsDamaged is set to true, the stock will be moved to Damaged. Otherwise, the stock will be moved to Sellable.
     *
     * @export
     * @class ReturnUserBorrowedProducts
     * @implements {EvaService}
     */
    class ReturnUserBorrowedProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ReturnUserBorrowedProducts;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Revoke the wishlist share.
     *
     * @export
     * @class RevokeWishlistShare
     * @implements {EvaService}
     */
    class RevokeWishlistShare implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RevokeWishlistShare;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for updating a existing `EventExportConfiguration`.

- Make sure to catch & store the `SecretKey1` or `SecretKey2`, as there will be no other way of retrieving them after this service call!
     *
     * @export
     * @class RotateEventExportConfigurationKey
     * @implements {EvaService}
     */
    class RotateEventExportConfigurationKey implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.RotateEventExportConfigurationKey;
        response?: EVA.Core.RotateEventExportConfigurationKeyResponse;
    }
    /**
     * Searches for consumer companies based on the provided query and filter criteria.
     *
     * @export
     * @class SearchConsumerCompany
     * @implements {EvaService}
     */
    class SearchConsumerCompany implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SearchConsumerCompany;
        response?: EVA.Core.SearchConsumerCompanyResponse;
    }
    /**
     * Search for documents in a specified category. This will return a maximum of 100 records.

:warning: For debugging and support purposes only.
     *
     * @export
     * @class SearchDocumentStore
     * @implements {EvaService}
     */
    class SearchDocumentStore implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SearchDocumentStore;
        response?: EVA.Core.SearchDocumentStoreResponse;
    }
    /**
     * Search for any type of orders by providing the filter details properties
Important search criterias are: OrganizationUnitID, CurrentOrganizationUnitType, OrderIDs, CustomerID(ClientContext.Type), PaymentStatus.

When no StartTime is provided this will be defaulted to 3 months ago. The only exception is when specific OrderIDs are provided, then it's allowed to not specify a Date.

To include the custom fields of the customer, the property `AdditionalOrderDataOptions.CustomerCustomFields` can be set to `true`.

E.g. request:
{
  "OrderIDs": [
    "orderID1",
    "orderID2"
  ],
  "PaymentStatus": "PaymentStatus",
  "IsShipped": "isShipped"
}


Payment status options:
    NotPaid = 0,
    Paid = 1,
    Partial = 2
     *
     * @export
     * @class SearchOrders
     * @implements {EvaService}
     */
    class SearchOrders implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SearchOrders;
        response?: EVA.Core.SearchOrdersResponse;
    }
    /**
     * Performs a search for products, given a `Query` string.
     *
     * @export
     * @class SearchProducts
     * @implements {EvaService}
     */
    class SearchProducts implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SearchProducts;
        response?: EVA.Core.SearchProductsResponse;
    }
    /**
     * Execute a search through the users

Filtering is possible on all kinds of properties. By default the search results will be restricted to users in the current country.
     *
     * @export
     * @class SearchUsers
     * @implements {EvaService}
     */
    class SearchUsers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SearchUsers;
        response?: EVA.Core.SearchUsersResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class SendConfigurationPage
     * @implements {EvaService}
     */
    class SendConfigurationPage implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SendConfigurationPage;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for sending a contact form to the email set in settings called `ContactForm:MailTo`
     *
     * @export
     * @class SendContactForm
     * @implements {EvaService}
     */
    class SendContactForm implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SendContactForm;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Requests a magic login link, which will send an email to the user (if found).

- Setting `MagicLink:Enabled` must be enabled explicitly
- When using a `RedirectUrl`, configure setting `MagicLink:RedirectDomains` to whitelist supported redirect hosts (recommended!)

Using the returned `RequestToken`, you can either subscribe to the `/magicLink` SignalR hub, or poll the `CheckMagicLink` service. As soon as the emailed magic link is clicked, that hub and service
will return either a `TemporaryToken` you can call `Login` with for this user, or an `APIKey` that's already valid and ready to proceed with.

- The email sent contains a link that will work for 10 minutes
- When clicked, this will generate either a `TemporaryToken` that can be used to `Login` with for 3 minutes, or an `ApiKey` that can be used for 1 hour
- This data is additionally exposed through `CheckMagicLink` for 1 minute after clicking
- When using a `TemporaryToken` to log in with, the resulting `AuthenticationToken` is valid for 1 week (or `SessionDuration:MagicLink`) and will refresh every time when used

Finally, when specifying `IncludeManualCode`, the email sent will also contain a short code for the user to _manually_ input into your application - call `ConfirmMagicLink` with both the `RequestToken`
and that `ManualCode` to obtain either the `TemporaryToken` or `ApiKey` like above, when valid.
     *
     * @export
     * @class SendMagicLink
     * @implements {EvaService}
     */
    class SendMagicLink implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SendMagicLink;
        response?: EVA.Core.SendMagicLinkResponse;
    }
    /**
     * Sends a `ReturnMail` stencil to the customer of given return order, attaching order blobs of type `ReturnLabel` (3) and `ReturnForm` (4) if present.
     *
     * @export
     * @class SendReturnMail
     * @implements {EvaService}
     */
    class SendReturnMail implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SendReturnMail;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update user custom data
     *
     * @export
     * @class SetCustomUserData
     * @implements {EvaService}
     */
    class SetCustomUserData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetCustomUserData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service can be used when in local mode to set some UserIdentifiers on an Order

If ParseBarcode responds with ParsedButNotFound, this service can be used to set the UserIdentifiers on the Order.
Or an EmailAddress can be manually set.

When the order is exported to the cloud, the UserIdentifiers will be used to match the order to the correct user.
     *
     * @export
     * @class SetCustomerIdentifiersOnOrder
     * @implements {EvaService}
     */
    class SetCustomerIdentifiersOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetCustomerIdentifiersOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class SetCustomerReferencesOnOrder
     * @implements {EvaService}
     */
    class SetCustomerReferencesOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetCustomerReferencesOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the given Address Book Item as default BillingAddress for the User of the Address Book
     *
     * @export
     * @class SetDefaultBillingAddress
     * @implements {EvaService}
     */
    class SetDefaultBillingAddress implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetDefaultBillingAddress;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the given Address Book Item as default PickupAddress for the User of the Address Book
     *
     * @export
     * @class SetDefaultPickupAddress
     * @implements {EvaService}
     */
    class SetDefaultPickupAddress implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetDefaultPickupAddress;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the given Address Book Item as default ShippingAddress for the User of the Address Book
     *
     * @export
     * @class SetDefaultShippingAddress
     * @implements {EvaService}
     */
    class SetDefaultShippingAddress implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetDefaultShippingAddress;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set additional order information intended for delivery data.
     *
     * @export
     * @class SetDeliveryOrderData
     * @implements {EvaService}
     */
    class SetDeliveryOrderData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetDeliveryOrderData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the product selection for a specific discount line
     *
     * @export
     * @class SetDiscountProductOption
     * @implements {EvaService}
     */
    class SetDiscountProductOption implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetDiscountProductOption;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Sets the additional e-invoicing data on the given order. To retrieve the information that has been set use: `GetElectronicInvoiceData`
     *
     * @export
     * @class SetElectronicInvoiceData
     * @implements {EvaService}
     */
    class SetElectronicInvoiceData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetElectronicInvoiceData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the employees for the financial period. This only works for pending financial periods.
     *
     * @export
     * @class SetEmployeesForFinancialPeriod
     * @implements {EvaService}
     */
    class SetEmployeesForFinancialPeriod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetEmployeesForFinancialPeriod;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the external full stock count result.
The service accepts partial or full result lists.
The service requires a ActualCompletionTime. There can only be one completion time. The value will be updated with every service call.

CountResults must either have ProductID (preferred) or ProductBarcode.
     *
     * @export
     * @class SetExternalFullStockCountResult
     * @implements {EvaService}
     */
    class SetExternalFullStockCountResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetExternalFullStockCountResult;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update the fiscal properties on an Order.
     *
     * @export
     * @class SetFiscalOrderData
     * @implements {EvaService}
     */
    class SetFiscalOrderData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetFiscalOrderData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the GiftWrapping options on an Order.

There are 4 options:

 - Wrap the complete order: set `WrapOrder` to `true`
 - Wrap the complete order individually: set `WrapIndividually` to `true`
 - Wrap parts of the order: set `GiftWraps` with a set of wraps
 - Wrap nothing: set everything to `null`

A `GiftWrap` consists out of an optional `Message` (could be printed on a card), an optional `GreetingCardProductID` and a set of `WrapLines`. A
`WrapLine` has an `OrderLineID` and a `Quantity`.

If a `GreetingCardProductID` and/or `Message` is desired when the order should be completely wrapped, or without any giftwrapping at all, set these
properties on root level. In all other cases they should be set on the `GiftWraps`.
     *
     * @export
     * @class SetGiftWrappingOptionsOnOrder
     * @implements {EvaService}
     */
    class SetGiftWrappingOptionsOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetGiftWrappingOptionsOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Adds or removes a lines to/from the ReturnToSupplier tasks shipment.

When 'Exclude' is enabled, all lines matching the given ProductID will be removed from the task shipment with a maximum of the given Quantity.
When 'Exclude' is disabled, all lines from the ReturnToSupplier order matching the given ProductID will be added to the tasks shipment to the maximum of the given Quantity.

Overshipment is not allowed.
     *
     * @export
     * @class SetLineForReturnToSupplierShipment
     * @implements {EvaService}
     */
    class SetLineForReturnToSupplierShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetLineForReturnToSupplierShipment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Sets the lottery number for a given order
     *
     * @export
     * @class SetLotteryNumber
     * @implements {EvaService}
     */
    class SetLotteryNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetLotteryNumber;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the loyalty program for the financial period. This only works for pending financial periods.
     *
     * @export
     * @class SetLoyaltyProgramForFinancialPeriod
     * @implements {EvaService}
     */
    class SetLoyaltyProgramForFinancialPeriod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetLoyaltyProgramForFinancialPeriod;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Sets `BackendID`/`BackendSystemID` on specified `OrderID`.

This combination may not already exist, and the targeted order may not be completed, or  already have a `BackendID`/`BackendSystemID` set.

:warning: Careful! Use at your own risk!

An order may already have any of the following things, which will not automatically be regenerated/resent because of this identifier change, and could
result in mapping issues on any external system listening in on these. Any of these would also apply to return orders referencing this order, or stock
mutations that have been caused by this order.

* Order exports to third parties
* Event exports (order created, order updated, ...)
* Data Lake exports
* Shipment labels with third party carriers
* Financial exports (Cookbook)
* Created documents (receipts, invoices, ...)
     *
     * @export
     * @class SetOrderBackendIdentifier
     * @implements {EvaService}
     */
    class SetOrderBackendIdentifier implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetOrderBackendIdentifier;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class SetOrderFulfillmentOptions
     * @implements {EvaService}
     */
    class SetOrderFulfillmentOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetOrderFulfillmentOptions;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Sets the `SoldBy` of the specified sales order to the provided `UserID`.

- The user must have type `Employee`
- This service can only be called on orders that have not been invoiced yet
- As `SoldBy` is a line-level property this service will update all lines on the order that have not been invoiced yet
     *
     * @export
     * @class SetOrderSoldBy
     * @implements {EvaService}
     */
    class SetOrderSoldBy implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetOrderSoldBy;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class SetOrganizationUnitNotes
     * @implements {EvaService}
     */
    class SetOrganizationUnitNotes implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetOrganizationUnitNotes;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Add or update an Organization's openinghours exception
     *
     * @export
     * @class SetOrganizationUnitOpeningHoursException
     * @implements {EvaService}
     */
    class SetOrganizationUnitOpeningHoursException implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetOrganizationUnitOpeningHoursException;
        response?: EVA.Core.ValidateOpeningHoursTemplateResponse;
    }
    /**
     * Set the answer to the PCI question for the provided `FinancialPeriodID`.
     *
     * @export
     * @class SetPCIQuestionAnswerForFinancialPeriod
     * @implements {EvaService}
     */
    class SetPCIQuestionAnswerForFinancialPeriod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetPCIQuestionAnswerForFinancialPeriod;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Pick a product for a PickProductAction for an OrderLine
     *
     * @export
     * @class SetPickProductDiscountOptionsForOrderLine
     * @implements {EvaService}
     */
    class SetPickProductDiscountOptionsForOrderLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetPickProductDiscountOptionsForOrderLine;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class SetPickupOrganizationUnit
     * @implements {EvaService}
     */
    class SetPickupOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetPickupOrganizationUnit;
        response?: EVA.Core.SimpleShoppingCartResponse;
    }
    /**
     * Set a preferred method to use when this order will be refunded.
Can be used on ReturnOrders to indicate how the money should be transferred back to the customer.

Available methods can be listed with the GetAvailableRefundPaymentMethodsForOrder service.
     *
     * @export
     * @class SetPreferredRefundMethod
     * @implements {EvaService}
     */
    class SetPreferredRefundMethod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetPreferredRefundMethod;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class SetRemarkOnOrder
     * @implements {EvaService}
     */
    class SetRemarkOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetRemarkOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the `RequestedDate` on OrderLines.
The RequestedDate is the date on which the OrderLine should be ready for shipping/pickup/etc.

It is required to provide an `OrderID`.
If the RequestedDate is null, it will be removed from the lines.
     *
     * @export
     * @class SetRequestedDate
     * @implements {EvaService}
     */
    class SetRequestedDate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetRequestedDate;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class SetReturnOrderRefundCorrection
     * @implements {EvaService}
     */
    class SetReturnOrderRefundCorrection implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetReturnOrderRefundCorrection;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Sets the first serial number on given order line, and splits off new order lines for each additional serial number given.
     *
     * @export
     * @class SetSerialNumbers
     * @implements {EvaService}
     */
    class SetSerialNumbers implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetSerialNumbers;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used in combination with the `GetShipmentSettings` to manage OrganizationUnit settings for Shipments.
Optionally an OrganizationUnitID can be provided, but by default the settings are updated for the current OrganizationUnit.

- DefaultReceiveMethod: How shipments should be received if no custom implementation is available
     *
     * @export
     * @class SetShipmentSettings
     * @implements {EvaService}
     */
    class SetShipmentSettings implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetShipmentSettings;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Sets the shipping method for the given order(lines). Use `ListAvailableShippingMethods` to request the shipping methods that can be used as input to this service.
     *
     * @export
     * @class SetShippingMethod
     * @implements {EvaService}
     */
    class SetShippingMethod implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetShippingMethod;
        response?: EVA.Core.SimpleShoppingCartResponse;
    }
    /**
     * Add or update a special date on an OpeningHoursTemplate
     *
     * @export
     * @class SetSpecialDateOnOpeningHoursTemplate
     * @implements {EvaService}
     */
    class SetSpecialDateOnOpeningHoursTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetSpecialDateOnOpeningHoursTemplate;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Change the settings for a specific OU/StockLabel combination. Use a value of null to make it inherit from it’s parent.
     *
     * @export
     * @class SetStockLabelSettings
     * @implements {EvaService}
     */
    class SetStockLabelSettings implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetStockLabelSettings;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the suppliers for OrganizationUnit where this could not be automatically determined

To view the available suppliers, sue the ValidateReturnToSupplierRequest service.

The SupplierOrganizationUnitID is nullable, if it is set to `null`, the OrganizationUnit will be skipped when generating the orders.
     *
     * @export
     * @class SetSuppliersForReturnToSupplierRequest
     * @implements {EvaService}
     */
    class SetSuppliersForReturnToSupplierRequest implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetSuppliersForReturnToSupplierRequest;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set additional order line information intended for transport data. Supports a single `OrderLineID`, or a list of `OrderLineIDs`.
     *
     * @export
     * @class SetTransportOrderLineData
     * @implements {EvaService}
     */
    class SetTransportOrderLineData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetTransportOrderLineData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set the user Consignment property.
     *
     * @export
     * @class SetUserConsignment
     * @implements {EvaService}
     */
    class SetUserConsignment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetUserConsignment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update a user login to single sign on only
     *
     * @export
     * @class SetUserToSingleSignOnOnly
     * @implements {EvaService}
     */
    class SetUserToSingleSignOnOnly implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetUserToSingleSignOnOnly;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set additional order information intended for warehouse data.
     *
     * @export
     * @class SetWarehouseOrderData
     * @implements {EvaService}
     */
    class SetWarehouseOrderData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SetWarehouseOrderData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Share a wishlist.
     *
     * @export
     * @class ShareWishlist
     * @implements {EvaService}
     */
    class ShareWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ShareWishlist;
        response?: EVA.Core.ShareWishlistResponse;
    }
    /**
     * Ships an order that has been exported to an external system by another proces
     *
     * @export
     * @class ShipExternalOrder
     * @implements {EvaService}
     */
    class ShipExternalOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ShipExternalOrder;
        response?: EVA.Core.ShipExternalOrderResponse;
    }
    /**
     * Ships the specified order. When Lines are specified, only those lines will be shipped.

Shipping of Lines includes:
- Activating giftcards
- Mutating the stock of the products
- Potentially invoicing of the lines
- Validating requirements
     *
     * @export
     * @class ShipOrder
     * @implements {EvaService}
     */
    class ShipOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ShipOrder;
        response?: EVA.Core.ShipOrderResponse;
    }
    /**
     * Ships an OrderFulfillment, thereby creating one or more Shipments for the underlying Order.
     *
     * @export
     * @class ShipOrderFulfillment
     * @implements {EvaService}
     */
    class ShipOrderFulfillment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ShipOrderFulfillment;
        response?: EVA.Core.ShipOrderFulfillmentResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ShipOrderLines
     * @implements {EvaService}
     */
    class ShipOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ShipOrderLines;
        response?: EVA.Core.ShipOrderLinesResponse;
    }
    /**
     * Creates a shipment for given purchase order lines and ships it!
     *
     * @export
     * @class ShipPurchaseOrder
     * @implements {EvaService}
     */
    class ShipPurchaseOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ShipPurchaseOrder;
        response?: EVA.Core.ShipPurchaseOrderResponse;
    }
    /**
     * Ship an existing shipment that has status Pending.

Works only for SalesOrders at this moment.
     *
     * @export
     * @class ShipShipment
     * @implements {EvaService}
     */
    class ShipShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ShipShipment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Add a signature to an Order.
     *
     * @export
     * @class SignOrder
     * @implements {EvaService}
     */
    class SignOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SignOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Split an OrderLine in two lines.

The new line will have a QuantityOrdered of QuantityToSplit and on the existing line there will be QuantityToShip items cancelled.
If the OrderLine has a negative QuantityToShip, the QuantityOrdered of the new line will be -QuantityToSplit.

The QuantityToSplit should always be greather than 0 and smaller than the absolute value of the QuantityToShip of the OrderLine.
     *
     * @export
     * @class SplitOrderLine
     * @implements {EvaService}
     */
    class SplitOrderLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SplitOrderLine;
        response?: EVA.Core.SplitOrderLineResponse;
    }
    /**
     * Initiates the closing of a cash journal. This does not yet mutate anything, but returns all the details you need about a cash journal to be able to close it.

This consists of:

- The information about the device and station
- Which expenses and deposits were made from this device.
- What payments were made.
- What the `ExpectedClosingAmount` is, which is a summary of the previous journal's closing amount plus all transactions.
- Possible pending cash deposits on this device
     *
     * @export
     * @class StartCloseCashJournal
     * @implements {EvaService}
     */
    class StartCloseCashJournal implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartCloseCashJournal;
        response?: EVA.Core.StartCloseCashJournalResponse;
    }
    /**
     * Starts a Composite User Task, returning child meta data.
     *
     * @export
     * @class StartCompositeUserTask
     * @implements {EvaService}
     */
    class StartCompositeUserTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartCompositeUserTask;
        response?: EVA.Core.StartCompositeUserTaskResponse;
    }
    /**
     * Starts a ConsolidatedZonePickTask
     *
     * @export
     * @class StartConsolidatedZonePickTask
     * @implements {EvaService}
     */
    class StartConsolidatedZonePickTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartConsolidatedZonePickTask;
        response?: EVA.Core.StartConsolidatedZonePickTaskResponse;
    }
    /**
     * Starts a new custom usertask
     *
     * @export
     * @class StartCustomUserTask
     * @implements {EvaService}
     */
    class StartCustomUserTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartCustomUserTask;
        response?: EVA.Core.StartCustomUserTaskResponse;
    }
    /**
     * Starts a full stock count task. Requires it to be ready.
     *
     * @export
     * @class StartFullStockCount
     * @implements {EvaService}
     */
    class StartFullStockCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartFullStockCount;
        response?: EVA.Core.StartFullStockCountResponse;
    }
    /**
     * Starts a Full Stock Count label task.
     *
     * @export
     * @class StartFullStockCountLabel
     * @implements {EvaService}
     */
    class StartFullStockCountLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartFullStockCountLabel;
        response?: EVA.Core.StartFullStockCountLabelResponse;
    }
    /**
     * Starts the opening of a cash journal for the given DeviceID.

It's only possible to open a cash journal when an open financial period exists and when the device is a cash device. Retrieving a list of valid devices
can be accomplished through the `ListDevices` service.

Calling this service does not yet do anything, except return the data you need to open a cash journal:

- The available denominations (coins and notes)
- The details about the previous cash journal's closing amounts (if it exists), which can be used as an extra check
- Possible pending cash deposits on this device
     *
     * @export
     * @class StartOpenCashJournal
     * @implements {EvaService}
     */
    class StartOpenCashJournal implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartOpenCashJournal;
        response?: EVA.Core.StartOpenCashJournalResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class StartPrintPriceLabelTask
     * @implements {EvaService}
     */
    class StartPrintPriceLabelTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartPrintPriceLabelTask;
        response?: EVA.Core.StartPrintPriceLabelTaskResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class StartPrintTask
     * @implements {EvaService}
     */
    class StartPrintTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartPrintTask;
        response?: EVA.Core.StartPrintTaskResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class StartReceiveShipment
     * @implements {EvaService}
     */
    class StartReceiveShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartReceiveShipment;
        response?: EVA.Core.StartReceiveShipmentResponse;
    }
    /**
     * Start repair user task
     *
     * @export
     * @class StartRepairUserTask
     * @implements {EvaService}
     */
    class StartRepairUserTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartRepairUserTask;
        response?: EVA.Core.StartRepairUserTaskResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class StartReservationCleanup
     * @implements {EvaService}
     */
    class StartReservationCleanup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartReservationCleanup;
        response?: EVA.Core.StartReservationCleanupResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class StartReservationPickupTask
     * @implements {EvaService}
     */
    class StartReservationPickupTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartReservationPickupTask;
        response?: EVA.Core.StartReservationPickupTaskResponse;
    }
    /**
     * Starts ReturnToSupplier task.
     *
     * @export
     * @class StartReturnToSupplierTask
     * @implements {EvaService}
     */
    class StartReturnToSupplierTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartReturnToSupplierTask;
        response?: EVA.Core.StartReturnToSupplierResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class StartSecondChanceProductRegistration
     * @implements {EvaService}
     */
    class StartSecondChanceProductRegistration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartSecondChanceProductRegistration;
        response?: EVA.Core.StartSecondChanceProductRegistrationResponse;
    }
    /**
     * Starts a ShipFromStore task, identified by its UserTaskID.

The service returns the order that should be shipped from the user's current store as well as the OrderLines
that should be part of the shipment. Not necessarily all orderlines of the order should be shipped, so the
OrderLineIDs returned by this service should be respected.
     *
     * @export
     * @class StartShipFromStoreTask
     * @implements {EvaService}
     */
    class StartShipFromStoreTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartShipFromStoreTask;
        response?: EVA.Core.StartShipFromStoreTaskResponse;
    }
    /**
     * Starts a Stock Movement Follow Up task. Proceed with the `CompleteStockMovementFollowUp` service when finished.
     *
     * @export
     * @class StartStockMovementFollowUp
     * @implements {EvaService}
     */
    class StartStockMovementFollowUp implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartStockMovementFollowUp;
        response?: EVA.Core.StartStockMovementFollowUpResponse;
    }
    /**
     * Starts one or more Stock Replenishment tasks, returning a workset. Pass this workset to the `CompleteStockReplenishment` service when all tasks are complete.
     *
     * @export
     * @class StartStockReplenishment
     * @implements {EvaService}
     */
    class StartStockReplenishment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartStockReplenishment;
        response?: EVA.Core.StartStockReplenishmentResponse;
    }
    /**
     * Starts a user task of type Stock Reservation.

- The task will return all required data for display purposes
- When no lines are returned, the task is already completed or in progress by someone else
- To complete the StockReservation task, use the `CompleteStockReservationTask` service
     *
     * @export
     * @class StartStockReservation2
     * @implements {EvaService}
     */
    class StartStockReservation2 implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartStockReservation2;
        response?: EVA.Core.StartStockReservationResponse2;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class StartZonedCycleCount
     * @implements {EvaService}
     */
    class StartZonedCycleCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartZonedCycleCount;
        response?: EVA.Core.StartZonedCycleCountResponse;
    }
    /**
     * Starts a ZonedCycleCountPreCount by the ID of its UserTask.

The response contains:

- Which StockLabels to count (`StockLabelsToCount`)
- The product that needs to be counted (`Product`); this product contains all the fields in its `Content` object that were requested through the `IncludedFields` property on the request, or a default set of fields if not specified
- Zone that needs to be counted (`CycleCountZone`)
- List of stock labels to be counted (`StockLabelsToCount`)
     *
     * @export
     * @class StartZonedCycleCountPreCount
     * @implements {EvaService}
     */
    class StartZonedCycleCountPreCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StartZonedCycleCountPreCount;
        response?: EVA.Core.StartZonedCycleCountPreCountResponse;
    }
    /**
     * Service combining StoreBlob and AttachBlobToOrder.
     *
     * @export
     * @class StoreAndAttachBlobToOrder
     * @implements {EvaService}
     */
    class StoreAndAttachBlobToOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.StoreAndAttachBlobToOrder;
        response?: EVA.Core.StoreAndAttachBlobToOrderResponse;
    }
    /**
     * Submit an inquiry
     *
     * @export
     * @class SubmitInquiry
     * @implements {EvaService}
     */
    class SubmitInquiry implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SubmitInquiry;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Subscribe to the specified feed for the current User's `ApplicationID`.

`Name` is the name of the file you wish to watch. The `WebhookUrl` is called with a JSON POST request, in which the Feed's `name` and `applicationID` are specified. This request does not contain the data
itself, your application will have to fetch the data using your current approaches to download this feed. This may seem verbose, but we have no guarantee the endpoint is secure, and we can't just send
possibly sensitive information there!

To unsubscribe, call the `UnsubscribeFromFeed` service with the returned `ID` value.
     *
     * @export
     * @class SubscribeToFeed
     * @implements {EvaService}
     */
    class SubscribeToFeed implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SubscribeToFeed;
        response?: EVA.Core.SubscribeToFeedResponse;
    }
    /**
     * Subscribe an User to a Subscription

The service returns the status of the subscription
     *
     * @export
     * @class SubscribeUser
     * @implements {EvaService}
     */
    class SubscribeUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SubscribeUser;
        response?: EVA.Core.SubscribeUserResponse;
    }
    /**
     * Creates a purchase order (or updates it if it already exists).

When the order already exists, only the lines can be updated (see `SupplierUpdatePurchaseOrder`) - other properties will be ignored.

:lightbulb: Using `SupplierCreateOrder` and `SupplierUpdatePurchaseOrder` is preferred.
     *
     * @export
     * @class SupplierCreateOrUpdatePurchaseOrder
     * @implements {EvaService}
     */
    class SupplierCreateOrUpdatePurchaseOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SupplierCreateOrUpdatePurchaseOrder;
        response?: EVA.Core.CreatePurchaseOrderResponseWithUpdateIndicator;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class SupplierCreatePurchaseOrder
     * @implements {EvaService}
     */
    class SupplierCreatePurchaseOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SupplierCreatePurchaseOrder;
        response?: EVA.Core.CreatePurchaseOrderResponse;
    }
    /**
     * Ships a Purchase Order with specified lines. Supply either `ID` or a `BackendID`/`BackendSystemID` combination to
identify the order.

If you wish to receive on behalf of a different organization, identify that organization by specifying a
`ShipToOrganizationUnitBackendID`. Should this organization reside in a different cluster, this call will be proxied
to that cluster for you.
     *
     * @export
     * @class SupplierShipPurchaseOrder
     * @implements {EvaService}
     */
    class SupplierShipPurchaseOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SupplierShipPurchaseOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class SupplierUpdatePurchaseOrder
     * @implements {EvaService}
     */
    class SupplierUpdatePurchaseOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SupplierUpdatePurchaseOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * When you would like to temporarily suspend an order, you can use `SuspendOrder` to do so.

- To identify your suspended order it is possible to supply a `Description`
- To print a receipt of the suspended order, to make it easier to continue later on, supply `PrintReceipt` (and optionally a `StationID`)
- To retreive the suspended orders, see the `ListSuspendedOrders` service
     *
     * @export
     * @class SuspendOrder
     * @implements {EvaService}
     */
    class SuspendOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.SuspendOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * *WARNING* NewBlack only service

Toggle on/off some core features of this environment
     *
     * @export
     * @class ToggleFeatures
     * @implements {EvaService}
     */
    class ToggleFeatures implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ToggleFeatures;
        response?: EVA.Core.ToggleFeaturesResponse;
    }
    /**
     * Transfers the provided Order to the OrganizationUnit of the currently logged in user, updating both the SoldFromOrganizationUnit and PickupOrganizationUnit.

If this is already the case, calling this service has no effect. In case the order cannot be transferred, this service will return an error.
     *
     * @export
     * @class TransferOrderToOrganizationUnit
     * @implements {EvaService}
     */
    class TransferOrderToOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.TransferOrderToOrganizationUnit;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Flag this order to trigger manual AML checks.
     *
     * @export
     * @class TriggerAMLChecksOnOrder
     * @implements {EvaService}
     */
    class TriggerAMLChecksOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.TriggerAMLChecksOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class TriggerRecurringTask
     * @implements {EvaService}
     */
    class TriggerRecurringTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.TriggerRecurringTask;
        response?: EVA.Core.TriggerRecurringTaskResponse;
    }
    /**
     * Restore an archived organization unit.
     *
     * @export
     * @class UnarchiveOrganizationUnit
     * @implements {EvaService}
     */
    class UnarchiveOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UnarchiveOrganizationUnit;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service allows you to undo a commitment on the given OrderLineIDs, making the committed stock available to other orders again.
     *
     * @export
     * @class UncommitOrderLines
     * @implements {EvaService}
     */
    class UncommitOrderLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UncommitOrderLines;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Removes stock from transit for an unreceived shipment
     *
     * @export
     * @class UndoShipment
     * @implements {EvaService}
     */
    class UndoShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UndoShipment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Removes stock for a received shipment
     *
     * @export
     * @class UndoShipmentReceipt
     * @implements {EvaService}
     */
    class UndoShipmentReceipt implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UndoShipmentReceipt;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Unlinks the shared wishlist from the user. Will also remove it from the users overview. Shared wishlist can still be retrieved by the token.
     *
     * @export
     * @class UnlinkSharedWishlist
     * @implements {EvaService}
     */
    class UnlinkSharedWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UnlinkSharedWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Unlock the Order, allowing future editing on it.
     *
     * @export
     * @class UnlockOrder
     * @implements {EvaService}
     */
    class UnlockOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UnlockOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Unsubscribe from the specified feed.
     *
     * @export
     * @class UnsubscribeFromFeed
     * @implements {EvaService}
     */
    class UnsubscribeFromFeed implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UnsubscribeFromFeed;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Unsubscribes an User from a Subscription

The service returns the status of the subscription
     *
     * @export
     * @class UnsubscribeUser
     * @implements {EvaService}
     */
    class UnsubscribeUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UnsubscribeUser;
        response?: EVA.Core.UnsubscribeUserResponse;
    }
    /**
     * Update the addresses on a User

This is deprecated in favour of using the AddressBook services
     *
     * @export
     * @class UpdateAddress
     * @implements {EvaService}
     */
    class UpdateAddress implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateAddress;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an existing Address in the AddressBook of an User
     *
     * @export
     * @class UpdateAddressBookItem
     * @implements {EvaService}
     */
    class UpdateAddressBookItem implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateAddressBookItem;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an existing Address Requirement object.

This service is using the ID or a PropertyType-CountryID par as input for the finding of the
correct address requirement we want to update.
If the address requirement is found the visibility and validation of this will be updated.
     *
     * @export
     * @class UpdateAddressRequirement
     * @implements {EvaService}
     */
    class UpdateAddressRequirement implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateAddressRequirement;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an anonymous wishlist.
     *
     * @export
     * @class UpdateAnonymousWishlist
     * @implements {EvaService}
     */
    class UpdateAnonymousWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateAnonymousWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an exisiting API key

All other properties need a new key to be created as these would influence the token authorization.
     *
     * @export
     * @class UpdateApiKey
     * @implements {EvaService}
     */
    class UpdateApiKey implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateApiKey;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an app rule
     *
     * @export
     * @class UpdateAppRule
     * @implements {EvaService}
     */
    class UpdateAppRule implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateAppRule;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates an existing AuditChainConfiguration, which allows you to configure the starting number of a sequence and its prefix.
Updating a configuration for an audit chain that already exists does not apply this updated configuration. A new chain would need to be created by incrementing the Auditing:Version setting.
     *
     * @export
     * @class UpdateAuditChainConfiguration
     * @implements {EvaService}
     */
    class UpdateAuditChainConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateAuditChainConfiguration;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update a cash correction reason
     *
     * @export
     * @class UpdateCashCorrectionReason
     * @implements {EvaService}
     */
    class UpdateCashCorrectionReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateCashCorrectionReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates the current Company on specified `UserID`, or the current user when omitted.

:no_entry: Requires employee rights when using a `UserID` different than the current user.
:warning: Will overwrite *all* company fields, those fields omitted in your request will be removed! Except for FiscalID, that will be kept if not provided.
     *
     * @export
     * @class UpdateCompanyForUser
     * @implements {EvaService}
     */
    class UpdateCompanyForUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateCompanyForUser;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates the details of companies associated with the consumer type.
     *
     * @export
     * @class UpdateConsumerCompany
     * @implements {EvaService}
     */
    class UpdateConsumerCompany implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateConsumerCompany;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateContractNumber
     * @implements {EvaService}
     */
    class UpdateContractNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateContractNumber;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update custom fields for multiple organization units at once.
     *
     * @export
     * @class UpdateCustomFieldsForOrganizationUnits
     * @implements {EvaService}
     */
    class UpdateCustomFieldsForOrganizationUnits implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateCustomFieldsForOrganizationUnits;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates a template for custom user tasks. If a property is not provided, it will be set to null.
     *
     * @export
     * @class UpdateCustomUserTaskTemplate
     * @implements {EvaService}
     */
    class UpdateCustomUserTaskTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateCustomUserTaskTemplate;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateCycleCountOriginType
     * @implements {EvaService}
     */
    class UpdateCycleCountOriginType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateCycleCountOriginType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateCycleCountZone
     * @implements {EvaService}
     */
    class UpdateCycleCountZone implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateCycleCountZone;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates an existing CycleCountZoneGroup.
     *
     * @export
     * @class UpdateCycleCountZoneGroup
     * @implements {EvaService}
     */
    class UpdateCycleCountZoneGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateCycleCountZoneGroup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateDigitalGiftCardMailOptions
     * @implements {EvaService}
     */
    class UpdateDigitalGiftCardMailOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateDigitalGiftCardMailOptions;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Sets the new mail values for the GiftCard identified by the `OrderID` and `GiftCardID`.

Omitting the `FromEmailAddress` parameter will default to the email address of the customer attached to the order.
     *
     * @export
     * @class UpdateDigitalGiftCardOptions
     * @implements {EvaService}
     */
    class UpdateDigitalGiftCardOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateDigitalGiftCardOptions;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service allows updating giftcard lines created with the `AddGiftCardToOrder` service.
 * `Data` will overwrite all the gift card data.
 * `ImageWidth` and `ImageHeight` describe the image used
     *
     * @export
     * @class UpdateDigitalGiftCardOrderLine
     * @implements {EvaService}
     */
    class UpdateDigitalGiftCardOrderLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateDigitalGiftCardOrderLine;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for updating a existing `EventExportConfiguration`
     *
     * @export
     * @class UpdateEventExportConfiguration
     * @implements {EvaService}
     */
    class UpdateEventExportConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateEventExportConfiguration;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an existing Flight
     *
     * @export
     * @class UpdateFlight
     * @implements {EvaService}
     */
    class UpdateFlight implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateFlight;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Mutate Interbranch properties on the given `Order`.
     *
     * @export
     * @class UpdateInterbranchOrder
     * @implements {EvaService}
     */
    class UpdateInterbranchOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInterbranchOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates an existing invoice. See the `CreateInvoice` service for more details about the request.
     *
     * @export
     * @class UpdateInvoice
     * @implements {EvaService}
     */
    class UpdateInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInvoice;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an InvoiceAdditionalAmount
     *
     * @export
     * @class UpdateInvoiceAdditionalAmount
     * @implements {EvaService}
     */
    class UpdateInvoiceAdditionalAmount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInvoiceAdditionalAmount;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update the Description and LedgerClassID of a InvoiceAdditionalAmountType
     *
     * @export
     * @class UpdateInvoiceAdditionalAmountType
     * @implements {EvaService}
     */
    class UpdateInvoiceAdditionalAmountType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInvoiceAdditionalAmountType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an InvoiceDispute
     *
     * @export
     * @class UpdateInvoiceDispute
     * @implements {EvaService}
     */
    class UpdateInvoiceDispute implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInvoiceDispute;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update the Description and LedgerClassID of a InvoiceDisputeReason
     *
     * @export
     * @class UpdateInvoiceDisputeReason
     * @implements {EvaService}
     */
    class UpdateInvoiceDisputeReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInvoiceDisputeReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an InvoiceDisputeResolveAction
     *
     * @export
     * @class UpdateInvoiceDisputeResolveAction
     * @implements {EvaService}
     */
    class UpdateInvoiceDisputeResolveAction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInvoiceDisputeResolveAction;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateInvoiceHoldStatus
     * @implements {EvaService}
     */
    class UpdateInvoiceHoldStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInvoiceHoldStatus;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates lines on the given invoice (`InvoiceID`).

When the Quantity is 0 and the ExpectedQuantity is `null`, the line will be deleted.
     *
     * @export
     * @class UpdateInvoiceLines
     * @implements {EvaService}
     */
    class UpdateInvoiceLines implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInvoiceLines;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates the invoice user data. To retrieve the information that has been set use: `GetInvoice`
     *
     * @export
     * @class UpdateInvoiceUserData
     * @implements {EvaService}
     */
    class UpdateInvoiceUserData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateInvoiceUserData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update a manual tax exemption reason.
     *
     * @export
     * @class UpdateManualTaxExemptionReason
     * @implements {EvaService}
     */
    class UpdateManualTaxExemptionReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateManualTaxExemptionReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an existing OpeningHoursTemplate
     *
     * @export
     * @class UpdateOpeningHoursTemplate
     * @implements {EvaService}
     */
    class UpdateOpeningHoursTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOpeningHoursTemplate;
        response?: EVA.Core.ValidateOpeningHoursTemplateResponse;
    }
    /**
     * Update various fields on an Order.
     *
     * @export
     * @class UpdateOrder
     * @implements {EvaService}
     */
    class UpdateOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateOrderAddresses
     * @implements {EvaService}
     */
    class UpdateOrderAddresses implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderAddresses;
        response?: EVA.Core.UpdateOrderAddressesResponse;
    }
    /**
     * Update the BillingAddress on an Order.

This can be done by providing a full Address in the Address property, or by referencing an Address in an AddressBook in the AddressBookID property.
     *
     * @export
     * @class UpdateOrderBillingAddress
     * @implements {EvaService}
     */
    class UpdateOrderBillingAddress implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderBillingAddress;
        response?: EVA.Core.UpdateOrderAddressResponse;
    }
    /**
     * Update the data for the checkout options returned in the `AdditionalData` of a `GetOrder` or `GetShoppingCart` call
     *
     * @export
     * @class UpdateOrderCheckoutOptions
     * @implements {EvaService}
     */
    class UpdateOrderCheckoutOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderCheckoutOptions;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateOrderCurrency
     * @implements {EvaService}
     */
    class UpdateOrderCurrency implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderCurrency;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create or update existing custom field on order
     *
     * @export
     * @class UpdateOrderCustomFields
     * @implements {EvaService}
     */
    class UpdateOrderCustomFields implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderCustomFields;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Set order custom status
     *
     * @export
     * @class UpdateOrderCustomOrderStatus
     * @implements {EvaService}
     */
    class UpdateOrderCustomOrderStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderCustomOrderStatus;
        response?: EVA.Core.UpdateOrderCustomOrderStatusResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateOrderCustomStatus
     * @implements {EvaService}
     */
    class UpdateOrderCustomStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderCustomStatus;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateOrderCustomType
     * @implements {EvaService}
     */
    class UpdateOrderCustomType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderCustomType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates the price and/or quantity of a fulfillment line based on its ID.

When modifying the quantity, it's not possible currently to increase the quantity, only to decrease it. The modification must also happen on the physical fulfillment line,
that is a fulfillment line with Type = Physical.
     *
     * @export
     * @class UpdateOrderFulfillmentLine
     * @implements {EvaService}
     */
    class UpdateOrderFulfillmentLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderFulfillmentLine;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Create or update existing custom field on order line level by ID
     *
     * @export
     * @class UpdateOrderLineCustomFields
     * @implements {EvaService}
     */
    class UpdateOrderLineCustomFields implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderLineCustomFields;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update the StockLabel for a single `OrderLine` or for the complete `Order`. It is required to supply an `OrderID` or an `OrderLineID`.
If one of the lines cannot be updated, it will be skipped.

Only unshipped orderlines with LineActionType Ship can be updated.
     *
     * @export
     * @class UpdateOrderLineStockLabel
     * @implements {EvaService}
     */
    class UpdateOrderLineStockLabel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderLineStockLabel;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateOrderLink
     * @implements {EvaService}
     */
    class UpdateOrderLink implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderLink;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateOrderLinkType
     * @implements {EvaService}
     */
    class UpdateOrderLinkType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderLinkType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This services can be used to toggle some of the OrderOptions.

- IgnoreDiscounts: Discounts will not be added/removed/recalculated
- IgnoreShippingCosts: ShippingCosts will not be added/removed/recalculated
- IgnoreGiftWrappingCosts: GiftWrappingCosts will not be added/removed/recalculated
- AllowPartialFulfillment: The order will potentially be fulfilled in multiple batches
     *
     * @export
     * @class UpdateOrderOptions
     * @implements {EvaService}
     */
    class UpdateOrderOptions implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderOptions;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update the PickupAddress on an Order.

This can be done by providing a full Address in the Address property, or by referencing an Address in an AddressBook in the AddressBookID property.
     *
     * @export
     * @class UpdateOrderPickupAddress
     * @implements {EvaService}
     */
    class UpdateOrderPickupAddress implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderPickupAddress;
        response?: EVA.Core.UpdateOrderAddressResponse;
    }
    /**
     * Update the ShippingAddress on an Order.

This can be done by providing a full Address in the Address property, or by referencing an Address in an AddressBook in the AddressBookID property.
     *
     * @export
     * @class UpdateOrderShippingAddress
     * @implements {EvaService}
     */
    class UpdateOrderShippingAddress implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderShippingAddress;
        response?: EVA.Core.UpdateOrderAddressResponse;
    }
    /**
     * Updates the order shipping status, creating ledger entries on order level.

Will be processed as a whole; a single failing line in `Lines` will prevent all other updates in a single call from persisting.
     *
     * @export
     * @class UpdateOrderShippingStatus
     * @implements {EvaService}
     */
    class UpdateOrderShippingStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderShippingStatus;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an order verification reason
     *
     * @export
     * @class UpdateOrderVerificationReason
     * @implements {EvaService}
     */
    class UpdateOrderVerificationReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrderVerificationReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates the basic information associated with an OrganizationUnit. This service can be used to only update a subset of the settings of the OrganizationUnit by providing only the fields you want to update.

If a field is not part of the request message (or has value `undefined`) then the setting is not updated. If the field is present but has a value of `null` then it's assumed you want to unset the setting.
     *
     * @export
     * @class UpdateOrganizationUnit
     * @implements {EvaService}
     */
    class UpdateOrganizationUnit implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrganizationUnit;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates the OrganizationUnit's settings. This service can be used to only update a subset of the settings of the OrganizationUnit by providing only the fields you want to update.
If a field is not part of the request message (or has value `undefined`) then the setting is not updated. If the field is present but has a value of `null` then it's assumed you want to unset the setting.
     *
     * @export
     * @class UpdateOrganizationUnitSettings
     * @implements {EvaService}
     */
    class UpdateOrganizationUnitSettings implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrganizationUnitSettings;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for updating a existing `OrganizationUnitSupplier`
     *
     * @export
     * @class UpdateOrganizationUnitSupplier
     * @implements {EvaService}
     */
    class UpdateOrganizationUnitSupplier implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateOrganizationUnitSupplier;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is largely the same as CreateProductBundle.
The model passed into the request body is considered to be the entire bundle configuration.
Any lines that are currently present but not present in the update model are removed from the bundle.
The only parameter that is different is that the model requires an ID to be set for the root object and its lines, which you get back from the `GetProductBundle` and `ListProductBundles` services.
     *
     * @export
     * @class UpdateProductBundle
     * @implements {EvaService}
     */
    class UpdateProductBundle implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateProductBundle;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Replace the given ProductRequirement values for an OrderLine with new values
     *
     * @export
     * @class UpdateProductRequirementValuesForOrderLine
     * @implements {EvaService}
     */
    class UpdateProductRequirementValuesForOrderLine implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateProductRequirementValuesForOrderLine;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates an existing ProductZone.
     *
     * @export
     * @class UpdateProductZone
     * @implements {EvaService}
     */
    class UpdateProductZone implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateProductZone;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates an existing ProductZoneGroup.
     *
     * @export
     * @class UpdateProductZoneGroup
     * @implements {EvaService}
     */
    class UpdateProductZoneGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateProductZoneGroup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service for updating an existing `QuickBuy` by ID
     *
     * @export
     * @class UpdateQuickBuy
     * @implements {EvaService}
     */
    class UpdateQuickBuy implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateQuickBuy;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateRecurringTask
     * @implements {EvaService}
     */
    class UpdateRecurringTask implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateRecurringTask;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateRepair
     * @implements {EvaService}
     */
    class UpdateRepair implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateRepair;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates the deadline of a pending ReservationPickup task.
     *
     * @export
     * @class UpdateReservationPickupTaskDeadline
     * @implements {EvaService}
     */
    class UpdateReservationPickupTaskDeadline implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateReservationPickupTaskDeadline;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update the return to supplier order with Reason, ShipFrom- and ShipTo organization unit
     *
     * @export
     * @class UpdateReturnToSupplierOrder
     * @implements {EvaService}
     */
    class UpdateReturnToSupplierOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateReturnToSupplierOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update a ReturnToSupplierRequest

This is only allowed for requests in the state New

Properties that are not provided in the request, will not be updated.
     *
     * @export
     * @class UpdateReturnToSupplierRequest
     * @implements {EvaService}
     */
    class UpdateReturnToSupplierRequest implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateReturnToSupplierRequest;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates a script
     *
     * @export
     * @class UpdateScript
     * @implements {EvaService}
     */
    class UpdateScript implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateScript;
        response?: EVA.Core.UpdateScriptResponse;
    }
    /**
     * Update the serial number on an order line - when no serialnumber is provided, it will be removed.
     *
     * @export
     * @class UpdateSerialNumber
     * @implements {EvaService}
     */
    class UpdateSerialNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateSerialNumber;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates an existing Serial Number Configuration by it's `ID`.
     *
     * @export
     * @class UpdateSerialNumberConfiguration
     * @implements {EvaService}
     */
    class UpdateSerialNumberConfiguration implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateSerialNumberConfiguration;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update a Shipment. For shipments with status > Pending, only `ReceiveMethod`, `ShipmentDate` and `ExpectedDeliveryDate` can be updated.
Updating a shipment creates the new necessary shipping lines and packages.
Using the RemoveOtherPackagesAndLines property removes the existing ones.
     *
     * @export
     * @class UpdateShipment
     * @implements {EvaService}
     */
    class UpdateShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateShipment;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an existing shipment identifier.
     *
     * @export
     * @class UpdateShipmentIdentifier
     * @implements {EvaService}
     */
    class UpdateShipmentIdentifier implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateShipmentIdentifier;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates the shipment status, creating ledger entries on shipment level.

Will be processed as a whole; a single failing line in `Lines` will prevent all other updates in a single call from persisting.
     *
     * @export
     * @class UpdateShipmentStatus
     * @implements {EvaService}
     */
    class UpdateShipmentStatus implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateShipmentStatus;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update the prices in the pricelist specified.

This service takes a simplified approach of the pricing tools EVA offers, but that limits the options you have available. The price list to update may
only have a single active adjustment, and that has to be of type `MANUAL_INPUT`.

* The pricelist is identified by the `PriceListID` or the `BackendSystemID` and `BackendID`.
* Products can be identified by their (EVA) `ID`.
* Updates from this service are persisted in the background, which could take up to 15 minutes to process (unless you set `ForcePriceListRecalculation` to `true`)
     *
     * @export
     * @class UpdateSimplePrices
     * @implements {EvaService}
     */
    class UpdateSimplePrices implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateSimplePrices;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates the Sellable stock of the given products for specified `OrganizationUnitID`, or when omitted, your current organization unit.

Specify the new, absolute quantities as values in the `Products` list. Products that are not mentioned will not be modified.
     *
     * @export
     * @class UpdateStock
     * @implements {EvaService}
     */
    class UpdateStock implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateStock;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates Stock Replenishment task data.
     *
     * @export
     * @class UpdateStockReplenishmentTaskData
     * @implements {EvaService}
     */
    class UpdateStockReplenishmentTaskData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateStockReplenishmentTaskData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update the user identifier on a user subscription
     *
     * @export
     * @class UpdateSubscriptionUserIdentifier
     * @implements {EvaService}
     */
    class UpdateSubscriptionUserIdentifier implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateSubscriptionUserIdentifier;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateTaxCode
     * @implements {EvaService}
     */
    class UpdateTaxCode implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateTaxCode;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an existing TaxCodeMapping. Only one mapping per TaxCode & TaxProvider is allowed.
     *
     * @export
     * @class UpdateTaxCodeMapping
     * @implements {EvaService}
     */
    class UpdateTaxCodeMapping implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateTaxCodeMapping;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update an existing TaxProvider
     *
     * @export
     * @class UpdateTaxProvider
     * @implements {EvaService}
     */
    class UpdateTaxProvider implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateTaxProvider;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Modifies a tax rate.

:point_right: See `CreateTaxRate` service for more info.
     *
     * @export
     * @class UpdateTaxRate
     * @implements {EvaService}
     */
    class UpdateTaxRate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateTaxRate;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update properties on an existing User. Will throw an exception if one of the properties are invalid.

Properties not specified in the request will be ignored.
     *
     * @export
     * @class UpdateUser
     * @implements {EvaService}
     */
    class UpdateUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUser;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * This service is used to change the AccountType of a Customer.

When changing the AccountType from Standard to Incognito or Basic, all credentials and roles are removed.

When changing the AccountType from Incognito or Basic to Standard:
- Uniqueness checks are done for the EmailAddress and Nickname
- Default customer roles are generated
- When no password is provided, one is generated and returned in the response
- If configured, the user will receive an `welcome to EVA` email. If no valid password is provided, the email will also contain a generated password.
     *
     * @export
     * @class UpdateUserAccountType
     * @implements {EvaService}
     */
    class UpdateUserAccountType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserAccountType;
        response?: EVA.Core.UpdateUserAccountTypeResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateUserAssociation
     * @implements {EvaService}
     */
    class UpdateUserAssociation implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserAssociation;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateUserAssociationType
     * @implements {EvaService}
     */
    class UpdateUserAssociationType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserAssociationType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateUserDebtorData
     * @implements {EvaService}
     */
    class UpdateUserDebtorData implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserDebtorData;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateUserGroup
     * @implements {EvaService}
     */
    class UpdateUserGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserGroup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates a UserInteraction. A UserInteraction logs an interaction between an employee and a customer, for example the fact the customer was called.

UserID is optional but only if OrderID is also provided and the given Order is attached to a customer. If so then the UserInteraction is attached to the Order.Customer.
     *
     * @export
     * @class UpdateUserInteraction
     * @implements {EvaService}
     */
    class UpdateUserInteraction implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserInteraction;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates a UserPhoneNumber based on its ID. This service allows you to change the PhoneNumber (required), Type, Description.
    If you want to change what number is the primary number, use the `MakeUserPhoneNumberPrimary` service.

This service only allows the number's own user or an employee to update it.
     *
     * @export
     * @class UpdateUserPhoneNumber
     * @implements {EvaService}
     */
    class UpdateUserPhoneNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserPhoneNumber;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateUserTaskNotCompletedReason
     * @implements {EvaService}
     */
    class UpdateUserTaskNotCompletedReason implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserTaskNotCompletedReason;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates an existing User Task Priority.
     *
     * @export
     * @class UpdateUserTaskPriority
     * @implements {EvaService}
     */
    class UpdateUserTaskPriority implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserTaskPriority;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateUserTaskType
     * @implements {EvaService}
     */
    class UpdateUserTaskType implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserTaskType;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class UpdateUserTaskTypeOrganizationUnitSet
     * @implements {EvaService}
     */
    class UpdateUserTaskTypeOrganizationUnitSet implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserTaskTypeOrganizationUnitSet;
        response?: EVA.Core.UserTaskTypeOrganizationUnitSetResponse;
    }
    /**
     * This service is used to change the VisibilityGroup of a User.

To change the VisibilityGroup of a User, the following conditions must be met:
- The User must be visable and editable by the current User
- The VisibilityGroup must be visible for Users by the current User, the service GetUsersVisibilityGroups can be used to get a list of all available VisibilityGroups for the current User.
- If the User is a standard account, there must be no duplicate standard account in the new VisibilityGroup.
     *
     * @export
     * @class UpdateUserVisibilityGroup
     * @implements {EvaService}
     */
    class UpdateUserVisibilityGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserVisibilityGroup;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Update the user wishlist.
     *
     * @export
     * @class UpdateUserWishlist
     * @implements {EvaService}
     */
    class UpdateUserWishlist implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateUserWishlist;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Updates a Zoned Cycle Count Schedule.
     *
     * @export
     * @class UpdateZonedCycleCountSchedule
     * @implements {EvaService}
     */
    class UpdateZonedCycleCountSchedule implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpdateZonedCycleCountSchedule;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Service to validate and upgrade tickets to invoices based on order details.
     *
     * @export
     * @class UpgradeInvoice
     * @implements {EvaService}
     */
    class UpgradeInvoice implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UpgradeInvoice;
        response?: EVA.Core.UpgradeInvoiceResponse;
    }
    /**
     * Upload stock Excel (delta)file. The service processes the data async. This service is intended for small files (up to 5MB). Please view the documentation for processing larger files.
     *
     * @export
     * @class UploadAndProcessStockExcelFile
     * @implements {EvaService}
     */
    class UploadAndProcessStockExcelFile implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UploadAndProcessStockExcelFile;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Upload stock JSON (delta)file. The service processes the data async.This service is intended for small files (up to 5MB). Please view the documentation for processing larger files.
     *
     * @export
     * @class UploadAndProcessStockJSONFile
     * @implements {EvaService}
     */
    class UploadAndProcessStockJSONFile implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UploadAndProcessStockJSONFile;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Uploads an Excel retrieved from `DownloadCycleCountSample`.
     *
     * @export
     * @class UploadCycleCountExcel
     * @implements {EvaService}
     */
    class UploadCycleCountExcel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UploadCycleCountExcel;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Uploads an Excel retrieved from `DownloadCycleCountSample`.
     *
     * @export
     * @class UploadCycleCountExcel_Async
     * @implements {EvaService}
     */
    class UploadCycleCountExcel_Async implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UploadCycleCountExcel;
        response?: EVA.Core.AsyncRequestResponse;
    }
    /**
     * Uploads an Excel retrieved from `DownloadCycleCountSample`.
     *
     * @export
     * @class UploadCycleCountExcel_AsyncResult
     * @implements {EvaService}
     */
    class UploadCycleCountExcel_AsyncResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.AsyncRequestResultRequest;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Upload full stock count result. Previous uploads will be overwritten. The service processes the data async.
     *
     * @export
     * @class UploadExternalFullStockCountResult
     * @implements {EvaService}
     */
    class UploadExternalFullStockCountResult implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UploadExternalFullStockCountResult;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Upload an excel with labels to the specified full stock count.
Caution! The Name column can only be max 50 chars. Names larger then that will be rejected.
Caution! This service will not change existing labels, but only add new ones. Duplicate detection is done on case insensitve Name in combination with StockLabel.
     *
     * @export
     * @class UploadFullStockCountLabels
     * @implements {EvaService}
     */
    class UploadFullStockCountLabels implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UploadFullStockCountLabels;
        response?: EVA.Core.UploadFullStockCountLabelsResponse;
    }
    /**
     * Upload a filled Excel file with special opening hours for an opening hour template. The template Excel file should be downloaded from the `DownloadSpecialOpeningHoursSample` service.
     *
     * @export
     * @class UploadSpecialOpeningHoursExcel
     * @implements {EvaService}
     */
    class UploadSpecialOpeningHoursExcel implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.UploadSpecialOpeningHoursExcel;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Before calling Create/UpdateCycleCountZone group you can call this service to perform some checks on the validity of the new zone group.
Currently it only returns warnings and calling it is not required before creating or updating the group, but it is recommended as it prevents confusing situations such as the same OrganizationUnit being part
of multiple groups at the same time.
     *
     * @export
     * @class ValidateCycleCountZoneGroup
     * @implements {EvaService}
     */
    class ValidateCycleCountZoneGroup implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateCycleCountZoneGroup;
        response?: EVA.Core.ValidateCycleCountZoneGroupResponse;
    }
    /**
     * Validate if all conditions are met to schedule the (specified lines of) specified order.

_Will check the default ExportValidator and check OrderRequirements for all actions._
     *
     * @export
     * @class ValidateExportSchedule
     * @implements {EvaService}
     */
    class ValidateExportSchedule implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateExportSchedule;
        response?: EVA.Core.ValidateExportScheduleResponse;
    }
    /**
     * Validates if the given PIN is valid for the current user/ou.
     *
     * @export
     * @class ValidateIdentificationPin
     * @implements {EvaService}
     */
    class ValidateIdentificationPin implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateIdentificationPin;
        response?: EVA.Core.ValidateIdentificationPinResponse;
    }
    /**
     * Check if given `InvoiceNumber` is valid (eg. does not already exist & passes other validations).
     *
     * @export
     * @class ValidateInvoiceNumber
     * @implements {EvaService}
     */
    class ValidateInvoiceNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateInvoiceNumber;
        response?: EVA.Core.ValidateInvoiceNumberResponse;
    }
    /**
     * Validate an OpeningHoursTemplate
     *
     * @export
     * @class ValidateOpeningHoursTemplate
     * @implements {EvaService}
     */
    class ValidateOpeningHoursTemplate implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateOpeningHoursTemplate;
        response?: EVA.Core.ValidateOpeningHoursTemplateResponse;
    }
    /**
     * Validates if a (part of an) Order can be shipped in the current state.

Returns if it is valid (`IsValid`) and if not, how much still needs to be paid (`OpenAmountForShipment`).
     *
     * @export
     * @class ValidateOrderShipment
     * @implements {EvaService}
     */
    class ValidateOrderShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateOrderShipment;
        response?: EVA.Core.ValidateOrderShipmentResponse;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ValidatePreCount
     * @implements {EvaService}
     */
    class ValidatePreCount implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidatePreCount;
        response?: EVA.Core.ValidatePreCountResponse;
    }
    /**
     * Validate the required data for the order
     *
     * @export
     * @class ValidateRequiredDataForOrder
     * @implements {EvaService}
     */
    class ValidateRequiredDataForOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateRequiredDataForOrder;
        response?: EVA.Core.ValidateRequiredDataForOrderResponse;
    }
    /**
     * Validate a ReturnToSupplierRequest

This will check if for all OrganizationUnits in the set a Supplier could be determined.

The service will return all available suppliers for each OrganizationUnit and will pick it when a single one is available.
When no Suppliers are available, no Order can be generated.
If more than one Supplier is available, the correct one should be picked with the SetSuppliersForReturnToSupplierRequest service.
     *
     * @export
     * @class ValidateReturnToSupplierRequest
     * @implements {EvaService}
     */
    class ValidateReturnToSupplierRequest implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateReturnToSupplierRequest;
        response?: EVA.Core.ValidateReturnToSupplierRequestResponse;
    }
    /**
     * Validates if a Shipment can be shipped in the current state.

Returns if it is valid (`IsValid`) and if not, how much still needs to be paid (`OpenAmountForShipment`).
     *
     * @export
     * @class ValidateShipment
     * @implements {EvaService}
     */
    class ValidateShipment implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateShipment;
        response?: EVA.Core.ValidateShipmentResponse;
    }
    /**
     * Stub service to validate the current token
     *
     * @export
     * @class ValidateToken
     * @implements {EvaService}
     */
    class ValidateToken implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateToken;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * TODO: Needs documentation
     *
     * @export
     * @class ValidateUser
     * @implements {EvaService}
     */
    class ValidateUser implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateUser;
        response?: EVA.Core.ValidateUserResponse;
    }
    /**
     * Sends an email validation code to your current user's email address.
     *
     * @export
     * @class ValidateUserEmailAddress
     * @implements {EvaService}
     */
    class ValidateUserEmailAddress implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateUserEmailAddress;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Validates the given VAT number by the current organization unit auditing provider or by providing a country in the request
     *
     * @export
     * @class ValidateVatNumber
     * @implements {EvaService}
     */
    class ValidateVatNumber implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.ValidateVatNumber;
        response?: EVA.Core.ValidateVatNumberResponse;
    }
    /**
     * Verify the customer of an order by a QR code check. This process is used to validate that the person that picks up an order is really the person that ordered it.
     *
     * @export
     * @class VerifyCustomer
     * @implements {EvaService}
     */
    class VerifyCustomer implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.VerifyCustomer;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Flag the Order that the customer has been verified for AML purposes.
     *
     * @export
     * @class VerifyCustomerOnOrder
     * @implements {EvaService}
     */
    class VerifyCustomerOnOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.VerifyCustomerOnOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
    /**
     * Verify an Order that has been flagged with a CustomOrderStatus `NeedsEmployeeVerification`. Will try to place the order after the verification.
     *
     * @export
     * @class VerifyOrder
     * @implements {EvaService}
     */
    class VerifyOrder implements IEvaServiceDefinition {
        name: string;
        path: string;
        request?: EVA.Core.VerifyOrder;
        response?: EVA.Core.EmptyResponseMessage;
    }
}
//# sourceMappingURL=EVA.Core.Services.d.ts.map