declare module EVA.Core.Management {

  export namespace Errors {
    export const enum UserOrigins {
      /**
       * This UserOrigin already exists
       */
      AlreadyExists = 'UserOrigins:AlreadyExists'
    }

    export const enum CreateEmployee {
      /**
       * This user already exists, use the UpdateUser service.
       */
      UserAlreadyExists = 'CreateEmployee:UserAlreadyExists',
      /**
       * There is already an existing customer.
       */
      ExistingCustomer = 'CreateEmployee:ExistingCustomer'
    }

    export const enum GenerateIdentificationCode {
      /**
       * Cannot generate identification codes for Single Sign-On users.
       */
      CannotGenerateIdentificationCodeForSingleSignOn = 'GenerateIdentificationCode:CannotGenerateIdentificationCodeForSingleSignOn'
    }

    export const enum TaxService {
      /**
       * StartDate ({0:string}) cannot be after EndDate: ({1:string})
       */
      StartDateShouldBeBeforeEndDate = 'TaxService:StartDateShouldBeBeforeEndDate',
      /**
       * StartDate cannot exist without EndDate and vice-versa
       */
      StartAndEndDateRequired = 'TaxService:StartAndEndDateRequired',
      /**
       * Validation of the Excel file failed: {0:string}
       */
      InvalidExcelFile = 'TaxService:InvalidExcelFile'
    }

    export const enum StockAllocations {
      /**
       * It's required to specify an OrganizationUnit to create this StockAllocationRule.
       */
      OrganizationUnitRequired = 'StockAllocations:OrganizationUnitRequired',
      /**
       * OrganizationUnit {0:number} is not directly supplied by OrganizationUnit {1:number}.
       */
      InvalidOrganizationUnit = 'StockAllocations:InvalidOrganizationUnit'
    }

    export const enum StockAllocationRules {
      /**
       * Validation of the Excel file failed: {0:string}
       */
      UploadStockAllocationRuleValidationError = 'StockAllocationRules:UploadStockAllocationRuleValidationError'
    }

    export const enum StockLabels {
      /**
       * StockLabel with name '{0:string}' already exists
       */
      AlreadyExists = 'StockLabels:AlreadyExists'
    }

    export const enum OrganizationUnitShippingMethod {
      /**
       * OrganizationUnitShippingMethod already exists
       */
      OrganizationUnitShippingMethodAlreadyExists = 'OrganizationUnitShippingMethod:OrganizationUnitShippingMethodAlreadyExists'
    }

    export const enum Roles {
      /**
       * The code `{0:string}` is preserved for system roles.
       */
      CodeIsPreservedForSystemRoles = 'Roles:CodeIsPreservedForSystemRoles',
      /**
       * Cannot modify or delete New Black system roles.
       */
      CannotModifyOrDeleteNewBlackRole = 'Roles:CannotModifyOrDeleteNewBlackRole'
    }

    export const enum RoleSets {
      /**
       * A role set with BackendID '{0:string}' already exists
       */
      RoleSetWithBackendIDAlreadyExists = 'RoleSets:RoleSetWithBackendIDAlreadyExists',
      /**
       * Role set '{0:string}' is still attached to organization units
       */
      RoleSetIsStillAttached = 'RoleSets:RoleSetIsStillAttached'
    }

    export const enum ProductBarcodes {
      /**
       * Barcode {barcode:string} already exists for ProductID {productID:number}
       */
      BarcodeAlreadyExists = 'ProductBarcodes:BarcodeAlreadyExists',
      /**
       * Product not found
       */
      ProductNotFound = 'ProductBarcodes:ProductNotFound',
      /**
       * UnitOfMeasure not found
       */
      UnitOfMeasureNotFound = 'ProductBarcodes:UnitOfMeasureNotFound'
    }

    export const enum ProductRequirements {
      /**
       * The supplied backendid already exists.
       */
      DuplicateBackendID = 'ProductRequirements:DuplicateBackendID'
    }

    export const enum ProductUnitOfMeasures {
      /**
       * Product not found
       */
      ProductNotFound = 'ProductUnitOfMeasures:ProductNotFound',
      /**
       * UnitOfMeasure not found
       */
      UnitOfMeasureNotFound = 'ProductUnitOfMeasures:UnitOfMeasureNotFound',
      /**
       * The Quantity should be greather than 0
       */
      InvalidQuantity = 'ProductUnitOfMeasures:InvalidQuantity'
    }

    export const enum UnitOfMeasures {
      /**
       * This UnitOfMeasure already exists
       */
      AlreadyExists = 'UnitOfMeasures:AlreadyExists'
    }

    export const enum Pricing {
      /**
       * A BackendID cannot be defined without a BackendSystemID and vice versa.
       */
      BackendIdentifiersRequired = 'Pricing:BackendIdentifiersRequired',
      /**
       * PriceList {0:number} has the same BackendID.
       */
      DuplicateBackendID = 'Pricing:DuplicateBackendID',
      /**
       * You do not have access to pricelist {0:number}.
       */
      CannotViewPriceList = 'Pricing:CannotViewPriceList'
    }

    export const enum PriceLists {
      /**
       * Cannot create relation to historical price list
       */
      CannotCreateRelationToHistoricalPriceList = 'PriceLists:CannotCreateRelationToHistoricalPriceList',
      /**
       * The given pricelist has a SpecialPriceListID, which is already attached to the given OU. SpecialPriceList is deprecated and should be remove from the pricelist.
       */
      SpecialPriceListIsAlreadyAttachedToOrganizationUnit = 'PriceLists:SpecialPriceListIsAlreadyAttachedToOrganizationUnit'
    }

    export const enum PaymentTransactionLedgerTypes {
      /**
       * This PaymentTransactionLedgerType already exists
       */
      AlreadyExists = 'PaymentTransactionLedgerTypes:AlreadyExists',
      /**
       * This PaymentTransactionLedgerType is internal and can't be modified
       */
      IsInternal = 'PaymentTransactionLedgerTypes:IsInternal'
    }

    export const enum OrganizationUnitSets {
      /**
       * The OrganizationUnitSetScope is still being used, cannot delete.
       */
      CannotDeleteScopeInUse = 'OrganizationUnitSets:CannotDeleteScopeInUse',
      /**
       * The OrganizationUnitSet is still being used, cannot delete.
       */
      CannotDeleteInUse = 'OrganizationUnitSets:CannotDeleteInUse',
      /**
       * Set {0:string} is a set managed by EVA and cannot be modified.
       */
      CannotModifySystemSet = 'OrganizationUnitSets:CannotModifySystemSet',
      /**
       * It's not possible to use OrganizationUnitSetTypes.System to create new sets.
       */
      CannotCreateSystemSet = 'OrganizationUnitSets:CannotCreateSystemSet',
      /**
       * It's not possible to use an AdHoc set as a subset of another set.
       */
      CannotUseAdHocSetsAsSubset = 'OrganizationUnitSets:CannotUseAdHocSetsAsSubset',
      /**
       * Name is required when creating a non-AdHoc set.
       */
      NameIsRequired = 'OrganizationUnitSets:NameIsRequired'
    }

    export const enum OpeningHoursTypes {
      /**
       * This OpeningHoursType already exists
       */
      AlreadyExists = 'OpeningHoursTypes:AlreadyExists',
      /**
       * The name of an internal OpeningHoursType cannot be modified.
       */
      NameOfInternalOpeningHourTypeCannotBeModified = 'OpeningHoursTypes:NameOfInternalOpeningHourTypeCannotBeModified',
      /**
       * This OpeningHoursType is internal and cannot be deleted
       */
      InternalTypesCannotBeDeleted = 'OpeningHoursTypes:InternalTypesCannotBeDeleted',
      /**
       * This OpeningHoursType has (exception)templates attached and cannot be deleted
       */
      TypeHasTemplatesAndCannotBeDeleted = 'OpeningHoursTypes:TypeHasTemplatesAndCannotBeDeleted'
    }

    export const enum Cultures {
      /**
       * This currency already exists for this OrganizationUnit
       */
      CurrencyAlreadyExists = 'Cultures:CurrencyAlreadyExists',
      /**
       * Additional currency must differ from main currency.
       */
      AdditionalCurrencyMustDifferFromMainCurrency = 'Cultures:AdditionalCurrencyMustDifferFromMainCurrency',
      /**
       * The selected cashhandler is not available for the given currency
       */
      CashHandlerCurrencyMismatch = 'Cultures:CashHandlerCurrencyMismatch',
      /**
       * This language already exists for this organization unit
       */
      LanguageAlreadyExists = 'Cultures:LanguageAlreadyExists',
      /**
       * The organization unit must have at least one (inherited) language. Add a new language before removing this language.
       */
      MustHaveAtLeastOneLanguage = 'Cultures:MustHaveAtLeastOneLanguage'
    }

    export const enum OpeningHours {
      /**
       * There already exists opening hours for organization {organizationUnitName:string} and day {dayOfWeek:EVA.Core.DayOfWeek}
       */
      DuplicateOpeningHoursDayOfWeek = 'OpeningHours:DuplicateOpeningHoursDayOfWeek',
      /**
       * There already exists opening hours for organization {organizationUnitName:string} and date {date:string}
       */
      DuplicateOpeningHoursDate = 'OpeningHours:DuplicateOpeningHoursDate'
    }

    export const enum CreateOrderLedgerType {
      /**
       * Can't create OrderLedgerType because its name already exists
       */
      OrderLedgerTypeAlreadyExists = 'CreateOrderLedgerType:OrderLedgerTypeAlreadyExists'
    }

    export const enum DeleteOrderLedgerType {
      /**
       * Can't delete OrderLedgerType because it's not custom
       */
      OrderLedgerTypeNotCustom = 'DeleteOrderLedgerType:OrderLedgerTypeNotCustom'
    }

    export const enum RefundCorrectionReasons {
      /**
       * RefundCorrectionReason with BackendID '{0:string}' already exists.
       */
      AlreadyExists = 'RefundCorrectionReasons:AlreadyExists'
    }

    export const enum UnitPriceCorrectionReasons {
      /**
       * UnitPriceCorrectionReason with BackendID '{0:string}' already exists.
       */
      AlreadyExists = 'UnitPriceCorrectionReasons:AlreadyExists'
    }

    export const enum UpdateOrderLedgerType {
      /**
       * Can't update OrderLedgerType because it's not custom
       */
      OrderLedgerTypeNotCustom = 'UpdateOrderLedgerType:OrderLedgerTypeNotCustom',
      /**
       * Can't update OrderLedgerType because its name already exists
       */
      OrderLedgerTypeAlreadyExists = 'UpdateOrderLedgerType:OrderLedgerTypeAlreadyExists'
    }

    export const enum ReturnReasons {
      /**
       * This return reason already exists.
       */
      AlreadyExists = 'ReturnReasons:AlreadyExists',
      /**
       * Remark is required for this return reason.
       */
      RemarkIsRequired = 'ReturnReasons:RemarkIsRequired'
    }

    export const enum ClientApplications {
      /**
       * The requested BackendID is already in use
       */
      DuplicateBackendID = 'ClientApplications:DuplicateBackendID'
    }

    export const enum GiftCardConfigurations {
      /**
       * Handler is required
       */
      MissingHandler = 'GiftCardConfigurations:MissingHandler',
      /**
       * The given handler doesn't exist
       */
      UnknownHandler = 'GiftCardConfigurations:UnknownHandler',
      /**
       * The data is not in a valid format for the selected type
       */
      InvalidData = 'GiftCardConfigurations:InvalidData',
      /**
       * Usage type needs at least Sale or Payment, never None
       */
      InvalidUsageType = 'GiftCardConfigurations:InvalidUsageType',
      /**
       * The selected usage type is not supported for the selected handler
       */
      InvalidUsageTypeForHandler = 'GiftCardConfigurations:InvalidUsageTypeForHandler'
    }

    export const enum GiftCards {
      /**
       * This product already has a giftcard product
       */
      ProductAlreadyExists = 'GiftCards:ProductAlreadyExists',
      /**
       * Handler is required
       */
      MissingHandler = 'GiftCards:MissingHandler',
      /**
       * The given type doesn't exist
       */
      UnknownType = 'GiftCards:UnknownType',
      /**
       * The data is not in a valid format for the selected type
       */
      InvalidData = 'GiftCards:InvalidData',
      /**
       * The given product isn't a giftcard
       */
      InvalidProductType = 'GiftCards:InvalidProductType'
    }

    export const enum CashExpenseTypes {
      /**
       * CashExpenseType {0:string} already exists.
       */
      AlreadyExists = 'CashExpenseTypes:AlreadyExists'
    }

    export const enum ExportProductCostPriceLedgerSummary {
      /**
       * You can only export up to 100 products at a time.
       */
      TooManyProductsSelected = 'ExportProductCostPriceLedgerSummary:TooManyProductsSelected'
    }

    export const enum DiscountCouponServices {
      /**
       * UserID is required when logged in user is a Employee or API user.
       */
      UserIDRequired = 'DiscountCouponServices:UserIDRequired'
    }

    export const enum DiscountService {
      /**
       * Discount adheres to campaign start- and end date/time
       */
      DiscountAdheresToCampaignTime = 'DiscountService:DiscountAdheresToCampaignTime',
      /**
       * You don't have permission to set discount budget information.
       */
      NotAllowedToSetDiscountBudgetData = 'DiscountService:NotAllowedToSetDiscountBudgetData'
    }

    export const enum Culture {
      /**
       * Culture for country {language:string} and language {country:string} already exists.
       */
      CultureAlreadyExists = 'Culture:CultureAlreadyExists'
    }

    export const enum SettingsExcelImporter {
      /**
       * OrganizationUnit with ID {0:string} not found on row {1:number}
       */
      OrganizationUnitByIDNotFound = 'SettingsExcelImporter:OrganizationUnitByIDNotFound',
      /**
       * OrganizationUnit with BackendID {0:string} not found on row {1:number}
       */
      OrganizationUnitByBackendIDNotFound = 'SettingsExcelImporter:OrganizationUnitByBackendIDNotFound',
      /**
       * No BackendID or OrganizationUnitID provided for row {0:number}
       */
      NoOrganizationUnitIdentifier = 'SettingsExcelImporter:NoOrganizationUnitIdentifier',
      /**
       * Value on row {0:number} looks like a masked value
       */
      MaskedValueImport = 'SettingsExcelImporter:MaskedValueImport'
    }

    export const enum LoyaltyProgram {
      /**
       * Field '{0:string}' is required for the creation of a new Loyalty Program.
       */
      MissingFieldOnLoyaltyProgramCreation = 'LoyaltyProgram:MissingFieldOnLoyaltyProgramCreation'
    }

  }


  /**
   * List the users bought products.
   */
  export interface ListUserBoughtProducts {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListUserBoughtProductsFilter>;
    PageToken?: string;
  }

  export interface ListUserBoughtProductsResponse {
    Results?: ListUserBoughtProductsResponse.BoughtProduct[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListUserBoughtProductsResponse {
    export interface BoughtProduct {
      /**
       * Entity type: UserBoughtProduct
       */
      ID: number;
      /**
       * Entity type: User
       */
      UserID: number;
      UserFullName: string;
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductBackendID: string;
      ProductCustomID: string;
      ProductName: string;
      PrimaryImage: EVA.Core.IProductSearchItemPrimaryImage;
      SerialNumber?: string;
      LastPurchaseDate?: string;
      LastPurchasedQuantity?: number;
      TotalPurchasedQuantity?: number;
      IsUnderWarranty: boolean;
      WarrantyExpiryDate?: string;
      ManufacturerWarrantyExpiryDate?: string;
    }

  }

  /**
   * Get the user bought product by ID.
   */
  export interface GetUserBoughtProduct {
    /**
     * Entity type: UserBoughtProduct
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
  }

  export interface GetUserBoughtProductResponse {
    /**
     * Entity type: UserBoughtProduct
     */
    ID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    UserFullName: string;
    Product: any;
    SerialNumber?: string;
    LastPurchaseDate?: string;
    LastPurchasedQuantity?: number;
    TotalPurchasedQuantity?: number;
    IsUnderWarranty: boolean;
    WarrantyExpiryDate?: string;
    ManufacturerWarrantyExpiryDate?: string;
    AdditionalWarrantyInfo?: string;
    Details: GetUserBoughtProductResponse.Detail[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetUserBoughtProductResponse {
    export interface Detail {
      /**
       * Entity type: UserBoughtProductDetail
       */
      ID?: number;
      FromExternal?: boolean;
      BackendSystemID?: string;
      BackendID?: string;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      OrderDisplayID?: string;
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      PurchaseDate?: string;
      PurchasedQuantity?: number;
    }

  }

  /**
   * Get a user bought product by a product's serial number.
   */
  export interface GetUserBoughtProductBySerialNumber {
    SerialNumber: string;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
  }

  /**
   * Register user bought product.
   */
  export interface RegisterUserBoughtProduct {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    SerialNumber?: string;
    IsUnderWarranty?: boolean;
    WarrantyExpiryDate?: string;
    ManufacturerWarrantyExpiryDate?: string;
    AdditionalWarrantyInfo?: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Organization unit that the user bought product is registered in. Ignored when an order/line organization unit is present. When omitted, registration is under the current organization unit.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    PurchaseDate?: string;
    PurchasedQuantity?: number;
  }

  export interface UserBoughtProductResponse {
    /**
     * Entity type: UserBoughtProduct
     */
    UserBoughtProductID: number;
    /**
     * Entity type: UserBoughtProductDetail
     */
    UserBoughtProductDetailID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update user bought product warranty.
   */
  export interface UpdateUserBoughtProductWarranty {
    /**
     * Entity type: UserBoughtProduct
     */
    UserBoughtProductID?: number;
    IsUnderWarranty?: boolean;
    WarrantyExpiryDate?: string;
    ManufacturerWarrantyExpiryDate?: string;
    AdditionalWarrantyInfo?: string;
  }

  /**
   * Register user bought product.
   */
  export interface AddUserBoughtProductDetail {
    /**
     * Entity type: UserBoughtProduct
     */
    UserBoughtProductID: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    PurchaseDate?: string;
    PurchasedQuantity: number;
  }

  /**
   * Register user bought product.
   */
  export interface UpdateUserBoughtProductDetail {
    /**
     * Entity type: UserBoughtProductDetail
     */
    ID: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Organization unit that the user bought product is registered in. Ignored when an order/line organization unit is present. When omitted, registration is under the current organization unit.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    PurchaseDate?: string;
    PurchasedQuantity: number;
  }

  /**
   * Deletes a user bought product detail.
   */
  export interface DeleteUserBoughtProductDetail {
    /**
     * Entity type: UserBoughtProductDetail
     */
    ID: number;
  }

  /**
   * Archive the user bought product.
   */
  export interface ArchiveUserBoughtProduct {
    /**
     * Entity type: UserBoughtProduct
     */
    ID: number;
  }

  /**
   * Push user bought product.
   */
  export interface PushUserBoughtProduct {
    /**
     * Entity type: User
     */
    UserID: number;
    BoughtProducts?: PushUserBoughtProduct.PushedBoughtProduct[];
  }

  export namespace PushUserBoughtProduct {
    export interface PushedBoughtProduct {
      /**
       * Entity type: Product
       */
      ProductID: number;
      SerialNumber?: string;
      IsUnderWarranty?: boolean;
      WarrantyExpiryDate?: string;
      ManufacturerWarrantyExpiryDate?: string;
      AdditionalWarrantyInfo?: string;
      Archive: boolean;
      Details: PushUserBoughtProduct.PushedBoughtProductDetail[];
    }

    export interface PushedBoughtProductDetail {
      BackendID?: string;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Organization unit that the user bought product is registered in. Ignored when an order/line organization unit is present. When omitted, registration is under the current organization unit.
       */
      OrganizationUnitID?: string;
      PurchaseDate?: string;
      PurchasedQuantity?: number;
    }

  }

  /**
   * Push bought products for users. Only accepts 10000 users per request
   */
  export interface PushBoughtProductsForUsers {
    BoughtProductsForUsers?: PushBoughtProductsForUsers.PushedBoughtProductForUser[];
  }

  export namespace PushBoughtProductsForUsers {
    export interface PushedBoughtProductForUser {
      /**
       * Entity type: User
       */
      UserID: number;
      BoughtProducts: PushedBoughtProduct[];
    }

  }

  export interface PushedBoughtProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    SerialNumber?: string;
    IsUnderWarranty?: boolean;
    WarrantyExpiryDate?: string;
    ManufacturerWarrantyExpiryDate?: string;
    AdditionalWarrantyInfo?: string;
    Archive: boolean;
    Details: PushedBoughtProduct.PushedBoughtProductDetail[];
  }

  export namespace PushedBoughtProduct {
    export interface PushedBoughtProductDetail {
      BackendID?: string;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Organization unit that the user bought product is registered in. Ignored when an order/line organization unit is present. When omitted, registration is under the current organization unit.
       */
      OrganizationUnitID?: string;
      PurchaseDate?: string;
      PurchasedQuantity?: number;
    }

  }

  export interface PushBoughtProductsForUsersResponse {
    FailedBoughtProductsForUsers?: PushBoughtProductsForUsersResponse.FailedUserBoughtProductsForUser[];
    SuccessfulBoughtProductsForUsers?: PushBoughtProductsForUsersResponse.SuccessfulUserBoughtProductsForUser[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace PushBoughtProductsForUsersResponse {
    export interface SuccessfulUserBoughtProductsForUser {
      /**
       * Entity type: User
       */
      UserID: number;
      UserBackendID: string;
      BoughtProducts: PushBoughtProductsForUsersResponse.BoughtProduct[];
    }

    export namespace SuccessfulUserBoughtProductsForUser {
      export interface SuccessfulBoughtProduct {
        /**
         * Entity type: UserBoughtProduct
         */
        UserBoughtProductID: number;
        /**
         * Entity type: Product
         */
        ProductID?: number;
        ProductBackendID?: string;
        SerialNumber?: string;
      }

    }

    export interface FailedUserBoughtProductsForUser {
      /**
       * Entity type: User
       */
      UserID: number;
      UserBackendID: string;
      BoughtProducts: PushBoughtProductsForUsersResponse.BoughtProduct[];
    }

    export interface BoughtProduct {
      /**
       * Entity type: Product
       */
      ProductID?: number;
      ProductBackendID?: string;
      SerialNumber?: string;
    }

  }

  /**
   * Creates a new PIN login for current user.
   * 
   * This PIN will only be valid for a limited amount of time, and only for the provided `OrganizationUnitID` (or your current organization unit if left null).
   */
  export interface CreateIdentificationPinForEmployee {
    Pin: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface CreateIdentificationPinForEmployeeResponse {
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GenerateIdentificationPinForEmployee {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface GenerateIdentificationPinForEmployeeResponse {
    Pin: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Download an Excel sample which can be filled with employees. The populated excel can be uploaded through the `UploadEmployees` service.
   */
  export interface DownloadEmployeesSample {
  }

  /**
   * Upload the Excel based on the sample from `DownloadEmployeesSample`. The uploaded file will be processed in the background and the results will be mailed to the uploader.
   */
  export interface UploadEmployees {
    Data: string;
    /**
     * Skip the configured UserRequirement for Employees when processing the excelsheet, these requirements however will be enforced when modifying the employee.
     */
    IgnoreUserRequirements?: boolean;
  }

  export interface DownloadInitialInventorySample {
  }

  export interface UploadInitialInventory {
    Data: string;
    IgnoreInTransitStock?: boolean;
  }

  export interface UploadInitialInventoryResponse {
    Messages: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListRestockedProducts {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    From: string;
    To?: string;
    /**
     * Entity type: StockLabel
     */
    StockLabelIDs: number[];
  }

  export interface ListRestockedProductsResponse {
    /**
     * Entity type: StockLabel
     */
    Results: { [key: number]: number[] };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the Stock per product/stocklabel for a subset of OrganizationUnits.
   * 
   * The page size `Limit` has a maximum value of `1.024` for this service (unless `DownloadOverview` is `true`).
   * 
   * If neither `OrganizationUnitIDs` nor `OrganizationUnitSetID` are specified, the service will return an empty result.
   */
  export interface ListStockForOrganizationUnits {
    /**
     * The organization units to list the stock for, will be ignored if `OrganizationUnitSetID` is specified
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * The organization units to list the stock for, when specified, `OrganizationUnitIDs` will be ignored
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Filter on products, is used as input in ElasticSearch
     */
    Query?: string;
    /**
     * View the stock on a certain moment in time
     */
    DateTime?: string;
    /**
     * When `true`, the stock will be grouped by OrganizationUnit, if `false`, the stock is summed across all (specified) organizationunits.
     */
    SplitByOrganizationUnit?: boolean;
    /**
     * If specified as true, stock with a quantity of zero is also returned by this service. Defaults to false, meaning that products with a zero stock quantity will not be returned by this service.
     */
    IncludeZeroQuantityStock?: boolean;
    PageConfig?: EVA.Core.PageConfig<ListStockForOrganizationUnitsFilter>;
  }

  export interface ListStockForOrganizationUnitsFilter {
    /**
     * If other `BackendID` and/or `CustomID` are specified as well, only products that match all those filters will be returned
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * If other filters are specified as well, only products matching all the filters will be returned
     */
    SerialNumber?: string;
  }

  export interface ListStockForOrganizationUnitsResponse {
    /**
     * Entity type: Product
     */
    Products: { [key: number]: ListStockForOrganizationUnitsResponse.ProductDto };
    Url: string;
    Result?: EVA.Core.PagedResult<ListStockForOrganizationUnitsResponse.StockDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListStockForOrganizationUnitsResponse {
    export interface StockDto {
      /**
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Entity type: StockLabel
       */
      StockLabel: number;
      StockLabelName: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      OrganizationUnitName: string;
      OrganizationUnitCurrencyID: string;
      QuantityOnHand: number;
      QuantityCommitted: number;
      QuantityAvailable: number;
      /**
       * `UnitPriceInTax` times the `QuantityOnHand`. Rounded to the currency's precision
       */
      TotalAmountInTax: number;
      TaxRate: number;
      /**
       * Entity type: TaxCode
       */
      TaxCodeID: number;
      UnitCost: number;
      UnitPrice: number;
      UnitPriceInTax: number;
      /**
       * `UnitCost` times the `QuantityOnHand`. Rounded to the currency's precision
       */
      TotalUnitCost: number;
    }

    export interface ProductDto {
      Product: any;
      SerialNumberRequirementID?: number;
    }

  }

  export interface ExportUnshippedPurchaseOrdersToExcel {
    /**
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnitID: number;
  }

  export interface ExportUnshippedPurchaseOrdersToExcelResponse {
    DownloadUrl: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ProcessUnshippedPurchaseOrdersFromExcel {
    Data: string;
  }

  export interface GeneratePurchaseOrderExcelSample {
  }

  export interface UploadPurchaseOrderExcel {
    Data: string;
  }

  /**
   * Generates a sample file for `UploadSalesOrderExcel` service
   */
  export interface GenerateSalesOrderExcelSample {
  }

  /**
   * Uploads a new sales order, attached to specified customer (but not paid, placed, or anything!)
   */
  export interface UploadSalesOrderExcel {
    Data: string;
    EmailAddress?: string;
  }

  export interface GetFraudItem {
    /**
     * Entity type: FraudItem
     */
    ID: number;
  }

  export interface GetFraudItemResponse {
    /**
     * Entity type: FraudItem
     */
    ID: number;
    TypeID: number;
    Type: EVA.Core.FraudDataType;
    Data: string;
    Object: any;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListFraudItems {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListFraudItemsFilter>;
  }

  export interface ListFraudItemsResponse {
    Result?: EVA.Core.PagedResult<ListFraudItemsResponse.Item>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListFraudItemsResponse {
    export interface Item {
      /**
       * Entity type: FraudItem
       */
      ID: number;
      TypeID: number;
      Type: EVA.Core.FraudDataType;
      Data: string;
      Object: any;
    }

  }

  export interface UpdateFraudItem {
    /**
     * Entity type: FraudItem
     */
    ID: number;
    Data?: string;
    Object?: any;
  }

  export interface DeleteFraudItem {
    /**
     * Entity type: FraudItem
     */
    ID: number;
  }

  export interface CreateFraudItem {
    Type: EVA.Core.FraudDataType;
    Data?: string;
    Object?: any;
  }

  export interface CreateFraudItemResponse {
    /**
     * Entity type: FraudItem
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface DownloadFraudItems {
  }

  export interface UploadFraudItems {
    Data: string;
  }

  export interface ExportGeneralLedgerSummaryToExcel {
    Filter?: EVA.Core.ListGeneralLedgersFilter;
  }

  /**
   * Create a new assortment.
   * You can provide either OrganizationUnitIDs or and OrganizationUnitSetID (which will translate to OrganizationUnitIDs) to be attached to the assortment that is being created.
   * OrganizationUnitIDs will take precedence over OrganizationUnitSetID.
   */
  export interface CreateAssortment {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Name: string;
    Code?: string;
    /**
     * Products to add to the assortment.
     */
    Products?: CreateAssortment.AssortmentProduct[];
  }

  export namespace CreateAssortment {
    export interface AssortmentProduct {
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductStatus: EVA.Core.ProductStatus;
      AnnouncementDate?: string;
      PreSaleDate?: string;
      ShipmentDate?: string;
      StartDate?: string;
      EndDate?: string;
    }

  }

  export interface CreateAssortmentResponse {
    /**
     * Entity type: Assortment
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing assortment
   */
  export interface UpdateAssortment {
    /**
     * Entity type: Assortment
     */
    ID: number;
    Name?: string;
    Code?: string;
  }

  /**
   * Delete an existing assortment
   */
  export interface DeleteAssortment {
    /**
     * Entity type: Assortment
     */
    ID: number;
  }

  /**
   * List existing assortments
   */
  export interface ListAssortments {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListAssortmentsFilter>;
  }

  export interface ListAssortmentsResponse {
    Result?: EVA.Core.PagedResult<ListAssortmentsResponse.AssortmentDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListAssortmentsResponse {
    export interface AssortmentDto {
      /**
       * Entity type: Assortment
       */
      ID: number;
      Name: string;
      IsDefault: boolean;
      Code: string;
    }

  }

  /**
   * Get an existing Assortment by ID
   */
  export interface GetAssortmentByID {
    /**
     * Entity type: Assortment
     */
    ID: number;
  }

  export interface GetAssortmentByIDResponse {
    /**
     * Entity type: Assortment
     */
    ID: number;
    Name: string;
    IsDefault: boolean;
    Code: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get all OrganizationUnits that are connected to the Assortment
   */
  export interface GetAssortmentOrganizationUnits {
    /**
     * Entity type: Assortment
     */
    ID: number;
  }

  export interface GetAssortmentOrganizationUnitsResponse {
    OrganizationUnits: GetAssortmentOrganizationUnitsResponse.OrganizationUnitDTO[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAssortmentOrganizationUnitsResponse {
    export interface OrganizationUnitDTO {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Attach an assortment to specified organization units or -set
   */
  export interface AttachAssortmentToOrganizationUnit {
    /**
     * Entity type: Assortment
     */
    ID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  /**
   * Adds or updates the given products to the assortment.
   * 
   * When the given products are all root products, the request will automaticly update all childeren.
   * But if the list is a mixed product list, all updates will be applied according to the given information.
   */
  export interface AddProductsToAssortment {
    /**
     * Entity type: Assortment
     */
    AssortmentID: number;
    /**
     * Specific product IDs that will be added to the assortment.
     */
    Products?: AddProductsToAssortment.AssortmentProduct[];
    /**
     * Only update products that are already part of the assortment
     */
    UpdateOnly?: boolean;
  }

  export namespace AddProductsToAssortment {
    export interface AssortmentProduct {
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductStatus: EVA.Core.ProductStatus;
      StartDate?: string;
      EndDate?: string;
      AnnouncementDate?: string;
      PreSaleDate?: string;
      ShipmentDate?: string;
      ProductStatusesToIgnoreToChildProducts?: EVA.Core.ProductStatus;
      IgnoreStartDateToChildProducts?: boolean;
      IgnoreEndDateToChildProducts?: boolean;
      IgnoreAnnouncementDateToChildProducts?: boolean;
      IgnorePreSaleDateToChildProducts?: boolean;
      IgnoreShipmentDateToChildProducts?: boolean;
    }

  }

  /**
   * Removes products from an assortment. When the product is not part of the assortment an error is returned.
   */
  export interface RemoveProductsFromAssortment {
    /**
     * Entity type: Assortment
     */
    AssortmentID: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
  }

  /**
   * Returns a paged result of all products in all assortments, unless a filter is provided on AssortmentID or ProductID.
   * In this manner, you can either get all products for an assortment (by filtering on AssortmentID), all assortments for a product (by filtering on ProductID)
   * or to get a specific product in an assortment by filtering on both.
   */
  export interface ListAssortmentProducts {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListAssortmentProductsFilter>;
  }

  export interface ListAssortmentProductsResponse {
    Result?: EVA.Core.PagedResult<ListAssortmentProductsResponse.AssortmentProduct>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListAssortmentProductsResponse {
    export interface AssortmentProduct {
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductCustomID: string;
      ProductBackendID: string;
      ProductType: EVA.Core.ProductTypes;
      /**
       * Entity type: Assortment
       */
      AssortmentID: number;
      AssortmentName: string;
      AssortmentCode: string;
      ProductStatus: EVA.Core.ProductStatus;
      StartDate?: string;
      EndDate?: string;
      AnnouncementDate?: string;
      PreSaleDate?: string;
      ShipmentDate?: string;
    }

  }

  /**
   * Returns a paged result of root products in an assortment.
   */
  export interface GetAssortmentRootProducts {
    /**
     * Entity type: Assortment
     */
    AssortmentID: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface GetAssortmentRootProductsResponse {
    Result?: EVA.Core.PagedResult<GetAssortmentRootProductsResponse.AssortmentProduct>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAssortmentRootProductsResponse {
    export interface AssortmentProduct {
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductBackendID: string;
      ProductType: EVA.Core.ProductTypes;
      ProductDetails: any;
      ProductStatus: EVA.Core.ProductStatus;
      MixedProductStatuses: EVA.Core.ProductStatus;
      StartDate?: string;
      EndDate?: string;
      AnnouncementDate?: string;
      PreSaleDate?: string;
      ShipmentDate?: string;
      HasChildren: boolean;
      IsPartiallyIncludedInAssortment: boolean;
    }

  }

  /**
   * Returns a paged result of root products in an assortment.
   */
  export interface GetAssortmentProductTree {
    /**
     * Entity type: Assortment
     */
    AssortmentID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface GetAssortmentProductTreeResponse {
    Parent: GetAssortmentProductTreeResponse.GetAssortmentProductTreeProduct;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAssortmentProductTreeResponse {
    export interface GetAssortmentProductTreeProduct {
      IsRequestedProduct: boolean;
      IsIncludedInAssortment: boolean;
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductBackendID: string;
      ProductType: EVA.Core.ProductTypes;
      ProductDetails: any;
      ProductStatus: EVA.Core.ProductStatus;
      MixedProductStatuses: EVA.Core.ProductStatus;
      StartDate?: string;
      EndDate?: string;
      AnnouncementDate?: string;
      PreSaleDate?: string;
      ShipmentDate?: string;
      Children: GetAssortmentProductTreeResponse.GetAssortmentProductTreeProduct[];
    }

  }

  /**
   * Returns an Excel file that contains all products in an assortment, which can then be updated and reuploaded using UploadAssortmentProducts.
   */
  export interface DownloadAssortmentProducts {
    /**
     * Entity type: Assortment
     */
    AssortmentID: number;
  }

  /**
   * Performs a bulk update of an assortment's products.
   * The data for this service can be gotten by first calling DownloadAssortmentProducts, then make your changes and reupload it using this service.
   */
  export interface UploadAssortmentProducts {
    /**
     * Entity type: Assortment
     */
    AssortmentID: number;
    Data: string;
    /**
     * When a product is currently part of the assortment but is not present in the Excel that is uploaded, it's removed from the assortment.
     */
    DeleteMissing?: boolean;
  }

  export interface UploadAssortmentProductsResponse {
    Messages: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Gets an VisibilityGroup by its ID and returns the basic information.
   */
  export interface GetVisibilityGroupByID {
    /**
     * Entity type: VisibilityGroup
     */
    ID: number;
  }

  export interface GetVisibilityGroupByIDResponse {
    /**
     * Entity type: VisibilityGroup
     */
    ID: number;
    Name: string;
    Description?: string;
    AllowCustomerStandardAccounts: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new VisibilityGroup.
   */
  export interface CreateVisibilityGroup {
    Name: string;
    Description?: string;
    AllowCustomerStandardAccounts: boolean;
  }

  export interface CreateVisibilityGroupResponse {
    /**
     * Entity type: VisibilityGroup
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing VisibilityGroup.
   */
  export interface UpdateVisibilityGroup {
    /**
     * Entity type: VisibilityGroup
     */
    ID: number;
    Name?: string;
    Description?: string;
    AllowCustomerStandardAccounts?: boolean;
  }

  /**
   * Delete a VisibilityGroup
   */
  export interface DeleteVisibilityGroup {
    /**
     * Entity type: VisibilityGroup
     */
    ID: number;
  }

  /**
   * List VisibilityGroups. Returns a paged response.
   */
  export interface ListVisibilityGroups {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListVisibilityGroupsFilter>;
  }

  export interface ListVisibilityGroupsResponse {
    Result?: EVA.Core.PagedResult<ListVisibilityGroupsResponse.VisibilityGroupDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListVisibilityGroupsResponse {
    export interface VisibilityGroupDto {
      /**
       * Entity type: VisibilityGroup
       */
      ID: number;
      Name: string;
      Description?: string;
      AllowCustomerStandardAccounts: boolean;
    }

  }

  /**
   * Gets an VisibilityGroup to VisibilityGroup configuration by its unique identifiers and returns the basic information.
   */
  export interface GetVisibilityGroupConfiguration {
    /**
     * Entity type: VisibilityGroup
     */
    FromID: number;
    /**
     * Entity type: VisibilityGroup
     */
    ToID: number;
  }

  export interface GetVisibilityGroupConfigurationResponse {
    From: GetVisibilityGroupConfigurationResponse.VisibilityGroupDto;
    To: GetVisibilityGroupConfigurationResponse.VisibilityGroupDto;
    Users: boolean;
    Orders: boolean;
    OrganizationUnits: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetVisibilityGroupConfigurationResponse {
    export interface VisibilityGroupDto {
      /**
       * Entity type: VisibilityGroup
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Create or update a VisibilityGroup to VisibilityGroup configuration.
   */
  export interface CreateOrUpdateVisibilityGroupConfiguration {
    /**
     * Entity type: VisibilityGroup
     */
    FromID: number;
    /**
     * Entity type: VisibilityGroup
     */
    ToID: number;
    Users?: boolean;
    Orders?: boolean;
    OrganizationUnits?: boolean;
  }

  /**
   * Delete a VisibilityGroup to VisibilityGroup configuration
   */
  export interface DeleteVisibilityGroupConfiguration {
    /**
     * Entity type: VisibilityGroup
     */
    FromID: number;
    /**
     * Entity type: VisibilityGroup
     */
    ToID: number;
  }

  /**
   * List VisibilityGroupVisibilityGroups. Returns a paged response.
   */
  export interface ListVisibilityGroupConfigurations {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListVisibilityGroupConfigurationsFilter>;
  }

  export interface ListVisibilityGroupConfigurationsResponse {
    Result?: EVA.Core.PagedResult<ListVisibilityGroupConfigurationsResponse.VisibilityGroupVisibilityGroupDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListVisibilityGroupConfigurationsResponse {
    export interface VisibilityGroupVisibilityGroupDto {
      From: ListVisibilityGroupConfigurationsResponse.VisibilityGroupDto;
      To: ListVisibilityGroupConfigurationsResponse.VisibilityGroupDto;
      Users: boolean;
      Orders: boolean;
      OrganizationUnits: boolean;
    }

    export interface VisibilityGroupDto {
      /**
       * Entity type: VisibilityGroup
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Export all UserSubscriptions in an excel file
   */
  export interface ExportUserSubscriptions {
  }

  /**
   * Get the available UserProperties on which requirements can be set.
   */
  export interface GetAvailableUserRequirementsProperties {
  }

  export interface GetAvailableUserRequirementsPropertiesResponse {
    Properties?: string[];
    Result: { [key: string]: EVA.Core.UserRequirementProperties };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get the configured UserRequirements for the current, or given, `OrganizationUnit`.
   */
  export interface GetUserRequirements {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Subscription
     */
    SubscriptionID?: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    UserType?: EVA.Core.UserTypes;
  }

  export interface GetUserRequirementsResponse {
    Requirements: { [key: string]: EVA.Core.UserRequirementResponse };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get the configured UserRequirements for the given Subscriptions.
   */
  export interface GetUserRequirementsForSubscriptions {
    /**
     * Entity type: Subscription
     */
    SubscriptionIDs: number[];
  }

  export interface GetUserRequirementsForSubscriptionsResponse {
    Requirements: { [key: number]: EVA.Core.UserRequirementForSubscriptionResponse[] };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * The service enables endpoints to push user subscriptions based on the user backend id and subscription backend id.
   * Multiple subscriptions can be pushed at once.
   * 
   * Additionally, a SubscriptionID and Balance can be provided
   * NOTE THAT Only subscriptions that allow for local storage can be pushed. Subscriptions fully depending on external services cannot be pushed.
   * NOTE THAT Balance can only be provided for loyalty subscriptions that have a EVA loyalty handler.
   */
  export interface PushUserSubscription {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Subscription
     */
    SubscriptionID?: number;
    UserSubscriptionIdentifier?: string;
    UserSubscriptionSourceIdentifier?: string;
    UserUnsubscribeReason?: string;
    IsSubscribed?: boolean;
    Balance?: number;
    SubscribedOn?: string;
  }

  /**
   * Gets all the SubscriptionOrganizationUnitSets and returns a paged response.
   */
  export interface GetSubscriptionOrganizationUnitSets {
  }

  export interface GetSubscriptionOrganizationUnitSetsResponse {
    SubscriptionOrganizationUnitSets: GetSubscriptionOrganizationUnitSetsResponse.SubscriptionOrganizationUnitSetDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSubscriptionOrganizationUnitSetsResponse {
    export interface SubscriptionOrganizationUnitSetDto {
      /**
       * Entity type: SubscriptionOrganizationUnitSet
       */
      ID: number;
      /**
       * Entity type: Subscription
       */
      SubscriptionID: number;
      SubscriptionName: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      OrganizationUnitSetName: string;
      OrganizationUnitSetDescription: string;
      OrganizationUnitSetType?: EVA.Core.OrganizationUnitSetTypes;
      Enabled: boolean;
      Default: boolean;
      SubscriptionConfirmation: EVA.Core.SubscriptionConfirmation;
      Hidden: boolean;
      Preferred: boolean;
      Priority: number;
    }

  }

  /**
   * Creates a new SubscriptionOrganizationUnitSet and returns the ID of the new record.
   */
  export interface CreateSubscriptionOrganizationUnitSet {
    /**
     * Entity type: Subscription
     */
    SubscriptionID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    Enabled: boolean;
    Default: boolean;
    SubscriptionConfirmation: EVA.Core.SubscriptionConfirmation;
    Hidden?: boolean;
    Preferred?: boolean;
    Priority?: number;
  }

  export interface CreateSubscriptionOrganizationUnitSetResponse {
    /**
     * Entity type: SubscriptionOrganizationUnitSet
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates an existing SubscriptionOrganizationUnitSet.
   */
  export interface UpdateSubscriptionOrganizationUnitSet {
    /**
     * Entity type: SubscriptionOrganizationUnitSet
     */
    ID: number;
    /**
     * Entity type: Subscription
     * @deprecated Updating this property makes no sence. Just remove SubscriptionOrganizationUnitSet registration and create a new one. No longer processed.
     */
    SubscriptionID?: number;
    /**
     * Entity type: OrganizationUnitSet
     * @deprecated Updating this property makes no sence. Just remove SubscriptionOrganizationUnitSet registration and create a new one. No longer processed.
     */
    OrganizationUnitSetID?: number;
    Enabled: boolean;
    Default: boolean;
    SubscriptionConfirmation?: EVA.Core.SubscriptionConfirmation;
    Hidden?: boolean;
    Preferred?: boolean;
    Priority?: number;
  }

  /**
   * Deletes a SubscriptionOrganizationUnitSet by its ID.
   */
  export interface DeleteSubscriptionOrganizationUnitSet {
    /**
     * Entity type: SubscriptionOrganizationUnitSet
     */
    ID: number;
  }

  /**
   * List the Subscriptions
   */
  export interface ListSubscriptions {
  }

  export interface ListSubscriptionsResponse {
    Subscriptions: ListSubscriptionsResponse.SubscriptionDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListSubscriptionsResponse {
    export interface SubscriptionDto {
      /**
       * Entity type: Subscription
       */
      ID?: number;
      Name?: string;
      Description?: string;
      BackendID?: string;
      Type?: EVA.Core.SubscriptionType;
      Handler?: string;
      /**
       * Entity type: LoyaltyProgram
       */
      LoyaltyProgramID?: number;
      LoyaltyProgramName?: string;
      /**
       * Entity type: CustomField
       */
      IdentifierCustomFieldID?: number;
      IdentifierCustomFieldName?: string;
      ManualUserIdentifiers?: boolean;
      InquiryID?: number;
      InquiryName?: string;
      IsExternal?: boolean;
      UserIdentifierPrefix?: string;
      UserIdentifierStartNumber?: number;
      UserIdentifierCurrentNumber?: number;
      UserIdentifierSuffix?: string;
      ResubscribeHandling?: EVA.Core.ResubscribeHandlingType;
      AllowUnsubscribeBy?: EVA.Core.AllowUnsubscribeByType;
    }

  }

  /**
   * Get the subscription details
   */
  export interface GetSubscriptionByID {
    /**
     * Entity type: Subscription
     */
    ID: number;
  }

  export interface GetSubscriptionByIDResponse {
    /**
     * Entity type: Subscription
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
    AllowUnsubscribeBy?: EVA.Core.AllowUnsubscribeByType;
    ResubscribeHandling?: EVA.Core.ResubscribeHandlingType;
    Type?: EVA.Core.SubscriptionType;
    Handler?: string;
    IsExternal?: boolean;
    ManualUserIdentifiers?: boolean;
    UserIdentifierPrefix?: string;
    UserIdentifierStartNumber?: number;
    UserIdentifierCurrentNumber?: number;
    UserIdentifierSuffix?: string;
    IdentifierCustomField?: GetSubscriptionByIDResponse.CustomFieldDto;
    Inquiry?: GetSubscriptionByIDResponse.InquiryDto;
    LoyaltyProgram?: GetSubscriptionByIDResponse.LoyaltyProgramDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSubscriptionByIDResponse {
    export interface CustomFieldDto {
      /**
       * Entity type: CustomField
       */
      ID: number;
      Name: string;
    }

    export interface InquiryDto {
      /**
       * Entity type: Inquiry
       */
      ID: number;
      Name: string;
    }

    export interface LoyaltyProgramDto {
      /**
       * Entity type: LoyaltyProgram
       */
      ID: number;
      Name: string;
      Options?: EVA.Core.LoyaltyProgramOptions;
      PointUsageOptions?: EVA.Core.LoyaltyPointUsageOptions;
      Company: GetSubscriptionByIDResponse.CompanyDto;
    }

    export interface CompanyDto {
      /**
       * Entity type: Company
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Create a new Subscription
   */
  export interface CreateSubscription {
    /**
     * The name of the subscription
     */
    Name?: string;
    Description?: string;
    /**
     * The (optional) BackendID of the subscription
     */
    BackendID?: string;
    /**
     * The handler for the actions on this subscription. Available handlers can be listed with the `GetSubscriptionHandlers` service.
     */
    Handler?: string;
    /**
     * The LoyaltyProgram the subscription is bound to.
     * Entity type: LoyaltyProgram
     * @deprecated Loyalty programs can now be created with the subscription info.
     */
    LoyaltyProgramID?: number;
    /**
     * The (optional) custom field the UserIdentifier of the subscription is written back to. Values from in the custom field are informational only.
     * Entity type: CustomField
     */
    IdentifierCustomFieldID?: number;
    /**
     * When set to true, subscribing the user will require a UserIdentifier to be given.
     */
    ManualUserIdentifiers?: boolean;
    UserIdentifierPrefix?: string;
    /**
     * Min: 0
     */
    UserIdentifierStartNumber?: number;
    UserIdentifierSuffix?: string;
    /**
     * Entity type: Inquiry
     */
    InquiryID?: number;
    /**
     * Handling when a user is unsubscribed and re-subscribes.
     */
    ResubscribeHandling?: EVA.Core.ResubscribeHandlingType;
    /**
     * Indicates if the subscription can be unsubscribed and if so, who can do the unsubscribe.
     */
    AllowUnsubscribeBy?: EVA.Core.AllowUnsubscribeByType;
  }

  export interface CreateSubscriptionResponse {
    /**
     * Entity type: Subscription
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing Subscription
   */
  export interface UpdateSubscription {
    /**
     * Entity type: Subscription
     */
    ID: number;
    /**
     * The name of the subscription
     */
    Name?: string;
    Description?: string;
    /**
     * The (optional) BackendID of the subscription
     */
    BackendID?: string;
    /**
     * Entity type: Inquiry
     */
    InquiryID?: number;
    /**
     * Handling when a user is unsubscribed and re-subscribes.
     */
    ResubscribeHandling?: EVA.Core.ResubscribeHandlingType;
    /**
     * Indicates if the subscription can be unsubscribed and if so, who can do the unsubscribe.
     */
    AllowUnsubscribeBy?: EVA.Core.AllowUnsubscribeByType;
  }

  /**
   * Delete a Subscription.
   */
  export interface DeleteSubscription {
    /**
     * Entity type: Subscription
     */
    ID: number;
    /**
     * You can provide the property `Force` to force remaining subscribed users to be unsubscribed. Doing so requires ForceDeleteSubscription functionality.
     */
    Force?: boolean;
  }

  /**
   * Get the available subscription handlers
   */
  export interface GetSubscriptionHandlers {
  }

  export interface GetSubscriptionHandlersResponse {
    HandlersDetails: GetSubscriptionHandlersResponse.Handler[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSubscriptionHandlersResponse {
    export interface Handler {
      Name: string;
      AllowRequestConfirmation: boolean;
    }

  }

  /**
   * Block a users subscription without unsubscribing.
   */
  export interface BlockUserSubscription {
    /**
     * Entity type: Subscription
     */
    SubscriptionID: number;
    /**
     * Entity type: User
     */
    UserID: number;
  }

  /**
   * Unblock a users subscription.
   */
  export interface UnblockUserSubscription {
    /**
     * Entity type: Subscription
     */
    SubscriptionID: number;
    /**
     * Entity type: User
     */
    UserID: number;
  }

  /**
   * Generate a user subscription identifier barcode for the given user and subscription
   */
  export interface GetUserSubscriptionBarcode {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: Subscription
     */
    SubscriptionID: number;
  }

  export interface GetUserSubscriptionBarcodeResponse {
    Barcode: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List all users attached to a subscription. Requires View rights on Subscriptions and Customers functionalities.
   */
  export interface ListSubscriptionUsers {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListSubscriptionUsersFilter>;
    PageToken?: string;
  }

  export interface ListSubscriptionUsersResponse {
    Results?: ListSubscriptionUsersResponse.SubscriptionUser[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListSubscriptionUsersResponse {
    export interface SubscriptionUser {
      /**
       * Entity type: Subscription
       */
      SubscriptionID: number;
      Status: EVA.Core.SubscriptionStatus;
      IsBlocked: boolean;
      UserIdentifier?: string;
      /**
       * Entity type: OrganizationUnit
       */
      OriginatingOrganizationUnitID?: number;
      /**
       * Entity type: User
       */
      UserID: number;
      User: ListSubscriptionUsersResponse.UserInfo;
    }

    export interface UserInfo {
      FirstName: string;
      LastName: string;
      FullName: string;
      EmailAddress: string;
    }

  }

  /**
   * Update all UserRequirements on a Subscription.
   * 
   * Updates the given requirements and removes all other requirements from the Subscription
   */
  export interface UpdateSubscriptionUserRequirements {
    /**
     * Entity type: Subscription
     */
    ID: number;
    Requirements: UpdateSubscriptionUserRequirements.UserRequirementDto[];
  }

  export namespace UpdateSubscriptionUserRequirements {
    export interface UserRequirementDto {
      /**
       * Entity type: UserRequirement
       */
      ID?: number;
      Property?: string;
      UserType?: EVA.Core.UserTypes;
      AccountType?: EVA.Core.UserAccountType;
      RequiredFor?: EVA.Core.UserRequirementFor;
      Preferred?: boolean;
      Display?: boolean;
      CustomValidators?: { [key: string]: any };
    }

  }

  /**
   * List the UserOrigins
   */
  export interface GetUserOrigins {
  }

  export interface GetUserOriginsResponse {
    UserOrigins: GetUserOriginsResponse.UserOriginDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetUserOriginsResponse {
    export interface UserOriginDto {
      BackendID: string;
      /**
       * Entity type: Enum
       */
      ID: number;
      /**
       * Usually an internal & unchangable code - for display, use `Description`
       */
      Name: string;
      Description: string;
    }

  }

  /**
   * Create a new UserOrigin
   */
  export interface CreateUserOrigin {
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  export interface CreateUserOriginResponse {
    /**
     * Entity type: UserOrigin
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing UserOrigin
   */
  export interface UpdateUserOrigin {
    /**
     * Entity type: UserOrigin
     */
    ID: number;
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  /**
   * Delete a UserOrigin
   */
  export interface DeleteUserOrigin {
    /**
     * Entity type: UserOrigin
     */
    ID: number;
  }

  /**
   * Get all personalized promotions for the logged in user or the given user.
   */
  export interface ListUserPersonalizedPromotions {
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  export interface ListUserPersonalizedPromotionsResponse {
    PersonalizedPromotions: ListUserPersonalizedPromotionsResponse.PersonalizedPromotion[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListUserPersonalizedPromotionsResponse {
    export interface PersonalizedPromotion {
      /**
       * Entity type: PersonalizedPromotion
       */
      ID: number;
      BackendID: string;
      Name: string;
      Description: string;
      MaximumDistinctProducts?: number;
      LockAfterFirstApply: boolean;
      IsLocked: boolean;
    }

  }

  /**
   * Get all products for a personalized promotion for the logged in user or the given user.
   */
  export interface GetUserPersonalizedPromotionProducts {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: PersonalizedPromotion
     */
    PersonalizedPromotionID: number;
  }

  export interface GetUserPersonalizedPromotionProductsResponse {
    /**
     * Entity type: PersonalizedPromotion
     */
    PersonalizedPromotionID: number;
    PersonalizedPromotionBackendID: string;
    Name: string;
    Description: string;
    ApplicableProducts: any[];
    SelectedProducts: any[];
    MaximumDistinctProducts?: number;
    IsLocked: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Set selected products for personalized promotion for the logged in user or the given user. Overwrites existing selection.
   */
  export interface SelectUserPersonalizedPromotionProducts {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: PersonalizedPromotion
     */
    PersonalizedPromotionID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
  }

  /**
   * List the UserRequirements.
   */
  export interface ListUserRequirements {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListUserRequirementsFilter>;
  }

  export interface ListUserRequirementsResponse {
    Result?: EVA.Core.PagedResult<ListUserRequirementsResponse.UserRequirementDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListUserRequirementsResponse {
    export interface UserRequirementDto {
      /**
       * Entity type: UserRequirement
       */
      ID: number;
      Property: string;
      UserType?: EVA.Core.UserTypes;
      AccountType?: EVA.Core.UserAccountType;
      RequiredFor: EVA.Core.UserRequirementFor;
      Preferred: boolean;
      Display: boolean;
      CustomValidators?: { [key: string]: any };
    }

  }

  /**
   * Creates a new UserRequirement and returns the ID of the new record.
   */
  export interface CreateUserRequirement {
    /**
     * Entity type: UserRequirementSet
     */
    UserRequirementSetID?: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    Property: string;
    /**
     * Entity type: Subscription
     */
    SubscriptionID?: number;
    UserType: EVA.Core.UserTypes;
    AccountType?: EVA.Core.UserAccountType;
    RequiredFor: EVA.Core.UserRequirementFor;
    Preferred?: boolean;
    Display?: boolean;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    CustomValidators?: { [key: string]: any };
  }

  export interface CreateUserRequirementResponse {
    /**
     * Entity type: UserRequirement
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates an existing UserRequirement.
   */
  export interface UpdateUserRequirement {
    /**
     * Entity type: UserRequirement
     */
    ID: number;
    Property?: string;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    UserType?: EVA.Core.UserTypes;
    AccountType?: EVA.Core.UserAccountType;
    RequiredFor?: EVA.Core.UserRequirementFor;
    Preferred?: boolean;
    Display?: boolean;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    CustomValidators?: { [key: string]: any };
  }

  /**
   * Deletes a UserRequirement by its ID.
   */
  export interface DeleteUserRequirement {
    /**
     * Entity type: UserRequirement
     */
    ID: number;
  }

  /**
   * Gets UserRequirement by its ID.
   */
  export interface GetUserRequirement {
    /**
     * Entity type: UserRequirement
     */
    ID: number;
  }

  export interface GetUserRequirementResponse {
    /**
     * Entity type: UserRequirement
     */
    ID: number;
    /**
     * Entity type: UserRequirementSet
     */
    UserRequirementSetID?: number;
    UserRequirementSetName?: string;
    Property: string;
    UserType?: EVA.Core.UserTypes;
    AccountType?: EVA.Core.UserAccountType;
    RequiredFor: EVA.Core.UserRequirementFor;
    Preferred: boolean;
    Display: boolean;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    CustomValidators?: { [key: string]: any };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the UserRequirementSets.
   */
  export interface ListUserRequirementSets {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListUserRequirementSetsFilter>;
  }

  export interface ListUserRequirementSetsResponse {
    Result?: EVA.Core.PagedResult<ListUserRequirementSetsResponse.UserRequirementSetDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListUserRequirementSetsResponse {
    export interface UserRequirementSetDto {
      /**
       * Entity type: UserRequirementSet
       */
      ID: number;
      Name: string;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      OrganizationUnitSetName: string;
      OrganizationUnitSetDescription: string;
      OrganizationUnitSetType?: EVA.Core.OrganizationUnitSetTypes;
    }

  }

  /**
   * Creates a new UserRequirementSet and returns the ID of the new record.
   */
  export interface CreateUserRequirementSet {
    Name: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
  }

  export interface CreateUserRequirementSetResponse {
    /**
     * Entity type: UserRequirementSet
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates an existing UserRequirementSet.
   */
  export interface UpdateUserRequirementSet {
    /**
     * Entity type: UserRequirementSet
     */
    ID: number;
    Name?: string;
    /**
     * Entity type: OrganizationUnitSet
     * @deprecated Updating this property makes no sense. Create a duplicate and remove this one if you want to move to another OU set.
     */
    OrganizationUnitSetID?: number;
  }

  /**
   * Deletes a UserRequirementSet by its ID. Will also delete the underlying requirements.
   */
  export interface DeleteUserRequirementSet {
    /**
     * Entity type: UserRequirementSet
     */
    ID: number;
  }

  /**
   * Update all requirements on a UserRequirementSet.
   */
  export interface UpdateUserRequirementSetRequirements {
    /**
     * Entity type: UserRequirementSet
     */
    ID: number;
    Requirements: UpdateUserRequirementSetRequirements.UserRequirementDto[];
  }

  export namespace UpdateUserRequirementSetRequirements {
    export interface UserRequirementDto {
      /**
       * Entity type: UserRequirement
       */
      ID?: number;
      Property?: string;
      /**
       * Entity type: ShippingMethod
       */
      ShippingMethodID?: number;
      UserType?: EVA.Core.UserTypes;
      AccountType?: EVA.Core.UserAccountType;
      RequiredFor?: EVA.Core.UserRequirementFor;
      Preferred?: boolean;
      Display?: boolean;
      /**
       * Entity type: Script
       */
      ScriptID?: number;
      CustomValidators?: { [key: string]: any };
    }

  }

  /**
   * Duplicate the requirements of an existing set to a new set.
   */
  export interface DuplicateUserRequirementSet {
    /**
     * Entity type: UserRequirementSet
     */
    SetToDuplicateID: number;
    Name: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
  }

  export interface DuplicateUserRequirementSetResponse {
    /**
     * Entity type: UserRequirementSet
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the wishlists.
   */
  export interface ListWishlists {
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListWishlistsFilter>;
    PageToken?: string;
  }

  export interface ListWishlistsResponse {
    Results?: ListWishlistsResponse.Wishlist[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListWishlistsResponse {
    export interface Wishlist {
      /**
       * Entity type: Wishlist
       */
      ID: number;
      /**
       * Entity type: User
       */
      UserID: number;
      UserFullName: string;
      Name: string;
      Description: string;
      CreatedOn: string;
      LastReadTime: string;
      CreatedByEmployee: boolean;
      HiddenFromUser: boolean;
      FirstProduct?: any;
      ItemCount: number;
      IsShared: boolean;
    }

  }

  /**
   * Get the wishlist by ID.
   */
  export interface GetWishlist {
    /**
     * Entity type: Wishlist
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
  }

  export interface GetWishlistResponse {
    /**
     * Entity type: Wishlist
     */
    ID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    UserFullName: string;
    Name: string;
    Description: string;
    CreatedOn: string;
    LastModifiedOn?: string;
    LastReadTime: string;
    CreatedByEmployee: boolean;
    HiddenFromUser: boolean;
    IsShared: boolean;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: EVA.Core.CustomFieldValueWithOptions };
    FirstProduct?: any;
    Products?: any[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new Employee
   */
  export interface CreateEmployeeUser {
    FirstName?: string;
    LastName?: string;
    Nickname?: string;
    EmailAddress?: string;
    Gender?: string;
    PhoneNumber?: string;
    DateOfBirth?: string;
    Password?: string;
    LanguageID?: string;
    CountryID?: string;
    EmployeeNumber?: string;
    Function?: string;
    /**
     * Entity type: OrganizationUnit
     */
    PrimaryOrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    ManagerID?: number;
    UserRoles?: CreateEmployeeUser.UserRole[];
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
    PricingGroupID?: string;
    IsSingleSignOnOnly?: boolean;
  }

  export namespace CreateEmployeeUser {
    export interface UserRole {
      /**
       * Entity type: Role
       */
      RoleID: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
    }

  }

  export interface CreateEmployeeUserResponse {
    /**
     * Entity type: User
     */
    UserID?: number;
    Result: CreateEmployeeResults;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export const enum CreateEmployeeResults {
    CreatedNewUser = 0,
    UpgradedExistingUser = 1,
    UpdatedExistingUser = 2
  }

  /**
   * Fetch a users employee data by `UserID`.
   */
  export interface GetEmployeeData {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  export interface GetEmployeeDataResponse {
    /**
     * Entity type: User
     */
    UserID?: number;
    UserFullName?: string;
    UserEmailAddress?: string;
    EmployeeNumber?: string;
    Function?: string;
    PrimaryOrganizationUnitName?: string;
    /**
     * Entity type: User
     */
    ManagerID?: number;
    ManagerFullName?: string;
    ManagerEmailAddress?: string;
    /**
     * Entity type: OrganizationUnit
     */
    PrimaryOrganizationUnitID?: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create or update a users employee data.
   */
  export interface CreateOrUpdateEmployeeData {
    /**
     * Entity type: User
     */
    UserID: number;
    EmployeeNumber?: string;
    Function?: string;
    /**
     * Entity type: User
     */
    ManagerID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    PrimaryOrganizationUnitID?: number;
  }

  /**
   * Remove a users employee data.
   */
  export interface DeleteEmployeeData {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  export interface GenerateIdentificationCodeForEmployee {
  }

  export interface GenerateIdentificationCodeForEmployeeResponse {
    Result: GenerateIdentificationCodeForEmployeeResponse.UserIdWithIdentificationCodeDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GenerateIdentificationCodeForEmployeeResponse {
    export interface UserIdWithIdentificationCodeDto {
      /**
       * Entity type: User
       */
      UserId: number;
      IdentificationCode: string;
    }

  }

  /**
   * Export all Employees with their roles in an excel file
   */
  export interface ExportEmployeeRoles {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  /**
   * Download an Excel sample which can be filled with customers. The populated excel can be uploaded through the `UploadCustomers` service.
   */
  export interface DownloadCustomersSample {
  }

  /**
   * Upload the Excel based on the sample from `DownloadCustomersSample`.
   * 
   * The uploaded file will be processed in the background and the results will be mailed to the uploader.
   */
  export interface UploadCustomers {
    Data: string;
  }

  /**
   * A service to upload an Excel containing tax codes info that.
   * The actions that can be taken on the tax codes are: create(C), update(U) or delete(D), according to the value specified in 'Action' column.
   */
  export interface UploadTaxCodesExcel {
    Data: string;
  }

  /**
   * Downloads an Excel file containing tax codes according to the specified PageConfig. To get an empty template Excel, specify a page size of zero.
   */
  export interface DownloadTaxCodesExcel {
  }

  /**
   * A service to upload an Excel containing tax rates.
   * The actions that can be applied on the tax rates are: Create, Update or Delete, according to the value specified in 'Action' column.
   */
  export interface UploadTaxRateExcel {
    Data: string;
  }

  /**
   * Downloads an Excel file containing tax rates according to the specified PageConfig.
   */
  export interface DownloadTaxRateExcel {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListTaxRateModelFilters>;
  }

  /**
   * Returns available survey triggers.
   */
  export interface GetAvailableSurveyTriggers {
    SurveyContext: EVA.Core.SurveyContext;
  }

  export interface GetAvailableSurveyTriggersResponse {
    Triggers: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns available survey distribution handlers.
   */
  export interface GetAvailableSurveyDistributionHandlers {
  }

  export interface GetAvailableSurveyDistributionHandlersResponse {
    Handlers: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the surveys
   */
  export interface ListSurveys {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListSurveyFilter>;
    PageToken?: string;
  }

  export interface ListSurveysResponse {
    Results?: ListSurveysResponse.SurveyDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListSurveysResponse {
    export interface SurveyDto {
      /**
       * Entity type: Survey
       */
      ID: number;
      Name: string;
      Description?: string;
      BackendID?: string;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      SurveyContext: EVA.Core.SurveyContext;
      SurveyTrigger: string;
      SurveyDistributionHandler: string;
      StartDate?: string;
      EndDate?: string;
      DeactivationReasons: EVA.Core.SurveyDeactivationReasons;
      HasUnpublishedChanges: boolean;
      IsPublishing: boolean;
      HasActivePublishedVersion: boolean;
      LastPublishedVersion?: number;
      StartedResponses: number;
      CompletedResponses: number;
    }

  }

  /**
   * Get the survey
   */
  export interface GetSurveyByID {
    /**
     * Entity type: Survey
     */
    ID: number;
  }

  export interface GetSurveyByIDResponse {
    EditAllowed?: boolean;
    /**
     * Entity type: Survey
     */
    ID?: number;
    Name?: string;
    Description?: string;
    IntroductionTitle?: string;
    IntroductionMessage?: string;
    /**
     * @deprecated Use IntroductionBlob.URL instead.
     */
    IntroductionBlobURL?: string;
    IntroductionBlob?: EVA.Core.BlobDto;
    ClosingRemarkTitle?: string;
    ClosingRemarkMessage?: string;
    /**
     * @deprecated Use ClosingRemarkBlob.URL instead.
     */
    ClosingRemarkBlobURL?: string;
    ClosingRemarkBlob?: EVA.Core.BlobDto;
    BackendID?: string;
    SurveyContext?: EVA.Core.SurveyContext;
    SurveyTrigger?: string;
    SurveyTriggerData?: any;
    SurveyDistributionHandler?: string;
    SurveyDistributionHandlerData?: any;
    StartDate?: string;
    EndDate?: string;
    DeactivationReasons?: EVA.Core.SurveyDeactivationReasons;
    StartedResponses?: number;
    LastResponseStartedOn?: string;
    CompletedResponses?: number;
    LastResponseCompletedOn?: string;
    HasUnpublishedChanges?: boolean;
    IsPublishing?: boolean;
    HasActivePublishedVersion?: boolean;
    LastPublishedVersion?: number;
    LastPublishedBy?: GetSurveyByIDResponse.User;
    LastPublishedOn?: string;
    DeactivatedBy?: GetSurveyByIDResponse.User;
    DeactivatedOn?: string;
    OrganizationUnitSet?: GetSurveyByIDResponse.OrganizationUnitSetDto;
    AfterEffects?: EVA.Core.SurveyAfterEffectDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSurveyByIDResponse {
    export interface User {
      /**
       * Entity type: User
       */
      ID: number;
      FullName: string;
    }

    export interface OrganizationUnitSetDto {
      /**
       * Entity type: OrganizationUnitSet
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Type: EVA.Core.OrganizationUnitSetTypes;
    }

  }

  /**
   * Creates a new survey
   */
  export interface CreateSurvey {
    Name?: string;
    Description?: string;
    IntroductionTitle?: string;
    IntroductionMessage?: string;
    IntroductionBlobID?: string;
    ClosingRemarkTitle?: string;
    ClosingRemarkMessage?: string;
    ClosingRemarkBlobID?: string;
    BackendID?: string;
    SurveyContext?: EVA.Core.SurveyContext;
    SurveyTrigger?: string;
    SurveyTriggerData?: any;
    SurveyDistributionHandler?: string;
    SurveyDistributionHandlerData?: any;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    StartDate?: string;
    EndDate?: string;
    AfterEffects?: EVA.Core.SurveyAfterEffectDto[];
  }

  export interface CreateSurveyResponse {
    /**
     * Entity type: Survey
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a survey.
   * NOTE THAT after update, the published survey will remain active and the current data will be taken into draft mode for republishing.
   * After publishing you can no longer change the OrganizationUnitSetID, SurveyContext, Trigger and (depending on the Trigger) some of the TriggerData properties.
   */
  export interface UpdateSurvey {
    /**
     * Entity type: Survey
     */
    ID: number;
    Name?: string;
    Description?: string;
    IntroductionTitle?: string;
    IntroductionMessage?: string;
    IntroductionBlobID?: string;
    ClosingRemarkTitle?: string;
    ClosingRemarkMessage?: string;
    ClosingRemarkBlobID?: string;
    BackendID?: string;
    SurveyContext?: EVA.Core.SurveyContext;
    SurveyTrigger?: string;
    SurveyTriggerData?: any;
    SurveyDistributionHandler?: string;
    SurveyDistributionHandlerData?: any;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    StartDate?: string;
    EndDate?: string;
    AfterEffects?: EVA.Core.SurveyAfterEffectDto[];
  }

  /**
   * Delete a survey
   */
  export interface DeleteSurvey {
    /**
     * Entity type: Survey
     */
    ID: number;
  }

  /**
   * Create and start a preview.
   */
  export interface StartSurveyPreview {
    /**
     * Entity type: Survey
     */
    SurveyID: number;
  }

  export interface StartSurveyPreviewResponse {
    Token: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Publishes all changes made to the survey.
   * 
   * Service will return an error when the survey is already published.
   * 
   * Survey cannot be published by the same user as the one who created / modified the survey, unless they have the right 'SurveySelfPublish'.
   * The service will return an error if the same user without the right 'SurveySelfPublish' tries to publish the survey.
   */
  export interface PublishSurvey {
    /**
     * Entity type: Survey
     */
    SurveyID: number;
    ValidateOnly?: boolean;
  }

  /**
   * Deactivates a survey.
   * NOTE THAT this deactivates all published versions of the survey, so it can no longer be responded to at the end of the grace period.
   * It will also take the survey back into draft.
   * The survey is re-enabled when a new version is published.
   */
  export interface DeactivateSurvey {
    /**
     * Entity type: Survey
     */
    SurveyID: number;
  }

  /**
   * Returns available survey after effects.
   */
  export interface GetAvailableSurveyAfterEffectTypes {
    SurveyContext: EVA.Core.SurveyContext;
  }

  export interface GetAvailableSurveyAfterEffectTypesResponse {
    AfterEffects: GetAvailableSurveyAfterEffectTypesResponse.AfterEffect[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAvailableSurveyAfterEffectTypesResponse {
    export interface AfterEffect {
      Type: string;
      MessageTemplates?: GetAvailableSurveyAfterEffectTypesResponse.MessageTemplate[];
    }

    export interface MessageTemplate {
      Name: string;
      IsAvailable: boolean;
    }

  }

  /**
   * List the survey questions
   */
  export interface GetSurveyQuestions {
    /**
     * Entity type: Survey
     */
    SurveyID: number;
  }

  export interface GetSurveyQuestionsResponse {
    Questions: GetSurveyQuestionsResponse.Question[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSurveyQuestionsResponse {
    export interface Question {
      /**
       * Entity type: SurveyQuestion
       */
      ID?: number;
      BackendID?: string;
      Sequence?: number;
      Text?: string;
      Description?: string;
      Remark?: string;
      /**
       * @deprecated Use Blob.URL instead.
       */
      BlobURL?: string;
      Blob?: EVA.Core.BlobDto;
      /**
       * Entity type: CustomField
       */
      SurveyContextCustomFieldID?: number;
      StoreAnswerOnContextCustomField?: boolean;
      UseContextCustomFieldDataAsPrefill?: boolean;
      DataType?: EVA.Core.CustomFieldDataTypes;
      OptionalAnswers?: { [key: string]: string };
      AllowMultipleAnswers?: boolean;
      DisplayType?: string;
      InputHint?: string;
      MinimumValue?: number;
      MaximumValue?: number;
      MinimumLength?: number;
      MaximumLength?: number;
      MinimumDate?: string;
      MaximumDate?: string;
      Required?: boolean;
      Routes?: GetSurveyQuestionsResponse.Route[];
    }

    export interface Route {
      /**
       * Entity type: SurveyQuestionRoute
       */
      ID?: number;
      Sequence?: number;
      Filter?: EVA.Core.SurveyQuestionRouteFilter;
      /**
       * Entity type: SurveyQuestion
       */
      RouteToQuestionID?: number;
      ExitRemarkTitle?: string;
      ExitRemarkMessage?: string;
      /**
       * @deprecated Use ExitBlob.URL instead.
       */
      ExitBlobURL?: string;
      ExitBlob?: EVA.Core.BlobDto;
    }

  }

  /**
   * Creates a new survey
   */
  export interface CreateSurveyQuestion {
    /**
     * Entity type: Survey
     */
    SurveyID?: number;
    BackendID?: string;
    Text?: string;
    Description?: string;
    Remark?: string;
    /**
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * A custom field that is available from the SurveyContext.
     * If context for example is `User`, all custom fields of the user are available.
     * If context for example is `Order`, all custom fields from the order and customer (user) on the order are available.
     * 
     * NOTE THAT this value cannot be update. If this question is based upon the custom field, any changes to the markup are rejected.
     * 
     * NOTE THAT any security settings on the custom field are ignored.
     * 
     * NOTE THAT when this value is set, the markup of the custom field is copied as markup for the survey question, and the response data is saved within the survey.
     * If the value also needs to be stored on the context entity self (e.g. order, user, etc), the setting `StoreAnswerOnContextCustomField` needs to be set.
     * When `StoreAnswerOnContextCustomField` is set to true, changes to the custom field itself will also be blocked while the survey is active.
     * Entity type: CustomField
     */
    SurveyContextCustomFieldID?: number;
    StoreAnswerOnContextCustomField?: boolean;
    UseContextCustomFieldDataAsPrefill?: boolean;
    DataType?: EVA.Core.CustomFieldDataTypes;
    OptionalAnswers?: { [key: string]: string };
    AllowMultipleAnswers?: boolean;
    DisplayType?: string;
    InputHint?: string;
    MinimumValue?: number;
    MaximumValue?: number;
    MinimumLength?: number;
    MaximumLength?: number;
    MinimumDate?: string;
    MaximumDate?: string;
    Required?: boolean;
  }

  export interface CreateSurveyQuestionResponse {
    /**
     * Entity type: SurveyQuestion
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a survey question
   */
  export interface UpdateSurveyQuestion {
    /**
     * Entity type: SurveyQuestion
     */
    ID: number;
    Text?: string;
    Description?: string;
    Remark?: string;
    BackendID?: string;
    /**
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * A custom field that is available from the SurveyContext.
     * If context for example is `User`, all custom fields of the user are available.
     * If context for example is `Order`, all custom fields from the order and customer (user) on the order are available.
     * 
     * NOTE THAT this value cannot be update. If this question is based upon the custom field, any changes to the markup are rejected.
     * 
     * NOTE THAT any security settings on the custom field are ignored.
     * 
     * NOTE THAT when this value is set, the markup of the custom field is copied as markup for the survey question, and the response data is saved within the survey.
     * If the value also needs to be stored on the context entity self (e.g. order, user, etc), the setting `StoreAnswerOnContextCustomField` needs to be set.
     * When `StoreAnswerOnContextCustomField` is set to true, changes to the custom field itself will also be blocked while the survey is active.
     * Entity type: CustomField
     */
    SurveyContextCustomFieldID?: number;
    StoreAnswerOnContextCustomField?: boolean;
    UseContextCustomFieldDataAsPrefill?: boolean;
    DataType?: EVA.Core.CustomFieldDataTypes;
    OptionalAnswers?: { [key: string]: string };
    AllowMultipleAnswers?: boolean;
    DisplayType?: string;
    InputHint?: string;
    MinimumValue?: number;
    MaximumValue?: number;
    MinimumLength?: number;
    MaximumLength?: number;
    MinimumDate?: string;
    MaximumDate?: string;
    Required?: boolean;
  }

  /**
   * Delete a survey question
   */
  export interface DeleteSurveyQuestion {
    /**
     * Entity type: SurveyQuestion
     */
    ID: number;
  }

  /**
   * Creates a new survey question route
   */
  export interface CreateSurveyQuestionRoute {
    /**
     * Entity type: SurveyQuestion
     */
    ParentQuestionID?: number;
    /**
     * Entity type: SurveyQuestion
     */
    RouteToQuestionID?: number;
    Filter?: EVA.Core.SurveyQuestionRouteFilter;
    ExitBlobID?: string;
    ExitRemarkTitle?: string;
    ExitRemarkMessage?: string;
  }

  export interface CreateSurveyQuestionRouteResponse {
    /**
     * Entity type: SurveyQuestionRoute
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a survey question route
   */
  export interface UpdateSurveyQuestionRoute {
    /**
     * Entity type: SurveyQuestionRoute
     */
    ID: number;
    /**
     * Entity type: SurveyQuestion
     */
    RouteToQuestionID?: number;
    Filter?: EVA.Core.SurveyQuestionRouteFilter;
    ExitBlobID?: string;
    ExitRemarkTitle?: string;
    ExitRemarkMessage?: string;
  }

  /**
   * Delete a survey question route.
   */
  export interface DeleteSurveyQuestionRoute {
    /**
     * Entity type: SurveyQuestionRoute
     */
    ID: number;
  }

  /**
   * Takes in all questions of a survey and sets their sequences.
   */
  export interface SetSurveyQuestionSequences {
    /**
     * Entity type: Survey
     */
    SurveyID: number;
    /**
     * Entity type: SurveyQuestion
     */
    SurveyQuestionIDs: number[];
  }

  /**
   * Takes in all routings of a question and sets their sequences.
   */
  export interface SetSurveyQuestionRoutingSequences {
    /**
     * Entity type: SurveyQuestion
     */
    SurveyQuestionID: number;
    /**
     * Entity type: SurveyQuestionRoute
     */
    SurveyQuestionRoutingIDs: number[];
  }

  /**
   * List the available custom fields for a given survey context
   */
  export interface ListCustomFieldsForSurvey {
    SurveyContext?: EVA.Core.SurveyContext;
    CustomFieldName?: string;
    /**
     * Entity type: CustomFieldType
     */
    CustomFieldTypeIDs?: number[];
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListCustomFieldsForSurveyResponse {
    AvailableCustomFieldTypes?: ListCustomFieldsForSurveyResponse.AvailableCustomFieldType[];
    Result?: EVA.Core.PagedResult<EVA.Core.CustomFieldMetadata>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCustomFieldsForSurveyResponse {
    export interface AvailableCustomFieldType {
      /**
       * Entity type: CustomFieldType
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Get all available surveys for user
   */
  export interface GetAvailableSurveys {
    /**
     * Organization unit in which surveys are available. When omitted, falls back to the current organization. Ignored when logged in user is a customer
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * User for which surveys are available. When omitted, falls back to the current user.
     * Entity type: User
     */
    UserID?: number;
  }

  export interface GetAvailableSurveysResponse {
    Surveys: GetAvailableSurveysResponse.AvailableSurvey[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAvailableSurveysResponse {
    export interface AvailableSurvey {
      /**
       * Entity type: Survey
       */
      ID: number;
      Name: string;
      Description?: string;
      Context: EVA.Core.SurveyContext;
      ContextSourceID?: number;
      Status: EVA.Core.SurveyResponseStatus;
      /**
       * Time the survey was started or the first question was answered.
       */
      FirstInteractionTime?: string;
      /**
       * Time the last question was answered.
       */
      LastInteractionTime?: string;
    }

  }

  /**
   * Get all available surveys for user and given context
   */
  export interface GetAvailableSurveysForContext {
    SurveyContext: EVA.Core.SurveyContext;
    SurveyContextSourceID: number;
    /**
     * Organization unit in which surveys are available. When omitted, falls back to the current organization. Ignored when logged in user is a customer
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  /**
   * Start a survey
   */
  export interface StartSurvey {
    Token?: string;
    /**
     * Entity type: Survey
     */
    SurveyID?: number;
    SurveyContextSourceID?: number;
    /**
     * Entity type: User
     * @deprecated Use `SurveyContextSourceID` instead.
     */
    UserID?: number;
  }

  export interface StartSurveyResponse {
    Token?: string;
    /**
     * The ID of the previous question.
     * Entity type: SurveyQuestion
     */
    PreviousQuestionID?: number;
    /**
     * Question to be answered. When null, the survey is done.
     */
    Question?: SurveyQuestionDto;
    /**
     * Indicates if the question has been answered before.
     */
    HasBeenAnswered?: boolean;
    /**
     * The answer given previously.
     */
    GivenAnswer?: EVA.Core.CustomFieldValue;
    /**
     * Filled when it's the first question of the survey.
     */
    Introduction?: QuestionResponse.SurveyRemark;
    /**
     * Filled when there is no question left to answer.
     */
    ClosingRemark?: QuestionResponse.SurveyRemark;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface SurveyQuestionDto {
    /**
     * Entity type: SurveyQuestion
     */
    ID?: number;
    Text?: string;
    Description?: string;
    Remark?: string;
    BackendID?: string;
    /**
     * @deprecated Use Blob.URL instead.
     */
    BlobURL?: string;
    Blob?: EVA.Core.BlobDto;
    DataType?: EVA.Core.CustomFieldDataTypes;
    OptionalAnswers?: { [key: string]: string };
    AllowMultipleAnswers?: boolean;
    DisplayType?: string;
    InputHint?: string;
    MinimumValue?: number;
    MaximumValue?: number;
    MinimumLength?: number;
    MaximumLength?: number;
    MinimumDate?: string;
    MaximumDate?: string;
    Required?: boolean;
  }

  /**
   * Get a question that is already answered (to navigate back and forth in the survey)
   */
  export interface GetSurveyQuestionAnswer {
    Token: string;
    /**
     * Entity type: SurveyQuestion
     */
    SurveyQuestionID: number;
  }

  export interface GetSurveyQuestionAnswerResponse {
    /**
     * The ID of the next question.
     * Entity type: SurveyQuestion
     */
    NextQuestion?: number;
    /**
     * Indicates if the next question has been answered before.
     */
    NextQuestionHasBeenAnswered: boolean;
    /**
     * The ID of the previous question.
     * Entity type: SurveyQuestion
     */
    PreviousQuestionID?: number;
    /**
     * Question to be answered. When null, the survey is done.
     */
    Question?: SurveyQuestionDto;
    /**
     * Indicates if the question has been answered before.
     */
    HasBeenAnswered?: boolean;
    /**
     * The answer given previously.
     */
    GivenAnswer?: EVA.Core.CustomFieldValue;
    /**
     * Filled when it's the first question of the survey.
     */
    Introduction?: QuestionResponse.SurveyRemark;
    /**
     * Filled when there is no question left to answer.
     */
    ClosingRemark?: QuestionResponse.SurveyRemark;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Answer a survey question
   */
  export interface AnswerSurveyQuestion {
    Token?: string;
    /**
     * Entity type: SurveyQuestion
     */
    SurveyQuestionID?: number;
    /**
     * Required unless the question is optional.
     */
    Answer?: EVA.Core.CustomFieldValue;
  }

  export interface QuestionResponse {
    /**
     * The ID of the previous question.
     * Entity type: SurveyQuestion
     */
    PreviousQuestionID?: number;
    /**
     * Question to be answered. When null, the survey is done.
     */
    Question?: SurveyQuestionDto;
    /**
     * Indicates if the question has been answered before.
     */
    HasBeenAnswered?: boolean;
    /**
     * The answer given previously.
     */
    GivenAnswer?: EVA.Core.CustomFieldValue;
    /**
     * Filled when it's the first question of the survey.
     */
    Introduction?: QuestionResponse.SurveyRemark;
    /**
     * Filled when there is no question left to answer.
     */
    ClosingRemark?: QuestionResponse.SurveyRemark;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace QuestionResponse {
    export interface SurveyRemark {
      Title?: string;
      Message?: string;
      /**
       * @deprecated Use Blob.URL instead.
       */
      BlobURL?: string;
      Blob?: EVA.Core.BlobDto;
    }

  }

  /**
   * Restart the survey response
   */
  export interface RestartSurveyResponse {
    Token: string;
  }

  /**
   * Cancel the survey response
   */
  export interface CancelSurveyResponse {
    Token?: string;
    /**
     * Entity type: Survey
     */
    SurveyID?: number;
    SurveyContextSourceID?: number;
    /**
     * Entity type: User
     * @deprecated Use `SurveyContextSourceID` instead.
     */
    UserID?: number;
  }

  /**
   * Returns all translatable items within the survey.
   */
  export interface GetSurveyTranslatableItems {
    /**
     * Entity type: Survey
     */
    SurveyID: number;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Filter on specific country. Only returns exact matches. Translations set solely on language will not be returned because the combination does not exist.
     * Entity type: Country
     */
    CountryID?: string;
  }

  export interface GetSurveyTranslatableItemsResponse {
    TranslatableItems: TranslatableItem[];
    QuestionTranslatableItems?: QuestionTranslatableItem[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface TranslatableItem {
    TranslationKey?: string;
    OriginalValue?: string;
    TranslationValue?: string;
  }

  export interface QuestionTranslatableItem {
    /**
     * Entity type: SurveyQuestion
     */
    QuestionID: number;
    TranslatableItems?: TranslatableItem[];
    QuestionRouteTranslatableItems?: QuestionRouteTranslatableItem[];
  }

  export interface QuestionRouteTranslatableItem {
    /**
     * Entity type: SurveyQuestionRoute
     */
    QuestionRouteID: number;
    TranslatableItems: TranslatableItem[];
  }

  /**
   * Set all translatable items within the survey.
   */
  export interface SetSurveyTranslatableItems {
    /**
     * Entity type: Survey
     */
    SurveyID: number;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    TranslatableItems?: TranslatableItem[];
    QuestionTranslatableItems?: QuestionTranslatableItem[];
  }

  /**
   * Returns the translation status of the survey.
   */
  export interface GetSurveyTranslationStatus {
    /**
     * Entity type: Survey
     */
    SurveyID?: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  export interface GetSurveyTranslationStatusResponse {
    Statuses: GetSurveyTranslationStatusResponse.TranslationStatus[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSurveyTranslationStatusResponse {
    export interface TranslationStatus {
      /**
       * Entity type: Language
       */
      LanguageID: string;
      /**
       * Entity type: Country
       */
      CountryID: string;
      /**
       * Percentage of translations that are set. 0-100
       */
      Progress: number;
    }

  }

  export interface ExportStockMutationSummary {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetIDs?: number[];
    ProductSearchFilters?: { [key: string]: EVA.Core.FilterModel };
    /**
     * The date to use for the stock mutation summary. Only the date component is used, which means if you send in 2023-06-29T16:00 all stock mutations on 2023-06-29 will be included.
     */
    Date?: string;
    /**
     * The date and time to use for the stock mutation summary, in UTC. Unlike the `Date` field, the time component is used.
     */
    DateTime?: string;
    EmailAddress?: string;
    SplitUnitCost?: boolean;
  }

  /**
   * Returns a list of stock mutation reasons that are currently active for the supplied OrganizationUnitID.
   */
  export interface GetRecentlyUsedStockMutationReasons {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetRecentlyUsedStockMutationReasonsResponse {
    Reasons: GetRecentlyUsedStockMutationReasonsResponse.StockMutationReason[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetRecentlyUsedStockMutationReasonsResponse {
    export interface StockMutationReason {
      ID: number;
      /**
       * Usually an internal & unchangable code - for display, use `Description`
       */
      Name: string;
      Description: string;
    }

  }

  /**
   * This service allows retrieval of StockMutations, filtered by a Query and/or a set of filters.
   */
  export interface SearchStockMutations {
    Query?: string;
    PageConfig?: EVA.Core.ScrollablePageConfig<EVA.Core.StockMutationFilters>;
    IncludedFields?: string[];
  }

  export interface SearchStockMutationsResponse {
    Result: EVA.Core.ScrollablePagedResult<EVA.Core.SearchStockMutationResult>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List internal and custom stock mutation reasons.
   */
  export interface ListStockMutationReasons {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListStockMutationReasonsModelFilters>;
  }

  export interface ListStockMutationReasonsResponse {
    Result?: EVA.Core.PagedResult<StockMutationReason>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface StockMutationReason {
    /**
     * Entity type: StockMutationReasons
     */
    ID?: number;
    /**
     * Usually an internal & unchangable code - for display, use `Description`
     */
    Name?: string;
    Description?: string;
    BackendID?: string;
    RequireRemarkForMove?: boolean;
    RequireRemarkForAdjust?: boolean;
    IsSystem?: boolean;
  }

  /**
   * Create a custom stock mutation reason.
   */
  export interface CreateStockMutationReason {
    Name: string;
    Description?: string;
    BackendID?: string;
    RequireRemarkForMove: boolean;
    RequireRemarkForAdjust: boolean;
  }

  export interface CreateStockMutationReasonResponse {
    /**
     * Entity type: StockMutationReasons
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a custom or internal stock mutation reason. The name cannot be modified for internal reasons.
   */
  export interface UpdateStockMutationReason {
    /**
     * Entity type: StockMutationReasons
     */
    ID: number;
    Name?: string;
    Description?: string;
    BackendID?: string;
    RequireRemarkForMove?: boolean;
    RequireRemarkForAdjust?: boolean;
  }

  /**
   * Delete a custom stock mutation reason.
   */
  export interface DeleteStockMutationReason {
    /**
     * Entity type: StockMutationReasons
     */
    ID?: number;
  }

  /**
   * Download an overview of the stock for the specified organization units or the specified organization unit set.
   * NOTE THAT when `OrganizationUnitSetID` is provided, `OrganizationUnitIDs` is ignored.
   */
  export interface DownloadStockOverview {
    /**
     * The organization units to list the stock for
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs: number[];
    /**
     * The organization units set to list the stock for
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Filter on products, is used as input in ElasticSearch
     */
    Query?: string;
    /**
     * View the stock on a certain moment in time
     */
    DateTime?: string;
    /**
     * Only these products will be returned
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
  }

  export interface InitialReplenishment {
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
    EmailAddress: string;
  }

  export interface InitialReplenishmentResponse {
    TaskID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListCumulativeStock {
    PageConfig: EVA.Core.PageConfig<EVA.Core.ListCumulativeStockFilter>;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface ListCumulativeStockResponse {
    Result?: EVA.Core.PagedResult<ListCumulativeStockResponse.Model>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCumulativeStockResponse {
    export interface Model {
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      OrganizationUnitBackendID: string;
      Product: any;
      QuantityOnHand: number;
    }

  }

  /**
   * Creates a StockAllocationRule for the SupplierOrganizationUnitID and OrganizationUnitID.
   * 
   * - If `ProductID` is left null, this is a 'catch-all' rule, meaning that the rule applies to all products (but it can still be overriden by a rule with a specific `ProductID`).
   * Only a reservation can be a catch-all rule, a limitation must specify a `ProductID` or a `ProductSearchTemplateID`.
   * - `RefillPeriodInDays` is ignored when the Type is not a `Limitation`, as only a `Limitation` can be refilled.
   * - If `ValueType` is `Percentage`, then `Value` must be between 0 and 100.
   * - `OrganizationUnitSupplierID` is a reference to a supplier relationship between OrganizationUnits, not to an OrganizationUnit.
   * This is for example the supplier relationship between a warehouse and a store. Available OrganizationUnitSuppliers can be found through `ListOrganizationUnitSuppliers`.
   * - When left empty, OrganizationUnitID defaults to the OrganizationUnitID on the OrganizationUnitSupplier. So if the supplier relationship is between a warehouse and a store, this will
   * default to the store. When specified, the OrganizationUnitID *must* be one that the supplier of the relationship *directly* supplies to.
   * For example, if a supplier relationship exists between a warehouse and a country OrganizationUnit then you *can't* specify a store that falls
   * under that country OrganizationUnit. In that case, you should create a StockAllocationRule between the country and the store,
   * not between the warehouse and the store.
   */
  export interface CreateStockAllocationRule {
    /**
     * Entity type: OrganizationUnitSupplier
     */
    OrganizationUnitSupplierID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    Value: number;
    ValueType: EVA.Core.StockAllocationRuleValueTypes;
    RefillPeriodInDays?: number;
    Type: EVA.Core.StockAllocationRuleTypes;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    StartDate?: string;
    EndDate?: string;
    /**
     * In case there is not enough stock, Priority is used to determine who gets preference. A higher Priority means a higher preference.
     */
    Priority: number;
  }

  export interface CreateStockAllocationRuleResponse {
    /**
     * Entity type: StockAllocationRule
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Allows updating a StockAllocationRule. Only `Value`, `ValueType`, `RefillPeriodInDays`, `Priority`, `StartDate` and `EndDate` can be updated. To update the other fields, the rule must be re-created.
   */
  export interface UpdateStockAllocationRule {
    /**
     * Entity type: StockAllocationRule
     */
    ID: number;
    Value?: number;
    ValueType?: EVA.Core.StockAllocationRuleValueTypes;
    RefillPeriodInDays?: number;
    StartDate?: string;
    EndDate?: string;
    /**
     * In case there is not enough stock, Priority is used to determine who gets preference. A higher Priority means a higher preference.
     */
    Priority?: number;
  }

  /**
   * Deletes a StockAllocationRule.
   */
  export interface DeleteStockAllocationRule {
    /**
     * Entity type: StockAllocationRule
     */
    ID: number;
  }

  export interface ListStockAllocationRules {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListStockAllocationRulesFilter>;
  }

  export interface ListStockAllocationRulesResponse {
    Result?: EVA.Core.PagedResult<EVA.Core.ListStockAllocationRulesItem>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Reallocates the stock for a StockLocationRule with `Type` = `Limitation`.
   */
  export interface RefillStockAllocation {
    /**
     * Entity type: StockAllocationRule
     */
    ID: number;
    ValueOverride?: number;
  }

  /**
   * A service to upload an Excel containing StockAllocationRules that should be created or updated according to the values specified in the sheet.
   * 
   * An example Excel file containing the expected structure can be retrieved by calling the `DownloadStockAllocationRulesExcel` service. Important to note however
   * is that the first column ('C/U/D' which stands for Create/Update/Delete) will be empty in the Excel file returned by `DownloadStockAllocationRulesExcel`, but to update
   * or create a record the first column must have a value according to what you want to do with it. To create a row, the first column must contain 'C', to update it must contain 'U' (and the ID column must have a value pointing to the correct record)
   * and to delete it it must contain 'D'.
   */
  export interface UploadStockAllocationRulesExcel {
    Data: string;
  }

  /**
   * Downloads an Excel file containing StockAllocationRules according to the specified PageConfig. To get an empty template Excel, specify a page size of zero.
   */
  export interface DownloadStockAllocationRulesExcel {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListStockAllocationRulesFilter>;
  }

  /**
   * Create a new StockLabel
   * 
   * Name is required
   */
  export interface CreateStockLabel {
    Name?: string;
    Description?: string;
    DisplayName?: string;
    LedgerClassID?: string;
  }

  export interface CreateStockLabelResponse {
    /**
     * Entity type: StockLabel
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update the description or ledgerclass of StockLabel
   */
  export interface UpdateStockLabel {
    /**
     * Entity type: StockLabel
     */
    ID: number;
    Description?: string;
    DisplayName?: string;
    LedgerClassID?: string;
  }

  /**
   * Create a new Carrier
   */
  export interface CreateCarrier {
    Name: string;
    Code?: string;
    BackendID?: string;
  }

  export interface CreateCarrierResponse {
    /**
     * Entity type: Carrier
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get a Carrier
   */
  export interface GetCarrierByID {
    /**
     * Entity type: Carrier
     */
    ID: number;
  }

  export interface GetCarrierByIDResponse {
    ID: number;
    Name: string;
    Code: string;
    BackendID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the Carriers
   */
  export interface ListCarriers {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListCarriersFilter>;
  }

  export interface ListCarriersResponse {
    Result?: EVA.Core.PagedResult<CarrierDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface CarrierDto {
    /**
     * Entity type: Carrier
     */
    ID: number;
    Name: string;
    Code: string;
    BackendID: string;
  }

  /**
   * Update a Carrier
   */
  export interface UpdateCarrier {
    /**
     * Entity type: Carrier
     */
    ID: number;
    Name: string;
    Code?: string;
    BackendID?: string;
  }

  /**
   * Returns available handlers for shipping methods
   */
  export interface GetAvailableShippingMethodHandlers {
  }

  export interface GetAvailableShippingMethodHandlersResponse {
    Handlers: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListShippingMethods {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListShippingMethodsFilter>;
  }

  export interface ListShippingMethodsResponse {
    Result?: EVA.Core.PagedResult<ListShippingMethodsResponse.ShippingMethodDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListShippingMethodsResponse {
    export interface ShippingMethodDto {
      /**
       * Entity type: ShippingMethod
       */
      ID: number;
      Name: string;
      /**
       * Entity type: Carrier
       */
      CarrierID: number;
      CarrierName: string;
      /**
       * Entity type: ProductSearchTemplate
       */
      ProductSearchTemplateID?: number;
      ProductSearchTemplateName: string;
    }

  }

  /**
   * Get an OrganizationUnitShippingMethod
   */
  export interface GetOrganizationUnitShippingMethodByID {
    /**
     * Entity type: OrganizationUnitShippingMethod
     */
    ID: number;
  }

  export interface GetOrganizationUnitShippingMethodByIDResponse {
    /**
     * Entity type: OrganizationUnitShippingMethod
     */
    ID: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID: number;
    ShippingMethodName: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    Priority: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create an OrganizationUnitShippingMethod
   * 
   * A ShippingMethod, OrganizationUnit and Priority are required
   */
  export interface CreateOrganizationUnitShippingMethod {
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    Priority: number;
  }

  export interface CreateOrganizationUnitShippingMethodResponse {
    /**
     * Entity type: OrganizationUnitShippingMethod
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an OrganizationUnitShippingMethod
   * 
   * A ShippingMethod, OrganizationUnit and Priority are required
   */
  export interface UpdateOrganizationUnitShippingMethod {
    /**
     * Entity type: OrganizationUnitShippingMethod
     */
    ID: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    Priority: number;
  }

  /**
   * Delete an OrganizationUnitShippingMethod
   */
  export interface DeleteOrganizationUnitShippingMethod {
    /**
     * Entity type: OrganizationUnitShippingMethod
     */
    ID: number;
  }

  /**
   * List the OrganizationUnitShippingMethods
   * 
   * There are filters available on:
   *  - ShippingMethodID
   *  - OrganizationUnitID
   */
  export interface ListOrganizationUnitShippingMethods {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListOrganizationUnitShippingMethodsFilter>;
  }

  export interface ListOrganizationUnitShippingMethodsResponse {
    Result?: EVA.Core.PagedResult<OrganizationUnitShippingMethodDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface OrganizationUnitShippingMethodDto {
    /**
     * Entity type: OrganizationUnitShippingMethod
     */
    ID: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID: number;
    ShippingMethodName: string;
    ShippingMethodCarrierName: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    Priority: number;
  }

  /**
   * Create a new package, with dimensions in millimeters (mm) and weight in grams (g).
   */
  export interface CreatePackage {
    Description?: string;
    Code?: string;
    /**
     * mm
     * Min: 1
     */
    Length: number;
    /**
     * mm
     * Min: 1
     */
    Width: number;
    /**
     * mm
     * Min: 1
     */
    Height: number;
    /**
     * g
     * Min: 1
     */
    Weight: number;
  }

  export interface CreatePackageResponse {
    /**
     * Entity type: Package
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get all packages, with dimensions in millimeters (mm) and weight in grams (g).
   */
  export interface GetPackages {
  }

  export interface GetPackagesResponse {
    Result: PackageDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface PackageDto {
    IsDefault: boolean;
    /**
     * Entity type: Package
     */
    ID: number;
    Description: string;
    Code?: string;
    Length: number;
    Width: number;
    Height: number;
    Weight: number;
  }

  /**
   * Update a package, with dimensions in millimeters (mm) and weight in grams (g).
   */
  export interface UpdatePackage {
    /**
     * Entity type: Package
     */
    ID: number;
    Description?: string;
    Code?: string;
    /**
     * mm
     * Min: 1
     */
    Length: number;
    /**
     * mm
     * Min: 1
     */
    Width: number;
    /**
     * mm
     * Min: 1
     */
    Height: number;
    /**
     * g
     * Min: 1
     */
    Weight: number;
  }

  /**
   * Delete a package
   */
  export interface DeletePackage {
    /**
     * Entity type: Package
     */
    ID: number;
  }

  /**
   * Get all packages for a carrier, with dimensions in millimeters (mm) and weight in grams (g).
   */
  export interface GetPackagesForCarrier {
    /**
     * Entity type: Carrier
     */
    CarrierID: number;
  }

  /**
   * Add a package to a carrier
   */
  export interface AddPackageToCarrier {
    /**
     * Entity type: Package
     */
    PackageID: number;
    /**
     * Entity type: Carrier
     */
    CarrierID: number;
    IsDefault?: boolean;
  }

  /**
   * Remove a package from a carrier
   */
  export interface RemovePackageFromCarrier {
    /**
     * Entity type: Package
     */
    PackageID: number;
    /**
     * Entity type: Carrier
     */
    CarrierID: number;
  }

  /**
   * Get a `ShippingCost`
   */
  export interface GetShippingCostByID {
    /**
     * Entity type: ShippingCost
     */
    ID: number;
  }

  export interface GetShippingCostByIDResponse {
    /**
     * Entity type: ShippingCost
     */
    ID: number;
    BackendID: string;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    ShippingMethodName: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    MinimumOrderAmount?: number;
    MaximumOrderAmount?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Amount: number;
    ZipCodes: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a `ShippingCost`
   */
  export interface CreateShippingCost {
    BackendID?: string;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    ZipCodes?: string[];
    MinimumOrderAmount?: number;
    MaximumOrderAmount?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Amount?: number;
  }

  export interface CreateShippingCostResponse {
    /**
     * Entity type: ShippingCost
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a `ShippingCost`
   */
  export interface UpdateShippingCost {
    /**
     * Entity type: ShippingCost
     */
    ID: number;
    BackendID?: string;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    ZipCodes?: string[];
    MinimumOrderAmount?: number;
    MaximumOrderAmount?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Amount?: number;
  }

  /**
   * Delete a `ShippingCost`
   */
  export interface DeleteShippingCost {
    /**
     * Entity type: ShippingCost
     */
    ID: number;
  }

  /**
   * List the available `ShippingCost`
   */
  export interface ListShippingCosts {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListShippingCostsFilter>;
  }

  export interface ListShippingCostsResponse {
    Result?: EVA.Core.PagedResult<ShippingCostDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ShippingCostDto {
    /**
     * Entity type: ShippingCost
     */
    ID: number;
    BackendID: string;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    ShippingMethodName: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    ZipCodes: string[];
    MinimumOrderAmount?: number;
    MaximumOrderAmount?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Amount: number;
  }

  /**
   * Create a new ShippingMethod
   */
  export interface CreateShippingMethod {
    Name: string;
    Code?: string;
    Handler?: string;
    /**
     * Entity type: Carrier
     */
    CarrierID: number;
    DeliveryType: EVA.Core.ShippingMethodDeliveryTypes;
    ExcludeProductsFromShippingMethod?: boolean;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    PaymentHandledByCarrier?: boolean;
    ShippingHandledByCarrier?: boolean;
    InvoicingHandledByCarrier?: boolean;
    SupportsPickupAddress?: boolean;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  export interface CreateShippingMethodResponse {
    /**
     * Entity type: ShippingMethod
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get an existing ShippingMethod by ID
   */
  export interface GetShippingMethodByID {
    /**
     * Entity type: ShippingMethod
     */
    ID: number;
  }

  export interface GetShippingMethodByIDResponse {
    /**
     * Entity type: ShippingMethod
     */
    ID: number;
    Name: string;
    Code: string;
    Handler: string;
    PaymentHandledByCarrier: boolean;
    ShippingHandledByCarrier: boolean;
    InvoicingHandledByCarrier: boolean;
    /**
     * Entity type: Carrier
     */
    CarrierID: number;
    CarrierName: string;
    CarrierCode: string;
    DeliveryType: EVA.Core.ShippingMethodDeliveryTypes;
    ExcludeProductsFromShippingMethod: boolean;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: EVA.Core.CustomFieldValueWithOptions };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the ShippingMethods
   */
  export interface ListManagementShippingMethods {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListShippingMethodsFilter>;
  }

  export interface ListManagementShippingMethodsResponse {
    Result?: EVA.Core.PagedResult<ListManagementShippingMethodsResponse.Data>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListManagementShippingMethodsResponse {
    export interface Data {
      /**
       * Entity type: ShippingMethod
       */
      ID: number;
      Name: string;
      Code: string;
      Handler: string;
      PaymentHandledByCarrier: boolean;
      ShippingHandledByCarrier: boolean;
      InvoicingHandledByCarrier: boolean;
      /**
       * Entity type: Carrier
       */
      CarrierID: number;
      CarrierName: string;
      CarrierCode: string;
      DeliveryType: EVA.Core.ShippingMethodDeliveryTypes;
      /**
       * Entity type: ProductSearchTemplate
       */
      ProductSearchTemplateID?: number;
      ProductSearchTemplateName: string;
      /**
       * Entity type: CustomField
       */
      CustomFieldValuesWithOptions?: { [key: number]: EVA.Core.CustomFieldValueWithOptions };
    }

  }

  /**
   * Update an existing ShippingMethod
   */
  export interface UpdateShippingMethod {
    /**
     * Entity type: ShippingMethod
     */
    ID: number;
    Name: string;
    Code?: string;
    Handler?: string;
    /**
     * Entity type: Carrier
     */
    CarrierID: number;
    DeliveryType: EVA.Core.ShippingMethodDeliveryTypes;
    ExcludeProductsFromShippingMethod?: boolean;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    PaymentHandledByCarrier?: boolean;
    ShippingHandledByCarrier?: boolean;
    InvoicingHandledByCarrier?: boolean;
    SupportsPickupAddress?: boolean;
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  /**
   * Creates a new ShippingMethodTransportationTime.
   */
  export interface CreateShippingMethodTransportationTime {
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    /**
     * Entity type: Country
     */
    FromCountryID?: string;
    /**
     * Entity type: Country
     */
    ToCountryID?: string;
    TimeInDays: number;
  }

  export interface CreateShippingMethodTransportationTimeResponse {
    /**
     * Entity type: ShippingMethodTransportationTime
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates a ShippingMethodTransportationTime.
   * If you want to preserve the current value of a field, provide it to this service as the current value;
   *  if it's sent in as null the value will be set to null.
   */
  export interface UpdateShippingMethodTransportationTime {
    /**
     * Entity type: ShippingMethodTransportationTime
     */
    ID: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    /**
     * Entity type: Country
     */
    FromCountryID?: string;
    /**
     * Entity type: Country
     */
    ToCountryID?: string;
    TimeInDays: number;
  }

  /**
   * Deletes a ShippingMethodTransportationTime.
   */
  export interface DeleteShippingMethodTransportationTime {
    /**
     * Entity type: ShippingMethodTransportationTime
     */
    ID: number;
  }

  /**
   * Lists current ShippingMethodTransportationTimes, optionally filtered.
   */
  export interface ListShippingMethodTransportationTimes {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListTransportationTimesFilter>;
  }

  export interface ListShippingMethodTransportationTimesResponse {
    Result: EVA.Core.PagedResult<EVA.Core.ShippingMethodTransportationTimeItem>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Gets a ShippingRestriction by its ID and returns the set's basic information.
   */
  export interface GetShippingRestrictionByID {
    /**
     * Entity type: ShippingRestriction
     */
    ID: number;
  }

  export interface GetShippingRestrictionByIDResponse {
    /**
     * Entity type: ShippingRestriction
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    OrganizationUnitSetName?: string;
    OrganizationUnitSetDescription?: string;
    OrganizationUnitSetType?: EVA.Core.OrganizationUnitSetTypes;
    CountryID?: string;
    CountryName?: string;
    ZipCodeFrom: string;
    ZipCodeTo: string;
    /**
     * Entity type: ProductPropertyType
     */
    ProductPropertyTypeID: string;
    ProductPropertyTypeValues: string[];
    Type: EVA.Core.ShippingRestrictionType;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    CheckScriptOnly: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new ShippingRestriction.
   */
  export interface CreateShippingRestriction {
    /**
     * Either an OrganizationUnitID or an OrganizationUnitSetID is required when there is no ScriptID set or CheckScriptOnly is 'false'
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Either an OrganizationUnitID or an OrganizationUnitSetID is required when there is no ScriptID set or CheckScriptOnly is 'false'
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Required when there is no ScriptID set or CheckScriptOnly is 'false'
     * Entity type: Country
     */
    CountryID?: string;
    ZipCodeFrom?: string;
    ZipCodeTo?: string;
    /**
     * Entity type: ProductPropertyType
     */
    ProductPropertyTypeID?: string;
    ProductPropertyTypeValues?: string[];
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    /**
     * When set to `true`, the script will be leading for the entire order. Default is `true`
     */
    CheckScriptOnly?: boolean;
    Type: EVA.Core.ShippingRestrictionType;
  }

  export interface CreateShippingRestrictionResponse {
    /**
     * Entity type: ShippingRestriction
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates a ShippingRestriction.
   */
  export interface UpdateShippingRestriction {
    /**
     * Entity type: ShippingRestriction
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    ZipCodeFrom?: string;
    ZipCodeTo?: string;
    /**
     * Entity type: ProductPropertyType
     */
    ProductPropertyTypeID?: string;
    ProductPropertyTypeValues?: string[];
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    /**
     * When set to `true`, the script will be leading for the entire order.
     */
    CheckScriptOnly?: boolean;
    Type?: EVA.Core.ShippingRestrictionType;
  }

  /**
   * Deletes a ShippingRestriction by its ID.
   */
  export interface DeleteShippingRestriction {
    /**
     * Entity type: ShippingRestriction
     */
    ID: number;
  }

  /**
   * List ShippingRestrictions. Returns a paged response. Filter options are;
   */
  export interface ListShippingRestrictions {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListShippingRestrictionsFilter>;
  }

  export interface ListShippingRestrictionsResponse {
    Result?: EVA.Core.PagedResult<ShippingRestrictionDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ShippingRestrictionDto {
    /**
     * Entity type: ShippingRestriction
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    OrganizationUnitSetName?: string;
    OrganizationUnitSetDescription?: string;
    OrganizationUnitSetType?: EVA.Core.OrganizationUnitSetTypes;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    CountryName?: string;
    ZipCodeFrom: string;
    ZipCodeTo: string;
    /**
     * Entity type: ProductPropertyType
     */
    ProductPropertyTypeID: string;
    ProductPropertyTypeValues: string[];
    Type: EVA.Core.ShippingRestrictionType;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    /**
     * ID of the script that is associated with this shipping restriction.
     * Entity type: Script
     */
    ScriptID?: number;
    /**
     * Name of the script that is associated with this shipping restriction.
     */
    ScriptName?: string;
  }

  /**
   * @deprecated Excel download/upload not supported anymore
   * Download ShippingRestrictions. Service returns an URL to an Excel file.
   */
  export interface DownloadShippingRestrictions {
  }

  /**
   * @deprecated Excel download/upload not supported anymore
   * Upload ShippingRestrictions. Expected format is an Excel file.
   */
  export interface UploadShippingRestrictions {
    Data: string;
  }

  export interface UploadShippingRestrictionsResponse {
    Messages: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetAvailableRoles {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  export interface GetAvailableRolesResponse {
    AvailableRoles: RoleDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface RoleDto {
    /**
     * Entity type: Role
     */
    ID: number;
    Name: string;
    UserType: EVA.Core.UserTypes;
    Code: string;
    /**
     * If this is set, any users that have not logged in after said days will get disabled
     */
    DeactivateUserAfterInactiveDays?: number;
    IsSystemDefined: boolean;
  }

  /**
   * @deprecated Just use `GetCurrentUser`
   * Retrieve the currently assigned functionalities flattened on specified OU.
   */
  export interface GetFlattenedFunctionalityByUserID {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetFlattenedFunctionalityByUserIDResponse {
    Functionalities: string[];
    ScopedFunctionalities: FunctionalityWithScope[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface FunctionalityWithScope {
    Functionality: string;
    Scope: EVA.Core.FunctionalityScope;
    ElevationType: EVA.Core.ElevationType;
    /**
     * @deprecated Replaced by `ElevationType`
     */
    RequiresElevation: boolean;
  }

  /**
   * Returns all available categories for functionalities
   */
  export interface GetFunctionalityCategories {
  }

  export interface GetFunctionalityCategoriesResponse {
    Categories: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetSupportedFunctionalities {
    Category?: string;
  }

  export interface GetSupportedFunctionalitiesResponse {
    ScopedFunctionalities: GetSupportedFunctionalitiesResponse.FunctionalityDefinition[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSupportedFunctionalitiesResponse {
    export interface FunctionalityDefinition {
      Name: string;
      Description: string;
      Unscoped: boolean;
      Categories: string[];
    }

  }

  /**
   * Fetch the current attached roles to an User in a specific OrganizationUnit
   */
  export interface GetUserRolesByUserID {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetUserRolesByUserIDResponse {
    Result: GetUserRolesByUserIDResponse.UserRoleDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetUserRolesByUserIDResponse {
    export interface UserRoleDto {
      /**
       * Entity type: UserRole
       */
      ID: number;
      /**
       * Entity type: User
       */
      UserID: number;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID?: number;
      /**
       * Entity type: Role
       */
      RoleID: number;
      EndDate?: string;
      Role: RoleDto;
    }

  }

  /**
   * Return the current Roles for an User.
   */
  export interface GetUserRoles {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface GetUserRolesResponse {
    Roles: UserRoleModel[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface UserRoleModel {
    /**
     * Entity type: Role
     */
    RoleID?: number;
    RoleName?: string;
    RoleCode?: string;
    EndDate?: string;
    UserType?: EVA.Core.UserTypes;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    OrganizationUnitSetName?: string;
    OrganizationUnitSetBackendID?: string;
    /**
     * Entity type: OrganizationUnit
     * @deprecated No longer used
     */
    OrganizationUnitID?: number;
    /**
     * @deprecated No longer used
     */
    OrganizationUnitBackendID?: string;
    /**
     * @deprecated No longer used
     */
    OrganizationUnitName?: string;
  }

  /**
   * Lists organization units and roles for user
   */
  export interface ListUserOrganizationUnitRoles {
    /**
     * Entity type: User
     */
    UserID: number;
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListUserOrganizationUnitRolesFilter>;
  }

  export interface ListUserOrganizationUnitRolesResponse {
    Result?: EVA.Core.PagedResult<ListUserOrganizationUnitRolesResponse.OrganizationUnit>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListUserOrganizationUnitRolesResponse {
    export interface OrganizationUnit {
      ID: number;
      OrganizationUnitSetID: number;
      Name: string;
      Roles: OrganizationUnit.Role[];
    }

    export namespace OrganizationUnit {
      export interface Role {
        ID: number;
        Name: string;
        Code: string;
      }

    }

  }

  /**
   * Get role with its functionalities by it's ID.
   */
  export interface GetRole {
    /**
     * Entity type: Role
     */
    ID: number;
    /**
     * Merge the normal and elevated functionalities into single records.
     * @deprecated Only controls the deprecated response property `ScopedFunctionalities`
     */
    FlattenElevationFunctionalities?: boolean;
  }

  export interface GetRoleResponse {
    Result: RoleWithFunctionalitiesDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface RoleWithFunctionalitiesDto {
    /**
     * Entity type: Role
     */
    ID: number;
    Name: string;
    UserType: EVA.Core.UserTypes;
    Code: string;
    /**
     * If this is set, any users that have not logged in after said days will get disabled
     */
    DeactivateUserAfterInactiveDays?: number;
    IsSystemDefined: boolean;
    Functionalities: Record<'None' | 'Elevation' | 'Verification', { [key: string]: EVA.Core.FunctionalityScope }>;
    /**
     * @deprecated Replaced by `Functionalities`
     */
    ScopedFunctionalities: RoleWithFunctionalitiesDto.GroupedFunctionalityWithScope[];
  }

  export namespace RoleWithFunctionalitiesDto {
    /**
     * @deprecated Replaced by `Functionalities`
     */
    export interface GroupedFunctionalityWithScope {
      Functionality: string;
      /**
       * Scopes available for this functionality. Should be set to `None (0)` for unscoped functionalities.
       */
      Scope?: EVA.Core.FunctionalityScope;
      /**
       * Scopes available only when elevated for this functionality. Should be set to `None (0)` for unscoped functionalities.
       */
      ElevationScope?: EVA.Core.FunctionalityScope;
      /**
       * This is not required anymore, this is determined based on the availability of the ElevationScope property.
       */
      RequiresElevation?: boolean;
    }

  }

  /**
   * List all roles.
   */
  export interface ListRoles {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListRolesFilter>;
  }

  export interface ListRolesResponse {
    Result?: EVA.Core.PagedResult<RoleDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new role.
   */
  export interface CreateRole {
    Name: string;
    Code?: string;
    UserType?: EVA.Core.UserTypes;
    /**
     * If this is set, any users that have not logged in after said days will get disabled
     */
    DeactivateUserAfterInactiveDays?: number;
  }

  export interface CreateRoleResponse {
    /**
     * Entity type: Role
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a role.
   */
  export interface UpdateRole {
    /**
     * Entity type: Role
     */
    ID: number;
    Name?: string;
    UserType?: EVA.Core.UserTypes;
    Code?: string;
    /**
     * If this is set, any users that have not logged in after said days will get disabled
     */
    DeactivateUserAfterInactiveDays?: number;
  }

  /**
   * Delete a role.
   */
  export interface DeleteRole {
    /**
     * Entity type: Role
     */
    ID: number;
  }

  /**
   * @deprecated Replaced by `UpdateRoleFunctionalities`
   * Detaches functionalities from a role.
   */
  export interface DetachFunctionalitiesFromRole {
    /**
     * Entity type: Role
     */
    RoleID: number;
    ScopedFunctionalities?: FunctionalityWithScope[];
  }

  /**
   * @deprecated Replaced by `UpdateRoleFunctionalities`
   * Attach functionalities to a role.
   */
  export interface AttachFunctionalitiesToRole {
    /**
     * Entity type: Role
     */
    RoleID: number;
    ScopedFunctionalities?: FunctionalityWithScope[];
  }

  /**
   * Update functionalities on a role.
   */
  export interface UpdateRoleFunctionalities {
    /**
     * Entity type: Role
     */
    RoleID: number;
    Functionalities: UpdateRoleFunctionalities.FunctionalityToUpdate[];
  }

  export namespace UpdateRoleFunctionalities {
    export interface FunctionalityToUpdate {
      Functionality: string;
      /**
       * Full list of scopes defined for this functionality - anything omitted will be deleted, submitting an empty object will remove the functionality for this role entirely.
       */
      Scopes: Record<'None' | 'Create' | 'Edit' | 'Delete' | 'View' | 'Manage' | 'Settings' | 'Scripting' | 'All', EVA.Core.ElevationType>;
      /**
       * Scopes available for this functionality. Should be set to `None (0)` for unscoped functionalities. To remove this functionality, set to `null`.
       * @deprecated Replaced by `Scopes`
       */
      Scope?: EVA.Core.FunctionalityScope;
      /**
       * Scopes available only when elevated for this functionality. Should be set to `None (0)` for unscoped functionalities. To remove this functionality, set to `null`.
       * @deprecated Replaced by `Scopes`
       */
      ElevationScope?: EVA.Core.FunctionalityScope;
    }

  }

  /**
   * List role functionalities.
   */
  export interface ListRoleFunctionalities {
    /**
     * Entity type: Role
     */
    RoleID: number;
    ShowAll?: boolean;
    PageConfig?: EVA.Core.PageConfig<ListRoleFunctionalitiesFilter>;
  }

  export interface ListRoleFunctionalitiesFilter {
    Functionality?: string;
  }

  export interface ListRoleFunctionalitiesResponse {
    Result?: EVA.Core.PagedResult<ListRoleFunctionalitiesItem>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListRoleFunctionalitiesItem {
    Name: string;
    Description: string;
    Categories: string[];
    Scopes: Record<'None' | 'Create' | 'Edit' | 'Delete' | 'View' | 'Manage' | 'Settings' | 'Scripting' | 'All', EVA.Core.ElevationType>;
  }

  export interface GetRoleSetByID {
    /**
     * Entity type: RoleSet
     */
    ID: number;
  }

  export interface GetRoleSetByIDResponse {
    /**
     * Entity type: RoleSet
     */
    ID: number;
    Name: string;
    BackendID?: string;
    Roles: RoleDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListRoleSets {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListRoleSetsResponse {
    Result?: EVA.Core.PagedResult<ListRoleSetsResponse.RoleSetDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListRoleSetsResponse {
    export interface RoleSetDto {
      /**
       * Entity type: RoleSet
       */
      ID: number;
      Name: string;
      BackendID?: string;
    }

  }

  export interface CreateRoleSet {
    Name: string;
    BackendID?: string;
  }

  export interface CreateRoleSetResponse {
    /**
     * Entity type: RoleSet
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface UpdateRoleSet {
    /**
     * Entity type: RoleSet
     */
    ID: number;
    Name: string;
    BackendID?: string;
  }

  export interface DeleteRoleSet {
    /**
     * Entity type: RoleSet
     */
    ID: number;
  }

  export interface SetRolesForRoleSet {
    /**
     * Entity type: RoleSet
     */
    RoleSetID: number;
    /**
     * Entity type: Role
     */
    Roles?: number[];
  }

  export interface GetRolesForOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetRolesForOrganizationUnitResponse {
    Roles: RoleDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetRolesForOrganizationUnitSet {
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Returns roles that are assignable for all organization units within the given `OrganizationUnitSetID`
     */
    AssignableOnly?: boolean;
  }

  export interface GetRolesForOrganizationUnitSetResponse {
    Roles: RoleDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update the Roles for an User. The roles provided will replace any existing roles on this user!
   * 
   * When an empty list is provided all roles will be removed.
   * 
   * The UserType defines as which UserType the User will act when logging in. For employees this wil normally be UserTypes.Employee (1)
   */
  export interface SetUserRoles {
    /**
     * Entity type: User
     */
    UserID: number;
    Roles: RoleOrganizationUnitSet[];
  }

  export interface RoleOrganizationUnitSet {
    /**
     * Entity type: Role
     */
    RoleID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * This will be mapped to the matching OrganizationUnitSet
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    EndDate?: string;
    UserType?: EVA.Core.UserTypes;
  }

  export interface UpdateUserRoles {
    /**
     * Entity type: User
     */
    UserID: number;
    Roles?: UpdateUserRoles.UpdateRole[];
  }

  export namespace UpdateUserRoles {
    export interface UpdateRole {
      /**
       * Entity type: Role
       */
      RoleID: number;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID?: number;
      IsRemoved: boolean;
      EndDate?: string;
      UserType?: EVA.Core.UserTypes;
    }

  }

  /**
   * Creates a barcode for a product for a specific unit of measure.
   * 
   * Optionally adds a record in the ProductUnitOfMeasure conversion table for the supplied Quantity.
   */
  export interface CreateProductBarcode {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Barcode: string;
    /**
     * Entity type: UnitOfMeasure
     */
    UnitOfMeasureID: number;
    Name?: string;
    Quantity?: number;
  }

  /**
   * Delete a barcode from eva
   * 
   * When it's SupplierBarcode all matching barcodes will be removed
   */
  export interface DeleteProductBarcode {
    Barcode: string;
    /**
     * @default false
     */
    IsSupplierBarcode?: boolean;
  }

  /**
   * The page size `Limit` has a maximum value of `1.000` for this service.
   */
  export interface ListProductBarcodes {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListProductBarcodesFilter>;
  }

  export interface ListProductBarcodesResponse {
    Result?: EVA.Core.PagedResult<ListProductBarcodesResponse.ProductBarcodeDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListProductBarcodesResponse {
    export interface ProductBarcodeDto {
      ProductID: number;
      ProductCustomID: string;
      ProductBackendID: string;
      CatalogID?: number;
      Origin: EVA.Core.ProductBarcodeOrigin;
      Barcode: string;
      ProductDisplayValue: string;
      Quantity: number;
      Name: string;
      UnitOfMeasure: EVA.Core.EnumDto;
      IsSupplierProduct: boolean;
    }

  }

  /**
   * Get all available custom handler for ProductRequirements
   * 
   * These handlers are custom build to fetch values for ProductRequirements from external systems
   */
  export interface GetProductRequirementHandlers {
  }

  export interface GetProductRequirementHandlersResponse {
    Handlers: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get all info for a Product Requirement
   */
  export interface GetProductRequirementByID {
    /**
     * Entity type: ProductRequirement
     */
    ID: number;
  }

  export interface GetProductRequirementByIDResponse {
    /**
     * Entity type: ProductRequirement
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Name: string;
    ProductName: string;
    DataType: EVA.Core.ProductRequirementDataTypes;
    BackendID?: string;
    Handler?: string;
    OrderType?: EVA.Core.OrderTypes;
    ProductBackendID?: string;
    ProductCustomID?: string;
    RequirementOptions?: EVA.Core.ProductRequirementOptions;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new Product Requirement
   */
  export interface CreateProductRequirement {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Name: string;
    DataType: EVA.Core.ProductRequirementDataTypes;
    BackendID?: string;
    Handler?: string;
    OrderType?: EVA.Core.OrderTypes;
    RequirementOptions?: EVA.Core.ProductRequirementOptions;
  }

  export interface CreateProductRequirementResponse {
    /**
     * Entity type: ProductRequirement
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing Product Requirement
   */
  export interface UpdateProductRequirement {
    /**
     * Entity type: ProductRequirement
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Name: string;
    DataType: EVA.Core.ProductRequirementDataTypes;
    BackendID?: string;
    Handler?: string;
    OrderType?: EVA.Core.OrderTypes;
    RequirementOptions?: EVA.Core.ProductRequirementOptions;
  }

  /**
   * Delete an existing Product Requirement
   */
  export interface DeleteProductRequirement {
    /**
     * Entity type: ProductRequirement
     */
    ID: number;
  }

  /**
   * List existing Product Requirements
   */
  export interface ListProductRequirements {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListProductRequirementsFilter>;
  }

  export interface ListProductRequirementsResponse {
    Result?: EVA.Core.PagedResult<ProductRequirementDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ProductRequirementDto {
    /**
     * Entity type: ProductRequirement
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Name: string;
    ProductName: string;
    DataType: EVA.Core.ProductRequirementDataTypes;
    BackendID?: string;
    Handler?: string;
    OrderType?: EVA.Core.OrderTypes;
    ProductBackendID?: string;
    ProductCustomID?: string;
    RequirementOptions?: EVA.Core.ProductRequirementOptions;
  }

  /**
   * List the ProductSearchTemplates
   */
  export interface ListProductSearchTemplates {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ProductSearchTemplateFilters>;
  }

  export interface ListProductSearchTemplatesResponse {
    Result?: EVA.Core.PagedResult<ListProductSearchTemplatesResponse.ProductSearchDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListProductSearchTemplatesResponse {
    export interface ProductSearchDto {
      /**
       * Entity type: ProductSearchTemplate
       */
      ID: number;
      BackendID?: string;
      Name: string;
      /**
       * Entity type: Language
       */
      LanguageID: string;
      /**
       * Entity type: Country
       */
      CountryID: string;
      Filters: { [key: string]: EVA.Core.FilterModel };
      AggregationOptions?: { [key: string]: EVA.Core.AggregationFilterModel };
      Type: EVA.Core.ProductSearchTemplateTypes;
      MinimumRefreshInterval?: string;
      IncludeLeafProducts: boolean;
    }

  }

  /**
   * Create a new ProductSearchTemplate
   */
  export interface CreateProductSearchTemplate {
    BackendID?: string;
    Name: string;
    Filters: { [key: string]: EVA.Core.FilterModel };
    AggregationOptions?: { [key: string]: EVA.Core.AggregationFilterModel };
    Type: EVA.Core.ProductSearchTemplateTypes;
    MinimumRefreshInterval?: string;
    IncludeLeafProducts?: boolean;
  }

  export interface CreateProductSearchTemplateResponse {
    /**
     * Entity type: ProductSearchTemplate
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing ProductSearchTemplate
   */
  export interface UpdateProductSearchTemplate {
    /**
     * Entity type: ProductSearchTemplate
     */
    ID: number;
    BackendID?: string;
    Name?: string;
    Filters?: { [key: string]: EVA.Core.FilterModel };
    AggregationOptions?: { [key: string]: EVA.Core.AggregationFilterModel };
    Type?: EVA.Core.ProductSearchTemplateTypes;
    MinimumRefreshInterval?: string;
    IncludeLeafProducts?: boolean;
  }

  /**
   * Delete a ProductSearchTemplate
   */
  export interface DeleteProductSearchTemplate {
    /**
     * Entity type: ProductSearchTemplate
     */
    ID: number;
  }

  /**
   * Get an exisiting ProductSearchTemplate by ID
   */
  export interface GetProductSearchTemplateByID {
    /**
     * Entity type: ProductSearchTemplate
     */
    ID: number;
  }

  export interface GetProductSearchTemplateByIDResponse {
    BackendID?: string;
    Name: string;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    Filters: { [key: string]: EVA.Core.FilterModel };
    AggregationOptions?: { [key: string]: EVA.Core.AggregationFilterModel };
    Type: EVA.Core.ProductSearchTemplateTypes;
    MinimumRefreshInterval?: string;
    IncludeLeafProducts: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns the product count within the persisted product search template
   * When product id is given, it wil checks if a product is part of the persisted product search template result.
   * NOTE THAT after product search template creation/update if can take a few minutes for the result to be updated.
   */
  export interface CheckProductSearchTemplate {
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
  }

  export interface CheckProductSearchTemplateResponse {
    /**
     * Total count of products withing the product search template result.
     */
    ResultCount: number;
    /**
     * Will return true/false is the given product is present. If no product is given it will return null
     */
    IsPresent?: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the ProductUnitOfMeasures
   */
  export interface ListProductUnitOfMeasures {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListProductUnitOfMeasuresFilter>;
  }

  export interface ListProductUnitOfMeasuresResponse {
    Result?: EVA.Core.PagedResult<ListProductUnitOfMeasuresResponse.ProductUnitOfMeasureDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListProductUnitOfMeasuresResponse {
    export interface ProductUnitOfMeasureDto {
      ID: number;
      ProductID: number;
      ProductName: string;
      UnitOfMeasureID: number;
      UnitOfMeasure: EVA.Core.EnumDto;
      Quantity: number;
    }

  }

  /**
   * Create a new ProductUnitOfMeasure
   */
  export interface CreateProductUnitOfMeasure {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: UnitOfMeasure
     */
    UnitOfMeasureID: number;
    Quantity: number;
  }

  export interface CreateProductUnitOfMeasureResponse {
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing ProductUnitOfMeasure
   */
  export interface UpdateProductUnitOfMeasure {
    /**
     * Entity type: ProductUnitOfMeasure
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: UnitOfMeasure
     */
    UnitOfMeasureID: number;
    Quantity: number;
  }

  /**
   * Delete a ProductUnitOfMeasure
   */
  export interface DeleteProductUnitOfMeasure {
    /**
     * Entity type: ProductUnitOfMeasure
     */
    ID: number;
  }

  /**
   * List the SupplierProducts
   * 
   * `SupplierProduct`: an item that describes the giftcard properties of a product with type GiftCard
   */
  export interface ListSupplierProducts {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListSupplierProductsFilter>;
  }

  export interface ListSupplierProductsResponse {
    Result?: EVA.Core.PagedResult<ListSupplierProductsResponse.SupplierProductDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListSupplierProductsResponse {
    export interface SupplierProductDto {
      ID: number;
      BackendID: string;
      BrandName: string;
      PrimitiveName: string;
      Status: EVA.Core.ProductStatus;
      Description: string;
      PreferredMinimumOrderQuantity?: number;
      TaxCode: EVA.Core.EnumDto;
      QuantityOnHand: number;
      CostPrice?: number;
      RecommendedRecommendedRetailPrice?: number;
      CurrencyID: string;
    }

  }

  /**
   * Get a SupplierProduct by ID
   */
  export interface GetSupplierProductByID {
    /**
     * Entity type: Product
     */
    ID: number;
  }

  export interface GetSupplierProductByIDResponse {
    ID: number;
    BackendID: string;
    BrandName: string;
    PrimitiveName: string;
    Status: EVA.Core.ProductStatus;
    Description: string;
    PreferredMinimumOrderQuantity?: number;
    TaxCode: EVA.Core.EnumDto;
    QuantityOnHand: number;
    CostPrice?: number;
    RecommendedRecommendedRetailPrice?: number;
    CurrencyID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new SupplierProduct
   */
  export interface CreateSupplierProduct {
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID: number;
    BackendID: string;
    BrandName?: string;
    PrimitiveName: string;
    Status: EVA.Core.ProductStatus;
    Description?: string;
    PreferredMinimumOrderQuantity?: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID: number;
  }

  export interface CreateSupplierProductResponse {
    /**
     * Entity type: Product
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing SupplierProduct
   */
  export interface UpdateSupplierProduct {
    /**
     * Entity type: Product
     */
    ID: number;
    BackendID: string;
    BrandName?: string;
    PrimitiveName: string;
    Status: EVA.Core.ProductStatus;
    Description?: string;
    PreferredMinimumOrderQuantity?: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID: number;
  }

  /**
   * Update the stock of a SupplierProduct, this is handled in the background
   */
  export interface UpdateSupplierProductStock {
    /**
     * Entity type: Product
     */
    ID: number;
    Quantity: number;
  }

  /**
   * Update the prices of a SupplierProduct, this is handled in the background
   */
  export interface UpdateSupplierProductPrices {
    /**
     * Entity type: Product
     */
    ID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    UnitCost?: number;
    RecommendedRetailPrice?: number;
  }

  /**
   * Delete a SupplierProduct
   */
  export interface DeleteSupplierProduct {
    /**
     * Entity type: Product
     */
    ID: number;
  }

  /**
   * Retrieves the suppliers (Organization Units) for the given Product.
   */
  export interface GetSuppliersForProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  export interface GetSupplierForProductResponse {
    Result: GetSupplierForProductResponse.SupplierDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSupplierForProductResponse {
    export interface SupplierDto {
      ID: number;
      Name: string;
      UnitCost: number;
      Stock: number;
    }

  }

  /**
   * Upload new products for a Supplier.
   * Fetch a sample with the `DownloadSupplierProductsSample`.
   * Fetch the current products with the `DownloadSupplierProducts`.
   */
  export interface UploadSupplierProducts {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Data: string;
  }

  export interface UploadSupplierProductsResponse {
    TotalProductCount: number;
    FailedCount: number;
    Messages: { [key: string]: string };
    DeletedCount: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Upload new stock for the products of a Supplier.
   * Fetch a sample with the `DownloadSupplierProductsStockSample`.
   * Fetch the current stock with the `DownloadSupplierProductsStock`.
   */
  export interface UploadSupplierProductsStock {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Data: string;
  }

  /**
   * Upload new prices for the products of a Supplier.
   * Fetch a sample with the `DownloadSupplierProductsPricingSample`.
   * Fetch the current prices with the `DownloadSupplierProductsPricing`.
   */
  export interface UploadSupplierProductsPricing {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Data: string;
  }

  /**
   * Upload new barcodes for the products of a Supplier.
   * Fetch a sample with the `DownloadSupplierProductsBarcodesSample`.
   * Fetch the current prices with the `DownloadSupplierProductsBarcodes`.
   */
  export interface UploadSupplierProductsBarcodes {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Data: string;
  }

  /**
   * Download a sample for the `UploadSupplierProducts`.
   */
  export interface DownloadSupplierProductsSample {
  }

  /**
   * Download a sample for the `UploadSupplierProductsStock`.
   */
  export interface DownloadSupplierProductsStockSample {
  }

  /**
   * Download a sample for the `UploadSupplierProductsPricing`.
   */
  export interface DownloadSupplierProductsPricingSample {
  }

  /**
   * Download a sample for the `UploadSupplierProductsBarcodes`.
   */
  export interface DownloadSupplierProductsBarcodesSample {
  }

  /**
   * Download the current supplier products.
   */
  export interface DownloadSupplierProducts {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  /**
   * Download the current stock for the supplier products.
   */
  export interface DownloadSupplierProductsStock {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  /**
   * Download the current prices for the supplier products in the given Currency.
   */
  export interface DownloadSupplierProductsPricing {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  /**
   * Download the current barcodes for the supplier products.
   */
  export interface DownloadSupplierProductsBarcodes {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  /**
   * List the UnitOfMeasures
   */
  export interface GetUnitOfMeasures {
  }

  export interface GetUnitOfMeasuresResponse {
    UnitOfMeasures: GetUnitOfMeasuresResponse.UnitOfMeasureDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetUnitOfMeasuresResponse {
    export interface UnitOfMeasureDto {
      BackendID: string;
      /**
       * Entity type: Enum
       */
      ID: number;
      /**
       * Usually an internal & unchangable code - for display, use `Description`
       */
      Name: string;
      Description: string;
    }

  }

  /**
   * Create a new UnitOfMeasure
   */
  export interface CreateUnitOfMeasure {
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  export interface CreateUnitOfMeasureResponse {
    /**
     * Entity type: UnitOfMeasure
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing UnitOfMeasure
   */
  export interface UpdateUnitOfMeasure {
    /**
     * Entity type: UnitOfMeasure
     */
    ID: number;
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  /**
   * Delete a UnitOfMeasure
   */
  export interface DeleteUnitOfMeasure {
    /**
     * Entity type: UnitOfMeasure
     */
    ID: number;
  }

  /**
   * Creates a printer type
   */
  export interface CreatePrinterType {
    /**
     * Name of the type
     */
    Name: string;
    /**
     * Description of the type
     */
    Description?: string;
    PaperWidth: number;
  }

  export interface CreatePrinterTypeResponse {
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Deletes a printer type
   */
  export interface DeletePrinterType {
    /**
     * Entity type: PrinterType
     */
    ID: number;
  }

  /**
   * Gets a printer type
   */
  export interface GetPrinterTypeByID {
    /**
     * Entity type: PrinterType
     */
    ID: number;
  }

  export interface GetPrinterTypeByIDResponse {
    ID: number;
    Name: string;
    Description?: string;
    PaperWidth: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Gets all printer types
   */
  export interface GetPrinterTypes {
  }

  export interface GetPrinterTypesResponse {
    PrinterTypes: GetPrinterTypesResponse.PrinterTypeModel[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPrinterTypesResponse {
    export interface PrinterTypeModel {
      ID: number;
      Name: string;
      Description?: string;
      PaperWidth: number;
    }

  }

  /**
   * Updates a printer type
   */
  export interface UpdatePrinterType {
    /**
     * Entity type: PrinterType
     */
    ID: number;
    Name: string;
    Description?: string;
    PaperWidth: number;
  }

  /**
   * Retrieve a list of OrganizationUnit PriceLists
   * 
   * It's required to set the OrganizationUnitID filter
   */
  export interface ListPriceListsForOrganizationUnit {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListPriceListsForOrganizationUnitFilter>;
  }

  export interface ListPriceListsForOrganizationUnitResponse {
    Result?: EVA.Core.PagedResult<ListPriceListsForOrganizationUnitResponse.PriceListDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPriceListsForOrganizationUnitResponse {
    export interface PriceListDto {
      /**
       * Entity type: PriceList
       */
      PriceListID: number;
      Name: string;
      BackendID: string;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      IsActive: boolean;
      IsFinalSalePriceList: boolean;
      IncludingVat: boolean;
      /**
       * Entity type: PriceListUsageType
       */
      PriceListUsageTypeID: number;
      PriceListUsageTypeName: string;
      PricingGroupID?: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      OrganizationUnitBackendID: string;
      /**
       * Entity type: OrganizationUnit
       */
      InheritedFromOrganizationUnitID?: number;
      InheritedFromOrganizationUnitName?: string;
      InheritedFromOrganizationUnitBackendID?: string;
      /**
       * Entity type: OrganizationUnit
       */
      TargetOrganizationUnitID?: number;
      TargetOrganizationUnitName?: string;
      TargetOrganizationUnitBackendID?: string;
    }

  }

  /**
   * List the pricechanges
   * 
   * It is required to supply some OrganizationUnitIDs or some ProductIDs in the PageConfig filter.
   */
  export interface ListProductPriceLedger {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListProductPriceLedgerFilter>;
  }

  export interface ListProductPriceLedgerResponse {
    Result?: EVA.Core.PagedResult<ListProductPriceLedgerResponse.OrganizationUnitPriceChange>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListProductPriceLedgerResponse {
    export interface OrganizationUnitPriceChange {
      ProductID: number;
      PriceListID: number;
      PriceListUsageTypeID: number;
      PriceListUsageTypeName: string;
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      OldPrice?: number;
      NewPrice?: number;
      CreationTime: string;
      CurrencyID: string;
    }

  }

  /**
   * Given the data on how rounding should be applied this service will perform the rounding on the given example prices to preview what the rounding would look like.
   */
  export interface PreviewPriceListRoundingAdjustment {
    ExamplePrices?: number[];
    Data?: EVA.Core.RoundingData;
  }

  export interface PreviewPriceListRoundingAdjustmentResponse {
    Results: PreviewPriceListRoundingAdjustmentResponse.RoundingResult[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace PreviewPriceListRoundingAdjustmentResponse {
    export interface RoundingResult {
      InputPrice?: number;
      OutputPrice?: number;
    }

  }

  /**
   * Get a PriceListAdjustment
   */
  export interface GetPriceListAdjustmentByID {
    /**
     * Entity type: PriceListAdjustment
     */
    ID: number;
  }

  export interface GetPriceListAdjustmentByIDResponse {
    /**
     * Entity type: PriceListAdjustment
     */
    ID: number;
    BackendID?: string;
    /**
     * Entity type: PriceList
     */
    PriceListID: number;
    PriceList: GetPriceListAdjustmentByIDResponse.PriceListDto;
    /**
     * Entity type: PriceListAdjustment
     */
    ParentAdjustmentID?: number;
    ParentAdjustmentName?: string;
    Sequence: number;
    Name: string;
    EffectiveDate: string;
    ExpireDate?: string;
    OverridePrice: boolean;
    Type: string;
    Data?: { [key: string]: any };
    /**
     * Entity type: PriceListAdjustmentLabel
     */
    LabelID?: number;
    Label?: string;
    PricingGroupID?: string;
    SystemType: EVA.Core.PriceListAdjustmentSystemType;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPriceListAdjustmentByIDResponse {
    export interface PriceListDto {
      Name: string;
      CurrencyID: string;
      TimeZone: string;
    }

  }

  /**
   * Create a new PriceListAdjustment
   */
  export interface CreatePriceListAdjustment {
    /**
     * Entity type: PriceList
     */
    PriceListID: number;
    BackendID?: string;
    /**
     * Entity type: PriceListAdjustment
     */
    ParentAdjustmentID?: number;
    Sequence: number;
    Name: string;
    Label?: string;
    EffectiveDate: string;
    ExpireDate?: string;
    OverridePrice?: boolean;
    Type: string;
    Data?: { [key: string]: any };
    PricingGroupID?: string;
    /**
     * If set to true, this will skip the recalculation of the price list after the adjustment is updated, defaults to false so it will always recalculate. Recalculation happens periodically thereafter.
     */
    SkipRecalculation?: boolean;
  }

  export interface CreatePriceListAdjustmentResponse {
    /**
     * Entity type: PriceListAdjustment
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing PriceListAdjustment
   */
  export interface UpdatePriceListAdjustment {
    /**
     * Entity type: PriceListAdjustment
     */
    ID: number;
    /**
     * Entity type: PriceListAdjustment
     */
    ParentAdjustmentID?: number;
    BackendID?: string;
    Sequence: number;
    Name: string;
    Label?: string;
    EffectiveDate: string;
    ExpireDate?: string;
    OverridePrice?: boolean;
    Type: string;
    Data?: { [key: string]: any };
    PricingGroupID?: string;
    /**
     * If set to true, this will skip the recalculation of the price list after the adjustment is updated, defaults to false so it will always recalculate. Recalculation happens periodically thereafter.
     */
    SkipRecalculation?: boolean;
  }

  /**
   * Delete an existing PriceListAdjustment
   */
  export interface DeletePriceListAdjustment {
    /**
     * Entity type: PriceListAdjustment
     */
    ID: number;
    /**
     * If set to true, this will skip the recalculation of the price list after the adjustment is updated, defaults to false so it will always recalculate. Recalculation happens periodically thereafter.
     */
    SkipRecalculation?: boolean;
  }

  /**
   * List the PriceListAdjustments
   */
  export interface ListPriceListAdjustments {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListPriceListAdjustmentsFilter>;
  }

  export interface ListPriceListAdjustmentsResponse {
    Result?: EVA.Core.PagedResult<ListPriceListAdjustmentsResponse.PriceListAdjustmentDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPriceListAdjustmentsResponse {
    export interface PriceListAdjustmentDto {
      /**
       * Entity type: PriceListAdjustment
       */
      ID: number;
      BackendID?: string;
      /**
       * Entity type: PriceList
       */
      PriceListID: number;
      PriceList: PriceListAdjustmentDto.PriceListDto;
      /**
       * Entity type: PriceListAdjustment
       */
      ParentAdjustmentID?: number;
      ParentAdjustmentName?: string;
      Sequence: number;
      Name: string;
      Label?: string;
      EffectiveDate: string;
      ExpireDate?: string;
      OverridePrice: boolean;
      Type: string;
      Data?: { [key: string]: any };
      PricingGroupID?: string;
      SystemType: EVA.Core.PriceListAdjustmentSystemType;
    }

    export namespace PriceListAdjustmentDto {
      export interface PriceListDto {
        Name: string;
        CurrencyID: string;
        TimeZone: string;
      }

    }

  }

  /**
   * Get a PriceList
   * 
   * A PriceList has a TimeZone, this TimeZone should be used to Display certain DateTime's on the frontend.
   * For these specific fields the TimeZone of the user should be ignored and instead the one on the PriceList should be used.
   */
  export interface GetPriceListByID {
    /**
     * Entity type: PriceList
     */
    ID: number;
  }

  export interface GetPriceListByIDResponse {
    /**
     * Entity type: PriceList
     */
    ID: number;
    Name: string;
    BackendID?: string;
    BackendSystemID?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    TimeZone: string;
    IncludingVat: boolean;
    IsActive: boolean;
    IsFinalSalePriceList: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface PriceListDto {
    /**
     * Entity type: PriceList
     */
    ID: number;
    Name: string;
    BackendID?: string;
    BackendSystemID?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    TimeZone: string;
    IncludingVat: boolean;
    IsActive: boolean;
    IsFinalSalePriceList: boolean;
    /**
     * Indicates if the user can view the pricelist details. This is the case when:
     * - User has 'CanViewCostPrices' AND 'CanViewPurchasePrices' functionality
     * OR
     * - User has access to an organization unit
     */
    CanViewPriceList: boolean;
  }

  /**
   * Create a new PriceList
   */
  export interface CreatePriceList {
    Name: string;
    BackendID?: string;
    BackendSystemID?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    TimeZone?: string;
    IncludingVat?: boolean;
    IsActive?: boolean;
    IsFinalSalePriceList?: boolean;
  }

  export interface CreatePriceListResponse {
    /**
     * Entity type: PriceList
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing PriceList
   */
  export interface UpdatePriceList {
    /**
     * Entity type: PriceList
     */
    ID: number;
    Name?: string;
    BackendID?: string;
    BackendSystemID?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    TimeZone?: string;
    IncludingVat?: boolean;
    IsActive?: boolean;
    IsFinalSalePriceList?: boolean;
  }

  /**
   * Delete a PriceList
   */
  export interface DeletePriceList {
    /**
     * Entity type: PriceList
     */
    ID: number;
  }

  /**
   * List PriceLists
   */
  export interface ListPriceLists {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListPriceListsFilter>;
  }

  export interface ListPriceListsResponse {
    Result?: EVA.Core.PagedResult<PriceListDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get a PriceListManualInputAdjustment by ID
   */
  export interface GetPriceListManualInputAdjustmentByID {
    /**
     * Entity type: PriceListManualInputAdjustment
     */
    ID: number;
  }

  export interface GetPriceListManualInputAdjustmentByIDResponse {
    /**
     * Entity type: PriceListManualInputAdjustment
     */
    ID: number;
    /**
     * Entity type: PriceListAdjustment
     */
    PriceListAdjustmentID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductName: string;
    ProductCustomID: string;
    ProductBackendID: string;
    Value: number;
    EffectiveDate: string;
    ExpireDate?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new PriceListManualInputAdjustment
   */
  export interface CreatePriceListManualInputAdjustment {
    /**
     * Entity type: PriceListAdjustment
     */
    PriceListAdjustmentID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Value: number;
    EffectiveDate: string;
    ExpireDate?: string;
  }

  export interface CreatePriceListManualInputAdjustmentResponse {
    /**
     * Entity type: PriceListManualInputAdjustment
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing new PriceListManualInputAdjustment
   */
  export interface UpdatePriceListManualInputAdjustment {
    /**
     * Entity type: PriceListManualInputAdjustment
     */
    ID: number;
    Value: number;
    EffectiveDate: string;
    ExpireDate?: string;
  }

  /**
   * Delete a PriceListManualInputAdjustment
   */
  export interface DeletePriceListManualInputAdjustment {
    /**
     * Entity type: PriceListManualInputAdjustment
     */
    ID: number;
  }

  /**
   * List the PriceListManualInputAdjustments, it is required to supply at least an PriceListAdjustmentID in the Filter
   */
  export interface ListPriceListManualInputAdjustments {
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListManualInputAdjustmentsFilter>;
  }

  export interface ListPriceListManualInputAdjustmentsResponse {
    Result?: EVA.Core.PagedResult<ListPriceListManualInputAdjustmentsResponse.PriceListManualInputAdjustmentDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPriceListManualInputAdjustmentsResponse {
    export interface PriceListManualInputAdjustmentDto {
      /**
       * Entity type: PriceListManualInputAdjustment
       */
      ID: number;
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductName: string;
      ProductCustomID: string;
      ProductBackendID: string;
      Product?: any;
      Value: number;
      EffectiveDate: string;
      ExpireDate?: string;
    }

  }

  /**
   * Generate an excel file containing all ManualInputPridceListAdjustments for the given PriceListAdjustment.
   */
  export interface DownloadPriceListManualInputAdjustments {
    /**
     * Entity type: PriceListAdjustment
     */
    PriceListAdjustmentID: number;
  }

  /**
   * Generate an excel sample file containing all ManualInputPridceListAdjustments for the given PriceListAdjustment.
   */
  export interface DownloadPriceListManualInputAdjustmentsSample {
  }

  /**
   * Upload an excel file containing all ManualInputPriceListAdjustments for the given PriceListAdjustment. These will replace all exisiting adjustments.
   */
  export interface UploadPriceListManualInputAdjustments {
    /**
     * Entity type: PriceListAdjustment
     */
    PriceListAdjustmentID: number;
    Data: string;
    ProcessAsync?: boolean;
  }

  export interface UploadPriceListManualInputAdjustmentsResponse {
    IsProcessedAsync: boolean;
    Messages: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get an existing PriceListOrganizationUnit
   */
  export interface GetPriceListOrganizationUnitByID {
    /**
     * Entity type: PriceListOrganizationUnit
     */
    ID: number;
  }

  export interface GetPriceListOrganizationUnitByIDResponse {
    ID: number;
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    /**
     * Entity type: OrganizationUnit
     */
    TargetOrganizationUnitID?: number;
    TargetOrganizationUnitName?: string;
    PriceListID: number;
    PriceList: GetPriceListOrganizationUnitByIDResponse.PriceListDto;
    PriceListUsageTypeID: number;
    PriceListUsageTypeName: string;
    PricingGroupID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPriceListOrganizationUnitByIDResponse {
    export interface PriceListDto {
      Name: string;
      CurrencyID: string;
    }

  }

  /**
   * Create a new PriceListOrganizationUnit
   */
  export interface CreatePriceListOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    TargetOrganizationUnitID?: number;
    /**
     * Entity type: PriceList
     */
    PriceListID?: number;
    /**
     * Entity type: PriceListUsageType
     */
    PriceListUsageTypeID?: number;
    PricingGroupID?: string;
  }

  export interface CreatePriceListOrganizationUnitResponse {
    /**
     * Entity type: PriceListOrganizationUnit
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing PriceListOrganizationUnit
   */
  export interface UpdatePriceListOrganizationUnit {
    /**
     * Entity type: PriceListOrganizationUnit
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    TargetOrganizationUnitID?: number;
    /**
     * Entity type: PriceList
     */
    PriceListID: number;
    /**
     * Entity type: PriceListUsageType
     */
    PriceListUsageTypeID: number;
    PricingGroupID?: string;
  }

  /**
   * Delete a PriceListOrganizationUnit
   */
  export interface DeletePriceListOrganizationUnit {
    /**
     * Entity type: PriceListOrganizationUnit
     */
    ID: number;
  }

  /**
   * List the PriceListOrganizationUnits
   */
  export interface ListPriceListOrganizationUnits {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListPriceListOrganizationUnitsFilter>;
  }

  export interface ListPriceListOrganizationUnitsResponse {
    Result?: EVA.Core.PagedResult<ListPriceListOrganizationUnitsResponse.PriceListOrganizationUnitDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPriceListOrganizationUnitsResponse {
    export interface PriceListOrganizationUnitDto {
      /**
       * Entity type: PriceListOrganizationUnit
       */
      ID: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      /**
       * Entity type: OrganizationUnit
       */
      TargetOrganizationUnitID?: number;
      TargetOrganizationUnitName?: string;
      /**
       * Entity type: PriceList
       */
      PriceListID: number;
      PriceList: PriceListOrganizationUnitDto.PriceListDto;
      /**
       * Entity type: PriceListUsageType
       */
      PriceListUsageTypeID: number;
      PriceListUsageTypeName: string;
      PricingGroupID: string;
    }

    export namespace PriceListOrganizationUnitDto {
      export interface PriceListDto {
        Name: string;
        CurrencyID: string;
      }

    }

  }

  export interface GetPriceListUsageTypeByID {
    /**
     * Entity type: PriceListUsageType
     */
    ID: number;
  }

  export interface GetPriceListUsageTypeByIDResponse {
    ID: number;
    Name: string;
    Description: string;
    RequiredUserType: EVA.Core.UserTypes;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface CreatePriceListUsageType {
    Name: string;
    Description?: string;
    RequiredUserType: EVA.Core.UserTypes;
  }

  export interface CreatePriceListUsageTypeResponse {
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface UpdatePriceListUsageType {
    /**
     * Entity type: PriceListUsageType
     */
    ID: number;
    Description?: string;
    RequiredUserType: EVA.Core.UserTypes;
  }

  export interface DeletePriceListUsageType {
    /**
     * Entity type: PriceListUsageType
     */
    ID: number;
  }

  export interface GetPriceListUsageTypes {
  }

  export interface GetPriceListUsageTypesResponse {
    PriceListUsageTypes: GetPriceListUsageTypesResponse.PriceListUsageTypeDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPriceListUsageTypesResponse {
    export interface PriceListUsageTypeDto {
      ID: number;
      Name: string;
      Description: string;
      RequiredUserType: EVA.Core.UserTypes;
    }

  }

  /**
   * List personalized promotions. Returns a paged response.
   */
  export interface ListPersonalizedPromotions {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListPersonalizedPromotionsFilter>;
    PageToken?: string;
  }

  export interface ListPersonalizedPromotionsResponse {
    Results?: ListPersonalizedPromotionsResponse.PersonalizedPromotion[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPersonalizedPromotionsResponse {
    export interface PersonalizedPromotion {
      /**
       * Entity type: PersonalizedPromotion
       */
      ID: number;
      Name: string;
      Description: string;
      BackendID: string;
      LockAfterFirstApply: boolean;
      MaximumDistinctProducts?: number;
    }

  }

  /**
   * Get personalized promotion by its ID.
   */
  export interface GetPersonalizedPromotionByID {
    /**
     * Entity type: PersonalizedPromotion
     */
    ID: number;
  }

  export interface GetPersonalizedPromotionByIDResponse {
    /**
     * Entity type: PersonalizedPromotion
     */
    ID: number;
    Name: string;
    Description: string;
    BackendID: string;
    LockAfterFirstApply: boolean;
    MaximumDistinctProducts?: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new personalized promotion.
   */
  export interface CreatePersonalizedPromotion {
    Name: string;
    Description?: string;
    BackendID?: string;
    LockAfterFirstApply?: boolean;
    MaximumDistinctProducts?: number;
  }

  export interface CreatePersonalizedPromotionResponse {
    /**
     * Entity type: PersonalizedPromotion
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing personalized promotion.
   */
  export interface UpdatePersonalizedPromotion {
    /**
     * Entity type: PersonalizedPromotion
     */
    ID: number;
    Name?: string;
    Description?: string;
    BackendID?: string;
    LockAfterFirstApply?: boolean;
    MaximumDistinctProducts?: number;
  }

  /**
   * Archive a personalized promotion.
   */
  export interface ArchivePersonalizedPromotion {
    /**
     * Entity type: PersonalizedPromotion
     */
    ID: number;
  }

  /**
   * Add personalized promotion products for a user.
   */
  export interface AddPersonalizedPromotionsProducts {
    /**
     * Entity type: PersonalizedPromotion
     */
    PersonalizedPromotionID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
  }

  /**
   * Remove personalized promotion products attached to a user. Either ProductIDs or RemoveAll is required.
   */
  export interface RemovePersonalizedPromotionProducts {
    /**
     * Entity type: PersonalizedPromotion
     */
    PersonalizedPromotionID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    RemoveAll?: boolean;
  }

  /**
   * Set products for personalized promotion for the given user. Overwrites existing products and product selection. Will unset products and selection when no products are provided.
   */
  export interface PushUserPersonalizedPromotionProducts {
    UserID: string;
    PersonalizedPromotionID: string;
    /**
     * The products that the user has in this PersonalizedPromotion. Can have a maximum of 100 products.
     */
    Products?: PushUserPersonalizedPromotionProducts.PushPersonalizedPromotionProduct[];
    /**
     * When set to true, products that couldn't be resolved will be skipped. When false request will fail when a product couldn't be resolved.
     */
    SkipUnresolvedProducts?: boolean;
  }

  export namespace PushUserPersonalizedPromotionProducts {
    export interface PushPersonalizedPromotionProduct {
      ProductID: string;
      Selected: boolean;
    }

  }

  /**
   * Delete a failed or unprocessed settlement file
   * 
   * File in state processing or processed can't be deleted due to generated financial events. This is only allowed when no settlents were generated for the file.
   */
  export interface DeleteUnprocessedPaymentSettlementFile {
    /**
     * Entity type: PaymentTransactionSettlementFile
     */
    PaymentSettlementFileID: number;
  }

  /**
   * Download a settlement result file
   */
  export interface DownloadPaymentSettlementFileResult {
    /**
     * Entity type: PaymentTransactionSettlementFile
     */
    PaymentSettlementFileID: number;
  }

  /**
   * Returns the available handlers for settlement files
   */
  export interface GetAvailablePaymentSettlementFileHandlers {
  }

  export interface GetAvailablePaymentSettlementFileHandlersResponse {
    Handlers: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List uploaded settlement files
   * 
   * Filters available:
   * - Name
   * - Status
   * - Handler
   */
  export interface ListPaymentSettlementFiles {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListSettlementFilesFilter>;
  }

  export interface ListPaymentSettlementFilesResponse {
    Result?: EVA.Core.PagedResult<ListPaymentSettlementFilesResponse.PaymentSettlementFile>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPaymentSettlementFilesResponse {
    export interface PaymentSettlementFile {
      /**
       * Entity type: PaymentTransactionSettlementFile
       */
      ID: number;
      Name: string;
      OriginalBlobID: string;
      OriginalDownloadUrl: string;
      Status: EVA.Core.PaymentTransactionSettlementFileStatus;
      /**
       * Entity type: OrganizationUnit
       */
      FallbackOrganizationUnitID: number;
      FallbackOrganizationUnitName: string;
      Handler: string;
    }

  }

  /**
   * List created settlements
   * 
   * Filters available:
   * - PaymentTransactionID
   * - TypeID
   * - FinancialPeriodID
   * - OrganizationUnitID
   */
  export interface ListPaymentSettlements {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListSettlementsFilter>;
  }

  export interface ListPaymentSettlementsResponse {
    Result?: EVA.Core.PagedResult<ListPaymentSettlementsResponse.PaymentSettlement>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPaymentSettlementsResponse {
    export interface PaymentSettlement {
      /**
       * Entity type: PaymentTransactionSettlement
       */
      ID: number;
      /**
       * Entity type: PaymentTransaction
       */
      PaymentTransactionID?: number;
      /**
       * Entity type: FinancialPeriod
       */
      FinancialPeriodID: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      /**
       * Entity type: PaymentTransactionSettlementType
       */
      TypeID: number;
      TypeName: string;
      Amount: number;
      CurrencyID: string;
      LedgerClassID?: string;
      Offset1?: string;
      Offset2?: string;
      Offset3?: string;
      Offset4?: string;
    }

  }

  /**
   * Upload a new settlement file.
   * 
   * - The settlements in the file will be imported and booked into financial events to allow processing in cookbook
   * - It is required to supply the `Handler` to use (available handlers can be fetched with the `GetAvailablePaymentSettlementFileHandlers` service)
   * - For settlement records that don't have a matching `PaymentTransaction` in EVA there will be financial events generated (which will happen for supplied `OrganizationUnitID`)
   */
  export interface UploadPaymentSettlementFile {
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
    Name?: string;
    Handler?: string;
    /**
     * The OrganizationUnit that will be used to book the Settlements for payments that are not known in EVA.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  /**
   * Return all PaymentMethods
   */
  export interface GetPaymentMethods {
  }

  export interface GetPaymentMethodsResponse {
    PaymentMethods: GetPaymentMethodsResponse.PaymentMethodDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPaymentMethodsResponse {
    export interface PaymentMethodDto {
      /**
       * Entity type: PaymentMethod
       */
      ID: number;
      Name: string;
      Code: string;
    }

  }

  /**
   * List paymenttransactionCaptures
   * 
   * All available filters are typed in the PageConfig.Filter property.
   */
  export interface ListPaymentTransactionCaptures {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListPaymentTransactionCapturesFilter>;
  }

  export interface ListPaymentTransactionCapturesResponse {
    Result?: EVA.Core.PagedResult<ListPaymentTransactionCapturesResponse.PaymentTransactionCaptureDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPaymentTransactionCapturesResponse {
    export interface PaymentTransactionCaptureDto {
      /**
       * Entity type: PaymentTransactionCapture
       */
      ID: number;
      /**
       * Entity type: PaymentTransaction
       */
      PaymentTransactionID: number;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      BackendID?: string;
      Status: EVA.Core.PaymentTransactionCaptureStatuses;
      Amount: number;
      ConfirmationDate?: string;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      SettledAmount?: number;
      IsSettled?: boolean;
      CreationTime: string;
    }

  }

  /**
   * List paymenttransactions.
   */
  export interface ListPaymentTransactions {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListPaymentTransactionsFilter>;
  }

  export interface ListPaymentTransactionsResponse {
    Result?: EVA.Core.PagedResult<ListPaymentTransactionsResponse.PaymentTransactionDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPaymentTransactionsResponse {
    export interface PaymentTransactionDto {
      /**
       * Entity type: PaymentTransaction
       */
      ID: number;
      /**
       * Entity type: PaymentType
       */
      TypeID: number;
      /**
       * Entity type: PaymentMethod
       */
      PaymentMethodID?: number;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      /**
       * Entity type: Device
       */
      DeviceID?: number;
      /**
       * Entity type: FinancialPeriod
       */
      FinancialPeriodID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      TypeName: string;
      PaymentMethodName: string;
      OrderDisplayID?: string;
      BackendID: string;
      BackendSystemID: string;
      Status: EVA.Core.PaymentStatuses;
      Amount: number;
      PaidAmount: number;
      Change: number;
      PaymentDate?: string;
      OrganizationUnitName: string;
      RefundedAmount?: number;
      BackendRelationID?: string;
      Description?: string;
      SettledAmount?: number;
      CreationTime: string;
      LastModificationTime?: string;
      IsSettled?: boolean;
    }

  }

  /**
   * Create or update OrganizationUnitSetPaymentType
   * 
   * A PaymentType, OrganizationUnitSet and RefundPriority are required
   * 
   * When this OrganizationUnitSet doesn't have this PaymentType yet, it will be created. If it already exists the fields will be updated.
   */
  export interface CreateOrUpdateOrganizationUnitSetPaymentType {
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    RefundPriority?: number;
    CaptureMoment?: EVA.Core.PaymentTypeCaptureMoment;
    AuthorizationExpirationInDays?: number;
    /**
     * Entity type: Product
     */
    DigitalGiftCardProductID?: number;
    DisableForCarryOutOrders?: boolean;
    DisableForReserveOrders?: boolean;
    DisableForDeliveryOrders?: boolean;
  }

  /**
   * Delete an OrganizationUnitSetPaymentType
   * 
   * A PaymentType and OrganizationUnitSet are required
   */
  export interface DeleteOrganizationUnitSetPaymentType {
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
  }

  /**
   * List the OrganizationUnitSetPaymentTypes
   * 
   * There are filters available on:
   *  - PaymentTypeID
   *  - OrganizationUnitSetID
   *  - RefundPriority
   *  - CaptureMoment
   *  - DisableForDeliveryOrders
   *  - DisableForReserveOrders
   *  - DisableForCarryOutOrders
   */
  export interface ListOrganizationUnitSetPaymentTypes {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListOrganizationUnitSetPaymentTypesFilter>;
  }

  export interface ListOrganizationUnitSetPaymentTypesResponse {
    Result?: EVA.Core.PagedResult<OrganizationUnitSetPaymentTypeDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface OrganizationUnitSetPaymentTypeDto {
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
    PaymentTypeName: string;
    /**
     * Entity type: PaymentMethod
     */
    PaymentMethodID: number;
    PaymentMethodName: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    OrganizationUnitSetName: string;
    RefundPriority?: number;
    CaptureMoment?: EVA.Core.PaymentTypeCaptureMoment;
    AuthorizationExpirationInDays?: number;
    /**
     * Entity type: Product
     */
    DigitalGiftCardProductID?: number;
    DisableForCarryOutOrders?: boolean;
    DisableForReserveOrders?: boolean;
    DisableForDeliveryOrders?: boolean;
  }

  /**
   * Gets an PaymentMethod by its ID and returns the basic information.
   */
  export interface GetPaymentMethodByID {
    /**
     * Entity type: PaymentMethod
     */
    ID: number;
  }

  export interface GetPaymentMethodByIDResponse {
    /**
     * Entity type: PaymentMethod
     */
    ID: number;
    Name: string;
    Code: string;
    IsCustom: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new custom PaymentMethod.
   */
  export interface CreatePaymentMethod {
    Name: string;
    Code: string;
  }

  export interface CreatePaymentMethodResponse {
    /**
     * Entity type: PaymentMethod
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing custom PaymentMethod.
   */
  export interface UpdatePaymentMethod {
    /**
     * Entity type: PaymentMethod
     */
    ID: number;
    Name: string;
    Code: string;
  }

  /**
   * Delete a PaymentMethod
   */
  export interface DeletePaymentMethod {
    /**
     * Entity type: PaymentMethod
     */
    ID: number;
  }

  /**
   * List PaymentMethods. Returns a paged response.
   */
  export interface ListPaymentMethods {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListPaymentMethodsFilter>;
  }

  export interface ListPaymentMethodsResponse {
    Result?: EVA.Core.PagedResult<ListPaymentMethodsResponse.PaymentMethodDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPaymentMethodsResponse {
    export interface PaymentMethodDto {
      /**
       * Entity type: PaymentMethod
       */
      ID: number;
      Name: string;
      Code: string;
      IsCustom: boolean;
    }

  }

  /**
   * Create a new PaymentTransactionLedgerType
   */
  export interface CreatePaymentTransactionLedgerType {
    Name: string;
    Description?: string;
  }

  export interface CreatePaymentTransactionLedgerTypeResponse {
    /**
     * Entity type: PaymentTransactionLedgerType
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update the description of a PaymentTransactionLedgerType. Only custom made types can be updated.
   */
  export interface UpdatePaymentTransactionLedgerType {
    /**
     * Entity type: PaymentTransactionLedgerType
     */
    ID: number;
    Description?: string;
  }

  /**
   * Delete a PaymentTransactionLedgerType. Only custom made types can be deleted that are not in use.
   */
  export interface DeletePaymentTransactionLedgerType {
    /**
     * Entity type: PaymentTransactionLedgerType
     */
    ID: number;
  }

  /**
   * Gets an PaymentType by its ID and returns the basic information.
   */
  export interface GetPaymentTypeByID {
    /**
     * Entity type: PaymentType
     */
    ID: number;
  }

  export interface GetPaymentTypeByIDResponse {
    /**
     * Entity type: PaymentType
     */
    ID: number;
    /**
     * Entity type: PaymentMethod
     */
    PaymentMethodID: number;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID: string;
    PaymentMethodName: string;
    PaymentMethodCode: string;
    PaymentMethodIsCustom: boolean;
    OrganizationUnitSetName: string;
    OrganizationUnitSetDescription: string;
    OrganizationUnitSetType?: EVA.Core.OrganizationUnitSetTypes;
    Name: string;
    Code: string;
    IsActive: boolean;
    IsRoundingType: boolean;
    ReturnAction: EVA.Core.PaymentReturnActions;
    PrintOnDocuments: boolean;
    BackendRelationID: string;
    BackendID?: string;
    AutoFinalizeOnOrderPaid: boolean;
    CashJournalMethod: EVA.Core.PaymentCashJournalMethod;
    Category: EVA.Core.PaymentTypeCategory;
    Options?: EVA.Core.PaymentTypeOptions;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    ScriptName?: string;
    Roles: GetPaymentTypeByIDResponse.PaymentTypeRole[];
    AllowMultiCurrency: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPaymentTypeByIDResponse {
    export interface PaymentTypeRole {
      RoleID: number;
      Name: string;
    }

  }

  /**
   * Create a new PaymentType.
   */
  export interface CreatePaymentType {
    /**
     * Entity type: PaymentMethod
     */
    PaymentMethodID: number;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID?: string;
    Name: string;
    Code: string;
    IsActive?: boolean;
    ReturnAction: EVA.Core.PaymentReturnActions;
    PrintOnDocuments?: boolean;
    BackendRelationID?: string;
    BackendID?: string;
    AutoFinalizeOnOrderPaid?: boolean;
    CashJournalMethod: EVA.Core.PaymentCashJournalMethod;
    Category?: EVA.Core.PaymentTypeCategory;
    Options?: EVA.Core.PaymentTypeOptions;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Roles?: CreatePaymentType.PaymentTypeRole[];
    /**
     * @default false
     */
    AllowMultiCurrency?: boolean;
  }

  export namespace CreatePaymentType {
    export interface PaymentTypeRole {
      /**
       * Entity type: Role
       */
      RoleID: number;
    }

  }

  export interface CreatePaymentTypeResponse {
    /**
     * Entity type: PaymentType
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing PaymentType.
   */
  export interface UpdatePaymentType {
    /**
     * Entity type: PaymentType
     */
    ID: number;
    /**
     * Entity type: PaymentMethod
     */
    PaymentMethodID?: number;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID?: string;
    Name?: string;
    Code?: string;
    IsActive?: boolean;
    ReturnAction?: EVA.Core.PaymentReturnActions;
    PrintOnDocuments?: boolean;
    BackendRelationID?: string;
    BackendID?: string;
    AutoFinalizeOnOrderPaid?: boolean;
    CashJournalMethod?: EVA.Core.PaymentCashJournalMethod;
    Category?: EVA.Core.PaymentTypeCategory;
    Options?: UpdatePaymentType.PaymentTypeOptionsMaybe;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Roles?: UpdatePaymentType.PaymentTypeRole[];
    AllowMultiCurrency?: boolean;
  }

  export namespace UpdatePaymentType {
    export interface PaymentTypeRole {
      /**
       * Entity type: Role
       */
      RoleID: number;
    }

    export interface PaymentTypeOptionsMaybe {
      RequiredUserType?: EVA.Core.UserTypes;
      RequiredFunctionality?: string;
      RequiredFunctionalityScope?: EVA.Core.FunctionalityScope;
      Pending?: boolean;
      RefundPending?: boolean;
      /**
       * Setting this to false results in refund only, will therefor only work with CanBeRefunded:true and CanRefundWithoutOriginalPaymentTransaction:true
       */
      CanBeUsedForPayments?: boolean;
      CanBeRefunded?: boolean;
      CanBeCancelled?: boolean;
      CanRefundWithoutOriginalPaymentTransaction?: boolean;
      RequiresInteractionForRefunds?: boolean;
      /**
       * Entity type: GiftCardConfiguration
       */
      GiftCardConfigurationID?: number;
      /**
       * Entity type: LoyaltyProgram
       */
      LoyaltyProgramID?: number;
      PinHandler?: string;
      PinManualKeyEntry?: boolean;
      RequiresAmount?: boolean;
      RequiresConfirmationForPayments?: boolean;
      RequiresConfirmationForRefunds?: boolean;
      OnlyAvailableInLocalMode?: boolean;
      BlockOverPayments?: boolean;
    }

  }

  /**
   * Delete a PaymentType
   */
  export interface DeletePaymentType {
    /**
     * Entity type: PaymentType
     */
    ID: number;
  }

  /**
   * List PaymentTypes. Returns a paged response.
   */
  export interface ListPaymentTypes {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListPaymentTypesFilter>;
  }

  export interface ListPaymentTypesResponse {
    Result?: EVA.Core.PagedResult<ListPaymentTypesResponse.PaymentTypeDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPaymentTypesResponse {
    export interface PaymentTypeDto {
      /**
       * Entity type: PaymentType
       */
      ID: number;
      /**
       * Entity type: PaymentMethod
       */
      PaymentMethodID: number;
      /**
       * Entity type: LedgerClass
       */
      LedgerClassID: string;
      PaymentMethodName: string;
      Name: string;
      Code: string;
      IsActive: boolean;
      IsRoundingType: boolean;
      ReturnAction: EVA.Core.PaymentReturnActions;
      PrintOnDocuments: boolean;
      BackendRelationID: string;
      BackendID?: string;
      AutoFinalizeOnOrderPaid: boolean;
      CashJournalMethod: EVA.Core.PaymentCashJournalMethod;
      Options?: EVA.Core.PaymentTypeOptions;
      /**
       * Entity type: Script
       */
      ScriptID?: number;
      OrganizationUnitSetPaymentTypes: ListPaymentTypesResponse.OrganizationUnitSetPaymentTypeDto[];
    }

    export interface OrganizationUnitSetPaymentTypeDto {
      OrganizationUnitSetID: number;
      OrganizationUnitSetName: string;
      RefundPriority?: number;
      CaptureMoment?: EVA.Core.PaymentTypeCaptureMoment;
    }

  }

  /**
   * Get's the ID/Name of all paymenttypes to use for filter inputs
   */
  export interface GetPaymentTypesForFilter {
    Name?: string;
  }

  export interface GetPaymentTypesForFilterResponse {
    PaymentTypes: GetPaymentTypesForFilterResponse.PaymentType[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPaymentTypesForFilterResponse {
    export interface PaymentType {
      /**
       * Entity type: PaymentType
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Check or perform the migration for setting `PaymentMethods:PaymentTypeAvailabilityByOuSetType`.
   * 
   * Omitting `SaveChanges` will default to `false`, returning the changes it would have done, but don't actually persist them. Pass `true` explicitly if you want to actually migrate (use with caution!).
   * 
   * The payment type property `OrganizationUnitSetID` is hence-forth known as 'the legacy filter', where as the configured list of `OrganizationUnitSetPaymentType` is called 'the sets' or 'a set'.
   * 
   * Will execute for each active payment type;
   * 
   * - If it has no legacy filter configured, will remove any set that might be configured for this payment type
   * - If it *has* a legacy filter configured;
   *  - This legacy filter will be removed
   *  - If there is no matching set configured, it will add it
   *  - If there is a matching set configured but it is limited (disabled for delivery etc.), those limitations will be lifted
   *  - If there are any other set configured, those will be removed
   * 
   * And finally, the setting will be enabled on root level.
   * 
   * This migration will not run if the setting is already set (at any level!).
   */
  export interface MigratePaymentTypeOrganizationUnitSetting {
    /**
     * @default false
     */
    SaveChanges?: boolean;
  }

  export interface MigratePaymentTypeOrganizationUnitSettingResponse {
    /**
     * Entity type: PaymentType
     */
    Output: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get CustomFields for the given payment type.
   */
  export interface GetPaymentTypeCustomFields {
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
  }

  export interface GetPaymentTypeCustomFieldsResponse {
    CustomFields: GetPaymentTypeCustomFieldsResponse.CustomFieldDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPaymentTypeCustomFieldsResponse {
    export interface CustomFieldDto {
      /**
       * Entity type: CustomField
       */
      ID: number;
      /**
       * Entity type: CustomFieldType
       */
      TypeID: number;
      TypeName: string;
      Name: string;
      DisplayName?: string;
      DataType: EVA.Core.CustomFieldDataTypes;
      Order: number;
      BackendID?: string;
      IsArray: boolean;
      EnumValues?: { [key: string]: string };
    }

  }

  export interface SetPaymentTypeRoles {
    /**
     * Entity type: PaymentMethod
     */
    ID: number;
    Roles?: SetPaymentTypeRoles.PaymentTypeRole[];
  }

  export namespace SetPaymentTypeRoles {
    export interface PaymentTypeRole {
      /**
       * Entity type: Role
       */
      RoleID: number;
    }

  }

  /**
   * List paymenttransactionssettlements
   * 
   * Filters available to be sent in the request:
   * - FinancialPeriodID
   * - PaymentReference
   * - MerchantReference
   * - Type
   * - SubType
   */
  export interface ListPaymentTransactionSettlements {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    PaymentReference?: string;
    MerchantReference?: string;
    /**
     * Entity type: PaymentTransactionSettlementType
     */
    TypeID?: number;
    SubType?: string;
  }

  export interface ListPaymentTransactionsSettlementsResponse {
    PaymentTransactions: ListPaymentTransactionsSettlementsResponse.ResultPaymentTransaction[];
    Settlements: ListPaymentTransactionsSettlementsResponse.ResultSettlement[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPaymentTransactionsSettlementsResponse {
    export interface ResultPaymentTransaction {
      /**
       * Entity type: PaymentTransaction
       */
      PaymentTransactionID: number;
      SettledAmount: number;
      PaymentTransactionCaptures: ListPaymentTransactionsSettlementsResponse.ResultPaymentTransactionCapture[];
      Settlements: ListPaymentTransactionsSettlementsResponse.ResultSettlement[];
    }

    export interface ResultPaymentTransactionCapture {
      /**
       * Entity type: PaymentTransactionCapture
       */
      PaymentTransactionCaptureID: number;
      Settlements: ListPaymentTransactionsSettlementsResponse.ResultSettlement[];
      SettledAmount: number;
    }

    export interface ResultSettlement {
      Type: EVA.Core.EnumDto;
      Subtype: string;
      Amount: number;
      PaymentReference: string;
      MerchantReference: string;
    }

  }

  /**
   * Creates a new OrganizationUnitSetScope.
   * 
   * When OrganizationUnitUniqueInScope is specified as true, then a single OrganizationUnit can only occur once
   * among all OrganizationUnitSets that have this scope.
   */
  export interface CreateOrganizationUnitSetScope {
    Name: string;
    OrganizationUnitUniqueInScope?: boolean;
  }

  export interface CreateOrganizationUnitSetScopeResponse {
    /**
     * Entity type: OrganizationUnitSetScope
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns all current scopes.
   */
  export interface GetOrganizationUnitSetScopes {
  }

  export interface GetOrganizationUnitSetScopesResponse {
    Scopes: GetOrganizationUnitSetScopesResponse.OrganizationUnitSetScope[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetOrganizationUnitSetScopesResponse {
    export interface OrganizationUnitSetScope {
      /**
       * Entity type: OrganizationUnitSetScope
       */
      ID: number;
      Name: string;
      Description: string;
      OrganizationUnitUniqueInScope: boolean;
    }

  }

  /**
   * Deletes a OrganizationUnitSetScope.
   * 
   * It's not possible to delete a scope that is still in use by an OrganizationUnitSet.
   */
  export interface DeleteOrganizationUnitSetScope {
    /**
     * Entity type: OrganizationUnitSetScope
     */
    ID: number;
  }

  /**
   * Updates a OrganizationUnitSetScope.
   * 
   * When OrganizationUnitUniqueInScope is specified as true, then a single OrganizationUnit can only occur once
   * among all OrganizationUnitSets that have this scope.
   */
  export interface UpdateOrganizationUnitSetScope {
    /**
     * Entity type: OrganizationUnitSetScope
     */
    ID: number;
    Name: string;
    Description?: string;
    OrganizationUnitUniqueInScope?: boolean;
  }

  /**
   * Creates a new `OrganizationUnitSet`.
   * 
   *   Optionally a definition can be provided, which defines a set of filters.
   *   Any `OrganizationUnit` that matches the filters are automatically part of the set.
   * 
   *   If a definition is provided without any filters, it's ignored.
   * 
   *   The ScopeID refers to an `OrganizationUnitSetScope` and can be used to create collections of sets for specific purposes, such as discounts.
   */
  export interface CreateOrganizationUnitSet {
    /**
     * Name is only optional when creating an AdHoc set.
     */
    Name?: string;
    /**
     * Description.
     */
    Description?: string;
    BackendID?: string;
    Type?: EVA.Core.OrganizationUnitSetTypes;
    /**
     * Entity type: OrganizationUnitSetScope
     */
    ScopeID?: number;
    Definition?: EVA.Core.OrganizationUnitSetDefinition;
    /**
     * When specified, adds the sets contained in Subsets as a subset of the new set.
     */
    Subsets?: CreateOrganizationUnitSet.OrganizationUnitSubSetDefinition[];
    SubsetOrdering?: OrganizationSubsetOrdering;
  }

  export namespace CreateOrganizationUnitSet {
    export interface OrganizationUnitSubSetDefinition {
      /**
       * Entity type: OrganizationUnitSet
       */
      ID: number;
      OperatorType: EVA.Core.OrganizationUnitSetOperatorTypes;
    }

  }

  export const enum OrganizationSubsetOrdering {
    Ascending = 0,
    Descending = 1
  }

  export interface CreateOrganizationUnitSetResponse {
    /**
     * Entity type: OrganizationUnitSet
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Gets a detailed view of an `OrganizationUnitSet` by its ID and returns it. This service recursively returns the set with its subsets and so on.
   */
  export interface GetOrganizationUnitSetDetails {
    /**
     * Entity type: OrganizationUnitSet
     */
    ID: number;
    SubsetOrdering?: OrganizationSubsetOrdering;
  }

  export interface GetOrganizationUnitSetDetailsResponse {
    Set: OrganizationUnitSetDetails;
    DeclaredSubsets: GetOrganizationUnitSetDetailsResponse.OrganizationUnitSubset[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetOrganizationUnitSetDetailsResponse {
    export interface OrganizationUnitSubset {
      /**
       * Entity type: OrganizationUnitSubset
       */
      ID: number;
      SequenceNumber: number;
      Name: string;
      Description: string;
      Type: EVA.Core.OrganizationUnitSetOperatorTypes;
      BackendID?: string;
    }

  }

  export interface OrganizationUnitSetDetails {
    /**
     * Entity type: OrganizationUnitSubset
     */
    ID: number;
    Name: string;
    Description: string;
    Type: EVA.Core.OrganizationUnitSetTypes;
    OperatorType: EVA.Core.OrganizationUnitSetOperatorTypes;
    Definition: EVA.Core.OrganizationUnitSetDefinition;
    Subsets: OrganizationUnitSetDetails[];
    Scope: EVA.Core.EnumDto;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    BackendID?: string;
  }

  /**
   * Updates an OrganizationUnitSet.
   * 
   *   Optionally a definition can be provided, which defines a set of filters.
   *   Any `OrganizationUnit` that matches the filters are automatically part of the set.
   * 
   *   To remove a definition from a set, pass it as `null` or as an empty object to this service. If you don't want to override
   *   it, pass the definition you got back from the `GetOrganizationUnitSet` service.
   */
  export interface UpdateOrganizationUnitSet {
    /**
     * Entity type: OrganizationUnitSet
     */
    ID: number;
    Name?: string;
    BackendID?: string;
    Description?: string;
    /**
     * Entity type: OrganizationUnitSetScope
     */
    ScopeID?: number;
    Definition?: EVA.Core.OrganizationUnitSetDefinition;
  }

  /**
   * Lists OrganizationUnitSets, optionally filtered and paged.
   */
  export interface ListOrganizationUnitSets {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListOrganizationUnitSetsFilter>;
  }

  export interface ListOrganizationUnitSetsResponse {
    Result?: EVA.Core.PagedResult<EVA.Core.ListOrganizationUnitSetsItem>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Deletes an OrganizationUnitSet by its ID.
   * 
   *   It's currently not possible to delete a set that is still in use. Trying to do so will return an OrganizationUnitSets:CannotDeleteInUse error.
   */
  export interface DeleteOrganizationUnitSet {
    /**
     * Entity type: OrganizationUnitSet
     */
    ID: number;
  }

  /**
   * Adds one set (`SubsetID`) as the subset of another set (`SetID`).
   *   If this subset relation already exists, the type is updated.
   * 
   *   The Type determines whether the relation is additive or subtractive.
   *   If it's additive, the SubsetID and and all its subsets (recursively) becomes part of this set.
   * 
   *   If it's subtractive, the SubsetID and all its subsets will not be part of this set.
   */
  export interface CreateOrUpdateOrganizationUnitSubset {
    /**
     * Entity type: OrganizationUnitSet
     */
    SetID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    SubsetID: number;
    Type: EVA.Core.OrganizationUnitSetOperatorTypes;
  }

  /**
   * Removes a subset (SubsetID) from a set (SetID). If it doesn't exist, nothing happens.
   */
  export interface DeleteOrganizationUnitSubset {
    /**
     * Entity type: OrganizationUnitSet
     */
    SetID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    SubsetID: number;
  }

  /**
   * Gets an `OrganizationUnitSet` by its ID and returns the set's basic information.
   */
  export interface GetOrganizationUnitSet {
    /**
     * Entity type: OrganizationUnitSet
     */
    ID: number;
  }

  export interface GetOrganizationUnitSetResponse {
    /**
     * Entity type: OrganizationUnitSet
     */
    ID: number;
    Name: string;
    Description: string;
    Type: EVA.Core.OrganizationUnitSetTypes;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Scope: EVA.Core.EnumDto;
    BackendID?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns a what-if preview of what the set would contain if it existed with the provided definition (optional) or has provided sets as its subsets.
   */
  export interface PreviewOrganizationUnitSet {
    Definition?: EVA.Core.OrganizationUnitSetDefinition;
    Subsets?: PreviewOrganizationUnitSet.PreviewOrganizationUnitSubSetDefinition[];
    SubsetOrdering?: OrganizationSubsetOrdering;
  }

  export namespace PreviewOrganizationUnitSet {
    export interface PreviewOrganizationUnitSubSetDefinition {
      /**
       * Entity type: OrganizationUnitSubset
       */
      ID: number;
      OperatorType: EVA.Core.OrganizationUnitSetOperatorTypes;
      SequenceNumber?: number;
    }

  }

  export interface PreviewOrganizationUnitSetResponse {
    Set: OrganizationUnitSetDetails;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Takes in all subsets of an organization unit set and sets their sequences. All subsets are required.
   */
  export interface SetOrganizationUnitSubsetSequences {
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    SubsetIDs: number[];
  }

  /**
   * Get a CashHandler
   */
  export interface GetCashHandlerByID {
    /**
     * Entity type: CashHandler
     */
    ID: number;
  }

  export interface GetCashHandlerByIDResponse {
    ID: number;
    Name: string;
    CurrencyID: string;
    RoundingFactor: number;
    Coins: number[];
    BankNotes: number[];
    MaxAmountPerOrder?: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new CashHandler
   */
  export interface CreateCashHandler {
    Name: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Range: 1-10000
     */
    RoundingFactor: number;
    /**
     * Min: 0
     */
    Coins: number[];
    /**
     * Min: 0
     */
    BankNotes: number[];
    /**
     * Min: 0
     */
    MaxAmountPerOrder?: number;
  }

  export interface CreateCashHandlerResponse {
    /**
     * Entity type: CashHandler
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing CashHandler
   */
  export interface UpdateCashHandler {
    /**
     * Entity type: CashHandler
     */
    ID: number;
    Name?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Range: 1-10000
     */
    RoundingFactor?: number;
    /**
     * Min: 0
     */
    Coins?: number[];
    /**
     * Min: 0
     */
    BankNotes?: number[];
    /**
     * Min: 0
     */
    MaxAmountPerOrder?: number;
  }

  /**
   * Delete a CashHandler
   */
  export interface DeleteCashHandler {
    /**
     * Entity type: CashHandler
     */
    ID: number;
  }

  /**
   * List CashHandlers
   */
  export interface ListCashHandlers {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListCashHandlersFilter>;
  }

  export interface ListCashHandlersResponse {
    Result?: EVA.Core.PagedResult<ListCashHandlersResponse.CashHandlerDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCashHandlersResponse {
    export interface CashHandlerDto {
      /**
       * Entity type: CashHandler
       */
      ID: number;
      Name: string;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      RoundingFactor: number;
      Coins: number[];
      BankNotes: number[];
      MaxAmountPerOrder?: number;
    }

  }

  /**
   * Create a new OpeningHoursType
   */
  export interface CreateOpeningHoursType {
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  export interface CreateOpeningHoursTypeResponse {
    /**
     * Entity type: OpeningHoursType
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing OpeningHoursType
   */
  export interface UpdateOpeningHoursType {
    /**
     * Entity type: OpeningHoursType
     */
    ID: number;
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  /**
   * Delete a OpeningHoursType
   */
  export interface DeleteOpeningHoursType {
    /**
     * Entity type: OpeningHoursType
     */
    ID: number;
  }

  /**
   * Get OpeningHoursTypes
   */
  export interface GetOpeningHoursTypes {
  }

  export interface GetOpeningHoursTypesResponse {
    OpeningHoursTypes: GetOpeningHoursTypesResponse.OpeningHoursTypeDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetOpeningHoursTypesResponse {
    export interface OpeningHoursTypeDto {
      BackendID: string;
      /**
       * Entity type: Enum
       */
      ID: number;
      /**
       * Usually an internal & unchangable code - for display, use `Description`
       */
      Name: string;
      Description: string;
    }

  }

  /**
   * List the configured additional currencies for the given OrganizationUnit
   * 
   * It is required to set the OrganizationUnitID on the filter.
   */
  export interface ListOrganizationUnitCurrencies {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.OrganizationUnitCurrencyFilter>;
  }

  export interface ListOrganizationUnitCurrenciesResponse {
    Result?: EVA.Core.PagedResult<ListOrganizationUnitCurrenciesResponse.OrganizationUnitCurrencyDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListOrganizationUnitCurrenciesResponse {
    export interface OrganizationUnitCurrencyDto {
      /**
       * Entity type: OrganizationUnitCurrency
       */
      ID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      /**
       * Used to override the default exchange rates
       */
      ExchangeRate?: number;
      /**
       * Entity type: CashHandler
       */
      CashHandlerID: number;
      CashHandlerName: string;
      /**
       * Entity type: OrganizationUnit
       */
      InheritedFromOrganizationUnitID?: number;
      InheritedFromOrganizationUnitName?: string;
    }

  }

  /**
   * Create a new additional currency for an OrganizationUnit
   * 
   * It is required to define the CashHandler
   */
  export interface CreateOrganizationUnitCurrency {
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: CashHandler
     */
    CashHandlerID: number;
    /**
     * Used to override the default exchange rates
     */
    ExchangeRate?: number;
  }

  export interface CreateOrganizationUnitCurrencyResponse {
    /**
     * Entity type: OrganizationUnitCurrency
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update the Cashhandler for an additional Currency
   */
  export interface UpdateOrganizationUnitCurrency {
    /**
     * Entity type: OrganizationUnitCurrency
     */
    ID: number;
    /**
     * Entity type: CashHandler
     */
    CashHandlerID: number;
    /**
     * Used to override the default exchange rates
     */
    ExchangeRate?: number;
  }

  /**
   * Delete an additional Currency
   */
  export interface DeleteOrganizationUnitCurrency {
    /**
     * Entity type: OrganizationUnitCurrency
     */
    ID: number;
  }

  /**
   * List organizationunit openinghours
   */
  export interface ListOpeningHours {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListOldOpeningHoursFilter>;
  }

  export interface ListOpeningHoursResponse {
    Result?: EVA.Core.PagedResult<EVA.Core.OpeningHoursDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create new OpeningHours for OrganizationUnits
   * 
   * Multiple items can be created in a single request. Also multiple OrganizationUnits are allowed.
   * 
   * It's not allowed to create multiple items for a single day on the same store. This is checked for Day based openinghours and Date based openinghours.
   * 
   * Openinghours wit a Date component override the openinghours on Day level. They are called SpecialOpeningHours most of the time.
   * 
   * To close a store on a regular day, just don't create an openinghour for that day (or delete it).
   * To close a store on a specific date, create an openinghour with StartTime=null, EndTime=null
   */
  export interface CreateOrganizationUnitOpeningHours {
    OpeningHours: EVA.Core.OpeningHoursDataDto[];
  }

  /**
   * Update existing OpeningHours for OrganizationUnits
   * 
   * Multiple items can be updated in a single request. Also multiple OrganizationUnits are allowed.
   */
  export interface UpdateOrganizationUnitOpeningHours {
    OpeningHours: EVA.Core.OpeningHoursDto[];
  }

  /**
   * Delete a single existing OpeningHour for an OrganizationUnit
   */
  export interface DeleteOrganizationUnitOpeningHours {
    /**
     * Entity type: OpeningHours
     */
    ID: number;
  }

  export interface GetOrganizationUnitSupplierDataByOrganizationUnitID {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetOrganizationUnitSupplierDataByOrganizationUnitIDResponse {
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    EstimatedDeliveryDays?: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface SetOrganizationUnitSupplierData {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Min: 0
     */
    EstimatedDeliveryDays?: number;
  }

  /**
   * Update the regular openinghours for an OrganizationUnit
   * 
   * This service accepts a request like this:
   * ```
   * {
   *   "OrganizationUnitID": 123, // required
   *   "TypeID": 2, // required, OpeningHoursType
   *   "Sunday": {
   *     "IsClosed": false, // required
   *     "StartTime": "10:00:00", // optional
   *     "EndTime": "18:00:00", // optional
   *     "Description": "Repairs closed"// optional
   *   }, // required
   *   "Monday": ..., //required
   *   "Tuesday": ..., //required
   *   "Wednesday": ..., //required
   *   "Thursday": ..., //required
   *   "Friday": ..., //required
   *   "Saturday": ..., //required
   *   }
   * ```
   * All days are required, days can be closed and contain a description
   * 
   * To manage non-regular OpeningHours the existing CreateOrganizationUnitOpeningHours, UpdateOrganizationUnitOpeningHours, DeleteOrganizationUnitOpeningHours can be used.
   * They have a TypeID property on the OpeningHours object to specify the OpeningHoursType.
   */
  export interface UpdateOrganizationUnitRegularOpeningHours {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: OpeningHoursType
     */
    TypeID: number;
    Sunday: EVA.Core.DayOpeningHours;
    Monday: EVA.Core.DayOpeningHours;
    Tuesday: EVA.Core.DayOpeningHours;
    Wednesday: EVA.Core.DayOpeningHours;
    Thursday: EVA.Core.DayOpeningHours;
    Friday: EVA.Core.DayOpeningHours;
    Saturday: EVA.Core.DayOpeningHours;
  }

  /**
   * List the CheckoutOptionCategories
   */
  export interface ListCheckoutOptionCategories {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListCheckoutOptionCategoriesFilter>;
    PageToken?: string;
  }

  export interface ListCheckoutOptionCategoriesResponse {
    Results?: CheckoutOptionCategoryDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface CheckoutOptionCategoryDto {
    /**
     * Entity type: CheckoutOptionCategory
     */
    ID: number;
    Name: string;
    Description?: string;
    Sequence: number;
  }

  /**
   * Create a new CheckoutOptionCategory
   */
  export interface CreateCheckoutOptionCategory {
    Name: string;
    Description?: string;
    Sequence: number;
  }

  export interface CreateCheckoutOptionCategoryResponse {
    /**
     * Entity type: CheckoutOptionCategory
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing CheckoutOptionCategory
   */
  export interface UpdateCheckoutOptionCategory {
    /**
     * Entity type: CheckoutOptionCategory
     */
    ID: number;
    Name?: string;
    Description?: string;
    Sequence?: number;
  }

  /**
   * Delete a CheckoutOptionCategory
   */
  export interface DeleteCheckoutOptionCategory {
    /**
     * Entity type: CheckoutOptionCategory
     */
    ID: number;
  }

  /**
   * Create a new CheckoutOption
   */
  export interface CreateCheckoutOption {
    /**
     * Entity type: CheckoutOptionCategory
     */
    CategoryID?: number;
    IsActive: boolean;
    Name: string;
    Description?: string;
    Handler: string;
    Data?: any;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Sequence: number;
  }

  export interface CreateCheckoutOptionResponse {
    /**
     * Entity type: CheckoutOption
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing CheckoutOption
   */
  export interface UpdateCheckoutOption {
    /**
     * Entity type: CheckoutOption
     */
    ID: number;
    /**
     * Entity type: CheckoutOptionCategory
     */
    CategoryID?: number;
    IsActive?: boolean;
    Name?: string;
    Description?: string;
    Handler?: string;
    Data?: any;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Sequence?: number;
  }

  /**
   * Delete a CheckoutOption
   */
  export interface DeleteCheckoutOption {
    /**
     * Entity type: CheckoutOption
     */
    ID: number;
  }

  /**
   * Get the available checkout option handlers
   */
  export interface GetCheckoutOptionHandlers {
  }

  export interface GetCheckoutOptionHandlersResponse {
    Handlers: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates the sequence of supplied CheckoutOptions `IDs` matching the order they were submitted. CheckoutOptions not supplied are not touched.
   * 
   * The first supplied CheckoutOption will get `Sequence=0` and thus will be returned first. The second will get `Sequence=1`, and so on.
   */
  export interface UpdateCheckoutOptionSequences {
    /**
     * Entity type: CheckoutOption
     */
    IDs?: number[];
  }

  /**
   * Create custom "OrderLedgerType"
   */
  export interface CreateOrderLedgerType {
    Name: string;
    Description?: string;
  }

  export interface CreateOrderLedgerTypeResponse {
    /**
     * Entity type: OrderLedgerType
     */
    ID: number;
    Name: string;
    Description?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Delete existing "OrderLedgerType"
   */
  export interface DeleteOrderLedgerType {
    /**
     * Entity type: OrderLedgerType
     */
    ID: number;
  }

  /**
   * Returns a list of "OrderLedgerType"
   */
  export interface ListOrderLedgerTypes {
  }

  export interface ListOrderLedgerTypesResponse {
    OrderLedgerTypes: ListOrderLedgerTypesResponse.ResponseModel[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListOrderLedgerTypesResponse {
    export interface ResponseModel {
      /**
       * Entity type: OrderLedgerType
       */
      ID: number;
      Name: string;
      Description?: string;
    }

  }

  /**
   * Returns a paged result of all the refund reasons.
   */
  export interface ListRefundCorrectionReasons {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListRefundCorrectionReasonsFilter>;
    PageToken?: string;
  }

  export interface ListRefundCorrectionReasonsResponse {
    Results?: ListRefundCorrectionReasonsResponse.Model[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListRefundCorrectionReasonsResponse {
    export interface Model {
      /**
       * Entity type: RefundCorrectionReason
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Description?: string;
      Priority: number;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      OrganizationUnitSetName: string;
    }

  }

  /**
   * Returns a single refund reason.
   */
  export interface GetRefundCorrectionReason {
    /**
     * Entity type: RefundCorrectionReason
     */
    ID: number;
  }

  export interface GetRefundCorrectionReasonResponse {
    BackendID?: string;
    Name: string;
    Description?: string;
    Priority: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    OrganizationUnitSetName: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new refund reason.
   */
  export interface CreateRefundCorrectionReason {
    BackendID?: string;
    Name?: string;
    Description?: string;
    Priority?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export interface CreateRefundCorrectionReasonResponse {
    /**
     * Entity type: RefundCorrectionReason
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface UpdateRefundCorrectionReason {
    /**
     * Entity type: RefundCorrectionReason
     */
    ID: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
    Priority?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  /**
   * Deletes a refund reason. Note that it can still be seen on orders that have this field, but you can no longer use it.
   */
  export interface DeleteRefundCorrectionReason {
    /**
     * Entity type: RefundCorrectionReason
     */
    ID: number;
  }

  /**
   * Create a new UnitPriceCorrectionReason
   */
  export interface CreateUnitPriceCorrectionReason {
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Name?: string;
    Description?: string;
    BackendID?: string;
    Priority?: number;
  }

  export interface CreateUnitPriceCorrectionReasonResponse {
    /**
     * Entity type: UnitPriceCorrectionReason
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing UnitPriceCorrectionReason
   */
  export interface UpdateUnitPriceCorrectionReason {
    /**
     * Entity type: UnitPriceCorrectionReason
     */
    ID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Name?: string;
    Description?: string;
    BackendID?: string;
    Priority?: number;
  }

  /**
   * Delete a UnitPriceCorrectionReason
   */
  export interface DeleteUnitPriceCorrectionReason {
    /**
     * Entity type: UnitPriceCorrectionReason
     */
    ID: number;
  }

  /**
   * Returns a paged result of all the unit price correction reasons.
   */
  export interface ListUnitPriceCorrectionReasons {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListUnitPriceCorrectionReasonsFilter>;
    PageToken?: string;
  }

  export interface ListUnitPriceCorrectionReasonsResponse {
    Results?: ListUnitPriceCorrectionReasonsResponse.Model[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListUnitPriceCorrectionReasonsResponse {
    export interface Model {
      /**
       * Entity type: UnitPriceCorrectionReason
       */
      ID: number;
      Description?: string;
      BackendID?: string;
      Name: string;
      Priority: number;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      OrganizationUnitSetName: string;
    }

  }

  /**
   * Update existing "OrderLedgerType"
   */
  export interface UpdateOrderLedgerType {
    /**
     * Entity type: OrderLedgerType
     */
    ID: number;
    Name: string;
    Description?: string;
  }

  export interface UpdateOrderLedgerTypeResponse {
    /**
     * Entity type: OrderLedgerType
     */
    ID: number;
    Name: string;
    Description?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List all return reasons. Does not support sorting. Always sorted by `Priority`
   */
  export interface ListReturnReasons {
    PageConfig?: EVA.Core.PageConfig<ListReturnReasonsFilter>;
  }

  export interface ListReturnReasonsFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Type?: EVA.Core.ReturnReasonType;
    /**
     * Entity type: ReturnReason
     */
    ID?: number;
    Name?: string;
    RemarkType?: EVA.Core.ReturnReasonRemarkType;
  }

  export interface ListReturnReasonsResponse {
    Result?: EVA.Core.PagedResult<ListReturnReasonsResponse.Reason>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListReturnReasonsResponse {
    export interface Reason {
      /**
       * Entity type: ReturnReason
       */
      ID?: number;
      Name?: string;
      Description?: string;
      BackendID?: string;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID?: number;
      OrganizationUnitSetName?: string;
      Type?: EVA.Core.ReturnReasonType;
      RemarkType?: EVA.Core.ReturnReasonRemarkType;
      Priority?: number;
    }

  }

  /**
   * Get the reasons by type.
   */
  export interface GetReturnReasonsByType {
    Type: EVA.Core.ReturnReasonType;
  }

  export interface GetReturnReasonsByTypeResponse {
    Reasons: GetReturnReasonsByTypeResponse.Reason[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetReturnReasonsByTypeResponse {
    export interface Reason {
      ID: number;
      Name: string;
      Description: string;
      BackendID: string;
      RemarkType: EVA.Core.ReturnReasonRemarkType;
    }

  }

  /**
   * @deprecated Use `ListReturnReasons` instead.
   * Get all return reasons
   */
  export interface GetReturnReasons {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Type?: EVA.Core.ReturnReasonType;
  }

  export interface GetReturnReasonsResponse {
    Reasons: GetReturnReasonsResponse.ReturnReasonDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetReturnReasonsResponse {
    export interface ReturnReasonDto {
      /**
       * Entity type: ReturnReason
       */
      ID?: number;
      Name?: string;
      Description?: string;
      BackendID?: string;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID?: number;
      OrganizationUnitSetName?: string;
      Type?: EVA.Core.ReturnReasonType;
      RemarkType?: EVA.Core.ReturnReasonRemarkType;
      Priority?: number;
    }

  }

  /**
   * Create a return reason
   */
  export interface CreateReturnReason {
    Name: string;
    Description?: string;
    BackendID?: string;
    Priority: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Optional for backwards compatibility and defaults to OrganizationUnit, but should be entered. Will be required in the future.
     */
    Type?: EVA.Core.ReturnReasonType;
    /**
     * Optional for backwards compatibility and defaults to `Hidden`, but should be entered. Will be required in the future.
     */
    RemarkType?: EVA.Core.ReturnReasonRemarkType;
  }

  export interface CreateReturnReasonResponse {
    /**
     * Entity type: ReturnReason
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Delete a return reason
   */
  export interface DeleteReturnReason {
    /**
     * Entity type: ReturnReason
     */
    ID: number;
  }

  /**
   * Update a return reason
   */
  export interface UpdateReturnReason {
    /**
     * Entity type: ReturnReason
     */
    ID: number;
    Name?: string;
    Description?: string;
    BackendID?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Optional for backwards compatibility and defaults to OrganizationUnit, but should be entered. Will be required in the future.
     */
    Type?: EVA.Core.ReturnReasonType;
    Priority?: number;
    RemarkType?: EVA.Core.ReturnReasonRemarkType;
  }

  /**
   * Duplicate an existing MessageTemplate
   */
  export interface DuplicateMessageTemplate {
    /**
     * Entity type: MessageTemplate
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Layout?: string;
  }

  export interface DuplicateMessageTemplateResponse {
    /**
     * Entity type: MessageTemplate
     */
    ID: number;
    Type: EVA.Core.MessageTemplateTypes;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetDataModelForTemplateHandler {
    Handler?: string;
  }

  export interface GetDataModelForTemplateHandlerResponse {
    DataModel: { [key: string]: any };
    SampleData: { [key: string]: any };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListMessageTemplateLayouts {
    Destination?: EVA.Core.MessageTemplateDestinations;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    LanguageID?: string;
    CountryID?: string;
  }

  export interface ListMessageTemplateLayoutsResponse {
    Layouts: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListTemplateHandlers {
  }

  export interface ListTemplateHandlersResponse {
    Handlers: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get a MessageTemplate
   */
  export interface GetMessageTemplateByID {
    /**
     * Entity type: MessageTemplate
     */
    ID: number;
  }

  export interface GetMessageTemplateByIDResponse {
    /**
     * Entity type: MessageTemplate
     */
    ID: number;
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName?: string;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    Header: string;
    Template: string;
    Footer: string;
    Helpers: string;
    Type: EVA.Core.MessageTemplateTypes;
    Layout: string;
    Destination: EVA.Core.MessageTemplateDestinations;
    PaperProperties: EVA.Core.PaperProperties;
    IsDisabled: boolean;
    LastModifiedByEmail?: string;
    /**
     * Entity type: User
     */
    LastModifiedByUserID?: number;
    LastModifiedAt?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new MessageTemplate
   */
  export interface CreateMessageTemplate {
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Header?: string;
    Template: string;
    Footer?: string;
    Helpers?: string;
    Type: EVA.Core.MessageTemplateTypes;
    Layout?: string;
    Destination: EVA.Core.MessageTemplateDestinations;
    PaperProperties?: EVA.Core.PaperProperties;
    IsDisabled?: boolean;
  }

  export interface CreateMessageTemplateResponse {
    /**
     * Entity type: MessageTemplate
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing MessageTemplate
   */
  export interface UpdateMessageTemplate {
    /**
     * Entity type: MessageTemplate
     */
    ID: number;
    Name?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Header?: string;
    Template?: string;
    Footer?: string;
    Helpers?: string;
    Layout?: string;
    Destination?: EVA.Core.MessageTemplateDestinations;
    PaperProperties?: EVA.Core.PaperProperties;
    IsDisabled?: boolean;
  }

  /**
   * Delete a MessageTemplate
   */
  export interface DeleteMessageTemplate {
    /**
     * Entity type: MessageTemplate
     */
    ID: number;
  }

  /**
   * List the MessageTemplates
   */
  export interface ListMessageTemplates {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.MessageTemplateFilter>;
  }

  export interface ListMessageTemplatesResponse {
    Result?: EVA.Core.PagedResult<ListMessageTemplatesResponse.MessageTemplateDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListMessageTemplatesResponse {
    export interface MessageTemplateDto {
      /**
       * Entity type: MessageTemplate
       */
      ID: number;
      Name: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      OrganizationUnitName: string;
      /**
       * Entity type: Language
       */
      LanguageID: string;
      /**
       * Entity type: Country
       */
      CountryID: string;
      Type: EVA.Core.MessageTemplateTypes;
      Layout: string;
      Destination: EVA.Core.MessageTemplateDestinations;
      PaperProperties: EVA.Core.PaperProperties;
      IsDisabled: boolean;
      LastModifiedByEmail?: string;
      /**
       * Entity type: User
       */
      LastModifiedByUserID?: number;
      LastModifiedAt?: string;
      ReadOnly: boolean;
    }

  }

  /**
   * Get the history of a MessageTemplate
   */
  export interface GetMessageTemplateHistoryByID {
    /**
     * Entity type: MessageTemplate
     */
    ID: number;
  }

  export interface GetMessageTemplateHistoryByIDResponse {
    Entries: GetMessageTemplateHistoryByIDResponse.MessageTemplateHistoryEntry[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetMessageTemplateHistoryByIDResponse {
    export interface MessageTemplateHistoryEntry {
      /**
       * Entity type: MessageTemplate
       */
      ID: number;
      Name: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      /**
       * Entity type: Language
       */
      LanguageID: string;
      /**
       * Entity type: Country
       */
      CountryID: string;
      Header: string;
      Template: string;
      Footer: string;
      Helpers: string;
      Type: EVA.Core.MessageTemplateTypes;
      Layout: string;
      Destination: EVA.Core.MessageTemplateDestinations;
      PaperProperties: EVA.Core.PaperProperties;
      IsDisabled: boolean;
      IsDeleted: boolean;
      ExistedFrom: string;
      ExistedTill?: string;
      ModifiedByEmail?: string;
      /**
       * Entity type: User
       */
      ModifiedByUserID?: number;
      ModifiedAt?: string;
    }

  }

  /**
   * Preview the result of the MessageTemplate
   */
  export interface PreviewMessageTemplate {
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Header?: string;
    Template: string;
    Footer?: string;
    Helpers?: string;
    Layout?: string;
    Destination: EVA.Core.MessageTemplateDestinations;
    PaperProperties?: EVA.Core.PaperProperties;
    Type: EVA.Core.MessageTemplateTypes;
    TargetContentType: EVA.Core.MessageTargetContentTypes;
    SampleData?: { [key: string]: any };
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * A preview will be generated, even if device is known
     */
    ForcePreview?: boolean;
  }

  export interface PreviewMessageTemplateResponse {
    Url: string;
    Success: boolean;
    Message: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface SendSampleMailMessageTemplate {
    Name?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Template?: string;
    Helpers?: string;
    Type: EVA.Core.MessageTemplateTypes;
    Layout?: string;
    Destination: EVA.Core.MessageTemplateDestinations;
    EmailAddress?: string;
    SampleData?: { [key: string]: any };
  }

  /**
   * List the LoyaltyProgramCondition
   */
  export interface ListLoyaltyProgramConditions {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    InitialPageConfig?: EVA.Core.PageTokenConfig<{ [key:string]: string }>;
    PageToken?: string;
  }

  export interface ListLoyaltyProgramConditionsResponse {
    Results?: ListLoyaltyProgramConditionsResponse.Item[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListLoyaltyProgramConditionsResponse {
    export interface Item {
      /**
       * Entity type: LoyaltyProgramCondition
       */
      ID: number;
      /**
       * Entity type: LoyaltyProgram
       */
      LoyaltyProgramID: number;
      BackendID?: string;
      Type: string;
    }

  }

  /**
   * Get a LoyaltyProgramCondition
   */
  export interface GetLoyaltyProgramCondition {
    /**
     * Entity type: LoyaltyProgramCondition
     */
    ID: number;
  }

  export interface GetLoyaltyProgramConditionResponse {
    /**
     * Entity type: LoyaltyProgramCondition
     */
    ID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    BackendID?: string;
    Type?: string;
    Data?: any;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new LoyaltyProgramCondition
   */
  export interface CreateLoyaltyProgramCondition {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    BackendID?: string;
    Type?: string;
    Data?: any;
  }

  export interface CreateLoyaltyProgramConditionResponse {
    /**
     * Entity type: LoyaltyProgramCondition
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing LoyaltyProgramCondition
   */
  export interface UpdateLoyaltyProgramCondition {
    /**
     * Entity type: LoyaltyProgramCondition
     */
    ID?: number;
    Data?: any;
  }

  /**
   * Delete a LoyaltyProgramCondition
   */
  export interface DeleteLoyaltyProgramCondition {
    /**
     * Entity type: LoyaltyProgramCondition
     */
    ID: number;
  }

  /**
   * List the LoyaltyPrograms
   */
  export interface ListLoyaltyPrograms {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListLoyaltyProgramsFilter>;
    PageToken?: string;
  }

  export interface ListLoyaltyProgramsResponse {
    Results?: ListLoyaltyProgramsResponse.LoyaltyProgramDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListLoyaltyProgramsResponse {
    export interface LoyaltyProgramDto {
      /**
       * Entity type: LoyaltyProgram
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Handler: string;
      Options: EVA.Core.LoyaltyProgramOptions;
      /**
       * Entity type: Currency
       */
      BudgetCurrencyID: string;
      StartDate?: string;
      EndDate?: string;
      Status: EVA.Core.LoyaltyProgramStatus;
      DeactivationReason: EVA.Core.LoyaltyProgramDeactivationReasons;
    }

  }

  /**
   * Get a LoyaltyProgram
   */
  export interface GetLoyaltyProgram {
    /**
     * Entity type: LoyaltyProgram
     */
    ID: number;
  }

  export interface GetLoyaltyProgramResponse {
    /**
     * Entity type: LoyaltyProgram
     */
    ID?: number;
    /**
     * Entity type: LoyaltyProgramGroup
     */
    LoyaltyProgramGroupID?: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
    Options?: EVA.Core.LoyaltyProgramOptions;
    StartDate?: string;
    EndDate?: string;
    Status?: EVA.Core.LoyaltyProgramStatus;
    Handler?: string;
    Data?: any;
    ReloadPolicy?: string;
    /**
     * When true, the loyalty program will only be applied when a customer is attached to the order.
     * 
     * When false, the loyalty program can be attached to the order even if there is no customer attached.
     */
    RequireCustomer?: boolean;
    /**
     * When true, the loyalty program will only be applied when a customer is attached to the order that is already subscribed to the loyalty program.
     * Also, this will make the `RequireCustomer` automatically true and `AutoSubscribeCustomer` false.
     * 
     * When false, the loyalty program can be attached to the order if a customer is attached that isn't already subscribed to the loyalty program.
     */
    RequiresSubscribedCustomer?: boolean;
    /**
     * When true, if a non anonymous customer is attached to the order they will be automatically subscribed on finishing the order.
     * When false, the customer will not be subscribed.
     * 
     * This is ignore when`RequiresSubscribedCustomer` or `RequiresSubscriptionValidation` is set to true.
     */
    AutoSubscribeCustomer?: boolean;
    /**
     * When true, the order will require that the attached customer on the order will be validated by providing the UserIdentifier in the `AttachBenefitProgramToOrder` service.
     * Also, this will make the `RequireCustomer` and `RequiresSubscribedCustomer` automatically true and `AutoSubscribeCustomer` false.
     * 
     * When false, this validation isn't required.
     */
    RequiresSubscriptionValidation?: boolean;
    LoyaltyPointUsageOptions?: EVA.Core.LoyaltyPointUsageOptions;
    PointPendingPolicyType?: string;
    PointPendingPolicyData?: { [key: string]: any };
    PointExpirationPolicyType?: string;
    PointExpirationPolicyData?: { [key: string]: any };
    /**
     * Entity type: Currency
     */
    BudgetCurrencyID?: string;
    Budget?: number;
    UsedBudget?: number;
    BudgetUsagePercentage?: number;
    BudgetRemaining?: number;
    DeactivateWhenBudgetIsReached?: boolean;
    UserMaximumUsage?: number;
    UserBudget?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    SubscriptionID?: number;
    RequireManualUserIdentifier?: boolean;
    UserIdentifierPrefix?: string;
    UserIdentifierStartNumber?: number;
    UserIdentifierSuffix?: string;
    ResubscribeHandling?: EVA.Core.ResubscribeHandlingType;
    AllowUnsubscribeBy?: EVA.Core.AllowUnsubscribeByType;
    DeactivationReason?: EVA.Core.LoyaltyProgramDeactivationReasons;
    Company?: GetLoyaltyProgramResponse.CompanyDto;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetLoyaltyProgramResponse {
    export interface CompanyDto {
      /**
       * Entity type: Company
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Create a new LoyaltyProgram
   */
  export interface CreateLoyaltyProgram {
    /**
     * Entity type: LoyaltyProgramGroup
     */
    LoyaltyProgramGroupID?: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
    Options?: EVA.Core.LoyaltyProgramOptions;
    /**
     * Entity type: Currency
     */
    BudgetCurrencyID?: string;
    StartDate?: string;
    EndDate?: string;
    Status?: EVA.Core.LoyaltyProgramStatus;
    Handler?: string;
    Data?: any;
    ReloadPolicy?: string;
    LoyaltyPointUsageOptions?: EVA.Core.LoyaltyPointUsageOptions;
    PointPendingPolicyType?: string;
    PointPendingPolicyData?: { [key: string]: any };
    PointExpirationPolicyType?: string;
    PointExpirationPolicyData?: { [key: string]: any };
    /**
     * When true, the loyalty program will only be applied when a customer is attached to the order.
     * 
     * When false, the loyalty program can be attached to the order even if there is no customer attached.
     */
    RequireCustomer?: boolean;
    /**
     * When true, the loyalty program will only be applied when a customer is attached to the order that is already subscribed to the loyalty program.
     * Also, this will make the `RequireCustomer` automatically true and `AutoSubscribeCustomer` false.
     * 
     * When false, the loyalty program can be attached to the order if a customer is attached that isn't already subscribed to the loyalty program.
     */
    RequiresSubscribedCustomer?: boolean;
    /**
     * When true, if a non anonymous customer is attached to the order they will be automatically subscribed on finishing the order.
     * When false, the customer will not be subscribed.
     * 
     * This is ignore when`RequiresSubscribedCustomer` or `RequiresSubscriptionValidation` is set to true.
     */
    AutoSubscribeCustomer?: boolean;
    /**
     * When true, the order will require that the attached customer on the order will be validated by providing the UserIdentifier in the `AttachBenefitProgramToOrder` service.
     * Also, this will make the `RequireCustomer` and `RequiresSubscribedCustomer` automatically true and `AutoSubscribeCustomer` false.
     * 
     * When false, this validation isn't required.
     */
    RequiresSubscriptionValidation?: boolean;
    UserMaximumUsage?: number;
    UserBudget?: number;
    /**
     * When set to true, subscribing the user will require a UserIdentifier to be given.
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * When this is not the case on create, this can be introduced on update. This value cannot be changed after it is set in either the create or update.
     */
    RequireManualUserIdentifier?: boolean;
    /**
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * When this is not the case on create, this can be introduced on update. This value cannot be changed after it is set in either the create or update.
     */
    UserIdentifierPrefix?: string;
    /**
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * When this is not the case on create, this can be introduced on update. This value cannot be changed after it is set in either the create or update.
     * Min: 0
     */
    UserIdentifierStartNumber?: number;
    /**
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * When this is not the case on create, this can be introduced on update. This value cannot be changed after it is set in either the create or update.
     */
    UserIdentifierSuffix?: string;
    /**
     * Handling when a user is unsubscribed and re-subscribes.
     */
    ResubscribeHandling?: EVA.Core.ResubscribeHandlingType;
    /**
     * Indicates if the subscription can be unsubscribed and if so, who can do the unsubscribe.
     */
    AllowUnsubscribeBy?: EVA.Core.AllowUnsubscribeByType;
    /**
     * Attaches the loyalty program to a consumer company.
     * Entity type: Company
     */
    CompanyID?: number;
  }

  export interface CreateLoyaltyProgramResponse {
    /**
     * Entity type: LoyaltyProgram
     */
    ID: number;
    /**
     * Entity type: Subscription
     */
    SubscriptionID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing LoyaltyProgram
   */
  export interface UpdateLoyaltyProgram {
    /**
     * Entity type: LoyaltyProgram
     */
    ID: number;
    /**
     * Entity type: LoyaltyProgramGroup
     */
    LoyaltyProgramGroupID?: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
    Options?: EVA.Core.LoyaltyProgramOptions;
    /**
     * Entity type: Currency
     */
    BudgetCurrencyID?: string;
    StartDate?: string;
    EndDate?: string;
    Status?: EVA.Core.LoyaltyProgramStatus;
    Data?: any;
    ReloadPolicy?: string;
    LoyaltyPointUsageOptions?: EVA.Core.LoyaltyPointUsageOptions;
    PointPendingPolicyType?: string;
    PointPendingPolicyData?: { [key: string]: any };
    PointExpirationPolicyType?: string;
    PointExpirationPolicyData?: { [key: string]: any };
    /**
     * When true, the loyalty program will only be applied when a customer is attached to the order.
     * 
     * When false, the loyalty program can be attached to the order even if there is no customer attached.
     */
    RequireCustomer?: boolean;
    /**
     * When true, the loyalty program will only be applied when a customer is attached to the order that is already subscribed to the loyalty program.
     * Also, this will make the `RequireCustomer` automatically true and `AutoSubscribeCustomer` false.
     * 
     * When false, the loyalty program can be attached to the order if a customer is attached that isn't already subscribed to the loyalty program.
     */
    RequiresSubscribedCustomer?: boolean;
    /**
     * When true, if a non anonymous customer is attached to the order they will be automatically subscribed on finishing the order.
     * When false, the customer will not be subscribed.
     * 
     * This is ignore when`RequiresSubscribedCustomer` or `RequiresSubscriptionValidation` is set to true.
     */
    AutoSubscribeCustomer?: boolean;
    /**
     * When true, the order will require that the attached customer on the order will be validated by providing the UserIdentifier in the `AttachBenefitProgramToOrder` service.
     * Also, this will make the `RequireCustomer` and `RequiresSubscribedCustomer` automatically true and `AutoSubscribeCustomer` false.
     * 
     * When false, this validation isn't required.
     */
    RequiresSubscriptionValidation?: boolean;
    UserMaximumUsage?: number;
    UserBudget?: number;
    /**
     * When set to true, subscribing the user will require a UserIdentifier to be given. Will be ignored is this was already set on create.
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * This value cannot be changed after it is set in either the create or update.
     */
    RequireManualUserIdentifier?: boolean;
    /**
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * This value cannot be changed after it is set in either the create or update.
     */
    UserIdentifierPrefix?: string;
    /**
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * This value cannot be changed after it is set in either the create or update.
     * Min: 0
     */
    UserIdentifierStartNumber?: number;
    /**
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * This value cannot be changed after it is set in either the create or update.
     */
    UserIdentifierSuffix?: string;
    /**
     * Handling when a user is unsubscribed and re-subscribes.
     */
    ResubscribeHandling?: EVA.Core.ResubscribeHandlingType;
    /**
     * Indicates if the subscription can be unsubscribed and if so, who can do the unsubscribe.
     */
    AllowUnsubscribeBy?: EVA.Core.AllowUnsubscribeByType;
    /**
     * Attaches the loyalty program to a consumer company.
     * Entity type: Company
     */
    CompanyID?: number;
  }

  /**
   * Update the budget on the loyalty program group.
   * ATTENTION! When the currency is changed, the used budget registration will be reset to 0!
   * When the budget is lowered below the current used budget and the `DeactivateWhenBudgetIsReached` is set to true, all related programs will be deactivated.
   * When the budget is cleared or raised above the current used budget, or `DeactivateWhenBudgetIsReached` is set to false, and the `ReactivateProgramsWhenPossible` is set to true, all programs that were deactivate by the group budget will be reactivated when possible.
   */
  export interface UpdateLoyaltyProgramBudget {
    /**
     * Entity type: LoyaltyProgram
     */
    ID: number;
    Budget?: number;
    DeactivateWhenBudgetIsReached?: boolean;
    ReactivateProgramWhenPossible?: boolean;
  }

  /**
   * Delete a LoyaltyProgram
   */
  export interface DeleteLoyaltyProgram {
    /**
     * Entity type: LoyaltyProgram
     */
    ID: number;
    /**
     * You can provide the property `Force` to force remaining subscribed users to be unsubscribed. Doing so requires ForceDeleteLoyaltyProgram functionality.
     */
    Force?: boolean;
  }

  /**
   * Get the available loyalty program handlers
   */
  export interface GetLoyaltyProgramHandlers {
  }

  export interface GetLoyaltyProgramHandlersResponse {
    Handlers: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Deposit loyalty points to a user
   */
  export interface DepositLoyaltyPoints {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Min: 1
     */
    Points: number;
  }

  /**
   * Withdraw loyalty points from a user
   */
  export interface WithdrawLoyaltyPoints {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Min: 1
     */
    Points: number;
  }

  /**
   * Rebalance loyalty points of a user
   */
  export interface RebalanceLoyaltyPoints {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * If this is set to true, pending points will be kept. When false, all pending points are also removed.
     */
    KeepPendingPoints?: boolean;
    /**
     * If this is set, the balance will be adjusted with the given point balance. This will ignore pending points when they are kept.
     * Min: 0
     */
    NewPointBalance: number;
    /**
     * Expiration date will be calculated based on the loyalty program expiration policy and current date and applied to the given NewPointBalance. Ignored when NewPointBalance is 0.
     */
    ApplyProgramExpiration?: boolean;
    /**
     * Date when the new points will be expired. When null provided, the loyalty points will not expire. When date provided, the new balance will expire on that date. Ignored when `ApplyProgramExpiration` is set to true. Ignored when NewPointBalance is 0.
     */
    ExpirationDate?: string;
  }

  /**
   * Asynchronously rebalance loyalty points of multiple users for a loyalty program.
   * This service is not transactional, meaning that if a mutation fails, other mutations that have already succeeded will not be reverted.
   * The maximum amount of users that can be rebalanced in a single request is 10,000.
   */
  export interface RebalanceLoyaltyPointsForUsers {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    UserLoyaltyPointAdjustments: UserLoyaltyPointAdjustment[];
  }

  export interface UserLoyaltyPointAdjustment {
    /**
     * Entity type: User
     */
    UserID: number;
    KeepPendingPoints: boolean;
    /**
     * How to adjust the balance. Points can be deposited, withdrew or 'rebalanced', the latter will set the balance to the given points.
     */
    AdjustmentType: LoyaltyPointAdjustmentType;
    /**
     * The amount of points to adjust.
     */
    Points: number;
    /**
     * Expiration date will be calculated based on the loyalty program expiration policy and current date and applied to the given `NewPointBalance`. Ignored if the new balance after adjustment is 0.
     */
    ApplyProgramExpiration: boolean;
    /**
     * Date when the new points will be expired. When null provided, the loyalty points will not expire. When date provided, the new balance will expire on that date. Ignored when `ApplyProgramExpiration` is set to true. Ignored if the new balance after adjustment is 0.
     */
    ExpirationDate?: string;
  }

  export const enum LoyaltyPointAdjustmentType {
    Deposit = 0,
    Withdraw = 1,
    Rebalance = 2
  }

  export interface RebalanceLoyaltyPointsForUsersResponse {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * The users that were successfully adjusted. Null when no users were successfully adjusted.
     */
    SuccessfullyAdjustedUsers?: RebalanceLoyaltyPointsForUsersResponse.SuccessfullyAdjustedUser[];
    /**
     * The users that failed to adjust. Null when no users failed to adjust.
     */
    FailedToAdjustUsers?: RebalanceLoyaltyPointsForUsersResponse.FailedToAdjustUser[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace RebalanceLoyaltyPointsForUsersResponse {
    export interface SuccessfullyAdjustedUser {
      /**
       * Entity type: User
       */
      UserID: number;
      NewLoyaltyPointBalance: number;
    }

    export interface FailedToAdjustUser {
      /**
       * Entity type: User
       */
      UserID: number;
      KeepPendingPoints: boolean;
      ApplyProgramExpiration: boolean;
      ExpirationDate?: string;
      AdjustmentType: LoyaltyPointAdjustmentType;
      Points: number;
      ErrorIdentifier: string;
    }

  }

  /**
   * Get the available point policies
   */
  export interface GetLoyaltyPointPolicies {
  }

  export interface GetLoyaltyPointPoliciesResponse {
    PendingPolicies: string[];
    ExpirationPolicies: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Attach a loyalty program to an order
   */
  export interface AttachLoyaltyProgramToOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * When set, the UserID will also become required so we can verify the UserIdentifier.
     */
    UserIdentifier?: string;
  }

  /**
   * Get the available loyalty program payment methods for the given order.
   */
  export interface GetOrderLoyaltyProgramPaymentTypes {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetOrderLoyaltyProgramPaymentTypesResponse {
    Options: GetOrderLoyaltyProgramPaymentTypesResponse.Option[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetOrderLoyaltyProgramPaymentTypesResponse {
    export interface Option {
      /**
       * Entity type: LoyaltyProgramPaymentType
       */
      ID: number;
      BackendID?: string;
      /**
       * Entity type: PaymentType
       */
      PaymentTypeID: number;
      PaymentTypeName: string;
      Required: boolean;
      Sequence: number;
    }

  }

  /**
   * Set the optional loyalty program payment types for the order.
   */
  export interface SetOrderLoyaltyProgramPaymentTypes {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * When left empty no optional payment types will be available. Required payment types cannot be unselected.
     * Entity type: LoyaltyProgramPaymentType
     */
    SelectedLoyaltyProgramPaymentTypes: number[];
  }

  /**
   * Detach a loyalty program from an order
   */
  export interface DetachLoyaltyProgramFromOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Get the loyalty program user usage
   */
  export interface GetLoyaltyProgramUserUsage {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
  }

  export interface GetLoyaltyProgramUserUsageResponse {
    ProductUsages: GetLoyaltyProgramUserUsageResponse.ProductUsage[];
    PaymentUsages: GetLoyaltyProgramUserUsageResponse.PaymentUsage[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetLoyaltyProgramUserUsageResponse {
    export interface ProductUsage {
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductBackendID: string;
      ProductName: string;
      PrimaryImage: EVA.Core.IProductSearchItemPrimaryImage;
      ConsumedQuantity: number;
    }

    export interface PaymentUsage {
      /**
       * Entity type: LoyaltyProgramPaymentType
       */
      LoyaltyProgramPaymentTypeID: number;
      AmountUsage: number;
      ProductUsage: GetLoyaltyProgramUserUsageResponse.ProductUsage[];
    }

  }

  /**
   * List the loyalty program CustomFieldRequirements
   */
  export interface ListLoyaltyProgramRequiredCustomFields {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    InitialPageConfig?: EVA.Core.PageTokenConfig<{ [key:string]: string }>;
    PageToken?: string;
  }

  export interface ListLoyaltyProgramRequiredCustomFieldsResponse {
    Results?: ListLoyaltyProgramRequiredCustomFieldsResponse.CustomField[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListLoyaltyProgramRequiredCustomFieldsResponse {
    export interface CustomField {
      /**
       * Entity type: CustomField
       */
      ID: number;
      Name: string;
      BackendID: string;
    }

  }

  /**
   * Add a custom field requirement to the loyalty program. NOTE THAT a custom field can only be set once as a requirement.
   */
  export interface AddLoyaltyProgramRequiredCustomField {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: CustomField
     */
    CustomFieldID: number;
  }

  /**
   * Removes a custom field requirement from a loyalty program.
   */
  export interface RemoveLoyaltyProgramRequiredCustomField {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: CustomField
     */
    CustomFieldID: number;
  }

  /**
   * List the LoyaltyProgramGroups
   */
  export interface ListLoyaltyProgramGroups {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListLoyaltyProgramGroupFilter>;
    PageToken?: string;
  }

  export interface ListLoyaltyProgramGroupsResponse {
    Results?: ListLoyaltyProgramGroupsResponse.Item[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListLoyaltyProgramGroupsResponse {
    export interface Item {
      /**
       * Entity type: LoyaltyProgramGroup
       */
      ID?: number;
      BackendID?: string;
      Name?: string;
      TaxExemptionCode?: string;
      /**
       * Entity type: Currency
       */
      BudgetCurrencyID?: string;
      Status?: EVA.Core.LoyaltyProgramStatus;
      StartDate?: string;
      EndDate?: string;
      DeactivationReason?: EVA.Core.LoyaltyProgramGroupDeactivationReasons;
    }

  }

  /**
   * Get a LoyaltyProgramGroup
   */
  export interface GetLoyaltyProgramGroup {
    /**
     * Entity type: LoyaltyProgramGroup
     */
    ID: number;
  }

  export interface GetLoyaltyProgramGroupResponse {
    /**
     * Entity type: LoyaltyProgramGroup
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    StartDate?: string;
    EndDate?: string;
    Status?: EVA.Core.LoyaltyProgramStatus;
    TaxExemptionCode?: string;
    /**
     * Entity type: Currency
     */
    BudgetCurrencyID?: string;
    Budget?: number;
    UsedBudget?: number;
    BudgetUsagePercentage?: number;
    BudgetRemaining?: number;
    DeactivateWhenBudgetIsReached?: boolean;
    DeactivationReason?: EVA.Core.LoyaltyProgramGroupDeactivationReasons;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new LoyaltyProgramGroup
   */
  export interface CreateLoyaltyProgramGroup {
    BackendID?: string;
    Name?: string;
    TaxExemptionCode?: string;
    /**
     * Entity type: Currency
     */
    BudgetCurrencyID?: string;
    StartDate?: string;
    EndDate?: string;
    Status?: EVA.Core.LoyaltyProgramStatus;
  }

  export interface CreateLoyaltyProgramGroupResponse {
    /**
     * Entity type: LoyaltyProgramGroup
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing LoyaltyProgramGroup
   * 
   * ATTENTION!
   * When the currency is changed, the budget usages will be reset to 0!
   * 
   * ATTENTION!
   * When a change to Status, StartDate, EndDate or Currency leads to the group being deactivated, all related loyalty programs will also be deactivated.
   * When a change to Status, StartDate, EndDate or Currency leads to the group being reactivate, all related loyalty programs will also be reactive when the option ReactivateProgramsWhenPossible is set to true. Otherwise all deactivated programs need to be manually reactivated.
   */
  export interface UpdateLoyaltyProgramGroup {
    /**
     * Entity type: LoyaltyProgramGroup
     */
    ID: number;
    BackendID?: string;
    Name?: string;
    TaxExemptionCode?: string;
    /**
     * Entity type: Currency
     */
    BudgetCurrencyID?: string;
    Status?: EVA.Core.LoyaltyProgramStatus;
    StartDate?: string;
    EndDate?: string;
    ReactivateProgramsWhenPossible?: boolean;
  }

  /**
   * Update the budget on the loyalty program group.
   * ATTENTION!
   * When the budget is lowered below the current used budget and the `DeactivateWhenBudgetIsReached` is set to true, all related programs will be deactivated.
   * When the budget is cleared or raised above the current used budget, or `DeactivateWhenBudgetIsReached` is set to false, and the `ReactivateProgramsWhenPossible` is set to true, all programs that were deactivate by the group budget will be reactivated when possible.
   */
  export interface UpdateLoyaltyProgramGroupBudget {
    /**
     * Entity type: LoyaltyProgramGroup
     */
    ID: number;
    /**
     * Min: 0
     */
    Budget?: number;
    DeactivateWhenBudgetIsReached?: boolean;
    ReactivateProgramsWhenPossible?: boolean;
  }

  /**
   * Delete the LoyaltyProgramGroup
   */
  export interface DeleteLoyaltyProgramGroup {
    /**
     * Entity type: LoyaltyProgramGroup
     */
    ID: number;
    /**
     * When set to true, all related programs are deleted. When false, the request will fail when there are related programs active.
     */
    Force?: boolean;
  }

  /**
   * List the LoyaltyProgramPaymentType
   */
  export interface ListLoyaltyProgramPaymentTypes {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    InitialPageConfig?: EVA.Core.PageTokenConfig<{ [key:string]: string }>;
    PageToken?: string;
  }

  export interface ListLoyaltyProgramPaymentTypesResponse {
    Results?: ListLoyaltyProgramPaymentTypesResponse.Item[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListLoyaltyProgramPaymentTypesResponse {
    export interface Item {
      /**
       * Entity type: LoyaltyProgramPaymentType
       */
      ID: number;
      BackendID?: string;
      Sequence: number;
      Required: boolean;
      /**
       * Entity type: PaymentType
       */
      PaymentTypeID: number;
      PaymentTypeName: string;
      ExcludeFromProgramBudget: boolean;
      Budget?: number;
      HasUserLimitations: boolean;
      AllowPartialPayment: boolean;
    }

  }

  /**
   * Get a LoyaltyProgramPaymentType
   */
  export interface GetLoyaltyProgramPaymentType {
    /**
     * Entity type: LoyaltyProgramPaymentType
     */
    ID: number;
  }

  export interface GetLoyaltyProgramPaymentTypeResponse {
    /**
     * Entity type: LoyaltyProgramPaymentType
     */
    ID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    BackendID?: string;
    Sequence?: number;
    Required?: boolean;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    PaymentTypeName?: string;
    TaxHandling?: EVA.Core.LoyaltyPaymentTaxHandlingType;
    ExcludeFromProgramBudget?: boolean;
    Budget?: number;
    UsedBudget?: number;
    BudgetUsagePercentage?: number;
    BudgetRemaining?: number;
    UserMaximumUsage?: number;
    UserBudget?: number;
    UserBudgetDeduction?: EVA.Core.UserBudgetDeduction;
    MaximumAmountPerOrder?: number;
    MaximumPercentageOfUserBudgetPerOrder?: number;
    ProductLimitation?: LoyaltyProgramProductLimitationData;
    AllowPartialPayment?: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface LoyaltyProgramProductLimitationData {
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchID?: number;
    Filters?: { [key: string]: EVA.Core.FilterModel };
    /**
     * The quantity limit over the entire product limitation product set per user.
     * Min: 1
     */
    OverallQuantityLimitPerUser?: number;
    /**
     * The quantity limit over the entire product limitation product set per order.
     * Min: 1
     */
    OverallQuantityLimitPerOrder?: number;
    /**
     * The quantity limit per product within the product limitation product set per user.
     * Min: 1
     */
    QuantityLimitPerUser?: number;
    /**
     * The quantity limit per product within the product limitation product set per order.
     * Min: 1
     */
    QuantityLimitPerOrder?: number;
  }

  /**
   * Create a new LoyaltyProgramPaymentType
   */
  export interface CreateLoyaltyProgramPaymentType {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    BackendID?: string;
    /**
     * Min: 0
     */
    Sequence?: number;
    AllowPartialPayment?: boolean;
    Required?: boolean;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    TaxHandling?: EVA.Core.LoyaltyPaymentTaxHandlingType;
    ExcludeFromProgramBudget?: boolean;
    /**
     * Min: 0
     */
    Budget?: number;
    /**
     * Min: 1
     */
    UserMaximumUsage?: number;
    /**
     * Min: 0
     */
    UserBudget?: number;
    UserBudgetDeduction?: EVA.Core.UserBudgetDeduction;
    /**
     * Min: 0
     */
    MaximumAmountPerOrder?: number;
    /**
     * Range: 0-100
     */
    MaximumPercentageOfUserBudgetPerOrder?: number;
    ProductLimitation?: LoyaltyProgramProductLimitationData;
  }

  export interface CreateLoyaltyProgramPaymentTypeResponse {
    /**
     * Entity type: LoyaltyProgramPaymentType
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing LoyaltyProgramPaymentType
   */
  export interface UpdateLoyaltyProgramPaymentType {
    /**
     * Entity type: LoyaltyProgramPaymentType
     */
    ID?: number;
    BackendID?: string;
    /**
     * Min: 0
     */
    Sequence?: number;
    AllowPartialPayment?: boolean;
    Required?: boolean;
    TaxHandling?: EVA.Core.LoyaltyPaymentTaxHandlingType;
    ExcludeFromProgramBudget?: boolean;
    /**
     * Min: 0
     */
    Budget?: number;
    /**
     * Min: 1
     */
    UserMaximumUsage?: number;
    /**
     * Min: 0
     */
    UserBudget?: number;
    UserBudgetDeduction?: EVA.Core.UserBudgetDeduction;
    /**
     * Min: 0
     */
    MaximumAmountPerOrder?: number;
    /**
     * Range: 0-100
     */
    MaximumPercentageOfUserBudgetPerOrder?: number;
    ProductLimitation?: LoyaltyProgramProductLimitationData;
  }

  /**
   * Delete a LoyaltyProgramPaymentType
   */
  export interface DeleteLoyaltyProgramPaymentType {
    /**
     * Entity type: LoyaltyProgramPaymentType
     */
    ID: number;
  }

  /**
   * Lists all payment types under the Loyalty payment method that aren't already connected to a loyalty program.
   */
  export interface ListLoyaltyPaymentMethodTypes {
  }

  export interface ListLoyaltyPaymentMethodTypesResponse {
    Types: ListLoyaltyPaymentMethodTypesResponse.LoyaltyPaymentType[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListLoyaltyPaymentMethodTypesResponse {
    export interface LoyaltyPaymentType {
      /**
       * Entity type: PaymentType
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * List the loyalty program pricelists
   */
  export interface ListLoyaltyProgramPriceLists {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    InitialPageConfig?: EVA.Core.PageTokenConfig<{ [key:string]: string }>;
    PageToken?: string;
  }

  export interface ListLoyaltyProgramPriceListsResponse {
    Results?: ListLoyaltyProgramPriceListsResponse.Item[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListLoyaltyProgramPriceListsResponse {
    export interface Item {
      /**
       * Entity type: PriceList
       */
      PriceListID: number;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      Name: string;
    }

  }

  /**
   * Add a price list to a loyalty program. NOTE THAT you can only add one pricelist per currency to a loyalty program.
   */
  export interface AddLoyaltyProgramPriceList {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: PriceList
     */
    PriceListID: number;
  }

  /**
   * Removes a price list from a loyalty program.
   */
  export interface RemoveLoyaltyProgramPriceList {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: PriceList
     */
    PriceListID: number;
  }

  /**
   * List the loyalty program product limitations
   */
  export interface ListLoyaltyProgramProductLimitations {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    InitialPageConfig?: EVA.Core.PageTokenConfig<{ [key:string]: string }>;
    PageToken?: string;
  }

  export interface ListLoyaltyProgramProductLimitationsResponse {
    Results?: ListLoyaltyProgramProductLimitationsResponse.Item[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListLoyaltyProgramProductLimitationsResponse {
    export interface Item {
      /**
       * Entity type: LoyaltyProgramProductLimitation
       */
      ID: number;
      BackendID?: string;
      Name: string;
      /**
       * The quantity limit over the entire product limitation product set per user.
       * Min: 1
       */
      OverallQuantityLimitPerUser?: number;
      /**
       * The quantity limit over the entire product limitation product set per order.
       * Min: 1
       */
      OverallQuantityLimitPerOrder?: number;
      /**
       * The quantity limit per product within the product limitation product set per user.
       * Min: 1
       */
      QuantityLimitPerUser?: number;
      /**
       * The quantity limit per product within the product limitation product set per order.
       * Min: 1
       */
      QuantityLimitPerOrder?: number;
    }

  }

  /**
   * Get a loyalty program product limitations
   */
  export interface GetLoyaltyProgramProductLimitation {
    /**
     * Entity type: LoyaltyProgramPaymentType
     */
    ID: number;
  }

  export interface GetLoyaltyProgramProductLimitationResponse {
    /**
     * Entity type: LoyaltyProgramProductLimitation
     */
    ID: number;
    BackendID?: string;
    Name: string;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    ApplyToPriceList: boolean;
    ProductLimitation: LoyaltyProgramProductLimitationData;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a loyalty program product limitation.
   */
  export interface CreateLoyaltyProgramProductLimitation {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    BackendID?: string;
    Name: string;
    ApplyToPriceList?: boolean;
    ProductLimitation?: LoyaltyProgramProductLimitationData;
  }

  export interface CreateLoyaltyProgramProductLimitationResponse {
    /**
     * Entity type: LoyaltyProgramProductLimitation
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a loyalty program product limitation.
   */
  export interface UpdateLoyaltyProgramProductLimitation {
    /**
     * Entity type: LoyaltyProgramProductLimitation
     */
    ID: number;
    BackendID?: string;
    Name: string;
    ApplyToPriceList?: boolean;
    ProductLimitation?: LoyaltyProgramProductLimitationData;
  }

  /**
   * Delete a loyalty program product limitation.
   */
  export interface DeleteLoyaltyProgramProductLimitation {
    /**
     * Entity type: LoyaltyProgramProductLimitation
     */
    ID: number;
  }

  /**
   * List loyalty point rules. Returns a paged response.
   */
  export interface ListPointAwardingRules {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListPointAwardingRuleFilter>;
    PageToken?: string;
  }

  export interface ListPointAwardingRulesResponse {
    Results?: ListPointAwardingRulesResponse.Rule[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListPointAwardingRulesResponse {
    export interface Rule {
      /**
       * Entity type: PointAwardingRule
       */
      ID: number;
      /**
       * Entity type: LoyaltyProgram
       */
      LoyaltyProgramID: number;
      LoyaltyProgramName: string;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      OrganizationUnitSetName: string;
      Name: string;
      Description?: string;
      Enabled: boolean;
      StartDate: string;
      EndDate?: string;
    }

  }

  /**
   * Get loyalty point rule.
   */
  export interface GetPointAwardingRule {
    /**
     * Entity type: PointAwardingRule
     */
    ID: number;
  }

  export interface GetPointAwardingRuleResponse {
    /**
     * Entity type: PointAwardingRule
     */
    ID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    LoyaltyProgramName?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    OrganizationUnitSetName?: string;
    Name?: string;
    Description?: string;
    MarketingDescription?: string;
    BackendID?: string;
    BackendSystemID?: string;
    Enabled?: boolean;
    StartDate?: string;
    EndDate?: string;
    ApplyRecursive?: boolean;
    PointsToAward?: number;
    PointPendingPolicyType?: string;
    PointPendingPolicyData?: { [key: string]: any };
    PointExpirationPolicyType?: string;
    PointExpirationPolicyData?: { [key: string]: any };
    Conditions?: GetPointAwardingRuleResponse.Condition[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetPointAwardingRuleResponse {
    export interface Condition {
      /**
       * Entity type: PointAwardingCondition
       */
      ID: number;
      Type: string;
      Data?: { [key: string]: any };
    }

  }

  /**
   * Create a point awarding rule.
   */
  export interface CreatePointAwardingRule {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Name?: string;
    Description?: string;
    MarketingDescription?: string;
    BackendID?: string;
    Enabled?: boolean;
    StartDate?: string;
    EndDate?: string;
    ApplyRecursive?: boolean;
    PointsToAward?: number;
    PointPendingPolicyType?: string;
    PointPendingPolicyData?: { [key: string]: any };
    PointExpirationPolicyType?: string;
    PointExpirationPolicyData?: { [key: string]: any };
    Conditions?: CreatePointAwardingRule.Condition[];
  }

  export namespace CreatePointAwardingRule {
    export interface Condition {
      BackendID?: string;
      Type: string;
      Data?: { [key: string]: any };
    }

  }

  export interface CreatePointAwardingRuleResponse {
    /**
     * Entity type: PointAwardingRule
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a point awarding rule
   */
  export interface UpdatePointAwardingRule {
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: PointAwardingRule
     */
    ID: number;
    Name?: string;
    Description?: string;
    MarketingDescription?: string;
    BackendID?: string;
    Enabled?: boolean;
    StartDate?: string;
    EndDate?: string;
    ApplyRecursive?: boolean;
    PointsToAward?: number;
    PointPendingPolicyType?: string;
    PointPendingPolicyData?: { [key: string]: any };
    PointExpirationPolicyType?: string;
    PointExpirationPolicyData?: { [key: string]: any };
    Conditions?: UpdatePointAwardingRule.Condition[];
  }

  export namespace UpdatePointAwardingRule {
    export interface Condition {
      /**
       * Entity type: PointAwardingCondition
       */
      ID?: number;
      BackendID?: string;
      Type: string;
      Data?: { [key: string]: any };
    }

  }

  /**
   * Enable a point awarding rule
   */
  export interface EnablePointAwardingRule {
    /**
     * Entity type: PointAwardingRule
     */
    ID: number;
  }

  /**
   * Disable a point awarding rule
   */
  export interface DisablePointAwardingRule {
    /**
     * Entity type: PointAwardingRule
     */
    ID: number;
  }

  /**
   * Archive a point awarding rule
   */
  export interface ArchivePointAwardingRule {
    /**
     * Entity type: PointAwardingRule
     */
    ID: number;
  }

  /**
   * Push a LoyaltyProgramGroup.
   * If it doesn't exist it will be created, otherwise is will be updated.
   * 
   * ATTENTION!
   * When the currency is changed, the budget usages will be reset to 0!
   * 
   * ATTENTION!
   * When a change to Status, StartDate, EndDate or Currency leads to the group being deactivated, all related loyalty programs will also be deactivated.
   * When a change to Status, StartDate, EndDate or Currency leads to the group being reactivated, all related loyalty programs will also be reactivated when the option ReactivateProgramsWhenPossible is set to true. Otherwise all deactivated programs need to be manually reactivated.
   * 
   * ATTENTION!
   * When the budget is lowered below the current used budget and the `DeactivateWhenBudgetIsReached` is set to true, all related programs will be deactivated.
   * When the budget is cleared or raised above the current used budget, or `DeactivateWhenBudgetIsReached` is set to false, and the `ReactivateProgramsWhenPossible` is set to true, all programs that were deactivate by the group budget will be reactivated when possible.
   */
  export interface PushLoyaltyProgramGroup {
    ID: string;
    Name: string;
    TaxExemptionCode?: string;
    /**
     * Entity type: Currency
     */
    BudgetCurrencyID?: string;
    /**
     * Min: 0
     */
    Budget?: number;
    DeactivateWhenBudgetIsReached?: boolean;
    StartDate?: string;
    EndDate?: string;
    Status: EVA.Core.LoyaltyProgramStatus;
  }

  export interface PushLoyaltyProgramGroupResponse {
    /**
     * Entity type: LoyaltyProgram
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get the predefined LedgerClasses, in most services it is allowed to use other classes.
   */
  export interface GetLedgerClasses {
  }

  export interface GetLedgerClassesResponse {
    LedgerClasses: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the inquiries
   */
  export interface ListInquiries {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListInquiriesFilter>;
    PageToken?: string;
  }

  export interface ListInquiriesResponse {
    Results?: InquiryData[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface InquiryData {
    /**
     * Entity type: Inquiry
     */
    ID: number;
    Name: string;
    Description?: string;
  }

  /**
   * Creates a new inquiry
   */
  export interface CreateInquiry {
    Name: string;
    Description?: string;
    PromptUser?: boolean;
    OneTimeUseOnly?: boolean;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Items?: CreateInquiry.ItemData[];
  }

  export namespace CreateInquiry {
    export interface ItemData {
      Name: string;
      Description?: string;
      IsRequired: boolean;
      /**
       * Min: 0
       */
      Sequence: number;
      /**
       * Entity type: CustomField
       */
      CustomFieldID: number;
    }

  }

  export interface CreateInquiryResponse {
    /**
     * Entity type: Inquiry
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an inquiry
   */
  export interface UpdateInquiry {
    /**
     * Entity type: Inquiry
     */
    ID: number;
    Name?: string;
    Description?: string;
    PromptUser?: boolean;
    OneTimeUseOnly?: boolean;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
  }

  /**
   * Delete an inquiry
   */
  export interface DeleteInquiry {
    /**
     * Entity type: Inquiry
     */
    ID: number;
  }

  /**
   * Creates an inquiry item
   */
  export interface CreateInquiryItem {
    /**
     * Entity type: Inquiry
     */
    InquiryID: number;
    Name: string;
    Description?: string;
    IsRequired?: boolean;
    /**
     * Min: 0
     */
    Sequence: number;
    /**
     * Entity type: CustomField
     */
    CustomFieldID: number;
  }

  export interface CreateInquiryItemResponse {
    /**
     * Entity type: InquiryItem
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates an inquiry item
   */
  export interface UpdateInquiryItem {
    /**
     * Entity type: InquiryItem
     */
    ID: number;
    Name?: string;
    Description?: string;
    IsRequired?: boolean;
    /**
     * Min: 0
     */
    Sequence?: number;
  }

  /**
   * Deletes an inquiry item
   */
  export interface DeleteInquiryItem {
    /**
     * Entity type: InquiryItem
     */
    ID: number;
  }

  /**
   * Create a new device
   */
  export interface CreateDevice {
    Name?: string;
    BackendID?: string;
    /**
     * Entity type: DeviceType
     */
    TypeID?: number;
    Address?: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    AssemblyName?: string;
    Data?: { [key: string]: any };
    /**
     * Entity type: PrinterType
     */
    PrinterTypeID?: number;
  }

  export interface CreateDeviceResponse {
    /**
     * Entity type: Device
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates a device
   */
  export interface UpdateDevice {
    /**
     * Entity type: Device
     */
    ID: number;
    Name?: string;
    BackendID?: string;
    /**
     * Entity type: DeviceType
     */
    TypeID?: number;
    Address?: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: PrinterType
     */
    PrinterTypeID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    AssemblyName?: string;
    Data?: { [key: string]: any };
  }

  /**
   * Delete a device
   */
  export interface DeleteDevice {
    /**
     * Entity type: Device
     */
    ID: number;
  }

  /**
   * Get the configuration required to set up a CustomerFacingDisplay.
   */
  export interface GetCFDConfiguration {
    GlobalID: string;
  }

  export interface GetCFDConfigurationResponse {
    /**
     * Entity type: Device
     */
    ID: number;
    GlobalID: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    CloudEndpoint: string;
    LiveGuardEndpoint?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get a qrcode to configure/enroll a device when available.
   * 
   * At this moment the supported DeviceTypes are:
   *  - CustomerDisplay
   */
  export interface GetConfigurationQrForDevice {
    /**
     * Entity type: Device
     */
    DeviceID: number;
  }

  export interface GetConfigurationQrForDeviceResponse {
    QrData?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new OpenCashDrawerReason. Name and Description can both be localized through the LocalizeOpenCashDrawerReason service later.
   */
  export interface CreateOpenCashDrawerReason {
    Name: string;
    Description?: string;
  }

  export interface CreateOpenCashDrawerReasonResponse {
    /**
     * Entity type: OpenCashDrawerReason
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates an existing OpenCashDrawerReason with a new Name and Description.
   */
  export interface UpdateOpenCashDrawerReason {
    /**
     * Entity type: OpenCashDrawerReason
     */
    ID: number;
    Name: string;
    Description?: string;
  }

  /**
   * Deletes an OpenCashDrawerReason
   */
  export interface DeleteOpenCashDrawerReason {
    /**
     * Entity type: OpenCashDrawerReason
     */
    ID: number;
  }

  /**
   * Creates a localized Name and Description for the OpenCashDrawerReason.
   * Language is required, Country is optional.
   */
  export interface LocalizeOpenCashDrawerReason {
    /**
     * Entity type: OpenCashDrawerReason
     */
    ID: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Name?: string;
    Description?: string;
  }

  /**
   * Print a qrcode on a thermal receipt.
   * 
   * It's required to select a Station, this can be done by providing the `EVA-StationID` header, or by setting the StationID property.
   * 
   * Also the MessageTemplate `QRCodePrintingTemplate` can be configured in Stencil.
   */
  export interface PrintQRCode {
    /**
     * Only required when not providing the Station in the header
     * Entity type: Station
     */
    StationID?: number;
    QRData: string;
  }

  /**
   * Create a new station for an OrganizationUnit.
   */
  export interface CreateStation {
    BackendID?: string;
    Name?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Special identifier for fiscalized stations, some certifications require this
     */
    FiscalSystemID?: string;
    /**
     * Used to differentiate between fiscalized stations and other (e.g. safes) devices
     */
    IsPointOfSale?: boolean;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  export namespace CreateStation {
    export interface StationToCreate {
      BackendID?: string;
      Name: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      /**
       * Special identifier for fiscalized stations, some certifications require this
       */
      FiscalSystemID?: string;
      /**
       * Used to differentiate between fiscalized stations and other (e.g. safes) devices
       */
      IsPointOfSale?: boolean;
      /**
       * Optional additional custom fields
       */
      CustomFields?: EVA.Core.CustomFieldKeyValue[];
    }

  }

  export interface CreateStationResponse {
    /**
     * Entity type: Station
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates a station.
   */
  export interface UpdateStation {
    /**
     * Entity type: Station
     */
    ID: number;
    Name?: string;
    BackendID?: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  /**
   * Delete a station
   */
  export interface DeleteStation {
    /**
     * Entity type: Station
     */
    ID: number;
  }

  export interface DeleteStationResponse {
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns available handlers for gift card configurations
   */
  export interface GetGiftCardHandlers {
  }

  export interface GetGiftCardHandlersResponse {
    Handlers: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the GiftCardConfigurations
   * 
   * `GiftCardConfiguration`: an item that describes the giftcard properties of a product with type GiftCard
   */
  export interface ListGiftCardConfigurations {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListGiftCardConfigurationsFilter>;
    PageToken?: string;
  }

  export interface ListGiftCardConfigurationsResponse {
    Results?: ListGiftCardConfigurationsResponse.GiftCardConfigurationDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListGiftCardConfigurationsResponse {
    export interface GiftCardConfigurationDto {
      /**
       * Entity type: GiftCardConfiguration
       */
      ID: number;
      Name: string;
      Handler: string;
      CardType?: string;
      UsageType: EVA.Core.GiftCardConfigurationUsageTypes;
      AllowRefundWithoutTransaction: boolean;
      AllowCurrencyConversion: boolean;
    }

  }

  /**
   * Get a single GiftCardConfiguration
   */
  export interface GetGiftCardConfiguration {
    /**
     * Entity type: GiftCardConfiguration
     */
    ID: number;
  }

  export interface GetGiftCardConfigurationResponse {
    Name: string;
    UsageType: EVA.Core.GiftCardConfigurationUsageTypes;
    Handler: string;
    /**
     * Some gift card handlers are able to handle multiple types of cards. With this property you can set the card the handler should work with.
     */
    CardType?: string;
    Data?: any;
    AllowRefundWithoutTransaction: boolean;
    AllowCurrencyConversion: boolean;
    AllowMultipleCardsPayment?: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new GiftCardConfiguration
   */
  export interface CreateGiftCardConfiguration {
    Name: string;
    UsageType: EVA.Core.GiftCardConfigurationUsageTypes;
    Handler: string;
    /**
     * Some gift card handlers are able to handle multiple types of cards. With this property you can set the card the handler should work with.
     */
    CardType?: string;
    Data?: any;
    AllowRefundWithoutTransaction: boolean;
    AllowCurrencyConversion: boolean;
    AllowMultipleCardsPayment?: boolean;
  }

  export interface CreateGiftCardConfigurationResponse {
    /**
     * Entity type: GiftCardConfiguration
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing GiftCardConfiguration
   */
  export interface UpdateGiftCardConfiguration {
    /**
     * Entity type: GiftCardConfiguration
     */
    ID: number;
    Name?: string;
    UsageType?: EVA.Core.GiftCardConfigurationUsageTypes;
    Handler?: string;
    /**
     * Some gift card handlers are able to handle multiple types of cards. With this property you can set the card the handler should work with.
     */
    CardType?: string;
    Data?: any;
    AllowRefundWithoutTransaction?: boolean;
    AllowCurrencyConversion?: boolean;
    AllowMultipleCardsPayment?: boolean;
  }

  /**
   * Delete a GiftCardConfiguration
   */
  export interface DeleteGiftCardConfiguration {
    /**
     * Entity type: GiftCardConfiguration
     */
    ID: number;
  }

  /**
   * List the ProductGiftCards
   * 
   * `ProductGiftCard`: an item that describes the giftcard properties of a product with type GiftCard
   */
  export interface ListProductGiftCards {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListProductGiftCardsFilter>;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
  }

  export interface ListProductGiftCardsResponse {
    Result?: EVA.Core.PagedResult<ListProductGiftCardsResponse.ProductGiftCardDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListProductGiftCardsResponse {
    export interface ProductGiftCardDto {
      /**
       * Entity type: ProductGiftCard
       */
      ID?: number;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      ProductName?: string;
      Product?: any;
      /**
       * Entity type: GiftCardConfiguration
       */
      GiftCardConfigurationID?: number;
      GiftCardConfigurationName?: string;
      /**
       * Communication options for digital giftcards
       */
      CommunicationOptions?: EVA.Core.ProductGiftCardCommunicationOptions;
      IsDigital?: boolean;
    }

  }

  /**
   * Create a new ProductGiftCard
   */
  export interface CreateProductGiftCard {
    /**
     * Entity type: GiftCardConfiguration
     */
    GiftCardConfigurationID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    IsDigital?: boolean;
    /**
     * Communication options for digital giftcards
     */
    CommunicationOptions?: EVA.Core.ProductGiftCardCommunicationOptions;
    /**
     * @default true
     */
    IsReturnable?: boolean;
  }

  export interface CreateProductGiftCardResponse {
    /**
     * Entity type: ProductGiftCard
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing ProductGiftCard
   */
  export interface UpdateProductGiftCard {
    /**
     * Entity type: ProductGiftCard
     */
    ID: number;
    /**
     * Entity type: GiftCardConfiguration
     */
    GiftCardConfigurationID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    IsDigital?: boolean;
    /**
     * Communication options for digital giftcards
     */
    CommunicationOptions?: EVA.Core.ProductGiftCardCommunicationOptions;
    IsReturnable?: boolean;
  }

  /**
   * Delete a ProductGiftCard
   */
  export interface DeleteProductGiftCard {
    /**
     * Entity type: ProductGiftCard
     */
    ID: number;
  }

  /**
   * List the ExchangeRates
   */
  export interface ListExchangeRates {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListExchangeRatesFilter>;
  }

  export interface ListExchangeRatesResponse {
    Result?: EVA.Core.PagedResult<ListExchangeRatesResponse.ExchangeRateDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListExchangeRatesResponse {
    export interface ExchangeRateDto {
      /**
       * Entity type: ExchangeRate
       */
      ID: number;
      /**
       * Entity type: Currency
       */
      BaseCurrencyID: string;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      ExchangeRate: number;
    }

  }

  /**
   * Returns a cash expense type by its ID.
   */
  export interface GetCashExpenseTypeByID {
    /**
     * Entity type: CashExpenseType
     */
    ID: number;
  }

  export interface GetCashExpenseTypeByIDResponse {
    /**
     * Entity type: CashExpenseType
     */
    ID: number;
    BackendID?: string;
    Name: string;
    Description?: string;
    LedgerClassID?: string;
    AmountType: EVA.Core.CashExpenseAmountTypes;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
    TaxCodeName?: string;
    OrganizationUnitSet?: CashExpenseTypeOrganizationUnitSet;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface CashExpenseTypeOrganizationUnitSet {
    /**
     * Entity type: OrganizationUnitSet
     */
    ID: number;
    Name: string;
  }

  /**
   * Creates a new cash expense type.
   * It's not possible to create a cash expense type with a name that is already in use.
   * 
   * The amount type specifies whether or not this type can be used to also register positive 'expenses' (so: income).
   * An example of this might be tips left by a customer.
   */
  export interface CreateCashExpenseType {
    BackendID?: string;
    Name: string;
    Description?: string;
    LedgerClassID?: string;
    AmountType: EVA.Core.CashExpenseAmountTypes;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export interface CreateCashExpenseTypeResponse {
    /**
     * Entity type: CashExpenseType
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates a cash expense type. All properties except for its Name can be updated.
   */
  export interface UpdateCashExpenseType {
    /**
     * Entity type: CashExpenseType
     */
    ID: number;
    Name?: string;
    Description?: string;
    LedgerClassID?: string;
    BackendID?: string;
    AmountType: EVA.Core.CashExpenseAmountTypes;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
  }

  /**
   * Deletes a cash expense type.
   */
  export interface DeleteCashExpenseType {
    /**
     * Entity type: CashExpenseType
     */
    ID: number;
  }

  /**
   * Returns a paged result of all the cash expsense types.
   */
  export interface ListCashExpenseTypes {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListCashExpenseTypesFilter>;
    PageToken?: string;
  }

  export interface ListCashExpenseTypesResponse {
    Results?: ListCashExpenseTypesResponse.Model[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCashExpenseTypesResponse {
    export interface Model {
      /**
       * Entity type: CashExpenseType
       */
      ID?: number;
      BackendID?: string;
      Name?: string;
      Description?: string;
      LedgerClassID?: string;
      AmountType?: EVA.Core.CashExpenseAmountTypes;
      /**
       * Entity type: TaxCode
       */
      TaxCodeID?: number;
      TaxCodeName?: string;
      OrganizationUnitSet?: CashExpenseTypeOrganizationUnitSet;
    }

  }

  /**
   * Returns a paged result of all the cash correction reasons.
   */
  export interface ListCashCorrectionReasons {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListCashCorrectionReasonsFilter>;
    PageToken?: string;
  }

  export interface ListCashCorrectionReasonsResponse {
    Results?: ListCashCorrectionReasonsResponse.Model[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCashCorrectionReasonsResponse {
    export interface Model {
      /**
       * Entity type: CashCorrectionReason
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Sequence: number;
    }

  }

  /**
   * -
   */
  export interface CorrectProductCostPriceLedgers {
    Corrections: CorrectProductCostPriceLedgers.Correction[];
  }

  export namespace CorrectProductCostPriceLedgers {
    export interface Correction {
      /**
       * Entity type: ProductCostPriceLedger
       */
      ID: number;
      UnitCost: number;
    }

  }

  /**
   * You can only export up to 100 products at a time.
   */
  export interface ExportProductCostPriceLedgerSummary {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetIDs?: number[];
    /**
     * Filters to select which products to export. Will be ignored if `ProductIDs` is set. Required if `ProductIDs` is not set.
     */
    ProductSearchFilters?: { [key: string]: EVA.Core.FilterModel };
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    EmailAddress?: string;
    SplitUnitCost?: boolean;
  }

  /**
   * This service lets you view the ProductCostPriceLedger, optionally filtered by ProductID/OrganizationUnitID and others.
   * 
   * The ProductCostPriceLedger is used to determine the cost prices of stock for OrganizationUnits that use CostPriceCalculationMethod = Fifo.
   * 
   * For those OrganizationUnits, whenever stock is received in them, a record is stored in the ProductCostPriceLedger noting the quantity and the purchase price of that stock and then whenever
   * stock is reduced this ledger is consulted in a first-in-first-out manner to determine the cost price of that stock mutation. This service gives you insight into that ledger.
   */
  export interface ListProductCostPriceLedger {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListProductCostPriceLedgerFilter>;
    PageToken?: string;
  }

  export interface ListProductCostPriceLedgerResponse {
    Results?: EVA.Core.ListProductCostPriceLedgerItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface ListStockMutationUnitCostSourceLedger {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListStockMutationUnitCostSourceLedgerFilter>;
    PageToken?: string;
  }

  export interface ListStockMutationUnitCostSourceLedgerResponse {
    Results?: EVA.Core.ListStockMutationUnitCostSourceLedgerItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new entity field validator
   */
  export interface CreateEntityFieldValidator {
    Entity: EVA.Core.EntityFieldValidatorEntityTypes;
    EntityFieldName: string;
    ValidatorType: EVA.Core.EntityFieldValidatorType;
    ApplyFor: EVA.Core.EntityFieldValidatorUserTypes;
    /**
     * This must be of object that inherits the class EntityFieldValidatorBase. For example StringEntityFieldValidator or DefaultEntityFieldValidator
     */
    ValidatorData: { [key: string]: any };
  }

  export interface CreateEntityFieldValidatorResponse {
    /**
     * Entity type: EntityFieldValidator
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get a entity field validator
   */
  export interface GetEntityFieldValidator {
    /**
     * Entity type: EntityFieldValidator
     */
    ID: number;
  }

  export interface GetEntityFieldValidatorResponse {
    ID: number;
    Entity: EVA.Core.EntityFieldValidatorEntityTypes;
    ValidatorType: EVA.Core.EntityFieldValidatorType;
    ApplyFor: EVA.Core.EntityFieldValidatorUserTypes;
    EntityFieldName: string;
    ValidatorData: { [key: string]: any };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a entity field validator
   */
  export interface UpdateEntityFieldValidator {
    /**
     * Entity type: EntityFieldValidator
     */
    ID: number;
    ApplyFor?: EVA.Core.EntityFieldValidatorUserTypes;
    ValidatorData?: { [key: string]: any };
  }

  /**
   * Delete a entity field validator
   */
  export interface DeleteEntityFieldValidator {
    /**
     * Entity type: EntityFieldValidator
     */
    ID: number;
  }

  /**
   * list all entity field validators
   */
  export interface ListEntityFieldValidators {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListEntityFieldValidatorsFilter>;
    PageToken?: string;
  }

  export interface ListEntityFieldValidatorsResponse {
    Results?: ListEntityFieldValidatorsResponse.ListEntityFieldValidatorItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListEntityFieldValidatorsResponse {
    export interface ListEntityFieldValidatorItem {
      /**
       * Entity type: EntityFieldValidator
       */
      ID: number;
      Entity: EVA.Core.EntityFieldValidatorEntityTypes;
      ValidatorType: EVA.Core.EntityFieldValidatorType;
      ApplyFor: EVA.Core.EntityFieldValidatorUserTypes;
      EntityFieldName: string;
      ValidatorData: { [key: string]: any };
    }

  }

  /**
   * Get all entity field validators for given entity
   */
  export interface GetFieldValidatorsForEntity {
    EntityType: EVA.Core.EntityFieldValidatorEntityTypes;
  }

  export interface GetFieldValidatorsForEntityResponse {
    Validators?: GetFieldValidatorsForEntityResponse.Validator[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetFieldValidatorsForEntityResponse {
    export interface Validator {
      ValidatorType: EVA.Core.EntityFieldValidatorType;
      EntityFieldName: string;
      ValidatorData: { [key: string]: any };
    }

  }

  /**
   * List all entity field validator entities and entity names
   */
  export interface ListEntityFieldValidatorEntityFieldNames {
  }

  export interface ListEntityFieldValidatorEntityFieldNamesResponse {
    EntityFieldValidatorEntityFields: ListEntityFieldValidatorEntityFieldNamesResponse.EntityFieldValidatorEntityField[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListEntityFieldValidatorEntityFieldNamesResponse {
    export interface EntityFieldValidatorEntityField {
      Entity: EVA.Core.EntityFieldValidatorEntityTypes;
      EntityFieldName: string;
      Type: string;
    }

  }

  /**
   * Create endpoint configurations.
   * 
   * Endpoint Configurations allow you to configure specific endpoints taht can be used for example for EventExportConfigurations.
   * 
   * - Make sure to catch & store the `SecretKey1` and `SecretKey2`, as there will be no other way of retrieving them after this service call!
   */
  export interface CreateEndpointConfiguration {
    /**
     * A name to identify the configuration.
     */
    Name: string;
    /**
     * An identifier for the configuration.
     */
    BackendID?: string;
    /**
     * The status of the configuration.
     */
    Status: EVA.Core.EndpointConfigurationStatus;
    /**
     * The endpoint to send the events to. This must be a valid HTTPS endpoint.
     */
    Endpoint: string;
    /**
     * Authorization configuration used when calling this endpoint.
     */
    Authorization?: EndpointConfigurationAuthorizationModel;
    /**
     * Timeout after which requests are cancelled. Defaults to 5 seconds. Not recommended to change.
     */
    TimeoutSeconds?: number;
    /**
     * If true, the endpoint will use string IDs instead of long IDs.
     */
    UseStringIDs?: boolean;
    /**
     * If false, the endpoint will also send `null` values. Preferred is true.
     */
    IgnoreNullValues?: boolean;
  }

  export interface EndpointConfigurationAuthorizationModel {
    /**
     * This will result in an authorization header that looks like: `Authorization: Bearer TOKEN`
     */
    StaticBearerToken?: string;
    /**
     * This will result in an authorization header that looks like: `Authorization: TOKEN`. Use this only when the other options are not sufficient.
     */
    StaticUnschemedToken?: string;
    /**
     * Configuration for Mutual TLS (mTLS)
     */
    ClientCertificate?: EndpointConfigurationCertificateModel;
  }

  export interface EndpointConfigurationCertificateModel {
    /**
     * A client certificate that can be used for mTLS. Must be X509 in PKCS12 format. Also, it must not be expired, have a private key and have clientAuth in it's extended key usage.
     */
    Certificate: string;
    /**
     * The password that is used to encrypt the certificate.
     */
    Password?: string;
  }

  export interface CreateEndpointConfigurationResponse {
    /**
     * Entity type: EndpointConfiguration
     */
    ID: number;
    /**
     * Make sure to store this information, this is the only time EVA will expose it for you!
     */
    SecretKey1: string;
    /**
     * Make sure to store this information, this is the only time EVA will expose it for you!
     */
    SecretKey2: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Service for updating a existing `EndpointConfiguration`
   */
  export interface UpdateEndpointConfiguration {
    /**
     * Entity type: EndpointConfiguration
     */
    ID: number;
    Name?: string;
    BackendID?: string;
    Status?: EVA.Core.EndpointConfigurationStatus;
    Endpoint?: string;
    /**
     * Authorization configuration used when calling this endpoint.
     */
    Authorization?: EndpointConfigurationAuthorizationUpdateModel;
    /**
     * Timeout after which requests are cancelled. Defaults to 5 seconds. Not recommended to change.
     */
    TimeoutInSeconds?: number;
    /**
     * If true, the endpoint will use string IDs instead of long IDs.
     */
    UseStringIDs?: boolean;
    /**
     * If false, the endpoint will also send `null` values. Default and preferred is true.
     */
    IgnoreNullValues?: boolean;
  }

  export interface EndpointConfigurationAuthorizationUpdateModel {
    /**
     * This will result in an authorization header that looks like: `Authorization: Bearer TOKEN`
     */
    StaticBearerToken?: string;
    /**
     * This will result in an authorization header that looks like: `Authorization: TOKEN`. Use this only when the other options are not sufficient.
     */
    StaticUnschemedToken?: string;
    /**
     * Configuration for Mutual TLS (mTLS)
     */
    ClientCertificate?: EndpointConfigurationCertificateUpdateModel;
  }

  export interface EndpointConfigurationCertificateUpdateModel {
    /**
     * A client certificate that can be used for mTLS. Must be X509 in PKCS12 format. Also, it must not be expired, have a private key and have clientAuth in it's extended key usage.
     */
    Certificate?: string;
    /**
     * The password that is used to encrypt the certificate.
     */
    Password?: string;
  }

  /**
   * Service for deleting a existing `EndpointConfiguration`
   */
  export interface DeleteEndpointConfiguration {
    /**
     * Entity type: EndpointConfiguration
     */
    ID: number;
  }

  /**
   * Service for retrieving an existing `EndpointConfiguration`.
   */
  export interface GetEndpointConfiguration {
    /**
     * Entity type: EndpointConfiguration
     */
    ID: number;
  }

  export interface GetEndpointConfigurationResponse {
    Result: EndpointConfigurationModel;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface EndpointConfigurationModel {
    /**
     * Entity type: EndpointConfiguration
     */
    ID: number;
    Name: string;
    BackendID?: string;
    Status: EVA.Core.EndpointConfigurationStatus;
    Endpoint: string;
    SecretKey1: string;
    SecretKey2: string;
    Authorization?: EndpointConfigurationAuthorizationModel;
    TimeoutInSeconds?: number;
    UseStringIDs?: boolean;
    IgnoreNullValues?: boolean;
    ErrorReason?: string;
    LastAccessTime?: string;
  }

  /**
   * Service for retrieving a list of existing `EndpointConfiguration`'s
   */
  export interface GetEndpointConfigurations {
  }

  export interface GetEndpointConfigurationsResponse {
    Items: EndpointConfigurationModel[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Service for generating a new SecretKey for an `EndpointConfiguration`.
   * 
   * - Make sure to catch & store the `SecretKey1` or `SecretKey2`, as there will be no other way of retrieving them after this service call!
   */
  export interface RotateEndpointConfigurationKey {
    /**
     * Entity type: EndpointConfiguration
     */
    ID: number;
    KeyID: number;
  }

  export interface RotateEndpointConfigurationKeyResponse {
    SecretKey1?: string;
    SecretKey2?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the DiscountCampaigns
   */
  export interface ListDiscountCampaigns {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListDiscountCampaignsFilter>;
  }

  export interface ListDiscountCampaignsResponse {
    Result?: EVA.Core.PagedResult<ListDiscountCampaignsResponse.DiscountCampaignDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListDiscountCampaignsResponse {
    export interface DiscountCampaignDto {
      /**
       * Entity type: DiscountCampaign
       */
      ID: number;
      Name: string;
      Description: string;
      StartDate?: string;
      EndDate?: string;
      BackendSystemID: string;
      BackendID: string;
      /**
       * Entity type: Currency
       */
      CurrencyID?: string;
      Budget?: number;
      BudgetProgress?: number;
    }

  }

  /**
   * Get a DiscountCampaign by ID
   */
  export interface GetDiscountCampaignByID {
    /**
     * Entity type: DiscountCampaign
     */
    ID: number;
  }

  export interface GetDiscountCampaignByIDResponse {
    /**
     * Entity type: DiscountCampaign
     */
    ID: number;
    Name: string;
    Description: string;
    StartDate?: string;
    EndDate?: string;
    KeepDiscountsWithinCampaignTimespan: boolean;
    BackendSystemID: string;
    BackendID: string;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    Budget?: number;
    DeactivateDiscountsWhenBudgetIsReached: boolean;
    TotalDiscountGiven?: number;
    BudgetAvailable?: number;
    BudgetProgress?: number;
    BudgetReached?: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new DiscountCampaign
   */
  export interface CreateDiscountCampaign {
    Name: string;
    Description?: string;
    StartDate?: string;
    EndDate?: string;
    KeepDiscountsWithinCampaignTimespan?: boolean;
    BackendSystemID?: string;
    BackendID?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  export interface CreateDiscountCampaignResponse {
    /**
     * Entity type: DiscountCampaign
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing DiscountCampaign
   */
  export interface UpdateDiscountCampaign {
    /**
     * Entity type: DiscountCampaign
     */
    ID: number;
    Name?: string;
    Description?: string;
    StartDate?: string;
    EndDate?: string;
    KeepDiscountsWithinCampaignTimespan?: boolean;
    BackendSystemID?: string;
    BackendID?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  /**
   * Update the budget on the discount campaign.
   * When the budget is lowerd below the current `TotalDiscountGiven` and the `DeactivateWhenBudgetReached` is set to true, all related discounts will be deactivated.
   * When the budget is cleared or raised above the current `TotalDiscountGiven`, or `DeactivateWhenBudgetReached` is set to false, all discounts that were deactivate by the discount campaign budget will be reactivated.
   */
  export interface UpdateDiscountCampaignBudget {
    /**
     * Entity type: DiscountCampaign
     */
    ID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Min: 0
     */
    Budget?: number;
    DeactivateDiscountsWhenBudgetIsReached?: boolean;
    ReactivateDeactivatedDiscounts?: boolean;
  }

  /**
   * Delete a DiscountCampaign
   */
  export interface DeleteDiscountCampaign {
    /**
     * Entity type: DiscountCampaign
     */
    ID: number;
    ArchiveAllRelatedDiscounts?: boolean;
    ReactivateAllRelatedDiscounts?: boolean;
  }

  /**
   * List the coupons
   */
  export interface ListCouponsByQuery {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListDiscountCouponsFilter>;
  }

  export interface ListCouponsByQueryResponse {
    Result?: EVA.Core.PagedResult<ListCouponsByQueryResponse.Coupon>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCouponsByQueryResponse {
    export interface Coupon {
      /**
       * Entity type: DiscountCoupon
       */
      ID: number;
      /**
       * Entity type: Discount
       */
      DiscountID: number;
      DiscountMarketingDescription: string;
      CouponCode: string;
      MaximumUsage?: number;
      UsageCount: number;
      FirstUsage?: string;
      LastUsage?: string;
      IsActive: boolean;
      IsGenerated: boolean;
      IsExternal: boolean;
    }

  }

  /**
   * Get discount coupons by discount ID
   */
  export interface GetDiscountCouponByID {
    /**
     * Entity type: DiscountCoupon
     */
    ID: number;
  }

  export interface GetDiscountCouponByIDResponse {
    /**
     * Entity type: DiscountCoupon
     */
    ID: number;
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    DiscountMarketingDescription: string;
    /**
     * Entity type: Discount
     */
    OriginatingDiscountID?: number;
    OriginatingDiscountMarketingDescription: string;
    /**
     * Entity type: Order
     */
    OriginatingOrderID?: number;
    CouponCode: string;
    IsExternal: boolean;
    Handler: string;
    IsGenerated: boolean;
    IsActive: boolean;
    MaximumUsage?: number;
    UsageCount: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    UserFullName?: string;
    FirstUsage?: string;
    LastUsage?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a DiscountCoupon
   */
  export interface CreateDiscountCoupon {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    CouponCode: string;
    IsActive?: boolean;
    MaximumUsage?: number;
  }

  export interface CreateDiscountCouponResponse {
    /**
     * Entity type: DiscountCoupon
     */
    ID: number;
    IsDuplicateCoupon: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing DiscountCoupon
   */
  export interface UpdateDiscountCoupon {
    /**
     * Entity type: DiscountCoupon
     */
    ID: number;
    IsActive?: boolean;
    MaximumUsage?: number;
  }

  export interface UpdateDiscountCouponResponse {
    IsDuplicateCoupon: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Generate a batch DiscountCoupons, they will be emailed to the current user
   */
  export interface GenerateDiscountCoupons {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    /**
     * Min: 1
     */
    Quantity: number;
    Prefix?: string;
    Suffix?: string;
    /**
     * Min: 1
     */
    MaximumUsage?: number;
  }

  /**
   * List the coupons that are generated on an order
   */
  export interface ListOrderGeneratedCoupons {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ListOrderGeneratedCouponsResponse {
    Coupons: ListOrderGeneratedCouponsResponse.Coupon[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListOrderGeneratedCouponsResponse {
    export interface Coupon {
      /**
       * Entity type: DiscountCoupon
       */
      ID: number;
      Code: string;
      /**
       * Entity type: Discount
       */
      TargetDiscountID: number;
      TargetDiscountDescription: string;
      TargetDiscountMarketingDescription: string;
    }

  }

  /**
   * Get the coupons for a specific user
   */
  export interface GetUserCoupons {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * You can provide an additional `OrganizationUnitID` to filter the coupons. This only works when you are logged in as User. Employees/API's can only see coupons that are within the ClientContext CurrentOrganizationUnit
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Filter on usability. Optional values are; null = all coupons, true = usable only, false = unusable only.
     */
    IsUsable?: boolean;
  }

  export interface GetUserCouponsResponse {
    Coupons: GetUserCouponsResponse.Coupon[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetUserCouponsResponse {
    export interface Coupon {
      /**
       * Entity type: DiscountCoupon
       */
      ID: number;
      Code: string;
      /**
       * Entity type: Discount
       */
      DiscountID: number;
      DiscountDescription: string;
      DiscountMarketingDescription: string;
      ValidFrom: string;
      ValidTill?: string;
    }

  }

  /**
   * Get a coupon by it's token. When a customer calls this service, the service will only resolve tokens from coupons on which the user is registered. Employees and API's can resolve all coupon tokens.
   */
  export interface GetCouponByToken {
    CouponToken: string;
  }

  export interface GetCouponByTokenResponse {
    Code: string;
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    DiscountDescription: string;
    DiscountMarketingDescription: string;
    ValidFrom: string;
    ValidTill?: string;
    /**
     * Entity type: OrderLine
     */
    FromCancelledLines?: { [key: number]: number };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Download template to upload an excel with coupons.
   */
  export interface DownloadCouponExcelTemplate {
  }

  /**
   * Upload an excel with coupons.
   */
  export interface UploadCouponExcel {
    /**
     * Entity type: Discount
     */
    DiscountID?: number;
    /**
     * Entity type: Blob
     */
    BlobID?: string;
  }

  export interface UploadCouponExcelResponse {
    IsProcessedAsync: boolean;
    Errors: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Exports the coupons for the given discount to Excel.
   */
  export interface DownloadCouponExcel {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    /**
     * Filter on active state. Optional values are; null = active and inactive, true = active only, false = inactive only.
     */
    IsActive?: boolean;
  }

  export interface DownloadCouponExcelResponse {
    CouponLimitReached: boolean;
    ResourceInfo: EVA.Core.ResourceInfo;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Push a DiscountCoupon
   */
  export interface PushDiscountCoupons {
    DiscountID?: string;
    Coupons?: PushDiscountCoupons.PushedCoupon[];
  }

  export namespace PushDiscountCoupons {
    export interface PushedCoupon {
      UserID?: string;
      OrderID?: string;
      CouponCode?: string;
      IsActive?: boolean;
      MaximumUsage?: number;
    }

  }

  export interface PushDiscountCouponsResponse {
    CreatedCoupons: PushDiscountCouponsResponse.CreatedCoupon[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace PushDiscountCouponsResponse {
    export interface CreatedCoupon {
      /**
       * Entity type: DiscountCoupon
       */
      ID: number;
      IsDuplicateCoupon: boolean;
    }

  }

  /**
   * Get a DiscountLayer by ID
   */
  export interface GetDiscountLayerByID {
    /**
     * Entity type: DiscountLayer
     */
    ID: number;
  }

  export interface GetDiscountLayerByIDResponse {
    ID: number;
    Name: string;
    Description: string;
    Sequence: number;
    IsExclusive: boolean;
    /**
     * This layer is managed by EVA and cannot be changed.
     */
    IsSystem: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new DiscountLayer
   */
  export interface CreateDiscountLayer {
    Name: string;
    Description?: string;
    /**
     * Min: 0
     */
    Sequence?: number;
    IsExclusive?: boolean;
  }

  export interface CreateDiscountLayerResponse {
    /**
     * Entity type: DiscountLayer
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing DiscountLayer
   */
  export interface UpdateDiscountLayer {
    /**
     * Entity type: DiscountLayer
     */
    ID: number;
    Name: string;
    Description?: string;
    /**
     * Min: 0
     */
    Sequence: number;
    IsExclusive?: boolean;
  }

  /**
   * Delete a DiscountLayer
   */
  export interface DeleteDiscountLayer {
    /**
     * Entity type: DiscountLayer
     */
    ID: number;
    /**
     * When set to `true` and there are active discounts within the layer, they will be archived. Otherwise will throw an error.
     */
    ArchiveDiscounts?: boolean;
  }

  /**
   * List the DiscountLayers
   */
  export interface ListDiscountLayers {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListDiscountLayerFilter>;
  }

  export interface ListDiscountLayersResponse {
    Result?: EVA.Core.PagedResult<ListDiscountLayersResponse.DiscountLayerDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListDiscountLayersResponse {
    export interface DiscountLayerDto {
      /**
       * Entity type: DiscountLayer
       */
      ID: number;
      Name: string;
      Description: string;
      Sequence: number;
      IsExclusive: boolean;
      /**
       * This layer is managed by EVA and cannot be changed.
       */
      IsSystem: boolean;
    }

  }

  /**
   * Move an existing DiscountLayer to a new postion.
   * 
   * The Layer will be placed after the `AfterDiscountLayerID`. If no `AfterDiscountLayerID` is given, the layer will be placed on the first position.
   */
  export interface MoveDiscountLayer {
    /**
     * Entity type: DiscountLayer
     */
    ID: number;
    /**
     * Entity type: DiscountLayer
     */
    AfterDiscountLayerID: number;
  }

  /**
   * Takes in all discount layers and sets their sequences. All layers except the system layer are required.
   */
  export interface SetDiscountLayerSequences {
    /**
     * Entity type: DiscountLayer
     */
    Layers: number[];
  }

  /**
   * Validates the given discount information
   */
  export interface ValidateDiscount {
    /**
     * Entity type: Discount
     */
    ID?: number;
    BackendID?: string;
    IsActive?: boolean;
    Description?: string;
    MarketingDescription?: string;
    NeedsReason?: boolean;
    /**
     * For a manual discount, these will indicate custom fields that need to be filled in order to apply the discount.
     * Entity type: CustomField
     */
    RequiredCustomFields?: number[];
    DiscountOrderType: EVA.Core.DiscountOrderTypes;
    StartDate: string;
    EndDate?: string;
    DaysApplicable?: EVA.Core.DaysOfWeek;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID?: string;
    /**
     * Min: 1
     */
    MaximumUsage?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerOrder?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerUser?: number;
    ActionType?: string;
    ActionData?: { [key: string]: any };
    Conditions?: ValidateDiscount.DiscountConditionDto[];
    ConditionType: EVA.Core.DiscountConditionTypes;
    Trigger: EVA.Core.DiscountTriggers;
    /**
     * Entity type: DiscountLayer
     */
    LayerID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    AppliesTo?: EVA.Core.DiscountAppliesTo;
    CouponHandler?: string;
    /**
     * @deprecated No longer supported, moved to the action data.
     */
    FinancialDispersion: EVA.Core.FinancialDespersionType;
    CouponsGeneratedByOtherDiscount?: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
    /**
     * Entity type: CustomField
     */
    CustomUserUsageLimitFieldID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    /**
     * Min: 1
     */
    LoyaltyPointWithdrawal?: number;
    CompliancyRule: EVA.Core.DiscountCompliancyRule;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  export namespace ValidateDiscount {
    export interface DiscountConditionDto {
      Type: string;
      Data: { [key: string]: any };
      UseAsActionCondition: boolean;
    }

  }

  export interface ValidateDiscountResponse {
    Messages: string[];
    ActionMessages: string[];
    /**
     * Entity type: DiscountCondition
     */
    ConditionMessages: { [key: number]: string[] };
    CustomFieldMessages?: ValidateDiscountResponse.CustomFieldMessage;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ValidateDiscountResponse {
    export interface CustomFieldMessage {
      Type: string;
      Message: string;
      Data: EVA.Core.CustomFieldValidationErrorData;
    }

  }

  /**
   * Create a new discount
   */
  export interface CreateDiscount {
    BackendID?: string;
    Enable: boolean;
    Description: string;
    MarketingDescription?: string;
    PromotionLabel?: string;
    /**
     * Entity type: DiscountCampaign
     */
    CampaignID?: number;
    NeedsReason?: boolean;
    /**
     * For a manual discount, these will indicate custom fields that need to be filled in order to apply the discount.
     * Entity type: CustomField
     */
    RequiredCustomFields?: number[];
    DiscountOrderType: EVA.Core.DiscountOrderTypes;
    StartDate?: string;
    EndDate?: string;
    DaysApplicable?: EVA.Core.DaysOfWeek;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID?: string;
    /**
     * Min: 1
     */
    MaximumUsage?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerOrder?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerUser?: number;
    CannotApplyToOwnOrder?: boolean;
    ActionType: string;
    ActionData?: { [key: string]: any };
    Conditions?: CreateDiscount.DiscountConditionDto[];
    ConditionType: EVA.Core.DiscountConditionTypes;
    Trigger: EVA.Core.DiscountTriggers;
    CouponHandler?: string;
    /**
     * Entity type: DiscountLayer
     */
    LayerID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Min: 0
     */
    Budget?: number;
    DeactivateWhenBudgetReached?: boolean;
    /**
     * Entity type: CustomField
     */
    CustomUserUsageLimitFieldID?: number;
    /**
     * Min: 0
     */
    UserUsageLimit?: number;
    UserUsageReloadStrategy?: string;
    ApplyTo?: EVA.Core.DiscountAppliesTo;
    /**
     * @deprecated No longer supported, moved to the action data.
     */
    FinancialDispersion: EVA.Core.FinancialDespersionType;
    /**
     * Entity type: Role
     */
    RoleIDs?: number[];
    /**
     * Entity type: Blob
     */
    ImageBlobID?: string;
    CouponsGeneratedByOtherDiscount?: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    /**
     * Min: 1
     */
    LoyaltyPointWithdrawal?: number;
    CompliancyRule: EVA.Core.DiscountCompliancyRule;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  export namespace CreateDiscount {
    export interface DiscountConditionDto {
      Type: string;
      Data: { [key: string]: any };
    }

  }

  export interface CreateDiscountResponse {
    /**
     * Entity type: Discount
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing discount
   */
  export interface UpdateDiscount {
    /**
     * Entity type: Discount
     */
    ID: number;
    BackendID?: string;
    Enable: boolean;
    Description: string;
    MarketingDescription?: string;
    PromotionLabel?: string;
    /**
     * Entity type: DiscountCampaign
     */
    CampaignID?: number;
    NeedsReason?: boolean;
    /**
     * For a manual discount, these will indicate custom fields that need to be filled in order to apply the discount.
     * Entity type: CustomField
     */
    RequiredCustomFields?: number[];
    DiscountOrderType: EVA.Core.DiscountOrderTypes;
    StartDate?: string;
    EndDate?: string;
    DaysApplicable?: EVA.Core.DaysOfWeek;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID?: string;
    /**
     * Min: 1
     */
    MaximumUsage?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerOrder?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerUser?: number;
    CannotApplyToOwnOrder?: boolean;
    ActionType: string;
    ActionData?: { [key: string]: any };
    Conditions?: UpdateDiscount.DiscountConditionDto[];
    ConditionType: EVA.Core.DiscountConditionTypes;
    Trigger: EVA.Core.DiscountTriggers;
    CouponHandler?: string;
    /**
     * Entity type: DiscountLayer
     */
    LayerID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Min: 0
     */
    Budget?: number;
    DeactivateWhenBudgetReached?: boolean;
    /**
     * Entity type: CustomField
     */
    CustomUserUsageLimitFieldID?: number;
    /**
     * Min: 0
     */
    UserUsageLimit?: number;
    UserUsageReloadStrategy?: string;
    ApplyTo?: EVA.Core.DiscountAppliesTo;
    /**
     * @deprecated No longer supported, moved to the action data.
     */
    FinancialDispersion: EVA.Core.FinancialDespersionType;
    /**
     * Entity type: Role
     */
    RoleIDs?: number[];
    ImageBlobID?: string;
    CouponsGeneratedByOtherDiscount?: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    /**
     * Min: 1
     */
    LoyaltyPointWithdrawal?: number;
    CompliancyRule: EVA.Core.DiscountCompliancyRule;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  export namespace UpdateDiscount {
    export interface DiscountConditionDto {
      /**
       * Entity type: DiscountCondition
       */
      ID?: number;
      Type: string;
      Data: { [key: string]: any };
    }

  }

  /**
   * Marks the Discount with the given Discount ID as verified.
   * 
   * Service will return an error when de discount is already verified.
   * 
   * Discount cannot be verified by the same user as the one who created / modified the discount, unless they have the right 'DiscountSelfVerification'.
   * The service will return an error if the same user without the right 'DiscountSelfVerification' tries to verify the discount.
   */
  export interface VerifyDiscount {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
  }

  /**
   * Enable discounts
   */
  export interface EnableDiscounts {
    /**
     * Entity type: Discount
     */
    DiscountIDs: number[];
  }

  /**
   * Disable discounts
   */
  export interface DisableDiscounts {
    /**
     * Entity type: Discount
     */
    DiscountIDs: number[];
  }

  /**
   * Archive an existing discount
   */
  export interface ArchiveDiscount {
    /**
     * Entity type: Discount
     */
    DiscountIDs?: number[];
  }

  /**
   * Duplicates the requested discount x times with it's conditions and action.
   * A duplicated discount will not be Active and Verified.
   */
  export interface DuplicateDiscount {
    /**
     * The ID of the discount to create a copy of.
     * Entity type: Discount
     */
    DiscountID: number;
    /**
     * Indicate how many duplicates need to be made. When left empty, only 1 duplicate will be made.
     * Range: 1-500
     */
    NumberOfDuplicates?: number;
  }

  export interface DuplicateDiscountResponse {
    /**
     * Entity type: Discount
     */
    IDs: number[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Searches the Discounts with the given filter and returns a paged response.
   */
  export interface SearchDiscountsByQuery {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.SearchDiscountsByQueryFilter>;
  }

  export interface SearchDiscountsByQueryResponse {
    Result?: EVA.Core.PagedResult<SearchDiscountsByQueryResponse.DiscountDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace SearchDiscountsByQueryResponse {
    export interface DiscountDto {
      /**
       * Entity type: Discount
       */
      ID: number;
      BackendID: string;
      /**
       * Entity type: DiscountCampaign
       */
      CampaignID?: number;
      Description?: string;
      IsUsed: boolean;
      Enabled: boolean;
      DeactivationReasons: EVA.Core.DiscountDeactivationReasons;
      MaximumUsage?: number;
      CurrencyID: string;
      Budget?: number;
      DeactivateWhenBudgetReached: boolean;
      BudgetAvailable?: number;
      BudgetProgress?: number;
      TotalDiscountGiven: number;
      UsageCount: number;
      /**
       * Entity type: User
       */
      CreatedByID: number;
      CreatedByFullName: string;
      DiscountOrderType: EVA.Core.DiscountOrderTypes;
      /**
       * Entity type: DiscountLayer
       */
      LayerID: number;
      LayerName: string;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      OrganizationUnitSetName: string;
      OrganizationUnitSetDescription: string;
      OrganizationUnitSetType?: EVA.Core.OrganizationUnitSetTypes;
      StartDate: string;
      EndDate?: string;
      /**
       * Entity type: CustomField
       */
      CustomUserUsageLimitFieldID?: number;
      UserUsageLimit?: number;
      UserUsageReloadStrategy: string;
      GeneratedFromTemplate: boolean;
      ImageBlobID?: string;
      CouponsGeneratedByOtherDiscount: boolean;
      /**
       * Entity type: OrganizationUnit
       */
      RestitutionOrganizationUnitID?: number;
      /**
       * Entity type: LoyaltyProgram
       */
      LoyaltyProgramID?: number;
    }

  }

  /**
   * Searches the Discounts with the given filter and returns a tree structure based on discount layer and discount.
   */
  export interface GetLayerGroupedDiscountsByQuery {
    Query?: EVA.Core.SearchDiscountsByQueryFilter;
  }

  export interface GetLayerGroupedDiscountsByQueryResponse {
    Layers: GetLayerGroupedDiscountsByQueryResponse.LayerModel[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetLayerGroupedDiscountsByQueryResponse {
    export interface LayerModel {
      LayerID: number;
      LayerName: string;
      Sequence: number;
      IsExclusive: boolean;
      IsSystem: boolean;
      Discounts: GetLayerGroupedDiscountsByQueryResponse.DiscountModel[];
    }

    export interface DiscountModel {
      /**
       * Entity type: Discount
       */
      ID: number;
      BackendID: string;
      /**
       * Entity type: DiscountCampaign
       */
      CampaignID?: number;
      Description?: string;
      IsUsed: boolean;
      Enabled: boolean;
      DeactivationReasons: EVA.Core.DiscountDeactivationReasons;
      MaximumUsage?: number;
      CurrencyID: string;
      Budget?: number;
      DeactivateWhenBudgetReached: boolean;
      BudgetAvailable?: number;
      BudgetProgress?: number;
      TotalDiscountGiven: number;
      UsageCount: number;
      /**
       * Entity type: User
       */
      CreatedByID: number;
      CreatedByFullName: string;
      DiscountOrderType: EVA.Core.DiscountOrderTypes;
      /**
       * Entity type: DiscountLayer
       */
      LayerID: number;
      LayerName: string;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      OrganizationUnitSetName: string;
      OrganizationUnitSetDescription: string;
      OrganizationUnitSetType?: EVA.Core.OrganizationUnitSetTypes;
      StartDate: string;
      EndDate?: string;
      /**
       * Entity type: CustomField
       */
      CustomUserUsageLimitFieldID?: number;
      UserUsageLimit?: number;
      UserUsageReloadStrategy: string;
      GeneratedFromTemplate: boolean;
      ImageBlobID?: string;
      CouponsGeneratedByOtherDiscount: boolean;
      /**
       * Entity type: OrganizationUnit
       */
      RestitutionOrganizationUnitID?: number;
      /**
       * Entity type: LoyaltyProgram
       */
      LoyaltyProgramID?: number;
      LoyaltyPointWithdrawal?: number;
    }

  }

  /**
   * Get Discount by the given Discount ID
   */
  export interface GetDiscountByID {
    /**
     * Entity type: Discount
     */
    ID: number;
  }

  export interface GetDiscountByIDResponse {
    /**
     * Entity type: Discount
     */
    ID: number;
    BackendID: string;
    Enabled: boolean;
    IsEditAllowed: boolean;
    IsUsed: boolean;
    DeactivationReasons: EVA.Core.DiscountDeactivationReasons;
    Description: string;
    MarketingDescription: string;
    PromotionLabel?: string;
    /**
     * Entity type: DiscountCampaign
     */
    CampaignID?: number;
    NeedsReason: boolean;
    /**
     * Entity type: CustomField
     */
    RequiredCustomFields: number[];
    DiscountOrderType: EVA.Core.DiscountOrderTypes;
    StartDate: string;
    EndDate?: string;
    DaysApplicable?: EVA.Core.DaysOfWeek;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID: string;
    MaximumUsage?: number;
    UsageCount: number;
    MaximumUsagePerOrder?: number;
    MaximumUsagePerUser?: number;
    ActionType: string;
    ActionData: { [key: string]: any };
    Conditions: GetDiscountByIDResponse.DiscountCondition[];
    ConditionType: EVA.Core.DiscountConditionTypes;
    Trigger: EVA.Core.DiscountTriggers;
    CouponHandler: string;
    TotalCouponCount?: number;
    UsableCouponCount?: number;
    UsedCouponCount?: number;
    /**
     * Entity type: DiscountLayer
     */
    LayerID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    OrganizationUnitSetName: string;
    /**
     * This could be the backend id of the OU if this is the system generated OU set of the OU
     */
    OrganizationUnitSetBackendID: string;
    OrganizationUnitSetType?: EVA.Core.OrganizationUnitSetTypes;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Budget?: number;
    DeactivateWhenBudgetReached: boolean;
    BudgetAvailable?: number;
    BudgetProgress?: number;
    TotalDiscountGiven: number;
    /**
     * Entity type: CustomField
     */
    CustomUserUsageLimitFieldID?: number;
    UserUsageLimit?: number;
    UserUsageReloadStrategy: string;
    CreatedByFullName: string;
    CreationTime: string;
    LastModifiedByFullName: string;
    LastModificationTime?: string;
    VerifiedByFullName: string;
    VerificationTime?: string;
    ApplyTo?: EVA.Core.DiscountAppliesTo;
    /**
     * @deprecated No longer supported, moved to the action data.
     */
    FinancialDispersion: EVA.Core.FinancialDespersionType;
    GeneratedFromTemplate: boolean;
    ImageBlobID?: string;
    CouponsGeneratedByOtherDiscount: boolean;
    Roles: GetDiscountByIDResponse.Role[];
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
    RestitutionOrganizationUnitName?: string;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    LoyaltyPointWithdrawal?: number;
    CompliancyRule: EVA.Core.DiscountCompliancyRule;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: EVA.Core.CustomFieldValueWithOptions };
    CannotApplyToOwnOrder: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetDiscountByIDResponse {
    export interface DiscountCondition {
      /**
       * Entity type: DiscountCondition
       */
      ID: number;
      Type: string;
      Data: { [key: string]: any };
      UseAsActionCondition: boolean;
    }

    export interface Role {
      /**
       * Entity type: Role
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Add security roles to ManualDiscount
   */
  export interface AddRolesToManualDiscount {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    /**
     * Entity type: Role
     */
    RoleIDs: number[];
  }

  /**
   * Remove security roles from ManualDiscount
   */
  export interface RemoveRolesFromManualDiscount {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    /**
     * Entity type: Role
     */
    RoleIDs?: number[];
    RemoveAllRoles?: boolean;
  }

  /**
   * Add required custom fields to ManualDiscount
   */
  export interface AddRequiredCustomFieldsToManualDiscount {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    /**
     * Entity type: CustomField
     */
    CustomFieldIDs: number[];
  }

  /**
   * Remove required custom fields from ManualDiscount
   */
  export interface RemoveRequiredCustomFieldsFromManualDiscount {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    /**
     * Entity type: CustomField
     */
    CustomFieldIDs?: number[];
    RemoveAllRequiredCustomFields?: boolean;
  }

  /**
   * Lists the available discount conditions and actions.
   */
  export interface ListAvailableDiscountActionsAndConditions {
  }

  export interface ListAvailableDiscountActionsAndConditionsResponse {
    Conditions: string[];
    Actions: string[];
    TriggerAvailability: ListAvailableDiscountActionsAndConditionsResponse.TriggerItem[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListAvailableDiscountActionsAndConditionsResponse {
    export interface TriggerItem {
      Trigger: EVA.Core.DiscountTriggers;
      Conditions: string[];
      Actions: string[];
    }

  }

  /**
   * Lists the available discount user usage reload strategies.
   */
  export interface ListDiscountUserUsageReloadStrategies {
  }

  export interface ListDiscountUserUsageReloadStrategiesResponse {
    Strategies: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Validates the given discount template information
   */
  export interface ValidateDiscountTemplate {
    Description: string;
    Trigger: EVA.Core.DiscountTriggers;
    CouponsGeneratedByOtherDiscount?: boolean;
    CouponHandler?: string;
    DiscountOrderType: EVA.Core.DiscountOrderTypes;
    AppliesTo: EVA.Core.DiscountAppliesTo;
    ActionType: string;
    ActionData: { [key: string]: any };
    ConditionType: EVA.Core.DiscountConditionTypes;
    Conditions?: ValidateDiscountTemplate.Condition[];
    NeedsReason?: boolean;
    IsExclusive?: boolean;
    /**
     * Min: 1
     */
    MaximumUsage?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerUser?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerOrder?: number;
    CannotApplyToOwnOrder?: boolean;
    DaysApplicable?: EVA.Core.DaysOfWeek;
    /**
     * Min: 0
     */
    UserUsageLimit?: number;
    UserUsageReloadStrategy?: string;
    BackendID?: string;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
  }

  export namespace ValidateDiscountTemplate {
    export interface Condition {
      Type: string;
      Data: { [key: string]: any };
    }

  }

  export interface ValidateDiscountTemplateResponse {
    GeneralErrors: string[];
    ActionErrors: string[];
    ConditionErrors: ValidateDiscountTemplateResponse.ConditionError[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ValidateDiscountTemplateResponse {
    export interface ConditionError {
      ConditionID: number;
      Messages: string[];
    }

  }

  /**
   * Create a new discount template
   */
  export interface CreateDiscountTemplate {
    Description: string;
    BackendID?: string;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID?: string;
    Trigger: EVA.Core.DiscountTriggers;
    CouponsGeneratedByOtherDiscount?: boolean;
    CouponHandler?: string;
    DiscountOrderType: EVA.Core.DiscountOrderTypes;
    AppliesTo: EVA.Core.DiscountAppliesTo;
    ActionType: string;
    ActionData: { [key: string]: any };
    ConditionType: EVA.Core.DiscountConditionTypes;
    Conditions?: CreateDiscountTemplate.Condition[];
    NeedsReason?: boolean;
    IsExclusive?: boolean;
    /**
     * Min: 1
     */
    MaximumUsage?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerUser?: number;
    CannotApplyToOwnOrder?: boolean;
    DaysApplicable?: EVA.Core.DaysOfWeek;
    /**
     * Min: 1
     */
    MaximumUsagePerOrder?: number;
    /**
     * Min: 0
     */
    UserUsageLimit?: number;
    UserUsageReloadStrategy?: string;
    ImageBlobID?: string;
    /**
     * Entity type: DiscountCampaign
     */
    CampaignID?: number;
    IsActive?: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
  }

  export namespace CreateDiscountTemplate {
    export interface Condition {
      Type: string;
      Data: { [key: string]: any };
    }

  }

  export interface CreateDiscountTemplateResponse {
    /**
     * Entity type: DiscountTemplate
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing discount template
   */
  export interface UpdateDiscountTemplate {
    /**
     * Entity type: DiscountTemplate
     */
    ID: number;
    Description: string;
    BackendID?: string;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID?: string;
    Trigger: EVA.Core.DiscountTriggers;
    CouponsGeneratedByOtherDiscount?: boolean;
    CouponHandler?: string;
    DiscountOrderType: EVA.Core.DiscountOrderTypes;
    AppliesTo: EVA.Core.DiscountAppliesTo;
    ActionType: string;
    ActionData: { [key: string]: any };
    ConditionType: EVA.Core.DiscountConditionTypes;
    Conditions?: UpdateDiscountTemplate.Condition[];
    NeedsReason?: boolean;
    IsExclusive?: boolean;
    /**
     * Min: 1
     */
    MaximumUsage?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerUser?: number;
    /**
     * Min: 1
     */
    MaximumUsagePerOrder?: number;
    CannotApplyToOwnOrder?: boolean;
    DaysApplicable?: EVA.Core.DaysOfWeek;
    /**
     * Min: 0
     */
    UserUsageLimit?: number;
    UserUsageReloadStrategy?: string;
    ImageBlobID?: string;
    /**
     * Entity type: DiscountCampaign
     */
    CampaignID?: number;
    IsActive?: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
  }

  export namespace UpdateDiscountTemplate {
    export interface Condition {
      /**
       * Entity type: DiscountTemplateCondition
       */
      ID?: number;
      Type: string;
      Data: { [key: string]: any };
    }

  }

  /**
   * Delete an existing discount template
   */
  export interface DeleteDiscountTemplate {
    /**
     * Entity type: DiscountTemplate
     */
    ID: number;
    ArchiveAllRelatedDiscounts?: boolean;
  }

  /**
   * Duplicates the given discount template.
   */
  export interface DuplicateDiscountTemplate {
    /**
     * Entity type: DiscountTemplate
     */
    ID: number;
  }

  /**
   * Searches the DiscountTemplates with the given filter and returns a paged response.
   */
  export interface ListDiscountTemplates {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListDiscountTemplatesFilter>;
  }

  export interface ListDiscountTemplatesResponse {
    Result?: EVA.Core.PagedResult<ListDiscountTemplatesResponse.Data>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListDiscountTemplatesResponse {
    export interface Data {
      /**
       * Entity type: DiscountTemplate
       */
      ID: number;
      Description: string;
      Trigger: EVA.Core.DiscountTriggers;
      DiscountOrderType: EVA.Core.DiscountOrderTypes;
      AppliesTo: EVA.Core.DiscountAppliesTo;
      ActionType: string;
      CreatedByFullName: string;
      CreationTime: string;
      LastModifiedByFullName: string;
      LastModificationTime?: string;
      ImageBlobID?: string;
      IsActive: boolean;
      /**
       * Entity type: OrganizationUnit
       */
      RestitutionOrganizationUnitID?: number;
    }

  }

  /**
   * Get DiscountTemplate by the given ID
   */
  export interface GetDiscountTemplateByID {
    /**
     * Entity type: DiscountTemplate
     */
    ID: number;
  }

  export interface GetDiscountTemplateByIDResponse {
    /**
     * Entity type: DiscountTemplate
     */
    ID: number;
    Description: string;
    BackendID?: string;
    LedgerClassID?: string;
    Trigger: EVA.Core.DiscountTriggers;
    CouponsGeneratedByOtherDiscount: boolean;
    CouponHandler: string;
    DiscountOrderType: EVA.Core.DiscountOrderTypes;
    AppliesTo: EVA.Core.DiscountAppliesTo;
    ActionType: string;
    ActionData: { [key: string]: any };
    ConditionType: EVA.Core.DiscountConditionTypes;
    Conditions: GetDiscountTemplateByIDResponse.Condition[];
    NeedsReason: boolean;
    IsExclusive: boolean;
    MaximumUsage?: number;
    MaximumUsagePerUser?: number;
    MaximumUsagePerOrder?: number;
    CannotApplyToOwnOrder: boolean;
    DaysApplicable?: EVA.Core.DaysOfWeek;
    UserUsageLimit?: number;
    UserUsageReloadStrategy: string;
    ImageBlobID?: string;
    /**
     * Entity type: DiscountCampaign
     */
    CampaignID?: number;
    IsActive: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
    RestitutionOrganizationUnitName?: string;
    CreatedByID: number;
    CreatedByFullName: string;
    CreationTime: string;
    ModifiedByID?: number;
    LastModifiedByFullName: string;
    LastModificationTime?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetDiscountTemplateByIDResponse {
    export interface Condition {
      /**
       * Entity type: DiscountTemplateCondition
       */
      ID: number;
      Type: string;
      Data: { [key: string]: any };
    }

  }

  /**
   * Create a discount from the given template with the overwrites with the values given in the request.
   */
  export interface CreateDiscountFromTemplate {
    /**
     * Entity type: DiscountTemplate
     */
    ID: number;
    /**
     * Entity type: DiscountLayer
     */
    LayerID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Description: string;
    MarketingDescription?: string;
    PromotionLabel?: string;
    /**
     * Entity type: DiscountCampaign
     */
    CampaignID?: number;
    /**
     * Entity type: LedgerClass
     */
    LedgerClassID?: string;
    FinancialDispersion?: EVA.Core.FinancialDespersionType;
    BackendID?: string;
    NeedsReason?: boolean;
    IsExclusive?: boolean;
    MaximumUsage?: number;
    MaximumUsagePerUser?: number;
    MaximumUsagePerOrder?: number;
    CannotApplyToOwnOrder?: boolean;
    DaysApplicable?: EVA.Core.DaysOfWeek;
    UserUsageLimit?: number;
    UserUsageReloadStrategy?: string;
    ImageBlobID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
    CreateAsActive?: boolean;
    StartDate?: string;
    EndDate?: string;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    /**
     * Min: 1
     */
    LoyaltyPointWithdrawal?: number;
    CompliancyRule: EVA.Core.DiscountCompliancyRule;
    /**
     * Entity type: Role
     */
    RoleIDs?: number[];
    /**
     * For a manual discount, these will indicate custom fields that need to be filled in order to apply the discount.
     * Entity type: CustomField
     */
    RequiredCustomFields?: number[];
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  /**
   * Gets an custom field by its ID and returns the basic information.
   */
  export interface GetCustomFieldByID {
    /**
     * Entity type: CustomField
     */
    ID: number;
  }

  export interface GetCustomFieldByIDResponse {
    /**
     * Entity type: CustomField
     */
    ID: number;
    /**
     * Entity type: CustomFieldType
     */
    TypeID: number;
    TypeName: string;
    TypeKey?: string;
    Name: string;
    DisplayName?: string;
    DataType: EVA.Core.CustomFieldDataTypes;
    Order: number;
    BackendID?: string;
    IsArray?: boolean;
    EnumValues?: { [key: string]: string };
    InputHint?: string;
    VisibleByUserTypes?: EVA.Core.UserTypes;
    EditableByUserTypes?: EVA.Core.UserTypes;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Create a new custom field.
   */
  export interface CreateCustomField {
    /**
     * Entity type: CustomFieldType
     */
    TypeID?: number;
    TypeKey?: string;
    BackendID?: string;
    Name?: string;
    DisplayName?: string;
    DataType?: EVA.Core.CustomFieldDataTypes;
    Order?: number;
    IsArray?: boolean;
    EnumValues?: { [key: string]: string };
    InputHint?: string;
  }

  export interface CreateCustomFieldResponse {
    /**
     * Entity type: CustomField
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing custom CustomField.
   */
  export interface UpdateCustomField {
    /**
     * Entity type: CustomField
     */
    ID: number;
    TypeKey?: string;
    Name?: string;
    DisplayName?: string;
    Order?: number;
    BackendID?: string;
    EnumValues?: { [key: string]: string };
    InputHint?: string;
  }

  /**
   * Delete a CustomField
   */
  export interface DeleteCustomField {
    /**
     * Entity type: CustomField
     */
    ID: number;
  }

  /**
   * List CustomFields. Returns a paged response.
   */
  export interface ListCustomFields {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListCustomFieldsFilter>;
  }

  export interface ListCustomFieldsResponse {
    Result?: EVA.Core.PagedResult<ListCustomFieldsResponse.CustomFieldDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCustomFieldsResponse {
    export interface CustomFieldDto {
      /**
       * Entity type: CustomField
       */
      ID: number;
      /**
       * Entity type: CustomFieldType
       */
      TypeID: number;
      TypeName: string;
      TypeKey?: string;
      Name: string;
      DisplayName?: string;
      DataType: EVA.Core.CustomFieldDataTypes;
      Order: number;
      BackendID?: string;
      EnumValues?: { [key: string]: string };
      IsArray: boolean;
      InputHint?: string;
    }

  }

  /**
   * Lists all filter operators for every customfield datatype.
   */
  export interface ListCustomFieldDataTypesOperators {
  }

  export interface ListCustomFieldDataTypeOperatorsResponse {
    DataTypeOperators: ListCustomFieldDataTypeOperatorsResponse.DataTypeOperator[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCustomFieldDataTypeOperatorsResponse {
    export interface DataTypeOperator {
      DataType: EVA.Core.CustomFieldDataTypes;
      Operators: EVA.Core.CustomFieldValueOperator[];
    }

  }

  /**
   * Get all the custom fields for the given organization unit and type.
   */
  export interface GetCustomFields {
    /**
     * Entity type: CustomFieldType
     */
    TypeID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    DataTypes?: EVA.Core.CustomFieldDataTypes[];
  }

  export interface GetCustomFieldsResponse {
    /**
     * Entity type: CustomField
     */
    CustomFieldOptions: { [key: number]: EVA.Core.CustomFieldOptions };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetCustomFieldMetadata {
  }

  export interface GetCustomFieldMetadataResponse {
    CustomFieldsMetadata: { [key: string]: EVA.Core.CustomFieldMetadata[] };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get the CustomFieldTypes
   */
  export interface GetCustomFieldTypes {
  }

  export interface GetCustomFieldTypesResponse {
    CustomFieldTypes: EVA.Core.EnumDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get all eligibility fields for the given custom field type.
   */
  export interface ListCustomFieldEligibilityScripts {
    /**
     * Entity type: CustomFieldType
     */
    TypeID: number;
  }

  export interface ListCustomFieldEligibilityScriptsResponse {
    Scripts: ListCustomFieldEligibilityScriptsResponse.Script[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCustomFieldEligibilityScriptsResponse {
    export interface Script {
      /**
       * Entity type: Script
       */
      ID: number;
      Name: string;
      Dialect: string;
      Type?: string;
    }

  }

  /**
   * List options for custom fields
   */
  export interface ListCustomFieldOptions {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListCustomFieldOptionsFilter>;
    PageToken?: string;
  }

  export interface ListCustomFieldOptionsResponse {
    Results?: ListCustomFieldOptionsResponse.CustomFieldOptionsDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCustomFieldOptionsResponse {
    export interface CustomFieldOptionsDto {
      IsRequired?: boolean;
      MinimumValue?: number;
      MaximumValue?: number;
      MinimumLength?: number;
      MaximumLength?: number;
      MinimumDate?: string;
      MaximumDate?: string;
      CustomFieldDefaultValue?: EVA.Core.CustomFieldValue;
      OrganizationUnitSet?: CustomFieldOptionsDto.OrganizationUnitSetData;
      VisibleByUserTypes?: EVA.Core.UserTypes;
      EditableByUserTypes?: EVA.Core.UserTypes;
      /**
       * Entity type: Script
       */
      SecurityScriptID?: number;
      /**
       * Entity type: Script
       */
      EligibilityScriptID?: number;
    }

    export namespace CustomFieldOptionsDto {
      export interface OrganizationUnitSetData {
        /**
         * Entity type: OrganizationUnitSet
         */
        ID: number;
        Name: string;
      }

    }

  }

  /**
   * Set options for custom fields
   */
  export interface SetCustomFieldOptions {
    /**
     * Entity type: CustomField
     */
    CustomFieldID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    MinimumValue?: number;
    MaximumValue?: number;
    MinimumLength?: number;
    MaximumLength?: number;
    MinimumDate?: string;
    MaximumDate?: string;
    CustomFieldDefaultValue?: EVA.Core.CustomFieldValue;
    IsRequired?: boolean;
    VisibleByUserTypes?: EVA.Core.UserTypes;
    EditableByUserTypes?: EVA.Core.UserTypes;
    /**
     * Entity type: Script
     */
    SecurityScriptID?: number;
    /**
     * Entity type: Script
     */
    EligibilityScriptID?: number;
  }

  export interface UnsetCustomFieldOptions {
    /**
     * Entity type: CustomField
     */
    CustomFieldID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
  }

  /**
   * List the available Cultures
   */
  export interface ListCultures {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.CultureFilter>;
  }

  export interface ListCulturesResponse {
    Result?: EVA.Core.PagedResult<ListCulturesResponse.CultureDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCulturesResponse {
    export interface CultureDto {
      ID: string;
      /**
       * Entity type: Language
       */
      LanguageID: string;
      /**
       * Entity type: Country
       */
      CountryID: string;
    }

  }

  /**
   * Create a new Culture
   */
  export interface CreateCulture {
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
  }

  export interface CreateCultureResponse {
    ID: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Delete a Culture
   */
  export interface DeleteCulture {
    ID?: string;
  }

  /**
   * Get all available languages
   */
  export interface GetLanguages {
  }

  export interface GetLanguagesResponse {
    Languages: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the organization unit countries.
   */
  export interface ListOrganizationUnitCountries {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.OrganizationUnitCountryFilter>;
  }

  export interface ListOrganizationUnitCountriesResponse {
    Result?: EVA.Core.PagedResult<ListOrganizationUnitCountriesResponse.OrganizationUnitCountryDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListOrganizationUnitCountriesResponse {
    export interface OrganizationUnitCountryDto {
      /**
       * Entity type: OrganizationUnitCountry
       */
      ID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      /**
       * Entity type: Country
       */
      CountryID: string;
      /**
       * Entity type: OrganizationUnit
       */
      InheritedFromOrganizationUnitID?: number;
      InheritedFromOrganizationUnitName: string;
    }

  }

  /**
   * Create an organization unit country
   */
  export interface CreateOrganizationUnitCountry {
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface CreateOrganizationUnitCountryResponse {
    /**
     * Entity type: OrganizationUnitCountry
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Delete an organization unit country
   */
  export interface DeleteOrganizationUnitCountry {
    /**
     * Entity type: OrganizationUnitCountry
     */
    ID: number;
  }

  /**
   * Retrieve a list of OrganizationUnit languages
   */
  export interface ListOrganizationUnitLanguages {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.OrganizationUnitLanguageFilter>;
  }

  export interface ListOrganizationUnitLanguagesResponse {
    Result?: EVA.Core.PagedResult<ListOrganizationUnitLanguagesResponse.OrganizationUnitLanguageDto>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListOrganizationUnitLanguagesResponse {
    export interface OrganizationUnitLanguageDto {
      /**
       * Entity type: OrganizationUnitLanguage
       */
      ID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      /**
       * Entity type: Language
       */
      LanguageID: string;
      /**
       * Entity type: OrganizationUnit
       */
      InheritedFromOrganizationUnitID?: number;
      InheritedFromOrganizationUnitName: string;
    }

  }

  /**
   * Attach a language directly to an organization unit.
   */
  export interface CreateOrganizationUnitLanguage {
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface CreateOrganizationUnitLanguageResponse {
    /**
     * Entity type: OrganizationUnitLanguage
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Detach a language that is directly attached to an organization unit.
   */
  export interface DeleteOrganizationUnitLanguage {
    /**
     * Entity type: OrganizationUnitLanguage
     */
    ID: number;
  }

  export interface CreateCostPriceCalculation {
    BackendID?: string;
    Name: string;
    Factor: number;
  }

  export interface CreateCostPriceCalculationResponse {
    /**
     * Entity type: CostPriceCalculation
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface DeleteCostPriceCalculation {
    /**
     * Entity type: CostPriceCalculation
     */
    ID: number;
  }

  export interface GetCostPriceCalculation {
    /**
     * Entity type: CostPriceCalculation
     */
    ID: number;
  }

  export interface GetCostPriceCalculationResponse {
    /**
     * Entity type: CostPriceCalculation
     */
    ID: number;
    BackendID?: string;
    Name: string;
    Factor: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface GetCostPriceCalculations {
  }

  export interface GetCostPriceCalculationsResponse {
    Items: GetCostPriceCalculationsResponse.Item[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetCostPriceCalculationsResponse {
    export interface Item {
      /**
       * Entity type: CostPriceCalculation
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Factor: number;
    }

  }

  export interface UpdateCostPriceCalculation {
    /**
     * Entity type: CostPriceCalculation
     */
    ID: number;
    BackendID?: string;
    Name?: string;
    Factor?: number;
  }

  /**
   * Set the value for an AppSetting
   * 
   * - OrganizationUnitID defaults to the root OrganizationUnit
   * - Name is required
   * 
   * Values for a specific App wil only be returned in the ApplicationConfiguration if the requesting UserAgent matches {App}/*
   */
  export interface SetAppSetting {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Setting are not stored on sets, the set is evaluated when this service is called, and the setting is applied to all OrganizationUnits in the set.
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Name?: string;
    Value?: string;
    App?: string;
  }

  /**
   * Set multiple AppSettings at once
   * 
   * - OrganizationUnitID defaults to the root OrganizationUnit
   * - Name is required
   * 
   * Values for a specific App wil only be returned in the ApplicationConfiguration if the requesting UserAgent matches {App}/*
   */
  export interface SetAppSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Setting are not stored on sets, the set is evaluated when this service is called, and the setting is applied to all OrganizationUnits in the set.
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Settings: SetAppSettings.Setting[];
  }

  export namespace SetAppSettings {
    export interface Setting {
      Name?: string;
      Value?: string;
      App?: string;
    }

  }

  /**
   * Removes the value for an AppSetting
   * 
   * - OrganizationUnitID is required
   * - Name is required
   */
  export interface UnsetAppSetting {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    Name: string;
    App?: string;
  }

  /**
   * List all configured AppSettings
   * 
   * Available filters
   * - OrganizationUnitID
   * - Name (partial match)
   * - App (exact match)
   */
  export interface ListAppSettings {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListAppSettingsFilter>;
  }

  export interface ListAppSettingsResponse {
    Result?: EVA.Core.PagedResult<AppSettingModel>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface AppSettingModel {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName?: string;
    Name?: string;
    Value?: string;
    App?: string;
    /**
     * Entity type: User
     */
    LastModifiedByID?: number;
    LastModifiedByFullName?: string;
    LastModificationTime?: string;
    Description?: string;
  }

  /**
   * Removes all unused settings. Set ReportOnly to let this service return all settings that would be deleted.
   */
  export interface CleanupSettings {
    ReportOnly?: boolean;
    Reasons?: EVA.Core.SettingRemovalReason[];
  }

  export interface CleanupSettingsResponse {
    Settings: EVA.Core.SettingModel[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Removes all untyped settings. Set ReportOnly to let this service return all settings that would be deleted.
   */
  export interface CleanupUntypedSettings {
    ReportOnly?: boolean;
  }

  export interface CleanupUntypedSettingsResponse {
    RemovedSettings: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Returns an Excel file containing all settings
   */
  export interface ExportSettings {
  }

  /**
   * Get the available info about appsettings that are defined in the templates repo.
   * 
   * This can be used to support some sort of autocompletion while editing appsettings.
   */
  export interface GetAppSettingsAutocompleteInfo {
  }

  export interface GetAppSettingsAutocompleteInfoResponse {
    Settings: EVA.Core.AppSettingAutocompleteInfo[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Gets the appsettings related to given `OrganizationUnitID`, optionally only those starting with `NamePrefix`
   */
  export interface GetAppSettingsForOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Name?: string;
    SortProperty?: string;
    /**
     * @default Ascending
     */
    SortDirection?: EVA.Core.SortDirection;
  }

  export interface GetAppSettingsForOrganizationUnitResponse {
    Result: AppSettingDto[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface AppSettingDto {
    Name: string;
    App?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitParentID?: number;
    OrganizationUnitName: string;
    Value: string;
    Description?: string;
    /**
     * Entity type: User
     */
    LastModifiedByID: number;
    LastModifiedByFullName?: string;
    LastModificationTime: string;
  }

  /**
   * Get all available typed settings
   * 
   * If available a Description and/or DefaultValue will be returned
   */
  export interface GetAvailableSettings {
    Functionality?: string;
  }

  export interface GetAvailableSettingsResponse {
    Settings: GetAvailableSettingsResponse.Setting[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetAvailableSettingsResponse {
    export interface Setting {
      Key: string;
      Description: string;
      DefaultValue: any;
      Sensitivity: EVA.Core.SettingSensitivityTypes;
      Type: string;
      Deprecation?: string;
      EntityType?: EVA.Core.SettingValueEntityTypes;
      AllowMultipleValues: boolean;
      RootLevelOnly: boolean;
      Functionality: string;
    }

  }

  /**
   * Yields the history of given setting.
   */
  export interface GetSettingHistory {
    Key: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface GetSettingHistoryResponse {
    Key: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    History: GetSettingHistoryResponse.Setting[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSettingHistoryResponse {
    export interface Setting {
      Value: string;
      SensitivityType: EVA.Core.SettingSensitivityTypes;
      ModifiedBy: string;
      Timestamp?: EVA.Core.DateTimeOffset;
    }

  }

  /**
   * Service for retrieving the value of a specific setting.
   */
  export interface GetSetting {
    Key: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * @default false
     */
    IncludeChildValues?: boolean;
    /**
     * @default false
     */
    FlattenChildValues?: boolean;
  }

  export interface GetSettingResponse {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    InheritedFromOrganizationUnitID?: number;
    Description?: string;
    DefaultValue?: any;
    Value?: string;
    OrganizationUnitName?: string;
    SensitivityType?: EVA.Core.SettingSensitivityTypes;
    Deprecation?: string;
    IsTypedSetting?: boolean;
    ChildValues?: GetSettingResponse.SettingChild[];
    EntityType?: EVA.Core.SettingValueEntityTypes;
    AllowMultipleValues?: boolean;
    RootLevelOnly?: boolean;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetSettingResponse {
    export interface SettingChild {
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      InheritedFromOrganizationUnitID?: number;
      Value?: string;
      OrganizationUnitName?: string;
      SensitivityType?: EVA.Core.SettingSensitivityTypes;
      ChildValues?: GetSettingResponse.SettingChild[];
      Error?: EVA.Core.ServiceError;
      Metadata?: EVA.Core.ResponseMessageMetadata;
    }

  }

  /**
   * Batch update/create settings through an Excel file (file can be retrieved by calling ExportSettings)
   */
  export interface ImportSettings {
    Data: string;
  }

  export interface ImportSettingsResponse {
    Count: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Yields the history of given app setting.
   */
  export interface ListAppSettingHistory {
    Name: string;
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListAppSettingsHistoryFilter>;
    PageToken?: string;
  }

  export interface ListAppSettingHistoryResponse {
    Results?: ListAppSettingHistoryResponse.Setting[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListAppSettingHistoryResponse {
    export interface Setting {
      Value?: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      OrganizationUnitBackendID?: string;
      OrganizationUnitName?: string;
      StartTime?: string;
      EndTime?: string;
      /**
       * Entity type: User
       */
      UserID?: number;
      UserFullName?: string;
    }

  }

  /**
   * Yields the history of given setting.
   */
  export interface ListSettingHistory {
    Key: string;
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListSettingsHistoryFilter>;
    PageToken?: string;
  }

  export interface ListSettingHistoryResponse {
    Results?: ListSettingHistoryResponse.Setting[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListSettingHistoryResponse {
    export interface Setting {
      Value?: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      OrganizationUnitBackendID?: string;
      OrganizationUnitName?: string;
      SensitivityType?: EVA.Core.SettingSensitivityTypes;
      StartTime?: string;
      EndTime?: string;
      /**
       * Entity type: User
       */
      UserID?: number;
      UserFullName?: string;
    }

  }

  /**
   * List the settings related to given `OrganizationUnitID`.
   */
  export interface ListSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Key?: string;
    TypedFilter?: ListSettingsTypedFilters;
    ShowDeprecationsOnly?: boolean;
    FunctionalityCategory?: string;
    Functionality?: string;
    SortProperty?: string;
    /**
     * @default Ascending
     */
    SortDirection?: EVA.Core.SortDirection;
  }

  export const enum ListSettingsTypedFilters {
    ShowAll = 0,
    ShowTyped = 1,
    ShowUntyped = 2
  }

  export interface ListSettingsResponse {
    Result: { [key: string]: SettingDto };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface SettingDto {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitParentID?: number;
    OrganizationUnitName: string;
    Value?: string;
    SensitivityType: EVA.Core.SettingSensitivityTypes;
    Description?: string;
    Deprecation?: string;
    IsTypedSetting: boolean;
    CanView: boolean;
    CanEdit: boolean;
    ViewFunctionality: SettingDto.FunctionalityInfo;
    EditFunctionality: SettingDto.FunctionalityInfo;
    RootLevelOnly: boolean;
    /**
     * Entity type: User
     */
    LastModifiedByID: number;
    LastModifiedByFullName: string;
    LastModificationTime: string;
  }

  export namespace SettingDto {
    export interface FunctionalityInfo {
      Categories?: string[];
      Functionality: string;
      Scope: EVA.Core.FunctionalityScope;
    }

  }

  /**
   * List all settings of the user's current organization unit and its 'children', per organization unit.
   */
  export interface ListSettingsPerOrganizationUnit {
    Key?: string;
    TypedFilter?: ListSettingsTypedFilters;
    ShowDeprecationsOnly?: boolean;
    FunctionalityCategory?: string;
    Functionality?: string;
    SortProperty?: string;
    /**
     * @default Ascending
     */
    SortDirection?: EVA.Core.SortDirection;
  }

  export interface ListSettingsPerOrganizationUnitResponse {
    /**
     * Entity type: OrganizationUnit
     */
    Result: { [key: number]: { [key: string]: SettingDto } };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Set a new value for a setting within an OrganizationUnit or apply it to all OrganizationUnits in the selected set
   */
  export interface SetSetting {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Setting are not stored on sets, the set is evaluated when this service is called, and the setting is applied to all OrganizationUnits in the set.
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Key: string;
    Value: string;
  }

  /**
   * Service for setting multiple settings at once
   * 
   * Will apply all changes or no changes in case of an error
   */
  export interface SetSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Setting are not stored on sets, the set is evaluated when this service is called, and the settings are applied to all OrganizationUnits in the set.
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Settings: SetSettings.Setting[];
  }

  export namespace SetSettings {
    export interface Setting {
      Key: string;
      Value: string;
    }

  }

  /**
   * Service for unsetting a setting
   */
  export interface UnsetSetting {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Setting are not stored on sets, the set is evaluated when this service is called, and the unsetting is applied to all OrganizationUnits in the set.
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Key: string;
    /**
     * When a Value is given, the setting will only be removed when the Value matches.
     */
    Value?: string;
  }

  /**
   * Creates a new Company, which represents a legal entity in EVA. Both Users and OrganizationUnits can have a Company that they are a part of.
   */
  export interface CreateCompany {
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCity?: string;
    /**
     * Entity type: Country
     */
    RegistrationCountryID?: string;
    /**
     * Entity type: CountrySubdivision
     */
    RegistrationCountrySubdivisionID?: string;
    VatNumber?: string;
    FiscalID?: string;
    EstablishedDate?: string;
    VisitorsAddress?: EVA.Core.AddressDataDto;
    InvoiceAddress?: EVA.Core.AddressDataDto;
    InvoiceEmailAddress?: string;
    ContactEmailAddress?: string;
    ContactPhoneNumber?: string;
    AccountHolderName?: string;
    IBAN?: string;
    BIC?: string;
    LogoID?: string;
    WasteDisposalNumber?: string;
    TaxOfficeNumber?: string;
    BackendID?: string;
    EntityType?: string;
    SocialCapital?: number;
    IndustryCode?: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  export interface CreateCompanyResponse {
    /**
     * Entity type: Company
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates a Company. Fields not provided in the request are ignored.
   */
  export interface UpdateCompany {
    /**
     * Entity type: Company
     */
    ID: number;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCity?: string;
    RegistrationCountryID?: string;
    RegistrationCountrySubdivisionID?: string;
    VatNumber?: string;
    FiscalID?: string;
    EstablishedDate?: string;
    VisitorsAddress?: EVA.Core.AddressDataDto;
    InvoiceAddress?: EVA.Core.AddressDataDto;
    InvoiceEmailAddress?: string;
    ContactEmailAddress?: string;
    ContactPhoneNumber?: string;
    AccountHolderName?: string;
    IBAN?: string;
    BIC?: string;
    LogoID?: string;
    WasteDisposalNumber?: string;
    TaxOfficeNumber?: string;
    Type?: EVA.Core.CompanyType;
    BackendID?: string;
    EntityType?: string;
    SocialCapital?: number;
    IndustryCode?: string;
    /**
     * Optional additional custom fields.
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  /**
   * Deletes a Company. If a Company is still in use by Users or OrganizationUnits this service will return an error.
   */
  export interface DeleteCompany {
    /**
     * Entity type: Company
     */
    ID: number;
  }

  /**
   * List Companies, optionally filtered.
   */
  export interface ListCompanies {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListCompaniesFilter>;
    PageToken?: string;
  }

  export interface ListCompaniesResponse {
    Results?: EVA.Core.ListCompaniesItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Get the details of a specific Company.
   */
  export interface GetCompany {
    /**
     * Entity type: Company
     */
    ID: number;
  }

  export interface GetCompanyResponse {
    /**
     * Entity type: Company
     */
    ID?: number;
    Name?: string;
    RegistrationNumber?: string;
    BackendID?: string;
    RegistrationCity?: string;
    /**
     * Entity type: Country
     */
    RegistrationCountryID?: string;
    /**
     * Entity type: CountrySubdivision
     */
    RegistrationCountrySubdivisionID?: string;
    VatNumber?: string;
    FiscalID?: string;
    EstablishedDate?: string;
    VisitorsAddress?: EVA.Core.AddressDto;
    InvoiceAddress?: EVA.Core.AddressDto;
    InvoiceEmailAddress?: string;
    ContactEmailAddress?: string;
    ContactPhoneNumber?: string;
    AccountHolderName?: string;
    IBAN?: string;
    BIC?: string;
    LogoID?: string;
    WasteDisposalNumber?: string;
    TaxOfficeNumber?: string;
    Type?: EVA.Core.CompanyType;
    EntityType?: string;
    SocialCapital?: number;
    IndustryCode?: string;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: EVA.Core.CustomFieldValueWithOptions };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new company associated with the organization unit type
   */
  export interface CreateOrganizationUnitCompany {
    BackendID?: string;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCountryID?: string;
    RegistrationCountrySubdivisionID?: string;
    TaxRegistrationNumber?: string;
    FiscalID?: string;
    TaxOfficeNumber?: string;
    SocialCapital?: number;
    EntityType?: string;
    IndustryCode?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    BillingAddress?: EVA.Core.AddressDataDto;
    ShippingAddress?: EVA.Core.AddressDataDto;
    RegistrationAddress?: EVA.Core.AddressDataDto;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  export interface CreateOrganizationUnitCompanyResponse {
    /**
     * Entity type: Company
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Retrieves the details of the organization unit company corresponding to the specified ID.
   */
  export interface GetOrganizationUnitCompany {
    /**
     * Entity type: Company
     */
    ID: number;
  }

  export interface GetOrganizationUnitCompanyResponse {
    /**
     * Entity type: Company
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCountryID?: string;
    RegistrationCountrySubdivisionID?: string;
    TaxRegistrationNumber?: string;
    FiscalID?: string;
    TaxOfficeNumber?: string;
    SocialCapital?: number;
    EntityType?: string;
    IndustryCode?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    BillingAddress?: EVA.Core.AddressDto;
    ShippingAddress?: EVA.Core.AddressDto;
    RegistrationAddress?: EVA.Core.AddressDto;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: EVA.Core.CustomFieldValueWithOptions };
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Push a company. If it doesn't exist it will be created, otherwise is will be updated.
   */
  export interface PushCompany {
    BackendID: string;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCity?: string;
    /**
     * Entity type: Country
     */
    RegistrationCountryID?: string;
    /**
     * Entity type: CountrySubdivision
     */
    RegistrationCountrySubdivisionID?: string;
    VatNumber?: string;
    FiscalID?: string;
    EstablishedDate?: string;
    VisitorsAddress?: EVA.Core.AddressDataDto;
    InvoiceAddress?: EVA.Core.AddressDataDto;
    RegistrationAddress?: EVA.Core.AddressDataDto;
    InvoiceEmailAddress?: string;
    ContactEmailAddress?: string;
    ContactPhoneNumber?: string;
    AccountHolderName?: string;
    IBAN?: string;
    BIC?: string;
    LogoID?: string;
    WasteDisposalNumber?: string;
    TaxOfficeNumber?: string;
    EntityType?: string;
    SocialCapital?: number;
    IndustryCode?: string;
    CompanyType?: EVA.Core.CompanyType;
  }

  export interface PushCompanyResponse {
    /**
     * Entity type: Company
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Push multiple companies at once.
   * If a company doesn't exist it will be created, otherwise is will be updated.
   * There is a limit (1000) of companies that can be pushed a once.
   */
  export interface PushCompanies {
    Companies: PushCompany[];
    /**
     * Enabling this will try to suppress as many event exports as possible that might result from this call.
     */
    DisableEventExports?: boolean;
  }

  export interface PushCompaniesResponse {
    SuccessfullyAdjustedCompanies: PushCompaniesResponse.SuccessfullyAdjustedCompany[];
    FailedToAdjustCompanies: PushCompaniesResponse.FailedToAdjustCompany[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace PushCompaniesResponse {
    export interface SuccessfullyAdjustedCompany {
      /**
       * Entity type: Company
       */
      ID: number;
    }

    export interface FailedToAdjustCompany {
      BackendID: string;
      ErrorIdentifier: string;
    }

  }

  /**
   * Searches for organization unit companies based on the provided query and filter criteria.
   */
  export interface SearchOrganizationUnitCompany {
    Query?: string;
    PageConfig?: EVA.Core.ScrollablePageConfig<EVA.Core.SearchCompanyFilter>;
  }

  export interface SearchOrganizationUnitCompanyResponse {
    Result: EVA.Core.ScrollablePagedResult<EVA.Core.SearchCompanyResult>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Updates the details of companies associated with the organization unit type.
   */
  export interface UpdateOrganizationUnitCompany {
    /**
     * Entity type: Company
     */
    ID: number;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCountryID?: string;
    RegistrationCountrySubdivisionID?: string;
    TaxRegistrationNumber?: string;
    FiscalID?: string;
    TaxOfficeNumber?: string;
    SocialCapital?: number;
    EntityType?: string;
    IndustryCode?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    BillingAddress?: EVA.Core.AddressDataDto;
    ShippingAddress?: EVA.Core.AddressDataDto;
    RegistrationAddress?: EVA.Core.AddressDataDto;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  export interface ValidateCompany {
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    BackendID?: string;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCity?: string;
    /**
     * Entity type: Country
     */
    RegistrationCountryID?: string;
    /**
     * Entity type: CountrySubdivision
     */
    RegistrationCountrySubdivisionID?: string;
    VatNumber?: string;
    FiscalID?: string;
    EstablishedDate?: string;
    VisitorsAddress?: EVA.Core.AddressDataDto;
    RegistrationAddress?: EVA.Core.AddressDataDto;
    InvoiceEmailAddress?: string;
    ContactEmailAddress?: string;
    ContactPhoneNumber?: string;
    AccountHolderName?: string;
    IBAN?: string;
    BIC?: string;
    LogoID?: string;
    WasteDisposalNumber?: string;
    TaxOfficeNumber?: string;
    ShowValidDetails?: boolean;
    EntityType?: string;
    SocialCapital?: number;
    IndustryCode?: string;
  }

  export interface ValidateCompanyResponse {
    IsValid: boolean;
    ValidationDetails: ValidateCompanyResponse.ValidationDetail[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ValidateCompanyResponse {
    export interface ValidationDetail {
      Field: string;
      Constraint: string;
      Message: string;
      IsValid: boolean;
    }

  }

  /**
   * List cases. Returns a paged response.
   */
  export interface ListCases {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListCasesFilter>;
    PageToken?: string;
  }

  export interface ListCasesResponse {
    Results?: ListCasesResponse.Case[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCasesResponse {
    export interface Case {
      /**
       * Entity type: Case
       */
      ID?: number;
      BackendID?: string;
      /**
       * Entity type: CaseTopic
       */
      TopicID?: number;
      TopicName?: string;
      /**
       * Entity type: CaseStatus
       */
      StatusID?: number;
      StatusName?: string;
      StatusColor?: string;
      Priority?: EVA.Core.CasePriority;
      Title?: string;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      OrderDisplayID?: string;
      ExternalOrderIdentifier?: string;
      /**
       * Entity type: User
       */
      AssigneeID?: number;
      AssigneeName?: string;
      ReportingTime?: string;
      LastInteractionTime?: string;
      /**
       * Entity type: User
       */
      CustomerID?: number;
      Customer?: ListCasesResponse.Customer;
      /**
       * Entity type: OrganizationUnit
       */
      OriginatingOrganizationUnitID?: number;
      OriginatingOrganizationUnitName?: string;
      CreationTime?: string;
      LastModificationTime?: string;
      CreatedBy?: ListCasesResponse.User;
      LastModifiedBy?: ListCasesResponse.User;
      /**
       * Entity type: OrganizationUnit
       */
      AssignedOrganizationUnitID?: number;
      AssignedOrganizationUnitName?: string;
    }

    export interface Customer {
      FirstName: string;
      LastName: string;
      EmailAddress: string;
      FullName: string;
    }

    export interface User {
      /**
       * Entity type: User
       */
      ID?: number;
      FirstName: string;
      LastName: string;
      FullName: string;
    }

  }

  /**
   * Gets a case by its ID.
   */
  export interface GetCaseByID {
    /**
     * Entity type: Case
     */
    ID: number;
  }

  export interface GetCaseByIDResponse {
    /**
     * Entity type: Case
     */
    ID: number;
    BackendID: string;
    /**
     * Entity type: CaseStatus
     */
    StatusID: number;
    StatusName: string;
    StatusColor: string;
    Priority: EVA.Core.CasePriority;
    Title: string;
    IsTitleEditable: boolean;
    Description: string;
    IsDescriptionEditable: boolean;
    Summary: string;
    SummaryLastModificationDate?: string;
    /**
     * Entity type: User
     */
    SummaryLastModifiedByID?: number;
    Solution: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    OrderDisplayID?: string;
    OrderCreationTime?: string;
    ExternalOrderIdentifier?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OriginatingOrganizationUnitID?: number;
    OriginatingOrganizationUnitName?: string;
    /**
     * Entity type: User
     */
    AssigneeID?: number;
    AssigneeName: string;
    AssigneeEmail: string;
    AssignedOn?: string;
    ReportingTime: string;
    LastInteractionTime?: string;
    ClosingTime?: string;
    /**
     * Entity type: Blob
     */
    BlobIDs: string[];
    Blobs: GetCaseByIDResponse.CaseBlobModel[];
    RelatedItems: GetCaseByIDResponse.RelatedItem[];
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: EVA.Core.CustomFieldValueWithOptions };
    Customer?: GetCaseByIDResponse.UserInfo;
    CreationTime: string;
    LastModificationTime?: string;
    CreatedBy?: GetCaseByIDResponse.User;
    LastModifiedBy?: GetCaseByIDResponse.User;
    Repairs: GetCaseByIDResponse.CaseRepair[];
    CaseTopic: GetCaseByIDResponse.CaseTopicData;
    /**
     * Entity type: OrganizationUnit
     */
    AssignedOrganizationUnitID?: number;
    AssignedOrganizationUnitName?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace GetCaseByIDResponse {
    export interface CaseTopicData {
      /**
       * Entity type: CaseTopic
       */
      TopicID?: number;
      TopicName?: string;
      TopicDisplayName?: string;
      TopicBackendID?: string;
      CaseTopic?: GetCaseByIDResponse.CaseTopicData;
    }

    export interface RelatedItem {
      /**
       * Entity type: CaseRelatedItem
       */
      ID?: number;
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      Product?: any;
      SerialNumber?: string;
      IsWarranty?: boolean;
      WarrantyExpiration?: string;
      WarrantyAdditionalInformation?: string;
    }

    export interface UserInfo {
      /**
       * Entity type: User
       */
      ID: number;
      BackendID: string;
      FirstName: string;
      LastName: string;
      EmailAddress: string;
      FullName: string;
      PhoneNumber: string;
    }

    export interface CaseRepair {
      /**
       * Entity type: Repair
       */
      ID?: number;
      CreationTime?: string;
      CompletionTime?: string;
      Status?: EVA.Core.RepairStatus;
      SerialNumber?: string;
      RepairTask?: GetCaseByIDResponse.CaseRepairTask;
      RepairAssignee?: GetCaseByIDResponse.UserInfo;
    }

    export interface CaseRepairTask {
      /**
       * Entity type: RepairTask
       */
      ID: number;
      UserTaskSubType: EVA.Core.EnumDto;
    }

    export interface User {
      /**
       * Entity type: User
       */
      ID?: number;
      FirstName: string;
      LastName: string;
      FullName: string;
    }

    export interface CaseBlobModel {
      /**
       * Entity type: Blob
       */
      ID: string;
      Url: string;
    }

  }

  /**
   * Create a new case.
   */
  export interface CreateCase {
    BackendID?: string;
    /**
     * Entity type: CaseTopic
     */
    TopicID: number;
    Title: string;
    Description: string;
    /**
     * Entity type: User
     */
    CustomerID?: number;
    ExternalOrderIdentifier?: string;
    /**
     * Can only be set once. Can be updated when not set on create.
     * Entity type: OrganizationUnit
     */
    OriginatingOrganizationUnitID?: number;
    /**
     * OU the case is assigned to. When set this will influence visibility.
     * Entity type: OrganizationUnit
     */
    AssignedOrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    AssigneeID?: number;
    Priority?: EVA.Core.CasePriority;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  export interface CreateCaseResponse {
    /**
     * Entity type: Case
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing case.
   */
  export interface UpdateCase {
    /**
     * Entity type: Case
     */
    ID: number;
    BackendID?: string;
    /**
     * Entity type: CaseTopic
     */
    TopicID?: number;
    Title?: string;
    Description?: string;
    /**
     * Entity type: User
     */
    CustomerID?: number;
    ExternalOrderIdentifier?: string;
    /**
     * Can only be set once. Can be updated when not set on create.
     * Entity type: OrganizationUnit
     */
    OriginatingOrganizationUnitID?: number;
    /**
     * OU the case is assigned to. When set this will influence visibility.
     * Entity type: OrganizationUnit
     */
    AssignedOrganizationUnitID?: number;
    /**
     * Optional additional custom fields
     */
    CustomFields?: EVA.Core.CustomFieldKeyValue[];
  }

  /**
   * Update an existing case state.
   */
  export interface UpdateCaseState {
    /**
     * Entity type: Case
     */
    ID: number;
    /**
     * Entity type: CaseStatus
     */
    StatusID: number;
    /**
     * Entity type: User
     */
    AssigneeID?: number;
    Priority?: EVA.Core.CasePriority;
    /**
     * The solution for the case
     */
    Solution?: string;
  }

  export interface UpdateCaseStateResponse {
    StatusAction: EVA.Core.CaseStatusAction;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing case summary.
   */
  export interface UpdateCaseSummary {
    /**
     * Entity type: Case
     */
    ID: number;
    /**
     * A summary of the current state of the case
     */
    Summary: string;
  }

  /**
   * Archive a case.
   */
  export interface ArchiveCase {
    /**
     * Entity type: Case
     */
    ID: number;
  }

  /**
   * Attach blob to case.
   */
  export interface AttachBlobToCase {
    /**
     * Entity type: Case
     */
    CaseID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     * @deprecated Use `BlobUrls` instead.
     */
    BlobUrl?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrls?: string[];
  }

  /**
   * Detaches and deletes a blob from a case.
   */
  export interface DeleteCaseBlob {
    /**
     * Entity type: Case
     */
    CaseID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
  }

  /**
   * List all the blobs that are attached to the case.
   */
  export interface ListBlobsForCase {
    /**
     * Entity type: Case
     */
    CaseID: number;
  }

  export interface ListBlobsForCaseResponse {
    Result: ListBlobsForCaseResponse.Blob[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListBlobsForCaseResponse {
    export interface Blob {
      Name: string;
      MimeType: string;
      BlobID: string;
      Url: string;
      CreationTime: string;
    }

  }

  /**
   * Set case related data.
   * You can provide an OrderID (optional)
   * You can provide an ExternalOrderID (optional)
   * You can provide further details by setting RelatedItems. These can either
   * - contain a OrderLineID (can only be provided if OrderID is also provided. The ProductID of the orderline will copied to the ProductID of the RelatedItem)
   * - contain a ProductID
   * - a SerialNumber (is settable whether you provided a OrderLineID or ProductID)
   * - a IsWarranty indicator (is settable when there's an associated ProductID or the orderline contains a product)
   */
  export interface SetCaseRelatedData {
    /**
     * Entity type: Case
     */
    CaseID: number;
    ExternalOrderIdentifier?: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Overwrite case customer with the customer bound to the order.
     */
    OverwriteCaseCustomerWithCustomerOfOrder?: boolean;
    RelatedItems?: SetCaseRelatedData.RelatedItem[];
  }

  export namespace SetCaseRelatedData {
    export interface RelatedItem {
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      SerialNumber?: string;
      IsWarranty?: boolean;
      WarrantyExpiration?: string;
      WarrantyAdditionalInformation: string;
    }

  }

  /**
   * Remove case related items.
   */
  export interface RemoveCaseRelatedItems {
    /**
     * Entity type: Case
     */
    CaseID?: number;
    /**
     * Entity type: CaseRelatedItem
     */
    ItemIDs?: number[];
    RemoveAll?: boolean;
  }

  /**
   * List the case interactions
   */
  export interface ListCaseInteractions {
    /**
     * Entity type: Case
     */
    CaseID: number;
    IncludeRepairInteractions?: boolean;
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListCaseInteractionsFilter>;
    PageToken?: string;
  }

  export interface ListCaseInteractionsResponse {
    Results?: ListCaseInteractionsResponse.CaseInteraction[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCaseInteractionsResponse {
    export interface CaseInteraction {
      /**
       * Entity type: Interaction
       */
      ID?: number;
      BackendID?: string;
      /**
       * Entity type: Repair
       */
      RepairID?: number;
      InteractionType?: EVA.Core.InteractionType;
      CanBeMutated?: boolean;
      Text?: string;
      CreationTime?: string;
      LastModificationTime?: string;
      CreatedBy?: ListCaseInteractionsResponse.User;
    }

    export interface User {
      /**
       * Entity type: User
       */
      CreatedByID?: number;
      FirstName: string;
      LastName: string;
      FullName: string;
    }

  }

  /**
   * Get case interaction
   */
  export interface GetCaseInteraction {
    /**
     * Entity type: Interaction
     */
    ID: number;
  }

  export interface GetCaseInteractionResponse {
    /**
     * Entity type: Interaction
     */
    ID: number;
    BackendID?: string;
    /**
     * Entity type: Case
     */
    CaseID: number;
    InteractionType: EVA.Core.InteractionType;
    CanBeMutated: boolean;
    Text: string;
    CreationTime: string;
    LastModificationTime?: string;
    BlobIDs: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new case interaction
   */
  export interface CreateCaseInteraction {
    /**
     * Entity type: Case
     */
    CaseID: number;
    InteractionType: EVA.Core.InteractionType;
    Text: string;
    BackendID?: string;
  }

  export interface CreateCaseInteractionResponse {
    /**
     * Entity type: Interaction
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a case interaction
   */
  export interface UpdateCaseInteraction {
    /**
     * Entity type: Interaction
     */
    ID: number;
    BackendID?: string;
    Text: string;
  }

  /**
   * Delete a case interaction
   */
  export interface DeleteCaseInteraction {
    /**
     * Entity type: Interaction
     */
    ID: number;
  }

  /**
   * Add a blob to a case interacion.
   */
  export interface AddBlobToCaseInteraction {
    /**
     * Entity type: Interaction
     */
    InteractionID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     * @deprecated Use `BlobUrls` instead.
     */
    BlobUrl?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrls?: string[];
  }

  /**
   * Remove a blob from a case interaction.
   */
  export interface RemoveCaseInteractionBlob {
    /**
     * Entity type: Interaction
     */
    InteractionID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
  }

  /**
   * List the case statuses
   */
  export interface ListCaseStatuses {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListCaseStatusesFilter>;
    PageToken?: string;
  }

  export interface ListCaseStatusesResponse {
    Results?: ListCaseStatusesResponse.CaseStatus[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCaseStatusesResponse {
    export interface CaseStatus {
      /**
       * Entity type: CaseStatus
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Description: string;
      Action: EVA.Core.CaseStatusAction;
      IsClosed: boolean;
      Color?: string;
    }

  }

  /**
   * Get case status
   */
  export interface GetCaseStatus {
    /**
     * Entity type: CaseStatus
     */
    ID: number;
  }

  export interface GetCaseStatusResponse {
    /**
     * Entity type: CaseStatus
     */
    ID: number;
    BackendID?: string;
    Name: string;
    Description: string;
    IsClosed: boolean;
    Action: EVA.Core.CaseStatusAction;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    IsSystem: boolean;
    Color?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new case status
   */
  export interface CreateCaseStatus {
    Name?: string;
    BackendID?: string;
    Description?: string;
    IsClosed?: boolean;
    Action?: EVA.Core.CaseStatusAction;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Color?: string;
  }

  export interface CreateCaseStatusResponse {
    /**
     * Entity type: CaseStatus
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a case status
   */
  export interface UpdateCaseStatus {
    /**
     * Entity type: CaseStatus
     */
    ID: number;
    Name?: string;
    BackendID?: string;
    Description?: string;
    IsClosed?: boolean;
    Action?: EVA.Core.CaseStatusAction;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Color?: string;
  }

  /**
   * Delete a case status
   */
  export interface DeleteCaseStatus {
    /**
     * Entity type: CaseStatus
     */
    ID: number;
  }

  /**
   * List the case topics
   */
  export interface ListCaseTopics {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListCaseTopicsFilter>;
    PageToken?: string;
  }

  export interface ListCaseTopicsResponse {
    Results?: ListCaseTopicsResponse.CaseTopic[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace ListCaseTopicsResponse {
    export interface CaseTopic {
      /**
       * Entity type: CaseTopic
       */
      ID: number;
      /**
       * Entity type: CaseTopic
       */
      ParentID?: number;
      BackendID?: string;
      Name: string;
      DisplayName: string;
      Description: string;
    }

  }

  /**
   * Get case topic
   */
  export interface GetCaseTopic {
    /**
     * Entity type: CaseTopic
     */
    ID: number;
  }

  export interface GetCaseTopicResponse {
    /**
     * Entity type: CaseTopic
     */
    ID: number;
    /**
     * Entity type: CaseTopic
     */
    ParentID?: number;
    BackendID?: string;
    Name: string;
    DisplayName: string;
    Description: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Creates a new case topic
   */
  export interface CreateCaseTopic {
    /**
     * Can only be set on creation
     * Entity type: CaseTopic
     */
    ParentID?: number;
    Name?: string;
    BackendID?: string;
    Description?: string;
  }

  export interface CreateCaseTopicResponse {
    /**
     * Entity type: CaseTopic
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update a case topic
   */
  export interface UpdateCaseTopic {
    /**
     * Entity type: CaseTopic
     */
    ID: number;
    Name?: string;
    BackendID?: string;
    Description?: string;
  }

  /**
   * Delete a case topic. NOTE THAT sub topics will also be deleted.
   */
  export interface DeleteCaseTopic {
    /**
     * Entity type: CaseTopic
     */
    ID: number;
  }

  /**
   * Push and import a case.
   */
  export interface PushCase {
    /**
     * The ID field can be used to update an existing Case (specifically to be able to update the backend ID). If the ID is provided, the Update field does not need to be set.
     * Entity type: Case
     */
    ID?: number;
    /**
     * Setting this to true will try to suppress as many event exports as possible that might result from this call.
     */
    DisableEventExports?: boolean;
    /**
     * Setting this to true will update the case information if it already exists instead of throwing an error.
     */
    Update?: boolean;
    BackendID: string;
    /**
     * The topic that this case is a part of. If a BackendID is provided and a CaseTopic exists with that ID, that CaseTopic is used, no matter what Name or ParentBackendID are passed and if they match.
     *   If no CaseTopic exists with the given BackendID, but one does exist with the given Name and ParentBackendID, that CaseTopic is used instead, disregarding what BackendID was passed in the request.
     *   If none of these conditions are met, a new CaseTopic is created.
     */
    CaseTopic: PushCase.PushCaseTopic;
    Title: string;
    Description: string;
    Summary?: string;
    Solution?: string;
    Customer?: PushCase.PushCaseCustomer;
    OriginatingOrganizationUnitBackendID?: string;
    /**
     * OU the case is assigned to. When set this will influence visibility.
     */
    AssignedOrganizationUnitBackendID?: string;
    Assignee?: PushCase.PushCaseAssignee;
    AssignedOn?: string;
    Priority?: EVA.Core.CasePriority;
    Status?: string;
    ReportingTime?: string;
    LastInteractionTime?: string;
    ClosingTime?: string;
    CustomFields?: { [key: string]: any };
    Interactions?: PushCase.PushCaseInteraction[];
    CaseRelatedItems?: PushCase.PushCaseRelatedItems;
  }

  export namespace PushCase {
    export interface PushCaseRelatedItems {
      OrderID?: string;
      Items?: PushCase.PushCaseRelatedItem[];
    }

    export interface PushCaseRelatedItem {
      OrderLineID?: string;
      ProductID?: string;
      SerialNumber?: string;
      IsWarranty?: boolean;
      WarrantyExpiration?: string;
      WarrantyAdditionalInformation?: string;
      /**
       * If specified as true, the item will be deleted.
       */
      Delete?: boolean;
    }

    export interface PushCaseCustomer {
      BackendID?: string;
      EmailAddress?: string;
      /**
       * Possible values are: M, F, O
       */
      Gender?: string;
      Initials?: string;
      FirstName?: string;
      LastName?: string;
      PhoneNumber?: string;
      DateOfBirth?: string;
      PlaceOfBirth?: string;
      BankAccount?: string;
      Nickname?: string;
      ShippingAddress?: PushCase.PushCaseAddress;
      BillingAddress?: PushCase.PushCaseAddress;
      /**
       * ISO 639-1
       * Entity type: Language
       */
      LanguageID?: string;
      /**
       * ISO 3166-1
       * Entity type: Country
       */
      CountryID?: string;
      FiscalID?: string;
      SocialSecurityNumber?: string;
      Title?: string;
      Salutation?: string;
      /**
       * Is this a Standard/Basic/Incognito account?
       * @default Standard
       */
      AccountType?: EVA.Core.UserAccountType;
      Company?: PushCase.PushCaseCustomerCompany;
      BackendRelationID?: string;
    }

    export interface PushCaseCustomerCompany {
      Name?: string;
      RegistrationNumber?: string;
      VatNumber?: string;
      FiscalID?: string;
    }

    export interface PushCaseAddress {
      Address1?: string;
      Address2?: string;
      /**
       * House number, including possible extensions
       */
      HouseNumber?: string;
      FirstName?: string;
      LastName?: string;
      ZipCode?: string;
      City?: string;
      Region?: string;
      District?: string;
      Subdistrict?: string;
      EmailAddress?: string;
      PhoneNumber?: string;
      /**
       * ISO 3166-2
       */
      State?: string;
      /**
       * ISO 3166-1
       */
      CountryID?: string;
    }

    export interface PushCaseAssignee {
      /**
       * If specified as true, the assignee will be removed from the case.
       */
      Remove?: boolean;
      /**
       * Entity type: User
       */
      ID?: number;
      CustomID?: string;
      EmailAddress?: string;
    }

    export interface PushCaseInteraction {
      BackendID: string;
      /**
       * If specified as true, the interaction will be deleted.
       */
      Delete: boolean;
      Type: EVA.Core.InteractionType;
      Text: string;
    }

    export interface PushCaseTopic {
      /**
       * Required if pushing and creating a previously non-existent CaseTopic
       */
      Name?: string;
      BackendID?: string;
      /**
       * Provide a ParentID, or nothing if the CaseTopic should be root-level.
       */
      ParentBackendID?: string;
    }

  }

  export interface PushCaseResponse {
    /**
     * Entity type: Case
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Validates the current settings and properties of an auditing provider.
   */
  export interface AuditingValidateConfiguration {
    AuditingProvider: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface AuditingConfigurationResponse {
    Success: boolean;
    Errors: string[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Resets the Financial Periods for given OU and all children to which the auditing provider applies. **Use with care!**
   */
  export interface AuditingResetFinancialPeriods {
    AuditingProvider: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    MarkAsProcessed: boolean;
  }

  /**
   * Update the private key version and reset the auditing chain.
   */
  export interface AuditingSetPrivateKey {
    PrivateKeyFile?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  /**
   * Retrieves the required auditing settings for the specified organization unit, including their value, if set.
   */
  export interface GetAuditingSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetAuditingSettingsResponse {
    AuditingSettings: AuditingSetting[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface AuditingSetting {
    Key: string;
    Name: string;
    Value: string;
    Group: string;
    Type: EVA.Core.AuditingSettingType;
  }

  /**
   * Sets the required auditing settings for the specific organization unit
   */
  export interface SetAuditingSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    AuditingSettings: { [key: string]: string };
  }

  /**
   * Auditing service for uploading your public and private key configuration
   */
  export interface AuditingUploadKeySet {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    PrivateKey: string;
    PublicKey: string;
  }

  /**
   * Set the AML rules for a specific country. When no rules are provided, the configuration will be deleted.
   */
  export interface CreateOrUpdateAMLConfiguration {
    /**
     * Entity type: Country
     */
    CountryID: string;
    Rules: AMLRuleDto[];
  }

  export interface AMLRuleDto {
    TriggerType: EVA.Core.AMLTriggerType;
    Threshold?: number;
    SlidingWindowDays?: number;
    Blocked: boolean;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    RequireIdentification: boolean;
    RequireIdentificationBlob: boolean;
    RequireCustomer: boolean;
    RequireCustomerVerification: boolean;
    RequireSurvey: boolean;
    /**
     * Entity type: Survey
     */
    SurveyID?: number;
    SystemSurvey?: EVA.Core.AMLSystemSurveys;
    RequireCustomerSignature: boolean;
    RequireEmployeeSignature: boolean;
    Foreigner: boolean;
    Business: boolean;
  }

  /**
   * List all AML configurations.
   */
  export interface ListAMLConfigurations {
    InitialPageConfig?: EVA.Core.PageTokenConfig<EVA.Core.ListConfigurationsFilter>;
    PageToken?: string;
  }

  export interface ListAMLConfigurationsResponse {
    Results?: AMLConfigurationDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export interface AMLConfigurationDto {
    /**
     * Entity type: Country
     */
    CountryID: string;
    Rules: AMLRuleDto[];
  }

  /**
   * Create a new Account to be used in Cookbook. Name and ObjectAccount are required.
   */
  export interface CreateAccount {
    Name: string;
    ObjectAccount: string;
    Subsidiary?: string;
    BookingFlags?: EVA.Core.BookingFlags;
    BackendCode?: string;
    Type?: EVA.Core.AccountType;
  }

  export interface CreateAccountResponse {
    /**
     * Entity type: Account
     */
    ID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Update an existing Account. Make sure all properties are filled.
   */
  export interface UpdateAccount {
    /**
     * Entity type: Account
     */
    ID: number;
    Name?: string;
    ObjectAccount?: string;
    Subsidiary?: string;
    BookingFlags?: EVA.Core.BookingFlags;
    BackendCode?: string;
    Type?: EVA.Core.AccountType;
  }

  /**
   * Delete an existing Account
   */
  export interface DeleteAccount {
    /**
     * Entity type: Account
     */
    ID: number;
  }

  /**
   * Get all the properties of an existing Account
   */
  export interface GetAccount {
    /**
     * Entity type: Account
     */
    ID: number;
  }

  export interface GetAccountResponse {
    /**
     * Entity type: Account
     */
    ID: number;
    Name: string;
    ObjectAccount: string;
    Subsidiary?: string;
    BookingFlags: EVA.Core.BookingFlags;
    BackendCode?: string;
    Type: EVA.Core.AccountType;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * List the active accounts
   * 
   * Available filters:
   * - Name
   * - ObjectAccount
   */
  export interface ListAccounts {
    PageConfig?: EVA.Core.PageConfig<EVA.Core.ListAccountsFilter>;
  }

  export interface ListAccountsResponse {
    Result?: EVA.Core.PagedResult<EVA.Core.ListAccountsItem>;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Push a LoyaltyProgram.
   * If it doesn't exist it will be created, otherwise is will be updated.
   * Sub entities will also be created or updated, but will not be removed.
   */
  export interface PushLoyaltyProgram {
    ID: string;
    LoyaltyProgramGroupID?: string;
    /**
     * Attaches the loyalty program to a consumer company.
     */
    CompanyID?: string;
    Name?: string;
    Description?: string;
    Options?: EVA.Core.LoyaltyProgramOptions;
    /**
     * Entity type: Currency
     */
    BudgetCurrencyID?: string;
    Budget?: number;
    DeactivateWhenBudgetIsReached?: boolean;
    ReactivateProgramWhenPossible?: boolean;
    StartDate?: string;
    EndDate?: string;
    Status?: EVA.Core.LoyaltyProgramStatus;
    Handler?: string;
    Data?: any;
    ReloadPolicy?: string;
    LoyaltyPointUsageOptions?: EVA.Core.LoyaltyPointUsageOptions;
    PointPendingPolicyType?: string;
    PointPendingPolicyData?: { [key: string]: any };
    PointExpirationPolicyType?: string;
    PointExpirationPolicyData?: { [key: string]: any };
    /**
     * When true, the loyalty program will only be applied when a customer is attached to the order.
     * 
     * When false, the loyalty program can be attached to the order even if there is no customer attached.
     */
    RequireCustomer?: boolean;
    /**
     * When true, the loyalty program will only be applied when a customer is attached to the order that is already subscribed to the loyalty program.
     * Also, this will make the `RequireCustomer` automatically true and `AutoSubscribeCustomer` false.
     * 
     * When false, the loyalty program can be attached to the order if a customer is attached that isn't already subscribed to the loyalty program.
     */
    RequiresSubscribedCustomer?: boolean;
    /**
     * When true, if a non anonymous customer is attached to the order they will be automatically subscribed on finishing the order.
     * When false, the customer will not be subscribed.
     * 
     * This is ignore when`RequiresSubscribedCustomer` or `RequiresSubscriptionValidation` is set to true.
     */
    AutoSubscribeCustomer?: boolean;
    /**
     * When true, the order will require that the attached customer on the order will be validated by providing the UserIdentifier in the `AttachBenefitProgramToOrder` service.
     * Also, this will make the `RequireCustomer` and `RequiresSubscribedCustomer` automatically true and `AutoSubscribeCustomer` false.
     * 
     * When false, this validation isn't required.
     */
    RequiresSubscriptionValidation?: boolean;
    UserMaximumUsage?: number;
    UserBudget?: number;
    /**
     * When set to true, subscribing the user will require a UserIdentifier to be given.
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * When this is not the case on create, this can be introduced on update. This value cannot be changed after it is set in either the create or update.
     */
    RequireManualUserIdentifier?: boolean;
    /**
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * When this is not the case on create, this can be introduced on update. This value cannot be changed after it is set in either the create or update.
     */
    UserIdentifierPrefix?: string;
    /**
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * When this is not the case on create, this can be introduced on update. This value cannot be changed after it is set in either the create or update.
     * Min: 0
     */
    UserIdentifierStartNumber?: number;
    /**
     * Attention!
     * This value will be used to create a subscription when either `RequiredCustomer` or `AutoSubscribeCustomer` is set to true.
     * When this is not the case on create, this can be introduced on update. This value cannot be changed after it is set in either the create or update.
     */
    UserIdentifierSuffix?: string;
    /**
     * Handling when a user is unsubscribed and re-subscribes.
     */
    ResubscribeHandling?: EVA.Core.ResubscribeHandlingType;
    /**
     * Indicates if the subscription can be unsubscribed and if so, who can do the unsubscribe.
     */
    AllowUnsubscribeBy?: EVA.Core.AllowUnsubscribeByType;
    OrganizationUnitSets?: PushLoyaltyProgram.PushLoyaltyProgramOrganizationUnitSet[];
    PriceLists?: string[];
    Conditions?: PushLoyaltyProgram.PushLoyaltyProgramCondition[];
    PaymentTypes?: PushLoyaltyProgram.PushLoyaltyProgramPaymentType[];
    ProductLimitations?: PushLoyaltyProgram.PushLoyaltyProgramProductLimitation[];
    PointAwardingRules?: PushLoyaltyProgram.PushPointAwardingRule[];
    RequiredCustomFields?: string[];
  }

  export namespace PushLoyaltyProgram {
    export interface PushLoyaltyProgramOrganizationUnitSet {
      OrganizationUnitSetID: string;
      SubscriptionConfirmation?: EVA.Core.SubscriptionConfirmation;
      Enabled: boolean;
      Hidden: boolean;
    }

    export interface PushLoyaltyProgramCondition {
      ID: string;
      Type: string;
      Data: { [key: string]: any };
    }

    export interface PushLoyaltyProgramPaymentType {
      ID: string;
      /**
       * Min: 0
       */
      Sequence?: number;
      Required: boolean;
      AllowPartialPayment: boolean;
      PaymentTypeID: string;
      TaxHandling: EVA.Core.LoyaltyPaymentTaxHandlingType;
      Budget?: number;
      ExcludeFromProgramBudget?: boolean;
      /**
       * Min: 0
       */
      UserMaximumUsage?: number;
      UserBudgetDeduction?: EVA.Core.UserBudgetDeduction;
      /**
       * Min: 0
       */
      UserBudget?: number;
      /**
       * Min: 0
       */
      MaximumAmountPerOrder?: number;
      /**
       * Range: 0-100
       */
      MaximumPercentageOfUserBudgetPerOrder?: number;
      ProductLimitation?: PushLoyaltyProgramPaymentType.ProductLimitationData;
    }

    export namespace PushLoyaltyProgramPaymentType {
      export interface ProductLimitationData {
        ProductIDs?: string[];
        ProductSearchID?: string;
        Filters?: { [key: string]: EVA.Core.FilterModel };
        /**
         * The quantity limit over the entire product limitation product set per user.
         * Min: 1
         */
        OverallQuantityLimitPerUser?: number;
        /**
         * The quantity limit over the entire product limitation product set per order.
         * Min: 1
         */
        OverallQuantityLimitPerOrder?: number;
        /**
         * The quantity limit per product within the product limitation product set per user.
         * Min: 1
         */
        QuantityLimitPerUser?: number;
        /**
         * The quantity limit per product within the product limitation product set per order.
         * Min: 1
         */
        QuantityLimitPerOrder?: number;
      }

    }

    export interface PushLoyaltyProgramProductLimitation {
      ID: string;
      Name: string;
      ApplyToPriceList: boolean;
      ProductIDs?: string[];
      ProductSearchID?: string;
      Filters?: { [key: string]: EVA.Core.FilterModel };
      /**
       * Min: 0
       */
      QuantityLimitPerUser?: number;
      /**
       * Min: 0
       */
      OverallQuantityLimitPerUser?: number;
      /**
       * Min: 0
       */
      QuantityLimitPerOrder?: number;
      /**
       * Min: 0
       */
      OverallQuantityLimitPerOrder?: number;
    }

    export interface PushPointAwardingRule {
      ID?: string;
      OrganizationUnitSetID?: string;
      Name?: string;
      Description?: string;
      MarketingDescription?: string;
      Enabled?: boolean;
      StartDate?: string;
      EndDate?: string;
      ApplyRecursive?: boolean;
      PointsToAward?: number;
      PointPendingPolicyType?: string;
      PointPendingPolicyData?: any;
      PointExpirationPolicyType?: string;
      PointExpirationPolicyData?: any;
      Conditions?: PushLoyaltyProgram.PushPointAwardingRuleCondition[];
    }

    export interface PushPointAwardingRuleCondition {
      ID: string;
      Type: string;
      Data?: any;
    }

  }

  export interface PushLoyaltyProgramResponse {
    /**
     * Entity type: LoyaltyProgram
     */
    ID: number;
    /**
     * Entity type: Subscription
     */
    SubscriptionID: number;
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  /**
   * Push multiple loyalty programs at once.
   * If a loyalty program doesn't exist it will be created, otherwise it will be updated.
   * Sub entities will also be created or updated, but will not be removed.
   * There is a limit (1000) of loyalty programs that can be pushed a once.
   */
  export interface PushLoyaltyPrograms {
    LoyaltyPrograms: PushLoyaltyProgram[];
    /**
     * Enabling this will try to suppress as many event exports as possible that might result from this call.
     */
    DisableEventExports?: boolean;
  }

  export interface PushLoyaltyProgramsResponse {
    SuccessfullyAdjustedLoyaltyPrograms: PushLoyaltyProgramsResponse.SuccessfullyAdjustedLoyaltyProgram[];
    FailedToAdjustLoyaltyPrograms: PushLoyaltyProgramsResponse.FailedToAdjustLoyaltyProgram[];
    Error?: EVA.Core.ServiceError;
    Metadata?: EVA.Core.ResponseMessageMetadata;
  }

  export namespace PushLoyaltyProgramsResponse {
    export interface SuccessfullyAdjustedLoyaltyProgram {
      /**
       * Entity type: LoyaltyProgram
       */
      ID: number;
    }

    export interface FailedToAdjustLoyaltyProgram {
      BackendID: string;
      ErrorIdentifier: string;
    }

    export interface PushLoyaltyProgramService {
    }

  }

}
