declare module EVA.Core {

  export namespace Errors {
    export const enum AsyncResult {
      /**
       * JobID cannot be null
       */
      JobIDRequired = 'AsyncResult:JobIDRequired'
    }

    export const enum MessageMiddleware {
      /**
       * Service call was locked
       */
      ServiceLocked = 'MessageMiddleware:ServiceLocked'
    }

    export const enum Wishlist {
      /**
       * Name must be unique. '{0:string}' already exists.
       */
      NameMustBeUnique = 'Wishlist:NameMustBeUnique',
      /**
       * Cannot attach wishlist to employee or api.
       */
      CannotAttachWishlistToEmployeeOrApi = 'Wishlist:CannotAttachWishlistToEmployeeOrApi',
      /**
       * Cannot add more than {0:number} products to a wishlist.
       */
      CannotAddMoreThanXProductsToWishlist = 'Wishlist:CannotAddMoreThanXProductsToWishlist',
      /**
       * Cannot provide more then {0:number} email addresses to share to.
       */
      CannotProvideMoreThenXEmailAddressesToShareTo = 'Wishlist:CannotProvideMoreThenXEmailAddressesToShareTo',
      /**
       * You are already linked to this wishlist.
       */
      AlreadyLinkedToWishlist = 'Wishlist:AlreadyLinkedToWishlist'
    }

    export const enum VisibilityGroups {
      /**
       * The default visibility group cannot be mutated.
       */
      DefaultIsImmutable = 'VisibilityGroups:DefaultIsImmutable',
      /**
       * Name '{0}' already exists.
       */
      NameAlreadyExists = 'VisibilityGroups:NameAlreadyExists',
      /**
       * The selected visibility group is not visible for the current user.
       */
      SelectedVisibilityGroupIsNotVisibleForUser = 'VisibilityGroups:SelectedVisibilityGroupIsNotVisibleForUser',
      /**
       * There is already a User with the same EmailAddress in the selected visibility group.
       */
      EmailAddressAlreadyInUse = 'VisibilityGroups:EmailAddressAlreadyInUse',
      /**
       * There is already a User with the same Nickname in the selected visibility group.
       */
      NicknameAlreadyInUse = 'VisibilityGroups:NicknameAlreadyInUse'
    }

    export const enum FullStockCount {
      /**
       * There is currently a Full Stock Count in progress.
       */
      CountInProgress = 'FullStockCount:CountInProgress',
      /**
       * An active/uncompleted (External) FullStockCount already exists.
       */
      FullStockCountAlreadyExists = 'FullStockCount:FullStockCountAlreadyExists',
      /**
       * Full Stock Count is not available for this action.
       */
      CountNotAvailable = 'FullStockCount:CountNotAvailable',
      /**
       * Full Stock Count label is not available for this action.
       */
      LabelNotAvailable = 'FullStockCount:LabelNotAvailable',
      /**
       * Specify a FullStockCountID or FullStockCountLabelID.
       */
      SpecifyCountOrLabel = 'FullStockCount:SpecifyCountOrLabel',
      /**
       * Please recount the specific product and that product only.
       */
      IncorrectProduct = 'FullStockCount:IncorrectProduct',
      /**
       * Recount must be performed by another user.
       */
      RecountMustBePerformedByAnotherUser = 'FullStockCount:RecountMustBePerformedByAnotherUser',
      /**
       * The is no shelf planning present.
       */
      NoShelfPlanningPresent = 'FullStockCount:NoShelfPlanningPresent',
      /**
       * Full stock count hasn't started yet.
       */
      FullStockCountHasntStartedYet = 'FullStockCount:FullStockCountHasntStartedYet',
      /**
       * Label {0:string} cannot be used for full stock count.
       */
      LabelCannotBeUsedForFullStockCount = 'FullStockCount:LabelCannotBeUsedForFullStockCount',
      /**
       * Cannot recount products when there are still labels open.
       */
      CannotRecountProductsWhenThereAreUncompletedLabels = 'FullStockCount:CannotRecountProductsWhenThereAreUncompletedLabels',
      /**
       * Cannot recount StockLabels {0:string} because there a no FullStockCountLabels present for these StockLabels.
       */
      CannotRecountStockLabelsWithoutFullStockCountLabels = 'FullStockCount:CannotRecountStockLabelsWithoutFullStockCountLabels',
      /**
       * Cannot recount uncompleted label
       */
      CannotRecountUncompletedLabel = 'FullStockCount:CannotRecountUncompletedLabel',
      /**
       * Cannot recount label with uncompleted recounts
       */
      CannotRecountLabelWithUncompletedRecounts = 'FullStockCount:CannotRecountLabelWithUncompletedRecounts',
      /**
       * Recount label {labelID:number} refers to non exisisting origin label {originLabelID:number}
       */
      RecountLabelRefersToANonExisistingOriginLabel = 'FullStockCount:RecountLabelRefersToANonExisistingOriginLabel',
      /**
       * Label only accepts counts from predefined products within the label.
       */
      LabelOnlyAcceptsPredefinedProducts = 'FullStockCount:LabelOnlyAcceptsPredefinedProducts',
      /**
       * Cannot add label to an external full stock count. Use either the 'SetExternalFullStockCountResult' or 'UploadExternalFullStockCountResult' service to set the results of the external full stock count.
       */
      CannotAddLabelToExternalFullStockCount = 'FullStockCount:CannotAddLabelToExternalFullStockCount',
      /**
       * External full stock counts cannot be recounted. Use either the 'SetExternalFullStockCountResult' or 'UploadExternalFullStockCountResult' service to set the results of the external full stock count.
       */
      ExternalFullStockCountsCannotBeRecounted = 'FullStockCount:ExternalFullStockCountsCannotBeRecounted',
      /**
       * The external full stock count reached the completion threshold. Result will no longer be accepted and task can only be cancelled.
       */
      FullStockCountCompletionThresholdReached = 'FullStockCount:FullStockCountCompletionThresholdReached',
      /**
       * External full stock counts cannot be started.
       */
      ExternalFullStockCountsCannotBeStarted = 'FullStockCount:ExternalFullStockCountsCannotBeStarted',
      /**
       * Can only upload results to external full stock counts.
       */
      CannotUploadResultToANonExternalFullStockCount = 'FullStockCount:CannotUploadResultToANonExternalFullStockCount',
      /**
       * External full stock count already completed.
       */
      ExternalFullStockCountAlreadyCompleted = 'FullStockCount:ExternalFullStockCountAlreadyCompleted',
      /**
       * External full stock count cancelled.
       */
      ExternalFullStockCountCancelled = 'FullStockCount:ExternalFullStockCountCancelled',
      /**
       * External full stock count results still processing.
       */
      ExternalFullStockCountDataStillProcessing = 'FullStockCount:ExternalFullStockCountDataStillProcessing',
      /**
       * Cannot complete FullStockCount with active FullStockCountLabel tasks.
       */
      CannotCompleteFullStockCountWithActiveFullStockCountLabelTasks = 'FullStockCount:CannotCompleteFullStockCountWithActiveFullStockCountLabelTasks',
      /**
       * Either `RecountItems` or `ProductIDs` is required.
       */
      EitherRecountItemsOrProductIDsIsRequired = 'FullStockCount:EitherRecountItemsOrProductIDsIsRequired',
      /**
       * Full stock count {fullStockCountID:number} stock snapshot blob '{blobID:string}' was no longer present. Could not process the full stock count.
       */
      StockSnapshotExpired = 'FullStockCount:StockSnapshotExpired',
      /**
       * Negative quantity is forbidden
       */
      NegativeQuantityForbidden = 'FullStockCount:NegativeQuantityForbidden',
      /**
       * Cannot provide multiple serial number products in a single AddProductToFullStockCountLabel call.
       */
      CannotProvideMultipleSerialNumberProducts = 'FullStockCount:CannotProvideMultipleSerialNumberProducts',
      /**
       * Cannot mutate more than a single serial number in a single call.
       */
      CannotMutateMoreThanSingleSerialNumber = 'FullStockCount:CannotMutateMoreThanSingleSerialNumber',
      /**
       * Serial number {0:string} has already been counted as part of this full stock count.
       */
      SerialNumberAlreadyCounted = 'FullStockCount:SerialNumberAlreadyCounted'
    }

    export const enum ConsolidatedPickTask {
      /**
       * There are no valid ship from store tasks to consolidate
       */
      NoShipFromStoreTasks = 'ConsolidatedPickTask:NoShipFromStoreTasks',
      /**
       * The provided organization unit could not be found
       */
      OrganizationUnitNotFound = 'ConsolidatedPickTask:OrganizationUnitNotFound',
      /**
       * A ConsolidatedPickTask is already in progress for OU '{0:number}' with user task ID '{1:number}'. Only one task can be active at a time.
       */
      ConsolidatedPickTaskInProgress = 'ConsolidatedPickTask:ConsolidatedPickTaskInProgress'
    }

    export const enum UserFieldRequirements {
      /**
       * Requirements for field are not met.
       */
      RequirementsNotMet = 'UserFieldRequirements:RequirementsNotMet'
    }

    export const enum Subscriptions {
      /**
       * A subscription with this name already exists
       */
      DuplicateName = 'Subscriptions:DuplicateName',
      /**
       * A subscription with this BackendID already exists
       */
      DuplicateBackendID = 'Subscriptions:DuplicateBackendID',
      /**
       * User '{0:number}' is not or no longer subscribed to subscription {1:number}
       */
      UserIsNotSubscribed = 'Subscriptions:UserIsNotSubscribed',
      /**
       * Cannot delete subscription because there are active subscriptions on it.
       */
      CannotDeleteBecauseOfActiveSubscriptions = 'Subscriptions:CannotDeleteBecauseOfActiveSubscriptions',
      /**
       * Loyalty program is required for loyalty subscription
       */
      LoyaltyProgramIsRequiredForLoyaltySubscription = 'Subscriptions:LoyaltyProgramIsRequiredForLoyaltySubscription',
      /**
       * Loyalty program is invalid
       */
      InvalidLoyaltyProgramProvidedForSubscription = 'Subscriptions:InvalidLoyaltyProgramProvidedForSubscription',
      /**
       * Can only set loyalty program when handler is loyalty handler
       */
      CanOnlySetLoyaltyProgramWhenHandlerIsLoyaltyHandler = 'Subscriptions:CanOnlySetLoyaltyProgramWhenHandlerIsLoyaltyHandler',
      /**
       * Subscription handler does not allow user identifier input
       */
      SubscriptionHandlerDoesNotAllowUserIdentifierInput = 'Subscriptions:SubscriptionHandlerDoesNotAllowUserIdentifierInput',
      /**
       * Subscription does not allow user identifier input
       */
      SubscriptionDoesNotAllowUserIdentifierInput = 'Subscriptions:SubscriptionDoesNotAllowUserIdentifierInput',
      /**
       * Loyalty handler does not allow user identifier input
       */
      LoyaltyHandlerDoesNotAllowUserIdentifierInput = 'Subscriptions:LoyaltyHandlerDoesNotAllowUserIdentifierInput',
      /**
       * Unknown loyalty program
       */
      UnknownLoyaltyProgram = 'Subscriptions:UnknownLoyaltyProgram',
      /**
       * Subscription requires either a manual user identifier or a policy to generate one
       */
      SubscriptionRequiresEitherAManualUserIdentifierOrAPolicyToGenerateOne = 'Subscriptions:SubscriptionRequiresEitherAManualUserIdentifierOrAPolicyToGenerateOne',
      /**
       * Cannot combine manual user identifier and user identifier generation
       */
      CannotCombineManualUserIdentifierAndUserIdentifierGeneration = 'Subscriptions:CannotCombineManualUserIdentifierAndUserIdentifierGeneration',
      /**
       * Start number is required for user identifier generation
       */
      StartNumberIsRequiredForUserIdentifierGeneration = 'Subscriptions:StartNumberIsRequiredForUserIdentifierGeneration',
      /**
       * The user should have an EmailAddress to subscribe to a newsletter.
       */
      EmailAddressRequired = 'Subscriptions:EmailAddressRequired',
      /**
       * Subscription handler does not allow confirmation
       */
      SubscriptionHandlerDoesNotAllowConfirmation = 'Subscriptions:SubscriptionHandlerDoesNotAllowConfirmation',
      /**
       * {0:string} is not a supported confirmation method.
       */
      UnsupportedSubscriptionConfirmationMethod = 'Subscriptions:UnsupportedSubscriptionConfirmationMethod',
      /**
       * A loyalty program cannot be attached to multiple subscriptions
       */
      ALoyaltyProgramCannotBeAttachedToMultipleSubscriptions = 'Subscriptions:ALoyaltyProgramCannotBeAttachedToMultipleSubscriptions'
    }

    export const enum LoyaltyPrograms {
      /**
       * Handler for subscription {0:number} is unknown.
       */
      UnknownSubscriptionHandler = 'LoyaltyPrograms:UnknownSubscriptionHandler',
      /**
       * Subscription is fully external and cannot be pushed.
       */
      ExternalSubscriptionsCannotBePushed = 'LoyaltyPrograms:ExternalSubscriptionsCannotBePushed',
      /**
       * Subscription is not a loyalty subscription so it cannot accept Balance.
       */
      SubscriptionIsNotALoyaltySubscription = 'LoyaltyPrograms:SubscriptionIsNotALoyaltySubscription',
      /**
       * Unable to unsubscribe user.
       */
      UnableToUnsubscribeUser = 'LoyaltyPrograms:UnableToUnsubscribeUser',
      /**
       * Only loyalty programs with the `EVALOYALTY` handler can be provided with a balance.
       */
      OnlyEVALoyaltyHandlerAcceptsBalance = 'LoyaltyPrograms:OnlyEVALoyaltyHandlerAcceptsBalance',
      /**
       * Cannot deposit negative balance.
       */
      CannotDepositNegativeBalance = 'LoyaltyPrograms:CannotDepositNegativeBalance',
      /**
       * Handler is required
       */
      MissingHandler = 'LoyaltyPrograms:MissingHandler',
      /**
       * The given handler doesn't exist
       */
      UnknownHandler = 'LoyaltyPrograms:UnknownHandler',
      /**
       * The data is not in a valid format for the selected handler
       */
      InvalidHandlerData = 'LoyaltyPrograms:InvalidHandlerData',
      /**
       * Loyalty program is not set to 'Awarding'.
       */
      UsageTypeIsNotSetToAwarding = 'LoyaltyPrograms:UsageTypeIsNotSetToAwarding',
      /**
       * Handler does not allow deposits.
       */
      HandlerDoesNotAllowDeposits = 'LoyaltyPrograms:HandlerDoesNotAllowDeposits',
      /**
       * Handler does not allow withdrawals.
       */
      HandlerDoesNotAllowWithdrawals = 'LoyaltyPrograms:HandlerDoesNotAllowWithdrawals',
      /**
       * Point deposit failed.
       */
      PointDepositFailed = 'LoyaltyPrograms:PointDepositFailed',
      /**
       * Point withdraw failed.
       */
      PointWithdrawFailed = 'LoyaltyPrograms:PointWithdrawFailed',
      /**
       * Cannot delete loyalty program because it still has active subscriptions and the deletion isn't forced.
       */
      CannotDeleteLoyaltyProgramBecauseOfActiveSubscriptions = 'LoyaltyPrograms:CannotDeleteLoyaltyProgramBecauseOfActiveSubscriptions',
      /**
       * Cannot change budget currency.
       */
      CannotChangeBudgetCurrency = 'LoyaltyPrograms:CannotChangeBudgetCurrency',
      /**
       * Cannot have multiple price list of the same currency.
       */
      CannotHaveMultiplePriceListOfTheSameCurrency = 'LoyaltyPrograms:CannotHaveMultiplePriceListOfTheSameCurrency',
      /**
       * Can only connect a payment type once.
       */
      CanOnlyConnectAPaymentTypeOnce = 'LoyaltyPrograms:CanOnlyConnectAPaymentTypeOnce',
      /**
       * Only payment types bound to the loyalty program payment method are allowed.
       */
      OnlyPaymentTypesBoundToTheLoyaltyProgramPaymentMethodAreAllowed = 'LoyaltyPrograms:OnlyPaymentTypesBoundToTheLoyaltyProgramPaymentMethodAreAllowed',
      /**
       * Condition {0:string} already present. Can only add a condition once.
       */
      LoyaltyProgramConditionAlreadyPresent = 'LoyaltyPrograms:LoyaltyProgramConditionAlreadyPresent',
      /**
       * Condition {ConditionID:number} (type '{Name:string}') for loyalty program {ProgramID:number} was malformed or its build-up resulted in an exception.
       */
      InvalidLoyaltyProgramCondition = 'LoyaltyPrograms:InvalidLoyaltyProgramCondition',
      /**
       * Data for condition {ConditionID:number} (type '{Name:string}') for loyalty program {ProgramID:number} was not initialized.
       */
      LoyaltyProgramConditionDataNotInitialized = 'LoyaltyPrograms:LoyaltyProgramConditionDataNotInitialized',
      /**
       * Condition {conditionID:number} for loyalty point awarding rule {ruleID:number} was malformed or its build-up resulted in an exception.
       */
      InvalidPointAwardingCondition = 'LoyaltyPrograms:InvalidPointAwardingCondition',
      /**
       * No products are provided
       */
      NoProductsGiven = 'LoyaltyPrograms:NoProductsGiven',
      /**
       * Value of property `{0:string}` must be higher than or equal to 0.
       */
      ValueMustBePositive = 'LoyaltyPrograms:ValueMustBePositive',
      /**
       * Value of property {PropertyNameX:string} must be equal to or larger than property {PropertyNameY:string}
       */
      ValueOfXMustBeLargerThanOrEqualToValueOfY = 'LoyaltyPrograms:ValueOfXMustBeLargerThanOrEqualToValueOfY',
      /**
       * Value of property {PropertyNameX:string} must be larger than property {PropertyNameY:string}
       */
      ValueOfXMustBeLargerThanValueOfY = 'LoyaltyPrograms:ValueOfXMustBeLargerThanValueOfY',
      /**
       * Can only set one product condition on a rule.
       */
      CanOnlySetOnePointAwardingProductCondition = 'LoyaltyPrograms:CanOnlySetOnePointAwardingProductCondition',
      /**
       * The requested BackendID is already in use
       */
      DuplicateBackendID = 'LoyaltyPrograms:DuplicateBackendID',
      /**
       * Attached company is required to be a consumer company.
       */
      ConsumerCompanyRequired = 'LoyaltyPrograms:ConsumerCompanyRequired',
      /**
       * Either property ApplyToPriceList, ProductIDs, ProductSearchID, or Filters is required.
       */
      RequireAtLeaseOneProductLimitationProperty = 'LoyaltyPrograms:RequireAtLeaseOneProductLimitationProperty',
      /**
       * Properties ApplyToPriceList, ProductIDs, ProductSearchID, and Filters cannot be combined.
       */
      ProductLimitationPropertiesAreMutuallyExclusive = 'LoyaltyPrograms:ProductLimitationPropertiesAreMutuallyExclusive',
      /**
       * Can only create one product limitation for now.
       */
      CanOnlyCreateOneProductLimitationForNow = 'LoyaltyPrograms:CanOnlyCreateOneProductLimitationForNow',
      /**
       * Cannot create a limitless limitation when pricelists are applied.
       */
      CannotCreateALimitlessLimitationWhenPriceListsAreApplied = 'LoyaltyPrograms:CannotCreateALimitlessLimitationWhenPriceListsAreApplied',
      /**
       * Can only use cached product search templates (static or periodically refreshed).
       */
      CanOnlyUseCachedProductSearchTemplates = 'LoyaltyPrograms:CanOnlyUseCachedProductSearchTemplates',
      /**
       * `UserID` is required when providing the `UserIdentifier`.
       */
      UserIDIsRequiredWhenProvidingTheUserIdentifier = 'LoyaltyPrograms:UserIDIsRequiredWhenProvidingTheUserIdentifier',
      /**
       * Can only attach this loyalty program to an order when either the order already has a customer or the customer is provided in the request.
       */
      LoyaltyProgramRequiresCustomerToBeAttachedToTheOrder = 'LoyaltyPrograms:LoyaltyProgramRequiresCustomerToBeAttachedToTheOrder',
      /**
       * Cannot attach loyalty program to (partially) paid or completed order.
       */
      CannotAttachLoyaltyProgramToPartiallyPaidOrCompletedOrder = 'LoyaltyPrograms:CannotAttachLoyaltyProgramToPartiallyPaidOrCompletedOrder',
      /**
       * Cannot detach loyalty program from (partially) paid or completed order.
       */
      CannotDetachLoyaltyProgramFromPartiallyPaidOrCompletedOrder = 'LoyaltyPrograms:CannotDetachLoyaltyProgramFromPartiallyPaidOrCompletedOrder',
      /**
       * Can only attach this loyalty program to an order when that has a customer that is subscribed to the loyalty program.
       */
      LoyaltyProgramsRequiresSubscribedCustomerToBeAttachedToTheOrder = 'LoyaltyPrograms:LoyaltyProgramsRequiresSubscribedCustomerToBeAttachedToTheOrder',
      /**
       * Order does not meet the condition `{0:string}` of the loyalty program.
       */
      OrderDoesNotMeetConditionOfTheLoyaltyProgram = 'LoyaltyPrograms:OrderDoesNotMeetConditionOfTheLoyaltyProgram',
      /**
       * Either LoyaltyProgramID or SubscriptionID is required
       */
      EitherLoyaltyProgramIDOrSubscriptionIDIsRequired = 'LoyaltyPrograms:EitherLoyaltyProgramIDOrSubscriptionIDIsRequired',
      /**
       * LoyaltyProgram does not match given SubscriptionID
       */
      LoyaltyProgramDoesNotMatchGivenSubscriptionID = 'LoyaltyPrograms:LoyaltyProgramDoesNotMatchGivenSubscriptionID',
      /**
       * LoyaltyProgram has required payment types that need to be selected/used.
       */
      LoyaltyProgramHasRequiredPaymentTypes = 'LoyaltyPrograms:LoyaltyProgramHasRequiredPaymentTypes',
      /**
       * Cannot update the type of a loyalty program condition.
       */
      CannotUpdateConditionType = 'LoyaltyPrograms:CannotUpdateConditionType',
      /**
       * Can only require order custom fields.
       */
      CanOnlyRequireOrderCustomFields = 'LoyaltyPrograms:CanOnlyRequireOrderCustomFields',
      /**
       * Push loyalty programs limit exceeded.
       */
      PushLoyaltyProgramsLimitExceeded = 'LoyaltyPrograms:PushLoyaltyProgramsLimitExceeded',
      /**
       * Unknown error while pushing loyalty program with BackendID `{0:string}`
       */
      UnknownPushLoyaltyProgramError = 'LoyaltyPrograms:UnknownPushLoyaltyProgramError'
    }

    export const enum UserSubscriptions {
      /**
       * User identifier required.
       */
      UserIdentifierRequired = 'UserSubscriptions:UserIdentifierRequired',
      /**
       * Cannot provide user identifier when user identifier generation is enabled.
       */
      CannotProvideUserIdentifierWhenUserIdentifierGenerationIsEnabled = 'UserSubscriptions:CannotProvideUserIdentifierWhenUserIdentifierGenerationIsEnabled',
      /**
       * UserIdentifier already exists.
       */
      UserIdentifierAlreadyExists = 'UserSubscriptions:UserIdentifierAlreadyExists',
      /**
       * User already subscribed.
       */
      UserAlreadySubscribed = 'UserSubscriptions:UserAlreadySubscribed',
      /**
       * User has pending subscription awaiting confirmation
       */
      UserHasPendingSubscriptionAwaitingConfirmation = 'UserSubscriptions:UserHasPendingSubscriptionAwaitingConfirmation',
      /**
       * Cannot resubscribe to this subscription.
       */
      CannotResubscribeToThisSubscription = 'UserSubscriptions:CannotResubscribeToThisSubscription',
      /**
       * Can only resubscribe with original user identifier.
       */
      CanOnlyResubscribeWithOriginalUserIdentifier = 'UserSubscriptions:CanOnlyResubscribeWithOriginalUserIdentifier',
      /**
       * Can only resubscribe with new user identifier.
       */
      CanOnlyResubscribeWithNewUserIdentifier = 'UserSubscriptions:CanOnlyResubscribeWithNewUserIdentifier',
      /**
       * Subscription does not allow unsubscribing.
       */
      SubscriptionDoesNotAllowUnsubscribing = 'UserSubscriptions:SubscriptionDoesNotAllowUnsubscribing',
      /**
       * Unsubscribe can only be done by subscribed user.
       */
      UnsubscribeCanOnlyBeDoneBySubscribedUser = 'UserSubscriptions:UnsubscribeCanOnlyBeDoneBySubscribedUser',
      /**
       * Unsubscribe can only be done by employee or api user.
       */
      UnsubscribeCanOnlyBeDoneByEmployeeOrAPIUser = 'UserSubscriptions:UnsubscribeCanOnlyBeDoneByEmployeeOrAPIUser',
      /**
       * Unsubscribe from handler '{0:string}' failed.
       */
      UnsubscribeFromHandlerFailed = 'UserSubscriptions:UnsubscribeFromHandlerFailed',
      /**
       * Subscribe to handler '{0:string}' failed.
       */
      SubscribeToHandlerFailed = 'UserSubscriptions:SubscribeToHandlerFailed'
    }

    export const enum ExternalUserProvider {
      /**
       * Failed to execute the call to the external user provider
       */
      CallFailed = 'ExternalUserProvider:CallFailed',
      /**
       * The user could not be found in the external user provider
       */
      UnknownUser = 'ExternalUserProvider:UnknownUser'
    }

    export const enum UserCards {
      /**
       * UserCards with Type Discount can only be created internally by the Discount engine.
       */
      DiscountTypeNotAllowed = 'UserCards:DiscountTypeNotAllowed',
      /**
       * UserCards with Type Loyalty can only be managed internally.
       */
      LoyaltyTypeNotAllowed = 'UserCards:LoyaltyTypeNotAllowed',
      /**
       * UserCards with Type Loyalty can only be mutated internally.
       */
      LoyaltyTypeMutationsNotAllowed = 'UserCards:LoyaltyTypeMutationsNotAllowed',
      /**
       * CurrencyID is required when creating Currency UserCards.
       */
      CurrencyRequired = 'UserCards:CurrencyRequired',
      /**
       * UserCards with Type Discount can't create paymenttransactions.
       */
      DiscountTypePayment = 'UserCards:DiscountTypePayment'
    }

    export const enum UserBoughtProduct {
      /**
       * Product is already registered.
       */
      AlreadyRegistered = 'UserBoughtProduct:AlreadyRegistered',
      /**
       * Unknown user bought product.
       */
      UnknownUserBoughtProduct = 'UserBoughtProduct:UnknownUserBoughtProduct',
      /**
       * Unknown user bought product detail.
       */
      UnknownUserBoughtProductDetail = 'UserBoughtProduct:UnknownUserBoughtProductDetail',
      /**
       * The given OrderID/OrderLineID is an invalid combination.
       */
      InvalidOrderAndLineCombination = 'UserBoughtProduct:InvalidOrderAndLineCombination',
      /**
       * User limit of {0:number} exceeded.
       */
      PushBoughtProductForUsersLimitExceeded = 'UserBoughtProduct:PushBoughtProductForUsersLimitExceeded'
    }

    export const enum UserBorrowedProducts {
      /**
       * User cannot register borrowed products on it's own account.
       */
      UserCannotBorrowProductsForHimSelf = 'UserBorrowedProducts:UserCannotBorrowProductsForHimSelf',
      /**
       * User cannot return register borrowed products from it's own account.
       */
      UserCannotReturnBorrowedProductsForHimSelf = 'UserBorrowedProducts:UserCannotReturnBorrowedProductsForHimSelf',
      /**
       * User cannot return purchase borrowed products from it's own account.
       */
      UserCannotPurchaseBorrowedProductsForHimSelf = 'UserBorrowedProducts:UserCannotPurchaseBorrowedProductsForHimSelf',
      /**
       * User doesn't have this product registerd as borrowed.
       */
      UserDoesNotHaveThisProductRegisterdAsBorrowed = 'UserBorrowedProducts:UserDoesNotHaveThisProductRegisterdAsBorrowed',
      /**
       * Cannot purchase more products then that are still borrowed
       */
      CannotReturnMoreProductsThenBorrowed = 'UserBorrowedProducts:CannotReturnMoreProductsThenBorrowed',
      /**
       * Cannot purchase more products then that are still borrowed
       */
      CannotPurchaseMoreProductsThenBorrowed = 'UserBorrowedProducts:CannotPurchaseMoreProductsThenBorrowed',
      /**
       * Cannot purchase borrowed products for multiple users on the same order.
       */
      CannotPurchaseBorrowedItemsForMultipleUsersOnTheSameOrder = 'UserBorrowedProducts:CannotPurchaseBorrowedItemsForMultipleUsersOnTheSameOrder',
      /**
       * Cannot register borrowed products that require a serial number on stock movement.
       */
      CannotBorrowProductRequiringSerialNumberWithStockMovement = 'UserBorrowedProducts:CannotBorrowProductRequiringSerialNumberWithStockMovement'
    }

    export const enum TaxCodeMapping {
      /**
       * An entry for the given tax code and tax provider already exists.
       */
      InvalidMapping = 'TaxCodeMapping:InvalidMapping'
    }

    export const enum TaxProvider {
      /**
       * The configuration data is invalid. 
       * {0:string}
       */
      InvalidConfigurationData = 'TaxProvider:InvalidConfigurationData',
      /**
       * The provided Type is invalid.
       */
      InvalidType = 'TaxProvider:InvalidType',
      /**
       * Name already exists.
       */
      InvalidName = 'TaxProvider:InvalidName',
      /**
       * No tax provider configuration exists for {0:number}.
       */
      NoConfiguration = 'TaxProvider:NoConfiguration',
      /**
       * Name and Type cannot be null.
       */
      InvalidRequiredFields = 'TaxProvider:InvalidRequiredFields'
    }

    export const enum GenerateCouponSurveyAfterEffect {
      /**
       * The value of property {0:string} is missing
       */
      MissingValue = 'GenerateCouponSurveyAfterEffect:MissingValue',
      /**
       * The discount is invalid. It should be a valid coupon triggered discount.
       */
      InvalidDiscount = 'GenerateCouponSurveyAfterEffect:InvalidDiscount'
    }

    export const enum ShippingMethod {
      /**
       * Either Code or Handler should have a value.
       */
      CodeOrHandlerShouldBeSet = 'ShippingMethod:CodeOrHandlerShouldBeSet',
      /**
       * An invalid handler has been provided.
       */
      InvalidShippingMethodHandler = 'ShippingMethod:InvalidShippingMethodHandler'
    }

    export const enum ModifyQuantityShipped {
      /**
       * Order {0:number} is not a PurchaseOrder
       */
      NoPurchaseOrder = 'ModifyQuantityShipped:NoPurchaseOrder',
      /**
       * Shipment {0:number} is already completed
       */
      ShipmentIsCompleted = 'ModifyQuantityShipped:ShipmentIsCompleted',
      /**
       * Shipment {0:number} is already exported
       */
      ShipmentIsExported = 'ModifyQuantityShipped:ShipmentIsExported',
      /**
       * ShipmentLine {0:number} is already completed
       */
      ShipmentLineIsCompleted = 'ModifyQuantityShipped:ShipmentLineIsCompleted',
      /**
       * ShipmentLine {0:number} is already fully delivered
       */
      ShipmentLineIsFullyDelivered = 'ModifyQuantityShipped:ShipmentLineIsFullyDelivered',
      /**
       * OrderLine {OrderLineID:number} of ShipmentLine {ShipmentLineID:number} is already invoiced
       */
      OrderLineIsInvoiced = 'ModifyQuantityShipped:OrderLineIsInvoiced',
      /**
       * ShipmentLine {0:number} has already more Delivered
       */
      ShipmentLineHasMoreDeliveredThanRequested = 'ModifyQuantityShipped:ShipmentLineHasMoreDeliveredThanRequested'
    }

    export const enum SerialNumbers {
      /**
       * Serial number is required
       */
      SerialNumberRequired = 'SerialNumbers:SerialNumberRequired',
      /**
       * Product is required when setting a serial number
       */
      ProductRequired = 'SerialNumbers:ProductRequired',
      /**
       * Quantity does not match serial number count
       */
      InvalidQuantity = 'SerialNumbers:InvalidQuantity',
      /**
       * Serial number {0:string} is invalid
       */
      SerialNumberInvalid = 'SerialNumbers:SerialNumberInvalid',
      /**
       * The usage of serial number {0:string} is denied.
       */
      SerialNumberUsageDenied = 'SerialNumbers:SerialNumberUsageDenied',
      /**
       * Duplicate serial number {0:string}
       */
      DuplicateSerialNumber = 'SerialNumbers:DuplicateSerialNumber',
      /**
       * Serial number {0:string} cannot be received because it was never shipped.
       */
      SerialNumberNotShipped = 'SerialNumbers:SerialNumberNotShipped',
      /**
       * Serial Number Configuration cannot be deleted because it is still used by existing Serial Number Products.
       */
      CannotDeleteUsedConfiguration = 'SerialNumbers:CannotDeleteUsedConfiguration',
      /**
       * It it not possible to sell to the stock location of serial number {0:string}.
       */
      StockLabelMismatch = 'SerialNumbers:StockLabelMismatch',
      /**
       * The serial number {0:string} is owned by a different organization unit.
       */
      InvalidOrganizationUnit = 'SerialNumbers:InvalidOrganizationUnit',
      /**
       * The serial number {0:string} is owned by customer.
       */
      OwnedByCustomer = 'SerialNumbers:OwnedByCustomer',
      /**
       * The serial number {0:string} belongs to a different product.
       */
      DifferentProduct = 'SerialNumbers:DifferentProduct',
      /**
       * Request contains duplicate serial numbers
       */
      SerialNumberDuplicates = 'SerialNumbers:SerialNumberDuplicates',
      /**
       * The provided Serial Number does not match with the existing one.
       */
      SerialNumberDoesNotMatch = 'SerialNumbers:SerialNumberDoesNotMatch'
    }

    export const enum Sellability {
      /**
       * Product {0:number} is not sellable.
       */
      ProductNotSellable = 'Sellability:ProductNotSellable'
    }

    export const enum ElevationManager {
      /**
       * You have insufficient rights to request this elevation.
       */
      InsufficientRightsToRequestElevation = 'ElevationManager:InsufficientRightsToRequestElevation',
      /**
       * Elevation type not supported.
       */
      ElevationTypeNotSupported = 'ElevationManager:ElevationTypeNotSupported'
    }

    export const enum SecurityPolicy {
      /**
       * New password is required.
       */
      NewPasswordIsRequired = 'SecurityPolicy:NewPasswordIsRequired',
      /**
       * User or password is not valid.
       */
      UserOrPasswordInvalid = 'SecurityPolicy:UserOrPasswordInvalid',
      /**
       * Password is too short - it must be at least 8 characters.
       */
      TooShort = 'SecurityPolicy:TooShort',
      /**
       * Password authentication is disabled for this user.
       */
      PasswordAuthenticationDisabled = 'SecurityPolicy:PasswordAuthenticationDisabled',
      /**
       * Password is too long - 1024 seems like a decent enough length for a strong password, no?
       */
      TooLong = 'SecurityPolicy:TooLong',
      /**
       * This password has been found in one or more breaches and should not be used (see https://haveibeenpwned.com/Passwords).
       */
      Pwned = 'SecurityPolicy:Pwned',
      /**
       * Password safety check could not be performed.
       */
      PwnedUnavailable = 'SecurityPolicy:PwnedUnavailable'
    }

    export const enum Url {
      /**
       * Url is empty.
       */
      UrlEmpty = 'Url:UrlEmpty',
      /**
       * Url is not valid.
       */
      UrlInvalid = 'Url:UrlInvalid',
      /**
       * Url does not match expectation.
       */
      UrlDoesNotMatch = 'Url:UrlDoesNotMatch'
    }

    export const enum Passkey {
      /**
       * Passkeys are disabled
       */
      PasskeysDisabled = 'Passkey:PasskeysDisabled',
      /**
       * Invalid challenge
       */
      InvalidChallenge = 'Passkey:InvalidChallenge',
      /**
       * Invalid client data
       */
      InvalidClientData = 'Passkey:InvalidClientData',
      /**
       * Invalid authenticator data
       */
      InvalidAuthenticatorData = 'Passkey:InvalidAuthenticatorData',
      /**
       * Invalid user
       */
      InvalidUser = 'Passkey:InvalidUser',
      /**
       * Invalid signature
       */
      InvalidSignature = 'Passkey:InvalidSignature',
      /**
       * Invalid key
       */
      InvalidKey = 'Passkey:InvalidKey'
    }

    export const enum MagicLink {
      /**
       * Magic links are disabled by your administrator.
       */
      Disabled = 'MagicLink:Disabled',
      /**
       * The redirect URL was not configured as supported domain.
       */
      NotConfiguredDomain = 'MagicLink:NotConfiguredDomain',
      /**
       * The provided combination of Email Address and OrderID yielded no results.
       */
      EmailOrderMismatch = 'MagicLink:EmailOrderMismatch',
      /**
       * Manual codes through magic links are disabled by your administrator.
       */
      ManualCodeDisabled = 'MagicLink:ManualCodeDisabled'
    }

    export const enum ReceiveShipment {
      /**
       * Shipment {0:number} cannot be found.
       */
      ShipmentNotFound = 'ReceiveShipment:ShipmentNotFound',
      /**
       * Shipment {0:number} has already been completed, can no longer receive it.
       */
      ShipmentCompleted = 'ReceiveShipment:ShipmentCompleted',
      /**
       * Shipment {0:number} is still pending, delete the shipment instead of cancelling it..
       */
      ShipmentPending = 'ReceiveShipment:ShipmentPending',
      /**
       * Shipment {0:number} has already been (partly) received, can no longer cancel it.
       */
      ShipmentReceived = 'ReceiveShipment:ShipmentReceived',
      /**
       * Cannot receive product {0:number} without reference because it is already in the shipment
       */
      CannotReceiveProductWithoutReference = 'ReceiveShipment:CannotReceiveProductWithoutReference'
    }

    export const enum ProductRequirements {
      /**
       * The given productrequirement doesn't match the product on this orderline.
       */
      InvalidProductRequirement = 'ProductRequirements:InvalidProductRequirement',
      /**
       * ProductRequirement '{0:string}' is an array, but the given value is not.
       */
      IsArray = 'ProductRequirements:IsArray',
      /**
       * ProductRequirement '{ProductRequirementName:string}' is an {ValueType:string}, but the given value is not.
       */
      InvalidType = 'ProductRequirements:InvalidType',
      /**
       * ProductRequirement '{0:string}' is an enum, but the given value is not a valid enum value.
       */
      InvalidArrayValue = 'ProductRequirements:InvalidArrayValue',
      /**
       * ProductRequirement '{0:string}' is required.
       */
      IsRequired = 'ProductRequirements:IsRequired'
    }

    export const enum ProductBundles {
      /**
       * Cannot create a ProductBundle for Product {0:number} because it's not a BundleProduct
       */
      BundleProductRequired = 'ProductBundles:BundleProductRequired',
      /**
       * A ProductBundleLine requires at least one ProductBundleLineOption.
       */
      OptionsRequired = 'ProductBundles:OptionsRequired',
      /**
       * Creating a ProductBundleLine when there are multiple Options requires selecting a DefaultProductID.
       */
      DefaultProductRequired = 'ProductBundles:DefaultProductRequired',
      /**
       * Product {0:number} cannot be found.
       */
      UnknownProduct = 'ProductBundles:UnknownProduct',
      /**
       * Creating a ProductBundleLine where one of the options is a Configurable product requires a DefaultProductID.
       */
      DefaultProductRequiredForConfigurable = 'ProductBundles:DefaultProductRequiredForConfigurable',
      /**
       * Product {0:number} is not one of the options nor part of the hierarchy of the options.
       */
      InvalidDefaultProduct = 'ProductBundles:InvalidDefaultProduct',
      /**
       * When creating/updating a ProductBundle a ProductBundleID or a BundleProductID is required
       */
      ProductBundleIDOrBundleProductIDRequired = 'ProductBundles:ProductBundleIDOrBundleProductIDRequired',
      /**
       * Product {0:number} can not be used as default product for bundle line {1:string} because it can be excluded by an option on bundle line {2:string}.
       */
      DefaultProductCouldBeExcluded = 'ProductBundles:DefaultProductCouldBeExcluded',
      /**
       * The ProductBundleLine contains the same product ({0:number}) multiple times.
       */
      DuplicateOption = 'ProductBundles:DuplicateOption',
      /**
       * Only ProductBundleLines of type Optional allow selecting multiple options.
       */
      MultiSelectOnlyPossibleOnOptionalLines = 'ProductBundles:MultiSelectOnlyPossibleOnOptionalLines',
      /**
       * An option refers to the same product as the root bundle product, causing an infinite recursion!
       */
      InfiniteRecursion = 'ProductBundles:InfiniteRecursion',
      /**
       * ProductBundleLine {0:number} cannot be found on this bundle.
       */
      UnknownProductBundleLine = 'ProductBundles:UnknownProductBundleLine',
      /**
       * Product {selectedProductID:number} is not a valid option for ProductBundleLineID {productBundleLineID:number}
       */
      InvalidProductSelection = 'ProductBundles:InvalidProductSelection',
      /**
       * It's no longer possible to switch products for ProductBundleLineID {productBundleLineID:number} on Order {orderID:number}
       */
      CannotChangeProductID = 'ProductBundles:CannotChangeProductID',
      /**
       * The selection contains multiple lines for ProductBundleLineID {0:number}
       */
      DuplicateProductBundleLines = 'ProductBundles:DuplicateProductBundleLines',
      /**
       * The selection contains {SelectedCount:number} for ProductBundleLineID {ProductBundleLineID:number}, while only a maximum of {MaximumCount:number} is allowed.
       */
      MoreOptionsSelectedThanAllowed = 'ProductBundles:MoreOptionsSelectedThanAllowed',
      /**
       * Selected product {selectedProductID:number} on ProductBundleLine {productBundleLineID:number} is excluded by the selected option on ProductBundleLine {excludingProductBundleLineID:number}
       */
      ProductIsExcludedByOtherProductBundleLine = 'ProductBundles:ProductIsExcludedByOtherProductBundleLine'
    }

    export const enum PriceCalculation {
      /**
       * PriceList {0:number} references itself.
       */
      PriceListReferencesItself = 'PriceCalculation:PriceListReferencesItself',
      /**
       * PriceList {priceListID:number} depends on deleted pricelist {adjustmentPriceListID:number}.
       */
      PriceListDependsOnDeletedPriceList = 'PriceCalculation:PriceListDependsOnDeletedPriceList',
      /**
       * PriceList {priceListID:number} has a component {adjustmentPriceListID:number} that does not have the required data object.
       */
      MissingData = 'PriceCalculation:MissingData'
    }

    export const enum PriceLists {
      /**
       * Currency mismatch: OrganizationUnitID {organizationUnitID:number} uses CurrencyID {ouCurrency:string} but PriceList {priceListID:number} uses CurrencyID {priceListCurrency:string}
       */
      CurrencyMismatch = 'PriceLists:CurrencyMismatch',
      /**
       * OrganizationUnit {organizationUnit:string} already uses pricelist {priceListName:string} for {usageType:string}, remove it first before attaching a new pricelist.
       */
      PriceListAlreadyAttachedToOrganizationUnit = 'PriceLists:PriceListAlreadyAttachedToOrganizationUnit',
      /**
       * You are not allowed to attach a pricelist of type {UsageType:string} to organization unit {OrganizationUnitName:string}.
       */
      NotAllowed = 'PriceLists:NotAllowed',
      /**
       * The adjustment or provided data for the adjustment is invalid.
       */
      InvalidAdjustment = 'PriceLists:InvalidAdjustment',
      /**
       * Cannot modify or delete second chance product adjustment.
       */
      CannotModifyOrDeleteSecondChanceProductAdjustment = 'PriceLists:CannotModifyOrDeleteSecondChanceProductAdjustment',
      /**
       * Historical pricelists are protected and cannot be modified.
       */
      HistoricalPricesAreProtectedAndCannotBeModified = 'PriceLists:HistoricalPricesAreProtectedAndCannotBeModified'
    }

    export const enum PaymentRequiredDataValidation {
      /**
       * Order {orderID:number} can't be paid, because it has validation errors: {errors:string}
       */
      OrderRequiredDataValidationFailed = 'PaymentRequiredDataValidation:OrderRequiredDataValidationFailed'
    }

    export const enum OpenAmountValidation {
      /**
       * This order has already been paid for.
       */
      OrderAlreadyFullyPaid = 'OpenAmountValidation:OrderAlreadyFullyPaid'
    }

    export const enum RefundAmountValidation {
      /**
       * This order has only {openAmount:number} available for refund and {givenAmount:number} was requested.
       */
      AmountNotAvailableForRefund = 'RefundAmountValidation:AmountNotAvailableForRefund'
    }

    export const enum CreatePayment {
      /**
       * Could not calculate sales tax for this order.
       */
      TaxCalculationFailed = 'CreatePayment:TaxCalculationFailed',
      /**
       * MultiCurrency is not available for this payment method.
       */
      MultiCurrencyNotAvailable = 'CreatePayment:MultiCurrencyNotAvailable',
      /**
       * MultiCurrency is not available for this payment type.
       */
      MultiCurrencyNotAvailableForPaymentType = 'CreatePayment:MultiCurrencyNotAvailableForPaymentType',
      /**
       * MultiCurrency is not available for refunds.
       */
      MultiCurrencyNotAvailableForRefunds = 'CreatePayment:MultiCurrencyNotAvailableForRefunds',
      /**
       * There is no PaymentMethod available for the requested PaymentType {paymentTypeOrCode:string}, available methods are: {availableMethods:string}
       */
      NoPaymentMethodAvailable = 'CreatePayment:NoPaymentMethodAvailable'
    }

    export const enum Payments {
      /**
       * PaymentTransaction {0:number} can no longer be cancelled.
       */
      CancellationNotAvailable = 'Payments:CancellationNotAvailable',
      /**
       * Failed to cancel PaymentTransaction {0:number}.
       */
      CancellationFailed = 'Payments:CancellationFailed',
      /**
       * Auto shipment after confirm is only allowed in the context of a store.
       */
      AutoShipOnlyAllowedInShops = 'Payments:AutoShipOnlyAllowedInShops',
      /**
       * The current order is not valid for auto shipment after confirm.
       */
      InvalidOrderForAutoShip = 'Payments:InvalidOrderForAutoShip'
    }

    export const enum PaymentMethods {
      /**
       * No internet connection available and payment method not usable offline
       */
      NotUsableOffline = 'PaymentMethods:NotUsableOffline',
      /**
       * EVA is running on watchtower and this payment method is not supported
       */
      PaymentMethodNotUsableOnWatchtower = 'PaymentMethods:PaymentMethodNotUsableOnWatchtower',
      /**
       * EVA is running normally and this payment method is only supported in ops/offline
       */
      PaymentMethodNotUsableInCloud = 'PaymentMethods:PaymentMethodNotUsableInCloud'
    }

    export const enum Settlements {
      /**
       * The summed amount of the settlements ({summedAmount:number}) doesn't match the totalamount ({totalAmount:number}).
       */
      AmountMismatch = 'Settlements:AmountMismatch',
      /**
       * No financialperiod for settlement.
       */
      NoFinancialPeriod = 'Settlements:NoFinancialPeriod'
    }

    export const enum PaymentDispute {
      /**
       * PaymentTransaction {0:number} was already resolved.
       */
      AlreadyResolved = 'PaymentDispute:AlreadyResolved',
      /**
       * PaymentTransaction {0:number} has no dipute.
       */
      NoDispute = 'PaymentDispute:NoDispute',
      /**
       * PaymentTransaction {0:number} has no order attached.
       */
      NoOrderAttached = 'PaymentDispute:NoOrderAttached'
    }

    export const enum CustomPaymentMethod {
      /**
       * This payment needs to be confirmed
       */
      ConfirmationRequired = 'CustomPaymentMethod:ConfirmationRequired',
      /**
       * Over payments are blocked on this payment method
       */
      BlockedOverPayments = 'CustomPaymentMethod:BlockedOverPayments',
      /**
       * Currency not available for the current organization unit.
       */
      CurrencyNotAvailable = 'CustomPaymentMethod:CurrencyNotAvailable',
      /**
       * Exchange rate is required.
       */
      ExchangeRateRequired = 'CustomPaymentMethod:ExchangeRateRequired'
    }

    export const enum OrganizationUnitSets {
      /**
       * An OrganizationUnit can only be part of a single set within the scope {scope:string}, but it appears in multiple sets: {setNames:string}
       */
      OrganizationUnitNotUniqueInScope = 'OrganizationUnitSets:OrganizationUnitNotUniqueInScope',
      /**
       * OrganizationUnitSet {set:string} has scope {scope:string}, cannot add subset {subSet:string} with scope {subScope:string}
       */
      SubsetCannotHaveDifferentScope = 'OrganizationUnitSets:SubsetCannotHaveDifferentScope',
      /**
       * OrganizationUnitSet {set:string} does not have a scope, cannot add subset {subset:string} with scope {subScope:string} to it.
       */
      SetWithoutScopeCannotHaveSubsetWithScope = 'OrganizationUnitSets:SetWithoutScopeCannotHaveSubsetWithScope',
      /**
       * OrganizationUnitSet {set:string} has a Scope {scope:string} with OrganizationUnitUniqueInScope = true, which means it cannot contain dynamic sets.
       */
      ScopeCannotContainDynamicSet = 'OrganizationUnitSets:ScopeCannotContainDynamicSet',
      /**
       * OrganizationUnitSet {setID:number} is its own subset. Reference chain: {recursionChain:string}
       */
      CircularReferenceInSet = 'OrganizationUnitSets:CircularReferenceInSet',
      /**
       * All subsets are required for reordering.
       */
      AllSubsetsAreRequiredForReordering = 'OrganizationUnitSets:AllSubsetsAreRequiredForReordering',
      /**
       * Duplicate subsets are not allowed when setting sequences.
       */
      DuplicateSubsetsNotAllowedWhenSequencing = 'OrganizationUnitSets:DuplicateSubsetsNotAllowedWhenSequencing'
    }

    export const enum OrganizationUnit {
      /**
       * ParentID is required.
       */
      MissingParentID = 'OrganizationUnit:MissingParentID',
      /**
       * Name is required.
       */
      MissingName = 'OrganizationUnit:MissingName',
      /**
       * EmailAddress is invalid.
       */
      InvalidEmailAddress = 'OrganizationUnit:InvalidEmailAddress',
      /**
       * Subnet is invalid.
       */
      InvalidSubnet = 'OrganizationUnit:InvalidSubnet',
      /**
       * IpAddress is invalid.
       */
      InvalidIpAddress = 'OrganizationUnit:InvalidIpAddress',
      /**
       * Cannot enable endless stock on non supplier organizationUnit.
       */
      CannotEnableEndlessStockOnNonSupplierOrganizationUnit = 'OrganizationUnit:CannotEnableEndlessStockOnNonSupplierOrganizationUnit',
      /**
       * Cannot remove the VisibilityGroup from an organizationUnit.
       */
      CannotRemoveVisibilityGroup = 'OrganizationUnit:CannotRemoveVisibilityGroup',
      /**
       * Cannot update organization unit while it is archived.
       */
      CannotUpdateWhileArchived = 'OrganizationUnit:CannotUpdateWhileArchived',
      /**
       * Container cannot also be shop.
       */
      ContainerCannotAlsoBeShop = 'OrganizationUnit:ContainerCannotAlsoBeShop',
      /**
       * Cash handler currency must be equal to organizationUnit currency.
       */
      InvalidCashHandler = 'OrganizationUnit:InvalidCashHandler',
      /**
       * BackendID {0:string} is already in use.
       */
      DuplicateBackendID = 'OrganizationUnit:DuplicateBackendID',
      /**
       * Cannot delete root organization unit.
       */
      CannotDeleteRootOrganizationUnit = 'OrganizationUnit:CannotDeleteRootOrganizationUnit',
      /**
       * Cannot delete organization unit with children. Please delete the children first.
       */
      CannotDeleteOrganizationUnitWithChildren = 'OrganizationUnit:CannotDeleteOrganizationUnitWithChildren',
      /**
       * Cannot delete organization unit with archived children. Please delete the children first.
       */
      CannotDeleteOrganizationUnitWithArchivedChildren = 'OrganizationUnit:CannotDeleteOrganizationUnitWithArchivedChildren',
      /**
       * Cannot archive organization unit with children. Please archive the children first.
       */
      CannotArchiveOrganizationUnitWithChildren = 'OrganizationUnit:CannotArchiveOrganizationUnitWithChildren',
      /**
       * Cannot unarchive organization unit with archived parent. Please unarchive the parent first.
       */
      CannotUnArchiveOrganizationUnitWithArchivedParent = 'OrganizationUnit:CannotUnArchiveOrganizationUnitWithArchivedParent',
      /**
       * It's not possible to change the CostPriceCurrencyMethod to FIFO for OrganizationUnit {0:string} when it has existing stock. Remove the stock first and re-create it by receiving a Purchase Order
       */
      CannotChangeToFifoWithExistingStock = 'OrganizationUnit:CannotChangeToFifoWithExistingStock'
    }

    export const enum LoyaltyAPI {
      /**
       * The configuration for this LoyaltyAPI is incomplete.
       */
      ConfigurationIncomplete = 'LoyaltyAPI:ConfigurationIncomplete',
      /**
       * Could not validate the payment. Service returned the following error: {0:string}.
       */
      CouldNotValidatePayment = 'LoyaltyAPI:CouldNotValidatePayment'
    }

    export const enum EVALoyalty {
      /**
       * User is not (or no longer) subscribed.
       */
      UserIsNotOrNoLongerSubscribed = 'EVALoyalty:UserIsNotOrNoLongerSubscribed',
      /**
       * Withdrawal not allowed.
       */
      WithdrawalNotAllowed = 'EVALoyalty:WithdrawalNotAllowed',
      /**
       * Refund not allowed.
       */
      RefundNotAllowed = 'EVALoyalty:RefundNotAllowed',
      /**
       * Invalid deposit transaction id.
       */
      InvalidDepositTransactionID = 'EVALoyalty:InvalidDepositTransactionID',
      /**
       * Unknown deposit transaction id.
       */
      UnknownDepositTransactionID = 'EVALoyalty:UnknownDepositTransactionID',
      /**
       * Cannot revoke more than deposited.
       */
      CannotRevokeMoreThanDeposited = 'EVALoyalty:CannotRevokeMoreThanDeposited',
      /**
       * Cannot withdraw more than available.
       */
      CannotWithdrawMoreThanAvailable = 'EVALoyalty:CannotWithdrawMoreThanAvailable',
      /**
       * Cannot refund more than there is left to refund.
       */
      CannotRefundMoreThanThereIsLeftToRefund = 'EVALoyalty:CannotRefundMoreThanThereIsLeftToRefund',
      /**
       * Original transaction does not exist.
       */
      OriginalTransactionDoesNotExist = 'EVALoyalty:OriginalTransactionDoesNotExist',
      /**
       * Can only refund on a withdraw transaction.
       */
      CanOnlyRefundOnAWithdrawTransaction = 'EVALoyalty:CanOnlyRefundOnAWithdrawTransaction',
      /**
       * Can only revoke on a deposit transaction.
       */
      CanOnlyRevokeOnADepositTransaction = 'EVALoyalty:CanOnlyRevokeOnADepositTransaction',
      /**
       * The LoyaltyPointAdjustmentType provided is not supported for LoyaltyPoint rebalances.
       */
      UnsupportedLoyaltyPointAdjustmentType = 'EVALoyalty:UnsupportedLoyaltyPointAdjustmentType',
      /**
       * Too many mutations requested, the limit of this service is 10.000 mutations per request.
       */
      TooManyMutationsRequested = 'EVALoyalty:TooManyMutationsRequested',
      /**
       * Only the EVALOYALTY can be used for this service.
       */
      UnsupportedHandlerForService = 'EVALoyalty:UnsupportedHandlerForService',
      /**
       * Unknown error while mutating the user's loyalty point balance.
       */
      UnknownLoyaltyPointBalanceError = 'EVALoyalty:UnknownLoyaltyPointBalanceError'
    }

    export const enum PointPolicy {
      /**
       * The point policy `{0:string}` is unknown.
       */
      UnknownPointPolicy = 'PointPolicy:UnknownPointPolicy',
      /**
       * Could not calculate the data because the data isn't initialized.
       */
      DataNotInitialized = 'PointPolicy:DataNotInitialized',
      /**
       * {PolicyName:string} data for the {Type:PointPolicyType} policy in {parentType:string} {parentID:number} is required.
       */
      MissingPolicyData = 'PointPolicy:MissingPolicyData',
      /**
       * {PolicyName:string} data for the {Type:PointPolicyType} policy in {parentType:string} {parentID:number} is invalid.
       */
      InvalidPolicyData = 'PointPolicy:InvalidPolicyData'
    }

    export const enum CustomPointPolicy {
      /**
       * The custom policy time interval `{0:string}` is not implemented.
       */
      NotImplementedCustomPolicyTimeInterval = 'CustomPointPolicy:NotImplementedCustomPolicyTimeInterval',
      /**
       * Interval value must be higher then zero.
       */
      IntervalValueMustBeHigherThanZero = 'CustomPointPolicy:IntervalValueMustBeHigherThanZero'
    }

    export const enum ExpireEndOfYearPointPolicy {
      /**
       * LeniencyDays cannot be negative.
       */
      LeniencyDaysCannotBeNegative = 'ExpireEndOfYearPointPolicy:LeniencyDaysCannotBeNegative'
    }

    export const enum ExpireEndOfMonthPointPolicy {
      /**
       * LeniencyDays cannot be negative.
       */
      LeniencyDaysCannotBeNegative = 'ExpireEndOfMonthPointPolicy:LeniencyDaysCannotBeNegative'
    }

    export const enum ExpireSetDatePointPolicy {
      /**
       * Invalid month value.
       */
      InvalidMonthValue = 'ExpireSetDatePointPolicy:InvalidMonthValue',
      /**
       * Invalid day value.
       */
      InvalidDayValue = 'ExpireSetDatePointPolicy:InvalidDayValue',
      /**
       * LeniencyDays cannot be negative.
       */
      LeniencyDaysCannotBeNegative = 'ExpireSetDatePointPolicy:LeniencyDaysCannotBeNegative'
    }

    export const enum EvaClient {
      /**
       * Failed to call service {serviceName:string}: {error:string}
       */
      CallFailed = 'EvaClient:CallFailed'
    }

    export const enum LiveGuard {
      /**
       * No TOTP token generated for this store.
       */
      NoTOTPTokenGenerated = 'LiveGuard:NoTOTPTokenGenerated'
    }

    export const enum Invoices {
      /**
       * Missing billing details for invoice {0:string}
       */
      MissingBillingDetailsForInvoice = 'Invoices:MissingBillingDetailsForInvoice',
      /**
       * Invoice {0:number} is not open for modification
       */
      CannotBeModified = 'Invoices:CannotBeModified',
      /**
       * Could not calculate sales tax for order {0:number}.
       */
      TaxCalculationFailed = 'Invoices:TaxCalculationFailed',
      /**
       * Dispute {0:number} is not open for modification
       */
      DisputeCannotBeModified = 'Invoices:DisputeCannotBeModified',
      /**
       * Invoice {0:number} is already closed
       */
      AlreadyClosed = 'Invoices:AlreadyClosed',
      /**
       * Invoice {invoiceID:number} has an original amount of {originalTotalAmount:number}, but the total amount is {totalAmount:number}.
       */
      TotalAmountMismatch = 'Invoices:TotalAmountMismatch',
      /**
       * Invoice {invoiceID:number} has an expected amount of {expectedTotalAmount:number}, but the total amount is {totalAmount:number}.
       */
      ExpectedTotalAmountMismatch = 'Invoices:ExpectedTotalAmountMismatch',
      /**
       * InvoiceAdditionalAmount {invoiceAdditionalAmountID:number} has an original amount of {originalTotalAmount:number}, but the total amount is {totalAmount:number}.
       */
      AdditionalAmountMismatch = 'Invoices:AdditionalAmountMismatch',
      /**
       * Invoice {0:number} has another CalculationMethod than the OrderLine.
       */
      CalculationMismatch = 'Invoices:CalculationMismatch',
      /**
       * An OrderLineID or ShipmentLineID must be supplied to create an InvoiceLine
       */
      OrderLineOrShipmentLineRequired = 'Invoices:OrderLineOrShipmentLineRequired',
      /**
       * A Discount OrderLine cannot be invoiced, it will be invoiced based on the other lines
       */
      DiscountOrderLinesCantBeInvoiced = 'Invoices:DiscountOrderLinesCantBeInvoiced',
      /**
       * A Dispute cannot be attached to an InvoiceLine and an AdditionalAmount
       */
      DisputeCannotBeAttachedTwice = 'Invoices:DisputeCannotBeAttachedTwice',
      /**
       * There already exists a blob for invoice {invoiceID:number} and order {orderID:number}
       */
      InvoiceBlobAlreadyExists = 'Invoices:InvoiceBlobAlreadyExists',
      /**
       * There already exists a blob for invoice {invoiceID:number}, attached to order {orderID:number}
       */
      InvoiceBlobAlreadyExistsForDifferentOrder = 'Invoices:InvoiceBlobAlreadyExistsForDifferentOrder',
      /**
       * There is no email available to send the invoice.
       */
      EmailNotAvailable = 'Invoices:EmailNotAvailable',
      /**
       * Invoice {0:number} has unresolved disputes
       */
      HasUnresolvedDisputes = 'Invoices:HasUnresolvedDisputes',
      /**
       * There are no point of sale available for organization: {0:number}.
       */
      PointOfSaleNotAvailable = 'Invoices:PointOfSaleNotAvailable',
      /**
       * The payment is not available: {0:string}.
       */
      PaymentNotAvailable = 'Invoices:PaymentNotAvailable',
      /**
       * Manual Invoices are not available for this country: {0:string}.
       */
      ManualInvoicesDisabled = 'Invoices:ManualInvoicesDisabled'
    }

    export const enum InvoiceExport {
      /**
       * Invoice {0:number} must be closed before it can be exported.
       */
      InvoiceNotClosed = 'InvoiceExport:InvoiceNotClosed'
    }

    export const enum ExecutableAction {
      /**
       * Invalid action.
       */
      InvalidAction = 'ExecutableAction:InvalidAction'
    }

    export const enum OpeningHours {
      /**
       * Either DayOfWeek or Date is required for an OpeningHour
       */
      DayOfWeekOrDateRequired = 'OpeningHours:DayOfWeekOrDateRequired',
      /**
       * Cannot create OpeningHours in the past
       */
      CannotCreateHoursInThePast = 'OpeningHours:CannotCreateHoursInThePast',
      /**
       * Time slots start time can not be higher or equal to the end time
       */
      TimeSlotStartTimeCanNotBeHigherOrEqualToTheEndTime = 'OpeningHours:TimeSlotStartTimeCanNotBeHigherOrEqualToTheEndTime',
      /**
       * Multiple time slots on the same day cannot overlap each other
       */
      TimeSlotsCannotOverlap = 'OpeningHours:TimeSlotsCannotOverlap',
      /**
       * Time slot start time and end time must be between 00:00:00.0 and 23:59:59.0
       */
      InvalidTimeSlotTimeSpan = 'OpeningHours:InvalidTimeSlotTimeSpan',
      /**
       * OpeninghoursTemplate can not be removed untill all organization unit sets are disconnected from it.
       */
      CannotRemoveOpeninghoursTemplateWhenOUSetsAreConnected = 'OpeningHours:CannotRemoveOpeninghoursTemplateWhenOUSetsAreConnected',
      /**
       * Special openinghours can not be added or updated when the date is in the past.
       */
      CannotAddOrUpdateSpecialOpeningHoursInThePast = 'OpeningHours:CannotAddOrUpdateSpecialOpeningHoursInThePast',
      /**
       * Either `DayOfWeek` or `Date` is required to set openinghours exception.
       */
      EitherDayOfWeekOrDateIsRequiredToSetOpeningHoursException = 'OpeningHours:EitherDayOfWeekOrDateIsRequiredToSetOpeningHoursException',
      /**
       * Invalid period given.
       */
      InvalidPeriodGiven = 'OpeningHours:InvalidPeriodGiven',
      /**
       * OrganizationUnitSet {0:number} is already attached to a template of the same type. When you wish to overwrite this, you must provide the 'OverrideIfOpeningHoursTypeAlreadyAttached' or remove the already attached template of the same type.
       */
      AlreadyAttached = 'OpeningHours:AlreadyAttached'
    }

    export const enum PhoneNumber {
      /**
       * The phonenumber is invalid: {0:string}
       */
      InvalidPhoneNumber = 'PhoneNumber:InvalidPhoneNumber'
    }

    export const enum GiftWrapping {
      /**
       * The selected product is not a greetingcard.
       */
      NoGreetingCardProduct = 'GiftWrapping:NoGreetingCardProduct',
      /**
       * The selected orderline is not available for giftwrapping.
       */
      LineNotAvailableForWrapping = 'GiftWrapping:LineNotAvailableForWrapping',
      /**
       * An OrderLine in the giftwrapping is not in the duplicated order, can't duplicate the giftwrapping.
       */
      LineInGiftWrappingNotInDuplicatedOrder = 'GiftWrapping:LineInGiftWrappingNotInDuplicatedOrder'
    }

    export const enum GiftCards {
      /**
       * The given amount {amount:number} is not in the allowed values {values:string}.
       */
      ActivateValueNotInValues = 'GiftCards:ActivateValueNotInValues',
      /**
       * The given amount {givenAmount:number} is lower than {minAmount:number}.
       */
      ActivateValueTooLow = 'GiftCards:ActivateValueTooLow',
      /**
       * The given amount {givenAmount:number} is higher than {maxAmount:number}.
       */
      ActivateValueTooHigh = 'GiftCards:ActivateValueTooHigh',
      /**
       * Failed to execute the action {action:string} for orderline {orderLineID:number}
       */
      ActionFailed = 'GiftCards:ActionFailed',
      /**
       * Failed to execute the action {0:string}
       */
      UnknownError = 'GiftCards:UnknownError',
      /**
       * It's required to supply a PIN when calling this services without being logged in
       */
      PinRequiredForAnonymous = 'GiftCards:PinRequiredForAnonymous',
      /**
       * Unable to determine the cardhandler for this card.
       */
      CardHandlerNotFound = 'GiftCards:CardHandlerNotFound',
      /**
       * No Giftcard configuration for product {0:number}
       */
      CardHandlerNotFoundForProduct = 'GiftCards:CardHandlerNotFoundForProduct',
      /**
       * Unable to fetch cardinfo: {0:string}
       */
      FailedToFetchCardInfo = 'GiftCards:FailedToFetchCardInfo',
      /**
       * The delivery of this giftcard is scheduled within an hour, updates are not allowed.
       */
      DeliveryScheduleToClose = 'GiftCards:DeliveryScheduleToClose',
      /**
       * Redemption of card {0:string} failed
       */
      RedemptionFailed = 'GiftCards:RedemptionFailed',
      /**
       * Card with number {0:string} was not found.
       */
      CardNotFound = 'GiftCards:CardNotFound',
      /**
       * Card with number {0:string} is expired.
       */
      CardExpired = 'GiftCards:CardExpired',
      /**
       * The card has currency {cardCurrency:string}, but the requested currency is {transactionCurrency:string}.
       */
      CurrencyMismatch = 'GiftCards:CurrencyMismatch',
      /**
       * The total balance of all the cards cannot cover the paid amount
       */
      CardsBalanceTooLow = 'GiftCards:CardsBalanceTooLow',
      /**
       * The current card handler does not support multiple cards at once.
       */
      MultipleCardsPaymentNotSupported = 'GiftCards:MultipleCardsPaymentNotSupported',
      /**
       * It is not allowed to pay for GiftCards with a GiftCard.
       */
      PayingForGiftCardsWithGiftCards = 'GiftCards:PayingForGiftCardsWithGiftCards',
      /**
       * A PaymentTransaction for this type of giftcard can only be refunded for the full amount
       */
      RefundsCanOnlyBeDoneForTheFullAmount = 'GiftCards:RefundsCanOnlyBeDoneForTheFullAmount',
      /**
       * No CardActionLedger could be found for the PaymentTransaction
       */
      NoCardActionLedgerFound = 'GiftCards:NoCardActionLedgerFound',
      /**
       * No refund option available for this giftcard
       */
      RefundNotAvailable = 'GiftCards:RefundNotAvailable',
      /**
       * This Giftcard does not allow for refunding this directly.
       */
      RefundWithoutTransactionNotAllowed = 'GiftCards:RefundWithoutTransactionNotAllowed',
      /**
       * This Giftcard has a higher value than the allowed amount for refunding directly.
       */
      CardBalanceTooHigh = 'GiftCards:CardBalanceTooHigh',
      /**
       * An expired card cannot be used.
       */
      Expired = 'GiftCards:Expired',
      /**
       * A card must be activated before use.
       */
      NotActivated = 'GiftCards:NotActivated',
      /**
       * Insufficient balance on card.
       */
      InsufficientBalance = 'GiftCards:InsufficientBalance'
    }

    export const enum APIGiftCard {
      /**
       * The cards have these currencies {cardCurrency:string}, but the requested ones are {transactionCurrency:string}.
       */
      CurrencyMismatches = 'APIGiftCard:CurrencyMismatches'
    }

    export const enum Finance {
      /**
       * There is still an open CashJournal for this Device.
       */
      OpenCashJournal = 'Finance:OpenCashJournal',
      /**
       * There is still an open CashJournal for this Station.
       */
      OpenCashJournalForStation = 'Finance:OpenCashJournalForStation'
    }

    export const enum FinancialPeriods {
      /**
       * There is no financial period available for this organization.
       */
      NoFinancialPeriod = 'FinancialPeriods:NoFinancialPeriod'
    }

    export const enum OpenFinancialPeriod {
      /**
       * The period has already been opened.
       */
      PeriodAlreadyOpened = 'OpenFinancialPeriod:PeriodAlreadyOpened'
    }

    export const enum CloseFinancialPeriod {
      /**
       * The period can only be closed when its status is Open or Closing, it's currently {0:string}.
       */
      PeriodNotOpen = 'CloseFinancialPeriod:PeriodNotOpen',
      /**
       * Financial period management for webshops is temporarily disabled.
       */
      WebshopFinancialPeriodManagementDisabled = 'CloseFinancialPeriod:WebshopFinancialPeriodManagementDisabled'
    }

    export const enum FinancialPeriodOpeningImpediments {
      /**
       * No employees linked to the financial period.
       */
      LinkedEmployees = 'FinancialPeriodOpeningImpediments:LinkedEmployees',
      /**
       * Loyalty program is not linked to the financial period.
       */
      LoyaltyProgram = 'FinancialPeriodOpeningImpediments:LoyaltyProgram'
    }

    export const enum FinancialPeriodClosingImpediments {
      /**
       * There are {0:number} active user tasks remaining that must be completed or deactivated before the period can be closed.
       */
      ActiveUserTasks = 'FinancialPeriodClosingImpediments:ActiveUserTasks',
      /**
       * There are {0:number} products with negative stock that must be corrected before the period can be closed.
       */
      NegativeStock = 'FinancialPeriodClosingImpediments:NegativeStock',
      /**
       * There are {0:number} open cash journals remaining. These must be closed before the period can be closed.
       */
      OpenCashJournals = 'FinancialPeriodClosingImpediments:OpenCashJournals',
      /**
       * The PCI question is not yet answered, this is required to close the period.
       */
      PCIQuestionNotAnswered = 'FinancialPeriodClosingImpediments:PCIQuestionNotAnswered'
    }

    export const enum FinancialPeriodExport {
      /**
       * No endpoint configured for period ledger export.
       */
      EndpointNotConfigured = 'FinancialPeriodExport:EndpointNotConfigured',
      /**
       * Financial period ledger export failed: {0:string}.
       */
      LedgerExportFailed = 'FinancialPeriodExport:LedgerExportFailed'
    }

    export const enum CashServices {
      /**
       * The given TotalAmount does not match with the counted coins and banknotes.
       */
      InvalidTotalAmount = 'CashServices:InvalidTotalAmount'
    }

    export const enum Cash {
      /**
       * The amount of {amount:number} is too precise, an amount in currency {currencyID:string} can only have a precision of {maxDigits:number} digits.
       */
      TooMuchPrecision = 'Cash:TooMuchPrecision',
      /**
       * Depositing cash requires a number.
       */
      CashDepositNumberRequired = 'Cash:CashDepositNumberRequired',
      /**
       * Depositing cash is only allowed from the safe.
       */
      CashDepositIsOnlyAllowedFromSafe = 'Cash:CashDepositIsOnlyAllowedFromSafe',
      /**
       * A required (pending) cash deposit is missing.
       */
      CashDepositMissing = 'Cash:CashDepositMissing',
      /**
       * Safe can not result in a negative balance
       */
      NegativeSafeBalance = 'Cash:NegativeSafeBalance',
      /**
       * This sealbag number was already used on {0:string}
       */
      CashDepositNumberAlreadyInUse = 'Cash:CashDepositNumberAlreadyInUse'
    }

    export const enum CashCorrections {
      /**
       * Remark is required when creating a cash correction.
       */
      RemarkIsRequired = 'CashCorrections:RemarkIsRequired'
    }

    export const enum CashDepositManager {
      /**
       * Cannot drop-off cash deposit because it is not pending.
       */
      CashDepositNotPending = 'CashDepositManager:CashDepositNotPending'
    }

    export const enum DeleteCashDeposit {
      /**
       * Cannot delete cash deposit because the cash journal has already been closed.
       */
      CashJournalClosed = 'DeleteCashDeposit:CashJournalClosed',
      /**
       * Cannot delete cash deposit because the financial period is not open.
       */
      FinancialPeriodClosed = 'DeleteCashDeposit:FinancialPeriodClosed'
    }

    export const enum CashJournal {
      /**
       * There is no open cash journal for device {0:string}
       */
      NoOpenCashJournal = 'CashJournal:NoOpenCashJournal',
      /**
       * There is still an open cash journal for device {0:string}
       */
      OpenCashJournal = 'CashJournal:OpenCashJournal',
      /**
       * This PaymentMethod requires a Device
       */
      DeviceRequired = 'CashJournal:DeviceRequired',
      /**
       * It's not possible to open a cash journal for a device when you're not currently logged into that OrganizationUnit.
       */
      InvalidOrganizationUnit = 'CashJournal:InvalidOrganizationUnit',
      /**
       * This device could not be found, is not a cash device, or it's not accessible to you.
       */
      InvalidDevice = 'CashJournal:InvalidDevice',
      /**
       * This cashjournal has CashJournalMethod None and can't be opened.
       */
      InvalidCashJournalMethod = 'CashJournal:InvalidCashJournalMethod',
      /**
       * This currency is not available for this organizationunit.
       */
      InvalidCurrency = 'CashJournal:InvalidCurrency'
    }

    export const enum OpenCashJournal {
      /**
       * There is no open financial period available for device {0:string}.
       */
      FinancialPeriodNotOpened = 'OpenCashJournal:FinancialPeriodNotOpened',
      /**
       * There is already an open cash journal for device {device:string}, paymenttype {paymentType:string} in currency {currencyID:string}.
       */
      AlreadyOpened = 'OpenCashJournal:AlreadyOpened',
      /**
       * The previous cash journal closed with a different amount than the new journal's opening, this should be corrected before the new journal can be opened.
       */
      UnexplainedOpeningDeviation = 'OpenCashJournal:UnexplainedOpeningDeviation'
    }

    export const enum CloseCashJournal {
      /**
       * The amount of cash in device {0:string} exceeds the allowed limit for this device.
       */
      ExceedsCashLimit = 'CloseCashJournal:ExceedsCashLimit',
      /**
       * The cash journal cannot be closed when the difference between the expected and the counted amount has not been corrected.
       */
      UnexplainedClosingDeviation = 'CloseCashJournal:UnexplainedClosingDeviation',
      /**
       * The cash journal cannot be closed with a correction being made without providing a reason or description.
       */
      MissingReasonOrDescriptionForCashCorrection = 'CloseCashJournal:MissingReasonOrDescriptionForCashCorrection'
    }

    export const enum MoveCash {
      /**
       * Source device ({source:string}) cannot be used for moving cash to destination device ({destination:string}).
       */
      InvalidDevice = 'MoveCash:InvalidDevice',
      /**
       * An amount of {currencyID:string} {amount:number} is not valid.
       */
      InvalidAmount = 'MoveCash:InvalidAmount',
      /**
       * When moving cash between two journals they must both be in the same financial period, but the source period was {sourceFinancialPeriodID:number} while the destination period was {destinationFinancialPeriodID:number}.
       */
      CashJournalsMustBeInSamePeriod = 'MoveCash:CashJournalsMustBeInSamePeriod',
      /**
       * Cannot supply Total and Coins or Notes at the same time. Try supplying either the Total or the Coins and Notes.
       */
      CannotSupplyTotalAndCoinsOrNotesAtTheSameTime = 'MoveCash:CannotSupplyTotalAndCoinsOrNotesAtTheSameTime'
    }

    export const enum ExchangeRates {
      /**
       * There is no exchange rate available from {baseCurrencyID:string} to {currencyID:string}
       */
      NoExchangeRateAvailable = 'ExchangeRates:NoExchangeRateAvailable'
    }

    export const enum EventExportConfiguration {
      /**
       * The provided endpoint is either invalid or is not secure
       */
      InvalidEndpoint = 'EventExportConfiguration:InvalidEndpoint',
      /**
       * Cannot change status to Error
       */
      CannotSetToError = 'EventExportConfiguration:CannotSetToError',
      /**
       * The provided config is invalid
       */
      InvalidConfig = 'EventExportConfiguration:InvalidConfig',
      /**
       * The key ID is invalid (should be either 1 or 2)
       */
      InvalidKeyID = 'EventExportConfiguration:InvalidKeyID'
    }

    export const enum Auditing {
      /**
       * There are multiple orders for invoice {0:number}, this is not allowed.
       */
      MultipleOrdersForInvoice = 'Auditing:MultipleOrdersForInvoice',
      /**
       * Not allowed to autocomplete orders in an audited country when invoicing is required.
       */
      AutoCompletionNotAllowed = 'Auditing:AutoCompletionNotAllowed',
      /**
       * '{0}' with value '{1}' doesn't pass validation.
       */
      ValidationError = 'Auditing:ValidationError'
    }

    export const enum Printing {
      /**
       * Unable to find a printer.
       */
      PrinterNotFound = 'Printing:PrinterNotFound',
      /**
       * IPP printing requires that a host name is configured.
       */
      MissingHostName = 'Printing:MissingHostName',
      /**
       * Device {0:number} has no network name or ip address specified, one of these is required for IPP printing
       */
      MissingNetworkName = 'Printing:MissingNetworkName',
      /**
       * Converting PDF to PCL through URL {0:string} failed.
       */
      FailureConvertingPDF2PCL = 'Printing:FailureConvertingPDF2PCL',
      /**
       * Failed to print: {0:string}
       */
      PrintFailure = 'Printing:PrintFailure'
    }

    export const enum Devices {
      /**
       * Unable to find a device.
       */
      DeviceNotFound = 'Devices:DeviceNotFound',
      /**
       * There is no connection available to the requested device.
       */
      NoConnectionAvailable = 'Devices:NoConnectionAvailable',
      /**
       * Watchtowers can only be created with the CreateWatchtowerDevice service.
       */
      WatchtowerTypeNotAllowed = 'Devices:WatchtowerTypeNotAllowed',
      /**
       * Devices need a Station or OrganizationUnit.
       */
      StationOrOrganizationUnitRequired = 'Devices:StationOrOrganizationUnitRequired',
      /**
       * Device {0:number} has no Address configured.
       */
      AddressNotConfigured = 'Devices:AddressNotConfigured',
      /**
       * Device {0:number} has no valid Address ({1:string}) configured.
       */
      UnableToParseAddress = 'Devices:UnableToParseAddress'
    }

    export const enum Stations {
      /**
       * Station is required.
       */
      StationRequired = 'Stations:StationRequired',
      /**
       * Station '{0:number}' is not a point of sale
       */
      StationIsNotPointOfSale = 'Stations:StationIsNotPointOfSale',
      /**
       * Station {0:number} cannot be used from your current user context.
       */
      InvalidStation = 'Stations:InvalidStation'
    }

    export const enum Global {
      /**
       * The client to communicate with the global eva instance is not configured.
       */
      GlobalClientNotConfigured = 'Global:GlobalClientNotConfigured',
      /**
       * This service has been deprecated from version {deprecationVersion:number} and will be removed in version {wopVersion:number}.
       */
      ServiceNoLongerInUse = 'Global:ServiceNoLongerInUse'
    }

    export const enum Users {
      /**
       * User does not exist.
       */
      DoesNotExist = 'Users:DoesNotExist',
      /**
       * Another user with this BackendID already exists.
       */
      DuplicateBackendID = 'Users:DuplicateBackendID',
      /**
       * Can only generate a PIN for an employee.
       */
      CanOnlyGeneratePINForEmployee = 'Users:CanOnlyGeneratePINForEmployee',
      /**
       * This PIN is not secure enough.
       */
      InsecurePIN = 'Users:InsecurePIN',
      /**
       * Tried to generate a PIN with length {attemptLength:number} but it must be length {requiredLength:number}.
       */
      InvalidLengthForPIN = 'Users:InvalidLengthForPIN',
      /**
       * EmailAddress '{0:string}' is already in use.
       */
      EmailAddressInUse = 'Users:EmailAddressInUse',
      /**
       * EmailAddress of a Customer can't be changed.
       */
      CantChangeCustomerEmailAddress = 'Users:CantChangeCustomerEmailAddress',
      /**
       * It is not allowed to change the EmailAddress of another employee.
       */
      CantChangeEmailAddressOfAnotherEmployee = 'Users:CantChangeEmailAddressOfAnotherEmployee',
      /**
       * Nickname '{0:string}' is already in use.
       */
      NicknameInUse = 'Users:NicknameInUse',
      /**
       * Fiscal ID '{0:string}' is already in use.
       */
      FiscalIDInUse = 'Users:FiscalIDInUse',
      /**
       * When no emailaddress is supplied, a password is required.
       */
      PasswordRequired = 'Users:PasswordRequired',
      /**
       * It is not allowed to set a password for employee accounts.
       */
      PasswordNotAllowedForEmployees = 'Users:PasswordNotAllowedForEmployees',
      /**
       * EmailAddress '{0:string}' is not valid.
       */
      InvalidEmailAddress = 'Users:InvalidEmailAddress',
      /**
       * PhoneNumber '{0:string}' is not valid.
       */
      InvalidPhoneNumber = 'Users:InvalidPhoneNumber',
      /**
       * This nickname '{0:string}' is not valid.
       */
      InvalidNickname = 'Users:InvalidNickname',
      /**
       * This password reset token is not valid
       */
      PasswordResetTokenInvalid = 'Users:PasswordResetTokenInvalid',
      /**
       * This password reset token has already been used to reset a password
       */
      PasswordResetTokenAlreadyUsed = 'Users:PasswordResetTokenAlreadyUsed',
      /**
       * This password is not valid: {0:string}
       */
      PasswordPolicyViolation = 'Users:PasswordPolicyViolation',
      /**
       * It is not allowed to create an employee that is also customer
       */
      MixedAccountsNotAllowed = 'Users:MixedAccountsNotAllowed',
      /**
       * It is not allowed to create a customer standard account in this visibility group
       */
      NoCustomerAccountsAllowedInThisVisibilityGroup = 'Users:NoCustomerAccountsAllowedInThisVisibilityGroup',
      /**
       * It is not allowed to recover your own account
       */
      CantRecoverOwnAccount = 'Users:CantRecoverOwnAccount',
      /**
       * The selected user is not visible.
       */
      NotVisible = 'Users:NotVisible',
      /**
       * Delete users limit of {0:number} exceeded.
       */
      DeleteUsersLimitExceeded = 'Users:DeleteUsersLimitExceeded',
      /**
       * Duplicate custom fields are invalid.
       */
      DuplicateCustomFields = 'Users:DuplicateCustomFields',
      /**
       * A company a user is attached to must be a consumer company.
       */
      UserCompanyMustBeConsumerCompany = 'Users:UserCompanyMustBeConsumerCompany'
    }

    export const enum Roles {
      /**
       * The Role {roleName:string} is not contained in the RoleSet of OrganizationUnit {ouName:string}.
       */
      RoleNotInRoleSet = 'Roles:RoleNotInRoleSet',
      /**
       * You are not allowed to set the role {roleName:string} for organization unit {ouName:string}
       */
      NotAllowedToSetRole = 'Roles:NotAllowedToSetRole',
      /**
       * The selected Role `{0:number}` has a non-matching UserType.
       */
      RoleUserTypeMismatch = 'Roles:RoleUserTypeMismatch',
      /**
       * This role still has users attached - please remove them first!
       */
      RoleStillHasUsers = 'Roles:RoleStillHasUsers'
    }

    export const enum Login {
      /**
       * Username is a required field when authenticating with credentials.
       */
      UsernameRequired = 'Login:UsernameRequired',
      /**
       * Failed to execute the custom login.
       */
      CustomAuthenticationFailed = 'Login:CustomAuthenticationFailed'
    }

    export const enum Barcodes {
      /**
       * Invalid barcode ({0:string}) provided
       */
      InvalidBarcode = 'Barcodes:InvalidBarcode',
      /**
       * Invalid handler type was provided to ExecuteBarcode.
       */
      InvalidExecuteHandlerType = 'Barcodes:InvalidExecuteHandlerType'
    }

    export const enum OrderLine {
      /**
       * Quantity cannot be modified: {0:string}
       */
      CannotModifyQuantity = 'OrderLine:CannotModifyQuantity',
      /**
       * Quantity to split must be positive and cannot be greather or equal to the quantity of the orderline.
       */
      InvalidSplitQuantity = 'OrderLine:InvalidSplitQuantity',
      /**
       * Order line #{0:number} has already been shipped.
       */
      AlreadyShipped = 'OrderLine:AlreadyShipped',
      /**
       * Require a product to mutate the order line
       */
      ProductRequiredToMutatePrice = 'OrderLine:ProductRequiredToMutatePrice',
      /**
       * Require a product with custom pricing to mutate the order line
       */
      ProductRequiresCustomPricingToMutatePrice = 'OrderLine:ProductRequiresCustomPricingToMutatePrice',
      /**
       * A GiftCard with CustomPricing requires a positive UnitPrice
       */
      GiftCardsRequireAPositiveUnitPrice = 'OrderLine:GiftCardsRequireAPositiveUnitPrice',
      /**
       * This lineactiontype is not valid for the orderline {OrderLineID:number}: {Error:string}
       */
      LineActionTypeChangeNotValid = 'OrderLine:LineActionTypeChangeNotValid',
      /**
       * OrderLine {0:number} has already been completed/shipped/invoiced and can therefor no longer be corrected in price.
       */
      UnitPriceCanNoLongerBeCorrected = 'OrderLine:UnitPriceCanNoLongerBeCorrected',
      /**
       * It is required to supply a reason for a UnitPriceCorrection.
       */
      UnitPriceCorrectionReasonRequired = 'OrderLine:UnitPriceCorrectionReasonRequired',
      /**
       * UnitPriceCorrections are only allowed on manual returnlines.
       */
      UnitPriceCorrectionNotAManualReturn = 'OrderLine:UnitPriceCorrectionNotAManualReturn',
      /**
       * Correction amount is limited to {Limit:number}, but {Requested:number} was requested.
       */
      CorrectionAmountExceeded = 'OrderLine:CorrectionAmountExceeded'
    }

    export const enum Shipment {
      /**
       * Cannot receive an unshipped shipment.
       */
      CannotReceiveUnshippedShipment = 'Shipment:CannotReceiveUnshippedShipment',
      /**
       * Cannot fully receive shipment {0:number}, because all lines are already completed.
       */
      CannotFullyReceiveCompletedShipment = 'Shipment:CannotFullyReceiveCompletedShipment',
      /**
       * Shipment already exists with that BackendSystemID/BackendID combination: {backendSystemID:string}/{backendID:string}
       */
      ShipmentAlreadyExists = 'Shipment:ShipmentAlreadyExists',
      /**
       * ShipmentReceipt already exists with that BackendSystemID/BackendID combination: {backendSystemID:string}/{backendID:string}
       */
      ShipmentReceiptAlreadyExists = 'Shipment:ShipmentReceiptAlreadyExists',
      /**
       * Can only ship purchase orders.
       */
      CanOnlyShipPurchaseOrders = 'Shipment:CanOnlyShipPurchaseOrders',
      /**
       * Orders should be supplied with full details.
       */
      OrderWithFullDetailsRequired = 'Shipment:OrderWithFullDetailsRequired',
      /**
       * ShipFromOrganizationUnit does not match for order #{orderID:number} which has #{shipFromOnOrder:number} and the shipment being created which has #{shipFromOnShipment:number}
       */
      ShipFromOrganizationUnitMismatch = 'Shipment:ShipFromOrganizationUnitMismatch',
      /**
       * The shipment contains an order of the wrong type for shipmentreceiver {0:string}
       */
      OrderTypeMismatch = 'Shipment:OrderTypeMismatch',
      /**
       * Surplus are not allowed.
       */
      SurplusNotAllowed = 'Shipment:SurplusNotAllowed',
      /**
       * Overshipment is not allowed; attempted to ship {quantityShipped:number} on order line {orderLine:number} even though it only has an open quantity of {quantityOpen:number}.
       */
      OvershipmentNotAllowed = 'Shipment:OvershipmentNotAllowed',
      /**
       * Shipment {0:number} is not Pending and cannot be mutated.
       */
      IsNotPending = 'Shipment:IsNotPending',
      /**
       * Pending shipments are not yet supported for non-sales orders.
       */
      PendingShipmentNotSupported = 'Shipment:PendingShipmentNotSupported',
      /**
       * Can't return OrderLine {returnOrderLineID:number} on Order {returnOrderID:number} for original OrderLine {originalOrderLineID:number}. The QuantityReturned ({quantityReturned:number}) is greather than the QuantityShipped ({quantityShipped:number})
       */
      CannotReturnMoreThanShipped = 'Shipment:CannotReturnMoreThanShipped',
      /**
       * Multiple shipping methods are not supported for one shipment.
       */
      MultipleShippingMethodsNotSupported = 'Shipment:MultipleShippingMethodsNotSupported',
      /**
       * There are no lines to receive specified for shipment {0:number}
       */
      NoReceiptLinesSpecified = 'Shipment:NoReceiptLinesSpecified',
      /**
       * Can't ship order {orderID:number}. There is still an open amount of {openAmount:number}.
       */
      OpenAmount = 'Shipment:OpenAmount',
      /**
       * A shipment can either contain lines or packages with lines, not both.
       */
      ShipmentCanOnlyContainLinesOrPackagesWithLines = 'Shipment:ShipmentCanOnlyContainLinesOrPackagesWithLines',
      /**
       * Shipment with multiple packages aren't allowed.
       */
      ShipmentWithMultiplePackagesNotAllowed = 'Shipment:ShipmentWithMultiplePackagesNotAllowed',
      /**
       * Preventing creation of empty shipment!
       */
      EmptyShipment = 'Shipment:EmptyShipment',
      /**
       * Invalid delivery type for shipping method
       */
      InvalidShippingMethodDeliveryType = 'Shipment:InvalidShippingMethodDeliveryType',
      /**
       * Package does not match shipping method
       */
      PackageDoesNotMatchShippingMethod = 'Shipment:PackageDoesNotMatchShippingMethod'
    }

    export const enum EntityFieldValidator {
      /**
       * The provided field name is not available or doesn't exist for this entity. To see available fields, execute ListEntityFieldValidatorEntityFieldNames.
       */
      FieldDoesNotExist = 'EntityFieldValidator:FieldDoesNotExist',
      /**
       * There is already a validator that covers this usertype.
       */
      UserTypeAlreadyExists = 'EntityFieldValidator:UserTypeAlreadyExists',
      /**
       * EntityFieldValidators on {entity:string} -> {property:string} with value {value:string} doesn't comply with rule {rule:string} -> {ruleValue:string}.
       */
      FieldValidationError = 'EntityFieldValidator:FieldValidationError'
    }

    export const enum Payment {
      /**
       * An Order with an attached customer is required for this payment.
       */
      CustomerRequired = 'Payment:CustomerRequired',
      /**
       * Transaction {0:number} has already been confirmed.
       */
      TransactionAlreadyConfirmed = 'Payment:TransactionAlreadyConfirmed',
      /**
       * Transaction {0:number} is already being processed.
       */
      TransactionAlreadyProcessing = 'Payment:TransactionAlreadyProcessing',
      /**
       * TransactionCapture {0:number} has already been confirmed.
       */
      CaptureAlreadyConfirmed = 'Payment:CaptureAlreadyConfirmed',
      /**
       * Transaction {0:number} is failed, create a new transaction.
       */
      TransactionFailed = 'Payment:TransactionFailed',
      /**
       * Refund failed: {0:string}
       */
      RefundFailed = 'Payment:RefundFailed',
      /**
       * Refund is not available for this order
       */
      RefundNotAvailable = 'Payment:RefundNotAvailable',
      /**
       * Requested to refund {requested:number} on the order, but only {available:number} is available for refund
       */
      RefundAmountNotAvailable = 'Payment:RefundAmountNotAvailable',
      /**
       * PaymentMethod {0:string} is not available
       */
      PaymentMethodNotAvailable = 'Payment:PaymentMethodNotAvailable',
      /**
       * Can't pay more than is due. Order {orderID:number} has an openamount of {openAmount:number} and a payment of {amountToPay:number} is requested.
       */
      OverPayment = 'Payment:OverPayment',
      /**
       * CashJournalMethod on PaymentType must be None/Open/OpenAndClose. {0:PaymentCashJournalMethod} is not allowed.
       */
      InvalidCashJournalMethod = 'Payment:InvalidCashJournalMethod',
      /**
       * Only Custom PaymentMethods can be mutated.
       */
      PaymentMethodIsNotCustom = 'Payment:PaymentMethodIsNotCustom',
      /**
       * A PaymentType must be of the same PaymentMethod.
       */
      PaymentTypeCannotBeChangedToOtherMethod = 'Payment:PaymentTypeCannotBeChangedToOtherMethod',
      /**
       * A PaymentTransaction must be committed before it can be captured.
       */
      UncommittedPaymentsCantBeCaptured = 'Payment:UncommittedPaymentsCantBeCaptured',
      /**
       * This paymenttransaction cannot be captured.
       */
      CaptureNotAvailable = 'Payment:CaptureNotAvailable',
      /**
       * This paymenttransaction can no langer be captured, the authorization has been expired.
       */
      AuthorizationExpired = 'Payment:AuthorizationExpired',
      /**
       * The selected device is already in use for a pin transaction.
       */
      PinDeviceAlreadyInUse = 'Payment:PinDeviceAlreadyInUse',
      /**
       * Cannot perform this action when setting `PaymentMethods:PaymentTypeAvailabilityByOuSetType` is enabled!
       */
      PaymentTypeAvailabilityByOuSetTypeEnabled = 'Payment:PaymentTypeAvailabilityByOuSetTypeEnabled',
      /**
       * The current payment method does not support multiple cards at once.
       */
      MultipleCardsPaymentNotSupported = 'Payment:MultipleCardsPaymentNotSupported',
      /**
       * Payment initialization not supported
       */
      InitializationNotSupported = 'Payment:InitializationNotSupported',
      /**
       * The current payment method does not support multi currency.
       */
      PaymentMethodDoesNotSupportMultiCurrency = 'Payment:PaymentMethodDoesNotSupportMultiCurrency'
    }

    export const enum Discounts {
      /**
       * Coupon {0:string} is invalid.
       */
      InvalidDiscountCoupon = 'Discounts:InvalidDiscountCoupon',
      /**
       * Discount {DiscountID:number} has already been used the maximum number of times.
       */
      MaximumDiscountUsageReached = 'Discounts:MaximumDiscountUsageReached',
      /**
       * Discount {0:number} has already been used the maximum number of times by the user.
       */
      MaximumDiscountUserUsageCountReached = 'Discounts:MaximumDiscountUserUsageCountReached',
      /**
       * User reached the maximum usage of discount {0:number}.
       */
      MaximumDiscountUserUsageBalanceReached = 'Discounts:MaximumDiscountUserUsageBalanceReached',
      /**
       * User {CustomerID:number} has insufficient credit to use Discount {DiscountID:number}. Available: {Balance:number}, Discount: {DiscountAmount:number}, Order: {OrderID:number}
       */
      InsufficientUserCardBalance = 'Discounts:InsufficientUserCardBalance',
      /**
       * Coupon resolved on different handler then registerd on the discount.
       */
      InvalidCouponHandlerResolvement = 'Discounts:InvalidCouponHandlerResolvement',
      /**
       * Coupon '{0:string}' not usable in current organization unit.
       */
      CouponNotUsableInCurrentOrganizationUnit = 'Discounts:CouponNotUsableInCurrentOrganizationUnit',
      /**
       * Coupon '{couponCode:string}' not usable for currency '{currency:string}'.
       */
      CouponNotUsableInCurrentCurrency = 'Discounts:CouponNotUsableInCurrentCurrency',
      /**
       * Coupon '{couponCode:string}' from '{handler:string}' not longer usable.
       */
      CouponNoLongerUsable = 'Discounts:CouponNoLongerUsable',
      /**
       * Coupon maximum usage reached
       */
      CouponMaximumUsageReached = 'Discounts:CouponMaximumUsageReached',
      /**
       * Coupon has expired
       */
      ExpiredCoupon = 'Discounts:ExpiredCoupon',
      /**
       * Coupon already added
       */
      CouponAlreadyAdded = 'Discounts:CouponAlreadyAdded',
      /**
       * Discount can not be used as a manual discount. Trigger is not set as manual.
       */
      DiscountNotSetAsManualDiscount = 'Discounts:DiscountNotSetAsManualDiscount',
      /**
       * Discount can not be used as a loyalty discount. Trigger is not set as manual.
       */
      DiscountNotSetAsLoyaltyDiscount = 'Discounts:DiscountNotSetAsLoyaltyDiscount',
      /**
       * Coupon {couponCode:string} with DiscountBackendID `{discountBackendID:string}` did not resolve to a valid discount.
       */
      CouponDidNotResolveToAValidDiscount = 'Discounts:CouponDidNotResolveToAValidDiscount',
      /**
       * Invalid coupon
       */
      InvalidCoupon = 'Discounts:InvalidCoupon',
      /**
       * Line level discount requires lines te be applied to
       */
      LineLevelDiscountRequiresLinesToApplyTo = 'Discounts:LineLevelDiscountRequiresLinesToApplyTo',
      /**
       * Discount reason required
       */
      DiscountReasonRequired = 'Discounts:DiscountReasonRequired',
      /**
       * Customer is required in order to apply this discount.
       */
      CustomerRequiredForDiscount = 'Discounts:CustomerRequiredForDiscount',
      /**
       * Required custom field `{0:number}` is missing.
       */
      RequiredCustomFieldMissing = 'Discounts:RequiredCustomFieldMissing',
      /**
       * Manual discount amount is invalid
       */
      ManualDiscountAmountIsInvalid = 'Discounts:ManualDiscountAmountIsInvalid',
      /**
       * Manual discount already applied to the order.
       */
      ManualDiscountAlreadyAppliedToTheOrder = 'Discounts:ManualDiscountAlreadyAppliedToTheOrder',
      /**
       * Manual discount given on invalid orderline.
       */
      ManualDiscountGivenOnInvalidOrderLine = 'Discounts:ManualDiscountGivenOnInvalidOrderLine',
      /**
       * Manual discount on line can not exceed orderline quantity.
       */
      ManualDiscountOnLineCanNotExceedOrderLineQuantity = 'Discounts:ManualDiscountOnLineCanNotExceedOrderLineQuantity',
      /**
       * Discount is already verified
       */
      DiscountAlreadyVerified = 'Discounts:DiscountAlreadyVerified',
      /**
       * Discount is not verified
       */
      DiscountIsNotVerified = 'Discounts:DiscountIsNotVerified',
      /**
       * Discount is not active
       */
      DiscountIsntActive = 'Discounts:DiscountIsntActive',
      /**
       * Discount is archived and can no longer be used
       */
      DiscountArchived = 'Discounts:DiscountArchived',
      /**
       * Discount is system managed and can not be displayed.
       */
      DiscountIsSystemManaged = 'Discounts:DiscountIsSystemManaged',
      /**
       * Order is not of correct type for the discount
       */
      OrderDoesNotMeetDiscountOrderType = 'Discounts:OrderDoesNotMeetDiscountOrderType',
      /**
       * Discount is disabled on order
       */
      DiscountIsDisabledOnOrder = 'Discounts:DiscountIsDisabledOnOrder',
      /**
       * Coupon is invalid
       */
      CouponIsInvalid = 'Discounts:CouponIsInvalid',
      /**
       * Allow price riles for interbranch order is disabled
       */
      AllowPriceRulesForInterbranchOrdersIsDisabled = 'Discounts:AllowPriceRulesForInterbranchOrdersIsDisabled',
      /**
       * Automatic discounts aren't allowed on returntosupplier orders
       */
      AutomaticDiscountsArentAllowedOnReturnToSupplierOrders = 'Discounts:AutomaticDiscountsArentAllowedOnReturnToSupplierOrders',
      /**
       * Discount on return order is disabled
       */
      DiscountOnReturnOrderIsDisabled = 'Discounts:DiscountOnReturnOrderIsDisabled',
      /**
       * Discount on return reversal order disabled
       */
      DiscountOnReturnReversalIsDisabled = 'Discounts:DiscountOnReturnReversalIsDisabled',
      /**
       * Order did not meet all of the discount conditions. Condition(s) not met; {0:string}
       */
      OrderDidNotMeetAllTheDiscountConditions = 'Discounts:OrderDidNotMeetAllTheDiscountConditions',
      /**
       * Order did not meet any of the discount conditions. Condition(s) not met; {0:string}
       */
      OrderDidNotMeetAnyOfTheDiscountConditions = 'Discounts:OrderDidNotMeetAnyOfTheDiscountConditions',
      /**
       * Roles can only be applied to manual discounts.
       */
      RolesCanOnlyBeAppliedToManualDiscounts = 'Discounts:RolesCanOnlyBeAppliedToManualDiscounts',
      /**
       * Discount with trigger {0:string} can not be disabled on order. Cancel or removed them.
       */
      CannotDisableDiscountOfTypeX = 'Discounts:CannotDisableDiscountOfTypeX',
      /**
       * {0:string} discount action type is invalid.
       */
      InvalidActionType = 'Discounts:InvalidActionType',
      /**
       * {0:string} discount condition type is invalid.
       */
      InvalidConditionType = 'Discounts:InvalidConditionType',
      /**
       * Discount is generated from a template and can not be modified
       */
      DiscountGeneratedFromTemplate = 'Discounts:DiscountGeneratedFromTemplate',
      /**
       * Discount campaign name is required.
       */
      DiscountCampaignNameIsRequired = 'Discounts:DiscountCampaignNameIsRequired',
      /**
       * Discount campaign names must be unique. '{0:string}' is already taken.
       */
      DiscountCampaignNameMustBeUnique = 'Discounts:DiscountCampaignNameMustBeUnique',
      /**
       * Discount can only applied with order generated coupons.
       */
      DiscountRequiresOrderGeneratedCoupon = 'Discounts:DiscountRequiresOrderGeneratedCoupon',
      /**
       * Coupon originating order does not exist.
       */
      OriginatingOrderDoesNotExist = 'Discounts:OriginatingOrderDoesNotExist',
      /**
       * Coupon originating order isn't paid.
       */
      OriginatingOrderIsntPaid = 'Discounts:OriginatingOrderIsntPaid',
      /**
       * Coupon originating order isn't shipped.
       */
      OriginatingOrderIsntShipped = 'Discounts:OriginatingOrderIsntShipped',
      /**
       * Coupon originating order is fully returned and coupon can no longer be used.
       */
      OriginatingOrderIsFullyReturned = 'Discounts:OriginatingOrderIsFullyReturned',
      /**
       * Can not create coupon for discount that hasn't got triggertype 'Coupon'.
       */
      CannotCreateCouponForDiscountWhereTriggerTypeIsntCoupon = 'Discounts:CannotCreateCouponForDiscountWhereTriggerTypeIsntCoupon',
      /**
       * Coupons for this discount are generated by another discount.
       */
      CouponsForThisDiscountAreGeneratedByOtherDiscount = 'Discounts:CouponsForThisDiscountAreGeneratedByOtherDiscount',
      /**
       * Cannot modify active discount template.
       */
      CannotModifyActiveDiscountTemplate = 'Discounts:CannotModifyActiveDiscountTemplate',
      /**
       * Cannot create discount from inactive discount template.
       */
      CannotCreateDiscountFromInActiveDiscountTemplate = 'Discounts:CannotCreateDiscountFromInActiveDiscountTemplate',
      /**
       * Service only accepts discount lines to cancel.
       */
      ServiceOnlyAcceptsDiscountLinesToCancel = 'Discounts:ServiceOnlyAcceptsDiscountLinesToCancel',
      /**
       * DiscountLines of discount type {0:string} cannot be cancelled
       */
      DiscountLinesOfDiscountTypeXCannotBeCancelled = 'Discounts:DiscountLinesOfDiscountTypeXCannotBeCancelled',
      /**
       * CustomUserUsageField {CustomFieldID:number} on Discount {DiscountID:number} is not (or no longer) a decimal data type.
       */
      CustomUserUsageFieldTypeInvalid = 'Discounts:CustomUserUsageFieldTypeInvalid',
      /**
       * The value in CustomUserUsageField {CustomFieldID:number} for customer {CustomerID:number} is to low to use Discount {DiscountID:number}. CustomUserUsageField value: {CustomUserUsageFieldValue:number}, Discount amount: {DiscountAmount:number}, Order: {OrderID:number}
       */
      CustomUserUsageFieldValueToLow = 'Discounts:CustomUserUsageFieldValueToLow',
      /**
       * Discount {0:number} does not support product selection.
       */
      DiscountDoesNotSupportProductSelection = 'Discounts:DiscountDoesNotSupportProductSelection',
      /**
       * Failed to load discount product selection data for orderline {0:number}.
       */
      FailedToLoadSelectionDataForDiscountLine = 'Discounts:FailedToLoadSelectionDataForDiscountLine',
      /**
       * Invalid orderline for discount product selection.
       */
      InvalidOrderlineForDiscountProductSelection = 'Discounts:InvalidOrderlineForDiscountProductSelection',
      /**
       * Coupon `{0:string}` already exists on discount.
       */
      CouponAlreadyExistsOnDiscount = 'Discounts:CouponAlreadyExistsOnDiscount',
      /**
       * Coupon `{0:string}` is bound to a specific user.
       */
      CouponIsUserBound = 'Discounts:CouponIsUserBound',
      /**
       * There are duplicate coupon codes in the upload.
       */
      DuplicateCouponCodeInUpload = 'Discounts:DuplicateCouponCodeInUpload',
      /**
       * The selected product requirement does not (or does no longer) exist.
       */
      InvalidProductRequirement = 'Discounts:InvalidProductRequirement',
      /**
       * The given product requirement value (or one of the values) is empty.
       */
      InvalidProductRequirementValue = 'Discounts:InvalidProductRequirementValue',
      /**
       * Cannot apply this discount to an order of the logged in user.
       */
      CannotApplyDiscountToOwnOrder = 'Discounts:CannotApplyDiscountToOwnOrder',
      /**
       * Loyalty program no longer valid.
       */
      LoyaltyProgramNoLongerValid = 'Discounts:LoyaltyProgramNoLongerValid',
      /**
       * Not subscribed to loyalty program.
       */
      NotSubscribedToLoyaltyProgram = 'Discounts:NotSubscribedToLoyaltyProgram',
      /**
       * Loyalty program subscription blocked.
       */
      LoyaltyProgramSubscriptionBlocked = 'Discounts:LoyaltyProgramSubscriptionBlocked',
      /**
       * Cannot withdraw from loyalty program.
       */
      CannotWithdrawFromLoyaltyProgram = 'Discounts:CannotWithdrawFromLoyaltyProgram',
      /**
       * There are insufficient loyalty points to apply the discount.
       */
      InsufficientLoyaltyPoints = 'Discounts:InsufficientLoyaltyPoints',
      /**
       * Discount cannot be applied to lines, only on order.
       */
      DiscountCannotBeAppliedToLines = 'Discounts:DiscountCannotBeAppliedToLines',
      /**
       * All discount layers are required for reordering.
       */
      AllDiscountLayersAreRequiredForReordering = 'Discounts:AllDiscountLayersAreRequiredForReordering',
      /**
       * The system discount layer cannot be changed
       */
      SystemLayerCannotBeChanged = 'Discounts:SystemLayerCannotBeChanged',
      /**
       * Layer has active discounts.
       */
      LayerHasActiveDiscounts = 'Discounts:LayerHasActiveDiscounts',
      /**
       * A description is required.
       */
      DescriptionRequired = 'Discounts:DescriptionRequired',
      /**
       * A currency is required.
       */
      CurrencyRequired = 'Discounts:CurrencyRequired',
      /**
       * An action is required.
       */
      ActionRequired = 'Discounts:ActionRequired',
      /**
       * An OrganizationUnitSet is required.
       */
      OrganizationUnitSetRequired = 'Discounts:OrganizationUnitSetRequired',
      /**
       * A Layer is required.
       */
      LayerRequired = 'Discounts:LayerRequired',
      /**
       * The selected date range is invalid.
       */
      InvalidDateRange = 'Discounts:InvalidDateRange',
      /**
       * A (Discount)OrderType is required.
       */
      OrderTypeRequired = 'Discounts:OrderTypeRequired',
      /**
       * A customer is required.
       */
      CustomerRequired = 'Discounts:CustomerRequired',
      /**
       * DiscountValue in LookupList cannot be negative
       */
      DiscountValueInLookupListCannotBeNegative = 'Discounts:DiscountValueInLookupListCannotBeNegative',
      /**
       * DiscountValue in LookupList must be a valid percentage
       */
      DiscountValueInLookupListMustBeAValidPercentage = 'Discounts:DiscountValueInLookupListMustBeAValidPercentage',
      /**
       * CustomField to use as discount value must be of type integer or decimal
       */
      CustomFieldToUseAsDiscountValueMustBeOfTypeIntegerOrDecimal = 'Discounts:CustomFieldToUseAsDiscountValueMustBeOfTypeIntegerOrDecimal',
      /**
       * CustomField enum values must be convertible to numeric (integer or decimal) values
       */
      CustomFieldEnumValuesMustBeConvertibleToNumericValues = 'Discounts:CustomFieldEnumValuesMustBeConvertibleToNumericValues'
    }

    export const enum DiscountsV2 {
      /**
       * Old discount actions and conditions are no longer supported. Please update the discount or archive it and create a new one (recommended).
       */
      OldDiscountActionsAndConditionsAreNoLongerSupported = 'DiscountsV2:OldDiscountActionsAndConditionsAreNoLongerSupported',
      /**
       * The discount calculation failed with an unexpected error.
       */
      DiscountCalculationFailedWithUnexpectedError = 'DiscountsV2:DiscountCalculationFailedWithUnexpectedError',
      /**
       * DiscountCondition {0:string} is not allowed for discount trigger {1:string}.
       */
      ConditionNotEnabledForTriggerType = 'DiscountsV2:ConditionNotEnabledForTriggerType',
      /**
       * CouponCondition {ConditionID:number} for discount {DiscountID:number} was malformed or its build-up resulted in an exception.
       */
      InvalidDiscountCouponCondition = 'DiscountsV2:InvalidDiscountCouponCondition',
      /**
       * The data for CouponCondition {ConditionID:number} on discount {DiscountID:number} is not initialized properly.
       */
      DiscountCouponConditionDataNotInitialized = 'DiscountsV2:DiscountCouponConditionDataNotInitialized',
      /**
       * The DiscountAction for discount {0:number} was malformed or its build-up resulted in an exception.
       */
      InvalidDiscountAction = 'DiscountsV2:InvalidDiscountAction',
      /**
       * The DiscountAction {0:string} is not allowed for discount trigger {1:string}.
       */
      ActionNotEnabledForTriggerType = 'DiscountsV2:ActionNotEnabledForTriggerType',
      /**
       * Action data for discount {0:number} is not initialized properly.
       */
      ActionDataNotInitialized = 'DiscountsV2:ActionDataNotInitialized',
      /**
       * Discount action type can not be updated when discount is used.
       */
      ActionTypeCanNotBeChangedWhenDiscountIsUsed = 'DiscountsV2:ActionTypeCanNotBeChangedWhenDiscountIsUsed',
      /**
       * Discount cannot be applied because order {0:number} is already invoiced.
       */
      OrderIsAlreadyInvoiced = 'DiscountsV2:OrderIsAlreadyInvoiced',
      /**
       * Discount cannot be applied because order {0:number} is placed and `FreezeOrderAfterPlacement` is enabled.
       */
      OrderIsPlacedAndFreezeOrderAfterPlacementIsEnabled = 'DiscountsV2:OrderIsPlacedAndFreezeOrderAfterPlacementIsEnabled',
      /**
       * Discount cannot be applied because order {0:number} has IgnoreDiscounts enabled.
       */
      IgnoreDiscountsEnabled = 'DiscountsV2:IgnoreDiscountsEnabled',
      /**
       * Condition '{0:string}' is deprecated.
       */
      ConditionIsDeprecated = 'DiscountsV2:ConditionIsDeprecated',
      /**
       * Action '{0:string}' is deprecated.
       */
      ActionIsDeprecated = 'DiscountsV2:ActionIsDeprecated',
      /**
       * Action '{ActionType:string}' requires at least {X:number} occurences of the condition '{ConditionType:string}'.
       */
      ActionRequiresTheConditionAtLeastXTimes = 'DiscountsV2:ActionRequiresTheConditionAtLeastXTimes',
      /**
       * Action '{ActionType:string}' allows a maximum of {X:number} occurences of the condition '{ConditionType:string}'.
       */
      ActionAllowsTheConditionForAMaximumOfXTimes = 'DiscountsV2:ActionAllowsTheConditionForAMaximumOfXTimes',
      /**
       * Action '{ActionType:string}' can not be combined with condition type '{ConditionType:string}'.
       */
      ActionDoesNotAllowConditions = 'DiscountsV2:ActionDoesNotAllowConditions',
      /**
       * Action '{0:string}' can not be applied to a particular line.
       */
      ActionDoesNotAllowLineLevelDiscount = 'DiscountsV2:ActionDoesNotAllowLineLevelDiscount',
      /**
       * The value of property {0:string} is missing
       */
      MissingValue = 'DiscountsV2:MissingValue',
      /**
       * The value of property {0:string} is cannot be fractional.
       */
      FractionalPercentageNotSupported = 'DiscountsV2:FractionalPercentageNotSupported',
      /**
       * At least one of the following values is required: {0:string}
       */
      RequiresAtLeastOneOfTheFollowingValues = 'DiscountsV2:RequiresAtLeastOneOfTheFollowingValues',
      /**
       * The value of property {0:string} must be higher then 0.
       */
      ValueCannotBeNegativeOrZero = 'DiscountsV2:ValueCannotBeNegativeOrZero',
      /**
       * The value of property {0:string} must be higher then or equals to 0.
       */
      ValueCannotBeNegative = 'DiscountsV2:ValueCannotBeNegative',
      /**
       * The value of property {PropertyName:string} must be between {1:number} and {2:number}.
       */
      IntValueMustBeBetweenXAndY = 'DiscountsV2:IntValueMustBeBetweenXAndY',
      /**
       * Value of property {PropertyNameX:string} must be equal to or larger then property {PropertyNameY:string}
       */
      ValueOfXMustBeLargerThenOrEqualToValueOfY = 'DiscountsV2:ValueOfXMustBeLargerThenOrEqualToValueOfY',
      /**
       * Value of property {PropertyNameX:string} must be larger then property {PropertyNameY:string}
       */
      ValueOfXMustBeLargerThenValueOfY = 'DiscountsV2:ValueOfXMustBeLargerThenValueOfY',
      /**
       * The date of property {0:string} is invalid.
       */
      InvalidDate = 'DiscountsV2:InvalidDate',
      /**
       * The value of property {PropertyName:string} must be between {1:string} and {2:string}.
       */
      DateValueMustBeBetweenXAndY = 'DiscountsV2:DateValueMustBeBetweenXAndY',
      /**
       * At least two tiers is required.
       */
      NoTiersGiven = 'DiscountsV2:NoTiersGiven',
      /**
       * Duplicate tier with value '{0:any}' detected.
       */
      DuplicateTiersGiven = 'DiscountsV2:DuplicateTiersGiven',
      /**
       * ProductSets in {0:string} can not be mixed. All ProductSets must be either including or exluding.
       */
      ProductSetsIncludingExcludingCanNotBeMixed = 'DiscountsV2:ProductSetsIncludingExcludingCanNotBeMixed',
      /**
       * No product sets are provided.
       */
      NoProductSetsGiven = 'DiscountsV2:NoProductSetsGiven',
      /**
       * At least one product set must be selected as the discountable product set.
       */
      NoDiscountableProductSetSelected = 'DiscountsV2:NoDiscountableProductSetSelected',
      /**
       * The value of property {PropertyName:string} in tier {TierNumber:number} is missing
       */
      TierMissingValue = 'DiscountsV2:TierMissingValue',
      /**
       * Tier {TierNumber:number} requires at least one of the following values: {PropertyNames:string}
       */
      TierRequiresAtLeastOneOfTheFollowingValues = 'DiscountsV2:TierRequiresAtLeastOneOfTheFollowingValues',
      /**
       * The value of property {PropertyName:string} in tier {TierNumber:number} must be higher then 0.
       */
      TierValueCannotBeNegativeOrZero = 'DiscountsV2:TierValueCannotBeNegativeOrZero',
      /**
       * The value of property {PropertyName:string} in tier {TierNumber:number} must be higher then or equals to 0.
       */
      TierValueCannotBeNegative = 'DiscountsV2:TierValueCannotBeNegative',
      /**
       * The value of property {PropertyName:string} in tier {TierNumber:number} must be between {2:number} and {3:number}.
       */
      TierValueMustBeBetweenXAndY = 'DiscountsV2:TierValueMustBeBetweenXAndY',
      /**
       * Product with ID {ID:number} in property {PropertyName:string} for tier {TierNumber:number} not found.
       */
      TierProductNotFound = 'DiscountsV2:TierProductNotFound',
      /**
       * All product sets must have a name.
       */
      AllProductsSetsRequireANameToBePresent = 'DiscountsV2:AllProductsSetsRequireANameToBePresent',
      /**
       * Product set {ProductSetName:string} requires at least one of the following values: {PropertyName:string}
       */
      ProductSetRequiresAtLeastOneOfTheFollowingValues = 'DiscountsV2:ProductSetRequiresAtLeastOneOfTheFollowingValues',
      /**
       * The value of product set property {PropertyName:string} in product set {ProductSetName:string} must be higher then 0.
       */
      ProductSetValueCannotBeNegativeOrZero = 'DiscountsV2:ProductSetValueCannotBeNegativeOrZero',
      /**
       * The value of product set property {PropertyName:string} in product set {ProductSetName:string} must be higher then or equals to 0.
       */
      ProductSetValueCannotBeNegative = 'DiscountsV2:ProductSetValueCannotBeNegative',
      /**
       * The value of product set property {PropertyNameX:string} in product set {ProductSetName:string} must be higher then or equals to property {PropertyNameY:string}.
       */
      ProductSetValueOfXMustBeLargerThenOrEqualToValueOfY = 'DiscountsV2:ProductSetValueOfXMustBeLargerThenOrEqualToValueOfY',
      /**
       * The product set {ProductSetName:string} requires a connected loyalty program when property {PropertyName:string} is set.
       */
      ProductSetUseLoyaltyProgramProductsRequiresALoyaltyProgram = 'DiscountsV2:ProductSetUseLoyaltyProgramProductsRequiresALoyaltyProgram',
      /**
       * Product set {0:string} did not resolve to a valid list of products.
       */
      ProductSetDidNotResolveToAListOfValidProducts = 'DiscountsV2:ProductSetDidNotResolveToAListOfValidProducts',
      /**
       * Product set {0:string} product requirement values must be present.
       */
      ProductSetProductRequirementValuesMustBePresent = 'DiscountsV2:ProductSetProductRequirementValuesMustBePresent',
      /**
       * Product set properties ProductsIDs, ProductSearchID, Filters, PersonalizedPromotionID and UseLoyaltyProgramProducts cannot be combined in product sets.
       */
      ProductSetPropertiesAreMutuallyExclusive = 'DiscountsV2:ProductSetPropertiesAreMutuallyExclusive',
      /**
       * A productset price filter requires a valid from/to value.
       */
      ProductSetPriceFilterRequiresAValidFromOrToValue = 'DiscountsV2:ProductSetPriceFilterRequiresAValidFromOrToValue',
      /**
       * The value {ID:number} of property {PropertyName:string} is an invalid {ObjectName:string} ID.
       */
      InvalidID = 'DiscountsV2:InvalidID',
      /**
       * Product {0:number} is not found
       */
      ProductNotFound = 'DiscountsV2:ProductNotFound',
      /**
       * Product {0:number} has an invalid type for GetAProductAction: {1:ProductTypes}.
       */
      ProductInvalidType = 'DiscountsV2:ProductInvalidType',
      /**
       * The provided stocklabel {0:number} could not be found.
       */
      InvalidStockLabel = 'DiscountsV2:InvalidStockLabel',
      /**
       * The orderlinetype should be ShippingCosts, ReturnCosts or GiftWrappingCosts.
       */
      InvalidLineType = 'DiscountsV2:InvalidLineType',
      /**
       * The coupon handler {0:string} is unknown.
       */
      UnknownCouponHandler = 'DiscountsV2:UnknownCouponHandler',
      /**
       * The discount to be applied for the generated coupon is invalid.
       */
      InvalidPostponedDiscount = 'DiscountsV2:InvalidPostponedDiscount',
      /**
       * The coupon distribution handler is missing.
       */
      CouponDistributionHandlerMissing = 'DiscountsV2:CouponDistributionHandlerMissing',
      /**
       * The coupon distribution handler '{0:string}' is invalid.
       */
      InvalidCouponDistributionHandler = 'DiscountsV2:InvalidCouponDistributionHandler',
      /**
       * The given RestitutionOrganizationUnit is invalid. OrganizationUnit needs to exist and typed as DiscountRestitutionOrganizationUnit.
       */
      InvalidRestitutionOrganizationUnit = 'DiscountsV2:InvalidRestitutionOrganizationUnit',
      /**
       * Action does not allow excluding product filters on discountable product sets
       */
      ActionDoesNotAllowExcludingProductFiltersOnDiscountableProductSets = 'DiscountsV2:ActionDoesNotAllowExcludingProductFiltersOnDiscountableProductSets',
      /**
       * StringFilter {0:string} must have a value to compare.
       */
      StringFilterMustHaveAValue = 'DiscountsV2:StringFilterMustHaveAValue',
      /**
       * Invalid user usage reload strategy `{0:string}`.
       */
      InvalidUserUsageReloadStrategy = 'DiscountsV2:InvalidUserUsageReloadStrategy',
      /**
       * A discount of trigger type `{0:string}` cannot be combined with (custom)UserUsageLimit.
       */
      TriggertypeCannotBeCombinedWithUserUsageLimit = 'DiscountsV2:TriggertypeCannotBeCombinedWithUserUsageLimit',
      /**
       * A discount of trigger type `{0:string}` cannot have other conditions.
       */
      TriggertypeDoesNotAllowForConditions = 'DiscountsV2:TriggertypeDoesNotAllowForConditions',
      /**
       * Cannot apply discount to orderline {0:number}.
       */
      InvalidOrderLineForDiscount = 'DiscountsV2:InvalidOrderLineForDiscount',
      /**
       * OrderLine {0:number} does not support discount product pick selection.
       */
      InvalidOrderLineForProductPicking = 'DiscountsV2:InvalidOrderLineForProductPicking',
      /**
       * Invalid product for product picking.
       */
      InvalidProductForProductPicking = 'DiscountsV2:InvalidProductForProductPicking',
      /**
       * Loyalty program is required when discount trigger is 'Loyalty'
       */
      LoyaltyProgramRequired = 'DiscountsV2:LoyaltyProgramRequired',
      /**
       * The given loyalty program is unknown
       */
      UnknowLoyaltyProgram = 'DiscountsV2:UnknowLoyaltyProgram',
      /**
       * Only EVA loyalty handler can be used
       */
      OnlyEVALoyaltyHandlerCanBeUsed = 'DiscountsV2:OnlyEVALoyaltyHandlerCanBeUsed',
      /**
       * Either `Data` or `BlobID` is required for upload.
       */
      EitherDataOrBlobIDIsRequired = 'DiscountsV2:EitherDataOrBlobIDIsRequired',
      /**
       * Non async processing only accepts `Data`.
       */
      NonAsyncProcessingOnlyAcceptsData = 'DiscountsV2:NonAsyncProcessingOnlyAcceptsData',
      /**
       * Async processing only accepts `BlobID`.
       */
      AsyncProcessingOnlyAcceptsBlobID = 'DiscountsV2:AsyncProcessingOnlyAcceptsBlobID',
      /**
       * Invalid custom field type `{0:number}`.
       */
      InvalidCustomFieldType = 'DiscountsV2:InvalidCustomFieldType',
      /**
       * Required custom fields are not supported for discount trigger `{0:DiscountTriggers}`.
       */
      RequiredCustomFieldsAreNotSupportedForDiscountTrigger = 'DiscountsV2:RequiredCustomFieldsAreNotSupportedForDiscountTrigger',
      /**
       * Custom field type `{0:number}` cannot be required on discount that applies to `{1:DiscountAppliesTo}`.
       */
      DiscountApplyToMismatchOnCustomFieldType = 'DiscountsV2:DiscountApplyToMismatchOnCustomFieldType'
    }

    export const enum PurchaseOrders {
      /**
       * Expecting UnitPrice to be greater than 0
       */
      UnitPriceNeedsToBeGreaterThenZero = 'PurchaseOrders:UnitPriceNeedsToBeGreaterThenZero',
      /**
       * Pruchase order already has a line with the same ProductID and StockLabel.
       */
      PurchaseOrderAlreadyHasALineWithTheSameProductAndStockLabelID = 'PurchaseOrders:PurchaseOrderAlreadyHasALineWithTheSameProductAndStockLabelID'
    }

    export const enum CustomFieldFilterErrors {
      /**
       * No fields given to validate filter
       */
      NoFieldsGivenToValidateFilter = 'CustomFieldFilterErrors:NoFieldsGivenToValidateFilter',
      /**
       * Invalid custom field value given
       */
      InvalidCustomFieldValueGiven = 'CustomFieldFilterErrors:InvalidCustomFieldValueGiven',
      /**
       * Filter validate failed because there are not custom fields of type {0:string}
       */
      CouldNotValidateFilterWhenNoCustomFieldsAreSetToCustomFieldTypeX = 'CustomFieldFilterErrors:CouldNotValidateFilterWhenNoCustomFieldsAreSetToCustomFieldTypeX',
      /**
       * Custom field {0:number} in filter is unknown.
       */
      InvalidCustomFieldInFilter = 'CustomFieldFilterErrors:InvalidCustomFieldInFilter',
      /**
       * Customfield {0:number} is an array valued field which only supports the `Contains` and `DoesNotContain` operators
       */
      InvalidArrayValueOperator = 'CustomFieldFilterErrors:InvalidArrayValueOperator',
      /**
       * Customfield {CustomFieldId:number} of type {DataType:string} does not support the operator {Operator:string}
       */
      InvalidValueOperator = 'CustomFieldFilterErrors:InvalidValueOperator'
    }

    export const enum Cultures {
      /**
       * This country already exists for this OrganizationUnit
       */
      CountryAlreadyExists = 'Cultures:CountryAlreadyExists'
    }

    export const enum Stock {
      /**
       * Not enough stock available for product {0:number}.
       */
      InsufficientStock = 'Stock:InsufficientStock',
      /**
       * Not enough stock available for product {0:number}.
       */
      InsufficientStockForCommit = 'Stock:InsufficientStockForCommit'
    }

    export const enum StockMutationReason {
      /**
       * Duplicate name or backendID.
       */
      DuplicateReason = 'StockMutationReason:DuplicateReason',
      /**
       * Cannot delete internal stock mutation reason.
       */
      InternalReason = 'StockMutationReason:InternalReason'
    }

    export const enum Products {
      /**
       * Product {duplicateProductID:number} already uses BackendID {backendID:string} and BackendID must be unique per catalog.
       */
      DuplicateBackendID = 'Products:DuplicateBackendID',
      /**
       * Cannot copy product of this type.
       */
      CannotCopyProductOfThisType = 'Products:CannotCopyProductOfThisType',
      /**
       * Can't identify product. Serial number or Orderline ID is missing or Orderline does not contain serial number.
       */
      CannotIdentifyProduct = 'Products:CannotIdentifyProduct'
    }

    export const enum TaxRates {
      /**
       * There is no TaxRate for TaxCode {taxCodeName:string} in country {countryID:string} / {countrySubdivisionID:string}.
       */
      NotFoundForTaxCode = 'TaxRates:NotFoundForTaxCode',
      /**
       * There is no TaxRate for Product {productID:number} in country {countryID:string} / {countrySubdivisionID:string}.
       */
      NotFoundForProduct = 'TaxRates:NotFoundForProduct',
      /**
       * There is no TaxRate for Rate {taxRate:number} in country {countryID:string} / {countrySubdivisionID:string}.
       */
      NotFoundForTaxRate = 'TaxRates:NotFoundForTaxRate',
      /**
       * Cannot create a tax rate for tax code {0:string} without specifying a TaxRateCategory.
       */
      CategoryIsRequired = 'TaxRates:CategoryIsRequired'
    }

    export const enum ReturnOrders {
      /**
       * Not allowed to add lines of an extax order to an intax order and vice versa.
       */
      CannotCreateReturnLinesWithDifferentTaxHandling = 'ReturnOrders:CannotCreateReturnLinesWithDifferentTaxHandling',
      /**
       * Not allowed to return invalid order line {ID:number} ({reason:string})
       */
      CannotCreateReturnOrderWithInvalidLines = 'ReturnOrders:CannotCreateReturnOrderWithInvalidLines',
      /**
       * Can't return orderline {0:number} because the original orderline can not be found.
       */
      CannotReturnLineWithInvalidOriginalLine = 'ReturnOrders:CannotReturnLineWithInvalidOriginalLine',
      /**
       * Can't return orderline {returnOrderLineID:number} because the original orderline ({originalOrderLineID:number}) is not returnable.
       */
      CannotReturnNonReturnableLine = 'ReturnOrders:CannotReturnNonReturnableLine',
      /**
       * Can't send invoices when the original order has no customer or when no customer is provided for unreferenced returns.
       */
      CannotSendInvoiceWithoutCustomer = 'ReturnOrders:CannotSendInvoiceWithoutCustomer',
      /**
       * Unreferenced return reversals are not allowed.
       */
      CannotCreateUnreferencedReturnReversals = 'ReturnOrders:CannotCreateUnreferencedReturnReversals',
      /**
       * The return reason {0:number} requires a remark.
       */
      RemarkRequired = 'ReturnOrders:RemarkRequired'
    }

    export const enum ReturnReason {
      /**
       * Return reason {0:number} requires a remark.
       */
      RemarkRequired = 'ReturnReason:RemarkRequired',
      /**
       * Return reason {0:number} doesn't exist.
       */
      ReasonNotFound = 'ReturnReason:ReasonNotFound',
      /**
       * Return reason {0:number} doesn't exist.
       */
      ReasonTypeMismatch = 'ReturnReason:ReasonTypeMismatch'
    }

    export const enum Countries {
      /**
       * Subdivision doesn't match the selected country.
       */
      InvalidSubdivision = 'Countries:InvalidSubdivision'
    }

    export const enum Companies {
      /**
       * VAT Number is already in use.
       */
      VatNumberAlreadyInUse = 'Companies:VatNumberAlreadyInUse',
      /**
       * Backend ID is already in use.
       */
      BackendIDAlreadyInUse = 'Companies:BackendIDAlreadyInUse',
      /**
       * Company is still attached to some organization unit(s). Please detach those first before proceeding.
       */
      CannotDeleteCompanyWithRelatedOrganizationUnits = 'Companies:CannotDeleteCompanyWithRelatedOrganizationUnits',
      /**
       * Company {0:number} does not have the required type {1:CompanyType}.
       */
      CompanyHasInvalidType = 'Companies:CompanyHasInvalidType'
    }

    export const enum Company {
      /**
       * It is not allowed to modify the registration number with closed transactions
       */
      ModifyRegistrationNumberNotAllowed = 'Company:ModifyRegistrationNumberNotAllowed',
      /**
       * It is not allowed to modify the vat number with closed transactions
       */
      ModifyVatNumberNotAllowed = 'Company:ModifyVatNumberNotAllowed',
      /**
       * Push companies limit exceeded.
       */
      PushCompaniesLimitExceeded = 'Company:PushCompaniesLimitExceeded',
      /**
       * Unknown error while pushing company with BackendID `{0:string}`
       */
      UnknownPushCompanyError = 'Company:UnknownPushCompanyError',
      /**
       * Invalid user for consumer company.
       */
      InvalidUserForConsumerCompany = 'Company:InvalidUserForConsumerCompany'
    }

    export const enum Offers {
      /**
       * Offers cannot be shipped.
       */
      ShipmentsNotAllowed = 'Offers:ShipmentsNotAllowed',
      /**
       * Offers cannot have payments.
       */
      PaymentsNotAllowed = 'Offers:PaymentsNotAllowed',
      /**
       * Offers cannot be invoiced, yet.
       */
      InvoicesNotSupportedYet = 'Offers:InvoicesNotSupportedYet',
      /**
       * This offer requires a customer based on configured settings.
       */
      OfferRequiresCustomer = 'Offers:OfferRequiresCustomer',
      /**
       * Expired Offers cannot be mutated.
       */
      Expired = 'Offers:Expired',
      /**
       * Order {0:number} cannot be changed, changing an offer is not allowed.
       */
      Unchangeble = 'Offers:Unchangeble'
    }

    export const enum OpeningHoursTemplateExcelUpload {
      /**
       * [row {0:number}, column {1:string}]: Status must be either 'Open' or 'Closed'
       */
      NoValidStatusProvided = 'OpeningHoursTemplateExcelUpload:NoValidStatusProvided',
      /**
       * [row {0:number}, column {1:string}]: Date must be after today
       */
      InvalidDateProvided = 'OpeningHoursTemplateExcelUpload:InvalidDateProvided',
      /**
       * [row {0:number}, column {1:string}]: From and To must be set if status is 'Open'
       */
      NoOpeningHoursProvided = 'OpeningHoursTemplateExcelUpload:NoOpeningHoursProvided',
      /**
       * [row {0:number}, column {1:string}]: From and To must not be set if status is 'Closed'
       */
      NoOpeningHoursAllowedIfStatusIsClosed = 'OpeningHoursTemplateExcelUpload:NoOpeningHoursAllowedIfStatusIsClosed',
      /**
       * [row {0:number}, column {1:string}]: The hour of opening must be before the hour of closing
       */
      InvalidOpeningHoursProvided = 'OpeningHoursTemplateExcelUpload:InvalidOpeningHoursProvided',
      /**
       * [row {0:number}, column {1:string}]: Multiple time slots on the same day cannot overlap each other
       */
      OverlappingTimeSlots = 'OpeningHoursTemplateExcelUpload:OverlappingTimeSlots',
      /**
       * Multiple rows have been provided with Date '{0:string}'. Please only provide one row per date
       */
      MultipleRowsWithSameDate = 'OpeningHoursTemplateExcelUpload:MultipleRowsWithSameDate'
    }

    export const enum Assortments {
      /**
       * It's not possible to delete the default Assortment.
       */
      CannotDeleteDefault = 'Assortments:CannotDeleteDefault'
    }

    export const enum AssortmentProducts {
      /**
       * When adding products to an assortment, either Products or Query needs to have a value.
       */
      NoProductsSpecified = 'AssortmentProducts:NoProductsSpecified',
      /**
       * It's not possible to remove products from the default assortment.
       */
      CannotRemoveProductsFromDefaultAssortment = 'AssortmentProducts:CannotRemoveProductsFromDefaultAssortment',
      /**
       * Invalid AssortmentProduct date sequence: {0:string} (value: {1:string}) cannot be after {2:string} (value: {3:string}).
       */
      PreorderDatesOutOfOrder = 'AssortmentProducts:PreorderDatesOutOfOrder'
    }

    export const enum ProductRelation {
      /**
       * ProductRelationType of type `Group` cannot have a StockRelationType.
       */
      ProductRelationTypeOfTypeGroupCannotHaveAStockRelationType = 'ProductRelation:ProductRelationTypeOfTypeGroupCannotHaveAStockRelationType',
      /**
       * Only ProductRelationType `Group` or types with the DynamicRelationProductID set accept a ProductSearchTemplateID or ProductSearchFilters.
       */
      DynamicRelationTypeOnlyAllowedForGroupOrWhenDynamicRelationProductIDIsSet = 'ProductRelation:DynamicRelationTypeOnlyAllowedForGroupOrWhenDynamicRelationProductIDIsSet',
      /**
       * Cannot add static relation to a dynamic relationtype
       */
      CannotCreateStaticRelationOnADynamicRelationType = 'ProductRelation:CannotCreateStaticRelationOnADynamicRelationType',
      /**
       * Product cannot have a relation with itself
       */
      ProductCannotHaveARelationWithItself = 'ProductRelation:ProductCannotHaveARelationWithItself',
      /**
       * Uni- or bidirectional relations require a primary and related product ID
       */
      UniOrBidirectionalRelationsRequireAPrimaryAndRelatedProductID = 'ProductRelation:UniOrBidirectionalRelationsRequireAPrimaryAndRelatedProductID',
      /**
       * A unidirectional relation already exists from product {relatedProductID:number} to product {primaryProductID:number}. Can not register a unidirectional relation from {primaryProductID:number} to {relatedProductID:number} on the same relation type.
       */
      UnidirectionalRelationAlreadyExistsFromRelatedProduct = 'ProductRelation:UnidirectionalRelationAlreadyExistsFromRelatedProduct',
      /**
       * Product relation not found with primary product '{primaryProductID:number}' and related product '{relatedProductID:number}'.
       */
      ProductRelationNotFound = 'ProductRelation:ProductRelationNotFound',
      /**
       * There are no product relations for type the given product relation type '{0:number}'.
       */
      NoProductRelationsForGivenType = 'ProductRelation:NoProductRelationsForGivenType',
      /**
       * Filters are required for a dynamic product relation type with uni or bi directional
       */
      DynamicUniOrBiDirectionalRelationTypesRequireAProductFilter = 'ProductRelation:DynamicUniOrBiDirectionalRelationTypesRequireAProductFilter',
      /**
       * DynamicRelationProductID required for a dynamic product relation type with uni or bi directional
       */
      DynamicUniOrBiDirectionalRelationTypesRequireADynamicRelationProductID = 'ProductRelation:DynamicUniOrBiDirectionalRelationTypesRequireADynamicRelationProductID',
      /**
       * Cannot convert product relation from static to dynamic
       */
      CannotConvertProductRelationFromStaticToDynamic = 'ProductRelation:CannotConvertProductRelationFromStaticToDynamic',
      /**
       * Cannot change second chance relation type.
       */
      CannotChangeSecondChanceRelationType = 'ProductRelation:CannotChangeSecondChanceRelationType'
    }

    export const enum Inquiries {
      /**
       * The type for custom field {customFieldId:number} should by User, got {type:string}
       */
      InvalidType = 'Inquiries:InvalidType',
      /**
       * Items should point to unique custom fields.
       */
      CustomFieldsShouldBeUnique = 'Inquiries:CustomFieldsShouldBeUnique'
    }

    export const enum Inquiry {
      /**
       * Inquire can only be filled in once.
       */
      InquireCanOnlyBeFilledInOnce = 'Inquiry:InquireCanOnlyBeFilledInOnce',
      /**
       * You cannot submit duplicate inquire items.
       */
      DuplicateInquireItemSubmitted = 'Inquiry:DuplicateInquireItemSubmitted',
      /**
       * Missing the following required inquiry item(s): {0:string}.
       */
      MissingRequiredInquiryItems = 'Inquiry:MissingRequiredInquiryItems'
    }

    export const enum CaseStatus {
      /**
       * Name '{0:string}' already exists.
       */
      StatusNameAlreadyExists = 'CaseStatus:StatusNameAlreadyExists',
      /**
       * Script is not valid.
       */
      ScriptIsNotAValid = 'CaseStatus:ScriptIsNotAValid',
      /**
       * The name of internal case status '{0:string}' cannot be modified.
       */
      NameOfInternalStatusCannotBeModified = 'CaseStatus:NameOfInternalStatusCannotBeModified',
      /**
       * Internal case status '{0:string}' cannot be deleted.
       */
      InternalStatusCannotBeDeleted = 'CaseStatus:InternalStatusCannotBeDeleted'
    }

    export const enum CaseTopic {
      /**
       * Name '{0:string}' already exists.
       */
      TopicNameAlreadyExists = 'CaseTopic:TopicNameAlreadyExists',
      /**
       * Can not filter on both RootOnly and ParentID, please provide only one.
       */
      CanNotFilterOnBothRootOnlyAndParent = 'CaseTopic:CanNotFilterOnBothRootOnlyAndParent'
    }

    export const enum CaseInteraction {
      /**
       * Can only update own interactions.
       */
      CanOnlyUpdateOwnInteraction = 'CaseInteraction:CanOnlyUpdateOwnInteraction',
      /**
       * Can only delete own interactions.
       */
      CanOnlyDeleteOwnInteraction = 'CaseInteraction:CanOnlyDeleteOwnInteraction'
    }

    export const enum Case {
      /**
       * Case already closed.
       */
      CaseAlreadyClosed = 'Case:CaseAlreadyClosed',
      /**
       * Can only archive closed cases.
       */
      CanOnlyArchiveClosedCases = 'Case:CanOnlyArchiveClosedCases',
      /**
       * Cannot update title and description of customer created case.
       */
      CannotUpdateTitleAndDescriptionOfCustomerCreatedCase = 'Case:CannotUpdateTitleAndDescriptionOfCustomerCreatedCase',
      /**
       * Cannot update title or description after assignment.
       */
      CannotUpdateTitleOrDescriptionAfterAssignment = 'Case:CannotUpdateTitleOrDescriptionAfterAssignment',
      /**
       * Can only update case title and description of own cases.
       */
      CanOnlyUpdateCaseTitleAndDescriptionOfOwnCases = 'Case:CanOnlyUpdateCaseTitleAndDescriptionOfOwnCases',
      /**
       * Assignee is invalid user type to assign to case.
       */
      AssigneeIsInvalidUserTypeToAssignToCase = 'Case:AssigneeIsInvalidUserTypeToAssignToCase',
      /**
       * Related orderline does not match related order.
       */
      RelatedOrderLineDoesNotMatchRelatedOrder = 'Case:RelatedOrderLineDoesNotMatchRelatedOrder',
      /**
       * Can only set warranty on products.
       */
      CanOnlySetWarrantyOnProducts = 'Case:CanOnlySetWarrantyOnProducts',
      /**
       * Cannot revert case status to `new` status.
       */
      CannotRevertCaseStatusToNewStatus = 'Case:CannotRevertCaseStatusToNewStatus',
      /**
       * Either an OrderLineID or ProductID is required for case related items.
       */
      EitherOrderLineIDOrProductIDIsRequired = 'Case:EitherOrderLineIDOrProductIDIsRequired',
      /**
       * Cannot update originating organization unit after it is set.
       */
      CannotUpdateOriginatingOrganizationUnit = 'Case:CannotUpdateOriginatingOrganizationUnit',
      /**
       * The case with BackendID `{0:string}` already exists.
       */
      CaseWithBackendIDAlreadyExists = 'Case:CaseWithBackendIDAlreadyExists',
      /**
       * CaseTopics are required for Cases.
       */
      CaseTopicIsRequiredForCase = 'Case:CaseTopicIsRequiredForCase'
    }

    export const enum PushCase {
      /**
       * Pushed topic '{0:string}' is unknown.
       */
      UnknownPushedTopic = 'PushCase:UnknownPushedTopic',
      /**
       * No CaseTopic was found for the given BackendID, but no Name was provided either. Please provide a Name to be able to find the CaseTopic by the Name and Parent.
       */
      NameIsRequiredIfBackendIDIsNotFound = 'PushCase:NameIsRequiredIfBackendIDIsNotFound',
      /**
       * There are more than 1 CaseTopics with name `{0:string}`, please use the new field `CaseTopic` and specify the BackendID.
       */
      MultipleOptionsForCaseTopicName = 'PushCase:MultipleOptionsForCaseTopicName',
      /**
       * Pushed status '{0:string}' is unknown.
       */
      UnknownPushedStatus = 'PushCase:UnknownPushedStatus',
      /**
       * Invalid product backend ID for case related item.
       */
      InvalidProductBackendID = 'PushCase:InvalidProductBackendID'
    }

    export const enum Script {
      /**
       * Script {0:number} is unknown.
       */
      UnknownScript = 'Script:UnknownScript',
      /**
       * Script type `{givenType:string}` is invalid. Expected script of type `{expectedType:string}`.
       */
      InvalidScriptType = 'Script:InvalidScriptType',
      /**
       * Dialect {0:string} is not a recognized dialect
       */
      UnknownDialect = 'Script:UnknownDialect'
    }

    export const enum UserRequirement {
      /**
       * Can only set a script when the requirement is for a single action.
       */
      CanOnlySetScriptForSingleRequiredFor = 'UserRequirement:CanOnlySetScriptForSingleRequiredFor'
    }

    export const enum LoyaltyProgramPayment {
      /**
       * Cannot use given payment type: {0:string}
       */
      CannotUseGivenPaymentType = 'LoyaltyProgramPayment:CannotUseGivenPaymentType',
      /**
       * Can only pay for fixed amount
       */
      CanOnlyPayForFixedAmount = 'LoyaltyProgramPayment:CanOnlyPayForFixedAmount',
      /**
       * Amount exceeds maximum allowed amount
       */
      AmountExceedsMaximumAllowedAmount = 'LoyaltyProgramPayment:AmountExceedsMaximumAllowedAmount',
      /**
       * Can only refund full amount
       */
      CannotRefundMoreThenOriginalAmount = 'LoyaltyProgramPayment:CannotRefundMoreThenOriginalAmount',
      /**
       * Payment requires preceding payment to be executed
       */
      PaymentRequiresPrecedingPaymentToBeExecuted = 'LoyaltyProgramPayment:PaymentRequiresPrecedingPaymentToBeExecuted'
    }

    export const enum ShippingRestriction {
      /**
       * Invalid ProductPropertyType, only types with a Boolean DataType are allowed.
       */
      InvalidProductPropertyType = 'ShippingRestriction:InvalidProductPropertyType',
      /**
       * Invalid script type.
       */
      InvalidScriptType = 'ShippingRestriction:InvalidScriptType',
      /**
       * OrganizationUnitID or OrganizationUnitSetID is required unless a leading script is provided.
       */
      OrganizationUnitOrSetRequired = 'ShippingRestriction:OrganizationUnitOrSetRequired',
      /**
       * CountryID is required unless a leading script is provided.
       */
      CountryRequired = 'ShippingRestriction:CountryRequired'
    }

    export const enum PersonalizedPromotion {
      /**
       * Value of property `{0:string}` must be higher than 0.
       */
      ValueMustBeGreaterThanZero = 'PersonalizedPromotion:ValueMustBeGreaterThanZero',
      /**
       * Value of property `{0:string}` must be unique.
       */
      ValueMustBeUnique = 'PersonalizedPromotion:ValueMustBeUnique',
      /**
       * No products are provided.
       */
      NoProductsGiven = 'PersonalizedPromotion:NoProductsGiven',
      /**
       * No available products.
       */
      NoAvailableProducts = 'PersonalizedPromotion:NoAvailableProducts',
      /**
       * Product '{0}' not available.
       */
      ProductNotAvailable = 'PersonalizedPromotion:ProductNotAvailable',
      /**
       * Maximum distinct products exceeded.
       */
      MaximumDistinctProductsExceeded = 'PersonalizedPromotion:MaximumDistinctProductsExceeded',
      /**
       * Personalized promotions is already applied.
       */
      AlreadyApplied = 'PersonalizedPromotion:AlreadyApplied'
    }

    export const enum ProductSet {
      /**
       * Product set {ProductSetName:string} requires at least one of the following values: {PropertyName:string}
       */
      ProductSetRequiresAtLeastOneOfTheFollowingValues = 'ProductSet:ProductSetRequiresAtLeastOneOfTheFollowingValues',
      /**
       * The value of product set property {PropertyName:string} in product set {ProductSetName:string} must be higher then 0.
       */
      ProductSetValueCannotBeNegativeOrZero = 'ProductSet:ProductSetValueCannotBeNegativeOrZero',
      /**
       * The value of product set property {PropertyName:string} in product set {ProductSetName:string} must be higher then or equals to 0.
       */
      ProductSetValueCannotBeNegative = 'ProductSet:ProductSetValueCannotBeNegative',
      /**
       * The value of product set property {PropertyNameX:string} in product set {ProductSetName:string} must be higher then or equals to property {PropertyNameY:string}.
       */
      ProductSetValueOfXMustBeLargerThenOrEqualToValueOfY = 'ProductSet:ProductSetValueOfXMustBeLargerThenOrEqualToValueOfY',
      /**
       * Product set {0:string} did not resolve to a valid list of products.
       */
      ProductSetDidNotResolveToAListOfValidProducts = 'ProductSet:ProductSetDidNotResolveToAListOfValidProducts',
      /**
       * A productset price filter requires a valid from/to value.
       */
      ProductSetPriceFilterRequiresAValidFromOrToValue = 'ProductSet:ProductSetPriceFilterRequiresAValidFromOrToValue'
    }

    export const enum AddressRequirement {
      /**
       * Property does not support predefined values.
       */
      PropertyDoesNotSupportPredefinedValues = 'AddressRequirement:PropertyDoesNotSupportPredefinedValues',
      /**
       * Property '{0:PropertyType}' cannot exceed a maximum length higher than '{1:number}'.
       */
      PropertyMaxLengthExceeded = 'AddressRequirement:PropertyMaxLengthExceeded',
      /**
       * Predefined value for property '{0:PropertyType}' cannot exceed the maximum length allowed.
       */
      PropertyPredefinedValueExceedsMaxLength = 'AddressRequirement:PropertyPredefinedValueExceedsMaxLength'
    }

    export const enum Address {
      /**
       * Invalid country subdivision.
       */
      InvalidCountrySubdivision = 'Address:InvalidCountrySubdivision'
    }

    export const enum CashHandlers {
      /**
       * MaxAmountPerOrder {0:number} needs to be rounded based on RoundingFactor. Try again using {1:number} or another rounded value.
       */
      IncorrectMaxAmountPerOrder = 'CashHandlers:IncorrectMaxAmountPerOrder'
    }

    export const enum EntityTranslations {
      /**
       * Cannot copy translations from an entity to itself.
       */
      CannotCopyToItself = 'EntityTranslations:CannotCopyToItself',
      /**
       * Entity `{0:string}` does not have any translations.
       */
      EntityDoesNotHaveTranslations = 'EntityTranslations:EntityDoesNotHaveTranslations'
    }

    export const enum OrderVerificationReason {
      /**
       * Name '{0:string}' already exists.
       */
      NameAlreadyExists = 'OrderVerificationReason:NameAlreadyExists',
      /**
       * Internal order verification reason '{0:string}' cannot be deleted.
       */
      InternalOrderVerificationReasonCannotBeDeleted = 'OrderVerificationReason:InternalOrderVerificationReasonCannotBeDeleted',
      /**
       * The name of internal order verification reason '{0:string}' cannot be modified.
       */
      NameOfInternalOrderVerificationReasonCannotBeModified = 'OrderVerificationReason:NameOfInternalOrderVerificationReasonCannotBeModified'
    }

    export const enum Surveys {
      /**
       * This is a system survey, it can't be managed.
       */
      SystemSurvey = 'Surveys:SystemSurvey',
      /**
       * Another survey with this BackendID already exists.
       */
      DuplicateBackendID = 'Surveys:DuplicateBackendID',
      /**
       * The trigger type `{0:string}` is unknown.
       */
      UnknownTriggerType = 'Surveys:UnknownTriggerType',
      /**
       * The distribution handler `{0:string}` is unknown.
       */
      UnknownDistributionHandler = 'Surveys:UnknownDistributionHandler',
      /**
       * The after effect type `{0:string}` is unknown.
       */
      UnknownAfterEffectType = 'Surveys:UnknownAfterEffectType',
      /**
       * Survey '{SurveyID:number}' is missing data for trigger '{TriggerType:string}'.
       */
      MissingTriggerData = 'Surveys:MissingTriggerData',
      /**
       * Survey '{SurveyID:number}' has invalid data for trigger '{TriggerType:string}'.
       */
      InvalidTriggerData = 'Surveys:InvalidTriggerData',
      /**
       * The date of property '{0:string}' is invalid.
       */
      InvalidDate = 'Surveys:InvalidDate',
      /**
       * Email addresses '{0:string}' are invalid.
       */
      InvalidEmailAddresses = 'Surveys:InvalidEmailAddresses',
      /**
       * Value of property '{PropertyNameX:string}' must be larger then property '{PropertyNameY:string}'
       */
      ValueOfXMustBeLargerThenValueOfY = 'Surveys:ValueOfXMustBeLargerThenValueOfY',
      /**
       * Cannot publish survey without questions.
       */
      CannotPublishSurveyWithoutQuestions = 'Surveys:CannotPublishSurveyWithoutQuestions',
      /**
       * All questions have routings pointing to them. One question needs to have no routings pointing to it to act as the first question.
       */
      AllQuestionsRoutedTo = 'Surveys:AllQuestionsRoutedTo',
      /**
       * Multiple questions without routings pointing to them. Only one question needs to have no routings pointing to it to act as the first question.
       */
      MultipleQuestionsNotRoutedTo = 'Surveys:MultipleQuestionsNotRoutedTo',
      /**
       * Unknown error while verifying survey with ID `{0:number}`
       */
      UnknownVerifySurveyError = 'Surveys:UnknownVerifySurveyError',
      /**
       * Question recursion detected; {0:string}.
       */
      PossibleLoopDetectedForQuestion = 'Surveys:PossibleLoopDetectedForQuestion',
      /**
       * Routed to non existing question within survey `{0:number}`.
       */
      RoutedToNonExistingQuestionWithinSurvey = 'Surveys:RoutedToNonExistingQuestionWithinSurvey',
      /**
       * Possible loop detected.
       */
      PossibleLoopDetected = 'Surveys:PossibleLoopDetected',
      /**
       * Cannot update taken survey.
       */
      CannotUpdateTakenSurvey = 'Surveys:CannotUpdateTakenSurvey',
      /**
       * Cannot update {0:string} for published survey.
       */
      CannotUpdatePropertyForPublishedSurvey = 'Surveys:CannotUpdatePropertyForPublishedSurvey',
      /**
       * Survey is being published
       */
      SurveyIsBeingPublished = 'Surveys:SurveyIsBeingPublished',
      /**
       * Survey doesn't have pending changes
       */
      SurveyDoesntHavePendingChanges = 'Surveys:SurveyDoesntHavePendingChanges',
      /**
       * Cannot start survey because it has not been published.
       */
      SurveyIsNotPublished = 'Surveys:SurveyIsNotPublished',
      /**
       * Question not part of survey.
       */
      QuestionNotPartOfSurvey = 'Surveys:QuestionNotPartOfSurvey',
      /**
       * An answer is required.
       */
      QuestionAnswerIsRequired = 'Surveys:QuestionAnswerIsRequired',
      /**
       * Survey is not started.
       */
      SurveyIsNotStarted = 'Surveys:SurveyIsNotStarted',
      /**
       * Survey requires context.
       */
      SurveyRequiresContext = 'Surveys:SurveyRequiresContext',
      /**
       * Automatic survey cannot be started manually.
       */
      AutomaticSurveyCannotBeStartedManually = 'Surveys:AutomaticSurveyCannotBeStartedManually',
      /**
       * Either Token or a SurveyID is required
       */
      EitherTokenOrSurveyIDRequired = 'Surveys:EitherTokenOrSurveyIDRequired',
      /**
       * Unknown survey response token
       */
      UnknownSurveyResponseToken = 'Surveys:UnknownSurveyResponseToken',
      /**
       * Answer is invalid
       */
      AnswerInvalid = 'Surveys:AnswerInvalid',
      /**
       * Survey response cancelled
       */
      SurveyResponseCancelled = 'Surveys:SurveyResponseCancelled',
      /**
       * Survey response completed
       */
      SurveyResponseCompleted = 'Surveys:SurveyResponseCompleted',
      /**
       * Survey expired
       */
      SurveyExpired = 'Surveys:SurveyExpired',
      /**
       * Survey not available.
       */
      SurveyNotAvailable = 'Surveys:SurveyNotAvailable',
      /**
       * Question not answered yet
       */
      QuestionNotAnsweredYet = 'Surveys:QuestionNotAnsweredYet',
      /**
       * The value of property '{PropertyName:string}' is missing for after effect type '{AfterEffectType:string}'
       */
      AfterEffectDataMissing = 'Surveys:AfterEffectDataMissing',
      /**
       * Invalid after effect type '{0:string}'
       */
      InvalidAfterEffect = 'Surveys:InvalidAfterEffect',
      /**
       * After efect '{0:string}' isn't available for given survey context.
       */
      AfterEffectNotAvailableForContext = 'Surveys:AfterEffectNotAvailableForContext',
      /**
       * Cannot distribute survey to customer
       */
      CannotDistributeSurveyToCustomer = 'Surveys:CannotDistributeSurveyToCustomer'
    }

    export const enum SurveyQuestions {
      /**
       * Another survey question with this BackendID already exists.
       */
      DuplicateBackendID = 'SurveyQuestions:DuplicateBackendID',
      /**
       * Either ContextCustomField or question data is Required.
       */
      EitherContextCustomFieldOrQuestionDataIsRequired = 'SurveyQuestions:EitherContextCustomFieldOrQuestionDataIsRequired',
      /**
       * ContextCustomField and question data are mutually exclusive.
       */
      ContextCustomFieldAndQuestionDataAreMutuallyExclusive = 'SurveyQuestions:ContextCustomFieldAndQuestionDataAreMutuallyExclusive',
      /**
       * Custom field type is required to be of type 'User'.
       */
      InvalidCustomFieldType = 'SurveyQuestions:InvalidCustomFieldType',
      /**
       * Data type 'Enum' requires optional answers.
       */
      DataTypeEnumRequiresOptionalAnswers = 'SurveyQuestions:DataTypeEnumRequiresOptionalAnswers',
      /**
       * Optional answers require a non empty key and value.
       */
      EnumValuesRequireAKeyAndValue = 'SurveyQuestions:EnumValuesRequireAKeyAndValue',
      /**
       * Optional answer keys must be unique.
       */
      OptionalAnswerKeysMustBeUnique = 'SurveyQuestions:OptionalAnswerKeysMustBeUnique',
      /**
       * Optional answers are only allowed for datatype `Enum`.
       */
      OptionalAnswersAreOnlyAllowedForDataTypeEnum = 'SurveyQuestions:OptionalAnswersAreOnlyAllowedForDataTypeEnum',
      /**
       * Saving on customer requires a custom field.
       */
      SaveOnCustomerRequiresCustomField = 'SurveyQuestions:SaveOnCustomerRequiresCustomField',
      /**
       * The question cannot be added to a set belonging to a different survey.
       */
      QuestionSetBelongsToDifferentSurvey = 'SurveyQuestions:QuestionSetBelongsToDifferentSurvey',
      /**
       * All question routings are required for reordering.
       */
      AllQuestionRoutingsAreRequiredForReordering = 'SurveyQuestions:AllQuestionRoutingsAreRequiredForReordering',
      /**
       * Cannot switch question to other custom field after publish.
       */
      CannotSwitchQuestionToOtherCustomFieldAfterPublish = 'SurveyQuestions:CannotSwitchQuestionToOtherCustomFieldAfterPublish',
      /**
       * Question cannot be detached from custom field after creation.
       */
      QuestionCannotBeDetachedFromCustomFieldAfterCreation = 'SurveyQuestions:QuestionCannotBeDetachedFromCustomFieldAfterCreation',
      /**
       * Question cannot be attached to custom field after creation.
       */
      QuestionCannotBeAttachedToCustomFieldAfterCreation = 'SurveyQuestions:QuestionCannotBeAttachedToCustomFieldAfterCreation',
      /**
       * Cannot switch DataType after publish.
       */
      CannotSwitchDataTypeAfterPublish = 'SurveyQuestions:CannotSwitchDataTypeAfterPublish',
      /**
       * Cannot create multiple routes without filter.
       */
      CannotCreateMultipleRoutesWithoutFilter = 'SurveyQuestions:CannotCreateMultipleRoutesWithoutFilter',
      /**
       * Route should either have a `RouteToQuestionID` or exit remarks
       */
      RouteShouldEitherHaveARouteToQuestionIDOrExitRemarks = 'SurveyQuestions:RouteShouldEitherHaveARouteToQuestionIDOrExitRemarks'
    }

    export const enum EnvironmentInitialization {
      /**
       * Could not convert initialization state value in Redis to initialization state Enum.
       */
      InvalidEnvironmentInitializationState = 'EnvironmentInitialization:InvalidEnvironmentInitializationState',
      /**
       * Tried to set environment state to created but the environment state is already set.
       */
      EnvironmentStateAlreadySet = 'EnvironmentInitialization:EnvironmentStateAlreadySet',
      /**
       * Tried to set an already Initialized environment to Initialized.
       */
      EnvironmentAlreadyInitialized = 'EnvironmentInitialization:EnvironmentAlreadyInitialized',
      /**
       * The currency passed is not a valid option.
       */
      InvalidCurrencyPassed = 'EnvironmentInitialization:InvalidCurrencyPassed',
      /**
       * The country passed is not a valid option.
       */
      InvalidCountryPassed = 'EnvironmentInitialization:InvalidCountryPassed',
      /**
       * The language passed is not a valid option.
       */
      InvalidLanguagePassed = 'EnvironmentInitialization:InvalidLanguagePassed',
      /**
       * The e-mail passed is not a valid e-mail address.
       */
      InvalidEmailPassed = 'EnvironmentInitialization:InvalidEmailPassed'
    }

    export const enum EndpointConfiguration {
      /**
       * The provided endpoint is either invalid or is not secure
       */
      InvalidEndpoint = 'EndpointConfiguration:InvalidEndpoint',
      /**
       * Cannot change status to Error
       */
      CannotSetToError = 'EndpointConfiguration:CannotSetToError',
      /**
       * The provided config is invalid
       */
      InvalidConfig = 'EndpointConfiguration:InvalidConfig',
      /**
       * The key ID is invalid (should be either 1 or 2)
       */
      InvalidKeyID = 'EndpointConfiguration:InvalidKeyID',
      /**
       * The requested BackendID is already in use
       */
      DuplicateBackendID = 'EndpointConfiguration:DuplicateBackendID'
    }

    export const enum ConditionHandler {
      /**
       * Condition {ConditionName:string} (id {ConditionID:number}) for {ParentName:string} (id {ParentID:number}) was malformed or its build-up resulted in an exception.
       */
      InvalidCondition = 'ConditionHandler:InvalidCondition',
      /**
       * The data for condition {ConditionName:string} (id {ConditionID:number}) on {ParentName:string} (id {ParentID:number}) is not initialized properly.
       */
      ConditionDataNotInitialized = 'ConditionHandler:ConditionDataNotInitialized'
    }

    export const enum CouponOriginatingOrderOrganizationUnitCondition {
      /**
       * Coupon can only be used on originating order organization unit.
       */
      CouponCanOnlyBeUsedOnOriginatingOrderOrganizationUnit = 'CouponOriginatingOrderOrganizationUnitCondition:CouponCanOnlyBeUsedOnOriginatingOrderOrganizationUnit'
    }

    export const enum CouponValidityCondition {
      /**
       * Invalid condition configuration. At least one of the properties must be set.
       */
      InvalidConditionConfiguration = 'CouponValidityCondition:InvalidConditionConfiguration',
      /**
       * Originating order does not this coupon registered.
       */
      OriginatingOrderHasntGotThisCouponRegistered = 'CouponValidityCondition:OriginatingOrderHasntGotThisCouponRegistered',
      /**
       * Coupon can only be used on originating order requested date, but the originating order does not have a requested date registered.
       */
      OriginatingOrderDoesntHaveARequestedDate = 'CouponValidityCondition:OriginatingOrderDoesntHaveARequestedDate',
      /**
       * Coupon is usable from {0:string}
       */
      CouponIsUsableFrom = 'CouponValidityCondition:CouponIsUsableFrom',
      /**
       * Coupon is usable on {0:string}
       */
      CouponIsUsableOn = 'CouponValidityCondition:CouponIsUsableOn',
      /**
       * Coupon was usable till {0:string}
       */
      CouponWasUsableTill = 'CouponValidityCondition:CouponWasUsableTill'
    }

    export const enum OldUserFieldConditionErrors {
      /**
       * No userfield's defined.
       */
      NoUserFieldDefined = 'OldUserFieldConditionErrors:NoUserFieldDefined',
      /**
       * Userfield {0:number} is unknown.
       */
      UnknownUserField = 'OldUserFieldConditionErrors:UnknownUserField',
      /**
       * User field {0:number} ({1:string}) has options and expects an integer value
       */
      UserFieldWithOptionsExpectsAnIntegerValue = 'OldUserFieldConditionErrors:UserFieldWithOptionsExpectsAnIntegerValue',
      /**
       * Option value {2:number} for user field {0:number} ({1:string}) is invalid
       */
      InvalidUserFieldOptionValue = 'OldUserFieldConditionErrors:InvalidUserFieldOptionValue'
    }

    export namespace Document {
      export namespace Printer {
        export const enum Factory {
          /**
           * Was unable to determine the correct organization unit for device #{0:number}
           */
          UnknownOrganizationUnit = 'Document:Printer:Factory:UnknownOrganizationUnit',
          /**
           * Do not have a printer implementation for type {0:string}
           */
          UnknownPrinterType = 'Document:Printer:Factory:UnknownPrinterType',
          /**
           * Address of device #{0:string} has not been configured
           */
          NotConfigured = 'Document:Printer:Factory:NotConfigured'
        }

      }

    }

    export const enum ReceiptPrinter {
      /**
       * Not allowed to print this invoice
       */
      NotAllowedToPrint = 'ReceiptPrinter:NotAllowedToPrint',
      /**
       * Not allowed to print more than 1 copy
       */
      NoMoreThanOneDuplicateAllowed = 'ReceiptPrinter:NoMoreThanOneDuplicateAllowed'
    }

    export const enum ReprintThermalPrintResult {
      /**
       * Blob {0:string} cannot be rendered as it is not a thermal print result.
       */
      InvalidBlob = 'ReprintThermalPrintResult:InvalidBlob'
    }

    export const enum ThermalPrinter {
      /**
       * Printing to device failed: {0:string}
       */
      DeviceFailure = 'ThermalPrinter:DeviceFailure',
      /**
       * Template {0:string} was not found.
       */
      TemplateNotFound = 'ThermalPrinter:TemplateNotFound',
      /**
       * Template {0:string} was not found or was disabled.
       */
      TemplateNotFoundOrDisabled = 'ThermalPrinter:TemplateNotFoundOrDisabled',
      /**
       * There is no paper present in device {0:string}.
       */
      NoPaper = 'ThermalPrinter:NoPaper',
      /**
       * Device {0:string} can't currently print as the cover is open.
       */
      CoverOpen = 'ThermalPrinter:CoverOpen',
      /**
       * Device {0:string} is reporting its status as offline.
       */
      Offline = 'ThermalPrinter:Offline'
    }

    export const enum OrderField {
      /**
       * Value {value:string} does not match type {type:CustomFieldDataTypes}
       */
      InvalidDataType = 'OrderField:InvalidDataType'
    }

    export const enum CustomFields {
      /**
       * CustomField {0:number} is unknown for the given type.
       */
      UnknownFieldForType = 'CustomFields:UnknownFieldForType',
      /**
       * CustomField {0:string} is unknown for the given type.
       */
      UnknownFieldNameForType = 'CustomFields:UnknownFieldNameForType',
      /**
       * CustomField {0:number} is required
       */
      Required = 'CustomFields:Required',
      /**
       * You are not allowed to edit CustomField {0:number}
       */
      NotAllowed = 'CustomFields:NotAllowed',
      /**
       * CustomField {CustomFieldID:number} is {CurrentLength:number} chars long but should not be longer than {MaxLength:number}
       */
      TooLong = 'CustomFields:TooLong',
      /**
       * CustomField {CustomFieldID:number} is {CurrentLength:number} chars long but should be minimal {MinLength:number} chars
       */
      TooShort = 'CustomFields:TooShort',
      /**
       * CustomField {CustomFieldID:number} is {CurrentValue:number} but should not be larger than {MaxValue:number}
       */
      TooLarge = 'CustomFields:TooLarge',
      /**
       * CustomField {CustomFieldID:number} is {CurrentValue:number} but should be at least {MinValue:number}
       */
      TooSmall = 'CustomFields:TooSmall',
      /**
       * CustomField {CustomFieldID:number} is {CurrentValue:string} but should not be later than {MaxDate:string}
       */
      TooLate = 'CustomFields:TooLate',
      /**
       * CustomField {CustomFieldID:number} is {CurrentValue:string} but should not be earlier than {MinDate:string}
       */
      TooEarly = 'CustomFields:TooEarly',
      /**
       * CustomField {CustomFieldID:number} is an enum, but the value {Value:string} doesn't exist
       */
      WrongEnumValue = 'CustomFields:WrongEnumValue',
      /**
       * CustomField {0:number} is an integer field and can't have a fraction
       */
      IntegerCantHaveFraction = 'CustomFields:IntegerCantHaveFraction',
      /**
       * The requested BackendID is already in use
       */
      DuplicateBackendID = 'CustomFields:DuplicateBackendID',
      /**
       * A custom field with BackendID {0:string} does not exist
       */
      BackendIDDoesNotExist = 'CustomFields:BackendIDDoesNotExist',
      /**
       * Enum values are only allowed when CustomField is of DataType 'Enum'
       */
      EnumValuesAreOnlyAllowedWhenCustomFieldIsOfDataTypeEnum = 'CustomFields:EnumValuesAreOnlyAllowedWhenCustomFieldIsOfDataTypeEnum',
      /**
       * Enum values require a key and a value
       */
      EnumValuesRequireAKeyAndValue = 'CustomFields:EnumValuesRequireAKeyAndValue',
      /**
       * Cannot remove enum value that is in use
       */
      CannotRemoveEnumValueThatIsUsedAsDefaultValue = 'CustomFields:CannotRemoveEnumValueThatIsUsedAsDefaultValue',
      /**
       * Enum values are required when CustomField is of DataType 'Enum'
       */
      DataTypeEnumRequiresSomeEnumValues = 'CustomFields:DataTypeEnumRequiresSomeEnumValues',
      /**
       * Date options are only allowed on customfields with type Date or DateTime
       */
      DateOptionsOnNonDateField = 'CustomFields:DateOptionsOnNonDateField',
      /**
       * Range options are only allowed on customfields with type Integer or Decimal
       */
      RangeOptionsOnNonNumberField = 'CustomFields:RangeOptionsOnNonNumberField',
      /**
       * The requested CustomField has a wrong type. Handling {expected:string}, but got a field with {actual:string}
       */
      InvalidType = 'CustomFields:InvalidType',
      /**
       * The provided data type does not allow for array values.
       */
      ArrayNotAllowedForDataType = 'CustomFields:ArrayNotAllowedForDataType',
      /**
       * One or more custom fields are invalid.
       */
      InvalidCustomFields = 'CustomFields:InvalidCustomFields',
      /**
       * Organization unit and organization unit set are mutually exclusive.
       */
      OrganizationUnitAndSetAreMutuallyExclusive = 'CustomFields:OrganizationUnitAndSetAreMutuallyExclusive'
    }

    export const enum DiscountCampaign {
      /**
       * Budget of a discount campaign is required and cannot be negative.
       */
      BudgetCannotBeNegative = 'DiscountCampaign:BudgetCannotBeNegative',
      /**
       * Budget currency cannot be changed.
       */
      CurrencyCannotBeChanged = 'DiscountCampaign:CurrencyCannotBeChanged',
      /**
       * Currency is required when budget is set.
       */
      CurrencyRequiredWhenBudgetIsSet = 'DiscountCampaign:CurrencyRequiredWhenBudgetIsSet',
      /**
       * Start date required when discount need to adhere to the campaing start/end date.
       */
      StartDateRequired = 'DiscountCampaign:StartDateRequired',
      /**
       * cannot set EndDate without StartDate.
       */
      CannotSetEndDateWithoutStartDate = 'DiscountCampaign:CannotSetEndDateWithoutStartDate',
      /**
       * EndDate cannot be lower then StartDate
       */
      EndDateCannotBeLowerThanStartDate = 'DiscountCampaign:EndDateCannotBeLowerThanStartDate'
    }

    export const enum EndpointConfigurations {
      /**
       * The certificate could not be parsed.
       */
      InvalidCertificate = 'EndpointConfigurations:InvalidCertificate',
      /**
       * The certificate has no private key, please provide a certificate with a private key.
       */
      CertificateHasNoPrivateKey = 'EndpointConfigurations:CertificateHasNoPrivateKey',
      /**
       * The certificate does not have Client Authentication usage.
       */
      CertificateHasNoClientAuthentication = 'EndpointConfigurations:CertificateHasNoClientAuthentication',
      /**
       * The certificate does not have Digital Signature or Key Agreement usage.
       */
      CertificateHasNoDigitalSignatureOrKeyAgreement = 'EndpointConfigurations:CertificateHasNoDigitalSignatureOrKeyAgreement',
      /**
       * The certificate must not be expired.
       */
      CertificateMustNotBeExpired = 'EndpointConfigurations:CertificateMustNotBeExpired'
    }

    export const enum EventExportConfigurations {
      /**
       * The certificate could not be parsed.
       */
      InvalidCertificate = 'EventExportConfigurations:InvalidCertificate',
      /**
       * The certificate has no private key, please provide a certificate with a private key.
       */
      CertificateHasNoPrivateKey = 'EventExportConfigurations:CertificateHasNoPrivateKey',
      /**
       * The certificate does not have Client Authentication usage.
       */
      CertificateHasNoClientAuthentication = 'EventExportConfigurations:CertificateHasNoClientAuthentication',
      /**
       * The certificate does not have Digital Signature or Key Agreement usage.
       */
      CertificateHasNoDigitalSignatureOrKeyAgreement = 'EventExportConfigurations:CertificateHasNoDigitalSignatureOrKeyAgreement',
      /**
       * The certificate must not be expired.
       */
      CertificateMustNotBeExpired = 'EventExportConfigurations:CertificateMustNotBeExpired'
    }

    export const enum PriceListManualInputAdjustment {
      /**
       * ManualInputAdjustments ExpireDate cannot be before the EffectiveDate.
       */
      ManualInputAdjustmentsExpireDateCannotBeBeforeEffectiveDate = 'PriceListManualInputAdjustment:ManualInputAdjustmentsExpireDateCannotBeBeforeEffectiveDate'
    }

    export const enum Settings {
      /**
       * Setting with key '{0:string}' doesn't exist.
       */
      NotFound = 'Settings:NotFound',
      /**
       * Conversion failed for setting {key:string} to type {settingType:string} for value {value:string}
       */
      ConversionFailed = 'Settings:ConversionFailed',
      /**
       * Prevented submission of masked value '{0:string}'!
       */
      MaskedValue = 'Settings:MaskedValue',
      /**
       * This setting can only be set on rootlevel
       */
      RootLevelOnly = 'Settings:RootLevelOnly'
    }

    export const enum CompanyCustomerValidator {
      /**
       * The specified user does not belong to this company.
       */
      CompanyUserNotExists = 'CompanyCustomerValidator:CompanyUserNotExists'
    }

    export const enum Communications {
      /**
       * The template for this message cannot be found or has been disabled.
       */
      TemplateNotFoundOrDisabled = 'Communications:TemplateNotFoundOrDisabled'
    }

    export const enum Smtp {
      /**
       * The OAuth2 provider {0:string} is not configured correctly.
       */
      OAuth2ProviderNotConfigured = 'Smtp:OAuth2ProviderNotConfigured',
      /**
       * Failed to acquire OAuth2 token. {0:string}
       */
      OAuth2TokenAcquisitionFailed = 'Smtp:OAuth2TokenAcquisitionFailed'
    }

    export const enum CreateFinancialPeriodAudit {
      /**
       * No available Financial Periods were found to audit
       */
      NoFinancialPeriodsFound = 'CreateFinancialPeriodAudit:NoFinancialPeriodsFound',
      /**
       * The selected Financial Periods exceed a fiscal year, please limit your selection
       */
      FinancialPeriodsExceedFiscalYear = 'CreateFinancialPeriodAudit:FinancialPeriodsExceedFiscalYear',
      /**
       * The supplied organization unit must be a country ou
       */
      CountryOuNeeded = 'CreateFinancialPeriodAudit:CountryOuNeeded'
    }

    export const enum AntiMoneyLaundering {
      /**
       * Transaction blocked due to Anti-Money Laundering regulations.
       */
      Blocked = 'AntiMoneyLaundering:Blocked',
      /**
       * Order can't be finalized due to pending order requirements.
       */
      PendingOrderRequirements = 'AntiMoneyLaundering:PendingOrderRequirements',
      /**
       * There can only be a single Manual rule per country.
       */
      TooManyManualRules = 'AntiMoneyLaundering:TooManyManualRules',
      /**
       * All rules with trigger type Threshold require a Threshold value.
       */
      ThresholdWithoutValue = 'AntiMoneyLaundering:ThresholdWithoutValue',
      /**
       * All rules with trigger type Scripting require a ScriptID.
       */
      ScriptingWithoutValue = 'AntiMoneyLaundering:ScriptingWithoutValue',
      /**
       * All rules that require a Survey need a SurveyID.
       */
      SurveyWithoutValue = 'AntiMoneyLaundering:SurveyWithoutValue',
      /**
       * Only Threshold rules can have a SlidingWindow.
       */
      SlidingWindowWithoutThreshold = 'AntiMoneyLaundering:SlidingWindowWithoutThreshold',
      /**
       * Change of customer blocked due to Anti-Money Laundering regulations.
       */
      CustomerOnAMLOrderCantBeModified = 'AntiMoneyLaundering:CustomerOnAMLOrderCantBeModified'
    }

    export const enum CustomOrderStatus {
      /**
       * CustomOrderStatus with Name '{name:string}' already exists: {value:number}
       */
      StatusWithSameNameAlreadyExists = 'CustomOrderStatus:StatusWithSameNameAlreadyExists',
      /**
       * CustomOrderStatus with Value '{0:number}'
       */
      StatusWithSameValueAlreadyExists = 'CustomOrderStatus:StatusWithSameValueAlreadyExists',
      /**
       * Maximum allowed number of CustomOrderStatus rows has been reached
       */
      MaximumAllowedNumberOfStatusRowsReached = 'CustomOrderStatus:MaximumAllowedNumberOfStatusRowsReached',
      /**
       * Modifying objects created by EVA is not allowed
       */
      ModifyingEVAObjectsNotAllowed = 'CustomOrderStatus:ModifyingEVAObjectsNotAllowed',
      /**
       * Deleting objects created by EVA is not allowed
       */
      DeletingEVAObjectsNotAllowed = 'CustomOrderStatus:DeletingEVAObjectsNotAllowed'
    }

    export const enum AutoComplete {
      /**
       * Failed to create invoice while autocompleting order.
       */
      InvoiceNull = 'AutoComplete:InvoiceNull',
      /**
       * Failed to invocie the Order while autocompleting order.
       */
      InvoicingFailed = 'AutoComplete:InvoicingFailed',
      /**
       * Failed to cancel some of the lines while autocompleting order.
       */
      CancellationFailed = 'AutoComplete:CancellationFailed'
    }

    export const enum OrderGeneration {
      /**
       * Non-employee users cannot use LineActionType ShipLine on an Order with a shop as OrganizationUnit
       */
      ShipLinesNotAllowed = 'OrderGeneration:ShipLinesNotAllowed'
    }

    export const enum OrderLineCreator {
      /**
       * Missing pricing information provided for OrderLine and unable to determine this because no Product has been provided either.
       */
      NoPricingAndProduct = 'OrderLineCreator:NoPricingAndProduct',
      /**
       * A configurable product cannot be added to an order.
       */
      ConfigurableProduct = 'OrderLineCreator:ConfigurableProduct',
      /**
       * Can't add new lines to a signed order.
       */
      OrderIsSigned = 'OrderLineCreator:OrderIsSigned',
      /**
       * Can't add new lines to a completed order.
       */
      OrderIsCompleted = 'OrderLineCreator:OrderIsCompleted',
      /**
       * Can't add new lines to an order with a pending payment.
       */
      PendingPayment = 'OrderLineCreator:PendingPayment',
      /**
       * Can't add a line with another currency to an order.
       */
      InvalidCurrency = 'OrderLineCreator:InvalidCurrency',
      /**
       * Can't add a line with a product whose pre-sale date has not been reached.
       */
      OrderBeforePreSaleDate = 'OrderLineCreator:OrderBeforePreSaleDate',
      /**
       * OrderLine {lineToCloneID:number} is not part of source order {sourceOrderID:number}
       */
      LineToCloneNotPartOfSourceOrder = 'OrderLineCreator:LineToCloneNotPartOfSourceOrder',
      /**
       * LoyaltyProgram does not allow exchange orders
       */
      LoyaltyProgramDoesNotAllowExchangeOrders = 'OrderLineCreator:LoyaltyProgramDoesNotAllowExchangeOrders',
      /**
       * Cannot add return from different LoyaltyProgram
       */
      CannotAddReturnFromDifferentLoyaltyProgram = 'OrderLineCreator:CannotAddReturnFromDifferentLoyaltyProgram'
    }

    export const enum LineActionType {
      /**
       * Cannot mutate the line action type of line {0:number} because it has a related line.
       */
      CannotMutateRelatedLine = 'LineActionType:CannotMutateRelatedLine'
    }

    export const enum OrderCompanyManager {
      /**
       * The specified user does not belong to this company.
       */
      CompanyUserNotExists = 'OrderCompanyManager:CompanyUserNotExists',
      /**
       * The order is either already completed, currently exporting, or has been exported.
       */
      OrderNotModifiable = 'OrderCompanyManager:OrderNotModifiable'
    }

    export const enum OrderCurrency {
      /**
       * It's not possible to change the currency of Order {0:number} anymore, please create a new order
       */
      OrderImmutable = 'OrderCurrency:OrderImmutable',
      /**
       * It's not possible to change the currency of Order {orderID:number} from {oldCurrencyID:string} to {newCurrencyID:string}. The SoldFrom {orgID:number} has currency {ouCurrencyID:string}
       */
      Invalid = 'OrderCurrency:Invalid'
    }

    export const enum OrderCustomer {
      /**
       * Customer on this order can't be changed anymore because it has exported lines.
       */
      OrderHasExportedLines = 'OrderCustomer:OrderHasExportedLines',
      /**
       * The selected user {0:number} is not a customer.
       */
      UserIsNotACustomer = 'OrderCustomer:UserIsNotACustomer',
      /**
       * Attaching business customer to order is not allowed
       */
      AttachBusinessCustomerNotAllowed = 'OrderCustomer:AttachBusinessCustomerNotAllowed',
      /**
       * Attaching a customer to an offline order is not allowed
       */
      AttachCustomerToOfflineOrderNotAllowed = 'OrderCustomer:AttachCustomerToOfflineOrderNotAllowed',
      /**
       * Order is not applicable to attach customers
       */
      OrderNotApplicable = 'OrderCustomer:OrderNotApplicable',
      /**
       * Customer cannot be changed on orders that only have return lines.
       */
      CustomerCannotBeChangedOnFullyReturn = 'OrderCustomer:CustomerCannotBeChangedOnFullyReturn',
      /**
       * Customer cannot be change when loyalty program payments are done.
       */
      CustomerCannotBeChangeWhenLoyaltyProgramPaymentsAreDone = 'OrderCustomer:CustomerCannotBeChangeWhenLoyaltyProgramPaymentsAreDone'
    }

    export const enum ModifyQuantityOrdered {
      /**
       * Error: {0:string}
       */
      CannotModifyQuantityOrdered = 'ModifyQuantityOrdered:CannotModifyQuantityOrdered',
      /**
       * All lines should be in the same order
       */
      LinesShouldBeInSameOrder = 'ModifyQuantityOrdered:LinesShouldBeInSameOrder'
    }

    export const enum OrderOrganizationUnits {
      /**
       * SoldToOrganizationUnitID cannot be the same as the SoldFromOrganizationUnitID
       */
      InvalidOrganizationUnit = 'OrderOrganizationUnits:InvalidOrganizationUnit',
      /**
       * It's not possible to attach Order {orderID:number} to OrganizationUnit with ID {orgID:number} because it has Type {type:OrganizationUnitTypes}
       */
      InvalidOrganizationUnitType = 'OrderOrganizationUnits:InvalidOrganizationUnitType',
      /**
       * It's not possible to attach Order {orderID:number} to OrganizationUnit with ID {orgID:number} because the ou has currency {ouCurrencyID:string} and the order has currency {orderCurrencyID:string}
       */
      InvalidCurrency = 'OrderOrganizationUnits:InvalidCurrency',
      /**
       * Order {0:number} already has exported lines, can no longer change the OrganizationUnit of it.
       */
      OrderAlreadyExported = 'OrderOrganizationUnits:OrderAlreadyExported'
    }

    export const enum StockReservation {
      /**
       * Cannot reserve quantity {quantity:number} on OrderLine {orderLineID:number} because it already has {alreadyReserved:number} reserved and only {ordered:number} ordered
       */
      CannotReserveMoreThanOrdered = 'StockReservation:CannotReserveMoreThanOrdered',
      /**
       * Reservation task cannot be completed because order {0:number} is already completed.
       */
      OrderCompleted = 'StockReservation:OrderCompleted'
    }

    export const enum DeliverShipments {
      /**
       * Shipment {0:number} is not a sales order shipment, cannot deliver it.
       */
      MustBeSalesOrder = 'DeliverShipments:MustBeSalesOrder',
      /**
       * Shipment {0:number} has already been completed, no deliveries of it can be made anymore.
       */
      ShipmentAlreadyCompleted = 'DeliverShipments:ShipmentAlreadyCompleted'
    }

    export const enum ReturnToSupplierRequests {
      /**
       * The ReturnToSupplierRequest {0:number} is not New and cannot be mutated.
       */
      NotNew = 'ReturnToSupplierRequests:NotNew',
      /**
       * The ReturnToSupplierRequest {0:number} is not New or Error and cannot be processed or deleted.
       */
      NotNewOrError = 'ReturnToSupplierRequests:NotNewOrError',
      /**
       * A ReturnToSupplierRequest already exists with the Name {0:string}.
       */
      DuplicateName = 'ReturnToSupplierRequests:DuplicateName'
    }

    export const enum OrderOrchestration {
      /**
       * Cannot determine a fulfillment proposition for order {0:number}
       */
      CannotFulfill = 'OrderOrchestration:CannotFulfill'
    }

    export const enum OrderExportRouter {
      /**
       * Could not determine warehouse for Order {0:number}
       */
      NoWarehouseFound = 'OrderExportRouter:NoWarehouseFound',
      /**
       * Could not determine warehouse as there are multiple candidates: {0:string}
       */
      MultipleWarehousesFound = 'OrderExportRouter:MultipleWarehousesFound'
    }

    export const enum ReturnToSupplierOrderExporter {
      /**
       * Endpoint did not respond with a 200 OK, body [ACK]
       */
      InvalidResponse = 'ReturnToSupplierOrderExporter:InvalidResponse'
    }

    export const enum WarehouseOrderExporter {
      /**
       * Endpoint did not respond with a 200 OK, body [ACK]
       */
      InvalidResponse = 'WarehouseOrderExporter:InvalidResponse'
    }

    export const enum OrderDocuments {
      /**
       * Output type not allowed: {0:string}
       */
      NotAllowed = 'OrderDocuments:NotAllowed'
    }

    export const enum CheckoutOptions {
      /**
       * Handler is required
       */
      MissingHandler = 'CheckoutOptions:MissingHandler',
      /**
       * The given handler doesn't exist
       */
      UnknownHandler = 'CheckoutOptions:UnknownHandler',
      /**
       * The data is not in a valid format for the selected type
       */
      InvalidData = 'CheckoutOptions:InvalidData',
      /**
       * Updating data of this checkout option type is not supported
       */
      UpdatingNotSupported = 'CheckoutOptions:UpdatingNotSupported'
    }

    export const enum AddressBook {
      /**
       * There is already an addressbookitem with this backendid.
       */
      DuplicateBackendID = 'AddressBook:DuplicateBackendID',
      /**
       * It's not possible to delete a default address when you have two or more other addresses in your address book.
       */
      CannotDeleteDefaultAddress = 'AddressBook:CannotDeleteDefaultAddress'
    }

    export const enum Orders {
      /**
       * Order {0:number} does not have the required type {1:OrderTypes}.
       */
      OrderHasInvalidType = 'Orders:OrderHasInvalidType',
      /**
       * Order {0:number} does not have the required property {1:OrderProperties}.
       */
      OrderHasInvalidProperty = 'Orders:OrderHasInvalidProperty',
      /**
       * The current PickupOrganizationUnit of Order {OrderID:number} is not a shop ({OrganizationUnitID:number}).
       */
      PickupOrganizationUnitMustBeShop = 'Orders:PickupOrganizationUnitMustBeShop',
      /**
       * Cannot change the PickupOrganizationUnit of an Order to an OrganizationUnit that is not of type Shop.
       */
      OrganizationUnitMustBeShop = 'Orders:OrganizationUnitMustBeShop'
    }

    export const enum ReturnReasons {
      /**
       * ReturnReason {0:number} does not have the required type {1:ReturnReasonType}.
       */
      ReturnReasonHasInvalidType = 'ReturnReasons:ReturnReasonHasInvalidType'
    }

    export const enum CustomerOrderReturns {
      /**
       * Not allowed to create returns on your current organization unit.
       */
      CannotCreateReturnsOnOrganizationUnit = 'CustomerOrderReturns:CannotCreateReturnsOnOrganizationUnit',
      /**
       * Not allowed to create returns from order with type other than Sales.
       */
      OriginalOrderMustBeSalesOrder = 'CustomerOrderReturns:OriginalOrderMustBeSalesOrder',
      /**
       * No OrderLine found on Order {orderToReturnID:number} to ship {quantity:number} items of product/orderline {productOrderLineID:number}
       */
      NoOrderLineForReturn = 'CustomerOrderReturns:NoOrderLineForReturn'
    }

    export namespace UserTasks {
      export const enum Deactivate {
        /**
         * Cannot deactivate {taskType:string} task {taskID:number} because it has already been completed
         */
        UserTaskAlreadyCompleted = 'UserTasks:Deactivate:UserTaskAlreadyCompleted',
        /**
         * Cannot deactivate {taskType:string} task {taskID:number} because it has already been deactivated.
         */
        UserTaskAlreadyDeactivated = 'UserTasks:Deactivate:UserTaskAlreadyDeactivated'
      }

      export const enum Start {
        /**
         * Cannot start {taskType:string} task {taskID:number} because it has already been completed
         */
        UserTaskAlreadyCompleted = 'UserTasks:Start:UserTaskAlreadyCompleted',
        /**
         * Cannot start {taskType:string} task {taskID:number} because it is not active
         */
        UserTaskNotActive = 'UserTasks:Start:UserTaskNotActive',
        /**
         * Cannot start {taskType:string} task {taskID:number} because it requires a functionality you don't currently have.
         */
        UserTaskRequiresFunctionality = 'UserTasks:Start:UserTaskRequiresFunctionality'
      }

      export const enum Print {
        /**
         * Cannot print task {taskID:number} because its type ({taskTypeID:number}) is not supported
         */
        UserTaskTypeNotSupported = 'UserTasks:Print:UserTaskTypeNotSupported'
      }

      export const enum Complete {
        /**
         * Cannot complete {taskType:string} task {taskID:number} because it has already been completed
         */
        UserTaskAlreadyCompleted = 'UserTasks:Complete:UserTaskAlreadyCompleted',
        /**
         * Cannot complete {taskType:string} task {taskID:number} because it is not active
         */
        UserTaskNotActive = 'UserTasks:Complete:UserTaskNotActive',
        /**
         * Cannot complete {taskType:string} task {taskID:number} because it is not attached to the current user ({currentUserID:number}) but to {attachedUserID:number}
         */
        UserNotAttachedToUserTask = 'UserTasks:Complete:UserNotAttachedToUserTask',
        /**
         * Cannot complete {taskType:string} task {taskID:number} because it is not attached to any user
         */
        UserTaskNotAttached = 'UserTasks:Complete:UserTaskNotAttached'
      }

      export const enum PrintTask {
        /**
         * Cannot complete print task, it was not yet processed.
         */
        NotYetPrinted = 'UserTasks:PrintTask:NotYetPrinted'
      }

    }

    export const enum NamedInstanceFactory {
      /**
       * Handler {0:string} not found.
       */
      HandlerNotFound = 'NamedInstanceFactory:HandlerNotFound'
    }

    export const enum TaskScheduler {
      /**
       * Task ID cannot contain '$'
       */
      InvalidCharacters = 'TaskScheduler:InvalidCharacters',
      /**
       * Invalid cron expression: {0:string}
       */
      InvalidCronExpression = 'TaskScheduler:InvalidCronExpression'
    }

    export const enum Sorting {
      /**
       * Sortingproperty {SortingProperty:string} does not exist in object {ObjectName:string}
       */
      SortingPropertyDoesNotExistInObject = 'Sorting:SortingPropertyDoesNotExistInObject'
    }

    export const enum CreateApiKey {
      /**
       * Incorrect user type - must be of type `API`.
       */
      IncorrectUserType = 'CreateApiKey:IncorrectUserType',
      /**
       * The ExpirationDate should be in the future.
       */
      InvalidExpirationDate = 'CreateApiKey:InvalidExpirationDate',
      /**
       * A PermanentApiKey can't have an ExpirationDate.
       */
      PermanentKeyCantHaveExpirationDate = 'CreateApiKey:PermanentKeyCantHaveExpirationDate'
    }

    export const enum PhoneBook {
      /**
       * It's not possible to delete a primary phone number unless it's one of two or the only one.
       */
      CannotDeletePrimaryPhoneNumber = 'PhoneBook:CannotDeletePrimaryPhoneNumber'
    }

    export const enum UserAssociation {
      /**
       * Primary and related user IDs should be different.
       */
      SameIDsError = 'UserAssociation:SameIDsError',
      /**
       * User association type was deleted, please use another one.
       */
      DeletedAssociationTypeError = 'UserAssociation:DeletedAssociationTypeError'
    }

    export const enum ValidateUserEmailAddress {
      /**
       * Email address already validated.
       */
      EmailAddressAlreadyValidated = 'ValidateUserEmailAddress:EmailAddressAlreadyValidated',
      /**
       * Validation code incorrect.
       */
      InvalidValidationCode = 'ValidateUserEmailAddress:InvalidValidationCode'
    }

    export const enum UserInteractions {
      /**
       * A UserInteraction must be attached to a user. Either provide a UserID in the request or an OrderID that has a CustomerID.
       */
      UserRequired = 'UserInteractions:UserRequired'
    }

    export const enum CreateOrganizationUnitSupplier {
      /**
       * When creating a supplier relationship either SupplierOrganizationUnitID, SupplierOrganizationUnitSetID or SupplierOrganizationUnitIDs must be specified.
       */
      SupplierIsRequired = 'CreateOrganizationUnitSupplier:SupplierIsRequired',
      /**
       * The requested supplier relationship(s) already exist.
       */
      RelationshipAlreadyExists = 'CreateOrganizationUnitSupplier:RelationshipAlreadyExists'
    }

    export const enum DeleteOrganizationUnitSupplier {
      /**
       * Please delete the supplier stock allocation rules before deleting the supplier.
       */
      HasStockAllocationRules = 'DeleteOrganizationUnitSupplier:HasStockAllocationRules'
    }

    export const enum UpdateOrganizationUnitSupplier {
      /**
       * Cannot update supplier relationship as it would create a duplicate. A supplier relationship with the same organization unit, supplier, and target already exists.
       */
      WouldCreateDuplicate = 'UpdateOrganizationUnitSupplier:WouldCreateDuplicate'
    }

    export const enum AdjustStockService {
      /**
       * No adjustments defined
       */
      NoAdjustmentsDefined = 'AdjustStockService:NoAdjustmentsDefined',
      /**
       * ProductID or ProductBackendID required
       */
      ProductIDOrProductBackendIDRequired = 'AdjustStockService:ProductIDOrProductBackendIDRequired'
    }

    export const enum CommitOrderLines {
      /**
       * It's not possible to choose a commitment status other than ManualCommit or ForcedCommit.
       */
      CannotChooseCommitmentStatus = 'CommitOrderLines:CannotChooseCommitmentStatus'
    }

    export const enum MoveStock {
      /**
       * No movements defined.
       */
      NoMovementsDefined = 'MoveStock:NoMovementsDefined',
      /**
       * Parameter ProductID or ProductBackendID required.
       */
      ProductRequired = 'MoveStock:ProductRequired',
      /**
       * Mutation {0} ({1}) already exists.
       */
      AlreadyExists = 'MoveStock:AlreadyExists'
    }

    export const enum ProcessStockMutationFile {
      /**
       * You are not allowed to update the stock of the specified OrganizationUnit.
       */
      UnauthorizedForStockMutation = 'ProcessStockMutationFile:UnauthorizedForStockMutation',
      /**
       * StockmutationReason 'FullStockCount' can only be used by the FullStockCount process.
       */
      StockmutationReasonFSCCanOnlyBeUsedByFSCProcress = 'ProcessStockMutationFile:StockmutationReasonFSCCanOnlyBeUsedByFSCProcress'
    }

    export const enum StockLabelSettings {
      /**
       * Cannot change the default settings for a stocklabel.
       */
      CannotChangeDefaultSettings = 'StockLabelSettings:CannotChangeDefaultSettings',
      /**
       * The feature shouldn't be inherited when setting it as default.
       */
      CannotSetIsDefaultForInheritedFeatue = 'StockLabelSettings:CannotSetIsDefaultForInheritedFeatue'
    }

    export const enum CreateShipment {
      /**
       * An Order identifier is required, either `OrderID`, `OrderFulfillmentID` or `OrderBackendID` and `OrderBackendSystemID` must be provided on the top-level request or in the `Lines` or `Packages`
       */
      MissingOrderReference = 'CreateShipment:MissingOrderReference',
      /**
       * Cannot have duplicate identifiers
       */
      NoDuplicateIdentifiersAllowed = 'CreateShipment:NoDuplicateIdentifiersAllowed'
    }

    export const enum UndoShipment {
      /**
       * Shipment line {ShipmentLineID:number} on shipment {ShipmentID:number} has already been received
       */
      ShipmentLineAlreadyReceived = 'UndoShipment:ShipmentLineAlreadyReceived',
      /**
       * Shipment {0:number} has not yet been received and cannot be undone.
       */
      ShipmentNotReceived = 'UndoShipment:ShipmentNotReceived'
    }

    export const enum UpdateShipment {
      /**
       * RequireReceiveMethodReason is required
       */
      ReceiveMethodReasonNeedsToBeProvided = 'UpdateShipment:ReceiveMethodReasonNeedsToBeProvided',
      /**
       * Cannot have duplicate identifiers
       */
      NoDuplicateIdentifiersAllowed = 'UpdateShipment:NoDuplicateIdentifiersAllowed'
    }

    export const enum CreateSerialNumber {
      /**
       * Cannot create serial numbers for second chance products using this service.
       */
      InvalidProduct = 'CreateSerialNumber:InvalidProduct'
    }

    export const enum Scripts {
      /**
       * The Script is still being used, cannot delete.
       */
      ScriptInUse = 'Scripts:ScriptInUse'
    }

    export const enum CreatePurchaseOrder {
      /**
       * A SupplierID is required when Unified Orders are enabled.
       */
      SupplierRequiredForUnifiedOrders = 'CreatePurchaseOrder:SupplierRequiredForUnifiedOrders'
    }

    export const enum SupplierPurchaseOrder {
      /**
       * Inconsistent use of product identifiers.
       */
      InconsistentProductIdentifiers = 'SupplierPurchaseOrder:InconsistentProductIdentifiers'
    }

    export const enum PrintProductPriceLabel {
      /**
       * No barcode found for product with ID {0:number}
       */
      ProductBarcodeNotFound = 'PrintProductPriceLabel:ProductBarcodeNotFound'
    }

    export const enum RenderThermalPrintResult {
      /**
       * Cannot use Blob with MimeType {0:string}.
       */
      InvalidMimeType = 'RenderThermalPrintResult:InvalidMimeType'
    }

    export const enum ExplainProductPrices {
      /**
       * PriceList is deactivated.
       */
      PriceListIsDeactivated = 'ExplainProductPrices:PriceListIsDeactivated',
      /**
       * Must provide between 1 and 100 product ID's.
       */
      RequestedProductIDsInvalid = 'ExplainProductPrices:RequestedProductIDsInvalid'
    }

    export const enum PushPriceList {
      /**
       * When creating a new price list, you must specify a currency ID.
       */
      MissingCurrencyID = 'PushPriceList:MissingCurrencyID',
      /**
       * Cannot create a markup component that does not have its associated MarkupData
       */
      MissingMarkupData = 'PushPriceList:MissingMarkupData',
      /**
       * Cannot create a CopyPricesFromOtherPriceList component that does not have its associated CopyPricesFromOtherPriceListData
       */
      MissingCopyPricesFromOtherPriceListData = 'PushPriceList:MissingCopyPricesFromOtherPriceListData'
    }

    export const enum Pricing {
      /**
       * This price list is not valid for simple updates.
       */
      NotValidForSimpleUpdates = 'Pricing:NotValidForSimpleUpdates',
      /**
       * Duplicate prices for products are not supported in simple price lists.
       */
      DuplicatePricesForProduct = 'Pricing:DuplicatePricesForProduct',
      /**
       * Expiration dates are not supported for simple price lists.
       */
      ExpirationDatesNotSupported = 'Pricing:ExpirationDatesNotSupported',
      /**
       * Prices must have either `ProductID`, `CustomID` or `BackendID` and cannot be mixed!
       */
      InvalidProductIdentifiers = 'Pricing:InvalidProductIdentifiers',
      /**
       * One or more products could not be found: {0:string}
       */
      ProductsNotFound = 'Pricing:ProductsNotFound'
    }

    export const enum PaymentTransactionLedger {
      /**
       * This PaymentTransactionLedgerType is internal and can't be used to create ledgers
       */
      IsInternal = 'PaymentTransactionLedger:IsInternal'
    }

    export const enum OrganizationUnitHierarchy {
      /**
       * If a Functionality Scope is passed, a Functionality must also be passed
       */
      NeedFunctionalityWithScope = 'OrganizationUnitHierarchy:NeedFunctionalityWithScope'
    }

    export const enum ImportOrganizationUnits {
      /**
       * ParentID {0:string} does not exist.
       */
      ParentNotFound = 'ImportOrganizationUnits:ParentNotFound',
      /**
       * ReturnOrganizationUnitID {0:string} does not exist.
       */
      ReturnOrganizationUnitNotFound = 'ImportOrganizationUnits:ReturnOrganizationUnitNotFound',
      /**
       * There is an infinite loop reference starting with {0:string} -> {1:string}.
       */
      InfiniteLoop = 'ImportOrganizationUnits:InfiniteLoop'
    }

    export const enum AddServiceProductToOrder {
      /**
       * Product {0:string} is not a service product.
       */
      ProductRequiresServiceType = 'AddServiceProductToOrder:ProductRequiresServiceType'
    }

    export const enum CreateOrderAppToken {
      /**
       * Order already has a customer attached.
       */
      OrderHasCustomerAttached = 'CreateOrderAppToken:OrderHasCustomerAttached'
    }

    export const enum CreateOrderLedger {
      /**
       * OrderLine is either not found or doesn't belong to the Order
       */
      OrderLineNotFound = 'CreateOrderLedger:OrderLineNotFound'
    }

    export const enum CreatePurchaseOrderLineUnitPriceCorrection {
      /**
       * Missing required parameters for creating a new purchase order line correction.
       */
      MissingRequiredParameters = 'CreatePurchaseOrderLineUnitPriceCorrection:MissingRequiredParameters'
    }

    export const enum OrderCustomStatus {
      /**
       * This name already exists!
       */
      AlreadyExists = 'OrderCustomStatus:AlreadyExists',
      /**
       * This Backend ID is already used!
       */
      AlreadyUsedBackendID = 'OrderCustomStatus:AlreadyUsedBackendID'
    }

    export const enum OrderCustomType {
      /**
       * This name already exists!
       */
      AlreadyExists = 'OrderCustomType:AlreadyExists',
      /**
       * This Backend ID is already used!
       */
      AlreadyUsedBackendID = 'OrderCustomType:AlreadyUsedBackendID'
    }

    export const enum OrderLink {
      /**
       * Primary and related order IDs should be different.
       */
      SameIDsError = 'OrderLink:SameIDsError',
      /**
       * Order link type was deleted, please use another one.
       */
      DeletedLinkTypeError = 'OrderLink:DeletedLinkTypeError'
    }

    export const enum PushSalesOrders {
      /**
       * The open amount on the order {backendID:string} is supposed to be zero, but it was {openAmount:number}
       */
      OpenAmountNotZero = 'PushSalesOrders:OpenAmountNotZero'
    }

    export const enum SetOrderBackendID {
      /**
       * Order already has a BackendID/BackendSystemID set.
       */
      OrderAlreadyHasBackendIdentifier = 'SetOrderBackendID:OrderAlreadyHasBackendIdentifier',
      /**
       * This BackendID/BackendSystemID already exists.
       */
      BackendIdentifierAlreadyExists = 'SetOrderBackendID:BackendIdentifierAlreadyExists',
      /**
       * Cannot use reserved BackendSystemID {0:string}.
       */
      CannotUseBackendSystemID = 'SetOrderBackendID:CannotUseBackendSystemID'
    }

    export const enum SoldBy {
      /**
       * The SoldByID {0:number} is not an Employee-type user.
       */
      SoldByMustBeEmployee = 'SoldBy:SoldByMustBeEmployee',
      /**
       * The SoldBy can no longer be updated on Order {0:number} as it has already been invoiced.
       */
      CanNoLongerUpdateSoldBy = 'SoldBy:CanNoLongerUpdateSoldBy',
      /**
       * The SoldBy cannot be updated on Order {0:number} because it is not a Sales Order.
       */
      OrderMustBeSalesOrder = 'SoldBy:OrderMustBeSalesOrder'
    }

    export const enum ShipExternalOrderService {
      /**
       * Cannot perform shipment on {0:string}, no lines could be shipped.
       */
      CannotPerformShipment = 'ShipExternalOrderService:CannotPerformShipment'
    }

    export const enum ShipOrder {
      /**
       * Nothing shipped: {0}
       */
      NothingShipped = 'ShipOrder:NothingShipped'
    }

    export const enum UpdateOrderCustomFields {
      /**
       * Either CustomFields or CustomFieldsByBackendID needs to be specified
       */
      MissingCustomFields = 'UpdateOrderCustomFields:MissingCustomFields'
    }

    export const enum UpdateOrderLineStockLabel {
      /**
       * Either OrderID or OrderLineID is required.
       */
      OrderIDOrOrderLineIDRequired = 'UpdateOrderLineStockLabel:OrderIDOrOrderLineIDRequired'
    }

    export const enum CreateReturnToSupplierOrder {
      /**
       * Cannot create a ReturnToSupplier order from an Interbranch order
       */
      CannotCreateRTSFromInterbranch = 'CreateReturnToSupplierOrder:CannotCreateRTSFromInterbranch'
    }

    export const enum UpdateReturnToSupplierOrder {
      /**
       * Order is not a return to supplier order
       */
      NotAnRTSOrder = 'UpdateReturnToSupplierOrder:NotAnRTSOrder',
      /**
       * Cannot update placed order
       */
      CannotUpdatePlacedOrder = 'UpdateReturnToSupplierOrder:CannotUpdatePlacedOrder'
    }

    export const enum AutoTransferInterbranchOrder {
      /**
       * One of the organization units must be the current organization unit.
       */
      OneOfOrganizationUnitsMustBeCurrent = 'AutoTransferInterbranchOrder:OneOfOrganizationUnitsMustBeCurrent',
      /**
       * Cannot transfer between these organization units.
       */
      CannotTransferBetweenOrganizationUnits = 'AutoTransferInterbranchOrder:CannotTransferBetweenOrganizationUnits',
      /**
       * Order is not an interbranch order.
       */
      OrderNotInterbranch = 'AutoTransferInterbranchOrder:OrderNotInterbranch',
      /**
       * Order cannot be placed: {0:string}.
       */
      OrderCannotBePlaced = 'AutoTransferInterbranchOrder:OrderCannotBePlaced',
      /**
       * Order must be unified to be shipped.
       */
      OrderMustBeUnified = 'AutoTransferInterbranchOrder:OrderMustBeUnified'
    }

    export const enum OrderFulfillment {
      /**
       * It was not possible to split OrderLine {0:number} to complete the partial cancellation of the OrderLine's fulfillment
       */
      SplitOrderLineFailed = 'OrderFulfillment:SplitOrderLineFailed',
      /**
       * It's not currently allowed to partially cancel an OrderLine fulfillment, this is controlled by the setting `OrderFulfillment:AllowPartialCancellation`.
       */
      PartialCancellationNotAllowed = 'OrderFulfillment:PartialCancellationNotAllowed'
    }

    export const enum SetOrderFulfillmentOptions {
      /**
       * The selected fulfillment option is not valid.
       */
      InvalidOptionSelected = 'SetOrderFulfillmentOptions:InvalidOptionSelected'
    }

    export const enum ShipOrderFulfillment {
      /**
       * It's only possible to ship OrderFulfillmentLines of type Physical.
       */
      CannotShipNonPhysicalLines = 'ShipOrderFulfillment:CannotShipNonPhysicalLines',
      /**
       * The OrderLine {OrderLineID:number} behind OrderFulfillmentLineID {OrderFulfillmentLineID:number} has already been shipped or otherwise completed.
       */
      AlreadyShipped = 'ShipOrderFulfillment:AlreadyShipped'
    }

    export const enum UpdateOrderFulfillmentLine {
      /**
       * It's not possible to increase the quantity of an OrderFulfillmentLine
       */
      CannotIncreaseQuantity = 'UpdateOrderFulfillmentLine:CannotIncreaseQuantity',
      /**
       * It's only possible to change the quantity of a physical fulfillment line.
       */
      PhysicalLineRequired = 'UpdateOrderFulfillmentLine:PhysicalLineRequired'
    }

    export const enum RequeueErrorMessage {
      /**
       * Can only requeue a message that is in status New or FailureAfterRetry.
       */
      CannotRetrySuccessfulMessage = 'RequeueErrorMessage:CannotRetrySuccessfulMessage'
    }

    export const enum CreateInvoice {
      /**
       * When creating an invoice it's required to specify CreditorOrganizationUnitID.
       */
      CreditorRequired = 'CreateInvoice:CreditorRequired'
    }

    export const enum InvoiceAdditionalAmountType {
      /**
       * InvoiceAdditionalAmountType with name `{0:string}` already exists
       */
      AlreadyExists = 'InvoiceAdditionalAmountType:AlreadyExists'
    }

    export const enum InvoiceDispute {
      /**
       * InvoiceDisputeReason with name `{0:string}` already exists
       */
      AlreadyExists = 'InvoiceDispute:AlreadyExists',
      /**
       * InvoiceDisputeReasons with AutoResolve require a LedgerClass
       */
      LedgerClassRequired = 'InvoiceDispute:LedgerClassRequired'
    }

    export const enum ListManualInvoices {
      /**
       * Use at least one filter.
       */
      InvalidRequestMessage = 'ListManualInvoices:InvalidRequestMessage'
    }

    export const enum ListShipmentsToInvoice {
      /**
       * Either ShipFromOrganizationUnitID or ShipToOrganizationUnitID must be specified.
       */
      OrganizationUnitFilterRequired = 'ListShipmentsToInvoice:OrganizationUnitFilterRequired'
    }

    export const enum UpdateInvoice {
      /**
       * When creating an invoice it's required to specify CreditorOrganizationUnitID.
       */
      CreditorRequired = 'UpdateInvoice:CreditorRequired'
    }

    export const enum ExternalFullStockCountService {
      /**
       * A product identifier is missing for line {0:number}.
       */
      MissingProductIdentifierForLine = 'ExternalFullStockCountService:MissingProductIdentifierForLine',
      /**
       * The product identifier for line {0:number} did not resolve to a valid product.
       */
      InvalidProductIdentifierForLine = 'ExternalFullStockCountService:InvalidProductIdentifierForLine',
      /**
       * The StockLabelID for line {0:number} is invalid.
       */
      InvalidStockLabelIDForLine = 'ExternalFullStockCountService:InvalidStockLabelIDForLine'
    }

    export const enum CashDeposit {
      /**
       * SignatureBlobID or SignatureBlobUrl is required
       */
      SignatureRequired = 'CashDeposit:SignatureRequired'
    }

    export const enum CashExpense {
      /**
       * SignatureBlobID is required
       */
      SignatureRequired = 'CashExpense:SignatureRequired'
    }

    export const enum DeleteCashExpense {
      /**
       * Cannot delete cash expense because the cash journal has already been closed.
       */
      CashJournalClosed = 'DeleteCashExpense:CashJournalClosed',
      /**
       * Cannot delete cash expense because the financial period is not open.
       */
      FinancialPeriodClosed = 'DeleteCashExpense:FinancialPeriodClosed'
    }

    export const enum SetEmployeesForFinancialPeriod {
      /**
       * Cannot set commission for some employees and not for others. Either set commission for all employees or none.
       */
      CannotSetPartialCommission = 'SetEmployeesForFinancialPeriod:CannotSetPartialCommission',
      /**
       * The financial period is not pending, so employees cannot be changed.
       */
      FinancialPeriodNotPendingEmployees = 'SetEmployeesForFinancialPeriod:FinancialPeriodNotPendingEmployees',
      /**
       * The commission shares must add up to 100%.
       */
      CommissionMustAddUp = 'SetEmployeesForFinancialPeriod:CommissionMustAddUp'
    }

    export const enum SetLoyaltyProgramForFinancialPeriod {
      /**
       * The financial period is not pending, so the loyalty program cannot be changed.
       */
      FinancialPeriodNotPendingLoyaltyProgram = 'SetLoyaltyProgramForFinancialPeriod:FinancialPeriodNotPendingLoyaltyProgram'
    }

    export const enum CashCorrectionReasons {
      /**
       * A correction reason with this name already exists
       */
      CashCorrectionReasonWithNameAlreadyExists = 'CashCorrectionReasons:CashCorrectionReasonWithNameAlreadyExists',
      /**
       * A correction reason with this backendID already exists
       */
      CashCorrectionReasonWithBackendIDAlreadyExists = 'CashCorrectionReasons:CashCorrectionReasonWithBackendIDAlreadyExists'
    }

    export const enum ReplayEvents {
      /**
       * This event export configuration does not support replaying events
       */
      CannotReplayEvents = 'ReplayEvents:CannotReplayEvents',
      /**
       * This operation is not supported on a replay-configuration
       */
      NotSupportedOnReplay = 'ReplayEvents:NotSupportedOnReplay'
    }

    export const enum Currency {
      /**
       * Currencies are not mutable
       */
      Immutable = 'Currency:Immutable'
    }

    export const enum GetActiveShoppingCartService {
      /**
       * The current organization unit should be a webshop.
       */
      OrganizationUnitShouldBeWebshop = 'GetActiveShoppingCartService:OrganizationUnitShouldBeWebshop'
    }

    export const enum ValidateToken {
      /**
       * Cloud is currently not available
       */
      CloudUnavailable = 'ValidateToken:CloudUnavailable'
    }

    export const enum AddBoardingPassToOrder {
      /**
       * This boarding pass barcode does not match the expected format.
       */
      FormatInvalid = 'AddBoardingPassToOrder:FormatInvalid',
      /**
       * This boarding pass is not valid for this airport.
       */
      InvalidFlight = 'AddBoardingPassToOrder:InvalidFlight'
    }

    export const enum ApplyScriptEdits {
      /**
       * Invalid value for field {Field:string}, expected a string, number, boolean or null but got {ActualValue:string}
       */
      InvalidValue = 'ApplyScriptEdits:InvalidValue'
    }

    export const enum CustomerReturn {
      /**
       * `OriginalOrderTimeStamp` is not allowed in the future
       */
      FutureTimeStampNotAllowed = 'CustomerReturn:FutureTimeStampNotAllowed'
    }

    export const enum CreateInStoreReturn {
      /**
       * This order cannot be returned through this service.
       */
      CannotBeReturned = 'CreateInStoreReturn:CannotBeReturned',
      /**
       * This order cannot be refunded.
       */
      CannotBeRefunded = 'CreateInStoreReturn:CannotBeRefunded'
    }

    export const enum PrintSecondChanceProductPriceLabelService {
      /**
       * No barcode found for product with ID {0:number}
       */
      ProductBarcodeNotFound = 'PrintSecondChanceProductPriceLabelService:ProductBarcodeNotFound'
    }

    export const enum Repairs {
      /**
       * Customer must be attached to case for deferred shipment repair.
       */
      DeferredShippingMustHaveAttachedCustomer = 'Repairs:DeferredShippingMustHaveAttachedCustomer',
      /**
       * Customer must have an email address.
       */
      DeferredShippingCustomerMustHaveEmailAddress = 'Repairs:DeferredShippingCustomerMustHaveEmailAddress',
      /**
       * Either a ProductID or an OrderLineID for a product is required.
       */
      EitherOrderLineIDOrProductIDIsRequired = 'Repairs:EitherOrderLineIDOrProductIDIsRequired',
      /**
       * EVAPay not loaded
       */
      EVAPayNotLoaded = 'Repairs:EVAPayNotLoaded',
      /**
       * EVAPay not configured
       */
      EVAPayNotConfigured = 'Repairs:EVAPayNotConfigured',
      /**
       * Cannot use EVAPay because order has no customer attached
       */
      EVAPayNoCustomer = 'Repairs:EVAPayNoCustomer',
      /**
       * UserTaskID or RepairID must be specified when printing repair thermal receipt
       */
      UserTaskOrRepairIDRequiredWhenPrintingReceipt = 'Repairs:UserTaskOrRepairIDRequiredWhenPrintingReceipt'
    }

    export const enum StockMovementFollowUp {
      /**
       * Invalid quantity
       */
      InvalidQuantity = 'StockMovementFollowUp:InvalidQuantity'
    }

    export const enum ShipFromStoreUserTaskComposer {
      /**
       * Station is required for this step.
       */
      StationRequired = 'ShipFromStoreUserTaskComposer:StationRequired',
      /**
       * Order is not paid and thus cannot be delivered.
       */
      OrderNotPaid = 'ShipFromStoreUserTaskComposer:OrderNotPaid',
      /**
       * Picked quantity is too much.
       */
      ExcessPickedQuantity = 'ShipFromStoreUserTaskComposer:ExcessPickedQuantity',
      /**
       * Product {replacementProductID:number} is not a replacement product for {originalProductID:number}.
       */
      NotAReplacementProduct = 'ShipFromStoreUserTaskComposer:NotAReplacementProduct'
    }

    export const enum ReturnToSupplierTaskErrors {
      /**
       * You can only have one active return to supplier task.
       */
      UserCanOnlyHaveOneActiveReturnToSupplierTask = 'ReturnToSupplierTaskErrors:UserCanOnlyHaveOneActiveReturnToSupplierTask',
      /**
       * Can not exclude more products for shipping then there are available on the shipment.
       */
      CanNotExcludeMoreProductThenPresentOnTheShipment = 'ReturnToSupplierTaskErrors:CanNotExcludeMoreProductThenPresentOnTheShipment',
      /**
       * Can not ship more products then present on the order.
       */
      CanNotShipMoreProductsThenPresentOnTheOrder = 'ReturnToSupplierTaskErrors:CanNotShipMoreProductsThenPresentOnTheOrder',
      /**
       * Task no longer has a linked shipment. You can complete this task.
       */
      NoLinkidShipmentOnTask = 'ReturnToSupplierTaskErrors:NoLinkidShipmentOnTask',
      /**
       * Can not finalize return to supplier order when other tasks are active.
       */
      CanNotFinalizeReturnToSupplierOrderWithOpenTasks = 'ReturnToSupplierTaskErrors:CanNotFinalizeReturnToSupplierOrderWithOpenTasks',
      /**
       * Selected return to supplier task is already started.
       */
      ReturnToSupplierTaskAlreadyStarted = 'ReturnToSupplierTaskErrors:ReturnToSupplierTaskAlreadyStarted'
    }

    export const enum SetLineForReturnToSupplierShipment {
      /**
       * Cannot provide more than one serial number
       */
      CannotProvideMoreThanOneSerialNumber = 'SetLineForReturnToSupplierShipment:CannotProvideMoreThanOneSerialNumber'
    }

    export const enum PriceLabelPrinter {
      /**
       * Product not found
       */
      ProductNotFound = 'PriceLabelPrinter:ProductNotFound'
    }

    export const enum CreateCustomUserTaskType {
      /**
       * Can't create CustomUserTaskType because its name already exists
       */
      CustomUserTaskTypeAlreadyExists = 'CreateCustomUserTaskType:CustomUserTaskTypeAlreadyExists'
    }

    export const enum DeactivateAllUserTasks {
      /**
       * Could not deactivate all user tasks for this environment.
       */
      InvalidEnvironment = 'DeactivateAllUserTasks:InvalidEnvironment'
    }

    export const enum UserTaskNotCompletedReasonErrors {
      /**
       * Can't create UserTaskNotCompletedReason because its name already exists
       */
      UserTaskNotCompletedReasonAlreadyExists = 'UserTaskNotCompletedReasonErrors:UserTaskNotCompletedReasonAlreadyExists',
      /**
       * Cannot update a system UserTaskNotCompletedReason
       */
      CannotUpdateSystemUserTaskNotCompletedReason = 'UserTaskNotCompletedReasonErrors:CannotUpdateSystemUserTaskNotCompletedReason'
    }

    export const enum UserTaskPriorityServices {
      /**
       * This User Task Priority already exists
       */
      UserTaskPriorityAlreadyExists = 'UserTaskPriorityServices:UserTaskPriorityAlreadyExists',
      /**
       * Cannot modify the Default User Task Priority
       */
      CannotModifyDefaultPriority = 'UserTaskPriorityServices:CannotModifyDefaultPriority'
    }

    export const enum CompleteZonedCycleCounts {
      /**
       * Deviation Reason is required for zoned cycle count {0:number}
       */
      DeviationReasonRequired = 'CompleteZonedCycleCounts:DeviationReasonRequired'
    }

    export const enum CreateZonedCycleCountService {
      /**
       * Cannot create cycle count for product {productID:number} in organization unit {ouID:number} because {reason:string}
       */
      CannotCreateCount = 'CreateZonedCycleCountService:CannotCreateCount'
    }

    export const enum CycleCountZoneGroup {
      /**
       * This name is already in use
       */
      DuplicateName = 'CycleCountZoneGroup:DuplicateName'
    }

    export const enum ZonedCycleCountSchedule {
      /**
       * Date needs to be at least one day in the future.
       */
      DateTooSoon = 'ZonedCycleCountSchedule:DateTooSoon',
      /**
       * Require at least a product query or filter to be set.
       */
      NoProductFilters = 'ZonedCycleCountSchedule:NoProductFilters',
      /**
       * Require at least a Date or CycleCountIntervalInDays set.
       */
      NoDateOrInterval = 'ZonedCycleCountSchedule:NoDateOrInterval'
    }

    export const enum SecondChangeProductRegistration {
      /**
       * Product does not match expected product for the given usertask.
       */
      ProductMismatch = 'SecondChangeProductRegistration:ProductMismatch',
      /**
       * MimeType {MimeType:string} for BlobID {BlobID:string} is an invalid MimeType for an image
       */
      InvalidMimeTypeForImage = 'SecondChangeProductRegistration:InvalidMimeTypeForImage'
    }

  }


  export interface AsyncRequestResponse {
    JobID: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ServiceError {
    /**
     * A human-readable message describing the error.
     */
    Message: string;
    /**
     * A short description of the error type.
     */
    Type: string;
    /**
     * A unique ID identifying this error.
     */
    Code: string;
    /**
     * A unique ID identifying the request. Please provide this ID when contacting support.
     */
    RequestID: string;
    Data?: any;
  }

  export interface ResponseMessageMetadata {
    /**
     * When using the `ExternalIDs` mode, this will hold the resolved external IDs.
     */
    ExternalIDs?: { [key: string]: { [key: string]: string } };
    /**
     * When using the `ExternalIDs` mode, this will hold the external IDs we couldn't resolve.
     */
    UnresolvedExternalIDs?: { [key: string]: string[] };
    /**
     * This is a flag indicating whether the async request is done. If it is `null`, it is not an async service. If it is `false`, the response is not yet available. Do not process the properties in that case. It is is `true` the properties are filled with the result of the call.
     */
    IsAsyncResultAvailable?: boolean;
  }

  export interface AsyncRequestResultRequest {
    JobID: string;
  }

  /**
   * Lists tax code mappings.
   */
  export interface ListTaxCodeMappings {
    InitialPageConfig?: PageTokenConfig<ListTaxCodeMappingsFilter>;
    PageToken?: string;
  }

  export interface PageTokenConfig<T = { [key: string]: string }> {
    Filter?: Partial<T>;
    /**
     * Min: 1
     */
    Limit?: number;
  }

  export interface ListTaxCodeMappingsFilter {
    TaxCodeID?: number;
    TaxProviderID?: number;
    BackendID?: string;
  }

  export interface ListTaxCodeMappingsResponse {
    Results?: TaxCodeMappingDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface TaxCodeMappingDto {
    /**
     * Entity type: TaxCodeMapping
     */
    ID: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID: number;
    /**
     * Entity type: TaxProvider
     */
    TaxProviderID: number;
    BackendID: string;
  }

  /**
   * Create a new TaxCodeMapping. Only one mapping per TaxCode & TaxProvider is allowed.
   */
  export interface CreateTaxCodeMapping {
    BackendID: string;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID: number;
    /**
     * Entity type: TaxProvider
     */
    TaxProviderID: number;
  }

  export interface CreateTaxCodeMappingResponse {
    /**
     * Entity type: TaxCodeMapping
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an existing TaxCodeMapping. Only one mapping per TaxCode & TaxProvider is allowed.
   */
  export interface UpdateTaxCodeMapping {
    /**
     * Entity type: TaxCodeMapping
     */
    ID: number;
    BackendID?: string;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
    /**
     * Entity type: TaxProvider
     */
    TaxProviderID?: number;
  }

  export interface EmptyResponseMessage {
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Delete a tax mapping.
   */
  export interface DeleteTaxCodeMapping {
    /**
     * Entity type: TaxCodeMapping
     */
    ID: number;
  }

  /**
   * Lists tax providers.
   */
  export interface ListTaxProviders {
    InitialPageConfig?: PageTokenConfig<ListTaxProvidersFilter>;
    PageToken?: string;
  }

  export interface ListTaxProvidersFilter {
    Name?: string;
    Type?: string;
  }

  export interface ListTaxProvidersResponse {
    Results?: TaxProviderDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface TaxProviderDto {
    /**
     * Entity type: TaxProvider
     */
    ID?: number;
    Name?: string;
    Type?: string;
    BackendID?: string;
    Description?: string;
    Configuration?: { [key: string]: any };
  }

  /**
   * Create a new TaxProvider
   */
  export interface CreateTaxProvider {
    Name: string;
    Type: string;
    BackendID?: string;
    Description?: string;
    Configuration?: any;
  }

  export interface CreateTaxProviderResponse {
    /**
     * Entity type: TaxProvider
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an existing TaxProvider
   */
  export interface UpdateTaxProvider {
    /**
     * Entity type: TaxProvider
     */
    ID: number;
    Name?: string;
    Type?: string;
    Description?: string;
    BackendID?: string;
    /**
     * Updating a configuration means overwriting the previous one. If the update will contain a type change, the configuration will be reset to null unless otherwise specified.
     */
    Configuration?: any;
  }

  /**
   * Delete a TaxProvider
   */
  export interface DeleteTaxProvider {
    /**
     * Entity type: TaxProvider
     */
    ID: number;
  }

  /**
   * Get the available tax provider types.
   */
  export interface GetTaxProviderTypes {
  }

  export interface GetTaxProviderTypesResponse {
    Types: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get a tax provider by ID.
   */
  export interface GetTaxProvider {
    /**
     * Entity type: TaxProvider
     */
    ID: number;
  }

  export interface GetTaxProviderResponse {
    TaxProvider: TaxProviderDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Requests a magic login link, which will send an email to the user (if found).
   * 
   * - Setting `MagicLink:Enabled` must be enabled explicitly
   * - When using a `RedirectUrl`, configure setting `MagicLink:RedirectDomains` to whitelist supported redirect hosts (recommended!)
   * 
   * Using the returned `RequestToken`, you can either subscribe to the `/magicLink` SignalR hub, or poll the `CheckMagicLink` service. As soon as the emailed magic link is clicked, that hub and service
   * will return either a `TemporaryToken` you can call `Login` with for this user, or an `APIKey` that's already valid and ready to proceed with.
   * 
   * - The email sent contains a link that will work for 10 minutes
   * - When clicked, this will generate either a `TemporaryToken` that can be used to `Login` with for 3 minutes, or an `ApiKey` that can be used for 1 hour
   * - This data is additionally exposed through `CheckMagicLink` for 1 minute after clicking
   * - When using a `TemporaryToken` to log in with, the resulting `AuthenticationToken` is valid for 1 week (or `SessionDuration:MagicLink`) and will refresh every time when used
   * 
   * Finally, when specifying `IncludeManualCode`, the email sent will also contain a short code for the user to _manually_ input into your application - call `ConfirmMagicLink` with both the `RequestToken`
   * and that `ManualCode` to obtain either the `TemporaryToken` or `ApiKey` like above, when valid.
   */
  export interface SendMagicLink {
    EmailAddress: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    RedirectUrl?: string;
    IncludeManualCode?: boolean;
  }

  export interface SendMagicLinkResponse {
    RequestToken: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Confirms a magic login by `ManualCode`.
   */
  export interface ConfirmMagicLink {
    RequestToken: string;
    ManualCode?: string;
  }

  export interface ConfirmMagicLinkResponse {
    TemporaryToken?: string;
    ApiKey?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Checks if a magic login was confirmed already.
   */
  export interface CheckMagicLink {
    RequestToken: string;
  }

  export interface CheckMagicLinkResponse {
    TemporaryToken?: string;
    ApiKey?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get the AvailableGiftCardConfigurations taking into account the OrganizationUnitID from ClientContext.
   * 
   * `GiftCardConfiguration`: an item that describes the giftcard properties of a product with type GiftCard
   */
  export interface GetAvailableGiftCardConfigurations {
  }

  export interface GetAvailableGiftCardConfigurationsResponse {
    GiftCardConfigurations: GetAvailableGiftCardConfigurationsResponse.AvailableGiftCardConfigurationDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAvailableGiftCardConfigurationsResponse {
    export interface AvailableGiftCardConfigurationDto {
      /**
       * Entity type: GiftCardConfiguration
       */
      ID: number;
      Name: string;
      Handler: string;
      CardType?: string;
      UsageType: GiftCardConfigurationUsageTypes;
      AllowRefundWithoutTransaction: boolean;
      AllowCurrencyConversion: boolean;
    }

  }

  export const enum GiftCardConfigurationUsageTypes {
    None = 0,
    Sale = 1,
    Payment = 2
  }

  /**
   * Gets the initialization state for the EVA environment.
   * Can either way be Created or Initialized.
   * If Created, the environment is not configured and can not be used.
   * If Initialized, the environment is configured and ready to be used.
   */
  export interface GetInitializationState {
  }

  export interface InitializationStateResponse {
    InitializationState: EnvironmentInitializationState;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum EnvironmentInitializationState {
    Created = 0,
    Initialized = 1
  }

  /**
   * Get the CheckoutOptions
   */
  export interface GetCheckoutOptions {
    Filters?: GetCheckoutOptionsFilter;
  }

  export interface GetCheckoutOptionsFilter {
    Name?: string;
    Handler?: string;
    IsActive?: boolean;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export interface GetCheckoutOptionsResponse {
    Options: CheckoutOptionDto[];
    Categories: CheckoutOptionCategoryDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CheckoutOptionDto {
    /**
     * Entity type: CheckoutOption
     */
    ID: number;
    /**
     * Entity type: CheckoutOptionCategory
     */
    CategoryID?: number;
    CategoryName: string;
    CategoryDescription?: string;
    IsActive: boolean;
    Name: string;
    Description?: string;
    Handler: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    OrganizationUnitSetName?: string;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    ScriptName: string;
    Sequence: number;
    ConfigData?: { [key: string]: any };
  }

  export interface CheckoutOptionCategoryDto {
    /**
     * Entity type: CheckoutOptionCategory
     */
    ID: number;
    Name: string;
    Description?: string;
    Sequence: number;
  }

  export interface PageConfig<T = { [key: string]: string }> {
    Filter?: Partial<T>;
    /**
     * @default 0
     * Min: 0
     */
    Start?: number;
    /**
     * @default 50
     * Range: 1-10000
     */
    Limit?: number;
    SortProperty?: string;
    /**
     * @default Ascending
     */
    SortDirection?: SortDirection;
  }

  export const enum SortDirection {
    Ascending = 0,
    Descending = 1
  }

  export interface PagedResult<TModel> {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
    Page?: TModel[];
    Offset?: number;
    Limit?: number;
    Total?: number;
    SortProperty?: string;
    SortDirection?: SortDirection;
    Filters?: { [key: string]: string };
    NumberOfPages?: number;
    CurrentPage?: number;
  }

  export const enum AuditingFieldDataType {
    String = 0,
    Bool = 1,
    Integer = 2,
    Decimal = 3
  }

  export interface ResourceResponseMessage {
    Url: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum AuditingFiscalIDValidationResult {
    Unknown = 0,
    Success = 1,
    Failure = -1
  }

  export interface ListBlobsForDocumentationFilter {
    Category?: string;
    OriginalName?: string;
  }

  export interface ListBlobsForStencilFilter {
    OriginalName?: string;
  }

  export interface ResourceInfo {
    Url: string;
    BlobID: string;
  }

  export const enum FinancialEventTypes {
    Sales = 10,
    SalesTax = 11,
    SalesDiscounts = 12,
    Purchase = 20,
    PurchaseTax = 21,
    PurchasePriceVariance = 22,
    PurchaseInvoiceDispute = 23,
    PurchaseInvoiceDisputeResolved = 24,
    PurchaseDiscounts = 25,
    CostOfGoods = 30,
    Payment = 40,
    PaymentEndRounding = 41,
    PaymentSettlement = 42,
    PaymentCapture = 43,
    PaymentRelease = 44,
    CashAdjustment = 50,
    StockMutation = 60,
    StockMutationAutomaticCorrection = 61,
    StockSold = 62,
    StockReceived = 63,
    Shipment = 70
  }

  export const enum UserTypes {
    None = 0,
    Employee = 1,
    Customer = 2,
    Anonymous = 4,
    Business = 8,
    System = 17,
    Debtor = 64,
    Tester = 512,
    RemovedByRequest = 1024,
    Api = 2048,
    Impersonated = 4096
  }

  export const enum FunctionalityScope {
    None = 0,
    Create = 1,
    Edit = 2,
    Delete = 4,
    View = 8,
    Manage = 31,
    Settings = 32,
    Scripting = 64,
    All = 127
  }

  export const enum ElevationType {
    None = 0,
    Elevation = 1,
    Verification = 2
  }

  export const enum AuditChainTypes {
    EventLedger = 0,
    InvoiceTotals = 1,
    FinancialPeriodTotals = 2,
    MonthlyFinancialReportingPeriodTotals = 3,
    YearlyFinancialReportingPeriodTotals = 4,
    InvoicePrint = 5,
    Invoice = 6,
    FinancialPeriodAudit = 7,
    InvoiceReprint = 8,
    TerminalTotals = 9,
    FinancialPeriod = 10,
    WorkingDocument = 11,
    TaxFreeForm = 12
  }

  export interface TOTPSecretWrapper {
    NextWeek: string;
    CurrentWeek: string;
    WeekNumber: number;
  }

  export interface AddressDataDto {
    FirstName?: string;
    LastName?: string;
    HouseNumber?: string;
    Address1?: string;
    Address2?: string;
    Remark?: string;
    ZipCode?: string;
    Subdistrict?: string;
    District?: string;
    City?: string;
    State?: string;
    Region?: string;
    /**
     * ISO 3166-2
     * Entity type: CountrySubdivision
     */
    CountrySubdivisionID?: string;
    CountrySubdivisionName?: string;
    /**
     * ISO 3166-1
     * Entity type: Country
     */
    CountryID?: string;
    Latitude?: number;
    Longitude?: number;
    PhoneNumber?: string;
    EmailAddress?: string;
  }

  export const enum TaxRateCategory {
    High = 0,
    Low = 1,
    Zero = 2,
    Exempt = 3,
    Intermediate = 4,
    Dynamic = 5
  }

  export const enum ReturnReasonType {
    OrganizationUnit = 0,
    Customer = 1
  }

  export const enum OrderBlobTypes {
    None = 0,
    Invoice = 1,
    InvoiceThermal = 2,
    ReturnLabel = 3,
    ReturnForm = 4,
    ShipmentLabel = 5,
    Identification = 6,
    PackingSlip = 7,
    PickSlip = 8,
    ThermalPrint = 9,
    PinReceipt = 10,
    Export = 11,
    ElectronicReceipt = 12,
    TaxFreeForm = 13,
    Offline = 14,
    AMLSurvey = 15
  }

  export const enum InvoiceLineTypes {
    Default = 0,
    Discount = 1,
    Return = 2
  }

  export const enum InvoiceStatuses {
    Open = 0,
    Closed = 1
  }

  export const enum InvoiceTypes {
    Sales = 0,
    Purchase = 1,
    ControlDocument = 2,
    InitializationDocument = 3,
    ClosingDocument = 4,
    MonthlyControlDocument = 5,
    YearlyControlDocument = 6,
    IntegrityCheckDocument = 7,
    Offer = 8
  }

  export interface CompanyDto {
    /**
     * Entity type: Company
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    RegistrationNumber?: string;
    TaxRegistrationNumber?: string;
    FiscalID?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    RegistrationCountryID?: string;
    RegistrationCountrySubdivisionID?: string;
    TaxRecipientCode?: string;
    TaxRecipientEmail?: string;
    BillingAddress?: AddressDto;
    ShippingAddress?: AddressDto;
    SingleUse?: boolean;
    IsInvoicing?: boolean;
    /**
     * @deprecated Use `TaxRegistrationNumber` instead.
     */
    VatNumber?: string;
    /**
     * @deprecated Use `EmailAddress` instead.
     */
    ContactEmailAddress?: string;
    /**
     * @deprecated Use `PhoneNumber` instead.
     */
    ContactPhoneNumber?: string;
    /**
     * @deprecated Use `EmailAddress` instead.
     */
    InvoiceEmailAddress?: string;
    /**
     * @deprecated The property `RegistrationCity` will be obsolete
     */
    RegistrationCity?: string;
    /**
     * @deprecated The property `VisitorsAddress` will be obsolete
     */
    VisitorsAddress?: AddressDto;
    /**
     * @deprecated Use `BillingAddress` instead
     */
    InvoiceAddress?: AddressDto;
    /**
     * @deprecated The property `RegistrationAddress` will be obsolete
     */
    RegistrationAddress?: AddressDto;
    /**
     * @deprecated The property `EstablishedDate` will be obsolete
     */
    EstablishedDate?: string;
    /**
     * @deprecated The property `AccountHolderName` will be obsolete
     */
    AccountHolderName?: string;
    /**
     * @deprecated The property `IBAN` will be obsolete
     */
    IBAN?: string;
    /**
     * @deprecated The property `BIC` will be obsolete
     */
    BIC?: string;
    /**
     * @deprecated The property `LogoID` will be obsolete
     */
    LogoID?: string;
  }

  export interface AddressDto {
    /**
     * Entity type: Address
     */
    ID: number;
    FirstName?: string;
    LastName?: string;
    HouseNumber?: string;
    Address1?: string;
    Address2?: string;
    Remark?: string;
    ZipCode?: string;
    Subdistrict?: string;
    District?: string;
    City?: string;
    State?: string;
    Region?: string;
    /**
     * ISO 3166-2
     * Entity type: CountrySubdivision
     */
    CountrySubdivisionID?: string;
    CountrySubdivisionName?: string;
    /**
     * ISO 3166-1
     * Entity type: Country
     */
    CountryID?: string;
    Latitude?: number;
    Longitude?: number;
    PhoneNumber?: string;
    EmailAddress?: string;
  }

  export const enum InvoiceDocumentLedgerTypes {
    Physical = 0,
    Digital = 1
  }

  export interface InvoiceDocumentLedgerData {
    PaperProperties: PaperProperties;
  }

  export interface PaperProperties {
    WaitForNetworkIdle: boolean;
    WaitForJS: boolean;
    Size: PaperProperties.PaperSize;
    Format?: PaperFormats;
    Orientation?: PaperOrientations;
    Margin: PaperProperties.PaperMargin;
    ThermalPrinterTemplateType?: ThermalPrinterTemplateTypes;
  }

  export namespace PaperProperties {
    export interface PaperMargin {
      Top?: number;
      Left?: number;
      Bottom?: number;
      Right?: number;
    }

    export interface PaperSize {
      Width: string;
      Height: string;
      DeviceScaleFactor?: number;
    }

  }

  export const enum PaperFormats {
    A3 = 1,
    A4 = 2,
    A5 = 3,
    Legal = 4,
    Letter = 5,
    Tabloid = 6,
    Auto = 7,
    Ledger = 8,
    A0 = 9,
    A1 = 10,
    A2 = 11,
    A6 = 12
  }

  export const enum PaperOrientations {
    Portrait = 1,
    Landscape = 2
  }

  export const enum ThermalPrinterTemplateTypes {
    Xml = 0,
    Html = 1
  }

  export const enum InvoiceDocumentLedgerFormats {
    Receipt = 0,
    Invoice = 1
  }

  export const enum InvoiceCalculationMethod {
    InTax = 0,
    ExTax = 1,
    InTaxNew = 2,
    ExTaxNew = 3
  }

  export const enum TaskState {
    Scheduled = 1,
    Running = 2,
    Completed = 3
  }

  export interface WatchtowerLog {
    Date: string;
    Message: string;
  }

  export const enum OrganizationUnitStatus {
    Default = 0,
    Open = 1,
    Closed = 2,
    OpeningSoon = 4,
    TemporarilyClosed = 8,
    Sale = 16,
    Hidden = 32
  }

  /**
   * Generate a barcode image.
   * 
   * The `Format` is one of;
   * 
   * - QR = 0
   * - EAN = 1
   * - Code39 = 2
   * - Code128 = 3
   * 
   * :lock: This is the safer variant to our `/barcode` endpoint, because the data moves _inside_ the POST body instead of through GET query params
   * :bulb: You can use the returned `Data` in combination with `MimeType` to render the image directly, for example;
   * 
   * ```
   *   <img src='data:{MimeType};base64,{Data}' />
   * ```
   */
  export interface Barcode {
    /**
     * Range: 10-1000
     */
    Width?: number;
    /**
     * Range: 10-1000
     */
    Height?: number;
    Code?: string;
    Format?: Barcode.BarcodeFormat;
    BlobID?: string;
    NoMargin?: boolean;
    NoText?: boolean;
    BackgroundHex?: string;
    ForegroundHex?: string;
  }

  export namespace Barcode {
    export const enum BarcodeFormat {
      QR = 0,
      EAN = 1,
      Code39 = 2,
      Code128 = 3
    }

  }

  export interface BarcodeResponse {
    MimeType: string;
    Data: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for sending a contact form to the email set in settings called `ContactForm:MailTo`
   */
  export interface SendContactForm {
    Type?: string;
    Properties?: { [key: string]: string };
  }

  /**
   * Creates an API key for specified user
   */
  export interface CreateApiKey {
    /**
     * Requires a user of type API, unless `ApiKeyForNonAPIUsers` setting is enabled
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * To request a permanent key, set this to true. The key will not expire.
     */
    RequestPermanentApiKey?: boolean;
    /**
     * When set, the key will be locked to the organization unit, so the key can't be used to access different OrganizationUnits
     */
    LockApiKeyToOrganizationUnit?: boolean;
    /**
     * Strict API keys will not automatically identify as employees, and so have restricted access to only the functionalities assigned to it
     */
    Strict?: boolean;
    Description?: string;
    /**
     * Will default to setting `ApiKeyExpirationInDays` when null and `RequestPermanentApiKey` is false
     */
    ExpirationDate?: string;
  }

  export interface CreateApiKeyResponse {
    /**
     * Entity type: ApiKey
     */
    ID: number;
    ApiKey: string;
    ExpirationDate?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an exisiting API key
   * 
   * All other properties need a new key to be created as these would influence the token authorization.
   */
  export interface UpdateApiKey {
    /**
     * Entity type: ApiKey
     */
    ID: number;
    Description?: string;
  }

  /**
   * List API keys, optionally filtered.
   */
  export interface ListApiKeys {
    InitialPageConfig?: PageTokenConfig<ListApiKeysFilter>;
    PageToken?: string;
  }

  export interface ListApiKeysFilter {
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  export interface ListApiKeysResponse {
    Results?: ListKeysItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListKeysItem {
    /**
     * Entity type: ApiKey
     */
    ID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    UserName: string;
    /**
     * Entity type: User
     */
    CreatedByUserID: number;
    CreatedByUserName: string;
    IsActive: boolean;
    ExpirationDate?: string;
    CreationTime: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName: string;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    Description: string;
    IsLockedToOrganizationUnit: boolean;
    IsStrict: boolean;
    LastAccessTime?: string;
  }

  /**
   * Deletes an API key
   */
  export interface DeleteApiKey {
    /**
     * Entity type: ApiKey
     */
    KeyID: number;
  }

  /**
   * Attaches an user to a consumer company.
   */
  export interface AttachUserToConsumerCompany {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: Company
     */
    CompanyID: number;
  }

  /**
   * Change the password of a user.
   * 
   * When invalid, the service will return a 400 BAD REQUEST;
   * 
   * - SecurityPolicy:UserOrPasswordInvalid
   * - SecurityPolicy:TooShort
   * - SecurityPolicy:TooLong
   * - SecurityPolicy.NewPasswordIsRequired
   * - SecurityPolicy.Pwned
   * - General
   */
  export interface ChangeUserPassword {
    NewPassword: string;
    OldPassword: string;
  }

  /**
   * Checks if the given EmailAddress is available to create an account for.
   * 
   * When creating Employee accounts, the `AsEmployee` property should be set to true.
   */
  export interface CheckEmailAddressAvailability {
    EmailAddress: string;
    AsEmployee?: boolean;
  }

  export interface CheckUsernameAvailabilityResponse {
    /**
     * Indicates if the username is available.
     */
    IsAvailable: boolean;
    /**
     * Indicates that the username is not available, but only exists in the external user provider.
     */
    IsExternal: boolean;
    /**
     * Indicates if the username is valid.
     */
    IsValid: boolean;
    /**
     * Indicates if a password reset is required.
     */
    PasswordResetRequired: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Checks if the given Nickname is available to create an account for.
   * 
   * When creating Employee accounts, the `AsEmployee` property should be set to true.
   */
  export interface CheckNicknameAvailability {
    Nickname: string;
    AsEmployee?: boolean;
  }

  /**
   * Creates a API user
   */
  export interface CreateApiUser {
    User: CreateApiUser.ApiUser;
  }

  export namespace CreateApiUser {
    export interface ApiUser {
      Name: string;
      Nickname: string;
    }

  }

  export interface CreateApiUserResponse {
    Result: CreateApiUserResults;
    /**
     * Entity type: User
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum CreateApiUserResults {
    CreatedApiUser = 0,
    NicknameInUse = 1
  }

  /**
   * Creates a customer.
   * 
   * This is allowed when you are an employee, or by anonymous if the CustomerCanRegister (default true) is enabled.
   * 
   * Be sure to handle errors for registering, which could be (amongst others);
   * 
   *   * InvalidEmailAddress
   *   * EmailAddressInUse
   *   * NicknameInUse
   *   * FiscalIDInUse
   */
  export interface CreateCustomer {
    User: CreateCustomer.Customer;
    /**
     * Is this a Standard/Basic/Incognito account?
     * @default Standard
     */
    AccountType?: UserAccountType;
    /**
     * Set to `true` to automatically authenticate as this new user.
     * @default false
     */
    AutoLogin?: boolean;
    /**
     * Automatically attach the new customer to the order identified by `OrderID`.
     * @default true
     */
    AttachToOrder?: boolean;
    /**
     * Identifier for the order to automatically attach the new customer to (see `AttachToOrder` parameter).
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Optional associations to other users, e.g. child, parent, neighbour, friend, colleague, ...
     */
    Associations?: CreateCustomer.UserAssociations;
  }

  export namespace CreateCustomer {
    export interface Customer {
      /**
       * Ignored if the user calling this service is not an employee
       */
      BackendID?: string;
      EmailAddress?: string;
      Gender?: string;
      Initials?: string;
      FirstName?: string;
      LastName?: string;
      PhoneNumber?: string;
      DateOfBirth?: string;
      PlaceOfBirth?: string;
      BankAccount?: string;
      Nickname?: string;
      Password?: string;
      /**
       * Entity type: Language
       */
      LanguageID?: string;
      /**
       * Entity type: Country
       */
      CountryID?: string;
      FiscalID?: string;
      SocialSecurityNumber?: string;
      /**
       * Entity type: UserOrigin
       */
      OriginID?: number;
      Title?: string;
      Salutation?: string;
      TimeZone?: string;
      ShippingAddress?: AddressDataDto;
      ShippingAddressDescription?: string;
      BillingAddress?: AddressDataDto;
      BillingAddressDescription?: string;
      Company?: CreateCustomer.CustomerCompany;
      /**
       * Entity type: OrganizationUnit
       */
      PreferredStoreID?: number;
      CustomID?: string;
      BackendRelationID?: string;
      /**
       * Optional additional custom fields
       */
      CustomFields?: CustomFieldKeyValue[];
      /**
       * Can only be set when used from an Employee or API type user, will be ignored otherwise
       */
      PricingGroupID?: string;
      IsSingleSignOnOnly?: boolean;
    }

    export interface CustomerCompany {
      /**
       * Entity type: Company
       */
      ID?: number;
      Name?: string;
      RegistrationNumber?: string;
      VatNumber?: string;
      FiscalID?: string;
    }

    export interface UserAssociations {
      /**
       * The new user is something *to* existing users, e.g. it is a parent to X
       */
      PrimaryAssociations?: UserAssociations.UserAssociation[];
      /**
       * The new user is something *of* existing users, e.g. it is the child of Y
       */
      RelatedAssociations?: UserAssociations.UserAssociation[];
    }

    export namespace UserAssociations {
      export interface UserAssociation {
        /**
         * Entity type: User
         */
        UserID: number;
        /**
         * Entity type: UserAssociationType
         */
        TypeID: number;
      }

    }

  }

  export interface CustomFieldKeyValue {
    /**
     * Identifier for the custom field
     * Entity type: CustomField
     */
    CustomFieldID: number;
    /**
     * Used for datatype String, Text and Enum
     */
    StringValue?: string;
    /**
     * Used for datatype Blob
     */
    BlobValue?: string;
    /**
     * Used for datatype Integer and Decimal
     */
    NumberValue?: number;
    /**
     * Used for datatype Bool
     */
    BoolValue?: boolean;
    /**
     * Used for datatype DateTime and Date
     */
    DateTimeValue?: string;
    /**
     * Used for multiple values
     */
    ArrayValues?: CustomFieldValue[];
  }

  export interface CustomFieldValue {
    /**
     * Used for datatype String, Text and Enum
     */
    StringValue?: string;
    /**
     * Used for datatype Blob
     */
    BlobValue?: string;
    /**
     * Used for datatype Integer and Decimal
     */
    NumberValue?: number;
    /**
     * Used for datatype Bool
     */
    BoolValue?: boolean;
    /**
     * Used for datatype DateTime and Date
     */
    DateTimeValue?: string;
    /**
     * Used for multiple values
     */
    ArrayValues?: CustomFieldValue[];
  }

  export const enum UserAccountType {
    Standard = 0,
    Basic = 1,
    Incognito = 2
  }

  export interface CreateCustomerResponse {
    User: LoggedInUserDto;
    Barcode: string;
    Result: CreateCustomerResults;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface LoggedInUserDto {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs: number[];
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupIDs: number[];
    /**
     * Entity type: OrganizationUnit
     */
    CurrentOrganizationID?: number;
    CurrentOrganizationName?: string;
    /**
     * Entity type: Currency
     */
    CurrentCurrencyID: string;
    AuthenticationToken: string;
    RefreshToken?: string;
    RefreshTokenExpireDate?: string;
    Functionalities: Record<'None' | 'Elevation' | 'Verification', { [key: string]: FunctionalityScope }>;
    CurrentOrganizationUnitType: OrganizationUnitTypes;
    /**
     * Entity type: Country
     */
    CurrentCountryID: string;
    /**
     * Entity type: Language
     */
    CurrentLanguageID: string;
    /**
     * Entity type: Assortment
     */
    CurrentAssortmentID: number;
    /**
     * @deprecated Replaced by `Functionalities`
     */
    ScopedFunctionalities: { [key: string]: FunctionalityScope };
    /**
     * @deprecated Replaced by `Functionalities`
     */
    ElevationFunctionalities: { [key: string]: FunctionalityScope };
    /**
     * Entity type: User
     */
    ID: number;
    BackendID: string;
    CustomID: string;
    EmployeeNumber: string;
    BackendRelationID: string;
    EmailAddress: string;
    EmailAddressValidated: boolean;
    Initials: string;
    FirstName: string;
    LastName: string;
    FullName: string;
    DateOfBirth?: string;
    PlaceOfBirth: string;
    Gender: string;
    Type: UserTypes;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    IsDeleted: boolean;
    AccountType: UserAccountType;
    SingleSignOnOnly: boolean;
    AllowPasswordLogin: boolean;
    Nickname: string;
    Company: CompanyDto;
    BankAccount: string;
    IsDebtor: boolean;
    PhoneNumber: string;
    GravatarHash: string;
    Data: { [key: string]: string };
    TimeZone: string;
    FiscalID: string;
    SocialSecurityNumber: string;
    Title: string;
    Salutation: string;
    CreationTime: string;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreatedByFullName: string;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions: { [key: number]: CustomFieldValueWithOptions };
    Barcode: string;
    LastSuccessfulLogin?: string;
  }

  export const enum OrganizationUnitTypes {
    None = 0,
    /**
     * A shop represents a physical store where products can be sold.
     */
    Shop = 1,
    /**
     * A WebShop represents an online channel that allows delivery and reservation orders, but no carry out sales.
     */
    WebShop = 2,
    /**
     * A container is an OrganizationUnit purely used to group some other OrganizationUnits, to allow easier configuration.
     */
    Container = 4,
    /**
     * Pickup can be combined with type Shop to allow reservation orders in the store.
     */
    Pickup = 8,
    /**
     * A warehouse represents an OrganizationUnit where delivery orders can be shipped. The stock of these organizationunits can be made available for delivery orders from (web)shops.
     */
    Warehouse = 16,
    /**
     * A Country is a special case of the Container type that represents a Country division in the OrganizationUnits structure.
     */
    Country = 36,
    /**
     * A shop can be flagged as franchiser to allow some special flows.
     */
    Franchise = 64,
    /**
     * The type EVA indicates that the shop is running EVA in the store. This will trigger Tasks etc that will not be generated for Shops that are not (yet) converted to running EVA. P/a non-EVA stores will receive an email for pickupordrs instead of a StockReservationTask.
     */
    EVA = 128,
    /**
     * TestOrganizationUnit can be used to test some things in a production environment. This is not advised :warning:. These stores will be excluded from a lot of processes.
     */
    TestOrganizationUnit = 256,
    /**
     * OrganizationUnits with DisableLogin cannot be selected in the Login process.
     */
    DisableLogin = 512,
    /**
     * @deprecated The Supplier naming was unclear, renamed to ExternalSupplier to better convey the meaning.
     * Suppliers are used to replenish warehouses or to export BackOrder/MTO-Products. These OrganizationUnits can have their own set of SupplierProducts, prices and stock.
     */
    Supplier = 1024,
    /**
     * An external supplier is an organization that is not part of your internal organization structure but that you would still like to have available in EVA to for example create purchase Orders for to replenish your warehouse or stores.
     */
    ExternalSupplier = 1024,
    /**
     * Some suppliers deliver their stock in consignment.
     */
    Consignment = 3072,
    /**
     * For Business-to-business orders this type can be set. Orders in these organizationunits will be ex-tax.
     */
    B2b = 4096,
    /**
     * A Region is a special case of the Container type that represents a subdivision under Country OrganizationUnits.
     */
    Region = 8196,
    /**
     * An OrganizationUnit that is meant to be used by customers for returning Orders.
     */
    ReturnsPortal = 16384,
    /**
     * An OrganizationUnit that is meant to be invoiced for restitutions.
     */
    Restitution = 32768,
    /**
     * An OrganizationUnit that also does repairs.
     */
    RepairCenter = 65536,
    /**
     * An OrganizationUnit that is mobile. This indicates that it might have a different address for every financial period.
     */
    Mobile = 131072,
    /**
     * An OrganizationUnit representing a Store-In-Store.
     */
    ConcessionStore = 262144
  }

  export interface CustomFieldResponse {
    Options: CustomFieldOptions;
    Value?: any;
    IsEditableByUser: boolean;
    /**
     * Entity type: CustomField
     */
    CustomFieldID: number;
    Name: string;
    DisplayName?: string;
    DataType: string;
    DataTypeID: number;
    BackendID?: string;
    IsArray: boolean;
    EnumValues?: { [key: string]: string };
    Order: number;
  }

  export interface CustomFieldOptions {
    IsRequired?: boolean;
    MinimumValue?: number;
    MaximumValue?: number;
    MinimumLength?: number;
    MaximumLength?: number;
    MinimumDate?: string;
    MaximumDate?: string;
    DefaultCustomFieldValue?: CustomFieldValue;
    IsEditable?: boolean;
    DefaultValue?: string;
  }

  export interface CustomFieldValueWithOptions {
    CustomFieldBackendID?: string;
    Value: CustomFieldValue;
    Options: CustomFieldOptions;
  }

  export const enum CreateCustomerResults {
    CreatedCustomer = 0,
    InvalidEmail = 1,
    EmailAlreadyInUse = 3,
    NicknameAlreadyInUse = 4,
    PhoneNumberAlreadyInUse = 5,
    AutoLoginFailed = 6,
    FiscalIDAlreadyInUse = 7
  }

  /**
   * Delete a User by it's ID.
   */
  export interface DeleteUser {
    /**
     * Entity type: User
     */
    ID: number;
  }

  /**
   * Delete multiple users by ID's.
   * This service is async only and only accepts 1000 ID's per request
   */
  export interface DeleteUsers {
    /**
     * Entity type: User
     */
    IDs: number[];
  }

  export interface DeleteUsersResponse {
    /**
     * Entity type: User
     */
    FailedUserIDs?: number[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Detaches an user from a consumer company.
   */
  export interface DetachUserFromConsumerCompany {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: Company
     */
    CompanyID: number;
  }

  /**
   * Creates a temporary, numeric password for specified user, overwriting his/her current password, intended for employees to be able to help resetting a user password.
   */
  export interface GenerateTemporaryPasswordForUser {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  export interface GenerateTemporaryPasswordForUserResponse {
    Password: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Return identification info for the current employee.
   */
  export interface GetCurrentUserIdentificationInfo {
  }

  export interface GetCurrentUserIdentificationInfoResponse {
    HasPin: boolean;
    ResetDate?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Return the current user for the given Authorization header
   */
  export interface GetCurrentUser {
  }

  export interface GetCurrentUserResponse {
    User: LoggedInUserDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This service returns a list of the most likely email address domains (so 'gmail.com', 'hotmail.com', etc) based on
   * your current context.
   */
  export interface GetEmailAddressDomainSuggestions {
    /**
     * The number of suggestions to return. Defaults to 5 and is limited to 10.
     * Range: 1-10
     */
    MaxSuggestions?: number;
  }

  export interface GetEmailAddressDomainSuggestionsResponse {
    /**
     * A list of the domain names, ordered by usage count. Each domain is returned in the form of `domain.tld`.
     */
    Domains: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns all organization units where this user can login
   */
  export interface GetLoginOrganizationUnitsForUser {
    /**
     * Filter by login type
     */
    UserTypeFilter?: UserTypes;
  }

  export interface GetLoginOrganizationUnitsForUserResponse {
    Result: GetLoginOrganizationUnitsForUserResponse.OrganizationUnit[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetLoginOrganizationUnitsForUserResponse {
    export interface OrganizationUnit {
      ID: number;
      Name: string;
    }

  }

  /**
   * @deprecated Use `GetLoginOrganizationUnitsForUser`
   */
  export interface GetOrganizationUnitsForUser {
    IncludeNonLoginOrganizationUnits?: boolean;
    UserTypeFilter?: UserTypes;
  }

  export interface GetOrganizationUnitsForUserResponse {
    Result: OrganizationUnitDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrganizationUnitDto {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    BackendID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    ParentID?: number;
    Name: string;
    Description?: string;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    /**
     * Entity type: Address
     */
    AddressID?: number;
    Address?: AddressDto;
    EmailAddress?: string;
    PhoneNumber?: string;
    Type: OrganizationUnitTypes;
    Status: OrganizationUnitStatus;
    OpeningHours: OpeningHoursDto[];
    ExtendedOpeningHours: ExtendedOpeningHours[];
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    Latitude?: number;
    Longitude?: number;
    /**
     * Entity type: Country
     */
    CountryID: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: Currency
     */
    CostPriceCurrencyID: string;
    Subnet?: string;
    Website?: string;
    BIC?: string;
    EstablishmentNumber?: string;
  }

  export interface OpeningHoursDto {
    /**
     * Entity type: OpeningHours
     */
    ID: number;
    /**
     * Entity type: OpeningHoursType
     */
    TypeID?: number;
    TypeName?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Only set when returned outside OrganizationUnit context
     */
    OrganizationUnitName?: string;
    DayOfWeek?: DayOfWeek;
    StartTime?: string;
    EndTime?: string;
    Date?: string;
    Description?: string;
    IsSpecial?: boolean;
    IsClosed?: boolean;
    IsActive?: boolean;
  }

  export const enum DayOfWeek {
    Sunday = 0,
    Monday = 1,
    Tuesday = 2,
    Wednesday = 3,
    Thursday = 4,
    Friday = 5,
    Saturday = 6
  }

  export interface ExtendedOpeningHours {
    /**
     * Entity type: OpeningHoursType
     */
    TypeID: number;
    TypeName: string;
    TypeBackendID?: string;
    Sunday: DayOpeningHours;
    Monday: DayOpeningHours;
    Tuesday: DayOpeningHours;
    Wednesday: DayOpeningHours;
    Thursday: DayOpeningHours;
    Friday: DayOpeningHours;
    Saturday: DayOpeningHours;
    SpecialOpeningHours: SpecialOpeningHours[];
  }

  export interface DayOpeningHours {
    Description?: string;
    IsClosed: boolean;
    TimeSlots?: TimeSlot[];
  }

  export interface TimeSlot {
    StartTime: string;
    EndTime: string;
  }

  export interface SpecialOpeningHours {
    Date: string;
    Description?: string;
    IsClosed: boolean;
    TimeSlots?: TimeSlot[];
  }

  /**
   * Retrieve information about a temporary access token from specified `Authorization` parameter.
   */
  export interface GetTokenInfo {
    Authorization: string;
  }

  export interface GetTokenInfoResponse {
    /**
     * Entity type: User
     */
    UserID: number;
    ExpirationDate?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    TemporaryAccessType: string;
    TemporaryAccessData?: { [key: string]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Return the current user associations for the given user
   */
  export interface GetUserAssociations {
    /**
     * Entity type: User
     */
    ID?: number;
  }

  export interface GetUserAssociationsResponse {
    PrimaryRelations: SingleUserAssociation[];
    RelatedRelations: SingleUserAssociation[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface SingleUserAssociation {
    User: UserDataForSingleUserAssociation;
    Type: AssociationTypeForSingleUserAssociation;
  }

  export interface UserDataForSingleUserAssociation {
    /**
     * Entity type: User
     */
    ID: number;
    FullName: string;
  }

  export interface AssociationTypeForSingleUserAssociation {
    /**
     * Entity type: UserAssociationType
     */
    ID: number;
    Name: string;
  }

  /**
   * Get a user by ID
   */
  export interface GetUser {
    /**
     * Entity type: User
     */
    ID?: number;
    AdditionalDataOptions?: GetUser.AdditionalUserDataOptions;
  }

  export namespace GetUser {
    export interface AdditionalUserDataOptions {
      IncludeLoyaltyPrograms: boolean;
      IncludeSubscriptions: boolean;
      IncludeCoupons: boolean;
      IncludeInquiries: boolean;
    }

  }

  export interface GetUserResponse {
    /**
     * Entity type: User
     */
    ID: number;
    BackendID: string;
    BackendSystemID: string;
    BackendRelationID: string;
    EmailAddress: string;
    EmailAddressValidated: boolean;
    Initials: string;
    FirstName: string;
    LastName: string;
    FullName: string;
    DateOfBirth?: string;
    PlaceOfBirth: string;
    Gender: string;
    Company: CompanyDto;
    Type: UserTypes;
    LanguageID: string;
    CountryID: string;
    IsIncognito: boolean;
    IsDeactivated: boolean;
    DeactivationDate?: string;
    LastSuccessfulLogin?: string;
    Nickname: string;
    BankAccount: string;
    IsDebtor: boolean;
    PhoneNumber: string;
    GravatarHash: string;
    Data: { [key: string]: string };
    TimeZone: string;
    FiscalID: string;
    SocialSecurityNumber: string;
    Title: string;
    Salutation: string;
    /**
     * Entity type: OrganizationUnit
     */
    PreferredStoreID?: number;
    PreferredStore?: GetUserResponse.OrganizationUnitDto;
    OriginatingOrganizationUnit?: GetUserResponse.OrganizationUnitDto;
    CustomID: string;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreatedByFullName: string;
    CreationTime: string;
    /**
     * Entity type: User
     */
    LastModifiedByID?: number;
    ModifiedByFullName?: string;
    LastModificationTime?: string;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    PricingGroupID?: string;
    AllowPasswordLogin: boolean;
    SingleSignOnOnly: boolean;
    AccountType: UserAccountType;
    HasOpenCases: boolean;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID: number;
    VisibilityGroupName: string;
    SingleSignOnID?: string;
    HasCompanies: boolean;
    AdditionalData?: GetUserResponse.AdditionalUserData;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetUserResponse {
    export interface AdditionalUserData {
      Coupons?: GetUserResponse.Coupon[];
      LoyaltySubscriptions?: UserSubscriptionDto[];
      Subscriptions?: UserSubscriptionDto[];
      Inquiries?: UserInquiryDto[];
    }

    export interface Coupon {
      /**
       * Entity type: DiscountCoupon
       */
      ID: number;
      Code: string;
      /**
       * Entity type: Discount
       */
      DiscountID: number;
      DiscountDescription: string;
      DiscountMarketingDescription: string;
      ValidFrom: string;
      ValidTill?: string;
    }

    export interface OrganizationUnitDto {
      /**
       * Entity type: OrganizationUnit
       */
      ID?: number;
      BackendID?: string;
      Name?: string;
    }

  }

  export interface GetUserLegacyResult {
    /**
     * Entity type: User
     */
    ID: number;
    BackendID: string;
    BackendSystemID: string;
    BackendRelationID: string;
    EmailAddress: string;
    EmailAddressValidated: boolean;
    Initials: string;
    FirstName: string;
    LastName: string;
    FullName: string;
    DateOfBirth?: string;
    PlaceOfBirth: string;
    Gender: string;
    Company: CompanyDto;
    Type: UserTypes;
    LanguageID: string;
    CountryID: string;
    IsDeleted: boolean;
    IsIncognito: boolean;
    IsDeactivated: boolean;
    DeactivationDate?: string;
    LastSuccessfulLogin?: string;
    Nickname: string;
    BankAccount: string;
    IsDebtor: boolean;
    PhoneNumber: string;
    GravatarHash: string;
    Data: { [key: string]: string };
    TimeZone: string;
    FiscalID: string;
    SocialSecurityNumber: string;
    Title: string;
    Salutation: string;
    /**
     * Entity type: OrganizationUnit
     */
    PreferredStoreID?: number;
    CustomID: string;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreatedByFullName: string;
    CreationTime: string;
    /**
     * Entity type: User
     */
    LastModifiedByID?: number;
    ModifiedByFullName?: string;
    LastModificationTime?: string;
    AllowPasswordLogin: boolean;
  }

  export interface UserSubscriptionDto {
    Status?: SubscriptionStatus;
    UserAgent?: string;
    SubscribeFromSourceOrganizationUnit?: UserSubscriptionDto.OrganizationUnitDto;
    SubscribeFromSourceUserAgent?: string;
    SubscribeFromSourceIdentifier?: string;
    UnsubscribeFromSourceOrganizationUnit?: UserSubscriptionDto.OrganizationUnitDto;
    UnsubscribeFromSourceUserAgent?: string;
    UnsubscribeFromSourceIdentifier?: string;
    UnsubscribeFromSourceReason?: string;
    IsBlocked?: boolean;
    UserIdentifier?: string;
    IsUnsubscribeAllowed?: boolean;
    CreationTime?: string;
    LastModificationTime?: string;
    CreatedBy?: SubscriptionSimpleUserDto;
    LastModifiedBy?: SubscriptionSimpleUserDto;
    Subscription?: SubscriptionDto;
  }

  export namespace UserSubscriptionDto {
    export interface OrganizationUnitDto {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
      BackendID?: string;
    }

  }

  export const enum SubscriptionStatus {
    None = 0,
    Requested = 1,
    Subscribed = 2,
    Unsubscribed = 3
  }

  export interface SubscriptionSimpleUserDto {
    /**
     * Entity type: User
     */
    ID: number;
    FirstName: string;
    LastName: string;
    FullName: string;
  }

  export interface SubscriptionDto {
    /**
     * Entity type: Subscription
     */
    ID?: number;
    /**
     * Entity type: LoyaltyProgramGroup
     */
    LoyaltyProgramGroupID?: number;
    LoyaltyProgramGroupName?: string;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    LoyaltyProgramName?: string;
    LoyaltyProgramOptions?: LoyaltyProgramOptions;
    LoyaltyProgramPointUsageOptions?: LoyaltyPointUsageOptions;
    LoyaltyBalance?: UserLoyaltyPointBalance;
    BackendID?: string;
    Name?: string;
    Description?: string;
    ConfirmationRequired?: boolean;
    Default?: boolean;
    Hidden?: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    OriginatingOrganizationUnitID?: number;
    Preferred?: boolean;
    UserIdentifierRequiredForSubscription?: boolean;
    HasCustomFieldIdentifier?: boolean;
    Inquiry?: InquiryDto;
    Priority?: number;
    IsExternal?: boolean;
    ResubscribeHandling?: ResubscribeHandlingType;
    AllowUnsubscribeBy?: AllowUnsubscribeByType;
  }

  export const enum LoyaltyProgramOptions {
    None = 0,
    EnablePaymentMethods = 1,
    EnableLoyaltyPoints = 2,
    EnableDiscounts = 4
  }

  export const enum LoyaltyPointUsageOptions {
    None = 0,
    AllowAwarding = 1,
    UsableAsPaymentMethod = 2,
    UsableForWithdraw = 4
  }

  export interface UserLoyaltyPointBalance {
    Points?: number;
    PointRegistrations?: UserLoyaltyPointBalance.Mutations[];
    PointRegistrationsPreviousPageToken?: string;
    PointRegistrationsNextPageToken?: string;
    CurrencyID: string;
    CurrencyValue: number;
  }

  export namespace UserLoyaltyPointBalance {
    export interface Mutations {
      Type: UserLoyaltyMutationType;
      Points: number;
      PendingTill?: string;
      ExpireAfter?: string;
      CreationTime: string;
    }

  }

  export const enum UserLoyaltyMutationType {
    Deposit = 0,
    Revoke = 1,
    Withdraw = 2,
    Refund = 3,
    Rebalance = 4
  }

  export interface InquiryDto {
    /**
     * Entity type: Inquiry
     */
    ID: number;
    Name: string;
    OneTimeUseOnly: boolean;
    CompletionTime?: string;
  }

  export const enum ResubscribeHandlingType {
    UpdateUserIdentifier = 0,
    KeepOriginalUserIdentifier = 1,
    Block = 2
  }

  export const enum AllowUnsubscribeByType {
    None = 0,
    Subscriber = 1,
    Employee = 2,
    All = 3
  }

  export interface UserInquiryDto {
    /**
     * Entity type: Inquiry
     */
    ID: number;
    /**
     * Entity type: Subscription
     */
    SubscriptionID?: number;
    Name: string;
    Description?: string;
    SubmittedOn?: string;
    PromptUser: boolean;
  }

  export interface ListAvailableTimeZones {
  }

  export interface ListAvailableTimeZonesResponse {
    Result: ListAvailableTimeZonesResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListAvailableTimeZonesResponse {
    export interface Model {
      ID: string;
      CountryID: string;
      CountryName: string;
      Offset: number;
    }

  }

  export interface ListUserApiKeys {
    /**
     * Entity type: User
     */
    UserID: number;
    PageConfig?: PageConfig<ListApiKeysFilter>;
  }

  export interface ListUserApiKeysResponse {
    Result?: PagedResult<ListApiKeysItem>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListApiKeysItem {
    /**
     * Entity type: ApiKey
     */
    ID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    Description?: string;
    ExpirationDate?: string;
  }

  /**
   * Creates a UserPhoneNumber for the current user, or the UserID specified if you're allowed to do so.
   */
  export interface CreateUserPhoneNumber {
    /**
     * Entity type: User
     */
    UserID?: number;
    PhoneNumber: string;
    Type: PhoneNumberTypes;
    Description?: string;
    IsPrimary?: boolean;
  }

  export const enum PhoneNumberTypes {
    Unspecified = 0,
    Home = 1,
    Mobile = 2,
    Work = 3,
    Fax = 4,
    Other = 999
  }

  export interface CreateUserPhoneNumberResponse {
    /**
     * Entity type: UserPhoneNumber
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates a UserPhoneNumber based on its ID. This service allows you to change the PhoneNumber (required), Type, Description.
   *     If you want to change what number is the primary number, use the `MakeUserPhoneNumberPrimary` service.
   * 
   * This service only allows the number's own user or an employee to update it.
   */
  export interface UpdateUserPhoneNumber {
    /**
     * Entity type: UserPhoneNumber
     */
    ID: number;
    PhoneNumber: string;
    Type: PhoneNumberTypes;
    Description?: string;
  }

  /**
   * Get the requested user's phone numbers. If UserID is left null, this service will return the phone numbers for your current user.
   * Only employee users are allowed to access the phone numbers of other users.
   */
  export interface GetUserPhoneNumbers {
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  export interface GetUserPhoneNumbersResponse {
    Result: UserPhoneNumberModel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UserPhoneNumberModel {
    /**
     * Entity type: UserPhoneNumber
     */
    ID: number;
    PhoneNumber: string;
    Description: string;
    Type: PhoneNumberTypes;
    IsPrimary: boolean;
  }

  /**
   * Delete a user's phone number based on the provided ID. If you're trying to delete a user's primary phone number and that user has
   * more than two phone numbers currently, this service will return a `CannotDeletePrimaryPhoneNumber` error,
   * because it cannot determine what number should now become the user's primary phone number. If the user has two numbers, the other number
   * will become the primary.
   * 
   * This service only allows the number's own user or an employee to delete it.
   */
  export interface DeleteUserPhoneNumber {
    /**
     * Entity type: UserPhoneNumber
     */
    ID: number;
  }

  /**
   * This service marks the provided phone number as the user's primary phone number,
   * which will be used for communication with the user.
   * 
   * This service only allows the number's own user or an employee to mark it as the primary number.
   */
  export interface MakeUserPhoneNumberPrimary {
    /**
     * Entity type: UserPhoneNumber
     */
    ID: number;
  }

  /**
   * Used to create or update a user from 3rd party system.
   */
  export interface PushUser {
    /**
     * ID of the EVA user
     * Entity type: User
     */
    ID?: number;
    /**
     * Code of the 3rd party system that wants to create this user
     */
    BackendSystemID?: string;
    /**
     * ID of the user in the 3rd party system
     */
    BackendID?: string;
    CustomID?: string;
    /**
     * Defaults to Customer
     */
    UserType?: UserTypes;
    /**
     * Defaults to Standard
     */
    AccountType?: UserAccountType;
    EmailAddress?: string;
    Nickname?: string;
    Initials?: string;
    FirstName?: string;
    LastName?: string;
    PhoneNumber?: string;
    DateOfBirth?: string;
    BackendRelationID?: string;
    FiscalID?: string;
    SocialSecurityNumber?: string;
    Title?: string;
    Salutation?: string;
    SingleSignOnID?: string;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    PreferredStoreID?: number;
    /**
     * Sets the originating organization unit when creating a customer. Ignored on updates.
     * Entity type: OrganizationUnit
     */
    OriginatingOrganizationUnitID?: number;
    /**
     * Gender of the user, either M or F
     */
    Gender?: string;
    /**
     * Language code of the user (ISO 639-1)
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Country code of the user (ISO 3166-1)
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Company of the user
     * @deprecated Use `Companies` instead.
     */
    CompanyID?: string;
    /**
     * Companies the user is attached to
     */
    Companies?: string[];
    ShippingAddress?: AddressDataDto;
    BillingAddress?: AddressDataDto;
    /**
     * With this property the whole address book can be mutated. When used ShippingAddress and BillingAddress will be ignored.
     */
    AddressBook?: PushUser.AddressBookData;
    /**
     * NOTE THAT Only subscriptions that allow for local storage can be pushed. Subscriptions fully depending on external services cannot be pushed.
     * NOTE THAT Balance can only be provided for loyalty subscriptions that have a EVA loyalty handler.
     */
    Subscriptions?: PushUser.UserSubscription[];
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    PricingGroupID?: string;
    IsSingleSignOnOnly?: boolean;
    WishlistData?: WishlistData;
  }

  export namespace PushUser {
    export interface AddressBookData {
      /**
       * If this is set to true, all existing addresses will be removed from the addressbook
       */
      ReplaceAll: boolean;
      Addresses: PushUser.AddressBookItem[];
    }

    export interface AddressBookItem {
      /**
       * Entity type: AddressBook
       */
      ID?: number;
      BackendID?: string;
      Description?: string;
      Address?: AddressDataDto;
      UseAsDefaultShippingAddress: boolean;
      UseAsDefaultBillingAddress: boolean;
      UseAsDefaultPickupAddress: boolean;
    }

    export interface UserSubscription {
      /**
       * Entity type: Subscription
       */
      SubscriptionID?: number;
      Subscribed?: boolean;
      /**
       * Identifier for this user for the given Subscription. P/a: a loyaltyID
       */
      UserIdentifier?: string;
      /**
       * Identifier for the source of subscription.
       */
      SourceIdentifier?: string;
      /**
       * Reason for unsubscribing.
       */
      UnsubscribeReason?: string;
      Balance?: number;
      SubscribedOn?: string;
    }

    export interface UserFieldValues {
      /**
       * Entity type: UserField
       */
      UserFieldID?: number;
      Value?: any;
    }

  }

  export interface WishlistData {
    ReplaceAll: boolean;
    Wishlists: WishlistData.WishlistItem[];
  }

  export namespace WishlistData {
    export interface WishlistItem {
      BackendID?: string;
      BackendSystemID: string;
      Name: string;
      Description: string;
      /**
       * Entity type: Product
       */
      ProductIDs: number[];
    }

  }

  export interface PushUserResponse {
    /**
     * Entity type: User
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Use this to push multiple users at once. This is more efficient than pushing them one by one. Please get in touch with your contact at New Black if you plan on pushing a lot of users. This service follows the "all-or-nothing" approach, either everything is imported, or nothing is imported. There is a limit (100) on the number of users that can be pushed at once.
   */
  export interface PushUsers {
    /**
     * Enabling this will try to suppress as many eventexports as possible that might result from this call.
     */
    DisableEventExports?: boolean;
    /**
     * Will ignore any faulty users. Everything that looks right is imported and a 200 OK is always returned.
     */
    IgnoreErrors?: boolean;
    Users: PushUser[];
  }

  export interface PushUsersResponse {
    Success: boolean;
    /**
     * Will be empty if the import was not successful
     */
    ValidUsers: ValidUser[];
    InvalidUsers: InvalidUser[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ValidUser {
    /**
     * Entity type: User
     */
    ID: number;
    BackendID: string;
    Addresses: ValidUser.Address[];
  }

  export namespace ValidUser {
    export interface Address {
      /**
       * Entity type: AddressBook
       */
      AddressBookID: number;
      AddressBookBackendID: string;
    }

  }

  export interface InvalidUser {
    BackendID: string;
    Reason: string;
  }

  /**
   * Reactivates an User that was deactivated due to no login
   */
  export interface ReactivateUser {
    /**
     * Entity type: User
     */
    ID: number;
  }

  /**
   * Recovers an employee account by changing the email address and optionally generating a temporary password.
   *   This is intended as last resort for when an employee has lost access to their emailaddress.
   */
  export interface RecoverEmployeeAccount {
    /**
     * Entity type: User
     */
    UserID: number;
    NewEmailAddress?: string;
    GenerateTemporaryPassword?: boolean;
  }

  export interface RecoverEmployeeAccountResponse {
    Password: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface RequestPasswordResetToken {
    EmailAddress: string;
    AsEmployee?: boolean;
    /**
     * Some identifier to use in the mailtemplate (admin, checkout, website, ...)
     */
    Context?: string;
  }

  export interface ResetUserPassword {
    Token: string;
    NewPassword: string;
  }

  export interface ResetUserPasswordResponse {
    EmailAddress: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Execute a search through the users
   * 
   * Filtering is possible on all kinds of properties. By default the search results will be restricted to users in the current country.
   */
  export interface SearchUsers {
    IncludeCustomers?: boolean;
    IncludeEmployees?: boolean;
    /**
     * By default the search will only return users from the same Country as the current OrganizationUnit.
     * Optionally an extensive search can be performed by enabling this option. But keep in mind this will take some time in real production scenario's.
     * It is advised to only show this option in the frontend after an intial search has been done in the current country.
     */
    IncludeAllCountries?: boolean;
    /**
     * Entity type: User
     */
    UserIDs?: number[];
    EmailAddress?: string;
    Name?: string;
    SearchQuery?: string;
    Types?: UserTypes;
    ZipCode?: string;
    PhoneNumber?: string;
    EmployeeNumber?: string;
    /**
     * Entity type: OrganizationUnit
     */
    PrimaryOrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    AssignedEmployeeID?: number;
    VatNumber?: string;
    SocialSecurityNumber?: string;
    DateOfBirth?: string;
    FiscalID?: string;
    /**
     * Entity type: Role
     */
    RoleIDs?: number[];
    PricingGroupID?: string;
    Data?: { [key: string]: string };
    /**
     * Entity type: CustomField
     */
    CustomFields?: { [key: number]: string };
    /**
     * When source is set to External the search will be performed on the external user provider.
     */
    Source?: SearchUsers.SearchUserSources;
    /**
     * Entity type: Company
     */
    CompanyIDs?: number[];
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export namespace SearchUsers {
    export const enum SearchUserSources {
      Internal = 1,
      External = 2
    }

  }

  export interface SearchUsersResponse {
    Result?: PagedResult<SearchUsersResponse.UserSearchDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace SearchUsersResponse {
    export interface UserSearchDto {
      /**
       * Entity type: User
       */
      ID?: number;
      CompanyName?: string;
      BackendID?: string;
      BackendSystemID?: string;
      BackendRelationID?: string;
      EmailAddress?: string;
      FirstName?: string;
      LastName?: string;
      FullName?: string;
      DateOfBirth?: string;
      ShippingAddress?: AddressDataDto;
      BillingAddress?: AddressDataDto;
      Type?: UserTypes;
      AccountType?: UserAccountType;
      IsDeactivated?: boolean;
      DeactivationDate?: string;
      LastSuccessfulLogin?: string;
      PhoneNumber?: string;
      GravatarHash?: string;
      Data?: { [key: string]: string };
      CustomFields?: { [key: string]: string };
      EmployeeNumber?: string;
      /**
       * Entity type: OrganizationUnit
       */
      PrimaryOrganizationUnitID?: number;
      PrimaryOrganizationUnitName?: string;
    }

  }

  /**
   * Update user custom data
   */
  export interface SetCustomUserData {
    /**
     * Entity type: User
     */
    ID: number;
    CustomFields?: CustomFieldKeyValue[];
  }

  /**
   * Update a user login to single sign on only
   */
  export interface SetUserToSingleSignOnOnly {
    /**
     * Entity type: User
     */
    ID: number;
  }

  /**
   * This service is used to change the AccountType of a Customer.
   * 
   * When changing the AccountType from Standard to Incognito or Basic, all credentials and roles are removed.
   * 
   * When changing the AccountType from Incognito or Basic to Standard:
   * - Uniqueness checks are done for the EmailAddress and Nickname
   * - Default customer roles are generated
   * - When no password is provided, one is generated and returned in the response
   * - If configured, the user will receive an `welcome to EVA` email. If no valid password is provided, the email will also contain a generated password.
   */
  export interface UpdateUserAccountType {
    /**
     * Entity type: User
     */
    UserID?: number;
    AccountType: UserAccountType;
    Password?: string;
  }

  export interface UpdateUserAccountTypeResponse {
    Password?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update properties on an existing User. Will throw an exception if one of the properties are invalid.
   * 
   * Properties not specified in the request will be ignored.
   */
  export interface UpdateUser {
    /**
     * Entity type: User
     */
    ID: number;
    BackendRelationID?: string;
    BackendID?: string;
    BackendSystemID?: string;
    Initials?: string;
    FirstName?: string;
    LastName?: string;
    DateOfBirth?: string;
    PlaceOfBirth?: string;
    Gender?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Nickname?: string;
    EmailAddress?: string;
    BankAccount?: string;
    PhoneNumber?: string;
    TimeZone?: string;
    FiscalID?: string;
    SocialSecurityNumber?: string;
    Title?: string;
    Salutation?: string;
    /**
     * Entity type: OrganizationUnit
     */
    PreferredStoreID?: number;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    PricingGroupID?: string;
  }

  /**
   * This service is used to change the VisibilityGroup of a User.
   * 
   * To change the VisibilityGroup of a User, the following conditions must be met:
   * - The User must be visable and editable by the current User
   * - The VisibilityGroup must be visible for Users by the current User, the service GetUsersVisibilityGroups can be used to get a list of all available VisibilityGroups for the current User.
   * - If the User is a standard account, there must be no duplicate standard account in the new VisibilityGroup.
   */
  export interface UpdateUserVisibilityGroup {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID: number;
  }

  /**
   * Get all visibility groups that the current user can see.
   */
  export interface GetAvailableVisibilityGroups {
  }

  export interface GetAvailableVisibilityGroupsResponse {
    UserVisibilityGroups: VisibilityGroupModel[];
    OrganizationUnitVisibilityGroups: VisibilityGroupModel[];
    OrderVisibilityGroups: VisibilityGroupModel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface VisibilityGroupModel {
    /**
     * Entity type: VisibilityGroup
     */
    ID: number;
    Name: string;
    Description: string;
    AllowCustomerStandardAccounts: boolean;
  }

  export interface CreateUserAssociation {
    /**
     * Entity type: User
     */
    PrimaryUserID: number;
    /**
     * Entity type: User
     */
    RelatedUserID: number;
    /**
     * Entity type: UserAssociationType
     */
    TypeID: number;
  }

  export interface CreateUserAssociationResponse {
    /**
     * Entity type: UserAssociation
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateUserAssociation {
    /**
     * Entity type: UserAssociation
     */
    ID: number;
    /**
     * Entity type: User
     */
    PrimaryUserID?: number;
    /**
     * Entity type: User
     */
    RelatedUserID?: number;
    /**
     * Entity type: UserAssociationType
     */
    TypeID?: number;
  }

  export interface DeleteUserAssociation {
    /**
     * Entity type: UserAssociation
     */
    ID: number;
  }

  export interface CreateUserAssociationType {
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  export interface CreateUserAssociationTypeResponse {
    /**
     * Entity type: UserAssociationType
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateUserAssociationType {
    /**
     * Entity type: UserAssociationType
     */
    ID: number;
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  export interface DeleteUserAssociationType {
    /**
     * Entity type: UserAssociationType
     */
    ID: number;
  }

  export interface GetUserAssociationTypeByID {
    /**
     * Entity type: UserAssociationType
     */
    ID: number;
  }

  export interface GetUserAssociationTypeByIDResponse {
    UserAssociationType: UserAssociationTypeModel;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UserAssociationTypeModel {
    /**
     * Entity type: UserAssociationType
     */
    ID: number;
    Name: string;
    Description: string;
    BackendID: string;
    IsDeleted: boolean;
  }

  export interface GetUserAssociationTypes {
  }

  export interface GetUserAssociationTypesResponse {
    UserAssociationTypes: UserAssociationTypeModel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get the user Consignment property.
   */
  export interface GetUserConsignment {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  export interface GetUserConsignmentResponse {
    UseConsignment: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Set the user Consignment property.
   */
  export interface SetUserConsignment {
    /**
     * Entity type: User
     */
    UserID: number;
    UseConsignment?: boolean;
  }

  export interface GetUserDebtorData {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  export interface GetUserDebtorDataResponse {
    PaymentTerms: string;
    CreditLimit?: number;
    ColloPrice?: number;
    PalletPrice?: number;
    OversizedPrice?: number;
    AssignedToUser: AssignedToUser;
    IsTaxExempt: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface AssignedToUser {
    /**
     * Entity type: User
     */
    ID: number;
    FullName: string;
    EmailAddress: string;
  }

  export interface UpdateUserDebtorData {
    /**
     * Entity type: User
     */
    UserID: number;
    PaymentTerms?: string;
    CreditLimit?: number;
    /**
     * Entity type: User
     */
    AssignedToUserID?: number;
    ColloPrice?: number;
    PalletPrice?: number;
    OversizedPrice?: number;
    IsTaxExempt?: boolean;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface GetUserGroupByID {
    /**
     * Entity type: UserGroup
     */
    ID: number;
  }

  export interface GetUserGroupByIDResponse {
    Name: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface CreateUserGroup {
    Name: string;
  }

  export interface CreateUserGroupResponse {
    /**
     * Entity type: UserGroup
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface UpdateUserGroup {
    /**
     * Entity type: UserGroup
     */
    ID: number;
    Name: string;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface DeleteUserGroup {
    /**
     * Entity type: UserGroup
     */
    ID: number;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListUserGroups {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListUserGroupsResponse {
    Result?: PagedResult<UserGroupDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UserGroupDto {
    /**
     * Entity type: User
     */
    ID: number;
    Name: string;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface AddUserToGroup {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: UserGroup
     */
    GroupID: number;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface RemoveUserFromGroup {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: UserGroup
     */
    GroupID: number;
  }

  /**
   * @deprecated Feature not supported anymore.
   */
  export interface ListUsersForUserGroup {
    /**
     * Entity type: UserGroup
     */
    UserGroupID: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListUsersForUserGroupResponse {
    Result?: PagedResult<SimpleUserDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface SimpleUserDto {
    /**
     * Entity type: User
     */
    ID: number;
    FirstName: string;
    LastName: string;
  }

  export interface ValidateUser {
    /**
     * Entity type: User
     */
    UserID?: number;
    IsEmployee?: boolean;
    FirstName?: string;
    Initials?: string;
    LastName?: string;
    Nickname?: string;
    EmailAddress?: string;
    Gender?: string;
    PhoneNumber?: string;
    DateOfBirth?: string;
    LanguageID?: string;
    CountryID?: string;
    ShowValidDetails?: boolean;
  }

  export interface ValidateUserResponse {
    IsValid: boolean;
    ValidationDetails: ValidateUserResponse.ValidationDetail[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ValidateUserResponse {
    export interface ValidationDetail {
      Field: string;
      Message: string;
      IsValid: boolean;
    }

  }

  /**
   * Share a wishlist.
   */
  export interface ShareWishlist {
    /**
     * Description to identify the share with
     */
    Description: string;
    /**
     * Entity type: Wishlist
     */
    WishlistID: number;
    CanEdit?: boolean;
    /**
     * Emailadresses to send the share token to. Can provide a maximum of 50 email adresses.
     */
    EmailAddresses?: string[];
  }

  export interface ShareWishlistResponse {
    Token: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Revoke the wishlist share.
   */
  export interface RevokeWishlistShare {
    /**
     * Entity type: Wishlist
     */
    WishlistID: number;
  }

  /**
   * Get the shared wishlist by token. Requires the share token.
   */
  export interface GetSharedWishlist {
    Token: string;
  }

  export interface GetSharedWishlistResponse {
    Name: string;
    Products: any[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Links the shared wishlist to the user. Requires the share token. When linked the wishlist is returned in the users overview.
   */
  export interface LinkSharedWishlist {
    Token: string;
  }

  /**
   * Unlinks the shared wishlist from the user. Will also remove it from the users overview. Shared wishlist can still be retrieved by the token.
   */
  export interface UnlinkSharedWishlist {
    /**
     * Entity type: Wishlist
     */
    WishlistID: number;
  }

  /**
   * Get an anonymous wishlist by token.
   */
  export interface GetAnonymousWishlist {
    Token: string;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
  }

  export interface GetAnonymousWishlistResponse {
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    Products: any[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create an anonymous wishlist.
   */
  export interface CreateAnonymousWishlist {
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
  }

  export interface CreateAnonymousWishlistResponse {
    Token: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an anonymous wishlist.
   */
  export interface UpdateAnonymousWishlist {
    Token: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
  }

  /**
   * Deletes an anonymous wishlist.
   */
  export interface DeleteAnonymousWishlist {
    Token: string;
  }

  /**
   * Add product(s) to an anonymous wishlist.
   */
  export interface AddProductsToAnonymousWishlist {
    Token: string;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
  }

  /**
   * Removes product(s) from an anonymous wishlist.
   */
  export interface RemoveProductsFromAnonymousWishlist {
    Token: string;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
  }

  /**
   * If the wishlist is created by anonymous, but the user logs in afterwards, this service can be used to attach the anonymous wishlist to the logged in user.
   * The service requires a unique name.
   */
  export interface AttachAnonymousWishlistToUser {
    Token: string;
    Name: string;
    Description?: string;
  }

  /**
   * List the users wishlists.
   */
  export interface ListUserWishlists {
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
    InitialPageConfig?: PageTokenConfig<ListUserWishlistsFilter>;
    PageToken?: string;
  }

  export interface ListUserWishlistsFilter {
    Name?: string;
    /**
     * Entity type: Product
     */
    ProductID?: number;
  }

  export interface ListUserWishlistsResponse {
    Results?: ListUserWishlistsResponse.Wishlist[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListUserWishlistsResponse {
    export interface Wishlist {
      /**
       * Entity type: Wishlist
       */
      ID: number;
      Name: string;
      Description: string;
      FirstProduct?: any;
      ItemCount: number;
      SharedByMe: boolean;
      SharedWithMe: boolean;
      CanEdit: boolean;
    }

  }

  /**
   * Get the user wishlist by ID.
   */
  export interface GetUserWishlist {
    /**
     * Entity type: Wishlist
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
  }

  export interface GetUserWishlistResponse {
    /**
     * Entity type: Wishlist
     */
    ID: number;
    Name: string;
    Description: string;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    FirstProduct?: any;
    Products?: any[];
    SharedByMe: boolean;
    SharedWithMe: boolean;
    CanEdit: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates a user wishlist.
   */
  export interface CreateUserWishlist {
    /**
     * Entity type: User
     */
    UserID?: number;
    Name: string;
    Description?: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    /**
     * When `HideForUser` is set to true, only employees can see this wishlist. The name must be unique within this context, but a user can potentially create a wishlist with the same name.
     */
    HideFromUser?: boolean;
  }

  export interface CreateUserWishlistResponse {
    /**
     * Entity type: Wishlist
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update the user wishlist.
   */
  export interface UpdateUserWishlist {
    /**
     * Entity type: Wishlist
     */
    ID: number;
    Name?: string;
    Description?: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    /**
     * When `HideForUser` is set to true, only employees can see this wishlist. The name must be unique within this context, but a user can potentially create a wishlist with the same name.
     */
    HideFromUser?: boolean;
  }

  /**
   * Deletes the user wishlist.
   */
  export interface DeleteUserWishlist {
    /**
     * Entity type: Wishlist
     */
    ID: number;
  }

  /**
   * Add product(s) to the user wishlist.
   */
  export interface AddProductsToUserWishlist {
    /**
     * Entity type: Wishlist
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
  }

  /**
   * Removes product(s) from the user wishlist.
   */
  export interface RemoveProductsFromUserWishlist {
    /**
     * Entity type: Wishlist
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
  }

  /**
   * Fetch user information from an external API based on the social security number. This can be used to enrich the user in the frontend.
   * Depending on the SSN provider, this may require Customer or Employee rights.
   */
  export interface GetUserInfoBySocialSecurityNumber {
    /**
     * Entity type: Country
     */
    CountryID: string;
    SocialSecurityNumber: string;
  }

  export interface GetUserInfoResponse {
    Data: GetUserInfoResponse.UserData;
    Address: GetUserInfoResponse.UserAddress;
    ExistingUsers: GetUserInfoResponse.ExistingUser[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetUserInfoResponse {
    export interface UserData {
      ID: number;
      SocialSecurityNumber: string;
      FirstName: string;
      LastName: string;
      DateOfBirth?: string;
      PhoneNumber: string;
      Company: GetUserInfoResponse.UserCompany;
      BillingAddress: GetUserInfoResponse.UserAddress;
    }

    export interface UserCompany {
      Name: string;
    }

    export interface UserAddress {
      AddressedTo: string;
      Street: string;
      HouseNumber: string;
      ZipCode: string;
      City: string;
      CountryID: string;
    }

    export interface ExistingUser {
      ID: number;
      FirstName: string;
      LastName: string;
      DateOfBirth?: string;
    }

  }

  /**
   * Fetch user information from an external API based on the phone number. This can be used to enrich the user in the frontend.
   * Depending on the phone number provider, this may require Customer or Employee rights.
   */
  export interface GetUserInfoByPhoneNumber {
    /**
     * Entity type: Country
     */
    CountryID: string;
    PhoneNumber: string;
  }

  /**
   * Find an User by their SocialSecurityNumber. This service only returns an UserID.
   * When more info is needed, additional data can be fetched with the `GetUser` service.
   */
  export interface GetUserBySocialSecurityNumber {
    SocialSecurityNumber: string;
    AsEmployee?: boolean;
  }

  export interface GetUserBySocialSecurityNumberResponse {
    UserID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List the user loyalty transactions.
   */
  export interface ListUserLoyaltyTransactions {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    InitialPageConfig?: PageTokenConfig<ListUserLoyaltyMutationFilter>;
    PageToken?: string;
  }

  export interface ListUserLoyaltyMutationFilter {
    Type?: UserLoyaltyMutationType;
  }

  export interface ListUserLoyaltyTransactionsResponse {
    Results?: ListUserLoyaltyTransactionsResponse.LoyaltyTransaction[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListUserLoyaltyTransactionsResponse {
    export interface LoyaltyTransaction {
      Type: UserLoyaltyMutationType;
      Points: number;
      PendingTill?: string;
      ExpireAfter?: string;
      CreationTime: string;
    }

  }

  /**
   * Get the user loyalty balance.
   */
  export interface GetUserLoyaltyBalance {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  export interface GetUserLoyaltyBalanceResponse {
    LoyaltyProgramID: number;
    LoyaltyProgramName: string;
    CurrentBalance: number;
    IsExternalProvider: boolean;
    /**
     * @deprecated Use service ListUserLoyaltyTransactions instead.
     */
    Mutations?: GetUserLoyaltyBalanceResponse.Mutation[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetUserLoyaltyBalanceResponse {
    export interface Mutation {
      Type: UserLoyaltyMutationType;
      Points: number;
      PendingTill?: string;
      ExpireAfter?: string;
      CreationTime: string;
    }

  }

  /**
   * Confirm a Subscription with a Token
   * 
   * The service returns the status of the subscription
   */
  export interface ConfirmSubscription {
    Token: string;
  }

  export interface ConfirmSubscriptionResponse {
    /**
     * Entity type: Subscription
     */
    ID: number;
    Status: SubscriptionStatus;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get all available subscriptions
   */
  export interface GetAvailableSubscriptions {
  }

  export interface GetAvailableSubscriptionsResponse {
    Subscriptions: GetAvailableSubscriptionsResponse.SubscriptionDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAvailableSubscriptionsResponse {
    export interface SubscriptionDto {
      /**
       * Entity type: Subscription
       */
      ID?: number;
      /**
       * Entity type: LoyaltyProgram
       */
      LoyaltyProgramID?: number;
      BackendID?: string;
      Name?: string;
      Description?: string;
      ConfirmationRequired?: boolean;
      SubscriptionConfirmation?: SubscriptionConfirmation;
      Default?: boolean;
      Hidden?: boolean;
      Preferred?: boolean;
      UserIdentifierRequiredForSubscription?: boolean;
      HasCustomFieldIdentifier?: boolean;
      /**
       * Entity type: Inquiry
       */
      InquiryID?: number;
      InquiryName?: string;
      Priority?: number;
      IsExternal?: boolean;
    }

  }

  export const enum SubscriptionConfirmation {
    None = 0,
    Email = 1,
    Webhook = 2
  }

  /**
   * Get all User subscriptions, this takes into account the OU this request is done on and will only show subscriptions available in that OU.
   */
  export interface GetUserSubscriptions {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Setting this to true will also include the subscriptions in the response that are available, but where the user has no Subscription status
     */
    IncludeAvailableSubscriptions?: boolean;
    /**
     * Setting this to true will also include the subscriptions in the response that are not available to subscribe the user to anymore, such as those that are already unsubscribed and resubscription is blocked.
     */
    IncludeHiddenSubscriptions?: boolean;
    /**
     * Setting this will filter subscriptions based on who can unsubscribe from it.
     */
    AllowUnsubscribeBy?: AllowUnsubscribeByType;
    /**
     * Entity type: User
     */
    LoyaltyProgramGroupID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    Name?: string;
    LoyaltyProgramOptions?: LoyaltyProgramOptions;
  }

  export interface GetUserSubscriptionsResponse {
    UserAgent: string;
    Subscriptions: UserSubscriptionDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get all user loyalty subscriptions, this takes into account the OU this request is done on and will only show subscriptions available in that OU.
   */
  export interface GetUserLoyaltySubscriptions {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Setting this to true will also include the subscriptions in the response that are available, but where the user has no Subscription status
     */
    IncludeAvailableSubscriptions?: boolean;
    /**
     * Setting this to true will also include the subscriptions in the response that are not available to subscribe the user to anymore, such as those that are already unsubscribed and resubscription is blocked.
     */
    IncludeHiddenSubscriptions?: boolean;
    /**
     * Setting this will filter subscriptions based on who can unsubscribe from it.
     */
    AllowUnsubscribeBy?: AllowUnsubscribeByType;
    /**
     * Entity type: LoyaltyProgramGroup
     */
    LoyaltyProgramGroupID?: number;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    Name?: string;
    LoyaltyProgramOptions?: LoyaltyProgramOptions;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface GetUserLoyaltySubscriptionsResponse {
    Result?: PagedResult<UserSubscriptionDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get all User subscriptions for all OUs.
   */
  export interface GetAllSubscriptionsByUserID {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  /**
   * Subscribe an User to a Subscription
   * 
   * The service returns the status of the subscription
   */
  export interface SubscribeUser {
    /**
     * Entity type: User
     */
    UserID?: number;
    Subscriptions: SubscribeUser.SubscriptionWithIdentifier[];
  }

  export namespace SubscribeUser {
    export interface SubscriptionWithIdentifier {
      /**
       * Entity type: Subscription
       */
      SubscriptionID?: number;
      /**
       * Entity type: LoyaltyProgram
       */
      LoyaltyProgramID?: number;
      /**
       * Identifier for this user for the given Subscription. P/a: UserSubscriptionIdentifier or a LoyaltyID
       */
      UserIdentifier?: string;
      /**
       * Identifier for the source the user is being subscribed by.
       */
      SourceIdentifier?: string;
      /**
       * If the user is already subscribed, but the UserIdentifier is different, the UserIdentifier will be updated.
       * NOTE THAT the old UserIdentifier will be overwritten and when the given UserIdentifier is already used, subscribing the user will fail.
       */
      UpdateUserIdentifierWhenChanged?: boolean;
    }

  }

  export interface SubscribeUserResponse {
    Subscriptions?: SubscribeUserResponse.SubscriptionWithStatus[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace SubscribeUserResponse {
    export interface SubscriptionWithStatus {
      /**
       * Entity type: UserSubscription
       */
      ID?: number;
      /**
       * Entity type: Subscription
       */
      SubscriptionID?: number;
      /**
       * Entity type: LoyaltyProgram
       */
      LoyaltyProgramID?: number;
      /**
       * Identifier for this user for the given Subscription. P/a: UserSubscriptionIdentifier or a LoyaltyID
       */
      UserIdentifier?: string;
      Status?: SubscriptionStatus;
    }

  }

  /**
   * Unsubscribes an User from a Subscription
   * 
   * The service returns the status of the subscription
   */
  export interface UnsubscribeUser {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Subscription
     */
    SubscriptionID?: number;
    /**
     * Identifier for the source the user is being unsubscribed by.
     */
    SourceIdentifier?: string;
    /**
     * Reason for unsubscribing.
     */
    UnsubscribeReason?: string;
  }

  export interface UnsubscribeUserResponse {
    Status: SubscriptionStatus;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update the user identifier on a user subscription
   */
  export interface UpdateSubscriptionUserIdentifier {
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: Subscription
     */
    SubscriptionID: number;
    UserIdentifier: string;
  }

  /**
   * Fetches the user by its BackendID from the configured external provider. This service is usually used in combination with a call to SearchUsers. This service will create a user in EVA based on the details returned by the external provider if we don't already have a user with the same BackendID. Passing in the OrderID will attach the (created) user to the given order directly as well.
   */
  export interface FetchUserFromExternalProvider {
    /**
     * BackendID that is returned from the `SearchUsers` with the source set to external
     */
    BackendID: string;
  }

  export interface FetchUserFromExternalProviderResponse {
    /**
     * Entity type: User
     */
    ID?: number;
    BackendID?: string;
    BackendSystemID?: string;
    BackendRelationID?: string;
    EmailAddress?: string;
    EmailAddressValidated?: boolean;
    Initials?: string;
    FirstName?: string;
    LastName?: string;
    FullName?: string;
    DateOfBirth?: string;
    PlaceOfBirth?: string;
    Gender?: string;
    Company?: CompanyDto;
    Type?: UserTypes;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    IsDeactivated?: boolean;
    DeactivationDate?: string;
    LastSuccessfulLogin?: string;
    Nickname?: string;
    BankAccount?: string;
    IsDebtor?: boolean;
    PhoneNumber?: string;
    TimeZone?: string;
    FiscalID?: string;
    SocialSecurityNumber?: string;
    Title?: string;
    Salutation?: string;
    /**
     * Entity type: OrganizationUnit
     */
    PreferredStoreID?: number;
    CustomID?: string;
    /**
     * Entity type: User
     */
    CreatedByID?: number;
    CreatedByFullName?: string;
    CreationTime?: string;
    /**
     * Entity type: User
     */
    LastModifiedByID?: number;
    ModifiedByFullName?: string;
    LastModificationTime?: string;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    PricingGroupID?: string;
    AllowPasswordLogin?: boolean;
    SingleSignOnOnly?: boolean;
    AccountType?: UserAccountType;
    HasOpenCases?: boolean;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    VisibilityGroupName?: string;
    SingleSignOnID?: string;
    ShippingAddress?: AddressDataDto;
    BillingAddress?: AddressDataDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Sends an email validation code to your current user's email address.
   */
  export interface ValidateUserEmailAddress {
  }

  /**
   * Confirms an email address validation by entering the code that was sent through `ValidateUserEmailAddress`.
   */
  export interface ConfirmUserEmailAddressValidation {
    Code: string;
  }

  /**
   * Create and attach a new company to an User
   * 
   * Only Name is required.
   */
  export interface CreateCompanyForUser {
    /**
     * Entity type: User
     */
    UserID?: number;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCity?: string;
    /**
     * Entity type: Country
     */
    RegistrationCountryID?: string;
    VatNumber?: string;
    FiscalID?: string;
    EstablishedDate?: string;
    ContactEmailAddress?: string;
    ContactPhoneNumber?: string;
    AccountHolderName?: string;
    IBAN?: string;
    BIC?: string;
  }

  export interface DetachCompanyFromUser {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  /**
   * Return the Company attached to the given User
   */
  export interface GetCompanyForUser {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  export interface GetCompanyForUserResponse {
    Result: CompanyDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates the current Company on specified `UserID`, or the current user when omitted.
   * 
   * :no_entry: Requires employee rights when using a `UserID` different than the current user.
   * :warning: Will overwrite *all* company fields, those fields omitted in your request will be removed! Except for FiscalID, that will be kept if not provided.
   */
  export interface UpdateCompanyForUser {
    /**
     * Entity type: User
     */
    UserID?: number;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCity?: string;
    RegistrationCountryID?: string;
    VatNumber?: string;
    FiscalID?: string;
    EstablishedDate?: string;
    ContactEmailAddress?: string;
    ContactPhoneNumber?: string;
    AccountHolderName?: string;
    IBAN?: string;
    BIC?: string;
    VisitorsAddress?: AddressDataDto;
    InvoiceAddress?: AddressDataDto;
  }

  /**
   * List borrowed products based on the given filter
   */
  export interface ListUserBorrowedProducts {
    PageConfig?: PageConfig<ListUserBorrowedProductsFilter>;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
  }

  export interface ListUserBorrowedProductsFilter {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    States?: BorrowedState[];
  }

  export const enum BorrowedState {
    Open = 0,
    Returned = 1,
    Purchased = 2
  }

  export interface ListUserBorrowedProductsResponse {
    UserProductOpenQuantities: ListUserBorrowedProductsResponse.UserProductQuantity[];
    Result?: PagedResult<ListUserBorrowedProductsResponse.Model>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListUserBorrowedProductsResponse {
    export interface Model {
      /**
       * Entity type: User
       */
      UserID: number;
      UserFullName: string;
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductName: string;
      ProductBackendID: string;
      Product: any;
      State: BorrowedState;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      CreatedByUser: ListUserBorrowedProductsResponse.UserModel;
      CreationTime: string;
      ModifiedByUser: ListUserBorrowedProductsResponse.UserModel;
      ModificationTime?: string;
    }

    export interface UserModel {
      /**
       * Entity type: User
       */
      ID: number;
      FullName: string;
    }

    export interface UserProductQuantity {
      /**
       * Entity type: User
       */
      UserID: number;
      /**
       * Entity type: Product
       */
      ProductID: number;
      Quantity: number;
    }

  }

  /**
   * Register user borrowed products
   */
  export interface RegisterUserBorrowedProducts {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Min: 0
     */
    Quantity?: number;
    SerialNumber?: string;
  }

  /**
   * Return user borrowed products. When IsDamaged is set to true, the stock will be moved to Damaged. Otherwise, the stock will be moved to Sellable.
   */
  export interface ReturnUserBorrowedProducts {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Min: 0
     */
    Quantity?: number;
    IsDamaged?: boolean;
    SerialNumber?: string;
  }

  /**
   * Add user borrowed products to order. When no quantity is given, all borrowed products with the given product ID are added to the order.
   */
  export interface AddUserBorrowedProductToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Min: 0
     */
    Quantity?: number;
  }

  export interface AddUserBorrowedProductToOrderResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Created OrderLineIDs
     * Entity type: OrderLine
     */
    OrderLineIDs: number[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Render a UserInteraction of Type Communication in something that can be displayed.
   * 
   * This can be used to view the send emails in a webview/iframe.
   */
  export interface PreviewUserInteractionCommunication {
    /**
     * Entity type: Interaction
     */
    ID: number;
  }

  export interface PreviewUserInteractionCommunicationResponse {
    Content?: string;
    ToName?: string;
    ToAddress?: string;
    FromName?: string;
    FromAddress?: string;
    Attachments?: PreviewUserInteractionCommunicationResponse.Attachment[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace PreviewUserInteractionCommunicationResponse {
    export interface Attachment {
      ID: string;
      Name: string;
      MediaType: string;
    }

  }

  /**
   * Prepare an attachment of an UserInteraction to be downloaded, will return an url that is valid for 5min.
   */
  export interface DownloadUserInteractionCommunicationAttachment {
    /**
     * Entity type: Interaction
     */
    ID: number;
    AttachmentID: string;
  }

  export interface ResourceWithBlobIDResponseMessage {
    ResourceInfo: ResourceInfo;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Resend a UserInteraction of Type Communication
   * 
   * When no EmailAddress is supplied, the original recipient will be used.
   */
  export interface ResendUserInteractionCommunication {
    /**
     * Entity type: Interaction
     */
    ID?: number;
    EmailAddress?: string;
  }

  /**
   * Get a UserInteration by its ID. A UserInteraction logs an interaction between an employee and a customer, for example the fact the customer was called.
   */
  export interface GetUserInteractionByID {
    /**
     * Entity type: Interaction
     */
    ID: number;
  }

  export interface GetUserInteractionByIDResponse {
    /**
     * Entity type: Interaction
     */
    ID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    User: GetUserInteractionByIDResponse.UserDto;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreatedBy: GetUserInteractionByIDResponse.UserDto;
    /**
     * Entity type: User
     */
    LastModifiedByID?: number;
    LastModifiedBy: GetUserInteractionByIDResponse.UserDto;
    Text?: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    InteractionType: InteractionType;
    Subject?: string;
    CreationTime: string;
    /**
     * Entity type: Interaction
     */
    OriginalUserInteractionID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetUserInteractionByIDResponse {
    export interface UserDto {
      FirstName: string;
      LastName: string;
      EmailAddress: string;
      FullName: string;
    }

  }

  export const enum UserInteractionTypes {
    /**
     * @deprecated Use InteractionType.PrivateNote instead
     */
    Manual = 0,
    /**
     * @deprecated Use InteractionType.Email instead
     */
    Communication = 1
  }

  export const enum InteractionType {
    /**
     * @deprecated Use InteractionType.PublicNote instead
     */
    Communication = 0,
    PublicNote = 0,
    /**
     * @deprecated Use InteractionType.PrivateNote instead
     */
    Note = 1,
    PrivateNote = 1,
    Ledger = 2,
    Email = 3
  }

  /**
   * Creates a UserInteraction. A UserInteraction logs an interaction between an employee and a customer, for example the fact the customer was called.
   * 
   * UserID is optional but only if OrderID is also provided and the given Order is attached to a customer. If so then the UserInteraction is attached to the Order.Customer.
   */
  export interface CreateUserInteraction {
    /**
     * Entity type: User
     */
    UserID?: number;
    Text: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
  }

  export interface CreateUserInteractionResponse {
    /**
     * Entity type: Interaction
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates a UserInteraction. A UserInteraction logs an interaction between an employee and a customer, for example the fact the customer was called.
   * 
   * UserID is optional but only if OrderID is also provided and the given Order is attached to a customer. If so then the UserInteraction is attached to the Order.Customer.
   */
  export interface UpdateUserInteraction {
    /**
     * Entity type: Interaction
     */
    ID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    Text: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
  }

  /**
   * Deletes a UserInteraction.
   */
  export interface DeleteUserInteraction {
    /**
     * Entity type: Interaction
     */
    ID: number;
  }

  export interface ListUserInteractions {
    PageConfig?: PageConfig<ListUserInteractionsFilter>;
  }

  export interface ListUserInteractionsFilter {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    Text?: string;
    InteractionType?: InteractionType;
    IncludeResends?: boolean;
  }

  export interface ListUserInteractionsResponse {
    Result?: PagedResult<ListUserInteractionsResponse.UserInteractionDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListUserInteractionsResponse {
    export interface UserInteractionDto {
      /**
       * Entity type: Interaction
       */
      ID?: number;
      /**
       * Entity type: User
       */
      UserID?: number;
      User?: UserInteractionDto.UserDto;
      /**
       * Entity type: User
       */
      CreatedByID?: number;
      CreatedBy?: UserInteractionDto.UserDto;
      /**
       * Entity type: User
       */
      LastModifiedByID?: number;
      LastModifiedBy?: UserInteractionDto.UserDto;
      Text?: string;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      /**
       * Entity type: UserTask
       */
      UserTaskID?: number;
      InteractionType?: InteractionType;
      CreationTime?: string;
      OriginalUserInteraction?: ListUserInteractionsResponse.UserInteractionDto;
      /**
       * Entity type: MessageTemplate
       */
      MessageTemplateID?: number;
    }

    export namespace UserInteractionDto {
      export interface UserDto {
        FullName: string;
        FirstName: string;
        LastName: string;
        EmailAddress: string;
      }

    }

  }

  export interface PushUserInteractionContent {
    /**
     * Entity type: Interaction
     */
    ID: number;
    ToEmailAddress: string;
    ToName?: string;
    FromEmailAddress: string;
    FromName?: string;
    Subject: string;
    BodyHTML: string;
  }

  /**
   * Service for listing transput jobs
   */
  export interface ListTransputJobs {
    InitialPageConfig?: PageTokenConfig<ListTransputJobsFilter>;
    PageToken?: string;
  }

  export interface ListTransputJobsFilter {
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    Status: number[];
    /**
     * Entity type: TransputJobType
     */
    Types: number[];
    Identifier: string;
    Name: string;
    StartTimeBefore?: string;
    StartTimeOnOrAfter?: string;
    EndTimeBefore?: string;
    EndTimeOnOrAfter?: string;
    CreationTimeBefore?: string;
    CreationTimeOnOrAfter?: string;
    /**
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnit?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnit?: number;
  }

  export interface ListTransputJobsResponse {
    Results?: TransputJobWithRelatedExportDataDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface TransputJobWithRelatedExportDataDto {
    /**
     * Entity type: TransputJob
     */
    ID: number;
    Status: TransputJobStatuses;
    TypeID: number;
    Identifier: string;
    CreationTime: string;
    StartTime?: string;
    EndTime?: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderExport
     */
    OrderExportID?: number;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * Entity type: ShipmentExport
     */
    ShipmentExportID?: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    /**
     * Entity type: InvoiceExport
     */
    InvoiceExportID?: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    /**
     * Entity type: FinancialPeriodExport
     */
    FinancialPeriodExportID?: number;
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnit?: number;
    ShipFromOrganizationUnitName?: string;
    /**
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnit?: number;
    ShipToOrganizationUnitName?: string;
    Type: EnumDto;
  }

  export const enum TransputJobStatuses {
    New = 0,
    Running = 1,
    Finished = 2,
    Failed = -1
  }

  export interface EnumDto {
    /**
     * Entity type: Enum
     */
    ID: number;
    /**
     * Usually an internal & unchangable code - for display, use `Description`
     */
    Name: string;
    Description: string;
  }

  /**
   * Creates an EntityTranslation if one does not exist yet, otherwise it replaces the current one.
   */
  export interface ReplaceEntityTranslation {
    /**
     * The Identifier of the object you are translating, in case of a Discount this would be the DiscountID
     */
    EntityID: string;
    /**
     * The name of the object you are translating, in case of a Discount this would be 'Discount'
     */
    EntityType: string;
    /**
     * The name of the property you are translating, in case of translating the Description of a Discount this would be 'Description'
     */
    EntityField: string;
    LanguageID: string;
    CountryID?: string;
    /**
     * The translated value for the property you are translating
     */
    Value: string;
  }

  /**
   * Service for retrieving a list of `EntityTranslation`
   */
  export interface ListEntityTranslations {
    PageConfig?: PageConfig<ListEntityTranslationsFilter>;
  }

  export interface ListEntityTranslationsFilter {
    EntityID: string;
    EntityType: string;
    EntityField: string;
    LanguageID: string;
    CountryID: string;
    Value: string;
  }

  export interface ListEntityTranslationsResponse {
    Result?: PagedResult<ListEntityTranslation>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListEntityTranslation {
    EntityID: string;
    EntityType: string;
    EntityField: string;
    LanguageID: string;
    CountryID: string;
    Value: string;
  }

  /**
   * Service for deleting an existing `EntityTranslation`
   */
  export interface DeleteEntityTranslation {
    EntityID: string;
    EntityType: string;
    EntityField: string;
    LanguageID: string;
    CountryID?: string;
  }

  /**
   * Copies EntityTranslations from one entity to another. Replaces any existing translations of the target entity.
   */
  export interface CopyEntityTranslations {
    /**
     * The Identifier of the object you are translating from, in case of a Discount this would be the DiscountID
     */
    SourceEntityID: string;
    /**
     * The Identifier of the object you are translating to, in case of a Discount this would be the DiscountID
     */
    TargetEntityID: string;
    /**
     * The name of the object you are translating, in case of a Discount this would be 'Discount'
     */
    EntityType: string;
    /**
     * When provided, will only copy the translations for the given language.
     */
    LanguageID?: string;
    /**
     * When provided, will only copy the translations for the given country.
     */
    CountryID?: string;
  }

  /**
   * Service for retrieving an existing `EntityTranslation`
   */
  export interface GetEntityTranslation {
    EntityID: string;
    EntityType: string;
    EntityField: string;
    LanguageID: string;
    CountryID?: string;
  }

  export interface GetEntityTranslationResponse {
    EntityID: string;
    EntityType: string;
    EntityField: string;
    LanguageID: string;
    CountryID: string;
    Value: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates an StringTranslation if one does not exist yet, otherwise it replaces the current one.
   */
  export interface ReplaceStringTranslation {
    Key: string;
    LanguageID: string;
    CountryID: string;
    Value: string;
  }

  /**
   * Service for retrieving a list of `StringTranslation`
   */
  export interface ListStringTranslations {
    PageConfig?: PageConfig<ListStringTranslationsFilter>;
  }

  export interface ListStringTranslationsFilter {
    Key: string;
    LanguageID: string;
    CountryID: string;
    Value: string;
  }

  export interface ListStringTranslationsResponse {
    Result?: PagedResult<ListStringTranslation>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListStringTranslation {
    Key: string;
    LanguageID: string;
    CountryID: string;
    Value: string;
  }

  /**
   * Service for deleting an existing `StringTranslation`
   */
  export interface DeleteStringTranslation {
    Key: string;
    LanguageID: string;
    CountryID: string;
  }

  /**
   * Service for retrieving a `StringTranslation`
   */
  export interface GetStringTranslation {
    Key: string;
    LanguageID: string;
    CountryID: string;
  }

  export interface GetStringTranslationResponse {
    Key: string;
    LanguageID: string;
    CountryID: string;
    Value: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * To continue with a suspended order, use the `ResumeSuspendedOrder` service.
   */
  export interface ListSuspendedOrders {
    PageConfig?: PageConfig<ListSuspendedOrdersFilter>;
  }

  export interface ListSuspendedOrdersFilter {
    IsPaid?: boolean;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Description: string;
  }

  export interface ListSuspendedOrdersResponse {
    Result?: PagedResult<ListSuspendedOrdersResponse.SuspendedOrderDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListSuspendedOrdersResponse {
    export interface SuspendedOrderDto {
      /**
       * Entity type: Order
       */
      OrderID: number;
      OrderDisplayID: string;
      Description: string;
      TotalAmount: number;
      TotalAmountInTax: number;
      CurrencyID: string;
      CustomerName: string;
      SuspendedByName: string;
      SuspensionTime: string;
    }

  }

  /**
   * When you would like to temporarily suspend an order, you can use `SuspendOrder` to do so.
   * 
   * - To identify your suspended order it is possible to supply a `Description`
   * - To print a receipt of the suspended order, to make it easier to continue later on, supply `PrintReceipt` (and optionally a `StationID`)
   * - To retreive the suspended orders, see the `ListSuspendedOrders` service
   */
  export interface SuspendOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Description?: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
    PrintReceipt?: boolean;
  }

  /**
   * When you would like to resume a suspended order, you can use `ResumeSuspendedOrder` to do so. This will remove the suspended state from the order.
   */
  export interface ResumeSuspendedOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Service for retrieving a list of `OrganizationUnitSupplier` by `OrganizationUnit`
   */
  export interface GetSuppliersForOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetSuppliersForOrganizationUnitResponse {
    Result: GetSuppliersForOrganizationUnitResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetSuppliersForOrganizationUnitResponse {
    export interface Model {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Service for creating a `OrganizationUnitSupplier`
   */
  export interface CreateOrganizationUnitSupplier {
    /**
     * This will be the OrganizationUnit that will be supplied to by the supplier.
     * 
     * If neither OrganizationUnitID or OrganizationUnitSetID or OrganizationUnitIDs are specified then all children of the supplier are automatically supplied to.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * The OrganizationUnits in this set will be the OrganizationUnits that will be supplied to by the supplier.
     * Adding OrganizationUnits to this set in the future will not result in those OrganizationUnits being automatically added to this collection.
     * 
     * If neither OrganizationUnitID or OrganizationUnitSetID or OrganizationUnitIDs are specified then all children of the supplier are automatically supplied to.
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * These will be the OrganizationUnits that will be supplied to by the supplier.
     * 
     * If neither OrganizationUnitID or OrganizationUnitSetID or OrganizationUnitIDs are specified then all children of the supplier are automatically supplied to.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    Target?: OrganizationUnitSupplierTargets;
    /**
     * This will be the OrganizationUnit that becomes the supplier in this supplier relationship.
     * 
     * Either SupplierOrganizationUnitID or SupplierOrganizationUnitSetID or SupplierOrganizationUnitIDs must be specified to be a valid request.
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
    /**
     * The OrganizationUnits in this set will be the OrganizationUnits that will be the suppliers
     * 
     * Either SupplierOrganizationUnitID or SupplierOrganizationUnitSetID or SupplierOrganizationUnitIDs must be specified to be a valid request.
     * Entity type: OrganizationUnitSet
     */
    SupplierOrganizationUnitSetID?: number;
    /**
     * These will be the OrganizationUnits that becomes the suppliers in this supplier relationship.
     * 
     * Either SupplierOrganizationUnitID or SupplierOrganizationUnitSetID or SupplierOrganizationUnitIDs must be specified to be a valid request.
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitIDs?: number[];
  }

  export const enum OrganizationUnitSupplierTargets {
    Default = 0,
    Replenishment = 1,
    SupplierReturns = 2,
    Pickup = 3
  }

  export const enum OrganizationUnitSupplierTypes {
    Internal = 0,
    External = 1
  }

  export interface CreateOrganizationUnitSupplierResponse {
    /**
     * Entity type: OrganizationUnitSupplier
     */
    OrganizationUnitSupplierIDs: number[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for updating a existing `OrganizationUnitSupplier`
   */
  export interface UpdateOrganizationUnitSupplier {
    /**
     * Entity type: OrganizationUnitSupplier
     */
    ID: number;
    Target?: OrganizationUnitSupplierTargets;
  }

  export interface DeleteOrganizationUnitSupplier {
    /**
     * Entity type: OrganizationUnitSupplier
     */
    ID: number;
  }

  /**
   * Service for retrieving a list of `OrganizationUnitSupplier`
   */
  export interface ListOrganizationUnitSuppliers {
    PageConfig?: PageConfig<ListOrganizationUnitSuppliersFilter>;
  }

  export interface ListOrganizationUnitSuppliersFilter {
    /**
     * Entity type: OrganizationUnitSupplier
     */
    ID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName?: string;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
    SupplierOrganizationUnitName?: string;
    Target?: OrganizationUnitSupplierTargets;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
  }

  export interface ListOrganizationUnitSuppliersResponse {
    Result?: PagedResult<ListOrganizationUnitSuppliersModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListOrganizationUnitSuppliersModel {
    /**
     * Entity type: OrganizationUnitSupplier
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID: number;
    SupplierOrganizationUnitName: string;
    Target: OrganizationUnitSupplierTargets;
  }

  /**
   * Book a set of Adjustments to modify stock.
   * 
   * An Adjustment should have a `ProductID` to identify the product, an `OrganizationUnitID` to identify the stock keeping organization, a `StockLabelID` (see `GetEnumValues` with type `StockLabel`),
   * a `StockMutationReasonID` (see `GetEnumValues` with type `StockMutationReasons`) and of course a `Quantity` to actually book.
   * 
   * Supply a `BackendSystemID` on the request and `BackendID`'s on the Adjustments to make the request idempotent (optional). When one adjustment in your request fails, all of them will fail. Duplicate
   * keys will not be processed again and yield a warning (but thus will not block!).
   * 
   * Optionally supports a `Remark` for each Adjustment.
   * 
   * When specifying a `SerialNumber`, please note the `Quantity` of this adjustment then should either be -1 or 1!
   */
  export interface AdjustStock {
    /**
     * Optional, unique identifier for the system that owns the `Adjustment.BackendID` as key for the mutation
     */
    BackendSystemID?: string;
    Adjustments: AdjustStock.Adjustment[];
  }

  export namespace AdjustStock {
    export interface Adjustment {
      /**
       * Optional, unique identifier for the mutation, combined with the root `BackendSystemID`
       */
      BackendID?: string;
      /**
       * Required product identifier (use the `EVA-IDs-Mode` header to switch identifier keys (`Product.ID`, `Product.BackendID`, ...)
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
      /**
       * Entity type: StockMutationReasons
       */
      StockMutationReasonID: number;
      Quantity: number;
      Remark?: string;
      SerialNumber?: string;
    }

  }

  /**
   * This service allows you to manually commit the stock for the given OrderLineIDs.
   * The response contains whether or not the commitment was successful, per OrderLine.
   * 
   * Commitment can be unsucessful for two reasons:
   * 
   * - No commitment is necessary. This is because it's an OrderLine that can't be committed, for example one without a stock product, or because the order type doesn't allow commitments.
   * - Insufficient stock. This service only allows creating a commitment when there's sufficient available stock. Use the MoveCommitments or UncommitOrderLines services to create available stock.
   */
  export interface CommitOrderLines {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    OrderLines?: CommitOrderLines.CommitOrderLine[];
    CommitmentStatus?: OrderLineCommitmentStatus;
  }

  export namespace CommitOrderLines {
    export interface CommitOrderLine {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      /**
       * If specified as non-null and less than the open quantity on the OrderLine this will causse the OrderLine to be split into two.
       * The ID of the new OrderLine will be returned in the response and the object will have a reference to the OriginalOrderLineID, which is the ID that was specified in the request.
       */
      QuantityToCommit?: number;
    }

  }

  export const enum OrderLineCommitmentStatus {
    Uncommitted = 0,
    SoftCommit = 1,
    HardCommit = 2,
    ManualCommit = 5,
    ForcedCommit = 10
  }

  export interface CommitOrderLinesResponse {
    Results: CommitOrderLinesResponse.CommitmentResultForOrderLine[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace CommitOrderLinesResponse {
    export interface CommitmentResultForOrderLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      IsCommitted: boolean;
      /**
       * Entity type: OrderLine
       */
      OriginalOrderLineID?: number;
    }

  }

  /**
   * This service allows you to move stock commitments from one OrderLine (`SourceOrderLineID`) to another OrderLine (`DestinationOrderLineID`).
   * 
   * - If the Source and Destination do not share the same ProductID, a `CannotMoveCommitment` error is returned.
   * - If the Source does not have a commitment, a `CannotMoveCommitment` error is returned.
   */
  export interface MoveCommitments {
    Movements?: MoveCommitments.CommitmentMovement[];
  }

  export namespace MoveCommitments {
    export interface CommitmentMovement {
      /**
       * Entity type: OrderLine
       */
      SourceOrderLineID: number;
      /**
       * Entity type: OrderLine
       */
      DestinationOrderLineID: number;
    }

  }

  /**
   * This service allows you to undo a commitment on the given OrderLineIDs, making the committed stock available to other orders again.
   */
  export interface UncommitOrderLines {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
  }

  /**
   * Creates a stock mutation for the given organization unit and product. Note that this service supports creating delta's as well as modifying in absolute quantities based on the `Delta` property.'
   */
  export interface CreateStockMutations {
    OrganizationUnitBackendID?: string;
    Mutations: CreateStockMutations.StockMutation[];
  }

  export namespace CreateStockMutations {
    export interface StockMutation {
      ProductBackendID: string;
      Timestamp: string;
      Quantity: number;
      Reason: string;
      Remark: string;
      Delta: boolean;
      SerialNumber?: string;
    }

  }

  /**
   * @deprecated Use GetProductAvailability instead.
   * Returns the pickup and/or delivery availability of the given products
   */
  export interface GetAvailabilityIndication {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Providing this will limit the pickup availability indication to only these OrganizationUnits.
     * Entity type: OrganizationUnit
     */
    PickupOrganizationUnitIDs?: number[];
    /**
     * The products for which to retrieve the availability.
     * 
     * The QuantityRequested will default to 1 if not provided. If this field is left null then OrderID must have a value and visa versa.
     */
    Products?: ProductQuantityDto[];
    /**
     * If provided this will return the availability indication for the products on the Order.
     * 
     * This can be used instead of manually providing Products on this request.
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * If true, limits the pickup availability results to only the OrganizationUnit that has the product available the quickest.
     * 
     * If multiple OrganizationUnits have the same availability, it's not defined which OrganizationUnit is returned.
     */
    FastestPickupShopOnly?: boolean;
    /**
     * If true, the OrganizationUnits returned not only have their basic information returned, but also their OpeningHours.
     */
    DetailedShopInformation?: boolean;
    IncludeAvailabilityTexts?: boolean;
    /**
     * If provided, limits the availability to indication to the provided types.
     * 
     * The types can be combined to provide both Delivery and Pickup indications, but this is also the default behavior so specifying this is not necessary.
     * 
     * Specify this only if you only need either Delivery or Pickup indications, this can result in better performance due to a smaller response size.
     */
    Type?: GetAvailabilityTypes;
  }

  export interface ProductQuantityDto {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * @default 1
     */
    QuantityRequested?: number;
    ShippingMethodID?: number;
  }

  export const enum GetAvailabilityTypes {
    None = 0,
    Pickup = 1,
    Delivery = 2,
    CarryOut = 4
  }

  export interface GetAvailabilityIndicationResponse {
    Products: GetAvailabilityIndicationResponse.ProductAvailabilityIndication[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAvailabilityIndicationResponse {
    export interface ProductAvailabilityIndication {
      /**
       * Entity type: Product
       */
      ProductID: number;
      StoreAvailabilityCount: number;
      StoreCount: number;
      ProductStatus: ProductStatus;
      ExpectedAvailabilityStatus?: ProductPromisedDeliveryStatuses;
      PickupText: string;
      PickupIndications: GetAvailabilityIndicationResponse.PickupAvailabilityIndication[];
      Delivery: GetAvailabilityIndicationResponse.DeliveryAvailabilityIndication;
    }

    export interface DeliveryAvailabilityIndication {
      AvailabilityDate?: string;
      QuantityAvailable?: number;
      IsAvailable: boolean;
      AvailabilityText: string;
    }

    export interface PickupAvailabilityIndication {
      PickupOrganizationUnit: OrganizationUnitDto;
      PickupIndicationDate: string;
      QuantityOnHand: number;
      QuantityAvailable: number;
      IsAvailable: boolean;
      /**
       * When the store has a known time to finish a pickup this contains the first available time
       */
      PickupIndicationTime?: string;
    }

  }

  export const enum ProductStatus {
    None = 0,
    PreRelease = 1,
    DeliveryOnly = 4,
    DisableDelivery = 8,
    DisablePickup = 16,
    DisableBackorder = 32,
    UseUp = 34,
    NonSellable = 64
  }

  export const enum ProductPromisedDeliveryStatuses {
    Unknown = 0,
    KnownUnSure = 1,
    KnownQuiteSure = 2,
    KnownSure = 3,
    OnDay = 4,
    Stock = 5
  }

  /**
   * This service returns information about product availability.
   */
  export interface GetProductAvailability {
    Products?: GetProductAvailability.Product[];
    /**
     * By default the availability is calculated from the perspective of your current OrganizationUnit, which determines which warehouse(s) stock is sourced
     * from, among other things. You can override the default with an OrganizationUnitID of your choosing and calculate availability from that
     * OrganizationUnit's perspective.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Contains options that let you influence the availability calculation.
     */
    Options?: GetProductAvailabilityOptions;
  }

  export namespace GetProductAvailability {
    export interface Product {
      /**
       * Entity type: Product
       * 
       * Allows missing/invalid values when using EVA-IDs-Mode: ExternalIDs.
       */
      ID?: number;
      /**
       * Required when `ID` is omitted.
       */
      BackendID?: string;
      /**
       * @default 1
       */
      QuantityRequested?: number;
      /**
       * Entity type: ShippingMethod
       */
      ShippingMethodID?: number;
    }

  }

  export interface GetProductAvailabilityOptions {
    /**
     * Contains options that let you influence if and how delivery availability is calculated. If not specified at all then no delivery availability is
     * calculated and returned.
     */
    Delivery?: GetProductAvailabilityOptions.DeliveryAvailability;
    /**
     * Contains options that let you influence if and how pickup availability is calculated. If not specified at all then no pickup availability is
     * calculated and returned.
     */
    Pickup?: GetProductAvailabilityOptions.PickupAvailability;
    CarryOut?: GetProductAvailabilityOptions.CarryOutAvailability;
    /**
     * If set to true then the QuantityAvailable is returned. By default this information is only accessible to Employee-type users.
     */
    QuantityAvailable?: boolean;
    /**
     * In addition to the normal data, additional detailed information can be returned for the provided OrganizationUnits and Products.
     * 
     * This information includes the entire supplier hierarchy for each OrganizationUnit, telling you where stock originates from, where it's currently committed.
     * 
     * Also includes information about expected future stock and allocation assignments.
     * 
     * Providing this field as an empty object is interpreted as a request to include this information for all organizations and products.
     */
    ProductAvailabilityDetails?: GetProductAvailabilityOptions.ProductAvailabilityDetailsOptions;
    /**
     * If set to true then the availability of related stock replacing products is returned.
     */
    IncludeRelatedStockReplacingProducts?: boolean;
    /**
     * Contains options that allow you to influence how bundle availability is calculated.
     */
    Bundles?: GetProductAvailabilityOptions.ProductBundleAvailabilityOptions;
  }

  export namespace GetProductAvailabilityOptions {
    export interface ProductAvailabilityDetailsOptions {
      /**
       * Which organization(s) to get the details for, defaults to the OrganizationUnitID provided in the request or to your current organization.
       * Entity type: OrganizationUnit
       * 
       * Allows missing/invalid values when using EVA-IDs-Mode: ExternalIDs.
       */
      OrganizationUnitIDs?: number[];
      /**
       * Entity type: Product
       * 
       * Allows missing/invalid values when using EVA-IDs-Mode: ExternalIDs.
       */
      ProductIDs?: number[];
      /**
       * Determines whether or not the detailed information for the requested organization(s) recursively includes the stock details of their suppliers as well.
       * Defaults to false if both OrganizationUnitIDs and ProductIDs is left empty, otherwise it defaults to true.
       * Setting this to false limits the response size in case you only care about the stock details of the specific requested organization(s).
       */
      IncludeSuppliers?: boolean;
    }

    export interface ProductBundleAvailabilityOptions {
      /**
       * Determines how bundle availability is calculated. By default the bundle is said to be available when all required lines of the bundle have at least
       * one option that has stock. The alternative is that the bundle is only considered available when the default product for each line has stock.
       * 
       * If not provided then the value of the `ProductBundleAvailabilityCalculationMethod` EVA setting is used.
       */
      AvailabilityCalculationMethod?: ProductBundleAvailabilityCalculationMethods;
      /**
       * When the availability of a bundle is calculated it means that all the products that are part of the bundle become part of the output of
       * GetProductAvailability. If this option is turned on, the service will also return information about the bundle to make sense of those extra products.
       */
      IncludeBundleInformation?: boolean;
    }

    export const enum PickupTypes {
      /**
       * Returns the shop that has the earliest availability date for each product.
       */
      FastestShop = 0,
      /**
       * Returns the pickup availability date for your current shop.
       */
      CurrentShop = 1
    }

    export const enum CarryOutTypes {
      /**
       * Returns the shop that has the most stock available for each product.
       */
      MostStock = 0,
      /**
       * Returns the carry-out availability date for your current shop.
       */
      CurrentShop = 1
    }

    export interface IAvailabilityRequestOptions {
      InRadius: GetProductAvailabilityOptions.PickupRadius;
    }

    export interface CarryOutAvailability {
      /**
       * If specified as true then all relevant shop OrganizationUnits are returned, whether they have available stock or not.
       * 
       * Defaults to false and only has effect if `IncludeCarryOutOrganizationUnits` is set to `true`.
       */
      IncludeOrganizationUnitsWithoutStock?: boolean;
      /**
       * If specified as true the response will contain the pickup organizations. Which organizations are included depends on the rest of the request:
       * 
       * - If a rectangular area is specified through `InArea` it will contain the organizations within the area.
       * - If the radius is specified through `InRadius` it will contain the organizations within the radius of the given center co-ordinates.
       * - If an explicit list of organizations is specified through `OrganizationUnitIDs` then those are returned.
       * 
       * In all cases, whether or not all of them are returned or only those that contain stock is determined by `IncludeOrganizationUnitsWithoutStock`. By
       * default only organizations with stock are returned.
       * 
       * If the request specifies more than one product, the organization will only be returned if *all* products have stock (unless
       * `IncludeOrganizationUnitsWithoutStock` is `true`).
       */
      IncludeCarryOutOrganizationUnits?: boolean;
      /**
       * This can be optionally used to provide a list of shops for which you would like to check carry-out availability.
       * Entity type: OrganizationUnit
       */
      OrganizationUnitIDs?: number[];
      /**
       * Select the organizations that are considered for carry-out based on a rectangular area defined by the top-left and bottom-right co-ordinates.
       */
      InArea?: GetProductAvailabilityOptions.PickupArea;
      /**
       * Select the organizations that are considered for carry-out based on a circular area defined by the center co-ordinate and a radius (specified in km).
       */
      InRadius?: GetProductAvailabilityOptions.PickupRadius;
      /**
       * When the carry-out organizations are returned, this determines what returned opening-hours are returned. See each value of the enum for details.
       */
      OpeningHoursType?: GetProductAvailabilityOptions.PickupOrganizationUnitOpeningHoursType;
      Type?: GetProductAvailabilityOptions.CarryOutTypes;
    }

    export interface PickupAvailability {
      /**
       * Include the expected date on which the provided products can be picked-up from a shop.
       */
      AvailabilityDate?: boolean;
      /**
       * This field lets you specify whether the pickup availability date you get back (in case you requested it) for each product should be based on your
       * current OrganizationUnit, or whether it should be the OrganizationUnit that has the earliest availability for the product. Defaults to FastestShop.
       */
      Type?: GetProductAvailabilityOptions.PickupTypes;
      OrganizationUnitType?: OrganizationUnitTypes;
      /**
       * If specified as true then all relevant pick-up OrganizationUnits are returned, whether they have available stock or not.
       * 
       * Defaults to false and only has effect if `IncludePickupOrganizationUnits` is set to `true`.
       */
      IncludeOrganizationUnitsWithoutStock?: boolean;
      /**
       * If specified as true the response will contain the pickup organizations. Which organizations are included depends on the rest of the request:
       * 
       * - If a rectangular area is specified through `InArea` it will contain the organizations within the area.
       * - If the radius is specified through `InRadius` it will contain the organizations within the radius of the given center co-ordinates.
       * - If an explicit list of organizations is specified through `OrganizationUnitIDs` then those are returned.
       * 
       * In all cases, whether or not all of them are returned or only those that contain stock is determined by `IncludeOrganizationUnitsWithoutStock`. By
       * default only organizations with stock are returned.
       * 
       * If the request specifies more than one product, the organization will only be returned if *all* products have stock (unless
       * `IncludeOrganizationUnitsWithoutStock` is `true`).
       */
      IncludePickupOrganizationUnits?: boolean;
      /**
       * This can be optionally used to provide a list of shops for which you would like to check pick-up availability.
       * Entity type: OrganizationUnit
       */
      OrganizationUnitIDs?: number[];
      /**
       * Select the organizations that are considered for pickup based on a rectangular area defined by the top-left and bottom-right co-ordinates.
       */
      InArea?: GetProductAvailabilityOptions.PickupArea;
      /**
       * Select the organizations that are considered for pickup based on a circular area defined by the center co-ordinate and a radius (specified in km).
       */
      InRadius?: GetProductAvailabilityOptions.PickupRadius;
      /**
       * When the pickup organizations are returned, this determines what returned opening-hours are returned. See each value of the enum for details.
       */
      OpeningHoursType?: GetProductAvailabilityOptions.PickupOrganizationUnitOpeningHoursType;
    }

    export const enum PickupOrganizationUnitOpeningHoursType {
      /**
       * Openinghours are not returned.
       */
      None = 0,
      /**
       * Openinghours for the next seven days are returned. Each result will contain the date and day-of-week. This will take into account any special
       * upcoming openinghours.
       */
      NextSevenDays = 1,
      /**
       * Openinghours for the current week are returned. Similar to NextSevenDays with the difference that the first item will always be Monday.
       */
      CurrentWeek = 2,
      /**
       * The regular openinghours for each day of the week will be returned without taking into account any special openinghours or holidays.
       */
      Regular = 3
    }

    export interface Coordinate {
      Latitude: number;
      Longitude: number;
    }

    export interface PickupRadius {
      RadiusInKm?: number;
      /**
       * Defaults to the coordinates of the selected OrganizationUnit when not set, but a RadiusInKm is set.
       */
      Center?: GetProductAvailabilityOptions.Coordinate;
      /**
       * Limit OrganizationUnits within the radius to those with the provided country. Optional.
       */
      CountryID?: string;
    }

    export interface PickupArea {
      TopLeft: GetProductAvailabilityOptions.Coordinate;
      BottomRight: GetProductAvailabilityOptions.Coordinate;
      /**
       * Limit OrganizationUnits within the area to those with the provided country. Optional.
       */
      CountryID?: string;
    }

    export interface DeliveryAvailability {
      /**
       * Include the expected date on which the provided products can be home-delivered.
       */
      AvailabilityDate?: boolean;
    }

  }

  export const enum ProductBundleAvailabilityCalculationMethods {
    AnyOptionHasStock = 0,
    DefaultOptionHasStock = 1
  }

  export interface GetProductAvailabilityResponse {
    Products: ProductAvailabilityResult[];
    PickupOrganizationUnits: ProductAvailabilityPickupOrganizationUnit[];
    CarryOutOrganizationUnits: ProductAvailabilityCarryOutOrganizationUnit[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ProductAvailabilityResult {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    ProductStatus?: ProductStatus;
    Delivery?: ProductAvailabilityResult.DeliveryAvailabilityResult;
    Pickup?: ProductAvailabilityResult.PickupAvailabilityResult;
    CarryOut?: ProductAvailabilityResult.CarryOutAvailabilityResult;
    /**
     * Contains the lines and option that this bundle consists of (if the product is a bundle product).
     */
    BundleInformation?: ProductAvailabilityResult.ProductAvailabilityResultBundleInformation;
    StockReplacementInformation?: ProductAvailabilityResult.ProductStockReplacementInformation;
    AvailabilityDetails?: ProductAvailabilityResult.ProductAvailabilityDetails[];
  }

  export namespace ProductAvailabilityResult {
    export interface ProductAvailabilityDetails {
      OrganizationUnit: ProductAvailabilityDetails.OrganizationUnitModel;
      /**
       * The quantity of stock that the OrganizationUnit physically has.
       */
      QuantityOnHand: number;
      /**
       * The quantity that has been committed to this OrganizationUnit.
       */
      QuantityCommitted: number;
      /**
       * The quantity of stock that is still available for the OrganizationUnit, so the amount of physical stock minus its commitments.
       */
      QuantityAvailable: number;
      /**
       * The total quantity of orders that are currently have a fulfillment status of WaitingForStock, meaning that they are waiting until stock becomes available on this OrganizationUnit.
       */
      OrdersWaitingForStockQuantity: number;
      /**
       * The quantity of the physical stock that is available to the OrganizationUnit for which this OrganizationUnit is a supplier.
       * 
       * This can differ from the QuantityOnHand when it comes to stock allocation. It's possible that the supplier reserves stock for certain other OrganizationUnits, meaning
       * that only a portion of the QuantityOnHand is available.
       * 
       * Example: you request stock details for the website, which is supplied by the warehouse organization. There is a stock allocation rule that 80% of the warehouse stock
       * is reserved to shops, so not available to the website. In that case, the website organization will have a stock detail with warehouse as a supplier and the stock detail
       * for the warehouse will (for example) have a QuantityOnHand of 100 but only an EffectiveQuantityOnHand of 20.
       */
      EffectiveQuantityOnHand?: number;
      /**
       * See EffectiveQuantityOnHand, but here the commitments are subtracted from the on hand stock. So if physical stock is 100, 80% is allocated to other organizations and there is a commitment of 11 then EffectiveQuantityAvailable will be 9.
       */
      EffectiveQuantityAvailable?: number;
      /**
       * The quantity committed on all suppliers of this organization
       */
      SupplierQuantityCommitted: number;
      /**
       * The quantity that all suppliers have on hand as far as we know from the perspective of the organization that is supplied by them.
       * 
       * The actual supplier might have more stock physically on hand, but due to allocation rules this might not be available to this organization.
       */
      SupplierQuantityOnHand: number;
      /**
       * The SupplierQuantityOnHand minus the SupplierQuantityCommitted
       */
      SupplierQuantityAvailable: number;
      /**
       * The sum of all StockReservations, meaning the quantities that were reserved on this supplier for other organizations.
       */
      QuantityReservedForOtherOrganizationUnits: number;
      /**
       * The date that this organization is expected to have stock.
       */
      ExpectedAvailabilityDate?: string;
      /**
       * The earliest date that one of the suppliers of this organization has stock.
       */
      ExpectedSupplierAvailabilityDate?: string;
      /**
       * The suppliers for this organization.
       */
      Suppliers?: ProductAvailabilityResult.ProductAvailabilityDetails[];
      /**
       * The stock that is reserved for other organizations that is not available to this organization.
       */
      StockReservations?: ProductAvailabilityDetails.StockReservation[];
      /**
       * The stock that is expected to be received in the future.
       */
      ExpectedStock?: ProductAvailabilityDetails.ExpectedStockItem[];
    }

    export namespace ProductAvailabilityDetails {
      export interface StockReservation {
        /**
         * The organization unit for which the stock is reserved.
         */
        ReservedForOrganizationUnit: ProductAvailabilityDetails.OrganizationUnitModel;
        /**
         * The quantity that is reserved, so this is the quantity that is not available to our organization
         */
        Quantity: number;
      }

      export interface ExpectedStockItem {
        /**
         * The OrderID that the expected stock is expected to be received on.
         * Entity type: Order
         */
        OrderID: number;
        /**
         * The date on which the stock is expected to be received.
         */
        ExpectedOnStockDate: string;
        /**
         * The quantity that is expected to be received.
         */
        Quantity: number;
        /**
         * How much of this expected stock is already spoken for by orders.
         */
        QuantityCommitted: number;
      }

      export interface OrganizationUnitModel {
        /**
         * Entity type: OrganizationUnit
         */
        ID: number;
        Name: string;
        Description: string;
        BackendID: string;
        Type: OrganizationUnitTypes;
      }

    }

    export interface ProductStockReplacementInformation {
      StockReplacementProducts: ProductStockReplacementInformation.StockReplacementProduct[];
    }

    export namespace ProductStockReplacementInformation {
      export interface StockReplacementProduct {
        /**
         * Entity type: Product
         */
        ProductID: number;
      }

    }

    export interface ProductAvailabilityResultBundleInformation {
      Lines: ProductAvailabilityResultBundleInformation.ProductBundleLine[];
    }

    export namespace ProductAvailabilityResultBundleInformation {
      export interface ProductBundleLine {
        /**
         * Entity type: ProductBundleLine
         */
        ID: number;
        Type: ProductBundleLineTypes;
        /**
         * Entity type: Product
         */
        DefaultProductID?: number;
        /**
         * Entity type: Product
         */
        OptionProductIDs: number[];
        Description: string;
        MaxSelectedOptionsCount?: number;
      }

    }

    export interface PickupAvailabilityResult {
      AvailabilityDate?: string;
      IsAvailable: boolean;
      QuantityAvailable?: number;
      /**
       * Indicates whether or not there is currently stock available for pick-up.
       */
      HasStock: boolean;
      EstimatedPickupTime?: string;
    }

    export interface DeliveryAvailabilityResult {
      AvailabilityDate?: string;
      IsAvailable: boolean;
      QuantityAvailable?: number;
      /**
       * Indicates whether or not there is currently stock available for home-delivery.
       */
      HasStock: boolean;
    }

    export interface CarryOutAvailabilityResult {
      IsAvailable: boolean;
      QuantityAvailable?: number;
      /**
       * Indicates whether or not there is currently stock available for pick-up.
       */
      HasStock: boolean;
    }

  }

  export const enum ProductBundleLineTypes {
    Required = 0,
    Optional = 1,
    OptionalIncludedInPrice = 2
  }

  export interface ProductAvailabilityPickupOrganizationUnit {
    /**
     * HasStock will be true if the OrganizationUnit has stock for *all* requested products.
     */
    HasStock: boolean;
    Products: ProductAvailabilityPickupOrganizationUnit.PickupOrganizationProductQuantity[];
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    BackendID: string;
    Name: string;
    Description: string;
    Type: OrganizationUnitTypes;
    IsPickup: boolean;
    Status: OrganizationUnitStatus;
    Address: AddressDto;
    Latitude?: number;
    Longitude?: number;
    EmailAddress: string;
    PhoneNumber: string;
    TimeZone: string;
    DistanceInKm?: number;
    OpeningHours: OpeningHoursInfo[];
  }

  export namespace ProductAvailabilityPickupOrganizationUnit {
    export interface PickupOrganizationProductQuantity {
      /**
       * Entity type: Product
       */
      ProductID: number;
      QuantityAvailable: number;
      AvailabilityDate?: string;
    }

  }

  export interface OpeningHoursInfo {
    DayOfWeek: DayOfWeek;
    OpeningTime?: string;
    ClosingTime?: string;
    Date?: string;
    Closed: boolean;
  }

  export interface ProductAvailabilityCarryOutOrganizationUnit {
    /**
     * HasStock will be true if the OrganizationUnit has stock for *all* requested products.
     */
    HasStock: boolean;
    Products: ProductAvailabilityCarryOutOrganizationUnit.CarryOutOrganizationProductQuantity[];
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    BackendID: string;
    Name: string;
    Description: string;
    Type: OrganizationUnitTypes;
    IsPickup: boolean;
    Status: OrganizationUnitStatus;
    Address: AddressDto;
    Latitude?: number;
    Longitude?: number;
    EmailAddress: string;
    PhoneNumber: string;
    TimeZone: string;
    DistanceInKm?: number;
    OpeningHours: OpeningHoursInfo[];
  }

  export namespace ProductAvailabilityCarryOutOrganizationUnit {
    export interface CarryOutOrganizationProductQuantity {
      /**
       * Entity type: Product
       */
      ProductID: number;
      QuantityAvailable: number;
    }

  }

  /**
   * Calculates the run-rates for the provided products for the provided warehouse OrganizationUnit, based on a date (`Since`).
   * 
   * The run-rates are returned per SoldFromOrganizationUnit.
   */
  export interface GetProductRunRates {
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    /**
     * Entity type: OrganizationUnit
     */
    WarehouseOrganizationUnitID: number;
    Since: string;
  }

  export interface GetProductRunRatesResponse {
    TotalQuantity: number;
    DailyRunRate: number;
    MonthlyRunRate: number;
    RunRates: GetProductRunRatesResponse.RunRate[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetProductRunRatesResponse {
    export interface RunRate {
      ProductID: number;
      TotalQuantity: number;
      DailyRunRate: number;
      MonthlyRunRate: number;
      SoldFromOrganizationUnitID: number;
      SoldFromOrganizationUnitName: string;
    }

  }

  /**
   * Calculate and return the current `QuantityOnHand` for supplied `ProductID`s on given organization unit.
   * 
   * Returns a dictionary with `ProductID` as key, and the `QuantityOnHand` as value.
   */
  export interface GetQuantityOnHandForProducts {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
  }

  export interface GetQuantityOnHandForProductsResponse {
    Result: { [key: number]: number };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Calculates the stock availability for the provided OrderLines that are part of the provided Order.
   * 
   * The return value of this service is a list that contains, per OrderLine, whether or not it can be fulfilled (`CanBeFulfilled`) on the `RequestedDate`.
   * 
   * When an OrderLine doesn't already have a RequestedDate, a RequestedDate of tomorrow is used.
   * 
   * Additionally, a list of AlternativeDates is returned per OrderLine with dates on which it is expected that the OrderLine can be fulfilled.
   * 
   * When OrderLineIDs is left empty, this service will return an estimate for all unshipped lines of the order.
   * 
   * Doesn't work for orders with more than 25 lines.
   */
  export interface GetStockAvailabilityEstimateForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    OrderLines?: GetStockAvailabilityEstimateForOrder.OrderLine[];
  }

  export namespace GetStockAvailabilityEstimateForOrder {
    export interface OrderLine {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      RequestedDate?: string;
    }

  }

  export interface GetStockAvailabilityEstimateForOrderResponse {
    OrderLines: GetStockAvailabilityEstimateForOrderResponse.OrderLine[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetStockAvailabilityEstimateForOrderResponse {
    export interface OrderLine {
      OrderLineID: number;
      ProductID: number;
      RequestedDate: string;
      QuantityAvailable?: number;
      IsCommitted: boolean;
      ShipmentDate?: string;
      CanBeFulfilled: boolean;
      AlternativeDates: GetStockAvailabilityEstimateForOrderResponse.SuggestedAlternativeDate[];
      ExpectedFulfillmentDate?: string;
      AvailabilityStatus: OrderLineAvailabilityStatus;
      CommitmentDate?: string;
      ExportDate?: string;
      MadeToOrderStatus: GetStockAvailabilityEstimateForOrderResponse.OrderLineMadeToOrderStatus;
      DependsOnPurchaseOrderLines: GetStockAvailabilityEstimateForOrderResponse.DependsOnPurchaseOrderLine[];
      ShouldBeSplit: boolean;
      ExportStatus: OrderExportStatuses;
      CommitmentStatus: OrderLineCommitmentStatus;
      IsBlocked: boolean;
    }

    export interface DependsOnPurchaseOrderLine {
      ID: number;
      BackendID: string;
      ProductID: number;
      OrderID: number;
      QuantityToShip: number;
      QuantityShipped: number;
      QuantityDelivered: number;
      RequestedDate: string;
      CreationTime: string;
    }

    export interface OrderLineMadeToOrderStatus {
      PurchaseOrderLine: OrderLineMadeToOrderStatus.MadeToOrderPurchaseOrderLine;
      HasBeenRequested: boolean;
      HasBeenCompleted: boolean;
    }

    export namespace OrderLineMadeToOrderStatus {
      export interface MadeToOrderPurchaseOrderLine {
        ID: number;
        OrderID: number;
        RequestedDate?: string;
        IsOverdue: boolean;
        BackendID: string;
        QuantityToShip: number;
        QuantityShipped: number;
        QuantityDelivered: number;
        IsConfirmed: boolean;
        ShipFromOrganizationUnit: OrderLineMadeToOrderStatus.MadeToOrderShipFromOrganizationUnit;
      }

      export interface MadeToOrderShipFromOrganizationUnit {
        ID: number;
        Name: string;
      }

    }

    export interface PurchaseOrderLine {
      ID: number;
      OrderID: number;
      RequestedDate: string;
      Quantity: number;
    }

    export interface SuggestedAlternativeDate {
      Date: string;
      QuantityAvailable?: number;
      PurchaseOrderLines: GetStockAvailabilityEstimateForOrderResponse.PurchaseOrderLine[];
    }

  }

  export const enum OrderLineAvailabilityStatus {
    Unknown = 0,
    Blocked = 5,
    CannotFulfill = 10,
    CannotFulfillOnTime = 12,
    CanFulfill = 15
  }

  export const enum OrderExportStatuses {
    NotExported = 0,
    ShouldBeExported = 1,
    Exporting = 2,
    Exported = 3,
    WillNotBeExported = 4
  }

  /**
   * Given a set of products this service returns a detailed estimate for each product when it's expected to be available.
   */
  export interface GetStockAvailabilityEstimateForProducts {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Products: GetStockAvailabilityEstimateForProducts.Product[];
    /**
     * No longer supported.
     */
    IncludeReservedOrders?: boolean;
    /**
     * If set to true the service will also return purchase OrderLines meant for shops
     */
    IncludeShopReplenishments?: boolean;
  }

  export namespace GetStockAvailabilityEstimateForProducts {
    export interface Product {
      /**
       * Entity type: Product
       */
      ID: number;
      RequestedDate?: string;
    }

  }

  export interface GetStockAvailabilityEstimateForProductsResponse {
    Products: GetStockAvailabilityEstimateForProductsResponse.Product[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetStockAvailabilityEstimateForProductsResponse {
    export interface Product {
      ID: number;
      AvailabilityDates: GetStockAvailabilityEstimateForProductsResponse.AvailabilityDate[];
      WarehouseQuantityOnHand: number;
      WarehouseQuantityCommitted: number;
      WarehouseQuantityAvailable: number;
      QuantityToShipOnOrder: number;
      SalesOrderLines: GetStockAvailabilityEstimateForProductsResponse.SalesOrderLine[];
      PurchaseOrderLines: GetStockAvailabilityEstimateForProductsResponse.PurchaseOrderLine[];
      ShopReplenishments: GetStockAvailabilityEstimateForProductsResponse.ShopReplenishment[];
      WarehouseStock: GetStockAvailabilityEstimateForProductsResponse.WarehouseStock[];
      /**
       * No longer supported.
       */
      ReservedOrderLines: GetStockAvailabilityEstimateForProductsResponse.ReservedOrderLine[];
    }

    export interface AvailabilityDate {
      Date: string;
      QuantityAvailable?: number;
      PurchaseOrderLines: GetStockAvailabilityEstimateForProductsResponse.PurchaseOrderLine[];
    }

    export interface PurchaseOrderLine {
      ID: number;
      OrderID: number;
      RequestedDate: string;
      Quantity: number;
      IsOverdue: boolean;
      QuantityAvailable: number;
      IsConfirmed: boolean;
    }

    export interface SalesOrderLine {
      OrderLineID: number;
      OrderID: number;
      Quantity: number;
      ExportStatus: OrderExportStatuses;
      CommitmentStatus: OrderLineCommitmentStatus;
      CommitmentDate?: string;
      FulfillmentStatus: OrderLineAvailabilityStatus;
      ExportDate?: string;
      RequestedDate: string;
      OrderType: OrderTypes;
    }

    export interface ShopReplenishment {
      OrderLineID: number;
      OrderID: number;
      Quantity: number;
      RequestedDate: string;
      IsConfirmed: boolean;
    }

    export interface ReservedOrderLine {
      OrderLineID: number;
      OrderID: number;
      QuantityReserved: number;
      ShopName: string;
    }

    export interface WarehouseStock {
      StockLabel: EnumDto;
      QuantityOnHand: number;
      WarehouseID: number;
      WarehouseName: string;
    }

  }

  export const enum OrderTypes {
    Sales = 0,
    Purchase = 1
  }

  /**
   * Returns the expected availability timeline for the provided `ProductIDs`, from the perspective of the provided `OrganizationUnitID` (optional, defaults to the user's current OrganizationUnitID).
   * 
   * The timeline is ordered chronologically and includes all events that are relevant to the availability of the requested products, including events that happen on the suppliers of the requested OrganizationUnit (such as PurchaseOrder receipts).
   * 
   * Additionally, a `FromDate` (inclusive) and `ToDate` (exclusive) can be used to filter the response by date and a `Type` can be provided to only return timeline items of a certain type.
   */
  export interface GetStockAvailabilityTimeline {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    FromDate?: string;
    ToDate?: string;
    Type?: AvailabilityTimelineItemTypes;
  }

  export const enum AvailabilityTimelineItemTypes {
    CurrentAvailability = 0,
    ExpectedCommitment = 1,
    ExpectedReplenishment = 2,
    ExpectedShipment = 3
  }

  export interface GetStockAvailabilityTimelineResponse {
    TimelineItems: GetStockAvailabilityTimelineResponse.TimelineItem[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetStockAvailabilityTimelineResponse {
    export interface TimelineItem {
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      Type: AvailabilityTimelineItemTypes;
      Date: string;
      OrderID?: number;
      OrderLineID?: number;
      ProductID: number;
      DeltaQuantityOnHand: number;
      DeltaQuantityCommitted: number;
      CumulativeSumQuantityCommitted: number;
      CumulativeSumQuantityOnHand: number;
      CumulativeSumQuantityAvailable: number;
      RequestedDate?: string;
      OrderBackendID: string;
    }

  }

  /**
   * Requests the stock per stock label, per product for the given organization unit. If there is no stock at all for a product the product is not
   * returned in the response.
   */
  export interface GetStockByStockLabelForProducts {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    /**
     * Entity type: StockLabel
     */
    StockLabelIDs: number[];
  }

  export interface GetStockByStockLabelForProductsResponse {
    /**
     * Entity type: Product
     */
    Stock: { [key: number]: GetStockByStockLabelForProductsResponse.StockLabelStock[] };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetStockByStockLabelForProductsResponse {
    export interface StockLabelStock {
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
      QuantityOnHand: number;
    }

  }

  export interface GetStockDetailsForProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShowWarehousesForOrganizationUnitID?: number;
  }

  export interface GetStockDetailsForProductResponse {
    PromisedDeliveryDate?: string;
    PromisedDeliveryStatus: ProductPromisedDeliveryStatuses;
    QuantityOnHand: number;
    SellableQuantityAvailable: number;
    /**
     * Entity type: StockLabel
     */
    StockPerLabel: { [key: number]: number };
    SellableStockInStoresWithinApplication: number;
    SellableStockInWarehouses: OrganizationUnitWithStock[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrganizationUnitWithStock {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    Name: string;
    SellableStock: number;
  }

  export interface ProductWithDeliveryPromisedDelivery {
    PromisedDeliveryDate: string;
    PromisedDeliveryStatus: ProductPromisedDeliveryStatuses;
    PromisedDeliveryQuantity: number;
    PromisedDeliveryStatusID: number;
    /**
     * Entity type: Product
     */
    ID: number;
    BackendID: string;
    CustomID: string;
    BrandID?: number;
    BrandName: string;
    PrimitiveName: string;
    Type: ProductTypes;
    CatalogID?: number;
    LedgerClassID: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Properties: any;
  }

  export const enum ProductTypes {
    None = 0,
    Stock = 1,
    Insurance = 2,
    Marketing = 4,
    GiftCard = 8,
    Service = 16,
    GreetingCard = 32,
    CustomPricing = 64,
    External = 128,
    OrderCosts = 256,
    SystemGenerated = 512,
    ProductSet = 1024,
    BundleProduct = 2048,
    VirtualProduct = 4096,
    MadeToOrder = 8192,
    Configurable = 16384,
    SupplierProduct = 32768,
    Template = 65536,
    SecondChance = 131072,
    NotReturnable = 262144
  }

  /**
   * This service lists the OrderLines for the given ProductID that currently have committed Stock.
   */
  export interface ListCommittedOrderLines {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListCommittedOrderLinesResponse {
    Result?: PagedResult<CommittedOrderLine>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CommittedOrderLine {
    /**
     * Entity type: OrderLine
     */
    ID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrganizationUnit
     */
    SoldFromOrganizationUnitID: number;
    SoldFromOrganizationUnitName: string;
    Description: string;
    TotalQuantityToShip: number;
    CommitmentStatus: OrderLineCommitmentStatus;
    QuantityCommitted: number;
    RequestedDate?: string;
    CreationTime: string;
  }

  /**
   * The page size `Limit` has a maximum value of `1.000` for this service.
   */
  export interface ListStockMutations {
    PageConfig?: PageConfig<ListStockMutationsFilter>;
  }

  export interface ListStockMutationsFilter {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    CreationTime?: string;
    /**
     * Entity type: StockLabel
     */
    SourceStockLabelID?: number;
    /**
     * Entity type: StockLabel
     */
    DestinationStockLabelID?: number;
    /**
     * Entity type: StockMutationReasons
     */
    ReasonID?: number;
    StartDateTime?: string;
    EndDateTime?: string;
    MutationQuantity?: number;
  }

  export interface ListStockMutationsResponse {
    FullStockCountInProgress: boolean;
    FullStockCountIsExternal: boolean;
    Result: PagedResult<StockMutationDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StockMutationDto {
    /**
     * Entity type: StockMutation
     */
    ID: number;
    CreationTime: string;
    OrganizationUnit: OrganizationUnitDto;
    /**
     * Entity type: StockLabel
     */
    SourceStockLabelID: number;
    /**
     * Entity type: StockLabel
     */
    SourceStockLabel: number;
    /**
     * Entity type: StockLabel
     */
    DestinationStockLabelID: number;
    /**
     * Entity type: StockLabel
     */
    DestinationStockLabel: number;
    /**
     * Entity type: StockMutationReasons
     */
    ReasonID: number;
    Reason: EnumDto;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Product: ProductDto;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    OrderLine: OrderLineDto;
    MutationQuantity: number;
    Remark: string;
    UnitCost: number;
    AutomaticCorrectionQuantity: number;
    SerialNumber?: string;
    SerialNumberID?: number;
  }

  export interface ProductDto {
    /**
     * Entity type: Product
     */
    ID: number;
    BackendID: string;
    CustomID: string;
    BrandID?: number;
    BrandName: string;
    PrimitiveName: string;
    Type: ProductTypes;
    CatalogID?: number;
    LedgerClassID: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Properties: any;
  }

  export interface OrderLineDto {
    /**
     * Entity type: OrderLine
     */
    ID: number;
    BackendReference: string;
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    ParentID?: number;
    Children?: OrderLineDto[];
    Type: OrderLineTypes;
    /**
     * Entity type: StockLabel
     */
    StockLabel: number;
    Description: string;
    AdditionalDescription: string;
    /**
     * This is the quantity that was ordered. Make sure you really need this property, the total of the line is TotalQuantityToShip.
     */
    QuantityOrdered: number;
    QuantityOpen: number;
    QuantityCommitted: number;
    CommitmentStatus: OrderLineCommitmentStatus;
    QuantityShipped: number;
    QuantityDelivered: number;
    QuantityReserved: number;
    QuantityBackordered: number;
    QuantityReturned: number;
    QuantityCancelled: number;
    QuantityInvoiced: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    Product?: ProductDto;
    UnitPrice: number;
    UnitPriceBeforeCorrection?: number;
    UnitPriceBeforeCorrectionInTax?: number;
    UnitPriceCorrectionReason?: EnumDto;
    TaxRate: number;
    TaxCode: EnumDto;
    ShowTaxInfo: boolean;
    SerialNumber?: string;
    UnitPriceInTax: number;
    IsCancelled: boolean;
    TotalQuantityToShip: number;
    TotalAmountInTax: number;
    TotalAmount: number;
    NetTotalAmountInTax: number;
    NetTotalAmount: number;
    TotalTaxAmount: number;
    NetTotalTaxAmount: number;
    OriginalTotalAmountInTax?: number;
    OriginalTotalAmount?: number;
    EcoTaxInTax?: number;
    TotalEcoTaxInTax?: number;
    DisplayPriceInTax?: number;
    DisplayPrice?: number;
    ModificationAllowed: boolean;
    Remark: string;
    Reference: string;
    LineActionType: LineActionTypes;
    /**
     * Entity type: Order
     */
    ReturnedOrderID?: number;
    /**
     * Entity type: OrderLine
     */
    ReturnedOrderLineID?: number;
    ExportStatus: OrderExportStatuses;
    /**
     * @deprecated Replaced with `ExportStatus`
     */
    ExportStatusID: number;
    ContractNumber: string;
    IsCompleted: boolean;
    IsShipped: boolean;
    IsInvoiced: boolean;
    ExpectedAvailabilityDate?: string;
    RequestedDate?: string;
    CreationTime: string;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    LastModificationTime?: string;
    IsDigitalGiftCard: boolean;
    /**
     * Entity type: Discount
     */
    DiscountID?: number;
    DiscountBackendID: string;
    DiscountCouponCode: string;
    DiscountCampaignName: string;
    DiscountAmount: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    ShippingMethod?: ShippingMethodDto;
    ExternalModificationStatus: ExternalModificationStatuses;
    CustomOrderLineType: string;
    /**
     * Entity type: OrderLine
     */
    RelatedOrderLineID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    ProductVariation: { [key: string]: string };
    /**
     * Entity type: ProductBundleLine
     */
    ProductBundleLineID?: number;
    TaxExemptionCode: string;
    TaxExemptionReason: string;
    TaxExemptionVerification: string;
    FulfillmentStatus: OrderLineFulfillmentStatuses;
    AvailabilityStatus?: OrderLineAvailabilityStatus;
    ReturnReason?: EnumWithBackendIDDto;
    RefundCorrectionReason?: RefundCorrectionReasonDto;
    /**
     * Entity type: User
     */
    SoldByID?: number;
    /**
     * Properties that should be used for display in the frontends to keep a matching experience in all apps.
     */
    DisplayProperties: OrderLineDisplayProperties;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    /**
     * Entity type: PriceList
     */
    UnitPricePriceListID?: number;
    Discounts: OrderLineDiscount[];
    LoyaltyWithdrawals?: OrderLineLoyaltyMutation[];
    LoyaltyDeposits?: OrderLineLoyaltyMutation[];
    GroupID?: string;
    ImageBlobID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    FulfillmentOrganizationUnitID?: number;
    FulfillmentOrganizationUnitName?: string;
    FulfillmentOrganizationUnitBackendID?: string;
    IsPricedByLoyaltyProgram?: boolean;
    IsLimitedByLoyaltyProgram?: boolean;
    PreSaleDate?: string;
  }

  export const enum OrderLineTypes {
    NormalProduct = 0,
    Discount = 1,
    ExtraCosts = 2,
    ShippingCosts = 5,
    ReturnCosts = 6,
    PriceCorrection = 7,
    Service = 8,
    GiftWrappingCosts = 9,
    DownPayment = 10
  }

  export const enum LineActionTypes {
    None = 0,
    ReserveLine = 1,
    OrderLine = 2,
    ShipLine = 3,
    Delivery = 4
  }

  export interface ShippingMethodDto {
    /**
     * Entity type: ShippingMethod
     */
    ID: number;
    /**
     * Entity type: Carrier
     */
    CarrierID: number;
    Carrier: CarrierDto;
    Name: string;
    Description: string;
    Code: string;
    Priority: number;
    PaymentHandledByCarrier: boolean;
    DeliveryType: ShippingMethodDeliveryTypes;
    CustomFieldValues: { [key: number]: CustomFieldValue };
  }

  export interface CarrierDto {
    /**
     * Entity type: Carrier
     */
    ID: number;
    Name: string;
    Code: string;
  }

  export const enum ShippingMethodDeliveryTypes {
    None = 0,
    Default = 1,
    ShipFromStore = 2,
    Interbranch = 4
  }

  export const enum ExternalModificationStatuses {
    None = 0,
    CancellationRequested = 1,
    QuantityChangeRequested = 2,
    CancellationRequestDenied = 4,
    CancellationRequestApproved = 8,
    QuantityChangeRequestDenied = 16,
    QuantityChangeRequestApproved = 32
  }

  export const enum OrderLineFulfillmentStatuses {
    None = 0,
    WaitingForStock = 300,
    Pending = 400,
    InProgress = 500,
    Cancelled = 800,
    Completed = 1000
  }

  export interface EnumWithBackendIDDto {
    BackendID: string;
    /**
     * Entity type: Enum
     */
    ID: number;
    /**
     * Usually an internal & unchangable code - for display, use `Description`
     */
    Name: string;
    Description: string;
  }

  export interface RefundCorrectionReasonDto {
    /**
     * Entity type: RefundCorrectionReason
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
  }

  export interface OrderLineDisplayProperties {
    TotalAmountInTax?: number;
    TotalAmount?: number;
    OriginalAmountInTax?: number;
    OriginalAmount?: number;
    UnitPriceInTax?: number;
    UnitPrice?: number;
    OriginalUnitPrice?: number;
    OriginalUnitPriceInTax?: number;
    IsFree?: boolean;
    EcoTaxInTax?: number;
    TotalEcoTaxInTax?: number;
  }

  export interface OrderLineDiscount {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    DiscountAmount: number;
    Description: string;
    PromotionLabel: string;
  }

  export interface OrderLineLoyaltyMutation {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    BackendID: string;
    Name: string;
    Points: number;
    /**
     * Entity type: Discount
     */
    DiscountID?: number;
    /**
     * Entity type: PointAwardingRule
     */
    PointAwardingRuleID?: number;
  }

  export const enum OrderLineHoldStatus {
    None = 0,
    SerialNumberValidationRequired = 1
  }

  /**
   * Returns a paginated list of all stock, optionally filtered.
   */
  export interface ListStock {
    InitialPageConfig?: PageTokenConfig<ListStockFilter>;
    PageToken?: string;
  }

  export interface ListStockFilter {
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: StockLabel
     */
    StockLabelIDs?: number[];
  }

  export interface ListStockResponse {
    Results?: ListStockResponse.ListStockItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListStockResponse {
    export interface ListStockItem {
      Product: ListStockItem.ProductInfo;
      OrganizationUnit: ListStockItem.OrganizationUnitInfo;
      StockLabel: EnumDto;
      QuantityOnHand: number;
      QuantityCommitted: number;
    }

    export namespace ListStockItem {
      export interface ProductInfo {
        /**
         * Entity type: Product
         */
        ID?: number;
        BackendID?: string;
        CustomID?: string;
      }

      export interface OrganizationUnitInfo {
        /**
         * Entity type: OrganizationUnit
         */
        ID: number;
        BackendID?: string;
        Name: string;
        Type: OrganizationUnitTypes;
      }

    }

  }

  /**
   * Books a set of Movements.
   * 
   * A Movement should have a `ProductID` to identify the product, an `OrganizationUnitID` to identify the stock keeping
   * organization, a `SourceStockLabelID` and `DestinationStockLabelID` (see `GetEnumValues` with type `StockLabel`) and (of course) a `Quantity`
   * to actually book.
   * 
   * Supply a `BackendSystemID` on the request and `BackendID`'s on the Movements to make the request idempotent.
   * 
   * Optionally supports a `StockMutationReasonID` and `Remark` for each Movement.
   */
  export interface MoveStock {
    BackendSystemID?: string;
    Movements?: MoveStock.Movement[];
  }

  export namespace MoveStock {
    export interface Movement {
      BackendID?: string;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      /**
       * Entity type: StockLabel
       */
      SourceStockLabelID: number;
      /**
       * Entity type: StockLabel
       */
      DestinationStockLabelID: number;
      /**
       * Entity type: StockMutationReasons
       */
      StockMutationReasonID?: number;
      Quantity: number;
      Remark?: string;
      ProductBackendID?: string;
      SerialNumber?: string;
    }

  }

  /**
   * Processes a (delta)stock mutation file.
   * 
   * Expected format of the file is JSON with the following markup:
   * 
   * ```
   * [
   *   {
   *     "ProductID": [Optional, int],
   *     "ProductBackendID": [Optional, string],
   *     "QuantityOnHand": [Required, int, value >= 0],
   *     "StockMutationReasonID": [Optional, int],
   *     "Remark": [Optional, string]
   *   },
   *   ...
   * ]
   * ```
   * 
   * Mutation will only be processed for valid ProductIDs or ProductBackendIDs. In case ProductID and ProductBackendID are provided,
   * ProductBackendID will be selected as the search parameter.
   */
  export interface ProcessStockMutationFile {
    FileBlobID: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * Entity type: StockMutationReasons
     */
    StockMutationReasonID?: number;
    /**
     * The `BackendSystemID` that belongs to the `ProductBackendID`s
     */
    BackendSystemID?: string;
    /**
     * If set to false (default), only the stock for the products that are present in the file will be updated (delta).
     * 
     * If set to true, all the products will be updated - products not present in the file will be updated to quantity 0.
     * @default false
     */
    IsFullStockMutationFile?: boolean;
    /**
     * An email address to mail the processing result to. When left empty, the logged in user context will be used
     */
    EmailAddress?: string;
  }

  /**
   * Download a template format for the stock file to upload.
   */
  export interface DownloadStockExcelFileTemplate {
  }

  /**
   * Upload stock Excel (delta)file. The service processes the data async. This service is intended for small files (up to 5MB). Please view the documentation for processing larger files.
   */
  export interface UploadAndProcessStockExcelFile {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * Entity type: StockMutationReasons
     */
    StockMutationReasonID?: number;
    /**
     * If set to false (default), only the stock for the products that are present in the file will be updated (delta).
     * 
     * If set to true, all the products will be updated - products not present in the file will be updated to quantity 0.
     * @default false
     */
    IsFullStockMutationFile?: boolean;
    Data: string;
    /**
     * The `BackendSystemID` that belongs to the `ProductBackendID`s
     */
    BackendSystemID?: string;
    /**
     * An email address to mail the processing result to. When left empty, the logged in user context will be used
     */
    EmailAddress?: string;
  }

  /**
   * Upload stock JSON (delta)file. The service processes the data async.This service is intended for small files (up to 5MB). Please view the documentation for processing larger files.
   */
  export interface UploadAndProcessStockJSONFile {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * Entity type: StockMutationReasons
     */
    StockMutationReasonID?: number;
    /**
     * If set to false (default), only the stock for the products that are present in the file will be updated (delta).
     * 
     * If set to true, all the products will be updated - products not present in the file will be updated to quantity 0.
     * @default false
     */
    IsFullStockMutationFile?: boolean;
    /**
     * Should be a JSON formatted array of object with these 3 properties;
     * 
     * - `ProductID` (optional, int)
     * - `ProductBackendID` (optional, string)
     * - `QuantityOnHand` (required, int, value >= 0)
     * - `StockMutationReasonID` (optional, int)
     * - `Remark` (optional, string)
     * 
     * Mutation will only be processed for valid ProductID's or ProductBackendID's. In case ProductID and ProductBackendID are provided,
     * ProductBackendID will be selected as the search parameter.
     */
    Data: string;
    /**
     * The `BackendSystemID` that belongs to the `ProductBackendID`s
     */
    BackendSystemID?: string;
    /**
     * An email address to mail the processing result to. When left empty, the logged in user context will be used
     */
    EmailAddress?: string;
  }

  /**
   * Get stock labels
   */
  export interface GetStockLabels {
  }

  export interface GetStockLabelsResponse {
    StockLabels: GetStockLabelsResponse.StockLabelDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetStockLabelsResponse {
    export interface StockLabelDto {
      /**
       * Entity type: StockLabel
       */
      ID: number;
      Name: string;
      DisplayName: string;
      Description: string;
    }

  }

  /**
   * Returns all stocklabels with the configured setting for a single organizationunit.
   */
  export interface GetStockLabelsForFeature {
    Feature: StockLabelFeature;
    /**
     * Optional organizationunit to filter by. If not set, the stocklabels for the current organizationunit will be returned.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export const enum StockLabelFeature {
    Sellable = 0,
    Movable = 1,
    Adjustable = 2,
    Returnable = 3,
    CycleCountable = 4,
    FullStockCountable = 5,
    ReturnToSupplierable = 6
  }

  export interface GetStockLabelsForFeatureResponse {
    StockLabels: StockLabelDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StockLabelDto {
    ID: number;
    Name: string;
    DisplayName: string;
    Description: string;
    IsDefault: boolean;
  }

  /**
   * Returns all stocklabels with the configured setting for a single organizationunit.
   */
  export interface GetStockLabelsForFeatures {
    /**
     * Optional organizationunit to filter by. If not set, the stocklabels for the current organizationunit will be returned.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface GetStockLabelsForFeaturesResponse {
    Sellable: StockLabelDto[];
    Movable: StockLabelDto[];
    Adjustable: StockLabelDto[];
    Returnable: StockLabelDto[];
    CycleCountable: StockLabelDto[];
    FullStockCountable: StockLabelDto[];
    ReturnToSupplierable: StockLabelDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns all organizationunits with the configured setting for a single stocklabel.
   * 
   * Every organizationunit that has any setting set for the stocklabel will be returned.
   * If an organizationunit has no setting for a specific feature, it will inherit the setting from it’s parent, this is indicated by the `IsInherited` property.
   */
  export interface GetStockLabelSettingsForLabel {
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
  }

  export interface GetStockLabelSettingsForLabelResponse {
    StockLabelSettings: GetStockLabelSettingsForLabelResponse.StockLabelSettingDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetStockLabelSettingsForLabelResponse {
    export interface StockLabelSettingDto {
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      /**
       * The name of the organizationunit.
       */
      OrganizationUnitName: string;
      /**
       * Indicates that these are the default settings for the stocklabel. These cannot be changed.
       */
      IsDefault: boolean;
      Sellable: GetStockLabelSettingsForLabelResponse.InheritedValue<boolean>;
      Movable: GetStockLabelSettingsForLabelResponse.InheritedValue<boolean>;
      Adjustable: GetStockLabelSettingsForLabelResponse.InheritedValue<boolean>;
      Returnable: GetStockLabelSettingsForLabelResponse.InheritedValue<boolean>;
      CycleCountable: GetStockLabelSettingsForLabelResponse.InheritedValue<boolean>;
      FullStockCountable: GetStockLabelSettingsForLabelResponse.InheritedValue<boolean>;
      ReturnToSupplierable: GetStockLabelSettingsForLabelResponse.InheritedValue<boolean>;
    }

    export interface InheritedValue<T> {
      Value: T;
      IsDefault: boolean;
      /**
       * Whether this value is inherited from a parent organization unit.
       */
      IsInherited: boolean;
    }

  }

  /**
   * Change the settings for a specific OU/StockLabel combination. Use a value of null to make it inherit from it’s parent.
   */
  export interface SetStockLabelSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    Movable?: boolean;
    Adjustable?: boolean;
    CycleCountable?: boolean;
    FullStockCountable?: boolean;
    Sellable?: boolean;
    IsDefaultSellable?: boolean;
    Returnable?: boolean;
    IsDefaultReturnable?: boolean;
    ReturnToSupplierable?: boolean;
    IsDefaultReturnToSupplierable?: boolean;
  }

  /**
   * Retrieve a StockNotification for a product and organization, for an EmailAddress - will return its ID, or null when it does not exist.
   */
  export interface GetStockNotification {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    EmailAddress: string;
  }

  export interface StockNotificationResponse {
    /**
     * Entity type: StockNotification
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Retrieve a StockNotification for a product and organization, for the current user - will return its ID, or null when it does not exist.
   */
  export interface GetStockNotificationForCurrentUser {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  /**
   * Creates a StockNotification for a product and organization, for specified EmailAddress - will return its ID.
   */
  export interface CreateStockNotification {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    EmailAddress: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
  }

  /**
   * Creates a StockNotification for a product and organization (or current when null), for the current user - will return its ID.
   */
  export interface CreateStockNotificationForCurrentUser {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
  }

  /**
   * Deletes a StockNotification by ID. When it was created with an EmailAddress, repeat that in this request by means of authentication.
   */
  export interface DeleteStockNotification {
    /**
     * Entity type: StockNotification
     */
    ID: number;
    EmailAddress?: string;
  }

  /**
   * Updates the Sellable stock of the given products for specified `OrganizationUnitID`, or when omitted, your current organization unit.
   * 
   * Specify the new, absolute quantities as values in the `Products` list. Products that are not mentioned will not be modified.
   */
  export interface UpdateStock {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Products: UpdateStock.ProductDto[];
  }

  export namespace UpdateStock {
    export interface ProductDto {
      /**
       * Entity type: Product
       * 
       * Allows missing/invalid values when using EVA-IDs-Mode: ExternalIDs.
       */
      ID: number;
      Quantity: number;
    }

    export const enum UpdateStockIdentifier {
      ID = 0,
      BackendID = 1
    }

  }

  /**
   * Cancels shipments without receipt of goods, also removing stock in transit.
   */
  export interface CancelShipments {
    /**
     * Entity type: Shipment
     */
    ShipmentIDs: number[];
  }

  export interface CancelShipmentsResponse {
    Total: number;
    Errors: number;
    Message: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create a new shipment
   * 
   * It is required to specify an Order. This can be done with `OrderID` or by sending `OrderBackendID` and `OrderBackendSystemID`.
   * 
   * To create a pending Shipment, set `IsPending` to true. A pending shipment has to be shipped with the ShipShipment service.
   */
  export interface CreateShipment {
    /**
     * ID of the order that is being shipped - required when `OrderBackendID` and `OrderBackendSystemID` are omitted
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderFulfillment
     */
    OrderFulfillmentID?: number;
    /**
     * Reference to the order that is being shipped - required when `OrderID` is omitted
     */
    OrderBackendID?: string;
    /**
     * Reference to the system from which the order originated - required when `OrderID` is omitted
     */
    OrderBackendSystemID?: string;
    /**
     * Optional, will be determined based on the Order
     * Entity type: OrganizationUnit
     */
    ShippedFromOrganizationUnitID?: number;
    ShipmentDate?: string;
    ExpectedDeliveryDate?: string;
    /**
     * Optional. When only lines are provided, the PackageID can be used to create a package for all lines.
     * Entity type: Package
     */
    PackageID?: number;
    Lines?: CreateShipment.ShipmentLine[];
    Packages?: CreateShipment.ShipmentPackage[];
    BackendID?: string;
    BackendSystemID?: string;
    TrackingCode?: string;
    TrackingLink?: string;
    ReceiveMethod?: ShipmentReceiveMethods;
    IsPending?: boolean;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    ShipmentIdentifiers?: CreateShipment.ShipmentIdentifier[];
  }

  export namespace CreateShipment {
    export interface ShipmentIdentifier {
      /**
       * Entity type: ShipmentIdentifier
       */
      ID: number;
      Value: string;
    }

    export interface ShipmentPackage {
      /**
       * Entity type: Package
       */
      PackageID: number;
      Lines: CreateShipment.ShipmentLine[];
    }

    export interface ShipmentLine {
      /**
       * Entity type: Order
       */
      OrderID?: number;
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Entity type: OrderFulfillmentLine
       */
      OrderFulfillmentLineID?: number;
      OrderLineBackendID?: string;
      /**
       * When not provided, the QuantityOpen of the OrderLine will be used
       */
      Quantity?: number;
      BackendReference: string;
    }

  }

  export const enum ShipmentReceiveMethods {
    Manual = 0,
    Automatic = 1,
    UserDefined = 2
  }

  export interface CreateShipmentResponse {
    /**
     * Entity type: Shipment
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Delete an existing shipment that has status Pending.
   * 
   * Works only for SalesOrders at this moment.
   */
  export interface DeleteShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    BackendID?: string;
    BackendSystemID?: string;
  }

  /**
   * Delivers an undelivered SalesOrder shipment. This will give both the shipment and the orderline the status that they have been delivered.
   */
  export interface DeliverShipment {
    /**
     * Either this or ShipmentBackendID is required.
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * Either this or ShipmentID is required. If ShipmentBackendID is specified, ShipmentBackendSystem is also required.
     */
    ShipmentBackendID?: string;
    /**
     * Required if ShipmentBackendID is specified.
     */
    ShipmentBackendSystemID?: string;
  }

  export interface GetPurchaseOrderShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
  }

  export interface GetPurchaseOrderShipmentResponse {
    Shipment: PurchaseOrderShipmentDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface PurchaseOrderShipmentDto {
    /**
     * Entity type: Shipment
     */
    ID: number;
    IsDeleted: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    ShipmentDate?: string;
    OriginalDeliveryDate?: string;
    ExpectedDeliveryDate?: string;
    BackendID: string;
    BackendSystemID: string;
    TrackingCode: string;
    TrackingLink: string;
    CreationTime: string;
    TotalQuantityShipped: number;
    TotalQuantityDelivered: number;
    NetTotalQuantityShipped: number;
    NetTotalQuantityDelivered: number;
    IsCompleted: boolean;
    StatusID: number;
    TotalPackageCountIndication: number;
    Orders: PurchaseOrderShipmentDto.OrderDto[];
    ReceiveMethodID?: number;
    ReceiptCreatedBy: string;
    ReceiptCreatedOn?: string;
    /**
     * Indicates that this shipment has lines where the delivered quantity and the shipped quantity differ.
     */
    HasDeviations: boolean;
    ShipmentIdentifiers: PurchaseOrderShipmentDto.ShipmentIdentifier[];
  }

  export namespace PurchaseOrderShipmentDto {
    export interface ShipmentIdentifier {
      /**
       * Entity type: ShipmentIdentifier
       */
      ID: number;
      Name: string;
      Value: string;
      Type: ShipmentIdentifierTypes;
    }

    export interface OrderDto {
      /**
       * Entity type: Order
       */
      ID: number;
      BackendID: string;
      DisplayID?: string;
    }

  }

  export const enum ShipmentIdentifierTypes {
    Crate = 0,
    Seal = 1
  }

  /**
   * Get a Shipment by its ID, returning information about where it came from, where it's shipped to, its shipping or delivery status and payment status
   * of the order(s) contained in the shipment.
   */
  export interface GetShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
  }

  export interface GetShipmentResponse {
    /**
     * Entity type: Shipment
     */
    ID: number;
    ShippedToOrganizationUnit: GetShipmentResponse.OrganizationUnit;
    ShippedFromOrganizationUnit?: GetShipmentResponse.OrganizationUnit;
    ShipmentDate?: string;
    OriginalDeliveryDate?: string;
    ExpectedDeliveryDate?: string;
    BackendID: string;
    BackendSystemID: string;
    TrackingCode: string;
    TrackingLink: string;
    TotalQuantityShipped: number;
    NetTotalQuantityShipped: number;
    TotalQuantityDelivered: number;
    NetTotalQuantityDelivered: number;
    LastDeliveryDate?: string;
    Packages: GetShipmentResponse.ShipmentPackage[];
    ReceiveMethod?: ShipmentReceiveMethods;
    Status: ShipmentStatuses;
    IsCompleted: boolean;
    IsDelivered: boolean;
    IsExported: boolean;
    CompletionTime?: string;
    OrderType: OrderTypes;
    OrdersInShipment: GetShipmentResponse.OrderInShipment[];
    PaymentHandledByCarrier: boolean;
    LineCount: number;
    IsDeleted: boolean;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    ShipmentIdentifiers: GetShipmentResponse.ShipmentIdentifier[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetShipmentResponse {
    export interface ShipmentIdentifier {
      /**
       * Entity type: ShipmentIdentifier
       */
      ID: number;
      Name: string;
      Value: string;
      RequiredFor: ShipmentIdentifierRequiredFor;
      Type: ShipmentIdentifierTypes;
    }

    export interface OrganizationUnit {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
      Type: OrganizationUnitTypes;
    }

    export interface ShipmentPackage {
      ID: number;
      Description: string;
      Quantity: number;
      Length: number;
      Width: number;
      Height: number;
      Weight: number;
    }

    export interface OrderInShipment {
      /**
       * Entity type: Order
       */
      OrderID?: number;
      OrderDisplayID?: string;
      TotalAmountInTax?: number;
      PaidAmount?: number;
      OpenAmount?: number;
    }

  }

  export const enum ShipmentStatuses {
    Open = 0,
    Completed = 10,
    Cancelled = 11,
    Pending = -1
  }

  export const enum ShipmentIdentifierRequiredFor {
    None = 0,
    RTS = 1,
    PurchaseOrder = 2
  }

  export interface ListPurchaseOrderShipments {
    PageConfig?: PageConfig<ListPurchaseOrderShipmentsFilter>;
  }

  export interface ListPurchaseOrderShipmentsFilter {
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    OrderBackendID?: string;
    OrderBackendIDs?: string[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    IncludeChildOrganizationUnits: boolean;
    BackendID?: string;
    ShipmentBackendIDs?: string[];
    TrackingCode: string;
    IsCompleted?: boolean;
    StatusID?: number;
    StatusIDs: number[];
    /**
     * Only return shipments that have deviations. That is: shipments where the delivered quantity and the shipped quantity differ on any line.
     */
    HasDeviations?: boolean;
    ReceiveMethod?: ShipmentReceiveMethods;
  }

  export interface ListPurchaseOrderShipmentsResponse {
    Result?: PagedResult<PurchaseOrderShipmentDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Retrieves a paged list of Shipment Ledgers for specified ShipmentID.
   */
  export interface ListShipmentLedgerForShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    PageConfig?: PageConfig<ListShipmentLedgerFilter>;
  }

  export interface ListShipmentLedgerFilter {
    /**
     * Entity type: ShipmentLedgerType
     */
    TypeIDs: number[];
  }

  export interface ListShipmentLedgerForShipmentResponse {
    Result?: PagedResult<ListShipmentLedgerForShipmentResponse.ShipmentLedgerDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListShipmentLedgerForShipmentResponse {
    export interface ShipmentLedgerDto {
      /**
       * Entity type: ShipmentLedger
       */
      ID: number;
      /**
       * Entity type: Shipment
       */
      ShipmentID: number;
      Type: EnumDto;
      /**
       * Entity type: ShipmentLedgerType
       */
      TypeID: number;
      OldValue: string;
      NewValue: string;
      Description: string;
      /**
       * Entity type: User
       */
      CreatedByID: number;
      CreationTime: string;
      CreatedBy: ListShipmentLedgerForShipmentResponse.UserDto;
    }

    export interface UserDto {
      /**
       * Entity type: User
       */
      ID: number;
      FullName: string;
    }

  }

  /**
   * Returns serial numbers on a shipment line with pagination and filtering capabilities.
   */
  export interface ListShipmentLineSerialNumbers {
    /**
     * Entity type: ShipmentLine
     */
    ShipmentLineID: number;
    PageConfig?: PageConfig<ListShipmentLineSerialNumbersFilter>;
  }

  export interface ListShipmentLineSerialNumbersFilter {
    /**
     * Filter serial numbers containing this text
     */
    SerialNumber?: string;
  }

  export interface ListShipmentLineSerialNumbersResponse {
    Result?: PagedResult<ListShipmentLineSerialNumber>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListShipmentLineSerialNumber {
    SerialNumber: string;
  }

  /**
   * List shipmentlines of a shipment, this service replaces the GetShipmentLines and the ListPurchaseOrderShipmentLines services.
   * 
   * The page size `Limit` has a maximum value of `10.000` for this service.
   */
  export interface ListShipmentLines {
    PageConfig?: PageConfig<ListShipmentLinesFilter>;
  }

  export interface ListShipmentLinesFilter {
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * Entity type: ShipmentPackage
     */
    ShipmentPackageID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    /**
     * Entity type: Order
     */
    OrderIDs: number[];
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs: number[];
    ExcludeEmptyLines?: boolean;
  }

  export interface ListShipmentLinesResponse {
    Result?: PagedResult<ShipmentLineDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ShipmentLineDto {
    /**
     * Entity type: ShipmentLine
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    ProductBackendID: string;
    ProductCustomID: string;
    Type: OrderLineTypes;
    Description: string;
    BackendReference: string;
    TotalQuantityToShip: number;
    QuantityShipped: number;
    QuantityDelivered: number;
    CurrencyID: string;
    UnitPrice: number;
    IsCompleted: boolean;
    TotalAmount: number;
    TotalAmountInTax: number;
    UnitPriceInTax: number;
    TaxRate: number;
    DeliveryType: ShipmentLineDeliveryTypes;
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    ShipmentDate?: string;
    NetQuantityShipped: number;
    NetQuantityDelivered: number;
    ShipmentPackageID?: number;
    HasSerialNumbers: boolean;
  }

  export const enum ShipmentLineDeliveryTypes {
    Default = 0,
    Surplus = 1,
    Deficiency = 2
  }

  /**
   * Lists shipment packages for a specified shipment
   */
  export interface ListShipmentPackages {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    PageConfig?: PageConfig<ListShipmentPackagesFilter>;
  }

  export interface ListShipmentPackagesFilter {
  }

  export interface ListShipmentPackagesResponse {
    Result?: PagedResult<ShipmentPackageDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ShipmentPackageDto {
    ID: number;
    Package: ShipmentPackageDto.PackageDto;
  }

  export namespace ShipmentPackageDto {
    export interface PackageDto {
      ID: number;
      Description: string;
      Length: number;
      Width: number;
      Height: number;
      Weight: number;
    }

  }

  /**
   * Modify the QuantityShipped on one or more ShipmentLines of a Shipment.
   * 
   * When the new QuantityShipped is 0 it will be cancelled.
   */
  export interface ModifyQuantityShipped {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    Lines: ModifyQuantityShipped.ShipmentLine[];
  }

  export namespace ModifyQuantityShipped {
    export interface ShipmentLine {
      /**
       * Entity type: ShipmentLine
       */
      ShipmentLineID: number;
      QuantityShipped: number;
    }

  }

  export interface ProducePackingSlip {
    /**
     * Entity type: Station
     */
    StationID?: number;
    Download?: boolean;
    Print?: boolean;
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
  }

  export interface ProducePackingSlipResponse {
    ResourceInfo: ResourceInfo;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ProducePickSlip {
    /**
     * Entity type: Order
     */
    OrderIDs: number[];
    /**
     * Entity type: Station
     */
    StationID?: number;
    Print?: boolean;
  }

  export interface ProducePickSlipResponse {
    ResourceInfos: ResourceInfo[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListShipmentExportsForShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListShipmentExportsForShipmentResponse {
    Result?: PagedResult<ListShipmentExportsForShipmentResponse.ShipmentExportDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListShipmentExportsForShipmentResponse {
    export interface ShipmentExportDto {
      ID: number;
      Name: string;
      Status: TransputJobStatuses;
      CreationTime: string;
      LastModificationTime?: string;
    }

  }

  export interface GetShipmentExportByID {
    /**
     * Entity type: ShipmentExport
     */
    ID: number;
  }

  export interface GetShipmentExportByIDResponse {
    ID: number;
    ShipmentID: number;
    Name: string;
    Status: TransputJobStatuses;
    ShippedFromOrganizationUnitID?: number;
    ShippedFromOrganizationUnitName: string;
    ShippedToOrganizationUnitID: number;
    ShippedToOrganizationUnitName: string;
    CreationTime: string;
    LastModificationTime?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface RetryShipmentExport {
    /**
     * Entity type: ShipmentExport
     */
    ID: number;
    Force?: boolean;
  }

  export interface GetShipmentExportDocuments {
    /**
     * Entity type: ShipmentExport
     */
    ID: number;
  }

  export interface GetShipmentExportDocumentsResponse {
    RequestDocuments: TransputJobDocument[];
    ResponseDocuments: TransputJobDocument[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface TransputJobDocument {
    CreationTime: string;
    Document: string;
  }

  /**
   * Get a shipment identifier by ID.
   */
  export interface GetShipmentIdentifier {
    /**
     * Entity type: ShipmentIdentifier
     */
    ID: number;
  }

  export interface GetShipmentIdentifierResponse {
    /**
     * Entity type: ShipmentIdentifier
     */
    ID: number;
    Name: string;
    Type: ShipmentIdentifierTypes;
    RequiredFor: ShipmentIdentifierRequiredFor;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List shipment identifiers.
   */
  export interface ListShipmentIdentifiers {
  }

  export interface ListShipmentIdentifiersResponse {
    ShipmentIdentifiers: ListShipmentIdentifiersResponse.ShipmentIdentifierDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListShipmentIdentifiersResponse {
    export interface ShipmentIdentifierDto {
      /**
       * Entity type: ShipmentIdentifier
       */
      ID: number;
      Name: string;
      Type: ShipmentIdentifierTypes;
      RequiredFor: ShipmentIdentifierRequiredFor;
    }

  }

  /**
   * Delete an existing shipment identifier.
   */
  export interface DeleteShipmentIdentifier {
    /**
     * Entity type: ShipmentIdentifier
     */
    ID: number;
  }

  /**
   * Update an existing shipment identifier.
   */
  export interface UpdateShipmentIdentifier {
    /**
     * Entity type: ShipmentIdentifier
     */
    ID: number;
    Name?: string;
    Type?: ShipmentIdentifierTypes;
    RequiredFor?: ShipmentIdentifierRequiredFor;
  }

  /**
   * Create a new shipment identifier.
   */
  export interface CreateShipmentIdentifier {
    Name: string;
    Type: ShipmentIdentifierTypes;
    /**
     * Which flow this identifier should be required for.
     */
    RequiredFor: ShipmentIdentifierRequiredFor;
  }

  export interface CreateShipmentIdentifierResponse {
    /**
     * Entity type: ShipmentIdentifier
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This service is used in combination with the `GetShipmentSettings` to manage OrganizationUnit settings for Shipments.
   * Optionally an OrganizationUnitID can be provided, but by default the settings are updated for the current OrganizationUnit.
   * 
   * - DefaultReceiveMethod: How shipments should be received if no custom implementation is available
   */
  export interface SetShipmentSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    DefaultReceiveMethod?: ShipmentReceiveMethods;
  }

  /**
   * @see: SetShipmentSettings
   */
  export interface GetShipmentSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface GetShipmentSettingsResponse {
    OrganizationUnitID: number;
    DefaultReceiveMethod: ShipmentReceiveMethods;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Ship an existing shipment that has status Pending.
   * 
   * Works only for SalesOrders at this moment.
   */
  export interface ShipShipment {
    BackendID?: string;
    BackendSystemID?: string;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * The moment the shipment was shipped, will be set to DateTime.Now when not set
     */
    ShipmentDate?: string;
    TrackingCode?: string;
    TrackingLink?: string;
    /**
     * Allow to force shipment. Requires additional rights
     */
    Force?: boolean;
  }

  /**
   * Removes stock from transit for an unreceived shipment
   */
  export interface UndoShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * The reason for undoing the shipment
     */
    Remark?: string;
  }

  /**
   * Removes stock for a received shipment
   */
  export interface UndoShipmentReceipt {
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * The reason for undoing the shipment
     */
    Remark?: string;
  }

  /**
   * Update a Shipment. For shipments with status > Pending, only `ReceiveMethod`, `ShipmentDate` and `ExpectedDeliveryDate` can be updated.
   * Updating a shipment creates the new necessary shipping lines and packages.
   * Using the RemoveOtherPackagesAndLines property removes the existing ones.
   */
  export interface UpdateShipment {
    BackendID?: string;
    BackendSystemID?: string;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    ReceiveMethod?: ShipmentReceiveMethods;
    /**
     * Explanation of why the receive method has been changed
     */
    ReceiveMethodChangedReason?: string;
    ShipmentDate?: string;
    ExpectedDeliveryDate?: string;
    /**
     * Optional. When only lines are provided, the PackageID can be used to create a package for all lines.
     * Entity type: Package
     */
    PackageID?: number;
    Lines?: UpdateShipment.ShipmentLine[];
    Packages?: UpdateShipment.ShipmentPackage[];
    TrackingCode?: string;
    TrackingLink?: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    /**
     * Overwrite the existing Lines and Packages, instead of adding to the existing ones
     * @default false
     */
    RemoveOtherPackagesAndLines?: boolean;
    ShipmentIdentifiers?: UpdateShipment.ShipmentIdentifier[];
  }

  export namespace UpdateShipment {
    export interface ShipmentIdentifier {
      /**
       * Entity type: ShipmentIdentifier
       */
      ID: number;
      Value: string;
    }

    export interface ShipmentPackage {
      /**
       * Entity type: ShipmentPackage
       */
      ID?: number;
      /**
       * Entity type: Package
       */
      PackageID: number;
      Lines: UpdateShipment.ShipmentLine[];
    }

    export interface ShipmentLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      OrderLineBackendID?: string;
      /**
       * When not provided, the QuantityOpen of the OrderLine will be used
       */
      Quantity?: number;
      BackendReference?: string;
    }

  }

  /**
   * Updates the shipment status, creating ledger entries on shipment level.
   * 
   * Will be processed as a whole; a single failing line in `Lines` will prevent all other updates in a single call from persisting.
   */
  export interface UpdateShipmentStatus {
    Lines: UpdateShipmentStatusLine[];
  }

  export interface UpdateShipmentStatusLine {
    /**
     * Identifier of the EVA shipment this update is specific for
     * Entity type: Shipment
     */
    ShipmentID: number;
    /**
     * Type of shipment status update; 0 = CarrierStatus
     */
    Type: ShipmentStatusType;
    /**
     * Date the event occurred - leave empty for 'now'.
     */
    Date?: string;
    /**
     * Optional, additional information, most commonly for the carrier status update code (B01, D01, ...) for CarrierStatus types
     */
    Value?: string;
    /**
     * Optional, additional description, most commonly for the carrier status update code description for CarrierStatus types
     */
    Description?: string;
  }

  export const enum ShipmentStatusType {
    CarrierStatus = 0
  }

  /**
   * Validates if a Shipment can be shipped in the current state.
   * 
   * Returns if it is valid (`IsValid`) and if not, how much still needs to be paid (`OpenAmountForShipment`).
   */
  export interface ValidateShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
  }

  export interface ValidateShipmentResponse {
    PaidAmount: number;
    OpenAmountForShipment: number;
    IsValid: boolean;
    ShipmentStatus: ShipmentStatuses;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List serial number registrations for a serial number.
   */
  export interface ListSerialNumberRegistrations {
    PageConfig?: PageConfig<ListSerialNumberRegistrationsFilter>;
  }

  export interface ListSerialNumberRegistrationsFilter {
    SerialNumber: string;
  }

  export interface ListSerialNumberRegistrationsResponse {
    Result?: PagedResult<ListSerialNumberRegistrationsResponse.SerialNumberRegistrationModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListSerialNumberRegistrationsResponse {
    export interface SerialNumberRegistrationModel {
      /**
       * Entity type: SerialNumberRegistration
       */
      ID?: number;
      /**
       * Entity type: User
       */
      CreatedByID?: number;
      CreationTime?: string;
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Entity type: CaseRelatedItem
       */
      CaseRelatedItemID?: number;
      /**
       * Entity type: Repair
       */
      RepairID?: number;
      /**
       * Entity type: UserBoughtProduct
       */
      UserBoughtProductID?: number;
      /**
       * Entity type: FullStockCountLabelProduct
       */
      FullStockCountLabelProductID?: number;
      /**
       * Entity type: ZonedCycleCountPreCountResult
       */
      ZonedCycleCountPreCountResultID?: number;
      /**
       * Entity type: ShipmentLine
       */
      ShipmentLineID?: number;
    }

  }

  export const enum SerialNumberRegistrationStatus {
    Pending = 0,
    Verified = 1,
    Rejected = -1
  }

  /**
   * Creates a new entity for Serial Number Configuration, defining a set of requirements where linked products should adhere to.
   */
  export interface CreateSerialNumberConfiguration {
    Name?: string;
    Description?: string;
    ValidationRegex?: string;
    IsStrict?: boolean;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Requirement?: SerialNumberConfigurationRequirement;
  }

  export const enum SerialNumberConfigurationRequirement {
    Sales = 0,
    OwnershipChange = 1,
    StockMovements = 2,
    None = -1
  }

  export interface CreateSerialNumberConfigurationResponse {
    /**
     * Entity type: SerialNumberConfiguration
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates an existing Serial Number Configuration by it's `ID`.
   */
  export interface UpdateSerialNumberConfiguration {
    /**
     * Entity type: SerialNumberConfiguration
     */
    ID: number;
    Name?: string;
    Description?: string;
    ValidationRegex?: string;
    IsStrict?: boolean;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Requirement?: SerialNumberConfigurationRequirement;
  }

  /**
   * Get all Serial Number Configurations, without paging.
   */
  export interface GetSerialNumberConfigurations {
  }

  export interface GetSerialNumberConfigurationsResponse {
    SerialNumberConfigurations: GetSerialNumberConfigurationsResponse.SerialNumberConfigurationDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetSerialNumberConfigurationsResponse {
    export interface SerialNumberConfigurationDto {
      /**
       * Entity type: SerialNumberConfiguration
       */
      ID?: number;
      Name?: string;
      Description?: string;
      ValidationRegex?: string;
      IsStrict?: boolean;
      /**
       * Entity type: Script
       */
      ScriptID?: number;
      Requirement?: SerialNumberConfigurationRequirement;
    }

  }

  /**
   * Deletes an existing Serial Number Configuration by it's `ID` if this is not linked to any existing Serial Number Product.
   */
  export interface DeleteSerialNumberConfiguration {
    /**
     * Entity type: SerialNumberConfiguration
     */
    ID: number;
  }

  /**
   * Lists all products associated with a Serial Number Configuration.
   */
  export interface ListSerialNumberProducts {
    PageConfig?: PageConfig<ListSerialNumberProductsFilter>;
  }

  export interface ListSerialNumberProductsFilter {
    /**
     * Entity type: SerialNumberConfiguration
     */
    SerialNumberConfigurationID: number;
  }

  export interface ListSerialNumberProductsResponse {
    Result?: PagedResult<ListSerialNumberProductsResponse.SerialNumberProductModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListSerialNumberProductsResponse {
    export interface SerialNumberProductModel {
      /**
       * Entity type: Product
       */
      ProductID: number;
    }

  }

  /**
   * Associate a product with a Serial Number Configuration, or re-associate with this Serial Number Configuration in case it was previously assigned to another.
   */
  export interface CreateSerialNumberProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: SerialNumberConfiguration
     */
    SerialNumberConfigurationID: number;
  }

  /**
   * Removes a product association with a Serial Number Configuration.
   */
  export interface DeleteSerialNumberProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  /**
   * Create a new Serial Number
   */
  export interface CreateSerialNumber {
    SerialNumber: string;
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * @default Allow
     */
    Status?: SerialNumberStatus;
  }

  export const enum SerialNumberStatus {
    Allow = 0,
    Deny = 1
  }

  export interface CreateSerialNumberResponse {
    /**
     * Entity type: SerialNumber
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets information about a serial number
   */
  export interface GetSerialNumber {
    SerialNumber: string;
  }

  export interface GetSerialNumberResponse {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    Status: SerialNumberStatus;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists Serial Numbers
   */
  export interface ListSerialNumbers {
    PageConfig?: PageConfig<ListSerialNumbersFilter>;
  }

  export interface ListSerialNumbersFilter {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    Status?: SerialNumberStatus;
    SerialNumbers?: string[];
  }

  export interface ListSerialNumbersResponse {
    Result?: PagedResult<ListSerialNumbersResponse.SerialNumberModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListSerialNumbersResponse {
    export interface SerialNumberModel {
      SerialNumber?: string;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      /**
       * Entity type: StockLabel
       */
      StockLabelID?: number;
      Status?: SerialNumberStatus;
    }

  }

  /**
   * Push multiple serial numbers into EVA.
   * 
   * - The `ProductID` is required
   * - You must either specify *both*, or *neither* of `StockLabelID` and `OrganizationUnitID`
   * 
   * :bulb: Our recommended JSON request size limit is 10 MB
   * :muscle: So when calling this service with just a `SerialNumber` and a `ProductID`, that should be somewhere between 100.000 and 200.000 serials per call
   * :warning: When a product belongs to a Serial Number Configuration that is configured with a validation regex, serial numbers pushed for that product must match this regex
   * 
   * This service is built for speeeeeeeeeeeed :rocket:, and so has a slightly different idempotency from other services;
   * 
   * - Skips over errors and attempts to persist as many serial numbers as possible
   * - Gathers any error encountered and returns (up to 1.000 of) them, so you can fix & retry
   * - Ignores serial numbers that already exist in EVA
   * - Ignores duplicates within the same call (but it's undefined which serial number is used)
   * - Ignores duplicates within multiple simultaneous calls (but it's undefined which serial number is used)
   * - Skips over serial number validation for second chance products
   */
  export interface PushSerialNumbers {
    SerialNumbers: PushSerialNumbers.SerialNumberModel[];
  }

  export namespace PushSerialNumbers {
    export interface SerialNumberModel {
      SerialNumber: string;
      /**
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      /**
       * Entity type: StockLabel
       */
      StockLabelID?: number;
      /**
       * Defaults to `Allow`
       * @default Allow
       */
      Status?: SerialNumberStatus;
    }

  }

  export interface PushSerialNumbersResponse {
    Errors: { [key: string]: string };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Parses a script for validation and auto-completion.
   * 
   * Can be used as the user types their script, for that purpose use these parameters:
   * - Type: Fast, doing Full performs a full compilation of the script which can take longer. Using Full can be done before the user hits the Save button, or for example every 10th keystroke or so.
   * 
   * Use IncludeCompletionSuggestions if you want to display auto-completion suggestions to the user.
   * 
   * You can verify the entire script and get completion results by passing in the full script as the Source and only the text until the user's cursor in the CompletionSource.
   * 
   * A list of the available dialects can be retrieved by calling GetScriptDialects.
   */
  export interface ParseScript {
    /**
     * The script to parse.
     */
    Source?: string;
    /**
     * The script to parse, in token format as returned by ParseScript when IncludeStructure is passed in as `true`.
     */
    SourceTokens?: ScriptToken[];
    Dialect?: string;
    Type?: ParseTypes;
    IncludeStructure?: boolean;
    DocumentationRequest?: ParseScript.DocumentationRequestInfo;
    AvailableVariablesRequest?: ParseScript.AvailableVariablesRequestInfo;
    CompletionRequest?: ParseScript.CompletionRequestInfo;
  }

  export namespace ParseScript {
    export interface CompletionRequestInfo {
      /**
       * The script that you want to have auto-completion suggestions for.
       * This can be different than the full Source as you only want to provide completion suggestions at the current cursor position,
       * so you can provide the full source for validation purposes and provide the script up to the cursor position for auto-completion suggestions.
       */
      Source?: string;
      SourceTokens?: ScriptToken[];
    }

    export interface DocumentationRequestInfo {
      /**
       * The 0-based column number of the position in the script that you would like documentation for.
       * Min: 0
       */
      Column: number;
      /**
       * The 1-based line number of the position in the script that you would like documentation for.
       * Min: 1
       */
      Line: number;
    }

    export interface AvailableVariablesRequestInfo {
      /**
       * The 0-based column number of the position in the script that you would like to know the available variables for.
       * Min: 0
       */
      Column: number;
      /**
       * The 1-based line number of the position in the script that you would like to know the available variables for.
       * Min: 1
       */
      Line: number;
    }

  }

  export interface ScriptToken {
    Index?: number;
    Type?: string;
    Text?: string;
    Value?: any;
    Column?: number;
    Line?: number;
    IsEditable?: boolean;
    LeadingTrivia?: ScriptTokenTrivia[];
    TrailingTrivia?: ScriptTokenTrivia[];
  }

  export interface ScriptTokenTrivia {
    Type?: string;
    Text?: string;
    Column?: number;
    Line?: number;
  }

  export const enum ParseTypes {
    None = 0,
    Parse = 1,
    Fast = 1,
    TypeCheck = 3,
    Transpile = 7,
    Full = 7
  }

  export interface ParseScriptResponse {
    HasErrors?: boolean;
    HasWarnings?: boolean;
    Errors?: ScriptError[];
    Warnings?: ScriptWarning[];
    CompletionSuggestions?: CompletionSuggestion[];
    Documentation?: DocumentationResult;
    AvailableVariables?: AvailableVariablesResult;
    Structure?: ScriptStructure;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ScriptError {
    Type: string;
    Message: string;
    SourceLine: string;
    SourceLineNumber: number;
    SourceColumn: number;
  }

  export interface ScriptWarning {
    Message: string;
    SourceLine: string;
    SourceLineNumber: number;
    SourceColumn: number;
  }

  export interface CompletionSuggestion {
    Category?: CompletionCategories;
    Label?: string;
    Code?: string;
    Priority?: number;
    Documentation?: string;
  }

  export const enum CompletionCategories {
    Keyword = 0,
    Member = 1,
    Snippet = 2
  }

  export interface DocumentationResult {
    Component?: string;
    Documentation?: string;
  }

  export interface AvailableVariablesResult {
    Variables?: VariableNode[];
  }

  export interface VariableNode {
    Name?: string;
    Children?: VariableNode[];
    Description?: string;
  }

  export interface ScriptStructure {
    RootNode?: ScriptNode;
    Tokens?: ScriptToken[];
  }

  export interface ScriptNode {
    ID?: number;
    ParentID?: number;
    Type?: string;
    Info?: string;
    Tokens?: number[];
    Children?: ScriptNode[];
  }

  /**
   * Creates a new script.
   */
  export interface CreateScript {
    Name?: string;
    Source?: string;
    Dialect?: string;
    IsActive?: boolean;
    IsPrivate?: boolean;
  }

  export interface CreateScriptResponse {
    /**
     * Entity type: Script
     */
    ID?: number;
    HasErrors?: boolean;
    HasWarnings?: boolean;
    Errors?: ScriptError[];
    Warnings?: ScriptWarning[];
    CompletionSuggestions?: CompletionSuggestion[];
    Documentation?: DocumentationResult;
    AvailableVariables?: AvailableVariablesResult;
    Structure?: ScriptStructure;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates a script
   */
  export interface UpdateScript {
    /**
     * Entity type: Script
     */
    ID?: number;
    RevisionComment?: string;
    Name?: string;
    Source?: string;
    Dialect?: string;
    IsActive?: boolean;
    SourceTokens?: ScriptToken[];
    EditabilityConfiguration?: ScriptEditabilityConfiguration;
  }

  export interface ScriptEditabilityConfiguration {
    Elements?: EditableElementConfiguration[];
  }

  export interface EditableElementConfiguration {
    ID?: string;
    IsEditable?: boolean;
    Name?: string;
    DisplayName?: string;
    Description?: string;
    FieldType?: EditableFieldTypes;
    VariableConfiguration?: EditableVariableConfiguration;
    TableConfiguration?: EditableTableConfiguration;
    IsMissing?: boolean;
  }

  export const enum EditableFieldTypes {
    SetVariable = 0,
    DeclareTable = 1
  }

  export interface EditableVariableConfiguration {
    ValueType?: EditableFieldValueTypes;
    Values?: EditableItemCurrentValue[];
    EntityType?: EditableFieldEntityTypes;
    AllowMultipleValues?: boolean;
    SuggestedAllowMultipleValues?: boolean;
    SuggestedValueType?: EditableFieldValueTypes;
    MinValue?: number;
    MaxValue?: number;
    MinLength?: number;
    MaxLength?: number;
  }

  export const enum EditableFieldValueTypes {
    String = 0,
    Integer = 1,
    Decimal = 2,
    Boolean = 3,
    Date = 4
  }

  export interface EditableItemCurrentValue {
    Type?: EditableItemValueTypes;
    Value?: any;
  }

  export const enum EditableItemValueTypes {
    NumberLiteral = 0,
    StringLiteral = 1,
    BooleanLiteral = 2,
    NullLiteral = 3,
    DynamicValue = 4
  }

  export const enum EditableFieldEntityTypes {
    OrganizationUnitBackendID = 0,
    ProductBackendID = 1,
    CountryID = 2
  }

  export interface EditableTableConfiguration {
    Columns?: EditableTableColumnConfiguration[];
    Rows?: EditableTableRow[];
  }

  export interface EditableTableColumnConfiguration {
    ID?: string;
    Name?: string;
    DisplayName?: string;
    Description?: string;
    Type?: EditableTableColumnType;
    EntityType?: EditableFieldEntityTypes;
    Number?: number;
    IsMissing?: boolean;
  }

  export const enum EditableTableColumnType {
    Unknown = 0,
    String = 1,
    Number = 2,
    Boolean = 3
  }

  export interface EditableTableRow {
    Cells?: EditableTableCell[];
  }

  export interface EditableTableCell {
    ColumnNumber?: number;
    Value?: any;
  }

  export interface UpdateScriptResponse {
    HasErrors?: boolean;
    HasWarnings?: boolean;
    Errors?: ScriptError[];
    Warnings?: ScriptWarning[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists scripts, optionally filtered.
   */
  export interface ListScripts {
    InitialPageConfig?: PageTokenConfig<ListScriptsFilter>;
    PageToken?: string;
  }

  export interface ListScriptsFilter {
    /**
     * Entity type: Script
     */
    ID?: number;
    IsActive?: boolean;
    Name?: string;
    Dialect?: string;
    Source?: string;
    Type?: string;
    IsPrivate?: boolean;
  }

  export interface ListScriptsResponse {
    Results?: ListScriptsItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListScriptsItem {
    /**
     * Entity type: Script
     */
    ID?: number;
    Name?: string;
    Source?: string;
    Dialect?: string;
    IsActive?: boolean;
    Revision?: string;
    UserID?: number;
    UserFullName?: string;
    Date?: string;
    Type?: string;
    RevisionComment?: string;
    IsPrivate?: boolean;
  }

  /**
   * Gets a script by ID
   */
  export interface GetScript {
    /**
     * Entity type: Script
     */
    ID?: number;
    IncludeEditabilityConfiguration?: boolean;
  }

  export interface GetScriptResponse {
    /**
     * Entity type: Script
     */
    ID?: number;
    Name?: string;
    Source?: string;
    Dialect?: string;
    IsActive?: boolean;
    Type?: string;
    RevisionComment?: string;
    IsPrivate?: boolean;
    EditabilityConfiguration?: ScriptEditabilityConfiguration;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Deletes a script
   */
  export interface DeleteScript {
    /**
     * Entity type: Script
     */
    ID: number;
  }

  /**
   * Attempts to explain the given script in plain English.
   */
  export interface ExplainScript {
    Source?: string;
    Dialect?: string;
  }

  export interface ExplainScriptResponse {
    Description?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns all available dialects
   */
  export interface GetScriptDialects {
  }

  export interface GetScriptDialectsResponse {
    Dialects?: GetScriptDialectsResponse.Dialect[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetScriptDialectsResponse {
    export interface Dialect {
      Name?: string;
      Keywords?: string[];
    }

  }

  export interface GetScriptExtensionPoints {
  }

  export interface GetScriptExtensionPointsResponse {
    ExtensionPoints?: GetScriptExtensionPointsResponse.ExtensionPoint[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetScriptExtensionPointsResponse {
    export interface ExtensionPoint {
      Name?: string;
    }

  }

  /**
   * Returns a list of revisions for a specific script.
   */
  export interface ListScriptHistory {
    /**
     * ID of the script
     * Entity type: Script
     */
    ID?: number;
    InitialPageConfig?: PageTokenConfig<ListScriptHistoryFilter>;
    PageToken?: string;
  }

  export interface ListScriptHistoryFilter {
  }

  export interface ListScriptHistoryResponse {
    Results?: ListScriptsItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ApplyScriptEdits {
    /**
     * Entity type: Script
     */
    ID?: number;
    Edits?: EditableScriptElementValue[];
  }

  export interface EditableScriptElementValue {
    ID?: string;
    TableRows?: EditableScriptElementValue.TableRow[];
    Values?: any[];
  }

  export namespace EditableScriptElementValue {
    export interface TableRow {
      Values?: any[];
    }

  }

  export interface CreateRecurringTask {
    ID: string;
    FullName: string;
    Cron?: string;
    Arguments?: { [key: string]: string };
  }

  export interface DeleteRecurringTask {
    ID?: string;
  }

  export interface EnqueueTask {
    FullName?: string;
    Arguments?: { [key: string]: string };
  }

  /**
   * Get the logging of the run of a recurring task.
   */
  export interface GetRecurringTaskLogging {
    ID?: string;
    /**
     * ID of the run, if not specified, the last run will be used.
     */
    RunID?: string;
  }

  export interface GetRecurringTaskLoggingResponse {
    LogEntries: GetRecurringTaskLoggingResponse.LogEntry[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetRecurringTaskLoggingResponse {
    export interface LogEntry {
      Timestamp: string;
      Content: string;
    }

  }

  /**
   * Get the list of runs for a recurring task.
   */
  export interface GetRecurringTaskRuns {
    ID?: string;
  }

  export interface GetRecurringTaskRunsResponse {
    Runs: GetRecurringTaskRunsResponse.RecurringTaskRun[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetRecurringTaskRunsResponse {
    export interface RecurringTaskRun {
      RunID: string;
      StartTime: string;
      EndTime: string;
      IsSuccessful: boolean;
    }

  }

  /**
   * Lists all tasks that are able to be configured to run.
   */
  export interface ListAvailableRecurringTasks {
  }

  export interface ListAvailableRecurringTasksResponse {
    Tasks: ListAvailableRecurringTasksResponse.TaskPluginDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListAvailableRecurringTasksResponse {
    export interface TaskPluginDto {
      Name: string;
      FullName: string;
      Description: string;
      AvailableSettings: TaskPluginSettings[];
      DefaultName: string;
      DefaultCron: string;
    }

  }

  export interface TaskPluginSettings {
    /**
     * Type of the underlying parameter, e.g., 'OrganizationUnit'
     */
    DataModelType: string;
    /**
     * Data type of the parameter, e.g., 'long'.
     */
    Type: string;
    Key: string;
    Description: string;
    Required: boolean;
    Default: any;
  }

  /**
   * @deprecated Replaced by `ListRecurringScheduledTasks`
   * Lists all tasks that are configured to run recurringly.
   */
  export interface ListRecurringTasks {
  }

  export interface ListRecurringTasksResponse {
    Tasks: RecurringTask[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface RecurringTask {
    Cron: string;
    ID: string;
    LastExecution?: string;
    NextExecution?: string;
    IsDeleted: boolean;
    Type: string;
    TypeName: string;
    Args: any[];
    LastStateID: LastState;
  }

  export const enum LastState {
    Unknown = 0,
    Succeeded = 1,
    Failed = 2
  }

  /**
   * @deprecated Replaced by `ListRecurringScheduledTasks`
   * Lists all tasks that are configured to run recurringly.
   */
  export interface ListRecurringTasks2 {
    PageConfig?: PageConfig<ListRecurringTasks2.ListRecurringTasksFilter>;
  }

  export namespace ListRecurringTasks2 {
    export interface ListRecurringTasksFilter {
      ID?: string;
      TypeName?: string;
      Query?: string;
      LastState?: string;
      LastStateID?: LastState;
    }

  }

  export interface ListRecurringTasks2Response {
    Result?: PagedResult<RecurringTask>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists all tasks that are scheduled recurringly.
   */
  export interface ListRecurringScheduledTasks {
    PageConfig?: PageConfig<ListRecurringScheduledTasks.Filter>;
  }

  export namespace ListRecurringScheduledTasks {
    export interface Filter {
      ID?: string;
      TypeName?: string;
      Query?: string;
      LastState?: string;
      LastStateID?: LastState;
    }

  }

  export interface ListRecurringScheduledTasksResponse {
    Result?: PagedResult<RecurringTask>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface TriggerRecurringTask {
    ID?: string;
  }

  export interface TriggerRecurringTaskResponse {
    JobId: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateRecurringTask {
    ID: string;
    /**
     * When provided will change the type of the task that is run.
     */
    FullName?: string;
    Cron: string;
    Arguments?: { [key: string]: string };
  }

  /**
   * Service for retrieving an list of `QuickBuy` options
   */
  export interface GetQuickBuyOptions {
    ProductProperties: string[];
  }

  export interface GetQuickBuyOptionsResponse {
    Options: GetQuickBuyOptionsResponse.QuickBuyOption[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetQuickBuyOptionsResponse {
    export interface QuickBuyOption {
      /**
       * Entity type: QuickBuy
       */
      ID: number;
      Name: string;
      Type: QuickBuyType;
      Products: any[];
    }

  }

  export const enum QuickBuyType {
    Checkout = 1,
    ShoppingCart = 2
  }

  /**
   * Service for creating a `QuickBuy`
   */
  export interface CreateQuickBuy {
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID: number;
    Type: QuickBuyType;
    Name: string;
  }

  export interface CreateQuickBuyResponse {
    /**
     * Entity type: QuickBuy
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for retrieving an existing `QuickBuy` by ID
   */
  export interface GetQuickBuy {
    /**
     * Entity type: QuickBuy
     */
    ID: number;
  }

  export interface GetQuickBuyResponse {
    /**
     * Entity type: QuickBuy
     */
    ID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID: number;
    Type: QuickBuyType;
    Name: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for retrieving an list of `QuickBuy`
   */
  export interface GetQuickBuys {
  }

  export interface GetQuickBuysResponse {
    Items: GetQuickBuysResponse.QuickBuyItem[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetQuickBuysResponse {
    export interface QuickBuyItem {
      /**
       * Entity type: QuickBuy
       */
      ID: number;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      /**
       * Entity type: ProductSearchTemplate
       */
      ProductSearchTemplateID: number;
      Type: QuickBuyType;
      Name: string;
    }

  }

  /**
   * Service for updating an existing `QuickBuy` by ID
   */
  export interface UpdateQuickBuy {
    /**
     * Entity type: QuickBuy
     */
    ID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID: number;
    Type: QuickBuyType;
    Name: string;
  }

  /**
   * Service for deleting an existing `QuickBuy` by ID
   */
  export interface DeleteQuickBuy {
    /**
     * Entity type: QuickBuy
     */
    ID: number;
  }

  /**
   * Register an app for push notifications. This will register the device token for the given application identifier so the app can receive PushNotifications for the current User/OrganizationUnit.
   */
  export interface RegisterAppForNotifications {
    DeviceToken: string;
    /**
     * Application identifier: BundleID
     */
    BackendID: string;
    /**
     * Only fill when multiple apps on the same device are involved. Identifier that uniquely identifies the device where the app is running on. Should be the same for multiple apps on the same device.
     */
    DeviceID?: string;
  }

  /**
   * Add lines to an existing purchase order
   * 
   * It is required to supply an OrderID and at least one line with a ProductID and a Quantity. All other line properties are optional.
   * It's not allowed to create multiple lines with the same product on the same stocklabel.
   */
  export interface AddPurchaseOrderLines {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Lines: AddPurchaseOrderLines.Line[];
  }

  export namespace AddPurchaseOrderLines {
    export interface Line {
      /**
       * Entity type: Product
       */
      ProductID?: number;
      /**
       * Min: 1
       */
      Quantity?: number;
      UnitPrice?: number;
      RequestedDate?: string;
      Reference?: string;
      /**
       * Entity type: StockLabel
       */
      StockLabelID?: number;
      /**
       * The currency of the unit price. If not supplied, the order's currency will be used. The UnitPrice will be converted to the order's currency. If no exchangerate is available, the UnitPrice will be ignored and the price is fetched from an available pricelist.
       * Entity type: Currency
       */
      CurrencyID?: string;
    }

  }

  export interface AddPurchaseOrderLinesResponse {
    Result: AddPurchaseOrderLinesResponse.ResultLine[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace AddPurchaseOrderLinesResponse {
    export interface ResultLine {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      /**
       * Entity type: Product
       */
      ProductID: number;
      Quantity: number;
      UnitPrice: number;
      RequestedDate?: string;
      Reference: string;
    }

  }

  /**
   * Confirms that specified Purchase Order will be delivered, identified by the `OrderID` parameter.
   * 
   * Optionally use the `BackendID` parameter to set the BackendID on the Purchase Order.
   */
  export interface ConfirmPurchaseOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    BackendID?: string;
  }

  /**
   * Creates a Purchase Order for supplied `OrganizationUnitID`, from optional `SupplierID`.
   * 
   * Use the `BackendID` to indicate the supplier order reference.
   */
  export interface CreatePurchaseOrder {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierID?: number;
    BackendID?: string;
    AutomaticCancellationDate?: string;
    /**
     * Entity type: CustomOrderType
     */
    CustomOrderTypeID?: number;
    /**
     * Entity type: OrderCustomType
     */
    OrderCustomTypeID?: number;
  }

  export interface CreatePurchaseOrderResponse {
    /**
     * Entity type: Order
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates a shipment receipt with `Lines` for specified `ShipmentID`.
   * 
   * * `CompletesShipment` / `Lines.CompletesShipmentLine` will close the shipment (line) for futher receipts
   * * `Lines.QuantityReceived` is the delta quantity received in this receipt, so not the new total quantity received
   * * `StockLabelID` is the target StockLabel for the received stock - overrides the StockLabel of the purchase order line
   */
  export interface CreateShipmentReceipt {
    BackendID?: string;
    BackendSystemID?: string;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    CompletesShipment?: boolean;
    Lines?: CreateShipmentReceipt.Line[];
  }

  export namespace CreateShipmentReceipt {
    export interface Line {
      /**
       * Entity type: ShipmentLine
       */
      ShipmentLineID: number;
      /**
       * Entity type: StockLabel
       */
      StockLabelID?: number;
      QuantityReceived: number;
      CompletesShipmentLine?: boolean;
    }

  }

  export interface CreateShipmentReceiptResponse {
    /**
     * Entity type: ShipmentReceipt
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Receives a shipment completely as-is. Supply either `ID` or `BackendID`.
   */
  export interface FullyReceiveShipment {
    /**
     * Entity type: Shipment
     */
    ID?: number;
    BackendID?: string;
  }

  export interface FullyReceiveShipmentResponse {
    ReservedOrdersCount: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FullyShipPurchaseOrder {
    /**
     * Entity type: Order
     */
    PurchaseOrderID: number;
    ShipmentBackendID?: string;
    BackendSystemID?: string;
    TrackingCode?: string;
    /**
     * The OrganizationUnit the shipment was physically shipped from.
     * Entity type: OrganizationUnit
     */
    ShippedFromOrganizationUnitID?: number;
  }

  /**
   * @deprecated No longer in use.
   */
  export interface GetOrganizationUnitCreditBalance {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetOrganizationUnitCreditBalanceResponse {
    CreditBalance: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get the supplier product info for a set of product IDs (not supplier product IDs) and the optional supplier OU.
   */
  export interface GetProductSupplierInfoForProducts {
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface GetProductSupplierInfoForProductsResponse {
    ProductSupplierInfos: GetProductSupplierInfoForProductsResponse.ProductSupplierInfo[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetProductSupplierInfoForProductsResponse {
    export interface ProductSupplierInfo {
      ProductID: number;
      BackendID: string;
      SupplierProductID: number;
      SupplierProductBackendID: string;
      PreferredMinimumOrderQuantity?: number;
      UnitCost: number;
      Supplier: GetProductSupplierInfoForProductsResponse.Supplier;
      UnitOfMeasures: GetProductSupplierInfoForProductsResponse.UnitOfMeasure[];
      Barcodes: GetProductSupplierInfoForProductsResponse.Barcode[];
    }

    export interface UnitOfMeasure {
      ID: number;
      Name: string;
      Quantity: number;
    }

    export interface Supplier {
      ID: number;
      BackendID: string;
      Name: string;
    }

    export interface Barcode {
      Code: string;
      UnitOfMeasureID: number;
      Quantity: number;
    }

  }

  /**
   * Get details of a shipment to receive, either by `ID` or by `BackendID`/`OrganizationUnitID` combination.
   */
  export interface GetShipmentDetails {
    /**
     * Entity type: Shipment
     */
    ID?: number;
    BackendID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface GetShipmentDetailsResponse {
    ReceiveMethod: ShipmentReceiveMethods;
    IsCompleted: boolean;
    WorkSet: ReceiveShipmentWorkSet;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    ShippedTo: GetShipmentDetailsResponse.ShipOrganizationUnitDetails;
    ShippedFrom: GetShipmentDetailsResponse.ShipOrganizationUnitDetails;
    TrackingCode?: string;
    ShipmentDate?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetShipmentDetailsResponse {
    export interface ShipOrganizationUnitDetails {
      ID: number;
      Name: string;
    }

  }

  export interface ReceiveShipmentWorkSet {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    BackendID: string;
    BackendSystemID: string;
    /**
     * Entity type: OrganizationUnit
     */
    ShippedToOrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShippedFromOrganizationUnitID?: number;
    ReceiveMethod: ShipmentReceiveMethods;
    Lines: ReceiveShipmentWorkSetLine[];
    Orders: GetShipmentDetailsOrder[];
  }

  export interface ReceiveShipmentWorkSetLine {
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductBackendID?: string;
    ProductPrimitiveName: string;
    ProductCustomID: string;
    ProductDetails?: any;
    QuantityOrdered: number;
    QuantityAlreadyReceived: number;
    QuantityShipped: number;
    NetQuantityShipped: number;
    NetQuantityDelivered: number;
    SerialNumbers?: string[];
    ReceivedSerialNumbers?: string[];
    SerialNumberRequired: boolean;
    SerialNumberConfigurationID?: number;
    Receipts: ReceiveShipmentWorkSetLineReceipt[];
    Packages: ReceiveShipmentWorkSetLinePackage[];
  }

  export interface ReceiveShipmentWorkSetLineReceipt {
    QuantityReceived: number;
    /**
     * Entity type: ShipmentLine
     */
    ShipmentLineID?: number;
  }

  export interface ReceiveShipmentWorkSetLinePackage {
    UnitOfMeasure: string;
    Barcode: string;
    Quantity: number;
    QuantityBarcode: number;
    QuantityTotal: number;
  }

  export interface GetShipmentDetailsOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    OrderDisplayID?: string;
    BackendID: string;
    CustomerReference: string;
  }

  /**
   * Get an existing ShipmentReceipt
   */
  export interface GetShipmentReceipt {
    /**
     * Entity type: ShipmentReceipt
     */
    ShipmentReceiptID: number;
  }

  export interface GetShipmentReceiptResponse {
    /**
     * Entity type: ShipmentReceipt
     */
    ID: number;
    BackendID?: string;
    BackendSystemID?: string;
    TotalQuantityReceived: number;
    Lines: GetShipmentReceiptResponse.LineDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetShipmentReceiptResponse {
    export interface LineDto {
      QuantityReceived: number;
      ShipmentLine: LineDto.ShipmentLineDto;
      StockLabel: EnumDto;
    }

    export namespace LineDto {
      export interface ShipmentLineDto {
        /**
         * Entity type: Order
         */
        OrderID: number;
        /**
         * Entity type: OrderLine
         */
        OrderLineID: number;
        QuantityShipped: number;
        QuantityDelivered: number;
        BackendReference?: string;
        DeliveryType: ShipmentLineDeliveryTypes;
      }

    }

  }

  /**
   * @deprecated No longer in use.
   */
  export interface GetSupplierStockForProduct {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  export interface GetSupplierStockForProductResponse {
    AvailableStock: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ModifyPurchaseOrderLine {
    /**
     * Entity type: OrderLine
     */
    ID: number;
    /**
     * Min: 0
     */
    Quantity?: number;
    UnitPrice?: number;
    RequestedDate?: string;
    Reference?: string;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
  }

  export interface RaisePurchaseOrderImportedEvent {
    /**
     * Entity type: Order
     */
    PurchaseOrderID?: number;
    BackendID?: string;
    BackendSystemID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID?: number;
  }

  /**
   * Receive (part of) a Shipment for a PurchaseOrder.
   * 
   * A ShipmentID is required.
   * 
   * When doing partial receipts it is strongly advised to supply a BackendSystemID and BackendID to make sure Receipts are not created twice.
   * 
   * To do partial receipts the FinalShipment property should be set to false, otherwise the shipment will be close afterwards and the remaining lines will be booked as missing.
   * 
   * When available it is preferred to identify the ShipmentLines with the ShipmentLineID and not with the ProductID.
   */
  export interface ReceivePurchaseOrderShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    BackendSystemID?: string;
    BackendID?: string;
    /**
     * If passed in as true this will close the shipment for further receipts.
     */
    CompleteShipment?: boolean;
    Lines: ReceivePurchaseOrderShipment.Line[];
  }

  export namespace ReceivePurchaseOrderShipment {
    export interface Line {
      /**
       * Entity type: ShipmentLine
       */
      ShipmentLineID?: number;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      QuantityReceived: number;
      /**
       * The serial numbers to receive.
       */
      SerialNumbers?: string[];
    }

  }

  export interface ReceivePurchaseOrderShipmentResponse {
    /**
     * Entity type: ShipmentReceipt
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Receive a shipment.
   */
  export interface ReceiveShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    CompleteShipment?: boolean;
    /**
     * The receipts to create. If left null, it's assumed you wish to receive the entire shipment. If left empty, it's assumed you wish to receive nothing.
     */
    Receipts?: ReceiveShipment.ShipmentReceipt[];
    /**
     * Receipts for products that are not present in the shipment. This can only be used for products that are not declard in the shipment.
     */
    UndeclaredProductsReceipts?: ReceiveShipment.ProductReceipt[];
    /**
     * @deprecated Use `Receipts` instead.
     */
    Lines?: ReceiveShipment.Line[];
  }

  export namespace ReceiveShipment {
    export interface ShipmentReceipt {
      /**
       * The quantity to receive. If left null, the quantity not yet received will be used.
       */
      QuantityReceived?: number;
      /**
       * The serial numbers to receive.
       */
      SerialNumbers?: string[];
      /**
       * The shipment line to receive.
       * Entity type: ShipmentLine
       */
      ShipmentLineID: number;
    }

    export interface ProductReceipt {
      /**
       * The quantity to receive.
       */
      QuantityReceived: number;
      /**
       * The product to receive. This can only be a product that is not in the shipment.
       * Entity type: Product
       */
      ProductID: number;
      /**
       * The serial numbers to receive.
       */
      SerialNumbers?: string[];
    }

    export interface Line {
      /**
       * Entity type: Product
       */
      ProductID: number;
      Receipts?: Line.Receipt[];
    }

    export namespace Line {
      export interface Receipt {
        QuantityReceived: number;
      }

    }

  }

  /**
   * Creates a shipment for given purchase order lines and ships it!
   */
  export interface ShipPurchaseOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    BackendID?: string;
    TrackingCode?: string;
    ShipmentDate?: string;
    Lines?: ShipPurchaseOrderLine[];
    /**
     * The OrganizationUnit the shipment was physically shipped from.
     * Entity type: OrganizationUnit
     */
    ShippedFromOrganizationUnitID?: number;
  }

  export interface ShipPurchaseOrderLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Min: 1
     */
    QuantityToShip: number;
    /**
     * Serial numbers for the items being shipped.
     */
    SerialNumbers?: string[];
  }

  export interface ShipPurchaseOrderResponse {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates a purchase order (or updates it if it already exists).
   * 
   * When the order already exists, only the lines can be updated (see `SupplierUpdatePurchaseOrder`) - other properties will be ignored.
   * 
   * :lightbulb: Using `SupplierCreateOrder` and `SupplierUpdatePurchaseOrder` is preferred.
   */
  export interface SupplierCreateOrUpdatePurchaseOrder {
    /**
     * OrganizationUnit who will be supplying the stock.
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnitID: number;
    /**
     * OrganizationUnit that will be receiving the purchase order shipment(s).
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID: number;
    /**
     * Identifer of the order from the 3rd party
     */
    BackendID: string;
    /**
     * Identifier of the 3rd party system that created the order
     */
    BackendSystemID?: string;
    /**
     * Date that the supplier expects to deliver the shipment(s) of the purchase order
     */
    ExpectedDeliveryDate?: string;
    CustomFields?: { [key: string]: any };
    /**
     * Ship the order immediately after creation. Order won't be shipped again after an update.
     */
    Ship?: boolean;
    /**
     * BackendID of the shipment to be created. Works only if `Ship` is set to true. Make sure this is unique.
     */
    ShipmentBackendID?: string;
    /**
     * Receive the order immediately after shipping, works only if `Ship` is set to true
     */
    Receive?: boolean;
    /**
     * Contents of the purchase order.
     */
    Lines: SupplierCreateOrUpdatePurchaseOrder.SupplierCreateOrUpdatePurchaseOrderLine[];
    /**
     * Identifiers to set on the shipment if any. Works only if `Ship` is set to true.
     */
    ShipmentIdentifiers?: SupplierCreateOrUpdatePurchaseOrder.SupplierCreateOrUpdatePurchaseOrderShipmentIdentifier[];
  }

  export namespace SupplierCreateOrUpdatePurchaseOrder {
    export interface SupplierCreateOrUpdatePurchaseOrderShipmentIdentifier {
      /**
       * Entity type: ShipmentIdentifier
       */
      ShipmentIdentifierID: number;
      Value: string;
    }

    export interface SupplierCreateOrUpdatePurchaseOrderLine {
      /**
       * Identifier of the product
       * Entity type: Product
       */
      ProductID?: number;
      /**
       * Quantity that is expected to be shipped
       */
      Quantity?: number;
      UnitPrice?: number;
      /**
       * The currency of the unit price. If not supplied, the order's currency will be used. The UnitPrice will be converted to the order's currency. If no exchangerate is available, the UnitPrice will be ignored and the price is fetched from an available pricelist.
       * Entity type: Currency
       */
      CurrencyID?: string;
      /**
       * The BackendReference of the line to create. Note that multiple lines with the same product are merged into a single line so the chosen BackendReference is arbitrary.
       */
      BackendReference?: string;
      /**
       * Date that the supplier expects to deliver this orderline. If you supply multiple lines with the same ProductID, we only take the latest date.
       */
      ExpectedDeliveryDate?: string;
    }

  }

  export interface CreatePurchaseOrderResponseWithUpdateIndicator {
    IsUpdated: boolean;
    IsCreated: boolean;
    /**
     * Entity type: Order
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface SupplierCreatePurchaseOrder {
    /**
     * Identifier of the 3rd party system that created the order
     */
    BackendSystemID?: string;
    /**
     * Identifier of the order from the 3rd party
     */
    BackendID?: string;
    /**
     * OrganizationUnit who will be supplying the stock.
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnitID: number;
    /**
     * OrganizationUnit that will be receiving the purchase order shipment(s).
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID: number;
    /**
     * Date that the supplier expects to deliver the shipment(s) of the purchase order. Can also be specified on the line level. If specified on both levels, the line level will take precedence.
     */
    ExpectedDeliveryDate?: string;
    CustomFields?: { [key: string]: any };
    /**
     * Ship the order immediately after creation.
     */
    Ship?: boolean;
    /**
     * List of shipment identifiers to attach to the shipment. This only works if `Ship` is set to true.
     */
    ShipmentIdentifiers?: SupplierCreatePurchaseOrder.SupplierCreatePurchaseOrderShipmentIdentifier[];
    /**
     * BackendID of the shipment to be created. Works only if `Ship` is set to true. Make sure this is unique.
     */
    ShipmentBackendID?: string;
    /**
     * Copy the orderlines backendreference to the shipmentlines backendreference, works only if `Ship` is set to true
     */
    CopyBackendReferenceOnShip?: boolean;
    /**
     * Receive the order immediately after shipping, works only if `Ship` is set to true
     */
    Receive?: boolean;
    /**
     * Contents of the purchase order.
     * Note: Only one key property can be used to identify the product. When different properties are used, the request will be invalid.
     * This will become redundant when ProductID is the only property left.
     */
    Lines: SupplierCreatePurchaseOrder.SupplierCreatePurchaseOrderBatchableLine[];
  }

  export namespace SupplierCreatePurchaseOrder {
    export interface SupplierCreatePurchaseOrderShipmentIdentifier {
      /**
       * Entity type: ShipmentIdentifier
       */
      ShipmentIdentifierID: number;
      Value: string;
    }

    export interface SupplierCreatePurchaseOrderBatchableLine {
      /**
       * Identifier of the product
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Quantity that is expected to be shipped
       */
      Quantity: number;
      /**
       * Date that the supplier expects to deliver this orderline. If you supply multiple lines with the same ProductID, we only take the latest date.
       */
      ExpectedDeliveryDate?: string;
      UnitPrice?: number;
      /**
       * The currency of the unit price. If not supplied, the order's currency will be used. The UnitPrice will be converted to the order's currency. If no exchangerate is available, the UnitPrice will be ignored and the price is fetched from an available pricelist.
       * Entity type: Currency
       */
      CurrencyID?: string;
      /**
       * The BackendReference of the line to create
       */
      BackendReference: string;
    }

  }

  /**
   * Ships a Purchase Order with specified lines. Supply either `ID` or a `BackendID`/`BackendSystemID` combination to
   * identify the order.
   * 
   * If you wish to receive on behalf of a different organization, identify that organization by specifying a
   * `ShipToOrganizationUnitBackendID`. Should this organization reside in a different cluster, this call will be proxied
   * to that cluster for you.
   */
  export interface SupplierShipPurchaseOrder {
    /**
     * Entity type: Order
     */
    ID?: number;
    BackendID?: string;
    BackendSystemID?: string;
    ShipToOrganizationUnitBackendID?: string;
    ExpectedDeliveryDate?: string;
    Confirmations: SupplierShipPurchaseOrderConfirmation[];
  }

  export interface SupplierShipPurchaseOrderConfirmation {
    BackendID: string;
    TrackingCode?: string;
    TrackingLink?: string;
    ExpectedDeliveryDate?: string;
    Lines: SupplierShipPurchaseOrderConfirmationLine[];
    CustomFieldsByBackendID?: { [key: string]: CustomFieldValue };
    ShipmentIdentifiers?: SupplierShipPurchaseOrderShipmentIdentifier[];
  }

  export interface SupplierShipPurchaseOrderConfirmationLine {
    /**
     * Identifier of the product
     */
    BackendID?: string;
    /**
     * Please use `BackendID`
     */
    CustomID?: string;
    /**
     * Please use `BackendID`
     */
    Barcode?: string;
    Quantity: number;
    SerialNumbers?: string[];
    BatchCode?: string;
    ExpiryDate?: string;
  }

  export interface SupplierShipPurchaseOrderShipmentIdentifier {
    /**
     * Entity type: ShipmentIdentifier
     */
    ShipmentIdentifierID: number;
    Value: string;
  }

  export interface SupplierUpdatePurchaseOrder {
    /**
     * Identifier of the 3rd party system that created the order
     */
    BackendSystemID?: string;
    /**
     * Identifier of the order from the 3rd party
     */
    BackendID?: string;
    /**
     * OrganizationUnit that will be receiving the purchase order shipment(s).
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID?: number;
    /**
     * Contents of the purchase order.
     * Note: Only one key property can be used to identify the product. When different properties are used, the request will be invalid.
     * This will become redundant when ProductID is the only property left.
     */
    Lines: SupplierUpdatePurchaseOrder.SupplierUpdatePurchaseOrderLine[];
  }

  export namespace SupplierUpdatePurchaseOrder {
    export interface SupplierUpdatePurchaseOrderLine {
      /**
       * Identifier of the product
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Quantity that is expected to be shipped
       */
      Quantity: number;
      /**
       * Date that the supplier expects to deliver this orderline. If you supply multiple lines with the same ProductID, we only take the latest date.
       */
      ExpectedDeliveryDate?: string;
      UnitPrice?: number;
      /**
       * The currency of the unit price. If not supplied, the order's currency will be used. The UnitPrice will be converted to the order's currency. If no exchangerate is available, the UnitPrice will be ignored and the price is fetched from an available pricelist.
       * Entity type: Currency
       */
      CurrencyID?: string;
      /**
       * The BackendReference of the line to create
       */
      BackendReference: string;
    }

  }

  export interface GetConfigurableProductDetail {
    /**
     * Entity type: Product
     */
    ID?: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
    Filter?: ConfigurableProductDetailFilter;
    ApplySearchFilters?: boolean;
  }

  export interface ConfigurableProductDetailFilter {
    Properties: ConfigurableProductDetailFilterProperty[];
    HideUnmatched: boolean;
  }

  export interface ConfigurableProductDetailFilterProperty {
    Property: string;
    Values: any[];
  }

  export interface GetConfigurableProductDetailResponse {
    Configurable: ConfigurableProductDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ConfigurableProductDto {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Type: ProductTypes;
    Status: ProductStatus;
    LogicalLevel: string;
    ConfigurableProperty: string;
    ConfigurablePropertyDisplayName: string;
    MatchesFilter?: boolean;
    Value: any;
    Values: any[];
    Children: ConfigurableProductDto[];
    Properties: any;
    HasDimensions: boolean;
    VariationIndex?: number;
  }

  export interface GetConfigurableProductsDetail {
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
    ApplySearchFilters?: boolean;
  }

  export interface GetConfigurableProductsDetailResponse {
    /**
     * Entity type: Product
     */
    Configurables: { [key: number]: ConfigurableProductDto };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns a list of greeting cards.
   */
  export interface GetGreetingCards {
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface GetGreetingCardsResponse {
    Products: any[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get product details by id
   */
  export interface GetProductDetail {
    /**
     * Entity type: Product
     */
    ID: number;
  }

  export interface GetProductDetailResponse {
    Result: any;
    IsPartOfBundle: boolean;
    Relations?: ProductRelationDTO[];
    /**
     * Entity type: SerialNumberConfiguration
     */
    SerialNumberConfigurationID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ProductRelationDTO {
    /**
     * Entity type: ProductRelationType
     */
    ProductRelationTypeID?: number;
    ProductRelationTypeBackendID?: string;
    PrimaryName?: string;
    SecondaryName?: string;
    Direction?: ProductRelationDirection;
    StockRelationType?: StockRelationType;
    /**
     * Entity type: Product
     */
    PrimaryProductIDs?: number[];
    /**
     * Entity type: Product
     */
    RelatedProductIDs?: number[];
    Sequence?: number;
  }

  export const enum ProductRelationDirection {
    Unidirectional = 0,
    Bidirectional = 1,
    Group = 2
  }

  export const enum StockRelationType {
    None = 0,
    Replacing = 1
  }

  /**
   * This service is used to list prices associated with the products determined by IDs.
   * 
   * Price - this value has TaxRate applied
   * PriceInTax - value without TaxRate
   * 
   * A product has more associated prices with different usage types, e.g. Cost, Sales..
   * 
   * Request e.g.:
   * {
   *   "ProductIDs": [
   *     "productId1",
   *     "productId2"
   *   ]
   * }
   */
  export interface GetProductPrices {
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    TargetOrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  export interface GetProductPricesResponse {
    Prices: PriceInfo[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface PriceInfo {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: PriceList
     */
    PriceListID: number;
    /**
     * Entity type: PriceListUsageType
     */
    PriceListUsageTypeID: number;
    PriceListUsageTypeName: string;
    CurrencyID: string;
    TaxRate: number;
    Price: number;
    PriceInTax: number;
    HistoricalPrice?: number;
    HistoricalPriceInTax?: number;
  }

  export interface GetProductPricing {
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  export interface GetProductPricingResponse {
    UnitPrice: number;
    OriginalUnitPrice?: number;
    TaxRate: number;
    UnitPriceInTax: number;
    OriginalUnitPriceInTax?: number;
    CurrencyID: string;
    UnitCost?: number;
    CostPriceCurrencyID: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get product relations by product id
   */
  export interface GetProductRelations {
    /**
     * Entity type: Product
     */
    ID: number;
  }

  export interface GetProductRelationsResponse {
    Relations?: ProductRelationDTO[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetProductSearchSuggestions {
    Text?: string;
    MaxSuggestions?: number;
  }

  export interface GetProductSearchSuggestionsResponse {
    Suggestions: SuggestionModel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface SuggestionModel {
    Score: number;
    Text: string;
    Payload: { [key: string]: any };
  }

  /**
   * Fetch one or more products by ID.
   */
  export interface GetProducts {
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields: string[];
  }

  export interface GetProductsResponse {
    /**
     * Entity type: Product
     */
    Products: { [key: number]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetProductUnitOfMeasureQuantities {
    Products?: GetProductUnitOfMeasureQuantities.Model[];
  }

  export namespace GetProductUnitOfMeasureQuantities {
    export interface Model {
      /**
       * Entity type: Product
       */
      ID: number;
      Quantity: number;
    }

  }

  export interface GetProductUnitOfMeasureQuantitiesResponse {
    Results: GetProductUnitOfMeasureQuantitiesResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetProductUnitOfMeasureQuantitiesResponse {
    export interface Model {
      /**
       * Entity type: Product
       */
      ProductID: number;
      UnitOfMeasures: GetProductUnitOfMeasureQuantitiesResponse.Result[];
    }

    export interface Result {
      Value: EnumDto;
      Quantity: number;
    }

  }

  /**
   * Get product warranty information by making a request to an endpoint set with the 'Product:WarrantyEndpoint' setting.
   * 
   * If 'SerialNumber' is not set in the request, we'll try to get it using the 'OrderLineID', so at least one or the other must be present and valid.
   */
  export interface GetProductWarranty {
    /**
     * This or 'OrderLineID' needed for search
     */
    SerialNumber?: string;
    /**
     * This or 'SerialNumber' needed for search
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    OrderDate?: string;
  }

  export interface GetProductWarrantyResponse {
    IsUnderWarranty: boolean;
    WarrantyExpiryDate?: string;
    ManufacturerWarrantyExpiryDate?: string;
    AdditionalInfo: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListBrands {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListBrandsResponse {
    Result?: PagedResult<ListBrandsResponse.BrandDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListBrandsResponse {
    export interface BrandDto {
      ID: number;
      Name: string;
      BackendID: string;
    }

  }

  /**
   * This service is used to print labels containing the price details of one product
   */
  export interface PrintProductPriceLabel {
    /**
     * The ID of the product
     * Entity type: Product
     */
    ID: number;
    /**
     * How many copies of the price label should be printed
     * Range: 1-999
     */
    Quantity: number;
    /**
     * ID of the printer where the label(s) will be printed
     * Entity type: Station
     */
    StationID?: number;
  }

  export interface PrintProductPriceLabelResponse {
    Success: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List all ProductRequirement values for an OrderLine
   */
  export interface GetProductRequirementValuesForOrderLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
  }

  export interface GetProductRequirementValuesForOrderLineResponse {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductPrimitiveName: string;
    /**
     * Entity type: ProductRequirement
     */
    Requirements: { [key: number]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List all ProductRequirement values for an Order
   */
  export interface GetProductRequirementValuesForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetProductRequirementValuesForOrderResponse {
    Requirements: GetProductRequirementValuesForOrderResponse.ProductRequirement[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetProductRequirementValuesForOrderResponse {
    export interface ProductRequirement {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      /**
       * Entity type: Product
       */
      ProductID: number;
      ProductPrimitiveName: string;
      /**
       * Entity type: ProductRequirement
       */
      Requirements: { [key: number]: any };
    }

  }

  /**
   * Replace the given ProductRequirement values for an OrderLine with new values
   */
  export interface UpdateProductRequirementValuesForOrderLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: ProductRequirement
     */
    Values?: { [key: number]: any };
  }

  /**
   * Retrieve the structure of products by `Code` (and for given `LanguageID`/`CountryID`).
   */
  export interface GetProductStructure {
    Code?: string;
    LanguageID?: string;
    CountryID?: string;
  }

  export interface GetProductStructureResponse {
    ID: number;
    Code: string;
    Name: string;
    LanguageID: string;
    CountryID: string;
    Definition: ProductStructureDefinition;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ProductStructureDefinition {
    Name: string;
    Filters: { [key: string]: FilterModel };
    Children: ProductStructureDefinition[];
  }

  export interface FilterModel {
    Values?: any[];
    From?: string;
    To?: string;
    /**
     * @default false
     */
    Negation?: boolean;
    /**
     * @default false
     */
    ExactMatch?: boolean;
    IncludeMissing?: boolean;
  }

  /**
   * Create a new structure of products.
   */
  export interface CreateProductStructure {
    Code: string;
    Name: string;
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Definition?: ProductStructureDefinition;
  }

  export interface CreateProductStructureResponse {
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Edits an existing structure of products by `ID`.
   */
  export interface EditProductStructure {
    /**
     * Entity type: ProductStructure
     */
    ID: number;
    Code: string;
    Name: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Definition?: ProductStructureDefinition;
  }

  export interface EditProductStructureResponse {
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Deletes an existing structure of products by `ID`.
   */
  export interface DeleteProductStructure {
    /**
     * Entity type: ProductStructure
     */
    ID: number;
  }

  /**
   * List product structures.
   */
  export interface ListProductStructures {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListProductStructuresResponse {
    Result?: PagedResult<GetProductStructureResponse>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Resolve product backend IDs
   */
  export interface ResolveProductBackendIDs {
    BackendIDs: string[];
  }

  export interface ResolveProductBackendIDsResponse {
    ResolvedBackendIDs: { [key: string]: number };
    UnresolvedBackendIDs: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Performs a search for products, given a `Query` string.
   */
  export interface SearchProducts {
    Query?: string;
    Filters?: { [key: string]: FilterModel };
    AggregationState?: string;
    AggregationOptions?: { [key: string]: AggregationFilterModel };
    PageSize?: number;
    Page?: number;
    Sort?: SortFieldDescriptor[];
    SearchStrategyCode?: string;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
    UserTypeOverride?: UserTypes;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    /**
     * If Explain is set to true the response will contain an explanation of how the search was performed. This is ignored if you do not have the ProductSearchExplanations functionality.
     */
    Explain?: boolean;
    Options?: SearchProductsOptions;
    ProductAvailabilityFilter?: ProductAvailabilityFilter;
  }

  export interface AggregationFilterModel {
    MaxAggregationTerms?: number;
    TermsAggregation: boolean;
    StatsAggregation: boolean;
    AggregationOptions: { [key: string]: AggregationFilterModel };
    Values?: any[];
    From?: string;
    To?: string;
    /**
     * @default false
     */
    Negation?: boolean;
    /**
     * @default false
     */
    ExactMatch?: boolean;
    IncludeMissing?: boolean;
  }

  export interface SortFieldDescriptor {
    FieldName: string;
    Direction: SortDirection;
  }

  export interface SearchProductsOptions {
    /**
     * Order to use for additional options
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Include the results of a PrefigureDiscounts call in the result
     */
    IncludePrefigureDiscounts?: boolean;
    /**
     * If something is defined in these options, the GetProductAvailability results will be added to the response
     */
    AvailabilityOptions?: GetProductAvailabilityOptions;
    /**
     * Include the product relations
     */
    IncludeProductRelations?: boolean;
    /**
     * Include the product serial number configuration
     */
    IncludeSerialNumberConfiguration?: boolean;
    /**
     * Include the information on whether or not a product is part of bundle.
     */
    IncludeProductBundleRelationship?: boolean;
    /**
     * Include the customer wishlists that contain this product.
     */
    IncludeConnectedWishlistForProduct?: boolean;
  }

  export interface ProductAvailabilityFilter {
    FilterByAvailability: boolean;
    Type: ProductAvailabilityFilterTypes;
  }

  export const enum ProductAvailabilityFilterTypes {
    CurrentOrganizationUnit = 0,
    Suppliers = 1
  }

  export interface SearchProductsResponse {
    Products?: any[];
    AggregationState?: string;
    Aggregations?: { [key: string]: any };
    AggregationOptions?: { [key: string]: AggregationFilterModel };
    Page?: number;
    PageSize?: number;
    Total?: number;
    Sort?: SortFieldDescriptor[];
    Explanation?: ProductSearchExplanationModel;
    /**
     * Contains results of requested options
     */
    Options?: SearchProductsOptionsResponse;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ProductSearchExplanationModel {
    SearchStrategy: any;
    Query: any;
  }

  export interface SearchProductsOptionsResponse {
    PrefiguredDiscounts?: PrefigureDiscountResult[];
    /**
     * Entity type: Product
     */
    AvailabilityResult?: { [key: number]: ProductAvailabilityResult };
    AvailabilityPickupOrganizationUnits?: ProductAvailabilityPickupOrganizationUnit[];
    /**
     * Entity type: Product
     */
    ProductRelations?: { [key: number]: SearchProductsOptionsResponse.ProductRelation[] };
    /**
     * Entity type: Product
     */
    ProductSerialNumberConfigurations?: { [key: number]: SearchProductsOptionsResponse.SerialNumberConfiguration };
    /**
     * Entity type: Product
     */
    IsPartOfBundleByProductID: { [key: number]: boolean };
    Wishlists?: SearchProductsOptionsResponse.Wishlist[];
    WishtListProductLinks: SearchProductsOptionsResponse.WishlistProductLink[];
  }

  export namespace SearchProductsOptionsResponse {
    export interface ProductRelation {
      PrimaryName?: string;
      SecondaryName?: string;
      Direction?: ProductRelationDirection;
      StockRelationType?: StockRelationType;
      /**
       * Entity type: Product
       */
      PrimaryProductIDs?: number[];
      /**
       * Entity type: Product
       */
      RelatedProductIDs?: number[];
      Sequence?: number;
    }

    export interface SerialNumberConfiguration {
      /**
       * Entity type: SerialNumberConfiguration
       */
      ID: number;
      Name: string;
    }

    export interface Wishlist {
      /**
       * Entity type: Wishlist
       */
      ID: number;
      Name: string;
    }

    export interface WishlistProductLink {
      /**
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Entity type: Wishlist
       */
      WishlistID: number;
    }

  }

  export interface PrefigureDiscountResult {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    /**
     * Entity type: DiscountLayer
     */
    LayerID?: number;
    Trigger: DiscountTriggers;
    BackendID: string;
    Description: string;
    MarketingDescription: string;
    PromotionLabel: string;
    ImageBlobID?: string;
    ApplyTo: DiscountAppliesTo;
    MaximumUsagePerOrder?: number;
    /**
     * Entity type: Product
     */
    EligibleProductIDs: number[];
    IsAlreadyApplied: boolean;
    RequirementsMet: boolean;
    RequiresCustomer: boolean;
    RequiresCustomerEmail: boolean;
    RequiresCustomerCustomData: boolean;
    RequiresMinimumAmount: boolean;
    RequiresLoyaltyPoints: boolean;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    LoyaltyPointWithdrawal?: number;
    LoyaltyPointsMissing?: number;
    LoyaltyCanApplyRecursive: boolean;
    RequiresCoupon: boolean;
    CouponPresent: boolean;
    AvailableUserCoupons?: PrefigureDiscountResult.Coupon[];
    ActionType: string;
    ActionData: { [key: string]: any };
    ActionResult: PrefigureDiscountResult.DataResult;
    ConditionsResults: PrefigureDiscountResult.DataResult[];
    IsPersonalizedPromotion: boolean;
    PersonalizedPromotionData: PrefigureDiscountResult.PersonalizedPromotionInformation[];
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
  }

  export namespace PrefigureDiscountResult {
    export interface Coupon {
      /**
       * Entity type: DiscountCoupon
       */
      ID: number;
      Code: string;
      /**
       * Entity type: Discount
       */
      DiscountID: number;
      ValidFrom: string;
      ValidTill?: string;
    }

    export interface DataResult {
      TypeName: string;
      RequirementsMet: boolean;
      RequiresCustomer: boolean;
      RequiresCustomerCustomData: boolean;
      RequiresMinimumAmount: boolean;
      RequiresLoyaltyPoints: boolean;
      CustomerFilterResult: CustomFieldFilterResult;
      ProductSetResults: PrefigureDiscountResult.ProductSetResult[];
      OrderAmountMissing?: number;
      OrderAmountMissingForNextTier?: number;
      CurrentTier: DataResult.ApplicableTier;
      MinimumLoyaltyPointsRequired?: number;
      LoyaltyPointsMissing?: number;
      LoyaltyPointsMissingForNextTier?: number;
    }

    export namespace DataResult {
      export interface ApplicableTier {
        ID?: string;
        OrderAmount: number;
      }

    }

    export interface ProductSetResult {
      ProductSet: DiscountProductSet;
      SetQuantityMissing?: number;
      SetQuantityMissingForNextTier?: number;
      ProductQuantityMissing?: PrefigureDiscountResult.ProductWithQuantity[];
      ProductQuantityMissingForNextTier?: PrefigureDiscountResult.ProductWithQuantity[];
      SetAmountMissing?: number;
      SetAmountMissingForNextTier?: number;
      ProductAmountMissing?: PrefigureDiscountResult.ProductWithAmount[];
      ProductAmountMissingForNextTier?: PrefigureDiscountResult.ProductWithAmount[];
    }

    export interface ProductWithQuantity {
      ProductID: number;
      Quantity: number;
    }

    export interface ProductWithAmount {
      ProductID: number;
      Amount: number;
    }

    export interface PersonalizedPromotionInformation {
      /**
       * Entity type: PersonalizedPromotion
       */
      PersonalizedPromotionID: number;
      RequiresSelection: boolean;
      LocksAfterFirstUsage: boolean;
      IsLocked: boolean;
    }

  }

  export const enum DiscountTriggers {
    Automatic = 0,
    Manual = 1,
    Coupon = 2,
    Bundles = 4,
    Loyalty = 5,
    SystemManaged = -1
  }

  export const enum DiscountAppliesTo {
    None = 0,
    Order = 1,
    Line = 2,
    Both = 3
  }

  export interface CustomFieldFilterResult {
    Matches: boolean;
    FilterType: CustomFieldFilterType;
    FilterResults: CustomFieldFilterResult[];
    ConditionResults: CustomFieldFilterConditionResult[];
  }

  export const enum CustomFieldFilterType {
    And = 0,
    Or = 1
  }

  export interface CustomFieldFilterConditionResult {
    CustomFieldId: number;
    ExpectedValue: CustomFieldValue;
    CurrentValue?: CustomFieldValue;
    CurrentValuePresent: boolean;
    Matches: boolean;
  }

  export interface DiscountProductSet {
    ID?: string;
    Name: string;
    RequiredProductAmount?: number;
    RequiredQuantityOfProducts?: number;
    IsDiscountableProductSet: boolean;
    DiscountableQuantityOfProducts?: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    ProductRequirements?: DiscountProductSet.ProductRequirement[];
    ProductSearchID?: number;
    /**
     * Entity type: PersonalizedPromotion
     */
    PersonalizedPromotionID?: number;
    UseLoyaltyProgramProducts: boolean;
    CountryID: string;
    LanguageID: string;
    Filters: DiscountProductSet.Filter[];
  }

  export namespace DiscountProductSet {
    export interface Filter {
      Exclude: boolean;
      PropertyName: string;
      Values: any[];
      ExactMatch: boolean;
      From?: string;
      To?: string;
    }

    export interface ProductRequirement {
      /**
       * Entity type: ProductRequirement
       */
      ProductRequirementID: number;
      Values: any[];
    }

  }

  /**
   * The normal `AddProductToOrder` cannot be used for adding a Product with type `BundleProduct` to an order. Instead, `AddBundleProductToOrder` must be
   * used. In addition to the `BundleProductID`, this service accepts an optional `LineSelection`, which represents the bundle's configuration in terms of
   * which products you've chosen.
   * 
   * The `LineSelection` data structure contains a `SelectedProductID`, which must refer to a ProductID that is a valid option for this bundle, according
   * to what `GetBundleProductDetails` returns.
   * 
   * If no `LineSelection` is provided then the default products of the bundle are added to the Order.
   */
  export interface AddBundleProductToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    LineActionType?: LineActionTypes;
    /**
     * Entity type: Product
     */
    BundleProductID: number;
    LineSelection?: BundleProductOrderLineSelection[];
    /**
     * If specified the new bundle OrderLine becomes a child of the specified OrderLine.
     * Entity type: OrderLine
     */
    ParentOrderLineID?: number;
  }

  export interface BundleProductOrderLineSelection {
    /**
     * Entity type: ProductBundleLine
     */
    ProductBundleLineID: number;
    /**
     * Entity type: Product
     */
    SelectedProductID: number;
    LineSelection: BundleProductOrderLineSelection[];
  }

  export interface AddBundleProductToOrderResponse {
    /**
     * Created or updated OrderLineID
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This service is used to apply a selection of products to the provided `BundleProductOrderLineID`.
   * The `BundleProductOrderLineID` field refers to the ID of an `OrderLine` on the provided `Order` that has a `BundleProduct` Product on it.
   * The `LineSelection` field is a list that couples a `SelectedProductID` to a `ProductBundleLineID` and is identical to how it's used in `AddBundleProductToOrder`.
   */
  export interface ApplyBundleProductSelection {
    /**
     * Entity type: Order
     */
    OrderID: number;
    LineActionType?: LineActionTypes;
    /**
     * Entity type: OrderLine
     */
    BundleProductOrderLineID: number;
    LineSelection?: BundleProductOrderLineSelection[];
  }

  export interface SimpleShoppingCartResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This service returns a list of bundle products that contain the `ProductID` in the request.
   * The service is meant to be used on a product detail page as a "this product is also available in these bundles" component.
   * The request allows you to specify what data you'd like to get back from the bundle product, through the `IncludedFields` field.
   */
  export interface GetBundleProductsForProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface GetBundleProductsForProductResponse {
    Result: BundleProductContainingProduct[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface BundleProductContainingProduct {
    /**
     * Entity type: Product
     */
    BundleProductID: number;
    BundlePricing: BundleProductPriceInfo;
    BundleProductContent: any;
    IsPromptSuppressed: boolean;
  }

  export interface BundleProductPriceInfo {
    UnitPrice: number;
    UnitPriceInTax: number;
    OriginalUnitPrice?: number;
    OriginalUnitPriceInTax?: number;
    TaxRate: number;
    CurrencyID: string;
  }

  /**
   * This service returns the bundles for which you currently have at least one product in the provided order.
   * This can then be used to suggest bundles to the end-user that they're missing out on.
   * 
   * The IncludedFields field controls which product content you get back for each bundle product. By default, only the product's display value is returned.
   * 
   * Specifying the ProductIDs field means that only suggestions for the given products are returned.
   * If the Order does not contain the provided ProductIDs, nothing will be returned.
   */
  export interface GetPotentialBundleProductsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
  }

  export interface GetPotentialBundleProductsForOrderResponse {
    Result: GetPotentialBundleProductsForOrderResponse.BundleProduct[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetPotentialBundleProductsForOrderResponse {
    export interface BundleProduct {
      BundleProductID: number;
      BundlePricing: BundleProductPriceInfo;
      BundleProductContent: any;
    }

  }

  /**
   * This service creates a ProductBundle for the given BundleProductID.
   * 
   * The request body object contains three parameters:
   * 
   * - BackendID: Optional. Can be filled in with the identifier of this bundle by which it is known in another system.
   * - BundleProductID: Required. This should refer to an EVA product with type `BundleProduct`.
   * - Type: Required. This specifies the type of bundle: discount or configurable. The type can not be changed after the bundle has been created.
   * - Lines: Optional. These specify the `ProductBundle`'s lines. If left empty or null at creation, these can be filled in by calling `UpdateProductBundle`. When specified, a line consists of the following:
   *     - BackendID: Optional. If the bundle exists in another system, this can refer to it.
   *     - Options: A list of available options for this line from which can be chosen. Only contains a ProductID for now, which may refer to a normal sellable product, a product with Type `ConfigurableProduct` or a `BundleProduct` (which would create bundles of bundles).
   *     - DefaultProductID: Optional, but required if there are multiple options or if the line option is a Configurable product. The DefaultProductID cannot be a `ConfigurableProduct` or a `BundleProduct`.
   *     - IsRequired: Is this line an optional part of the bundle or not?
   *     - IsPromptSuppressed: Should the FE suppress the modal that allows the user to select a matching bundle for the scanned product
   */
  export interface CreateProductBundle {
    BackendID?: string;
    /**
     * Entity type: Product
     */
    BundleProductID: number;
    Type: ProductBundleTypes;
    Lines?: CreateProductBundleLine[];
    /**
     * Should the FE suppress the modal that allows the user to select a matching bundle for the scanned product
     */
    IsPromptSuppressed?: boolean;
  }

  export const enum ProductBundleTypes {
    Discount = 0,
    Configurable = 1
  }

  export interface CreateProductBundleLine {
    /**
     * Entity type: Product
     */
    DefaultProductID?: number;
    Description?: string;
    BackendID?: string;
    Type: ProductBundleLineTypes;
    Options: CreateProductBundleLineOption[];
    MaxSelectedOptionsCount?: number;
  }

  export interface CreateProductBundleLineOption {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Quantity?: number;
    /**
     * Entity type: Product
     */
    AdditionalProductID?: number;
    AdditionalProductQuantity: number;
    /**
     * Entity type: Product
     */
    ExcludesProductIDs: number[];
  }

  export interface CreateProductBundleResponse {
    /**
     * Entity type: ProductBundle
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This service is largely the same as CreateProductBundle.
   * The model passed into the request body is considered to be the entire bundle configuration.
   * Any lines that are currently present but not present in the update model are removed from the bundle.
   * The only parameter that is different is that the model requires an ID to be set for the root object and its lines, which you get back from the `GetProductBundle` and `ListProductBundles` services.
   */
  export interface UpdateProductBundle {
    /**
     * Entity type: ProductBundle
     */
    ID: number;
    BackendID?: string;
    /**
     * Should the FE suppress the modal that allows the user to select a matching bundle for the scanned product
     */
    IsPromptSuppressed?: boolean;
    Lines?: UpdateProductBundleLine[];
  }

  export interface UpdateProductBundleLine {
    /**
     * Can be left null for new lines to be added to the bundle.
     * Entity type: ProductBundleLine
     */
    ID?: number;
    Description: string;
    /**
     * Entity type: Product
     */
    DefaultProductID?: number;
    BackendID: string;
    Type: ProductBundleLineTypes;
    Options: UpdateProductBundleLineOption[];
    MaxSelectedOptionsCount?: number;
  }

  export interface UpdateProductBundleLineOption {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Quantity?: number;
    /**
     * Entity type: Product
     */
    AdditionalProductID?: number;
    AdditionalProductQuantity: number;
    /**
     * Entity type: Product
     */
    ExcludesProductIDs: number[];
  }

  /**
   * Deletes a ProductBundle.
   */
  export interface DeleteProductBundle {
    /**
     * Entity type: ProductBundle
     */
    ID: number;
  }

  /**
   * This service returns a paged summary of `ProductBundles`, for each bundle it contains the `BundleProduct` and the pricing information for the bundle.
   */
  export interface ListProductBundles {
    PageConfig?: PageConfig<ListProductBundlesFilter>;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface ListProductBundlesFilter {
    /**
     * The ProductBundleID
     * Entity type: ProductBundle
     */
    ProductBundleID?: number;
    /**
     * The ProductBundle BackendID
     */
    ProductBundleBackendID?: string;
    ProductBundleType?: ProductBundleTypes;
    /**
     * The ProductBundle ProductID
     * Entity type: Product
     */
    ProductBundleProductID?: number;
    /**
     * The ProductBundle Product BackendID
     */
    ProductBundleProductBackendID?: string;
    /**
     * The ProductBundle Product CustomID
     */
    ProductBundleProductCustomID?: string;
    /**
     * The ProductBundle Product Name
     */
    ProductBundleProductName?: string;
  }

  export interface ListProductBundlesResponse {
    Result: PagedResult<ListProductBundlesResponseResult>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListProductBundlesResponseResult {
    /**
     * Entity type: ProductBundle
     */
    ID: number;
    BackendID: string;
    BundleProduct: ProductBundlesProductInfo;
    BundlePricing: BundleProductPriceInfo;
    /**
     * Indicates whether the bundle has any non-default options. If it doesn't, there is no need to configure it after adding it to an order.
     */
    HasNonDefaultOptions: boolean;
    /**
     * Should the FE suppress the modal that allows the user to select a matching bundle for the scanned product
     */
    IsPromptSuppressed: boolean;
  }

  export interface ProductBundlesProductInfo {
    /**
     * Entity type: Product
     */
    ID: number;
    CustomID: string;
    Content: any;
    BackendID: string;
  }

  /**
   * This service is used to fetch a `ProductBundle` by its ID. The requested bundle will be returned with its lines and for each line, its options.
   */
  export interface GetProductBundle {
    /**
     * Entity type: ProductBundle
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface GetProductBundleResponse {
    /**
     * Entity type: ProductBundle
     */
    ID: number;
    BackendID: string;
    BundleProduct: ProductBundlesProductInfo;
    BundlePricing: BundleProductPriceInfo;
    Lines: GetProductBundleResponse.Line[];
    Type: ProductBundleTypes;
    /**
     * Indicates whether the bundle has any non-default options. If it doesn't, there is no need to configure it after adding it to an order.
     */
    HasNonDefaultOptions: boolean;
    /**
     * Should the FE suppress the modal that allows the user to select a matching bundle for the scanned product
     */
    IsPromptSuppressed: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetProductBundleResponse {
    export interface Line {
      /**
       * Entity type: ProductBundleLine
       */
      ID: number;
      Description: string;
      BackendID: string;
      Type: ProductBundleLineTypes;
      DefaultProduct: ProductBundlesProductInfo;
      Options: GetProductBundleResponse.LineOption[];
      MaxSelectedOptionsCount?: number;
    }

    export interface LineOption {
      /**
       * Entity type: ProductBundleLineOption
       */
      ID: number;
      Sequence: number;
      Quantity: number;
      Product: ProductBundlesProductInfo;
      /**
       * Entity type: Product
       */
      ExcludesProductIDs: number[];
    }

  }

  /**
   * Gets the details of a bundle by its ProductID. The input to this service is `BundleProductID` which refers to the ID
   * of a `Product` that has `Type` = `BundleProduct`.
   * 
   * The details of a bundle is a structure containing the lines of the bundle, the available options for each line
   * of the bundle and, if it's a bundle of bundles or a bundle of configurable products, this structure is recursive; each option
   * can have lines of its own again. So the output of this service contains a tree of all the available products for the bundle.
   * 
   * The `IncludedFields` parameter is optional and allows you to specify which fields you wish to get back from this service.
   */
  export interface GetBundleProductDetails {
    /**
     * Entity type: Product
     */
    BundleProductID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface GetBundleProductDetailResponse {
    Lines: BundleProductLine[];
    /**
     * Entity type: Product
     */
    BundleProductID: number;
    Products: GetBundleProductProduct[];
    IsDeleted: boolean;
    /**
     * Should the FE suppress the modal that allows the user to select a matching bundle for the scanned product
     */
    IsPromptSuppressed: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface BundleProductLine {
    /**
     * Entity type: ProductBundleLine
     */
    ID: number;
    BackendID: string;
    /**
     * Entity type: Product
     */
    DefaultProductID?: number;
    Options: BundleProductLineOption[];
    IsRequired: boolean;
    IncludedInBundlePrice: boolean;
    Description: string;
    IsDeleted: boolean;
    MaxSelectedOptionsCount?: number;
  }

  export interface BundleProductLineOption {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Lines: BundleProductLine[];
    /**
     * Entity type: ProductBundleLine
     */
    ProductBundleLineID: number;
    AdditionalUnitPriceInTax?: number;
    /**
     * Entity type: ProductBundleLineOption
     */
    ID: number;
    Quantity: number;
    Sequence: number;
    /**
     * Entity type: Product
     */
    AdditionalProductID?: number;
    AdditionalProductQuantity: number;
    /**
     * Entity type: Product
     */
    ExcludesProductIDs: number[];
  }

  export interface GetBundleProductProduct {
    /**
     * Entity type: Product
     */
    ID: number;
    CustomID: string;
    Type: ProductTypes;
    Content: any;
    Pricing: BundleProductPriceInfo;
  }

  /**
   * Pushes a blob (by `BlobID` or `BlobUrl`) to a device of specified `Type` on given `StationID`.
   */
  export interface Print {
    /**
     * Entity type: Station
     */
    StationID?: number;
    Type?: PrintType;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
  }

  export const enum PrintType {
    Paper = 0,
    Zebra = 1
  }

  /**
   * Renders the result of a previous thermal print job to HTML and/or PDF. Supply either a `BlobID` or `BlobUrl`.
   */
  export interface RenderThermalPrintResult {
    /**
     * The output format. It's possible to provide both HTML and PDF. When Html is specified the response will contain the thermal print result inside the `Html` field.
     */
    OutputFormat?: RenderThermalPrintResultOutputFormat;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
  }

  export const enum RenderThermalPrintResultOutputFormat {
    Html = 1,
    Pdf = 2,
    Png = 4
  }

  export interface RenderThermalPrintResultResponse {
    Html: string;
    PdfUrl: string;
    PngUrl: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Reprints the results of a previous thermal print job to a specified station. Supply either a `BlobID` or `BlobUrl`.
   */
  export interface ReprintThermalPrintResult {
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  /**
   * Returns a tree representation of how the price was derived for the given ProductIDs in the given PriceListID. NOTICE THAT you can provide a maximum of 100 products to explain.
   */
  export interface ExplainProductPrices {
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    /**
     * Entity type: PriceList
     */
    PriceListID: number;
  }

  export interface ExplainProductPricesResponse {
    Products: ExplainProductPricesResponse.ProductPricingExplanation[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ExplainProductPricesResponse {
    export interface ProductPricingExplanation {
      ProductID: number;
      Explanations: ProductPricingExplanation.Explanation[];
    }

    export namespace ProductPricingExplanation {
      export interface PriceListAdjustment {
        ID: number;
        Name: string;
      }

      export interface Explanation {
        PriceListID?: number;
        PriceListAdjustment?: ProductPricingExplanation.PriceListAdjustment;
        NodeType?: string;
        OldPrice?: number;
        NewPrice?: number;
        Message?: string;
        Data?: any;
        Children?: ProductPricingExplanation.Explanation[];
      }

    }

  }

  export interface GetPotentialProductPricing {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Entity type: Order
     */
    OrderID?: number;
  }

  export interface GetPotentialProductPricingResponse {
    Products?: GetPotentialProductPricingResponse.ProductPriceInfo[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetPotentialProductPricingResponse {
    export interface ProductPriceInfo {
      CurrentPrice?: GetPotentialProductPricingResponse.CurrentPriceInfo;
      PotentialPrices?: GetPotentialProductPricingResponse.PotentialPriceInfo[];
    }

    export interface CurrentPriceInfo {
      CurrencyID?: string;
      UnitPrice?: number;
      UnitPriceInTax?: number;
      OriginalUnitPrice?: number;
      OriginalUnitPriceInTax?: number;
      PricingGroupID?: string;
    }

    export interface PotentialPriceInfo {
      CurrencyID?: string;
      UnitPrice?: number;
      UnitPriceInTax?: number;
      OriginalUnitPrice?: number;
      OriginalUnitPriceInTax?: number;
      PricingGroupID?: string;
      Condition?: PotentialPriceInfo.ConditionInfo;
    }

    export namespace PotentialPriceInfo {
      export interface ConditionInfo {
        CustomFields?: ConditionInfo.CustomFieldCondition[];
      }

      export namespace ConditionInfo {
        export interface CustomFieldCondition {
          FieldName?: string;
          RequiredValue?: string;
          PricingGroupID?: string;
        }

      }

    }

  }

  export interface ListManualTaxExemptionReasons {
    PageConfig?: PageConfig<ListTaxExemptionReasonModelFilters>;
  }

  export interface ListTaxExemptionReasonModelFilters {
    Code?: string;
    Name?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface ListManualTaxExemptionReasonsResponse {
    Result?: PagedResult<ManualTaxExemptionReason>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ManualTaxExemptionReason {
    ID: number;
    Code: string;
    Name: string;
    RequiresVerification: boolean;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    OrganizationUnitSetName: string;
  }

  export interface ListProductPrices {
    InitialPageConfig?: PageTokenConfig<ListProductPricesFilter>;
    PageToken?: string;
  }

  export interface ListProductPricesFilter {
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Entity type: PriceList
     */
    PriceListIDs?: number[];
    PriceListAdjustmentTypes?: string[];
    /**
     * Entity type: PriceListUsageType
     */
    PriceListUsageTypeIDs?: number[];
  }

  /**
   * Returns a paged list of all calculated prices, optionally filtered by ProductID and PriceListID.
   * 
   * In addition to filtering on ProductID and PriceListID, it's also possible to only return prices for pricelists that are built using certain adjustment types.
   * 
   * For example, to only return prices for pricelists that contain a MANUAL_INPUT adjustment, pass that string into the `PriceListAdjustmentTypes` array field.
   * 
   * Likewise, to only return pricelists that are attached to an organization with a certain PriceListUsageType you can add the PriceListUsageTypeID to the PriceListUsageTypeIDs filter.
   */
  export interface ListProductPricesResponse {
    Results?: ListProductPricesItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListProductPricesItem {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: PriceList
     */
    PriceListID: number;
    PriceListName: string;
    Price?: number;
    CurrencyID: string;
    LabelID?: number;
    LabelName: string;
  }

  export interface ListTaxCodes {
    PageConfig?: PageConfig<ListTaxCodesFilter>;
  }

  export interface ListTaxCodesFilter {
    /**
     * Entity type: TaxCode
     */
    ID?: number;
    Name: string;
    /**
     * Only returns tax codes which currently have an active tax rate for your current country. Defaults to true.
     */
    ActiveOnly: boolean;
  }

  export interface ListTaxCodesResponse {
    Result?: PagedResult<TaxCodeItem>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface TaxCodeItem {
    /**
     * Entity type: TaxCode
     */
    ID: number;
    Name: string;
    Description: string;
    LedgerClassID: string;
    BackendTaxCodeID: string;
    BackendID: string;
  }

  /**
   * A service that allows to manage the entirety of a price list.
   */
  export interface PushPriceList {
    /**
     * The unique identifier for this pricelist.
     */
    ID?: string;
    /**
     * A string that identifies the system that was used to call this service. In combination with the ID of the pricelist, this is what uniquely identifies the pricelist in EVA.
     * The SystemID must be the same for every call, if it's different from a previous call then a new pricelist will be created.
     */
    SystemID?: string;
    /**
     * If set to true then all prices specified on this price list will be interpreted as already VAT-included. If set to false then all prices will be interpreted as VAT-excluded. If not specified then the default value is true.
     * 
     * So if a price entry of 90 EUR is added to this pricelist with SpecifiedPricesIncludeTax = true (or not specified) then the price will be interpreted as 90 EUR already including VAT.
     * If SpecifiedPricesIncludeTax = false then the price will be interpreted as 90 EUR excluding VAT so that will be added on top for any orders created with this price.
     */
    SpecifiedPricesIncludeTax?: boolean;
    /**
     * The name of the pricelist. Must be present for the creation of a pricelist but can be left empty in subsequent calls.
     */
    Name?: string;
    /**
     * Whether or not the pricelist is active.
     */
    IsActive?: boolean;
    /**
     * The timezone that will be used to interpret activation times for prices.
     */
    TimeZone?: string;
    /**
     * The currency for the pricelist. Must be present for the creation of a pricelist but can be left empty in subsequent calls.
     */
    CurrencyID?: string;
    /**
     * The list of components that make up the pricelist. If left empty this will leave the current configuration unchanged. If specified only the components present will be modified. Components must be explictly deleted.
     */
    Components?: PushPriceList.Component[];
    /**
     * The organization units that use this pricelist. If left empty will leave the current configuration unchanged. If specified, will replace the current configuration.
     */
    ActiveForOrganizationUnits?: PushPriceList.ActiveOrganizationUnit[];
    /**
     * Whether or not to immediately recalculate the effective prices after processing the changes. Otherwise, the prices are automatically recalculated every 15 minutes. Until such a recalculation has been performed, the old price will remain active.
     * Defaults to false, only specify as true if you cannot wait for the normal recalculation cycle.
     */
    RecalculateEffectivePrices?: boolean;
    /**
     * If specified as true, the call to PushPriceList will not return until all uploaded prices have been stored and all prices have been recalculated. Normally this process runs in the background.
     */
    WaitForCompletion?: boolean;
    /**
     * The PushPriceList service is optimized for updating a pricelist in its entirety, so for many products at once when using a PriceEntry component. When using few products or even single products this service can perform unnecessary work. It's always recommended to
     * use this service in a large batch manner but if that's not possible you can specify your intent by using either LowProductCountOption = Auto or Optimize. Effectively this service will simply store the new price entries and return.
     * The updated prices won't become effective until the next calculation round, which happens every 15 minutes. The threshold for 'low product count' is set at 100 products.
     */
    LowProductCountOption?: PushPriceList.LowProductCountOptions;
  }

  export namespace PushPriceList {
    export interface Component {
      /**
       * The unique identifier for this component. Required.
       */
      ID?: string;
      /**
       * The name of the component. If left empty on creation of the component the ID will be used as the name.
       */
      Name?: string;
      /**
       * Determines the type of the component. Cannot be changed after creation.
       */
      Type: PushPriceList.ComponentTypes;
      Label?: string;
      /**
       * Determines when this component will become active. Until then the component will not be included in the calculation of the pricelist.
       */
      StartDate?: string;
      /**
       * Determines when this component will become inactive. After this date the component will not be included in the calculation of the pricelist.
       */
      EndDate?: string;
      /**
       * The PricingGroupID that this component is active for. Components with a PricingGroupID are only active for users that are members of that group. If left empty the component will be active for all users.
       */
      PricingGroupID?: string;
      /**
       * Components are applied in the order of their sequence number, from lowest to highest.
       */
      SequenceNumber?: number;
      /**
       * Determines if the price resulting from this component simply replaces the value as calculated by preceding components or if it's a relative value.
       * 
       * It defaults to false.
       * 
       * This affects the different component types in the following ways:
       * 
       * PriceEntries:
       * 
       * If OverrideCurrentCalculatedPrice = true then the value of the price entry replaces any value by a previous component. If false (the default) then the price entry is added to the existing value.
       * 
       * So if a previous component calculated that the price should be 100 and there is a price entry for 90, then with OverrideCurrentCalculatedPrice = true the resulting price will become 90. With false, it would become 190.
       * 
       * Markup:
       * 
       * It's only applied to a markup component if it has Type = Amount, it does not have any effect for Type = Percentage.
       * 
       * If set to true, a Amount markup component will function like a price entry compnent and replace the current value with the amount of the markup component.
       * 
       * If false (the default) it will add the Amount of the markup component to the current calculated value.
       */
      OverrideCurrentCalculatedPrice?: boolean;
      /**
       * If specified as true, the component will be deleted.
       */
      Delete: boolean;
      /**
       * The data that accompanies a Component with Type = PriceEntries, so a component that is simply a list of product/price pairs.
       * The simplest and most direct way to give a product a price.
       * 
       * This data consists of either a list of product/price pairs or a BlobID which is a reference to a previously created blob in EVA that contains data of the following format:
       * 
       * [
       *     {
       *       "ProductID": "<ProductID>",
       *       "Price": <Price>
       *     },
       *     ...
       * ]
       * 
       * If you only supply a handful of prices, the inline data works well, but for large quantities of prices you may want to use the BlobID method.
       * 
       * This data is ignored for all other Component types.
       * 
       * PriceEntries data is processed asynchronously in the background and will not be immediately visible after the service call returns.
       */
      PriceEntriesData?: PushPriceList.PriceEntriesData;
      /**
       * The data that accompanies a Component with Type = PricesFromOtherPriceList which is a component to copy prices from another pricelist.
       * 
       * Is useful for example for promotion prices, where you first 'import' the prices from the original pricelist, and then apply a negative markup to them to create a promotion discount.
       * 
       * Which prices you copy can be modified by specifying a ProductSearchTemplateID, only products that are contained in the results of the search will be copied,
       * unless ExcludeProductsInFilter is set to true, in which case the reverse is true; all products except the matching ones will have their prices copied.
       */
      CopyPricesFromOtherPriceListData?: PushPriceList.CopyPricesFromOtherPriceListData;
      /**
       * The data that accompanies a Component with Type = Markup which is a component that alters the price value of the previusly applied components,
       * either up or down. The markup/markdown can either be an absolute value or a percentage depending on the Type.
       */
      MarkupData?: PushPriceList.MarkupData;
    }

    export interface PriceEntriesData {
      /**
       * The blob that refers to the price entry data that was previously uploaded to a Blob in EVA. If left empty the inline data will be used.
       * 
       * The data in the blob must be in the following format:
       * 
       * [
       *     {
       *       "ProductID": "<ProductID>",
       *       "Price": <Price>
       *     },
       *     ...
       * ]
       */
      BlobID?: string;
      /**
       * The inline data that will be used to create the price entries. If left empty the blob will be used.
       */
      Prices?: PriceEntriesData.PriceEntryPrice[];
    }

    export namespace PriceEntriesData {
      export interface PriceEntryPrice {
        ID?: string;
        ProductID?: string;
        Price?: number;
        StartDate?: string;
        EndDate?: string;
      }

    }

    export interface CopyPricesFromOtherPriceListData {
      /**
       * The ID of the pricelist to copy prices from.
       */
      PriceListID?: string;
      /**
       * The ID of the ProductSearchTemplate to use to filter the products to copy. If left empty all products will be copied.
       * Entity type: ProductSearchTemplate
       */
      ProductSearchTemplateID?: number;
      /**
       * If true, all products except the matching ones will have their prices copied. If false, only the matching ones will have their prices copied.
       */
      ExcludeProductsInFilter?: boolean;
    }

    export interface MarkupData {
      /**
       * The type of markup/markdown. Either absolute amount or a percentage. Determines how FactorValue will be interpreted.
       */
      Type: FactorType;
      /**
       * The value of the markup/markdown. If Type is Amount, this is the amount to add to the price.
       * If Type is Percentage, this is the amount that the Price will be multiplied by.
       * 
       * A 10% markup should be specified as 1.10, a 25% discount should be specified as 0.75.
       */
      FactorValue: number;
      /**
       * The ID of the ProductSearchTemplate to use to filter the products to copy. If left empty all products will be copied.
       * Entity type: ProductSearchTemplate
       */
      ProductSearchTemplateID?: number;
      /**
       * If true, all products except the matching ones will have their prices copied. If false, only the matching ones will have their prices copied.
       */
      ExcludeProductsInFilter: boolean;
    }

    export const enum ComponentTypes {
      PriceEntries = 0,
      Markup = 1,
      CopyPricesFromOtherPriceList = 2
    }

    export const enum LowProductCountOptions {
      Auto = 0,
      Optimize = 1,
      DoNotOptimize = 2
    }

    export const enum PriceListUsageTypes {
      /**
       * The price that is used as the price that a product is sold for in the normal B2C scenarios.
       */
      Sales = 0,
      /**
       * The price that is used to calculat what the cost price is of products that are sold to the party who sells it.
       */
      Cost = 1,
      /**
       * A promotional price reduction of the normal Sales price.
       */
      Promotion = 2,
      /**
       * The price that is used for sales prices between organizations in a B2B scenario.
       */
      Purchase = 3,
      /**
       * RecommendedRetail prices can be used for display purposes, not used in any logic.
       */
      RecommendedRetail = 4,
      /**
       * Can be used to give an indication of effective product prices after applying discounts. This is purely for display purposes.
       */
      PotentialDiscountedPrice = 5,
      /**
       * Economic taxes (recycling fees) can be used for display purposes, not used in any logic.
       */
      EcoTax = 6
    }

    export interface ActiveOrganizationUnit {
      OrganizationUnitID?: string;
      UsageType?: PushPriceList.PriceListUsageTypes;
      TargetOrganizationUnitID?: string;
      PricingGroupID?: string;
      Delete?: boolean;
    }

  }

  export const enum FactorType {
    Amount = 0,
    Percentage = 1
  }

  export interface PushPriceListResponse {
    /**
     * Entity type: PriceList
     */
    PriceListID: number;
    /**
     * Returns unknown products that could not be processed. Only supported with low product counts.
     */
    UnknownProducts?: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CreateTaxCode {
    Name: string;
    Description?: string;
    LedgerClassID?: string;
    BackendID?: string;
    BackendTaxCodeID?: string;
  }

  export interface CreateTaxCodeResponse {
    /**
     * Entity type: TaxCode
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateTaxCode {
    /**
     * Entity type: TaxCode
     */
    ID: number;
    Name?: string;
    Description?: string;
    BackendID?: string;
    BackendTaxCodeID?: string;
    LedgerClassID?: string;
  }

  export interface DeleteTaxCode {
    /**
     * Entity type: TaxCode
     */
    ID: number;
  }

  /**
   * Create a new manual tax exemption reason.
   */
  export interface CreateManualTaxExemptionReason {
    Code: string;
    Name: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    RequiresVerification?: boolean;
  }

  export namespace CreateManualTaxExemptionReason {
    export interface CreateManualTaxExemptionReasonLine {
      TaxRate: number;
      TaxName: string;
    }

  }

  export interface CreateManualTaxExemptionReasonResponse {
    /**
     * Entity type: TaxExemptionReason
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update a manual tax exemption reason.
   */
  export interface UpdateManualTaxExemptionReason {
    /**
     * Entity type: TaxExemptionReason
     */
    ID: number;
    Code: string;
    Name: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    RequiresVerification?: boolean;
  }

  /**
   * Delete a manual tax exemption reason.
   */
  export interface DeleteManualTaxExemptionReason {
    /**
     * Entity type: TaxExemptionReason
     */
    ID: number;
  }

  /**
   * Delete a manual tax exemption reason line.
   */
  export interface DeleteManualTaxExemptionReasonLine {
    /**
     * Entity type: TaxExemptionReasonLine
     */
    ID: number;
  }

  /**
   * Creates a tax rate.
   * 
   * * `CountryID` is a foreign key to the Countries table, which uses the ISO 3166-2 standard as ID
   * * `CountrySubdivisionID` is a foreign key to the CountrySubdivisions table, which uses the ISO 3166-2 standard as ID
   * * `TaxCodeID` is a foreign key to the TaxCodes table, which can be listed using the `ListTaxCodes` service
   * 
   * Percentages are a human readable formatted result of a fraction - applications use a `Rate` instead as an actual result of fraction to maintain
   * precision and integrity. If you wish to set a 21% tax percentage, that means a 1.21 tax rate. Setting a rate of 1.00 implies a 0% tax percentage,
   * and setting to 2.00 would mean a rediculous 100% tax percentage, doubling prices. You can probably tell that a `Rate` of 2.00 here is much more
   * readable than a 100% increase, but hey; nobody's forcing you to stick to percentages. At least - we're not!
   * 
   * Oh, and you can set a tax rate of 0.00, which would mean it will be fully sponsored by the government and will be free to purchase. The services
   * block anything lower than 0.00 and higher than 2.00 for sanity reasons. Will recap when governments gone mad.
   * 
   * :key: The combination of `CountryID`, `CountrySubdivision` and `TaxCodeID` is unique and will be enforced by EVA.
   * 
   * :point_right: `StartDate` and `EndDate` are optional, and can be used to transition from one rate to a next when it changes. Both these dates
   * are inclusive, meaning you'd want to set the `EndDate` of the expiring tax rate to 2018-12-31 and the `StartDate` of the new tax rate to
   * 2019-01-01 (for example).
   */
  export interface CreateTaxRate {
    /**
     * Entity type: Country
     */
    CountryID: string;
    /**
     * Entity type: CountrySubdivision
     */
    CountrySubdivisionID?: string;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID: number;
    /**
     * Range: 1-2
     */
    Rate: number;
    StartDate?: string;
    EndDate?: string;
    /**
     * Defines in which category the provided Rate falls for the provided CountryID. For example, a rate of 25% in the country Denmark should have Category = High as 25% is the high VAT rate in Denmark.
     * 
     * If the TaxCode used for this TaxRate is one of the predefined ones (High, Low, Zero, Exempt, Intermediate) this parameter is ignored but Category must be provided when using a custom TaxCode.
     * 
     * The Category `Dynamic` should be used for countries such as the US where the actual tax rate is context dependent and calculated dynamically.
     */
    Category?: TaxRateCategory;
  }

  export interface CreateTaxRateResponse {
    /**
     * Entity type: TaxRateInfo
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets a tax rate.
   * 
   * :point_right: See `CreateTaxRate` service for more info.
   */
  export interface GetTaxRate {
    /**
     * Entity type: TaxRateInfo
     */
    ID: number;
  }

  export interface GetTaxRateResponse {
    Result: TaxRateModel;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface TaxRateModel {
    /**
     * Entity type: TaxRateInfo
     */
    ID: number;
    /**
     * Entity type: Country
     */
    CountryID: string;
    /**
     * Entity type: CountrySubdivision
     */
    CountrySubdivisionID?: string;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID: number;
    TaxCodeName: string;
    Rate: number;
    StartDate?: string;
    EndDate?: string;
    Category: TaxRateCategory;
  }

  /**
   * Lists all tax rates.
   * 
   * Supports the filter `OnlyActive`, and when specified, will filter out all tax rates that have a `StartDate` or `EndDate` out of bounds for
   * 'today'. The actual value of the filter is irrelevant.
   * 
   * :point_right: See `CreateTaxRate` service for more info.
   */
  export interface ListTaxRates {
    PageConfig?: PageConfig<ListTaxRateModelFilters>;
  }

  export interface ListTaxRateModelFilters {
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: CountrySubdivision
     */
    CountrySubdivisionID?: string;
    OnlyActive?: boolean;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
    TaxCodeName?: string;
    StartDate?: string;
    EndDate?: string;
  }

  export interface ListTaxRatesResponse {
    Result?: PagedResult<TaxRateModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Modifies a tax rate.
   * 
   * :point_right: See `CreateTaxRate` service for more info.
   */
  export interface UpdateTaxRate {
    /**
     * Entity type: TaxRateInfo
     */
    ID: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: CountrySubdivision
     */
    CountrySubdivisionID?: string;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
    /**
     * Min: 0
     */
    Rate?: number;
    StartDate?: string;
    EndDate?: string;
    /**
     * Defines in which category the provided Rate falls for the provided CountryID. For example, a rate of 25% in the country Denmark should have Category = High as 25% is the high VAT rate in Denmark.
     * 
     * If the TaxCode used for this TaxRate is one of the predefined ones (High, Low, Zero, Exempt, Intermediate) this parameter is ignored but Category must be provided when using a custom TaxCode.
     * 
     * The Category `Dynamic` should be used for countries such as the US where the actual tax rate is context dependent and calculated dynamically.
     */
    Category?: TaxRateCategory;
  }

  /**
   * Deletes a tax rate.
   */
  export interface DeleteTaxRate {
    /**
     * Entity type: TaxRateInfo
     */
    ID: number;
  }

  /**
   * Update the prices in the pricelist specified.
   * 
   * This service takes a simplified approach of the pricing tools EVA offers, but that limits the options you have available. The price list to update may
   * only have a single active adjustment, and that has to be of type `MANUAL_INPUT`.
   * 
   * * The pricelist is identified by the `PriceListID` or the `BackendSystemID` and `BackendID`.
   * * Products can be identified by their (EVA) `ID`.
   * * Updates from this service are persisted in the background, which could take up to 15 minutes to process (unless you set `ForcePriceListRecalculation` to `true`)
   */
  export interface UpdateSimplePrices {
    /**
     * Entity type: PriceList
     */
    PriceListID?: number;
    PriceListBackendID?: string;
    PriceListBackendSystemID?: string;
    Prices: UpdateSimplePrices.Price[];
    /**
     * Set this to `true` when an immediate persistance of these prices is required. Not recommended; background recalculation is usually done within 15 minutes.
     */
    ForcePriceListRecalculation?: boolean;
  }

  export namespace UpdateSimplePrices {
    export interface Price {
      /**
       * Entity type: Product
       * 
       * Allows missing/invalid values when using EVA-IDs-Mode: ExternalIDs.
       */
      ProductID?: number;
      /**
       * Leave empty to delete the price for this product
       */
      NewPrice?: number;
    }

  }

  /**
   * Approve a non-confirmed payment transaction capture
   */
  export interface ApproveCapture {
    /**
     * Entity type: PaymentTransactionCapture
     */
    PaymentTransactionCaptureID: number;
  }

  /**
   * Approve a non-confirmed payment transaction
   */
  export interface ApprovePayment {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentID: number;
  }

  export interface ApprovePaymentResponse {
    HasBeenApproved: boolean;
    Reason?: ApprovementFailureReasons;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum ApprovementFailureReasons {
    AlreadyApproved = 1,
    OrderAlreadyPaid = 2
  }

  /**
   * Cancels a non-confirmed payment transaction
   */
  export interface CancelPayment {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentID: number;
  }

  /**
   * Sends a capture request to the PSP for the given PaymentTransaction with the given amount.
   */
  export interface CapturePayment {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID: number;
    Amount?: number;
  }

  export interface CapturePaymentResponse {
    PaymentTransaction: PaymentTransactionDto;
    OpenAmount?: number;
    OrderAmountToCapture?: number;
    Success: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface PaymentTransactionDto {
    /**
     * Entity type: PaymentTransaction
     */
    ID: number;
    BackendID?: string;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    BackendSystemID: string;
    /**
     * Entity type: Device
     */
    DeviceID?: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    GroupID?: string;
    /**
     * Only exposed with the functionality PaymentDetails (View).
     */
    PaymentDetails?: string;
    /**
     * Only exposed with the functionality PaymentDetails (View).
     */
    ErrorDetails?: string;
    /**
     * Only exposed to API callers with the functionality PaymentData (View).
     */
    Data?: string;
    PaymentDate?: string;
    Amount: number;
    Status: PaymentStatuses;
    IsConfirmed: boolean;
    PendingOrConfirmed: boolean;
    PaidAmount: number;
    PaymentName: string;
    Change: number;
    /**
     * Entity type: PaymentType
     */
    TypeID: number;
    Type: PaymentTypeDto;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: Currency
     */
    ForeignCurrencyID?: string;
    ExchangeRate?: number;
    Description?: string;
    /**
     * Only exposed with the functionality PaymentSettlements (View).
     */
    SettledAmount?: number;
    IsSettled?: boolean;
    RequiresCapture: boolean;
    CapturedAmount?: number;
    CapturableAmount?: number;
    AmountPendingCapture?: number;
    IsCaptured?: boolean;
    ForeignAmount?: number;
    ForeignPaidAmount?: number;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    CaptureMoment?: PaymentTypeCaptureMoment;
    HasDispute: boolean;
    HasUnresolvedDispute: boolean;
    IsRefund: boolean;
    IsAuthorizationAdjustment: boolean;
    IsAuthorizationExpired: boolean;
    AuthorizationExpirationDate?: string;
    Properties?: PaymentTransactionProperties;
    /**
     * Only included when requested
     */
    AvailableActions?: PaymentTransactionActions;
    OrganizationUnit: PaymentOrganizationUnitDto;
  }

  export const enum PaymentStatuses {
    New = 0,
    Pending = 1,
    Partial = 2,
    Confirmed = 10,
    Failed = -1
  }

  export interface PaymentTypeDto {
    Code: string;
    Name: string;
    IsRoundingType: boolean;
    PrintOnDocuments: boolean;
  }

  export const enum PaymentTypeCaptureMoment {
    None = 0,
    Manual = 1,
    Ship = 2,
    Confirmation = 3
  }

  export const enum PaymentTransactionProperties {
    None = 0,
    /**
     * The transaction was reversed by the psp
     */
    Reversed = 1,
    /**
     * The transaction was charched back due to a dispute
     */
    Chargedback = 2,
    /**
     * The losses created by a Reversal or a Chargeback are booked
     */
    DisputeResolved = 4,
    /**
     * A request for cancelling the remaining capturable amount has been done
     */
    CaptureCancelRequested = 8,
    /**
     * A request for cancelling the remaining capturable amount has been confirmed
     */
    CaptureCancelConfirmed = 16,
    /**
     * A request for a fraud check has been issued, pending external verification
     */
    FraudCheckRequested = 32,
    /**
     * A request for a fraud check has passed
     */
    FraudCheckPassed = 64,
    /**
     * A request for a fraud check has been denied
     */
    FraudCheckDenied = 128,
    /**
     * A request for cancelling the remaining capturable amount was failed
     */
    CaptureCancelFailed = 256
  }

  export interface PaymentTransactionActions {
    Cancel: boolean;
    Refund: boolean;
    Update: boolean;
    Approve: boolean;
    Capture: boolean;
  }

  export interface PaymentOrganizationUnitDto {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    Name: string;
    BackendID?: string;
  }

  export interface CaptureShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
  }

  export interface CaptureShipmentResponse {
    Success: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CreatePayment {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
    /**
     * Range: 0-1000000
     */
    Amount?: number;
    Properties?: { [key: string]: any };
    /**
     * If the payment is done in another currency than the Order, this is required
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * When the created payment is confirmed and the order is paid, the order will be shipped automatically when possible.
     */
    AutoShipAfterConfirm?: boolean;
  }

  export interface CreatePaymentResponse {
    OpenAmount: number;
    Properties: { [key: string]: any };
    PaymentTransactions: PaymentTransactionDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create a new paymenttransaction ledger of the given type. Only custom types are allowed
   */
  export interface CreatePaymentTransactionLedger {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    LedgerType?: string;
    Value?: string;
    Description?: string;
    CreationTime?: string;
  }

  /**
   * Create a refund
   * 
   * With this service a refund can be done based on an existing PaymentTransaction, or without it for an Order.
   * 
   * It is required to specify a transaction or to specify the PaymentType to use.
   * 
   * When no Amount is specified the open amount of the order will be used.
   */
  export interface CreateRefund {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    /**
     * Min: 0
     */
    Amount?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    Properties?: { [key: string]: any };
  }

  export interface CreateRefundResponse {
    Success: boolean;
    RefundedOnOrder: boolean;
    PaymentTransactions: PaymentTransactionDto[];
    Properties: { [key: string]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetActivePaymentTypes {
  }

  export interface GetActivePaymentTypesResponse {
    Result: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Retrieve all available payment methods, in general, or for a specific order.
   * 
   * When ReturnUnusablePaymentMethods is set to true, the system will also return unusable payment methods
   */
  export interface GetAvailablePaymentMethods {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * @default false
     */
    ReturnUnusablePaymentMethods?: boolean;
  }

  export interface GetAvailablePaymentMethodsResponse {
    PaymentMethods: string[];
    Results: AvailablePaymentMethod[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface AvailablePaymentMethod {
    Name: string;
    LocalizedName: string;
    IsUsable: boolean;
    IsCustom: boolean;
    NotUsableError: PaymentMethodNotUsableError;
    Types: AvailablePaymentMethodType[];
  }

  export interface PaymentMethodNotUsableError {
    Type: string;
    Message: string;
  }

  export interface AvailablePaymentMethodType {
    /**
     * Entity type: PaymentType
     */
    ID: number;
    Name: string;
    PaymentMethodName: string;
    /**
     * Entity type: PaymentMethod
     */
    PaymentMethodID: number;
    Code: string;
    Data: any;
    /**
     * Only used for refunds.
     */
    CanRefundWithoutOriginalPaymentTransaction?: boolean;
    /**
     * When this field is set, the amount that can be paid with this method is limited.
     */
    MaximumAmount?: number;
    IsFixedAmount: boolean;
  }

  /**
   * Get all available options for refunds on the given order
   * 
   * Refunds can be executed with the CreateRefund service
   */
  export interface GetAvailableRefundPaymentMethodsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetAvailableRefundPaymentMethodsResponse {
    Methods: RefundPaymentMethod[];
    /**
     * The amount currently available for refund on the given order
     */
    OrderAmountAvailableForRefund?: number;
    /**
     * Indicates if this order can be (partially) refunded with RefundOrder.
     */
    IsAutoRefundAvailable: boolean;
    AutoRefundableAmount?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface RefundPaymentMethod {
    Code: string;
    IsCustom: boolean;
    Types: AvailablePaymentMethodType[];
    Transactions: RefundablePaymentTransaction[];
  }

  export interface RefundablePaymentTransaction {
    /**
     * Entity type: PaymentTransaction
     */
    ID: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    BackendID?: string;
    PaidAmount: number;
    RefundableAmount: number;
    ForeignPaidAmount?: number;
    Code: string;
    Type: string;
    /**
     * Entity type: PaymentType
     */
    TypeID: number;
    CreationTime: string;
    CanRefundWithoutInteraction: boolean;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: Currency
     */
    ForeignCurrencyID?: string;
  }

  export interface GetPaymentTransaction {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID: number;
  }

  export interface GetPaymentTransactionResponse {
    PaymentTransaction: PaymentTransactionDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get the PaymentTransactions attached to the given Order
   * 
   * The response also includes an object `AvailableActions`:
   * - Cancel: can the payment be cancelled
   * - Refund: can the payment be refunded
   * - Update: can an update for this payment be requested
   * - Approve: can the payment be approved
   */
  export interface GetPaymentTransactionsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetPaymentTransactionsResponse {
    PaymentTransactions: GetPaymentTransactionsResponse.PaymentTransaction[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetPaymentTransactionsResponse {
    export interface PaymentTransaction {
      DeviceSerialNumber?: string;
      /**
       * Entity type: PaymentTransaction
       */
      ID: number;
      BackendID?: string;
      /**
       * Entity type: User
       */
      CreatedByID: number;
      BackendSystemID: string;
      /**
       * Entity type: Device
       */
      DeviceID?: number;
      /**
       * Entity type: Order
       */
      OrderID: number;
      GroupID?: string;
      /**
       * Only exposed with the functionality PaymentDetails (View).
       */
      PaymentDetails?: string;
      /**
       * Only exposed with the functionality PaymentDetails (View).
       */
      ErrorDetails?: string;
      /**
       * Only exposed to API callers with the functionality PaymentData (View).
       */
      Data?: string;
      PaymentDate?: string;
      Amount: number;
      Status: PaymentStatuses;
      IsConfirmed: boolean;
      PendingOrConfirmed: boolean;
      PaidAmount: number;
      PaymentName: string;
      Change: number;
      /**
       * Entity type: PaymentType
       */
      TypeID: number;
      Type: PaymentTypeDto;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      /**
       * Entity type: Currency
       */
      ForeignCurrencyID?: string;
      ExchangeRate?: number;
      Description?: string;
      /**
       * Only exposed with the functionality PaymentSettlements (View).
       */
      SettledAmount?: number;
      IsSettled?: boolean;
      RequiresCapture: boolean;
      CapturedAmount?: number;
      CapturableAmount?: number;
      AmountPendingCapture?: number;
      IsCaptured?: boolean;
      ForeignAmount?: number;
      ForeignPaidAmount?: number;
      /**
       * Entity type: CustomField
       */
      CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
      CaptureMoment?: PaymentTypeCaptureMoment;
      HasDispute: boolean;
      HasUnresolvedDispute: boolean;
      IsRefund: boolean;
      IsAuthorizationAdjustment: boolean;
      IsAuthorizationExpired: boolean;
      AuthorizationExpirationDate?: string;
      Properties?: PaymentTransactionProperties;
      /**
       * Only included when requested
       */
      AvailableActions?: PaymentTransactionActions;
      OrganizationUnit: PaymentOrganizationUnitDto;
    }

  }

  /**
   * Get the PaymentTransactions attached to the given Invoice
   * 
   * The response also includes an object `AvailableActions`:
   * - Cancel: can the payment be cancelled
   * - Refund: can the payment be refunded
   * - Update: can an update for this payment be requested
   * - Approve: can the payment be approved
   */
  export interface GetPaymentTransactionsForInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
  }

  /**
   * When a PaymentTransaction is disputed due to a Chargeback or a Reversal, the difference can be booked in a different PaymentTransaction to allow the order to be completed.
   * 
   * Calling this service will result in an additional PaymentTransaction of type `DISPUTE`.
   */
  export interface HandlePaymentTransactionDispute {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID: number;
  }

  /**
   * Set up a payment type
   */
  export interface InitializePaymentType {
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
    Properties?: { [key: string]: any };
  }

  export interface InitializePaymentTypeResponse {
    Properties: { [key: string]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List PaymentTransactionLedger
   * 
   * All available filters are typed in the PageConfig.Filter property.
   */
  export interface ListPaymentTransactionLedger {
    PageConfig?: PageConfig<ListPaymentTransactionLedgerFilter>;
  }

  export interface ListPaymentTransactionLedgerFilter {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: PaymentTransactionLedgerType
     */
    TypeID?: number;
  }

  export interface ListPaymentTransactionLedgerResponse {
    Result?: PagedResult<ListPaymentTransactionLedgerResponse.PaymentTransactionLedgerDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListPaymentTransactionLedgerResponse {
    export interface PaymentTransactionLedgerDto {
      /**
       * Entity type: PaymentTransactionLedger
       */
      ID: number;
      /**
       * Entity type: PaymentTransaction
       */
      PaymentTransactionID: number;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      /**
       * Entity type: PaymentTransactionLedgerType
       */
      TypeID: number;
      TypeName: string;
      OldValue: string;
      NewValue: string;
      Description: string;
      CreationTime: string;
      CreatedBy: ListPaymentTransactionLedgerResponse.UserDto;
    }

    export interface UserDto {
      /**
       * Entity type: User
       */
      ID: number;
      FullName: string;
    }

  }

  /**
   * Set a preferred method to use when this order will be refunded.
   * Can be used on ReturnOrders to indicate how the money should be transferred back to the customer.
   * 
   * Available methods can be listed with the GetAvailableRefundPaymentMethodsForOrder service.
   */
  export interface SetPreferredRefundMethod {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    Properties?: { [key: string]: any };
  }

  /**
   * Get the selected preferred refund method to use when this order will be refunded.
   * 
   * See SetPreferredRefundMethod for more information.
   */
  export interface GetPreferredRefundMethod {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetPreferredRefundMethodResponse {
    PaymentMethod?: string;
    PaymentType?: PaymentTypeDto;
    PaymentTransaction?: PaymentTransactionDto;
    Properties?: { [key: string]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Certain payment methods support retrieving the current status. Given a PaymentTransactionID this service will attempt to poll thestatus for that particular payment and directly update its status.
   */
  export interface RequestPaymentUpdate {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID: number;
  }

  export interface RequestPaymentUpdateResponse {
    PaymentTransaction: PaymentTransactionDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Retrieve the anonymous token for specified `OrganizationUnitID` and `LanguageID`. The response is your token, or null when not present.
   */
  export interface GetAnonymousToken {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
  }

  export interface GetAnonymousTokenResponse {
    Token: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create an anonymous token for specified `OrganizationUnitID` and `LanguageID`. The response is your new token, or a current one if it already existed.
   */
  export interface CreateAnonymousToken {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    LanguageID?: string;
  }

  export interface CreateAnonymousTokenResponse {
    Token: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Deletes an existing anonymous token.
   */
  export interface DeleteAnonymousToken {
    Token?: string;
  }

  /**
   * Archive an organization unit.
   * NOTE: Only do this when it is no longer in use. While this action is reversible, settings and other configurations previously in place may be lost.
   */
  export interface ArchiveOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     * Min: 1
     */
    OrganizationUnitID: number;
    /**
     * Enabling this will archive all children of the organization unit
     */
    ForceArchiveArchiveChildren?: boolean;
  }

  /**
   * Restore an archived organization unit.
   */
  export interface UnarchiveOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  /**
   * Updates some of the non-unique the OrganizationUnit's settings. This service can be used to only update a subset of the settings of the OrganizationUnit by providing only the fields you want to update.
   * If a field is not part of the request message (or has value `undefined`) then the setting is not updated. If the field is present but has a value of `null` then it's assumed you want to unset the setting.
   * 
   * The targeted OrganizationUnits can be identified by providing `OrganizationUnitIDs` or an `OrganizationUnitSetID`.
   * 
   * All OrganizationUnits will be updated, if one of them fails, no changes will be persisted.
   */
  export interface BulkUpdateOrganizationUnitSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    RegisterCashLimit?: number;
    SafeCashLimit?: number;
    /**
     * Entity type: CashHandler
     */
    CashHandlerID?: number;
    Type?: OrganizationUnitTypes;
    /**
     * Entity type: TaxProvider
     */
    TaxProviderID?: number;
    /**
     * Min: 0
     */
    CashFloat?: number;
    Subnet?: string;
    UseForAccounting?: boolean;
    /**
     * Entity type: User
     */
    AttachedToUserID?: number;
    CountryID?: string;
    CountrySubdivisionID?: string;
    LanguageID?: string;
    TimeZone?: string;
    /**
     * Entity type: Assortment
     */
    AssortmentID?: number;
    /**
     * Entity type: RoleSet
     */
    RoleSetID?: number;
    CostPriceCalculationMethod?: CostPriceCalculationMethod;
    /**
     * Entity type: OrganizationUnit
     */
    ReturnOrganizationUnitID?: number;
    EnableEndlessSupplierStock?: boolean;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    ForceLocalMode?: boolean;
  }

  export const enum CostPriceCalculationMethod {
    Manual = 0,
    WeightedAverageCost = 1,
    LastIn = 2,
    Fifo = 3
  }

  /**
   * Create a new OrganizationUnit
   */
  export interface CreateOrganizationUnit {
    ToCreate?: CreateOrganizationUnitDto;
  }

  export interface CreateOrganizationUnitDto {
    BackendID?: string;
    Name?: string;
    Description?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    Type?: OrganizationUnitTypes;
    Status?: OrganizationUnitStatus;
    Latitude?: number;
    Longitude?: number;
    Subnet?: string;
    Address?: AddressDataDto;
    OpeningHours?: CreateOrganizationUnitDto.OpeningHour[];
    Website?: string;
    BIC?: string;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ParentID?: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: CountrySubdivision
     */
    CountrySubdivisionID?: string;
    /**
     * Entity type: Assortment
     */
    AssortmentID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ReturnOrganizationUnitID?: number;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Entity type: Currency
     */
    CostPriceCurrencyID?: string;
    Notes?: string;
    BankAccount?: string;
    BackendRelationID?: string;
    BranchNumber?: string;
    GlobalLocationNumber?: string;
    RegisterCashLimit?: number;
    SafeCashLimit?: number;
    /**
     * Min: 0
     */
    CashFloat?: number;
    /**
     * Entity type: TaxProvider
     */
    TaxProviderID?: number;
    /**
     * Entity type: CashHandler
     */
    CashHandlerID?: number;
    UseForAccounting?: boolean;
    /**
     * Entity type: User
     */
    AttachedToUserID?: number;
    IpAddress?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    TimeZone?: string;
    /**
     * Entity type: RoleSet
     */
    RoleSetID?: number;
    CostPriceCalculationMethod?: CostPriceCalculationMethod;
    EnableEndlessSupplierStock?: boolean;
    EstablishmentNumber?: string;
  }

  export namespace CreateOrganizationUnitDto {
    export interface OpeningHour {
      /**
       * Entity type: OpeningHoursType
       */
      TypeID?: number;
      DayOfWeek?: DayOfWeek;
      Date?: string;
      Description?: string;
      StartTime?: string;
      EndTime?: string;
    }

  }

  export interface CreateOrganizationUnitResponse {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface DeleteOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     * Min: 1
     */
    OrganizationUnitID: number;
    /**
     * Enabling this will delete all children of the organization unit
     */
    ForceDeleteArchiveChildren?: boolean;
  }

  /**
   * @deprecated Use `GetOrganizationUnitDetailed` (authenticated) or `GetOrganizationUnitSummary` (anonymous)
   */
  export interface GetOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetOrganizationUnitResponse {
    Result: OrganizationUnitDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * @deprecated Use `ListOrganizationUnitDetailed` (authenticated) or `ListOrganizationUnitSummaries` (anonymous)
   */
  export interface ListOrganizationUnits {
    PageConfig?: PageConfig<ListOrganizationUnitsFilter>;
  }

  export interface ListOrganizationUnitsFilter {
    /**
     * Entity type: OrganizationUnit
     */
    IDs?: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Name?: string;
    BackendIDs?: string[];
    NameOrBackendID?: string;
    /**
     * Entity type: Country
     */
    CountryIDs?: string[];
    TypeID?: number;
    StatusID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ParentID?: number;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    /**
     * Entity type: RoleSet
     */
    RoleSetID?: number;
    Functionality?: string;
    FunctionalityScope?: FunctionalityScope;
    IncludeArchived?: boolean;
  }

  export interface ListOrganizationUnitsResponse {
    Result?: PagedResult<ListOrganizationUnitsResponse.OrganizationUnit>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrganizationUnitsResponse {
    export interface OrganizationUnit {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      BackendID: string;
      /**
       * Entity type: Company
       */
      CompanyID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      ParentID?: number;
      Name: string;
      Description: string;
      EmailAddress: string;
      PhoneNumber: string;
      Type: OrganizationUnitTypes;
      Status: OrganizationUnitStatus;
      Latitude?: number;
      Longitude?: number;
      Address: AddressDto;
      Website?: string;
      BIC?: string;
      UpcomingOpeningHours: WeekOpeningsDto[];
      RegularOpeningHours: RegularOpeningHours[];
      CustomFields: CustomFieldResponse[];
      /**
       * Entity type: Currency
       */
      CurrencyID?: string;
      IsArchived: boolean;
    }

  }

  export interface WeekOpeningsDto {
    DayOfWeek: DayOfWeek;
    OpeningTime: string;
    ClosingTime: string;
    Date: string;
    Closed: boolean;
  }

  export interface RegularOpeningHours {
    DayOfWeek: DayOfWeek;
    OpeningTime: string;
    ClosingTime: string;
    Closed: boolean;
  }

  /**
   * Returns all `CashHandlers` available to use on `OrganizationUnits`
   */
  export interface GetAvailableCashHandlers {
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
  }

  export interface GetAvailableCashHandlersResponse {
    Handlers: GetAvailableCashHandlersResponse.CashHandler[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAvailableCashHandlersResponse {
    export interface CashHandler {
      ID: number;
      Name: string;
      CurrencyID: string;
    }

  }

  /**
   * Retrieve detailed organization unit information.
   */
  export interface GetOrganizationUnitDetailed {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
  }

  export interface GetOrganizationUnitDetailedResponse {
    /**
     * Entity type: OrganizationUnit
     */
    ID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ParentID?: number;
    ParentName?: string;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    CompanyName?: string;
    /**
     * Entity type: Address
     */
    AddressID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    /**
     * Entity type: TaxProvider
     */
    TaxProviderID?: number;
    VisibilityGroupName?: string;
    Name?: string;
    BackendID?: string;
    BackendRelationID?: string;
    CashFloat?: number;
    Description?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    Type?: OrganizationUnitTypes;
    Status?: OrganizationUnitStatus;
    OpeningHours?: OpeningHoursDto[];
    ExtendedOpeningHours?: ExtendedOpeningHours[];
    Subnet?: string;
    RegisterCashLimit?: number;
    SafeCashLimit?: number;
    Latitude?: number;
    Longitude?: number;
    BankAccount?: string;
    Website?: string;
    BIC?: string;
    TimeZone?: string;
    EstablishmentNumber?: string;
    Address?: AddressDto;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    CountryID?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetOrganizationUnitSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetOrganizationUnitSettingsResponse {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    ParentID?: number;
    BackendID: string;
    BackendRelationID: string;
    BranchNumber: string;
    GlobalLocationNumber: string;
    RegisterCashLimit?: number;
    SafeCashLimit?: number;
    /**
     * Entity type: CashHandler
     */
    CashHandlerID?: number;
    CashHandlerName: string;
    CashFloat?: number;
    Type: OrganizationUnitTypes;
    Subnet: string;
    UseForAccounting: boolean;
    /**
     * Entity type: User
     */
    AttachedToUserID?: number;
    AttachedToUserName?: string;
    IpAddress: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    /**
     * Entity type: CountrySubdivision
     */
    CountrySubdivisionID?: string;
    Region: string;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: Currency
     */
    CostPriceCurrencyID: string;
    TimeZone: string;
    /**
     * Entity type: Assortment
     */
    AssortmentID?: number;
    AssortmentName: string;
    /**
     * Entity type: RoleSet
     */
    RoleSetID: number;
    RoleSetName: string;
    CostPriceCalculationMethod: CostPriceCalculationMethod;
    /**
     * Entity type: OrganizationUnit
     */
    ReturnOrganizationUnitID?: number;
    ReturnOrganizationUnitName?: string;
    /**
     * Entity type: TaxProvider
     */
    TaxProviderID?: number;
    TaxProviderName?: string;
    ForceLocalMode: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns the full hierarchy of organization units, starting from the root and then containing a nested list of child OU's.
   */
  export interface GetOrganizationUnitsHierarchy {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Functionality?: string;
    FunctionalityScope?: FunctionalityScope;
  }

  export interface GetOrganizationUnitsHierarchyResponse {
    OrganizationUnits: OrganizationUnitHierarchy[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrganizationUnitHierarchy {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    Name: string;
    BackendID: string;
    /**
     * Entity type: OrganizationUnit
     */
    ParentID?: number;
    Children?: OrganizationUnitHierarchy[];
  }

  /**
   * Returns a summary of a specific organizationunit. Since this service is publicly available, this will return minimal information (aka a summary) about the OU.
   * Service can also be called for archived organization units to fetch minimal information.
   */
  export interface GetOrganizationUnitSummary {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetOrganizationUnitSummaryResponse {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    Name: string;
    Address?: AddressDto;
    PhoneNumber?: string;
    Latitude?: number;
    Longitude?: number;
    UpcomingOpeningHours: WeekOpeningsDto[];
    RegularOpeningHours: RegularOpeningHours[];
    /**
     * Entity type: OrganizationUnit
     */
    ParentID?: number;
    /**
     * Entity type: Country
     */
    CountryID: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetRequiredOrganizationUnitFields {
  }

  export interface GetRequiredOrganizationUnitFieldsResponse {
    RequiredFields: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Shows in which OU sets an OU is included.
   */
  export interface GetSetsByOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
  }

  export interface GetSetsByOrganizationUnitResponse {
    Result: GetSetsByOrganizationUnitResponse.OrganizationUnitSetDetails[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetSetsByOrganizationUnitResponse {
    export interface OrganizationUnitSetDetails {
      /**
       * Entity type: OrganizationUnitSet
       */
      ID: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      Name: string;
      Description: string;
      Subsets: GetSetsByOrganizationUnitResponse.OrganizationUnitSetDetails[];
    }

  }

  /**
   * Returns stores in proximity of a set of coordinates.
   * 
   * If `ProductID` and `HasStock` is set, only the stores with stock for that product will be returned.
   */
  export interface GetShopsByProximity {
    /**
     * Amount of stores to return, defaults to setting `GetShopsByProximityDefaultLimit` / 10
     */
    Limit?: number;
    /**
     * Range around the coordinates to look for shops, in meters, defaults to setting `DefaultShopProximityRange` / 50_000
     */
    Range?: number;
    /**
     * Country code of the user (ISO 3166-1)
     */
    CountryID?: string;
    /**
     * Filter only stores with available stock
     */
    HasStock?: boolean;
    Status?: OrganizationUnitStatus;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Range: -90-90
     */
    Latitude: number;
    /**
     * Range: -180-180
     */
    Longitude: number;
  }

  export interface GetShopsByProximityResponse {
    Shops: GetShopsByProximityResponse.ShopInfo[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetShopsByProximityResponse {
    export interface ShopInfo {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
      Address: AddressDto;
      PhoneNumber: string;
      Latitude: number;
      Longitude: number;
      Distance: number;
    }

  }

  /**
   * This service return the available webshop OrganizationUnits
   */
  export interface GetWebshops {
  }

  export interface GetWebshopsResponse {
    Webshops: GetWebshopsResponse.WebshopDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetWebshopsResponse {
    export interface WebshopDto {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
      /**
       * Entity type: Country
       */
      CountryID: string;
    }

  }

  /**
   * Creates or updates specified `OrganizationUnits`, identified by your `ID` (that we map to our `BackendID`).
   * 
   * Omit any field you don't want altered, or send in null values of nullable fields to unset their current value.
   * 
   * - Does not apply to `Address` - if you specify a new or different one from the current, a new Address will be created with only the data specify!
   * - Does not apply to `OpeningHours` - every value you specify will either update or create a record for that day
   */
  export interface ImportOrganizationUnits {
    OrganizationUnits?: ImportOrganizationUnits.ImportOrganizationUnit[];
  }

  export namespace ImportOrganizationUnits {
    export interface ImportOrganizationUnit {
      /**
       * Your identifier for an organization unit (mapped as `BackendID`)
       */
      ID: string;
      /**
       * Your identifier for a parent organization unit (mapped as `BackendID`)
       */
      ParentID: string;
      /**
       * Name of the organization unit
       */
      Name: string;
      Description?: string;
      EmailAddress?: string;
      PhoneNumber?: string;
      VatNumber?: string;
      Type?: OrganizationUnitTypes;
      Status?: OrganizationUnitStatus;
      /**
       * Entity type: Company
       */
      CompanyID?: number;
      CompanyBackendID?: string;
      Latitude?: number;
      Longitude?: number;
      Subnet?: string;
      TimeZone?: string;
      Region?: string;
      IsDeleted?: boolean;
      /**
       * Entity type: Country
       */
      CountryID?: string;
      /**
       * Entity type: CountrySubdivision
       */
      CountrySubdivisionID?: string;
      /**
       * Entity type: Language
       */
      LanguageID?: string;
      /**
       * Entity type: Currency
       */
      CurrencyID?: string;
      /**
       * Entity type: Currency
       */
      CostPriceCurrencyID?: string;
      ReturnOrganizationUnitID?: string;
      /**
       * Optional additional custom fields
       */
      CustomFields?: CustomFieldKeyValue[];
      Address?: ImportOrganizationUnits.ImportOrganizationUnitAddress;
      OpeningHours?: ImportOrganizationUnits.ImportOrganizationUnitOpeningHour[];
    }

    export interface ImportOrganizationUnitAddress {
      FirstName?: string;
      LastName?: string;
      HouseNumber?: string;
      Address1?: string;
      Address2?: string;
      ZipCode?: string;
      Subdistrict?: string;
      District?: string;
      City?: string;
      State?: string;
      Region?: string;
      CountryID?: string;
      Latitude?: number;
      Longitude?: number;
    }

    export interface ImportOrganizationUnitOpeningHour {
      DayOfWeek?: DayOfWeek;
      StartTime?: string;
      EndTime?: string;
      Date?: string;
      Description?: string;
      /**
       * Entity type: OpeningHoursType
       */
      TypeID?: number;
      /**
       * Name of the OpeningHoursType, used when TypeID is not set
       */
      TypeName?: string;
    }

  }

  export interface ImportOrganizationUnitsResponse {
    Created: { [key: string]: number };
    Updated: { [key: string]: number };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns all data of a list of organizationunits. Since this service is not publicly available, this will return all information about the OUs.
   */
  export interface ListOrganizationUnitsDetailed {
    /**
     * Indicate some properties that will be omitted in the response, to keep the responsesize down
     */
    Excluded?: ListOrganizationUnitsDetailed.ExcludedProperties;
    PageConfig?: PageConfig<ListOrganizationUnitsFilter>;
  }

  export namespace ListOrganizationUnitsDetailed {
    export const enum ExcludedProperties {
      CustomFields = 1,
      Openinghours = 2,
      Address = 4
    }

  }

  export interface ListOrganizationUnitsDetailedResponse {
    Result?: PagedResult<ListOrganizationUnitsDetailedResponse.OrganizationUnit>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrganizationUnitsDetailedResponse {
    export interface OrganizationUnit {
      /**
       * Entity type: OrganizationUnit
       */
      ID?: number;
      BackendID?: string;
      /**
       * Entity type: Company
       */
      CompanyID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      ParentID?: number;
      Name?: string;
      Description?: string;
      EmailAddress?: string;
      PhoneNumber?: string;
      Type?: OrganizationUnitTypes;
      Status?: OrganizationUnitStatus;
      Latitude?: number;
      Longitude?: number;
      Address?: AddressDto;
      Website?: string;
      BIC?: string;
      UpcomingOpeningHours?: WeekOpeningsDto[];
      RegularOpeningHours?: RegularOpeningHours[];
      CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
      /**
       * Entity type: Currency
       */
      CurrencyID?: string;
      CountryID?: string;
      IsArchived?: boolean;
    }

  }

  /**
   * Returns summaries of a list of organizationunits. Since this service is publicly available, this will return minimal information (aka a summary) about the OUs.
   */
  export interface ListOrganizationUnitSummaries {
    PageConfig?: PageConfig<ListOrganizationUnitsFilter>;
  }

  export interface ListOrganizationUnitSummariesResponse {
    Result?: PagedResult<ListOrganizationUnitSummariesResponse.OrganizationUnit>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrganizationUnitSummariesResponse {
    export interface OrganizationUnit {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
      /**
       * Entity type: OrganizationUnit
       */
      ParentID?: number;
      /**
       * Entity type: Country
       */
      CountryID: string;
      /**
       * Entity type: Language
       */
      LanguageID?: string;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      Address?: AddressDto;
      PhoneNumber?: string;
      Latitude?: number;
      Longitude?: number;
    }

  }

  /**
   * Service that returns OrganizationUnitDtos that are approximately in range of the given latitude and longitude. The result are returned in ascending order.
   * 
   * This service is useful for displaying a list of the nearest shops (eg: checkout and choosing a pick-up shop).
   * 
   * The page size `Limit` has a maximum value of `100` for this service.
   */
  export interface ListShopsByProximity {
    /**
     * Latitude of a user or device of some kind to search stores that are nearby.
     * Range: -90-90
     */
    Latitude: number;
    /**
     * Longitude of a user or device of some kind to search stores that are nearby.
     * Range: -180-180
     */
    Longitude: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListShopsByProximityResponse {
    Result?: PagedResult<ListShopsByProximityResponse.ShopInfo>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListShopsByProximityResponse {
    export interface ShopInfo {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
      Description: string;
      Address: AddressDto;
      PhoneNumber: string;
      Latitude: number;
      Longitude: number;
      UpcomingOpeningHours: WeekOpeningsDto[];
      RegularOpeningHours: RegularOpeningHours[];
    }

  }

  /**
   * List the Shop OrganizationUnits in a certain area
   * 
   * Included are the OpeningHours in the next 7 days, and the Regular OpeningHours.
   * 
   * This service is useful for showing all shops in a map, where you want to search in a given rectangle.
   */
  export interface ListShopsInArea {
    NorthWest: ListShopsInArea.Coordinate;
    SouthEast: ListShopsInArea.Coordinate;
    Status?: OrganizationUnitStatus;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export namespace ListShopsInArea {
    export interface Coordinate {
      Latitude: number;
      Longitude: number;
    }

  }

  export interface ListShopsInAreaResponse {
    Result?: PagedResult<ListShopsInAreaResponse.ShopInfo>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListShopsInAreaResponse {
    export interface ShopInfo {
      ID: number;
      Name: string;
      Address: AddressDto;
      PhoneNumber: string;
      Latitude: number;
      Longitude: number;
      UpcomingOpeningHours: WeekOpeningsDto[];
      RegularOpeningHours: RegularOpeningHours[];
    }

  }

  export interface GetOrganizationUnitNotes {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetOrganizationUnitNotesResponse {
    Notes: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface SetOrganizationUnitNotes {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    Notes: string;
  }

  /**
   * Update custom fields for multiple organization units at once.
   */
  export interface UpdateCustomFieldsForOrganizationUnits {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * When set, OrganizationUnitIDs will be ignored and instead all organization units for this set will be updated
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * The values in here will be merged with existing custom field values
     */
    CustomFields?: CustomFieldKeyValue[];
  }

  /**
   * Updates the basic information associated with an OrganizationUnit. This service can be used to only update a subset of the settings of the OrganizationUnit by providing only the fields you want to update.
   * 
   * If a field is not part of the request message (or has value `undefined`) then the setting is not updated. If the field is present but has a value of `null` then it's assumed you want to unset the setting.
   */
  export interface UpdateOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    Name?: string;
    Description?: string;
    Status?: OrganizationUnitStatus;
    Latitude?: number;
    Longitude?: number;
    BankAccount?: string;
    BranchNumber?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    Address?: AddressDataDto;
    Website?: string;
    BIC?: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    EstablishmentNumber?: string;
  }

  export namespace UpdateOrganizationUnit {
    export interface OpeningHour {
      /**
       * Entity type: OpeningHours
       */
      ID?: number;
      /**
       * Entity type: OpeningHoursType
       */
      TypeID?: number;
      DayOfWeek?: DayOfWeek;
      Date?: string;
      Description?: string;
      IsClosed?: boolean;
      StartTime?: string;
      EndTime?: string;
      IsActive?: boolean;
    }

  }

  /**
   * Updates the OrganizationUnit's settings. This service can be used to only update a subset of the settings of the OrganizationUnit by providing only the fields you want to update.
   * If a field is not part of the request message (or has value `undefined`) then the setting is not updated. If the field is present but has a value of `null` then it's assumed you want to unset the setting.
   */
  export interface UpdateOrganizationUnitSettings {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    ParentID?: number;
    BackendID?: string;
    BackendRelationID?: string;
    GlobalLocationNumber?: string;
    RegisterCashLimit?: number;
    SafeCashLimit?: number;
    /**
     * Entity type: CashHandler
     */
    CashHandlerID?: number;
    /**
     * Entity type: TaxProvider
     */
    TaxProviderID?: number;
    /**
     * Min: 0
     */
    CashFloat?: number;
    Type?: OrganizationUnitTypes;
    Subnet?: string;
    UseForAccounting?: boolean;
    /**
     * Entity type: User
     */
    AttachedToUserID?: number;
    IpAddress?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: CountrySubdivision
     */
    CountrySubdivisionID?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    TimeZone?: string;
    /**
     * Entity type: Assortment
     */
    AssortmentID?: number;
    /**
     * Entity type: RoleSet
     */
    RoleSetID?: number;
    CostPriceCalculationMethod?: CostPriceCalculationMethod;
    /**
     * Entity type: OrganizationUnit
     */
    ReturnOrganizationUnitID?: number;
    EnableEndlessSupplierStock?: boolean;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    Name?: string;
    Description?: string;
    Status?: OrganizationUnitStatus;
    Latitude?: number;
    Longitude?: number;
    BankAccount?: string;
    BranchNumber?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    Address?: AddressDataDto;
    Website?: string;
    BIC?: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    ForceLocalMode?: boolean;
  }

  /**
   * @deprecated Service is split in to two seperate services. Use 'AddDiscountCouponToOrder' to add a coupon to an order. Use 'AddManualDiscountToOrder' to add manual discounts to an order.
   * Add a Discount to an Order
   * 
   * - To add a Coupon discount, specify the `CouponCode`.
   * - To add a Manual discount, specify the `DiscountID` and the `DiscountAmount`. Optionally a `Reason` can be added.
   */
  export interface AddDiscountToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    CouponCode?: string;
    /**
     * Entity type: Discount
     */
    DiscountID?: number;
    DiscountAmount?: number;
    Reason?: string;
    OrderLines?: OrderLineWithQuantity[];
  }

  export interface OrderLineWithQuantity {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    Quantity?: number;
  }

  /**
   * Add a discount coupon to an order
   */
  export interface AddDiscountCouponToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    CouponCode: string;
  }

  export interface AddDiscountToOrderResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    DiscountIsApplied: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Add a manual Discount to an order
   */
  export interface AddManualDiscountToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    DiscountAmount?: number;
    Reason?: string;
    CustomFields?: CustomFieldKeyValue[];
    OrderLines?: OrderLineWithQuantity[];
  }

  /**
   * Add a loyalty Discount to an order
   */
  export interface AddLoyaltyDiscountToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    /**
     * Min: 1
     */
    NumberOfTimesToApply?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
  }

  /**
   * Cancel a discount order line.
   * 
   * When the discount of the given line is a coupon triggerd discount, the coupon will be removed from the order.
   * When the discount of the given line is a manual discount, the manual discount will be cancelled.
   * When the discount of the given line is an automatic discount, the discount will be disabled on the order. To re-enable it you must call the 'EnableDisabledDiscountOnOrder' service. You can check which discount are disabled on an order by calling 'ListDisabledDiscountsOnOrder'
   */
  export interface CancelDiscountOrderLine {
    /**
     * Entity type: Order
     */
    OrderLineID: number;
  }

  /**
   * Get discount product options for a specific discount line
   */
  export interface GetDiscountProductOptionsForOrderLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    IncludedProperties?: string[];
  }

  export interface GetDiscountProductOptionsForOrderLineResponse {
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    ActionType: string;
    ActionSelectionData: any;
    /**
     * Entity type: Product
     */
    CurrentSelection?: number;
    SelectionOptions: any[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get discount product options for order
   */
  export interface GetDiscountProductOptionsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    IncludedProperties?: string[];
  }

  export interface GetDiscountProductOptionsForOrderResponse {
    DiscountOptions: GetDiscountProductOptionsForOrderResponse.DiscountOption[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetDiscountProductOptionsForOrderResponse {
    export interface DiscountOption {
      /**
       * Entity type: Discount
       */
      DiscountID: number;
      ActionType: string;
      ActionSelectionData: any;
      LineOptions: GetDiscountProductOptionsForOrderResponse.DiscountLineOption[];
    }

    export interface DiscountLineOption {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      /**
       * Entity type: Product
       */
      CurrentSelection?: number;
      SelectionOptions: any[];
    }

  }

  /**
   * Set the product selection for a specific discount line
   */
  export interface SetDiscountProductOption {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * When set to 'true' selection must be within the given product options
     */
    DisallowSelectionOutsideProductOptions?: boolean;
  }

  /**
   * Add a Product to an Order
   * 
   * A product can be defined by `ProductID` or `Barcode`.
   * If the product is unstackable (e.g. gift cards, same product added with different serial numbers), new orderlines are created.
   */
  export interface AddProductToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Barcode?: string;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Min: 9000
     */
    QuantityOrdered?: number;
    LineActionType: LineActionTypes;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    GroupID?: string;
    /**
     * If the product allows its price to be modified, you can specify the desired price using this field.
     */
    UnitPriceInTax?: number;
    Children: AddProductToOrder.ChildOrderLine[];
    /**
     * This will always create a new line, regardless of whether a line with the same product ID was found in the order.
     */
    ForceNewLine?: boolean;
    /**
     * Optionally override the default description that is generated for the new OrderLine. If specified, this OrderLine will not be stacked on any existing OrderLines in the Order of the same Product (unless they use the same Description). Instead, a new OrderLine will be created.
     */
    Description?: string;
    /**
     * Specify this field to add the new line as a child of the specified parent
     * Entity type: OrderLine
     */
    ParentID?: number;
    SerialNumber?: string;
  }

  export namespace AddProductToOrder {
    export interface ChildOrderLine {
      /**
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Optionally override the default description that is generated for the new OrderLine.
       */
      Description: string;
      /**
       * Min: 9000
       */
      QuantityOrdered?: number;
      SerialNumber?: string;
    }

  }

  export interface AddProductToOrderResponse {
    /**
     * Created or updated OrderLineID
     * Entity type: OrderLine
     * @deprecated Use Result.OrderLineID
     */
    OrderLineID: number;
    /**
     * @deprecated Use Result.Children
     */
    Children: AddProductToOrderResponse.ChildOrderLine[];
    Result: AddProductToOrderResponse.AddProductToOrderResult[];
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace AddProductToOrderResponse {
    export interface ChildOrderLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      /**
       * Entity type: Product
       */
      ProductID?: number;
    }

    /**
     * Created or updated OrderLineIDs
     */
    export interface AddProductToOrderResult {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      Children: AddProductToOrderResponse.ChildOrderLine[];
    }

  }

  /**
   * Adds a service product to an order.
   */
  export interface AddServiceProductToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Description?: string;
    UnitPriceInTax?: number;
    UnitCost?: number;
  }

  export interface AddServiceProductToOrderResponse {
    /**
     * Created or updated `OrderLineID`
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Attach addresses to specified order(s), requiring either `OrderID` or `OrderIDs`.
   */
  export interface AttachAddressToOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Order
     */
    OrderIDs?: number[];
    /**
     * Entity type: Address
     */
    BillingAddressID?: number;
    /**
     * Entity type: Address
     */
    ShippingAddressID?: number;
    /**
     * Entity type: AddressBook
     */
    BillingAddressBookID?: number;
    /**
     * Entity type: AddressBook
     */
    ShippingAddressBookID?: number;
  }

  export interface AttachAddressToOrderResponse {
    CreditcardDisabled: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Attaches a consumer company to an order. Optionally, provide a user ID to associate with it.
   */
  export interface AttachConsumerCompanyToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Company
     */
    CompanyID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  /**
   * Attach a Customer to an Order. When no UserID is provided the current user will be attached. If the selected User has addresses, these will also be assigned to the Order.
   */
  export interface AttachCustomerToOrder {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Attachment Metadata to be added in the Order Ledger's Description.
     */
    AttachmentMetadata?: string;
  }

  /**
   * Detach a Customer from an Order. The addresses on the Order will also be cleared.
   */
  export interface DetachCustomerFromOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Attempts to autocomplete the order by:
   * 
   * - Cancelling the lines that can still be cancelled
   * - Invoicing the lines that've been (partially) shipped
   * - Optionally creating a payment to bring the open amount to 0
   * 
   * :bikini: __The actions this service performs are irreverisble__
   * 
   * *With great power comes great responsibility*
   * -Uncle Ben
   */
  export interface AutocompleteOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface AutocompleteOrderResponse {
    IsCompleted: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Cancel a complete order or a subset of lines.
   * 
   * The order can be identified by an OrderID or by BackendID + BackendSystemID, the lines can be identified by an OrderLineID or by BackendID.
   * 
   * The result will tell you if it was successful.
   */
  export interface CancelOrder {
    /**
     * Reference to the order that is being cancelled - required when `OrderID` is omitted
     */
    BackendID?: string;
    /**
     * Reference to the system from which the order originated - required when `OrderID` is omitted
     */
    BackendSystemID?: string;
    /**
     * ID of the order that is being cancelled - required when `BackendID` and `BackendSystemID` are omitted
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Lines to cancel - or, if none are specified, all open lines of the order will be cancelled
     */
    OrderLines?: CancelOrder.LineToCancel[];
    /**
     * Forcing cancellations is not recommended! Requires `ForceCancelOrder` functionality.
     * 
     *       Only use this when a normal cancellation does not work, because the exported party (warehouse) does not accept it, and they have agreed that
     *       they too will manually cancel it in their systems. This will skip all external validation and cause discrepancies in this order status and stock levels.
     */
    Force?: boolean;
  }

  export namespace CancelOrder {
    export interface LineToCancel {
      /**
       * Reference to the orderline that is being cancelled - required when `OrderLineID` is omitted
       */
      BackendID?: string;
      /**
       * ID of the orderline that is being cancelled - required when `BackendID` is omitted
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Quantity to be cancelled. If left blank the full order line will be cancelled
       */
      Quantity?: number;
    }

  }

  export interface CancelOrderResponse {
    Result: OrderCancellationOptions;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum OrderCancellationOptions {
    CanBeCancelled = 0,
    AlreadyCancelled = 1,
    ShouldBeRequested = 2,
    MustBeRequested = 3,
    CannotBeCancelled = 4
  }

  /**
   * Update some OrderLines to a state so that they can be carried out in a store
   * 
   * If the products should be ordered from a supplier first, the `OrderTheLinesFromASupplier` property should be true.
   * 
   * When no OrderLineIDs are provided all eligable lines in the Order are updated
   */
  export interface ChangeOrderLinesToCarryOut {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
    /**
     * Set to true to indicate that you don't have stock available for carry-out and that you want to order the products from a supplier when the order is placed.
     */
    MustBeOrdered?: boolean;
  }

  /**
   * Update some OrderLines to a state so that they can be delivered to the ShippingAddress on the order
   * 
   * When no OrderLineIDs are provided all eligable lines in the Order are updated
   */
  export interface ChangeOrderLinesToDelivery {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
  }

  /**
   * Update some OrderLines to a state so that they can be pickup up in a store by the customer attached to the order
   * 
   * If no PickupOrganizationUnit was choses yet, it should be provided in the `PickupOrganizationUnitID` property.
   * 
   * When no OrderLineIDs are provided all eligable lines in the Order are updated
   */
  export interface ChangeOrderLinesToPickup {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
    /**
     * Will keep the current value when not provided.
     * Entity type: OrganizationUnit
     */
    PickupOrganizationUnitID?: number;
  }

  /**
   * Check if order BackendID exists
   */
  export interface CheckOrderBackendIDExists {
    BackendSystemID?: string;
    BackendID?: string;
  }

  export interface BackendIDExistsResponse {
    Exists: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Check if shipment BackendID exists
   */
  export interface CheckShipmentBackendIDExists {
    BackendSystemID?: string;
    BackendID?: string;
  }

  /**
   * Check if the order has fraudulent shipping or billing addresses
   */
  export interface CheckOrderAddressFraud {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface CheckOrderAddressFraudResponse {
    HasFraudulentAddress: boolean;
    HasFraudulentShippingAddress: boolean;
    HasFraudulentBillingAddress: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create a correction on an already invoiced line.
   * 
   * This can be used to refund a partial amount on a completed OrderLine, in cases where the product should still be returnable.
   * 
   * Specify either `OrderLineID` and `CorrectionAmount`, or `Lines`.
   * 
   * - If the product is not returnable anymore, a return without retake should be created
   * - This will result in a new line that will be invoiced immediately and shipped by default (AutoProcess.Ship is the default variant).
   * - If the shipping isn't desired, use AutoProcess.None;
   * - If a refund is desired, use AutoProcess.ShipAndRefund.
   */
  export interface CreateInvoicedOrderLineCorrection {
    Lines?: CreateInvoicedOrderLineCorrection.CreateInvoicedOrderLineCorrectionLine[];
    /**
     * @default Ship
     */
    AutoProcess?: AutoProcess;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Min: 0
     */
    CorrectionAmount?: number;
    /**
     * Entity type: UnitPriceCorrectionReason
     */
    CorrectionReasonID?: number;
  }

  export namespace CreateInvoicedOrderLineCorrection {
    export interface CreateInvoicedOrderLineCorrectionLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      /**
       * Min: 0
       */
      CorrectionAmount: number;
      /**
       * Entity type: UnitPriceCorrectionReason
       */
      CorrectionReasonID?: number;
    }

  }

  export const enum AutoProcess {
    Ship = 0,
    ShipAndRefund = 1,
    None = 2
  }

  /**
   * Converts an order to an offer
   */
  export interface CreateOfferFromOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    OfferExpirationDate?: string;
  }

  /**
   * Creates and sets an app token on specified order, to allow accessing it from an anonymous context. The order may not have a Customer attached.
   */
  export interface CreateOrderAppToken {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface CreateOrderAppTokenResponse {
    Token: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create an Order from an Offer specified by `OrderID`.
   */
  export interface CreateOrderFromOffer {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface CreateOrderFromOfferResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create a new order ledger of the given type
   */
  export interface CreateOrderLedger {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    OrderLedgerType?: string;
    Value?: string;
    Description?: string;
    CreationTime?: string;
  }

  /**
   * Create a price correction on the order line, specified by `OrderID` and `OrderLineID`. Provide the new `UnitPrice` this order line should have, and include
   * a property `IncludingTax` to indicate if you mean IN or EX taxes.
   */
  export interface CreateOrderLineUnitPriceCorrection {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    UnitPrice: number;
    IncludingTax?: boolean;
    /**
     * Entity type: UnitPriceCorrectionReason
     */
    CorrectionReasonID?: number;
  }

  /**
   * Reset the unitprice on an OrderLine with a price correction.
   */
  export interface RemoveOrderLineUnitPriceCorrection {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
  }

  /**
   * Create a new Order of the given Type
   */
  export interface CreateOrder {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Type: OrderTypes;
    ClientOrderID?: string;
  }

  export interface CreateOrderResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create a price correction on the purchase order line.
   */
  export interface CreatePurchaseOrderLineUnitPriceCorrection {
    /**
     * Entity type: OrderLine
     */
    PurchaseOrderLineID?: number;
    /**
     * Min: 0
     */
    UnitPrice: number;
    /**
     * Entity type: OrganizationUnit
     */
    SoldFromOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    SoldToOrganizationUnitID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    OldPrice?: number;
    QuantityOrdered?: number;
  }

  export interface CreatePurchaseOrderLineUnitPriceCorrectionResponse {
    /**
     * Entity type: OrderLine
     */
    CorrectionPurchaseOrderLineID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get additional order information intended for delivery data.
   */
  export interface GetDeliveryOrderData {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetDeliveryOrderDataResponse {
    DeliveryOrderData: DeliveryOrderData;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface DeliveryOrderData {
    DefaultShippingMethodID?: number;
    ShippingType: string;
    ShippingPrice: number;
    DeliveryService: string;
    PickUpPointID: string;
    PickUpPointName: string;
    AdditionalPickUpPointInformation: string;
    ExpectedDeliveryDate?: string;
    DeliveryTimeFrom?: string;
    DeliveryTimeTo?: string;
  }

  /**
   * Set additional order information intended for delivery data.
   */
  export interface SetDeliveryOrderData {
    /**
     * Entity type: Order
     */
    OrderID: number;
    DeliveryOrderData: DeliveryOrderData;
  }

  /**
   * Detaches the consumer company from an order.
   */
  export interface DetachConsumerCompanyFromOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Detach the given Order from the attached customer and a session. A ReminderToken will be returned to be able to use the Order later with the `RecoverShoppingCartService`.
   * 
   * This service can be used to transfer an Order to a different User:
   * - DetachOrder
   * - Logout
   * - Login
   * - RecoverShoppingCart
   */
  export interface DetachOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface DetachOrderResponse {
    ReminderToken: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Indicate that this was the current order, but has been dismissed.
   * 
   * This is purely a ledger and does not affect the order in any other way.
   */
  export interface DismissOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Make a new `Order` based on an existing one.
   * 
   * * Optionally you can provide some OrderLineIDs to duplicate only a subset of the order.
   * * When an `OrganizationUnitID` is set, the Order will be created Originating and SoldFrom the specified OrganizationUnit
   * * To include the orignal GiftWrapping the flag `IncludeGiftWrapping` should be set to true. When combining this with a subset of lines, this can generate an error when some wrapped lines are not duplicated.
   * * To ignore shipping costs, set `IgnoreShippingCosts` to true (defaults to false)
   */
  export interface DuplicateOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
    /**
     * @default false
     */
    IncludeGiftWrapping?: boolean;
    /**
     * @default false
     */
    IncludeBundles?: boolean;
    /**
     * @default false
     */
    IgnoreShippingCosts?: boolean;
    DuplicationReason?: string;
    LineActionType?: LineActionTypes;
  }

  export interface DuplicateOrderResponse {
    /**
     * Entity type: Order
     */
    DuplicatedOrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get the fiscal properties of an Order.
   */
  export interface GetFiscalOrderData {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetFiscalOrderDataResponse {
    FiscalID: string;
    VatNumber: string;
    FiscalRemark?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update the fiscal properties on an Order.
   */
  export interface SetFiscalOrderData {
    /**
     * Entity type: Order
     */
    OrderID: number;
    FiscalID?: string;
    VatNumber?: string;
    FiscalRemark?: string;
  }

  export interface ForcePlaceOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ForcePlaceOrderResponse {
    ValidationResult: OrderExportValidationResult;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrderExportValidationResult {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Result: OrderExportValidationResults;
    Messages: OrderExportValidationResultMessage[];
  }

  export const enum OrderExportValidationResults {
    Unknown = 0,
    Valid = 1,
    Invalid = 2,
    NothingToBeExported = 3
  }

  export interface OrderExportValidationResultMessage {
    Code: string;
    Message: string;
  }

  export interface GetConfigurableOrderView {
    /**
     * Entity type: Order
     */
    OrderID: number;
    LogicalLevel: string;
    GroupByPropertyType: string;
    IncludeAllChildren?: boolean;
    ProductProperties?: string[];
  }

  export interface GetConfigurableOrderViewResponse {
    PropertyGroups: PropertyGroup[];
    OtherLines: Record<'NormalProduct' | 'Discount' | 'ExtraCosts' | 'ShippingCosts' | 'ReturnCosts' | 'PriceCorrection' | 'Service' | 'GiftWrappingCosts' | 'DownPayment', SimpleOrderLine[]>;
    TotalAmount: number;
    TotalAmountInTax: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface PropertyGroup {
    PropertyValue: any;
    TotalAmount: number;
    TotalAmountInTax: number;
    Quantity: number;
    ChildGroups: ConfigurableGroup[];
  }

  export interface ConfigurableGroup {
    ProductID: number;
    Properties: any;
    LogicalLevel: string;
    ConfigurableProperty: string;
    Value: any;
    Values: any[];
    TotalAmount: number;
    TotalAmountInTax: number;
    Quantity: number;
    Children: ConfigurableGroup[];
    OrderLines: ConfigurableOrderLine[];
  }

  export interface ConfigurableOrderLine {
    OrderLineID: number;
    Quantity: number;
    TotalAmount: number;
    TotalAmountInTax: number;
  }

  export interface SimpleOrderLine {
    OrderLineID: number;
    ProductID?: number;
    Properties: any;
    Description: string;
    Quantity: number;
    TotalAmount: number;
    TotalAmountInTax: number;
  }

  /**
   * Returns a list of custom status changes for a specific order
   */
  export interface GetOrderCustomStatusLedger {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetOrderCustomStatusLedgerResponse {
    Results: GetOrderCustomStatusLedgerResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOrderCustomStatusLedgerResponse {
    export interface Model {
      Description: string;
      BackendID: string;
      Name: string;
      CreationTime: string;
      Sequence: number;
      Data: { [key: string]: any };
    }

  }

  /**
   * Return the current order links for the given order
   */
  export interface GetOrderLinks {
    /**
     * Entity type: Order
     */
    ID: number;
  }

  export interface GetOrderLinksResponse {
    PrimaryRelations: SingleOrderLink[];
    RelatedRelations: SingleOrderLink[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface SingleOrderLink {
    OrderLink: OrderDataForSingleOrderLink;
    Type: AssociationTypeForSingleOrderLink;
  }

  export interface OrderDataForSingleOrderLink {
    /**
     * Entity type: Order
     */
    ID: number;
    /**
     * Entity type: OrderLink
     */
    OrderLinkID: number;
  }

  export interface AssociationTypeForSingleOrderLink {
    /**
     * Entity type: OrderLinkType
     */
    ID: number;
    Name: string;
  }

  /**
   * Get an Order by ID.
   * 
   * This service is readonly, it will not update the order just by calling it like the `GetShoppingCart` service does.
   * 
   * To include the result of `GetRequiredDataForOrder`, the property `AdditionalOrderDataOptions.RequiredFor` can be set to the desired level of requirements.
   * To include the result of `GetProductRequirementValuesForOrder`, the property `AdditionalOrderDataOptions.IncludeProductRequirements` can be set to `true`.
   * To include the result of `GetGiftWrappingOptionsForOrder` and `GetAvailableGiftWrappingLinesForOrder`, the property `AdditionalOrderDataOptions.IncludeGiftWrapping` can be set to `true`.
   * To include the result of `PrefigureDiscounts` and `GetPotentialDiscountsForOrder`, the property `AdditionalOrderDataOptions.IncludePrefigureDiscounts` can be set to `true`.
   * To include the result of `GetPickProductDiscountOptionsForOrder`, the property `AdditionalOrderDataOptions.IncludePickProductOptions` can be set to `true`.
   * To include the result of `ListAvailableShippingMethods`, the property `AdditionalOrderDataOptions.IncludeAvailableShippingMethods` can be set to `true`.
   * To include the result of `GetAvailablePaymentMethods`, the property `AdditionalOrderDataOptions.IncludeAvailablePaymentMethods` can be set to `true`.
   * To include the result of `GetAvailableRefundPaymentMethodsForOrder`, the property `AdditionalOrderDataOptions.IncludeAvailableRefundPaymentMethods` can be set to `true`.
   * To include the result of `ValidateOrderShipment`, the property `AdditionalOrderDataOptions.IncludeValidateShipment` can be set to `true`.
   * To include the available actions for the PaymentTransactions, the property `AdditionalOrderDataOptions.IncludePaymentTransactionActions` can be set to `true`.
   */
  export interface GetOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    IncludeAllPayments?: boolean;
    ShowOnlyShippableLines?: boolean;
    AdditionalOrderDataOptions?: AdditionalOrderDataOptions;
  }

  export interface AdditionalOrderDataOptions {
    ProductProperties: string[];
    RequiredFor?: RequiredFor;
    IncludeProductRequirements: boolean;
    IncludeGiftWrapping: boolean;
    IncludeGiftCardBusinessRules: boolean;
    IncludePrefigureDiscounts: boolean;
    IncludePickProductOptions: boolean;
    IncludeAvailableShippingMethods: boolean;
    IncludeAvailablePaymentMethods: boolean;
    IncludeAvailableRefundPaymentMethods: boolean;
    IncludeValidateShipment: boolean;
    IncludePaymentTransactionActions: boolean;
    IncludeCheckoutOptions: boolean;
    IncludeOrganizationUnitData: boolean;
    IncludeCustomerCustomFields: boolean;
  }

  export const enum RequiredFor {
    PlaceOrder = 1,
    Payment = 2,
    Ship = 4,
    Invoice = 8,
    All = 15
  }

  export interface GetOrderResponse {
    Result: OrderDto;
    Amounts: OpenOrderAmounts;
    HasMoreLines: boolean;
    AdditionalOrderData?: AdditionalOrderData;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrderDto {
    GlobalID: string;
    /**
     * Entity type: Order
     */
    ID: number;
    BackendID: string;
    BackendSystemID: string;
    Type: OrderTypes;
    Definition: OrderDefinitionModel;
    OrderCustomStatus?: OrderCustomStatusModel;
    OrderCustomType?: OrderCustomTypeModel;
    /**
     * Entity type: User
     */
    CustomerID?: number;
    Customer: UserDto;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    /**
     * Only populated in case the CreatedBy user in an employee
     */
    CreatedByFullName: string;
    CreationTime: string;
    Payments: PaymentTransactionDto[];
    /**
     * Entity type: Address
     */
    ShippingAddressID?: number;
    /**
     * Entity type: Address
     */
    BillingAddressID?: number;
    /**
     * Entity type: Address
     */
    PickupAddressID?: number;
    ShippingAddress: AddressDto;
    BillingAddress: AddressDto;
    PickupAddress: AddressDto;
    Lines: OrderLineDto[];
    OfferExpirationDate?: string;
    IsOffer: boolean;
    IsExpiredOffer?: boolean;
    IsLocked: boolean;
    OriginalTotalAmount?: number;
    OriginalTotalAmountInTax?: number;
    TotalAmount: number;
    TotalTaxAmount: number;
    TotalAmountInTax: number;
    TotalAmountInvoiced: number;
    TotalEcoTaxInTax?: number;
    TaxTable: TaxAmount[];
    IsCompleted: boolean;
    IsDelivered: boolean;
    IsShipped: boolean;
    IsPaid: boolean;
    IsCaptured: boolean;
    IsInvoiced: boolean;
    IsPlaced: boolean;
    CompletionDate?: string;
    CaptureDate?: string;
    ShipmentDate?: string;
    DeliveryDate?: string;
    InvoiceDate?: string;
    PaymentDate?: string;
    Status: OrderStatus;
    /**
     * Entity type: OrderHoldStatus
     */
    HoldStatusID?: number;
    Remark: string;
    CustomerOrderID: string;
    CustomerReference: string;
    TotalItems: number;
    HasDelivery: boolean;
    IdentificationID?: number;
    Properties: OrderProperties;
    Characteristics: OrderProperties;
    /**
     * Entity type: OrganizationUnit
     */
    OriginatingOrganizationUnitID: number;
    /**
     * Requested by setting AdditionalOrderDataOptions.IncludeOrganizationUnitData
     */
    OriginatingOrganizationUnitData?: OrganizationUnitData;
    /**
     * Entity type: OrganizationUnit
     */
    PickupOrganizationUnitID: number;
    /**
     * Requested by setting AdditionalOrderDataOptions.IncludeOrganizationUnitData
     */
    PickupOrganizationUnitData?: OrganizationUnitData;
    /**
     * Entity type: OrganizationUnit
     */
    SoldFromOrganizationUnitID: number;
    /**
     * Requested by setting AdditionalOrderDataOptions.IncludeOrganizationUnitData
     */
    SoldFromOrganizationUnitData?: OrganizationUnitData;
    /**
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnitID?: number;
    /**
     * Requested by setting AdditionalOrderDataOptions.IncludeOrganizationUnitData
     */
    ShipFromOrganizationUnitData?: OrganizationUnitData;
    /**
     * Entity type: OrganizationUnit
     */
    SoldToOrganizationUnitID?: number;
    /**
     * Requested by setting AdditionalOrderDataOptions.IncludeOrganizationUnitData
     */
    SoldToOrganizationUnitData?: OrganizationUnitData;
    /**
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID?: number;
    /**
     * Requested by setting AdditionalOrderDataOptions.IncludeOrganizationUnitData
     */
    ShipToOrganizationUnitData?: OrganizationUnitData;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupIDs: number[];
    CurrencyID: string;
    CustomStatusID?: number;
    PreferredPriceDisplayMode: OrderPreferredPriceDisplayMode;
    IsTaxExempt: boolean;
    FiscalID: string;
    FiscalRemark?: string;
    VatNumber: string;
    UserAgentApplication: string;
    UserAgentVersion: string;
    AllowPartialFulfillment: boolean;
    PlacementDate?: string;
    IgnoreDiscounts: boolean;
    IgnoreShippingCosts: boolean;
    IgnoreGiftWrappingCosts: boolean;
    /**
     * Entity type: Discount
     */
    DisabledDiscounts: number[];
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    IsConfirmed: boolean;
    Barcode: string;
    PriceLists?: PriceListModel[];
    BackfillInformation?: OrderBackfillInformation;
    IsDownPayment: boolean;
    TransferReference?: string;
    TransferredOrderID?: number;
    AutomaticCancellationDate?: string;
    /**
     * Entity type: CustomOrderType
     */
    CustomOrderTypeID?: number;
    SelectedFulfillmentOptions?: SelectedOrderFulfillmentOptions[];
    ClientOrderID?: string;
    TotalLineCount: number;
    TaxFreeInformation?: TaxFreeInformation;
    TotalLoyaltyProgramWithdrawals: OrderLoyaltyMutation[];
    TotalLoyaltyProgramDeposits: OrderLoyaltyMutation[];
    CustomerIdentifiers?: OrderCustomerIdentifiers;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    LoyaltyProgramValidated?: boolean;
    LoyaltyProgramName: string;
    LoyaltyProgramDescription: string;
    /**
     * The repair this order is associated with.
     */
    Repair?: RepairModel;
    /**
     * Information related to local mode, in case it was imported from LiveGuard.
     */
    LocalInformation?: LocalInformation;
    DisplayID: string;
    HasPresaleLines: boolean;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    Company?: CompanyDto;
    DisplayProperties: OrderDisplayProperties;
    /**
     * @deprecated Use `SearchOrders` with `IsClickAndCollect` filter
     */
    IsClickAndCollect: boolean;
  }

  export interface OrderDefinitionModel {
    Status: OrderDefinitionModel.Value;
    Type: OrderDefinitionModel.Value;
  }

  export namespace OrderDefinitionModel {
    export interface Value {
      BackendID: string;
      Title: string;
    }

  }

  export interface OrderCustomStatusModel {
    /**
     * Entity type: OrderCustomStatus
     */
    ID: number;
    BackendID: string;
    Name: string;
    Description: string;
  }

  export interface OrderCustomTypeModel {
    /**
     * Entity type: OrderCustomType
     */
    ID: number;
    BackendID: string;
    Name: string;
    Description: string;
  }

  export interface UserDto {
    /**
     * Entity type: User
     */
    ID: number;
    BackendID: string;
    CustomID: string;
    EmployeeNumber: string;
    BackendRelationID: string;
    EmailAddress: string;
    EmailAddressValidated: boolean;
    Initials: string;
    FirstName: string;
    LastName: string;
    FullName: string;
    DateOfBirth?: string;
    PlaceOfBirth: string;
    Gender: string;
    Type: UserTypes;
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    IsDeleted: boolean;
    AccountType: UserAccountType;
    SingleSignOnOnly: boolean;
    AllowPasswordLogin: boolean;
    Nickname: string;
    Company: CompanyDto;
    BankAccount: string;
    IsDebtor: boolean;
    PhoneNumber: string;
    GravatarHash: string;
    Data: { [key: string]: string };
    TimeZone: string;
    FiscalID: string;
    SocialSecurityNumber: string;
    Title: string;
    Salutation: string;
    CreationTime: string;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreatedByFullName: string;
    /**
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions: { [key: number]: CustomFieldValueWithOptions };
    Barcode: string;
    LastSuccessfulLogin?: string;
  }

  export interface TaxAmount {
    Rate: number;
    /**
     * Amount of tax for this rate
     */
    Amount: number;
    /**
     * TotalAmount that has this rate
     */
    Base: number;
    /**
     * TotalAmount including the tax (Base + Amount)
     */
    Total: number;
  }

  export const enum OrderStatus {
    Cart = 0,
    Order = 1
  }

  export const enum OrderProperties {
    None = 0,
    IsPickup = 1,
    CreatedByEmployee = 2,
    IsInterbranch = 4,
    IsB2B = 8,
    Duplicated = 16,
    /**
     * Indicates that the order was auto completed with force by a user.
     */
    Autocompleted = 32,
    /**
     * ReturnToSupplier indicates that the order will be returned from the store back to the supplier (RMA/CMA).
     */
    ReturnToSupplier = 128,
    /**
     * CustomTaxesApplied indicates that the order has custom taxes applied and other processes should not touch the taxes anymore.
     */
    CustomTaxesApplied = 256,
    /**
     * Replenishment indicates that this order is used to replenish a store/warehouse
     */
    Replenishment = 512,
    /**
     * Indicates that the SoldBy, which is normally determined automatically, has been manually overriden on the Order level.
     */
    SoldByOverridden = 1024,
    /**
     * Indicates that this order was created in localmode on liveguard and pushed to the cloud.
     */
    ImportedFromLiveGuard = 2048,
    IsTransferOrder = 4096,
    IsReturnsTransferOrder = 8192,
    IsOffer = 16384,
    IsUnified = 32768,
    /**
     * Indicates that this order has been returned, partially or completely, by another order.
     */
    IsReturned = 65536,
    /**
     * Indicates the order contains lines that have negative quantities to ship, aka, are returned lines.
     */
    HasReturnLines = 131072,
    DownPayment = 262144,
    /**
     * Indicates that the order is a restitution order
     */
    Restitution = 524288,
    /**
     * Indicates that the order is a repair order and contains products used during a repair
     */
    Repairs = 1048576,
    /**
     * Indicates that the order is the result of a return reversal.
     */
    IsReturnReversal = 2097152
  }

  export interface OrganizationUnitData {
    Name: string;
    BackendID?: string;
  }

  export const enum OrderPreferredPriceDisplayMode {
    InTax = 0,
    ExTax = 1
  }

  export const enum DiscountInvalidReasons {
    None = 0,
    /**
     * The coupon does not exist
     */
    NotExist = 1,
    /**
     * The coupon is not valid (yet)
     */
    NotValidYet = 2,
    /**
     * The coupon is expired
     */
    Expired = 3,
    /**
     * The coupon has already been used
     */
    HasAlreadyBeenUsed = 4,
    /**
     * The coupon code has already been added
     */
    CouponCodeAlreadyAdded = 5,
    /**
     * The manual discount does not meet all conditions
     */
    ManualValueIsInvalid = 6,
    /**
     * The discount does not apply to this order type
     */
    InvalidOrderType = 7,
    Invalid = 8
  }

  export interface PriceListModel {
    /**
     * Entity type: PriceList
     */
    ID: number;
    Name: string;
    BackendID: string;
  }

  export interface OrderBackfillInformation {
    /**
     * Original date when this order was created
     */
    OriginalCreationDate?: string;
    /**
     * Original date when this order was invoiced
     */
    OriginalInvoiceDate?: string;
    /**
     * Original invoicenumber of this order
     */
    OriginalInvoiceNumber?: string;
  }

  export interface SelectedOrderFulfillmentOptions {
    SameDayDelivery: SelectedOrderFulfillmentOptions.SameDayDeliveryOptions;
    OrderLineIDs: number[];
  }

  export namespace SelectedOrderFulfillmentOptions {
    export interface SameDayDeliveryOptions {
      UseSameDayDelivery: boolean;
    }

  }

  export interface TaxFreeInformation {
    Handler?: string;
    IsIssued?: boolean;
    IsVoided?: boolean;
    FormIdentifier?: string;
    UserIdentifier?: string;
    CanBePrinted?: boolean;
    DeviceTypeForPrinting?: FlagsEnumDto;
  }

  export interface FlagsEnumDto {
    /**
     * Entity type: Enum
     */
    ID: number;
    /**
     * Usually an internal & unchangable code - for display, use `Description`
     */
    Name: string;
    IDs: number[];
    Names: string[];
  }

  export interface OrderLoyaltyMutation {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID: number;
    BackendID: string;
    Name: string;
    Points: number;
  }

  export interface OrderCustomerIdentifiers {
    UserID?: string;
    UserSubscriptionID?: string;
    UserBackendID?: string;
    UserCustomID?: string;
    EmailAddress?: string;
  }

  export interface RepairModel {
    /**
     * Entity type: Repair
     */
    ID: number;
  }

  export interface LocalInformation {
    LocalInvoices: LocalInformation.LocalInvoice[];
  }

  export namespace LocalInformation {
    export interface LocalInvoice {
      InvoiceNumber: string;
      InvoiceDate?: string;
      PreviousSignature: string;
      Signature: string;
    }

  }

  export interface OrderDisplayProperties {
    Subtotal: number;
  }

  export interface OpenOrderAmounts {
    Open: OpenOrderAmount;
    OpenAmountIsZero: boolean;
    CurrencyID: string;
    Total: OrderAmount;
    UnroundedTotal: OrderAmount;
    Paid: PaidOrderAmount;
    Taxes: TaxAmount[];
    Types: Record<'NormalProduct' | 'Discount' | 'ExtraCosts' | 'ShippingCosts' | 'ReturnCosts' | 'PriceCorrection' | 'Service' | 'GiftWrappingCosts' | 'DownPayment', OrderAmount>;
    Discounts: DiscountAmounts;
    SubtotalExDiscount: number;
  }

  export interface OpenOrderAmount {
    InTax: number;
    PendingInTax: number;
    Capturable: number;
  }

  export interface OrderAmount {
    Amount: number;
    InTax: number;
    Tax: number;
  }

  export interface PaidOrderAmount {
    Pending: number;
    Amount: number;
    Captured: number;
  }

  export interface DiscountAmounts {
    TotalDiscountAmount: number;
    PerDiscount: DiscountAmounts.GroupedDiscountAmount[];
  }

  export namespace DiscountAmounts {
    export interface GroupedDiscountAmount {
      /**
       * Entity type: Discount
       */
      DiscountID: number;
      Discount: DiscountAmounts.DiscountAmountDiscount;
      Total: number;
    }

    export interface DiscountAmountDiscount {
      Description: string;
    }

  }

  export interface AdditionalOrderData {
    RequiredData?: RequiredData;
    ProductRequirements?: ProductRequirementModel[];
    GiftCardBusinessRules?: GiftCardBusinessRulesDto[];
    GiftWrappingLineAvailability?: GiftWrappingLineAvailabilityDto[];
    GiftWrappingOptions?: GiftWrappingOptionsForOrder;
    PrefiguredDiscounts?: PrefigureDiscountResult[];
    PotentialDiscounts?: PotentialDiscountDto[];
    /**
     * Entity type: OrderLine
     */
    DiscountPickProductOptions?: { [key: number]: OrderPickProductDiscountOptions };
    HasMoreDiscountPickProductOptions?: boolean;
    AvailableShippingMethods?: ShippingMethodByOrderLineDto[];
    AvailablePaymentMethods?: AvailablePaymentMethod[];
    AvailableRefundPaymentMethods?: RefundPaymentMethod[];
    OrderAmountAvailableForRefund?: number;
    /**
     * Amount required to pay when the whole order would be shipped.
     */
    OpenAmountForShipment?: number;
    /**
     * Is the amount required to pay when the whole order would be shipped valid.
     */
    IsPaidAmountValidForShipment?: boolean;
    /**
     * Amount required to pay when the carry out lines of the order would be shipped.
     */
    OpenAmountForShipmentOfCarryOut?: number;
    /**
     * Is the amount required to pay when the carry out lines of the order would be shipped valid.
     */
    IsPaidAmountValidForShipmentOfCarryOut?: boolean;
    IsAutoRefundAvailable?: boolean;
    AutoRefundableAmount?: number;
    IsAutoShipAvailable?: boolean;
    LineDiscountProductOptions?: LineDiscountProductOption[];
    HasMoreLineDiscountProductOptions?: boolean;
    AvailableCheckoutOptions?: CheckoutOrderOption[];
    /**
     * Entity type: CustomField
     */
    CustomerCustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
  }

  export interface RequiredData {
    Order: Requirement[];
    /**
     * Entity type: OrderLine
     */
    OrderLines: { [key: number]: Requirement[] };
    Customer: Requirement[];
    Company: Requirement[];
  }

  export interface Requirement {
    Name: string;
    IsValid: boolean;
    Message: string;
    Type: string;
    ValidationRule: string;
    DataType: string;
    RequiredFor: RequiredFor;
    Data: any;
  }

  export interface ProductRequirementModel {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductPrimitiveName: string;
    RequirementModels: ProductRequirementLineModel[];
  }

  export interface ProductRequirementLineModel {
    /**
     * Entity type: ProductRequirement
     */
    ID: number;
    Name: string;
    Value: any;
  }

  export interface GiftCardBusinessRulesDto {
    /**
     * The order line where the giftcard is stored at.
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * The date the giftcard was purchased.
     */
    DateOfPurchase?: string;
    /**
     * The date the giftcard expired / will expire.
     */
    DateOfExpiration?: string;
    /**
     * The expiration status of the giftcard.
     */
    IsExpired?: boolean;
  }

  export interface GiftWrappingLineAvailabilityDto {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    QuantityWrapped: number;
    QuantityAvailable: number;
    TotalQuantityWrappable: number;
  }

  export interface GiftWrappingOptionsForOrder {
    WrapOrder: boolean;
    WrapIndividually: boolean;
    GiftWraps: GiftWrapDto[];
    GreetingCardProductID?: number;
    Message: string;
  }

  export interface GiftWrapDto {
    /**
     * Entity type: Product
     */
    GreetingCardProductID?: number;
    Message?: string;
    Lines: WrapLine[];
  }

  export interface WrapLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    Quantity: number;
  }

  export interface PotentialDiscountDto {
    /**
     * Entity type: Discount
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    BackendID?: string;
    Description?: string;
    MarketingDescription?: string;
    PromotionLabel?: string;
    ImageBlobID?: string;
    AmountMissing?: number;
    /**
     * Entity type: Product
     */
    SecondaryProductIDs?: number[];
    ActionProperties?: any;
    IsAlreadyApplied: boolean;
  }

  export interface OrderPickProductDiscountOptions {
    BaseAmount: number;
    /**
     * Entity type: Product
     */
    Options: number[];
    Tiers: OrderPickProductDiscountOptions.PickProductTierOptions[];
    /**
     * Entity type: Product
     */
    CurrentSelection?: number;
    CurrentTier?: string;
  }

  export namespace OrderPickProductDiscountOptions {
    export interface PickProductTierOptions {
      ID: string;
      From: number;
      Difference: number;
      /**
       * Entity type: Product
       */
      DefaultProductID?: number;
      Prompt: boolean;
      /**
       * Entity type: Product
       */
      Products: any[];
      Description: string;
      IsCurrentTier: boolean;
    }

  }

  export interface ShippingMethodByOrderLineDto {
    /**
     * Entity type: OrderLine
     */
    AppliesToOrderLineIDs: number[];
    RequestedDeliveryDates: RequestedDateByOrderLineID[];
    Costs: ShippingMethodByOrderLineDto.ShippingMethodCosts;
    /**
     * Entity type: ShippingMethod
     */
    ID: number;
    /**
     * Entity type: Carrier
     */
    CarrierID: number;
    Carrier: CarrierDto;
    Name: string;
    Description: string;
    Code: string;
    Priority: number;
    PaymentHandledByCarrier: boolean;
    DeliveryType: ShippingMethodDeliveryTypes;
    CustomFieldValues: { [key: number]: CustomFieldValue };
  }

  export namespace ShippingMethodByOrderLineDto {
    export interface ShippingMethodCosts {
      Amount: number;
      /**
       * The ranges of the shipping costs based on the order amount. Empty if shipping costs are determined using scripting or no ranges are known for the shipping method.
       * The ranges are non-overlapping, gapless and sorted by the minimum amount (ascending). The first range has a minimum amount of 0. The last range has a maximum amount of null.
       */
      Ranges: ShippingCostRange[];
    }

  }

  export interface RequestedDateByOrderLineID {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    RequestedDate?: string;
  }

  export interface ShippingCostRange {
    MinAmount: number;
    MaxAmount?: number;
    Costs: number;
  }

  export interface LineDiscountProductOption {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: Discount
     */
    DiscountID: number;
    ActionType: string;
    /**
     * Entity type: Product
     */
    CurrentSelection?: number;
    Options: any[];
  }

  export interface CheckoutOrderOptionCategory {
    Options: CheckoutOrderOption[];
    /**
     * Entity type: CheckoutOptionCategory
     */
    ID: number;
    Name: string;
    Description?: string;
  }

  export interface CheckoutOrderOption {
    /**
     * Entity type: CheckoutOption
     */
    ID: number;
    Name: string;
    Description?: string;
    Handler: string;
    ConfigData?: ICheckoutOptionConfigData;
    ValueData?: ICheckoutOptionValueData;
    CheckoutOptionCategory?: CheckoutOrderOptionCategoryInfo;
  }

  export interface ICheckoutOptionConfigData {
  }

  export interface ICheckoutOptionValueData {
  }

  export interface CheckoutOrderOptionCategoryInfo {
    /**
     * Entity type: CheckoutOptionCategory
     */
    ID: number;
    Name: string;
    Description?: string;
  }

  export interface GetOrderStatistics {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetOrderStatisticsResponse {
    Statistics: OrderStatistics;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrderStatistics {
    GrossMargin: number;
    /**
     * Entity type: OrderLine
     */
    OrderLines: { [key: number]: OrderStatistics.OrderLineStatistics };
  }

  export namespace OrderStatistics {
    export interface OrderLineStatistics {
      UnitPrice: number;
      UnitCost: number;
      DiscountAmount: number;
    }

  }

  /**
   * Get a summary of an Order to be used when a shipment will be made.
   * 
   * Caution: the ProductID is nullable, all non-cancelled lines are returned
   */
  export interface GetOrderSummaryForShipping {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetOrderSummaryForShippingResponse {
    Lines: GetOrderSummaryForShippingResponse.OrderLineSummary[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOrderSummaryForShippingResponse {
    export interface OrderLineSummary {
      ID: number;
      ProductID?: number;
      TotalQuantityToShip: number;
      QuantityShipped: number;
      QuantityOpen: number;
    }

  }

  /**
   * Returns a summarized version of the order with less details than `GetOrder`. This service should be used to expose
   * less data to the frontend, also this variant to retrieve the order can be used when authenticated via the `GetTemporaryTokenForOrder`
   * service. Where `GetOrder` is not accessible.
   */
  export interface GetOrderSummary {
    /**
     * Entity type: Order
     */
    OrderID: number;
    ProductProperties?: string[];
  }

  export interface GetOrderSummaryResponse {
    /**
     * Entity type: Order
     */
    ID: number;
    ShippingAddress: GetOrderSummaryResponse.AddressModel;
    BillingAddress: GetOrderSummaryResponse.AddressModel;
    Amounts: OpenOrderAmounts;
    Lines: GetOrderSummaryResponse.OrderLineModel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOrderSummaryResponse {
    export interface AddressModel {
      Address1: string;
      Address2: string;
      ZipCode: string;
      City: string;
      CountryID: string;
    }

    export interface OrderLineModel {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      /**
       * Entity type: OrderLine
       */
      ParentID?: number;
      Product: any;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      Description: string;
      QuantityOrdered: number;
      QuantityCancelled: number;
      UnitPriceInTax: number;
      DiscountAmount: number;
      NetTotalAmountInTax: number;
      OriginalTotalAmountInTax?: number;
      Type: OrderLineTypes;
      QuantityExported: number;
      IsPlaced: boolean;
    }

  }

  /**
   * Get tax details on the specified order.
   */
  export interface GetOrderTaxDetails {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetOrderTaxDetailsResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Lines: ReceiptTaxDetailLine[];
    SummaryLines: ReceiptTaxSummaryLine[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ReceiptTaxDetailLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    Name: string;
    Type: string;
    Rate: number;
    Amount: number;
  }

  export interface ReceiptTaxSummaryLine {
    Name: string;
    Type: string;
    Rate: number;
    Amount: number;
  }

  /**
   * Returns the quantity that is to be shipped for order lines. Includes pending shipments.
   */
  export interface GetPendingShipmentInformationForOrderLines {
    Lines: GetPendingShipmentInformationForOrderLines.OrderLine[];
  }

  export namespace GetPendingShipmentInformationForOrderLines {
    export interface OrderLine {
      /**
       * Entity type: OrderLine
       */
      ID: number;
    }

  }

  export interface GetPendingShipmentInformationForOrderLinesResponse {
    Result: GetPendingShipmentInformationForOrderLinesResponse.LineInfo[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetPendingShipmentInformationForOrderLinesResponse {
    export interface LineInfo {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      /**
       * Total quantity of the line that is and needed to be shipped.
       */
      QuantityOpen: number;
      /**
       * This is what has already been shipped for this line
       */
      QuantityShipped: number;
      /**
       * This is what is currently in a shipment, but not shipped yet
       */
      QuantityShippedWithPending: number;
      /**
       * This is what currently still needs to be shipped for this line
       */
      QuantityOpenWithPending: number;
    }

  }

  /**
   * List all the related orderlines for the given OrderLineID.
   * 
   * The orderlines will be sorted by CreationTime.
   */
  export interface GetRelatedOrderLines {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
  }

  export interface GetRelatedOrderLinesResponse {
    Result: GetRelatedOrderLinesResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetRelatedOrderLinesResponse {
    export interface Model {
      OrderType: OrderTypes;
      SoldFromOrganizationUnit: GetRelatedOrderLinesResponse.OrganizationUnit;
      ShipFromOrganizationUnit: GetRelatedOrderLinesResponse.OrganizationUnit;
      /**
       * Entity type: OrderLine
       */
      ID: number;
      BackendReference: string;
      /**
       * Entity type: Order
       */
      OrderID: number;
      /**
       * Entity type: OrderLine
       */
      ParentID?: number;
      Children?: OrderLineDto[];
      Type: OrderLineTypes;
      /**
       * Entity type: StockLabel
       */
      StockLabel: number;
      Description: string;
      AdditionalDescription: string;
      /**
       * This is the quantity that was ordered. Make sure you really need this property, the total of the line is TotalQuantityToShip.
       */
      QuantityOrdered: number;
      QuantityOpen: number;
      QuantityCommitted: number;
      CommitmentStatus: OrderLineCommitmentStatus;
      QuantityShipped: number;
      QuantityDelivered: number;
      QuantityReserved: number;
      QuantityBackordered: number;
      QuantityReturned: number;
      QuantityCancelled: number;
      QuantityInvoiced: number;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      Product?: ProductDto;
      UnitPrice: number;
      UnitPriceBeforeCorrection?: number;
      UnitPriceBeforeCorrectionInTax?: number;
      UnitPriceCorrectionReason?: EnumDto;
      TaxRate: number;
      TaxCode: EnumDto;
      ShowTaxInfo: boolean;
      SerialNumber?: string;
      UnitPriceInTax: number;
      IsCancelled: boolean;
      TotalQuantityToShip: number;
      TotalAmountInTax: number;
      TotalAmount: number;
      NetTotalAmountInTax: number;
      NetTotalAmount: number;
      TotalTaxAmount: number;
      NetTotalTaxAmount: number;
      OriginalTotalAmountInTax?: number;
      OriginalTotalAmount?: number;
      EcoTaxInTax?: number;
      TotalEcoTaxInTax?: number;
      DisplayPriceInTax?: number;
      DisplayPrice?: number;
      ModificationAllowed: boolean;
      Remark: string;
      Reference: string;
      LineActionType: LineActionTypes;
      /**
       * Entity type: Order
       */
      ReturnedOrderID?: number;
      /**
       * Entity type: OrderLine
       */
      ReturnedOrderLineID?: number;
      ExportStatus: OrderExportStatuses;
      /**
       * @deprecated Replaced with `ExportStatus`
       */
      ExportStatusID: number;
      ContractNumber: string;
      IsCompleted: boolean;
      IsShipped: boolean;
      IsInvoiced: boolean;
      ExpectedAvailabilityDate?: string;
      RequestedDate?: string;
      CreationTime: string;
      /**
       * Entity type: User
       */
      CreatedByID: number;
      LastModificationTime?: string;
      IsDigitalGiftCard: boolean;
      /**
       * Entity type: Discount
       */
      DiscountID?: number;
      DiscountBackendID: string;
      DiscountCouponCode: string;
      DiscountCampaignName: string;
      DiscountAmount: number;
      /**
       * Entity type: ShippingMethod
       */
      ShippingMethodID?: number;
      ShippingMethod?: ShippingMethodDto;
      ExternalModificationStatus: ExternalModificationStatuses;
      CustomOrderLineType: string;
      /**
       * Entity type: OrderLine
       */
      RelatedOrderLineID?: number;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      ProductVariation: { [key: string]: string };
      /**
       * Entity type: ProductBundleLine
       */
      ProductBundleLineID?: number;
      TaxExemptionCode: string;
      TaxExemptionReason: string;
      TaxExemptionVerification: string;
      FulfillmentStatus: OrderLineFulfillmentStatuses;
      AvailabilityStatus?: OrderLineAvailabilityStatus;
      ReturnReason?: EnumWithBackendIDDto;
      RefundCorrectionReason?: RefundCorrectionReasonDto;
      /**
       * Entity type: User
       */
      SoldByID?: number;
      /**
       * Properties that should be used for display in the frontends to keep a matching experience in all apps.
       */
      DisplayProperties: OrderLineDisplayProperties;
      /**
       * Entity type: CustomField
       */
      CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
      /**
       * Entity type: PriceList
       */
      UnitPricePriceListID?: number;
      Discounts: OrderLineDiscount[];
      LoyaltyWithdrawals?: OrderLineLoyaltyMutation[];
      LoyaltyDeposits?: OrderLineLoyaltyMutation[];
      GroupID?: string;
      ImageBlobID?: string;
      /**
       * Entity type: OrganizationUnit
       */
      FulfillmentOrganizationUnitID?: number;
      FulfillmentOrganizationUnitName?: string;
      FulfillmentOrganizationUnitBackendID?: string;
      IsPricedByLoyaltyProgram?: boolean;
      IsLimitedByLoyaltyProgram?: boolean;
      PreSaleDate?: string;
    }

    export interface OrganizationUnit {
      ID: number;
      Name: string;
    }

  }

  /**
   * Get all the related orders for the given OrderID.
   * 
   * The orders will be sorted by CreationTime.
   */
  export interface GetRelatedOrders {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetRelatedOrdersResponse {
    Result: GetRelatedOrdersResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetRelatedOrdersResponse {
    export interface Model {
      /**
       * Entity type: Order
       */
      RelatedOrderID: number;
      BackendID: string;
      Description: string;
      CreationTime: string;
      TypeID: number;
      SoldFromOrganizationUnit: GetRelatedOrdersResponse.NamedDto;
      ShipFromOrganizationUnit?: GetRelatedOrdersResponse.NamedDto;
      SoldToOrganizationUnit?: GetRelatedOrdersResponse.NamedDto;
      ShipToOrganizationUnit?: GetRelatedOrdersResponse.NamedDto;
    }

    export interface NamedDto {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Get the required data for the order
   */
  export interface GetRequiredDataForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    RequiredFor?: RequiredFor;
  }

  export interface GetRequiredDataForOrderResponse {
    RequiredData: RequiredData;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets whether or not an Order is currently returnable.
   * 
   * For each OrderLine on the Order you'll get back a result that specifies whether or not it can be returned,
   * how much can be returned and which (if any) other OrderLines must be returned before the line itself can be returned
   * (think of parent-child lines where you can't return a child OrderLine without also returning the parent OrderLine).
   * 
   * In addition, this service may return `ReturnSets` which specify that 'if you return these OrderLines (specified by `OrderLineIDs`) then
   * you must *also* return these lines (specified by `RequiredOrderLineIDsToReturn`). For example, if you return all normal OrderLines on an Order
   * then you must also return all the free products you have received.
   * 
   * Finally, specify the `ReturningQuantities` parameter as an object where the key is an `OrderLineID` with a (positive) `Quantity` you wish to be
   * returned. The `Result` can be affected in case rules have been configured to, for example, allow _returns without products_ only when the total
   * return order amount is below a certain amount.
   */
  export interface GetReturnableStatusForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    ReturningQuantities?: { [key: number]: number };
    RequireLoginFunctionality?: boolean;
    HideCancelledLines?: boolean;
  }

  export interface GetReturnableStatusForOrderResponse {
    Result: ReturnableStatusForOrderLine[];
    ReturnSets: OrderLinesReturnSet[];
    StockOrganizationUnits: StockOrganizationUnit[];
    /**
     * Indicates whether the order needs to use the special return procedure for in-store returns. This is done using the `CreateInStoreReturn` service.
     */
    RequiresSpecialInStoreReturn: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ReturnableStatusForOrderLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    ReturnableQuantity: number;
    ReasonCode: string;
    Reason: string;
    IsReturnable: boolean;
    OverrideFunctionality?: FunctionalityWithScope;
    /**
     * When present, this OrderLine can only be returned when the OrderLines specified in this collection are also being returned.
     * Entity type: OrderLine
     */
    MustBeReturnedWithOrderLineIDs: number[];
    OrderLineType: OrderLineTypes;
    ReturnableAmount?: number;
    ReturnWithoutProducts: ReturnWithoutProductsResult;
    SerialNumberRequired: boolean;
    /**
     * Entity type: SerialNumberConfiguration
     */
    SerialNumberConfigurationID?: number;
    NonReturnableStockLabels?: ReturnableStatusForOrderLine.NonReturnableStockLabel[];
  }

  export namespace ReturnableStatusForOrderLine {
    export interface NonReturnableStockLabel {
      ID: number;
      Name: string;
      Description: string;
      DisplayName: string;
    }

  }

  export interface FunctionalityWithScope {
    Functionality: string;
    Scope: FunctionalityScope;
  }

  export const enum ReturnWithoutProductsResult {
    None = 0,
    ReturnWithoutProducts = 1,
    ReturnWithProducts = 2
  }

  export interface OrderLinesReturnSet {
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs: number[];
    /**
     * Entity type: OrderLine
     */
    RequiredOrderLineIDsToReturn: number[];
  }

  export interface StockOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    Reason: ReturnableStockOrganizationUnitReasons;
  }

  export const enum ReturnableStockOrganizationUnitReasons {
    CurrentOrganizationUnit = 0,
    ShipFromOrganizationUnit = 1,
    SupplierOrganizationUnit = 2,
    ReturnOrganizationUnit = 3,
    SoldFromOrganizationUnit = 4
  }

  /**
   * Returns all of the shipping methods that are being used on the order.
   */
  export interface GetShippingMethodsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetShippingMethodsForOrderResponse {
    ShippingMethods: ShippingMethodByOrderLineDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Allows customers to grab a token to be able to view their order for a limited amount of time. The token is
   * obtained through the user of either `OrderID` or both a `BackendSystemID` and a `BackendID` in combination
   * with the `EmailAddress` of the customer that is attached to the order.
   */
  export interface GetTemporaryTokenForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    EmailAddress: string;
  }

  export interface GetTemporaryTokenForOrderResponse {
    Token: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns a set of all the shipping methods that can be used for the given order. Use `SetShippingMethod` to update the shipping method on the order.
   */
  export interface ListAvailableShippingMethods {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
  }

  export interface ListAvailableShippingMethodsResponse {
    AvailableShippingMethods: ShippingMethodByOrderLineDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists the available custom order types that exists which can be used in the order editing process.
   */
  export interface GetCustomOrderTypes {
  }

  export interface GetCustomOrderTypesResponse {
    CustomOrderTypes: GetCustomOrderTypesResponse.CustomOrderType[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetCustomOrderTypesResponse {
    export interface CustomOrderType {
      /**
       * Entity type: CustomOrderType
       */
      ID: number;
      Name: string;
      Description: string;
    }

  }

  export interface ListExternalOrderStatusForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ListExternalOrderStatusForOrderResponse {
    Result: ListExternalOrderStatusForOrderResponse.ExternalOrderStatus[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListExternalOrderStatusForOrderResponse {
    export interface ExternalOrderStatus {
      Name: string;
      Status: string;
      CanPollStatus: boolean;
    }

  }

  export interface ListInvoicesForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ListInvoicesForOrderResponse {
    Result: ListInvoicesForOrderResponse.Invoice[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListInvoicesForOrderResponse {
    export interface Invoice {
      /**
       * Entity type: Invoice
       */
      ID: number;
      InvoiceNumber: string;
      InvoiceDate?: string;
      CreatedByID: number;
      CreationTime: string;
      TotalAmount: number;
      PaidAmount: number;
      OpenAmount: number;
      Payments: Invoice.Payment[];
    }

    export namespace Invoice {
      export interface Payment {
        /**
         * Entity type: PaymentTransaction
         */
        PaymentTransactionID: number;
        Amount: number;
      }

    }

  }

  /**
   * Retrieves a paged list of Order Ledgers for specified OrderID.
   */
  export interface ListOrderLedgerForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    PageConfig?: PageConfig<ListOrderLedgerFilter>;
  }

  export interface ListOrderLedgerFilter {
    /**
     * Entity type: OrderLedgerType
     */
    TypeIDs: number[];
  }

  export interface ListOrderLedgerForOrderResponse {
    Result?: PagedResult<ListOrderLedgerForOrderResponse.OrderLedgerDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrderLedgerForOrderResponse {
    export interface OrderLedgerDto {
      /**
       * Entity type: OrderLedger
       */
      ID: number;
      /**
       * Entity type: Order
       */
      OrderID: number;
      OrderDisplayID?: string;
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      Type: ListOrderLedgerForOrderResponse.OrderLedgerTypeDto;
      /**
       * Entity type: OrderLedgerType
       */
      TypeID: number;
      OldValue: string;
      NewValue: string;
      Description: string;
      /**
       * Entity type: User
       */
      CreatedByID: number;
      CreationTime: string;
      CreatedBy: ListOrderLedgerForOrderResponse.UserDto;
    }

    export interface UserDto {
      /**
       * Entity type: User
       */
      ID: number;
      FullName: string;
    }

    export interface OrderLedgerTypeDto {
      /**
       * Entity type: OrderLedgerType
       */
      ID: number;
      Name: string;
      Description: string;
      IsCustom: boolean;
    }

  }

  export interface ListOrderLinesInvoiceSummary {
    /**
     * Entity type: Order
     */
    OrderID: number;
    PageConfig?: PageConfig<ListOrderLinesFilter>;
  }

  export interface ListOrderLinesFilter {
    /**
     * Entity type: OrderLine
     */
    IDs: number[];
    Shippable: boolean;
    Invoiceable: boolean;
    OnlyShippableLines: boolean;
    ProductTypes?: ProductTypes;
    Query: string;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
  }

  export interface ListOrderLinesInvoiceSummaryResponse {
    Result?: PagedResult<ListOrderLinesInvoiceSummaryResponse.Model>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrderLinesInvoiceSummaryResponse {
    export interface Model {
      QuantityLeftToInvoice: number;
      /**
       * Entity type: OrderLine
       */
      ID: number;
      BackendReference: string;
      /**
       * Entity type: Order
       */
      OrderID: number;
      /**
       * Entity type: OrderLine
       */
      ParentID?: number;
      Children?: OrderLineDto[];
      Type: OrderLineTypes;
      /**
       * Entity type: StockLabel
       */
      StockLabel: number;
      Description: string;
      AdditionalDescription: string;
      /**
       * This is the quantity that was ordered. Make sure you really need this property, the total of the line is TotalQuantityToShip.
       */
      QuantityOrdered: number;
      QuantityOpen: number;
      QuantityCommitted: number;
      CommitmentStatus: OrderLineCommitmentStatus;
      QuantityShipped: number;
      QuantityDelivered: number;
      QuantityReserved: number;
      QuantityBackordered: number;
      QuantityReturned: number;
      QuantityCancelled: number;
      QuantityInvoiced: number;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      Product?: ProductDto;
      UnitPrice: number;
      UnitPriceBeforeCorrection?: number;
      UnitPriceBeforeCorrectionInTax?: number;
      UnitPriceCorrectionReason?: EnumDto;
      TaxRate: number;
      TaxCode: EnumDto;
      ShowTaxInfo: boolean;
      SerialNumber?: string;
      UnitPriceInTax: number;
      IsCancelled: boolean;
      TotalQuantityToShip: number;
      TotalAmountInTax: number;
      TotalAmount: number;
      NetTotalAmountInTax: number;
      NetTotalAmount: number;
      TotalTaxAmount: number;
      NetTotalTaxAmount: number;
      OriginalTotalAmountInTax?: number;
      OriginalTotalAmount?: number;
      EcoTaxInTax?: number;
      TotalEcoTaxInTax?: number;
      DisplayPriceInTax?: number;
      DisplayPrice?: number;
      ModificationAllowed: boolean;
      Remark: string;
      Reference: string;
      LineActionType: LineActionTypes;
      /**
       * Entity type: Order
       */
      ReturnedOrderID?: number;
      /**
       * Entity type: OrderLine
       */
      ReturnedOrderLineID?: number;
      ExportStatus: OrderExportStatuses;
      /**
       * @deprecated Replaced with `ExportStatus`
       */
      ExportStatusID: number;
      ContractNumber: string;
      IsCompleted: boolean;
      IsShipped: boolean;
      IsInvoiced: boolean;
      ExpectedAvailabilityDate?: string;
      RequestedDate?: string;
      CreationTime: string;
      /**
       * Entity type: User
       */
      CreatedByID: number;
      LastModificationTime?: string;
      IsDigitalGiftCard: boolean;
      /**
       * Entity type: Discount
       */
      DiscountID?: number;
      DiscountBackendID: string;
      DiscountCouponCode: string;
      DiscountCampaignName: string;
      DiscountAmount: number;
      /**
       * Entity type: ShippingMethod
       */
      ShippingMethodID?: number;
      ShippingMethod?: ShippingMethodDto;
      ExternalModificationStatus: ExternalModificationStatuses;
      CustomOrderLineType: string;
      /**
       * Entity type: OrderLine
       */
      RelatedOrderLineID?: number;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      ProductVariation: { [key: string]: string };
      /**
       * Entity type: ProductBundleLine
       */
      ProductBundleLineID?: number;
      TaxExemptionCode: string;
      TaxExemptionReason: string;
      TaxExemptionVerification: string;
      FulfillmentStatus: OrderLineFulfillmentStatuses;
      AvailabilityStatus?: OrderLineAvailabilityStatus;
      ReturnReason?: EnumWithBackendIDDto;
      RefundCorrectionReason?: RefundCorrectionReasonDto;
      /**
       * Entity type: User
       */
      SoldByID?: number;
      /**
       * Properties that should be used for display in the frontends to keep a matching experience in all apps.
       */
      DisplayProperties: OrderLineDisplayProperties;
      /**
       * Entity type: CustomField
       */
      CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
      /**
       * Entity type: PriceList
       */
      UnitPricePriceListID?: number;
      Discounts: OrderLineDiscount[];
      LoyaltyWithdrawals?: OrderLineLoyaltyMutation[];
      LoyaltyDeposits?: OrderLineLoyaltyMutation[];
      GroupID?: string;
      ImageBlobID?: string;
      /**
       * Entity type: OrganizationUnit
       */
      FulfillmentOrganizationUnitID?: number;
      FulfillmentOrganizationUnitName?: string;
      FulfillmentOrganizationUnitBackendID?: string;
      IsPricedByLoyaltyProgram?: boolean;
      IsLimitedByLoyaltyProgram?: boolean;
      PreSaleDate?: string;
    }

  }

  /**
   * List the OrderLines of an Order in a paginated fashion, allowing you to filter with a `PageConfig` and include product properties using `ProductProperties`.
   * 
   * Use this service when an Order contains a lot of lines, and the `GetOrder` service doesn't cut it - it will tell you it's time to switch using a property called `HasMoreLines`.
   */
  export interface ListOrderLines {
    /**
     * Entity type: Order
     */
    OrderID: number;
    ProductProperties?: string[];
    PageConfig?: PageConfig<ListOrderLinesFilter>;
    /**
     * When specified, order lines are returned nested under their parents recursively within their `Children` property. Filters will only be applied to the parent.
     */
    UseHierarchy?: boolean;
  }

  export interface ListOrderLinesResponse {
    Result?: PagedResult<OrderLineDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List orders for specific customer.
   */
  export interface ListOrdersForCustomer {
    /**
     * Entity type: User
     */
    ID: number;
    PageConfig?: PageConfig<ListOrdersForCustomerFilter>;
  }

  export interface ListOrdersForCustomerFilter {
    /**
     * Entity type: Order
     */
    ID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    IsShipped?: boolean;
    /**
     * @default false
     */
    OpenOrdersOnly?: boolean;
    /**
     * @default false
     */
    ShowOrdersWithoutLines?: boolean;
    /**
     * @default false
     */
    ModifiableOrdersOnly?: boolean;
    IsPlaced?: boolean;
    /**
     * @default EVA.Core.DataModels.OrderTypes[]
     */
    Type?: OrderTypes[];
    /**
     * @default EVA.Core.DataModels.OrderStatus[]
     */
    Status?: OrderStatus[];
    OrderProperties?: OrderProperties;
    ExcludeOrderProperties?: OrderProperties;
    /**
     * Entity type: OrganizationUnit
     */
    SoldFromOrganizationUnitID?: number;
  }

  export interface ListOrdersForCustomerResponse {
    Result?: PagedResult<OrderDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListOrderShipments {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ListOrderShipmentsResponse {
    Result: ListOrderShipmentsResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrderShipmentsResponse {
    export interface Model {
      /**
       * Entity type: Shipment
       */
      ID: number;
      BackendID: string;
      Status: ShipmentStatuses;
      ShipmentDate?: string;
      CompletionTime?: string;
      TrackingCode: string;
      TrackingLink: string;
      ShippedFromOrganizationUnit?: Model.OrganizationUnitModel;
      ShippedToOrganizationUnit: Model.OrganizationUnitModel;
      TotalQuantityShipped: number;
      TotalQuantityDelivered: number;
      NetTotalQuantityShipped: number;
      NetTotalQuantityDelivered: number;
      ReceiptCreatedBy: string;
      ReceiptCreatedOn?: string;
      CreationTime: string;
    }

    export namespace Model {
      export interface OrganizationUnitModel {
        /**
         * Entity type: OrganizationUnit
         */
        ID?: number;
        BackendID?: string;
        Name?: string;
        Description?: string;
      }

    }

  }

  export interface ListOrdersWithCustomerReferences {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListOrdersWithCustomerReferencesResponse {
    Result?: PagedResult<OrderWithCustomerReferences>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrderWithCustomerReferences {
    ID: number;
    CustomerReference: string;
    CustomerOrderID: string;
    CreationTime: string;
  }

  /**
   * Lock the Order in order to prevent some editing functionalities on it.
   */
  export interface LockOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Unlock the Order, allowing future editing on it.
   */
  export interface UnlockOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface MigrateOrdersToUnified {
    /**
     * Entity type: Order
     */
    OrderIDs?: number[];
  }

  export interface MigrateOrdersToUnifiedResponse {
    MigratedOrderIDs: number[];
    NotMigratedOrderIDs: number[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ModifyOrderLinePrice {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Range: 0-1000000
     */
    UnitPriceInTax: number;
    /**
     * Range: 0-1000000
     */
    UnitCost?: number;
  }

  export interface ModifyQuantityOrdered {
    Lines?: ModifyQuantityOrdered.OrderLineToModify[];
  }

  export namespace ModifyQuantityOrdered {
    export interface OrderLineToModify {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      /**
       * Min: 9000
       */
      NewQuantityOrdered: number;
    }

  }

  export interface ModifyQuantityOrderedResponse {
    /**
     * @deprecated Use Result.Messages
     */
    Messages: ModifyQuantityOrderedResponse.QuantityMutationResultMessage[];
    Result: ModifyQuantityOrderedResponse.ModifyQuantityResult[];
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ModifyQuantityOrderedResponse {
    export interface QuantityMutationResultMessage {
      Code: string;
      Message: string;
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
    }

    export interface ModifyQuantityResult {
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Entity type: OrderLine
       */
      CreatedOrderLineIDs?: number[];
      Messages?: ModifyQuantityOrderedResponse.ModifyQuantityOrderedMessage[];
    }

    export interface ModifyQuantityOrderedMessage {
      Code: string;
      Message: string;
    }

  }

  export interface ListOrderCustomStatus {
    PageConfig?: PageConfig<ListOrderCustomStatusFilter>;
  }

  export interface ListOrderCustomStatusFilter {
    /**
     * Entity type: OrderCustomStatus
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
    Color?: string;
  }

  export interface ListOrderCustomStatusResponse {
    Result?: PagedResult<ListOrderCustomStatusResponse.ListOrderCustomStatusModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrderCustomStatusResponse {
    export interface ListOrderCustomStatusModel {
      /**
       * Entity type: OrderCustomStatus
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Description?: string;
      Color?: string;
    }

  }

  export interface CreateOrderCustomStatus {
    BackendID?: string;
    Name: string;
    Description?: string;
    Color?: string;
  }

  export interface CreateOrderCustomStatusResponse {
    /**
     * Entity type: OrderCustomStatus
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateOrderCustomStatus {
    /**
     * Entity type: OrderCustomStatus
     */
    ID: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
    Color?: string;
  }

  export interface DeleteOrderCustomStatus {
    /**
     * Entity type: OrderCustomStatus
     */
    ID: number;
  }

  export interface ListOrderCustomTypes {
    PageConfig?: PageConfig<ListOrderCustomTypesFilter>;
  }

  export interface ListOrderCustomTypesFilter {
    /**
     * Entity type: OrderCustomType
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
  }

  export interface ListOrderCustomTypesResponse {
    Result?: PagedResult<ListOrderCustomTypesResponse.ListOrderCustomTypeModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrderCustomTypesResponse {
    export interface ListOrderCustomTypeModel {
      /**
       * Entity type: OrderCustomType
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Description?: string;
    }

  }

  export interface CreateOrderCustomType {
    BackendID?: string;
    Name: string;
    Description?: string;
  }

  export interface CreateOrderCustomTypeResponse {
    /**
     * Entity type: OrderCustomType
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateOrderCustomType {
    /**
     * Entity type: OrderCustomType
     */
    ID: number;
    BackendID?: string;
    Name?: string;
    Description?: string;
  }

  export interface DeleteOrderCustomType {
    /**
     * Entity type: OrderCustomType
     */
    ID: number;
  }

  export interface ListOrderExportsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListOrderExportsForOrderResponse {
    Result?: PagedResult<ListOrderExportsForOrderResponse.OrderExportDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrderExportsForOrderResponse {
    export interface OrderExportDto {
      /**
       * Entity type: OrderExport
       */
      ID: number;
      ShipToOrganizationUnit: ListOrderExportsForOrderResponse.OrganizationUnitDto;
      Name: string;
      Status: TransputJobStatuses;
      CreationTime: string;
      LastModificationTime?: string;
      /**
       * Entity type: OrderFulfillment
       */
      OrderFulfillmentID?: number;
      /**
       * Entity type: TransputJob
       */
      TransputJobID: number;
    }

    export interface OrganizationUnitDto {
      ID: number;
      Name: string;
      BackendID: string;
    }

  }

  export interface GetOrderExportByID {
    /**
     * Entity type: OrderExport
     */
    ID: number;
  }

  export interface GetOrderExportByIDResponse {
    ID: number;
    OrderID: number;
    Name: string;
    Status: TransputJobStatuses;
    IsDelivery: boolean;
    ShipFromOrganizationUnitID?: number;
    ShipFromOrganizationUnitName: string;
    ShipToOrganizationUnitID: number;
    ShipToOrganizationUnitName: string;
    Lines: GetOrderExportByIDResponse.OrderExportLineDto[];
    CreationTime: string;
    LastModificationTime?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOrderExportByIDResponse {
    export interface OrderExportLineDto {
      ID: number;
      OrderLineID: number;
      QuantityExported: number;
      QuantityCancelled: number;
      QuantityBackordered: number;
      BackendReference: string;
      CreationTime: string;
      LastModificationTime?: string;
    }

  }

  export interface RetryOrderExport {
    /**
     * Entity type: OrderExport
     */
    ID: number;
    Force?: boolean;
  }

  export interface GetOrderExportDocuments {
    /**
     * Entity type: OrderExport
     */
    ID: number;
  }

  export interface GetOrderExportDocumentsResponse {
    RequestDocuments: TransputJobDocument[];
    ResponseDocuments: TransputJobDocument[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Attach an identification document to an order. This will replace any existing identification document attached to the order.
   */
  export interface AttachIdentificationToOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Type?: IdentificationTypes;
    Number?: string;
    /**
     * ISO 3166-1 alpha-3 of the country
     */
    NationalityCode?: string;
    ValidTo?: string;
    IssuingDate?: string;
    IssuingCity?: string;
    /**
     * Entity type: Country
     */
    IssuingCountryID?: string;
    /**
     * Entity type: Country
     */
    OriginatingCountryID?: string;
    DocumentMimeType?: string;
    Document?: string;
  }

  export const enum IdentificationTypes {
    DiplomaticPassport = 1,
    DriverLicense = 2,
    EuropeanIdentification = 3,
    ForeignPassport = 4,
    INDSticker = 5,
    NatoPassport = 6,
    Passport = 7,
    PermanentResidencyDocument = 8,
    PrivilegedDocument = 9,
    ResidencePermitTypeEuEea = 10,
    ResidencePermitTypeI = 11,
    ResidencePermitTypeII = 12,
    ResidencePermitTypeIII = 13,
    ResidencePermitTypeIV = 14,
    ResidencePermitTypeW = 15,
    VNGCard = 16,
    ResidencePermitTypeV = 17
  }

  /**
   * Detach an identification document from an order. This will wipe the document and data.
   */
  export interface DetachIdentificationFromOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface CreateOrderLink {
    /**
     * Entity type: Order
     */
    PrimaryOrderID: number;
    /**
     * Entity type: Order
     */
    RelatedOrderID: number;
    /**
     * Entity type: OrderLinkType
     */
    TypeID: number;
  }

  export interface CreateOrderLinkResponse {
    /**
     * Entity type: OrderLink
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateOrderLink {
    /**
     * Entity type: OrderLink
     */
    ID: number;
    /**
     * Entity type: Order
     */
    PrimaryOrderID?: number;
    /**
     * Entity type: Order
     */
    RelatedOrderID?: number;
    /**
     * Entity type: OrderLinkType
     */
    TypeID?: number;
  }

  export interface DeleteOrderLink {
    /**
     * Entity type: OrderLink
     */
    ID: number;
  }

  export interface CreateOrderLinkType {
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  export interface CreateOrderLinkTypeResponse {
    /**
     * Entity type: OrderLinkType
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateOrderLinkType {
    /**
     * Entity type: OrderLinkType
     */
    ID: number;
    Name: string;
    Description?: string;
    BackendID?: string;
  }

  export interface DeleteOrderLinkType {
    /**
     * Entity type: OrderLinkType
     */
    ID: number;
  }

  export interface GetOrderLinkTypeByID {
    /**
     * Entity type: OrderLinkType
     */
    ID: number;
  }

  export interface GetOrderLinkTypeByIDResponse {
    OrderLinkType: OrderLinkType;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrderLinkType {
    ID: number;
    Name: string;
    Description: string;
    BackendID: string;
    IsDeleted: boolean;
  }

  export interface GetOrderLinkTypes {
  }

  export interface GetOrderLinkTypesResponse {
    OrderLinkTypes: OrderLinkType[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create a new order verification reason.
   */
  export interface CreateOrderVerificationReason {
    Name?: string;
    BackendID?: string;
    Description?: string;
  }

  export interface CreateOrderVerificationReasonResponse {
    /**
     * Entity type: OrderVerificationReason
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists order verification reasons.
   */
  export interface ListOrderVerificationReasons {
    InitialPageConfig?: PageTokenConfig<ListOrderVerificationReasonFilter>;
    PageToken?: string;
  }

  export interface ListOrderVerificationReasonFilter {
    Name: string;
    Description: string;
  }

  export interface ListOrderVerificationReasonsResponse {
    Results?: ListOrderVerificationReasonsResponse.OrderVerificationReason[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOrderVerificationReasonsResponse {
    export interface OrderVerificationReason {
      /**
       * Entity type: OrderVerificationReason
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Description: string;
    }

  }

  /**
   * Get order verification reason.
   */
  export interface GetOrderVerificationReason {
    /**
     * Entity type: OrderVerificationReason
     */
    ID: number;
  }

  export interface GetOrderVerificationReasonResponse {
    /**
     * Entity type: OrderVerificationReason
     */
    ID: number;
    BackendID?: string;
    Name: string;
    Description: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an order verification reason
   */
  export interface UpdateOrderVerificationReason {
    /**
     * Entity type: OrderVerificationReason
     */
    ID: number;
    Name?: string;
    BackendID?: string;
    Description?: string;
  }

  /**
   * Delete an order verification reason
   */
  export interface DeleteOrderVerificationReason {
    /**
     * Entity type: OrderVerificationReason
     */
    ID: number;
  }

  /**
   * Placing an order means it is final for all parties; both agree on the exchange in goods, services and/or money that this order implies.
   * 
   * Will set the `Order`.`IsPlaced` flag.
   * 
   * This service can be used to place a single order, or to place a batch at once.
   */
  export interface PlaceOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Order
     */
    OrderIDs?: number[];
  }

  export interface PlaceOrderResponse {
    ValidationResult?: OrderExportValidationResult;
    ValidationResults?: OrderExportValidationResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface PollExternalOrderStatus {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Name?: string;
  }

  /**
   * Prepare the order to go into the checkout
   * 
   * If the order still has status ShoppingCart, update it to Order and:
   *  - Validate lineactiontypes (customer are not allowed to use ShipLine p/a)
   *  - Set the SoldBy on the lines to the current user
   *  - Set the addresses of the customer on the order, if not already set
   *  - (Re)calculate the taxes
   *  - Recalculate the order: discounts, shippingcosts, giftwrappingcosts etc.
   */
  export interface PrepareOrderForCheckout {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ProduceTrackingInformation {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ProduceTrackingInformationResponse {
    Result: ProduceTrackingInformationResponse.TrackingInformation;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ProduceTrackingInformationResponse {
    export interface TrackingInformation {
      Link: string;
      Code: string;
    }

  }

  /**
   * Creates a returnorder with all specified data
   * 
   * It is required to send the identifier for the ReturnOrder of the external system by providing a BackendSystemID and a BackendID.
   * 
   * A return is based of an OriginalOrder. This order can be identifier by the OriginalBackendID or the OriginalOrderID. One of those is required.
   * 
   * If the stock is already received, set the `AlreadyReceived` to true. This will ship the order immediately.
   * 
   * If the stock is already received and the open amount should be refunded directly, set `AutoRefund` to true.
   * 
   * The Lines have to be identified by the original line, this should be done by providing the OriginalBackendID or the OriginalOrderLineID.
   * If those are not available the CustomID of the product can be set, but matching based on that is not advised.
   */
  export interface PushReturnOrder {
    /**
     * A code, free of choice for the calling 3rd party application creating the order
     */
    BackendSystemID: string;
    /**
     * A unique identifier for the order, within the context of given `BackendSystemID`
     */
    BackendID: string;
    /**
     * The unique identifier of the original order, within the context of given `BackendSystemID`
     */
    OriginalBackendID?: string;
    /**
     * Entity type: Order
     */
    OriginalOrderID?: number;
    /**
     * Remark on the order (optional)
     */
    Remark?: string;
    /**
     * Where the return will be booked. If not set, the logged in OU will be used.
     */
    ReturnToOrganizationUnitBackendID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    StockOrganizationUnitID?: number;
    /**
     * Ship the order immediately
     */
    AlreadyReceived?: boolean;
    /**
     * Try to refund the order immediately. Unreferenced returns cannot be auto refunded.
     */
    AutoRefund?: boolean;
    /**
     * Sends an invoice (credit note) - this is only done if AutoRefund or AlreadyReceived are also true, and requires a customer to be set
     */
    SendInvoice?: boolean;
    /**
     * If this order pending a customer return
     */
    IsCustomerReturn?: boolean;
    /**
     * Specifies if the order is an offer - this is only happening if AutoRefund or AlreadyReceived are set to false. If no value is specified, the default is false.
     * @default false
     */
    IsOffer?: boolean;
    /**
     * Specifies the offer expiration date.
     */
    OfferExpirationDate?: string;
    Lines: PushReturnOrder.PushReturnOrderLine[];
    ReturnCosts?: PushReturnOrder.PushReturnCostsLine;
    CustomFields?: { [key: string]: any };
    /**
     * Specify the customer for an unreferenced return.
     * Entity type: User
     */
    CustomerID?: number;
  }

  export namespace PushReturnOrder {
    export interface PushReturnOrderLine {
      /**
       * ID of the original orderline from the 3rd party
       */
      OriginalBackendID?: string;
      /**
       * Entity type: OrderLine
       */
      OriginalOrderLineID?: number;
      /**
       * Article number of the product from the 3rd party
       */
      CustomID?: string;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      /**
       * Quantity that has been returned, positive
       */
      Quantity: number;
      /**
       * Remark describing something return related
       */
      Remark?: string;
      /**
       * Entity type: ReturnReason
       */
      ReturnReasonID?: number;
      SerialNumber?: string;
      /**
       * Return stock label. Only relevant for unreferenced returns.
       * Entity type: StockLabel
       */
      ReturnStockLabel?: number;
    }

    export interface PushReturnCostsLine {
      Description: string;
      UnitPrice: number;
      /**
       * Range: 1-2
       */
      TaxRate?: number;
      TaxAmount?: number;
    }

  }

  export interface PushReturnOrderResponse {
    /**
     * Entity type: Order
     */
    ID: number;
    /**
     * If the order with the specified BackendID already existed
     */
    AlreadyExists: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates an order with all specified data, places it, and raises all follow-up events.
   * 
   * Whenever the order already exists with the same BackendSystemID/BackendID combination, nothing will be modified.
   */
  export interface PushSalesOrder {
    /**
     * A code, free of choice for the calling 3rd party application creating the order
     */
    BackendSystemID?: string;
    /**
     * A unique identifier for the order, within the context of given `BackendSystemID`
     */
    BackendID?: string;
    /**
     * ISO code of the currency of the order
     */
    CurrencyID?: string;
    /**
     * Identifier for the (web)shop that made the sale (`OrganizationUnit.BackendID`)
     */
    SoldFromOrganizationUnitBackendID?: string;
    /**
     * Identifier for the shop where the order will be picked up, if any (`OrganizationUnit.BackendID`
     */
    PickupOrganizationUnitBackendID?: string;
    /**
     * Identifier for the shop where the order will be shipped to, if any (`OrganizationUnit.BackendID`
     */
    ShipToOrganizationUnitBackendID?: string;
    /**
     * How this order is fulfilled, this can be overridden on line level. Default to Delivery or Pickup (when a PickupOrganizationUnit is defined)
     */
    FulfillmentMethod?: OrderImportFulfillmentMethod;
    /**
     * The reason for this ReturnToSupplier order, used in combination with Properties.ReturnToSupplier. Required for ReturnToSupplier orders
     */
    ReturnToSupplierReason?: string;
    /**
     * Enabling this will try to suppress as many eventexports as possible that might result from this call.
     */
    DisableEventExports?: boolean;
    /**
     * Lock Order and block any future changes on it.
     */
    LockOrder?: boolean;
    LoyaltyProgramBackendID?: string;
    Customer?: PushSalesOrderCustomer;
    ShippingAddress?: PushSalesOrderAddress;
    BillingAddress?: PushSalesOrderAddress;
    PickupAddress?: PushSalesOrderAddress;
    Lines?: PushSalesOrderLine[];
    BackfillInformation?: PushSalesOrderBackfillInformation;
    Remark?: string;
    CustomData?: { [key: string]: any };
    CustomFields?: { [key: string]: any };
    Discounts?: PushSalesOrderDiscount[];
    Payments?: PushSalesOrderPayment[];
    Shipments?: PushSalesOrderShipment[];
    ShippingInformation?: PushSalesOrderShippingInformation;
    Properties?: PushSalesOrderProperties;
    Options?: PushSalesOrderOptions;
    Coupons?: PushSalesOrderCoupon[];
    /**
     * @default false
     */
    IsOffer?: boolean;
    OfferExpirationDate?: string;
  }

  export const enum OrderImportFulfillmentMethod {
    Delivery = 1,
    Pickup = 2,
    CarryOut = 3
  }

  export interface PushSalesOrderCustomer {
    BackendID?: string;
    EmailAddress?: string;
    /**
     * Possible values are: M, F, O
     */
    Gender?: string;
    Initials?: string;
    FirstName?: string;
    LastName?: string;
    PhoneNumber?: string;
    DateOfBirth?: string;
    PlaceOfBirth?: string;
    BankAccount?: string;
    Nickname?: string;
    /**
     * ISO 639-1
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * ISO 3166-1
     * Entity type: Country
     */
    CountryID?: string;
    FiscalID?: string;
    SocialSecurityNumber?: string;
    Title?: string;
    Salutation?: string;
    /**
     * Is this a Standard/Basic/Incognito account?
     * @default Standard
     */
    AccountType?: UserAccountType;
    Company?: PushSalesOrderCustomerCompany;
    BackendRelationID?: string;
  }

  export interface PushSalesOrderCustomerCompany {
    Name?: string;
    RegistrationNumber?: string;
    VatNumber?: string;
    FiscalID?: string;
  }

  export interface PushSalesOrderAddress {
    /**
     * House number, including possible extensions
     */
    HouseNumber?: string;
    Address1?: string;
    Address2?: string;
    FirstName?: string;
    LastName?: string;
    ZipCode?: string;
    City?: string;
    Region?: string;
    District?: string;
    Subdistrict?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    /**
     * ISO 3166-2
     */
    State?: string;
    /**
     * ISO 3166-1
     * Entity type: Country
     */
    CountryID?: string;
  }

  export interface PushSalesOrderLine {
    /**
     * Optional grouping identifier to keep lines 'together'.
     */
    GroupID?: string;
    /**
     * Child lines for this line
     */
    ChildLines?: ChildPushSalesOrderLine[];
    /**
     * ID of the order line from the 3rd party
     */
    BackendID?: string;
    /**
     * Article number of the product from the 3rd party
     */
    CustomID?: string;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Price of a single unit, before discounts, based on the SoldFromOrganizationUnit settings this should be in or ex tax. See service documentation.
     */
    Amount?: number;
    /**
     * Original price of a single unit, before discounts, based on the SoldFromOrganizationUnit settings this should be in or ex tax. See service documentation.
     */
    OriginalAmount?: number;
    Quantity?: number;
    /**
     * When no tax rate is provided, the taxrate will be based on what is known in Eva. When no rate can be determined, an error will be thrown.
     */
    TaxRate?: number;
    /**
     * When known the exact TaxAmount can be specified. Used when tax calculations are done by an external party like Avatax.
     */
    TaxAmount?: number;
    Description?: string;
    RequestedDeliveryDate?: string;
    Discounts?: PushSalesOrderDiscount[];
    CustomData?: { [key: string]: any };
    FulfillmentMethod?: OrderImportFulfillmentMethod;
    Shipments?: PushSalesOrderLineShipment[];
    /**
     * Used for passing in the serial number of, for example, a gift card. This is used to support refunding and other operations on the gift card.
     */
    SerialNumber?: string;
    /**
     * Used for passing in the data for a digital giftcard.
     */
    GiftcardData?: GiftCardData;
    /**
     * Used for passing in the media information for a digital giftcard.
     */
    GiftcardMedia?: GiftCardMedia;
    /**
     * Used for passing the identifier used to identify this line at the payment provider. This is required for payments that need capture on line level like Klarna.
     */
    PaymentLineIdentifier?: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    /**
     * Product requirements
     */
    ProductRequirements?: PushSalesOrderProductRequirement[];
    /**
     * Marks the line as a final sale line, making it non returnable.
     */
    IsFinalSale?: boolean;
    /**
     * A code identifying the tax exemption.
     */
    TaxExemptionCode?: string;
    /**
     * A longer description with the justification for the tax exemption.
     */
    TaxExemptionReason?: string;
    TaxRateCategory?: TaxRateCategory;
    /**
     * Allows you to specify a preferred organization for doing the physical fulfillment of the OrderLine.
     * Depending on your order orchestration configuration this can be a hard filter or merely a preference.
     */
    PreferredFulfillmentOrganizationUnitBackendID?: string;
  }

  export interface ChildPushSalesOrderLine {
    /**
     * ID of the order line from the 3rd party
     */
    BackendID?: string;
    /**
     * Article number of the product from the 3rd party
     */
    CustomID?: string;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Price of a single unit, before discounts, based on the SoldFromOrganizationUnit settings this should be in or ex tax. See service documentation.
     */
    Amount?: number;
    /**
     * Original price of a single unit, before discounts, based on the SoldFromOrganizationUnit settings this should be in or ex tax. See service documentation.
     */
    OriginalAmount?: number;
    Quantity?: number;
    /**
     * When no tax rate is provided, the taxrate will be based on what is known in Eva. When no rate can be determined, an error will be thrown.
     */
    TaxRate?: number;
    /**
     * When known the exact TaxAmount can be specified. Used when tax calculations are done by an external party like Avatax.
     */
    TaxAmount?: number;
    Description?: string;
    RequestedDeliveryDate?: string;
    Discounts?: PushSalesOrderDiscount[];
    CustomData?: { [key: string]: any };
    FulfillmentMethod?: OrderImportFulfillmentMethod;
    Shipments?: PushSalesOrderLineShipment[];
    /**
     * Used for passing in the serial number of, for example, a gift card. This is used to support refunding and other operations on the gift card.
     */
    SerialNumber?: string;
    /**
     * Used for passing in the data for a digital giftcard.
     */
    GiftcardData?: GiftCardData;
    /**
     * Used for passing in the media information for a digital giftcard.
     */
    GiftcardMedia?: GiftCardMedia;
    /**
     * Used for passing the identifier used to identify this line at the payment provider. This is required for payments that need capture on line level like Klarna.
     */
    PaymentLineIdentifier?: string;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    /**
     * Product requirements
     */
    ProductRequirements?: PushSalesOrderProductRequirement[];
    /**
     * Marks the line as a final sale line, making it non returnable.
     */
    IsFinalSale?: boolean;
    /**
     * A code identifying the tax exemption.
     */
    TaxExemptionCode?: string;
    /**
     * A longer description with the justification for the tax exemption.
     */
    TaxExemptionReason?: string;
    TaxRateCategory?: TaxRateCategory;
    /**
     * Allows you to specify a preferred organization for doing the physical fulfillment of the OrderLine.
     * Depending on your order orchestration configuration this can be a hard filter or merely a preference.
     */
    PreferredFulfillmentOrganizationUnitBackendID?: string;
  }

  export interface PushSalesOrderDiscount {
    /**
     * Reference to the discount of the 3rd party
     */
    BackendID?: string;
    /**
     * Amount of the discount, typically with a negative value
     */
    Amount?: number;
    Description?: string;
    CampaignName?: string;
    CouponCode?: string;
  }

  export interface PushSalesOrderLineShipment {
    /**
     * Identifier of the shipment, must exist in the Order.Shipments
     */
    ShipmentBackendID: string;
    /**
     * Identifier of the shipment line
     */
    BackendID?: string;
    /**
     * Date when this shipment line was shipped, will fall-back to the Shipment.ShipmentDate
     */
    ShipmentDate?: string;
    Quantity: number;
  }

  export interface GiftCardData {
    Theme: GiftCardTheme;
    From: string;
    To: string;
    Text: string;
    FromEmailAddress?: string;
    ToEmailAddress?: string;
    PhoneNumber?: string;
    DeliverySchedule?: string;
    UpdateUrl: string;
    DeliveryMode?: GiftCardDeliveryMode;
  }

  export interface GiftCardTheme {
    ID: string;
    Name: string;
    Description: string;
    LongDescription: string;
    ForegroundColor: string;
    BackgroundColor: string;
    ImageUrl: string;
    ImageBlobID?: string;
    ImageWidth: number;
    ImageHeight: number;
    VideoUrl: string;
    VideoBlobID?: string;
    CloudinaryImagePublicID: string;
    CloudinaryVideoPublicID: string;
  }

  export const enum GiftCardDeliveryMode {
    Default = 0,
    Instant = 1,
    NotSet = 2
  }

  export interface GiftCardMedia {
    CloudinaryImageID?: string;
    CloudinaryImageMimeType?: string;
    CloudinaryVideoID?: string;
    CloudinaryVideoMimeType?: string;
  }

  export interface PushSalesOrderProductRequirement {
    Name: string;
    Value?: any;
  }

  export interface PushSalesOrderBackfillInformation {
    /**
     * Original date when this order was created
     */
    OriginalCreationDate?: string;
    /**
     * Original date when this order was invoiced
     */
    OriginalInvoiceDate?: string;
    /**
     * Original invoicenumber of this order
     */
    OriginalInvoiceNumber?: string;
  }

  export interface PushSalesOrderPayment {
    /**
     * Identifier of the payment by the 3rd party (e.g. the one from the payment service provider (PSP))
     */
    BackendID?: string;
    /**
     * Code of the payment method (e.g. `ADYEN_IDEAL`)
     */
    Method?: string;
    /**
     * Amount of the payment that was actually paid
     */
    Amount?: number;
    /**
     * Part (or all) of the amount that has already been captured for this payment.
     * This amount should always be in the base currency of the order.
     */
    CapturedAmount?: number;
    /**
     * If this is set to not None, the Payment will not be considered captured, but only Authorized. This means the capture will be done based on this value.
     * If it's not set, the CaptureMoment will be determined based on the PaymentType.
     */
    CaptureMoment?: PaymentTypeCaptureMoment;
    /**
     * Data that can be used to store anything that may be required later to verify the payment for example
     */
    Data?: string;
    /**
     * This property is optional but becomes mandatory when a payment is made in a currency different from the order base currency.
     * It indicates the currency used for the payment.
     */
    ForeignCurrencyID?: string;
    /**
     * This property is optional but becomes mandatory when a payment is made in a currency different from the order base currency.
     * It indicates the specific amount paid in the foreign currency.
     */
    ForeignAmount?: number;
    /**
     * Whether to automatically confirm the payment or adhere to the payment type configuration.
     * The default value is set to true.
     */
    AutoConfirm?: boolean;
  }

  export interface PushSalesOrderShipment {
    /**
     * Identifier of the shipment; should also be filled on Line.Shipments.ShipmentBackendID
     */
    BackendID?: string;
    /**
     * Date when this shipment was shipped
     */
    ShipmentDate?: string;
    TrackingCode?: string;
    TrackingLink?: string;
  }

  export interface PushSalesOrderShippingInformation {
    /**
     * Method that is used to ship the order (e.g. UPS, DHL, etc.)
     */
    Method?: string;
    /**
     * ShippingCosts, before discounts, based on the SoldFromOrganizationUnit settings this should be in or ex tax. See service documentation.
     */
    Amount?: number;
    /**
     * Tax rate of the shipping method, when `Amount` / `UnitPriceInTax` is set and it's not provided, the taxrate will be based on what is known in Eva. When no rate can be determined, an error will be thrown.
     */
    TaxRate?: number;
    /**
     * When known the exact TaxAmount can be specified. Used when tax calculations are done by an external party like Avatax.
     */
    TaxAmount?: number;
    Discount?: PushSalesOrderDiscount;
    /**
     * Used for passing the identifier used to identify the shipping costs at the payment provider. This is required for payments that need capture on line level like Klarna.
     */
    PaymentLineIdentifier?: string;
  }

  export interface PushSalesOrderProperties {
    IsInterbranch?: boolean;
    Replenishment?: boolean;
    ReturnToSupplier?: boolean;
  }

  export interface PushSalesOrderOptions {
    /**
     * Never generate an invoice for this order. The order will be completed when it's Paid+Shipped
     */
    DoNotInvoice?: boolean;
    /**
     * Allow the order to be fulfilled in multiple shipments
     */
    AllowPartialFulfillment?: boolean;
    /**
     * When true, this service call return with an error when the order does not end up with an open amount of exactly 0
     */
    CheckOpenAmountIsZero?: boolean;
  }

  export interface PushSalesOrderCoupon {
    DiscountBackendID: string;
    CouponCode: string;
  }

  export interface PushSalesOrderResponse {
    /**
     * Entity type: Order
     */
    ID: number;
    /**
     * Entity type: User
     */
    CustomerID?: number;
    AlreadyExists: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Remove offer expiration date for an order specified by `OrderID`.
   */
  export interface RemoveOfferStatusFromOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Reserve an Order.
   * 
   * The Lines on the Order with LineActionType ReserveLine will be reserved:
   * - The Stock will be moved from Sellable to Reserved
   * - A receipt will be printed
   */
  export interface ReserveOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Station to print the receipt on
     * Entity type: Station
     */
    StationID?: number;
  }

  /**
   * Search for any type of orders by providing the filter details properties
   * Important search criterias are: OrganizationUnitID, CurrentOrganizationUnitType, OrderIDs, CustomerID(ClientContext.Type), PaymentStatus.
   * 
   * When no StartTime is provided this will be defaulted to 3 months ago. The only exception is when specific OrderIDs are provided, then it's allowed to not specify a Date.
   * 
   * To include the custom fields of the customer, the property `AdditionalOrderDataOptions.CustomerCustomFields` can be set to `true`.
   * 
   * E.g. request:
   * {
   *   "OrderIDs": [
   *     "orderID1",
   *     "orderID2"
   *   ],
   *   "PaymentStatus": "PaymentStatus",
   *   "IsShipped": "isShipped"
   * }
   * 
   * 
   * Payment status options:
   *     NotPaid = 0,
   *     Paid = 1,
   *     Partial = 2
   */
  export interface SearchOrders {
    Query?: string;
    IsConfirmed?: boolean;
    IsPlaced?: boolean;
    IsShipped?: boolean;
    IsCompleted?: boolean;
    IsCaptured?: boolean;
    IsSuspended?: boolean;
    HasShipLines?: boolean;
    HasDeliveryLines?: boolean;
    HasReserveLines?: boolean;
    HasOrderLines?: boolean;
    /**
     * The StartTime will default to 3 months ago if not provided and OrderIDs are not provided.
     */
    StartTime?: string;
    EndTime?: string;
    StartCreationTime?: string;
    EndCreationTime?: string;
    StartPlacementDate?: string;
    EndPlacementDate?: string;
    StartRequestedDate?: string;
    EndRequestedDate?: string;
    StartShipmentDate?: string;
    EndShipmentDate?: string;
    StartConfirmationDate?: string;
    EndConfirmationDate?: string;
    StartMinExpectedAvailabilityDate?: string;
    EndMinExpectedAvailabilityDate?: string;
    StartMaxExpectedAvailabilityDate?: string;
    EndMaxExpectedAvailabilityDate?: string;
    /**
     * Entity type: Order
     */
    OrderIDs?: number[];
    OrderBackendID?: string;
    OrderDisplayID?: string;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    PaymentStatus?: PaymentStatus;
    InvoiceNumber?: string;
    CompanyName?: string;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    SerialNumber?: string;
    /**
     * Entity type: Discount
     */
    DiscountIDs?: number[];
    IsReserved?: boolean;
    IsOrder?: boolean;
    IsInvoiced?: boolean;
    IsEmpty?: boolean;
    IsExported?: boolean;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    PaymentTypeName?: string;
    PaymentTransactionBackendID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    MinTotalAmountInTax?: number;
    MaxTotalAmountInTax?: number;
    MinOpenAmountInTax?: number;
    MaxOpenAmountInTax?: number;
    MinTotalItems?: number;
    MaxTotalItems?: number;
    ProductVariation?: { [key: string]: string[] };
    Data?: { [key: string]: OrderSearchDataFilterModel };
    /**
     * Entity type: CustomField
     */
    CustomFields?: { [key: number]: string };
    CustomFieldsByBackendID?: { [key: string]: string };
    HasCancelledLines?: boolean;
    HasDisputedPayments?: boolean;
    HasUnresolvedDisputedPayments?: boolean;
    HasCustomer?: boolean;
    /**
     * Defaults to OrderTypes.Sales
     */
    OrderType?: OrderTypes;
    OrderStatus?: OrderStatus;
    /**
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    CustomerID?: number;
    /**
     * Entity type: User
     */
    CustomerIDs?: number[];
    CustomerBackendID?: string;
    /**
     * Entity type: User
     */
    CreatedByID?: number;
    /**
     * Entity type: User
     */
    LastModifiedByID?: number;
    CustomerReference?: string;
    /**
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    FulfillmentOrganizationUnitIDs?: number[];
    /**
     * Entity type: Country
     */
    ShipToCountryID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    SoldFromOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    SoldToOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OriginatingOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    PickupOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShippingOrganizationUnitID?: number;
    IncludeChildOrganizationUnits?: boolean;
    /**
     * Properties that should be present on the order
     */
    Properties?: number[];
    /**
     * Properties that should not be present on the order
     */
    NotProperties?: number[];
    CustomStatus?: number[];
    UserAgentApplication?: string[];
    FulfillmentStatus?: OrderSearchFulfillmentStatusFilterModel;
    /**
     * Entity type: Carrier
     */
    CarrierID?: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    IsOffer?: boolean;
    StartOfferExpirationDate?: string;
    EndOfferExpirationDate?: string;
    CustomerOrderStatus?: string[];
    CustomerOrderStatusBackendID?: string[];
    CustomerOrderTypeBackendID?: string[];
    /**
     * Entity type: ReturnToSupplierRequest
     */
    ReturnToSupplierRequestID?: number;
    OrderFulfillmentReferenceNumbers?: string[];
    /**
     * Entity type: CustomOrderType
     */
    CustomOrderTypeID?: number;
    /**
     * Entity type: OrderCustomStatus
     */
    OrderCustomStatusID?: number;
    /**
     * Entity type: OrderCustomType
     */
    OrderCustomTypeID?: number;
    IsCrossChannelReturn?: boolean;
    IsClickAndCollect?: boolean;
    IsEndlessAisle?: boolean;
    HasSoldToOrganizationUnit?: boolean;
    AdditionalOrderDataOptions?: SearchOrders.AdditionalSearchOrderDataOptions;
    IsReturnToSupplier?: boolean;
    HasPriceCorrection?: boolean;
    /**
     * Use only when really needed. This will include the first order line description in the search result.
     */
    IncludeFirstOrderLineDescription?: boolean;
    AggregationOptions?: OrderSearchAggregationOptions;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export namespace SearchOrders {
    export interface AdditionalSearchOrderDataOptions {
      CustomerCustomFields?: boolean;
    }

  }

  export const enum PaymentStatus {
    NotPaid = 0,
    Paid = 1,
    Partial = 2
  }

  export interface OrderSearchDataFilterModel {
    StringValue: string;
    BooleanValue?: boolean;
    StartDateTime?: string;
    EndDateTime?: string;
    StartDecimal?: number;
    EndDecimal?: number;
  }

  export interface OrderSearchFulfillmentStatusFilterModel {
    CanFulfill?: boolean;
    IsBlocked?: boolean;
  }

  export interface OrderSearchAggregationOptions {
    UserAgentApplication?: boolean;
    ProductVariations?: boolean;
  }

  /**
   * The Result propery contains the paged orders.
   * Result.Page contains the first sequence of the paged result.
   * Result.Page.ID is the orderID.
   */
  export interface SearchOrdersResponse {
    ProductVariations: { [key: string]: { [key: string]: number } };
    UserAgents: { [key: string]: { [key: string]: number } };
    CustomerOrderStatuses: { [key: string]: number };
    Result?: PagedResult<OrderSearchResultItem>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrderSearchResultItem {
    CreatedBy: OrderSearchResultItemUser;
    LastModifiedBy: OrderSearchResultItemUser;
    Customer: OrderSearchResultItemCustomer;
    OrganizationUnits: OrderSearchResultItemOrganizationUnit[];
    SoldFromOrganizationUnit: OrderSearchResultItemOrganizationUnit;
    SoldToOrganizationUnit: OrderSearchResultItemOrganizationUnit;
    ShipFromOrganizationUnit: OrderSearchResultItemOrganizationUnit;
    ShipToOrganizationUnit: OrderSearchResultItemOrganizationUnit;
    ShippingAddress: OrderSearchResultItemAddress;
    OriginatingOrganizationUnit: OrderSearchResultItemOrganizationUnit;
    PickupOrganizationUnitID: OrderSearchResultItemOrganizationUnit;
    PriceListIDs: number[];
    IsCompleted: boolean;
    IsPaid: boolean;
    IsCaptured: boolean;
    IsSuspended: boolean;
    IsPlaced: boolean;
    IsConfirmed: boolean;
    IsShipped: boolean;
    IsReserved: boolean;
    IsInvoiced: boolean;
    HasShipLines?: boolean;
    HasDeliveryLines?: boolean;
    HasReserveLines?: boolean;
    HasOrderLines?: boolean;
    /**
     * Entity type: Order
     */
    ID: number;
    BackendID: string;
    TotalAmountInTax: number;
    TotalAmount: number;
    PaidAmount: number;
    ForeignPaidAmount: number;
    OpenAmountInTax: number;
    OpenAmount: number;
    LastModificationTime: string;
    CreationTime: string;
    CurrencyID: string;
    Type: OrderTypes;
    Status: OrderStatus;
    IsOffer?: boolean;
    CustomerReference: string;
    HasCancelledLines: boolean;
    HasPriceCorrection: boolean;
    HasDisputedPayments: boolean;
    HasUnresolvedDisputedPayments: boolean;
    TotalItems: number;
    TotalQuantityToShip: number;
    TotalQuantityShipped: number;
    Properties: OrderProperties;
    PropertiesIDs: number[];
    CustomStatus: number;
    CustomStatusIDs: number[];
    OrderCustomTypeID?: number;
    OrderCustomStatusID?: number;
    RequestedDate?: string;
    ConfirmationDate?: string;
    Data: OrderSearchResultItemData[];
    IsEmpty: boolean;
    UserAgentApplication: string;
    UserAgentVersion: string;
    ProductImage: string;
    ProductImageUrl: string;
    FirstOrderLineDescription?: string;
    Definition: OrderSearchResultItemDefinition;
    ReturnToSupplierRequest: OrderSearchResultItemReturnToSupplierRequest;
    /**
     * Entity type: CustomOrderType
     */
    CustomOrderTypeID?: number;
    MinExpectedAvailabilityDate?: string;
    MaxExpectedAvailabilityDate?: string;
    PlacementDate?: string;
    DisplayID?: string;
  }

  export interface OrderSearchResultItemUser {
    /**
     * Entity type: User
     */
    ID: number;
    FullName: string;
  }

  export interface OrderSearchResultItemCustomer {
    EmailAddress: string;
    /**
     * Entity type: CustomField
     */
    CustomFieldValues: { [key: number]: CustomFieldValue };
    /**
     * Entity type: User
     */
    ID: number;
    FullName: string;
  }

  export interface OrderSearchResultItemOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    Name: string;
  }

  export interface OrderSearchResultItemAddress {
    CountryID: string;
  }

  export interface OrderSearchResultItemData {
    Name: string;
    StringValue: string;
    DecimalValue?: number;
    BooleanValue?: boolean;
    DateValue?: string;
  }

  export interface OrderSearchResultItemDefinition {
    Status: OrderSearchResultItemDefinition.Value;
    Type: OrderSearchResultItemDefinition.Value;
  }

  export namespace OrderSearchResultItemDefinition {
    export interface Value {
      BackendID: string;
      Title: string;
    }

  }

  export interface OrderSearchResultItemReturnToSupplierRequest {
    ID: number;
    Name: string;
  }

  /**
   * Update the serial number on an order line - when no serialnumber is provided, it will be removed.
   */
  export interface UpdateSerialNumber {
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    SerialNumber?: string;
  }

  /**
   * Sets the first serial number on given order line, and splits off new order lines for each additional serial number given.
   */
  export interface SetSerialNumbers {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    SerialNumbers: string[];
  }

  /**
   * This service can be used when in local mode to set some UserIdentifiers on an Order
   * 
   * If ParseBarcode responds with ParsedButNotFound, this service can be used to set the UserIdentifiers on the Order.
   * Or an EmailAddress can be manually set.
   * 
   * When the order is exported to the cloud, the UserIdentifiers will be used to match the order to the correct user.
   */
  export interface SetCustomerIdentifiersOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    UserID?: string;
    UserSubscriptionID?: string;
    UserBackendID?: string;
    UserCustomID?: string;
    EmailAddress?: string;
  }

  export interface SetCustomerReferencesOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    CustomerReference?: string;
    CustomerOrderID?: string;
  }

  /**
   * Sets `BackendID`/`BackendSystemID` on specified `OrderID`.
   * 
   * This combination may not already exist, and the targeted order may not be completed, or  already have a `BackendID`/`BackendSystemID` set.
   * 
   * :warning: Careful! Use at your own risk!
   * 
   * An order may already have any of the following things, which will not automatically be regenerated/resent because of this identifier change, and could
   * result in mapping issues on any external system listening in on these. Any of these would also apply to return orders referencing this order, or stock
   * mutations that have been caused by this order.
   * 
   * * Order exports to third parties
   * * Event exports (order created, order updated, ...)
   * * Data Lake exports
   * * Shipment labels with third party carriers
   * * Financial exports (Cookbook)
   * * Created documents (receipts, invoices, ...)
   */
  export interface SetOrderBackendIdentifier {
    /**
     * The EVA order identifier
     * Entity type: Order
     */
    OrderID: number;
    /**
     * A code, free of choice for the calling 3rd party application creating the order
     */
    BackendSystemID: string;
    /**
     * A unique identifier for the order, within the context of given `BackendSystemID`
     */
    BackendID: string;
  }

  /**
   * Sets the `SoldBy` of the specified sales order to the provided `UserID`.
   * 
   * - The user must have type `Employee`
   * - This service can only be called on orders that have not been invoiced yet
   * - As `SoldBy` is a line-level property this service will update all lines on the order that have not been invoiced yet
   */
  export interface SetOrderSoldBy {
    /**
     * The order to set the `SoldBy` for - must be of type `Sales`.
     * Entity type: Order
     */
    OrderID: number;
    /**
     * The user that will become the `SoldBy`- must be of type `Employee`.
     * Entity type: User
     */
    UserID: number;
    /**
     * If specified, sets the `SoldBy` only for the given order lines.
     * 
     * :warning: If `SetOrderSoldBy` is called again for the whole order (so without specifying `OrderLineIDs`) this value is overwritten
     * :notebook: Newly created order lines will not automatically inherit `SoldBy` set on invidivual order lines
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
  }

  export interface SetPickupOrganizationUnit {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface SetRemarkOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Remark?: string;
  }

  /**
   * Set the `RequestedDate` on OrderLines.
   * The RequestedDate is the date on which the OrderLine should be ready for shipping/pickup/etc.
   * 
   * It is required to provide an `OrderID`.
   * If the RequestedDate is null, it will be removed from the lines.
   */
  export interface SetRequestedDate {
    /**
     * Entity type: Order
     */
    OrderID: number;
    OrderLines?: SetRequestedDate.OrderLineDto[];
    RequestedDate?: string;
  }

  export namespace SetRequestedDate {
    export interface OrderLineDto {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      Quantity?: number;
      RequestedDate?: string;
    }

  }

  /**
   * Sets the shipping method for the given order(lines). Use `ListAvailableShippingMethods` to request the shipping methods that can be used as input to this service.
   */
  export interface SetShippingMethod {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
    /**
     * This date can be used to request a specific delivery date. If null, it will be sent at the first available date.
     */
    RequestedDeliveryDate?: string;
  }

  /**
   * Ships an order that has been exported to an external system by another proces
   */
  export interface ShipExternalOrder {
    /**
     * ID of the order that is being shipped - required when `BackendID` and `BackendSystemID` are omitted
     * Entity type: Order
     */
    ID?: number;
    /**
     * Reference to the order that is being shipped - required when `ID` is omitted
     */
    BackendID?: string;
    /**
     * Reference to the system from which the order originated - required when `ID` is omitted
     */
    BackendSystemID?: string;
    Shipments: ShipExternalOrder.Shipment[];
    /**
     * Indicates if this is the last shipment - when set to true will cancel any remaining lines that are not shipped
     */
    FinalShipment?: boolean;
  }

  export namespace ShipExternalOrder {
    export interface Shipment {
      /**
       * Reference number of the shipment
       */
      BackendID: string;
      /**
       * Identifier of the system that created the shipment
       */
      BackendSystemID: string;
      /**
       * Optional tracking code that is stored on the shipment.
       */
      TrackingCode?: string;
      /**
       * Optional tracking link that is stored on the shipment
       */
      TrackingLink?: string;
      /**
       * Entity type: Package
       */
      PackageID?: number;
      Lines?: ShipExternalOrder.ShipmentLine[];
      Packages?: ShipExternalOrder.ShipmentPackage[];
      /**
       * Optional additional custom fields
       */
      CustomFields?: CustomFieldKeyValue[];
      /**
       * Create shipment as being shipped from this OU, which affects which stock is mutated and the financial consequences of that
       * Entity type: OrganizationUnit
       */
      ShipFromOrganizationUnitID?: number;
    }

    export interface ShipmentLine {
      /**
       * Used to identify the order line - required if BackendID and Barcode are omitted
       * Entity type: OrderLine
       */
      ID?: number;
      /**
       * Used to identify the order line - required if ID and Barcode are omitted
       */
      BackendID?: string;
      /**
       * Used to identify the order line - required if ID and BackendID are omitted
       */
      Barcode?: string;
      /**
       * Number of products that have been shipped
       */
      Quantity: number;
      /**
       * In case of serial number products (like gift cards), this field is required and indicates the shipped serial number
       */
      SerialNumber?: string;
      /**
       * In case of shipping multiple serial numbers at once
       */
      SerialNumbers?: string[];
    }

    export interface ShipmentPackage {
      /**
       * Entity type: Package
       */
      PackageID: number;
      Lines: ShipExternalOrder.ShipmentLine[];
    }

  }

  export interface ShipExternalOrderResponse {
    /**
     * Entity type: Shipment
     */
    ShipmentIDs: number[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ShipOrderLines {
    ShipmentBackendID?: string;
    TrackingCode?: string;
    ShipmentDate: string;
    Lines: ShipOrderLines.Line[];
    /**
     * The OrganizationUnit the shipment was physically shipped from.
     * Entity type: OrganizationUnit
     */
    ShippedFromOrganizationUnitID?: number;
  }

  export namespace ShipOrderLines {
    export interface Line {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      Quantity: number;
      BackendReference: string;
    }

  }

  export interface ShipOrderLinesResponse {
    ShipmentID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Ships the specified order. When Lines are specified, only those lines will be shipped.
   * 
   * Shipping of Lines includes:
   * - Activating giftcards
   * - Mutating the stock of the products
   * - Potentially invoicing of the lines
   * - Validating requirements
   */
  export interface ShipOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Lines?: ShipOrder.ShipmentOrderLine[];
    /**
     * Use with caution.
     * Set this to true if you want to skip requirements that would otherwise cause validation errors. Like shipping delivery lines, which you usually don't want from a shop.
     */
    Force?: boolean;
    FinalShipment?: boolean;
    /**
     * The OrganizationUnit the shipment was physically shipped from.
     * Entity type: OrganizationUnit
     */
    ShippedFromOrganizationUnitID?: number;
  }

  export namespace ShipOrder {
    export interface ShipmentOrderLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      QuantityShipped?: number;
      ShipmentBackendID?: string;
      TrackingCode?: string;
      TrackingLink?: string;
    }

  }

  export interface ShipOrderResponse {
    /**
     * Entity type: Shipment
     */
    ShipmentIDs: number[];
    /**
     * Entity type: OrderLine
     */
    ShippedOrderLineIDs: number[];
    Messages: ShipmentResultMessage[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ShipmentResultMessage {
    Code: string;
    Message: string;
  }

  /**
   * Split an OrderLine in two lines.
   * 
   * The new line will have a QuantityOrdered of QuantityToSplit and on the existing line there will be QuantityToShip items cancelled.
   * If the OrderLine has a negative QuantityToShip, the QuantityOrdered of the new line will be -QuantityToSplit.
   * 
   * The QuantityToSplit should always be greather than 0 and smaller than the absolute value of the QuantityToShip of the OrderLine.
   */
  export interface SplitOrderLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    QuantityToSplit: number;
  }

  export interface SplitOrderLineResponse {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Marks the provided Order or OrderLine as exempt from tax, with specified code and reason.
   * 
   * When exempting the Order, this also marks all current OrderLines of the Order that haven't been invoiced yet as tax-exempt. All future
   * OrderLines created on this Order will then also automatically be made tax-exempt.
   * 
   * Calling this service is irreversible; once the Order is marked as tax exempt, it cannot be undone and can only be worked around by creating a new Order.
   */
  export interface ExemptOrderFromTax {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * A code identifying the tax exemption.
     */
    TaxExemptionCode?: string;
    /**
     * A longer description with the justification for the tax exemption.
     */
    TaxExemptionReason: string;
    /**
     * This verification number is a confirmation of the tax exemption reason.
     */
    TaxExemptionVerification?: string;
  }

  /**
   * Will return the estimated tax for specified order, but does not persist through the order amounts retrieved elsewhere.
   */
  export interface GetSalesTaxEstimateForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface SalesTaxEstimateForOrderResponse {
    IsSuccess: boolean;
    TotalTaxAmount: number;
    TaxTable: SalesTaxEstimateForOrderResponse.OrderLineTaxCalculationTaxAmountModel[];
    OrderLines: SalesTaxEstimateForOrderResponse.OrderLineTaxCalculationModel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace SalesTaxEstimateForOrderResponse {
    export interface OrderLineTaxCalculationTaxAmountModel {
      TaxRate: number;
      TaxAmount: number;
    }

    export interface OrderLineTaxCalculationModel {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      TaxRate: number;
      TaxAmount: number;
    }

  }

  /**
   * Will apply and return the estimated tax for specified order. Its effect is persistent through all subsequent calls for this orders' amounts.
   */
  export interface ApplySalesTaxEstimateForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Transfers the provided Order to the OrganizationUnit of the currently logged in user, updating both the SoldFromOrganizationUnit and PickupOrganizationUnit.
   * 
   * If this is already the case, calling this service has no effect. In case the order cannot be transferred, this service will return an error.
   */
  export interface TransferOrderToOrganizationUnit {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Get additional order line information intended for transport data.
   */
  export interface GetTransportOrderLineData {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
  }

  export interface GetTransportOrderLineDataResponse {
    TransportOrderLineData: TransportOrderLineData;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface TransportOrderLineData {
    TransportID: string;
    Priority?: number;
    PlannedDate?: string;
    Timeslot: string;
  }

  /**
   * Set additional order line information intended for transport data. Supports a single `OrderLineID`, or a list of `OrderLineIDs`.
   */
  export interface SetTransportOrderLineData {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
    TransportOrderLineData?: TransportOrderLineData;
  }

  export interface UpdateContractNumber {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    ContractNumber?: string;
  }

  export interface GetContractNumber {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
  }

  export interface GetContractNumberResponse {
    ContractNumber: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateOrderAddresses {
    /**
     * Entity type: Order
     */
    OrderID: number;
    CopyShippingAddressFromCustomer?: boolean;
    CopyBillingAddressFromCustomer?: boolean;
    ShipToAddress?: AddressDataDto;
    BillToAddress?: AddressDataDto;
    PickupAddress?: AddressDataDto;
  }

  export interface UpdateOrderAddressesResponse {
    CreditcardAllowed: boolean;
    ShipToAddress: AddressDto;
    BillToAddress: AddressDto;
    PickupAddress: AddressDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update the ShippingAddress on an Order.
   * 
   * This can be done by providing a full Address in the Address property, or by referencing an Address in an AddressBook in the AddressBookID property.
   */
  export interface UpdateOrderShippingAddress {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Address?: AddressDataDto;
    /**
     * Entity type: AddressBook
     */
    AddressBookID?: number;
  }

  export interface UpdateOrderAddressResponse {
    ShippingAddress?: AddressDto;
    BillingAddress?: AddressDto;
    PickupAddress?: AddressDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update the BillingAddress on an Order.
   * 
   * This can be done by providing a full Address in the Address property, or by referencing an Address in an AddressBook in the AddressBookID property.
   */
  export interface UpdateOrderBillingAddress {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Address?: AddressDataDto;
    /**
     * Entity type: AddressBook
     */
    AddressBookID?: number;
  }

  /**
   * Update the PickupAddress on an Order.
   * 
   * This can be done by providing a full Address in the Address property, or by referencing an Address in an AddressBook in the AddressBookID property.
   */
  export interface UpdateOrderPickupAddress {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Address?: AddressDataDto;
    /**
     * Entity type: AddressBook
     */
    AddressBookID?: number;
  }

  /**
   * Update the data for the checkout options returned in the `AdditionalData` of a `GetOrder` or `GetShoppingCart` call
   */
  export interface UpdateOrderCheckoutOptions {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Options: UpdateOrderCheckoutOptions.UpdateOrderCheckoutOption[];
  }

  export namespace UpdateOrderCheckoutOptions {
    export interface UpdateOrderCheckoutOption {
      /**
       * Entity type: CheckoutOption
       */
      ID: number;
      ValueData?: { [key: string]: any };
    }

  }

  export interface UpdateOrderCurrency {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  /**
   * Create or update existing custom field on order
   */
  export interface UpdateOrderCustomFields {
    /**
     * Entity type: Order
     */
    OrderID: number;
    CustomFields?: CustomFieldKeyValue[];
    CustomFieldsByBackendID?: CustomFieldByBackendValue[];
  }

  export interface CustomFieldByBackendValue {
    BackendID: string;
    /**
     * Used for datatype String, Text and Enum
     */
    StringValue?: string;
    /**
     * Used for datatype Blob
     */
    BlobValue?: string;
    /**
     * Used for datatype Integer and Decimal
     */
    NumberValue?: number;
    /**
     * Used for datatype Bool
     */
    BoolValue?: boolean;
    /**
     * Used for datatype DateTime and Date
     */
    DateTimeValue?: string;
    /**
     * Used for multiple values
     */
    ArrayValues?: CustomFieldValue[];
  }

  /**
   * Create or update existing custom field on order line level by ID
   */
  export interface UpdateOrderLineCustomFields {
    /**
     * Entity type: OrderLine
     */
    ID: number;
    CustomFields: CustomFieldKeyValue[];
  }

  /**
   * Update the StockLabel for a single `OrderLine` or for the complete `Order`. It is required to supply an `OrderID` or an `OrderLineID`.
   * If one of the lines cannot be updated, it will be skipped.
   * 
   * Only unshipped orderlines with LineActionType Ship can be updated.
   */
  export interface UpdateOrderLineStockLabel {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
  }

  /**
   * This services can be used to toggle some of the OrderOptions.
   * 
   * - IgnoreDiscounts: Discounts will not be added/removed/recalculated
   * - IgnoreShippingCosts: ShippingCosts will not be added/removed/recalculated
   * - IgnoreGiftWrappingCosts: GiftWrappingCosts will not be added/removed/recalculated
   * - AllowPartialFulfillment: The order will potentially be fulfilled in multiple batches
   */
  export interface UpdateOrderOptions {
    /**
     * Entity type: Order
     */
    OrderID: number;
    IgnoreDiscounts?: boolean;
    IgnoreShippingCosts?: boolean;
    IgnoreGiftWrappingCosts?: boolean;
    AllowPartialFulfillment?: boolean;
  }

  /**
   * Update various fields on an Order.
   */
  export interface UpdateOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    AutomaticCancellationDate?: string;
    /**
     * Entity type: CustomOrderType
     */
    CustomOrderTypeID?: number;
    /**
     * Entity type: OrderCustomType
     */
    OrderCustomTypeID?: number;
    /**
     * Entity type: OrderCustomStatus
     */
    OrderCustomStatusID?: number;
  }

  /**
   * Updates the order shipping status, creating ledger entries on order level.
   * 
   * Will be processed as a whole; a single failing line in `Lines` will prevent all other updates in a single call from persisting.
   */
  export interface UpdateOrderShippingStatus {
    Lines: UpdateOrderShippingStatusLine[];
  }

  export interface UpdateOrderShippingStatusLine {
    /**
     * Identifier of the EVA order this update is specific for
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Unique identifier for the order, in respect to the `BackendSystemID`
     */
    BackendOrderID?: string;
    /**
     * Owner of the `BackendOrderID`
     */
    BackendSystemID?: string;
    /**
     * Type of order shipping status update; 0 = Picked, 1 = Packed, 2 = CarrierStatus
     */
    Type: OrderShippingStatusType;
    /**
     * Identifier of the EVA order line this update is specific for
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * BackendID of the EVA order line this update is specific for
     */
    BackendOrderLineID?: string;
    /**
     * Date the event occurred - leave empty for 'now'.
     */
    Date?: string;
    /**
     * Optional, additional information, most commonly for the carrier status update code (B01, D01, ...) for CarrierStatus types
     */
    Value?: string;
    /**
     * Optional, additional description, most commonly for the carrier status update code description for CarrierStatus types
     */
    Description?: string;
  }

  export const enum OrderShippingStatusType {
    Picked = 0,
    Packed = 1,
    CarrierStatus = 2
  }

  /**
   * Validate if all conditions are met to schedule the (specified lines of) specified order.
   * 
   * _Will check the default ExportValidator and check OrderRequirements for all actions._
   */
  export interface ValidateExportSchedule {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineIDs?: number[];
  }

  export interface ValidateExportScheduleResponse {
    ValidationResult: OrderExportValidationResult;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Validates if a (part of an) Order can be shipped in the current state.
   * 
   * Returns if it is valid (`IsValid`) and if not, how much still needs to be paid (`OpenAmountForShipment`).
   */
  export interface ValidateOrderShipment {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Lines?: ValidateOrderShipment.ShipmentOrderLine[];
  }

  export namespace ValidateOrderShipment {
    export interface ShipmentOrderLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      QuantityShipped?: number;
    }

  }

  export interface ValidateOrderShipmentResponse {
    PaidAmount: number;
    OpenAmountForShipment: number;
    IsValid: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Validate the required data for the order
   */
  export interface ValidateRequiredDataForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    RequiredFor?: RequiredFor;
  }

  export interface ValidateRequiredDataForOrderResponse {
    ValidatedData: RequiredDataValidationResult;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface RequiredDataValidationResult {
    ValidationErrors: ValidationError[];
  }

  export interface ValidationError {
    Field: string;
    Message: string;
    Context: string;
  }

  /**
   * Verify the customer of an order by a QR code check. This process is used to validate that the person that picks up an order is really the person that ordered it.
   */
  export interface VerifyCustomer {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Retrieved from the `ParseBarcode` result `VerificationCode`
     */
    VerificationCode: string;
  }

  /**
   * Verify the customer of an order without QR code check.
   */
  export interface ForceVerifyCustomer {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetCustomerOrderVerificationCode {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetVerificationCodeResponse {
    VerificationCode: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetEmployeeOrderVerificationCode {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Verify an Order that has been flagged with a CustomOrderStatus `NeedsEmployeeVerification`. Will try to place the order after the verification.
   */
  export interface VerifyOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Get additional order information intended for warehouse data.
   */
  export interface GetWarehouseOrderData {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetWarehouseOrderDataResponse {
    WarehouseOrderData: WarehouseOrderData;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface WarehouseOrderData {
    AllowPartialPick: boolean;
  }

  /**
   * Set additional order information intended for warehouse data.
   */
  export interface SetWarehouseOrderData {
    /**
     * Entity type: Order
     */
    OrderID: number;
    WarehouseOrderData?: WarehouseOrderData;
  }

  /**
   * Gets the available reasons for tax exemption.
   * The `Code` property of each result can then be supplied to `ExemptOrderFromTax` as the `TaxExemptionCode`.
   */
  export interface GetTaxExemptionReasons {
    /**
     * If you have an OrderID available it can be specified to get a more accurate list of tax exemption reasons that is independent of your current OrganizationUnit.
     * For example if you have a US order but you are currently logged into a non-US organization, the result of this service won't be accurate unless you specify the OrderID.
     * Entity type: Order
     */
    OrderID?: number;
  }

  export interface GetTaxExemptionReasonsResponse {
    Reasons: GetTaxExemptionReasonsResponse.TaxExemptionReason[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetTaxExemptionReasonsResponse {
    export interface TaxExemptionReason {
      Code: string;
      Name: string;
      Description: string;
      RequiresVerification: boolean;
    }

  }

  /**
   * Get the signature for an Order
   */
  export interface GetOrderSignature {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Type?: SignatureTypes;
  }

  export const enum SignatureTypes {
    Untyped = 0,
    Shipping = 1,
    Scripted = 2,
    AMLEmployee = 3,
    AMLCustomer = 4
  }

  /**
   * Add a signature to an Order.
   */
  export interface SignOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Type?: SignatureTypes;
    Signature: string;
    MimeType: string;
  }

  /**
   * Convert the given `Order` to a ReturnToSupplierOrder. Existing none-product lines will be cancelled.
   */
  export interface CreateReturnToSupplierOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Create a new ReturnToSupplierRequest
   * 
   * A ReturnToSupplierRequest is used to generate ReturnToSupplier orders for Shops
   * 
   * - Name: For identification of the request (required)
   * - OrganizationUnitSetID: The organizationunits (filtered on Type = Shop) that will get a RTS (required)
   * - ProductSearchTemplateID: Only products matching this template will be added to the orders, if supplied (optional)
   * - RecallPercentage: Percentage of the current stock to recall. Value should be between 0.00 and 1.00. (required)
   * - Filters, CountryID, LanguageID: Instead of a productsearchtemplate, products can also be defined by providing a manual filter (all optional)
   */
  export interface CreateReturnToSupplierRequest {
    Name?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    /**
     * Range: 0.01-1
     */
    RecallPercentage?: number;
    Filters?: { [key: string]: FilterModel };
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * ReturnReason of type OrganizationUnit to indicate why these ReturnToSupplier orders are created
     * Entity type: ReturnReason
     */
    ReturnReasonID?: number;
    Remark?: string;
  }

  export interface CreateReturnToSupplierRequestResponse {
    /**
     * Entity type: ReturnToSupplierRequest
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Delete a ReturnToSupplierRequest
   * 
   * This is only allowed for Request in the state New or Error
   */
  export interface DeleteReturnToSupplierRequest {
    /**
     * Entity type: ReturnToSupplierRequest
     */
    ID: number;
  }

  export interface GetReturnToSupplierData {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetReturnToSupplierDataResponse {
    Reason: EnumDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get a ReturnToSupplierRequest
   */
  export interface GetReturnToSupplierRequestByID {
    /**
     * Entity type: ReturnToSupplierRequest
     */
    ID: number;
  }

  export interface GetReturnToSupplierRequestByIDResponse {
    Name: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    OrganizationUnitSetName: string;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    ProductSearchTemplateName: string;
    RecallPercentage: number;
    Filters?: { [key: string]: FilterModel };
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Status: ReturnToSupplierRequestStatus;
    /**
     * Entity type: ReturnReason
     */
    ReturnReasonID?: number;
    ReturnReasonName?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum ReturnToSupplierRequestStatus {
    New = 0,
    Processing = 1,
    Processed = 2,
    Error = 9
  }

  export interface ListReturnableSuppliersForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ListReturnableSuppliersForOrderResponse {
    Result: ListReturnableSuppliersForOrderResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListReturnableSuppliersForOrderResponse {
    export interface Model {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      BackendID: string;
      Name: string;
    }

  }

  /**
   * List ReturnToSupplierRequests
   * 
   * Available filters:
   * - Name
   * - Status
   */
  export interface ListReturnToSupplierRequests {
    PageConfig?: PageConfig<ListReturnToSupplierRequestsFilter>;
  }

  export interface ListReturnToSupplierRequestsFilter {
    Name?: string;
    /**
     * This will only match requests that have the exact specified organization unit set.
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Status?: ReturnToSupplierRequestStatus;
  }

  export interface ListReturnToSupplierRequestsResponse {
    Result?: PagedResult<ReturnToSupplierRequestModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ReturnToSupplierRequestModel {
    /**
     * Entity type: ReturnToSupplierRequest
     */
    ID: number;
    Name: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    OrganizationUnitSetName: string;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    ProductSearchTemplateName: string;
    RecallPercentage: number;
    Filters?: { [key: string]: FilterModel };
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Status: ReturnToSupplierRequestStatus;
  }

  /**
   * Print the return to supplier order label.
   */
  export interface PrintReturnToSupplierOrderTicket {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * Print the return to supplier order receipt.
   */
  export interface PrintReturnToSupplierReceipt {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * Start the processing of a ReturnToSupplierRequest
   * 
   * This will put the request in a processing state and will generate the orders based on the set options.
   */
  export interface ProcessReturnToSupplierRequest {
    /**
     * Entity type: ReturnToSupplierRequest
     */
    ID: number;
  }

  /**
   * Set the suppliers for OrganizationUnit where this could not be automatically determined
   * 
   * To view the available suppliers, sue the ValidateReturnToSupplierRequest service.
   * 
   * The SupplierOrganizationUnitID is nullable, if it is set to `null`, the OrganizationUnit will be skipped when generating the orders.
   */
  export interface SetSuppliersForReturnToSupplierRequest {
    /**
     * Entity type: ReturnToSupplierRequest
     */
    ID: number;
    Suppliers: SupplierSet[];
  }

  export interface SupplierSet {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
  }

  /**
   * Update the return to supplier order with Reason, ShipFrom- and ShipTo organization unit
   */
  export interface UpdateReturnToSupplierOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: ReturnReason
     */
    ReasonID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID?: number;
  }

  /**
   * Update a ReturnToSupplierRequest
   * 
   * This is only allowed for requests in the state New
   * 
   * Properties that are not provided in the request, will not be updated.
   */
  export interface UpdateReturnToSupplierRequest {
    /**
     * Entity type: ReturnToSupplierRequest
     */
    ID: number;
    Name?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    RecallPercentage?: number;
    Filters?: { [key: string]: FilterModel };
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * ReturnReason of type OrganizationUnit to indicate why these ReturnToSupplier orders are created
     * Entity type: ReturnReason
     */
    ReturnReasonID?: number;
    Remark?: string;
  }

  /**
   * Validate a ReturnToSupplierRequest
   * 
   * This will check if for all OrganizationUnits in the set a Supplier could be determined.
   * 
   * The service will return all available suppliers for each OrganizationUnit and will pick it when a single one is available.
   * When no Suppliers are available, no Order can be generated.
   * If more than one Supplier is available, the correct one should be picked with the SetSuppliersForReturnToSupplierRequest service.
   */
  export interface ValidateReturnToSupplierRequest {
    /**
     * Entity type: ReturnToSupplierRequest
     */
    ID: number;
  }

  export interface ValidateReturnToSupplierRequestResponse {
    OrganizationUnits: OrganizationUnitValidationModel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrganizationUnitValidationModel {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    OrganizationUnitBackendID: string;
    PickedSupplier?: SupplierValidationModel;
    Skipped: boolean;
    AvailableSuppliers: SupplierValidationModel[];
  }

  export interface SupplierValidationModel {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    OrganizationUnitBackendID: string;
  }

  /**
   * Get all refund reasons available in the current context
   */
  export interface GetRefundCorrectionReasons {
  }

  export interface GetRefundCorrectionReasonsResponse {
    Reasons: GetRefundCorrectionReasonsResponse.RefundCorrectionReasonDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetRefundCorrectionReasonsResponse {
    export interface RefundCorrectionReasonDto {
      /**
       * Entity type: RefundCorrectionReason
       */
      ID?: number;
      Name?: string;
      Description?: string;
      BackendID?: string;
    }

  }

  /**
   * Automatically places, ships and receives an interbranch order between 2 organization units.
   */
  export interface AutoTransferInterbranchOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID: number;
  }

  /**
   * Convert the given `Order` to a InterbranchOrder.
   */
  export interface CreateInterbranchOrder {
    /**
     * The Order to convert to an interbranch order.
     * Entity type: Order
     */
    OrderID: number;
    /**
     * The destination OrganizationUnit of this Interbranch order. This will become the SoldToOrganizationUnit of the order.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * The date when the shipment is expected in the other store
     */
    ExpectedDeliveryDate?: string;
  }

  /**
   * Print the InterbranchOrder receipt.
   */
  export interface PrintInterbranchOrderReceipt {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * Lists the OrganizationUnits that are available for the given OrderID to be used as the target for an interbranch order.
   */
  export interface ListInterbranchOrganizationUnitForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    ExcludeClosed?: boolean;
  }

  export interface ListInterbranchOrganizationUnitForOrderResponse {
    OrganizationUnits: InterbranchOrganizationUnit[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface InterbranchOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    Name: string;
  }

  /**
   * Mutate Interbranch properties on the given `Order`.
   */
  export interface UpdateInterbranchOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * The date when the shipment is expected in the other store
     */
    ExpectedDeliveryDate?: string;
  }

  /**
   * Set the GiftWrapping options on an Order.
   * 
   * There are 4 options:
   * 
   *  - Wrap the complete order: set `WrapOrder` to `true`
   *  - Wrap the complete order individually: set `WrapIndividually` to `true`
   *  - Wrap parts of the order: set `GiftWraps` with a set of wraps
   *  - Wrap nothing: set everything to `null`
   * 
   * A `GiftWrap` consists out of an optional `Message` (could be printed on a card), an optional `GreetingCardProductID` and a set of `WrapLines`. A
   * `WrapLine` has an `OrderLineID` and a `Quantity`.
   * 
   * If a `GreetingCardProductID` and/or `Message` is desired when the order should be completely wrapped, or without any giftwrapping at all, set these
   * properties on root level. In all other cases they should be set on the `GiftWraps`.
   */
  export interface SetGiftWrappingOptionsOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    WrapOrder?: boolean;
    WrapIndividually?: boolean;
    GiftWraps?: GiftWrapDto[];
    /**
     * Entity type: Product
     */
    GreetingCardProductID?: number;
    Message?: string;
  }

  /**
   * Get the GiftWrapping options for an Order.
   */
  export interface GetGiftWrappingOptionsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetGiftWrappingOptionsForOrderResponse {
    WrapOrder: boolean;
    WrapIndividually: boolean;
    GiftWraps: GiftWrapDto[];
    GreetingCardProductID?: number;
    Message: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get the available GiftWrapping options for an Order.
   */
  export interface GetAvailableGiftWrappingLinesForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetAvailableGiftWrappingLinesForOrderResponse {
    Lines: GiftWrappingLineAvailabilityDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This service is used only for downloading invoices.
   */
  export interface DownloadInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
  }

  /**
   * This service is used only for downloading offers.
   */
  export interface DownloadOffer {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * This service is used only for printing paper invoices.
   */
  export interface DownloadOrderConfirmation {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface DownloadOrderConfirmationResponse {
    Url: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This service is used only for emailing invoices.
   */
  export interface EmailInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    EmailAddress?: string;
  }

  /**
   * This service is used for emailing all invoices linked to an order ID.
   */
  export interface EmailInvoices {
    /**
     * Entity type: Order
     */
    OrderID: number;
    EmailAddress?: string;
  }

  /**
   * This service is used only for emailing offers.
   */
  export interface EmailOffer {
    /**
     * Entity type: Order
     */
    OrderID: number;
    EmailAddress: string;
  }

  /**
   * This service is used only for printing paper invoices.
   */
  export interface EmailOrderConfirmation {
    /**
     * Entity type: Order
     */
    OrderID: number;
    EmailAddress?: string;
  }

  /**
   * This service is used to retrieve the TaxFreeForms associated with an order & send them to the specified email address.
   */
  export interface EmailOrderTaxFreeForm {
    /**
     * Entity type: Order
     */
    OrderID: number;
    EmailAddress?: string;
  }

  /**
   * This service is used only for emailing thermal invoices
   */
  export interface EmailThermalInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    EmailAddress: string;
  }

  /**
   * This service is used only for emailing thermal invoices based on an order ID.
   */
  export interface EmailThermalInvoices {
    /**
     * Entity type: Order
     */
    OrderID: number;
    EmailAddress: string;
  }

  /**
   * Returns a list of available outputs for an order by taking an `OrderID`.
   */
  export interface GetAvailableDocumentsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetAvailableDocumentsForOrderResponse {
    /**
     * List of order document outputs available for a specific order.
     * Each document contains information regarding the following categories:
     * 
     * `Receipt`           - physically printing the order invoice
     * `Paper`             - printing the order invoice on a paper printer
     * `Email`             - sending the order invoice by email
     * `Digital`           - providing a downloadable version of the order invoice
     * `ElectronicReceipt` - providing an electronic receipt version of the order invoice
     * `Metadata`          - providing information about the order invoice.
     * 
     * The document category contains information on whether it's available for the order
     * and a reason code in case it's not available.
     */
    Documents: GetAvailableDocumentsForOrderResponse.OrderDocument[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAvailableDocumentsForOrderResponse {
    export interface DocumentStation {
      /**
       * ID of the Station in EVA. Can be used directly in services
       */
      ID: number;
      /**
       * Name of the station for display purposes
       */
      Name: string;
    }

    export interface DocumentOutput {
      /**
       * Indicates if this output should be preferred by the frontend. Can be used to, for example, set a checkbox or radio button by default so the user does not have to toggle it themselves
       */
      Preferred?: boolean;
      /**
       * Allowed is false when this option cannot be used, true otherwise
       */
      Allowed?: boolean;
      /**
       * Optional reason code that is included if Allowed is set to false
       */
      ReasonCode?: string;
      /**
       * Possbile stations where the document can be output
       */
      Stations?: GetAvailableDocumentsForOrderResponse.DocumentStation[];
    }

    export interface OrderDocument {
      Type: string;
      Email: GetAvailableDocumentsForOrderResponse.DocumentOutput;
      Receipt: GetAvailableDocumentsForOrderResponse.DocumentOutput;
      Digital: GetAvailableDocumentsForOrderResponse.DocumentOutput;
      Paper: GetAvailableDocumentsForOrderResponse.DocumentOutput;
      ElectronicReceipt: GetAvailableDocumentsForOrderResponse.DocumentOutput;
      Metadata: { [key: string]: any };
    }

  }

  /**
   * This is service is used only for printing couopons.
   */
  export interface PrintCouponsForOrder {
    /**
     * Entity type: Station
     */
    StationID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * This service is used only for printing paper invoices.
   */
  export interface PrintPaperInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * This is service is used for printing sales paper invoices for one order.
   */
  export interface PrintPaperInvoices {
    /**
     * Entity type: Station
     */
    StationID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * This service is used only for printing offers.
   */
  export interface PrintPaperOffer {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  /**
   * This service is used only for printing paper invoices.
   */
  export interface PrintPaperOrderConfirmation {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * Print a *gift* receipt; a regular receipt, but without any price- or payment details. This document is intended to accompany the purchase when gifted, so the recipient can return or exchange it without (direct) knowledge of the price.
   *     The `Lines` property is optional, when omitted the entire order (from `OrderID`) will be printed. Also, `Name` and `Remark` are optional and intended for the gift recipient, only.
   */
  export interface PrintThermalGiftReceipt {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
    Lines: PrintThermalGiftReceiptLine[];
    Name?: string;
    Remark?: string;
  }

  export interface PrintThermalGiftReceiptLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    Quantity: number;
  }

  /**
   * This is service is used only for printing thermal invoices.
   */
  export interface PrintThermalInvoice {
    /**
     * Entity type: Station
     */
    StationID: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
  }

  /**
   * This is service is used for printing sales thermal invoices for one order.
   */
  export interface PrintThermalInvoices {
    /**
     * Entity type: Station
     */
    StationID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface PrintThermalOrderConfirmation {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * Print an order summary as a thermal non-fiscal receipt.
   * It can be used when a payment is refunded or canceled during the sale process, the cashier needs to hand a proof to the customer with the current state of the order.
   */
  export interface PrintThermalOrderSummary {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  export interface ProduceDocuments {
    PrintInvoice?: boolean;
    EmailInvoice?: boolean;
    PrintReceipt?: boolean;
    DownloadInvoice?: boolean;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    EmailAddress?: string;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
  }

  export interface ProduceDocumentsResponse {
    InvoiceUrl?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ProduceInvoice {
    /**
     * Entity type: Order
     */
    OrderID: number;
    EmailAddress?: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
    EmailInvoice?: boolean;
    DownloadInvoice?: boolean;
    PrintInvoice?: boolean;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
  }

  export interface ProducePurchaseOrderDocument {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ProduceReceipt {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
  }

  export interface ProduceReceiptResponse {
    PrintSuccessful: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Generate shipment document for an order or shipment.
   * 
   * It is required to supply an `OrderID` or a `ShipmentID`.
   * 
   * When using an `OrderID` it is also required to set the `LinesToShip`.
   * 
   * When a `StationID` is supplied the documents will be printed, if not, the service will return urls for downloading the documents.
   */
  export interface ProduceShipmentDocuments {
    /**
     * Required when omitting `ShipmentID`.
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Required when omitting `OrderID`.
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * Required when passing an `OrderID`.
     */
    LinesToShip?: ProduceShipmentDocuments.OrderLineDto[];
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  export namespace ProduceShipmentDocuments {
    export interface OrderLineDto {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      QuantityToShip: number;
    }

  }

  export interface ProduceShipmentDocumentsResponse {
    ShipmentDocuments: ProduceShipmentDocumentsResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ProduceShipmentDocumentsResponse {
    export interface Model {
      Type: string;
      ID: string;
      OriginalName: string;
      MimeType: string;
      Category: string;
      ExpireDate?: string;
      LocationType: string;
      Url: string;
    }

  }

  /**
   * Creates a custom order status which can be set on an order
   */
  export interface CreateCustomOrderStatus {
    Name: string;
    Description?: string;
  }

  export interface CreateCustomOrderStatusResponse {
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Deletes a custom order status
   */
  export interface DeleteCustomOrderStatus {
    /**
     * Entity type: CustomOrderStatus
     */
    ID: number;
  }

  export interface ListCustomOrderStatus {
  }

  export interface ListCustomOrderStatusResponse {
    Result: ListCustomOrderStatusResponse.CustomOrderStatus[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListCustomOrderStatusResponse {
    export interface CustomOrderStatus {
      ID: number;
      Name: string;
      Description: string;
    }

  }

  /**
   * Set order custom status
   */
  export interface UpdateOrderCustomOrderStatus {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderCustomStatus
     */
    StatusID: number;
  }

  export interface UpdateOrderCustomOrderStatusResponse {
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get all unitPrice correction reasons
   */
  export interface GetUnitPriceCorrectionReasons {
  }

  export interface GetUnitPriceCorrectionReasonsResponse {
    Reasons: GetUnitPriceCorrectionReasonsResponse.UnitPriceCorrectionReasonData[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetUnitPriceCorrectionReasonsResponse {
    export interface UnitPriceCorrectionReasonData {
      /**
       * Entity type: UnitPriceCorrectionReason
       */
      ID: number;
      BackendID?: string;
      Priority: number;
      OrganizationUnitSet: UnitPriceCorrectionReasonData.OrganizationUnitSetDto;
      Name: string;
      Description?: string;
    }

    export namespace UnitPriceCorrectionReasonData {
      export interface OrganizationUnitSetDto {
        /**
         * Entity type: OrganizationUnitSet
         */
        ID?: number;
        Name?: string;
      }

    }

  }

  /**
   * Cancels the OrderFulfillmentLines for the given OrderID and provided OrderLineIDs.
   * It will cancel all not already cancelled OrderFulfillmentLines for the orderlines.
   * This will cause them to be re-routed and possible be fulfilled through another supplier or method.
   */
  export interface CancelOrderLineFulfillments {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
    Lines: CancelOrderLineFulfillments.Line[];
  }

  export namespace CancelOrderLineFulfillments {
    export interface Line {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      /**
       * The part of the OrderLine that couldn't be fulfilled. Can be left null if you want to cancel the entire fulfillment.
       * If a value is specified and splitting fulfillment lines is not enabled (so setting `OrderFulfillment:AllowPartialCancellation` is false or not defined) an error will be returned.
       * Min: 1
       */
      QuantityToCancel?: number;
    }

  }

  export interface GetOrderFulfillmentOptions {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetOrderFulfillmentOptionsResponse {
    AvailableFulfillmentOptions: GetOrderFulfillmentOptionsResponse.AvailableFulfillmentOptionsForOrder;
    BlockedFulfillmentOptions?: GetOrderFulfillmentOptionsResponse.BlockedFulfillmentOptionsForOrder;
    FulfillmentOptions: GetOrderFulfillmentOptionsResponse.FulfillmentOption[];
    PotentialFulfillmentOptions: GetOrderFulfillmentOptionsResponse.PotentialFulfillmentOption[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOrderFulfillmentOptionsResponse {
    export interface AvailableFulfillmentOptionsForOrder {
      SameDayDelivery: AvailableFulfillmentOptionsForOrder.SameDayDeliveryOption;
      NormalDelivery: AvailableFulfillmentOptionsForOrder.NormalDeliveryShippingOption;
    }

    export namespace AvailableFulfillmentOptionsForOrder {
      export interface SameDayDeliveryOption {
        Price: number;
        IsAvailable: boolean;
        /**
         * Entity type: OrderLine
         */
        AvailableForOrderLineIDs: number[];
        CutoffTime?: string;
        EstimatedDeliveryTimeframeFrom?: string;
        EstimatedDeliveryTimeframeTo?: string;
      }

      export interface NormalDeliveryShippingOption {
        Price: number;
        /**
         * Entity type: OrderLine
         */
        AvailableForOrderLineIDs: number[];
      }

    }

    export interface BlockedFulfillmentOptionsForOrder {
      BlockingReason: PotentialFulfillmentOptionBlockingReasons;
      FulfillmentOptions: GetOrderFulfillmentOptionsResponse.AvailableFulfillmentOptionsForOrder;
      /**
       * Entity type: OrderLine
       */
      BlockingOrderLineIDs: number[];
    }

    export interface FulfillmentOption {
      SameDayDelivery: FulfillmentOption.SameDayDeliveryOption;
      ShippingCosts: number;
      /**
       * Entity type: OrderLine
       */
      ApplicableToOrderLineIDs: number[];
      ID: string;
    }

    export namespace FulfillmentOption {
      export interface SameDayDeliveryOption {
        IsAvailable: boolean;
        CutoffTime?: string;
        EstimatedDeliveryTimeframeFrom?: string;
        EstimatedDeliveryTimeframeTo?: string;
      }

    }

    export interface PotentialFulfillmentOption {
      BlockingReason: PotentialFulfillmentOptionBlockingReasons;
      PotentialFulfillmentOptions: GetOrderFulfillmentOptionsResponse.FulfillmentOption[];
      /**
       * Entity type: OrderLine
       */
      BlockingOrderLineIDs: number[];
    }

  }

  export const enum PotentialFulfillmentOptionBlockingReasons {
    NoPartialFulfillmentAllowed = 0
  }

  /**
   * Gets the details of an OrderFulfillment, except for its Lines which can be paged in through ListOrderFulfillmentLines with a filter on OrderFulfillmentID.
   */
  export interface GetOrderFulfillment {
    /**
     * Entity type: OrderFulfillment
     */
    ID: number;
    /**
     * Whether or not to include the details on how this OrderFulfillment came to be. This information is by default only stored for 14 days. Defaults to false.
     */
    IncludeOrchestrationDetails?: boolean;
  }

  export interface GetOrderFulfillmentResponse {
    /**
     * Entity type: OrderFulfillment
     */
    ID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    ReferenceNumber: string;
    SupplierOrganizationUnit: GetOrderFulfillmentResponse.OrderFulfillmentOrganizationUnit;
    FulfillmentMethod: string;
    Data: { [key: string]: any };
    Source: string;
    /**
     * Entity type: TransputJob
     */
    TransputJobID?: number;
    Statuses: OrderFulfillmentLineStatuses[];
    CreationTime: string;
    LastModificationTime?: string;
    OrderSoldFromOrganizationUnit: GetOrderFulfillmentResponse.OrderFulfillmentOrganizationUnit;
    TotalLineCount: number;
    OrchestrationDetails?: OrderFulfillmentOrchestrationData;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOrderFulfillmentResponse {
    export interface OrderFulfillmentOrganizationUnit {
      ID: number;
      Name: string;
      Type: OrganizationUnitTypes;
      BackendID: string;
    }

  }

  export const enum OrderFulfillmentLineStatuses {
    New = 0,
    Deactivated = 50,
    Cancelled = 100,
    WaitingForStock = 500,
    Pending = 600,
    InProgress = 900,
    Completed = 1000
  }

  export interface OrderFulfillmentOrchestrationData {
    Fulfillments: OrderFulfillmentOrchestrationData.Fulfillment[];
    OrderLines: OrderFulfillmentOrchestrationData.OrderLineResult[];
    Rejections: OrderFulfillmentOrchestrationData.SupplierRejection[];
    ValidSuppliers: OrderFulfillmentOrchestrationData.ValidSupplier[];
    Sheets: OrderFulfillmentOrchestrationData.Sheet[];
  }

  export namespace OrderFulfillmentOrchestrationData {
    export interface Sheet {
      ID?: number;
      Name: string;
    }

    export interface Supplier {
      ID: number;
      Name: string;
      BackendID: string;
    }

    export interface OrderLine {
      ID: number;
      Score: number;
      Quantity: number;
      ScoreExplanations: OrderFulfillmentOrchestrationData.ScoreExplanation[];
    }

    export interface OrderLineResult {
      ID: number;
      GroupID?: string;
      PotentialSuppliers: OrderLineResult.PotentialSupplier[];
    }

    export namespace OrderLineResult {
      export interface PotentialSupplier {
        OrganizationUnitID: number;
        Score: number;
        Sheet: string;
      }

    }

    export interface Shipment {
      Score: number;
      ScoreExplanation?: OrderFulfillmentOrchestrationData.FulfillmentScoreExplanation;
      FulfillmentAction: string;
      Data: { [key: string]: any };
      Sheet: OrderFulfillmentOrchestrationData.Sheet;
      Supplier: OrderFulfillmentOrchestrationData.Supplier;
      OrderLines: OrderFulfillmentOrchestrationData.OrderLine[];
    }

    export interface Fulfillment {
      Shipments: OrderFulfillmentOrchestrationData.Shipment[];
      Score: number;
      ScoreExplanation?: OrderFulfillmentOrchestrationData.FulfillmentScoreExplanation;
    }

    export interface FulfillmentScoreExplanation {
      Description: string;
      ScoreExplanations: OrderFulfillmentOrchestrationData.ScoreExplanation[];
    }

    export interface ScoreExplanation {
      Description: string;
      Expression: string;
      PreviousScore: number;
      NewScore: number;
      DeltaScore: number;
      Variables: { [key: string]: any };
    }

    export interface RejectionReason {
      Description: string;
      Reason: string;
      ProductIDs: number[];
      OrderLineIDs: number[];
    }

    export interface FulfillmentMethod {
      Name: string;
      Reasons: OrderFulfillmentOrchestrationData.RejectionReason[];
    }

    export interface ValidSupplier {
      Score: number;
      FulfillmentMethod: string;
      SupplierID: number;
      SupplierName: string;
      ScoreExplanations: OrderFulfillmentOrchestrationData.ScoreExplanation[];
      FulfillableOrderLineIDs: number[];
    }

    export interface SupplierRejection {
      SupplierOrganizationUnitID?: number;
      SupplierOrganizationUnitName: string;
      FulfillmentMethods: OrderFulfillmentOrchestrationData.FulfillmentMethod[];
    }

  }

  export interface GetOrderFulfillmentSupplierPerformance {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetOrderFulfillmentSupplierPerformanceResponse {
    CancellationRate: number;
    AverageCompletionTime: string;
    FromDate: string;
    ToDate: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists the OrderFulfillmentLines for a given OrderID or OrderFulfillmentID.
   */
  export interface ListOrderFulfillmentLines {
    /**
     * Either provide OrderID or OrderFulfillmentID
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Either provide OrderID or OrderFulfillmentID
     * Entity type: OrderFulfillment
     */
    OrderFulfillmentID?: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
    InitialPageConfig?: PageTokenConfig<ListOrderFulfillmentLinesFilter>;
    PageToken?: string;
  }

  export interface ListOrderFulfillmentLinesFilter {
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    Status?: OrderFulfillmentLineStatuses;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
  }

  export interface ListOrderFulfillmentLinesResponse {
    Results?: ListOrderFulfillmentLineItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListOrderFulfillmentLineItem {
    /**
     * Entity type: OrderFulfillmentLine
     */
    ID: number;
    /**
     * Entity type: OrderFulfillment
     */
    OrderFulfillmentID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    OrderDisplayID?: string;
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    Product?: ListOrderFulfillmentLineItem.OrderLineProduct;
    SupplierOrganizationUnit: OrderFulfillmentLineOrganizationUnit;
    FulfillmentMethod: string;
    Data: { [key: string]: any };
    Status: OrderFulfillmentLineStatuses;
    Source: string;
    /**
     * Entity type: TransputJob
     */
    TransputJobID?: number;
    CreationTime: string;
    LastModificationTime?: string;
    FinancialOrderFulfillmentLines: ListOrderFulfillmentLineItem.FinancialOrderFulfillmentLine[];
    UnitPrice: number;
    Quantity: number;
    CurrencyID?: string;
    EarliestPossibleStartTime?: string;
    StartTime?: string;
    CompletionTime?: string;
  }

  export namespace ListOrderFulfillmentLineItem {
    export interface OrderLineProduct {
      /**
       * Entity type: Product
       */
      ID?: number;
      BackendID?: string;
      CustomID?: string;
      Description?: string;
      Details?: any;
    }

    export interface FinancialOrderFulfillmentLine {
      ID: number;
      SourceOrganizationUnit: OrderFulfillmentLineOrganizationUnit;
      DestinationOrganizationUnit: OrderFulfillmentLineOrganizationUnit;
      CurrencyID?: string;
      SequenceID?: number;
      UnitPrice?: number;
      Status: OrderFulfillmentLineStatuses;
    }

  }

  export interface OrderFulfillmentLineOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    ID: number;
    Name: string;
    Type: OrganizationUnitTypes;
    BackendID: string;
  }

  /**
   * List fulfillment rejections for one order.
   * Filter on: OrderID, SupplierOrganizationUnitID, OrderLineID, Reason, FulfillmentMethod
   */
  export interface ListOrderFulfillmentRejections {
    InitialPageConfig?: PageTokenConfig<ListOrderFulfillmentRejectionsFilter>;
    PageToken?: string;
  }

  export interface ListOrderFulfillmentRejectionsFilter {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
    Reason: string;
    FulfillmentMethod: string;
  }

  export interface ListOrderFulfillmentRejectionsResponse {
    Results?: ListOrderFulfillmentRejectionsItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListOrderFulfillmentRejectionsItem {
    /**
     * Entity type: OrderFulfillmentRejectionLedger
     */
    ID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    SupplierOrganizationUnit?: OrderFulfillmentLineOrganizationUnit;
    FulfillmentMethod?: string;
    Reason?: string;
    Description?: string;
    CreationTime?: string;
    ShippingAddress?: AddressDto;
  }

  /**
   * Lists OrderFulfillments, optionally filtered.
   */
  export interface ListOrderFulfillments {
    InitialPageConfig?: PageTokenConfig<ListOrderFulfillmentsFilter>;
    PageToken?: string;
  }

  export interface ListOrderFulfillmentsFilter {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
    OrderFulfillmentReferenceNumber?: string;
    /**
     * Entity type: OrganizationUnit
     */
    SoldFromOrganizationUnitID?: number;
    Statuses?: OrderFulfillmentLineStatuses[];
  }

  export interface ListOrderFulfillmentsResponse {
    Results?: ListOrderFulfillmentItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListOrderFulfillmentItem {
    /**
     * Entity type: OrderFulfillment
     */
    ID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    OrderType: OrderTypes;
    OrderDisplayID?: string;
    ReferenceNumber: string;
    SupplierOrganizationUnit: OrderFulfillmentLineOrganizationUnit;
    FulfillmentMethod: string;
    Data: { [key: string]: any };
    Source: string;
    /**
     * Entity type: TransputJob
     */
    TransputJobID?: number;
    Statuses: OrderFulfillmentLineStatuses[];
    CreationTime: string;
    LastModificationTime?: string;
    OrderSoldFromOrganizationUnit: OrderFulfillmentLineOrganizationUnit;
  }

  export interface SetOrderFulfillmentOptions {
    /**
     * Entity type: Order
     */
    OrderID: number;
    FulfillmentOptionsSelection?: SetOrderFulfillmentOptions.FulfillmentOptionsSelectionForOrder;
    SelectedFulfillmentOptions: SetOrderFulfillmentOptions.SelectedFulfillmentOption[];
  }

  export namespace SetOrderFulfillmentOptions {
    export interface FulfillmentOptionsSelectionForOrder {
      NormalDelivery: FulfillmentOptionsSelectionForOrder.NormalDeliverySelection;
      SameDayDelivery: FulfillmentOptionsSelectionForOrder.SameDayDeliverySelection;
    }

    export namespace FulfillmentOptionsSelectionForOrder {
      export interface NormalDeliverySelection {
        /**
         * Entity type: OrderLine
         */
        OrderLineIDs: number[];
      }

      export interface SameDayDeliverySelection {
        /**
         * Entity type: OrderLine
         */
        OrderLineIDs: number[];
      }

    }

    export interface SelectedFulfillmentOption {
      ID?: string;
      /**
       * Entity type: OrderLine
       */
      OrderLineIDs: number[];
    }

  }

  /**
   * Ships an OrderFulfillment, thereby creating one or more Shipments for the underlying Order.
   */
  export interface ShipOrderFulfillment {
    /**
     * Entity type: OrderFulfillment
     */
    OrderFulfillmentID: number;
    Lines?: ShipOrderFulfillment.ShipmentOrderFulfillmentLine[];
    FinalShipment?: boolean;
  }

  export namespace ShipOrderFulfillment {
    export interface ShipmentOrderFulfillmentLine {
      /**
       * Entity type: OrderFulfillmentLine
       */
      OrderFulfillmentLineID?: number;
      QuantityShipped?: number;
      ShipmentBackendID?: string;
      TrackingCode?: string;
      TrackingLink?: string;
    }

  }

  export interface ShipOrderFulfillmentResponse {
    Shipments: ShipOrderFulfillmentResponse.Shipment[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ShipOrderFulfillmentResponse {
    export interface Shipment {
      /**
       * Entity type: Shipment
       */
      ID?: number;
      TrackingCode?: string;
      TrackingLink?: string;
    }

  }

  /**
   * Updates the price and/or quantity of a fulfillment line based on its ID.
   * 
   * When modifying the quantity, it's not possible currently to increase the quantity, only to decrease it. The modification must also happen on the physical fulfillment line,
   * that is a fulfillment line with Type = Physical.
   */
  export interface UpdateOrderFulfillmentLine {
    /**
     * Entity type: OrderFulfillmentLine
     */
    ID: number;
    Quantity?: number;
    UnitPrice?: number;
  }

  /**
   * Download a sample Excel file which can be filled with special opening hours. The populated Excel file can be uploaded through the `ImportSpecialOpeningHours` service.
   */
  export interface DownloadSpecialOpeningHoursExcelTemplate {
  }

  /**
   * Upload a filled Excel file with special opening hours for an opening hour template. The template Excel file should be downloaded from the `DownloadSpecialOpeningHoursSample` service.
   */
  export interface UploadSpecialOpeningHoursExcel {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID: number;
    Data: string;
  }

  /**
   * Validate an OpeningHoursTemplate
   */
  export interface ValidateOpeningHoursTemplate {
    /**
     * Entity type: OpeningHoursType
     */
    OpeningHoursTypeID: number;
    Sunday?: OpeningHoursTemplateDay;
    Monday?: OpeningHoursTemplateDay;
    Tuesday?: OpeningHoursTemplateDay;
    Wednesday?: OpeningHoursTemplateDay;
    Thursday?: OpeningHoursTemplateDay;
    Friday?: OpeningHoursTemplateDay;
    Saturday?: OpeningHoursTemplateDay;
  }

  export interface OpeningHoursTemplateDay {
    IsClosed: boolean;
    Description?: string;
    DayOpeningTimes?: OpeningHoursTemplateTimeSlot[];
  }

  export interface OpeningHoursTemplateTimeSlot {
    StartTime: string;
    EndTime: string;
  }

  export interface ValidateOpeningHoursTemplateResponse {
    Successful: boolean;
    ValidationErrors: Error[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface Error {
    Type: string;
    Message: string;
    Prefix: string;
    MessageArguments: any[];
    Data: any;
    Identifier: string;
  }

  /**
   * Create a new OpeningHoursTemplate
   */
  export interface CreateOpeningHoursTemplate {
    /**
     * Entity type: OpeningHoursType
     */
    OpeningHoursTypeID: number;
    Description: string;
    Sunday?: OpeningHoursTemplateDay;
    Monday?: OpeningHoursTemplateDay;
    Tuesday?: OpeningHoursTemplateDay;
    Wednesday?: OpeningHoursTemplateDay;
    Thursday?: OpeningHoursTemplateDay;
    Friday?: OpeningHoursTemplateDay;
    Saturday?: OpeningHoursTemplateDay;
  }

  export interface CreateOpeningHoursTemplateResponse {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID: number;
    Successful: boolean;
    ValidationErrors: Error[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an existing OpeningHoursTemplate
   */
  export interface UpdateOpeningHoursTemplate {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID: number;
    Description: string;
    Sunday?: OpeningHoursTemplateDay;
    Monday?: OpeningHoursTemplateDay;
    Tuesday?: OpeningHoursTemplateDay;
    Wednesday?: OpeningHoursTemplateDay;
    Thursday?: OpeningHoursTemplateDay;
    Friday?: OpeningHoursTemplateDay;
    Saturday?: OpeningHoursTemplateDay;
  }

  /**
   * Delete an OpeningHoursTemplate
   */
  export interface DeleteOpeningHoursTemplate {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID: number;
  }

  /**
   * Add or update a special date on an OpeningHoursTemplate
   */
  export interface SetSpecialDateOnOpeningHoursTemplate {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID: number;
    Date: string;
    Description: string;
    StoreDescriptionAsTranslationKey?: boolean;
    IsClosed: boolean;
    DayOpeningTimes?: OpeningHoursTemplateTimeSlot[];
  }

  /**
   * Remove a special date from an OpeningHoursTemplate
   */
  export interface RemoveSpecialDateFromOpeningHoursTemplate {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID: number;
    Date: string;
  }

  /**
   * List OpeningHoursTemplates
   */
  export interface ListOpeningHoursTemplates {
    PageConfig?: PageConfig<ListOpeningHoursTemplatesFilter>;
  }

  export interface ListOpeningHoursTemplatesFilter {
    /**
     * Entity type: OpeningHoursType
     */
    OpeningHoursTypeID?: number;
    Description?: string;
  }

  export interface ListOpeningHoursTemplatesResponse {
    Result?: PagedResult<ListOpeningHoursTemplatesResponse.Data>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOpeningHoursTemplatesResponse {
    export interface Data {
      /**
       * Entity type: OpeningHoursTemplate
       */
      OpeningHoursTemplateID?: number;
      /**
       * Entity type: OpeningHoursType
       */
      OpeningHoursTypeID: number;
      Description: string;
      IsImported: boolean;
    }

  }

  /**
   * List all the attached OrganizationUnitSets for OpeningHoursTemplates
   */
  export interface ListOpeningHoursTemplatesOrganizationUnitSets {
    PageConfig?: PageConfig<ListOpeningHoursTemplatesOrganizationUnitSetsFilter>;
  }

  export interface ListOpeningHoursTemplatesOrganizationUnitSetsFilter {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: OpeningHoursType
     */
    OpeningHoursTypeID?: number;
  }

  export interface ListOpeningHoursTemplatesOrganizationUnitSetsResponse {
    Result?: PagedResult<ListOpeningHoursTemplatesOrganizationUnitSetsResponse.Data>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListOpeningHoursTemplatesOrganizationUnitSetsResponse {
    export interface Data {
      /**
       * Entity type: OpeningHoursTemplate
       */
      OpeningHoursTemplateID?: number;
      /**
       * Entity type: OpeningHoursType
       */
      OpeningHoursTypeID: number;
      Description: string;
      OrganizationUnitSets: ListOpeningHoursTemplatesOrganizationUnitSetsResponse.OrganizationUnitSetData[];
      IsImported: boolean;
    }

    export interface OrganizationUnitSetData {
      /**
       * Entity type: OrganizationUnitSet
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Get OpeningHoursTemplate by ID
   */
  export interface GetOpeningHoursTemplateByID {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID: number;
  }

  export interface GetOpeningHoursTemplateResponseByID {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID?: number;
    /**
     * Entity type: OpeningHoursType
     */
    OpeningHoursTypeID: number;
    Description: string;
    Sunday: OpeningHoursTemplateDay;
    Monday: OpeningHoursTemplateDay;
    Tuesday: OpeningHoursTemplateDay;
    Wednesday: OpeningHoursTemplateDay;
    Thursday: OpeningHoursTemplateDay;
    Friday: OpeningHoursTemplateDay;
    Saturday: OpeningHoursTemplateDay;
    SpecialDays: GetOpeningHoursTemplateResponseByID.SpecialDay[];
    IsImported: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOpeningHoursTemplateResponseByID {
    export interface SpecialDay {
      Date: string;
      OpeningHours: OpeningHoursTemplateDay;
    }

  }

  /**
   * Attaches an OpeningHoursTemplate to the given OrganizationUnitSets.
   * When `OverrideIfOpeningHoursTypeAlreadyAttached` is set to true, the service detaches the template of the same type before attaching the new template.
   * When `OverrideIfOpeningHoursTypeAlreadyAttached` is set to false, the service will fail if one of the given OrganizationUnitSets is already attached to a OpeningHoursTemplate of the same Type.
   */
  export interface AttachOpeningHoursTemplateToOrganizationUnitSets {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetIDs: number[];
    OverrideIfOpeningHoursTypeAlreadyAttached?: boolean;
  }

  /**
   * Attaches an OpeningHoursTemplate to the given OrganizationUnitSets.
   */
  export interface DetachOpeningHoursTemplateToOrganizationUnitSets {
    /**
     * Entity type: OpeningHoursTemplate
     */
    OpeningHoursTemplateID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetIDs?: number[];
  }

  /**
   * Get flattened organization unit opening hours
   */
  export interface GetOrganizationUnitOpeningHours {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetOrganizationUnitOpeningHoursResponse {
    OpeningHours: GetOrganizationUnitOpeningHoursResponse.Data[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOrganizationUnitOpeningHoursResponse {
    export interface Data {
      /**
       * Entity type: OpeningHoursType
       */
      OpeningHoursTypeID: number;
      Sunday: OpeningHoursTemplateDay;
      Monday: OpeningHoursTemplateDay;
      Tuesday: OpeningHoursTemplateDay;
      Wednesday: OpeningHoursTemplateDay;
      Thursday: OpeningHoursTemplateDay;
      Friday: OpeningHoursTemplateDay;
      Saturday: OpeningHoursTemplateDay;
      SpecialDays: Data.SpecialDay[];
    }

    export namespace Data {
      export interface SpecialDay {
        Date: string;
        OpeningHours: OpeningHoursTemplateDay;
      }

    }

  }

  /**
   * Get detailed organization unit opening hours
   */
  export interface GetDetailedOrganizationUnitOpeningHours {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetDetailedOrganizationUnitOpeningHoursResponse {
    OpeningHours: GetDetailedOrganizationUnitOpeningHoursResponse.Data[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetDetailedOrganizationUnitOpeningHoursResponse {
    export interface Data {
      /**
       * Entity type: OpeningHoursType
       */
      OpeningHoursTypeID: number;
      Sunday: Data.RegularDay;
      Monday: Data.RegularDay;
      Tuesday: Data.RegularDay;
      Wednesday: Data.RegularDay;
      Thursday: Data.RegularDay;
      Friday: Data.RegularDay;
      Saturday: Data.RegularDay;
      SpecialDays: Data.SpecialDay[];
    }

    export namespace Data {
      export interface RegularDay {
        IsFromTemplate: boolean;
        ElectedTemplateID?: number;
        ExceptionData: OpeningHoursTemplateDay;
        TemplateData: OpeningHoursTemplateDay;
      }

      export interface SpecialDay {
        Date: string;
        IsFromTemplate: boolean;
        ElectedTemplateID?: number;
        ExceptionData: OpeningHoursTemplateDay;
        TemplateData: OpeningHoursTemplateDay;
      }

    }

  }

  /**
   * Gets the organization unit visiting opening hours for the given period.
   */
  export interface GetOrganizationUnitOpeningHoursForPeriod {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    From: string;
    To: string;
  }

  export interface GetOrganizationUnitOpeningHoursForPeriodResponse {
    OpeningHours: { [key: number]: GetOrganizationUnitOpeningHoursForPeriodResponse.OrganizationUnitOpeningHours };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOrganizationUnitOpeningHoursForPeriodResponse {
    export interface OrganizationUnitOpeningHours {
      DayOfWeek: DayOfWeek;
      SpecialReason: string;
      StartTime?: string;
      EndTime?: string;
    }

  }

  /**
   * Add or update an Organization's openinghours exception
   */
  export interface SetOrganizationUnitOpeningHoursException {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: OpeningHoursType
     */
    OpeningHoursTypeID: number;
    DayOfWeek?: DayOfWeek;
    Date?: string;
    OpeningHours: OpeningHoursTemplateDay;
  }

  /**
   * Remove an Organization's openinghours exception. When neither a DayOfWeek or Date is given, all openinghours exception for the given type will be removed and all openinghours will revert to the template value (if present).
   */
  export interface RemoveOrganizationUnitOpeningHoursException {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: OpeningHoursType
     */
    OpeningHoursTypeID: number;
    DayOfWeek?: DayOfWeek;
    Date?: string;
  }

  /**
   * Get a list of all OpeningHoursTemplates translation keys for description translation.
   */
  export interface GetOpeningHoursTemplatesTranslationKeys {
  }

  export interface GetOpeningHoursTemplatesTranslationKeysResponse {
    EntityPropertyTranslations: GetOpeningHoursTemplatesTranslationKeysResponse.EntityPropertyTranslation[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOpeningHoursTemplatesTranslationKeysResponse {
    export interface EntityPropertyTranslation {
      TranslationKey: string;
      CurrentTranslation: string;
    }

  }

  /**
   * Lists the errors from the message queue.
   */
  export interface ListMessageQueueErrors {
    PageConfig?: PageConfig<ListMessageQueueErrorsFilter>;
  }

  export interface ListMessageQueueErrorsFilter {
    Status?: MessageQueueErrorStatuses;
    StartDate?: string;
    EndDate?: string;
    MessageTypeName: string;
    MessageID?: string;
    ExceptionMessage: string;
    ErrorCode: string;
  }

  export const enum MessageQueueErrorStatuses {
    New = 0,
    Republished = 1,
    SuccessAfterRetry = 2,
    FailureAfterRetry = 3,
    Ignored = 4
  }

  export interface ListMessageQueueErrorsResponse {
    Result?: PagedResult<MessageQueueErrorDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface MessageQueueErrorDto {
    ID: number;
    Status: MessageQueueErrorStatuses;
    MessageID?: string;
    MessageBody: string;
    MessageTypeName: string;
    ExceptionMessage: string;
    StackTrace: string;
    ErrorCode: string;
    HostInfo: string;
    Headers: string;
    Addresses: string;
    CreationTime: string;
    RetryCount: number;
    NextRetry?: string;
  }

  /**
   * Requeues specified error message(s).
   */
  export interface RequeueErrorMessage {
    /**
     * Entity type: MessageQueueError
     */
    MessageQueueErrorID?: number;
    /**
     * Entity type: MessageQueueError
     */
    MessageQueueErrorIDs?: number[];
  }

  /**
   * Books the given invoice by its `ID`.
   * 
   * Booking an invoice has the side effect of creating a payment on the order for the amount of the invoice.
   */
  export interface BookInvoice {
    /**
     * Entity type: Invoice
     */
    ID: number;
  }

  /**
   * Creates lines on the given invoice (`InvoiceID`).
   * 
   * It is required to supply an OrderLine or a ShipmentLine. All other properties are optional and will be copied from the shipment-/order- line.
   */
  export interface CreateInvoiceLines {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    Lines?: CreateInvoiceLine[];
  }

  export interface CreateInvoiceLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: ShipmentLine
     */
    ShipmentLineID?: number;
    Quantity?: number;
    ExpectedQuantity?: number;
    UnitPrice?: number;
    /**
     * Range: 1-2
     */
    TaxRate?: number;
  }

  /**
   * Creates a new invoice for the given `OrganizationUnitID` (the one who writes the invoice)
   * and the `SupplierOrganizationUnitID` (the one who receives good/services).
   * 
   * - `InvoiceDate` is the date on which the invoice was created.
   * - `InvoiceNumber` is the unique number for the invoice.
   * - `Blobs` can contain PDF's or other media that can for example be a copy of the invoice
   * - `OriginalTotalAmount` is the amount that is expected on the invoice. E.g. this can differ from the amount that is computed based on the invoice lines
   * - `Type` indicates if this is an invoice for a sales or purchase orders
   */
  export interface CreateInvoice {
    /**
     * Entity type: OrganizationUnit
     */
    CreditorOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    DebtorOrganizationUnitID?: number;
    PaymentTermStartDate?: string;
    PaymentTermDueDate?: string;
    InvoiceDate: string;
    InvoiceNumber?: string;
    Description?: string;
    Blobs?: InvoiceBlobDto[];
    OriginalTotalAmount?: number;
    Type: InvoiceTypes;
    /**
     * Entity type: InvoiceHoldStatus
     */
    HoldStatusID?: number;
    FiscalID?: string;
    TaxReverseCharge?: boolean;
    CalculationMethod?: InvoiceCalculationMethod;
  }

  export interface InvoiceBlobDto {
    BlobID: string;
  }

  export interface InvoiceAdditionalAmountDto {
    /**
     * Entity type: InvoiceAdditionalAmount
     */
    ID: number;
    Type: number;
    OriginalAmount: number;
    Amount: number;
    OriginalAmountInTax: number;
    AmountInTax: number;
    TaxRate: number;
    TaxCodeID: number;
    Disputes: InvoiceDisputeDto[];
  }

  export interface InvoiceDisputeDto {
    /**
     * Entity type: InvoiceDispute
     */
    ID: number;
    /**
     * Entity type: InvoiceDisputeReason
     */
    ReasonID: number;
    ReasonName: string;
    Amount: number;
    Description?: string;
    Type: InvoiceDisputeTypes;
    IsResolved: boolean;
    ResolvedAmount: number;
    ResolveActions: InvoiceDisputeResolveActionDto[];
    EmailAddressToSendNotificationTo?: string;
  }

  export const enum InvoiceDisputeTypes {
    Original = 0,
    Expected = 1
  }

  export interface InvoiceDisputeResolveActionDto {
    /**
     * Entity type: InvoiceDisputeResolveAction
     */
    ID: number;
    Amount: number;
    Description?: string;
    LedgerClassID?: string;
  }

  export interface CreateInvoiceResponse {
    /**
     * Entity type: Invoice
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create invoices for an Order. Optionally specify the specific lines with quantities.
   */
  export interface CreateInvoicesForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    LinesToInvoice?: CreateInvoicesForOrder.OrderLineToInvoice[];
  }

  export namespace CreateInvoicesForOrder {
    export interface OrderLineToInvoice {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      Quantity: number;
    }

  }

  export interface CreateInvoicesForOrderResponse {
    Invoices: CreateInvoicesForOrderResponse.Invoice[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace CreateInvoicesForOrderResponse {
    export interface Invoice {
      ID: number;
      InvoiceNumber: string;
      TypeID: number;
      StatusID: number;
    }

  }

  /**
   * Creates an order with a manually issued invoice. This will insert a committed Invoice with a custom sequencing number and additional
   * `InvoiceManualData`. It will be fully paid with a custom payment type.
   * 
   * The specified Order will be paid, placed, shipped and completed, but without any additional side effects like stock ledgers or commitments.
   */
  export interface CreateManualInvoice {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    Series: string;
    Number: string;
    InvoiceDate: string;
    Customer: ManualCustomer;
    Lines: ManualLine[];
  }

  export interface ManualCustomer {
    FirstName: string;
    LastName: string;
    FiscalID: string;
    CountryID: string;
    ZipCode: string;
    Address1: string;
    City: string;
  }

  export interface ManualLine {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    Description: string;
    /**
     * Entity type: TaxRateInfo
     */
    TaxRateInfoID: number;
    Quantity: number;
    TotalAmountInTax: number;
  }

  export interface CreateManualInvoiceResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Deletes the given invoice. Note, can only delete invoices that have not been booked yet.
   */
  export interface DeleteInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
  }

  /**
   * Returns all the details of an invoice by its ID.
   */
  export interface GetInvoice {
    /**
     * Entity type: Invoice
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
  }

  export interface GetInvoiceResponse {
    Invoice: InvoiceDetailsDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface InvoiceDetailsDto {
    CreditorOrganizationUnitName: string;
    CreditorOrganizationUnitBackendID?: string;
    DebtorOrganizationUnitName?: string;
    DebtorOrganizationUnitBackendID?: string;
    /**
     * Entity type: Language
     */
    CreditorOrganizationUnitLanguageID: string;
    /**
     * Entity type: Country
     */
    CreditorOrganizationUnitCountryID: string;
    AdditionalAmounts: InvoiceAdditionalAmountDto[];
    Disputes: InvoiceDisputeDto[];
    Lines: InvoiceDetailLineDto[];
    Blobs: InvoiceBlobDto[];
    PaidAmount?: number;
    OpenAmount?: number;
    BookStatus: InvoiceBookStatusDto;
    Shipments: InvoiceShipment[];
    /**
     * The ExpectedQuantity of all the InvoiceLines summed
     */
    ExpectedTotalQuantity?: number;
    /**
     * The Quantity of all the InvoiceLines summed
     */
    TotalQuantity: number;
    /**
     * The difference between TotalQuantity and ExpectedTotalQuantity
     */
    TotalQuantityDifference?: number;
    /**
     * The TotalAmount of the invoice minus the OriginalDisputes
     */
    SubTotalAmount: number;
    /**
     * The ExpectedTotalAmount of the invoice minus the Disputes
     */
    ExpectedSubTotalAmount?: number;
    /**
     * The Amount of the Disputes summed by Type
     */
    DisputeAmounts: DisputeAmounts;
    /**
     * The TotalAmountInvoicedInTax of all the InvoiceLines summed
     */
    TotalLineAmountInvoicedInTax: number;
    FiscalData: InvoiceFiscalData;
    ExportStatus: InvoiceFullExportStatus;
    IsIntraCompany: boolean;
    IsInterCompany: boolean;
    OrderID?: number;
    OrderDisplayID?: string;
    RecipientEmail?: string;
    RecipientCode?: string;
    PublicProjectCode?: string;
    TenderIdentificationCode?: string;
    UserData: InvoiceUserDetailsDto;
    /**
     * Entity type: Invoice
     */
    ReferencedInvoiceID?: number;
    /**
     * Entity type: Invoice
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    CreditorOrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    DebtorOrganizationUnitID?: number;
    PaymentTermStartDate?: string;
    PaymentTermDueDate?: string;
    Type: InvoiceTypes;
    InvoiceDate?: string;
    InvoiceNumber: string;
    Description: string;
    Status: InvoiceStatuses;
    TotalAmount: number;
    CurrencyID: string;
    HoldStatusID?: number;
    /**
     * The TotalAmount on the actual Invoice as provided by the Supplier
     */
    OriginalTotalAmount?: number;
    /**
     * The difference between the TotalAmount and the OriginalTotalAmount
     */
    OriginalTotalAmountDifference?: number;
    /**
     * The ExpectedTotalAmountInTax of all the InvoiceLines + AdditionalAmounts + ExpectedDisputes
     */
    ExpectedTotalAmount?: number;
    /**
     * The difference between the TotalAmount and the ExpectedTotalAmount
     */
    ExpectedTotalAmountDifference?: number;
    TaxReverseCharge: boolean;
    ZippedSignature: string;
    Signature: string;
    EcoTax: number;
  }

  export interface InvoiceDetailLineDto {
    /**
     * Entity type: InvoiceLine
     */
    ID: number;
    OrderLine: InvoiceDetailsOrderLineDto;
    ShipmentLine: InvoiceDetailsShipmentLineDto;
    InvoiceDate?: string;
    UnitCost?: number;
    UnitPrice: number;
    TaxRate: number;
    TaxCode?: string;
    TaxRateCategory?: TaxRateCategory;
    DiscountAmount: number;
    Disputes: InvoiceDisputeDto[];
    TotalAmount: number;
    TotalAmountInTax: number;
    TotalAmountInvoicedInTax: number;
    TotalAmountInvoiced: number;
    ExpectedUnitPrice?: number;
    ExpectedQuantity?: number;
    ExpectedDiscountAmount?: number;
    ExpectedTaxRate?: number;
    ExpectedTotalAmount?: number;
    ExpectedTotalAmountInTax?: number;
    /**
     * The amount that has to be resolved with disputes: ExpectedTotalAmountInTax - TotalAmountInTax - Disputes
     */
    AmountToBeDisputed?: number;
    Quantity: number;
  }

  export interface InvoiceDetailsOrderLineDto {
    Description: string;
    UnitCost?: number;
    UnitPrice: number;
    /**
     * Entity type: OrderLine
     */
    ID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    ProductCustomID: string;
    Product?: any;
    SupplierProductBackendID: string;
    Type: OrderLineTypes;
  }

  export interface InvoiceDetailsShipmentLineDto {
    /**
     * Entity type: ShipmentLine
     */
    ID: number;
    QuantityDelivered: number;
    TotalAmountDelivered: number;
    Shipment: InvoiceDetailsShipmentDto;
  }

  export interface InvoiceDetailsShipmentDto {
    /**
     * Entity type: Shipment
     */
    ID: number;
    BackendID: string;
  }

  export interface InvoiceBookStatusDto {
    IsBooked: boolean;
    CanBeBooked: boolean;
    Message: string;
  }

  export interface InvoiceShipment {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    ShipmentBackendID: string;
    LastDeliveryDate?: string;
    CompletionTime?: string;
    TotalAmount: number;
    ExpectedTotalAmount?: number;
    TotalQuantity: number;
    ExpectedTotalQuantity?: number;
  }

  export interface DisputeAmounts {
    Total: number;
    Original: number;
    Expected: number;
  }

  export interface InvoiceFiscalData {
    UniqueDocumentCode: string;
    QrCodeData: string;
    DocumentSequenceNumber: string;
    FiscalDocumentID: string;
    FiscalDocumentDate: string;
    ExternalDocumentConfirmationNumber: string;
    ExternalFiscalDeviceID: string;
  }

  export const enum InvoiceFullExportStatus {
    None = 0,
    Complete = 1,
    NotComplete = 2
  }

  export interface InvoiceUserDetailsDto {
    FirstName: string;
    LastName: string;
    EmailAddress: string;
    PhoneNumber: string;
    FiscalID: string;
    UserBillingAddress: InvoiceAddressDto;
    UserShippingAddress: InvoiceAddressDto;
    CompanyName: string;
    TaxRegistrationNumber: string;
    RegistrationNumber: string;
    CountryID: string;
    CompanyFiscalID?: string;
  }

  export interface InvoiceAddressDto {
    Address1: string;
    Address2: string;
    HouseNumber: string;
    ZipCode: string;
    Subdistrict: string;
    District: string;
    City: string;
    State: string;
    CountryID: string;
  }

  /**
   * Create an InvoiceAdditionalAmount
   */
  export interface CreateInvoiceAdditionalAmount {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    /**
     * Entity type: InvoiceAdditionalAmountTypes
     */
    TypeID: number;
    OriginalAmount: number;
    Amount: number;
    TaxRate?: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
  }

  export interface CreateInvoiceAdditionalAmountResponse {
    /**
     * Entity type: InvoiceAdditionalAmount
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an InvoiceAdditionalAmount
   */
  export interface UpdateInvoiceAdditionalAmount {
    /**
     * Entity type: InvoiceAdditionalAmount
     */
    ID: number;
    /**
     * Entity type: InvoiceAdditionalAmountTypes
     */
    TypeID: number;
    OriginalAmount: number;
    Amount: number;
    TaxRate?: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
  }

  /**
   * Delete an InvoiceAdditionalAmount
   */
  export interface DeleteInvoiceAdditionalAmount {
    /**
     * Entity type: InvoiceAdditionalAmount
     */
    ID: number;
  }

  /**
   * Get all AdditionalAmounts for an Invoice
   */
  export interface GetInvoiceAdditionalAmountsForInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
  }

  export interface GetInvoiceAdditionalAmountsForInvoiceResponse {
    InvoiceAdditionalAmounts: GetInvoiceAdditionalAmountsForInvoiceResponse.InvoiceAdditionalAmountDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetInvoiceAdditionalAmountsForInvoiceResponse {
    export interface InvoiceAdditionalAmountDto {
      /**
       * Entity type: InvoiceAdditionalAmount
       */
      ID: number;
      /**
       * Entity type: InvoiceAdditionalAmountTypes
       */
      TypeID: number;
      TypeName: string;
      OriginalAmount: number;
      Amount: number;
      OriginalAmountInTax: number;
      AmountInTax: number;
      TaxRate: number;
      /**
       * Entity type: TaxCode
       */
      TaxCodeID: number;
      TaxCodeName: string;
    }

  }

  /**
   * Get a InvoiceAdditionalAmountType by ID
   */
  export interface GetInvoiceAdditionalAmountTypeByID {
    /**
     * Entity type: InvoiceAdditionalAmountTypes
     */
    ID: number;
  }

  export interface GetInvoiceAdditionalAmountTypeByIDResponse {
    /**
     * Entity type: InvoiceAdditionalAmountTypes
     */
    ID: number;
    Name: string;
    Description?: string;
    LedgerClassID?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create a InvoiceAdditionalAmountType
   */
  export interface CreateInvoiceAdditionalAmountType {
    Name: string;
    Description?: string;
    LedgerClassID?: string;
  }

  export interface CreateInvoiceAdditionalAmountTypeResponse {
    /**
     * Entity type: InvoiceAdditionalAmountTypes
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update the Description and LedgerClassID of a InvoiceAdditionalAmountType
   */
  export interface UpdateInvoiceAdditionalAmountType {
    /**
     * Entity type: InvoiceAdditionalAmountTypes
     */
    ID: number;
    Description?: string;
    LedgerClassID?: string;
  }

  /**
   * Delete a InvoiceAdditionalAmountType, this is only allowed if the type is never used
   */
  export interface DeleteInvoiceAdditionalAmountType {
    /**
     * Entity type: InvoiceAdditionalAmountTypes
     */
    ID: number;
  }

  /**
   * Get all InvoiceAdditionalAmountTypes
   */
  export interface GetInvoiceAdditionalAmountTypes {
  }

  export interface GetInvoiceAdditionalAmountTypesResponse {
    InvoiceAdditionalAmountTypes: GetInvoiceAdditionalAmountTypesResponse.InvoiceAdditionalAmountTypeDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetInvoiceAdditionalAmountTypesResponse {
    export interface InvoiceAdditionalAmountTypeDto {
      /**
       * Entity type: InvoiceAdditionalAmountTypes
       */
      ID: number;
      Name: string;
      Description?: string;
      LedgerClassID?: string;
    }

  }

  /**
   * Attaches a blob to an invoice, either by `BlobID` or `BlobUrl`.
   */
  export interface AttachBlobToInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
  }

  /**
   * Removes a blob from an invoice, either by `BlobID` or `BlobUrl`.
   */
  export interface DeleteInvoiceBlob {
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
  }

  /**
   * Provides a mapping between an organization unit and a company based on a country. This mapping is used in scenarios for where delivery orders are
   * in play. E.g endless aisle (delivery order from a store) and an online order from a website for example.
   * 
   * To comply with regulations, these invoices that are generated require the tax registration number from the company that will be delivering the goods
   * with its specific registration from the country that the goods are send to.
   * 
   * If a mapping already exists for given `OrganizationUnitID` and `CountryID`, the `CompanyID` will be updated and `AlreadyExisted` in the response will be `true`.
   */
  export interface CreateOrUpdateInvoiceCompanyMapping {
    /**
     * Entity type: Country
     */
    CountryID: string;
    /**
     * Entity type: Company
     */
    CompanyID: number;
    VatNumber: string;
  }

  export interface CreateOrUpdateInvoiceCompanyMappingResponse {
    /**
     * Entity type: InvoiceCompanyMapping
     */
    ID: number;
    AlreadyExisted: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists the `InvoiceCompanyMapping` items.
   */
  export interface ListInvoiceCompanyMapping {
    InitialPageConfig?: PageTokenConfig<ListInvoiceCompanyMappingFilter>;
    PageToken?: string;
  }

  export interface ListInvoiceCompanyMappingFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
  }

  export interface ListInvoiceCompanyMappingResponse {
    Results?: ListInvoiceCompanyMappingModel[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListInvoiceCompanyMappingModel {
    /**
     * Entity type: InvoiceCompanyMapping
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitBackendID?: string;
    OrganizationUnitName?: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    /**
     * Entity type: Company
     */
    CompanyID: number;
    CompanyName: string;
    VatNumber: string;
  }

  /**
   * Deletes an `InvoiceCompanyMapping`.
   */
  export interface DeleteInvoiceCompanyMapping {
    /**
     * Entity type: InvoiceCompanyMapping
     */
    ID: number;
  }

  /**
   * Get a InvoiceDisputeReason by ID
   */
  export interface GetInvoiceDisputeReasonByID {
    /**
     * Entity type: InvoiceDisputeReason
     */
    ID: number;
  }

  export interface GetInvoiceDisputeReasonByIDResponse {
    /**
     * Entity type: InvoiceDisputeReason
     */
    ID: number;
    Name: string;
    Description?: string;
    LedgerClassID?: string;
    AutoResolve: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create a InvoiceDisputeReason
   */
  export interface CreateInvoiceDisputeReason {
    Name: string;
    Description?: string;
    LedgerClassID?: string;
    AutoResolve: boolean;
  }

  export interface CreateInvoiceDisputeReasonResponse {
    /**
     * Entity type: InvoiceDisputeReason
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update the Description and LedgerClassID of a InvoiceDisputeReason
   */
  export interface UpdateInvoiceDisputeReason {
    /**
     * Entity type: InvoiceDisputeReason
     */
    ID: number;
    Description?: string;
    LedgerClassID?: string;
    AutoResolve: boolean;
  }

  /**
   * Delete a InvoiceDisputeReason, this is only allowed if the reason is never used
   */
  export interface DeleteInvoiceDisputeReason {
    /**
     * Entity type: InvoiceDisputeReason
     */
    ID: number;
  }

  /**
   * Get all InvoiceDisputeReasons
   */
  export interface GetInvoiceDisputeReasons {
  }

  export interface GetInvoiceDisputeReasonsResponse {
    InvoiceDisputeReasons: GetInvoiceDisputeReasonsResponse.InvoiceDisputeReasonDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetInvoiceDisputeReasonsResponse {
    export interface InvoiceDisputeReasonDto {
      /**
       * Entity type: InvoiceDisputeReason
       */
      ID: number;
      Name: string;
      Description?: string;
      LedgerClassID?: string;
      AutoResolve: boolean;
    }

  }

  /**
   * Create an InvoiceDisputeResolveAction
   */
  export interface CreateInvoiceDisputeResolveAction {
    /**
     * Entity type: InvoiceDispute
     */
    InvoiceDisputeID: number;
    Amount: number;
    Description?: string;
    LedgerClassID?: string;
  }

  export interface CreateInvoiceDisputeResolveActionResponse {
    /**
     * Entity type: InvoiceDisputeResolveAction
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an InvoiceDisputeResolveAction
   */
  export interface UpdateInvoiceDisputeResolveAction {
    /**
     * Entity type: InvoiceDisputeResolveAction
     */
    ID: number;
    Amount: number;
    Description?: string;
    LedgerClassID?: string;
  }

  /**
   * Delete an InvoiceDisputeResolveAction
   */
  export interface DeleteInvoiceDisputeResolveAction {
    /**
     * Entity type: InvoiceDisputeResolveAction
     */
    ID: number;
  }

  /**
   * Create an InvoiceDispute
   */
  export interface CreateInvoiceDispute {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    /**
     * Entity type: InvoiceLine
     */
    InvoiceLineID?: number;
    /**
     * Entity type: InvoiceAdditionalAmount
     */
    InvoiceAdditionalAmountID?: number;
    /**
     * Entity type: InvoiceDisputeReason
     */
    ReasonID: number;
    Amount: number;
    Description?: string;
    Type?: InvoiceDisputeTypes;
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Email address to send notification to. This will overwrite the selected User when set.
     */
    EmailAddressToSendNotificationTo?: string;
  }

  export interface CreateInvoiceDisputeResponse {
    /**
     * Entity type: InvoiceDispute
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an InvoiceDispute
   */
  export interface UpdateInvoiceDispute {
    /**
     * Entity type: InvoiceDispute
     */
    ID: number;
    /**
     * Entity type: InvoiceDisputeReason
     */
    ReasonID?: number;
    Amount?: number;
    Description?: string;
    Type?: InvoiceDisputeTypes;
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  /**
   * Delete an InvoiceDispute
   */
  export interface DeleteInvoiceDispute {
    /**
     * Entity type: InvoiceDispute
     */
    ID: number;
  }

  /**
   * Get all Disputes for an Invoice
   */
  export interface GetInvoiceDisputesForInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
  }

  export interface GetInvoiceDisputesForInvoiceResponse {
    InvoiceDisputes: GetInvoiceDisputesForInvoiceResponse.InvoiceDisputeDto[];
    TotalDisputedAmount: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetInvoiceDisputesForInvoiceResponse {
    export interface InvoiceDisputeDto {
      /**
       * Entity type: InvoiceDispute
       */
      ID: number;
      /**
       * Entity type: InvoiceLine
       */
      InvoiceLineID?: number;
      /**
       * Entity type: InvoiceAdditionalAmount
       */
      InvoiceAdditionalAmountID?: number;
      /**
       * Entity type: InvoiceDisputeReason
       */
      ReasonID: number;
      ReasonName: string;
      Amount: number;
      Description?: string;
      IsResolved: boolean;
      ResolvedAmount: number;
      Type: InvoiceDisputeTypes;
      /**
       * Entity type: User
       */
      UserID?: number;
      UserFullName?: string;
      EmailAddressToSendNotificationTo?: string;
      ResolveActions: InvoiceDisputeResolveActionDto[];
    }

  }

  /**
   * List the exports for the given Invoice
   */
  export interface ListInvoiceExportsForInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListInvoiceExportsForInvoiceResponse {
    Result?: PagedResult<ListInvoiceExportsForInvoiceResponse.InvoiceExportDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListInvoiceExportsForInvoiceResponse {
    export interface InvoiceExportDto {
      /**
       * Entity type: InvoiceExport
       */
      ID: number;
      Name: string;
      Status: TransputJobStatuses;
      CreationTime: string;
      LastModificationTime?: string;
    }

  }

  /**
   * Get the details of an InvoiceExport
   */
  export interface GetInvoiceExport {
    /**
     * Entity type: InvoiceExport
     */
    ID: number;
  }

  export interface GetInvoiceExportResponse {
    /**
     * Entity type: InvoiceExport
     */
    ID: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    Name: string;
    Status: TransputJobStatuses;
    CreationTime: string;
    LastModificationTime?: string;
    ExternalStatus: InvoiceExportExternalStatus;
    ExternalStatusTime?: string;
    Errors: GetInvoiceExportResponse.GetInvoiceExportError[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetInvoiceExportResponse {
    export interface GetInvoiceExportError {
      Code?: string;
      Description?: string;
      Suggestion?: string;
      Timestamp?: string;
    }

  }

  export const enum InvoiceExportExternalStatus {
    None = 0,
    Pending = 1,
    Completed = 2,
    Failed = -1
  }

  /**
   * Retry the given InvoiceExports
   */
  export interface RetryInvoiceExport {
    /**
     * Entity type: InvoiceExport
     */
    InvoiceExportIDs: number[];
    Force?: boolean;
  }

  /**
   * Get the documents for an export when available
   */
  export interface GetInvoiceExportDocuments {
    /**
     * Entity type: InvoiceExport
     */
    ID: number;
  }

  export interface GetInvoiceExportDocumentsResponse {
    RequestDocuments: TransputJobDocument[];
    ResponseDocuments: TransputJobDocument[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for retrieving a list of available `InvoiceOutputType`'s providing a `Invoice`
   */
  export interface GetInvoiceOutputTypes {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
  }

  export interface GetInvoiceOutputTypesResponse {
    OutputTypes: InvoiceOutputType[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum InvoiceOutputType {
    None = 0,
    Mail = 1,
    PDF = 2,
    Paper = 4,
    Thermal = 8,
    All = 15
  }

  /**
   * Retrieves payments for an order, with context about allocated invoices, filtered on specified optional `InvoiceID`.
   */
  export interface GetInvoicePayments {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
  }

  export interface GetInvoicePaymentsResponse {
    Result: GetInvoicePaymentsResponse.InvoicePayment[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetInvoicePaymentsResponse {
    export interface InvoicePayment {
      /**
       * Entity type: PaymentTransaction
       */
      PaymentTransactionID: number;
      /**
       * Entity type: Invoice
       */
      InvoiceID?: number;
      Amount: number;
      TypeName: string;
      TypeCode: string;
      MethodName: string;
      MethodCode: string;
    }

  }

  /**
   * Allocates a payment transaction to either a specific invoice, or to all open invoices sequentially.
   */
  export interface CreateInvoicePayment {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
  }

  /**
   * List all the blobs that're attached to the given invoice (`InvoiceID`).
   */
  export interface ListBlobsForInvoice {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
  }

  export interface ListBlobsForInvoiceResponse {
    Result: ListBlobsForInvoiceResponse.Model[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListBlobsForInvoiceResponse {
    export interface Model {
      Name: string;
      MimeType: string;
      BlobID: string;
      Url: string;
      CreationTime: string;
      TypeID: number;
    }

  }

  export interface ListControlDocuments {
    PageConfig?: PageConfig<ListControlDocumentsFilter>;
  }

  export interface ListControlDocumentsFilter {
    Types: ControlDocumentTypes[];
    StartDate?: string;
    EndDate?: string;
    Number: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
  }

  export const enum ControlDocumentTypes {
    ControlDocument = 2,
    InitializationDocument = 3,
    ClosingDocument = 4,
    MonthlyControlDocument = 5,
    YearlyControlDocument = 6,
    IntegrityCheckDocument = 7,
    Unknown = 8
  }

  export interface ListControlDocumentsResponse {
    Result?: PagedResult<InvoiceControlDocumentData>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface InvoiceControlDocumentData {
    ID: number;
    Date?: string;
    Number: string;
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    Signature: string;
    ZippedSignature: string;
    Type: ControlDocumentTypes;
  }

  /**
   * List invoice exports
   */
  export interface ListInvoiceExports {
    InitialPageConfig?: PageTokenConfig<ListInvoiceExportFilter>;
    PageToken?: string;
  }

  export interface ListInvoiceExportFilter {
    /**
     * Entity type: InvoiceExport
     */
    ID?: number;
    /**
     * The organization units to list the invoices for, will be ignored if `OrganizationUnitSetID` is specified
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    StartDate?: string;
    EndDate?: string;
    ExternalStatus?: InvoiceExportExternalStatus;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    InvoiceNumber?: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Types?: string[];
  }

  export interface ListInvoiceExportResponse {
    Results?: ListInvoiceExportsDto[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListInvoiceExportsDto {
    /**
     * Entity type: InvoiceExport
     */
    ID?: number;
    Type?: string;
    ExternalStatus?: InvoiceExportExternalStatus;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    InvoiceNumber?: string;
    InvoiceDate?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitName?: string;
    TotalAmount?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    OrderDisplayID?: string;
  }

  /**
   * List ledgers for invoices
   * 
   * Available filters:
   * - InvoiceID
   * - InvoiceLineID
   * - InvoiceDisputeID
   * - Type
   */
  export interface ListInvoiceLedger {
    PageConfig?: PageConfig<ListInvoiceLedgerFilter>;
  }

  export interface ListInvoiceLedgerFilter {
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    /**
     * Entity type: InvoiceLine
     */
    InvoiceLineID?: number;
    /**
     * Entity type: InvoiceDispute
     */
    InvoiceDisputeID?: number;
    /**
     * Entity type: InvoiceLedgerType
     */
    Type?: number;
  }

  export interface ListInvoiceLedgerResponse {
    Result?: PagedResult<ListInvoiceLedgerResponse.InvoiceLedgerDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListInvoiceLedgerResponse {
    export interface InvoiceLedgerDto {
      InvoiceID: number;
      InvoiceLineID?: number;
      InvoiceDisputeID?: number;
      TypeID: number;
      TypeName: string;
      OldValue: string;
      NewValue: string;
      Description: string;
      CreatedByID: number;
      CreatedByFullName: string;
      CreationTime: string;
    }

  }

  export interface ListInvoices {
    PageConfig?: PageConfig<ListInvoiceFilter>;
  }

  export interface ListInvoiceFilter {
    Type?: InvoiceTypes;
    StartDate?: string;
    EndDate?: string;
    Number: string;
    /**
     * The organization units to list the invoices for, will be ignored if `OrganizationUnitSetID` is specified
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs: number[];
    /**
     * The supplier organization units to list the invoices for, will be ignored if `SupplierOrganizationUnitSetID` is specified
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitIDs: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    SupplierOrganizationUnitSetID?: number;
    Status?: InvoiceStatuses;
    /**
     * Entity type: Order
     */
    OrderIDs: number[];
    HasUnresolvedDisputes?: boolean;
    IsMatched?: boolean;
    HasSupplierOrganizationUnit?: boolean;
  }

  export interface ListInvoicesResponse {
    Result?: PagedResult<ListInvoicesDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListInvoicesDto {
    /**
     * Entity type: Invoice
     */
    ID: number;
    InvoiceDate?: string;
    InvoiceNumber: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    BackendID: string;
    OrganizationUnitName: string;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
    SupplierOrganizationUnitName: string;
    Description: string;
    CurrencyID: string;
    TotalAmount: number;
    Signature: string;
    Status: InvoiceStatuses;
    StatusID: number;
    HoldStatusID?: number;
    HasUnresolvedDisputes: boolean;
    /**
     * Entity type: Invoice
     */
    ReferencedInvoiceID?: number;
  }

  /**
   * List manually created invoices. These invoices are created via the `CreateManualInvoice` service and are used as a worst-case scenario when everything has gone offline and the retailer has to make invoices on actual paper. These services are then used to enter the paper invoices back in to EVA.
   */
  export interface ListManualInvoices {
    PageConfig?: PageConfig<ListManualInvoicesFilter>;
  }

  export interface ListManualInvoicesFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Number?: string;
    StartDate?: string;
    EndDate?: string;
  }

  export interface ListManualInvoicesResponse {
    Result?: PagedResult<ManualInvoiceSummary>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ManualInvoiceSummary {
    /**
     * Entity type: Invoice
     */
    ID: number;
    InvoiceDate?: string;
    Number: string;
    Series: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    FiscalID: string;
    Name: string;
  }

  /**
   * List all of the lines of a shipment that have not been completely invoiced.
   */
  export interface ListShipmentLinesToInvoice {
    PageConfig?: PageConfig<ListShipmentLinesToInvoiceFilter>;
  }

  export interface ListShipmentLinesToInvoiceFilter {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
  }

  export interface ListShipmentLinesToInvoiceResponse {
    Result?: PagedResult<ListShipmentLinesToInvoiceResponse.Model>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListShipmentLinesToInvoiceResponse {
    export interface Model {
      ID: number;
      QuantityShipped: number;
      QuantityDelivered: number;
      QuantityInvoiced: number;
      QuantityAvailableForInvoice: number;
      CurrencyID: string;
      UnitPrice: number;
      TaxRate: number;
      TotalAmountInvoiced: number;
      TotalAmountDelivered: number;
      ProductID: number;
      CustomID: string;
      ProductName: string;
      SupplierProductBackendID: string;
      DeliveryType: ShipmentLineDeliveryTypes;
    }

  }

  /**
   * List all of the shipments that have not been completely invoiced.
   */
  export interface ListShipmentsToInvoice {
    PageConfig?: PageConfig<ListShipmentsToInvoiceFilter>;
  }

  export interface ListShipmentsToInvoiceFilter {
    /**
     * Entity type: Shipment
     */
    ID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShipFromOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID?: number;
    From?: string;
    To?: string;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    BackendID: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    OrderBackendID: string;
  }

  export interface ListShipmentsToInvoiceResponse {
    Result?: PagedResult<ListShipmentsToInvoiceResponse.Model>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListShipmentsToInvoiceResponse {
    export interface Model {
      /**
       * Entity type: Shipment
       */
      ID: number;
      /**
       * Entity type: Order
       */
      OrderIDs: number[];
      OrderBackendIDs: string[];
      ShipmentDate: string;
      LastDeliveryDate?: string;
      CompletionTime?: string;
      BackendID: string;
      TrackingCode: string;
      TotalQuantityInvoiced: number;
      TotalQuantityReceived: number;
      TotalAmountReceived: number;
      TotalAmountInvoiced: number;
      BackendReference?: string;
    }

  }

  export interface UpdateInvoiceHoldStatus {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    /**
     * Entity type: InvoiceHoldStatus
     */
    HoldStatusID?: number;
  }

  /**
   * Updates lines on the given invoice (`InvoiceID`).
   * 
   * When the Quantity is 0 and the ExpectedQuantity is `null`, the line will be deleted.
   */
  export interface UpdateInvoiceLines {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    Lines?: UpdateInvoiceLine[];
  }

  export interface UpdateInvoiceLine {
    /**
     * Entity type: InvoiceLine
     */
    InvoiceLineID: number;
    Quantity: number;
    ExpectedQuantity?: number;
    UnitPrice: number;
    /**
     * Range: 1-2
     */
    TaxRate: number;
  }

  /**
   * Updates an existing invoice. See the `CreateInvoice` service for more details about the request.
   */
  export interface UpdateInvoice {
    /**
     * Entity type: Invoice
     */
    ID: number;
    InvoiceDate: string;
    PaymentTermStartDate?: string;
    PaymentTermDueDate?: string;
    InvoiceNumber?: string;
    Description?: string;
    /**
     * Entity type: OrganizationUnit
     */
    CreditorOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    DebtorOrganizationUnitID?: number;
    /**
     * Entity type: InvoiceHoldStatus
     */
    HoldStatusID?: number;
    Blobs?: InvoiceBlobDto[];
    OriginalTotalAmount?: number;
    FiscalID?: string;
    TaxReverseCharge?: boolean;
  }

  /**
   * Updates the invoice user data. To retrieve the information that has been set use: `GetInvoice`
   */
  export interface UpdateInvoiceUserData {
    /**
     * Entity type: Invoice
     */
    InvoiceID: number;
    RemoveCompany?: boolean;
    FirstName?: string;
    LastName?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    FiscalID?: string;
    Address1?: string;
    HouseNumber?: string;
    ZipCode?: string;
    City?: string;
    Subdistrict?: string;
    District?: string;
    State?: string;
    CountryID?: string;
    CompanyName?: string;
    CompanyVatNumber?: string;
    CompanyFiscalID?: string;
  }

  /**
   * Service to validate and upgrade tickets to invoices based on order details.
   */
  export interface UpgradeInvoice {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface UpgradeInvoiceResponse {
    /**
     * Entity type: Invoice
     */
    InvoiceIDs: number[];
    ValidationErrors: UpgradeInvoiceResponse.ValidationError[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace UpgradeInvoiceResponse {
    export interface ValidationError {
      Parameter: string;
      Description?: string;
    }

  }

  /**
   * Check if given `InvoiceNumber` is valid (eg. does not already exist & passes other validations).
   */
  export interface ValidateInvoiceNumber {
    InvoiceNumber: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    Type?: InvoiceTypes;
  }

  export interface ValidateInvoiceNumberResponse {
    IsValid: boolean;
    Invoices: InvoiceDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface InvoiceDto {
    /**
     * Entity type: Invoice
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    CreditorOrganizationUnitID: number;
    /**
     * Entity type: OrganizationUnit
     */
    DebtorOrganizationUnitID?: number;
    PaymentTermStartDate?: string;
    PaymentTermDueDate?: string;
    Type: InvoiceTypes;
    InvoiceDate?: string;
    InvoiceNumber: string;
    Description: string;
    Status: InvoiceStatuses;
    TotalAmount: number;
    CurrencyID: string;
    HoldStatusID?: number;
    /**
     * The TotalAmount on the actual Invoice as provided by the Supplier
     */
    OriginalTotalAmount?: number;
    /**
     * The difference between the TotalAmount and the OriginalTotalAmount
     */
    OriginalTotalAmountDifference?: number;
    /**
     * The ExpectedTotalAmountInTax of all the InvoiceLines + AdditionalAmounts + ExpectedDisputes
     */
    ExpectedTotalAmount?: number;
    /**
     * The difference between the TotalAmount and the ExpectedTotalAmount
     */
    ExpectedTotalAmountDifference?: number;
    TaxReverseCharge: boolean;
    ZippedSignature: string;
    Signature: string;
    EcoTax: number;
  }

  /**
   * Returns inquiry details
   */
  export interface GetInquiry {
    /**
     * Entity type: Inquiry
     */
    ID: number;
  }

  export interface GetInquiryResponse {
    /**
     * Entity type: Inquiry
     */
    ID: number;
    Name: string;
    Description?: string;
    PromptUser: boolean;
    OneTimeUseOnly: boolean;
    /**
     * Entity type: Script
     */
    ScriptID?: number;
    Items: GetInquiryResponse.InquiryItemData[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetInquiryResponse {
    export interface InquiryData {
      /**
       * Entity type: Inquiry
       */
      ID: number;
      Name: string;
      Description?: string;
      Items: GetInquiryResponse.InquiryItemData[];
    }

    export interface InquiryItemData {
      /**
       * Entity type: InquiryItem
       */
      ID: number;
      Name: string;
      Sequence: number;
      IsRequired: boolean;
      Description?: string;
      CustomFieldMetadata: CustomFieldMetadata;
    }

  }

  export interface CustomFieldMetadata {
    /**
     * Entity type: CustomField
     */
    CustomFieldID: number;
    Name: string;
    DisplayName?: string;
    DataType: string;
    DataTypeID: number;
    BackendID?: string;
    IsArray: boolean;
    EnumValues?: { [key: string]: string };
    Order: number;
  }

  /**
   * Returns user inquiries
   */
  export interface GetUserInquiries {
    /**
     * Entity type: User
     */
    UserID?: number;
    IgnoreSubscriptionInquiries?: boolean;
    ShowSubmittedOneTimeUseInquieries?: boolean;
  }

  export interface GetUserInquiriesResponse {
    Inquiries: UserInquiryDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns a user inquiry with data
   */
  export interface GetUserInquiry {
    /**
     * Entity type: Inquiry
     */
    InquiryID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  export interface GetUserInquiryResponse {
    /**
     * Entity type: Inquiry
     */
    ID: number;
    Name: string;
    Description?: string;
    OneTimeUseOnly: boolean;
    PromptUser: boolean;
    SubmittedOn?: string;
    Items: GetUserInquiryResponse.InquiryItemData[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetUserInquiryResponse {
    export interface InquiryItemData {
      /**
       * Entity type: InquiryItem
       */
      ID: number;
      Name: string;
      Sequence: number;
      IsRequired: boolean;
      Description?: string;
      CustomFieldMetadata: CustomFieldMetadata;
      CustomFieldValue: CustomFieldValue;
    }

  }

  /**
   * Submit an inquiry
   */
  export interface SubmitInquiry {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Inquiry
     */
    InquiryID: number;
    /**
     * Range: 0-10
     */
    Score?: number;
    Feedback?: string;
    Items: SubmitInquiry.ItemData[];
  }

  export namespace SubmitInquiry {
    export interface ItemData {
      /**
       * Entity type: InquiryItem
       */
      ID: number;
      Value: CustomFieldValue;
    }

  }

  /**
   * Returns the available assembly names for the device types. Can optionally filter by device types.
   */
  export interface GetAssemblyNamesForDeviceTypes {
    DeviceTypes?: string[];
  }

  export interface GetAssemblyNamesForDeviceTypesResponse {
    Result: GetAssemblyNamesForDeviceTypesResponse.DeviceTypeWithAssemblyName[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAssemblyNamesForDeviceTypesResponse {
    export interface DeviceTypeWithAssemblyName {
      DeviceType: EnumDto;
      AssemblyName: string[];
    }

  }

  /**
   * Service for retrieving a existing `Device`
   */
  export interface GetDevice {
    /**
     * Entity type: Device
     */
    ID: number;
  }

  export interface GetDeviceResponse {
    Result: GetDeviceResponse.DeviceDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetDeviceResponse {
    export interface DeviceDto {
      /**
       * Entity type: Device
       */
      ID?: number;
      Name?: string;
      BackendID?: string;
      /**
       * Entity type: DeviceType
       */
      TypeID?: number;
      Type?: FlagsEnumDto;
      Address?: string;
      /**
       * Entity type: Station
       */
      StationID?: number;
      /**
       * Entity type: PrinterType
       */
      PrinterTypeID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      AssemblyName?: string;
      Data?: string;
      Station?: StationDto;
      OrganizationUnit?: OrganizationUnitDto;
      PrinterType?: DeviceDto.PrinterTypeModel;
    }

    export namespace DeviceDto {
      export interface PrinterTypeModel {
        /**
         * Entity type: PrinterType
         */
        ID: number;
        Name: string;
        Description?: string;
        PaperWidth: number;
      }

    }

  }

  export interface StationDto {
    /**
     * Entity type: Station
     */
    ID: number;
    GlobalID?: string;
    BackendID?: string;
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnit: OrganizationUnitDto;
    OrganizationUnitDescription?: string;
    Devices: DeviceDto[];
    HasPinDevice: boolean;
    HasThermalPrinterDevice: boolean;
    HasFiscalThermalPrinterDevice: boolean;
    HasPrinterDevice: boolean;
    HasCashDrawerDevice: boolean;
    HasZebraPrinterDevice: boolean;
    HasCustomerFacingDisplayDevice: boolean;
    HasSafeDevice: boolean;
    HasVoidDevice: boolean;
    HasScannerDevice: boolean;
    FiscalSystemID?: string;
    Barcode: string;
    IsPointOfSale: boolean;
    CustomFields: CustomFieldResponse[];
    IsFiscalStationInitialized: boolean;
  }

  export interface DeviceDto {
    /**
     * Entity type: Device
     */
    ID?: number;
    GlobalID?: string;
    BackendID?: string;
    Name?: string;
    /**
     * Entity type: DeviceType
     */
    TypeID?: number;
    Address?: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    AssemblyName?: string;
    Data?: { [key: string]: any };
    Type?: FlagsEnumDto;
    OrganizationUnit?: OrganizationUnitDto;
  }

  /**
   * Service for retrieving a list of possible `Device` types
   */
  export interface GetDeviceTypes {
  }

  export interface GetDeviceTypesResponse {
    DeviceTypes: FlagsEnumDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets the possible valid reasons that have been defined for opening the cash drawer manually.
   */
  export interface GetOpenCashDrawerReasons {
  }

  export interface GetOpenCashDrawerReasonsResponse {
    Reasons: GetOpenCashDrawerReasonsResponse.OpenCashDrawerReason[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOpenCashDrawerReasonsResponse {
    export interface OpenCashDrawerReason {
      /**
       * Entity type: OpenCashDrawerReason
       */
      ID: number;
      Name: string;
      Description: string;
      /**
       * Will be localized (if a localized version has been defined) to the culture of your current user.
       */
      LocalizedName: string;
      /**
       * Will be localized (if a localized version has been defined) to the culture of your current user.
       */
      LocalizedDescription: string;
    }

  }

  /**
   * Service for retieving a existing `Station`
   */
  export interface GetStation {
    /**
     * Entity type: Station
     */
    ID: number;
  }

  export interface GetStationResponse {
    /**
     * Entity type: Station
     */
    ID: number;
    GlobalID?: string;
    BackendID?: string;
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    OrganizationUnitBackendID?: string;
    HasPinDevice: boolean;
    HasThermalPrinterDevice: boolean;
    HasFiscalThermalPrinterDevice: boolean;
    HasPrinterDevice: boolean;
    HasCashDrawerDevice: boolean;
    HasZebraPrinterDevice: boolean;
    HasCustomerFacingDisplayDevice: boolean;
    HasSafeDevice: boolean;
    HasVoidDevice: boolean;
    HasScannerDevice: boolean;
    FiscalSystemID?: string;
    Barcode: string;
    IsPointOfSale: boolean;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for retrieving a list of `Device`
   */
  export interface ListDevices {
    PageConfig?: PageConfig<ListDevicesFilter>;
  }

  export interface ListDevicesFilter {
    Name?: string;
    BackendID?: string;
    Address?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: DeviceType
     */
    TypeID?: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: PrinterType
     */
    PrinterTypeID?: number;
    /**
     * Entity type: Device
     */
    DeviceIDs?: number[];
  }

  export interface ListDevicesResponse {
    Result?: PagedResult<ListDevicesModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListDevicesModel {
    /**
     * Entity type: Device
     */
    ID?: number;
    Name?: string;
    BackendID?: string;
    TypeID?: number;
    Type?: FlagsEnumDto;
    Address?: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: PrinterType
     */
    PrinterTypeID?: number;
    AssemblyName?: string;
    Data?: { [key: string]: any };
    CashLimit?: number;
    Station?: ListDevicesModel.StationModel;
    OrganizationUnit?: ListDevicesModel.OrganizationUnitModel;
    PrinterType?: ListDevicesModel.PrinterTypeModel;
  }

  export namespace ListDevicesModel {
    export interface StationModel {
      /**
       * Entity type: Station
       */
      ID: number;
      BackendID?: string;
      Name: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnit?: ListDevicesModel.OrganizationUnitModel;
    }

    export interface OrganizationUnitModel {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      BackendID?: string;
      Name: string;
    }

    export interface PrinterTypeModel {
      /**
       * Entity type: PrinterType
       */
      ID: number;
      Name: string;
      Description?: string;
      PaperWidth: number;
    }

  }

  /**
   * List stations for specified organization unit.
   */
  export interface ListStationsForOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: DeviceType
     */
    DeviceTypeID?: number;
    PageConfig?: PageConfig<ListStationsFilter>;
  }

  export interface ListStationsFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    Name?: string;
    BackendID?: string;
    FiscalSystemID?: string;
    IncludeDeleted?: boolean;
    /**
     * Entity type: DeviceType
     */
    DeviceTypeID?: number;
  }

  export interface ListStationsForOrganizationUnitResponse {
    Result?: PagedResult<StationDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List stations
   * 
   * This service is meant for management purposes, will return all stations you're allowed to view.
   */
  export interface ListStations {
    PageConfig?: PageConfig<ListStationsFilter>;
  }

  export interface ListStationsResponse {
    Result?: PagedResult<ListStationsResponse.Station>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListStationsResponse {
    export interface Station {
      /**
       * Entity type: Station
       */
      ID: number;
      BackendID?: string;
      Name: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      FiscalSystemID?: string;
      IsPointOfSale: boolean;
    }

  }

  export interface SendConfigurationPage {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  /**
   * A specific implementation for `AddProductToOrder` for GiftCards. Adds a new order line for a digital GiftCard to specified order, where
   * `OrderID` identifies the order for the customer and `Amount` is the desired amount to create a GiftCard for.
   * 
   * Optionally supply a `DetailsUrl` pointing to the website you want the buyer of the GiftCard to land to see/manage the order.
   * 
   * Additionally, the `DisplayOptions` property need to be supplied, this contains a bunch of default options to personalize the giftcard.
   * It is also possible to provide some Metadata. We won't do any processing on DisplayOptions.
   * 
   * Except: DetailsUrl will be changed to include the `OrderID` in the URL as well as a `token` that can be used to access a few services.
   */
  export interface AddDigitalGiftCardToOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Range: 1E-06-1000000
     */
    Amount?: number;
    /**
     * The product ID to add as a giftcard. Will fallback to GiftCards:Digital:ProductID
     * Entity type: Product
     */
    ProductID?: number;
    Data?: GiftCardData;
    UpdateUrl?: string;
    GiftCardMedia?: GiftCardMedia;
  }

  /**
   * Checks a balance of given card, where `Handler` or `Barcode` determines the giftcard provider to consult.
   */
  export interface CardBalanceCheck {
    /**
     * Entity type: GiftCardConfiguration
     */
    ConfigurationID?: number;
    CardNumber?: string;
    CardPin?: string;
    Barcode?: string;
    Handler?: string;
    CardSubType?: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  export interface CardBalanceCheckResponse {
    CardID?: string;
    Balance: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * If the total amount on the card can be spend on this order.
     */
    CanSpendAll?: boolean;
    /**
     * The remaining balance on the card when this order is paid.
     */
    RemainingBalance?: number;
    CanRefundWithoutTransaction: boolean;
    BusinessRules?: GiftCardBusinessRulesDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets the GiftCard mail information identified by the `OrderID` and `GiftCardID`.
   */
  export interface GetDigitalGiftCardOptions {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    GiftCardID?: string;
    AccessToken?: string;
  }

  export interface GetDigitalGiftCardOptionsResponse {
    FromEmailAddress?: string;
    ToEmailAddress?: string;
    DeliverySchedule?: string;
    CanBeUpdated?: boolean;
    PhoneNumber?: string;
    /**
     * Available options how a digital giftcards can be delivered
     */
    AvailableCommunicationOptions?: ProductGiftCardCommunicationOptions;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum ProductGiftCardCommunicationOptions {
    Mail = 1,
    Sms = 2
  }

  /**
   * @deprecated Replaced by `GetDigitalGiftCardOptions`
   */
  export interface GetDigitalGiftCardMailOptions {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    GiftCardID?: string;
    AccessToken?: string;
  }

  /**
   * Sets the new mail values for the GiftCard identified by the `OrderID` and `GiftCardID`.
   * 
   * Omitting the `FromEmailAddress` parameter will default to the email address of the customer attached to the order.
   */
  export interface UpdateDigitalGiftCardOptions {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    GiftCardID?: string;
    AccessToken?: string;
    FromEmailAddress?: string;
    ToEmailAddress?: string;
    PhoneNumber?: string;
    DeliverySchedule?: string;
    DeliveryMode?: GiftCardDeliveryMode;
  }

  /**
   * @deprecated Replaced by `UpdateDigitalGiftCardOptions`
   */
  export interface UpdateDigitalGiftCardMailOptions {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    GiftCardID?: string;
    AccessToken?: string;
    FromEmailAddress?: string;
    ToEmailAddress?: string;
    PhoneNumber?: string;
    DeliverySchedule?: string;
    DeliveryMode?: GiftCardDeliveryMode;
  }

  /**
   * Download the GiftCard data for a specific order, based on given `OrderID` and `GiftCardID`. This order needs to be visible for the current user. Requires
   * the order to be shipped (automatically when it is fully paid).
   * 
   * Additionally, this service can be called with an `AccessToken` and `GiftCardID` combination. In this case, no user validation is done - you know both
   * keys to the GiftCard, so you are allowed to get it's information.
   * 
   * If you want, you can specify a `LanguageID` to use for rendering the report - when omitted, the `LanguageID` of the current user.
   * 
   * See `GetDigitalGiftCardsForOrder` service for more details.
   */
  export interface DownloadDigitalGiftCard {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    AccessToken?: string;
    GiftCardID?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
  }

  /**
   * Empty the full amount of a giftcard into an empty order.
   */
  export interface EmptyGiftCard {
    /**
     * Entity type: GiftCardConfiguration
     */
    ConfigurationID: number;
    CardNumber: string;
    Pin?: string;
  }

  export interface EmptyGiftCardResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    AmountPaid: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Retrieve the GiftCard data for a specific order, based on given `OrderID`. This order needs to be visible for the current user.
   * 
   * Will return a list of `GiftCard` data, for every orderline with a GiftCard. Will return the same data supplied in the `AddGiftCardToOrder`
   * service. When the order was shipped (automatically when it is fully paid), the details `GiftCardNumber` and
   * `Pin` are also included. An additional property `IsAvailable` let's you check if it was indeed shipped and paid.
   * 
   * The `GiftCardID` returned on the GiftCard is it's unique identifier, to be used in conjunction with the `AccessToken` to make it globally
   * unique. Using the `AccessToken` and `GiftCardID`, we are pretty sure you are allowed to see the GiftCard data. Use this data to build the link
   * to share directly, or use the approperiate services to mail/text/print this.
   * 
   * Additionally, this service can be called with an `AccessToken` and `GiftCardID` combination, as explained above. In this case, no user validation
   * is done - you know both keys to the GiftCard, so you are allowed to get it's information.
   */
  export interface GetDigitalGiftCardsForOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    AccessToken?: string;
    GiftCardID?: string;
  }

  export interface GetDigitalGiftCardsForOrderResponse {
    Result: GiftCard[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GiftCard {
    IsAvailable: boolean;
    Amount: number;
    CurrencyID: string;
    Identifiers: GiftCardIdentifiers;
    Data: GiftCardData;
    Details: GiftCardDetails;
    IsCleared: boolean;
    IsEmpty: boolean;
    CommunicationOptions?: ProductGiftCardCommunicationOptions;
  }

  export interface GiftCardIdentifiers {
    OrderID: number;
    OrderLineID: number;
    AccessToken: string;
    GiftCardID: string;
  }

  export interface GiftCardDetails {
    GiftCardNumber: string;
    PIN: string;
    DateOfPurchase?: string;
    DateOfExpiration?: string;
  }

  /**
   * Fetch the options for a giftcard to determine the value ranges, currency etc.
   */
  export interface GetGiftCardOptions {
    /**
     * Entity type: GiftCardConfiguration
     */
    GiftCardConfigurationID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    CardNumber?: string;
    /**
     * Uses current currency if not provided
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  export interface GetGiftCardOptionsResponse {
    CardID?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    BusinessRules?: GetGiftCardOptionsResponse.CardBusinessRules;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetGiftCardOptionsResponse {
    export interface CardBusinessRules {
      ActivateValues: number[];
      ActivateValueMin?: number;
      ActivateValueMax?: number;
      PurchaseValueMin?: number;
      PurchaseValueMax?: number;
      Reloadable?: boolean;
      Refundable?: boolean;
    }

  }

  /**
   * This service allows updating giftcard lines created with the `AddGiftCardToOrder` service.
   *  * `Data` will overwrite all the gift card data.
   *  * `ImageWidth` and `ImageHeight` describe the image used
   */
  export interface UpdateDigitalGiftCardOrderLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Range: 1E-06-1000000
     */
    Amount?: number;
    Data?: GiftCardData;
    GiftCardMedia?: GiftCardMedia;
  }

  /**
   * Set the external full stock count result.
   * The service accepts partial or full result lists.
   * The service requires a ActualCompletionTime. There can only be one completion time. The value will be updated with every service call.
   * 
   * CountResults must either have ProductID (preferred) or ProductBarcode.
   */
  export interface SetExternalFullStockCountResult {
    /**
     * Entity type: FullStockCount
     */
    FullStockCountID: number;
    CountCompletionTime: string;
    CountResults: SetExternalFullStockCountResult.CountResult[];
    /**
     * When this property is true, the results will processed as a partial part. Matching values will be updated, new values will be added and existing values no present in the CountResults will be left as is.
     *       When this property is false, the result will be processed as a full set. Matching values will be updated, new values will be added and existing values no present in the CountResults will be removed.
     */
    IsDeltaBatch: boolean;
  }

  export namespace SetExternalFullStockCountResult {
    export interface CountResult {
      /**
       * This optional field can be used to split te count into counted zones.
       */
      Label?: string;
      /**
       * Entity type: StockLabel
       */
      StockLabelID?: string;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      ProductBarcode?: string;
      /**
       * Min: 1
       */
      CountedQuantity?: number;
    }

  }

  /**
   * Download a template format for the full stock count result to upload.
   */
  export interface DownloadExternalFullStockCountResultTemplate {
  }

  /**
   * Upload full stock count result. Previous uploads will be overwritten. The service processes the data async.
   */
  export interface UploadExternalFullStockCountResult {
    /**
     * Entity type: FullStockCount
     */
    FullStockCountID: number;
    CountCompletionTime: string;
    Data: string;
    /**
     * An email address to mail the processing result to. When left empty, the logged in user context will be used
     */
    EmailAddress?: string;
  }

  /**
   * Retrieves details of a cash correction
   */
  export interface GetCashCorrection {
    /**
     * Entity type: CashCorrection
     */
    ID: number;
  }

  export interface GetCashCorrectionResponse {
    /**
     * Entity type: CashCorrection
     */
    ID?: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    /**
     * Entity type: Device
     */
    DeviceID?: number;
    Type?: CashCorrectionTypes;
    Amount?: number;
    Remark?: string;
    CreationTime?: string;
    Device?: GetCashCorrectionResponse.CorrectionDevice;
    CurrencyID?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetCashCorrectionResponse {
    export interface CorrectionDevice {
      /**
       * Entity type: Device
       */
      ID: number;
      Name: string;
      Type: FlagsEnumDto;
      Station?: GetCashCorrectionResponse.CorrectionStation;
    }

    export interface CorrectionStation {
      /**
       * Entity type: Station
       */
      ID: number;
      Name: string;
    }

  }

  export const enum CashCorrectionTypes {
    CashJournalOpening = 1,
    CashJournalClosing = 2
  }

  /**
   * Creates a cash deposit for the provided device, currency and payment type.
   * 
   * When not explicitly delaying the drop-off, it will be booked to the current active cash journal and financial period.
   */
  export interface CreateCashDeposit {
    /**
     * Entity type: Device
     */
    DeviceID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    /**
     * Min: 0
     */
    TotalAmount?: number;
    Coins?: { [key: number]: number };
    BankNotes?: { [key: number]: number };
    /**
     * Required for monetary deposits
     */
    Number?: string;
    /**
     * When set to `true`, the cash deposit remains on this device awaiting manual drop-off; see `DropOffCashDeposit` service
     * @default false
     */
    DelayDropOff?: boolean;
    /**
     * Optional image of the signature
     */
    SignatureBlobID?: string;
    /**
     * Optional url of the signature image
     */
    SignatureBlobUrl?: string;
  }

  export interface CreateCashDepositResponse {
    /**
     * Entity type: CashDeposit
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists all cash deposits for your current financial period.
   */
  export interface ListCashDeposits {
    PageConfig?: PageConfig<ListCashDepositsFilter>;
  }

  export interface ListCashDepositsFilter {
    /**
     * Entity type: Device
     */
    DeviceID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    Statuses?: CashDepositStatus[];
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    Number?: string;
  }

  export const enum CashDepositStatus {
    Done = 0,
    Pending = -1
  }

  export interface ListCashDepositsResponse {
    Result?: PagedResult<ListCashDepositsItem>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListCashDepositsItem {
    /**
     * Entity type: CashDeposit
     */
    ID: number;
    /**
     * Entity type: Device
     */
    DeviceID: number;
    DeviceName: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    PaymentTypeName?: string;
    Number: string;
    Amount: number;
    Status: CashDepositStatus;
    Type?: CashJournalTypes;
    CreatedBy: ListCashDepositsItem.ListCashDepositsItemUser;
    DroppedOffBy?: ListCashDepositsItem.ListCashDepositsItemUser;
    Coins: { [key: number]: number };
    BankNotes: { [key: number]: number };
    CreationTime: string;
    SignatureBlobID?: string;
    SignatureBlobUrl?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
  }

  export namespace ListCashDepositsItem {
    export interface ListCashDepositsItemUser {
      /**
       * Entity type: User
       */
      ID: number;
      Name: string;
      Date: string;
    }

  }

  export const enum CashJournalTypes {
    Default = 0,
    NonMonetary = 1
  }

  /**
   * Deletes a deposit for the given `ID`.
   * 
   * Cash deposits in status Pending (`-1`) can always be removed, status Done (`0`) ("Dropped-off") requires the cash journal and the financial period to still be open.
   */
  export interface DeleteCashDeposit {
    /**
     * Entity type: CashDeposit
     */
    ID: number;
  }

  /**
   * Drops-off a cash deposit previously created and stored on a device (`CreateCashDeposit.DelayDropOff`).
   * 
   * Cash deposits created this way will remain in status Pending (`-1`) until it is either dropped-off (`DropOffCashDeposit`) or deleted (`DeleteCashDeposit`). Whilst
   * pending, it will not create any events, will not be attached to any `CashTransactionLedger` and therefor will not be attached to any `FinancialPeriod`. You can find
   * these pending cash deposits using `ListCashDeposits`, but then only when filtering for a `DeviceID` and/or `OrganizationUnitID` (and optionally `StatusID`).
   */
  export interface DropOffCashDeposit {
    /**
     * Entity type: CashDeposit
     */
    ID: number;
  }

  /**
   * Service for producing a cash deposit document
   */
  export interface ProduceCashDepositDocument {
    /**
     * Entity type: CashDeposit
     */
    ID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * Retrieves details of a cash deposit
   */
  export interface GetCashDeposit {
    /**
     * Entity type: CashDeposit
     */
    ID?: number;
    /**
     * Get a cash deposit by number, for the current organization unit
     */
    Number?: string;
  }

  export interface GetCashDepositResponse {
    Result?: GetCashDepositResponse.CashDepositSummary;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetCashDepositResponse {
    export interface CashDepositSummary {
      /**
       * Entity type: CashDeposit
       */
      ID: number;
      /**
       * Entity type: Device
       */
      DeviceID: number;
      DeviceName: string;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
      /**
       * Entity type: PaymentType
       */
      PaymentTypeID?: number;
      PaymentTypeName?: string;
      Number: string;
      Amount: number;
      Status: CashDepositStatus;
      Type?: CashJournalTypes;
      CreatedBy: GetCashDepositResponse.CashDepositSummaryUser;
      DroppedOffBy?: GetCashDepositResponse.CashDepositSummaryUser;
      Coins: { [key: number]: number };
      BankNotes: { [key: number]: number };
      CreationTime: string;
      SignatureBlobUrl?: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      OrganizationUnitName: string;
      /**
       * @deprecated Use `SignatureBlobUrl` instead
       */
      SignatureUrl: string;
    }

    export interface CashDepositSummaryUser {
      /**
       * Entity type: User
       */
      ID: number;
      Name: string;
      Date: string;
    }

  }

  /**
   * Creates a cash expense.
   * 
   * This service expects:
   * - `DeviceID`, which is the cash device the money was used from.
   * - `TypeID` which is one of the available expense types.
   * - An `Amount`, which is the amount of cash that was taken from or added to the device.
   * The amount should always be positive, but the semantics of the amount depends on the AmountType of the ExpenseType:
   * if ExpenseType.AmountType = Expense, then the amount is booked as an expense, if it's Income then it's booked as income. Either way, it should be a positive number.
   * - A `TaxCodeID`, which refers to one of the available tax codes.
   * - A `Description`, which is the reason for this expense (for example: 'lunch').
   * - A `BlobID`, which contains an image of the receipt.
   * - A `SignatureBlobID`, which contains an image of the signature
   */
  export interface CreateCashExpense {
    /**
     * Entity type: Device
     */
    DeviceID: number;
    /**
     * Entity type: CashExpenseType
     */
    TypeID: number;
    /**
     * Min: 0
     */
    Amount: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID: number;
    Description: string;
    /**
     * Optional image of the receipt
     */
    BlobID?: string;
    /**
     * Optional image of the signature
     */
    SignatureBlobID?: string;
  }

  /**
   * Lists all cash expenses for your current financial period.
   */
  export interface ListCashExpenses {
    PageConfig?: PageConfig<ListCashExpensesFilter>;
  }

  export interface ListCashExpensesFilter {
    DeviceID?: number;
    FinancialPeriodID?: number;
  }

  export interface ListCashExpensesResponse {
    Result?: PagedResult<ListCashExpensesItem>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListCashExpensesItem {
    /**
     * Entity type: CashExpense
     */
    ID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Amount: number;
    TaxCode: EnumDto;
    Type: ListCashExpensesItem.ExpenseType;
    Description: string;
    /**
     * Entity type: Blob
     */
    BlobID?: string;
    BlobUrl?: string;
    CreationTime: string;
    /**
     * Entity type: CashJournal
     */
    CashJournalID: number;
    /**
     * Entity type: CashTransactionLedger
     */
    CashTransactionLedgerID: number;
    /**
     * Entity type: Blob
     */
    SignatureBlobID?: string;
    SignatureBlobUrl: string;
    Device: ListCashExpensesItem.ExpenseDevice;
  }

  export namespace ListCashExpensesItem {
    export interface ExpenseType {
      AmountType: CashExpenseAmountTypes;
      /**
       * Entity type: Enum
       */
      ID: number;
      /**
       * Usually an internal & unchangable code - for display, use `Description`
       */
      Name: string;
      Description: string;
    }

    export interface ExpenseDevice {
      /**
       * Entity type: Device
       */
      ID: number;
      Name: string;
      Type: FlagsEnumDto;
      Station: ListCashExpensesItem.ExpenseStation;
    }

    export interface ExpenseStation {
      /**
       * Entity type: Station
       */
      ID: number;
      Name: string;
    }

  }

  export const enum CashExpenseAmountTypes {
    Expense = 0,
    Income = 1
  }

  /**
   * Deletes an expense for the given `ID`. This is only possible while the cash journal and the financial period are still open.
   */
  export interface DeleteCashExpense {
    /**
     * Entity type: CashExpense
     */
    ID: number;
  }

  /**
   * Service for producing a cash expense document
   */
  export interface ProduceCashExpenseDocument {
    /**
     * Entity type: CashExpense
     */
    ID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * Retrieves details of a cash expense
   */
  export interface GetCashExpense {
    /**
     * Entity type: CashExpense
     */
    ID: number;
  }

  export interface GetCashExpenseResponse {
    /**
     * Entity type: CashExpense
     */
    ID: number;
    /**
     * Entity type: CashJournal
     */
    CashJournalID: number;
    /**
     * Entity type: CashTransactionLedger
     */
    CashTransactionLedgerID: number;
    Amount: number;
    TaxCode: EnumDto;
    Type: GetCashExpenseResponse.ExpenseType;
    Description: string;
    BlobID?: string;
    CreationTime: string;
    SignatureBlobID?: string;
    Device: GetCashExpenseResponse.ExpenseDevice;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetCashExpenseResponse {
    export interface ExpenseType {
      AmountType: CashExpenseAmountTypes;
      /**
       * Entity type: Enum
       */
      ID: number;
      /**
       * Usually an internal & unchangable code - for display, use `Description`
       */
      Name: string;
      Description: string;
    }

    export interface ExpenseDevice {
      /**
       * Entity type: Device
       */
      ID: number;
      Name: string;
      Type: FlagsEnumDto;
      Station?: GetCashExpenseResponse.ExpenseStation;
    }

    export interface ExpenseStation {
      /**
       * Entity type: Station
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Starts the opening of a cash journal for the given DeviceID.
   * 
   * It's only possible to open a cash journal when an open financial period exists and when the device is a cash device. Retrieving a list of valid devices
   * can be accomplished through the `ListDevices` service.
   * 
   * Calling this service does not yet do anything, except return the data you need to open a cash journal:
   * 
   * - The available denominations (coins and notes)
   * - The details about the previous cash journal's closing amounts (if it exists), which can be used as an extra check
   * - Possible pending cash deposits on this device
   */
  export interface StartOpenCashJournal {
    /**
     * Entity type: Device
     */
    DeviceID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
  }

  export interface StartOpenCashJournalResponse {
    CurrencyID: string;
    CountryID: string;
    AvailableCoins: number[];
    AvailableBankNotes: number[];
    Type: CashJournalTypes;
    PaymentType: StartOpenCashJournalResponse.PaymentTypeDto;
    PreviousCashJournal: StartOpenCashJournalResponse.CashJournal;
    PendingCashDeposits: StartOpenCashJournalResponse.PendingCashDeposit[];
    ExpectedOpeningAmount?: number;
    MinimalAllowedDeviation: number;
    MaximalAllowedDeviation: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace StartOpenCashJournalResponse {
    export interface CashJournal {
      /**
       * Entity type: CashJournal
       */
      ID: number;
      OpeningTime: string;
      ClosingTime?: string;
      OpeningAmount: number;
      ClosingAmount?: number;
    }

    export interface PaymentTypeDto {
      /**
       * Entity type: PaymentType
       */
      ID: number;
      Name: string;
      Code: string;
      CashJournalMethod: PaymentCashJournalMethod;
    }

    export interface PendingCashDeposit {
      ID: number;
      Number: string;
      Amount: number;
    }

  }

  export const enum PaymentCashJournalMethod {
    None = 0,
    Close = 1,
    OpenAndClose = 2,
    Ignore = 3
  }

  /**
   * Completes the opening of a cash journal. After this service has been called, it's possible to make cash transactions on this device.
   * 
   * This service expects a list of the coins and notes that were counted.
   * 
   * When the opening amount is different from last period's closing amount, it's required that a correction is provided for this difference
   * in `OpeningDeviationCorrection`. This simply consists of a description and an amount.
   * 
   * For cash journals that either do not use or ignore cash journals, most properties will automatically be set to the expected amounts. Should you
   * want to enforce validation, regardless, specify the `ForceValidation` as `true`.
   * 
   * Pending cash deposits need to be accounted for as well - supply their `ID`s as `PendingCashDeposits`. Do not add the `Amount` of cash deposits
   * to the `TotalAmount`, as the user cannot usually verify the cash deposit content.
   */
  export interface CompleteOpenCashJournal {
    /**
     * Entity type: Device
     */
    DeviceID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
    Coins?: { [key: number]: number };
    BankNotes?: { [key: number]: number };
    /**
     * Min: 0
     */
    TotalAmount: number;
    OpeningDeviationCorrection?: CompleteOpenCashJournal.CashCorrection;
    /**
     * Entity type: CashDeposit
     */
    PendingCashDeposits?: number[];
    ForceValidation?: boolean;
  }

  export namespace CompleteOpenCashJournal {
    export interface CashCorrection {
      Amount: number;
      Description: string;
      /**
       * Entity type: CashCorrectionReason
       */
      ReasonID?: number;
    }

  }

  /**
   * Initiates the closing of a cash journal. This does not yet mutate anything, but returns all the details you need about a cash journal to be able to close it.
   * 
   * This consists of:
   * 
   * - The information about the device and station
   * - Which expenses and deposits were made from this device.
   * - What payments were made.
   * - What the `ExpectedClosingAmount` is, which is a summary of the previous journal's closing amount plus all transactions.
   * - Possible pending cash deposits on this device
   */
  export interface StartCloseCashJournal {
    /**
     * Entity type: Device
     */
    DeviceID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
  }

  export interface StartCloseCashJournalResponse {
    CashJournalDetails: CashJournalDetailsModel;
    PendingCashDeposits: StartCloseCashJournalResponse.PendingCashDeposit[];
    ExpectedClosingAmount: number;
    MinimalAllowedDeviation: number;
    MaximalAllowedDeviation: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace StartCloseCashJournalResponse {
    export interface PendingCashDeposit {
      /**
       * Entity type: CashDeposit
       */
      ID: number;
      Number: string;
      Amount: number;
    }

  }

  export interface CashJournalDetailsModel {
    ID: number;
    OpeningAmount: number;
    ExpectedClosingAmount: number;
    OpeningDeviation?: number;
    OpeningTime: string;
    ClosingTime?: string;
    ClosingAmount?: number;
    Device: CashJournalDetailsModel.CashDevice;
    Type: CashJournalTypes;
    PaymentType: CashJournalDetailsModel.PaymentTypeDto;
    CurrencyID: string;
    CashTransactions: CashJournalDetailsModel.CashTransactionLedger[];
    Expenses: CashJournalDetailsModel.CashExpense[];
    Deposits: CashJournalDetailsModel.CashDeposit[];
    Corrections: CashJournalDetailsModel.CashCorrection[];
    OpeningDenominations: CashJournalDetailsModel.CashDenominations;
    ClosingDenominations: CashJournalDetailsModel.CashDenominations;
    OpenCashJournalsCount: number;
  }

  export namespace CashJournalDetailsModel {
    export interface CashDevice {
      /**
       * Entity type: Device
       */
      ID: number;
      Name: string;
      Type: FlagsEnumDto;
      Station: CashJournalDetailsModel.Station;
    }

    export interface PaymentTypeDto {
      /**
       * Entity type: PaymentType
       */
      ID: number;
      Name: string;
      Code: string;
      CashJournalMethod: PaymentCashJournalMethod;
    }

    export interface Station {
      /**
       * Entity type: Station
       */
      ID: number;
      Name: string;
    }

    export interface CashDeposit {
      ID: number;
      Amount: number;
      Number: string;
      CashTransactionLedger: CashJournalDetailsModel.CashTransactionLedger;
    }

    export interface CashCorrection {
      ID: number;
      Type: CashCorrectionTypes;
      Amount: number;
      Remark: string;
      CashTransactionLedger: CashJournalDetailsModel.CashTransactionLedger;
    }

    export interface CashExpense {
      ID: number;
      Amount: number;
      TaxCode: EnumDto;
      Description: string;
      CreationTime: string;
      BlobID?: string;
    }

    export interface CashTransactionLedger {
      ID: number;
      Description: string;
      Amount: number;
      CreationTime: string;
      Type: CashTransactionLedgerTypes;
      Denominations: CashJournalDetailsModel.CashDenominations;
    }

    export interface CashDenominations {
      Coins: { [key: number]: number };
      BankNotes: { [key: number]: number };
    }

  }

  export const enum CashTransactionLedgerTypes {
    MoveFrom = 0,
    MoveTo = 1,
    Deposit = 2,
    Correction = 3,
    Expense = 4,
    AmountGiven = 5,
    Change = 6
  }

  /**
   * Completes the closing of a cash journal, making the device once again unavailable for cash transactions until a new journal is opened.
   * 
   * This service expects a list of coins and notes, which summed up must equal the given `TotalAmount`.
   * 
   * If the `TotalAmount` is different from what was expected (returned as `ExpectedClosingAmount` in `StartCloseCashJournal` ) then this deviation
   * must be corrected and provided through a list of corrections in the `Corrections` field.
   * 
   * For cash journals that either do not use or ignore cash journals, most properties will automatically be set to the expected amounts. Should you
   * want to enforce validation, regardless, specify the `ForceValidation` as `true`.
   * 
   * Pending cash deposits need to be accounted for as well - supply their `ID`s as `PendingCashDeposits`. Do not add the `Amount` of cash deposits
   * to the `TotalAmount`, as the user cannot usually verify the cash deposit content.
   */
  export interface CompleteCloseCashJournal {
    /**
     * Entity type: Device
     */
    DeviceID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
    Coins?: { [key: number]: number };
    BankNotes?: { [key: number]: number };
    /**
     * Min: 0
     */
    TotalAmount: number;
    Corrections?: CompleteCloseCashJournal.CashCorrection[];
    Data?: { [key: string]: any };
    /**
     * Entity type: CashDeposit
     */
    PendingCashDeposits?: number[];
    ForceValidation?: boolean;
  }

  export namespace CompleteCloseCashJournal {
    export interface CashCorrection {
      Amount: number;
      Description?: string;
      /**
       * Entity type: CashCorrectionReason
       */
      ReasonID?: number;
    }

  }

  /**
   * Gets the currency and a list of available coins and notes for the provided `DeviceID`.
   */
  export interface GetCashJournalDenominations {
    /**
     * Entity type: Device
     */
    DeviceID: number;
  }

  export interface GetCashJournalDenominationsResponse {
    CashJournalDenominations: GetCashJournalDenominationsResponse.CashJournalDenomination[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetCashJournalDenominationsResponse {
    export interface CashJournalDenomination {
      CurrencyID: string;
      AvailableCoins: number[];
      AvailableBankNotes: number[];
    }

  }

  /**
   * Returns information about the current cash journals for the given DeviceID.
   */
  export interface GetCurrentCashJournals {
    /**
     * Entity type: Device
     */
    DeviceID: number;
  }

  export interface GetCurrentCashJournalsResponse {
    CashJournals: GetCurrentCashJournalsResponse.CashJournal[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetCurrentCashJournalsResponse {
    export interface CashJournal {
      /**
       * Entity type: CashJournal
       */
      ID?: number;
      OpeningTime?: string;
      OpeningAmount?: number;
      Type: CashJournalTypes;
      CurrencyID: string;
      PaymentType: GetCurrentCashJournalsResponse.PaymentTypeDto;
      PreviousCashJournal?: GetCurrentCashJournalsResponse.PreviousCashJournal;
      /**
       * Entity type: Device
       */
      DeviceID: number;
      ExpectedClosingAmount?: number;
    }

    export interface PreviousCashJournal {
      /**
       * Entity type: CashJournal
       */
      ID: number;
      OpeningTime: string;
      ClosingTime?: string;
      OpeningAmount: number;
      ClosingAmount?: number;
    }

    export interface PaymentTypeDto {
      /**
       * Entity type: PaymentType
       */
      ID: number;
      Name: string;
      Code: string;
      CashJournalMethod: PaymentCashJournalMethod;
    }

  }

  /**
   * Retrieves details of a cash movement
   */
  export interface GetCashMovement {
    /**
     * Entity type: CashMovement
     */
    ID: number;
  }

  export interface GetCashMovementResponse {
    /**
     * Entity type: CashMovement
     */
    ID: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    /**
     * Entity type: Device
     */
    SourceDeviceID: number;
    /**
     * Entity type: Device
     */
    DestinationDeviceID: number;
    CurrencyID?: string;
    Amount: number;
    SourceDevice: GetCashMovementResponse.MovementDevice;
    DestinationDevice: GetCashMovementResponse.MovementDevice;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetCashMovementResponse {
    export interface MovementDevice {
      /**
       * Entity type: Device
       */
      ID: number;
      Name: string;
      Type: FlagsEnumDto;
      Station?: GetCashMovementResponse.MovementStation;
    }

    export interface MovementStation {
      /**
       * Entity type: Station
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Moves cash!
   */
  export interface MoveCash {
    /**
     * Entity type: Device
     */
    SourceDeviceID?: number;
    /**
     * Entity type: Device
     */
    DestinationDeviceID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    Coins?: { [key: number]: number };
    BankNotes?: { [key: number]: number };
    /**
     * Instead of supplying Coins or BankNotes, supply directly the total.
     */
    Total?: number;
  }

  export interface DownloadFinancialPeriodDeposit {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
  }

  /**
   * Returns all cash expenses that are available for your current OrganizationUnit, or available to the provided OrganizationUnitID (optional).
   */
  export interface GetCashExpenseTypes {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface GetCashExpenseTypesResponse {
    CashExpenseTypes: GetCashExpenseTypesResponse.CashExpenseTypeDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetCashExpenseTypesResponse {
    export interface CashExpenseTypeDto {
      /**
       * Entity type: CashExpenseType
       */
      ID: number;
      BackendID?: string;
      Name: string;
      Description?: string;
      LedgerClassID?: string;
      AmountType: CashExpenseAmountTypes;
      /**
       * Entity type: TaxCode
       */
      TaxCodeID?: number;
      TaxCodeName?: string;
      OrganizationUnitSet?: CashExpenseTypeDto.CashExpenseTypeOrganizationUnitSet;
    }

    export namespace CashExpenseTypeDto {
      export interface CashExpenseTypeOrganizationUnitSet {
        /**
         * Entity type: OrganizationUnitSet
         */
        ID: number;
        Name: string;
      }

    }

  }

  /**
   * Get financial period export documents
   */
  export interface GetFinancialPeriodExportDocuments {
    /**
     * Entity type: FinancialPeriodExport
     */
    ID: number;
  }

  export interface GetFinancialPeriodExportDocumentsResponse {
    RequestDocuments: TransputJobDocument[];
    ResponseDocuments: TransputJobDocument[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Retry financial period export. Use `Force` to retry eventhough the status is `Finished`
   */
  export interface RetryFinancialPeriodExport {
    /**
     * Entity type: FinancialPeriodExport
     */
    ID: number;
    Force?: boolean;
  }

  /**
   * Opens a financial period for your current OrganizationUnit.
   */
  export interface OpenFinancialPeriod {
    /**
     * If there is an impediment to opening the period and it allows being ignored then passing the name of the impediment type in this collection will not block opening the period on that impediment. Also see `GetFinancialPeriodOpeningImpediments`.
     */
    OpeningImpedimentTypesToIgnore?: string[];
  }

  /**
   * Closes the financial period for your current OrganizationUnit.
   */
  export interface CloseFinancialPeriod {
    /**
     * If there is an impediment to closing the period and it allows being ignored then passing the name of the impediment type in this collection will not block closing the period on that impediment. Also see `GetFinancialPeriodClosingImpediments`.
     */
    ClosingImpedimentTypesToIgnore?: string[];
  }

  /**
   * Set the answer to the PCI question for the provided `FinancialPeriodID`.
   */
  export interface SetPCIQuestionAnswerForFinancialPeriod {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    Answer: boolean;
  }

  /**
   * Set the employees for the financial period. This only works for pending financial periods.
   */
  export interface SetEmployeesForFinancialPeriod {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    /**
     * The employees to set for the financial period. Can be left empty to remove all employees.
     */
    Employees?: SetEmployeesForFinancialPeriod.FinancialPeriodEmployee[];
  }

  export namespace SetEmployeesForFinancialPeriod {
    export interface FinancialPeriodEmployee {
      /**
       * Entity type: User
       */
      UserID: number;
      /**
       * Share of the commission fo this financial period. Either provide this for all employees or for none. It must of course add up to 1
       * Range: 0-1
       */
      CommissionShare?: number;
    }

  }

  /**
   * Set the loyalty program for the financial period. This only works for pending financial periods.
   */
  export interface SetLoyaltyProgramForFinancialPeriod {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    /**
     * The loyalty program to set for the financial period. Can be left empty to remove the loyalty program.
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
  }

  /**
   * Gets a detailed overview of the provided `FinancialPeriodID`. If no `FinancialPeriodID` is provided, the current period for your user is used.
   */
  export interface GetFinancialPeriodInformation {
    /**
     * If left null, defaults to the current period of the logged in user.
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
  }

  export interface GetFinancialPeriodInformationResponse {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    Status: FinancialPeriodStatus;
    OrganizationUnit: GetFinancialPeriodInformationResponse.FinancialPeriodOrganizationUnit;
    TotalRevenue: number;
    CreationTime: string;
    ExpectedClosingTime?: string;
    ShouldHaveBeenClosed: boolean;
    PCIQuestionAnswer?: boolean;
    CashJournals: CashJournalDetailsModel[];
    PaymentTypeSummaries: GetFinancialPeriodInformationResponse.PaymentTypeSummary[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetFinancialPeriodInformationResponse {
    export interface FinancialPeriodOrganizationUnit {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
    }

    export interface PaymentTypeSummary {
      Type: GetFinancialPeriodInformationResponse.PaymentType;
      TotalPaidAmount: number;
    }

    export interface PaymentType {
      /**
       * Entity type: PaymentType
       */
      ID: number;
      Name: string;
    }

  }

  export const enum FinancialPeriodStatus {
    Open = 0,
    Closing = 1,
    Closed = 2,
    Processed = 3,
    Pending = -1
  }

  /**
   * Gets a list of reasons (if any) why this period cannot be opened yet.
   * 
   * The current possible reasons are (which is available in the `Type` property of each item):
   * 
   * ## LINKED_EMPLOYEES
   * 
   * Employees should be linked to this financial periods (with an optional commission share).
   * 
   * ### Data
   * 
   * No Data.
   * 
   * ## LOYALTY_PROGRAM
   * 
   * A loyalty program should be linked to this financial period.
   * 
   * ### Data
   * 
   * No Data.
   * 
   * ## MOBILE_ORGANIZATION_UNIT_LOCATION
   * 
   * This organization unit should get an updated location before starting this financial period.
   * 
   * ### Data
   * 
   * No Data.
   */
  export interface GetFinancialPeriodOpeningImpediments {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
  }

  export interface GetFinancialPeriodOpeningImpedimentsResponse {
    Impediments: FinancialPeriodImpediment[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FinancialPeriodImpediment {
    Type: string;
    Description: string;
    AllowIgnore: boolean;
    Data: { [key: string]: any };
  }

  /**
   * Gets a list of reasons (if any) why this period cannot be closed yet.
   * 
   * The current possible reasons are (which is available in the `Type` property of each item):
   * 
   * ## CLOCKED_IN_USER
   * 
   * The user is still clocked in and must be clocked out.
   * 
   * ### Data
   * 
   * The `Data` property will contain:
   * 
   * ```json
   * {
   *   "UserID" : 1,
   *   "ClockedIn" : "2018-08-01 12:00",
   *   "FirstName" : "John",
   *   "LastName"  : "Doe"
   * }
   * ```
   * 
   * ## ACTIVE_USER_TASK
   * 
   * The user task hasn't been completed yet and it's either attached to this financial period, or its past its deadline.
   * 
   * ### Data
   * 
   * The `Data` property will contain:
   * 
   * ```json
   * {
   * 	"UserTaskID" : 5546,
   * 	"UserTaskTypeID" : 9,
   * 	"UserID" : null,
   * 	"UserFullName" : null,
   * 	"IsActive" : true,
   * 	"IsCompleted" : false,
   * 	"Deadline" : null,
   * 	"CreationTime" : "2018-08-01"
   * }
   * ```
   * 
   * ## OPEN_CASH_JOURNAL
   * 
   * There is still an open cash journal that needs to be closed before the period can be closed.
   * 
   * ### Data
   * 
   * The `Data` property will contain:
   * 
   * ```json
   * {
   * "CashJournalID" : 3,
   * "DeviceID" : 1,
   * "OpeningAmount" : 150.15
   * }
   * ```
   * 
   * ## PCI_QUESTION
   * 
   * The PCI compliancy question has not been answered yet, this is required to close the period if the OrganizationUnit has been configured to require it.
   * 
   * ### Data
   * 
   * No Data.
   * 
   * ## NEGATIVE_STOCK
   * 
   * There is negative stock that must be corrected (to at least zero) before the period can be closed.
   * 
   * ### Data
   * 
   * The `Data` property will contain:
   * 
   * ```json
   * {
   *   "ProductID" : 7684,
   *   "ProductBackendID" : "ABC123",
   *   "QuantityOnHand" : -10,
   *   "StockLabelID" : 1,
   *   "StockLabelName" : "Sellable",
   *   "OrganizationUnitID" : 182
   * }
   * ```
   * 
   * This data can then be used to correct the stock through AdjustStock/MoveStock.
   */
  export interface GetFinancialPeriodClosingImpediments {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
  }

  export interface GetFinancialPeriodClosingImpedimentsResponse {
    Impediments: FinancialPeriodImpediment[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets information regarding financial period stock
   */
  export interface GetFinancialPeriodClosingStockInformation {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  export interface GetFinancialPeriodClosingStockInformationResponse {
    PendingShipmentLines: GetFinancialPeriodClosingStockInformationResponse.PendingShipmentLine[];
    CurrentStock: GetFinancialPeriodClosingStockInformationResponse.CurrentStockForLabel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetFinancialPeriodClosingStockInformationResponse {
    export interface CurrentStockForLabel {
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
      StockLabelName: string;
      QuantityOnHand: number;
    }

    export interface PendingShipmentLine {
      /**
       * Entity type: Shipment
       */
      ShipmentID: number;
      ShipmentBackendID: string;
      TrackingCode: string;
      TrackingLink: string;
      /**
       * Entity type: ShipmentLine
       */
      ShipmentLineID: number;
      QuantityShipped: number;
      QuantityDelivered: number;
      /**
       * Entity type: Product
       */
      ProductID: number;
      ReceiveShipmentUserTaskID?: number;
    }

  }

  /**
   * Get status summary for the devices of a FinancialPeriod
   */
  export interface GetFinancialPeriodCashJournalDeviceSummary {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
  }

  export interface GetFinancialPeriodCashJournalDeviceSummaryResponse {
    Devices: GetFinancialPeriodCashJournalDeviceSummaryResponse.DeviceSummary[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetFinancialPeriodCashJournalDeviceSummaryResponse {
    export interface DeviceSummary {
      /**
       * Entity type: Device
       */
      ID: number;
      DeviceName: string;
      StationName?: string;
      DeviceType: FlagsEnumDto;
      Status: DeviceStatus;
    }

  }

  export const enum DeviceStatus {
    Unknown = 0,
    Open = 1,
    Opening = 2,
    Closed = 3,
    Missing = 4
  }

  /**
   * Retrieves the current available cash amount of given device, to be able to e.g. validate user given amounts in `CreateCashDeposit` or `CreateCashExpense` services.
   * 
   * Supply a `CurrencyID` and `PaymentTypeID` to indicate the Cash Journal to be used.
   */
  export interface GetAvailableCashAmountForDevice {
    /**
     * Entity type: Device
     */
    DeviceID: number;
    /**
     * The currency to get the available cash amount for.
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * The payment type to get the available cash amount for.
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
  }

  export interface GetAvailableCashAmountForDeviceResponse {
    Amount: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    ValidateSafeBalance: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns a summary of the current financial period of the organization unit that the user is currently logged on to. The
   * result of this service can be used to retrieve more details of the financial period via the
   * `GetFinancialPeriodDetails` service.
   */
  export interface GetCurrentFinancialPeriodSummary {
  }

  export interface GetCurrentFinancialPeriodSummaryResponse {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    FinancialPeriodStatus: FinancialPeriodStatus;
    ShouldHaveBeenClosed: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetFinancialPeriodAccountsSummary {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
  }

  export interface GetFinancialPeriodAccountsSummaryResponse {
    Accounts: AccountSummary[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface AccountSummary {
    /**
     * Entity type: Account
     */
    AccountID: number;
    Name: string;
    Amount: number;
    ObjectAccount: string;
    Subsidiary: string;
  }

  /**
   * Returns a `FinancialPeriodDetailsDto` that contains details about the given financial period (`FinancialPeriodID`).
   */
  export interface GetFinancialPeriodDetails {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
  }

  export interface GetFinancialPeriodDetailsResponse {
    Details: FinancialPeriodDetailsDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FinancialPeriodDetailsDto {
    /**
     * Entity type: FinancialPeriod
     */
    PreviousPeriodID?: number;
    OpeningTime?: string;
    ClosingTime?: string;
    Status: FinancialPeriodStatus;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    CashDeposits: CashDepositDto[];
    CashJournals: CashJournalDto[];
    PreviousCashJournal: CashJournalDto;
    Expenses: ExpenseDto[];
    CashCorrections: CashCorrectionDto[];
    PaymentTotalPerTypes: PaymentTotalPerPaymentTypeDto[];
    ExtendedStations: ExtendedStation[];
    Devices: DeviceWithCashJournals[];
    LoyaltyProgram?: LoyaltyProgramDetails;
    Employees: LinkedEmployee[];
    AmountOfRoundingDifference: number;
    AmountOfOpeningDifference: number;
    AmountOfCashDifference: number;
    TotalDepositedAmounts: CurrencyTotalAmount[];
    TotalAmountOfExpenses: number;
    TotalAmountOfCorrections: number;
    TotalRevenue: number;
    TotalOrderAmountInTax: number;
    TotalOrderAmount: number;
    TotalReturnAmountInTax: number;
    TotalReturnAmount: number;
    TotalPaidOrders: number;
    TotalReturnedOrders: number;
    TotalSoldProducts: number;
    TotalReturnedProducts: number;
    TotalNonSystemPayments: number;
    TotalSystemPayments: number;
    PCIQuestionAnswer?: boolean;
    /**
     * ID of the user who closed the period
     */
    UserID?: number;
    /**
     * Full name of the user who closed the period
     */
    UserFullName?: string;
    /**
     * ID of the user who opened the period
     */
    OpenedByUserID?: number;
    /**
     * Full name of the user who opened the period
     */
    OpenedByUserFullName?: string;
  }

  export interface CashDepositDto {
    Number: string;
    Description: string;
    Total: number;
    Lines: CashDepositLineDto[];
    ID: number;
    CashJournalID: number;
  }

  export interface CashDepositLineDto {
    Currency: number;
    Quantity: number;
  }

  export interface CashJournalDto {
    OpeningAmount: number;
    ClosingAmount?: number;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreatedByFullName: string;
    Type: CashJournalTypes;
    /**
     * Entity type: CashJournal
     */
    ID: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: Device
     */
    DeviceID: number;
    DeviceName: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID: number;
    OpeningTime: string;
    ClosingTime?: string;
    OpeningDeviation?: number;
    ClosingDeviation?: number;
    PaymentType: CashJournalPaymentType;
    PreviousCashJournal?: PreviousCashJournal;
  }

  export interface CashJournalPaymentType {
    ID: number;
    Name: string;
    Code: string;
    CashJournalMethod: PaymentCashJournalMethod;
  }

  export interface PreviousCashJournal {
    ID: number;
    OpeningTime: string;
    ClosingTime?: string;
    OpeningAmount: number;
    ClosingAmount?: number;
  }

  export interface ExpenseDto {
    /**
     * Entity type: CashExpense
     */
    ID: number;
    /**
     * Entity type: CashExpenseType
     */
    TypeID: number;
    TypeDescription: string;
    TaxRate: number;
    Amount: number;
    Description: string;
    CreationTime: string;
    BlobID?: string;
    AmountType: CashExpenseAmountTypes;
    /**
     * Entity type: CashJournal
     */
    CashJournalID: number;
  }

  export interface CashCorrectionDto {
    /**
     * Entity type: CashCorrection
     */
    ID: number;
    Type: CashCorrectionTypes;
    Amount: number;
    Remark: string;
    CreationTime: string;
    /**
     * Entity type: CashJournal
     */
    CashJournalID: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
    StationName?: string;
    /**
     * Entity type: Device
     */
    DeviceID: number;
    DeviceName: string;
    /**
     * Entity type: DeviceType
     */
    DeviceTypeID: number;
    DeviceTypeName: string;
  }

  export interface PaymentTotalPerPaymentTypeDto {
    Total: number;
    IsSystem: boolean;
    TypeName: string;
  }

  export interface StationClosing {
    /**
     * Entity type: Station
     */
    StationID?: number;
    InitialAmount: number;
    Name: string;
    Type: LegacyCashJournalTypes;
    TypeName: string;
    ClosingAmount: number;
  }

  export const enum LegacyCashJournalTypes {
    Register = 0,
    Safe = 1
  }

  export interface ExtendedStation {
    /**
     * Entity type: Station
     */
    ID: number;
    Name: string;
    BackendID: string;
    CashJournals: CashJournalDto[];
    FirstOpeningAmount?: number;
    LastClosingAmount?: number;
  }

  export interface DeviceWithCashJournals {
    /**
     * Entity type: Device
     */
    ID: number;
    DeviceName: string;
    DeviceTypeID: number;
    DeviceTypeName: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
    StationName?: string;
    CashJournals: CashJournalDto[];
    Status: DeviceStatus;
  }

  export interface LoyaltyProgramDetails {
    /**
     * Entity type: LoyaltyProgram
     */
    ID: number;
    Name: string;
  }

  export interface LinkedEmployee {
    /**
     * Entity type: User
     */
    ID: number;
    CommissionShare?: number;
  }

  export interface CurrencyTotalAmount {
    Currency: string;
    TotalAmount: number;
  }

  export interface GeneralLedgerDto {
    ID: number;
    OrganizationUnitID: number;
    OrganizationUnit: OrganizationUnitDto;
    Amount: number;
    Remark: string;
    PaymentTransactionID?: number;
    PaymentTransaction: PaymentTransactionDto;
    CreationTime: string;
    OrderID?: number;
    AccountName: string;
    AccountObjectAccount: string;
    AccountingID: number;
    StockMutationID?: number;
    StockMutation: StockMutationDto;
    CurrencyID: string;
  }

  /**
   * Lists all of the bookings that've been made for the given financial period. Which you can retrieve
   * by setting the filter `FinancialPeriodID` on the `PageConfig`.
   */
  export interface ListBookings {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListBookingsResponse {
    Result?: PagedResult<GeneralLedgerDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This service allows you to view the cash transaction ledger, which underlies all cash transactions,
   * such as received payments, moves between devices and expenses.
   * 
   * By default, you only get access to the cash transaction ledger for your current financial period. If
   * you have the `ManageCashTransactionLedger` permission, you're also allowed to filter on another financial
   * period.
   */
  export interface ListCashTransactionLedger {
    PageConfig?: PageConfig<ListCashTransactionLedgerFilter>;
  }

  export interface ListCashTransactionLedgerFilter {
    Types: CashTransactionLedgerTypes[];
    CurrencyID?: string;
    DeviceID?: number;
    CashJournalID?: number;
    FinancialPeriodID?: number;
    PaymentTypeID?: number;
    FromDate?: string;
    ToDate?: string;
  }

  export interface ListCashTransactionLedgerResponse {
    Result?: PagedResult<ListCashTransactionLedgerItem>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListCashTransactionLedgerItem {
    ID: number;
    Amount: number;
    DeviceName: string;
    DeviceTypeID: number;
    StationName: string;
    Description: string;
    Type: CashTransactionLedgerTypes;
    CreationTime: string;
    GroupID?: string;
    CurrencyID: string;
    PaymentTypeID: number;
    PaymentTypeName: string;
    CreatedByID: number;
    CreatedByFullName: string;
    Details: string;
    Coins: { [key: number]: number };
    BankNotes: { [key: number]: number };
  }

  /**
   * This service allows you to get a grouped view of the cash transaction ledger, which underlies all cash
   * transactions, such as received payments, moves between devices and expenses.
   * 
   * By default, you only get access to the cash transaction ledger for your current financial period. If
   * you have the `ManageCashTransactionLedger` permission, you're also allowed to filter on another financial
   * period.
   */
  export interface ListCashTransactionLedgerGroups {
    PageConfig?: PageConfig<ListCashTransactionLedgerFilter>;
  }

  export interface ListCashTransactionLedgerGroupsResponse {
    Result?: PagedResult<ListCashTransactionLedgerGroup>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListCashTransactionLedgerGroup {
    ID: string;
    Items: ListCashTransactionLedgerItem[];
  }

  /**
   * List financial events.
   */
  export interface ListFinancialEvents {
    InitialPageConfig?: PageTokenConfig<ListFinancialEventsFilter>;
    PageToken?: string;
  }

  export interface ListFinancialEventsFilter {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Order
     */
    OrderIDs?: number[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    FromDate?: string;
    ToDate?: string;
    Type?: FinancialEventTypes;
    StatusID?: number;
  }

  export interface ListFinancialEventsResponse {
    Results?: ListFinancialEventRow[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListFinancialEventRow {
    /**
     * Entity type: FinancialEvent
     */
    ID: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: StockMutation
     */
    StockMutationID?: number;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * Entity type: TaxCode
     */
    TaxCodeID?: number;
    OrganizationUnitName: string;
    TypeID: number;
    CurrencyID: string;
    Amount: number;
    Remark: string;
    StatusID: number;
    OrderDisplayID?: string;
    ProcessingError: string;
    TaxRate?: number;
    TaxAmount?: number;
    ExchangeRate?: number;
    TaxRateCategory?: TaxRateCategory;
    Type: FinancialEventTypes;
    Status: FinancialEventStatuses;
  }

  export const enum FinancialEventStatuses {
    Unprocessed = 0,
    Processed = 1,
    Ignored = 2,
    NoProcessingRequired = 3,
    NoMatchingRecipe = 4
  }

  /**
   * List financial period exports
   */
  export interface ListFinancialPeriodExports2 {
    PageConfig?: PageConfig<ListFinancialPeriodExports2Filter>;
  }

  export interface ListFinancialPeriodExports2Filter {
    /**
     * Entity type: FinancialPeriodExport
     */
    ID?: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    Status?: TransputJobStatuses;
    Name?: string;
    From?: string;
    To?: string;
  }

  export interface ListFinancialPeriodExports2Response {
    Result?: PagedResult<ListFinancialPeriodExports2Response.FinancialPeriodExport>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListFinancialPeriodExports2Response {
    export interface FinancialPeriodExport {
      /**
       * Entity type: FinancialPeriodExport
       */
      ID: number;
      Name: string;
      Trigger: FinancialPeriodExportTrigger;
      Status: TransputJobStatuses;
      CreationTime: string;
      LastModificationTime?: string;
      /**
       * Entity type: FinancialPeriod
       */
      FinancialPeriodID: number;
    }

  }

  export const enum FinancialPeriodExportTrigger {
    PeriodOpened = 0,
    PeriodClosed = 1,
    CashJournalOpened = 2,
    CashJournalClosed = 3,
    CashDeposit = 4,
    CashExpense = 5,
    CashMovement = 6,
    Difference = 7
  }

  /**
   * List financialperiods for an OrganizationUnit
   */
  export interface ListFinancialPeriods {
    PageConfig?: PageConfig<ListFinancialPeriodsFilter>;
  }

  export interface ListFinancialPeriodsFilter {
    IDs?: number[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    OpeningTimeFrom?: string;
    OpeningTimeTo?: string;
    ClosingTimeFrom?: string;
    ClosingTimeTo?: string;
    /**
     * Filter on the FinancialPeriodStatuses
     */
    Status?: FinancialPeriodStatus[];
  }

  export interface ListFinancialPeriodsResponse {
    Result?: PagedResult<FinancialPeriodDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FinancialPeriodDto {
    /**
     * Entity type: FinancialPeriod
     */
    ID: number;
    OpeningTime?: string;
    ClosingTime?: string;
    Status: FinancialPeriodStatus;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    UserFullName?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: Currency
     */
    CostPriceCurrencyID: string;
    /**
     * Entity type: User
     */
    OpenedByUserID?: number;
    OpenedByUserFullName?: string;
  }

  /**
   * Returns a paged view of all the general ledgers.
   * 
   * The page size `Limit` has a maximum value of `1.000` for this service.
   */
  export interface ListGeneralLedgers {
    InitialPageConfig?: PageTokenConfig<ListGeneralLedgersFilter>;
    PageToken?: string;
  }

  export interface ListGeneralLedgersFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * The organization units to list the invoices for, will be ignored if `OrganizationUnitSetID` is specified
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: Account
     */
    AccountID?: number;
    IsProcessed?: boolean;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    CreationTime?: string;
    FromDate?: string;
    ToDate?: string;
    ObjectAccount?: string;
    Amount?: number;
  }

  export interface ListGeneralLedgersResponse {
    Result: PagedResult<ListGeneralLedgersResponse.GeneralLedger>;
    Results?: ListGeneralLedgersResponse.GeneralLedger[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListGeneralLedgersResponse {
    export interface GeneralLedger {
      AccountName: string;
      AccountObjectAccount: string;
      Amount: number;
      CreationTime: string;
      CurrencyID: string;
      /**
       * Entity type: FinancialEvent
       */
      FinancialEventID?: number;
      /**
       * Entity type: FinancialPeriod
       */
      FinancialPeriodID: number;
      /**
       * Entity type: GeneralLedger
       */
      ID: number;
      Offset1: string;
      Offset2: string;
      Offset3: string;
      Offset4: string;
      Offset5: string;
      Offset6: string;
      Offset7: string;
      Offset8: string;
      Offset9: string;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      OrderDisplayID?: string;
      OrganizationUnit: ListGeneralLedgersResponse.OrganizationUnit;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID: number;
      /**
       * Entity type: PaymentTransaction
       */
      PaymentTransactionID?: number;
      Remark: string;
      /**
       * Entity type: StockMutation
       */
      StockMutationID?: number;
    }

    export interface OrganizationUnit {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Get the estimated amount of unprocessed financial events.
   */
  export interface GetUnprocessedFinancialEventsCount {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    Filter?: ListFinancialEventsFilter;
  }

  export interface GetUnprocessedFinancialEventsCountResponse {
    Count: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Process unprocessed financial events.
   */
  export interface ProcessUnprocessedFinancialEvents {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    Filter?: ListFinancialEventsFilter;
  }

  export interface RepostFinancialPeriod {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
  }

  /**
   * This service resets processed financial events back to unprocessed,
   * which means all associated financial ledgers are deleted.
   * 
   * Only financial events of periods that have not been fully processed (for example exported to an external ERP system)
   * can be reset.
   * 
   * As this can be a time consuming service, the request is handled in the background and upon completion a mail
   * is sent to the caller of this service with the results.
   * 
   * The scope of which events to reset can be limited by specifying a `Filter` in the request. Only the events
   * that match the filter will be reset.
   */
  export interface ResetFinancialEvents {
    Filter?: ListFinancialEventsFilter;
  }

  /**
   * Retry financial period exports.
   */
  export interface RetryFinancialPeriodExports {
    ToRetry?: RetryFinancialPeriodExports.FinancialPeriodExportToRetry[];
  }

  export namespace RetryFinancialPeriodExports {
    export interface FinancialPeriodExportToRetry {
      /**
       * Entity type: FinancialPeriodExport
       */
      ID: number;
      Force?: boolean;
    }

  }

  /**
   * Create a cash correction reason
   */
  export interface CreateCashCorrectionReason {
    Name: string;
    BackendID?: string;
    Sequence: number;
  }

  export interface CreateCashCorrectionReasonResponse {
    /**
     * Entity type: CashCorrectionReason
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Delete a cash correction reason
   */
  export interface DeleteCashCorrectionReason {
    /**
     * Entity type: CashCorrectionReason
     */
    ID: number;
  }

  /**
   * Get cash correction reason by it's ID
   */
  export interface GetCashCorrectionReason {
    /**
     * Entity type: CashCorrectionReason
     */
    ID: number;
  }

  export interface GetCashCorrectionReasonResponse {
    /**
     * Entity type: CashCorrectionReason
     */
    ID: number;
    Name: string;
    TranslatedName: string;
    BackendID?: string;
    Sequence: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get all cash correction reasons
   */
  export interface GetCashCorrectionReasons {
  }

  export interface GetCashCorrectionReasonsResponse {
    Reasons: GetCashCorrectionReasonsResponse.CashCorrectionReasonData[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetCashCorrectionReasonsResponse {
    export interface CashCorrectionReasonData {
      /**
       * Entity type: CashCorrectionReason
       */
      ID: number;
      Name: string;
      BackendID?: string;
    }

  }

  /**
   * Update a cash correction reason
   */
  export interface UpdateCashCorrectionReason {
    /**
     * Entity type: CashCorrectionReason
     */
    ID: number;
    Name?: string;
    BackendID?: string;
    Sequence?: number;
  }

  /**
   * List the active feed subscriptions for the current user's `ApplicationID`.
   */
  export interface ListFeedSubscriptions {
  }

  export interface ListFeedSubscriptionsResponse {
    Result: FeedSubscriptionDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FeedSubscriptionDto {
    ID: number;
    Name: string;
    WebhookUrl: string;
  }

  /**
   * Subscribe to the specified feed for the current User's `ApplicationID`.
   * 
   * `Name` is the name of the file you wish to watch. The `WebhookUrl` is called with a JSON POST request, in which the Feed's `name` and `applicationID` are specified. This request does not contain the data
   * itself, your application will have to fetch the data using your current approaches to download this feed. This may seem verbose, but we have no guarantee the endpoint is secure, and we can't just send
   * possibly sensitive information there!
   * 
   * To unsubscribe, call the `UnsubscribeFromFeed` service with the returned `ID` value.
   */
  export interface SubscribeToFeed {
    Name?: string;
    WebhookUrl?: string;
  }

  export interface SubscribeToFeedResponse {
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Unsubscribe from the specified feed.
   */
  export interface UnsubscribeFromFeed {
    /**
     * Entity type: FeedSubscription
     */
    ID: number;
  }

  /**
   * Create event export configurations (HTTPS only).
   * 
   * Event Export Configurations allow you to configure specific endpoints (webhooks) for EventTargets and EventTypes.
   * 
   * - Not setting EventTarget & EventType will configure the endpoint for all targets and types
   * - Set AuthenticationHeaderType and AuthenticationHeaderValue if you want to configure some sort of authentication
   * - Make sure to catch & store the `SecretKey1` and `SecretKey2`, as there will be no other way of retrieving them after this service call!
   */
  export interface CreateEventExportConfiguration {
    /**
     * A name to identify the configuration.
     */
    Name: string;
    /**
     * The configuration of the endpoint attached to this EventExport.
     * Entity type: EndpointConfiguration
     */
    EndpointConfigurationID: number;
    /**
     * The status of the configuration.
     */
    Status: EventExportConfigurationStatus;
    /**
     * Additional configuration for the endpoint.
     */
    Config?: EventExportConfigurationConfigModel;
  }

  export const enum EventExportConfigurationStatus {
    Enabled = 1,
    Disabled = 2,
    Error = 3,
    Done = 4
  }

  export interface EventExportConfigurationConfigModel {
    /**
     * Authorization configuration used when calling this endpoint.
     */
    Authorization?: EventExportConfigurationConfigAuthorizationModel;
    /**
     * Defines what kind of response EVA requires from this endpoint.
     */
    ResponseMode?: EventExportConfigurationResponseMode;
    /**
     * Filter for certain message types. If not set, all message types will be sent.
     */
    MessageTypes?: EventExportConfigurationTarget[];
    /**
     * Exclude certain message types. If not set, no message types will be excluded.
     */
    ExcludedMessageTypes?: EventExportConfigurationTarget[];
    /**
     * Filter for certain organization units. If not set, all organization units will be sent.
     * Entity type: OrganizationUnit
     */
    IncludeOrganizationUnits?: number[];
    /**
     * Exclude certain organization units. If not set, no organization units will be excluded.
     * Entity type: OrganizationUnit
     */
    ExcludeOrganizationUnits?: number[];
    /**
     * Filter for certain organization unit sets. If not set, all organization units will be sent.
     * Entity type: OrganizationUnitSet
     */
    IncludeOrganizationUnitSets?: number[];
    /**
     * Exclude certain organization unit sets. If not set, no organization units will be excluded.
     * Entity type: OrganizationUnitSet
     */
    ExcludeOrganizationUnitSets?: number[];
    /**
     * Optional script to run before sending the data. This can be used to filter out the event export.
     * Entity type: Script
     */
    ScriptID?: number;
    /**
     * Name of the script that ScriptID refers to.
     */
    ScriptName?: string;
    /**
     * Timeout after which event exports are cancelled. Defaults to 5 seconds. Not recommended to change.
     * Min: 1
     */
    TimeoutSeconds?: number;
    /**
     * The protocol to use when sending the data. If not set, the best available version of HTTP will be used.
     */
    Protocol?: ProtocolTypes;
  }

  export interface EventExportConfigurationConfigAuthorizationModel {
    /**
     * This will result in an authorization header that looks like: `Authorization: Bearer TOKEN`
     */
    StaticBearerToken?: string;
    /**
     * This will result in an authorization header that looks like: `Authorization: TOKEN`. Use this only when the other options are not sufficient.
     */
    StaticUnschemedToken?: string;
    /**
     * Configuration for Mutual TLS (mTLS)
     */
    ClientCertificate?: EventExportConfigurationCertificateModel;
  }

  export interface EventExportConfigurationCertificateModel {
    /**
     * A client certificate that can be used for mTLS. Must be X509 in PKCS12 format. Also, it must not be expired, have a private key and have clientAuth in it's extended key usage.
     */
    Certificate: string;
    /**
     * The password that is used to encrypt the certificate.
     */
    Password?: string;
  }

  export const enum EventExportConfigurationResponseMode {
    /**
     * Expects a 200 OK response with a body of exactly: '[ACK]' (without the singular quotes)
     */
    Normal = 0,
    /**
     * We don't care what you send back as long as it is a 2XX statuscode
     */
    Loose = 1
  }

  export interface EventExportConfigurationTarget {
    TargetID: EventExportTarget;
    EventType?: string;
  }

  export const enum EventExportTarget {
    Order = 1,
    Shipment = 2,
    PaymentTransaction = 3,
    Customer = 4,
    Invoice = 5,
    StockMutation = 6,
    OrganizationUnit = 7,
    FinancialPeriod = 8,
    FinancialPeriodAudit = 9,
    OrderFulfillmentLine = 10,
    System = 11,
    UserTasks = 12,
    CashCorrection = 13,
    CashDeposit = 14,
    CashExpense = 15,
    CashMovement = 16,
    RecurringTask = 17,
    OrderFulfillment = 18,
    Coupon = 19,
    ProductAvailability = 20,
    Repairs = 21,
    StockNotification = 22,
    SerialNumber = 23,
    LoyaltyProgram = 24,
    Cases = 25,
    Wishlist = 26,
    Assortment = 27,
    Survey = 28,
    SurveyResponse = 29,
    Company = 30
  }

  export const enum ProtocolTypes {
    /**
     * Use the best available HTTP version (default)
     */
    HTTP_ANY = 0,
    /**
     * Explicitly instruct EVA to use HTTP/1.1
     */
    HTTP_1_1 = 1
  }

  export interface CreateEventExportConfigurationResponse {
    /**
     * Entity type: EventExportConfiguration
     */
    ID?: number;
    /**
     * Make sure to store this information, this is the only time EVA will expose it for you!
     */
    SecretKey1?: string;
    /**
     * Make sure to store this information, this is the only time EVA will expose it for you!
     */
    SecretKey2?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for deleting a existing `EventExportConfiguration`
   */
  export interface DeleteEventExportConfiguration {
    /**
     * Entity type: EventExportConfiguration
     */
    ID: number;
  }

  /**
   * Service for retrieving an existing `EventExportConfiguration`.
   */
  export interface GetEventExportConfiguration {
    /**
     * Entity type: EventExportConfiguration
     */
    ID: number;
  }

  export interface GetEventExportConfigurationResponse {
    Result: EventExportConfigurationModel;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface EventExportConfigurationModel {
    /**
     * Entity type: EventExportConfiguration
     */
    ID: number;
    /**
     * Entity type: EndpointConfiguration
     */
    EndpointConfigurationID?: number;
    Name: string;
    Status: EventExportConfigurationStatus;
    IsReplay: boolean;
    Endpoint: string;
    SecretKey1: string;
    SecretKey2: string;
    Config: EventExportConfigurationConfigModel;
    ErrorReason?: string;
  }

  export interface EventExportConfigurationConfig {
    Authorization?: EventExportConfigurationConfigAuthorization;
    ResponseMode?: EventExportConfigurationResponseMode;
    MessageTypes?: EventExportConfigurationTarget[];
    ExcludedMessageTypes?: EventExportConfigurationTarget[];
    IncludeOrganizationUnits?: number[];
    ExcludeOrganizationUnits?: number[];
    IncludeOrganizationUnitSets?: number[];
    ExcludeOrganizationUnitSets?: number[];
    ScriptID?: number;
    TimeoutSeconds?: number;
    Protocol?: ProtocolTypes;
  }

  export interface EventExportConfigurationConfigAuthorization {
    StaticBearerToken: string;
    StaticUnschemedToken: string;
    ClientCertificate?: EventExportConfigurationCertificate;
  }

  export interface EventExportConfigurationCertificate {
    Certificate?: string;
    Password?: string;
  }

  /**
   * Service for retrieving a list of existing `EventExportConfiguration`'s
   */
  export interface GetEventExportConfigurations {
  }

  export interface GetEventExportConfigurationsResponse {
    Items: EventExportConfigurationModel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for retrieving a list of possible event types by `EventExportTarget`. If no target is provided, all targets and their types are returned.
   */
  export interface GetEventExportTargetTypes {
    EventTarget?: EventExportTarget;
  }

  export interface GetEventExportTargetTypesResponse {
    EventTypes?: string[];
    EventTargets?: EventTargetTypes[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface EventTargetTypes {
    Target: EventExportTarget;
    Types: string[];
  }

  export interface ReplayEvents {
    /**
     * Entity type: EventExportConfiguration
     */
    ID: number;
    From?: string;
    To?: string;
  }

  export interface ReplayEventsResponse {
    /**
     * Entity type: EventExportConfiguration
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for updating a existing `EventExportConfiguration`.
   * 
   * - Make sure to catch & store the `SecretKey1` or `SecretKey2`, as there will be no other way of retrieving them after this service call!
   */
  export interface RotateEventExportConfigurationKey {
    /**
     * Entity type: EventExportConfiguration
     */
    ID: number;
    KeyID: number;
  }

  export interface RotateEventExportConfigurationKeyResponse {
    SecretKey1?: string;
    SecretKey2?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for updating a existing `EventExportConfiguration`
   */
  export interface UpdateEventExportConfiguration {
    /**
     * Entity type: EventExportConfiguration
     */
    ID: number;
    Name?: string;
    /**
     * The configuration of the endpoint attached to this EventExport.
     * Entity type: EndpointConfiguration
     */
    EndpointConfigurationID?: number;
    Status?: EventExportConfigurationStatus;
    Endpoint?: string;
    Config?: EventExportConfigurationConfigModel;
  }

  /**
   * A generic service to return all possible values in specified list, e.g. `Currency`, `TaxCode` or `StockLabel`.
   * 
   * The `Name` of the enumeration is unique within EVA, and all of the `Values` returned are also unique and identifiable
   * by their respective `Name`. Services that consume a value of an enumeration will expect the numeric `Value` returned.
   * 
   * Some of these enumerations are static and are defined by code, meaning the `Values`' combination of `Name` and `Value`
   * is contant and will never change. It is important to know, though, that other enumerations may be dynamic and can be
   * altered per environment by their own respective services. When using these, keep in mind they can vary between
   * environments and new values could be added. It is guaranteed, however, that once a certain `Name`-`Value` pair is set,
   * it will never change in that particular environment and you are safe treat it as a constant.
   * 
   * :point_right: It is important to realize these dynamics, because at first sight, all of these values may seem contant between
   * environments - but that is the mere result of an initialization sequence that is usually identical, but definitely
   * not guaranteed!
   * 
   * :point_right: `IsFlags` will indicate if the values should be treated as bit fields (or, a set of flags). For example,
   * the enumeration `OrganizationUnitTypes` demonstrates that an organization unit can both be a Shop (`1`) and a Warehouse (`16`)
   * at the same time, indicated by value `17`. Always use bitwise operands on these fields!
   */
  export interface GetEnumValues {
    Name: string;
  }

  export interface GetEnumValuesResponse {
    Values: GetEnumValuesResponse.EnumValue[];
    IsFlags: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetEnumValuesResponse {
    export interface EnumValue {
      Name: string;
      Value: number;
      Description: string;
    }

  }

  export interface GetTransputJobTypes {
  }

  export interface GetTransputJobTypesResponse {
    Values: TransputJobTypeInfo[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface TransputJobTypeInfo {
    /**
     * Entity type: TransputJobType
     */
    ID: number;
    Name: string;
  }

  /**
   * Get a list of all document store categories.
   * 
   * :warning: For debugging and support purposes only.
   */
  export interface GetDocumentStoreCategories {
  }

  export interface GetDocumentStoreCategoriesResponse {
    Categories: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get a specific document from the document store.
   * 
   * :warning: For debugging and support purposes only.
   */
  export interface GetDocumentStoreDocument {
    /**
     * @The category of the document, see `GetDocumentStoreCategories`
     */
    CategoryID?: string;
    /**
     * @The partition key of the document
     */
    PartitionKey?: string;
  }

  export interface GetDocumentStoreDocumentResponse {
    Documents: Document[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface Document {
    ID: string;
    CreationTime: string;
    Data: any;
  }

  /**
   * Search for documents in a specified category. This will return a maximum of 100 records.
   * 
   * :warning: For debugging and support purposes only.
   */
  export interface SearchDocumentStore {
    /**
     * @The category of the document, see `GetDocumentStoreCategories`
     */
    CategoryID?: string;
    From?: string;
    To?: string;
  }

  export interface SearchDocumentStoreResponse {
    Documents: SearchDocumentStoreResponse.SearchDocumentStoreResponseResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace SearchDocumentStoreResponse {
    export interface SearchDocumentStoreResponseResult {
      PartitionKey: string;
      Id: string;
      CreationTime: string;
      /**
       * Entity type: User
       */
      CreatedByID: number;
      MimeType: string;
    }

  }

  /**
   * Disable a specific Discount for an Order, if the discount is active it will be removed from the Order, if it not active, it will not be applied anymore.
   */
  export interface DisableDiscountOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Discount
     */
    DiscountID: number;
  }

  /**
   * List disabled discounts on order
   */
  export interface ListDisabledDiscountsOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ListDisabledDiscountsOnOrderResponse {
    DisabledDiscounts: ListDisabledDiscountsOnOrderResponse.Item[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListDisabledDiscountsOnOrderResponse {
    export interface Item {
      /**
       * Entity type: Discount
       */
      ID: number;
      Description: string;
      MarketingDescription: string;
      CampaignName: string;
    }

  }

  /**
   * Enable a previous disabled Discount for an Order. If the Discount wasn't disabled, no error will be given.
   */
  export interface EnableDisabledDiscountOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Discount
     */
    DiscountID: number;
  }

  /**
   * Disable all discounts of the given trigger(s) on the order.
   */
  export interface DisableDiscountTriggersOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    DiscountTrigger?: DiscountTriggers[];
  }

  /**
   * Enable previously disabled discounts of the given trigger(s) on the order
   */
  export interface EnableDiscountTriggersOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    DiscountTrigger?: DiscountTriggers[];
  }

  /**
   * List disabled discounts triggers on order
   */
  export interface ListDisabledDiscountTriggersOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface ListDisabledDiscountTriggersOnOrderResponse {
    DisabledDiscountTriggers: DiscountTriggers[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This services returns info about the discounts on the given order
   */
  export interface GetOrderDiscountInfo {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetOrderDiscountInfoResponse {
    Discounts: GetOrderDiscountInfoResponse.DiscountInfo[];
    OrderLineDiscounts: GetOrderDiscountInfoResponse.OrderLineDiscount[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetOrderDiscountInfoResponse {
    export interface DiscountInfo {
      /**
       * Entity type: Discount
       */
      ID: number;
      Description: string;
      MarketingDescription: string;
      /**
       * Discount type: Default/System/Coupon/Bundle/Manual
       */
      Type: string;
      /**
       * If the manual discount needs a reason to apply it.
       */
      NeedsReason: boolean;
      BackendID: string;
      /**
       * If the discount is disabled with the DisableDiscountOnOrder service
       */
      IsDisabled: boolean;
      Coupons: GetOrderDiscountInfoResponse.CouponInfo[];
      Manuals: GetOrderDiscountInfoResponse.ManualInfo[];
      /**
       * The OrderLineID's of the discount lines.
       * Entity type: OrderLine
       */
      DiscountOrderLineIDs: number[];
      /**
       * The OrderLineID's on which the discount is applied.
       * Entity type: OrderLine
       */
      AppliedToOrderLines: number[];
      TotalDiscountAmount: number;
    }

    export interface CouponInfo {
      /**
       * Entity type: DiscountCoupon
       */
      ID: number;
      Code: string;
      /**
       * Entity type: OrderLine
       */
      DiscountOrderLineID: number;
    }

    export interface ManualInfo {
      /**
       * Entity type: OrderLine
       */
      DiscountOrderLineID: number;
      /**
       * If the manual discount is applied to a subset of lines, this will contain those lines. If empty, the discount is applied to the order.
       */
      Lines: OrderLineWithQuantity[];
      Amount: number;
      /**
       * Reason for the manual discount
       */
      Reason: string;
    }

    export interface OrderLineDiscount {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      Details: GetOrderDiscountInfoResponse.OrderLineDiscountDetail[];
    }

    export interface OrderLineDiscountDetail {
      /**
       * Entity type: Discount
       */
      DiscountID: number;
      DiscountAmount: number;
    }

  }

  /**
   * List the discounts that potentially could be activated.
   * 
   * A order or user can be given to look at the context of the order or user. When order is given the user will be ignored and the customer of the order will be used (if present)
   * 
   * If a `ProductIDs`, `ProductSearchID` or `ProductSearch` is set, the service will check if the resulting products are a eligible to one or more discounts. If an `OrderID` is provided, the context of the given order is taken into account, that can include user specific discounts, order OU specific discount, etc.
   * 
   * Be aware that:
   * - If `ProductIDs` is given, `ProductSearchID` and `ProductSearch` are ignored
   * - If `ProductSearchID` is given, `ProductIDs` and `ProductSearch` are ignored
   * - If `ProductSearch` is given, `ProductIDs` and `ProductSearchID` are ignored
   */
  export interface PrefigureDiscounts {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchID?: number;
    ProductSearch?: SearchProducts;
  }

  export interface PrefigureDiscountsResponse {
    Results: PrefigureDiscountResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * @deprecated This service will be replaced by `PrefigureDiscounts`.
   * List the discounts that potentially could be activated.
   * When an `Order` is given, identified by `OrderID`, the discounts will be based on the ordercontext. That will include possibly User specific discounts etc.
   * If a `ProductID` is set, also discounts that will be triggered when that Product is added to the current Order will be returned.
   * 
   * It is required to supply an Order a Product or both.
   */
  export interface GetPotentialDiscountsForOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
  }

  export interface GetPotentialDiscountsResponse {
    OrderDiscounts: PotentialDiscountDto[];
    /**
     * Entity type: Product
     */
    ProductDiscounts: { [key: number]: PotentialDiscountDto[] };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * @deprecated This service will be replaced by `PrefigureDiscounts`.
   * List the discounts that potentially could be activated.
   * This service has the same behavior as the `GetPotentialDiscountsForOrder`, only in this service the discounts are not calculated based on one `ProductID`, but on the result of a `ProductSearch` request.
   * This creates the possibility to fire a `ProductSearch` and a `GetPotentialDiscountsForProductSearch` at the same time.
   */
  export interface GetPotentialDiscountsForProductSearch {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    SearchProductsRequest?: SearchProducts;
  }

  /**
   * Returns all active discounts within an organization unit
   */
  export interface GetActiveDiscounts {
    /**
     * Defaults to current organization unit.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Include discounts based on trigger type. Defaults to automatic.
     */
    DiscountTriggers?: DiscountTriggers[];
  }

  export interface GetActiveDiscountsResponse {
    Layers: GetActiveDiscountsResponse.ActiveDiscountLayer[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetActiveDiscountsResponse {
    export interface ActiveDiscountLayer {
      /**
       * Entity type: DiscountLayer
       */
      LayerID: number;
      Discounts: GetActiveDiscountsResponse.ActiveDiscount[];
    }

    export interface ActiveDiscount {
      /**
       * Entity type: Discount
       */
      DiscountID: number;
      Description: string;
      DisplayDescription: string;
      MarketingDescription: string;
      DisplayMarketingDescription: string;
      DisplayPromotionLabel: string;
      PromotionLabel: string;
      ActionType: string;
      ActionData: { [key: string]: any };
      Conditions: GetActiveDiscountsResponse.ActiveDiscountCondition[];
      ImageBlobID?: string;
    }

    export interface ActiveDiscountCondition {
      /**
       * Entity type: DiscountCondition
       */
      ID: number;
      Type: string;
      Data: { [key: string]: any };
      UseAsActionCondition: boolean;
    }

  }

  /**
   * This services returns a list of discount coupon handlers
   */
  export interface ListDiscountCouponHandlers {
  }

  export interface ListDiscountCouponHandlersResponse {
    Handlers: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List the available manual discounts. Optionally, the OrderType or DiscountAppliesTo filter can be given.
   */
  export interface ListManualDiscounts {
    /**
     * Defaults to Sales
     */
    OrderType?: OrderTypes;
    /**
     * Defaults to Both
     */
    DiscountAppliesTo?: DiscountAppliesTo;
    /**
     * When OrderID is given, the manual discount conditions will be checked against the order.
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * When OrderLineID is given, the manual discount conditions will be checked against the order and orderline. When OrderLineID is given, OrderID will be ignored.
     * Entity type: OrderLine
     */
    OrderLineID?: number;
  }

  export interface ListManualDiscountsResponse {
    Result: ListManualDiscountsResponse.ManualDiscountDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListManualDiscountsResponse {
    export interface ManualDiscountDto {
      /**
       * Entity type: Discount
       */
      ID: number;
      BackendID?: string;
      Description?: string;
      MarketingDescription?: string;
      AppliesTo: DiscountAppliesTo;
      DiscountValueType: DiscountValueType;
      NeedsReason: boolean;
      /**
       * Entity type: CustomField
       */
      RequiredCustomFields: number[];
      FixedAmount?: number;
      MaxAmount?: number;
      /**
       * Entity type: Currency
       */
      CurrencyID: string;
    }

  }

  export const enum DiscountValueType {
    Percentage = 0,
    Amount = 1
  }

  /**
   * Get the PickProductDiscountOptions for an OrderLine
   * 
   * See: `GetPickProductDiscountOptionsForOrder`
   */
  export interface GetPickProductDiscountOptionsForOrderLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
  }

  export interface GetPickProductDiscountOptionsForOrderLineResponse {
    BaseAmount: number;
    /**
     * Entity type: Product
     */
    Options: number[];
    Tiers: PickProductTierOptions[];
    /**
     * Entity type: Product
     */
    CurrentSelection?: number;
    CurrentTier?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface PickProductTierOptions {
    ID: string;
    From: number;
    Difference: number;
    /**
     * Entity type: Product
     */
    DefaultProductID?: number;
    Prompt: boolean;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    Description: string;
    IsCurrentTier: boolean;
  }

  /**
   * Get the PickProductDiscountOptions for an Order
   * 
   * For each line there is an object with the options:
   * - Options: the available products to choose from
   * - CurrentSelection: the currently selected product
   * - CurrentTier: the current tier of the discount
   * - BaseAmount: the amount used to determine the tiers
   * - Tiers:
   *  - ID
   *  - Description
   *  - From: The amount at which this tier is available
   *  - Difference: The amount that should be added to be eligable for this tier
   *  - DefaultProductID: The product that should be selected by default when hitting this tier
   *  - Prompt: Should the frontend show a prompt to pick a product?
   *  - ProductIDs: The avaialble products in this tier
   *  - IsCurrentTier: Is this tier the current one?
   */
  export interface GetPickProductDiscountOptionsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetPickProductDiscountOptionsForOrderResponse {
    /**
     * Entity type: OrderLine
     */
    OptionsPerLine: { [key: number]: GetPickProductDiscountOptionsForOrderResponse.PickProductDiscountOptions };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetPickProductDiscountOptionsForOrderResponse {
    export interface PickProductDiscountOptions {
      BaseAmount: number;
      /**
       * Entity type: Product
       */
      Options: number[];
      Tiers: PickProductTierOptions[];
      /**
       * Entity type: Product
       */
      CurrentSelection?: number;
      CurrentTier?: string;
    }

  }

  /**
   * Pick a product for a PickProductAction for an OrderLine
   */
  export interface SetPickProductDiscountOptionsForOrderLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: Product
     */
    Selection?: number;
    /**
     * When set to 'true' selection must be within the given product options
     */
    DisallowSelectionOutsideProductOptions?: boolean;
  }

  export interface GetCurrency {
    /**
     * Entity type: Currency
     */
    ID: string;
  }

  export interface GetCurrencyResponse {
    ID: string;
    Name: string;
    Precision: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListCurrencies {
    PageConfig?: PageConfig<ListCurrenciesFilter>;
  }

  export interface ListCurrenciesFilter {
    /**
     * Entity type: Currency
     */
    ID: string;
    Name: string;
  }

  export interface ListCurrenciesResponse {
    Result?: PagedResult<CurrencyDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CurrencyDto {
    ID: string;
    Name: string;
    Precision: number;
  }

  /**
   * Services for retrieving a list of `Currency`
   */
  export interface GetAvailableCurrencies {
  }

  export interface GetAvailableCurrenciesResponse {
    Currencies: CurrencyDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates a new company associated with the consumer type
   */
  export interface CreateConsumerCompany {
    BackendID?: string;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCountryID?: string;
    RegistrationCountrySubdivisionID?: string;
    TaxRegistrationNumber?: string;
    FiscalID?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    BillingAddress?: AddressDataDto;
    ShippingAddress?: AddressDataDto;
    TaxRecipientCode?: string;
    TaxRecipientEmail?: string;
    SingleUse?: boolean;
    IsInvoicing?: boolean;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
  }

  export interface CreateConsumerCompanyResponse {
    /**
     * Entity type: Company
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Retrieves the details of the consumer company corresponding to the specified ID.
   */
  export interface GetConsumerCompany {
    /**
     * Entity type: Company
     */
    ID: number;
  }

  export interface GetConsumerCompanyResponse {
    /**
     * Entity type: Company
     */
    ID?: number;
    BackendID?: string;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCountryID?: string;
    RegistrationCountrySubdivisionID?: string;
    TaxRegistrationNumber?: string;
    FiscalID?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    BillingAddress?: AddressDto;
    ShippingAddress?: AddressDto;
    TaxRecipientCode?: string;
    TaxRecipientEmail?: string;
    SingleUse?: boolean;
    IsInvoicing?: boolean;
    HasUsers?: boolean;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Searches for consumer companies based on the provided query and filter criteria.
   */
  export interface SearchConsumerCompany {
    Query?: string;
    PageConfig?: ScrollablePageConfig<SearchCompanyFilter>;
  }

  export interface ScrollablePageConfig<T> {
    Filter: T;
    NextResultToken: string;
    Limit?: number;
  }

  export interface SearchCompanyFilter {
    /**
     * Entity type: Company
     */
    CompanyIDs?: number[];
    /**
     * Entity type: User
     */
    UserIDs?: number[];
    Name?: string;
    RegistrationNumber?: string;
    TaxRegistrationNumber?: string;
    FiscalID?: string;
    BillingAddressAddress1?: string;
    BillingAddressAddress2?: string;
    BillingAddressHouseNumber?: string;
    BillingAddressZipCode?: string;
    BillingAddressDistrict?: string;
    BillingAddressSubDistrict?: string;
    BillingAddressCity?: string;
    BillingAddressState?: string;
    BillingAddressCountryID?: string;
    BillingAddressCountrySubdivisionID?: string;
    IsInvoicing?: boolean;
  }

  export interface SearchConsumerCompanyResponse {
    Result: ScrollablePagedResult<SearchCompanyResult>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ScrollablePagedResult<T> {
    Page: T[];
    PreviousResultToken: string;
    NextResultToken: string;
  }

  export interface SearchCompanyResult {
    /**
     * Entity type: Company
     */
    ID?: number;
    Name?: string;
    RegistrationNumber?: string;
    TaxRegistrationNumber?: string;
    FiscalID?: string;
  }

  /**
   * Updates the details of companies associated with the consumer type.
   */
  export interface UpdateConsumerCompany {
    /**
     * Entity type: Company
     */
    ID: number;
    Name?: string;
    RegistrationNumber?: string;
    RegistrationCountryID?: string;
    RegistrationCountrySubdivisionID?: string;
    TaxRegistrationNumber?: string;
    FiscalID?: string;
    EmailAddress?: string;
    PhoneNumber?: string;
    BillingAddress?: AddressDataDto;
    ShippingAddress?: AddressDataDto;
    TaxRecipientCode?: string;
    TaxRecipientEmail?: string;
    IsInvoicing?: boolean;
    /**
     * Optional additional custom fields
     */
    CustomFields?: CustomFieldKeyValue[];
    /**
     * Entity type: Order
     */
    OrderID?: number;
  }

  /**
   * Service for customers to list case interactions
   */
  export interface CustomerListCaseInteractions {
    /**
     * Entity type: Case
     */
    CaseID: number;
    InitialPageConfig?: PageTokenConfig<CustomerListCaseInteractionsFilter>;
    PageToken?: string;
  }

  export interface CustomerListCaseInteractionsFilter {
    /**
     * Entity type: Interaction
     */
    ID?: number;
    Text?: string;
  }

  export interface CustomerListCaseInteractionsResponse {
    Results?: CustomerListCaseInteractionsResponse.CaseInteraction[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace CustomerListCaseInteractionsResponse {
    export interface CaseInteraction {
      /**
       * Entity type: Interaction
       */
      ID: number;
      /**
       * Entity type: Case
       */
      CaseID: number;
      Type: InteractionType;
      CanBeModified: boolean;
      Text: string;
      CreationTime: string;
      LastModificationTime?: string;
      /**
       * Entity type: User
       */
      CreatedByID?: number;
      CreatedBy?: CustomerListCaseInteractionsResponse.User;
    }

    export interface User {
      FirstName: string;
      LastName: string;
      FullName: string;
    }

  }

  /**
   * Service for customers to get a case interaction by id
   */
  export interface CustomerGetCaseInteraction {
    /**
     * Entity type: Interaction
     */
    ID: number;
  }

  export interface CustomerGetCaseInteractionResponse {
    /**
     * Entity type: Interaction
     */
    ID: number;
    /**
     * Entity type: Case
     */
    CaseID: number;
    Type: InteractionType;
    CanBeModified: boolean;
    Text: string;
    CreationTime: string;
    LastModificationTime?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for customers to create a case
   */
  export interface CustomerCreateCaseInteraction {
    /**
     * Entity type: Case
     */
    CaseID: number;
    Text?: string;
  }

  export interface CustomerCreateCaseInteractionResponse {
    /**
     * Entity type: Interaction
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for customers to update a case interaction
   */
  export interface CustomerUpdateCaseInteraction {
    /**
     * Entity type: Interaction
     */
    ID: number;
    Text?: string;
  }

  /**
   * Service for customers to delete a case interaction
   */
  export interface CustomerDeleteCaseInteraction {
    /**
     * Entity type: Interaction
     */
    ID: number;
  }

  /**
   * Service for customers to add a blob to a case interacion.
   */
  export interface CustomerAddBlobToCaseInteraction {
    /**
     * Entity type: Interaction
     */
    InteractionID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     * @deprecated Use `BlobUrls` instead.
     */
    BlobUrl?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrls?: string[];
  }

  /**
   * Service for customers to remove a blob from a case interaction.
   */
  export interface CustomerRemoveCaseInteractionBlob {
    /**
     * Entity type: Interaction
     */
    InteractionID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
  }

  /**
   * Service for customers to list cases. Returns a paged response.
   */
  export interface CustomerListCases {
    InitialPageConfig?: PageTokenConfig<CustomerListCasesFilter>;
    PageToken?: string;
  }

  export interface CustomerListCasesFilter {
    Title?: string;
  }

  export interface CustomerListCasesResponse {
    Results?: CustomerListCasesResponse.Case[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace CustomerListCasesResponse {
    export interface Case {
      /**
       * Entity type: Case
       */
      ID?: number;
      /**
       * Entity type: CaseTopic
       */
      TopicID?: number;
      TopicName?: string;
      /**
       * Entity type: CaseStatus
       */
      StatusID?: number;
      StatusName?: string;
      Priority?: CasePriority;
      Title?: string;
      /**
       * Entity type: Order
       */
      OrderID?: number;
      ReportingTime?: string;
    }

  }

  export const enum CasePriority {
    Low = 0,
    Medium = 1,
    High = 2,
    Critical = 9
  }

  /**
   * Service for customers to get a case by its ID.
   */
  export interface CustomerGetCaseByID {
    /**
     * Entity type: Case
     */
    ID: number;
  }

  export interface CustomerGetCaseByIDResponse {
    /**
     * Entity type: Case
     */
    ID: number;
    /**
     * Entity type: CaseTopic
     */
    TopicID?: number;
    TopicName?: string;
    /**
     * Entity type: CaseStatus
     */
    StatusID: number;
    StatusName?: string;
    Priority: CasePriority;
    Title: string;
    IsTitleEditable: boolean;
    Description: string;
    IsDescriptionEditable: boolean;
    Solution: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    ReportingTime: string;
    ClosingTime?: string;
    DescriptionCanBeModified: boolean;
    BlobIDs: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for customers to create a new case.
   */
  export interface CustomerCreateCase {
    /**
     * Entity type: CaseTopic
     */
    TopicID: number;
    Title: string;
    Description: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
  }

  export interface CustomerCreateCaseResponse {
    /**
     * Entity type: Case
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for customers to update an existing case.
   */
  export interface CustomerUpdateCase {
    /**
     * Entity type: Case
     */
    ID: number;
    /**
     * Entity type: CaseTopic
     */
    TopicID?: number;
    Title?: string;
    Description?: string;
  }

  /**
   * Service for customers to add a blob to a case.
   */
  export interface CustomerAddBlobToCase {
    /**
     * Entity type: Case
     */
    CaseID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     * @deprecated Use `BlobUrls` instead.
     */
    BlobUrl?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrls?: string[];
  }

  /**
   * Service for customers to remove a blob from a case.
   */
  export interface CustomerRemoveCaseBlob {
    /**
     * Entity type: Case
     */
    CaseID?: number;
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
  }

  /**
   * Returns the active shopping cart for the current user.
   * Works only for webshop OUs.
   * If there is no active shopping cart, no orderID will be returned.
   */
  export interface GetActiveShoppingCart {
  }

  export interface GetActiveShoppingCartResponse {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Fetch the summary of an Order. This summary contains:
   * - QuantityOrdered: the total count of products in the Order
   * - TotalAmount: the total amount of the Order
   * - ForeignTotalAmount: the total amount of the Order in the foreign currency
   * - CurrencyID: the currency of the Order
   * 
   * It is required to supply an OrderID
   */
  export interface GetShoppingCartInfo {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetShoppingCartInfoResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This returns the requested order. The order is retreived based on the OrderID.
   * 
   * The order will be recalculated:
   * - Discounts
   * - ShippingCosts
   * - GiftWrappingCosts
   * - OrderLine prices
   * 
   * If no recalculation is needed, use the GetOrder service instead. In general the approach would be: use GetShoppingCart while building the order, after PrepareOrderForCheckout has been called, switch to GetOrder.
   * 
   * To include the result of `GetRequiredDataForOrder`, the property `AdditionalOrderDataOptions.RequiredFor` can be set to the desired level of requirements.
   * To include the result of `GetProductRequirementValuesForOrder`, the property `AdditionalOrderDataOptions.IncludeProductRequirements` can be set to `true`.
   * To include the result of `GetGiftWrappingOptionsForOrder` and `GetAvailableGiftWrappingLinesForOrder`, the property `AdditionalOrderDataOptions.IncludeGiftWrapping` can be set to `true`.
   * To include the result of `PrefigureDiscounts` and `GetPotentialDiscountsForOrder`, the property `AdditionalOrderDataOptions.IncludePrefigureDiscounts` can be set to `true`.
   * To include the result of `GetPickProductDiscountOptionsForOrder`, the property `AdditionalOrderDataOptions.IncludePickProductOptions` can be set to `true`.
   * To include the result of `ListAvailableShippingMethods`, the property `AdditionalOrderDataOptions.IncludeAvailableShippingMethods` can be set to `true`.
   * To include the result of `GetAvailablePaymentMethods`, the property `AdditionalOrderDataOptions.IncludeAvailablePaymentMethods` can be set to `true`.
   * To include the result of `GetAvailableRefundPaymentMethodsForOrder`, the property `AdditionalOrderDataOptions.IncludeAvailableRefundPaymentMethods` can be set to `true`.
   * To include the result of `ValidateOrderShipment`, the property `AdditionalOrderDataOptions.IncludeValidateShipment` can be set to `true`.
   * To include the available actions for the PaymentTransactions, the property `AdditionalOrderDataOptions.IncludePaymentTransactionActions` can be set to `true`.
   */
  export interface GetShoppingCart {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * By default only confirmed payments are returned, enabling this flag will return all PaymentTransactions
     */
    IncludeAllPayments?: boolean;
    AdditionalOrderDataOptions?: AdditionalOrderDataOptions;
  }

  export interface ShoppingCartResponse {
    ShoppingCart: OrderDto;
    Amounts: OpenOrderAmounts;
    AdditionalOrderData?: AdditionalOrderData;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FlushCache {
    Pattern: string;
  }

  /**
   * Gets info about EVA caches, only accessible by New Black.
   */
  export interface GetCacheInfo {
    /**
     * Include more detail for the caches where available.
     */
    IncludeDetails?: boolean;
  }

  export interface GetCacheInfoResponse {
    CacheInfo: any;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Attaches given blob to an order, either by `BlobID` or `BlobUrl`.
   */
  export interface AttachBlobToOrder {
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
    Name: string;
    /**
     * @default None
     */
    Type?: OrderBlobTypes;
    /**
     * Required when `OrderLineID` is omitted
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Required when `OrderID` is omitted
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Optional link the blob to an Invoice of the Order
     * Entity type: Invoice
     */
    InvoiceID?: number;
    /**
     * Optional link the blob to a Shipment of the Order
     * Entity type: Shipment
     */
    ShipmentID?: number;
  }

  /**
   * Creates a blob intended for app diagnostics - will log the user context to determine user, organization unit and user agent.
   */
  export interface CreateDiagnosticsBlob {
  }

  export interface CreateDiagnosticsBlobResponse {
    /**
     * Entity type: Blob
     */
    Guid: string;
    Url: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List blobs intended for app diagnostics.
   */
  export interface ListDiagnosticsBlobs {
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListDiagnosticsBlobsResponse {
    Result?: PagedResult<DiagnosticsBlob>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface DiagnosticsBlob {
    /**
     * Entity type: Blob
     */
    ID: string;
    Url: string;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreationTime: string;
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    UserAgent?: string;
  }

  export interface ListBlobsForFinancialPeriod {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
  }

  export interface ListBlobsForFinancialPeriodResponse {
    Result: FinancialPeriodBlobDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FinancialPeriodBlobDto {
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID: number;
    Name: string;
    MimeType: string;
    /**
     * Entity type: Blob
     */
    BlobID: string;
    Url: string;
  }

  /**
   * Get the blobs that are attached to an Order
   * 
   * It's required to specify the order, optionally the blobs can also be filtered on `InvoiceID` of `ShipmentID`.
   */
  export interface ListBlobsForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
  }

  export interface ListBlobsForOrderResponse {
    Result: OrderBlobDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OrderBlobDto {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: Invoice
     */
    InvoiceID?: number;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    Type: OrderBlobTypes;
    Name: string;
    MimeType: string;
    BlobID: string;
    Url: string;
    CreationTime: string;
    /**
     * Optional OrganizationUnit based on Shipment.OrganizationUnit or Invoice.OrganizationUnit
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitBackendID?: string;
    OrganizationUnitName?: string;
    InvoiceNumber?: string;
    /**
     * Entity type: MessageTemplate
     */
    MessageTemplateID?: number;
  }

  /**
   * Service combining StoreBlob and AttachBlobToOrder.
   */
  export interface StoreAndAttachBlobToOrder {
    Name: string;
    /**
     * @default None
     */
    Type?: OrderBlobTypes;
    Category?: string;
    OriginalName?: string;
    MimeType: string;
    Data: string;
    /**
     * Required when `OrderLineID` is omitted
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Required when `OrderID` is omitted
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Optional link the blob to an Invoice of the Order
     * Entity type: Invoice
     */
    InvoiceID?: number;
    /**
     * Optional link the blob to a Shipment of the Order
     * Entity type: Shipment
     */
    ShipmentID?: number;
  }

  export interface StoreAndAttachBlobToOrderResponse {
    Guid: string;
    Url: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for generating a new device hub barcode
   */
  export interface GenerateDeviceHubBarcode {
  }

  export interface GenerateDeviceHubBarcodeResponse {
    DeviceID: string;
    Barcode: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for generating a new scan mode barcode
   */
  export interface GenerateScanModeBarcode {
  }

  export interface GenerateScanModeBarcodeResponse {
    DeviceID: string;
    Barcode: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for generating a new session barcode
   */
  export interface GenerateSessionBarcode {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GenerateSessionBarcodeResponse {
    Barcode: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Parses the given barcode with an available handler that can process it.
   */
  export interface ParseBarcode {
    Barcode?: string;
  }

  export interface ParseBarcodeResponse {
    Type: string;
    Value: string;
    /**
     * This indicates the barcode was parsed succesfully, but the resulting data could not be found in the current instance.
     */
    ParsedButNotFound: boolean;
    /**
     * The handler that handled the barcode has ExecuteBarcode functionality.
     */
    IsExecutable: boolean;
    Data: { [key: string]: any };
    Actions: ParseBarcodeResponse.Action[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ParseBarcodeResponse {
    export interface Action {
      Sequence: number;
      Type: string;
      Parameters: { [key: string]: any };
    }

  }

  /**
   * Executes the given barcode with an available handler that can execute it. To know if a barcode is executable, run ParseBarcode and look for the IsExecutable in the response.
   */
  export interface ExecuteBarcode {
    /**
     * The handler that parsed the barcode in parsebarcode.
     */
    Type: string;
    Barcode: string;
    Data: { [key: string]: any };
  }

  export interface ExecuteBarcodeResponse {
    Data?: { [key: string]: any };
    Actions?: ExecuteBarcodeResponse.Action[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ExecuteBarcodeResponse {
    export interface Action {
      Sequence: number;
      Type: string;
      Parameters: { [key: string]: any };
    }

  }

  /**
   * Service for retrieving a device authentication token
   */
  export interface RequestDeviceAuthenticationToken {
  }

  export interface RequestDeviceAuthenticationTokenResponse {
    AuthenticationToken: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Generates a temporary 5-digit code that can be used for elevation.
   * 
   * A manager generates this code on their own device, and can then manually copy this code on the user's device when requested.
   * 
   * :shield: Requires `Security:ElevatedFunctionalityProvider` to contain "TemporaryElevationCode".
   * 
   * :point_right: Send this temporary code as elevation header with your original call.
   * 
   * :bulb: Note it's only valid for a limited period of time (as per `Security:TemporaryElevationCode:ExpireInMinutes`) and is a /separate/ code from your login PIN!
   */
  export interface GenerateTemporaryElevationCode {
  }

  export interface GenerateTemporaryElevationCodeResponse {
    Code: string;
    ValidBefore: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Generates an elevation barcode to elevate this user's rights, which can be scanned by a manager with sufficient rights to grant elevation.
   * 
   * Show the returned `Barcode` to a manager for scanning through the `ParseBarcode` service, preferably as QR code through our `/barcode` endpoint.
   * 
   * :shield: Requires `Security:ElevatedFunctionalityProvider` to contain "ElevationBarcode".
   * 
   * :point_right: Send the returned `RequestToken` as elevation header with your original call, but before you do, it needs to be confirmed first through a manager scan. You can use this
   * same token to subscribe to the `/elevationBarcode` SignalR hub to listen for this scan event.
   * 
   * :bulb: Optionally supply the `OrganizationUnitID`, `Functionality` and `Scope` of your original request, so we can check the manager rights directly from the scan
   * on their device. Omitting these here will fall back to the default security enforced from the user attempting to use the (confirmed) elevation token, which just gives
   * a nasty 403 on the employee's device.
   */
  export interface GenerateElevationBarcode {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    Functionality?: string;
    Scope?: FunctionalityScope;
    ElevationType?: ElevationType;
  }

  export interface GenerateElevationBarcodeResponse {
    Barcode: string;
    RequestToken: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Returns the full authorizationstructure of the current user.
   * 
   * It is a list of all the roles the user has, with their functionalities and the OrganizationUnits where these are active.
   */
  export interface GetAuthorizationStructure {
  }

  export interface GetAuthorizationStructureResponse {
    Roles: RoleWithFunctionalitiesAndOrganizationUnits[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface RoleWithFunctionalitiesAndOrganizationUnits {
    /**
     * Entity type: Role
     */
    RoleID: number;
    RoleName: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs: number[];
    Functionalities: Record<'None' | 'Elevation' | 'Verification', { [key: string]: FunctionalityScope }>;
    /**
     * @deprecated Replaced by `Functionalities`
     */
    ScopedFunctionalities: { [key: string]: FunctionalityScope };
    /**
     * @deprecated Replaced by `Functionalities`
     */
    ElevationFunctionalities: { [key: string]: FunctionalityScope };
  }

  /**
   * Create a temporary token, based off of your current authentication token, which can be passed in query parameters for redirect requests. On the
   * receiving side, post this token in the `Login` service request body as `AuthenticationToken` to retrieve a new token, similar to the original
   * token, preserving user and context.
   * 
   * Note this token lifetime is only 30 seconds, and has no rights to do anything else.
   */
  export interface GetTemporaryRedirectToken {
  }

  export interface GetTemporaryRedirectTokenResponse {
    Token: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Authenticates a user
   */
  export interface Login {
    /**
     * Primary identifier of the user, can be an `EmailAddress` or `Nickname`
     */
    Username?: string;
    /**
     * The password associated with the `Username`
     */
    Password?: string;
    /**
     * Used to login to a specific organization unit
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Used to login to a specific visibility group
     * Entity type: VisibilityGroup
     */
    VisibilityGroupID?: number;
    /**
     * The two-factor authenticator code that granted the confirmation token
     */
    TwoFactorAuthenticator?: string;
    /**
     * The two-factor authenticator confirmation token
     */
    TwoFactorAuthenticationToken?: string;
    /**
     * If a successful two-factor authentication should be remembered on this IP address
     */
    TwoFactorRememberMe?: boolean;
    /**
     * Your current authenticationtoken;
     * - Can be used to select the same OrganizationUnit when logging in onto a website with an anonymous token
     * - Can be used when switching to another organization unit (no other properties are required in that case)
     */
    AuthenticationToken?: string;
    /**
     * The employee code - used to login by QR or PIN
     */
    IdentificationCode?: string;
    /**
     * Login on the first OrganizationUnit in the tree where the user has the Login functionality
     */
    SelectFirstOrganizationUnit?: boolean;
    /**
     * The preferred Language for the session
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Type of the custom authenticator, if used
     */
    CustomAuthenticatorType?: string;
    /**
     * Data for the custom authenticator, if used
     */
    CustomAuthenticateData?: { [key: string]: any };
    /**
     * Filters the available roles/functionalities when those are explicitly attached to a user type (only single flag values are allowed)
     */
    UserType?: UserTypes;
    /**
     * When set, the non-employees will be explicity filtered - used primarily when Customers and Employees are split, but can also be used to make sure no Customers can login
     */
    AsEmployee?: boolean;
    /**
     * When enabled this service will return a JWT token and refresh token.
     */
    UseJwtTokens?: boolean;
  }

  export interface LoginResponse {
    Authentication?: AuthenticationResults;
    FailureReason?: AuthenticationFailureReasons;
    OrganizationUnits?: LoginResponse.OrganizationUnitModel[];
    VisibilityGroups?: LoginResponse.VisibilityGroupModel[];
    User?: LoggedInUserDto;
    IsCustomAuthenticateResult?: boolean;
    /**
     * The available two-factor authenticators, returned when the Authentication result is NeedsTwoFactorAuthentication, for use in a sequential Login call
     */
    TwoFactorAuthenticators?: TwoFactorAuthenticatorData[];
    /**
     * Response data for the custom authenticator, if used
     */
    CustomAuthenticateResponseData?: { [key: string]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace LoginResponse {
    export interface OrganizationUnitModel {
      /**
       * Entity type: OrganizationUnit
       */
      ID?: number;
      /**
       * Entity type: OrganizationUnit
       */
      ParentID?: number;
      /**
       * Entity type: Country
       */
      CountryID?: string;
      Name?: string;
      BackendID?: string;
      Description?: string;
      Type?: OrganizationUnitTypes;
      Status?: OrganizationUnitStatus;
    }

    export interface VisibilityGroupModel {
      /**
       * Entity type: VisibilityGroup
       */
      ID?: number;
      Name?: string;
    }

  }

  export const enum AuthenticationResults {
    NotAuthenticated = 0,
    NotAuthorized = 1,
    Authenticated = 2,
    NeedsTwoFactorAuthentication = 3,
    NeedsOrganizationUnitID = 4,
    NeedsEmailVerification = 5,
    NeedsPasswordReset = 6,
    NeedsVisibilityGroupID = 7,
    NeedsChallenge = 8
  }

  export const enum AuthenticationFailureReasons {
    None = 0,
    InvalidRequest = 1,
    NeedsEmailVerification = 2,
    NotAuthorized = 3,
    InvalidCredentials = 4,
    ThirdPartyFailure = 5,
    InvalidUserType = 6,
    NeedsPasswordReset = 7,
    AuthenticationOnPrimaryRegionFailed = 8,
    SelectedOrganizationUnitNotAvailable = 9,
    UserDoesntExist = 10,
    UserDeactivated = 11,
    InvalidTwoFactorAuthentication = 12,
    CannotUseApiKeyForLogin = 13,
    InvalidChallenge = 14
  }

  export interface TwoFactorAuthenticatorData {
    Name: string;
    SecurityLevel: number;
    Status: TwoFactorAuthenticatorStatus;
    Type: TwoFactorAuthenticatorType;
  }

  export const enum TwoFactorAuthenticatorStatus {
    Disabled = 0,
    Pending = 1,
    Confirmed = 2,
    Failed = 3
  }

  export const enum TwoFactorAuthenticatorType {
    Passive = 0,
    Active = 1
  }

  /**
   * Ends the current session of the user and invalidates the used AuthenticationToken
   * 
   * Optionally the ExpireAllSessions property can be set to terminate all current sessions for the User.
   * 
   * The Clockout property can be used the register a clock out event for time tracking.
   */
  export interface Logout {
    ExpireAllSessions?: boolean;
    ClockOut?: boolean;
  }

  export interface LogoutResponse {
    LogoutSuccessful: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Retrieves some basic information about the current user's passkeys.
   */
  export interface GetPasskeyInfo {
  }

  export interface GetPasskeyInfoResponse {
    Passkeys?: GetPasskeyInfoResponse.GetPasskeyInfoItem[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetPasskeyInfoResponse {
    export interface GetPasskeyInfoItem {
      ID?: string;
    }

  }

  /**
   * As a logged in user, you can create a passkey as additional login method, which is a totally awesome, passwordless, usernameless method to log in!
   * 
   * Before you can create a new passkey, use this `GetPasskeyChallenge` service to obtain a challenge and some information from the server, push that through WebAuthn using
   * `navigator.credentials.create` to create the credential, which you use to call `CreatePasskey`.
   * 
   * :point_right: See `CreatePasskey` for more details!
   */
  export interface GetPasskeyChallenge {
  }

  export interface GetPasskeyChallengeResponse {
    Challenge?: string;
    RelyingParties?: GetPasskeyChallengeResponse.GetPasskeyChallengeResponseRelyingParty[];
    User?: GetPasskeyChallengeResponse.GetPasskeyChallengeResponseUser;
    Algorithms?: GetPasskeyChallengeResponse.GetPasskeyChallengeResponseAlgorithm[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetPasskeyChallengeResponse {
    export interface GetPasskeyChallengeResponseRelyingParty {
      ID?: string;
      Name?: string;
    }

    export interface GetPasskeyChallengeResponseUser {
      ID?: string;
      DisplayName?: string;
      Name?: string;
    }

    export interface GetPasskeyChallengeResponseAlgorithm {
      ID?: number;
      Type?: string;
    }

  }

  /**
   * After having called `GetPasskeyChallenge` to obtain a challenge and some information from the server, and after pushing that through WebAuthn using `navigator.credentials.create`,
   * you extract information from the generated credential and send it back to the server for storage.
   * 
   * 1. The `Challenge` matching the one originally provided through `GetPasskeyChallenge`
   * 2. The `PublicKey` generated (base64 encoded)
   * 3. The `AuthenticatorData` generated (base64 encoded)
   * 
   * The creation will succeed if the `Challenge` matches the one we created for you and all the data checks out, after which you can use your new key to log in!
   * 
   * When you want to log in, simply call `Login` using `CustomAuthenticatorType` set to "Passkey" and it will reply with a challenge and some information inside the
   * `CustomAuthenticateResponseData` property, which you can push through WebAuthn using `navigator.credentials.get` to sign that challenge with your private key. Exact the information
   * from the generated credential and call `Login` again with the same `CustomAuthenticatorType`, but this time include `CustomAuthenticateData` with an object of type
   * `PasskeyAuthenticateMessage` containing;
   * 
   * - The `ID` as the "rawId" of the used credential (base64 encoded)
   * - The `Signature` that was created to sign the challenge (base64 encoded)
   * - The `ClientData` to include some client information (base64 encoded json)
   * - The `AuthenticatorData` to include some authenticator data (base64 encoded)
   * 
   * If this all checks out, you're logged in! As with every `Login`, be sure to specify user types and organization units of your preference.
   */
  export interface CreatePasskey {
    Challenge?: string;
    PublicKey?: string;
    AuthenticatorData?: string;
  }

  /**
   * Remove a passkey configuration for the current user.
   */
  export interface DeletePasskey {
    ID?: string;
  }

  /**
   * Remove all passkey configurations for the specified user, as employee management feature.
   */
  export interface DeletePasskeyForUser {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  /**
   * Stub service to validate the current token for employees
   */
  export interface Ping {
  }

  /**
   * Stub service to validate the current token
   */
  export interface ValidateToken {
  }

  /**
   * Refresh a JWT token
   */
  export interface RefreshToken {
    Token?: string;
  }

  export interface RefreshTokenResponse {
    Token?: string;
    RefreshToken?: string;
    RefreshTokenExpireDate?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets the available two-factor authenticators for the current user
   */
  export interface GetAvailableTwoFactorAuthenticators {
    Status?: TwoFactorAuthenticatorStatus;
  }

  export interface GetAvailableTwoFactorAuthenticatorsResponse {
    Result: TwoFactorAuthenticatorData[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Enable a given two-factor authenticator for the current user
   */
  export interface EnableTwoFactorAuthentication {
    TwoFactorAuthenticator: string;
    TwoFactorAuthenticationToken?: string;
  }

  export interface EnableTwoFactorAuthenticationResponse {
    Status: TwoFactorAuthenticatorStatus;
    Data: { [key: string]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Disable a given two-factor authenticator for the current user
   */
  export interface DisableTwoFactorAuthentication {
    TwoFactorAuthenticator: string;
  }

  /**
   * Disables all two-factor authenticator for the given user
   */
  export interface DisableAllTwoFactorAuthentication {
    /**
     * Entity type: User
     */
    UserID: number;
  }

  /**
   * Validates if the given PIN is valid for the current user/ou.
   */
  export interface ValidateIdentificationPin {
    Pin: string;
  }

  export interface ValidateIdentificationPinResponse {
    IsValid: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates a new AuditChainConfiguration, which allows you to configure the starting number of a sequence and its prefix.
   * Creating a configuration for an audit chain that already exists does not apply this configuration. A new chain would need to be created by incrementing the Auditing:Version setting.
   */
  export interface CreateAuditChainConfiguration {
    AuditChainType?: AuditChainTypes;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    DesiredStartSequenceNumber?: number;
    DesiredPrefix?: string;
    DesiredIdentifier?: string;
  }

  export interface CreateAuditChainConfigurationResponse {
    /**
     * Entity type: AuditChainConfiguration
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets all current audit chain configurations.
   */
  export interface GetAuditChainConfigurations {
  }

  export interface GetAuditChainConfigurationsResponse {
    Configurations: GetAuditChainConfigurationsResponse.AuditChainConfiguration[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAuditChainConfigurationsResponse {
    export interface AuditChainConfiguration {
      /**
       * Entity type: AuditChainConfiguration
       */
      ID?: number;
      /**
       * Entity type: Country
       */
      CountryID?: string;
      /**
       * Entity type: OrganizationUnit
       */
      OrganizationUnitID?: number;
      /**
       * Entity type: Company
       */
      CompanyID?: number;
      AuditChainType?: AuditChainTypes;
      OrganizationUnitName?: string;
      CompanyName?: string;
      DesiredStartSequenceNumber?: number;
      DesiredPrefix?: string;
      DesiredIdentifier?: string;
      CreationTime?: string;
      LastModificationTime?: string;
    }

  }

  /**
   * Updates an existing AuditChainConfiguration, which allows you to configure the starting number of a sequence and its prefix.
   * Updating a configuration for an audit chain that already exists does not apply this updated configuration. A new chain would need to be created by incrementing the Auditing:Version setting.
   */
  export interface UpdateAuditChainConfiguration {
    /**
     * Entity type: AuditChainConfiguration
     */
    ID?: number;
    AuditChainType?: AuditChainTypes;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
    DesiredStartSequenceNumber?: number;
    DesiredPrefix?: string;
    DesiredIdentifier?: string;
  }

  /**
   * Deletes an AuditChainConfiguration
   */
  export interface DeleteAuditChainConfiguration {
    /**
     * Entity type: AuditChainConfiguration
     */
    ID: number;
  }

  /**
   * Downloads all generated audit files from the selected period as a single zip file, with a separate folder for each organization unit.
   * 
   * When selecting a container organization unit, all underlying organization units will be included.
   */
  export interface DownloadFinancialPeriodAudits {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    FiscalYear: number;
    FiscalMonth?: number;
  }

  /**
   * Used to retrieve the e-invoicing data that was set before via `SetElectronicInvoiceData`
   */
  export interface GetElectronicInvoiceData {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetElectronicInvoiceDataResponse {
    RecipientEmail: string;
    RecipientCode: string;
    PublicProjectCode: string;
    TenderIdentificationCode: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Sets the additional e-invoicing data on the given order. To retrieve the information that has been set use: `GetElectronicInvoiceData`
   */
  export interface SetElectronicInvoiceData {
    /**
     * Entity type: Order
     */
    OrderID: number;
    RecipientEmail?: string;
    RecipientCode?: string;
    PublicProjectCode?: string;
    TenderIdentificationCode?: string;
  }

  /**
   * Retrieves the lottery number for a given order
   */
  export interface GetLotteryNumber {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetLotteryNumberResponse {
    LotteryNumber: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * This service lets you query the EventLedger for auditing purposes.
   * 
   * The request provides a PageConfig (of type ScrollablePageConfig) that lets you filter on the given properties in the EventLedger.
   * Paging is done by providing the request with the `NextResultToken` that you got back from the previous request.
   * As this method of paging is forward-only, you can't go back to a previous page unless you store the token or the results,
   * and the total number of results is unknown, only whether or not there are more results (determined by if the NextResultToken is null).
   * 
   * The response contains the whole EventLedger document.
   * 
   * Possible filters:
   * 
   * - Types: A list of strings that refer to the `Type` field that is in every document.
   * - OrderID: Only return documents that contain a reference to the given OrderID
   * - OrganizationUnitID: Only return documents that were created by someone logged into the given OrganizationUnitID. It's recommended that this filter is always used, due to performance reasons.
   * - DeviceID: Filters on documents that have a reference to the given device, usually associated with a payment or a print command.
   * - FromDate: Only return documents newer than this date. It's recommended that this filter is always used, due to performance reasons.
   * - ToDate: Only return documents older than this date. It's recommended that this filter is always used, due to performance reasons.
   * - UserIDs: Only return documents created by these users. The IDs for this can be gotten by first doing a user search.
   * - FirstName: Filter on documents created by users whose first name starts with this. Achieves the same as filtering on UserIDs, but with one step less. However, due to privacy reasons, this information isn't stored for customers.
   * - FullName: Filter on documents created by users whose full name starts with this. Achieves the same as filtering on UserIDs, but with one step less. However, due to privacy reasons, this information isn't stored for customers.
   * - EmailAddress: Filter on documents created by users whose email starts with this. Achieves the same as filtering on UserIDs, but with one step less. However, due to privacy reasons, this information isn't stored for customers.
   */
  export interface ListEventLedger {
    Query?: string;
    PageConfig?: ScrollablePageConfig<EventLedgerFilters>;
  }

  export interface EventLedgerFilters {
    IDs: string[];
    Types: string[];
    OrderID?: number;
    OrganizationUnitID?: number;
    DeviceID?: number;
    FromDate?: string;
    ToDate?: string;
    UserIDs: number[];
    FinancialPeriodIDs: number[];
    FirstName: string;
    FullName: string;
    EmailAddress: string;
    OrderIDs: number[];
    OnlyAuditedDocuments: boolean;
    Amount: EventLedgerFilters.AmountFilter;
  }

  export namespace EventLedgerFilters {
    export interface AmountFilter {
      EqualTo?: number;
      GreaterThan?: number;
      LessThan?: number;
    }

  }

  export interface ListEventLedgerResponse {
    Result: ScrollablePagedResult<EventLedgerDocument>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface EventLedgerDocument {
    ID: string;
    Type: string;
    BackendType: string;
    CreationTime: string;
    OverrideCreationTime?: string;
    IpAddress: string;
    TenantID: string;
    PartitionKey: string;
    Auditing: EventLedgerDocument.AuditingDocument;
    CurrentUser: EventLedgerDocument.CurrentUserDocument;
    Order: EventLedgerDocument.OrderDocument;
    UserTask: EventLedgerDocument.UserTaskDocument;
    DailyTotals: EventLedgerDocument.DailyTotalDocument[];
    ExecutionContext: EventLedgerDocument.ExecutionContextDocument;
    AuthenticationResponse: EventLedgerDocument.AuthenticationResponseDocument;
    PaymentTransaction: EventLedgerDocument.PaymentTransactionDocument;
    FinancialPeriod: EventLedgerDocument.FinancialPeriodDocument;
    Station: EventLedgerDocument.StationDocument;
    Device: EventLedgerDocument.DeviceDocument;
    OrderLines: EventLedgerDocument.OrderLineDocument[];
    Description: string;
    Discount: EventLedgerDocument.DiscountDocument;
    Invoice: EventLedgerDocument.InvoiceDocument;
    Amount?: number;
    Attachments: EventLedgerDocument.AttachmentDocument[];
    MessageTemplate: EventLedgerDocument.MessageTemplateDocument;
    FinancialReportingPeriod: EventLedgerDocument.FinancialReportingPeriodDocument;
    Data: { [key: string]: any };
    BlobID?: string;
    OrganizationUnit: EventLedgerDocument.OrganizationUnitDocument;
    FinancialPeriodAudit: EventLedgerDocument.FinancialPeriodAuditDocument;
    Role: EventLedgerDocument.RoleDocument;
    User: EventLedgerDocument.UserDocument;
    TerminalReport: EventLedgerDocument.TerminalReportDocument;
    CashExpense: EventLedgerDocument.CashExpenseDocument;
    CashDeposit: EventLedgerDocument.CashDepositDocument;
    CashCorrection: EventLedgerDocument.CashCorrectionDocument;
    CashMovement: EventLedgerDocument.CashMovementDocument;
    PriceCorrection: EventLedgerDocument.PriceCorrectionDocument;
    SerialNumber: EventLedgerDocument.SerialNumberDocument;
    Customers: string[];
  }

  export namespace EventLedgerDocument {
    export interface SerialNumberDocument {
      ID: number;
      Number: string;
      ExpectedOrganizationUnitID?: number;
      ExpectedStockLabelID?: number;
      ActualOrganizationUnitID?: number;
      ActualStockLabelID?: number;
    }

    export interface CashDepositDocument {
      ID: number;
      Status: string;
      Number: string;
      Amount: number;
      CurrencyID: string;
      Details: CashDepositDocument.CashDepositDetails;
      SignatureBlobID?: string;
    }

    export namespace CashDepositDocument {
      export interface CashDepositDetails {
        Coins: { [key: number]: number };
        BankNotes: { [key: number]: number };
      }

    }

    export interface CashExpenseDocument {
      ID: number;
      Type: CashExpenseDocument.CashExpenseType;
      Amount: number;
      Description: string;
      TaxCodeID: number;
      TaxRate: number;
      BlobID?: string;
      SignatureBlobID?: string;
    }

    export namespace CashExpenseDocument {
      export interface CashExpenseType {
        ID: number;
        Type: string;
        Name: string;
        Description: string;
      }

    }

    export interface CashCorrectionDocument {
      ID: number;
      CreatedByID: number;
      FinancialPeriodID: number;
      DeviceID?: number;
      Type: string;
      Amount: number;
      Remark: string;
      Reason?: EventLedgerDocument.CashCorrectionReasonDocument;
      CurrencyID?: string;
    }

    export interface CashCorrectionReasonDocument {
      ID: number;
      Name: string;
      BackendID?: string;
    }

    export interface CashMovementDocument {
      ID?: number;
      FinancialPeriodID?: number;
      PaymentTypeID?: number;
      SourceDeviceID?: number;
      DestinationDeviceID?: number;
      CurrencyID?: string;
      Amount?: number;
    }

    export interface PriceCorrectionDocument {
      OrderLineID: number;
      OldValue: number;
      NewValue: number;
    }

    export interface AuditingDocument {
      CumulativeTotalTaxes: number;
      SequenceNumber?: number;
      PreviousSequenceNumber?: number;
      Signature: string;
      PreviousSignature: string;
      Type: string;
      Version: number;
      KeyBlobID?: string;
      ChainID: number;
    }

    export interface AttachmentDocument {
      Name: string;
      Category: string;
      MimeType: string;
      BlobID: string;
      Url: string;
    }

    export interface DailyTotalDocument {
      Description: string;
      ExternalTotal: number;
      ExpectedTotal: number;
      Details: string;
    }

    export interface OrderDocument {
      /**
       * Entity type: Order
       */
      ID: number;
      DisplayID: string;
      Customer: OrderDocument.CustomerDocument;
      HasReturns: boolean;
      CurrencyID: string;
      Type: OrderTypes;
      IsPaid: boolean;
      IsCompleted: boolean;
      IsShipped: boolean;
      IsInvoiced: boolean;
    }

    export namespace OrderDocument {
      export interface CustomerDocument {
        /**
         * Entity type: User
         */
        ID: number;
      }

    }

    export interface OrderLineDocument {
      /**
       * Entity type: OrderLine
       */
      ID: number;
      Description: string;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      TotalQuantityToShip: number;
      QuantityShipped: number;
      QuantityInvoiced: number;
      UnitPrice: number;
      TaxRate: number;
    }

    export interface ShipmentDocument {
      /**
       * Entity type: Shipment
       */
      ID: number;
      TrackingCode: string;
      BackendID: string;
    }

    export interface InvoiceDocument {
      /**
       * Entity type: Invoice
       */
      ID: number;
      ReferencedInvoiceID?: number;
      InvoiceNumber: string;
      FiscalID: string;
      TotalAmount: number;
      InvoiceData: EventLedgerDocument.InvoiceDataDocument[];
    }

    export interface InvoiceDataDocument {
      Type: string;
      Data: string;
    }

    export interface ShipmentLineDocument {
      /**
       * Entity type: ShipmentLine
       */
      ID: number;
      QuantityShipped: number;
    }

    export interface UserTaskDocument {
      /**
       * Entity type: UserTask
       */
      ID: number;
      /**
       * Entity type: User
       */
      UserID?: number;
      StartTime?: string;
      CompletionTime?: string;
      Type: UserTaskDocument.UserTaskTypeDocument;
      Description: string;
    }

    export namespace UserTaskDocument {
      export interface UserTaskTypeDocument {
        ID: number;
        Name: string;
      }

    }

    export interface OrganizationUnitDocument {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
      Type: OrganizationUnitTypes;
    }

    export interface CurrentUserDocument {
      /**
       * Entity type: User
       */
      ID: number;
      FirstName: string;
      FullName: string;
      EmailAddress: string;
      ApplicationID: number;
      LanguageID: string;
      CountryID: string;
      OrganizationUnitID: number;
      StationID?: number;
      Type: UserTypes;
      TimeZone: string;
      OrganizationUnit: EventLedgerDocument.OrganizationUnitDocument;
    }

    export interface ExecutionContextDocument {
      Name: string;
      UserAgent: string;
      ID: string;
      Version: string;
    }

    export interface PaymentTransactionDocument {
      /**
       * Entity type: PaymentTransaction
       */
      ID: number;
      BackendID: string;
      Amount: number;
      PaidAmount: number;
      Description: string;
      Type: PaymentTransactionDocument.PaymentTypeDocument;
      PaymentMethod: string;
      CurrencyID: string;
      AmountToRefund?: number;
      RefundedAmount?: number;
      Status: PaymentStatuses;
    }

    export namespace PaymentTransactionDocument {
      export interface PaymentTypeDocument {
        /**
         * Entity type: PaymentType
         */
        ID: number;
        Name: string;
      }

    }

    export interface StationDocument {
      /**
       * Entity type: Station
       */
      ID: number;
      Name: string;
    }

    export interface DeviceDocument {
      /**
       * Entity type: Device
       */
      ID: number;
      Name: string;
      IpAddress: string;
      Type: DeviceDocument.DeviceTypeDocument;
      OpenCashDrawerReason: DeviceDocument.OpenCashDrawerReasonDocument;
      EcrID: string;
      HardwareID?: string;
      ProxyID?: string;
      GlobalID?: string;
      StationID?: number;
    }

    export namespace DeviceDocument {
      export interface DeviceTypeDocument {
        ID: number;
        IDs: number[];
        Name: string;
        Names: string[];
      }

      export interface OpenCashDrawerReasonDocument {
        ID: number;
        Name: string;
        Description: string;
      }

    }

    export interface DiscountDocument {
      Trigger: DiscountTriggers;
      ActionType: string;
      ActionData: { [key: string]: any };
    }

    export interface MessageTemplateDocument {
      /**
       * Entity type: MessageTemplate
       */
      ID: number;
      Name: string;
      OrganizationUnitID?: number;
      LanguageID: string;
      CountryID: string;
      DestinationID: number;
    }

    export interface FinancialPeriodDocument {
      /**
       * Entity type: FinancialPeriod
       */
      ID: number;
    }

    export interface FinancialReportingPeriodDocument {
      ID: number;
      Type: FinancialReportingPeriodTypes;
      StartDate: string;
      EndDate: string;
    }

    export interface FinancialPeriodAuditDocument {
      /**
       * Entity type: FinancialPeriodAudit
       */
      ID: number;
    }

    export interface RoleDocument {
      /**
       * Entity type: Role
       */
      ID: number;
    }

    export interface UserDocument {
      /**
       * Entity type: User
       */
      ID: number;
      FirstName: string;
      FullName: string;
    }

    export interface TerminalReportDocument {
      CurrencyID: string;
      Number: string;
      Type: TerminalReportType;
      Supplier: Billable;
      Company: Billable;
      Date: string;
      Payments: TerminalReportDocument.Payment[];
      CreditTaxes: TerminalReportDocument.Tax[];
      DebitTaxes: TerminalReportDocument.Tax[];
      Taxes: TerminalReportDocument.Tax[];
      ProductGroups: TerminalReportDocument.ProductGroup[];
      PaymentsPerUser: TerminalReportDocument.PaymentPerUser[];
      CopyReceiptsPrinted: number;
      TotalCopyReceiptsAmount: number;
      ReceiptsPrinted: number;
      CashDrawerOpenings: number;
      ReturnCount: number;
      TotalReturnsAmount: number;
      DiscountCount: number;
      TotalDiscounts: number;
      Change: number;
      GrandTotalCash: number;
      GrandTotal: number;
      GrandTotalNet: number;
      GrandTotalReturns: number;
      OpeningAmount: number;
      ClosingAmount: number;
      CashDepositsTotalAmount: number;
      TotalAmount: number;
    }

    export namespace TerminalReportDocument {
      export interface Payment {
        Type: Payment.Metadata;
        Amount: number;
        Count: number;
      }

      export namespace Payment {
        export interface Metadata {
          ID: number;
          Name: string;
          Code: string;
        }

      }

      export interface Tax {
        Name: string;
        Code: string;
        Rate: number;
        Base: number;
        Amount: number;
      }

      export interface ProductGroup {
        Code: string;
        Amount: number;
        Count: number;
      }

      export interface PaymentPerUser {
        UserID: number;
        EmployeeNumber: string;
        Description: string;
        Amount: number;
        Count: number;
      }

    }

    export interface AuthenticationResponseDocument {
      Result: AuthenticationResults;
    }

  }

  export const enum FinancialReportingPeriodTypes {
    Month = 0,
    FiscalYear = 1
  }

  export const enum TerminalReportType {
    X = 0,
    Z = 1
  }

  export interface Billable {
    AuditingSystemID: string;
    SystemID: string;
    Name: string;
    RegistrationNumber: string;
    TaxRegistrationNumber: string;
    RegistrationCountryID?: string;
    TaxCountryRegion: string;
    PhoneNumber: string;
    Email: string;
    WasteDisposalNumber: string;
    TaxOfficeNumber: string;
    BankAccount?: string;
    Website?: string;
    BIC?: string;
    AuditingFields: { [key: string]: string };
    Address: Address;
    EstablishmentNumber?: string;
    EntityType?: string;
    SocialCapital?: number;
    IndustryCode?: string;
    IsBusinessToBusiness: boolean;
    CustomFields?: string;
    IsWebShop: boolean;
    IsShop: boolean;
  }

  export interface Address {
    Addressee?: string;
    Address1: string;
    Address2: string;
    Number: string;
    City: string;
    PostalCode: string;
    Country: string;
    Region: string;
    CountrySubdivisionID: string;
    State: string;
    Type: AddressType;
  }

  export const enum AddressType {
    Street = 0,
    Postal = 1,
    Billing = 2,
    ShipTo = 3,
    ShipFrom = 4
  }

  /**
   * List available event ledger types.
   */
  export interface GetEventLedgerTypes {
  }

  export interface GetEventLedgerTypesResponse {
    EventLedgerTypes: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists all `Financial Period Audit` with their possible BlobID's for audition purposes.
   */
  export interface ListFinancialPeriodAudits {
    PageConfig?: PageConfig<ListAuditsFilter>;
  }

  export interface ListAuditsFilter {
    /**
     * The user can see only the audit files for these organization units in case of limited visibility
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs: number[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    FiscalYear?: number;
    FiscalMonth?: number;
  }

  export interface ListFinancialPeriodAuditsResponse {
    Result?: PagedResult<FinancialPeriodAuditDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FinancialPeriodAuditDto {
    /**
     * Entity type: FinancialPeriodAudit
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    StatusID: number;
    /**
     * Entity type: Blob
     */
    BlobID?: string;
    BlobUrl?: string;
    FiscalYear?: number;
    FiscalMonth?: number;
    From?: string;
    To?: string;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreatedByFirstName: string;
    CreatedByLastName: string;
    CreationTime: string;
    Blobs: FinancialPeriodAuditBlobDto[];
    Status: FinancialPeriodAuditStatus;
    IsPreview: boolean;
  }

  export interface FinancialPeriodAuditBlobDto {
    Type?: string;
    BlobID: string;
    CreationTime: string;
    Url: string;
  }

  export const enum FinancialPeriodAuditStatus {
    New = 0,
    Processing = 10,
    ProcessingError = 11,
    Processed = 12,
    Exporting = 20,
    ExportingError = 21,
    Exported = 22,
    NothingToBeExported = 999
  }

  /**
   * Sets the lottery number for a given order
   */
  export interface SetLotteryNumber {
    /**
     * Entity type: Order
     */
    OrderID: number;
    LotteryNumber: string;
  }

  /**
   * Validates the given VAT number by the current organization unit auditing provider or by providing a country in the request
   */
  export interface ValidateVatNumber {
    VatNumber: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  export interface ValidateVatNumberResponse {
    Success: AuditingVatNumberValidationResult;
    Address: string;
    Name: string;
    CountryCode: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum AuditingVatNumberValidationResult {
    Unknown = 0,
    Success = 1,
    Failure = -1
  }

  /**
   * Returns all async requests for your current user
   */
  export interface GetAsyncRequestsForUser {
  }

  export interface GetAsyncRequestsForUserResponse {
    Requests: AsyncRequest[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface AsyncRequest {
    ID: string;
    Timestamp: string;
    Type: string;
    ResponseType: string;
    Status: AsyncServiceRequestStatus;
    ResponseBlobUrl: string;
  }

  export const enum AsyncServiceRequestStatus {
    InProgress = 0,
    Completed = 1,
    Failed = 2
  }

  /**
   * Returns a specific async request for your current user
   */
  export interface GetAsyncRequestForUser {
    /**
     * The ID of the async request to retrieve
     */
    ID: string;
  }

  export interface GetAsyncRequestForUserResponse {
    Request: AsyncRequest;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * @deprecated Feature not supported anymore.
   * Create app rule
   */
  export interface CreateAppRule {
    ClientName: string;
    FromVersion?: string;
    ToVersion: string;
    Type?: RuleType;
    Comment?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export const enum RuleType {
    Warning = 0,
    Error = 1
  }

  export interface CreateAppRuleResponse {
    /**
     * Entity type: AppRule
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * @deprecated Feature not supported anymore.
   * Delete app rule
   */
  export interface DeleteAppRule {
    /**
     * Entity type: AppRule
     */
    ID: number;
  }

  /**
   * @deprecated Feature not supported anymore.
   * Get all app rules
   */
  export interface GetAppRules {
  }

  export interface GetAppRulesResponse {
    Result: GetAppRulesResponse.AppRule[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAppRulesResponse {
    export interface AppRule {
      /**
       * Entity type: AppRule
       */
      ID: number;
      ClientName: string;
      FromVersion?: string;
      ToVersion: string;
      Type: RuleType;
      Comment?: string;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID?: number;
      IsSystem: boolean;
    }

  }

  /**
   * @deprecated Feature not supported anymore.
   * Update an app rule
   */
  export interface UpdateAppRule {
    /**
     * Entity type: AppRule
     */
    ID: number;
    FromVersion?: string;
    ToVersion?: string;
    Type?: RuleType;
    Comment?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export interface GetApplicationConfiguration {
  }

  export interface GetApplicationConfigurationResponse {
    Configuration: { [key: string]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetCurrentApplication {
  }

  export interface GetCurrentApplicationResponse {
    /**
     * Entity type: Application
     */
    ID: number;
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    RequiredUserTypeID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get a list of all active Applications
   */
  export interface ListApplications {
  }

  export interface ListApplicationsResponse {
    Result: ApplicationDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ApplicationDto {
    /**
     * Entity type: Application
     */
    ID: number;
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    BaseUrl: string;
    /**
     * Contains the base url where static files should be served from for the application.
     */
    AssetsBaseUrl: string;
  }

  /**
   * @deprecated This service is deprecated. Use the /api/definition.json endpoint instead.
   * Gets the description of the entire API
   */
  export interface GetApiDefinition {
  }

  export interface GetApiDefinitionResponse {
    ApiDefinition: { [key: string]: any };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for retrieving a list of available services
   */
  export interface GetAvailableServices {
  }

  export interface GetAvailableServicesResponse {
    Services: GetAvailableServicesResponse.Service[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAvailableServicesResponse {
    export interface Service {
      Name: string;
      Type: string;
      Namespace: string;
      Scope?: FunctionalityScope;
      Functionality?: string;
      AllowPublic: boolean;
      AvailableOffline: boolean;
      RequiresElevation: boolean;
      RequiresSudoMode: boolean;
    }

  }

  /**
   * Service for retrieving a list available service details
   */
  export interface GetAvailableServiceDetails {
    Type: string;
  }

  export interface GetAvailableServiceDetailsResponse {
    Description: string;
    Request: GetAvailableServiceDetailsResponse.TypeDefinition;
    Response: GetAvailableServiceDetailsResponse.TypeDefinition;
    Routes: GetAvailableServiceDetailsResponse.Route[];
    AvailableOffline: boolean;
    Deprecation: string;
    Security: GetAvailableServiceDetailsResponse.SecurityInfo;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetAvailableServiceDetailsResponse {
    export interface Route {
      Method: string;
      Path: string;
    }

    export interface SecurityInfo {
      RequiresAuthentication: boolean;
      RequiredFunctionalities: GetAvailableServiceDetailsResponse.FunctionalityInfo[];
      RequiredUserType: UserTypes;
      AllowPublic: boolean;
    }

    export interface FunctionalityInfo {
      Functionality: string;
      Scope: FunctionalityScope;
    }

    export interface TypeDefinition {
      Name: string;
      Description: string;
      Namespace: string;
      Type: string;
      Optional: boolean;
      Deprecation: string;
      Fields: GetAvailableServiceDetailsResponse.TypeDefinition[];
    }

  }

  /**
   * Get the version status of the client compared to the current version of the API. The header EVA-API-Version is required.
   */
  export interface GetCurrentVersion {
  }

  export interface GetCurrentVersionResponse {
    Status: VersionStatus;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum VersionStatus {
    Unknown = 0,
    None = 1,
    Old = 2,
    Ancient = 3
  }

  /**
   * *WARNING* NewBlack only service
   * 
   * Toggle on/off some core features of this environment
   */
  export interface ToggleFeatures {
    CacheRefreshDisabled?: boolean;
    PrefigureDiscountsDisabled?: boolean;
    DiscountEngineDisabled?: boolean;
    SearchUsersDisabled?: boolean;
    SearchProductsDisabled?: boolean;
    WaldoDisabled?: boolean;
    EventExportDisabled?: boolean;
    ScheduledTasksDisabled?: boolean;
    WebshopFinancialPeriodManagementDisabled?: boolean;
  }

  export interface ToggleFeaturesResponse {
    CacheRefreshDisabled: boolean;
    PrefigureDiscountsDisabled: boolean;
    DiscountEngineDisabled: boolean;
    SearchUsersDisabled: boolean;
    SearchProductsDisabled: boolean;
    WaldoDisabled: boolean;
    EventExportDisabled: boolean;
    ScheduledTasksDisabled: boolean;
    WebshopFinancialPeriodManagementDisabled: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Cancel the pending cash PaymentTransactions on the Order to remove the AML checks.
   */
  export interface CancelAMLChecksOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Validate all AML requirements and finalize the pending PaymentTransactions on the Order.
   */
  export interface FinalizeAMLChecksOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Force the AML checks to be finalized, even if the requirements are not met.
     */
    Force?: boolean;
  }

  export interface FinalizeAMLChecksOnOrderResponse {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionIDs: number[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Flag this order to trigger manual AML checks.
   */
  export interface TriggerAMLChecksOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Flag the Order that the customer has been verified for AML purposes.
   */
  export interface VerifyCustomerOnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * Adds a boarding pass to specified order, by barcode.
   */
  export interface AddBoardingPassToOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Barcode: string;
  }

  /**
   * Create a new Flight object
   */
  export interface CreateFlight {
    Flight: FlightDto;
  }

  export interface FlightDto {
    /**
     * Entity type: Flight
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    AirlineCompanyCode: string;
    FlightNumber: string;
    DestinationCode: number;
    DestinationAirportCode: string;
    FromDate: string;
    ToDate: string;
  }

  export interface CreateFlightResponse {
    /**
     * Entity type: Flight
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an existing Flight
   */
  export interface UpdateFlight {
    /**
     * Entity type: Flight
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    AirlineCompanyCode?: string;
    FlightNumber?: string;
    DestinationCode?: number;
    DestinationAirportCode?: string;
    FromDate?: string;
    ToDate?: string;
  }

  /**
   * Delete an existing Flight
   */
  export interface DeleteFlight {
    /**
     * Entity type: Flight
     */
    ID: number;
  }

  /**
   * Retrieves an existing Flight
   */
  export interface GetFlight {
    /**
     * Entity type: Flight
     */
    ID: number;
  }

  export interface GetFlightResponse {
    Flight: FlightDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List all the existing Flights mathing the specified filter
   */
  export interface ListFlights {
    PageConfig?: PageConfig<ListFlightsFilter>;
  }

  export interface ListFlightsFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    AirlineCompanyCode?: string;
    FlightNumber?: string;
    DestinationAirportCode?: string;
  }

  export interface ListFlightsResponse {
    Result?: PagedResult<FlightDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets boarding pass information set to given order.
   */
  export interface GetBoardingPass {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetBoardingPassResponse {
    Format: string;
    NumberOfLegs: number;
    Name: string;
    Type: string;
    OperatingCarrierPNRCode: string;
    DepartingAirportCode: string;
    DestinationAirportCode: string;
    AirlineCompanyCode: string;
    FlightNumber: number;
    JulianDate: number;
    Cabin: string;
    Seat: string;
    CheckinSequence: number;
    Status: number;
    Data: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates multiple Flight objects for a specific Organization Unit.
   *     The flights are identified from a GRC type file containing the necessary information.
   */
  export interface ImportFlights {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    Data: string;
  }

  /**
   * Download a template format for the flights file to upload.
   */
  export interface DownloadFlightsFileTemplate {
  }

  /**
   * Service for retrieving a list of `AddressBook`
   */
  export interface ListAddressBook {
    PageConfig?: PageConfig<ListAddressBookFilter>;
  }

  export interface ListAddressBookFilter {
    /**
     * Entity type: AddressBook
     */
    ID?: number;
    BackendID?: string;
    DefaultShippingAddress?: boolean;
    DefaultBillingAddress?: boolean;
    DefaultPickupAddress?: boolean;
    /**
     * Entity type: Address
     */
    AddressID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    Description?: string;
    /**
     * Entity type: Country
     */
    CountryIDs?: string[];
  }

  export interface ListAddressBookResponse {
    Result?: PagedResult<AddressBookDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface AddressBookDto {
    /**
     * Entity type: AddressBook
     */
    ID?: number;
    BackendID?: string;
    Address?: AddressDto;
    Description?: string;
    DefaultShippingAddress?: boolean;
    DefaultBillingAddress?: boolean;
    DefaultPickupAddress?: boolean;
    CreationTime?: string;
    LastModificationTime?: string;
    CreatedBy?: AddressBookDto.User;
    LastModifiedBy?: AddressBookDto.User;
  }

  export namespace AddressBookDto {
    export interface User {
      /**
       * Entity type: User
       */
      ID?: number;
      FirstName: string;
      LastName: string;
      FullName: string;
    }

  }

  /**
   * Set the given Address Book Item as default BillingAddress for the User of the Address Book
   */
  export interface SetDefaultBillingAddress {
    /**
     * Entity type: AddressBook
     */
    AddressBookID: number;
  }

  /**
   * Set the given Address Book Item as default PickupAddress for the User of the Address Book
   */
  export interface SetDefaultPickupAddress {
    /**
     * Entity type: AddressBook
     */
    AddressBookID: number;
  }

  /**
   * Set the given Address Book Item as default ShippingAddress for the User of the Address Book
   */
  export interface SetDefaultShippingAddress {
    /**
     * Entity type: AddressBook
     */
    AddressBookID: number;
  }

  /**
   * Create a new Address in the AddressBook of an User
   */
  export interface CreateAddressBookItem {
    /**
     * Entity type: User
     */
    UserID?: number;
    BackendID?: string;
    Description?: string;
    Address?: AddressDataDto;
    UseAsDefaultShippingAddress?: boolean;
    UseAsDefaultBillingAddress?: boolean;
    UseAsDefaultPickupAddress?: boolean;
  }

  export interface CreateAddressBookItemResponse {
    /**
     * Entity type: AddressBook
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Update an existing Address in the AddressBook of an User
   */
  export interface UpdateAddressBookItem {
    /**
     * Entity type: AddressBook
     */
    ID: number;
    BackendID?: string;
    Description?: string;
    Address?: AddressDataDto;
    UseAsDefaultShippingAddress?: boolean;
    UseAsDefaultBillingAddress?: boolean;
    UseAsDefaultPickupAddress?: boolean;
  }

  /**
   * Delete an Address in the AddressBook of an User
   * 
   * It's not allowed to delete an address when it's the default and there are more than 1 remaining items in the address book
   */
  export interface DeleteAddressBookItem {
    /**
     * Entity type: AddressBook
     */
    ID: number;
  }

  /**
   * Update an existing Address Requirement object.
   * 
   * This service is using the ID or a PropertyType-CountryID par as input for the finding of the
   * correct address requirement we want to update.
   * If the address requirement is found the visibility and validation of this will be updated.
   */
  export interface UpdateAddressRequirement {
    /**
     * Entity type: AddressRequirement
     */
    ID?: number;
    PropertyType?: PropertyType;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    Visibility?: VisibilityType;
    Validation?: string;
    /**
     * Min: 1
     */
    MinLength?: number;
    /**
     * Min: 1
     */
    MaxLength?: number;
    PredefinedValues?: string[];
  }

  export const enum PropertyType {
    FirstName = 1,
    LastName = 2,
    Address1 = 3,
    Address2 = 4,
    HouseNumber = 6,
    ZipCode = 7,
    Subdistrict = 8,
    District = 9,
    City = 10,
    State = 11,
    CountryID = 12,
    EmailAddress = 13,
    PhoneNumber = 14,
    CountrySubdivisionID = 15,
    Remark = 16
  }

  export const enum VisibilityType {
    Visible = 1,
    NotVisible = 2,
    VisibleAndRequired = 3
  }

  /**
   * Get all available address requirements.
   */
  export interface GetAddressRequirements {
    /**
     * Entity type: Country
     */
    CountryID: string;
    /**
     * When `UseAddressRequirements` isn't enable yet, this boolean can be used to force show the address requirements for configuration purposes.
     */
    ReturnAddressRequirements?: boolean;
  }

  export interface GetAddressRequirementsResponse {
    AddressRequirements: { [key: string]: AddressRequirementDto };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface AddressRequirementDto {
    /**
     * Entity type: AddressRequirement
     */
    ID?: number;
    Visible?: VisibilityType;
    Validation?: string;
    MinLength?: number;
    MaxLength?: number;
    PredefinedValues?: string[];
  }

  /**
   * Service for retrieving a list of suggested addresses
   */
  export interface AutocompleteAddress {
    Query: string;
    Countries?: string[];
    Language?: string;
  }

  export interface AutocompleteAddressResponse {
    Result: AddressSuggestion[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface AddressSuggestion {
    ID: string;
    Reference: string;
    Description: string;
  }

  /**
   * Service for retrieving a `Address` by specifying a Zip Code and House Number.
   * 
   * This only works for addresses in the Netherlands, for a more generic approach look a the AutocompleteAddress service
   */
  export interface GetAddressForZipCode {
    /**
     * Expected format: 9999ZZ
     */
    ZipCode: string;
    HouseNumber: string;
  }

  export interface GetAddressForZipCodeResponse {
    Result: AddressDataDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Service for retrieving address suggestion details by specifying a address suggestion retrieved from `AutocompleteAddress` service
   */
  export interface GetAutocompleteAddressByReference {
    Suggestion: AddressSuggestion;
  }

  export interface GetAutocompleteAddressByReferenceResponse {
    Result?: AddressSuggestionDetails;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface AddressSuggestionDetails {
    Formatted: string;
    Components: AddressSuggestionDetails.Component[];
    Geometry: AddressSuggestionDetails.GeometryDetails;
    UtcOffset: number;
    HouseNumber: AddressSuggestionDetails.Component;
    Street: AddressSuggestionDetails.Component;
    City: AddressSuggestionDetails.Component;
    Region: AddressSuggestionDetails.Component;
    District: AddressSuggestionDetails.Component;
    Subdistrict: AddressSuggestionDetails.Component;
    Country: AddressSuggestionDetails.Component;
    PostalCode: AddressSuggestionDetails.Component;
    IsValidCountry: boolean;
  }

  export namespace AddressSuggestionDetails {
    export interface Component {
      LongName: string;
      ShortName: string;
      Types: string[];
    }

    export interface GeometryDetails {
      Location: GeometryLocation;
    }

  }

  export interface GeometryLocation {
    Latitude: number;
    Longitude: number;
  }

  /**
   * Get the CountrySubdivisions
   * 
   * Optionally the request can be filtered on countryid
   */
  export interface GetCountrySubdivisions {
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  export interface GetCountrySubdivisionsResponse {
    CountrySubdivisions: CountrySubdivisionDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CountrySubdivisionDto {
    /**
     * Entity type: CountrySubdivision
     */
    ID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
    Name: string;
  }

  /**
   * Gets the full list of countries. When `FetchAll` is set to true, all the `Country` entries are returned, rather than just the ones limited by the `OrganizationUnitCountry` constraint (`false` or `null`, default).
   */
  export interface ListCountries {
    FetchAll?: boolean;
  }

  export interface ListCountriesResponse {
    Result: CountryDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CountryDto {
    /**
     * Entity type: Country
     */
    ID: string;
    Name: string;
    Code: string;
    Number: number;
    EU: boolean;
    EEA: boolean;
    EFTA: boolean;
  }

  /**
   * Update the addresses on a User
   * 
   * This is deprecated in favour of using the AddressBook services
   */
  export interface UpdateAddress {
    /**
     * Entity type: User
     */
    UserID: number;
    BillToShipping?: boolean;
    ShipToAddress?: AddressDataDto;
    BillToAddress?: AddressDataDto;
  }

  export interface ListUserBoughtProductsFilter {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
  }

  export interface IProductSearchItemPrimaryImage {
    Name: string;
    Type: string;
    Blob: string;
    Url: string;
  }

  export const enum FraudDataType {
    Address = 1,
    IBAN = 2,
    IdentificationDocument = 3
  }

  export interface ListFraudItemsFilter {
    Type?: FraudDataType;
    TypeID?: number;
  }

  export interface ListAssortmentsFilter {
    Name?: string;
    Code?: string;
    /**
     * Entity type: Assortment
     */
    AssortmentIDs?: number[];
  }

  export interface SimpleProductSearchModel {
    Query: string;
    Filters: { [key: string]: FilterModel };
  }

  export interface ListAssortmentProductsFilter {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    TypeID?: number;
    /**
     * Entity type: Assortment
     */
    AssortmentID?: number;
    /**
     * Entity type: Assortment
     */
    AssortmentIDs?: number[];
    IncludeDeleted?: boolean;
  }

  export interface ListVisibilityGroupsFilter {
    /**
     * Entity type: VisibilityGroup
     */
    IDs?: number[];
    Name?: string;
    Description?: string;
    /**
     * Using this filter will limit the result to a single item.
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface ListVisibilityGroupConfigurationsFilter {
    /**
     * Entity type: VisibilityGroup
     */
    FromID?: number;
    /**
     * Entity type: VisibilityGroup
     */
    ToID?: number;
    Orders?: boolean;
    Users?: boolean;
    OrganizationUnits?: boolean;
  }

  export interface UserRequirementProperties {
    Type: string;
    CustomValidators?: { [key: string]: { [key: string]: string } };
  }

  export interface UserRequirementResponse {
    ID: number;
    Display: boolean;
    RequiredFor: UserRequirementFor;
    UserType?: UserTypes;
    AccountType?: UserAccountType;
    Preferred: boolean;
    ShippingMethodID?: number;
    CustomValidators: { [key: string]: any };
  }

  export const enum UserRequirementFor {
    None = 0,
    Create = 1,
    PlaceOrder = 2,
    Payment = 4,
    All = 7
  }

  export interface UserRequirementForSubscriptionResponse {
    Property: string;
    Display: boolean;
    RequiredFor: UserRequirementFor;
    UserType?: UserTypes;
    AccountType?: UserAccountType;
    Preferred: boolean;
    CustomValidators: { [key: string]: any };
  }

  export const enum OrganizationUnitSetTypes {
    System = 0,
    Custom = 1,
    AdHoc = 2
  }

  export const enum SubscriptionType {
    Default = 0,
    Loyalty = 1
  }

  export interface ListSubscriptionUsersFilter {
    /**
     * Entity type: Subscription
     */
    SubscriptionID?: number;
    Status?: SubscriptionStatus;
  }

  export interface ListUserRequirementsFilter {
    /**
     * Entity type: UserRequirement
     */
    IDs?: number[];
    Property?: string;
    /**
     * Entity type: Subscription
     */
    SubscriptionID?: number;
    ShippingMethodID?: number;
    /**
     * Will filter on exact matches
     */
    UserType?: UserTypes;
    /**
     * Will filter on any matches
     */
    OneOfUserTypes?: UserTypes;
    RequiredFor?: UserRequirementFor;
    Preferred?: boolean;
    Display?: boolean;
    /**
     * Entity type: UserRequirementSet
     */
    UserRequirementSetID?: number;
  }

  export interface ListUserRequirementSetsFilter {
    /**
     * Entity type: UserRequirementSet
     */
    IDs?: number[];
    Name?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface ListWishlistsFilter {
    /**
     * Entity type: User
     */
    UserID?: number;
    Name?: string;
  }

  export const enum SurveyContext {
    None = 0,
    User = 1,
    Order = 2,
    OrganizationUnit = 3
  }

  export interface ListSurveyFilter {
    Name?: string;
    BackendID?: string;
    SurveyTrigger?: string;
    Description?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * You can filter surveys based on their deactivation reason.The array is interpreted as an 'or' value list
     * To get surveys with a specific deactivation reason, only provide the deactivation reason you want. For example:
     * - providing an empty value will return all surveys
     * - providing `None` (0) will return surveys that are active.
     * - providing `Unpublished` (1) will return surveys that are inactive because they are unpublished.
     * - providing `Unpublished` (1) and `DeactivatedByUser` (2) will return surveys that are inactive because they are either unpublished OR deactivated by user.
     * - providing `Unpublished` (1) and `DeactivatedByUser` (2) as a combined flag (3) will return surveys that are inactive because they are unpublished AND deactivated by user.
     */
    DeactivationReasons?: SurveyDeactivationReasons[];
    TimeRange?: ListSurveyFilter.TimeRangeDto;
  }

  export namespace ListSurveyFilter {
    export interface TimeRangeDto {
      StartDate?: string;
      EndDate?: string;
    }

  }

  export const enum SurveyDeactivationReasons {
    None = 0,
    Unpublished = 1,
    DeactivatedByUser = 2,
    BySurveyTimeFrame = 4
  }

  export interface BlobDto {
    Url: string;
    MimeType: string;
  }

  export interface SurveyAfterEffectDto {
    /**
     * Entity type: SurveyAfterEffect
     */
    ID?: number;
    Type: string;
    Data: any;
  }

  export const enum CustomFieldDataTypes {
    String = 0,
    Bool = 1,
    /**
     * Whole numbers, no fractions
     */
    Integer = 2,
    Decimal = 3,
    Enum = 4,
    /**
     * Multiline string
     */
    Text = 5,
    DateTime = 6,
    Date = 7,
    Blob = 8
  }

  export interface SurveyQuestionRouteFilter {
    FilterType: CustomFieldFilterType;
    Filters: SurveyQuestionRouteFilter[];
    Conditions: SurveyQuestionRouteFilterCondition[];
  }

  export interface SurveyQuestionRouteFilterCondition {
    Operator: CustomFieldValueOperator;
    CustomFieldValue: CustomFieldValue;
  }

  export const enum CustomFieldValueOperator {
    Equals = 0,
    NotEquals = 1,
    LowerThan = 2,
    LowerThanOrEqualTo = 3,
    HigherThanOrEqualTo = 4,
    HigherThan = 5,
    Contains = 7,
    DoesNotContain = 8,
    StartsWith = 9,
    DoesNotStartWith = 10,
    EndsWith = 11,
    DoesNotEndWith = 12,
    HasValue = 13,
    DoesntHaveValue = 14
  }

  export const enum SurveyResponseStatus {
    Open = 0,
    Started = 1,
    Completed = 2,
    Cancelled = 3,
    Terminated = 4
  }

  export interface StockMutationFilters {
    ID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    MutationQuantity?: number;
    Remark: string;
    /**
     * Entity type: StockLabel
     */
    SourceStockLabelID?: number;
    /**
     * Entity type: StockLabel
     */
    DestinationStockLabelID?: number;
    /**
     * Entity type: StockMutationReasons
     */
    ReasonIDs: number[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    FromDate?: string;
    ToDate?: string;
    /**
     * Entity type: User
     */
    UserIDs: number[];
    /**
     * Entity type: Order
     */
    OrderIDs: number[];
    AutoCorrectedQuantities?: boolean;
    SerialNumbers: string[];
  }

  export interface SearchStockMutationResult {
    /**
     * Entity type: StockMutation
     */
    ID: number;
    CreationTime: string;
    Product: SearchStockMutationResult.ProductResult;
    OrganizationUnit: SearchStockMutationResult.IdNameValue;
    SourceStockLabel: SearchStockMutationResult.IdNameValue;
    DestinationStockLabel: SearchStockMutationResult.IdNameValue;
    MutationQuantity: number;
    AutomaticCorrectionQuantity: number;
    UnitCost: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    Reason: SearchStockMutationResult.ReasonResult;
    Type: SearchStockMutationResult.IdNameValue;
    Remark: string;
    OrderLine: SearchStockMutationResult.OrderLineResult;
    UserTask: SearchStockMutationResult.UserTaskResult;
    ShipmentLine: SearchStockMutationResult.ShipmentLineResult;
    CreatedBy: SearchStockMutationResult.UserResult;
    SerialNumber?: string;
  }

  export namespace SearchStockMutationResult {
    export interface IdNameValue {
      /**
       * Entity type: Enum
       */
      ID: number;
      Name: string;
    }

    export interface UserResult {
      /**
       * Entity type: User
       */
      ID: number;
      EmailAddress: string;
      FirstName: string;
      LastName: string;
    }

    export interface ProductResult {
      /**
       * Entity type: Product
       */
      ID: number;
      CustomID: string;
      BackendID: string;
      PrimitiveName: string;
      Content?: any;
    }

    export interface OrderLineResult {
      /**
       * Entity type: OrderLine
       */
      ID?: number;
      OrderID?: number;
      OrderDisplayID?: string;
    }

    export interface UserTaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Type: SearchStockMutationResult.IdNameValue;
    }

    export interface ShipmentLineResult {
      /**
       * Entity type: ShipmentLine
       */
      ID: number;
      Shipment: SearchStockMutationResult.ShipmentResult;
    }

    export interface ShipmentResult {
      /**
       * Entity type: Shipment
       */
      ID: number;
      TrackingCode: string;
    }

    export interface ReasonResult {
      /**
       * Entity type: StockMutationReasons
       */
      ID: number;
      Name: string;
      BackendID: string;
    }

  }

  export interface ListStockMutationReasonsModelFilters {
    Name?: string;
    Query?: string;
    /**
     * Entity type: StockMutationReasons
     */
    IDs?: number[];
  }

  export interface ListCumulativeStockFilter {
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    OrganizationUnitType?: OrganizationUnitTypes;
    OrganizationUnitName: string;
    HasStock?: boolean;
  }

  export const enum StockAllocationRuleValueTypes {
    Quantity = 0,
    Percentage = 1
  }

  export const enum StockAllocationRuleTypes {
    Reservation = 0,
    Limitation = 1
  }

  export interface ListStockAllocationRulesFilter {
    /**
     * Entity type: OrganizationUnitSupplier
     */
    OrganizationUnitSupplierID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface ListStockAllocationRulesItem {
    /**
     * Entity type: StockAllocationRule
     */
    ID: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID: number;
    SupplierOrganizationUnitName: string;
    /**
     * Entity type: OrganizationUnitSupplier
     */
    OrganizationUnitSupplierID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    Value: number;
    ValueType: StockAllocationRuleValueTypes;
    RefillPeriodInDays?: number;
    LastRefill?: string;
    Type: StockAllocationRuleTypes;
    StartDate?: string;
    EndDate?: string;
    Priority?: number;
  }

  export interface ListCarriersFilter {
    Name: string;
  }

  export interface ListShippingMethodsFilter {
    /**
     * Entity type: Carrier
     */
    CarrierID?: number;
    Name?: string;
    Code?: string;
    CarrierName?: string;
  }

  export interface ListOrganizationUnitShippingMethodsFilter {
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export interface ListShippingCostsFilter {
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
  }

  export interface ListTransportationTimesFilter {
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    FromCountryID?: string;
    ToCountryID?: string;
    TimeInDays?: number;
  }

  export interface ShippingMethodTransportationTimeItem {
    /**
     * Entity type: ShippingMethodTransportationTime
     */
    ID: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    ShippingMethodName?: string;
    FromCountryID?: string;
    ToCountryID?: string;
    TimeInDays: number;
  }

  export const enum ShippingRestrictionType {
    SoldFromOrganizationUnit = 0,
    ShipFromOrganizationUnit = 1
  }

  export interface ListShippingRestrictionsFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    ProductPropertyTypeID?: string;
    Type?: ShippingRestrictionType;
    ZipCodeFrom?: string;
    ZipCodeTo?: string;
  }

  export interface ListUserOrganizationUnitRolesFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface ListRolesFilter {
    /**
     * Entity type: Role
     */
    IDs?: number[];
    Name?: string;
    Code?: string;
    UserTypeID?: number;
    DeactivateUserAfterInactiveDays?: number;
    Functionalities?: ListRolesFilter.RoleFilterFunctionality[];
    ExactFunctionalitiesOnly?: boolean;
  }

  export namespace ListRolesFilter {
    export interface RoleFilterFunctionality {
      Functionality: string;
      Scope?: FunctionalityScope;
    }

  }

  export interface ListProductBarcodesFilter {
    ProductID?: number;
    Barcode: string;
    UnitOfMeasureID?: number;
    IsSupplierProduct?: boolean;
    Origin?: ProductBarcodeOrigin;
  }

  export const enum ProductBarcodeOrigin {
    Imported = 0,
    UserDefined = 1
  }

  export const enum ProductRequirementDataTypes {
    String = 0,
    Bool = 1,
    Integer = 2,
    Decimal = 3,
    Enum = 4,
    Text = 5
  }

  export interface ProductRequirementOptions {
    EnumValues?: { [key: string]: string };
    IsArray?: boolean;
    IsRequired?: boolean;
  }

  export interface ListProductRequirementsFilter {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    Name?: string;
    DataType?: ProductRequirementDataTypes;
    OrderType?: OrderTypes;
    IsArray?: boolean;
    IsRequired?: boolean;
    Handler?: string;
    BackendID?: string;
  }

  export interface ProductSearchTemplateFilters {
    Name?: string;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  export const enum ProductSearchTemplateTypes {
    Uncached = 0,
    Cached = 1,
    Static = 2
  }

  export interface ListProductUnitOfMeasuresFilter {
    ProductID?: number;
    UnitOfMeasureID?: number;
    Quantity?: number;
  }

  export interface ListSupplierProductsFilter {
    SupplierOrganizationUnitID: number;
    BackendID: string;
    PrimitiveName: string;
  }

  export interface ListPriceListsForOrganizationUnitFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: PriceList
     */
    PriceListID?: number;
    /**
     * Entity type: PriceListUsageType
     */
    PriceListUsageTypeID?: number;
    IncludeInheritedOrganizationUnits: boolean;
  }

  export interface ListProductPriceLedgerFilter {
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    /**
     * Entity type: PriceList
     */
    PriceListID?: number;
    /**
     * Entity type: PriceListUsageType
     */
    PriceListUsageTypeID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs: number[];
    FromDate?: string;
  }

  export interface RoundingData {
    Targets: RoundingData.RoundingTarget[];
    Direction: RoundingDirection;
    MaximumTolerance?: number;
  }

  export namespace RoundingData {
    export interface RoundingTarget {
      Amount: number;
      Direction?: RoundingDirection;
      AmountType: RoundingAmountType;
      MaximumTolerance?: number;
    }

  }

  export const enum RoundingDirection {
    Closest = 0,
    Up = 1,
    Down = 2
  }

  export const enum RoundingAmountType {
    Fraction = 0,
    Divisor = 1
  }

  export const enum PriceListAdjustmentSystemType {
    Default = 0,
    SecondChance = 1
  }

  export interface ListPriceListAdjustmentsFilter {
    /**
     * Entity type: PriceList
     */
    PriceListID?: number;
    /**
     * Entity type: PriceListAdjustment
     */
    ParentID?: number;
    Type: string;
  }

  export interface ListPriceListsFilter {
    Name: string;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    IncludingVat?: boolean;
    IsActive?: boolean;
    /**
     * Only show price lists that are used somewhere with this usage type. Can be combined with a filter on `OrganizationUnitSetID` to only show pricelists that are used with a specific usagetype in specific organizationunits.
     * Entity type: PriceListUsageType
     */
    PriceListUsageTypeID?: number;
    /**
     * Only show price lists that are used in this organizationunit set. Can be combined with a filter on `PriceListUsageTypeID` to only show pricelists that are used with a specific usagetype in specific organizationunits.
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export interface ListManualInputAdjustmentsFilter {
    /**
     * Entity type: PriceListAdjustment
     */
    PriceListAdjustmentID: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    IsActive?: boolean;
  }

  export interface ListPriceListOrganizationUnitsFilter {
    /**
     * Entity type: PriceList
     */
    PriceListID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    TargetOrganizationUnitID?: number;
    /**
     * Entity type: PriceListUsageType
     */
    PriceListUsageTypeID?: number;
  }

  export interface ListPersonalizedPromotionsFilter {
    Name: string;
    Description: string;
    BackendID: string;
    LockAfterFirstApply?: boolean;
  }

  export interface ListSettlementFilesFilter {
    Name?: string;
    Status?: PaymentTransactionSettlementFileStatus;
    Handler?: string;
  }

  export const enum PaymentTransactionSettlementFileStatus {
    New = 0,
    Processing = 1,
    Processed = 2,
    Failed = -10
  }

  export interface ListSettlementsFilter {
    /**
     * Entity type: PaymentTransactionSettlementFile
     */
    SettlementFileID?: number;
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: PaymentTransactionSettlementType
     */
    Type?: number;
    Offset1?: string;
    Offset2?: string;
    Offset3?: string;
    Offset4?: string;
  }

  export interface ListPaymentTransactionCapturesFilter {
    /**
     * Entity type: PaymentTransaction
     */
    PaymentTransactionID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Statuses?: PaymentTransactionCaptureStatuses[];
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    IsSettled?: boolean;
  }

  export const enum PaymentTransactionCaptureStatuses {
    /**
     * Payment Transaction Capture is created, but not yet communicated with the PSP.
     */
    New = 0,
    /**
     * The PSP is aware of the payment transaction capture and allows (or is) processing it.
     */
    Pending = 1,
    /**
     * The PSP has confirmed that the payment capture is complete.
     */
    Confirmed = 10,
    /**
     * The PSP has confirmed that the capture has failed.
     */
    Failed = -1
  }

  export interface ListPaymentTransactionsFilter {
    /**
     * Entity type: PaymentType
     */
    TypeIDs?: number[];
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: Device
     */
    DeviceID?: number;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Only works with `OrganizationUnitID`.
     */
    IncludeChildOrganizationUnits?: boolean;
    BackendID?: string;
    BackendSystemID?: string;
    Statuses?: PaymentStatuses[];
    MinAmount?: number;
    MaxAmount?: number;
    MinPaidAmount?: number;
    MaxPaidAmount?: number;
    MinSettledAmount?: number;
    MaxSettledAmount?: number;
    IsSettled?: boolean;
    FromDate?: string;
    ToDate?: string;
    SettlementOffset1?: string;
    SettlementOffset2?: string;
    SettlementOffset3?: string;
    SettlementOffset4?: string;
  }

  export interface ListOrganizationUnitSetPaymentTypesFilter {
    /**
     * Entity type: PaymentType
     */
    PaymentTypeID?: number;
    PaymentTypeName?: string;
    PaymentTypeCode?: string;
    PaymentMethodName?: string;
    PaymentMethodCode?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetIDs?: number[];
    RefundPriority?: number;
    CaptureMoment?: PaymentTypeCaptureMoment;
    DisableForCarryOutOrders?: boolean;
    DisableForReserveOrders?: boolean;
    DisableForDeliveryOrders?: boolean;
    IsActive?: boolean;
  }

  export interface ListPaymentMethodsFilter {
    Name?: string;
    Code?: string;
    IsCustom?: boolean;
  }

  export const enum PaymentReturnActions {
    Default = 0,
    Blocked = 1,
    Forced = 2
  }

  export const enum PaymentTypeCategory {
    Other = 0,
    Debit = 1,
    Credit = 2,
    Cash = 3,
    Voucher = 4,
    Online = 5,
    Rounding = 6,
    Exchange = 7
  }

  export interface PaymentTypeOptions {
    PspType?: string;
    PspSubType?: string;
    /**
     * Entity type: GiftCardConfiguration
     */
    GiftCardConfigurationID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    PinHandler?: string;
    PinManualKeyEntry?: boolean;
    RequiredUserType?: UserTypes;
    RequiredFunctionality?: string;
    RequiredFunctionalityScope?: FunctionalityScope;
    Pending?: boolean;
    RefundPending?: boolean;
    /**
     * Setting this to false results in refund only, will therefor only work with CanBeRefunded:true and CanRefundWithoutOriginalPaymentTransaction:true
     */
    CanBeUsedForPayments?: boolean;
    CanBeRefunded?: boolean;
    CanBeCancelled?: boolean;
    CanRefundWithoutOriginalPaymentTransaction?: boolean;
    RequiresInteractionForRefunds?: boolean;
    RequiresAmount?: boolean;
    RequiresConfirmationForPayments?: boolean;
    RequiresConfirmationForRefunds?: boolean;
    OnlyAvailableInLocalMode?: boolean;
    BlockOverPayments?: boolean;
  }

  export interface ListPaymentTypesFilter {
    /**
     * Entity type: PaymentType
     */
    IDs?: number[];
    Name?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    IsActive?: boolean;
    IsRoundingType?: boolean;
    PrintOnDocuments?: boolean;
    ReturnAction?: PaymentReturnActions;
    AutoFinalizeOnOrderPaid?: boolean;
    CashJournalMethod?: PaymentCashJournalMethod;
    Code?: string;
    /**
     * Entity type: PaymentMethod
     */
    PaymentMethodID?: number;
    PaymentMethodName?: string;
    PaymentMethodCode?: string;
    PaymentMethodIsCustom?: boolean;
    BackendID?: string;
  }

  export interface OrganizationUnitSetDefinition {
    CountryFilter?: OrganizationUnitSetDefinition.FilterDefinition;
    LanguageFilter?: OrganizationUnitSetDefinition.FilterDefinition;
    TypeFilter?: OrganizationUnitTypes;
  }

  export namespace OrganizationUnitSetDefinition {
    export interface FilterDefinition {
      Values: string[];
      /**
       * @default false
       */
      Negate?: boolean;
    }

  }

  export const enum OrganizationUnitSetOperatorTypes {
    Add = 0,
    Remove = 1
  }

  export interface ListOrganizationUnitSetsFilter {
    Name?: string;
    BackendID?: string;
    NameOrBackendID?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    ID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    IDs?: number[];
    HasSubsets?: boolean;
    Types?: OrganizationUnitSetTypes[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Return only sets that contain all the given OrganizationUnits
     * Entity type: OrganizationUnit
     */
    ContainingOrganizationUnitIDs?: number[];
    IncludedOrganizationUnitTypes?: OrganizationUnitTypes;
    ExcludedOrganizationUnitTypes?: OrganizationUnitTypes;
    /**
     * Entity type: OrganizationUnitSetScope
     */
    ScopeID?: number;
    IncludeWithoutScopeID?: boolean;
    Functionality?: string;
    FunctionalityScope?: FunctionalityScope;
  }

  export interface ListOrganizationUnitSetsItem {
    /**
     * Entity type: OrganizationUnitSet
     */
    ID: number;
    BackendID?: string;
    Name: string;
    GlobalID: string;
    Description: string;
    Type: OrganizationUnitSetTypes;
    OrganizationUnitCount: number;
    OrganizationUnitSubsetCount: number;
    /**
     * Entity type: OrganizationUnitSetScope
     */
    ScopeID?: number;
    ScopeName: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OrganizationUnitBackendID?: string;
  }

  export interface ListCashHandlersFilter {
    Name: string;
    CurrencyID: string;
  }

  export interface OrganizationUnitCurrencyFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    IncludeInheritedOrganizationUnits?: boolean;
  }

  export interface ListOldOpeningHoursFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: OpeningHoursType
     */
    TypeIDs?: number[];
    DayOfWeeks?: DayOfWeek[];
    /**
     * Inclusive
     */
    StartDate?: string;
    /**
     * Inclusive
     */
    EndDate?: string;
    /**
     * When the OpeningHour has no date, but only a weekday
     */
    IsRegular?: boolean;
    IsActive?: boolean;
  }

  export interface OpeningHoursDataDto {
    /**
     * Entity type: OpeningHoursType
     */
    TypeID?: number;
    TypeName?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Only set when returned outside OrganizationUnit context
     */
    OrganizationUnitName?: string;
    DayOfWeek?: DayOfWeek;
    StartTime?: string;
    EndTime?: string;
    Date?: string;
    Description?: string;
    IsSpecial?: boolean;
    IsClosed?: boolean;
    IsActive?: boolean;
  }

  export interface ListCheckoutOptionCategoriesFilter {
    Name?: string;
  }

  export interface ListRefundCorrectionReasonsFilter {
    Name?: string;
    Description?: string;
    BackendID?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitSetIDs?: number[];
  }

  export interface ListUnitPriceCorrectionReasonsFilter {
    Name?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitSetIDs?: number[];
  }

  export const enum ReturnReasonRemarkType {
    Hidden = 0,
    Show = 1,
    Required = 2
  }

  export const enum MessageTemplateTypes {
    Template = 1,
    Partial = 2,
    Layout = 3
  }

  export const enum MessageTemplateDestinations {
    Mail = 1,
    Sms = 2,
    Pdf = 4,
    Thermal = 8,
    Notification = 16,
    Http = 32
  }

  export interface MessageTemplateFilter {
    Name?: string;
    ExactName?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    TypeID?: MessageTemplateTypes;
    DestinationID?: MessageTemplateDestinations;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export const enum MessageTargetContentTypes {
    Html = 0,
    Pdf = 1,
    Plain = 2,
    Png = 3
  }

  export interface PageTokenConfig<T = { [key:string]: string }> {
    /**
     * Min: 1
     */
    Limit?: number;
  }

  export interface ListLoyaltyProgramsFilter {
    /**
     * Entity type: LoyaltyProgramGroup
     */
    LoyaltyProgramGroupID?: number;
    Name?: string;
    BackendID?: string;
    Handler?: string;
    Options?: LoyaltyProgramOptions;
    LoyaltyPointUsageOptions?: LoyaltyPointUsageOptions;
    Status?: LoyaltyProgramStatus;
    /**
     * Entity type: Company
     */
    CompanyID?: number;
  }

  export const enum LoyaltyProgramUsageTypes {
    None = 0,
    AllowAwarding = 1,
    UsableAsPaymentMethod = 2,
    UsableForWithdraw = 4
  }

  export const enum LoyaltyProgramStatus {
    Inactive = 0,
    Active = 1
  }

  export const enum LoyaltyProgramDeactivationReasons {
    None = 0,
    DeactivatedByGroup = 1,
    ManuallyDeactivated = 2,
    TimeFrame = 4,
    BudgetReached = 8
  }

  export interface ListLoyaltyProgramGroupFilter {
    Name?: string;
    BackendID?: string;
    Status?: LoyaltyProgramStatus;
    DeactivationReason?: LoyaltyProgramGroupDeactivationReasons;
  }

  export const enum LoyaltyProgramGroupDeactivationReasons {
    None = 0,
    ManuallyDeactivated = 1,
    TimeFrame = 2,
    BudgetReached = 4
  }

  export const enum LoyaltyPaymentTaxHandlingType {
    InTax = 0,
    UpToTax = 1
  }

  export const enum UserBudgetDeduction {
    None = 0,
    Program = 1,
    PaymentMethod = 2,
    Both = 3
  }

  export interface ListPointAwardingRuleFilter {
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    Name?: string;
    BackendID?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Enabled?: boolean;
  }

  export interface ListInquiriesFilter {
    Name?: string;
    Description?: string;
  }

  export interface ListGiftCardConfigurationsFilter {
    Name?: string;
    Handler?: string;
    UsageType?: GiftCardConfigurationUsageTypes;
  }

  export interface ListProductGiftCardsFilter {
    /**
     * Entity type: GiftCardConfiguration
     */
    GiftCardConfigurationID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    Type?: string;
  }

  export interface ListExchangeRatesFilter {
    /**
     * Entity type: Currency
     */
    CurrencyID?: string;
    /**
     * Entity type: Currency
     */
    BaseCurrencyID?: string;
  }

  export interface ListCashExpenseTypesFilter {
    Name?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetIDs?: number[];
  }

  export interface ListCashCorrectionReasonsFilter {
    Name?: string;
    BackendID?: string;
  }

  export interface ListProductCostPriceLedgerFilter {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    FromDate?: string;
    ToDate?: string;
  }

  export interface ListProductCostPriceLedgerItem {
    ID: number;
    CreationTime: string;
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductBackendID: string;
    ProductName: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    /**
     * Entity type: Order
     */
    OrderID: number;
    OrderDisplayID?: string;
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    StockLabelName: string;
    CurrencyID: string;
    Quantity: number;
    UnitCost: number;
  }

  export interface ListStockMutationUnitCostSourceLedgerFilter {
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * Entity type: OrderLine
     */
    SourceOrderLineID?: number;
    /**
     * Entity type: Order
     */
    SourceOrderID?: number;
    /**
     * Entity type: OrderLine
     */
    DestinationOrderLineID?: number;
    /**
     * Entity type: Order
     */
    DestinationOrderID?: number;
    /**
     * Entity type: StockMutation
     */
    StockMutationID?: number;
  }

  export interface ListStockMutationUnitCostSourceLedgerItem {
    ID: number;
    /**
     * Entity type: StockMutation
     */
    StockMutationID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductBackendID: string;
    ProductName: string;
    MutationQuantity: number;
    Quantity: number;
    OldQuantity: number;
    NewQuantity: number;
    UnitCost: number;
    /**
     * Entity type: Order
     */
    SourceOrderID?: number;
    /**
     * Entity type: OrderLine
     */
    SourceOrderLineID?: number;
    /**
     * Entity type: Order
     */
    DestinationOrderID?: number;
    /**
     * Entity type: OrderLine
     */
    DestinationOrderLineID?: number;
    CreationTime: string;
    OrganizationUnitName: string;
    StockLabelName: string;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
  }

  export const enum EntityFieldValidatorEntityTypes {
    PriceList = 1,
    PriceListAdjustment = 2
  }

  export const enum EntityFieldValidatorType {
    DefaultValidator = 0,
    StringValidator = 1
  }

  export const enum EntityFieldValidatorUserTypes {
    Employee = 1,
    Customer = 2,
    Api = 4,
    All = 7
  }

  export interface ListEntityFieldValidatorsFilter {
    Entity?: EntityFieldValidatorEntityTypes;
    UserTypes?: EntityFieldValidatorUserTypes;
  }

  export const enum EndpointConfigurationStatus {
    Enabled = 1,
    Disabled = 2,
    Error = 3
  }

  export interface ListDiscountCampaignsFilter {
    Name?: string;
    StartDate?: string;
    EndDate?: string;
    BackendSystemID?: string;
    BackendID?: string;
    BudgetCurrencyID?: string;
  }

  export interface ListDiscountCouponsFilter {
    CouponCode?: string;
    /**
     * Entity type: Discount
     */
    DiscountID?: number;
  }

  export interface ListDiscountLayerFilter {
    Name: string;
    IsExclusive?: boolean;
  }

  export const enum DiscountOrderTypes {
    None = 0,
    Sales = 1,
    Purchase = 2,
    All = 3
  }

  export const enum DaysOfWeek {
    None = 0,
    Monday = 1,
    Tuesday = 2,
    Wednesday = 4,
    Thursday = 8,
    Friday = 16,
    Saturday = 32,
    Sunday = 64,
    All = 127
  }

  export const enum DiscountConditionTypes {
    OneOf = 0,
    All = 1
  }

  export const enum FinancialDespersionType {
    DivideInProportionToProductPrice = 0,
    MostExpensiveToCheapest = 1,
    CheapestToMostExpensive = 2,
    HighestToLowestTaxRate = 3,
    LowestToHighestTaxRate = 4
  }

  export const enum DiscountCompliancyRule {
    None = 0,
    EmployeeDiscountTaxExclusion = 1
  }

  export interface CustomFieldValidationErrorData {
    /**
     * Entity type: CustomField
     */
    Errors: { [key: number]: CustomFieldValidationErrorData.Error };
    /**
     * Entity type: CustomField
     */
    CustomFieldOptions: { [key: number]: CustomFieldOptions };
  }

  export namespace CustomFieldValidationErrorData {
    export interface Error {
      Type: string;
      Message: string;
    }

  }

  export interface SearchDiscountsByQueryFilter {
    /**
     * Entity type: Discount
     */
    ID?: number;
    /**
     * You can filter discounts based on their deactivation reason.The array is interpreted as an 'or' value list
     * To get discounts with a specific deactivation reason, only provide the deactivation reason you want. For example:
     * - providing an empty value will return all discounts
     * - providing `None` (0) will return discounts that are active.
     * - providing `Unverified` (1) will return discounts that are inactive because they are unverified.
     * - providing `Unverified` (1) and `DisabledByUser` (2) will return discounts that are inactive because they are either unverified OR disabled by user.
     * - providing `Unverified` (1) and `DisabledByUser` (2) as a combined flag (3) will return discounts that are inactive because they are unverified AND disabled by user.
     */
    DeactivationReasons: DiscountDeactivationReasons[];
    DiscountTriggerIDs: DiscountTriggers[];
    Description: string;
    CouponCode: string;
    /**
     * The name of the user who created the discount
     */
    CreatedBy: string;
    BackendID: string;
    OrderType?: OrderTypes;
    /**
     * The discount layer
     * Entity type: DiscountLayer
     */
    LayerID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: Currency
     */
    CurrencyID: string;
    /**
     * Entity type: DiscountTemplate
     */
    DiscountTemplateID?: number;
    /**
     * Entity type: DiscountCampaign
     */
    DiscountCampaignID?: number;
    ActiveRange?: SearchDiscountsByQueryFilter.DiscountActiveRange;
    HasRestitutionOrganizationUnit?: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    RestitutionOrganizationUnitID?: number;
  }

  export namespace SearchDiscountsByQueryFilter {
    export interface DiscountActiveRange {
      StartDate?: string;
      EndDate?: string;
    }

  }

  export const enum DiscountDeactivationReasons {
    None = 0,
    Unverified = 1,
    DisabledByUser = 2,
    ByDiscountUsage = 4,
    ByDiscountBudget = 8,
    ByDiscountTimeFrame = 16,
    ByDiscountCampaignBudget = 32,
    ByDiscountCampaignTimeFrame = 64
  }

  export interface ListDiscountTemplatesFilter {
    Description?: string;
    TriggerIDs?: DiscountTriggers[];
    ActionType?: string;
    IsActive?: boolean;
  }

  export interface ListCustomFieldsFilter {
    /**
     * Entity type: CustomFieldType
     */
    TypeID?: number;
    TypeKey?: string;
    Name?: string;
    DataType?: CustomFieldDataTypes;
    BackendID?: string;
  }

  export interface ListCustomFieldOptionsFilter {
    /**
     * Entity type: CustomField
     */
    CustomFieldID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export interface CultureFilter {
    /**
     * Entity type: Language
     */
    LanguageID: string;
    /**
     * Entity type: Country
     */
    CountryID: string;
  }

  export interface OrganizationUnitCountryFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    CountryID: string;
    IncludeInheritedOrganizationUnits: boolean;
  }

  export interface OrganizationUnitLanguageFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    LanguageID: string;
    IncludeInheritedOrganizationUnits: boolean;
  }

  export interface ListAppSettingsFilter {
    Name?: string;
    App?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export const enum SettingRemovalReason {
    Untyped = 0,
    DeletedOrganizationUnit = 1,
    ParentHasSameValue = 2,
    ValueIsSameAsDefault = 3,
    ArchivedOrganizationUnit = 4
  }

  export interface SettingModel {
    Reason: SettingRemovalReason;
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    Key: string;
    Value: string;
  }

  export interface AppSettingAutocompleteInfo {
    Name: string;
    Description: string;
    DataType: string;
    Default: string;
    Options: string[];
    Converted?: boolean;
  }

  export const enum SettingSensitivityTypes {
    Normal = 0,
    Sensitive = 1,
    Masked = 2,
    Encrypted = 6,
    CloudOnly = 8
  }

  export const enum SettingValueEntityTypes {
    None = 0,
    OrganizationUnit = 1
  }

  export interface DateTimeOffset {
    DateTime: string;
    UtcDateTime: string;
    LocalDateTime: string;
    Date: string;
    Day: number;
    DayOfWeek: DayOfWeek;
    DayOfYear: number;
    Hour: number;
    Millisecond: number;
    Microsecond: number;
    Nanosecond: number;
    Minute: number;
    Month: number;
    Offset: string;
    TotalOffsetMinutes: number;
    Second: number;
    Ticks: number;
    UtcTicks: number;
    TimeOfDay: string;
    Year: number;
  }

  export interface ListAppSettingsHistoryFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    Value?: string;
  }

  export interface ListSettingsHistoryFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    Value?: string;
  }

  export const enum CompanyType {
    Unknown = 0,
    Consumer = 1,
    OrganizationUnit = 2
  }

  export interface ListCompaniesFilter {
    /**
     * Entity type: Company
     */
    IDs?: number[];
    Name?: string;
    VatNumber?: string;
    Type?: CompanyType;
  }

  export interface ListCompaniesItem {
    /**
     * Entity type: Company
     */
    ID: number;
    BackendID?: string;
    Name: string;
    Type: CompanyType;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
  }

  export interface ListCasesFilter {
    /**
     * Entity type: Case
     */
    ID?: number;
    /**
     * Entity type: CaseTopic
     */
    TopicID?: number;
    Title?: string;
    BackendID?: string;
    /**
     * Entity type: CaseStatus
     */
    StatusID?: number;
    Priority?: CasePriority;
    /**
     * Entity type: User
     */
    CustomerID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    ExternalOrderIdentifier?: string;
    /**
     * Entity type: User
     */
    AssigneeID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OriginatingOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    AssignedOrganizationUnitID?: number;
  }

  export const enum RepairStatus {
    WaitingForGoods = 0,
    WaitingForRepair = 1,
    WaitingForPayment = 2,
    WaitingForShipment = 3,
    Finished = 4,
    Cancelled = 5,
    WaitingForPickup = 6,
    Started = 7,
    WaitingForGoods_OnHold = 8,
    WaitingForRepair_OnHold = 9,
    WaitingForShipment_OnHold = 10
  }

  export const enum CaseStatusAction {
    None = 0,
    CreateRepair = 1,
    CreateOrder = 2
  }

  export interface ListCaseInteractionsFilter {
    InteractionType?: InteractionType;
    Text?: string;
  }

  export interface ListCaseStatusesFilter {
    Name?: string;
    Description?: string;
  }

  export interface ListCaseTopicsFilter {
    /**
     * Entity type: CaseTopic
     */
    ParentID?: number;
    Name?: string;
    Description?: string;
    RootOnly?: boolean;
  }

  export const enum AuditingSettingType {
    String = 0,
    Bool = 1,
    Integer = 2,
    Decimal = 3
  }

  export const enum AMLTriggerType {
    Threshold = 1,
    Scripting = 2,
    Manual = 3
  }

  export const enum AMLSystemSurveys {
    US_IRS8300 = 1
  }

  export interface ListConfigurationsFilter {
    CountryID?: string;
  }

  export const enum BookingFlags {
    None = 0,
    WithTaxInformation = 1,
    WithoutOffsets = 2,
    WithOrderNumber = 4,
    WithReference = 8,
    WithInvoiceNumber = 16,
    WithCurrencyInformation = 32
  }

  export const enum AccountType {
    GeneralLedger = 1,
    Debtor = 2,
    Creditor = 3
  }

  export interface ListAccountsFilter {
    Name: string;
    ObjectAccount: string;
  }

  export interface ListAccountsItem {
    /**
     * Entity type: Account
     */
    ID: number;
    Name: string;
    ObjectAccount: string;
    Subsidiary?: string;
    BackendCode?: string;
    BookingFlagsID: number;
    TypeID: number;
    BookingFlags: BookingFlags;
    Type: AccountType;
  }

  /**
   * Create a return order from an existing sales order (`OrderID`).
   * 
   * When a `TargetOrderID` has been specified, newly created lines are added to that specific order. When omitted, the lines are either added to the
   * existing, original order (in case that is still permitted) or a new order is created automatically.
   * 
   * Returns information on the (newly created?) return order.
   */
  export interface CreateCustomerReturn {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Order
     */
    TargetOrderID?: number;
    /**
     * Lines to return
     */
    Lines: CustomerReturnLine[];
    /**
     * The products will not be returned, so the return order will be shipped immediately and the stock will be corrected. It does not work for return reversals.
     */
    ReturnWithoutProducts?: boolean;
    /**
     * When a ReturnWithoutProducts is done, but not the full amount should be refunded, a different amount can be provided - cannot exceed the worth of the returned products
     */
    AmountToRefund?: number;
    /**
     * When not the whole amount is refunded, a reason for this correction can be set. This will only have an affect if `AmountToRefund` is lower than the total amount of this return.
     * Entity type: RefundCorrectionReason
     */
    RefundCorrectionReasonID?: number;
    /**
     * Attach the order to the same OrganizationUnit as the returned order
     * @default false
     */
    CreateInOriginalOrganizationUnit?: boolean;
    /**
     * When the stock should be returned to a specific OrganizationUnit (see `GetReturnableStatusForOrder.StockOrganizationUnits`)
     * Entity type: OrganizationUnit
     */
    StockOrganizationUnitID?: number;
    /**
     * Optional, attached to a possible newly created (return) order - requires `BackendSystemID`
     */
    BackendID?: string;
    /**
     * Optional, attached to a possible newly created (return) order - requires `BackendID`
     */
    BackendSystemID?: string;
    /**
     * User is allowed to create return order when a valid security code is provided. The user has to have the ReturnWithoutSecurityCode role when omitted. `CRM:Return:RequiresOrderSecurityCode` needs to be enabled.
     */
    SecurityCode?: string;
    /**
     * Determines if the order will be auto-refunded or not. This value, when specified, will take precedence over CRM:Refund:AutoRefundOnReturn.
     */
    AutoRefund?: boolean;
  }

  export interface CustomerReturnLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    Quantity: number;
    /**
     * Entity type: ReturnReason
     */
    ReturnReasonID?: number;
    Remark?: string;
    /**
     * When a ReturnWithoutProducts is done, but not the full amount should be refunded, a different amount can be provided - cannot exceed the worth of the returned products
     */
    UnitAmountToRefund?: number;
    /**
     * When not the whole amount is refunded, a reason for this correction can be set. This will only have an affect if `UnitAmountToRefund` is lower than the total amount of this return line.
     * Entity type: RefundCorrectionReason
     */
    RefundCorrectionReasonID?: number;
    /**
     * Return it to the given stocklabel. This is only available for employees.
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * The custom fields to store on the orderline. The key of the dictionary should be the name of the custom field.
     */
    CustomFields?: { [key: string]: CustomFieldValue };
    SerialNumber?: string;
  }

  /**
   * Creates a return order for a set of products for which there is no original order in EVA.
   */
  export interface CreateCustomerReturnWithoutOrder {
    /**
     * Entity type: Order
     */
    TargetOrderID?: number;
    OriginalOrderReference?: string;
    OriginalOrderTimestamp?: string;
    Lines?: ProductReturnLine[];
    /**
     * Entity type: User
     */
    CustomerID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    SoldFromOrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    ShipToOrganizationUnitID?: number;
  }

  export interface ProductReturnLine {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * A positive number that indicates the quantity of products returned.
     * Min: 0
     */
    Quantity: number;
    /**
     * Entity type: ReturnReason
     */
    ReturnReasonID?: number;
    Remark?: string;
    SerialNumber?: string;
  }

  /**
   * Create a return that is returned in the store. This will handle the return in the original OU and move stock to the current OU. For now, this only supports orders sold from a webshop.
   */
  export interface CreateInStoreReturn {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Lines: CreateInStoreReturn.Line[];
  }

  export namespace CreateInStoreReturn {
    export interface Line {
      /**
       * The order line that should be returned. Must be part of the given order
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      /**
       * Min: 1
       */
      Quantity?: number;
      /**
       * Optional remark
       */
      Remark?: string;
      /**
       * Entity type: ReturnReason
       */
      ReturnReasonID?: number;
      /**
       * The stock label that the product should be returned to on the current store.
       * Entity type: StockLabel
       */
      ReturnStockLabel?: number;
    }

  }

  export interface CreateInStoreReturnResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Return the customer order return details originally set via `CreateCustomerReturn` or `CreateCustomerReturnWithoutOrder
   */
  export interface GetCustomerReturn {
    /**
     * ID of the returned order
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetCustomerReturnResponse {
    /**
     * Entity type: Order
     */
    OriginalOrderID?: number;
    OriginalOrderTimestamp?: string;
    ReferenceNumber: string;
    ReturnWithoutProducts: boolean;
    /**
     * Entity type: OrganizationUnit
     */
    StockOrganizationUnitID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetOrdersForReturnOrder {
    /**
     * Entity type: Order
     */
    ReturnOrderID: number;
    /**
     * Also return the original orders for cancelled return lines.
     */
    IncludeCancelledOrderLines?: boolean;
  }

  export interface GetOrdersForReturnOrderResponse {
    Orders: OriginalOrder[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface OriginalOrder {
    /**
     * Entity type: Order
     */
    ID: number;
    BackendID: string;
    BackendSystemID: string;
    IsPaid: boolean;
    IsCompleted: boolean;
    IsShipped: boolean;
  }

  export interface GetReturnOrdersForOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * If set to true, only return orders that have been placed will be returned. If set to false, only return orders that have not been placed will be returned. If not set, all return orders will be returned.
     */
    IsPlaced?: boolean;
  }

  export interface GetReturnOrdersForOrderResponse {
    Orders: ReturnOrder[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ReturnOrder {
    /**
     * Entity type: Order
     */
    ID: number;
    IsReturnWithoutProducts: boolean;
    CreationTime: string;
    TotalAmountInTax: number;
    CurrencyID: string;
    IsPaid: boolean;
    IsCompleted: boolean;
    IsShipped: boolean;
    PaidAmount: number;
    ForeignPaidAmount: number;
    ReturnedToOrganizationUnits: ReturnOrder.OrganizationUnit[];
  }

  export namespace ReturnOrder {
    export interface OrganizationUnit {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Receive a customer return order
   */
  export interface ReceiveReturnOrder {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Lines?: ReceiveReturnOrder.ReturnedLine[];
    FinalReceipt?: boolean;
  }

  export namespace ReceiveReturnOrder {
    export interface ReturnedLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      Remark?: string;
      /**
       * Min: 1
       */
      Quantity?: number;
    }

  }

  /**
   * Triggers a refund request for the given Order.
   * 
   * The amount to refund is optional, if not specified it will be the amount available to refund on the order.
   */
  export interface RefundOrder {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Reason?: string;
    /**
     * Min: 0
     */
    AmountToRefund?: number;
    /**
     * Handle all the refund in process, this can take a long time if multiple transactions are involved.
     */
    WaitForCompletion?: boolean;
  }

  export interface RefundOrderResponse {
    /**
     * Only set when WaitForCompletion is true
     */
    CreatedTransactions?: PaymentTransactionDto[];
    /**
     * Only set when WaitForCompletion is true
     */
    AmountAvailableToRefund?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Supports the creation of return orders and return reversal orders. Unreferenced reversals will always be treated as normal returns.
   */
  export interface ReturnOrderLines {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    Lines: ReturnLineDto[];
    /**
     * Entity type: Order
     */
    TargetOrderID?: number;
    LineActionType?: LineActionTypes;
    /**
     * Attach the order to the same OrganizationUnit as the returned order
     */
    CreateInOriginalOrganizationUnit?: boolean;
    /**
     * When the stock should be returned to a specific OrganizationUnit.
     * 
     * The available OrganizationUnits are returned from the GetReturnableStatusForOrder.StockOrganizationUnits
     * Entity type: OrganizationUnit
     */
    StockOrganizationUnitID?: number;
  }

  export interface ReturnLineDto {
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    Quantity: number;
    Remark: string;
    /**
     * Entity type: ReturnReason
     */
    ReturnReasonID?: number;
    /**
     * Entity type: StockLabel
     */
    ReturnStockLabel?: number;
    SerialNumber?: string;
  }

  /**
   * Sends a `ReturnMail` stencil to the customer of given return order, attaching order blobs of type `ReturnLabel` (3) and `ReturnForm` (4) if present.
   */
  export interface SendReturnMail {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface SetReturnOrderRefundCorrection {
    /**
     * Entity type: Order
     */
    OrderID?: number;
    NewOpenAmount?: number;
    Reason?: string;
    /**
     * When not the whole amount is refunded, a reason for this correction can be set.
     * Entity type: RefundCorrectionReason
     */
    RefundCorrectionReasonID?: number;
  }

  export interface RequestReindexUserSearchData {
    Script?: string;
    DeleteOldIndex?: boolean;
  }

  export interface RequestReindexOrderSearchData {
    Script?: string;
  }

  export const enum MessageGroups {
    General = 0,
    Events = 1,
    Global = 2
  }

  export const enum ProductCapacityTimeFrame {
    Daily = 0,
    Weekly = 1
  }

  export interface ProductCapacityPlanFilter {
    /**
     * Entity type: ProductCapacityPlan
     */
    ID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    SupplierOrganizationUnitID?: number;
    Name?: string;
    Description?: string;
    ProductCapacityTimeFrame?: ProductCapacityTimeFrame;
    Capacity?: number;
    IsDynamic?: boolean;
  }

  export interface ProductRelationTypesFilter {
    /**
     * Entity type: ProductRelationType
     */
    ID?: number;
    Name?: string;
    BackendID?: string;
    Direction?: ProductRelationDirection;
    StockRelationType?: StockRelationType;
  }

  export const enum ParsingErrorType {
    InvalidSyntax = 0,
    DuplicateVariableDeclaration = 1,
    UnknownVariable = 2,
    InvalidSheetType = 3,
    InvalidScopeType = 4,
    InvalidExpressionValueType = 5,
    InvalidOption = 6
  }

  export const enum NodeTypes {
    Root = 0,
    Comparison = 1,
    Number = 2,
    Calculation = 3,
    And = 4,
    ReadVariable = 5,
    Array = 6,
    String = 7,
    DeclareVariable = 8,
    Switch = 9,
    SwitchCase = 10,
    Boolean = 11,
    Or = 12,
    Require = 13,
    Scope = 14,
    BoostScore = 15,
    NullCheck = 16,
    Null = 17,
    Fanout = 18,
    InterpolatedString = 19,
    Group = 20,
    Block = 22,
    DeclareTable = 23,
    ForEach = 24,
    If = 25,
    Lookup = 26,
    Output = 27,
    SetVariable = 28,
    FunctionCall = 30,
    FunctionCallParameter = 31,
    Exit = 32,
    IfBranch = 33,
    TableRow = 34,
    TableCell = 35,
    ArrayAccessor = 36,
    Identifier = 37,
    MemberAccess = 38,
    LambdaFunction = 39,
    Not = 40,
    ExtensionPoint = 1000,
    Recipe = 2000,
    Book = 2001,
    BookLine = 2002,
    BookParameter = 2003,
    Ignore = 2003
  }

  export interface PrintSecondChanceProductPriceLabel {
    /**
     * Entity type: Product
     */
    ID: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  /**
   * Creates a new interaction associated with a specific repair
   */
  export interface CreateRepairInteraction {
    /**
     * Entity type: Repair
     */
    RepairID: number;
    Type: InteractionType;
    Text?: string;
  }

  export interface CreateRepairInteractionResponse {
    /**
     * Entity type: Interaction
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Create a repair
   */
  export interface CreateRepair {
    /**
     * Will copy as much information as possible from the case.
     * Entity type: Case
     */
    CaseID?: number;
    /**
     * Copy product, serial number and warranty information from the case item.
     * Entity type: CaseRelatedItem
     */
    CaseRelatedItemID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    SerialNumber?: string;
    IsWarranty?: boolean;
    WarrantyExpiration?: string;
    WarrantyAdditionalInformation?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    AssigneeID?: number;
    CustomFields?: CustomFieldKeyValue[];
    /**
     * How to deliver the repair to the store. Immediate means it is already there, Deferred means it will get there later.
     */
    ReceiveMethod?: RepairReceiveMethod;
    /**
     * The date when we expect the goods to arrive. Only used when ReceiveMethod is Deferred.
     */
    ExpectedArrivalDate?: string;
    ShippingOption?: RepairShippingOptions;
    /**
     * Where to print the thermal receipt
     * Entity type: Station
     */
    StationID?: number;
  }

  export const enum RepairReceiveMethod {
    Immediate = 0,
    Deferred = 1
  }

  export const enum RepairShippingOptions {
    Pickup = 0,
    Delivery = 1
  }

  export interface CreateRepairResponse {
    /**
     * Entity type: Repair
     */
    ID: number;
    /**
     * Entity type: Order
     */
    RepairOrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FinishRepair {
    /**
     * Entity type: Repair
     */
    ID: number;
    Result: RepairResult;
    Message?: string;
    Public?: boolean;
  }

  export const enum RepairResult {
    Completed = 0,
    Cancelled = 1
  }

  export interface GetRepair {
    /**
     * Entity type: Repair
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface GetRepairResponse {
    /**
     * Entity type: Repair
     */
    ID: number;
    Blobs: GetRepairResponse.BlobModel[];
    OrganizationUnit: GetRepairResponse.OrganizationUnitModel;
    CompletionTime?: string;
    CancellationTime?: string;
    CompletedBy?: GetRepairResponse.UserModel;
    CancelledBy?: GetRepairResponse.UserModel;
    AssignedTo?: GetRepairResponse.UserModel;
    Customer?: GetRepairResponse.UserModel;
    Product: any;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Entity type: Order
     */
    OriginalOrderID?: number;
    Case: GetRepairResponse.CaseModel;
    /**
     * Entity type: Order
     */
    RepairOrderID: number;
    RepairOrderDisplayID: string;
    /**
     * Entity type: UserTask
     */
    ActiveRepairTaskID?: number;
    ActiveTask: GetRepairResponse.ActiveTaskModel;
    HasOpenAmount: boolean;
    /**
     * Indicates if a repair can be cancelled through `FinishRepair`.
     */
    IsCancellable: boolean;
    /**
     * Indicates if the repair is a warranty repair.
     */
    IsWarranty: boolean;
    /**
     * Warranty information **currently** attached to this product. Might differ from `IsWarranty`.Also, if this is an old repair and the product has been resold (eg: refurbished) this will show the warranty information of the refurbished product,not the warranty at the time of the repair creation.
     */
    WarrantyInformation: GetRepairResponse.WarrantyInfo;
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    SerialNumber?: string;
    ReceiveMethod: RepairReceiveMethod;
    /**
     * Indicates when the repair is expected to arrive at the OrganizationUnit. Not applicable if ReceiveMethod is 'Immediate'.
     */
    ExpectedArrivalDate?: string;
    Status: RepairState;
    StatusHistory: GetRepairResponse.StatusHistoryModel[];
    ShippingOption?: RepairShippingOptions;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetRepairResponse {
    export interface StatusHistoryModel {
      Status: RepairState;
      CreationTime: string;
    }

    export interface WarrantyInfo {
      IsUnderWarranty?: boolean;
      WarrantyExpiryDate?: string;
      AdditionalWarrantyInfo?: string;
    }

    export interface Interaction {
      /**
       * Entity type: Interaction
       */
      ID: number;
      Message: string;
      Type: InteractionType;
      UserModel: GetRepairResponse.UserModel;
      Timestamp: string;
    }

    export interface UserModel {
      /**
       * Entity type: User
       */
      ID: number;
      BackendID: string;
      FullName: string;
      EmailAddress: string;
      PhoneNumber: string;
      BackendSystemID: string;
    }

    export interface BlobModel {
      /**
       * Entity type: Blob
       */
      ID: string;
      Name: string;
      Type: string;
      CreationTime: string;
    }

    export interface OrganizationUnitModel {
      /**
       * Entity type: OrganizationUnit
       */
      ID: number;
      Name: string;
    }

    export interface CaseModel {
      /**
       * Entity type: Case
       */
      ID: number;
      Title: string;
      Description: string;
      Summary: string;
      /**
       * Entity type: CaseTopic
       */
      TopicID?: number;
      TopicName?: string;
      CreationTime: string;
    }

    export interface ActiveTaskModel {
      /**
       * Entity type: UserTask
       */
      TaskID?: number;
      /**
       * Subtype of the task.
       */
      SubType?: EnumDto;
    }

  }

  export const enum RepairState {
    WaitingForGoods = 0,
    WaitingForRepair = 1,
    WaitingForPayment = 2,
    WaitingForShipment = 3,
    Finished = 4,
    Cancelled = 5,
    WaitingForPickup = 6,
    Started = 7,
    OnHold = 8
  }

  /**
   * List the repair interactions
   */
  export interface ListRepairInteractions {
    /**
     * Entity type: Repair
     */
    RepairID: number;
    IncludeCaseInteractions?: boolean;
    InitialPageConfig?: PageTokenConfig<ListRepairInteractionsFilter>;
    PageToken?: string;
  }

  export interface ListRepairInteractionsFilter {
    InteractionType?: InteractionType;
    Text?: string;
  }

  export interface ListRepairInteractionsResponse {
    Results?: ListRepairInteractionsResponse.RepairInteraction[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListRepairInteractionsResponse {
    export interface RepairInteraction {
      /**
       * Entity type: Interaction
       */
      ID: number;
      IsCaseInteraction: boolean;
      InteractionType: InteractionType;
      CanBeMutated: boolean;
      Text: string;
      CreationTime: string;
      LastModificationTime?: string;
      CreatedBy?: ListRepairInteractionsResponse.User;
    }

    export interface User {
      /**
       * Entity type: User
       */
      CreatedByID?: number;
      FirstName: string;
      LastName: string;
      FullName: string;
    }

  }

  /**
   * List Repairs, optionally filtered.
   */
  export interface ListRepairs {
    InitialPageConfig?: PageTokenConfig<ListRepairsFilter>;
    PageToken?: string;
  }

  export interface ListRepairsFilter {
    /**
     * Entity type: Repair
     */
    RepairID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    FromDate?: string;
    ToDate?: string;
    Status?: RepairStatus[];
    SerialNumber?: string;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: User
     */
    AssignedUserIDs?: number[];
    /**
     * Entity type: User
     */
    CustomerIDs?: number[];
  }

  export interface ListRepairsResponse {
    Results?: ListRepairsResponse.ListRepairsItem[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListRepairsResponse {
    export interface ListRepairsItem {
      /**
       * Entity type: Repair
       */
      ID: number;
      CreationTime: string;
      Case: ListRepairsItem.CaseModel;
      Product: any;
      OrganizationUnit: ListRepairsItem.OrganizationUnitModel;
      RepairTasks: ListRepairsItem.RepairTaskModel[];
      ReceiveMethod: RepairReceiveMethod;
      Status: RepairState;
      /**
       * Whether the repair can be cancelled.
       */
      IsCancellable: boolean;
      /**
       * Entity type: Order
       */
      RepairOrderID: number;
      RepairOrderDisplayID?: string;
    }

    export namespace ListRepairsItem {
      export interface StatusHistoryModel {
        Status: RepairState;
        CreationTime: string;
      }

      export interface OrganizationUnitModel {
        /**
         * Entity type: OrganizationUnit
         */
        ID: number;
        Name: string;
      }

      export interface CaseModel {
        /**
         * Entity type: Case
         */
        ID: number;
        Title: string;
        Description: string;
      }

      export interface RepairTaskModel {
        /**
         * Entity type: UserTask
         */
        TaskID?: number;
        IsIgnored?: boolean;
        /**
         * Whether the task is completed.
         */
        IsCompleted?: boolean;
        /**
         * Whether the task is active. This means it can be picked up or is being worked on.
         */
        IsActive?: boolean;
        /**
         * The user assigned to the task.
         */
        Assignee?: ListRepairsItem.RepairTaskAssignee;
        /**
         * Subtype of the task.
         */
        SubType?: EnumDto;
      }

      export interface RepairTaskAssignee {
        /**
         * Entity type: User
         */
        ID: number;
        Name: string;
      }

    }

  }

  export interface UpdateRepair {
    /**
     * Entity type: Repair
     */
    ID: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    OnHold?: boolean;
    /**
     * Entity type: User
     */
    AssigneeID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    CaseSummary?: string;
    CaseDescription?: string;
    SerialNumber?: string;
    CustomFields?: CustomFieldKeyValue[];
    IsWarranty?: boolean;
    WarrantyExpiration?: string;
    WarrantyAdditionalInformation?: string;
    ExpectedArrivalDate?: string;
    ShippingOption?: RepairShippingOptions;
    ReceiveMethod?: RepairReceiveMethod;
  }

  /**
   * Completes a Stock Movement Follow Up task, using the `WorkSet` returned by the `StartStockMovementFollowUp` service.
   */
  export interface CompleteStockMovementFollowUp {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * The quantity moved (optional, use in case of a deviation - defaults to the full original amount)
     */
    Quantity?: number;
    SerialNumber?: string;
  }

  export interface StockMovementFollowUpWorkSet {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: UserTask
     */
    StockMovementFollowUpTaskID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Product: any;
    Quantity: number;
    FromStockLabel: StockLabelDto;
    ToStockLabel: StockLabelDto;
  }

  export interface StockLabelDto {
    /**
     * Entity type: StockLabel
     */
    ID: number;
    Name: string;
    Description: string;
  }

  /**
   * Starts a Stock Movement Follow Up task. Proceed with the `CompleteStockMovementFollowUp` service when finished.
   */
  export interface StartStockMovementFollowUp {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartStockMovementFollowUpResponse {
    WorkSet: StockMovementFollowUpWorkSet;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Starts a Composite User Task, returning child meta data.
   */
  export interface StartCompositeUserTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartCompositeUserTaskResponse {
    /**
     * Entity type: UserTask
     */
    ID: number;
    Description?: string;
    Tasks: StartCompositeUserTaskResponse.CompositeUserTaskSubTask[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace StartCompositeUserTaskResponse {
    export interface CompositeUserTaskSubTask {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Type: UserTaskTypeDto;
      SubType?: EnumDto;
      Description?: string;
      Sequence: number;
      IsActive: boolean;
      IsCompleted: boolean;
    }

  }

  export interface UserTaskTypeDto {
    ExpectedTimeToComplete?: string;
    DefaultPriority: number;
    DefaultRequired: boolean;
    SubTypes: EnumDto[];
    /**
     * Entity type: Enum
     */
    ID: number;
    /**
     * Usually an internal & unchangable code - for display, use `Description`
     */
    Name: string;
    Description: string;
  }

  /**
   * Complete multiple print SFS tasks.
   */
  export interface CompleteMultipleShipFromStoreTasks {
    /**
     * Entity type: UserTask
     */
    UserTaskIDs: number[];
  }

  /**
   * Completes the ShipFromOrder task, indicating that the shipment of the order has taken place.
   * 
   * The order will be shipped and the shop stock will be moved to Transit.
   */
  export interface CompleteShipFromStoreTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Required for subtypes Pick and Ship
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Optional for subtype Pick
     */
    PickedOrderLines?: CompleteShipFromStoreTask.PickedOrderLine[];
    /**
     * Optional for subtype Ship
     */
    Shipments?: CompleteShipFromStoreTask.Shipment[];
    /**
     * Optional for subtype Ship, requesting to cancel all that was not shipped (also see `UnpickedLinesReasonID`)
     */
    FinalShipment?: boolean;
    /**
     * The reason why not all lines might be picked
     * Entity type: UserTaskNotCompletedReason
     */
    UnpickedLinesReasonID?: number;
    /**
     * If you want to override printing behavior - will default to the `ShouldBePrinting` value from `StartShipFromStoreTask`
     */
    ShouldBePrinting?: boolean;
  }

  export namespace CompleteShipFromStoreTask {
    export interface Shipment {
      /**
       * Entity type: ShippingMethod
       */
      ShippingMethodID?: number;
      Lines?: CompleteShipFromStoreTask.ShipmentLine[];
      Packages?: CompleteShipFromStoreTask.ShipmentPackage[];
    }

    export interface ShipmentLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      Quantity: number;
    }

    export interface ShipmentPackage {
      /**
       * Entity type: Package
       */
      PackageID: number;
      Lines?: CompleteShipFromStoreTask.ShipmentLine[];
    }

    export interface PickedOrderLine {
      /**
       * Entity type: OrderLine
       */
      ID?: number;
      /**
       * Assuming original `ProductID` when omitted - useful when using a replacement product
       * Entity type: Product
       */
      ProductID?: number;
      /**
       * Assuming full quantity when omitted - useful for partial picks or replacement products
       */
      Quantity?: number;
      SerialNumber?: string;
    }

  }

  export interface CompleteShipFromStoreTaskResponse {
    Next: NextUserTaskDto[];
    ShipmentIDs: number[];
    CompletionData: any;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface NextUserTaskDto {
    /**
     * Entity type: UserTask
     */
    ID: number;
    /**
     * Entity type: UserTask
     */
    ParentID?: number;
    Description: string;
    Priority: number;
    Type: UserTaskTypeDto;
    SubType: EnumDto;
  }

  /**
   * Retrieves a pre-picked ShipFromStoreTask based on a product ID.
   * Optionally filters by a ConsolidatedPickTask ID or defaults to the latest done, stopped, or paused task.
   */
  export interface GetPrePickedShipFromStoreTask {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
  }

  export interface GetPrePickedShipFromStoreTaskResponse {
    /**
     * The SFS task that was chosen as the most optimal based on the determination algorithm configured
     * Entity type: UserTask
     */
    UserTaskID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets an active ShipFromStore type User Task for your store, that contains a product with specified `Barcode`.
   * 
   * Use the optional `DeterminationType` to indicate if you prefer the `Oldest`, `Smallest` or `Largest` task that contains your product.
   */
  export interface GetShipFromStoreTaskByBarcode {
    Barcode: string;
    DeterminationType?: DeterminationType;
    Filter?: GetShipFromStoreTaskByBarcodeFilter;
  }

  export const enum DeterminationType {
    Oldest = 0,
    Smallest = 1,
    Largest = 2
  }

  export interface GetShipFromStoreTaskByBarcodeFilter {
    /**
     * Filter the tasks on the product count before choosing which one to return.
     */
    ProductCount?: ProductCountFilter;
  }

  export const enum ProductCountFilter {
    SingleProduct = 1,
    MultiProduct = 2,
    All = 3
  }

  export interface GetShipFromStoreTaskByBarcodeResponse {
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    /**
     * Entity type: UserTask
     */
    ParentID?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    UserFullName?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Generates a combined pick list for given Ship From Store tasks, starting and assigning them all to the current user.
   * 
   * These tasks need to be ShipFromStore type with subtype Pick, or a CompositeUserTask type with such a task as first child.
   */
  export interface PrintShipFromStoreTasks {
    /**
     * Entity type: UserTask
     */
    UserTaskIDs: number[];
    ProductProperties?: string[];
  }

  /**
   * Starts a ShipFromStore task, identified by its UserTaskID.
   * 
   * The service returns the order that should be shipped from the user's current store as well as the OrderLines
   * that should be part of the shipment. Not necessarily all orderlines of the order should be shipped, so the
   * OrderLineIDs returned by this service should be respected.
   */
  export interface StartShipFromStoreTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    ProductProperties?: string[];
  }

  export interface StartShipFromStoreTaskResponse {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    OrderDisplayID?: string;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    AvailableShippingMethods: ShippingMethodByOrderLineDto[];
    TravelDistanceText: string;
    TravelTimeText: string;
    TravelDistanceInMeters?: number;
    TravelTimeInSeconds?: number;
    ShouldBePrinting: boolean;
    SubTypeName: string;
    /**
     * All lines in the order fulfillment.
     */
    Lines: StartShipFromStoreTaskResponse.ShipFromStoreLine[];
    PreferredPriceDisplayMode: OrderPreferredPriceDisplayMode;
    GiftWrappingOptions: StartShipFromStoreTaskResponse.ShipFromStoreGiftWrappingOptions;
    Data: { [key: string]: any };
    /**
     * The lines that need to be packed. Only available for the Pack subtype.
     */
    PackLines?: StartShipFromStoreTaskResponse.ShipFromStoreLine[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace StartShipFromStoreTaskResponse {
    export interface ShipFromStoreLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      /**
       * Entity type: Product
       */
      ProductID?: number;
      /**
       * Entity type: Product
       */
      ReplacementProductIDs?: number[];
      Quantity: number;
      Properties: any;
      /**
       * The product requirements for this line
       */
      Requirements: StartShipFromStoreTaskResponse.ShipFromStoreLineProductRequirement[];
      /**
       * The TotalAmount of this line: Quantity * UnitPrice
       */
      TotalAmount: number;
      /**
       * The TotalAmountInTax of this line: Quantity * UnitPrice + Taxes
       */
      TotalAmountInTax: number;
      /**
       * The TotalAmount with the Discounts for this line included
       */
      NetTotalAmount: number;
      /**
       * The TotalAmountInTax with the Discounts for this line included
       */
      NetTotalAmountInTax: number;
      /**
       * The child lines for this line
       */
      ChildLines: StartShipFromStoreTaskResponse.ShipFromStoreLine[];
      /**
       * Entity type: OrderLine
       */
      ParentID?: number;
      GroupID?: string;
      /**
       * Entity type: ProductBundleLine
       */
      ProductBundleLineID?: number;
      CurrencyID: string;
      ProductVariation: { [key: string]: string };
      SerialNumber?: string;
      SerialNumberRequired: boolean;
      /**
       * Entity type: SerialNumberConfiguration
       */
      SerialNumberConfigurationID?: number;
    }

    export interface ShipFromStoreLineProductRequirement {
      Name: string;
      Value?: any;
    }

    export interface ShipFromStoreGiftWrappingOptions {
      GiftWraps: StartShipFromStoreTaskResponse.ShipFromStoreGiftWrap[];
      GreetingCardProductID?: number;
      Message: string;
    }

    export interface ShipFromStoreGiftWrap {
      GreetingCardProductID?: number;
      Message: string;
      Lines: StartShipFromStoreTaskResponse.ShipFromStoreWrapLine[];
    }

    export interface ShipFromStoreWrapLine {
      Line: StartShipFromStoreTaskResponse.ShipFromStoreLine;
      Quantity: number;
    }

  }

  /**
   * Add a tracking code to the ReturnToSupplier task shipment.
   * If tracking code is already present, it will be overwritten.
   */
  export interface AddTrackingCodeToReturnToSupplierShipment {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    TrackingCode?: string;
  }

  /**
   * Completes the ReturnToSupplier task.
   * 
   * When CompleteReturnToSupplierTask is called and there are no lines on the task shipment, the shipment will be removed.
   */
  export interface CompleteReturnToSupplierTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  /**
   * List available ReturnToSupplier order lines with quantities.
   */
  export interface ListReturnToSupplierTaskOrderLines {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    PageConfig?: PageConfig<ListReturnToSupplierTaskOrderLineFilter>;
    ProductProperties?: string[];
  }

  export interface ListReturnToSupplierTaskOrderLineFilter {
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
    ProductTypes?: ProductTypes;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    Query: string;
  }

  export interface ListReturnToSupplierTaskOrderLinesResponse {
    Result?: PagedResult<ReturnToSupplierTaskOrderLine>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ReturnToSupplierTaskOrderLine {
    OrderLineID: number;
    ParentID?: number;
    GroupID?: string;
    ProductID: number;
    CustomID: string;
    ProductName: string;
    /**
     * Entity type: StockLabel
     */
    StockLabel: number;
    ProductProperties: any;
    Quantity: number;
    SerialNumber?: string;
    SerialNumberRequired: boolean;
    SerialNumberConfigurationID?: number;
    UnitOfMeasures: ReturnToSupplierTaskOrderLine.UnitOfMeasure[];
    OrderDisplayID?: string;
  }

  export namespace ReturnToSupplierTaskOrderLine {
    export interface UnitOfMeasure {
      BackendID: string;
      Name: string;
      Description: string;
      UnitOfMeasureQuantity: number;
      Quantity: number;
      BaseQuantityTotal: number;
    }

  }

  /**
   * List lines for the tasks shipment with quantities.
   */
  export interface ListReturnToSupplierTaskShipmentLines {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    PageConfig?: PageConfig<ListReturnToSupplierTaskOrderLineFilter>;
    ProductProperties?: string[];
  }

  export interface ListReturnToSupplierTaskShipmentLinesResponse {
    Result?: PagedResult<ReturnToSupplierTaskShipmentLine>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ReturnToSupplierTaskShipmentLine {
    OrderLineID: number;
    ParentID?: number;
    GroupID?: string;
    ProductID: number;
    CustomID: string;
    ProductName: string;
    /**
     * Entity type: StockLabel
     */
    StockLabel: number;
    ProductProperties: any;
    Quantity: number;
    UnitOfMeasures: ReturnToSupplierTaskShipmentLine.UnitOfMeasure[];
    SerialNumber?: string;
    SerialNumberRequired: boolean;
    /**
     * Entity type: SerialNumberConfiguration
     */
    SerialNumberConfigurationID?: number;
  }

  export namespace ReturnToSupplierTaskShipmentLine {
    export interface UnitOfMeasure {
      BackendID: string;
      Name: string;
      Description: string;
      UnitOfMeasureQuantity: number;
      Quantity: number;
      BaseQuantityTotal: number;
    }

  }

  /**
   * Print the tasks shipment label.
   */
  export interface PrintReturnToSupplierTaskShipment {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  /**
   * Adds or removes a lines to/from the ReturnToSupplier tasks shipment.
   * 
   * When 'Exclude' is enabled, all lines matching the given ProductID will be removed from the task shipment with a maximum of the given Quantity.
   * When 'Exclude' is disabled, all lines from the ReturnToSupplier order matching the given ProductID will be added to the tasks shipment to the maximum of the given Quantity.
   * 
   * Overshipment is not allowed.
   */
  export interface SetLineForReturnToSupplierShipment {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    Exclude?: boolean;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID?: number;
    /**
     * Min: 0
     */
    Quantity: number;
    SerialNumbers?: string[];
  }

  /**
   * Starts ReturnToSupplier task.
   */
  export interface StartReturnToSupplierTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartReturnToSupplierResponse {
    UseExcludePolicyOnReturnToSupplierShipment: boolean;
    ShipmentIdentifiers: StartReturnToSupplierResponse.ShipmentIdentifier[];
    Lines: StartReturnToSupplierResponse.StartReturnToSupplierLineResponse[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace StartReturnToSupplierResponse {
    export interface ShipmentIdentifier {
      /**
       * Entity type: ShipmentIdentifier
       */
      ID: number;
      Name: string;
      RequiredFor: ShipmentIdentifierRequiredFor;
      Type: ShipmentIdentifierTypes;
    }

    export interface StartReturnToSupplierLineResponse {
      QuantityToAdd: number;
      /**
       * Entity type: OrderLine
       */
      OrderLineID: number;
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
      StockLabelName: string;
      SerialNumber?: string;
      SerialNumberRequired: boolean;
      /**
       * Entity type: SerialNumberConfiguration
       */
      SerialNumberConfigurationID?: number;
    }

  }

  /**
   * Complete a ReservationPickupTask task
   */
  export interface CompleteReservationPickupTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartReservationPickupTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartReservationPickupTaskResponse {
    /**
     * Entity type: Order
     */
    OrderID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates the deadline of a pending ReservationPickup task.
   */
  export interface UpdateReservationPickupTaskDeadline {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    Deadline: string;
  }

  export interface DetachFromRepairUserTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  /**
   * Finish repair user task
   */
  export interface FinishRepairUserTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    /**
     * Message associated with the completion of this task. This is only useful for the ExecuteRepair task type.
     */
    Message?: string;
    /**
     * Whether the message should be public (same disclaimer as Message applies).
     */
    Public?: boolean;
    Result?: RepairResult;
  }

  /**
   * Print repair thermal receipt
   */
  export interface PrintRepairThermalReceipt {
    /**
     * Entity type: Repair
     */
    RepairID?: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  /**
   * Start repair user task
   */
  export interface StartRepairUserTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartRepairUserTaskResponse {
    Repair: StartRepairUserTaskResponse.RepairModel;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace StartRepairUserTaskResponse {
    export interface RepairModel {
      /**
       * Entity type: Repair
       */
      ID: number;
    }

  }

  /**
   * Creates a new ProductZoneGroup.
   */
  export interface CreateProductZoneGroup {
    Name: string;
    Description?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: ProductZone
     */
    ZoneIDs?: number[];
  }

  export interface CreateProductZoneGroupResponse {
    /**
     * Entity type: ProductZoneGroup
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates an existing ProductZoneGroup.
   */
  export interface UpdateProductZoneGroup {
    /**
     * Entity type: ProductZoneGroup
     */
    ID: number;
    Name?: string;
    Description?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: ProductZone
     */
    ZoneIDs?: number[];
  }

  /**
   * Deletes a ProductZoneGroup by its ID
   */
  export interface DeleteProductZoneGroup {
    /**
     * Entity type: ProductZoneGroup
     */
    ID: number;
  }

  /**
   * Retrieves a ProductZoneGroup by its ID
   */
  export interface GetProductZoneGroup {
    /**
     * Entity type: ProductZoneGroup
     */
    ID: number;
  }

  export interface ProductZoneGroupDto {
    /**
     * Entity type: ProductZoneGroup
     */
    ID: number;
    Name: string;
    Description?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    Zones?: ProductZoneDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ProductZoneDto {
    /**
     * Entity type: ProductZone
     */
    ID?: number;
    Name?: string;
    Description?: string;
    BackendID?: string;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    FilterAttribute?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Gets all the ProductZoneGroups based on the given filter and returns a paged response.
   */
  export interface ListProductZoneGroups {
    PageConfig?: PageConfig<ListProductZoneGroupsFilter>;
  }

  export interface ListProductZoneGroupsFilter {
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: ProductZone
     */
    ProductZoneID?: number;
    /**
     * Entity type: ProductZoneGroup
     */
    ProductZoneGroupIDs: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetIDs?: number[];
  }

  export interface ListProductZoneGroupsResponse {
    Result?: PagedResult<ProductZoneGroupDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates a new ProductZone.
   */
  export interface CreateProductZone {
    Name?: string;
    Description?: string;
    BackendID?: string;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    FilterAttribute?: string;
  }

  export interface CreateProductZoneResponse {
    /**
     * Entity type: ProductZone
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates an existing ProductZone.
   */
  export interface UpdateProductZone {
    /**
     * Entity type: ProductZone
     */
    ID: number;
    Name?: string;
    Description?: string;
    BackendID?: string;
    /**
     * Entity type: ProductSearchTemplate
     */
    ProductSearchTemplateID?: number;
    FilterAttribute?: string;
  }

  /**
   * Deletes a ProductZone by its ID
   */
  export interface DeleteProductZone {
    /**
     * Entity type: ProductZone
     */
    ID: number;
  }

  /**
   * Retrieves a ProductZone by its ID
   */
  export interface GetProductZone {
    /**
     * Entity type: ProductZone
     */
    ID: number;
  }

  /**
   * Gets all the ProductZones based on the given filter and returns a paged response.
   */
  export interface ListProductZones {
    PageConfig?: PageConfig<ListProductZonesFilter>;
  }

  export interface ListProductZonesFilter {
    Name: string;
    FilterAttribute: string;
    /**
     * Entity type: ProductZoneGroup
     */
    ProductZoneGroupID?: number;
  }

  export interface ListProductZonesResponse {
    Result?: PagedResult<ProductZoneDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CompletePrintPriceLabelTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  /**
   * Creates a new Print Price Label task with specified products, or append these to existing tasks when applicable.
   */
  export interface CreatePrintPriceLabelTask {
    /**
     * OrganizationUnitID (defaults to current OU)
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * List of products to print price labels for
     */
    Products: CreatePrintPriceLabelTask.PrintPriceLabelTaskProduct[];
  }

  export namespace CreatePrintPriceLabelTask {
    export interface PrintPriceLabelTaskProduct {
      /**
       * Product ID
       * Entity type: Product
       */
      ProductID: number;
      /**
       * Quantity (defaults to 1)
       */
      Quantity?: number;
    }

  }

  export interface CreatePrintPriceLabelTaskResponse {
    Result: CreatedPrintPriceLabelTask[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CreatedPrintPriceLabelTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    AlreadyExisted: boolean;
  }

  export interface PrintPriceLabel {
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    Products?: PrintPriceLabel.ProductPrintRequest[];
    /**
     * Entity type: Station
     */
    StationID: number;
  }

  export namespace PrintPriceLabel {
    export interface ProductPrintRequest {
      /**
       * Entity type: Product
       */
      ProductID: number;
      Quantity?: number;
    }

  }

  export interface PrintPriceLabelResponse {
    Success: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StartPrintPriceLabelTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartPrintPriceLabelTaskResponse {
    Result: StartPrintPriceLabelTaskResponse.PriceLabelProduct[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace StartPrintPriceLabelTaskResponse {
    export interface PriceLabelProduct {
      ProductID: number;
      Quantity: number;
      ProductCustomID: string;
      ProductBackendID: string;
      ProductDisplayValue: string;
      ProductUnitPriceInTax: number;
      ProductUnitPrice: number;
      ProductOriginalUnitPriceInTax?: number;
      ProductOriginalUnitPrice?: number;
    }

  }

  export interface CompletePrintTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Pass as `true` to skip "NotYetPrinted" check and force task completion without it.
     */
    ManuallyPrinted?: boolean;
  }

  /**
   * Create a UserTask in specified organization unit to print a specific `BlobID` or `BlobUrl`.
   */
  export interface CreatePrintTask {
    /**
     * Can be used for standard blobs - for secure blobs, use `BlobUrl` instead.
     * Entity type: Blob
     */
    BlobID?: string;
    /**
     * Can be used for both standard- and secured blobs - must match environment assets URL.
     */
    BlobUrl?: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Optional description for the generated UserTask
     */
    Description?: string;
    /**
     * Assign the task to a specific user immediately
     * Entity type: User
     */
    UserID?: number;
    Type: PrintTaskType;
  }

  export const enum PrintTaskType {
    Paper = 0,
    Zebra = 1
  }

  export interface CreatePrintTaskResponse {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Processes the print tasks for each required ID.
   * 
   * This service does not throws an expeption in the case of mulptiple user task ids, but the output will be a dictionary containing
   * an Error (EVA.Framework.Error) for every print task that encounters an issue. The UserTaskID will be the key in that specific Dictionary.
   * 
   * In the case of a single user task id, the service will behave exactly as the depracated ProcessPrintTask service did, throwing the encountered exception.
   */
  export interface ProcessPrintTasks {
    /**
     * Entity type: UserTask
     */
    UserTaskIDs: number[];
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  export interface ProcessPrintTasksResponse {
    /**
     * Entity type: UserTask
     */
    FailedUserTasks: { [key: number]: Error };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StartPrintTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartPrintTaskResponse {
    /**
     * Entity type: Blob
     */
    BlobID: string;
    BlobUrl: string;
    Type: PrintTaskType;
    IsPrinted: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Completes a custom usertask
   */
  export interface CompleteCustomUserTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: CustomField
     */
    Values?: { [key: number]: CustomFieldValue };
  }

  /**
   * Creates a new custom usertask
   */
  export interface CreateCustomUserTask {
    /**
     * The organization unit to create this task for
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * The type of the custom usertask
     * Entity type: UserTaskSubType
     */
    CustomUserTaskTypeID?: number;
    /**
     * The template to create the usertask from
     * Entity type: CustomUserTaskTemplate
     */
    CustomUserTaskTemplateID?: number;
    /**
     * Custom fields that will override the ones from the template
     */
    Fields?: UserTaskField[];
    Deadline?: string;
    Description?: string;
    Priority?: number;
  }

  export interface UserTaskField {
    /**
     * Entity type: CustomField
     */
    CustomFieldID: number;
    /**
     * If the field is required, a value MUST be set when completing the task.
     */
    IsRequired: boolean;
  }

  export interface CreateCustomUserTaskResponse {
    /**
     * Entity type: UserTask
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates a new custom usertask type.
   */
  export interface CreateCustomUserTaskType {
    Name?: string;
  }

  export interface CreateCustomUserTaskTypeResponse {
    /**
     * Entity type: UserTaskSubType
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List all custom usertask types
   */
  export interface ListCustomUserTaskTypes {
  }

  export interface ListCustomUserTaskTypesResponse {
    Types: ListCustomUserTaskTypesResponse.CustomUserTaskType[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListCustomUserTaskTypesResponse {
    export interface CustomUserTaskType {
      /**
       * Entity type: UserTaskSubType
       */
      ID: number;
      Name: string;
    }

  }

  /**
   * Creates a new template for custom user tasks
   */
  export interface CreateCustomUserTaskTemplate {
    Name?: string;
    Description?: string;
    /**
     * Entity type: UserTaskSubType
     */
    TypeID: number;
    Options?: TaskOptions;
  }

  export interface TaskOptions {
    DefaultPriority: number;
    DefaultDeadlineMinutes?: number;
    Fields: UserTaskField[];
  }

  export interface CreateCustomUserTaskTemplateResponse {
    /**
     * Entity type: CustomUserTaskTemplate
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Deletes a specific custom usertask template
   */
  export interface DeleteCustomUserTaskTemplate {
    /**
     * Entity type: CustomUserTaskTemplate
     */
    ID: number;
  }

  /**
   * Updates a template for custom user tasks. If a property is not provided, it will be set to null.
   */
  export interface UpdateCustomUserTaskTemplate {
    /**
     * Entity type: CustomUserTaskTemplate
     */
    ID: number;
    Name?: string;
    Description?: string;
    /**
     * Entity type: UserTaskSubType
     */
    TypeID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Options?: TaskOptions;
  }

  /**
   * Lists all of the custom usertask templates
   */
  export interface ListCustomUserTaskTemplate {
    InitialPageConfig?: PageTokenConfig<ListCustomUserTaskTemplateFilter>;
    PageToken?: string;
  }

  export interface ListCustomUserTaskTemplateFilter {
  }

  export interface ListCustomUserTaskTemplateResponse {
    Results?: ListCustomUserTaskTemplateResponse.CustomUserTaskTemplate[];
    PreviousPageToken?: string;
    NextPageToken?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListCustomUserTaskTemplateResponse {
    export interface CustomUserTaskTemplate {
      /**
       * Entity type: CustomUserTaskTemplate
       */
      ID: number;
      Name: string;
      Description: string;
      /**
       * Entity type: UserTaskSubType
       */
      TypeID: number;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID?: number;
      Options: TaskOptions;
    }

  }

  /**
   * Retrieve the set values
   */
  export interface GetCustomUserTaskData {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface GetCustomUserTaskDataResponse {
    /**
     * Entity type: CustomField
     */
    CustomFieldValuesWithOptions?: { [key: number]: CustomFieldValueWithOptions };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Starts a new custom usertask
   */
  export interface StartCustomUserTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartCustomUserTaskResponse {
    CustomFields: StartCustomUserTaskResponse.CustomFieldInfo[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace StartCustomUserTaskResponse {
    export interface CustomFieldInfo {
      CustomFieldResponse: CustomFieldResponse;
      InputHint: string;
      IsRequired: boolean;
    }

  }

  /**
   * Completes a ConsolidatedPickTask task.
   */
  export interface CompleteConsolidatedPickTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  /**
   * Completes a ConsolidatedZonePickTask
   */
  export interface CompleteConsolidatedZonePickTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    Products: CompleteConsolidatedZonePickTaskProduct[];
  }

  export interface CompleteConsolidatedZonePickTaskProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Quantity: number;
  }

  /**
   * Returns a detailed view of a ConsolidatedPickTask by the ID of its UserTask.
   */
  export interface GetConsolidatedPickTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface GetConsolidatedPickTaskResponse {
    /**
     * Entity type: ConsolidatedPickTask
     */
    ID: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    Status: ConsolidatedPickTaskStatus;
    ZonePickTasks: GetConsolidatedPickTaskZonePickTasks[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum ConsolidatedPickTaskStatus {
    New = 0,
    InProgress = 1,
    Done = 2,
    Stopped = 3,
    Paused = 4
  }

  export interface GetConsolidatedPickTaskZonePickTasks {
    /**
     * Entity type: ConsolidatedZonePickTask
     */
    ID?: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    IsCompleted?: boolean;
    IsActive?: boolean;
    /**
     * Entity type: User
     */
    UserID?: number;
    UserName?: string;
    CreationTime?: string;
    StartTime?: string;
    CompletionTime?: string;
    ProductZone?: GetConsolidatedPickTaskZone;
    TotalProducts?: number;
    TotalQuantity?: number;
    TotalPickedQuantity?: number;
  }

  export interface GetConsolidatedPickTaskZone {
    /**
     * Entity type: ProductZone
     */
    ID: number;
    Name: string;
    FilterAttribute?: string;
  }

  /**
   * Pauses a ConsolidatedPickTask task.
   */
  export interface PauseConsolidatedPickTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  /**
   * Starts a ConsolidatedZonePickTask
   */
  export interface StartConsolidatedZonePickTask {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface StartConsolidatedZonePickTaskResponse {
    /**
     * Entity type: ConsolidatedZonePickTask
     */
    ID: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    Products: StartConsolidatedZonePickTaskProduct[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StartConsolidatedZonePickTaskProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Product: any;
    Quantity: number;
    PickedQuantity: number;
  }

  export interface DeactivateUserTask {
    /**
     * Entity type: UserTask
     */
    UserTaskIDs: number[];
    /**
     * A reason, if any, why the tasks were disabled. Optional.
     * Entity type: UserTaskNotCompletedReason
     */
    ReasonID?: number;
  }

  /**
   * Deactivates all active user tasks based on organization unit. Not supported on production environments!
   */
  export interface DeactivateAllUserTasks {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  /**
   * Return the total number of open tasks, and the count per task type.
   */
  export interface GetUserTaskCounts {
    /**
     * Defaults to the current organization when omitted - includes child organizations
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * When specified, will include all assigned tasks in the counts (regardless of whom it is assigned to)
     */
    IncludeAssignedTasks?: boolean;
    /**
     * When specified, will include all tasks, regardless if the user can execute them
     */
    IncludeAllTasks?: boolean;
    /**
     * When specified, will include only tasks from the specified organization unit, excluding child organizations
     */
    ExcludeChildOrganizations?: boolean;
    /**
     * Optionally (and additionally) filter on specified types
     */
    UserTaskTypes?: string[];
  }

  export interface GetUserTaskCountsResponse {
    NumberOfTasks: number;
    UrgentTasksCount: number;
    MyTasksCount: number;
    RepairTasksCount: number;
    DetailedNumberOfTasks: { [key: string]: GetUserTaskCountsResponse.UserTaskCount };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetUserTaskCountsResponse {
    export interface UserTaskCount {
      Count: number;
      SubTypeCounts: { [key: string]: number };
    }

  }

  export interface GetUserTaskDetails {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface GetUserTaskDetailsResponse {
    Details: UserTaskDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UserTaskDto {
    /**
     * Entity type: UserTask
     */
    ID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    User: UserDto;
    UserFullname: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnit: OrganizationUnitDto;
    /**
     * Entity type: FinancialPeriod
     */
    FinancialPeriodID?: number;
    FinancialPeriod: FinancialPeriodDto;
    Type: UserTaskTypeDto;
    Priority: number;
    IsCompleted: boolean;
    IsActive: boolean;
    CreationTime: string;
    Deadline?: string;
    ExpectedTimeToComplete?: string;
    ExpectedStartTime?: string;
    StartTime?: string;
    ExpectedCompletionTime?: string;
    CompletionTime?: string;
    HasUrgency: boolean;
    Description: string;
    Results: UserTaskDto.Result[];
    DeactivationReason: EnumDto;
  }

  export namespace UserTaskDto {
    export interface Result {
      Name: string;
      Description: string;
      Type: string;
      Value: string;
      Data: string;
    }

  }

  /**
   * List the available UserTasks for the current user.
   * 
   * The response has a `Aggregations` property. This property contains the available filters for this set of UserTasks.
   * To apply such a filter it should added in the `Filters` property on the RequestMessage.
   * 
   * Example response:
   * ```
   * {
   * ...
   * ,
   *   "Aggregations": {
   *       "ZonedCycleCountPreCount": {
   *         "CycleCountZoneID": [
   *           {
   *             "Name": "Shelves",
   *             "Value": 10,
   *             "Count": 1
   *           },
   *           {
   *             "Name": "Stockroom",
   *             "Value": 12,
   *             "Count": 6
   *           },
   *           {
   *             "Name": "Store",
   *             "Value": 13,
   *             "Count": 4
   *           }
   *         ]
   *       }
   *     }
   *   }
   * }
   * ```
   * 
   * To apply a matching filter:
   * ```
   * {
   *   "Filters": {
   *     "ZonedCycleCountPreCount": {
   *       "CycleCountZoneID": [10]
   *     }
   *   }
   * }
   * ```
   */
  export interface ListAvailableUserTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
    /**
     * Return only Hidden/Visible UserTasks
     * @default false
     */
    IsHidden?: boolean;
    /**
     * Return only UserTasks of one of the specified types
     */
    UserTaskTypes?: string[];
    /**
     * Return only UserTasks with one of the specified subtypes
     */
    UserTaskSubTypes?: string[];
    /**
     * Return only UserTasks with one of the specified labels
     */
    Labels?: string[];
    /**
     * Filter for a specific UserTaskType.Name using a property name and a list of values.
     */
    Filters?: { [key: string]: { [key: string]: any[] } };
  }

  export interface ListAvailableUserTasksResponse {
    AvailableTasks: AvailableUserTaskDto[];
    Aggregations: { [key: string]: { [key: string]: UserTaskDataAggregation[] } };
    Labels: { [key: string]: number };
    /**
     * Indicates if the store has a full stock count in progress.
     */
    HasActiveFullStockCount: boolean;
    /**
     * Indicates if one or more confirmation tasks (evem the ones without viewing rights) have AllowStockMutationsDuringCount on true
     */
    AllowStockMutationsDuringCount: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface AvailableUserTaskDto {
    /**
     * Entity type: UserTask
     */
    ID: number;
    /**
     * Entity type: UserTask
     */
    ParentID?: number;
    Description: string;
    Priority: number;
    PriorityDescription: string;
    Sequence: number;
    CanBeIgnored: boolean;
    Labels: string[];
    CreationTime: string;
    StartTime?: string;
    ExpectedTimeToComplete?: string;
    ExpectedCompletionTime?: string;
    Deadline?: string;
    User: UserDto;
    Type: UserTaskTypeDto;
    SubType: EnumDto;
    Children: AvailableUserTaskDto[];
    Data: { [key: string]: any };
    HasUrgency: boolean;
  }

  export interface UserTaskDataAggregation {
    Name: string;
    Value: any;
    Count: number;
  }

  /**
   * Returns the current tasks that prevent the current organization units' financial period from closing.
   */
  export interface ListTasksThatBlockPeriodClosing {
  }

  export interface ListTasksThatBlockPeriodClosingResponse {
    Tasks: UserTaskDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * @deprecated Not supported anymore
   * Returns a paged list of UserTasks.
   */
  export interface ListUserTasks {
    PageConfig?: PageConfig<ListUserTasksFilter>;
  }

  export interface ListUserTasksFilter {
    /**
     * Includes child organizations
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    IsActive?: boolean;
    IsCompleted?: boolean;
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypes?: number[];
  }

  export interface ListUserTasksResponse {
    Result?: PagedResult<ListUserTasksResponse.UserTask>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListUserTasksResponse {
    export interface UserTask {
      ID: number;
      Priority: number;
      IsActive: boolean;
      IsCompleted: boolean;
      Description: string;
      StartTime?: string;
      CompletionTime?: string;
      OrganizationUnit: ListUserTasksResponse.OrganizationUnit;
      UserTaskType: ListUserTasksResponse.UserTaskType;
      UserTaskSubType?: ListUserTasksResponse.UserTaskSubType;
      User?: ListUserTasksResponse.User;
    }

    export interface OrganizationUnit {
      ID: number;
      BackendID: string;
      Name: string;
    }

    export interface UserTaskType {
      ID: number;
      Name: string;
    }

    export interface UserTaskSubType {
      ID: number;
      Name: string;
    }

    export interface User {
      ID: number;
      FullName: string;
    }

  }

  export interface GetUserTaskNotCompletedReasons {
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
  }

  export interface GetUserTaskNotCompletedReasonsResponse {
    Reasons: GetUserTaskNotCompletedReasonsResponse.Reason[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetUserTaskNotCompletedReasonsResponse {
    export interface Reason {
      ReasonID: number;
      Name: string;
      IsSystem: boolean;
      Usages: ReasonUsageWithDescription[];
    }

  }

  export interface ReasonUsageWithDescription {
    UserTaskTypeID?: number;
    UserTaskSubTypeID?: number;
    UserTaskType?: string;
    UserTaskSubType?: string;
    OrganizationUnitSetID?: number;
    OrganizationUnitSetName?: string;
  }

  export interface CreateUserTaskNotCompletedReason {
    Name?: string;
    Usages?: ReasonUsage[];
  }

  export interface ReasonUsage {
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypeID?: number;
    /**
     * Entity type: UserTaskSubType
     */
    UserTaskSubTypeID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitSetID?: number;
  }

  export interface CreateUserTaskNotCompletedReasonResponse {
    /**
     * Entity type: UserTaskNotCompletedReason
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateUserTaskNotCompletedReason {
    /**
     * Entity type: UserTaskNotCompletedReason
     */
    ID?: number;
    Name?: string;
    Usages?: ReasonUsage[];
  }

  export interface DeleteUserTaskNotCompletedReason {
    /**
     * Entity type: UserTaskNotCompletedReason
     */
    ID: number;
  }

  /**
   * Return all of the configured User Task Priority configurations.
   * 
   * These are used at real-time on the User Task details whenever fetched, translating the User Task `Priority` into a `PriorityDescription`. The
   * `LanguageID` and `CountryID` filters are compared against the user context requesting this information, and the `UserTaskTypeID`, `UserTaskSubTypeID`
   * and `Priority` are taken from the User Task itself. The `Priority` from the User Task will be compared against the `From` and `To` ranges defined,
   * where the `From` priority is an *inclusive* lower bound, while `To` is the *exclusive* upper bound.
   * 
   * The `Description` will be used to describe the priority, but when left empty, will fall back to `Name`.
   * 
   * An example configuration could be as follows;
   * 
   * ```
   * Language | From | To | Description
   * ---------|------|----|------------
   *       NL |      |    | Standaard
   *       NL |   10 |    | Spoed
   *       DE |      | 10 | Unwichtig
   *          |      |    | Default
   * ```
   * 
   * A User Task with priority 9 for a Dutch employee would be labelled "Standaard", while for a German colleague this would be called "Unwichtig". A
   * User Task of priority 10 would be called "Spoed" in Dutch, but "Default" in German.
   * 
   * :sunglasses: For added awesomeness, if you define overlapping priority ranges, we'll select the most narrow range for you.
   */
  export interface GetUserTaskPriorities {
  }

  export interface GetUserTaskPrioritiesResponse {
    Result: UserTaskPriorityDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UserTaskPriorityDto {
    From?: number;
    To?: number;
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypeID?: number;
    /**
     * Entity type: UserTaskSubType
     */
    UserTaskSubTypeID?: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: Enum
     */
    ID: number;
    /**
     * Usually an internal & unchangable code - for display, use `Description`
     */
    Name: string;
    Description: string;
  }

  /**
   * Creates a new User Task Priority.
   */
  export interface CreateUserTaskPriority {
    Name?: string;
    Description?: string;
    /**
     * Inclusive
     */
    From?: number;
    /**
     * Exclusive
     */
    To?: number;
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypeID?: number;
    /**
     * Entity type: UserTaskSubType
     */
    UserTaskSubTypeID?: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  export interface CreateUserTaskPriorityResponse {
    /**
     * Entity type: UserTaskPriority
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates an existing User Task Priority.
   */
  export interface UpdateUserTaskPriority {
    /**
     * Entity type: UserTaskPriority
     */
    ID: number;
    Name?: string;
    Description?: string;
    /**
     * Inclusive
     */
    From?: number;
    /**
     * Exclusive
     */
    To?: number;
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypeID?: number;
    /**
     * Entity type: UserTaskSubType
     */
    UserTaskSubTypeID?: number;
    /**
     * Entity type: Language
     */
    LanguageID?: string;
    /**
     * Entity type: Country
     */
    CountryID?: string;
  }

  /**
   * Deletes a User Task Priority.
   */
  export interface DeleteUserTaskPriority {
    /**
     * Entity type: UserTaskPriority
     */
    ID: number;
  }

  export interface CreateUserTaskTypeOrganizationUnitSet {
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypeID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    Required: boolean;
    IgnoreForPeriodClosing?: boolean;
  }

  export interface UserTaskTypeOrganizationUnitSetResponse {
    Result: UserTaskTypeOrganizationUnitSetDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UserTaskTypeOrganizationUnitSetDto {
    ID: number;
    UserTaskTypeID: number;
    OrganizationUnitSetID: number;
    Required: boolean;
    IgnoreForPeriodClosing: boolean;
  }

  export interface GetUserTaskTypeOrganizationUnitSet {
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypeID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
  }

  export interface UpdateUserTaskTypeOrganizationUnitSet {
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypeID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    Required: boolean;
    IgnoreForPeriodClosing?: boolean;
  }

  export interface DeleteUserTaskTypeOrganizationUnitSet {
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypeID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
  }

  export interface ListUserTaskTypeOrganizationUnitSets {
    PageConfig?: PageConfig<ListUserTaskTypeOrganizationUnitSetsFilter>;
  }

  export interface ListUserTaskTypeOrganizationUnitSetsFilter {
    /**
     * Entity type: UserTaskType
     */
    UserTaskTypeID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    Required?: boolean;
  }

  export interface ListUserTaskTypeOrganizationUnitSetsResponse {
    Result?: PagedResult<UserTaskTypeOrganizationUnitSetDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetUserTaskTypes {
    /**
     * If set to true each UserTaskType will also include all of its subtypes, if any.
     */
    IncludeSubTypes?: boolean;
  }

  export interface GetUserTaskTypesResponse {
    Types?: UserTaskTypeDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateUserTaskType {
    /**
     * Entity type: UserTaskType
     */
    ID: number;
    DefaultPriority?: number;
  }

  /**
   * @deprecated Use `ListReturnReasons` instead.
   */
  export interface GetCycleCountSettings {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
  }

  export interface GetCycleCountSettingsResponse {
    OrganizationUnitID: number;
    GenerateCycleCountAfterNASC: boolean;
    MaxCycleCountsPerShop: number;
    MaxCycleCountsPerShopPerDay: number;
    PastDueProductsOnly: boolean;
    CycleCountDays: DaysOfWeek;
    StockLabels: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CompleteZonedCycleCountPreCount {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    Results: CompleteZonedCycleCountPreCountResult[];
  }

  export interface CompleteZonedCycleCountPreCountResult {
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    CountedQuantity?: number;
    SerialNumber?: string;
  }

  export interface CompleteZonedCycleCounts {
    ToComplete: ZonedCycleCountToComplete[];
    CompletionType: CompleteZonedCycleCountCompletionTypes;
  }

  export interface ZonedCycleCountToComplete {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    DeviationReason?: string;
    CountedQuantityCorrections?: ZonedCycleCountToCompleteQuantityCorrection[];
  }

  export interface ZonedCycleCountToCompleteQuantityCorrection {
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    NewCountedQuantity: number;
  }

  export const enum CompleteZonedCycleCountCompletionTypes {
    AcceptPreCount = 0,
    Recount = 1
  }

  export interface CountStockLabelForZonedCycleCount {
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    CountedQuantity?: number;
    SerialNumber?: string;
  }

  export interface CountStockLabelForZonedCycleCountResponse {
    CountResult: ZonedCycleCountResultTypes;
    IsCompleted: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum ZonedCycleCountResultTypes {
    Accepted = 0,
    Recount = 1,
    CompleteResources = 2
  }

  /**
   * Creates one cycle count for the requested Product and Organization Unit (with origin Manual)
   */
  export interface CreateZonedCycleCount {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  export interface CreateZonedCycleCountResponse {
    Created: CycleCountEntry[];
    Skipped: CycleCountEntry[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CycleCountEntry {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductName: string;
    ProductBackendID?: string;
    /**
     * Entity type: UserTask
     */
    TaskID?: number;
    NotCreatedReason?: CycleCountNotCreatedReason;
  }

  export const enum CycleCountNotCreatedReason {
    AlreadyExists = 0,
    ProductNotStock = 1,
    ProductNotInAssortment = 2
  }

  /**
   * Creates cycle counts for all requested combinations of Products and Organization Units (with origin Manual). Use this service through the async service endpoints.
   */
  export interface CreateZonedCycleCounts {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs: number[];
    /**
     * Entity type: Product
     */
    ProductIDs: number[];
  }

  export interface CreateZonedCycleCountsResponse {
    Created?: CycleCountEntry[];
    Skipped?: CycleCountEntry[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Creates cycle counts for all requested combinations of (type Stock) Products and Organization Units that match specified filters (with origin Manual). Use this service through the async service endpoints.
   */
  export interface CreateZonedCycleCountsByQuery {
    /**
     * Query for the product search
     */
    Query?: string;
    /**
     * Filters to apply to product search
     */
    Filters?: { [key: string]: FilterModel };
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export namespace CreateZonedCycleCountsByQuery {
    export interface OrganizationUnitFilterModel {
      /**
       * Entity type: OrganizationUnit
       */
      IDs: number[];
      BackendIDs: string[];
      /**
       * Entity type: Country
       */
      CountryIDs: string[];
      StatusID?: number;
    }

  }

  export interface CreateZonedCycleCountsByQueryResponse {
    Created?: CycleCountEntry[];
    Skipped?: CycleCountEntry[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetCycleCountOriginTypes {
  }

  export interface GetCycleCountOriginTypesResponse {
    Result: EnumDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateCycleCountOriginType {
    /**
     * Entity type: CycleCountOriginType
     */
    ID: number;
    Description?: string;
  }

  /**
   * Creates a new CycleCountZoneGroup.
   */
  export interface CreateCycleCountZoneGroup {
    Name?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: CycleCountZone
     */
    ZoneIDs?: number[];
  }

  export interface CreateCycleCountZoneGroupResponse {
    /**
     * Entity type: CycleCountZoneGroup
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates an existing CycleCountZoneGroup.
   */
  export interface UpdateCycleCountZoneGroup {
    /**
     * Entity type: CycleCountZoneGroup
     */
    ID: number;
    Name?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    /**
     * Entity type: CycleCountZone
     */
    ZoneIDs?: number[];
  }

  /**
   * Deletes a CycleCountZoneGroup by its ID
   */
  export interface DeleteCycleCountZoneGroup {
    /**
     * Entity type: CycleCountZoneGroup
     */
    ID: number;
  }

  /**
   * Gets all the CycleCountZoneGroups based on the given filter and returns a paged response.
   */
  export interface ListCycleCountZoneGroups {
    PageConfig?: PageConfig<ListCycleCountGroupsFilter>;
  }

  export interface ListCycleCountGroupsFilter {
    Name: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: CycleCountZone
     */
    CycleCountZoneID?: number;
    /**
     * Entity type: CycleCountZoneGroup
     */
    CycleCountZoneGroupIDs: number[];
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetIDs?: number[];
  }

  export interface ListCycleCountZoneGroupsResponse {
    Result?: PagedResult<ListCycleCountZoneGroupsResponse.CycleCountZoneGroupDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListCycleCountZoneGroupsResponse {
    export interface CycleCountZoneGroupDto {
      /**
       * Entity type: CycleCountZoneGroup
       */
      ID: number;
      Name: string;
      /**
       * Entity type: OrganizationUnitSet
       */
      OrganizationUnitSetID: number;
      OrganizationUnitSetName: string;
      Zones: EnumDto[];
    }

  }

  /**
   * Before calling Create/UpdateCycleCountZone group you can call this service to perform some checks on the validity of the new zone group.
   * Currently it only returns warnings and calling it is not required before creating or updating the group, but it is recommended as it prevents confusing situations such as the same OrganizationUnit being part
   * of multiple groups at the same time.
   */
  export interface ValidateCycleCountZoneGroup {
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
  }

  export interface ValidateCycleCountZoneGroupResponse {
    HasWarnings: boolean;
    Warnings: ValidateCycleCountZoneGroupResponse.Warning[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ValidateCycleCountZoneGroupResponse {
    export interface Warning {
      Type: string;
      Message: string;
    }

  }

  export interface CreateCycleCountZone {
    Name: string;
    Description?: string;
  }

  export interface CreateCycleCountZoneResponse {
    /**
     * Entity type: CycleCountZone
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface UpdateCycleCountZone {
    /**
     * Entity type: CycleCountZone
     */
    ID: number;
    Name: string;
    Description?: string;
  }

  export interface DeleteCycleCountZone {
    /**
     * Entity type: CycleCountZone
     */
    ID: number;
  }

  export interface GetUserWhoDeletedCycleCountZoneAndMakeItGreatAgain {
    /**
     * Entity type: CycleCountZone
     */
    ID: number;
  }

  export interface GetUserWhoDeletedCycleCountZoneAndMakeItGreatAgainResponse {
    LastModifiedByID?: number;
    LastModificationTime?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetCycleCountZones {
  }

  export interface GetCycleCountZonesResponse {
    Result: CycleCountZoneDto[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CycleCountZoneDto {
    /**
     * Entity type: CycleCountZone
     */
    ID: number;
    Name: string;
    Description: string;
  }

  export interface ReplaceCycleCountZonesForOrganizationUnit {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: CycleCountZone
     */
    CycleCountZoneIDs?: number[];
  }

  export interface ListCycleZonesForOrganizationUnits {
    PageConfig?: PageConfig<ListOrganizationUnitCycleCountZonesFilter>;
  }

  export interface ListOrganizationUnitCycleCountZonesFilter {
    OrganizationUnitID: number;
  }

  export interface ListCycleZonesForOrganizationUnitsResponse {
    Result?: PagedResult<CycleCountZonesForOrganizationUnitDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CycleCountZonesForOrganizationUnitDto {
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    Zones: CycleCountZoneDto[];
  }

  /**
   * Deactivate the given ZonedCycleCounts
   * 
   * Also deactivates linked PreCounts
   */
  export interface DeactivateZonedCycleCount {
    /**
     * Entity type: UserTask
     */
    UserTaskIDs: number[];
    /**
     * A reason, if any, why the tasks were disabled. Optional.
     */
    Reason?: string;
  }

  /**
   * Gets the status of the pre-counts of a cyclecount, if any. The input for this service is either the ID of the main count task (`ZonedCycleCountUserTaskID`)
   * or the ID of a pre-count task (`PreCountUserTaskID`). If `PreCountUserTaskID` is specified, it returns the status of the sibling pre-counts.
   */
  export interface GetStatusForZonedCycleCountPreCounts {
    /**
     * Entity type: UserTask
     */
    ZonedCycleCountUserTaskID?: number;
    /**
     * Entity type: UserTask
     */
    PreCountUserTaskID?: number;
  }

  export interface GetStatusForZonedCycleCountPreCountsResponse {
    PreCounts: GetStatusForZonedCycleCountPreCountsResponse.PreCountStatus[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetStatusForZonedCycleCountPreCountsResponse {
    export interface PreCountStatus {
      CycleCountZone: EnumDto;
      IsCompleted: boolean;
      IsStarted: boolean;
      /**
       * Entity type: User
       */
      UserID?: number;
      UserFullName: string;
      CompletionTime?: string;
      PreCountedQuantity?: number;
      CountResults: PreCountStatus.Result[];
    }

    export namespace PreCountStatus {
      export interface Result {
        StockLabel: EnumDto;
        CountedQuantity: number;
        CreationTime: string;
      }

    }

  }

  /**
   * Returns a detailed view of a ZonedCycleCountPreCount by the ID of its UserTask.
   * 
   * The response contains:
   * 
   * - Which StockLabels to count (`StockLabelsToCount`)
   * - The product that needs to be counted (`Product`); this product contains all the fields in its `Content` object that were requested through the `IncludedFields` property on the request, or a default set of fields if not specified
   * - Zone that needs to be counted (`CycleCountZone`)
   * - List of stock labels to be counted (`StockLabelsToCount`)
   * - List of required resource types (`RequiredResourceTypes`)
   */
  export interface GetZonedCycleCountPreCount {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface GetZonedCycleCountPreCountResponse {
    /**
     * Entity type: ZonedCycleCountPreCount
     */
    ID: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    CycleCountZone: EnumDto;
    StockLabelsToCount: EnumDto[];
    Product: GetZonedCycleCountPreCountResponse.GetZonedCycleCountPreCountProduct;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace GetZonedCycleCountPreCountResponse {
    export interface GetZonedCycleCountPreCountProduct {
      /**
       * Entity type: Product
       */
      ID: number;
      CustomID: string;
      Type: ProductTypes;
      Content: any;
      UnitPriceInTax: number;
      CurrencyID: string;
    }

  }

  /**
   * Returns a detailed view of a ZonedCycleCount by the ID of its UserTask.
   * 
   * The response contains:
   * 
   * - Which StockLabels to count (`StockLabelsToCount`)
   * - The product that needs to be counted. This product contains all the fields in its `Content` object that were requested through the `IncludedFields` property on the request, or a default set of fields if not specified.
   * - Which StockMutations have been created since the first pre-count (if any) per StockLabel (`StockMutationsSincePreCount`)
   * - A list of all pre-counts, complete with the zone that needs to be counted, the user who did the count (if the count already happened) and a list of count results per StockLabel
   * - A list of pre-counted quantityies per StockLabel (`PreCountedQuantities`)
   * - A list of the total quantity of stock mutated per StockLabel (a summary of `StockMutationsSincePreCount`).
   * - The current stock per StockLabel
   * - ExpectedPreCountedQuantity: This is stored on the ZonedCyleCount the moment the last pre-count has been completed, and is a sum of the expected Stock at that moment.
   * - PreCountedQuantity: The quantity that was actually counted in all pre-counts over all StockLabels. This is recorded when all pre-counts have been completed.
   */
  export interface GetZonedCycleCount {
    /**
     * Entity type: UserTask
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface GetZonedCycleCountResponse {
    ID: number;
    UserTaskID: number;
    IsCompleted: boolean;
    IsActive: boolean;
    UserID?: number;
    UserName: string;
    CreatedByID: number;
    CreatedByUserName: string;
    CreationTime: string;
    StartTime?: string;
    CompletionTime?: string;
    ResultStatus?: ZonedCycleCountResultStatus;
    OriginTypeID?: number;
    OriginID?: number;
    IsDeviationReasonRequired: boolean;
    StockLabelsToCount: EnumDto[];
    Product: GetZonedCycleCountProduct;
    StockMutationsSincePreCount: GetZonedCycleCountStockMutation[];
    PreCounts: GetZonedCycleCountPreCountDto[];
    PreCountedQuantities: GetZonedCycleCountPreCountedQuantity[];
    StockMutationQuantitiesSincePreCount: GetZonedCycleCountStockMutationQuantity[];
    StockLabelQuantities: GetZonedCyleCountStockLabelQuantities[];
    ExpectedPreCountedQuantity?: number;
    PreCountedQuantity?: number;
    SerialNumberRequired: boolean;
    SerialNumberConfigurationID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum ZonedCycleCountResultStatus {
    None = 0,
    /**
     * The results of the PreCounts matched the expected stock, so no stockmutations were created
     */
    Correct = 1,
    /**
     * The cyclecount results were discarded and a new one is created
     */
    RecountRequested = 2,
    /**
     * The results of the PreCounts are modified during finalizing
     */
    Modified = 4,
    /**
     * The finalizing resulted in at least one stockmutation
     */
    Deviated = 8
  }

  export interface GetZonedCycleCountProduct {
    ID: number;
    CustomID: string;
    Type: ProductTypes;
    Content: any;
  }

  export interface GetZonedCycleCountStockMutation {
    StockLabel: EnumDto;
    StockMutationReason: EnumDto;
    MutationQuantity: number;
    CreationTime: string;
  }

  export interface GetZonedCycleCountPreCountDto {
    /**
     * Entity type: ZonedCycleCountPreCount
     */
    ID: number;
    CycleCountZone: EnumDto;
    User: GetZonedCycleCountPreCountUserDto;
    Results: GetZonedCycleCountPreCountResult[];
  }

  export interface GetZonedCycleCountPreCountUserDto {
    ID: number;
    Name: string;
  }

  export interface GetZonedCycleCountPreCountResult {
    StockLabel: EnumDto;
    CountedQuantity: number;
    CreationTime: string;
    CurrentStock?: number;
  }

  export interface GetZonedCycleCountPreCountedQuantity {
    StockLabel: EnumDto;
    CountedQuantity?: number;
  }

  export interface GetZonedCycleCountStockMutationQuantity {
    TotalMutationQuantity: number;
    Since: string;
    StockLabel: EnumDto;
  }

  export interface GetZonedCyleCountStockLabelQuantities {
    StockLabel: EnumDto;
    CurrentQuantityOnHand: number;
    ExpectedQuantity: number;
    CountedQuantity: number;
    ModifiedQuantity?: number;
  }

  /**
   * Returns a count of potential cycle counts that would be created for the combinations of (type Stock) Products and Organization Units that match specified filters. Use this service to preview results before actual creation.
   */
  export interface GetZonedCycleCountsEstimateByQuery {
    /**
     * Query for the product search
     */
    Query?: string;
    /**
     * Filters to apply to product search
     */
    Filters?: { [key: string]: FilterModel };
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  export interface GetZonedCycleCountsEstimateByQueryResponse {
    /**
     * Estimated total number of cycle counts that would be created
     */
    CycleCountsEstimate: number;
    /**
     * Number of products matching the filters
     */
    ProductsCount: number;
    /**
     * Number of organization units matching the filters
     */
    OrganizationUnitsCount: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists results associated with a Zoned Cycle Count PreCount
   */
  export interface ListZonedCycleCountPreCountResult {
    /**
     * Entity type: ZonedCycleCountPreCount
     */
    ZonedCycleCountPreCountID: number;
    PageConfig?: EVA.Core.PageConfig<{ [key:string]: string }>;
  }

  export interface ListZonedCycleCountPreCountResultResponse {
    Result?: PagedResult<ListZonedCycleCountPreCountResultDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListZonedCycleCountPreCountResultDto {
    /**
     * Entity type: ZonedCycleCountPreCount
     */
    ID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * Entity type: SerialNumber
     */
    SerialNumberID?: number;
    SerialNumber?: string;
  }

  /**
   * Retrieves the status of serial numbers for a specific product and stock label within a Zoned Cycle Count.
   */
  export interface ListZonedCycleCountSerialNumbersStatus {
    /**
     * Entity type: ZonedCycleCount
     */
    ZonedCycleCountID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    PageConfig?: PageConfig<ListZonedCycleCountSerialNumbersStatusFilter>;
  }

  export interface ListZonedCycleCountSerialNumbersStatusFilter {
    Statuses?: ZonedCycleCountSerialNumberStatus[];
    SerialNumbers?: string[];
  }

  export const enum ZonedCycleCountSerialNumberStatus {
    /**
     * Serial number was expected and counted.
     */
    ExpectedAndCounted = 0,
    /**
     * Serial number was expected but not counted.
     */
    ExpectedNotCounted = 1,
    /**
     * Serial number was expected but counted on a different stock label.
     */
    CountedOnDifferentStockLabel = 2,
    /**
     * Serial number was expected but counted on a different organization unit.
     */
    CountedOnDifferentOrganizationUnit = 3,
    /**
     * Serial number was expected but counted as a different product.
     */
    CountedAsDifferentProduct = 4,
    /**
     * Serial number was counted but not expected.
     */
    Unexpected = 5
  }

  export interface ListZonedCycleCountSerialNumbersStatusResponse {
    Result?: PagedResult<ZonedCycleCountSerialNumberStatusDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ZonedCycleCountSerialNumberStatusDto {
    /**
     * Entity type: StockLabel
     */
    ExpectedStockLabelID?: number;
    /**
     * Entity type: StockLabel
     */
    CountedStockLabelID?: number;
    SerialNumber?: string;
    Status?: ZonedCycleCountSerialNumberStatus;
  }

  /**
   * The page size `Limit` has a maximum value of `250` for this service.
   */
  export interface ListZonedCycleCounts {
    PageConfig?: PageConfig<ListZonedCycleCountsFilter>;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
  }

  export interface ListZonedCycleCountsFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs: number[];
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: UserTask
     */
    UserTaskIDs: number[];
    /**
     * Entity type: CycleCountOriginType
     */
    OriginTypeID?: number;
    IsCompleted?: boolean;
    InProgress?: boolean;
    ResultStatus?: ZonedCycleCountResultStatus;
    IsActive?: boolean;
    FromDate?: string;
    ToDate?: string;
    /**
     * When set to `true`, will overwrite `OrganizationUnitIDs` with given `OrganizationUnitID` and all its children.
     */
    IncludeChildOrganizationUnits?: boolean;
  }

  export interface ListZonedCycleCountsResponse {
    Result?: PagedResult<ListZonedCycleCountsDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListZonedCycleCountsDto {
    ID: number;
    UserTaskID: number;
    IsCompleted: boolean;
    IsActive: boolean;
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    UserID?: number;
    UserName: string;
    CreatedByID: number;
    CreatedByUserName: string;
    CreationTime: string;
    StartTime?: string;
    CompletionTime?: string;
    ResultStatus?: ZonedCycleCountResultStatus;
    OriginTypeID?: number;
    OriginID?: number;
    ProductID: number;
    Product: any;
    BackendID: string;
    CustomID: string;
    ProductBarcode: string;
    ProductDisplayValue: string;
    ExpectedPreCountedQuantity?: number;
    PreCountedQuantity?: number;
    PreCountedDifference?: number;
    DeviationPercentage?: number;
    FinishedPreCountTasks: number;
    TotalPreCountTasks: number;
    PreCountProgression?: number;
  }

  /**
   * 
   */
  export interface ProduceZonedCycleCountHandout {
    /**
     * Entity type: CycleCountZone
     */
    CycleCountZoneIDs: number[];
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
    Channel: TemplateOutputChannel;
  }

  export const enum TemplateOutputChannel {
    Print = 1,
    Download = 2,
    Email = 4,
    Receipt = 8
  }

  export interface ProduceZonedCycleCountHandoutResponse {
    Url: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Starts a ZonedCycleCountPreCount by the ID of its UserTask.
   * 
   * The response contains:
   * 
   * - Which StockLabels to count (`StockLabelsToCount`)
   * - The product that needs to be counted (`Product`); this product contains all the fields in its `Content` object that were requested through the `IncludedFields` property on the request, or a default set of fields if not specified
   * - Zone that needs to be counted (`CycleCountZone`)
   * - List of stock labels to be counted (`StockLabelsToCount`)
   */
  export interface StartZonedCycleCountPreCount {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface StartZonedCycleCountPreCountResponse {
    ID: number;
    UserTaskID: number;
    CycleCountZone: EnumDto;
    StockLabelsToCount: EnumDto[];
    Product: StartZonedCycleCountPreCountResponse.StartZonedCycleCountPreCountProduct;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace StartZonedCycleCountPreCountResponse {
    export interface StartZonedCycleCountPreCountProduct {
      ID: number;
      CustomID: string;
      Type: ProductTypes;
      Content: any;
      UnitPriceInTax: number;
      CurrencyID: string;
      SerialNumberRequired: boolean;
      /**
       * Entity type: SerialNumberConfiguration
       */
      SerialNumberConfigurationID?: number;
    }

  }

  export interface StartZonedCycleCount {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
  }

  export interface StartZonedCycleCountResponse {
    ID: number;
    UserTaskID: number;
    StockLabelsToCount: EnumDto[];
    Product: StartZonedCycleCountProduct;
    StockMutationsSincePreCount: StartZonedCycleCountStockMutation[];
    StockMutationQuantitiesSincePreCount: StartZonedCycleCountStockMutationQuantity[];
    PreCounts: StartZonedCycleCountPreCountDto[];
    PreCountedQuantities: StartZonedCycleCountPreCountedQuantity[];
    CurrentStock: StartZonedCyleCountCurrentStock[];
    ExpectedPreCountedQuantity?: number;
    PreCountedQuantity?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StartZonedCycleCountProduct {
    ID: number;
    CustomID: string;
    Type: ProductTypes;
    Content: any;
    UnitPriceInTax: number;
    CurrencyID: string;
    SerialNumberRequired: boolean;
    /**
     * Entity type: SerialNumberConfiguration
     */
    SerialNumberConfigurationID?: number;
  }

  export interface StartZonedCycleCountStockMutation {
    StockLabel: EnumDto;
    StockMutationReason: EnumDto;
    MutationQuantity: number;
    CreationTime: string;
  }

  export interface StartZonedCycleCountStockMutationQuantity {
    TotalMutationQuantity: number;
    Since: string;
    StockLabel: EnumDto;
  }

  export interface StartZonedCycleCountPreCountDto {
    CycleCountZone: EnumDto;
    User: StartZonedCycleCountPreCountUserDto;
    Results: StartZonedCycleCountPreCountResult[];
    IsCompleted: boolean;
  }

  export interface StartZonedCycleCountPreCountUserDto {
    ID: number;
    Name: string;
  }

  export interface StartZonedCycleCountPreCountResult {
    StockLabel: EnumDto;
    CountedQuantity: number;
    CreationTime: string;
    SerialNumber?: string;
  }

  export interface StartZonedCycleCountPreCountedQuantity {
    StockLabel: EnumDto;
    CountedQuantity?: number;
  }

  export interface StartZonedCyleCountCurrentStock {
    StockLabel: EnumDto;
    CurrentQuantityOnHand: number;
  }

  /**
   * Download an Excel sample which can be filled with cycle count info. The populated excel can be uploaded through the `UploadCycleCountExcel` service.
   */
  export interface DownloadCycleCountSample {
  }

  /**
   * Uploads an Excel retrieved from `DownloadCycleCountSample`.
   */
  export interface UploadCycleCountExcel {
    Data: string;
  }

  /**
   * Creates a Zoned Cycle Count Schedule.
   * 
   * It is required to specify either a set `Date`, or a `CycleCountIntervalInDays`.
   */
  export interface CreateZonedCycleCountSchedule {
    BackendID?: string;
    Description?: string;
    /**
     * Range: 1-1000
     */
    CycleCountIntervalInDays?: number;
    Date?: string;
    ProductSearchModel: ZonedCycleCountScheduleProductSearchModel;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
  }

  export interface ZonedCycleCountScheduleProductSearchModel {
    Query?: string;
    Filters?: { [key: string]: ZonedCycleCountScheduleProductSearchFilterModel };
  }

  export interface ZonedCycleCountScheduleProductSearchFilterModel {
    Values?: any[];
    From?: string;
    To?: string;
    Negation?: boolean;
    ExactMatch?: boolean;
    IncludeMissing?: boolean;
  }

  export interface CreateZonedCycleCountScheduleResponse {
    /**
     * Entity type: ZonedCycleCountSchedule
     */
    ID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists the Zoned Cycle Count Schedules.
   */
  export interface ListZonedCycleCountSchedules {
    PageConfig?: PageConfig<ListZonedCycleCountSchedulesFilter>;
  }

  export interface ListZonedCycleCountSchedulesFilter {
    /**
     * Entity type: ZonedCycleCountSchedule
     */
    ID?: number;
    BackendID?: string;
    Description?: string;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
  }

  export interface ListZonedCycleCountSchedulesResponse {
    Result?: PagedResult<ZonedCycleCountScheduleDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ZonedCycleCountScheduleDto {
    /**
     * Entity type: ZonedCycleCountSchedule
     */
    ID: number;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID: number;
    BackendID: string;
    Description: string;
    CycleCountIntervalInDays?: number;
    Date?: string;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreatedByName: string;
    /**
     * Entity type: User
     */
    LastModifiedByID?: number;
    LastModifiedByName: string;
    OrganizationUnitSetName: string;
    ProductSearchModel: ZonedCycleCountScheduleProductSearchModel;
  }

  /**
   * Updates a Zoned Cycle Count Schedule.
   */
  export interface UpdateZonedCycleCountSchedule {
    /**
     * Entity type: ZonedCycleCountSchedule
     */
    ID: number;
    BackendID?: string;
    Description?: string;
    /**
     * Range: 1-1000
     */
    CycleCountIntervalInDays?: number;
    Date?: string;
    ProductSearchModel?: ZonedCycleCountScheduleProductSearchModel;
    /**
     * Entity type: OrganizationUnitSet
     */
    OrganizationUnitSetID?: number;
  }

  /**
   * Deletes a Zoned Cycle Count Schedule.
   */
  export interface DeleteZonedCycleCountSchedule {
    /**
     * Entity type: ZonedCycleCountSchedule
     */
    ID: number;
  }

  /**
   * Complete a StockReservation task that has been started with the StartStockReservation(2) service
   * 
   * The service should be called with a line for each orderline that should be reserved. It is not required anymore to send a line for each item on a line.
   * 
   * Provide each line with an OrderLineID and a QuantityTo(Cancel/Reserve/Order).
   */
  export interface CompleteStockReservationTask {
    Task: CompleteStockReservationTask.StockReservationTask;
    /**
     * Station to print the receipt on
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Should we print a receipt? Default true
     */
    PrintReceipt?: boolean;
  }

  export namespace CompleteStockReservationTask {
    export interface StockReservationTask {
      /**
       * Entity type: UserTask
       */
      UserTaskID: number;
      Lines: CompleteStockReservationTask.StockReservationTaskLine[];
    }

    export interface StockReservationTaskLine {
      /**
       * Entity type: OrderLine
       */
      OrderLineID?: number;
      QuantityToCancel?: number;
      QuantityToReserve?: number;
      QuantityToOrder?: number;
      ForceComplete?: boolean;
      SerialNumber?: string;
    }

  }

  /**
   * Retrieve details on a Stock Reservation Task based on an OrderID.
   */
  export interface GetStockReservationTask {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  export interface GetStockReservationTaskResponse {
    Result: StockReservationTaskDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StockReservationTaskDto {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    Order: OrderDto;
    OrderDisplayID?: string;
    Lines: StockReservationTaskLineDto[];
  }

  export interface StockReservationTaskLineDto {
    IsCancelled: boolean;
    IsReserved: boolean;
    Barcodes: string[];
    Product: { [key: string]: any };
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    QuantityOnHand: number;
    ForceComplete: boolean;
    MustBeOrdered: boolean;
    /**
     * Entity type: OrderLine
     */
    Requirements: { [key: number]: any };
  }

  /**
   * Print the Reservation receipt for the given Order
   */
  export interface PrintStockReservationReceipt {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Station
     */
    StationID?: number;
  }

  /**
   * Starts a user task of type Stock Reservation.
   * 
   * - The task will return all required data for display purposes
   * - When no lines are returned, the task is already completed or in progress by someone else
   * - To complete the StockReservation task, use the `CompleteStockReservationTask` service
   */
  export interface StartStockReservation2 {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Force a restart of the task, even when it's already started by a different user.
     * @default false
     */
    Force?: boolean;
  }

  export interface StartStockReservationResponse2 {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    OrderDisplayID?: string;
    Lines: StartStockReservationLine[];
    ServiceLines: StartStockReservationLine[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StartStockReservationLine {
    /**
     * Entity type: OrderLine
     */
    OrderLineID: number;
    Quantity: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductCustomID?: string;
    ProductBackendID?: string;
    ProductDisplayValue: string;
    ProductUnitPrice: number;
    ProductPrimaryImage?: string;
    TotalAmount: number;
    ProductVariation?: { [key: string]: string };
    SerialNumber?: string;
    SerialNumberRequired: boolean;
    SerialNumberConfigurationID?: number;
    /**
     * Current stock of this product in the pickuporganizationunit
     */
    QuantityOnHand: number;
    /**
     * The child lines for this line
     */
    ChildLines: StartStockReservationLine[];
    /**
     * Entity type: OrderLine
     */
    ParentID?: number;
    GroupID?: string;
    /**
     * Entity type: ProductBundleLine
     */
    ProductBundleLineID?: number;
    /**
     * Entity type: OrderLine
     */
    Requirements: { [key: number]: any };
  }

  export interface CompleteReservationCleanup {
    Task: CompleteReservationCleanup.ReservationCleanupTask;
    /**
     * Can be set to true so that lines missing from the workset are copied to a new cleanup task, so these can be done at a later point in time. This is
     * useful when the employee is handling a large order and wants to move a single batch first.
     */
    CreateNewTaskForRemainingLines?: boolean;
    /**
     * Should be set to true when the user did not confirm the order or user task (by means of a barcode scan or something). When set to true, the user will
     * be required to have (or be elevated for) the functionality `CompleteReservationCleanupWithoutConfirmation`.
     */
    NoConfirmation?: boolean;
  }

  export namespace CompleteReservationCleanup {
    export interface ReservationCleanupTask {
      /**
       * Entity type: UserTask
       */
      UserTaskID: number;
      /**
       * Every line on the cleanup task must be represented in this collection and each line must have an indication what needs
       * to happen to it in the context of the cleanup. The options are:
       * 
       * - The product was returned, in which case QuantityReturned must have a value.
       * - There was nothing there to return, in which case NothingToReturn must be set to true. A cyclecount will be generated for the product to make sure.
       * - The reservation must be extended, in which case ReservationExpirationOverride must have a value.
       */
      Lines: CompleteReservationCleanup.ReservationCleanupTaskLine[];
    }

    export interface ReservationCleanupTaskLine {
      QuantityToReturn: number;
      QuantityReturned: number;
      NothingToReturn?: boolean;
      ReservationExpirationOverride?: string;
      NothingToReturnAction?: CompleteReservationCleanup.NothingToReturnAction;
      OrderLine: CompleteReservationCleanup.ReservationCleanupTaskOrderLine;
    }

    export interface ReservationCleanupTaskOrderLine {
      /**
       * Entity type: OrderLine
       */
      ID?: number;
      SerialNumber?: string;
    }

    export const enum NothingToReturnAction {
      Cancel = 0,
      Complete = 1
    }

  }

  export interface StartReservationCleanup {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartReservationCleanupResponse {
    Task: ReservationCleanupWorkSet;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ReservationCleanupWorkSet {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: UserTask
     */
    ReservationCleanupTaskID: number;
    Type: ReservationCleanupTaskTypes;
    Order: OrderDto;
    Lines: ReservationCleanupWorkSetOrderLine[];
  }

  export const enum ReservationCleanupTaskTypes {
    Default = 0,
    ReturnToSupplier = 1
  }

  export interface ReservationCleanupWorkSetOrderLine {
    QuantityToReturn: number;
    QuantityReturned: number;
    NothingToReturn: boolean;
    ReservationExpirationOverride?: string;
    OrderLine: OrderLineDto;
    Type: ReservationCleanupTaskLineTypes;
  }

  export const enum ReservationCleanupTaskLineTypes {
    ExpiredReservation = 0,
    Cancellation = 1
  }

  /**
   * Completes a Stock Replenishment task, using the UserTaskIDs from the `WorkSet` returned by the `StartStockReplenishment` service.
   */
  export interface CompleteStockReplenishment {
    /**
     * Entity type: UserTask
     */
    UserTaskIDs?: number[];
  }

  export interface StockReplenishmentWorkSet {
    Items: StockReplenishmentWorkSetItem[];
  }

  export interface StockReplenishmentWorkSetItem {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: StockReplenishmentTask
     */
    StockReplenishmentTaskID: number;
    Quantity: number;
    Type: StockReplenishmentType;
    Zone: string;
    Product: { [key: string]: any };
    Originator: string;
  }

  export const enum StockReplenishmentType {
    Default = 0,
    Priority = 1,
    AutoGenerated = 2
  }

  /**
   * Creates a new Stock Replenishment task.
   */
  export interface CreateStockReplenishmentTask {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Quantity: number;
    Type?: StockReplenishmentType;
    Zone?: string;
  }

  /**
   * Creates new Stock Replenishment tasks in bulk, with `DetailedProducts` being an array of products details.
   */
  export interface CreateStockReplenishmentTasks {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    DetailedProducts: CreateStockReplenishmentProduct[];
  }

  export interface CreateStockReplenishmentProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Zone: string;
    Quantity: number;
    Type?: StockReplenishmentType;
  }

  /**
   * Get the additional Stock Replenishment task data.
   */
  export interface GetStockReplenishmentTaskData {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export interface GetStockReplenishmentTaskDataResponse {
    Disabled: boolean;
    AutoGenerateQuantity: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Get the configured zones for your current organization unit.
   */
  export interface GetStockReplenishmentTaskZones {
  }

  export interface GetStockReplenishmentTaskZonesResponse {
    Zones: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Lists the additional Stock Replenishment task data.
   */
  export interface ListStockReplenishmentTaskData {
    PageConfig?: PageConfig<ListStockReplenishmentTaskDataFilter>;
  }

  export interface ListStockReplenishmentTaskDataFilter {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    Disabled?: boolean;
  }

  export interface ListStockReplenishmentTaskDataResponse {
    Result?: PagedResult<StockReplenishmentTaskDataDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StockReplenishmentTaskDataDto {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Disabled: boolean;
    AutoGenerateQuantity: number;
  }

  /**
   * Starts one or more Stock Replenishment tasks, returning a workset. Pass this workset to the `CompleteStockReplenishment` service when all tasks are complete.
   */
  export interface StartStockReplenishment {
    /**
     * Entity type: UserTask
     */
    UserTaskIDs?: number[];
  }

  export interface StartStockReplenishmentResponse {
    WorkSet: StockReplenishmentWorkSet;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Updates Stock Replenishment task data.
   */
  export interface UpdateStockReplenishmentTaskData {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Disabled?: boolean;
    AutoGenerateQuantity?: number;
  }

  export interface StartSecondChanceProductRegistration {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  export interface StartSecondChanceProductRegistrationResponse {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    ProductName: string;
    Description: string;
    SerialNumber?: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CancelSecondChanceProductRegistration {
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    /**
     * A reason, if any, why the tasks were disabled. Optional.
     */
    Reason?: string;
  }

  export interface CompleteSecondChanceProductRegistration {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    ProductName: string;
    Description: string;
    /**
     * A list of blob urls.
     */
    Images?: string[];
    Price: number;
    NotReturnable?: boolean;
    SerialNumber?: string;
    /**
     * Stock label for original product after stock movement. If unspecified, defaults to Damaged.
     * Entity type: StockLabel
     */
    StockLabelID?: number;
  }

  export interface CompleteSecondChanceProductRegistrationResponse {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Barcode: string;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StartReceiveShipment {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProductProperties?: string[];
  }

  export interface StartReceiveShipmentResponse {
    ReceiveMethod: ShipmentReceiveMethods;
    IsCompleted: boolean;
    WorkSet: ReceiveShipmentWorkSet;
    AllowSurplus: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CanProcessInitialCycleCount {
    /**
     * Entity type: InitialCycleCountLabel
     */
    InitialCycleCountID: number;
  }

  export interface CanProcessInitialCycleCountResponse {
    CanProcess: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface CountProductForInitialCycleCount {
    /**
     * Entity type: InitialCycleCountLabel
     */
    LabelID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    Quantity: number;
  }

  export interface CreateInitialCycleCount {
  }

  export interface CreateInitialCycleCountResponse {
    Result: InitialCycleCountDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface InitialCycleCountDto {
    /**
     * Entity type: InitialCycleCount
     */
    ID: number;
    Status: InitialCycleCountStatus;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
  }

  export const enum InitialCycleCountStatus {
    New = 0,
    DetailCount = 1,
    Processing = 2,
    Finished = 3
  }

  export interface DeleteInitialCycleCountResult {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabel: number;
    /**
     * Entity type: InitialCycleCountLabel
     */
    LabelID: number;
  }

  export interface GetCurrentInitialCycleCount {
  }

  export interface GetCurrentInitialCycleCountResponse {
    Result: InitialCycleCountDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface GetInitialCycleCountLabel {
    /**
     * Entity type: InitialCycleCountLabel
     */
    LabelID: number;
  }

  export interface GetInitialCycleCountLabelResponse {
    Label: InitialCycleCountLabelDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface InitialCycleCountLabelDto {
    /**
     * Entity type: InitialCycleCountLabel
     */
    ID: number;
    Number: number;
    Status: InitialCycleCountLabelStatus;
    /**
     * Entity type: StockLabel
     */
    StockLabel?: number;
    ItemPreCount?: number;
    ItemEndCount?: number;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreationTime: string;
    InitialCycleCount: InitialCycleCountDto;
    PreCountedByFullName: string;
    CountedByFullName: string;
  }

  export const enum InitialCycleCountLabelStatus {
    New = 0,
    PreCounted = 1,
    Counted = 2
  }

  export interface ListInitialCycleCountLabels {
    /**
     * Entity type: InitialCycleCount
     */
    InitialCycleCountID: number;
    Start: number;
    Limit: number;
  }

  export interface ListInitialCycleCountLabelsResponse {
    Result: InitialCycleCountLabelDto[];
    Total: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListInitialCycleCountResults {
    /**
     * Min: 0
     */
    Start: number;
    /**
     * Min: 1
     */
    Limit: number;
    ShowAll?: boolean;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    CustomItemNumber?: string;
    /**
     * Entity type: StockLabel
     */
    StockLabel?: number;
    /**
     * Entity type: InitialCycleCountLabel
     */
    LabelID?: number;
  }

  export interface ListInitialCycleCountResultsResponse {
    Results: StockWithInitialCycleCountResultDto[];
    Total: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface StockWithInitialCycleCountResultDto {
    ProductID: number;
    OrganizationUnitID: number;
    QuantityOnHand: number;
    StockLabelID: number;
    StockLabel: number;
    Quantity: number;
    BackendID: string;
    LabelID: string;
    LabelNumber: string;
    ShortDescription: string;
  }

  export interface MissingProductForInitialCycleCount {
    /**
     * Entity type: InitialCycleCountLabel
     */
    LabelID: number;
    Barcode?: string;
    Quantity: number;
    Photo?: string;
    PhotoMimeType?: string;
  }

  export interface PreCountInitialCycleCountLabel {
    /**
     * Entity type: InitialCycleCountLabel
     */
    LabelID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    Quantity: number;
  }

  export interface ProcessInitialCycleCountResults {
  }

  export interface ProduceInitialCycleCountLabels {
    /**
     * Range: 1-100
     */
    Count: number;
    /**
     * The ID of the station, which will be used to determine what printer to use to print the bar codes
     * Entity type: Station
     */
    StationID?: number;
    AsPDF?: boolean;
    /**
     * If this is set all the labels that've currently been created will be re-generated, but no new labels will be created in the process.
     */
    ReprintLabels?: boolean;
    Destination: MessageTemplateDestinations;
  }

  export interface ValidatePreCount {
    /**
     * Entity type: InitialCycleCountLabel
     */
    LabelID: number;
    ConfirmedQuantity?: number;
  }

  export interface ValidatePreCountResponse {
    Result: ValidatePreCountResults;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export const enum ValidatePreCountResults {
    Correct = 1,
    Deviation = 2,
    RedoDetailCount = 3
  }

  /**
   * Lists the products of specified full stock count label.
   */
  export interface ListFullStockCountLabelProducts {
    /**
     * Entity type: FullStockCountLabel
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
    PageConfig?: PageConfig<ListFullStockCountLabelProductsFilter>;
  }

  export interface ListFullStockCountLabelProductsFilter {
    /**
     * Entity type: Product
     */
    ProductID?: number;
  }

  export interface ListFullStockCountLabelProductsResponse {
    Result?: PagedResult<ListFullStockCountLabelProductDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListFullStockCountLabelProductDto {
    /**
     * Entity type: Product
     */
    ProductID: number;
    BackendID: string;
    /**
     * Entity type: User
     */
    CurrentCountUserID?: number;
    Quantity: number;
    PreviousQuantity?: number;
    /**
     * Entity type: User
     */
    PreviousCountUserID?: number;
    Properties: any;
    CreationTime: string;
    LastModificationTime?: string;
    IsSerialNumberRequired: boolean;
  }

  /**
   * Returns the information of a single label product. Most notably, this also includes serialnumber information.
   */
  export interface GetFullStockCountLabelProduct {
    /**
     * Entity type: FullStockCountLabel
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
    /**
     * Entity type: Product
     */
    ProductID: number;
  }

  export interface GetFullStockCountLabelProductResponse {
    /**
     * Entity type: User
     */
    CurrentCountUserID?: number;
    Quantity?: number;
    PreviousQuantity?: number;
    /**
     * Entity type: User
     */
    PreviousCountUserID?: number;
    Properties?: any;
    CreationTime?: string;
    LastModificationTime?: string;
    /**
     * All serialnumbers scanned as part of this product
     */
    SerialNumbers?: string[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Initiates a recount of the given FullStockCountProduct for the given FullStockCount. FullStockCount will not be completed.
   */
  export interface RecountFullStockCountProduct {
    /**
     * Entity type: FullStockCount
     */
    FullStockCountID: number;
    Recounts?: RecountFullStockCountProduct.Recount[];
  }

  export namespace RecountFullStockCountProduct {
    export interface Recount {
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
      /**
       * Entity type: Product
       */
      ProductID: number;
    }

  }

  /**
   * Adds a product count result to specified Full Stock Count label. User task must be active and assigned to the current user.
   */
  export interface AddProductToFullStockCountLabel {
    /**
     * Entity type: FullStockCountLabel
     */
    ID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    Quantity?: number;
    /**
     * This will instead of overwriting, increment the quantity.
     */
    IncrementQuantity?: boolean;
    SerialNumber?: string;
  }

  export interface AddProductToFullStockCountLabelResponse {
    ID: number;
    Product: AddProductToFullStockCountLabelResponse.AddProductToFullStockCountLabelProductDto;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace AddProductToFullStockCountLabelResponse {
    export interface AddProductToFullStockCountLabelProductDto {
      /**
       * Entity type: Product
       */
      ProductID: number;
      BackendID: string;
      CurrentCountUserID?: number;
      Quantity: number;
      PreviousQuantity?: number;
      PreviousCountUserID?: number;
      Properties: any;
      CreationTime: string;
      LastModificationTime?: string;
      SerialNumbers?: string[];
    }

  }

  /**
   * Initiates a recount of the given FullStockCountLabel. FullStockCount will not be completed.
   */
  export interface RecountFullStockCountLabel {
    /**
     * Entity type: FullStockCountLabel
     */
    ID: number;
  }

  export interface RecountFullStockCountLabelResponse {
    ID: number;
    UserTaskID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Starts a Full Stock Count label task.
   */
  export interface StartFullStockCountLabel {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartFullStockCountLabelResponse {
    ID: number;
    UserTaskID: number;
    FullStockCountID: number;
    Name: string;
    StockLabelID: number;
    StockLabelName: string;
    IsRecount: boolean;
    PredefinedProducts: number[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Cancel a specific Full Stock Count label.
   */
  export interface CancelFullStockCountLabel {
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    /**
     * Entity type: FullStockCountLabel
     */
    FullStockCountLabelID?: number;
  }

  /**
   * Completes a Full Stock Count label task. User task must be active and assigned to the current user.
   */
  export interface CompleteFullStockCountLabel {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: Product
     */
    Products?: { [key: number]: number };
  }

  /**
   * Lists the Full Stock Count labels.
   */
  export interface ListFullStockCountLabels {
    /**
     * Entity type: FullStockCount
     */
    ID: number;
    PageConfig?: PageConfig<ListFullStockCountLabelsFilter>;
  }

  export interface ListFullStockCountLabelsFilter {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * When omitted, all FullStockCountLabels are returned.
     * When false, only uncompleted FullStockCountLabels are returned.
     * When true, only completed or cancelled FullStockCountLabels are returned.
     */
    Completed?: boolean;
  }

  export interface ListFullStockCountLabelsResponse {
    Result?: PagedResult<ListFullStockCountLabelDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListFullStockCountLabelDto {
    /**
     * Entity type: FullStockCountLabel
     */
    ID: number;
    /**
     * Entity type: FullStockCount
     */
    FullStockCountID: number;
    Name: string;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    StockLabelName: string;
    IsRecount: boolean;
    State: FullStockCountLabelState;
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    UserFullName: string;
  }

  export const enum FullStockCountLabelState {
    Waiting = 0,
    CountInProgress = 1,
    WaitingRecount = 2,
    RecountInProgress = 3,
    Cancelled = 8,
    Completed = 9
  }

  /**
   * Prints label(s) for specified Full Stock Count to specified station.
   */
  export interface PrintFullStockCountLabels {
    /**
     * Entity type: Station
     */
    StationID?: number;
    /**
     * Prints all labels for this count - required if FullStockCountLabelID is empty
     * Entity type: FullStockCount
     */
    FullStockCountID?: number;
    /**
     * Prints just this label - required if FullStockCountID is empty
     * Entity type: FullStockCountLabel
     */
    FullStockCountLabelID?: number;
  }

  /**
   * Initiate the Full Stock Count process for the current store. **Prevents basic store functionalities once started.**
   */
  export interface CreateFullStockCount {
    /**
     * Indicates if the full stock count is done by an external party.
     * When false, the full stock count will be created and all basic store functionalities will be blocked once started.
     * 
     * When true, the full stock count will be created with a snapshot from the current stock. To set the result of the external full stock count, the 'UploadExternalFullStockCount' can be called.
     * Basic store functionalities will remaining in working order. Deviations will be calculated based on stock mutations.
     * The UserTask will be started automatically.
     */
    IsExternal?: boolean;
    /**
     * When this value is set to true, EVA will allow for stockmutations to be done during the fullstockcount.
     * WARNING: setting this value to true will make the outcome of the fullstockcount unreliable, because quantities on the full stock count label products can be changed before or after a count has been done.
     */
    AllowStockMutationsDuringCount: boolean;
    /**
     * When supplied, it will automatically create a validation recount based on the given recount interval. Ignored when IsExternal is set to true.
     * Min: 1
     */
    RecountInterval?: number;
    /**
     * When supplied, it will automatically call `AddLabelToFullStockCount` for each Shelf defined in your replenishment configuration. Ignored when IsExternal is set to true.
     */
    CreateFullStockCountLabelsForShelves?: boolean;
    /**
     * When supplied, it will automatically call `AddLabelToFullStockCount` for each InitialLabel. Ignored when IsExternal or CreateFullStockCountLabelsForShelves is set to true.
     */
    InitialFullStockCountLabels?: CreateFullStockCount.InitialFullStockCountLabel[];
    /**
     * When set to true, the service will return the current stock information.
     */
    ReturnCurrentStock?: boolean;
  }

  export namespace CreateFullStockCount {
    export interface InitialFullStockCountLabel {
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
      Name?: string;
    }

  }

  export interface CreateFullStockCountResponse {
    /**
     * Entity type: FullStockCount
     */
    ID: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    CurrentStock: CreateFullStockCountResponse.ProductStock[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace CreateFullStockCountResponse {
    export interface ProductStock {
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
      /**
       * Entity type: Product
       */
      ProductID: number;
      QuantityOnHand: number;
    }

  }

  /**
   * Adds a label to specified full stock count.
   */
  export interface AddLabelToFullStockCount {
    /**
     * Entity type: FullStockCount
     */
    FullStockCountID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    Name?: string;
  }

  export interface AddLabelToFullStockCountResponse {
    /**
     * Entity type: FullStockCountLabel
     */
    ID: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface DownloadFullStockCountLabelUploadTemplate {
  }

  /**
   * Upload an excel with labels to the specified full stock count.
   * Caution! The Name column can only be max 50 chars. Names larger then that will be rejected.
   * Caution! This service will not change existing labels, but only add new ones. Duplicate detection is done on case insensitve Name in combination with StockLabel.
   */
  export interface UploadFullStockCountLabels {
    /**
     * Entity type: FullStockCount
     */
    FullStockCountID: number;
    Data: string;
  }

  export interface UploadFullStockCountLabelsResponse {
    Added: UploadFullStockCountLabelsResponse.AddedLabel[];
    Rejected: UploadFullStockCountLabelsResponse.RejectedLabel[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace UploadFullStockCountLabelsResponse {
    export interface AddedLabel {
      /**
       * Entity type: FullStockCountLabel
       */
      ID: number;
      Name: string;
      /**
       * Entity type: StockLabel
       */
      StockLabelID: number;
    }

    export interface RejectedLabel {
      Name: string;
      StockLabel: string;
      Reason: UploadFullStockCountLabelsResponse.RejectionReason;
    }

    export const enum RejectionReason {
      None = 0,
      NameToLong = 1,
      UknownStockLabel = 2,
      DuplicateInUpload = 3,
      AlreadyExists = 4
    }

  }

  /**
   * Lists the full stock counts.
   */
  export interface ListFullStockCounts {
    PageConfig?: PageConfig<ListFullStockCountsFilter>;
  }

  export interface ListFullStockCountsFilter {
    /**
     * Entity type: FullStockCount
     */
    FullStockCountID?: number;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitIDs?: number[];
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * This filters full stock counts with specific states;
     * `New` returns full stock counts that have been created but not yet started
     * `Active` returns full stock counts which are currenty in count process.
     * `Completed` return full stock counts that are completed (but not cancelled)
     * `Cancelled` return full stock counts that are cancelled
     * 
     * If no states are added (aka; empty list) ALL full stock counts will be returned.
     * If multiple states are desired, you can add both states to the list. For example: to get all completed and cancelled full stock count, both the `Completed` and the `Cancelled` state can be added to the list.
     */
    State?: ListFullStockCountsFilter.FullStockCountFilterState[];
  }

  export namespace ListFullStockCountsFilter {
    export const enum FullStockCountFilterState {
      Open = 0,
      Active = 1,
      Completed = 2,
      Cancelled = 3
    }

  }

  export interface ListFullStockCountsResponse {
    Result?: PagedResult<ListFullStockCountDto>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListFullStockCountDto {
    /**
     * Entity type: FullStockCount
     */
    ID: number;
    IsExternal: boolean;
    AllowStockMutationsDuringCount: boolean;
    RecountInterval: number;
    Type: FullStockCountType;
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    UserFullName: string;
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    OrganizationUnitName: string;
    IsActive: boolean;
    IsCompleted: boolean;
    /**
     * Entity type: User
     */
    CreatedByID: number;
    CreatedByFullName: string;
    CreationTime: string;
  }

  export const enum FullStockCountType {
    Initial = 1,
    InProgress = 2
  }

  /**
   * Get a Full Stock count task with details.
   */
  export interface GetFullStockCount {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface GetFullStockCountResponse {
    /**
     * Entity type: FullStockCount
     */
    ID: number;
    IsExternal: boolean;
    AllowStockMutationsDuringCount: boolean;
    RecountInterval: number;
    CreatedByFullName: string;
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    UserFullName: string;
    CreationTime: string;
    CompletionTime?: string;
    Type: FullStockCountType;
    Progress: number;
    IsReady: boolean;
    IsActive: boolean;
    IsCompleted: boolean;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Starts a full stock count task. Requires it to be ready.
   */
  export interface StartFullStockCount {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
  }

  export interface StartFullStockCountResponse {
    /**
     * Entity type: FullStockCount
     */
    ID: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    RecountInterval: number;
    Type: FullStockCountType;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * Cancel a full stock count, cancelling all related labels and their results.
   */
  export interface CancelFullStockCount {
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    /**
     * A reason, if any, why the tasks were disabled. Optional.
     */
    Reason?: string;
  }

  /**
   * Completes a full stock count task. User task must be active and assigned to the current user.
   */
  export interface CompleteFullStockCount {
    /**
     * Entity type: UserTask
     */
    UserTaskID: number;
    /**
     * When set to true, deviations will NOT be processed as stock mutations.
     */
    DismissDeviationsAsStockMutations?: boolean;
    /**
     * Recount all products in the full stock count.
     */
    RecountAllProducts?: boolean;
    /**
     * Recount all products in the full stock count that have a deviation.
     */
    RecountAllDeviations?: boolean;
    /**
     * Products to recount after completion of given full stock count.
     * Entity type: Product
     */
    ProductsToRecount?: number[];
  }

  /**
   * Exports the summary of specified Full Stock Count to Excel.
   */
  export interface DownloadFullStockCountSummaryExcel {
    /**
     * Entity type: FullStockCount
     */
    ID: number;
    OnlyShowDeviations?: boolean;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
  }

  /**
   * Exports the summary of specified Full Stock Count to PDF.
   */
  export interface DownloadFullStockCountSummaryPDF {
    /**
     * Entity type: FullStockCount
     */
    ID: number;
    OnlyShowDeviations?: boolean;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
  }

  /**
   * Lists the result of specified Full Stock Count.
   */
  export interface ListFullStockCountResult {
    /**
     * Entity type: FullStockCount
     */
    ID?: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
    PageConfig?: PageConfig<ListFullStockCountResultFilter>;
  }

  export interface ListFullStockCountResultFilter {
    /**
     * Entity type: User
     */
    UserID?: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    OnlyShowDeviations?: boolean;
  }

  export interface ListFullStockCountResultResponse {
    FullStockCountStartTime?: string;
    FullStockCountCompletionTime?: string;
    Result?: PagedResult<FullStockCountResult>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FullStockCountResult {
    /**
     * Entity type: FullStockCountLabel
     */
    FullStockCountLabelID: number;
    Name: string;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    StockLabelName: string;
    /**
     * Entity type: User
     */
    UserID?: number;
    UserFullName: string;
    /**
     * Entity type: Product
     */
    ProductID: number;
    BackendID: string;
    Quantity: number;
    Properties: any;
    /**
     * Entity type: User
     */
    RecountUserID?: number;
    RecountUserFullName: string;
    RecountQuantity?: number;
  }

  /**
   * Lists the summary of specified Full Stock Count.
   */
  export interface ListFullStockCountSummary {
    /**
     * Entity type: FullStockCount
     */
    ID?: number;
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedProperties?: string[];
    PageConfig?: PageConfig<ListFullStockCountSummaryFilter>;
  }

  export interface ListFullStockCountSummaryFilter {
    /**
     * Entity type: StockLabel
     */
    StockLabelID?: number;
    /**
     * Entity type: Product
     */
    ProductID?: number;
    /**
     * Entity type: Product
     */
    ProductIDs?: number[];
    ProductFilters?: { [key: string]: FilterModel };
    OnlyShowDeviations?: boolean;
    OnlyShowWithStockMutations?: boolean;
  }

  export interface ListFullStockCountSummaryResponse {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    CurrencyID: string;
    CreationTime: string;
    /**
     * Entity type: User
     */
    CreatedBy: number;
    CreatedByFullName: string;
    CompletionTime?: string;
    FinalSummaryValues: boolean;
    TotalProducts: number;
    TotalCounted: number;
    TotalDeviations: number;
    IsExternal: boolean;
    AllowStockMutationsDuringCount: boolean;
    IsCompleted: boolean;
    Progress: number;
    TotalPositiveDeviationAmount: number;
    TotalNegativeDeviationAmount: number;
    Result?: PagedResult<FullStockCountSummaryRecord>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface FullStockCountSummaryRecord {
    /**
     * Entity type: FullStockCountProductSummary
     */
    ID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    StockLabelName: string;
    /**
     * Entity type: Product
     */
    ProductID: number;
    BackendID: string;
    CustomID: string;
    Properties: any;
    /**
     * @deprecated Not returned anymore
     */
    Barcodes: string;
    CountQuantity: number;
    StockMutationQuantity: number;
    CurrentQuantity: number;
    DeviationQuantity: number;
    DeviationAmount: number;
    AbsoluteDeviationQuantity: number;
    AbsoluteDeviationAmount: number;
    CycleCountCreated: boolean;
    CycleCountCountResult?: number;
    SerialNumberRequired: boolean;
  }

  /**
   * Lists the summary of specified Full Stock Count.
   */
  export interface ListFullStockCountLabelSummary {
    /**
     * Entity type: FullStockCount
     */
    ID: number;
    /**
     * Entity type: ProductPropertyType
     */
    IncludedFields?: string[];
    PageConfig?: PageConfig<ListFullStockCountLabelSummaryFilter>;
  }

  export interface ListFullStockCountLabelSummaryFilter {
    /**
     * Entity type: FullStockCountLabel
     */
    FullStockCountLabelID?: number;
    FullStockCountLabelName?: string;
    /**
     * Will only return the labels that contain 1 of these products
     * Entity type: Product
     */
    ProductIDs?: number[];
    /**
     * Will only return the labels that have one of these states
     */
    States?: FullStockCountLabelState[];
    /**
     * Will not return empty labels (the ProductIDs filter does not have influence on this). Only labels that are really empty will be left out.
     */
    IgnoreEmptyLabels?: boolean;
  }

  export interface ListFullStockCountLabelSummaryResponse {
    /**
     * Entity type: FullStockCount
     */
    FullStockCountID: number;
    FullStockCountIsExternal: boolean;
    FullStockCountIsCompleted: boolean;
    AllowStockMutationsDuringCount: boolean;
    Result?: PagedResult<ListFullStockCountLabelSummaryResponse.Label>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListFullStockCountLabelSummaryResponse {
    export interface Label {
      /**
       * Entity type: FullStockCountLabel
       */
      FullStockLabelID: number;
      FullStockLabelName: string;
      State: FullStockCountLabelState;
      IsCompleted: boolean;
      CompletedBy?: ListFullStockCountLabelSummaryResponse.CompletionUser;
      Products: ListFullStockCountLabelSummaryResponse.Product[];
    }

    export interface CompletionUser {
      /**
       * Entity type: User
       */
      ID: number;
      Name: string;
    }

    export interface Product {
      /**
       * Entity type: Product
       */
      ProductID: number;
      BackendID: string;
      CustomID: string;
      Properties: any;
      CountQuantity: number;
    }

  }

  /**
   * Retrieves the status of serial numbers for a specific product and stock label within a Full Stock Count.
   */
  export interface ListFullStockCountSerialNumbersStatus {
    /**
     * Entity type: FullStockCount
     */
    FullStockCountID: number;
    /**
     * Entity type: StockLabel
     */
    StockLabelID: number;
    /**
     * Entity type: Product
     */
    ProductID: number;
    PageConfig?: PageConfig<ListFullStockCountSerialNumbersStatusFilter>;
  }

  export interface ListFullStockCountSerialNumbersStatusFilter {
    Statuses?: FullStockCountSerialNumberStatus[];
    SerialNumbers?: string[];
  }

  export const enum FullStockCountSerialNumberStatus {
    /**
     * Serial number was expected and counted.
     */
    ExpectedAndCounted = 0,
    /**
     * Serial number was expected but not counted.
     */
    ExpectedNotCounted = 1,
    /**
     * Serial number was expected but counted on a different stock label.
     */
    CountedOnDifferentStockLabel = 2,
    /**
     * Serial number was expected but counted on a different organization unit.
     */
    CountedOnDifferentOrganizationUnit = 3,
    /**
     * Serial number was expected but counted as a different product.
     */
    CountedAsDifferentProduct = 4,
    /**
     * Serial number was counted but not expected.
     */
    Unexpected = 5
  }

  export interface ListFullStockCountSerialNumbersStatusResponse {
    Result?: PagedResult<ListFullStockCountSerialNumbersStatusModel>;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export interface ListFullStockCountSerialNumbersStatusModel {
    /**
     * Entity type: StockLabel
     */
    ExpectedStockLabelID?: number;
    /**
     * Entity type: StockLabel
     */
    CountedStockLabelID?: number;
    SerialNumber?: string;
    Status?: FullStockCountSerialNumberStatus;
  }

  /**
   * Creates consolidated pick task
   */
  export interface CreateConsolidatedPickTask {
    /**
     * Entity type: OrganizationUnit
     */
    OrganizationUnitID: number;
    /**
     * @default OldestFirst
     */
    SelectionPriority?: PickTaskSelectionPriority;
    /**
     * The amount of Pick ShipFromStore tasks to be consumed by this operation. If not set, all the pending Pick ShipFromStore tasks will be consumed.
     */
    Limit?: number;
  }

  export const enum PickTaskSelectionPriority {
    /**
     * Oldest First
     */
    OldestFirst = 0,
    /**
     * Newest First
     */
    NewestFirst = 1
  }

  export interface CreateConsolidatedPickTaskResponse {
    /**
     * Entity type: UserTask
     */
    UserTaskID?: number;
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  /**
   * List the available ConsolidatedPickTask tasks for the current user.
   */
  export interface ListConsolidatedPickTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListConsolidatedPickTasksResponse {
    Results: ListConsolidatedPickTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListConsolidatedPickTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: ConsolidatedPickUserTaskData;
    }

  }

  export interface ConsolidatedPickUserTaskData {
    /**
     * Entity type: UserTask
     */
    ZonePickUserTaskIDs: number[];
  }

  /**
   * List the available ConsolidatedZonePickTask tasks for the current user.
   */
  export interface ListConsolidatedZonePickTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListConsolidatedZonePickTasksResponse {
    Results: ListConsolidatedZonePickTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListConsolidatedZonePickTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: ConsolidatedZonePickUserTaskData;
    }

  }

  export interface ConsolidatedZonePickUserTaskData {
    /**
     * Entity type: ProductZone
     */
    ProductZoneID?: number;
  }

  /**
   * List the available FullStockCountLabel tasks for the current user.
   */
  export interface ListFullStockCountLabelTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListFullStockCountLabelTasksResponse {
    Results: ListFullStockCountLabelTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListFullStockCountLabelTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: FullStockCountLabelUserTaskData;
    }

  }

  export interface FullStockCountLabelUserTaskData {
    ID: number;
    Name: string;
    StockLabelID: number;
    FullStockCountID: number;
    StockLabelName: string;
    IsRecount: boolean;
  }

  /**
   * List the available FullStockCount tasks for the current user.
   */
  export interface ListFullStockCountTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListFullStockCountTasksResponse {
    HasActiveFullStockCount: boolean;
    AllowStockMutationsDuringCount: boolean;
    Results: ListFullStockCountTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListFullStockCountTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: FullStockCountUserTaskData;
    }

  }

  export interface FullStockCountUserTaskData {
    Progress: number;
    IsReady: boolean;
    IsExternal: boolean;
    AllowStockMutationsDuringCount: boolean;
  }

  /**
   * List the UserTasks for the current user.
   */
  export interface ListMyUserTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
  }

  export interface ListMyUserTasksResponse {
    Results: ListMyUserTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListMyUserTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Type: UserTaskTypeDto;
      SubType: EnumDto;
    }

  }

  /**
   * List the available PrintPriceLabel tasks for the current user.
   */
  export interface ListPrintPriceLabelTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListPrintPriceLabelTasksResponse {
    Results: ListPrintPriceLabelTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListPrintPriceLabelTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: PrintPriceLabelUserTaskData;
    }

  }

  export interface PrintPriceLabelUserTaskData {
    Products: PrintPriceLabelUserTaskDataProduct[];
  }

  export interface PrintPriceLabelUserTaskDataProduct {
    /**
     * Entity type: Product
     */
    ProductID: number;
    Quantity: number;
  }

  /**
   * List the available Print tasks for the current user.
   */
  export interface ListPrintTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
    /**
     * Filter for a specific UserTaskType.Name using a property name and a list of values.
     */
    Filters?: { [key: string]: { [key: string]: any[] } };
  }

  export interface ListPrintTasksResponse {
    Results: ListPrintTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListPrintTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: PrintUserTaskData;
    }

  }

  export interface PrintUserTaskData {
    TypeID: PrintTaskType;
  }

  /**
   * List the available ReceiveShipment tasks for the current user.
   */
  export interface ListReceiveShipmentTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListReceiveShipmentTasksResponse {
    Results: ListReceiveShipmentTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListReceiveShipmentTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: ReceiveShipmentUserTaskData;
    }

  }

  export interface ReceiveShipmentUserTaskData {
    /**
     * Entity type: Shipment
     */
    ShipmentID: number;
    ShipmentIdentifiers: ReceiveShipmentUserTaskData.ShipmentIdentifier[];
    TotalQuantityShipped: number;
    NetTotalQuantityShipped: number;
    OriginalDeliveryDate?: string;
    ExpectedDeliveryDate?: string;
    OrderCustomTypeID?: number;
    OrderCustomType?: string;
  }

  export namespace ReceiveShipmentUserTaskData {
    export interface ShipmentIdentifier {
      Name: string;
      Value: string;
      Type: ShipmentIdentifierTypes;
    }

  }

  /**
   * List the available Repair tasks for the current user.
   */
  export interface ListRepairTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
    /**
     * Return only Hidden/Visible UserTasks
     * @default false
     */
    IsHidden?: boolean;
    /**
     * Return only UserTasks with one of the specified subtypes
     */
    UserTaskSubTypes?: string[];
    /**
     * Filter for a specific UserTaskType.Name using a property name and a list of values.
     */
    Filters?: { [key: string]: { [key: string]: any[] } };
  }

  export interface ListRepairTasksResponse {
    Results: ListRepairTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListRepairTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      SubType: EnumDto;
      Data: RepairUserTaskData;
    }

  }

  export interface RepairUserTaskData {
    ProductID?: number;
    ProductName: string;
    /**
     * Entity type: Repair
     */
    RepairID: number;
    ProductImage: string;
    OrderID: number;
    SerialNumber?: string;
  }

  /**
   * List the available ReservationCleanup tasks for the current user.
   */
  export interface ListReservationCleanupTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListReservationCleanupTasksResponse {
    Results: ListReservationCleanupTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListReservationCleanupTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: ReservationCleanupUserTaskData;
    }

  }

  export interface ReservationCleanupUserTaskData {
    OrderID: number;
    Type: ReservationCleanupTaskTypes;
    productIDs: number[];
  }

  /**
   * List the available ReservationDeviation tasks for the current user.
   */
  export interface ListReservationDeviationTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListReservationDeviationTasksResponse {
    Results: ListReservationDeviationTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListReservationDeviationTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: ReservationDeviationUserTaskData;
    }

  }

  export interface ReservationDeviationUserTaskData {
    ProductID: number;
  }

  /**
   * List the available ReservationPickup tasks for the current user.
   */
  export interface ListReservationPickupTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListReservationPickupTasksResponse {
    Results: ListReservationPickupTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListReservationPickupTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: ReservationPickupUserTaskData;
    }

  }

  export interface ReservationPickupUserTaskData {
    /**
     * Entity type: Order
     */
    OrderID: number;
  }

  /**
   * List the available ReturnToSupplier tasks for the current user.
   */
  export interface ListReturnToSupplierTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
    /**
     * Return only UserTasks with one of the specified subtypes
     */
    UserTaskSubTypes?: string[];
  }

  export interface ListReturnToSupplierTasksResponse {
    Results: ListReturnToSupplierTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListReturnToSupplierTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      SubType: EnumDto;
      Data: ReturnToSupplierUserTaskData;
    }

  }

  export interface ReturnToSupplierUserTaskData {
    /**
     * Entity type: Order
     */
    OrderID: number;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    Count?: number;
    Quantity?: number;
    TrackingCode: string;
    UseExcludePolicyOnReturnToSupplierShipment: boolean;
    HasMarkedProducts: boolean;
    ProductMarkerValue: string;
    OrderDisplayID?: string;
  }

  /**
   * List the available SecondChanceProductRegistration tasks for the current user.
   */
  export interface ListSecondChanceProductRegistrationTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListSecondChanceProductRegistrationTasksResponse {
    Results: ListSecondChanceProductRegistrationTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListSecondChanceProductRegistrationTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: SecondChanceProductRegistrationUserTaskData;
    }

  }

  export interface SecondChanceProductRegistrationUserTaskData {
    ProductID?: number;
    SerialNumber?: string;
  }

  /**
   * List the available ShipFromStore tasks for the current user.
   */
  export interface ListShipFromStoreTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only UserTasks with one of the specified labels
     */
    Labels?: string[];
  }

  export interface ListShipFromStoreTasksResponse {
    Results: ListShipFromStoreTasksResponse.TaskResult[];
    Labels: { [key: string]: number };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListShipFromStoreTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      Labels: string[];
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      SubType: EnumDto;
      Data: ShipFromStoreUserTaskData;
    }

  }

  export interface ShipFromStoreUserTaskData {
    Count?: number;
    Quantity?: number;
    /**
     * Entity type: Order
     */
    OrderID?: number;
    OrderDisplayID?: string;
    TravelDistanceText: string;
    TravelDistanceInMeters?: number;
    TravelTimeText: string;
    HasMarkedProducts: boolean;
    ProductMarkerValue: string;
    TravelTimeInSeconds?: number;
    /**
     * Entity type: Shipment
     */
    ShipmentID?: number;
    /**
     * Entity type: ShippingMethod
     */
    ShippingMethodID?: number;
    /**
     * Entity type: Product
     */
    productIDs: number[];
    IsPrePicked: boolean;
  }

  /**
   * List the available StockMovementFollowUp tasks for the current user.
   */
  export interface ListStockMovementFollowUpTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListStockMovementFollowUpTasksResponse {
    Results: ListStockMovementFollowUpTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListStockMovementFollowUpTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: StockMovementFollowUpUserTaskData;
    }

  }

  export interface StockMovementFollowUpUserTaskData {
    ProductID?: number;
    SerialNumber?: string;
  }

  /**
   * List the available StockReplenishment tasks for the current user.
   */
  export interface ListStockReplenishmentTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
    /**
     * Filter for a specific UserTaskType.Name using a property name and a list of values.
     */
    Filters?: { [key: string]: { [key: string]: any[] } };
  }

  export interface ListStockReplenishmentTasksResponse {
    Results: ListStockReplenishmentTasksResponse.TaskResult[];
    Aggregations: { [key: string]: UserTaskDataAggregation[] };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListStockReplenishmentTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      Labels: string[];
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: StockReplenishmentUserTaskData;
    }

  }

  export interface StockReplenishmentUserTaskData {
    Quantity: number;
    Type: StockReplenishmentType;
    Zone: string;
    ExpireDate?: string;
    Product?: any;
    Originator: string;
  }

  /**
   * List the available StockReservation tasks for the current user.
   */
  export interface ListStockReservationTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListStockReservationTasksResponse {
    Results: ListStockReservationTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListStockReservationTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: StockReservationUserTaskData;
    }

  }

  export interface StockReservationUserTaskData {
    Count?: number;
    Quantity?: number;
    /**
     * Entity type: Order
     */
    OrderID: number;
    HasMarkedProducts: boolean;
    ProductMarkerValue: string;
    OrderDisplayID: string;
  }

  /**
   * List the UserTasks for the current user.
   */
  export interface ListUrgentTasks {
  }

  export interface ListUrgentTasksResponse {
    Results: ListUrgentTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListUrgentTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Type: UserTaskTypeDto;
      SubType: EnumDto;
    }

  }

  /**
   * List the available ZonedCycleCountPreCount tasks for the current user.
   */
  export interface ListZonedCycleCountPreCountTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
    /**
     * Filter for a specific UserTaskType.Name using a property name and a list of values.
     */
    Filters?: { [key: string]: { [key: string]: any[] } };
  }

  export interface ListZonedCycleCountPreCountTasksResponse {
    Results: ListZonedCycleCountPreCountTasksResponse.TaskResult[];
    Aggregations: { [key: string]: UserTaskDataAggregation[] };
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListZonedCycleCountPreCountTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: ZonedCycleCountPreCountUserTaskData;
    }

  }

  export interface ZonedCycleCountPreCountUserTaskData {
    CycleCountZoneID: number;
    CycleCountZoneName: string;
    ProductID: number;
    OriginTypeID?: number;
    OriginID?: number;
    ProductDisplayValue: string;
    ProductImageBlob: string;
  }

  /**
   * List the available ZonedCycleCount tasks for the current user.
   */
  export interface ListZonedCycleCountTasks {
    /**
     * Return only UserTasks attached to this UserID
     * Entity type: User
     */
    UserID?: number;
    /**
     * Return only Started/Open UserTasks
     */
    IsStarted?: boolean;
  }

  export interface ListZonedCycleCountTasksResponse {
    Results: ListZonedCycleCountTasksResponse.TaskResult[];
    Error?: ServiceError;
    Metadata?: ResponseMessageMetadata;
  }

  export namespace ListZonedCycleCountTasksResponse {
    export interface TaskResult {
      /**
       * Entity type: UserTask
       */
      ID: number;
      Description: string;
      Priority: number;
      PriorityDescription: string;
      CanBeIgnored: boolean;
      CreationTime: string;
      StartTime?: string;
      ExpectedTimeToComplete?: string;
      ExpectedCompletionTime?: string;
      Deadline?: string;
      User: UserDto;
      Data: ZonedCycleCountUserTaskData;
    }

  }

  export interface ZonedCycleCountUserTaskData {
    /**
     * Entity type: Product
     */
    ProductID: number;
    /**
     * Entity type: ZonedCycleCount
     */
    ZonedCycleCountID?: number;
    Progress: number;
    DisplayValue: string;
    /**
     * Entity type: CycleCountOriginType
     */
    OriginTypeID?: number;
    OriginID?: number;
    Origin: string;
  }

  export interface UserCardTypeDto {
    AmountType: UserCardAmountTypes;
    /**
     * Entity type: Enum
     */
    ID: number;
    /**
     * Usually an internal & unchangable code - for display, use `Description`
     */
    Name: string;
    Description: string;
  }

  export const enum UserCardAmountTypes {
    Currency = 0,
    Points = 1
  }

  export interface ListUserCardsFilter {
    Barcode?: string;
    /**
     * Entity type: UserCardType
     */
    TypeID?: number;
    /**
     * Entity type: User
     */
    UserID?: number;
    User?: string;
  }

  export interface UserCardDto {
    /**
     * Entity type: UserCard
     */
    ID: number;
    /**
     * Entity type: User
     */
    UserID: number;
    TypeID: number;
    Barcode: string;
    IsActive: boolean;
    CurrencyID: string;
    Description: string;
    /**
     * Entity type: Discount
     */
    DiscountID?: number;
    /**
     * Entity type: LoyaltyProgram
     */
    LoyaltyProgramID?: number;
    User: UserDto;
    CurrentBalance: UserCardBalance;
    Type: UserCardTypeDto;
  }

  export interface UserCardBalance {
    CardBalance: number;
    CurrencyBalance: number;
    CurrencyID: string;
  }

  export const enum UserCardMutationStatuses {
    Pending = 0,
    Completed = 1,
    Cancelled = 2
  }

  export const enum OpenCashDrawerResults {
    Failure = 0,
    Opened = 1,
    AlreadyOpen = 2,
    OpenedAndClosed = 3
  }

  export interface ListByProximityFilter {
    Type?: OrganizationUnitTypes;
    /**
     * Range: -90-90
     */
    Latitude: number;
    /**
     * Range: -180-180
     */
    Longitude: number;
    /**
     * Entity type: Country
     */
    CountryID?: string;
    /**
     * Entity type: OrganizationUnit
     */
    IDs?: number[];
  }

  export const enum PointPolicyType {
    None = 0,
    Pending = 1,
    Expiration = 2
  }

}
